/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      8057 × 20 bytes =   161140 bytes
//   Child IDs:         7121 ×  4 bytes =    28484 bytes
//   Terminators:         85 ×  4 bytes =      340 bytes
//   Strings:           2377 strings      =    35285 bytes
//   Aux Data:         12663 ×  4 bytes =    50652 bytes
//   Aux Offsets:       8057 ×  4 bytes =    32228 bytes
//   Regex Patterns:      11 patterns     =      548 bytes
//   Simple Hints:      1358 hints        =    21728 bytes
//   Hint Indices:      3045 ×  4 bytes =    12180 bytes
//   TOTAL:                                    ≈   342585 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [4] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 7, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 12, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [6] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 12, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 12, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Ref(SecretKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 12, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 12, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 12, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 12, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 12, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 14, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 14, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 14, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 14, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 16, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 16, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 21, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 25, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 29, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 29, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] Ref(TypesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 29, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(SecretsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 29, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Ref(ConnectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 29, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 29, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 29, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 31, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 31, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 35, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 40, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 40, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Ref(TypesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 40, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(SecretsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 40, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(ConnectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 40, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 40, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 40, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 40, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(DatabasesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(ClustersKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 10, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [51] Ref(CreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] Ref(CreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(CreateclusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 54, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 54, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 54, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 1, _padding: 0 }, // [57] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 56, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 56, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 56, child_count: 14, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [60] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [61] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [62] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [63] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [64] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [65] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [66] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [67] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [68] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [69] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [70] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 70, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [71] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 72, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [72] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 72, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [73] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 72, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [74] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [75] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [76] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [77] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [78] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [79] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [80] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [81] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [82] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 14, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [83] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [84] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [85] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [86] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [87] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [88] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [89] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [90] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [91] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [92] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [93] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [94] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 94, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [95] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 94, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [96] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 94, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [97] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 96, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [98] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 96, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [99] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 96, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 98, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [101] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 98, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [102] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 98, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [103] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 100, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [104] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 100, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [105] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 100, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [106] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 102, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [107] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 102, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [108] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 102, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [109] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 104, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [110] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 104, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [111] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 106, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [112] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 106, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [113] Ref(SemiStructuredAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 106, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [114] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 106, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [115] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 106, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [116] StringParser("ACKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 106, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [117] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 106, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [118] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 106, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [119] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 109, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [120] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 109, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [121] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 109, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [122] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 109, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [123] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 109, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [124] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 109, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [125] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 109, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [126] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 109, child_count: 4, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [127] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 113, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [128] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 113, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [129] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 113, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [130] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 115, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [131] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 117, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [132] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 117, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [133] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [134] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [135] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [136] StringParser(")")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [137] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [138] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [139] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 125, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [140] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 127, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [141] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 129, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [142] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 129, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [143] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 129, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [144] Meta("dedent")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 129, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [145] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 129, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [146] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 131, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [147] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 131, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [148] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 131, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [149] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 131, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [150] StringParser("ALLOW_CONNECTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 131, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [151] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 131, child_count: 5, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [152] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 136, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [153] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 136, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [154] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 136, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [155] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 136, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [156] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 139, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [157] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [158] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [159] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [160] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [161] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 145, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [162] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [163] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [164] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [165] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [166] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 151, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [167] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 151, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [168] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 151, child_count: 4, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [169] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 155, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [170] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 155, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [171] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 155, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [172] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 155, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [173] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 155, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [174] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 158, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [175] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 158, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [176] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 158, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [177] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 158, child_count: 6, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [178] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [179] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [180] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [181] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [182] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [183] Ref(RotateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [184] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 4, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [185] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [186] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [187] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [188] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 168, child_count: 6, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [189] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [190] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 176, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [191] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 176, child_count: 3, min_times: 1, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [192] AnyNumberOf(3 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 179, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [193] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [194] Ref(Allow_connectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [195] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [196] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [197] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [198] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [199] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [200] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 186, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [201] Ref(Is_templateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 186, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [202] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 186, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [203] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [204] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [205] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [206] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [207] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [208] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [209] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 192, child_count: 4, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [210] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [211] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [212] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [213] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [214] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [215] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [216] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [217] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [218] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 3, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [219] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [220] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [221] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [222] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 204, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [223] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [224] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [225] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [226] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 4, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [227] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 212, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [228] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 212, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [229] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 212, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [230] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 212, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [231] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 212, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [232] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [233] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [234] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [235] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [236] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 216, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [237] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 218, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [238] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 218, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [239] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 218, child_count: 7, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [240] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [241] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [242] Ref(AlterDefaultPrivilegesObjectPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [243] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [244] Ref(AlterDefaultPrivilegesSchemaObjectsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [245] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 2, min_times: 0, first_terminator_idx: 1, terminator_count: 1, _padding: 0 }, // [246] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 227, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [247] Ref(AlterDefaultPrivilegesToFromRolesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 227, child_count: 0, min_times: 0, first_terminator_idx: 1, terminator_count: 0, _padding: 0 }, // [248] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 227, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [249] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [250] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [251] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [252] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 232, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [254] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 234, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [255] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 234, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [256] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 234, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [257] Delimited(10 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 236, child_count: 10, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [258] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [259] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [260] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [261] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [262] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [263] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [264] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [265] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [266] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [267] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [268] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [269] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 8, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [270] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 254, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] Ref(AlterDefaultPrivilegesObjectPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] Ref(AlterDefaultPrivilegesSchemaObjectsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 2, _padding: 0 }, // [280] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] Ref(AlterDefaultPrivilegesToFromRolesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [284] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [285] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [286] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [287] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [288] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [289] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [290] Ref(TypesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [291] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 9, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [292] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [293] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [294] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [295] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [296] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [297] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [298] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [299] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [300] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [301] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [302] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [303] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [304] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 285, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [305] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [306] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [307] Ref(RolesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 287, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [308] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [309] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [310] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [311] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [312] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [313] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [314] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [315] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [316] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 7, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [317] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [318] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [319] Ref(TypesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [320] Ref(SecretsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [321] Ref(ConnectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [322] Ref(DatabasesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [323] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [324] Ref(ClustersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [325] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [326] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [327] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [328] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 303, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [329] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [330] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [331] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [332] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [333] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [334] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [335] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 10, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [336] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 317, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [337] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 320, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [338] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 320, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [339] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 320, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [340] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 320, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [341] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [342] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [343] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [344] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [345] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [346] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [347] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [348] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [349] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [350] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [351] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 331, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [352] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [353] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [354] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [355] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [356] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [357] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [358] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [359] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [360] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [361] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 339, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [362] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [363] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [364] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [365] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 346, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [366] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 346, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [367] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 346, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [368] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 346, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [369] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 346, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [370] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [371] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [372] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [373] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 353, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [374] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 353, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [375] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 353, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [376] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 353, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [377] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 353, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [378] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 353, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [379] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [380] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [381] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [382] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [383] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 359, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [384] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 359, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [385] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 359, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [386] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [387] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [388] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [389] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [390] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [391] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 366, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [392] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 366, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [393] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 366, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [394] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 366, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [395] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [396] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [397] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [398] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [399] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 373, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [400] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 373, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [401] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 373, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [402] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 373, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [403] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 376, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [404] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [405] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 378, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [406] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [407] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [408] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [409] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [410] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [411] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [412] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 385, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [413] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 385, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [414] Ref(Current_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 385, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [415] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [416] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [417] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [418] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 7, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [419] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 396, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [420] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 398, child_count: 13, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [421] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 411, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [422] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [423] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [424] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [425] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [426] Ref(ConversionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [427] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [428] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [429] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [430] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [431] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [432] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [433] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [434] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [435] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 420, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [436] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 420, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [437] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 420, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [438] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [439] Ref(ProceduralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [440] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [441] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [442] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [443] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [444] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 425, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [445] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 425, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [446] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 425, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [447] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [448] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [449] Ref(DictionaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [450] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [451] Ref(TemplateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [452] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [453] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [454] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [455] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 434, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [456] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [457] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [458] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [459] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [460] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [461] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [462] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 439, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [463] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 439, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [464] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 439, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [465] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 442, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [466] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 443, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [467] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [468] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 445, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [469] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 445, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [470] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [471] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [472] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [473] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [474] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [475] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [476] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [477] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [478] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [479] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [480] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 458, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [481] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 460, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [482] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 460, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [483] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 460, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [484] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [485] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [486] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [487] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [488] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [489] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 469, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [490] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 471, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [491] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 474, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [492] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 474, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [493] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 474, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [494] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 474, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [495] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 478, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [496] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 480, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [497] Ref(ClassKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 480, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [498] Ref(FamilyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 480, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [499] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 480, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [500] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 480, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [501] Ref(IndexAccessMethodSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 480, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [502] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [503] Ref(TransformKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [504] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [505] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [506] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 24, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [507] OneOf(24 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 7, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [508] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [509] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [510] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [511] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [512] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [513] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 515, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [514] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 517, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [515] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 517, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [516] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 517, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [517] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [518] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [519] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 523, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [520] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 523, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [521] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 523, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [522] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 523, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [523] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 523, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [524] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 523, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [525] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [526] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [527] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [528] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 13, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [529] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [530] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 545, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [531] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 547, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [532] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 547, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [533] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 547, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [534] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 547, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [535] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 547, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [536] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [537] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [538] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 549, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [539] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [540] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [541] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [542] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [543] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [544] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [545] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [546] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 556, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [547] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [548] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [549] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [550] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [551] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [552] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [553] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [554] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [555] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 565, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [556] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [557] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [558] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [559] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [560] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [561] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 570, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [562] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 570, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [563] Ref(ExpressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 570, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [564] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 570, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [565] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [566] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [567] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [568] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [569] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 578, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [570] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [571] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [572] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [573] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [574] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 580, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [575] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 583, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [576] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [577] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [578] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [579] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [580] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [581] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [582] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [583] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 593, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [584] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 593, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [585] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [586] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [587] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [588] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [589] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [590] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [591] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 599, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [592] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 599, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [593] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [594] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [595] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [596] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [597] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [598] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [599] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [600] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [601] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [602] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [603] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [604] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 609, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [605] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 609, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [606] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 609, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [607] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [608] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [609] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [610] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 614, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [611] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 614, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [612] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 614, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [613] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 618, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [614] Ref(PlainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 618, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [615] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 618, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [616] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 618, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [617] Ref(MainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 618, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [618] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [619] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [620] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [621] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [622] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 623, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [623] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 623, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [624] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 623, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [625] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 625, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [626] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 625, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [627] Ref(TableConstraintUsingIndexSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 625, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [628] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [629] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [630] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [631] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 630, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [632] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [633] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [634] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 634, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [635] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 634, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [636] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 634, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [637] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [638] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [639] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [640] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [641] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [642] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [643] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [644] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 643, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [645] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 643, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [646] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 643, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [647] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 643, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [648] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [649] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [650] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [651] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [652] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [653] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [654] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 651, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [655] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [656] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [657] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [658] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [659] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 656, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [660] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 656, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [661] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 656, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [662] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 656, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [663] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 660, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [664] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 660, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [665] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [666] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [667] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [668] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [669] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [670] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [671] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 669, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [672] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 669, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [673] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 669, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [674] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [675] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [676] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [677] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 673, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [678] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 673, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [679] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 673, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [680] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 673, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [681] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 673, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [682] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 677, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [683] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [684] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [685] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [686] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [687] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [688] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [689] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [690] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [691] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [692] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [693] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [694] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [695] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [696] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [697] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [698] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [699] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [700] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 691, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [701] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 691, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [702] Ref(OidsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 691, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [703] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [704] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [705] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [706] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [707] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 696, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [708] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 696, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [709] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [710] Ref(LoggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [711] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [712] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [713] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [714] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [715] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [716] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [717] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [718] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [719] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [720] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [721] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [722] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 707, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [723] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 707, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [724] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 707, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [725] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [726] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [727] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [728] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 712, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [729] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 712, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [730] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 712, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [731] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [732] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [733] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [734] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [735] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [736] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [737] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [738] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [739] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [740] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [741] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 726, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [742] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 726, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [743] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 726, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [744] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 726, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [745] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 726, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [746] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 726, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [747] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 729, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [748] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [749] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [750] Ref(COLUMN)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [751] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [752] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [753] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [754] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [755] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 740, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [756] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [757] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [758] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [759] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [760] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [761] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [762] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [763] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [764] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [765] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [766] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [767] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [768] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [769] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [770] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [771] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 755, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [772] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 757, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [773] Ref(AlterForeignTableActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 757, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [774] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [775] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [776] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [777] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [778] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [779] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [780] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [781] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [782] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [783] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [784] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 767, child_count: 10, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [785] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [786] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 780, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [787] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [788] Ref(CalledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [789] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [790] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [791] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [792] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [793] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [794] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [795] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [796] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [797] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [798] Ref(StrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [799] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 792, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [800] Ref(ImmutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 792, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [801] Ref(StableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 792, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [802] Ref(VolatileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 792, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [803] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [804] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [805] Ref(LeakproofKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [806] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [807] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [808] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 797, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [809] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [810] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [811] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [812] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 801, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [813] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 801, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [814] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 804, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [815] Ref(UnsafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 804, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [816] Ref(RestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 804, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [817] Ref(SafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 804, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [818] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [819] Ref(CostKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [820] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [821] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [822] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [823] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [824] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [825] Ref(SupportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [826] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [827] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 813, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [828] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 813, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [829] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 813, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [830] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [831] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 817, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [832] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [833] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [834] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [835] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [836] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [837] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [838] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [839] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 824, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [840] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 824, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [841] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 824, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [842] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [843] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [844] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 828, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [845] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 828, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [846] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 828, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [847] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 828, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [848] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [849] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [850] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [851] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [852] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [853] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [854] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [855] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [856] Ref(AlterFunctionActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [857] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [858] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [859] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [860] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [861] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 847, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [862] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 847, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [863] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 847, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [864] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 847, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [865] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [866] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [867] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 850, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [868] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [869] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [870] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [871] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [872] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [873] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [874] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [875] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [876] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [877] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [878] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [879] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [880] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [881] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 865, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [882] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 865, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [883] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 865, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [884] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 865, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [885] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 867, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [886] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 867, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [887] Ref(EnabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 867, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [888] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 867, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [889] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 873, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [890] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [891] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [892] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [893] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [894] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 882, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [895] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [896] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [897] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [898] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [899] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [900] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [901] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [902] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [903] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [904] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [905] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [906] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [907] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [908] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [909] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [910] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [911] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [912] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [913] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [914] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [915] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [916] Ref(PlainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [917] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [918] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [919] Ref(MainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [920] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [921] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [922] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [923] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [924] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [925] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [926] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [927] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [928] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [929] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [930] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [931] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [932] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [933] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [934] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [935] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [936] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 927, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [937] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 927, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [938] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [939] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [940] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [941] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [942] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [943] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 934, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [944] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [945] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [946] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [947] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [948] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [949] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [950] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [951] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [952] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [953] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [954] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [955] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [956] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [957] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [958] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 950, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [959] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [960] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [961] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [962] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 957, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [963] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [964] Ref(AlterMaterializedViewActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [965] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [966] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 964, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [967] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 965, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [968] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 965, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [969] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 965, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [970] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 965, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [971] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 965, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [972] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [973] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [974] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [975] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [976] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 971, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [977] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 971, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [978] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 971, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [979] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 971, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [980] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [981] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [982] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [983] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [984] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [985] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [986] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 9, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [987] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 986, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [988] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 986, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [989] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 986, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [990] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 986, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [991] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 986, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [992] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 989, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [993] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 989, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [994] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 989, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [995] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [996] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [997] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [998] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [999] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 991, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1000] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 992, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1001] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 992, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1002] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1003] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 997, child_count: 10, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1004] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1007, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1005] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1007, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1006] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1007, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1007] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1008] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1009] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1010] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1011] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1012] Ref(SinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1013] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1014] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1015] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1016] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1017] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1018] Ref(SecretKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1019] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1020] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1021] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1022] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1023] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1024] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1025] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1026] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1027] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1028] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1029] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1030] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1031] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1032] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1033] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1034] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 3, min_times: 1, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1035] AnySetOf(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1027, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1036] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1037] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1038] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1031, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1039] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1040] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1041] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1042] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1043] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1044] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1045] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1038, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1046] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1038, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1047] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1048] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1049] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1050] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1051] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1052] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1053] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1054] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1055] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1052, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1056] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1057] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1058] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1059] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1060] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1061] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1062] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1063] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1064] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1065] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1064, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1067] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1066, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1068] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1066, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1069] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1066, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1070] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1071] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1072] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1073] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1074] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1075] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1076] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1077] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1078] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1079] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1080] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1081] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1082] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1083] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1084] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1085] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1086] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1087] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1088] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1083, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1089] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1090] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1089, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1091] Ref(AlterProcedureActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1092] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1092, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1093] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1092, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1094] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1092, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1095] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1092, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1096] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1097] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1098] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1099] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1100] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1101] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1102] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1103] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1104] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1105] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1106] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1107] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1108] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1109] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1110] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1111] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1112] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1113] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1114] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1115] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1113, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1116] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1113, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1117] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1113, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1118] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1117, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1119] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1120] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1121] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1122] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1123] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1124] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1125] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1126] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1127] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1128] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1129] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1130] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1131] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1132] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1133] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1134] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1135] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1136] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1137] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1138] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1139] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1138, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1140] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1138, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1141] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1140, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1142] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1140, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1143] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1144] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1145] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1146] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1147] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1148] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1149] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1150] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1151] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1152] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1153] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1154] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1155] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1154, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1156] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1154, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1157] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1154, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1158] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1158, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1159] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1158, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1160] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1158, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1161] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1158, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1162] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1158, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1163] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1164] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 9, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1165] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1166] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1167] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1168] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1169] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1170] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1171] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1172] Ref(SinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1173] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1175] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1176] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1177] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1178] Ref(SecretKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1179] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1181] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1182] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1183] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1184] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1181, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1185] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1181, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1186] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1187] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1188] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1189] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1190] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1189, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1191] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1192] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1193] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1194] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1195] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1196] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1193, child_count: 10, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1197] AnySetOf(10 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1203, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1198] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1205, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1199] Ref(SuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1205, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1200] Ref(NosuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1205, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1201] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1207, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1202] Ref(CreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1207, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1203] Ref(NocreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1207, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1204] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1205] Ref(CreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1206] Ref(NocreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1207] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1211, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1208] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1211, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1209] Ref(NoinheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1211, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1210] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1211] Ref(LoginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1212] Ref(NologinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1213] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1214] Ref(ReplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1215] Ref(NoreplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1216] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1217, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1217] Ref(BypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1217, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1218] Ref(NobypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1217, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1219] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1220] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1221] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1222] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1223] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1223, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1224] Ref(EncryptedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1223, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1225] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1223, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1226] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1227] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1228] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1229] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1230] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1231] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1232] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1233] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1234] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1235] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1236] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1237] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1238] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1239] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1240] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1241] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1242] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1243] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1244] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1245] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1241, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1246] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1247] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1248] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1249] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1250] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1251] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1249, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1252] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1249, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1253] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1249, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1254] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1251, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1255] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1256] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1257] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1258] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1259] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1257, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1260] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1257, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1261] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1259, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1262] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1263] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1264] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1265] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1266] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1267] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1268] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1269] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1270] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1266, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1271] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1272] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1273] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1274] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1272, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1275] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1272, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1276] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1272, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1277] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1272, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1278] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1274, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1279] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1280] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1281] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1282] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1283] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1280, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1284] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1280, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1285] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1280, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1286] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1280, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1287] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1286, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1288] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1286, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1289] Ref(SecretKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1286, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1290] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1286, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1291] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1286, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1292] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1286, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1293] Anything
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1286, child_count: 10, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1294] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1295] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1296] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1297] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1298, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1298] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1299] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1300] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1301] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1302] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1303] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1304] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1307, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1305] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1307, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1306] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1307, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1307] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1307, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1308] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1309] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1310] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1311] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1311, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1312] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1313] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1314] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1315] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1316] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1317] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1318] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1319] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1319, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1320] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1319, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1321] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1319, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1322] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1319, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1323] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1324] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1325] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1326] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1327] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1323, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1328] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1329] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1330] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1331] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1332] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1333] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1334] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1335] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1336] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1331, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1337] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1331, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1338] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1331, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1339] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1340] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1341] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1342] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1336, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1343] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1336, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1344] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1336, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1345] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1346] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1347] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1348] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1349] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1350] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1345, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1351] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1346, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1352] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1353] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1354] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1355] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1349, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1356] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1357] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1358] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1359] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1360] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1355, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1361] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1355, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1362] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1355, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1363] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1355, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1364] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1365] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1366] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1367] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1368] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1363, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1369] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1370] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1371] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1372] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1373] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1374] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1375] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1376] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1377] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1378] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1379] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1380] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1381] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1382] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1383] Ref(SinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1376, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1384] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1385] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1386] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1387] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1388] Ref(SizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1389] Ref(InstanceSizes)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1380, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1390] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1385, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1391] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1385, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1392] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1385, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1393] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1385, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1394] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1385, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1395] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1396] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1397] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1398] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1399] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1400] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1401] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1402] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1403] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1404] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1405] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1406] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1407] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1408] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1409] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1410] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1411] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1412] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1404, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1413] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1414] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1415] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1416] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1417] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1418] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1419] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1420] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1421] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1422] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 9, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1423] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1421, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1424] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1423, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1425] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1423, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1426] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1423, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1427] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1427, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1428] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1429] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1430] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1431] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1432] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1433] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1434] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1432, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1435] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1436] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1437] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1438] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1439] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1440] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1437, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1441] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1442] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1443] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1444] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1445] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1446] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1441, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1447] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1441, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1448] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1441, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1449] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1450] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1451] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1452] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1453] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1454] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1455] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1456] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1457] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1458] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1459] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1460] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1461] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1462] Ref(Current_sessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1463] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1458, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1464] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1458, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1465] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1458, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1466] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1458, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1467] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1461, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1468] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1461, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1469] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1461, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1470] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1471] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1472] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1473] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1467, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1474] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1475] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1476] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1477] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1478] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1479] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1473, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1480] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1481] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1482] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1483] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1484] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1485] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1486] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1479, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1487] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1488] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1489] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 23, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1490] OneOf(23 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 7, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1491] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1511, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1492] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1511, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1493] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1511, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1494] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1511, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1495] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1511, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1496] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1511, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1497] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1498] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1499] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1513, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1500] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1514, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1501] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1514, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1502] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1503] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1504] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1505] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1506] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1507] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1508] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1523, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1509] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1523, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1510] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1523, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1511] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1523, child_count: 13, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1512] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1513] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1541, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1514] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1515] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1516] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1517] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1518] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1543, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1519] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1545, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1520] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1545, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1521] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1545, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1522] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1523] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1524] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1525] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1526] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1527] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1528] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1529] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1552, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1530] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1531] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1532] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1533] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1534] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1556, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1535] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1558, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1536] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1558, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1537] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1558, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1538] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1561, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1539] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1540] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1541] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1542] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1543] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1544] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1545] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1546] Ref(ExpressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1547] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1548] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1549] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1550] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1551] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1552] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1553] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1554] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1555] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1556] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1557] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1576, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1558] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1579, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1559] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1560] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1561] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1581, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1562] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1563] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1564] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1565] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1566] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1589, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1567] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1589, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1568] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1569] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1570] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1591, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1571] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1572] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1573] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1574] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1595, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1575] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1595, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1576] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1577] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1578] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1579] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1580] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1581] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1582] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1583] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1584] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1585] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1586] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1587] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1588] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1589] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1590] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1591] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1592] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1593] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1610, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1594] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1610, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1595] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1610, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1596] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1597] Ref(PlainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1598] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1599] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1600] Ref(MainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1601] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1602] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1603] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1604] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1605] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1619, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1606] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1619, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1607] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1619, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1608] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1609] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1610] Ref(TableConstraintUsingIndexSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1611] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1612] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1613] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1614] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1626, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1615] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1616] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1617] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1618] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1619] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1630, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1620] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1621] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1622] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1623] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1624] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1625] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1626] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1627] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1639, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1628] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1639, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1629] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1639, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1630] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1639, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1631] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1632] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1633] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1634] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1635] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1636] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1637] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1647, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1638] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1639] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1640] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1641] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1642] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1643] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1644] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1645] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1646] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1647] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1648] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1649] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1650] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1651] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1652] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1653] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1661, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1654] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1655] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1656] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1657] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1658] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1659] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1660] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1661] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1662] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1663] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1664] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1665] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1666] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1667] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1668] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1669] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1670] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1671] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1672] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1673] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1674] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1675] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1676] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1677] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1678] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1679] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1680] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1685, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1681] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1685, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1682] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1685, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1683] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1684] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1685] Ref(OidsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1686] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1690, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1687] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1690, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1688] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1690, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1689] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1690, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1690] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1691] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1692] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1693] Ref(LoggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1694, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1694] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1694, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1695] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1696] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1697] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1698] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1699] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1700] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1701] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1701, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1702] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1701, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1703] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1701, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1704] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1701, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1705] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1706] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1707] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1708] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1709] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1710] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1711] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1712] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1713] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1714] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1715] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1716] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1717] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1718] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1719] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1715, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1720] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1715, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1721] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1715, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1722] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1719, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1723] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1719, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1724] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1725] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1726] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1727] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1728] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1729] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1730] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1731] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1732] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1733] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1734] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1735] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1730, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1736] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1733, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1737] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1733, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1738] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1733, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1739] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1740] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1741] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1742] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1743] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1744] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1745] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1746] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1747] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1741, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1748] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1743, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1749] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1745, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1750] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1751] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1752] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1753] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1754] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1755] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1756] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1757] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1750, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1758] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1759] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1760] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1761] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1762] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1763] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1764] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1765] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1766] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1767] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1768] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1769] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1770] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1771] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1766, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1772] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1773] Ref(AlterTableActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1774] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1775] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1776] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1777] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1778] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1779] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1780] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1781] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1782] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1783] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1784] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1785] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1786] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1787] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1788] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1789] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1790] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1791] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1792] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1793] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1794] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1795] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1796] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1797] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1798] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1799] Ref(AttachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1800] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1801] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1802] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1797, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1803] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1804] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1805] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1806] Ref(PartitionBoundSpecSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1807] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1808] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1809] Ref(DetachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1810] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1811] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1805, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1812] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1805, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1813] Ref(FinalizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 9, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1814] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1815] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1816] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1817] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1818] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1814, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1819] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1820] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1821] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1822] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1823] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1824] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1825] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1826] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1827] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1828] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1829] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1830] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1831] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1832] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1833] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 8, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1834] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1835] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1836] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1837] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1838] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1839] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1840] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1841] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1842] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1843, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1843] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1843, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1844] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1845] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1846] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1847] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1848] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1851, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1849] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1851, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1850] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1851, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1851] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1852] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1853] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1854] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1855] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1856] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1857] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1858] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1859] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 8, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1860] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1867, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1861] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1867, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1862] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1867, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1863] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1867, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1864] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1865] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1866] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1867] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1868] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1869] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1870] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1871] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1872] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1873] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1874] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1875] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1876, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1876] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1876, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1877] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1878] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1879] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1880] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1881] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1882] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1883] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1882, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1884] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1886, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1885] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1886, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1886] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1886, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1887] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1886, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1888] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1886, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1889] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1890] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1891] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1892] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1893] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1895, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1894] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1895, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1895] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1895, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1896] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1895, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1897] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1895, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1898] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1895, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1899] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1897, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1900] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1901] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1902] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1903] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1904] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1905] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1906] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1907] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1908] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1909] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1910] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1911] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1912] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1913] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1914] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1915] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1916] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1917] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1918] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1922, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1919] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1922, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1920] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1922, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1921] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1922, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1922] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1922, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1923] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1924] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1925] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1926] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1927] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1928] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1929] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1930] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1931] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1932] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1933] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1933, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1934] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1933, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1935] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1933, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1936] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1933, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1937] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1935, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1938] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1939] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1940] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1941] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1942] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1943] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1945, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1944] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1947, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1945] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1947, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1946] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1947, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1947] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1947, child_count: 8, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1948] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1949] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1950] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1951] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1955, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1952] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1953] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1954] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1955] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1956] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1957, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1957] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1959, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1958] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1959, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1959] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1959, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1960] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1959, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1961] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1962] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1963] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1964] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1965] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1966] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1967] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1968] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1969] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1970] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1971] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1972] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1973] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1974] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1975, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1975] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1975, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1976] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1975, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1977] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1975, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1978] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1975, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1979] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1980] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1981] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1982] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1983] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1984] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1985] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1986] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1987] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1988] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 7, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1989] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1990] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1995, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1991] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1995, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1992] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1995, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1993] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1995, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1994] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1997, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1995] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1996] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1997] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1998] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [1999] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2000] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2001] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2002] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2003] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2004] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2005] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2006] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2007] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2008] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2009] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2013, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2010] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2013, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2011] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2013, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2012] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2013, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2013] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2013, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2014] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2015] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2016] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2017] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2018] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2019] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2020] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2021, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2021] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2021, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2022] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2021, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2023] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2021, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2024] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2025] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2026] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2027] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2028] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2029] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2030] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2029, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2031] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2031, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2032] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2033] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2033, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2034] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2035] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2036] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2037] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2037, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2038] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2037, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2039] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2040, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2040] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2041] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2042] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2043] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2044] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2045] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2046] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2045, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2047] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2047, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2048] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2047, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2049] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2047, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2050] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2051] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2052, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2052] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2054, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2053] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2054] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2055] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2056] Ref(Skip_lockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2057] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2058] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2058, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2059] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2060, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2060] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2062, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2061] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2062, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2062] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2065, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2063] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2064] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2065] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2066] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2067] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2068] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 10, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2069] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2070] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2071] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2072] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2073] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2074] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2075] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2076] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2077] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2078] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2079] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2080] StringParser("ARN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2081] StringParser("ARRANGEMENT")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2082] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2080, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2083] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2084] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2085] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2086] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2087] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2087, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2088] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2089] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2090] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2091] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2092] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2090, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2093] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2094] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2095] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2096] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2097] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2098] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2099] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2100] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2101] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2096, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2102] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2103] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2104] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2105] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2106] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2101, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2107] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2105, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2108] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2105, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2109] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2105, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2110] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2111] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2112] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2113] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2114] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2115] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2116] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2117] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2118] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2119] StringParser("ATTACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2120] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2121] StringParser("AUCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2122] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2123] StringParser("AUTHORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2124] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2106, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2125] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2126] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2127] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2128] StringParser("AVAILABILITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2129] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2130] StringParser("AVRO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2131] StringParser("AWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2132] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2133] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2134] MultiStringParser(12 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2107, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2135] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2109, child_count: 7, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [2136] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2137] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2138] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2139] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2140] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2141] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2142] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2143] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2144] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2145] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2146] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [2147] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2148] Ref(CompositeValueExpansionSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2149] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2150] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2151] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2152] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2153] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2154] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2155] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2156] StringParser("BIGSERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2118, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2157] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2118, child_count: 5, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2158] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2159] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2160] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2161] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2162] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2163] Ref(JsonOperatorSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2164] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2165] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2166] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2167] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2168] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2123, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2169] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2170] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2171] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2172] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2125, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2173] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2174] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2175] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2176] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2177] StringParser("BODY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2178] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2179] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2180] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2181] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2129, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2182] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2183] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2184] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2185] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2186] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2187] StringParser("BOX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2188] StringParser("BPCHAR")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 3, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2189] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2134, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2190] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2191] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2136, child_count: 3, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2192] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2193] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2194] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2195] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 3, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2196] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2197] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2198] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2199] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2200] StringParser("BROKER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2201] StringParser("BROKERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2202] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2203] StringParser("BUCKET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2204] StringParser("BUFFERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2205] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2206] StringParser("BY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2207] StringParser("BYPASSRLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2208] StringParser("BYTEA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2144, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2209] StringParser("BYTES")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2144, child_count: 3, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2210] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2147, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2211] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2147, child_count: 6, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2212] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2153, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2213] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2153, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2214] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2153, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2215] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2153, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2216] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2155, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2217] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2218] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 3, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2219] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2158, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2220] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2158, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2221] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2160, child_count: 7, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2222] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2223] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2224] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2225] Ref(BreadthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2226] Ref(DepthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2227] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2228] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2229] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2230] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2231] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 6, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2232] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2233] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2234] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2235] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2236] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2237] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2238] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2239] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2240] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2241] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2242] Ref(CallKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2243] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2244] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2245] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2246] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2179, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2247] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2179, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2248] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2179, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2249] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2179, child_count: 2, min_times: 0, first_terminator_idx: 13, terminator_count: 3, _padding: 0 }, // [2250] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 6, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [2251] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2252] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2253] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2187, child_count: 1, min_times: 0, first_terminator_idx: 7, terminator_count: 2, _padding: 0 }, // [2254] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2188, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2255] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2188, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2256] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2188, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [2257] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2188, child_count: 0, min_times: 0, first_terminator_idx: 9, terminator_count: 1, _padding: 0 }, // [2258] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2188, child_count: 0, min_times: 0, first_terminator_idx: 9, terminator_count: 0, _padding: 0 }, // [2259] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2188, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [2260] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2188, child_count: 7, min_times: 0, first_terminator_idx: 13, terminator_count: 0, _padding: 0 }, // [2261] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2195, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [2262] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2195, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [2263] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2195, child_count: 1, min_times: 0, first_terminator_idx: 10, terminator_count: 2, _padding: 0 }, // [2264] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [2265] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [2266] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [2267] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 1, _padding: 0 }, // [2268] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2269] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 13, terminator_count: 0, _padding: 0 }, // [2270] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 13, terminator_count: 0, _padding: 0 }, // [2271] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 13, terminator_count: 0, _padding: 0 }, // [2272] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 13, terminator_count: 0, _padding: 0 }, // [2273] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2274] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2275] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2276] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2277] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2278] StringParser("CERTIFICATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2279] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2280] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2281] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2282] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2283] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2284] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2285] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2286] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2287] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2288] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2289] StringParser("CIDR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2290] StringParser("CIRCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2291] StringParser("CIRCULARSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2292] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2293] StringParser("CLIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2294] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2295] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2296] Ref(CloseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2297] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2298] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2299] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2201, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2300] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2201, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2301] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2201, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2302] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2203, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2303] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2304] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2205, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2305] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2306] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2307] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2308] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2309] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2310] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2311] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2312] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2313] StringParser("CLUSTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2314] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2315] Token("raw")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2316] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2317] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2318] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2319] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2212, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2320] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2212, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 1, _padding: 0 }, // [2321] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2322] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2323] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2324] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2325] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2326] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2327] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2328] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2218, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2329] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2218, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2330] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2218, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2331] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2218, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2332] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2218, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2333] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2222, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2335] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2336] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 8, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2337] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2338] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2234, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2339] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2234, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2340] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2234, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2341] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2342] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2343] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2240, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2344] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2240, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2345] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2346] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2347] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2348] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2349] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 5, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2350] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2351] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2352] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2353] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2354] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2355] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 5, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2356] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2357] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2358] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2359] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2360] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2361] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 5, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2362] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2259, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2363] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2259, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2364] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2261, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2365] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2261, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2366] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2367] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2368] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2369] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2370] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2263, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2371] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2266, child_count: 1, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2372] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2373] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2374] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2271, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2375] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2271, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2376] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2274, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2377] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2274, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2378] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2274, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2379] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2274, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2380] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2276, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2381] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2276, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2382] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2276, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2383] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2384] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2385] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2386] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2387] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2388] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2284, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2389] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2284, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2390] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2284, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2391] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2392] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2393] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2286, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2394] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2395] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2396] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2397] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2398] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2290, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2399] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2290, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2400] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2292, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2401] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2292, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2402] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2403] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2404] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2294, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2405] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2296, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2406] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2407] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2408] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2409] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2410] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2411] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2412] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2413] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2414] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2303, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2415] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2305, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2416] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2305, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2417] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2418] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2419] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2420] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2421] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2307, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2422] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2309, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2423] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 2309, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2424] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2425] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2313, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2426] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2313, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2427] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2315, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2428] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2315, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2429] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2315, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 12, _padding: 0 }, // [2430] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2317, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2431] Ref(SingleIdentifierFullGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2317, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2432] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2433] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2434] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2435] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2436] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2437] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2438] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2439] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2440] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2441] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2442] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2443] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2444] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2445] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2446] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2447] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2448] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2449] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2450] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2451] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2452] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2453] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2454] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2455] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2456] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2457] StringParser(",")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2458] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2459] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2460] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2461] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2462] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2329, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2463] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2329, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2464] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2329, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2465] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2331, child_count: 11, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2466] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2342, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2467] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2344, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2468] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2469] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2470] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2346, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2471] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2346, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2472] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2473] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2474] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2475] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2354, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2476] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2354, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2477] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2354, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2478] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2354, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2479] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2480] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2481] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2482] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2483] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2484] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2485] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2486] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2487] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2488] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2489] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2490] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2491] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2492] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2493] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2494] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2366, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2495] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2496] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2497] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2369, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2498] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2499] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2500] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2372, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2501] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2372, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2502] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2372, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2503] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2504] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2505] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2506] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2377, child_count: 24, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2507] OneOf(24 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2508] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2509] Ref(ConversionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2510] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2511] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2512] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2513] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2514] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2515] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2516] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2517] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2518] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2519] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2520] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2521] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2522] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2523] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2403, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2524] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2403, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2525] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2403, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2526] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2405, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2527] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2405, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2528] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2405, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2529] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2530] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2531] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2532] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2408, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2533] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2534] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2535] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2536] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2537] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2538] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2412, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2539] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2540] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2541] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2542] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2543] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2544] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2545] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2546] Ref(DictionaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2547] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2548] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2549] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2550] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2551] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2552] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2553] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2554] Ref(TemplateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2555] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2424, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2556] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2557] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2558] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2559] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2560] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2561] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2562] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2563] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2564] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2432, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2565] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2435, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2566] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2567] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2436, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2568] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2569] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2570] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2571] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2572] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2573] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2574] StringParser("COMMENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2575] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2576] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2577] StringParser("COMPACTION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2440, child_count: 17, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2578] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2579] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2580] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2581] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2582] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2583] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2584] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2585] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2586] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2460, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2587] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2460, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2588] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2460, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2589] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2460, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2590] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2591] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2592] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2593] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2594] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2595] Ref(OverlapSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2596] Ref(NotExtendRightSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2597] Ref(NotExtendLeftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2598] Ref(AdjacentSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2599] Ref(PostgisOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2600] Ref(PgvectorOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2601] Ref(PgTrgmOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2602] Ref(QualifiedOperatorSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2603] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2604] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2605] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2606] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2607] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2608] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2609] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2610] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2611] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2612] StringParser("COMPOUNDCURVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2613] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2614] StringParser("COMPRESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2615] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2616] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2617] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2618] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2619] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2620] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2621] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2622] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2623] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2475, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2624] StringParser("CONFIGURATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2625] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2626] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2477, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2627] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2628] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2629] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2630] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2631] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2632] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2489, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2633] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2634] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2635] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2636] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2637] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2638] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2639] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2640] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2501, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2641] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2642] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2643] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2644] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2645] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2506, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2646] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2509, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2647] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2511, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2648] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2649] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2650] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2651] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2516, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2652] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2519, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2653] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2521, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2654] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2521, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2655] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2521, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2656] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2657] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2524, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2658] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2659] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2660] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2526, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2661] StringParser("CONFLICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2662] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2528, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2663] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2530, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2664] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2533, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2665] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2535, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2666] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2538, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2667] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2668] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2669] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2544, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2670] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2544, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2671] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2544, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2672] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2673] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2674] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2675] Ref(OperationClassReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2546, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2676] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2677] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2678] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2548, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2679] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2680] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2681] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2682] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2683] StringParser("CONFLUENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2684] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2685] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2686] StringParser("CONNECTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2687] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2688] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2689] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2690] StringParser("CONTENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2691] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2692] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2693] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2557, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2694] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2557, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2695] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2557, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2696] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2560, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2697] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2560, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2698] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2560, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2699] Ref(StdinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2560, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2700] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2562, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2701] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2563, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2702] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2563, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2703] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2704] Anything
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2705] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2706] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2568, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2707] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2568, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2708] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2572, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2709] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2577, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2710] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2711] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2579, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2712] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2582, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2713] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2714] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2715] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2716] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2586, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2717] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2718] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2589, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2719] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2720] Ref(ProgramKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2721] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2722] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2723] Ref(PsqlVariableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2724] Ref(PsqlVariableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2725] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2594, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2726] Ref(StdinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2594, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2727] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2596, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2728] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2596, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2729] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2599, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2730] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2601, child_count: 11, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2731] AnySetOf(11 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2612, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2732] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2614, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2733] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2614, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2734] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2614, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2735] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2736] Ref(FreezeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2737] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2738] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2739] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2740] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2741] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2742] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2743] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2744] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2622, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2745] Ref(HeaderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2622, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2746] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2622, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2747] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2748] Ref(QuoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2749] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2750] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2626, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2751] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2626, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2752] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2626, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2753] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2754] Ref(Force_quoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2628, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2755] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2630, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2756] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2757] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2758] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2759] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2635, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2760] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2637, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2761] Ref(Force_not_nullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2637, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2762] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2640, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2763] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2764] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2765] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2644, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2766] Ref(Force_nullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2644, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2767] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2647, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2768] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2649, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2769] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2649, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2770] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2771] Ref(EncodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2772] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2651, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2773] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2653, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2774] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2653, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2775] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2653, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2776] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2657, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2777] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2657, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2778] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2779] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2780] Ref(PsqlVariableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2781] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2661, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2782] Ref(StdinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2661, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2783] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2663, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2784] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2663, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2785] AnySetOf(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2667, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2786] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2668, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2787] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2668, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2788] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2789] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2790] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2791] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2671, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2792] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2674, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2793] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2674, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2794] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2674, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2795] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2674, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2796] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2676, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2797] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2676, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2798] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2799] Ref(HeaderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2800] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2683, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2801] Ref(QuoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2683, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2802] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2683, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2803] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2683, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2804] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2686, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2805] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2686, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2806] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2686, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2807] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2686, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2808] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2690, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2809] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2690, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2810] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2690, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2811] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2690, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2812] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2692, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2813] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2692, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2814] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2696, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2815] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2698, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2816] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2701, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2817] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2701, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2818] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2701, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2819] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2820] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2704, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2821] Ref(StdoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2704, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2822] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2708, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2823] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2710, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2824] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2713, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2825] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2713, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2826] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2713, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2827] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2828] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2829] Ref(PsqlVariableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2830] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2717, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2831] Ref(StdoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2717, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2832] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2719, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2833] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2719, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2834] AnySetOf(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2723, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2835] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2836] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2837] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2727, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2838] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2727, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2839] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2727, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2840] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2727, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2841] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2730, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2842] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2730, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2843] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2730, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2844] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2730, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2845] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2732, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2846] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2732, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2847] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2848] Ref(HeaderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2736, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2849] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2850] Ref(QuoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2851] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2852] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2853] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2742, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2854] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2742, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2855] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2742, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2856] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2742, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2857] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2858] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2859] Ref(QuoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2860] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2747, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2861] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2750, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2862] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2752, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2863] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2752, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2864] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2752, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2865] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2757, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2866] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2757, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2867] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2760, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2868] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2763, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2869] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2763, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2870] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2765, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2871] Ref(SubscribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2765, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2872] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2765, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2873] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2767, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2874] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2767, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2875] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2769, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2876] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2769, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2877] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2769, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2878] Ref(StdoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2769, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2879] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2771, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2880] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2771, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2881] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2774, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2882] Anything
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2774, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2883] StringParser("CORRELATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2774, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2884] StringParser("COST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2774, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2885] StringParser("COSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2774, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2886] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2774, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2887] StringParser("COUNTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2774, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2888] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2780, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2889] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2780, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2890] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2780, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2891] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2780, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2892] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2780, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2893] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2783, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2894] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2783, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2895] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2783, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2896] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2897] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2898] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2899] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2793, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2900] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2793, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2901] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2793, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2902] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2793, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2903] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2796, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2904] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2800, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2905] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2800, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2906] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2800, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2907] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2800, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2908] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2800, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2909] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2910] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2911] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2912] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2804, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2913] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2804, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2914] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2804, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2915] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2806, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2916] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2808, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2917] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2808, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2918] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2808, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2919] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2920] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2921] Ref(ImplicitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2922] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2923] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2924] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2925] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2926] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2815, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2927] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2816, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2928] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2816, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2929] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2930] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2931] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2932] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2820, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2933] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2822, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2934] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2935] Ref(ReplicasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2936] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2827, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2937] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2829, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2938] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2829, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2939] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2830, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2940] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2941] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2942] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2943] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2944] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2945] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2946] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2837, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2947] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2840, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2948] Delimited(6 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2842, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2949] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2848, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2950] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2951] Ref(LocaleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2952] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2953] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2954] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2955] Ref(Lc_collateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2956] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2957] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2958] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2857, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2959] Ref(Lc_ctypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2857, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2960] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2857, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2961] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2857, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2962] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2860, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2963] Ref(ProviderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2860, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2964] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2860, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2965] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2966] Ref(IcuKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2967] Ref(LibcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2968] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2865, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2969] Ref(DeterministicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2865, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2970] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2865, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2971] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2865, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2972] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2973] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2974] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2975] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2976] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2870, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2977] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2870, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2978] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2870, child_count: 7, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2979] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2877, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2980] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2877, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2981] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2877, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2982] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2877, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2983] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2877, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2984] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2877, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2985] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2882, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2986] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2884, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2987] Ref(AwsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2884, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2988] Ref(PrivatelinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2884, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2989] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2990] Ref(ConfluentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2991] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2992] Ref(RegistryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2993] Ref(KafkaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2994] Ref(PostgresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2995] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2889, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2996] Ref(SshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2889, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2997] Ref(TunnelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2889, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2998] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2892, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2999] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2892, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3000] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3001] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3002] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3003] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3004] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2897, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3005] AnyNumberOf(8 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2905, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3006] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2908, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3007] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2908, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3008] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2908, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3009] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2908, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3010] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2911, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3011] Ref(TemplateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2911, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3012] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2911, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3013] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2911, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3014] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2914, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3015] Ref(EncodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2914, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3016] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2914, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3017] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3018] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3019] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2916, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3020] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3021] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2921, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3022] Ref(LocaleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2921, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3023] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2921, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3024] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2921, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3025] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2923, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3026] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2926, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3027] Ref(Lc_collateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2926, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3028] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2926, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3029] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2926, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3030] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3031] Ref(Lc_ctypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3032] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3033] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3034] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2932, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3035] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2932, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3036] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2932, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3037] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2934, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3038] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2934, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3039] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2934, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3040] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2937, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3041] Ref(Allow_connectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2937, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3042] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2937, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3043] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2937, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3044] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3045] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3046] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3047] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3048] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3049] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2944, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3050] Ref(Is_templateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2944, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3051] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2944, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3052] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2944, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3053] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3054] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3055] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3056] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2952, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3057] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2953, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3058] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2953, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3059] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2953, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3060] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3061] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3062] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2955, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3063] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3064] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3065] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2957, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3066] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2958, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3067] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2960, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3068] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3069] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3070] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2962, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3071] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2964, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3072] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2966, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3073] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3074] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3075] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3076] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3077] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3078] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3079] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3080] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3081] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3082] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3083] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2977, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3084] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2979, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3085] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2979, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3086] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2979, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3087] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2981, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3088] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2981, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3089] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2981, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3090] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2983, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3091] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2983, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3092] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2983, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3093] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2983, child_count: 11, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3094] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2994, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3095] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2994, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3096] Ref(ForeignDataWrapperGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2994, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3097] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2994, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3098] Ref(HandlerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2994, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3099] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2994, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3100] Ref(ValidatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2994, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3101] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2994, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3102] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3103] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3104] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3105] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3106] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2999, child_count: 7, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3107] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3006, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3108] Ref(CreateForeignTableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3006, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3109] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3006, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3110] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3006, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3111] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3009, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3112] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3011, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3113] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3013, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3114] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3018, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3115] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3018, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3116] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3018, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3117] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3018, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3118] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3020, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3119] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3020, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3120] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3020, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3121] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3021, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3122] Ref(ForeignTableColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3021, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3123] Ref(ForeignTableTableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3021, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3124] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3023, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3125] Ref(InheritsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3023, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3126] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3026, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3127] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3028, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3128] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3028, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3129] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3030, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3130] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3030, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3131] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3030, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3132] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3030, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3133] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3134] Ref(CreateForeignTableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3135] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3136] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3137] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3138] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3139] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3140] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3041, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3141] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3142] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3046, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3143] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3048, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3144] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3051, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3145] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3051, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3146] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3147] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3148] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3053, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3149] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3054, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3150] Ref(ForeignTableColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3054, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3151] Ref(ForeignTableTableConstraintSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3054, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3152] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3056, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3153] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3059, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3154] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3059, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3155] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3059, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3156] Ref(PartitionBoundSpecSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3059, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3157] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3059, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3158] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3159] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3160] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3161] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3162] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3163] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3164] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3165] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3166] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3167] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3168] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3169] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3070, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3170] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3171] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3172] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3075, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3173] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3077, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3174] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3077, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3175] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3080, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3176] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3082, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3177] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3084, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3178] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3084, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3179] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3086, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3180] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3086, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3181] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3088, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3182] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3088, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3183] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3088, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3184] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3185] Ref(SetofKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3186] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3187] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3188] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3090, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3189] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3092, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3190] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3092, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3191] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3094, child_count: 7, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3192] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3101, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3193] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3101, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3194] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3101, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3195] Ref(InCluster)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3101, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3196] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3101, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3197] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3101, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3198] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3103, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3199] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3103, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3200] Anything
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3103, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3201] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3106, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3202] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3108, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3203] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3108, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3204] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3205] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3206] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3207] Ref(InCluster)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3208] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3209] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3114, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3210] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3211] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3212] Anything
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3213] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3214] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3118, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3215] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3118, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3216] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3120, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3217] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3128, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3218] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3128, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3219] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3128, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3220] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3128, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3221] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3128, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3222] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3131, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3223] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3224] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3225] Ref(InCluster)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3226] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3227] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3228] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3141, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3229] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3141, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3230] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3141, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3231] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3141, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3232] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3141, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3233] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3144, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3234] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3146, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3235] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3146, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3236] Ref(InCluster)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3146, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3237] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3146, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3238] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3146, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3239] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3154, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3240] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3154, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3241] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3154, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3242] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3154, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3243] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3154, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3244] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3154, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3245] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3156, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3246] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3156, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3247] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3248] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3161, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3249] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3250] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3251] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3164, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3252] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3166, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3253] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3166, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3254] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3169, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3255] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3171, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3256] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3171, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3257] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3171, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3258] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3171, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3259] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3175, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3260] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3175, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3261] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3175, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3262] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3176, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3263] RegexParser
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3176, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3264] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3265] Delimited(6 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3181, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3266] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3187, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3267] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3190, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3268] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3192, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3269] Ref(LeftargKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3192, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3270] Ref(RightargKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3192, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3271] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3192, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3272] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3192, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3273] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3195, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3274] Ref(CommutatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3195, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3275] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3195, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3276] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3196, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3277] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3196, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3278] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3199, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3279] Ref(NegatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3199, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3280] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3199, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3281] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3200, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3282] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3200, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3283] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3203, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3284] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3285] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3286] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3287] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3208, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3288] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3208, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3289] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3208, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3290] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3208, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3291] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3208, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3292] Ref(HashesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3208, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3293] Ref(MergesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3208, child_count: 10, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3294] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3218, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3295] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3218, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3296] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3218, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3297] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3218, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3298] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3218, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3299] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3218, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3300] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3220, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3301] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3220, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3302] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3303] Ref(PermissiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3304] Ref(RestrictiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3222, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3305] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3224, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3306] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3224, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3307] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3229, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3308] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3229, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3309] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3229, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3310] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3229, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3311] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3229, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3312] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3229, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3313] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3231, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3314] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3231, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3315] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3233, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3316] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3238, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3317] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3238, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3318] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3238, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3319] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3238, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3320] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3238, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3321] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3238, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3322] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3240, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3323] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3240, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3324] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3243, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3325] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3243, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3326] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3246, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3327] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3246, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3328] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3246, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3329] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3249, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3330] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3249, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3331] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3332] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3255, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3333] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3334] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3335] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3336] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3337] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3338] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3339] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3340] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3341] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3260, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3342] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3262, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3343] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3265, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3344] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3265, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3345] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3265, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3346] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3265, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3347] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3267, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3348] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3267, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3349] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3350] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3269, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3351] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3271, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3352] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3271, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3353] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3271, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3354] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3355] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3356] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3357] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3358] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3359] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3277, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3360] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3279, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3361] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3279, child_count: 16, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3362] AnySetOf(16 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3295, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3363] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3297, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3364] Ref(SuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3297, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3365] Ref(NosuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3297, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3366] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3367] Ref(CreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3368] Ref(NocreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3299, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3369] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3301, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3370] Ref(CreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3301, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3371] Ref(NocreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3301, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3372] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3373] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3374] Ref(NoinheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3375] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3305, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3376] Ref(LoginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3305, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3377] Ref(NologinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3305, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3378] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3307, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3379] Ref(ReplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3307, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3380] Ref(NoreplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3307, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3381] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3309, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3382] Ref(BypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3309, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3383] Ref(NobypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3309, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3384] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3312, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3385] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3312, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3386] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3312, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3387] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3312, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3388] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3389] Ref(EncryptedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3390] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3391] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3317, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3392] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3317, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3393] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3317, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3394] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3395] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3396] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3397] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3320, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3398] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3323, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3399] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3323, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3400] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3323, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3401] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3323, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3402] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3403] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3404] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3405] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3406] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3328, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3407] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3328, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3408] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3328, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3409] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3410] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3411] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3412] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3413] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3414] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3415] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3416] Ref(SysidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3417] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3418] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3338, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3419] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3338, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3420] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3338, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3421] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3338, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3422] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3423] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3343, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3424] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3343, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3425] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3343, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3426] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3343, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3427] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3343, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3428] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3429] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3430] Ref(SecretKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3431] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3432] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3433] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3434] Anything
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3435] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3357, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3436] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3359, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3437] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3359, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3438] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3359, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3439] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3363, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3440] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3363, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3441] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3363, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3442] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3363, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3443] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3363, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3444] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3365, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3445] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3368, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3446] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3368, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3447] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3368, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3448] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3368, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3449] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3370, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3450] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3370, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3451] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3370, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3452] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3372, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3453] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3375, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3454] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3375, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3455] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3375, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3456] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3375, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3457] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3458] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3459] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3377, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3460] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3461] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3462] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3463] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3464] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3381, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3465] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3383, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3466] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3383, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3467] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3383, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3468] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3385, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3469] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3385, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3470] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3387, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3471] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3387, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3472] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3387, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3473] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3390, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3474] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3390, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3475] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3390, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3476] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3477] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3478] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3392, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3479] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3480] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3481] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3482] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3483] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3484] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3398, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3485] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3399, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3486] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3399, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3487] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3488] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3489] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3490] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3491] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3408, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3492] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3410, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3493] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3410, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3494] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3410, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3495] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3496] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3497] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3498] Ref(ForeignDataWrapperGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3499] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3500] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 13, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3501] Sequence(13 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3502] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3503] Ref(SinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3504] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3505] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3506] Ref(InCluster)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3507] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3508] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3509] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3510] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3425, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3511] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3427, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3512] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3427, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3513] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3430, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3514] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3432, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3515] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3432, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3516] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3434, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3517] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3434, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3518] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3434, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3519] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3436, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3520] Ref(EnvelopeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3436, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3521] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3438, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3522] Ref(DebeziumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3438, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3523] Ref(UpsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3438, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3524] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3525] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3440, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3526] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3443, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3527] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3445, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3528] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3445, child_count: 16, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3529] Sequence(16 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3461, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3530] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3461, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3531] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3461, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3532] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3461, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3533] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3461, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3534] Ref(InCluster)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3461, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3535] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3464, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3536] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3537] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3538] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3539] Ref(KafkaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3540] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3541] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3542] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3469, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3543] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3471, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3544] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3471, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3545] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3546] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3547] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3548] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3549] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3550] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3551] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3477, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3552] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3553] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3554] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3479, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3555] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3481, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3556] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3481, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3557] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3483, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3558] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3483, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3559] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3485, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3560] Ref(EnvelopeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3485, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3561] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3488, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3562] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3488, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3563] Ref(DebeziumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3488, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3564] Ref(UpsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3488, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3565] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3490, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3566] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3490, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3567] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3493, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3568] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3495, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3569] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3495, child_count: 10, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3570] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3571] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3572] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3573] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3574] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3575] Ref(InCluster)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3576] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3508, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3577] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3508, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3578] Ref(LoadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3508, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3579] Ref(GeneratorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3508, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3580] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3581] Ref(AuctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3582] Ref(CounterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3583] Ref(MarketingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3584] Ref(TpchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3512, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3585] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3515, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3586] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3517, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3587] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3517, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3588] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3589] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3590] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3591] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3520, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3592] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3522, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3593] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3522, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3594] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3595] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3527, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3596] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3527, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3597] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3535, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3598] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3535, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3599] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3535, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3600] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3535, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3601] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3535, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3602] Ref(InCluster)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3535, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3603] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3604] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3605] Ref(PostgresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3606] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3607] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3540, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3608] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3543, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3609] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3545, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3610] Anything
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3545, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3611] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3547, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3612] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3550, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3613] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3550, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3614] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3550, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3615] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3550, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3616] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3617] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3553, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3618] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3553, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3619] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3556, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3620] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3558, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3621] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3558, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3622] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3560, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3623] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3560, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3624] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3563, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3625] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3565, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3626] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3565, child_count: 10, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3627] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3575, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3628] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3575, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3629] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3575, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3630] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3575, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3631] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3575, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3632] Ref(InCluster)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3575, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3633] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3579, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3634] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3579, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3635] Ref(WebhookKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3579, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3636] Ref(BodyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3579, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3637] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3579, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3638] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3582, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3639] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3582, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3640] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3582, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3641] Ref(BytesKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3582, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3642] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3584, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3643] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3587, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3644] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3587, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3645] Ref(HeaderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3587, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3646] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3588, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3647] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3588, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3648] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3591, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3649] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3591, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3650] Ref(HeadersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3591, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3651] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3594, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3652] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3596, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3653] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3596, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3654] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3598, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3655] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3598, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3656] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3602, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3657] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3602, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3658] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3605, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3659] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3660] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3607, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3661] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3608, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3662] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3608, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3663] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3664] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3665] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3616, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3666] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3618, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3667] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3618, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3668] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3618, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3669] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3621, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3670] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3623, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3671] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3627, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3672] Ref(DependenciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3627, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3673] Ref(McvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3627, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3674] Ref(NdistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3627, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3675] Ref(CorrelationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3627, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3676] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3627, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3677] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3629, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3678] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3679] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3680] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3681] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3682] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3631, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3683] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3684] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3685] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3686] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3687] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3688] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3689] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3690] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3691] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3641, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3692] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3693] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3643, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3694] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3643, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3695] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3652, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3696] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3652, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3697] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3654, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3698] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3656, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3699] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3700] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3701] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3702] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3703] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3704] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3705] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3706] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3658, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3707] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3663, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3708] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3666, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3709] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3668, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3710] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3668, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3711] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3670, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3712] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3670, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3713] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3670, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3714] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3672, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3715] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3674, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3716] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3674, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3717] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3677, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3718] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3679, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3719] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3681, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3720] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3681, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3721] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3683, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3722] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3683, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3723] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3685, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3724] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3685, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3725] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3685, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3726] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3687, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3727] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3687, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3728] Ref(OidsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3687, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3729] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3690, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3730] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3690, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3731] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3690, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3732] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3693, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3733] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3734] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3735] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3736] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3697, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3737] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3697, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3738] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3697, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3739] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3697, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3740] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3741] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3742] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3743] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3744] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3703, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3745] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3705, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3746] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3708, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3747] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3708, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3748] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3710, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3749] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3713, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3750] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3715, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3751] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3715, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3752] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3715, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3753] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3715, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3754] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3717, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3755] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3720, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3756] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3757] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3758] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3722, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3759] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 7, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3760] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3729, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3761] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3729, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3762] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3731, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3763] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3733, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3764] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3765] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3766] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3767] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3768] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3769] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3770] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3735, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3771] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3735, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3772] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3738, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3773] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3740, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3774] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3743, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3775] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3745, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3776] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3748, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3777] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3748, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3778] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3748, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3779] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3751, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3780] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3751, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3781] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3751, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3782] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3752, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3783] Ref(LikeOptionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3752, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3784] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3754, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3785] Ref(InheritsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3754, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3786] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3757, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3787] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3788] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3789] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3762, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3790] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3762, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3791] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3762, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3792] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3765, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3793] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3767, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3794] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3769, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3795] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3772, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3796] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3772, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3797] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3774, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3798] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3774, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3799] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3774, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3800] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3775, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3801] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3775, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3802] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3775, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3803] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3780, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3804] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3780, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3805] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3780, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3806] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3780, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3807] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3783, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3808] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3785, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3809] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3787, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3810] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3790, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3811] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3790, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3812] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3792, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3813] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3792, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3814] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3792, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3815] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3793, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3816] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3793, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3817] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3793, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3818] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3795, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3819] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3798, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3820] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3798, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3821] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3798, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3822] Ref(PartitionBoundSpecSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3798, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3823] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3798, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3824] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3825] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3807, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3826] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3807, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3827] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3807, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3828] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3810, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3829] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3810, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3830] Ref(ListKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3810, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3831] Ref(HashKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3810, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3832] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3813, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3833] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3814, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3834] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3816, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3835] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3818, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3836] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3820, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3837] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3820, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3838] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3820, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3839] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3822, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3840] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3824, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3841] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3824, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3842] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3824, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3843] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3824, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3844] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3826, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3845] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3826, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3846] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3826, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3847] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3828, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3848] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3830, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3849] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3830, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3850] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3830, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3851] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3832, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3852] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3832, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3853] Ref(OidsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3832, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3854] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3835, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3855] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3835, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3856] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3835, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3857] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3838, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3858] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3840, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3859] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3840, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3860] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3840, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3861] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3842, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3862] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3842, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3863] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3842, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3864] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3842, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3865] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3844, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3866] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3844, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3867] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3844, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3868] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3850, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3869] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3850, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3870] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3850, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3871] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3850, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3872] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3850, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3873] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3850, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3874] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3853, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3875] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3855, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3876] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3858, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3877] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3858, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3878] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3858, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3879] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3858, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3880] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3881] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3882] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3883] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 11, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3884] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3872, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3885] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3872, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3886] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3872, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3887] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3872, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3888] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3872, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3889] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3872, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3890] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3875, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3891] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3875, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3892] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3875, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3877, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3894] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3877, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3895] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3877, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3896] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3879, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3897] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3883, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3898] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3883, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3899] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3883, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3900] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3883, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3901] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3885, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3902] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3885, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3903] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3887, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3904] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3887, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 2, _padding: 0 }, // [3905] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3889, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3906] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3889, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3907] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3889, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [3908] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3889, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3909] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3889, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3910] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3889, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3911] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3889, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3912] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3894, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3913] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3914] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3896, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3915] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3896, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3916] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3898, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3917] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3900, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3918] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3900, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3919] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3900, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3920] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3902, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3921] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3902, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3922] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3904, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3923] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3906, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3924] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3906, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3925] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3906, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3926] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3927] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3928] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3908, child_count: 6, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3929] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3914, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3930] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3914, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3931] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3932] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3933] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3934] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3935] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3916, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3936] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3916, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3937] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3920, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3938] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3922, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3939] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3922, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3940] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3922, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3941] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3922, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3942] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3922, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3943] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3922, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3944] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3925, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3945] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3925, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3946] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3925, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3947] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3927, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3948] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3927, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3949] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3927, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3950] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3929, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3951] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3929, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3952] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3932, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3953] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3932, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3954] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3935, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3955] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3935, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3956] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3937, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3957] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3937, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3958] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3937, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3959] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3937, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3960] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3961] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3962] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3963] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3964] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3943, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3965] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3945, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3966] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3945, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3967] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3948, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3968] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3950, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3969] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3970] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3971] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3952, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3972] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3955, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3973] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3955, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3974] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3957, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3975] Ref(ListKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3957, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3976] Ref(MapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3957, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3977] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3978] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3962, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3979] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3980] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3981] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3965, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3982] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3965, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3983] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3968, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3984] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3968, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3985] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3968, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3986] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3968, child_count: 7, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3987] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3975, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3988] Ref(CreateUserMappingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3975, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3989] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3975, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3990] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3975, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3991] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3992] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3993] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3994] Ref(SessionInformationUserFunctionsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3995] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3996] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3997] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3998] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3981, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3999] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3981, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4000] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3981, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4001] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3981, child_count: 8, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4002] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4003] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3989, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4004] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3991, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4005] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3991, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4006] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3991, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4007] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3991, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4008] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3991, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4009] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3991, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4010] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3994, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4011] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4012] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4013] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4014] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4015] StringParser("CREATECLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4016] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4017] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4018] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4019] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4020] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4021] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4022] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4023] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3998, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4024] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4000, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4025] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4000, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4026] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4000, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4027] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4028] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4029] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4030] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4031] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4032] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4033] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4034] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4035] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4036] StringParser("CURVEPOLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4037] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4038] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4039] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4003, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 1, _padding: 0 }, // [4040] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4005, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4041] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4005, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4042] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4005, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [4043] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4005, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4044] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4005, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4045] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4007, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4046] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4007, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4047] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4048] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4008, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4049] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4010, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4050] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 4013, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4051] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4015, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4052] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4015, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4053] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4015, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4054] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4055] Ref(WellKnownTextGeometrySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4056] Ref(DateTimeTypeIdentifier)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4057] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4058] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4021, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4059] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4022, child_count: 38, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4060] OneOf(38 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4061] Ref(SmallintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4062] Ref(IntegerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4063] Ref(IntKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4064] Ref(Int2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4065] Ref(Int4KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4066] Ref(Int8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4067] Ref(BigintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4068] Ref(Float4KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4069] Ref(Float8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4070] Ref(RealKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4071] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4062, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4072] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4062, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4073] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4062, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4074] Ref(SmallserialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4062, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4075] Ref(SerialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4062, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4076] Ref(Serial2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4062, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4077] Ref(Serial4KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4062, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4078] Ref(Serial8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4062, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4079] Ref(BigserialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4062, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4080] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4064, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4081] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4065, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4082] Ref(FloatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4065, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4083] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4065, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4084] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4067, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4085] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4086] Ref(DecimalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4087] Ref(NumericKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4088] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4089] Ref(MoneyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4069, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4090] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4071, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4091] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4073, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4092] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4093] Ref(BpcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4094] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4095] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4081, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4096] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4081, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4097] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4081, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4098] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4081, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4099] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4083, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4100] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4083, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4101] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4083, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4102] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4083, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4103] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4083, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4104] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4083, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4105] Ref(ByteaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4083, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4106] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4085, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4107] Ref(BooleanKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4085, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4108] Ref(BoolKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4085, child_count: 7, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4109] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4110] Ref(PointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4111] Ref(LineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4112] Ref(LsegKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4113] Ref(BoxKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4114] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4115] Ref(PolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4116] Ref(CircleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4117] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4118] Ref(CidrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4119] Ref(InetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4120] Ref(MacaddrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4121] Ref(Macaddr8KeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4096, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4122] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4098, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4123] Ref(TsvectorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4098, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4124] Ref(TsqueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4098, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4125] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4101, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4126] Ref(BitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4101, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4127] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4128] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4129] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4130] Ref(UuidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4131] Ref(XmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4132] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4133] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4134] Ref(JsonbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4135] Ref(Int4rangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4136] Ref(Int8rangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4137] Ref(NumrangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4138] Ref(TsrangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4139] Ref(TstzrangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4140] Ref(DaterangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4141] Ref(Pg_lsnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4104, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4142] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4143] Ref(VectorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4144] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4145] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4108, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4146] Ref(DatatypeIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4108, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4147] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4108, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4148] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4111, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4149] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4112, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4150] Ref(ArrayTypeSuffixSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4112, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4151] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4112, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4152] Ref(SizedArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4112, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4153] Ref(ListKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4112, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4154] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4112, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4155] MultiStringParser(0 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4112, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4156] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4112, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4157] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4113, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4158] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4116, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4159] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4118, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4160] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4120, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4161] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4120, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4162] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4120, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4163] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4164] Ref(DateTimeTypeIdentifier)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4165] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4122, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4166] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4125, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4167] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4125, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4168] Ref(TimeWithTZGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4125, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4169] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4127, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4170] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4130, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4171] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4130, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4172] Ref(TimetzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4130, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4173] Ref(TimestamptzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4130, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4174] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4175] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4176] StringParser("DATERANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4177] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4178] MultiStringParser(24 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4179] StringParser("DATUMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4180] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4181] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4182] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4183] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4184] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4185] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4186] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4187] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4188] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4189] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4190] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4133, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4191] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4136, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4192] Ref(DeallocateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4136, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4193] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4136, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4194] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4195] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4196] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4197] StringParser("DEBEZIUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4198] StringParser("DEBUG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4199] StringParser("DEBUGGING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4200] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4201] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4202] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4138, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4203] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4144, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4204] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4144, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4205] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4144, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4206] Ref(CursorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4144, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4207] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4146, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4208] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4146, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4209] Ref(HoldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4146, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4210] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4146, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4211] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4150, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4212] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4150, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4213] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4152, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4214] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4152, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4215] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4152, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4216] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4155, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4217] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4155, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4218] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4155, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4219] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4155, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4220] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4156, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4221] Anything
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4156, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4222] StringParser("DECORRELATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4156, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4223] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4156, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4224] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4158, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4225] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4158, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4226] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4158, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4227] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4158, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4228] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4158, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4229] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4158, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4230] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4158, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4231] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4160, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4232] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4160, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4233] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4160, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4234] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4162, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4235] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4162, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4236] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4164, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4237] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4164, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4238] Ref(DefinitionArgumentValueGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4164, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4239] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4167, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4240] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4169, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4241] Ref(DefinitionParameterSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4169, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4242] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4169, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4243] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4169, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4244] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4169, child_count: 10, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4245] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4179, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4246] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4179, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4247] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4179, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4248] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4179, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4249] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4179, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4250] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4179, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4251] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4179, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4252] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4183, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4253] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4183, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4254] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4185, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4255] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4187, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4256] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4187, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4257] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4187, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4258] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4187, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4259] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4189, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4260] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4261] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4262] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4263] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4264] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4265] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4193, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4266] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4197, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4267] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4197, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4268] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4199, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4269] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4199, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4270] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4201, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4271] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4272] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4273] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4274] StringParser("DELIMITED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4275] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4276] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4277] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4278] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4279] StringParser("DEPENDENCIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4280] StringParser("DEPENDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4281] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4282] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4283] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4203, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4284] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4285] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4286] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4287] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4288] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4289] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4290] StringParser("DETACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4291] StringParser("DETAILS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4292] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4293] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4294] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4295] StringParser("DISABLE_PAGE_SKIPPING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4296] StringParser("DISCARD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4206, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4297] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4298] Ref(DiscardKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4208, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4299] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4300] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4301] Ref(PlansKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4302] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4303] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4304] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4305] StringParser("DISCOVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4306] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4307] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4308] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4309] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4310] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4311] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4213, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4312] StringParser("DO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4213, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4313] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4215, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4314] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4215, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4315] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4217, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4316] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4219, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4317] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4219, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4318] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4219, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4319] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4221, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4320] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4221, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4321] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4221, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4322] StringParser("DOCUMENT")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4221, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4323] TypedParser("dollar_numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4221, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4324] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4221, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4325] StringParser("DOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4221, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4326] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4221, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4327] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4221, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4328] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4226, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4329] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4226, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4330] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4226, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4331] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4226, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4332] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4228, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4333] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4230, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4334] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4230, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4335] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4233, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4336] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4233, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4337] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4233, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4338] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4233, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4339] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4233, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4340] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4235, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4341] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4235, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4342] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4235, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4343] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4240, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4344] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4240, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4345] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4240, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4346] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4240, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4347] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4245, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4348] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4245, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4349] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4245, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4350] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4245, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4351] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4245, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4352] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4353] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4354] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4355] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4356] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4250, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4357] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4250, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4358] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4255, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4359] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4255, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4360] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4255, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4361] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4255, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4362] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4255, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4363] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4257, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4364] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4257, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4365] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4260, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4366] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4260, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4367] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4265, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4368] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4265, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4369] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4265, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4370] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4265, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4371] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4267, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4372] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4267, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4373] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4267, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4374] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4272, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4375] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4272, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4376] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4272, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4377] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4272, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4378] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4272, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4379] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4272, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4380] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4278, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4381] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4278, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4382] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4278, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4383] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4278, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4384] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4278, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4385] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4280, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4386] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4280, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4387] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4280, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4388] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4285, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4389] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4285, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4390] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4285, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4391] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4285, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4392] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4287, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4393] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4289, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4394] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4289, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4395] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4289, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4396] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4289, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4397] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4295, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4398] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4295, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4399] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4295, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4400] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4295, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4401] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4295, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4402] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4297, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4403] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4297, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4404] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4297, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4405] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4297, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4406] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4303, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4407] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4303, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4408] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4303, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4409] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4303, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4410] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4303, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4411] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4305, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4412] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4305, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4413] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4305, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4414] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4309, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4415] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4309, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4416] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4309, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4417] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4309, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4418] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4309, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4419] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4314, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4420] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4314, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4421] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4314, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4422] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4314, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4423] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4316, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4424] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4320, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4425] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4320, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4426] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4320, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4427] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4320, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4428] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4320, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4429] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4320, child_count: 7, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4430] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4327, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4431] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4327, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4432] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4327, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4433] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4327, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4434] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4327, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4435] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4327, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4436] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4327, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4437] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4327, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4438] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4332, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4439] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4332, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4440] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4332, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4441] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4332, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4442] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4334, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4443] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4336, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4444] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4336, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4445] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4336, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4446] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4338, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4447] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4338, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4448] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4338, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4449] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4343, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4450] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4343, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4451] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4343, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4452] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4343, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4453] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4345, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4454] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4345, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4455] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4345, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4456] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4349, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4457] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4349, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4458] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4349, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4459] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4349, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4460] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4349, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4461] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4354, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4462] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4354, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4463] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4354, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4464] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4354, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4465] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4354, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4466] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4354, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4467] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4359, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4468] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4359, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4469] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4359, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4470] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4359, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4471] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4361, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4472] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4361, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4473] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4361, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4474] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4366, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4475] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4366, child_count: 15, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4476] OneOf(15 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4381, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4477] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4381, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4478] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4381, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4479] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4383, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4480] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4383, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4481] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4383, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4482] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4383, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4483] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4383, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4484] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4485] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4486] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4487] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4488] Ref(SecretKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4489] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4490] Ref(SinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4491] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4492] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4493] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4494] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4495] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4496] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4385, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4497] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4385, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4498] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4387, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4499] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4388, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4500] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4388, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4501] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4389, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4502] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4389, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4503] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4394, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4504] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4394, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4505] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4394, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4506] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4394, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4507] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4396, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4508] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4396, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4509] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4398, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4510] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4398, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4511] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4398, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4512] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4403, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4513] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4403, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4514] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4403, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4515] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4403, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4516] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4403, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4517] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4405, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4518] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4405, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4519] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4405, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4520] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4411, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4521] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4411, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4522] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4411, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4523] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4411, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4524] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4411, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4525] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4413, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4526] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4413, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4527] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4413, child_count: 7, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4528] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4529] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4530] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4531] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4532] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4420, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4533] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4534] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4420, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4535] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 7, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4536] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4427, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4537] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4427, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4538] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4427, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4539] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4427, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4540] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4427, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4541] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4427, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4542] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4427, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4543] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4427, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4544] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4545] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4546] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4432, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4547] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4432, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4548] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4434, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4549] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4434, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4550] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4434, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4551] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4438, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4552] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4438, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4553] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4438, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4554] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4438, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4555] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4438, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4556] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4443, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4557] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4443, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4558] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4443, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4559] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4443, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4560] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4561] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4562] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4563] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4564] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4565] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4566] StringParser("EFFORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4567] StringParser("ELEMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4445, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4568] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4449, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4569] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4449, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4570] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4449, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4571] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4449, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4572] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4449, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4573] StringParser("EMPTY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4449, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4574] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4451, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4575] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4576] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4577] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4578] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4579] StringParser("ENABLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4580] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4581] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4582] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4583] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4584] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4585] StringParser("ENDPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4586] StringParser("ENFORCED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4587] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4588] StringParser("ENVELOPE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4589] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4590] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4591] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4592] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4593] StringParser("EVENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4594] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4595] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4596] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4453, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4597] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4456, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4598] Ref(IndexElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4456, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4599] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4456, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4600] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4456, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4601] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4456, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4602] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4456, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4603] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4459, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4604] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4459, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4605] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4459, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4606] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4462, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4607] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4608] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4609] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4610] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4611] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4612] StringParser("EXPECTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4464, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4613] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4464, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4614] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4466, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4615] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4468, child_count: 9, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4616] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4617] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4618] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4619] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4620] Ref(CostsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4621] Ref(SettingsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4622] Ref(BuffersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4623] Ref(WalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4624] Ref(TimingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4625] Ref(SummaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4626] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4627] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4479, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4628] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4479, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4629] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4483, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4630] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4483, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4631] Ref(XmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4483, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4632] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4483, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4633] Ref(YamlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4483, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4634] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4486, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4635] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4486, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4636] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4488, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4637] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4490, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4638] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4492, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4639] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4492, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4640] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4492, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4641] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4492, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4642] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4495, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4643] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4497, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4644] Ref(ExplainOptionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4497, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4645] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4501, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4646] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4501, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4647] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4501, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4648] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4501, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4649] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4501, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4650] StringParser("EXPRESSION")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4501, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4651] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4501, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4652] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4503, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4653] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4503, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4654] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4504, child_count: 10, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4655] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4514, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4656] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4514, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4657] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4516, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4658] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4516, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4659] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4516, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4660] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4516, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4661] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4516, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4662] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4519, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4663] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4519, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4664] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4519, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4665] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4519, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4666] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4519, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4667] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4519, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4668] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4519, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4669] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4524, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4670] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4524, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4671] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4524, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4672] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4524, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4673] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4524, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4674] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4524, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4675] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4526, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4676] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4526, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4677] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4526, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4678] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4530, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4679] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4531, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4680] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4681] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4682] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4683] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4684] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4685] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4534, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4686] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4534, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4687] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4535, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4688] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4536, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4689] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4538, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4690] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4541, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4691] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4541, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4692] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4541, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4693] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4541, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4694] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4541, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4695] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4543, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4696] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4544, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4697] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4545, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4698] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4545, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4699] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4545, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4700] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4547, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4701] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4547, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4702] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4549, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 1, _padding: 0 }, // [4703] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4552, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4704] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4554, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4705] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4554, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4706] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4557, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4707] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4557, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4708] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4559, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4709] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4561, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4710] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4561, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4711] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4561, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4712] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4713] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4714] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4562, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [4715] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4562, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4716] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4564, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4717] Ref(FullTextSearchOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4564, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4718] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4564, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4719] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4566, child_count: 9, min_times: 0, first_terminator_idx: 33, terminator_count: 1, _padding: 0 }, // [4720] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4575, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4721] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4575, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4722] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 4575, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4723] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4578, child_count: 4, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4724] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4582, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4725] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4582, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4726] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4582, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4727] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4582, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4728] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4584, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4729] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4730] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4731] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4732] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4733] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4734] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4735] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4736] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4589, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4737] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4591, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4738] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4593, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4739] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4593, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4740] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4593, child_count: 3, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4741] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4596, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4742] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4598, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4743] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4598, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4744] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4600, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4745] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4600, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4746] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4605, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4747] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4605, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4748] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4605, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4749] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4605, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4750] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4605, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4751] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4605, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4752] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4605, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4753] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4605, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [4754] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4605, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4755] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4605, child_count: 7, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4756] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4757] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4758] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4759] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4760] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4761] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4762] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4763] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4764] StringParser("EXTENDED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4612, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4765] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4766] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4615, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4767] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4615, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4768] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4617, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4769] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4617, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4770] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4619, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4771] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4622, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4772] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4622, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4773] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4774] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4775] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4776] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4777] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4778] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4779] StringParser("FACTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4780] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4781] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4782] StringParser("FAMILY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4624, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4783] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4629, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4784] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4629, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4785] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4631, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4786] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4631, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4787] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4631, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4788] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4633, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4789] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4633, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4790] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4634, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4791] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4634, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4792] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4636, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4793] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4636, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4794] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4636, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4795] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4638, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4796] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4638, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4797] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4640, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4798] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4640, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4799] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4640, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4800] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4640, child_count: 6, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4801] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4646, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4802] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4646, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4803] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4647, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4804] Ref(ForwardKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4647, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4805] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4649, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4806] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4649, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4807] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4649, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4808] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4650, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4809] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4650, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4810] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4650, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4811] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4652, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4812] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4652, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4813] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4814] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4657, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4815] Anything
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4657, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4816] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4657, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4817] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4657, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4818] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4661, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4819] Ref(PostgresCopyStdinDataStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4661, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4820] Ref(PsqlCopyMetaCommandStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4661, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4821] Ref(PsqlSetMetaCommandStatementSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 4661, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4822] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4663, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4823] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4663, child_count: 1, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4824] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4664, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4825] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4664, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4826] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4664, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4827] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4666, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4828] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4666, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4829] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4669, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4830] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4831] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4832] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4833] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4834] StringParser("FINALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4835] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4836] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4837] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4838] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4839] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4671, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4840] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4671, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4841] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4675, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4842] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4675, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4843] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4679, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4844] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4679, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4845] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4682, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4846] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4682, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4847] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4682, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4848] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4682, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4849] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4682, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4850] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4684, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4851] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4684, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4852] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4684, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4853] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4854] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4855] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4688, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4856] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4688, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4857] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4690, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4858] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4690, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4859] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4692, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4860] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4692, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4861] Ref(LockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4692, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4862] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4692, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4863] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4692, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4864] StringParser("FORCE_NOT_NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4692, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4865] StringParser("FORCE_NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4692, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4866] StringParser("FORCE_QUOTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4692, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4867] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4695, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4868] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4695, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4869] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4695, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4870] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4695, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4871] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4697, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4872] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4697, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4873] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4697, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4874] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4697, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4875] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4699, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4876] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4701, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4877] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4701, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4878] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4701, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4879] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4705, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4880] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4707, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4881] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4707, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4882] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4707, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4883] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4710, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4884] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4710, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4885] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4886] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4713, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4887] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4715, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4888] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4715, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4889] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4715, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4890] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4717, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4891] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4717, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4892] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4722, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4893] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4722, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4894] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4722, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4895] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4722, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4896] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4722, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4897] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4722, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4898] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4727, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4899] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4727, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4900] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4727, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4901] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4727, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4902] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4727, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4903] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4727, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4904] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4729, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4905] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4731, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4906] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4731, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4907] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4731, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4908] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4734, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4909] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4734, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4910] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4737, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4911] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4737, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4912] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4739, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4913] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4739, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4914] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4739, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4915] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4739, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4916] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4739, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4917] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4742, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4918] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4742, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4919] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4744, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4920] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4746, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4921] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4748, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4922] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4748, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4923] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4748, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4924] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4750, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4925] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4754, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4926] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4754, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4927] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4754, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4928] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4754, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4929] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4754, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4930] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4756, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4931] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4756, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4932] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4756, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4933] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4760, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4934] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4760, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4935] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4760, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4936] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4762, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4937] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4762, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4938] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4766, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4939] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4768, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4940] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4768, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4941] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4768, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4942] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4768, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4943] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4768, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4944] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4770, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4945] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4770, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4946] Ref(OthersKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4770, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4947] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4773, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4948] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4773, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4949] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4773, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4950] Ref(GroupsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4773, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4951] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4773, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4952] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4773, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4953] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4773, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4954] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4775, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4955] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4775, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4956] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4777, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4957] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4777, child_count: 13, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4958] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4790, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4959] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4790, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4960] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4790, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4961] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4792, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4962] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4792, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4963] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4792, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4964] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4965] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4966] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4967] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4968] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4969] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4970] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4971] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4972] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4973] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4974] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4975] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4976] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4796, child_count: 7, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4977] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4803, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4978] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4803, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4979] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4805, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4980] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4808, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4981] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4808, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4982] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4808, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4983] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4809, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4984] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4813, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4985] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4813, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4986] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4813, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4987] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4813, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4988] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4813, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4989] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4816, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4990] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4816, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4991] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4816, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4992] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4816, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4993] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4816, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4994] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4816, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4995] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4819, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4996] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4821, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4997] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4822, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4998] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4822, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4999] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4824, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5000] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4827, child_count: 6, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5001] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4833, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 2, _padding: 0 }, // [5002] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4836, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [5003] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4836, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [5004] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4836, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [5005] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4839, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [5006] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4839, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [5007] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4841, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [5008] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4841, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [5009] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4841, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [5010] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4843, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [5011] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4843, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [5012] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4843, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [5013] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4843, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 2, _padding: 0 }, // [5014] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4844, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [5015] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4845, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [5016] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4847, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [5017] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4847, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [5018] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4847, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [5019] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4849, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [5020] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4849, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [5021] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4849, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [5022] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [5023] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [5024] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5025] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5026] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5027] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5028] TypedParser("full_text_search_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5029] StringParser("FULLNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5030] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4851, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5031] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5032] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5033] Ref(NamedArgumentSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4854, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5034] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4856, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5035] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4856, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5036] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4858, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5037] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4858, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5038] Ref(NamedArgumentSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4858, child_count: 14, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5039] AnyNumberOf(14 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4872, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5040] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4872, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5041] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4874, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5042] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4877, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5043] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4877, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5044] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4880, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5045] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4880, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5046] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4880, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5047] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4880, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5048] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4884, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5049] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4884, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5050] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4885, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5051] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4885, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5052] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4885, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5053] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4885, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5054] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4887, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5055] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4889, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5056] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4889, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5057] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4889, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5058] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4891, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5059] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4893, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5060] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4893, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5061] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4893, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5062] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4895, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5063] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4895, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5064] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4895, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5065] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4897, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5066] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4897, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5067] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4897, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5068] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4899, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5069] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4899, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5070] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5071] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4901, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5072] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4903, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5073] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4903, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5074] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4903, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5075] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4905, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5076] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4905, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5077] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4905, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5078] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4908, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5079] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4912, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5080] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4912, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5081] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4912, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5082] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4912, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5083] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4912, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5084] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4912, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5085] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4916, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5086] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4916, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5087] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4916, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5088] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4916, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5089] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4916, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5090] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4916, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5091] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4916, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5092] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4916, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5093] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4918, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5094] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4921, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5095] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4921, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5096] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5097] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5098] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5099] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4923, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5100] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4924, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5101] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4927, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5102] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4927, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5103] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4929, child_count: 15, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5104] AnyNumberOf(15 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4944, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5105] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4944, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5106] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4948, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5107] Ref(TransformKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4948, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5108] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4948, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5109] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4948, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5110] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4948, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5111] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4948, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5112] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5113] Ref(ImmutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5114] Ref(StableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5115] Ref(VolatileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5116] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4953, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5117] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4953, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5118] Ref(LeakproofKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4953, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5119] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4956, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5120] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4960, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5121] Ref(CalledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4960, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5122] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4960, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5123] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4960, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5124] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4960, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5125] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4965, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5126] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4965, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5127] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4965, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5128] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4965, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5129] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4965, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5130] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4965, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5131] Ref(StrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4965, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5132] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4968, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5133] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4968, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5134] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4968, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5135] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4970, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5136] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4970, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5137] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4970, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5138] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4972, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5139] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4972, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5140] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4975, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5141] Ref(UnsafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4975, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5142] Ref(RestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4975, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5143] Ref(SafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4975, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5144] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4977, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5145] Ref(CostKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4977, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5146] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4977, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5147] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5148] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5149] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5150] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4981, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5151] Ref(SupportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4981, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5152] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4981, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5153] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4984, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5154] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4984, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5155] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4984, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5156] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4986, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5157] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4988, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5158] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4990, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5159] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4990, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5160] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4990, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5161] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4992, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5162] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4994, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5163] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4994, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5164] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4994, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5165] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4996, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5166] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4996, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5167] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4996, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5168] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4998, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5169] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4998, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5170] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5171] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5000, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5172] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5003, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5173] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5003, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5174] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5003, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5175] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5003, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5176] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5177] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5178] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5005, child_count: 6, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5179] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5011, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5180] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5011, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5181] Ref(AtomicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5011, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5182] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5015, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5183] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5017, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5184] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5017, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5185] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5017, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5186] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5019, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5187] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5019, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5188] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5019, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5189] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5021, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5190] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5024, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5191] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5024, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5192] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5024, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5193] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5024, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5194] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5024, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5195] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5027, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5196] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5027, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5197] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5027, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5198] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5027, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5199] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5027, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5200] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5029, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5201] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5029, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5202] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5032, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5203] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5034, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5204] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5034, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5205] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5034, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5206] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5034, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5207] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5036, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 1, _padding: 0 }, // [5208] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5037, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5209] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5039, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5210] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5039, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5211] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5039, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [5212] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5039, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 1, _padding: 0 }, // [5213] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5041, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [5214] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5041, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [5215] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5041, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [5216] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5041, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5217] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5044, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5218] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5219] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5220] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5221] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5222] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5223] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5050, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5224] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5050, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5225] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5053, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5226] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5053, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5227] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5057, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5228] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5057, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5229] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5057, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5230] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5057, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5231] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5057, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5232] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5059, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5233] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5059, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5234] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5059, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5235] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5061, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5236] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5064, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5237] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5064, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5238] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5064, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5239] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5064, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5240] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5064, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5241] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5067, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5242] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5069, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5243] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5069, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5244] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5072, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5245] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5074, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5246] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5074, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5247] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5074, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5248] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5074, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5249] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5076, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5250] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 5078, child_count: 1, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5251] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5079, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5252] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5253] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5254] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5255] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5256] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5257] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5258] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5259] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5260] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5261] StringParser("GENERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5262] StringParser("GEOGRAPHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5263] StringParser("GEOMETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5264] StringParser("GEOMETRYCOLLECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5265] StringParser("GEOMETRYCOLLECTIONM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5266] StringParser("GEOMETRYCOLLECTIONZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5267] StringParser("GEOMETRYCOLLECTIONZM")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5268] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5269] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5082, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5270] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5082, child_count: 6, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5271] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5088, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5272] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5088, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5273] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5088, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5274] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5088, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5275] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5088, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5276] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5088, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5277] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5088, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5278] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5088, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5279] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5088, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5280] StringParser("GRAPH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5088, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5281] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5090, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5282] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5090, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5283] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5090, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5284] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5090, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5285] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5090, child_count: 5, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5286] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5095, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5287] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5095, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5288] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5095, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 6, _padding: 0 }, // [5289] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5097, child_count: 6, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5290] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5103, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5291] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5103, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5292] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5103, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5293] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5103, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5294] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5103, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5295] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5103, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5296] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5105, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5297] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5298] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5299] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5300] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5301] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5302] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5303] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [5304] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 6, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5305] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5113, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5306] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5115, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5307] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5115, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5308] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5115, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5309] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5115, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5310] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5115, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5311] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5115, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5312] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5115, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5313] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5115, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5314] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5115, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5315] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5116, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 1, _padding: 0 }, // [5316] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5118, child_count: 4, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5317] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5122, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5318] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5122, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5319] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5122, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5320] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5122, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5321] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5124, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5322] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5124, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5323] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5124, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5324] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5127, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5325] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5127, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5326] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5127, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5327] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5130, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5328] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5132, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5329] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5330] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5331] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5332] StringParser("GROUPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5333] StringParser("GZIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5334] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5335] StringParser("HASH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5134, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5336] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5138, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5337] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5138, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5338] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5140, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5339] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5340] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5143, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5341] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5148, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5342] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5343] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5344] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5345] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5346] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5347] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5348] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5349] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5350] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5351] StringParser("HEADERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5352] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5353] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5354] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5355] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5356] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5357] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5358] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5359] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5360] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5361] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5362] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5363] StringParser("HOURS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5364] StringParser("ICU")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5365] StringParser("ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5366] StringParser("IDEMPOTENCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5367] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5368] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5369] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5370] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5371] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5372] StringParser("IDLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5373] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5152, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5374] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5152, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5375] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5152, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5376] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5152, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5377] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5155, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5378] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5155, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5379] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5155, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5380] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5155, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5381] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5155, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5382] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5157, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5383] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5159, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5384] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5159, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5385] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5159, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5386] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5159, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5387] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5159, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5388] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5159, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5389] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5159, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5390] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5159, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5391] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5162, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5392] Ref(ImportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5162, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5393] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5162, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5394] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5162, child_count: 9, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5395] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5171, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5396] Ref(ImportForeignSchemaGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5171, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5397] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5171, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5398] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5173, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5399] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5175, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5400] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5401] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5402] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5403] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5404] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5180, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5405] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5182, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5406] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5182, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5407] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5182, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5408] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5182, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5409] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5182, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5410] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5182, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5411] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5182, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5412] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5182, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5413] StringParser("IMPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5182, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5414] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5182, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5415] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5185, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5416] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5185, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5417] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5185, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5418] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5185, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5419] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5185, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5420] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5188, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5421] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5188, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5422] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5188, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5423] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5190, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5424] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5193, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5425] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5195, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5426] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5197, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5427] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5197, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5428] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5197, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5429] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5197, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5430] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5197, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5431] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5197, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5432] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5197, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5433] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5197, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5434] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5199, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5435] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5199, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5436] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5201, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5437] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5201, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5438] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5201, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5439] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5205, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5440] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5207, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5441] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5207, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5442] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5207, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5443] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 5209, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5444] Ref(OperatorClassReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5210, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5445] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5212, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5446] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5212, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5447] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5214, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5448] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5214, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5449] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5214, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5450] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5214, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5451] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5216, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5452] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5216, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5453] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5216, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5454] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5218, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5455] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5218, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5456] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5220, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5457] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5220, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5458] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5220, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5459] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5222, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5460] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5225, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5461] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5225, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5462] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5225, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5463] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5228, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5464] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5228, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5465] Ref(IndexElementOptionsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5228, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5466] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5228, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5467] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5231, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5468] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5233, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5469] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5233, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5470] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5233, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5471] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5235, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5472] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5235, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5473] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5235, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5474] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5475] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5476] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5477] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5478] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5479] StringParser("INDEX_CLEANUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5480] StringParser("INDEXES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5481] StringParser("INET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5482] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5483] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5484] StringParser("INFO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5485] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5486] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5487] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5488] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5489] StringParser("INLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5490] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5491] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5492] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5493] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5494] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5239, child_count: 9, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5495] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5496] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5497] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5498] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5499] Ref(AsAliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5500] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5248, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5501] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5251, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5502] Ref(OverridingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5251, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5503] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5253, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5504] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5253, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5505] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5253, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5506] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5253, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5507] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5255, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5508] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5257, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5509] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5257, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5510] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5257, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5511] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5257, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5512] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5261, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5513] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5261, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5514] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5261, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5515] Ref(ConflictTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5261, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5516] Ref(ConflictActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5261, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5517] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5265, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5518] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5265, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5519] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5520] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5267, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5521] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5269, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5522] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5271, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5523] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5271, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5524] Ref(AsAliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5271, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5525] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5271, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5526] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5527] MultiStringParser(12 templates)
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5528] MultiStringParser(12 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5529] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5530] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5531] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5532] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5533] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5534] StringParser("INT4RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5535] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5536] StringParser("INT8RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5537] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5538] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5539] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5540] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5541] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5273, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5542] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5275, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5543] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5275, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5544] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5278, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5545] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5280, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5546] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5280, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5547] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5282, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5548] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5282, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5549] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5282, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5550] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5282, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5551] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5284, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5552] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5284, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5553] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5286, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5554] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5286, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5555] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5286, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5556] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5286, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5557] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5558] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5559] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5560] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5561] Ref(HourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5562] Ref(MinuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5563] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5564] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5296, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5565] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5296, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5566] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5299, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5567] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5299, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5568] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5299, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5569] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5299, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5570] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5299, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5571] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5299, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5572] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5299, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5573] StringParser("INTROSPECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5299, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5574] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5299, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5575] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5305, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5576] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5305, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5577] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5305, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5578] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5305, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5579] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5305, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5580] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5305, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5581] Ref(JsonTestGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5305, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5582] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5583] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5584] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5585] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5586] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5587] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5588] Ref(IsnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5589] StringParser("IS_SUPERUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5590] StringParser("IS_TEMPLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5591] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5592] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5593] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5594] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5595] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5312, child_count: 7, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5596] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5319, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5597] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5319, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5598] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5319, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5599] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5319, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5600] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5320, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5601] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5320, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5602] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5324, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5603] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5324, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5604] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5324, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5605] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5606] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5607] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5608] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5609] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5332, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5610] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5332, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5611] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5332, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5612] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5332, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5613] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5332, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5614] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5336, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5615] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5336, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5616] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5336, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5617] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5336, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5618] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5337, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5619] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5337, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5620] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5337, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5621] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5341, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5622] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5341, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5623] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5341, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5624] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5343, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5625] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5346, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5626] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5346, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5627] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5346, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5628] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5629] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5348, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5630] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5350, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5631] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5353, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5632] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5353, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5633] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5353, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5634] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5353, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5635] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5353, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5636] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5357, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5637] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5357, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5638] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5360, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5639] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5362, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5640] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5362, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5641] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5362, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5642] TypedParser("json_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5362, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5643] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5365, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5644] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5365, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5645] Ref(JsonTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5365, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5646] Ref(JsonUniqueKeysGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5365, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5647] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5648] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5649] Ref(ScalarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5650] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5651] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5652] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5372, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5653] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5374, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5654] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5374, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5655] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5374, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5656] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5374, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5657] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5375, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5658] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5375, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5659] StringParser("JSONB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5375, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5660] StringParser("KAFKA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5375, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5661] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5375, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5662] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5375, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5663] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5375, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5664] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5375, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5665] StringParser("KINESIS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5375, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5666] StringParser("LABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5375, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5667] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5375, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5668] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5377, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5669] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5377, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5670] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5671] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5672] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5673] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5674] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5675] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5676] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5677] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5678] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5679] StringParser("LATEST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5680] StringParser("LC_COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5681] StringParser("LC_CTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5682] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5683] StringParser("LEAKPROOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5684] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5685] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5686] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5687] StringParser("LEFTARG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5688] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5379, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5689] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5381, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5690] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5381, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5691] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5381, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5692] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5381, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5693] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5381, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5694] StringParser("LIBC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5381, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5695] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5384, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5696] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5386, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5697] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5386, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5698] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5386, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5699] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5386, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5700] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5388, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5701] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5388, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5702] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5388, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5703] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5391, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5704] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5391, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5705] Ref(IlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5391, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5706] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5393, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5707] Ref(SimilarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5393, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5708] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5393, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5709] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5393, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5710] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5393, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5711] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5395, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5712] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5397, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5713] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5397, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5714] Ref(ExcludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5397, child_count: 9, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5715] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5716] Ref(CommentsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5717] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5718] Ref(DefaultsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5719] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5720] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5721] Ref(IndexesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5722] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5723] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5724] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5725] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5411, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5726] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5411, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5727] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5413, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5728] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5416, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5729] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5419, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5730] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5419, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5731] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5419, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5732] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5419, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5733] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5421, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5734] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5423, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5735] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5423, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5736] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5425, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5737] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5425, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5738] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5425, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5739] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5740] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5741] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5742] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5743] StringParser("LINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5744] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5745] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5746] StringParser("LINESTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5747] StringParser("LINESTRINGM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5748] StringParser("LINESTRINGZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5749] StringParser("LINESTRINGZM")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5750] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5751] StringParser("LIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5752] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5427, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5753] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5754] Ref(ListenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5755] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 11, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5756] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5757] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5758] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5759] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5760] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5761] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5762] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5763] Ref(DollarNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5764] Ref(PsqlVariableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5765] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5766] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5767] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5768] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5769] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5440, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5770] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5771] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5442, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5772] Ref(LoadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5442, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5773] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5442, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5774] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5442, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5775] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5442, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5776] StringParser("LOCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5442, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5777] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5442, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5778] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5442, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5779] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5442, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5780] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5442, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5781] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5448, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5782] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5448, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5783] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5448, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5784] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5448, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5785] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5450, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5786] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5452, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5787] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5452, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5788] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5452, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5789] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5455, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5790] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5455, child_count: 8, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5791] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5463, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5792] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5465, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5793] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5465, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5794] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5465, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5795] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5467, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5796] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5467, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5797] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5467, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5798] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5469, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5799] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5469, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5800] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5469, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5801] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5472, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5802] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5472, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5803] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5472, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5804] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5472, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5805] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5472, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5806] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5475, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5807] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5475, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5808] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5475, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5809] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5475, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5810] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5475, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5811] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5812] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5813] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5814] Ref(ModeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5815] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5816] StringParser("LOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5817] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5818] StringParser("LOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5819] StringParser("LOGGED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5820] StringParser("LOGICAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5821] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5822] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5823] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5824] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5825] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5826] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5827] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5828] StringParser("LSEG")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5477, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5829] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5481, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5830] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5481, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5831] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5481, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5832] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5481, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5833] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5485, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5834] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5487, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5835] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5487, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5836] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5487, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5837] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5838] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5489, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5839] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5840] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5841] StringParser("MACADDR8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5842] StringParser("MACADDR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5843] StringParser("MAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5844] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5845] StringParser("MANAGED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5846] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5847] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5848] StringParser("MAPPING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5849] StringParser("MARKETING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5850] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5851] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5852] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5853] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5854] StringParser("MATCHING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5855] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5498, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5856] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5498, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5857] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5499, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5858] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5501, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5859] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5505, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5860] Ref(RawKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5505, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5861] Ref(DecorrelatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5505, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5862] Ref(OptimizedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5505, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5863] Ref(PhysicalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5505, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5864] Ref(PlanKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5505, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5865] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5507, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5866] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5507, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5867] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5510, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5868] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5512, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5869] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5512, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5870] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5514, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5871] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5514, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5872] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5516, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5873] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5516, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5874] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5516, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5875] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5517, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5876] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5517, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5877] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5521, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5878] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5521, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5879] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5523, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5880] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5523, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5881] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5523, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5882] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5526, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5883] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5526, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5884] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5526, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5885] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5526, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5886] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5887] Anything
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5888] StringParser("MATERIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5889] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5890] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5891] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5892] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5893] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5894] StringParser("MCV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5895] StringParser("MECHANISMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5896] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5897] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5898] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5527, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5899] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5527, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5900] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5532, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5901] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5532, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5902] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5532, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5903] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5532, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5904] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5534, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5905] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5534, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5906] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5534, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5907] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5534, child_count: 3, min_times: 1, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5908] AnyNumberOf(3 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5537, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5909] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5537, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5910] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5537, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5911] Ref(MergeNotMatchedBySourceClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5537, child_count: 7, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5912] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5544, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5913] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5544, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5914] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5544, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5915] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5546, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5916] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5546, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5917] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5546, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5918] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5546, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5919] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5549, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5920] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5549, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5921] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5549, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5922] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5551, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5923] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5551, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5924] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5551, child_count: 10, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5925] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5561, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5926] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5561, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5927] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5561, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5928] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5561, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5929] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5561, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5930] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5561, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5931] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5563, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5932] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5563, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5933] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5563, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5934] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5563, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5935] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5566, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5936] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5566, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5937] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5566, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5938] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5939] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5568, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5940] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5568, child_count: 9, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5941] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5577, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5942] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5577, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5943] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5577, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5944] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5577, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5945] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5579, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5946] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5579, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5947] Ref(TargetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5579, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5948] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5581, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5949] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5581, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5950] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5581, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5951] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5581, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5952] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5583, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5953] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5583, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5954] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5585, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5955] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5585, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5956] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5585, child_count: 12, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5957] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5597, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5958] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5597, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5959] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5599, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5960] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5599, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5961] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5599, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5962] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5599, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5963] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5603, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5964] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5603, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5965] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5603, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5966] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5605, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5967] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5608, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5968] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5608, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5969] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5608, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5970] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5610, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5971] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5610, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5972] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5610, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5973] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5610, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5974] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5610, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5975] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5610, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5976] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5610, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5977] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5614, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5978] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5614, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5979] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5614, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5980] StringParser("MESSAGE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5614, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5981] TypedParser("meta_command_query_buffer")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5614, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5982] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5615, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5983] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5616, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5984] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5985] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5986] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5987] StringParser("METADATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5988] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5989] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5990] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5991] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5992] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5993] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5994] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5995] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5996] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5997] StringParser("MINUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5998] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5999] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6000] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6001] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6002] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6003] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6004] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6005] StringParser("MODULUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6006] StringParser("MONEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6007] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6008] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6009] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6010] StringParser("MONTHS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6011] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6012] StringParser("MS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6013] StringParser("MULTICURVE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5618, child_count: 1, min_times: 1, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6014] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6015] Ref(MultilineConcatenateNewline)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6016] TypedParser("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6017] StringParser("MULTILINESTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6018] StringParser("MULTILINESTRINGM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6019] StringParser("MULTILINESTRINGZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6020] StringParser("MULTILINESTRINGZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6021] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6022] StringParser("MULTIPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6023] StringParser("MULTIPOINTM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6024] StringParser("MULTIPOINTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6025] StringParser("MULTIPOINTZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6026] StringParser("MULTIPOLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6027] StringParser("MULTIPOLYGONM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6028] StringParser("MULTIPOLYGONZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6029] StringParser("MULTIPOLYGONZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6030] StringParser("MULTISURFACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6031] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6032] TypedParser("word")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6033] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6034] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5619, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6035] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5622, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6036] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5622, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6037] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5625, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6038] Ref(RightArrowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5625, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6039] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5625, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6040] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5625, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6041] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5625, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6042] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5628, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6043] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5628, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6044] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5628, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6045] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5630, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6046] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5630, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6047] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5633, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6048] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5633, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6049] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5637, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6050] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5637, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6051] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5639, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6052] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5639, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6053] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5639, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6054] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5639, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6055] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5639, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6056] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5639, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6057] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6058] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6059] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6060] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6061] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6062] StringParser("NDISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6063] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6064] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6065] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6066] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6067] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6068] StringParser("NFC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6069] StringParser("NFD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6070] StringParser("NFKC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6071] StringParser("NFKD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6072] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6073] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6074] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6075] StringParser("NOBYPASSRLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6076] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6077] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6078] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6079] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6080] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6081] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6082] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6083] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6084] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6085] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5647, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6086] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5647, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6087] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5647, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6088] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5650, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6089] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5650, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6090] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5653, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6091] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5653, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6092] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6093] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6094] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6095] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6096] OneOf(0 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5656, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6097] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5662, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6098] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5662, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6099] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5664, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6100] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5667, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6101] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5667, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6102] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5667, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6103] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5667, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6104] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5667, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6105] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5667, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6106] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5667, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6107] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5667, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6108] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5667, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6109] StringParser("NOREPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5667, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6110] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5667, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6111] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5669, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6112] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5673, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6113] Ref(NfcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5673, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6114] Ref(NfdKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5673, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6115] Ref(NfkcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5673, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6116] Ref(NfkdKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5673, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6117] Ref(NormalizedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5673, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6118] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5673, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6119] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5673, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6120] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5673, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6121] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5675, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6122] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5677, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6123] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5677, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6124] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5677, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6125] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6126] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6127] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5679, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6128] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5681, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6129] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5681, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6130] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5681, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6131] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5683, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6132] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5683, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6133] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5683, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6134] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5683, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6135] Ref(NotnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5683, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6136] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5683, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6137] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5683, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6138] StringParser("NOTICE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5683, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6139] StringParser("NOTIFICATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5683, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6140] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5683, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6141] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6142] Ref(NotifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6143] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5686, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6144] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6145] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6146] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6147] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6148] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6149] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6150] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6151] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6152] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6153] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6154] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6155] StringParser("NUMRANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5688, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6156] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5688, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6157] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5691, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6158] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5691, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6159] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5691, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6160] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5691, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6161] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5694, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6162] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5696, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6163] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5696, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6164] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5696, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6165] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5698, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6166] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5698, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6167] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5700, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6168] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5700, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6169] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5700, child_count: 12, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6170] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6171] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6172] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6173] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6174] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6175] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6176] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6177] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6178] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6179] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6180] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6181] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6182] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6183] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6184] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6185] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6186] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5716, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6187] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5716, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6188] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6189] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6190] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6191] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6192] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6193] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6194] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6195] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6196] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6197] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6198] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6199] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6200] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6201] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6202] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6203] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6204] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6205] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6206] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6207] StringParser("OPTIMIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6208] StringParser("OPTIMIZER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6209] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5718, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6210] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5718, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6211] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5720, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6212] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5720, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6213] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5723, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6214] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5724, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6215] Ref(OptionsListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5724, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6216] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5724, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6217] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5725, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6218] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5727, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6219] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5729, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6220] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5729, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6221] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5729, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6222] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5729, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6223] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5729, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6224] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5731, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6225] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5731, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6226] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5731, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6227] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5736, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6228] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5736, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6229] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5736, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 2, _padding: 0 }, // [6230] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5738, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6231] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5742, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6232] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5745, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6233] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5745, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6234] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5745, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6235] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5745, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6236] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5748, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6237] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5748, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6238] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5748, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6239] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5750, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6240] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5750, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6241] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5750, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6242] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5752, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6243] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5752, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6244] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5754, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6245] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5754, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6246] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5754, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6247] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5754, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6248] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5754, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [6249] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5754, child_count: 8, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6250] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6251] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6252] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6253] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6254] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6255] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6256] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6257] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5764, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6258] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5764, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6259] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5764, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6260] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5764, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6261] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5764, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6262] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5766, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6263] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5766, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6264] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5766, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6265] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5766, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6266] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5766, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6267] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5766, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6268] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5766, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6269] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5766, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6270] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6271] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5771, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6272] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5771, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6273] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5773, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6274] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5773, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6275] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5776, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6276] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5776, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6277] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5776, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6278] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5778, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6279] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5778, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6280] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5778, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6281] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5781, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6282] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5783, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6283] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5784, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6284] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5789, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6285] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5793, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6286] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5793, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6287] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5793, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6288] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5793, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6289] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5793, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6290] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5793, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6291] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6292] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6293] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6294] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6295] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6296] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6297] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6298] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5799, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6299] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5800, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6300] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5805, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6301] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5809, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6302] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5809, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6303] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5809, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6304] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5809, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6305] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5809, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6306] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5809, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6307] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6308] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6309] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6310] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6311] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6312] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6313] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6314] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6315] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6316] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6317] StringParser("OWNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6318] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6319] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6320] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6321] StringParser("PARALLEL")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6322] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6323] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6324] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6325] StringParser("PARSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6326] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5813, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6327] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5816, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6328] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5818, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6329] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5818, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6330] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5821, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6331] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5823, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6332] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5823, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6333] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5827, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6334] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5827, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6335] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5830, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6336] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5832, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6337] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5835, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6338] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5835, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6339] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5835, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6340] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5835, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6341] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5835, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6342] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5838, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6343] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5840, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6344] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5843, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6345] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5843, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6346] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5843, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6347] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5843, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6348] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5845, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6349] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5845, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6350] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5848, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6351] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5853, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6352] Ref(ModulusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5853, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6353] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5853, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6354] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5853, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6355] Ref(RemainderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5853, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6356] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5853, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6357] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5858, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6358] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5858, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6359] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5858, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6360] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5860, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6361] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5863, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6362] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5865, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6363] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5865, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6364] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5865, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6365] StringParser("PASSING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5865, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6366] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5865, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6367] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5865, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6368] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5867, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6369] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5869, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6370] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5869, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6371] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6372] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6373] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6374] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6375] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6376] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6377] StringParser("PERMISSIVE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6378] TypedParser("pg_trgm_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6379] StringParser("PG_LSN")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6380] TypedParser("pgvector_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6381] StringParser("PHYSICAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6382] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6383] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6384] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6385] StringParser("PLAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6386] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6387] StringParser("PLANS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6388] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6389] StringParser("POINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6390] StringParser("POINTM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6391] StringParser("POINTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6392] StringParser("POINTZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6393] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6394] StringParser("POLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6395] StringParser("POLYGONM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6396] StringParser("POLYGONZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6397] StringParser("POLYGONZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6398] StringParser("POLYHEDRALSURFACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6399] StringParser("PORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6400] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6401] StringParser("+")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5871, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6402] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5874, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6403] Ref(WithinGroupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5874, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6404] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5874, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6405] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5874, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6406] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5874, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6407] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5874, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6408] TypedParser("postgis_operator")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5874, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6409] TypedParser("postgres_copy_stdin_data_block")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5874, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6410] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5875, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6411] Ref(PostgresCopyStdinDataSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5875, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6412] StringParser("POSTGRES")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5875, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6413] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5876, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6414] Ref(LateralKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5876, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6415] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5876, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6416] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5876, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6417] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5876, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6418] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5876, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6419] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5876, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6420] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5881, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6421] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5881, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6422] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5881, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6423] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5884, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6424] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6425] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6426] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5886, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6427] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5888, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6428] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5888, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6429] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5888, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6430] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5888, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6431] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5888, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6432] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6433] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6434] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6435] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6436] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6437] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6438] StringParser("PRIVATELINK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6439] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6440] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6441] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6442] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6443] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6444] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6445] StringParser("PROCESS_TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6446] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6447] StringParser("PROGRAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6448] StringParser("PROGRESS")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6449] TypedParser("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6450] StringParser("PROTOBUF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6451] StringParser("PROVIDER")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5890, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6452] TypedParser("psql_copy_command")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5890, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6453] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5891, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6454] Ref(PsqlCopyMetaCommandSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5891, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6455] TypedParser("psql_set_command")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5891, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6456] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5892, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6457] Ref(PsqlSetMetaCommandSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5892, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6458] StringParser(":")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5892, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6459] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5893, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6460] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5895, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6461] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5898, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6462] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5900, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6463] Ref(PsqlVariableColonSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5900, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6464] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5902, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6465] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5902, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6466] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5902, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6467] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5902, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6468] StringParser("PUBLICATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5902, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6469] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5904, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6470] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5906, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6471] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5906, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 1, _padding: 0 }, // [6472] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5908, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6473] Ref(PublicationTableSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5908, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6474] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5910, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6475] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5910, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6476] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6477] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [6478] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 4, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6479] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5916, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6480] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5916, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6481] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5916, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6482] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5916, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 1, _padding: 0 }, // [6483] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5918, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6484] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5920, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6485] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5920, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6486] Ref(Current_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5920, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6487] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5922, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6488] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5922, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6489] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5924, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6490] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5924, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [6491] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5924, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6492] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5924, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6493] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5927, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6494] Ref(ExtendedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5927, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6495] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5927, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6496] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5929, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6497] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5929, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6498] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5932, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6499] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5932, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6500] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5932, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6501] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5934, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6502] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5934, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6503] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5934, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6504] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5936, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6505] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5936, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6506] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5939, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6507] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5942, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6508] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5942, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6509] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5942, child_count: 1, min_times: 1, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6510] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5943, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6511] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5943, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6512] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5945, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6513] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 5945, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6514] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5947, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6515] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5947, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6516] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5947, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6517] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5947, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6518] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5950, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6519] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5950, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6520] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6521] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6522] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6523] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6524] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5956, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6525] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5956, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6526] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5956, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6527] StringParser("QUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5956, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6528] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5956, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6529] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5958, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6530] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5958, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6531] TypedParser("unicode_double_quote")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5958, child_count: 4, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6532] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5962, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6533] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5964, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6534] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5966, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6535] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5966, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6536] TypedParser("escaped_single_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5966, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6537] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5968, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6538] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5968, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6539] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5968, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6540] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5970, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6541] TypedParser("bit_string_literal")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5970, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6542] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5972, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6543] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5972, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6544] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5972, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6545] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5975, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6546] TypedParser("unicode_single_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5975, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6547] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5977, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6548] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5977, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6549] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5977, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6550] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5979, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6551] Ref(UescapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5979, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6552] RegexParser
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5979, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6553] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5981, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6554] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5983, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6555] TypedParser("dollar_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5983, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6556] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6557] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6558] TypedParser("dollar_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6559] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6560] StringParser("RAISE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6561] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6562] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6563] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6564] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6565] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6566] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6567] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6568] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6569] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6570] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6571] StringParser("REASSIGN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5985, child_count: 6, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6572] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5991, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6573] Ref(ReassignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5991, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6574] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5991, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6575] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5991, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6576] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5993, child_count: 4, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6577] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6578] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6579] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6580] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6581] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6582] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5997, child_count: 4, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6583] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6001, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6584] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6001, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6585] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6001, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6586] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6001, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6587] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6001, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6588] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6001, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6589] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6001, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6590] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6001, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6591] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6001, child_count: 5, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6592] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6006, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6593] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6006, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6594] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6006, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6595] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6006, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6596] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6006, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6597] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6008, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6598] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6011, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6599] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6011, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6600] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6011, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6601] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6011, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6602] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6014, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6603] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6014, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6604] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6014, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6605] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6014, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6606] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6016, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6607] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6016, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6608] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6019, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6609] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6019, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6610] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6019, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6611] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6019, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6612] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6019, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6613] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6019, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6614] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6019, child_count: 4, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6615] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6023, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6616] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6023, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6617] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6026, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6618] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6026, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6619] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6028, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6620] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6028, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6621] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6028, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6622] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6031, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6623] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6033, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6624] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6033, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6625] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6033, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6626] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6035, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6627] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6035, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6628] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6035, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6629] StringParser("REFRESH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6035, child_count: 6, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6630] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6631] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6632] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6633] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6634] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6635] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6636] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6637] StringParser("REGEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6638] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6639] StringParser("REGION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6640] StringParser("REGISTRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6641] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6041, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6642] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6044, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6643] Ref(ReindexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6044, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6644] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6047, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6645] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6049, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6646] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6052, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6647] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6054, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6648] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6054, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6649] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6054, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6650] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6056, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6651] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6056, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6652] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6056, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6653] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6058, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6654] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6058, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6655] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6058, child_count: 4, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6656] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6062, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6657] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6065, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6658] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6065, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6659] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6065, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6660] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6065, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6661] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6068, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6662] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6068, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6663] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6068, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6664] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6068, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6665] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6071, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6666] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6071, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6667] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6071, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6668] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6071, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6669] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6074, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6670] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6076, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6671] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6076, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6672] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6076, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6673] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6076, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6674] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6076, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6675] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6079, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6676] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6079, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6677] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6081, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6678] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6081, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6679] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6081, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6680] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6083, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6681] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6083, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6682] Ref(DefinitionArgumentValueGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6083, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6683] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6086, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6684] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6685] Ref(RelationOptionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6686] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6687] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6688] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6689] StringParser("REMAINDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6690] StringParser("REMOTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6691] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6692] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6693] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6694] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6695] StringParser("REPLICA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6696] StringParser("REPLICAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6697] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6698] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6088, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6699] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6088, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6700] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6091, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6701] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6091, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6702] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6091, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6703] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6091, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6704] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6093, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6705] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6093, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6706] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6707] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6708] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6709] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6710] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6711] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6712] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6713] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6714] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6715] StringParser("RESTRICTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6716] StringParser("RESTRICTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6717] StringParser("RETENTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6718] StringParser("RETRIEVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6719] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6720] StringParser("RETURNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6721] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6096, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6722] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6096, child_count: 6, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6723] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6102, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6724] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6102, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6725] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6105, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6726] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6105, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6727] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6105, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6728] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6105, child_count: 4, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6729] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6109, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6730] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6112, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6731] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6112, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6732] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6112, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6733] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6112, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6734] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6114, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6735] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6114, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6736] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6114, child_count: 4, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6737] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6738] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6739] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6740] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6741] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6742] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6743] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6744] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6745] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6746] StringParser("=>")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6747] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6748] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6749] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6750] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6751] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6752] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6753] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6754] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6755] StringParser("ROTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6756] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6757] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6758] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6759] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6760] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6761] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6762] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6763] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6764] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6765] StringParser("S3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6118, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6766] StringParser("SAFE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6118, child_count: 4, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6767] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6122, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6768] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6122, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6769] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6124, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6770] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6124, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6771] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6124, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6772] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6127, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6773] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6127, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6774] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6129, child_count: 1, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6775] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6130, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6776] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6130, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6777] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6778] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6779] StringParser("SASL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6780] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6781] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6782] StringParser("SCALAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6783] StringParser("SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6784] StringParser("SCAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6785] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6786] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6787] StringParser("SCRIPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6788] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6789] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6790] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6791] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6792] StringParser("SECONDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6793] StringParser("SECRET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6794] StringParser("SECRETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6795] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6133, child_count: 7, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6796] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6140, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6797] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6140, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6798] Ref(LabelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6140, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6799] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6142, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6800] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6142, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6801] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6142, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6802] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6142, child_count: 21, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6803] OneOf(21 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6163, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6804] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6165, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6805] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6165, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6806] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6165, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6807] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6167, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6808] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6167, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6809] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6167, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6810] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6170, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6811] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6170, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6812] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6170, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6813] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6173, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6814] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6173, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6815] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6175, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6816] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6175, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6817] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6175, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6818] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6177, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6819] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6177, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6820] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6177, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6821] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6180, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6822] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6180, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6823] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6180, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6824] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6180, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6825] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6183, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6826] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6183, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6827] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6183, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6828] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6183, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6829] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6186, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6830] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6186, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6831] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6186, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6832] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6188, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6833] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6191, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6834] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6191, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6835] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6194, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6836] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6194, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6837] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6194, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6838] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6194, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6839] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6197, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6840] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6197, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6841] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6197, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6842] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6197, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6843] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6200, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6844] Ref(ProceduralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6200, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6845] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6200, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6846] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6200, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6847] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6203, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6848] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6203, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6849] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6203, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6850] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6205, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6851] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6208, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6852] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6208, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6853] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6210, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6854] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6210, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6855] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6210, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6856] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6212, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6857] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6212, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6858] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6212, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6859] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6215, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6860] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6215, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6861] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6215, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6862] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6217, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6863] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6220, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6864] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6220, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6865] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6222, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6866] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6222, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6867] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6222, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6868] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6224, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6869] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6224, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6870] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6224, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6871] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6226, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6872] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6226, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6873] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6226, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6874] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6228, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6875] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6228, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6876] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6228, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6877] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6230, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6878] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6230, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6879] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6230, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6880] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6232, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6881] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6232, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6882] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6232, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6883] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6232, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6884] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6234, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6885] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6234, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6886] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6234, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6887] StringParser("SEED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6234, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6888] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6236, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6889] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6236, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6890] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6238, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6891] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6238, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6892] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6238, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6893] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6240, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6894] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6242, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6895] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6242, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6896] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6244, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6897] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6244, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6898] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6247, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6899] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6249, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6900] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6249, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6901] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 6249, child_count: 5, min_times: 0, first_terminator_idx: 51, terminator_count: 12, _padding: 0 }, // [6902] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6254, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6903] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6254, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6904] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(13), first_child_idx: 6254, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6905] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6256, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6906] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6256, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6907] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6256, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6908] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6256, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6909] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6256, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6910] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6258, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6911] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6258, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6912] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6258, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6913] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6260, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6914] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6260, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6915] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6260, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6916] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6260, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6917] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6260, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6918] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6260, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6919] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6260, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6920] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6263, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6921] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6263, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6922] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6263, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6923] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6263, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6924] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6263, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [6925] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6263, child_count: 9, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6926] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6272, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6927] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 6272, child_count: 1, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6928] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6273, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6929] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6273, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6930] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6273, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6931] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6932] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6933] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6934] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6935] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6936] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6937] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6938] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6939] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 6275, child_count: 14, min_times: 0, first_terminator_idx: 63, terminator_count: 7, _padding: 0 }, // [6940] Sequence(14 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6941] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6942] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6943] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6944] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6945] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6946] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6947] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6948] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6949] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6950] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6951] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6952] Ref(OffsetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6953] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6954] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6955] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6956] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6289, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6957] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6289, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6958] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6291, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6959] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6291, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6960] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6291, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6961] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6291, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6962] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6291, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [6963] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6291, child_count: 4, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6964] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6295, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6965] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6297, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6966] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6300, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6967] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6300, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6968] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6302, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6969] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6305, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6970] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6305, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6971] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6305, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6972] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6308, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6973] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6308, child_count: 4, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6974] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6312, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6975] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6312, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6976] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6312, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6977] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6312, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6978] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6314, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6979] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6316, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6980] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6316, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6981] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6316, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6982] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6316, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6983] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6316, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6984] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6316, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6985] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6316, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6986] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6318, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6987] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6320, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6988] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6320, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6989] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6320, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6990] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6991] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6992] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6322, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6993] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6324, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6994] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6326, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6995] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6326, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6996] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6326, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6997] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6998] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6999] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7000] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7001] StringParser("SERIAL2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7002] StringParser("SERIAL4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7003] StringParser("SERIAL8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7004] StringParser("SERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7005] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7006] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7007] StringParser("SERVER_ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7008] StringParser("SERVER_VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6328, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7009] StringParser("SERVICE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6328, child_count: 4, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7010] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6332, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7011] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6332, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7012] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6332, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7013] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6332, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7014] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6332, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7015] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6332, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7016] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6332, child_count: 4, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7017] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6336, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7018] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6336, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7019] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6338, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7020] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6338, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7021] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6339, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7022] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6341, child_count: 5, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7023] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6346, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7024] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6346, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7025] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6346, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7026] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6346, child_count: 6, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7027] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6352, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7028] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6352, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7029] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6352, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7030] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6352, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7031] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6352, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7032] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6352, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7033] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6352, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7034] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6353, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7035] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6353, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7036] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6356, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7037] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6359, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7038] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7039] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7040] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6361, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7041] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6364, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7042] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6366, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7043] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6366, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7044] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6368, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7045] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6370, child_count: 6, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7046] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6376, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7047] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6376, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7048] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6376, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7049] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6376, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7050] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6376, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7051] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6376, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7052] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6376, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7053] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6377, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7054] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6377, child_count: 4, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7055] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6381, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7056] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6381, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7057] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6381, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7058] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6383, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7059] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6383, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7060] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7061] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7062] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6387, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7063] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6387, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7064] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6387, child_count: 5, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7065] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6392, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7066] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6392, child_count: 1, min_times: 1, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7067] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6393, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7068] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6395, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7069] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6395, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7070] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6395, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7071] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6395, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7072] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6395, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7073] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6395, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7074] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6395, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7075] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6398, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7076] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6398, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7077] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6400, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7078] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6402, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7079] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6402, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7080] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6402, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7081] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6402, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7082] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6402, child_count: 4, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7083] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6406, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7084] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6406, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7085] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6406, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7086] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6406, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7087] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6406, child_count: 4, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7088] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6410, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7089] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6410, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7090] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6412, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7091] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6414, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7092] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6414, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7093] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6414, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7094] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6416, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7095] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6416, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7096] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6416, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7097] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6416, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7098] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6418, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7099] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6418, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7100] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6418, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7101] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6421, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7102] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6421, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7103] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6423, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7104] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6423, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7105] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6423, child_count: 4, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7106] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6427, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7107] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6430, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7108] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6430, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7109] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7110] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7111] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7112] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6434, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7113] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6434, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7114] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6436, child_count: 4, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7115] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6440, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7116] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6440, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7117] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6440, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7118] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6440, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7119] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6440, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7120] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6443, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7121] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6443, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7122] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6443, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7123] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6446, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7124] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6446, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7125] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6446, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7126] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6446, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7127] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6448, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7128] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6448, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7129] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6448, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6450, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7131] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6450, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7132] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7133] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7134] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7135] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7136] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7137] StringParser("SETTINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7138] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7139] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6452, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7140] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6452, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7141] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6454, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7142] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6456, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7143] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6456, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7144] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6456, child_count: 1, min_times: 1, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7145] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6457, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7146] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6460, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7147] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6460, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7148] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6460, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7149] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6460, child_count: 4, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7150] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6464, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7151] Ref(ShowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6464, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7152] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6464, child_count: 7, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7153] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6471, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7154] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6472, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7155] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6472, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7156] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7157] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6473, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7158] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6475, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7159] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6475, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7160] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6475, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7161] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6476, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7162] Ref(SinkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6476, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7163] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6477, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7164] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6477, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7165] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6478, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7166] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6478, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7167] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6479, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7168] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6479, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7169] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6479, child_count: 6, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7170] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6485, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7171] Ref(ShowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6485, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7172] Ref(IndexesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6485, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7173] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6487, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7174] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6487, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7175] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6487, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7176] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7177] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7178] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7179] Ref(InCluster)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6489, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7180] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6491, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7181] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6493, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7182] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6493, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7183] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6493, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7184] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6495, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7185] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6495, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7186] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6495, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7187] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6495, child_count: 5, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7188] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6500, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7189] Ref(ShowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6500, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7190] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6500, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7191] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6500, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7192] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6502, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7193] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6502, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7194] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6502, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7195] Ref(InCluster)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6502, child_count: 5, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7196] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6507, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7197] Ref(ShowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6507, child_count: 15, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7198] OneOf(15 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6522, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7199] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6522, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7200] Ref(ConnectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6522, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7201] Ref(ClustersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6522, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7202] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7203] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7204] Ref(ReplicasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7205] Ref(DatabasesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7206] Ref(IndexesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6524, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7207] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7208] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7209] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7210] Ref(SecretsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7211] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7212] Ref(SinksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7213] Ref(SourcesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7214] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7215] Ref(TypesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7216] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7217] Ref(ObjectsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7218] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6526, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7219] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6528, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7220] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6528, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7221] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6528, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7222] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6530, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7223] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6532, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7224] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6532, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7225] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6532, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7226] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6534, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7227] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6534, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7228] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6534, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7229] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6534, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7230] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6534, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7231] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6536, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7232] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6536, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7233] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6536, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7234] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6536, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7235] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6537, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7236] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6537, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7237] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6537, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7238] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6540, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7239] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6540, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7240] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6540, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7241] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6540, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 1, _padding: 0 }, // [7242] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6542, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7243] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6542, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7244] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6542, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7245] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6542, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7246] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6544, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7247] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6544, child_count: 3, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7248] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6547, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7249] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6547, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7250] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6547, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7251] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6547, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7252] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6547, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7253] StringParser("SINK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6547, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7254] StringParser("SINKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6547, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7255] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6547, child_count: 2, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7256] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7257] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7258] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7259] StringParser("SKIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7260] StringParser("SKIP_LOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7261] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7262] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7263] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7264] StringParser("SMALLSERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7265] StringParser("SNAPSHOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7266] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7267] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7268] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7269] StringParser("SOURCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7270] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7271] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7272] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7273] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7274] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7275] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7276] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7277] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7278] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7279] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7280] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7281] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7282] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7283] StringParser("SQS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7284] StringParser("SSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7285] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7286] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7287] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7288] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7289] StringParser("STANDALONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7290] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7291] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7292] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7293] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6549, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7294] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6549, child_count: 69, min_times: 0, first_terminator_idx: 71, terminator_count: 1, _padding: 0 }, // [7295] OneOf(69 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7296] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7297] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7298] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7299] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7300] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7301] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7302] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7303] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7304] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7305] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7306] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7307] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7308] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7309] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7310] Ref(CreateSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7311] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7312] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7313] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7314] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7315] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7316] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7317] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7318] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7319] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7320] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7321] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7322] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7323] Ref(CreateModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7324] Ref(DropModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7325] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7326] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7327] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7328] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7329] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7330] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7331] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7332] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7333] Ref(AlterOwnerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7334] Ref(AlterConnectionRotateKeys)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7335] Ref(AlterDefaultPrivilegesStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7336] Ref(AlterIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7337] Ref(AlterRenameStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7338] Ref(AlterSecretStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7339] Ref(AlterSetClusterStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7340] Ref(AlterSourceSinkSizeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7341] Ref(CloseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7342] Ref(CopyToStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7343] Ref(CopyFromStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7344] Ref(CreateClusterStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7345] Ref(CreateClusterReplicaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7346] Ref(CreateConnectionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7347] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7348] Ref(CreateMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7349] Ref(CreateSecretStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7350] Ref(CreateSinkKafkaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7351] Ref(CreateSourceKafkaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7352] Ref(CreateSourceLoadGeneratorStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7353] Ref(CreateSourcePostgresStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7354] Ref(CreateSourceWebhookStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7355] Ref(CreateTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7356] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7357] Ref(DropStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7358] Ref(FetchStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7359] Ref(MaterializeExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7360] Ref(ShowStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7361] Ref(ShowCreateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7362] Ref(ShowIndexesStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7363] Ref(ShowMaterializedViewsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7364] Ref(DeclareStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 71, terminator_count: 0, _padding: 0 }, // [7365] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7366] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7367] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7368] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7369] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7370] StringParser("STORED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7371] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7372] StringParser("STRATEGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7373] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7374] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7375] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7376] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6620, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7377] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6620, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7378] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6620, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7379] StringParser("STRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6620, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7380] StringParser("STRIP")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6620, child_count: 3, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7381] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6623, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7382] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6625, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7383] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7384] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7385] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7386] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7387] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7388] StringParser("SUBSCRIBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7389] StringParser("SUBSCRIPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7390] StringParser("SUBSOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7391] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7392] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7393] StringParser("SUMMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7394] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7395] StringParser("SUPPORT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7396] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7397] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7398] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7399] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7400] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6627, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7401] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6627, child_count: 3, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7402] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6630, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7403] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7404] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7405] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6632, child_count: 5, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7406] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6637, child_count: 3, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7407] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6640, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7408] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6640, child_count: 3, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7409] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6643, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7410] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6643, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7411] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7412] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7413] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6645, child_count: 4, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7414] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6649, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7415] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6649, child_count: 3, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7416] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6652, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7417] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6652, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7418] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6652, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7419] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6652, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7420] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6652, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7421] Ref(IndexParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6652, child_count: 3, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7422] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6655, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7423] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6655, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7424] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6655, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7425] Ref(IndexParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6655, child_count: 5, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7426] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6660, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7427] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6660, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7428] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6662, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7429] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6662, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7430] Ref(IndexAccessMethodSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6662, child_count: 3, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7431] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6665, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7432] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6667, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7433] Ref(ExclusionConstraintElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6667, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7434] Ref(IndexParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6667, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7435] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6669, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7436] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6669, child_count: 3, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7437] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6672, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7438] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6672, child_count: 4, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7439] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6676, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7440] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6676, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7441] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6676, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7442] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6676, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7443] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6676, child_count: 4, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7444] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6680, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7445] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6682, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7446] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6682, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7447] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6684, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7448] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6684, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7449] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6684, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7450] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6686, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7451] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6688, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7452] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6688, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7453] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6688, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7454] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6690, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7455] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6690, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7456] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6690, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7457] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6692, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7458] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6692, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7459] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6692, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7460] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6694, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7461] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6694, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7462] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6694, child_count: 4, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7463] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6698, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7464] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6700, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7465] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6700, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7466] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6700, child_count: 4, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7467] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6704, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7468] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6706, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7469] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6706, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7470] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6706, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7471] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6706, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7472] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6706, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7473] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6706, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7474] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6708, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7475] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6708, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7476] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6710, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7477] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6710, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7478] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6710, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7479] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6712, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7480] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6714, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7481] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6714, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7482] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6714, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7483] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6716, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7484] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6716, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7485] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6716, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7486] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6716, child_count: 6, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7487] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6722, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7488] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6722, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7489] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6722, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7490] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6724, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7491] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6724, child_count: 2, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7492] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6726, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7493] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6726, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7494] Ref(OrdinalityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6726, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7495] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6726, child_count: 3, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7496] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6729, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7497] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6729, child_count: 3, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7498] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6732, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7499] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6732, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7500] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6732, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7501] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6732, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7502] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6732, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7503] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6732, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7504] StringParser("TAIL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6732, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7505] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6734, child_count: 1, min_times: 0, first_terminator_idx: 72, terminator_count: 1, _padding: 0 }, // [7506] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6735, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7507] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6735, child_count: 0, min_times: 0, first_terminator_idx: 72, terminator_count: 0, _padding: 0 }, // [7508] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6735, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7509] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6735, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7510] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6737, child_count: 1, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7511] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6738, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7512] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6738, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7513] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6738, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7514] StringParser("TARGET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6738, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7515] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6738, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7516] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6738, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7517] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6738, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7518] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6738, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7519] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6738, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7520] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6740, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7521] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6740, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7522] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6740, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7523] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6740, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7524] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7525] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7526] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7527] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7528] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7529] StringParser("TEST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7530] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7531] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7532] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7533] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7534] StringParser("TICK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7535] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7536] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6742, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7537] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6742, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7538] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6745, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7539] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6747, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7540] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6747, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7541] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6747, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7542] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6747, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7543] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6750, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7544] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6752, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7545] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6752, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7546] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6752, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7547] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6752, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7548] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6752, child_count: 1, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7549] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6753, child_count: 4, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7550] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7551] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7552] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7553] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7554] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7555] StringParser("TIMELINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7556] StringParser("TIMEOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7557] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7558] StringParser("TIMESTAMPTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7559] StringParser("TIMETZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7560] StringParser("TIMING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7561] StringParser("TIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7562] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7563] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7564] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7565] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7566] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7567] StringParser("TOKEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7568] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7569] StringParser("TOPIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7570] StringParser("TPCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7571] StringParser("TRACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7572] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7573] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6757, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7574] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6757, child_count: 4, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7575] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6761, child_count: 5, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7576] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6766, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7577] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6766, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7578] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6766, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7579] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6766, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7580] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6766, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7581] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6766, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7582] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6768, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7583] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6768, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7584] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6768, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7585] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6770, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7586] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6770, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7587] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6770, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7588] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6773, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7589] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6773, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7590] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6773, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7591] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6773, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7592] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6773, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7593] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6773, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7594] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6773, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7595] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6773, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7596] StringParser("TRIANGLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6773, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7597] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6773, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7598] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6773, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7599] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6776, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7600] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6776, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7601] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6776, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7602] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6776, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7603] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6776, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7604] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6776, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7605] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6776, child_count: 5, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7606] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6781, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7607] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6781, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7608] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6781, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7609] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6783, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7610] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6785, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7611] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6787, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7612] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6787, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7613] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6787, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7614] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6789, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7615] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6789, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7616] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6789, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7617] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6791, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7618] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7619] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7620] Ref(ContinueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7621] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7622] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7623] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7624] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7625] StringParser("TSQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7626] StringParser("TSRANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7627] StringParser("TSTZRANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7628] StringParser("TSVECTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7629] StringParser("TUNNEL")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6793, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7630] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6796, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7631] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6798, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7632] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6798, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7633] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6798, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7634] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6801, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7635] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6803, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7636] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6803, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7637] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6805, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7638] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6805, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7639] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6805, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7640] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6805, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7641] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6805, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7642] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7643] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7644] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7645] StringParser("TYPES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7646] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7647] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7648] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7649] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7650] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6807, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7651] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7652] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7653] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7654] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7655] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7656] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6810, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7657] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6812, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7658] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6812, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7659] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6814, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7660] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6814, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7661] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6814, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7662] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6814, child_count: 1, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7663] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6815, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7664] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6815, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7665] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6815, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7666] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6815, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7667] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6815, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7668] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6815, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7669] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6817, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7670] Ref(UnlistenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6817, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7671] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6819, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7672] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6819, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7673] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6819, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7674] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6819, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7675] StringParser("UNLOGGED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 6819, child_count: 8, min_times: 0, first_terminator_idx: 73, terminator_count: 10, _padding: 0 }, // [7676] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7677] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7678] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7679] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7680] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7681] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7682] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7683] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7684] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7685] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7686] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7687] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7688] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7689] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6827, child_count: 3, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7690] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6830, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7691] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6830, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7692] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6830, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7693] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6830, child_count: 2, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7694] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6832, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7695] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6832, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7696] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6832, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7697] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6832, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7698] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6832, child_count: 0, min_times: 0, first_terminator_idx: 73, terminator_count: 0, _padding: 0 }, // [7699] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6832, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7700] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6834, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7701] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6834, child_count: 1, min_times: 1, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7702] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6835, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7703] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7704] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7705] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7706] StringParser("UNSAFE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7707] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7708] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7709] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6837, child_count: 10, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7710] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6847, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7711] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6847, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7712] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6847, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7713] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 6847, child_count: 1, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7714] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6848, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7715] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6848, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7716] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6848, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7717] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6848, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7718] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6850, child_count: 4, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7719] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6854, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7720] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6854, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7721] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6854, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7722] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6854, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7723] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6854, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7724] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6854, child_count: 4, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7725] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6858, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7726] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6858, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7727] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6860, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7728] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6860, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7729] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6862, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7730] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6864, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7731] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6864, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7732] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6864, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7733] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6864, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7734] StringParser("UPSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6864, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7735] StringParser("URL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6864, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7736] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6864, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7737] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6864, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7738] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7739] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7740] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7741] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7742] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7743] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7744] StringParser("USERNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7745] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7746] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7747] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7748] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7749] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7750] StringParser("UUID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7751] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6866, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7752] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6869, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7753] Ref(VacuumKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6869, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7754] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6871, child_count: 4, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7755] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6875, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7756] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6875, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7757] Ref(FreezeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6875, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7758] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6875, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7759] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6877, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7760] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6877, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7761] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6877, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7762] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6880, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7763] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6882, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7764] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6884, child_count: 11, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7765] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7766] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7767] Ref(FreezeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7768] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7769] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7770] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7771] Ref(Disable_page_skippingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7772] Ref(Skip_lockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7773] Ref(Index_cleanupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7774] Ref(Process_toastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7775] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7776] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6895, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7777] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6898, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7778] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6898, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7779] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6898, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7780] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6898, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7781] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6900, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7782] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6902, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7783] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6902, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7784] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6902, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7785] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6902, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7786] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6902, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7787] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6902, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7788] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6902, child_count: 5, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7789] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6907, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7790] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6907, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7791] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 6909, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7792] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6912, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7793] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6914, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7794] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7795] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7796] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7797] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7798] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7799] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7800] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7801] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7802] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7803] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7804] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7805] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7806] StringParser("VARIADIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7807] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7808] StringParser("VECTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7809] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6916, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7810] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7811] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7812] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7813] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7814] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7815] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7816] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7817] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7818] StringParser("WAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7819] StringParser(":=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7820] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7821] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7822] StringParser("WARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7823] StringParser("WEBHOOK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7824] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6918, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7825] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6918, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7826] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6920, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7827] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6922, child_count: 36, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7828] OneOf(36 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7829] Ref(PointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7830] Ref(LinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7831] Ref(PolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7832] Ref(MultipointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7833] Ref(MultilinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7834] Ref(MultipolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7835] Ref(GeometrycollectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7836] Ref(PointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7837] Ref(LinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7838] Ref(PolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7839] Ref(MultipointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7840] Ref(MultilinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7841] Ref(MultipolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7842] Ref(GeometrycollectionzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7843] Ref(PointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7844] Ref(LinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7845] Ref(PolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7846] Ref(MultipointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7847] Ref(MultilinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7848] Ref(MultipolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7849] Ref(GeometrycollectionmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7850] Ref(PointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7851] Ref(LinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7852] Ref(PolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7853] Ref(MultipointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7854] Ref(MultilinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7855] Ref(MultipolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7856] Ref(GeometrycollectionzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7857] Ref(CircularstringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7858] Ref(CompoundcurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7859] Ref(CurvepolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7860] Ref(MulticurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7861] Ref(MultisurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7862] Ref(PolyhedralsurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7863] Ref(TriangleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7864] Ref(TinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6958, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7865] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6961, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7866] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6963, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7867] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6966, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7868] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6968, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7869] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6971, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7870] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6973, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7871] Ref(SimpleGeometryGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6973, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7872] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6976, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7873] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6978, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7874] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6981, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7875] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6983, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7876] Ref(SimpleGeometryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6983, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7877] Ref(WellKnownTextGeometrySegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6983, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7878] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6985, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7879] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6987, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7880] Ref(GeometryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6987, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7881] Ref(GeographyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6987, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7882] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6990, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7883] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6992, child_count: 38, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7884] OneOf(38 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7885] Ref(PointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7886] Ref(LinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7887] Ref(PolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7888] Ref(MultipointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7889] Ref(MultilinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7890] Ref(MultipolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7891] Ref(GeometrycollectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7892] Ref(PointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7893] Ref(LinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7894] Ref(PolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7895] Ref(MultipointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7896] Ref(MultilinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7897] Ref(MultipolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7898] Ref(GeometrycollectionzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7899] Ref(PointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7900] Ref(LinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7901] Ref(PolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7902] Ref(MultipointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7903] Ref(MultilinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7904] Ref(MultipolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7905] Ref(GeometrycollectionmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7906] Ref(PointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7907] Ref(LinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7908] Ref(PolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7909] Ref(MultipointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7910] Ref(MultilinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7911] Ref(MultipolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7912] Ref(GeometrycollectionzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7913] Ref(CircularstringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7914] Ref(CompoundcurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7915] Ref(CurvepolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7916] Ref(MulticurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7917] Ref(MultisurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7918] Ref(PolyhedralsurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7919] Ref(TriangleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7920] Ref(TinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7921] Ref(GeometryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7030, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7922] Ref(GeographyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7030, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7923] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7032, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7924] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7032, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7925] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7032, child_count: 8, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7926] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7040, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7927] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7040, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7928] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7929] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7930] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7931] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7932] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7933] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7934] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7935] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7936] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7043, child_count: 4, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7937] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7047, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7938] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7047, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7939] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7049, child_count: 3, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7940] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7052, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7941] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7052, child_count: 10, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7942] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7062, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7943] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7062, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7944] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7064, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7945] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7064, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7946] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7064, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7947] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7066, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7948] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7066, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7949] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7066, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7950] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7066, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7951] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7066, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7952] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7066, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7953] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7066, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7954] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7066, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7955] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7068, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7956] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7068, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7957] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7068, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7958] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7068, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7959] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7068, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7960] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7068, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7961] StringParser("WHITESPACE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7068, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7962] Token("whitespace")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7068, child_count: 1, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7963] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7069, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7964] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7069, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7965] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7071, child_count: 1, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7966] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7072, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7967] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7074, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7968] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7076, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7969] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7076, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7970] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7076, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7971] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7078, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7972] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7078, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7973] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7078, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7974] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7078, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7975] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7078, child_count: 4, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7976] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 7082, child_count: 1, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7977] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7083, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7978] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7085, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7979] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7085, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7980] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7085, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7981] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7085, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7982] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7085, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7983] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7085, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7984] StringParser("WIRE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7085, child_count: 4, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7985] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7089, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7986] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7089, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7987] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7091, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7988] Ref(CascadedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7091, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7989] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7091, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7990] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7091, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7991] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7091, child_count: 6, min_times: 0, first_terminator_idx: 84, terminator_count: 0, _padding: 0 }, // [7992] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7097, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7993] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7097, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7994] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7097, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7995] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7097, child_count: 2, min_times: 0, first_terminator_idx: 83, terminator_count: 1, _padding: 0 }, // [7996] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7099, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7997] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7099, child_count: 0, min_times: 0, first_terminator_idx: 83, terminator_count: 0, _padding: 0 }, // [7998] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7099, child_count: 0, min_times: 0, first_terminator_idx: 84, terminator_count: 0, _padding: 0 }, // [7999] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7099, child_count: 0, min_times: 0, first_terminator_idx: 84, terminator_count: 0, _padding: 0 }, // [8000] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7099, child_count: 6, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8001] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7105, child_count: 0, min_times: 0, first_terminator_idx: 84, terminator_count: 0, _padding: 0 }, // [8002] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7105, child_count: 0, min_times: 0, first_terminator_idx: 84, terminator_count: 0, _padding: 0 }, // [8003] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7105, child_count: 0, min_times: 0, first_terminator_idx: 84, terminator_count: 0, _padding: 0 }, // [8004] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7105, child_count: 2, min_times: 0, first_terminator_idx: 84, terminator_count: 1, _padding: 0 }, // [8005] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7107, child_count: 0, min_times: 0, first_terminator_idx: 84, terminator_count: 0, _padding: 0 }, // [8006] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7107, child_count: 0, min_times: 0, first_terminator_idx: 84, terminator_count: 0, _padding: 0 }, // [8007] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7107, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8008] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7107, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8009] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7107, child_count: 3, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8010] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7110, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8011] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7110, child_count: 1, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8012] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7111, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8013] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7111, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8014] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7111, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8015] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7111, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8016] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7111, child_count: 4, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8017] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7115, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8018] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7115, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8019] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7115, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8020] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7115, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8021] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7115, child_count: 3, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8022] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7118, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8023] Ref(WithinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7118, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8024] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7118, child_count: 3, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8025] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8026] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8027] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8028] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8029] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8030] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8031] StringParser("WORKERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8032] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8033] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8034] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8035] StringParser("X509")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8036] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8037] StringParser("XMLATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8038] StringParser("XMLCONCAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8039] StringParser("XMLELEMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8040] StringParser("XMLEXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8041] StringParser("XMLFOREST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8042] StringParser("XMLNAMESPACES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8043] StringParser("XMLPARSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8044] StringParser("XMLPI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8045] StringParser("XMLROOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8046] StringParser("XMLSERIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8047] StringParser("XMLTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8048] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8049] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8050] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8051] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8052] StringParser("YEARS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8053] StringParser("YES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8054] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8055] StringParser("ZONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7121, child_count: 0, min_times: 0, first_terminator_idx: 85, terminator_count: 0, _padding: 0 }, // [8056] StringParser("ZONES")
];

pub static CHILD_IDS: &[u32] = &[
    4, 16, 17, 5, 13, 6, 7, 8, 9, 10, 11, 12, 14, 15, 18, 19,
    20, 30, 41, 42, 43, 21, 26, 27, 28, 22, 23, 24, 25, 29, 15, 31,
    37, 38, 39, 32, 33, 34, 35, 36, 40, 15, 45, 46, 47, 48, 49, 50,
    51, 52, 53, 54, 55, 56, 58, 15, 61, 62, 63, 64, 65, 66, 67, 68,
    69, 70, 71, 74, 77, 80, 72, 73, 75, 76, 78, 79, 81, 82, 84, 85,
    86, 87, 88, 89, 90, 91, 92, 93, 94, 97, 100, 103, 95, 96, 98, 99,
    101, 102, 104, 105, 107, 108, 110, 15, 112, 113, 120, 121, 122, 128, 129, 130,
    144, 131, 137, 132, 133, 134, 135, 136, 138, 139, 140, 135, 136, 141, 15, 142,
    143, 147, 148, 153, 154, 155, 156, 161, 157, 135, 136, 158, 159, 160, 162, 166,
    174, 163, 164, 165, 167, 168, 169, 170, 171, 172, 173, 175, 176, 177, 179, 180,
    181, 182, 183, 184, 186, 187, 188, 189, 190, 203, 207, 215, 219, 235, 191, 192,
    193, 196, 200, 194, 195, 197, 198, 199, 201, 202, 204, 205, 206, 208, 209, 210,
    211, 212, 213, 214, 216, 217, 218, 220, 221, 222, 223, 232, 224, 227, 225, 226,
    228, 229, 230, 231, 233, 234, 236, 237, 238, 239, 241, 242, 243, 244, 245, 246,
    249, 247, 15, 250, 251, 252, 254, 257, 255, 256, 258, 15, 259, 260, 261, 262,
    263, 264, 265, 266, 267, 268, 271, 272, 276, 277, 278, 279, 280, 284, 273, 274,
    275, 281, 15, 286, 287, 288, 289, 290, 291, 293, 298, 308, 314, 315, 316, 317,
    325, 326, 294, 295, 296, 297, 299, 302, 305, 300, 301, 303, 304, 306, 307, 309,
    310, 313, 311, 312, 318, 319, 320, 321, 322, 323, 324, 328, 331, 329, 330, 333,
    334, 335, 336, 337, 341, 344, 350, 365, 373, 379, 383, 391, 395, 338, 339, 340,
    342, 343, 345, 348, 349, 346, 347, 351, 352, 355, 362, 353, 354, 356, 359, 357,
    358, 360, 361, 363, 364, 366, 367, 368, 369, 370, 371, 372, 374, 375, 376, 377,
    378, 380, 381, 382, 384, 385, 386, 387, 388, 389, 390, 392, 393, 394, 396, 397,
    398, 400, 401, 402, 403, 404, 409, 415, 405, 406, 407, 408, 410, 411, 412, 413,
    414, 416, 419, 417, 418, 420, 454, 462, 468, 475, 484, 502, 421, 453, 422, 425,
    426, 427, 428, 431, 435, 438, 441, 442, 443, 444, 452, 423, 424, 429, 430, 432,
    433, 434, 436, 437, 439, 440, 445, 446, 447, 448, 449, 450, 451, 455, 461, 456,
    459, 460, 457, 458, 463, 464, 465, 466, 135, 136, 467, 469, 470, 471, 135, 136,
    472, 473, 474, 476, 480, 477, 478, 479, 481, 15, 482, 483, 485, 486, 487, 495,
    488, 489, 490, 135, 136, 491, 15, 492, 493, 494, 496, 499, 500, 501, 497, 498,
    503, 504, 505, 506, 508, 519, 525, 622, 625, 628, 644, 648, 654, 663, 670, 682,
    693, 697, 703, 707, 712, 715, 718, 722, 725, 728, 736, 747, 509, 510, 511, 512,
    513, 514, 517, 515, 516, 518, 520, 521, 522, 523, 524, 526, 527, 528, 529, 530,
    543, 552, 555, 561, 565, 578, 596, 600, 604, 607, 610, 618, 531, 534, 535, 536,
    539, 532, 533, 537, 538, 540, 541, 542, 544, 545, 546, 547, 548, 549, 550, 551,
    553, 554, 556, 559, 560, 557, 558, 562, 563, 564, 566, 567, 568, 573, 574, 575,
    569, 570, 571, 572, 576, 135, 136, 577, 579, 580, 588, 591, 581, 582, 583, 584,
    585, 586, 587, 589, 590, 592, 593, 594, 595, 597, 598, 599, 601, 602, 603, 605,
    606, 608, 609, 611, 612, 613, 614, 615, 616, 617, 619, 620, 621, 623, 624, 626,
    627, 629, 630, 631, 632, 637, 633, 634, 635, 636, 638, 641, 639, 640, 642, 643,
    645, 646, 647, 649, 650, 651, 652, 653, 655, 658, 659, 656, 657, 660, 661, 662,
    664, 665, 668, 669, 666, 667, 671, 680, 681, 672, 673, 674, 677, 675, 676, 678,
    679, 683, 690, 691, 692, 684, 685, 686, 687, 688, 689, 694, 695, 696, 698, 699,
    700, 701, 702, 704, 705, 706, 708, 709, 710, 711, 713, 714, 716, 717, 719, 720,
    721, 723, 724, 726, 727, 729, 730, 731, 732, 733, 734, 735, 737, 738, 739, 740,
    741, 745, 746, 742, 743, 744, 748, 752, 753, 749, 750, 751, 754, 135, 136, 755,
    15, 756, 760, 761, 757, 758, 759, 763, 764, 765, 766, 767, 768, 769, 770, 771,
    772, 774, 780, 773, 15, 775, 776, 777, 778, 779, 781, 782, 783, 785, 847, 786,
    799, 803, 806, 812, 818, 821, 824, 827, 842, 787, 792, 798, 788, 789, 790, 791,
    793, 794, 795, 796, 797, 800, 801, 802, 804, 805, 807, 808, 809, 810, 811, 813,
    814, 815, 816, 817, 819, 820, 822, 823, 825, 826, 828, 829, 830, 831, 839, 832,
    835, 833, 834, 836, 837, 838, 840, 841, 843, 844, 845, 846, 849, 850, 851, 855,
    852, 15, 853, 854, 856, 857, 861, 865, 875, 858, 859, 860, 862, 863, 864, 866,
    867, 868, 869, 872, 873, 874, 870, 871, 876, 877, 878, 879, 880, 882, 883, 884,
    885, 886, 887, 890, 924, 928, 932, 941, 946, 891, 892, 893, 894, 895, 899, 907,
    912, 920, 896, 897, 898, 900, 901, 902, 135, 136, 903, 15, 904, 905, 906, 908,
    909, 910, 135, 136, 911, 15, 913, 914, 915, 916, 917, 918, 919, 921, 922, 923,
    925, 926, 927, 929, 930, 931, 933, 934, 935, 135, 136, 936, 15, 937, 938, 939,
    940, 942, 943, 944, 135, 136, 945, 15, 947, 948, 949, 950, 951, 952, 953, 955,
    956, 957, 958, 959, 980, 987, 960, 961, 962, 963, 965, 972, 976, 964, 15, 966,
    967, 969, 970, 971, 968, 973, 974, 975, 977, 978, 979, 981, 982, 983, 984, 985,
    986, 988, 989, 990, 991, 992, 997, 998, 999, 1000, 993, 994, 995, 996, 15, 1001,
    1003, 1004, 1019, 1020, 1023, 1005, 1006, 1007, 1010, 1011, 1012, 1013, 1014, 1017, 1018, 1008,
    1009, 1015, 1016, 1021, 1022, 1025, 1026, 1027, 1028, 1029, 1030, 1031, 1035, 1032, 1033, 1034,
    1036, 1045, 1049, 1037, 1038, 1039, 15, 1040, 1041, 1042, 1043, 1044, 1046, 1047, 1048, 135,
    136, 1050, 1051, 1052, 1053, 135, 136, 1055, 1082, 1056, 1062, 1077, 1057, 1058, 1059, 1060,
    1061, 1063, 1064, 1065, 1066, 1074, 1067, 1070, 1068, 1069, 1071, 1072, 1073, 1075, 1076, 1078,
    1079, 1080, 1081, 1084, 1085, 1086, 1090, 1087, 15, 1088, 1089, 1091, 1092, 1096, 1100, 1115,
    1125, 1093, 1094, 1095, 1097, 1098, 1099, 1101, 1102, 1103, 1104, 1112, 1105, 1108, 1106, 1107,
    1109, 15, 1110, 1111, 1113, 1114, 1116, 1117, 1118, 1119, 1122, 1123, 1124, 1120, 1121, 1126,
    1127, 1128, 1129, 1130, 1132, 1133, 1134, 1135, 1136, 1139, 1143, 1147, 1151, 1155, 1137, 1138,
    1140, 1141, 1142, 15, 1144, 1145, 1146, 15, 1148, 1149, 1150, 15, 1152, 1153, 1154, 1156,
    1157, 1158, 1159, 1160, 1161, 1162, 1164, 1165, 1179, 1180, 1183, 1166, 1167, 1170, 1171, 1172,
    1173, 1174, 1177, 1178, 1168, 1169, 1175, 1176, 1181, 1182, 1185, 1186, 1189, 1187, 1188, 1190,
    1233, 1239, 1191, 1196, 1197, 1192, 1193, 1194, 1195, 1198, 1201, 1204, 1207, 1210, 1213, 1216,
    1219, 1223, 1229, 1199, 1200, 1202, 1203, 1205, 1206, 1208, 1209, 1211, 1212, 1214, 1215, 1217,
    1218, 1220, 1221, 1222, 1224, 1225, 1226, 1227, 1228, 1230, 1231, 1232, 1234, 1235, 1236, 1237,
    1238, 1240, 1246, 1250, 1241, 1242, 1243, 1244, 1245, 1247, 1248, 1249, 1251, 1269, 1252, 1253,
    1254, 1255, 1266, 1256, 1259, 1257, 1258, 1260, 1261, 1262, 15, 1263, 1264, 1265, 1267, 1268,
    1270, 1271, 1272, 1273, 1275, 1276, 1277, 1278, 1279, 1283, 1280, 1281, 1282, 1284, 1285, 1286,
    1288, 1289, 1290, 1291, 1292, 1293, 1295, 1298, 1303, 1311, 1319, 1323, 1328, 1333, 1336, 1339,
    1296, 1297, 1299, 1300, 1301, 1302, 1304, 1308, 1305, 1306, 1307, 1309, 1310, 1312, 1316, 1313,
    1314, 1315, 1317, 1318, 1320, 1321, 1322, 1324, 1325, 1326, 1327, 1329, 1330, 1331, 1332, 1334,
    1335, 1337, 1338, 1340, 1341, 1342, 1343, 1344, 1346, 1347, 1348, 1349, 1350, 1351, 1353, 1360,
    1364, 1352, 1354, 1355, 1356, 1357, 1358, 1359, 1361, 1362, 1363, 1365, 1366, 1367, 1369, 1373,
    1374, 1375, 1378, 1370, 1371, 1372, 1376, 1377, 1380, 1381, 1384, 1385, 1386, 1387, 1382, 1383,
    1388, 1389, 135, 136, 1391, 1392, 1393, 1394, 1395, 1396, 1406, 1410, 1397, 1398, 1399, 1400,
    1403, 1404, 1405, 1401, 1402, 1407, 1408, 1409, 1411, 1412, 1413, 1416, 1414, 1415, 1417, 1418,
    1420, 1421, 1422, 1423, 1424, 1427, 1438, 1444, 1445, 1446, 1449, 1455, 1463, 1425, 1426, 1428,
    1432, 1433, 1435, 1429, 1430, 1431, 1434, 15, 1436, 1437, 1439, 1440, 1441, 1442, 1443, 1447,
    1448, 1450, 1451, 1452, 1453, 1454, 135, 136, 1456, 1457, 1458, 1459, 1460, 1461, 1462, 1464,
    1465, 1466, 1468, 1469, 1470, 1471, 1485, 1472, 1473, 1474, 1477, 1475, 1476, 1478, 1479, 1480,
    15, 1481, 1482, 1483, 1484, 1486, 1487, 1488, 1489, 1491, 1502, 1508, 1605, 1608, 1611, 1627,
    1631, 1637, 1646, 1653, 1665, 1676, 1680, 1686, 1690, 1695, 1698, 1701, 1705, 1708, 1711, 1719,
    1492, 1493, 1494, 1495, 1496, 1497, 1500, 1498, 1499, 1501, 1503, 1504, 1505, 1506, 1507, 1509,
    1510, 1511, 1512, 1513, 1526, 1535, 1538, 1544, 1548, 1561, 1579, 1583, 1587, 1590, 1593, 1601,
    1514, 1517, 1518, 1519, 1522, 1515, 1516, 1520, 1521, 1523, 1524, 1525, 1527, 1528, 1529, 1530,
    1531, 1532, 1533, 1534, 1536, 1537, 1539, 1542, 1543, 1540, 1541, 1545, 1546, 1547, 1549, 1550,
    1551, 1556, 1557, 1558, 1552, 1553, 1554, 1555, 1559, 135, 136, 1560, 1562, 1563, 1571, 1574,
    1564, 1565, 1566, 1567, 1568, 1569, 1570, 1572, 1573, 1575, 1576, 1577, 1578, 1580, 1581, 1582,
    1584, 1585, 1586, 1588, 1589, 1591, 1592, 1594, 1595, 1596, 1597, 1598, 1599, 1600, 1602, 1603,
    1604, 1606, 1607, 1609, 1610, 1612, 1613, 1614, 1615, 1620, 1616, 1617, 1618, 1619, 1621, 1624,
    1622, 1623, 1625, 1626, 1628, 1629, 1630, 1632, 1633, 1634, 1635, 1636, 1638, 1641, 1642, 1639,
    1640, 1643, 1644, 1645, 1647, 1648, 1651, 1652, 1649, 1650, 1654, 1663, 1664, 1655, 1656, 1657,
    1660, 1658, 1659, 1661, 1662, 1666, 1673, 1674, 1675, 1667, 1668, 1669, 1670, 1671, 1672, 1677,
    1678, 1679, 1681, 1682, 1683, 1684, 1685, 1687, 1688, 1689, 1691, 1692, 1693, 1694, 1696, 1697,
    1699, 1700, 1702, 1703, 1704, 1706, 1707, 1709, 1710, 1712, 1713, 1714, 1715, 1716, 1717, 1718,
    1720, 1721, 1722, 1723, 1724, 1728, 1729, 1725, 1726, 1727, 1731, 1732, 1733, 1734, 1736, 1742,
    1755, 1756, 1737, 1738, 1739, 1740, 1741, 1743, 1746, 1747, 1748, 1744, 1745, 1749, 1754, 1750,
    1753, 1751, 1752, 1757, 1758, 1761, 1759, 1760, 1763, 1764, 1765, 1766, 1786, 1814, 1767, 1768,
    1769, 1770, 1771, 1772, 1774, 1780, 1773, 15, 1775, 1776, 1777, 1778, 1779, 1781, 1782, 1783,
    1784, 1785, 1787, 1788, 1789, 1790, 1794, 1798, 1808, 1791, 1792, 1793, 1795, 1796, 1797, 1799,
    1800, 1801, 1802, 1803, 1807, 1804, 1805, 1806, 1809, 1810, 1811, 1812, 1813, 1815, 1816, 1817,
    1818, 1819, 1824, 1825, 1826, 1827, 1820, 1821, 1822, 1823, 15, 1829, 1830, 1831, 1832, 1833,
    1834, 1835, 1844, 1853, 1860, 1870, 1877, 1881, 1889, 1836, 1837, 1838, 1839, 1841, 1842, 1840,
    15, 1843, 15, 1845, 1846, 1847, 1848, 1850, 1851, 1849, 15, 1852, 15, 1854, 1855, 1856,
    1857, 1858, 1859, 1861, 1862, 1863, 1864, 1866, 1867, 1868, 1869, 1865, 15, 1871, 1872, 1873,
    1874, 1875, 1876, 15, 1878, 1879, 1880, 1882, 1883, 1884, 1885, 1886, 1887, 1888, 1890, 1891,
    1892, 1894, 1895, 1896, 1897, 1898, 1899, 1900, 1904, 1901, 1902, 1903, 1905, 1906, 1907, 1908,
    1909, 1911, 1912, 1913, 1914, 1915, 1923, 1929, 1933, 1937, 1974, 1916, 1917, 1918, 1919, 1920,
    1921, 1922, 1924, 1925, 1926, 1927, 1928, 1930, 1931, 1932, 1934, 1935, 1936, 1938, 15, 1939,
    1948, 1961, 1967, 1940, 1941, 1942, 1943, 1944, 1947, 1945, 1946, 1949, 1950, 1951, 1952, 1955,
    1956, 1957, 1960, 1953, 1954, 1958, 1959, 1962, 1963, 1964, 1965, 1966, 1968, 1969, 1970, 1971,
    1972, 1973, 1975, 1976, 1977, 1978, 1979, 1980, 1983, 1981, 1982, 1985, 1986, 1987, 1988, 1989,
    1990, 2006, 2014, 2020, 2024, 2028, 2037, 1991, 1992, 1993, 1994, 1995, 2003, 1996, 1997, 1998,
    1999, 2000, 2001, 2002, 2004, 2005, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2015, 2016, 2017,
    2018, 2019, 2021, 2022, 2023, 2025, 2026, 2027, 2029, 2030, 2031, 135, 136, 2032, 15, 2033,
    2034, 2035, 2036, 2038, 2039, 2040, 135, 136, 2041, 15, 2047, 2050, 2059, 2048, 2049, 2051,
    2058, 2052, 135, 136, 2053, 15, 2054, 2057, 2055, 2056, 2060, 15, 2061, 2062, 2063, 135,
    136, 2064, 15, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079, 2083, 2097, 2098,
    2084, 2085, 2086, 2087, 2088, 2092, 2093, 2089, 2090, 2091, 2094, 2095, 2096, 2102, 2097, 2098,
    2103, 15, 2106, 2097, 2098, 128, 2108, 2109, 144, 2111, 2126, 2136, 2148, 2137, 2138, 2139,
    2140, 2141, 2142, 2143, 2144, 2145, 2159, 2160, 2161, 2162, 2163, 2170, 2171, 2174, 2175, 2180,
    2181, 2184, 2185, 2190, 135, 136, 2191, 15, 2193, 135, 136, 2194, 15, 2197, 135, 136,
    2211, 135, 136, 2213, 2214, 2215, 2216, 2219, 2221, 2217, 2218, 2220, 135, 136, 2222, 2232,
    2223, 2224, 2227, 2228, 2229, 2230, 2231, 2225, 2226, 2233, 2234, 2235, 2236, 2237, 2238, 2242,
    2243, 2247, 2248, 2251, 2261, 2252, 2253, 2254, 2258, 144, 2260, 2255, 2262, 2263, 2253, 2264,
    2268, 144, 2270, 2265, 2296, 2297, 2300, 2301, 2302, 2303, 2308, 2304, 2305, 2306, 2307, 2309,
    2310, 2311, 2320, 2321, 2322, 2323, 2329, 2330, 2331, 2332, 2334, 2337, 2400, 2405, 2335, 2336,
    2338, 2341, 2348, 2356, 2362, 2374, 2388, 2399, 2339, 2340, 2342, 2343, 2345, 2344, 135, 136,
    2346, 2347, 2349, 2350, 2351, 2352, 2353, 2354, 2355, 2357, 2358, 2359, 2360, 2361, 2363, 2364,
    2369, 2370, 2371, 2365, 2366, 2367, 2368, 2372, 135, 136, 2373, 2375, 2376, 2380, 2383, 2377,
    2378, 2379, 2381, 2382, 2384, 2385, 2386, 2387, 2389, 2390, 2391, 2394, 2392, 2393, 2395, 2396,
    2397, 2398, 2401, 2402, 2403, 2404, 2406, 2409, 2407, 2408, 2410, 2411, 2413, 2414, 2415, 2416,
    2417, 2418, 2419, 2423, 2424, 2425, 2430, 2426, 2427, 2428, 2429, 2431, 2432, 2433, 2434, 2435,
    2436, 2449, 2450, 2451, 2459, 2460, 2463, 2464, 2465, 2466, 2568, 2467, 2472, 2479, 2482, 2489,
    2492, 2495, 2500, 2503, 2506, 2559, 2468, 2471, 2469, 2470, 2473, 2474, 2475, 135, 136, 2476,
    2477, 2478, 2480, 2481, 2483, 2484, 2485, 2486, 2487, 2488, 2490, 2491, 2493, 2494, 2496, 2497,
    2498, 2499, 2501, 2502, 2504, 2505, 2507, 2555, 2556, 2508, 2509, 2510, 2511, 2512, 2513, 2514,
    2515, 2516, 2517, 2518, 2519, 2520, 2521, 2522, 2523, 2526, 2529, 2533, 2536, 2539, 2543, 2547,
    2551, 2524, 2525, 2527, 2528, 2530, 2531, 2532, 2534, 2535, 2537, 2538, 2540, 2541, 2542, 2544,
    2545, 2546, 2548, 2549, 2550, 2552, 2553, 2554, 2557, 2558, 2560, 2564, 2565, 2561, 2562, 2563,
    2566, 135, 136, 2567, 2569, 2570, 2571, 2572, 2579, 2580, 2581, 2582, 2583, 2584, 2585, 2586,
    2590, 2595, 2596, 2597, 2598, 2599, 2600, 2601, 2602, 2587, 2588, 2589, 2591, 2592, 2593, 2594,
    2608, 2610, 2611, 2609, 135, 136, 2617, 2618, 2621, 2622, 2623, 2626, 2627, 2628, 2629, 2630,
    128, 2631, 2253, 2632, 144, 2658, 144, 2633, 15, 2634, 2640, 2651, 2635, 2636, 2637, 2638,
    2639, 2641, 2644, 2645, 2646, 2642, 135, 136, 2643, 15, 2647, 135, 136, 2648, 15, 2649,
    2650, 2652, 2655, 2656, 2653, 135, 136, 2654, 15, 2657, 135, 136, 2659, 2660, 2663, 2679,
    2664, 2676, 2665, 135, 136, 2666, 15, 2667, 2672, 2675, 2668, 2669, 2671, 2670, 135, 136,
    2673, 2674, 2677, 2678, 2680, 2681, 2682, 2694, 2695, 2696, 2698, 2699, 2700, 2697, 135, 136,
    2701, 2703, 2702, 2704, 135, 136, 2707, 2708, 2709, 2776, 2814, 2822, 2710, 2715, 2716, 2727,
    2773, 2711, 2712, 2713, 135, 136, 2714, 15, 2717, 2725, 2718, 2719, 2724, 2720, 2721, 2722,
    2723, 2726, 2728, 2729, 2730, 135, 136, 2731, 15, 2732, 2735, 2738, 2741, 2744, 2747, 2750,
    2753, 2760, 2765, 2770, 2733, 2734, 2736, 2737, 2739, 2740, 2742, 2743, 2745, 2746, 2748, 2749,
    2751, 2752, 2754, 2755, 2756, 2759, 2757, 135, 136, 2758, 15, 2761, 2762, 2763, 135, 136,
    2764, 15, 2766, 2767, 2768, 135, 136, 2769, 15, 2771, 2772, 2774, 2775, 2710, 2777, 2778,
    2783, 2779, 2780, 2781, 2782, 2784, 2785, 2786, 2788, 2792, 2796, 2787, 2789, 2790, 2791, 2793,
    2794, 2795, 2797, 2798, 2799, 2800, 2804, 2808, 2801, 2802, 2803, 2805, 2806, 2807, 2809, 2810,
    2811, 2812, 2813, 15, 2815, 2818, 2819, 2727, 2710, 2816, 2817, 135, 136, 2717, 2820, 2821,
    2823, 2826, 2827, 2832, 2710, 2824, 2825, 135, 136, 2828, 2829, 2830, 2831, 2833, 2834, 2835,
    2837, 2841, 2845, 2836, 2838, 2839, 2840, 2842, 2843, 2844, 2846, 2847, 2848, 2849, 2853, 2857,
    2850, 2851, 2852, 2854, 2855, 2856, 2858, 2859, 2860, 2861, 2864, 2862, 135, 136, 2863, 15,
    2866, 2867, 2877, 2878, 2879, 2868, 135, 136, 2869, 2870, 2873, 2871, 2872, 2874, 2875, 2876,
    15, 2880, 2881, 2882, 135, 136, 2889, 2890, 2891, 2892, 2893, 2895, 2894, 135, 136, 2897,
    2898, 2899, 2903, 2915, 2900, 2901, 2902, 135, 136, 2904, 2909, 2912, 2905, 2906, 2907, 2908,
    2910, 2911, 2913, 2914, 2916, 2919, 2917, 2918, 2920, 2921, 2923, 2924, 2925, 2926, 2927, 2928,
    2930, 2931, 2932, 2933, 2934, 2939, 2935, 2936, 2937, 135, 136, 2938, 15, 2940, 2942, 2943,
    2944, 2945, 2946, 2947, 2976, 2948, 135, 136, 2949, 15, 2950, 2954, 2958, 2962, 2968, 2972,
    2951, 2952, 2953, 2955, 2956, 2957, 2959, 2960, 2961, 2963, 2964, 2965, 2966, 2967, 2969, 2970,
    2971, 2973, 2974, 2975, 2977, 2978, 2980, 2981, 2982, 2983, 2984, 2985, 2998, 2986, 2989, 2993,
    2994, 2995, 2987, 2988, 2990, 2991, 2992, 2996, 2997, 2999, 135, 136, 3001, 3002, 3003, 3004,
    3005, 3006, 3010, 3014, 3020, 3034, 3040, 3044, 3049, 3007, 3008, 3009, 3011, 3012, 3013, 3015,
    3016, 3017, 3018, 3019, 3021, 3025, 3022, 3023, 3024, 3026, 3030, 3027, 3028, 3029, 3031, 3032,
    3033, 3035, 3036, 3037, 3038, 3039, 3041, 3042, 3043, 3045, 3046, 3047, 3048, 3050, 3051, 3052,
    3054, 3055, 3056, 3057, 3059, 3060, 3063, 3066, 3058, 3061, 3062, 3064, 3065, 3067, 3068, 3071,
    3069, 3070, 3072, 3075, 3073, 3074, 3076, 3077, 3079, 3080, 3081, 3082, 3083, 3084, 3087, 3090,
    3093, 3085, 3086, 3088, 3089, 3091, 3092, 3095, 3096, 3097, 128, 3098, 3099, 144, 128, 3100,
    3101, 144, 3103, 3104, 3105, 3107, 3133, 3108, 3109, 3110, 3111, 3124, 3129, 3132, 3112, 135,
    136, 3113, 15, 3114, 3123, 3115, 3116, 3117, 3118, 3121, 3119, 3120, 3122, 3125, 3126, 3127,
    135, 136, 3128, 15, 3130, 3131, 3134, 3135, 3136, 3137, 3158, 3161, 3138, 3139, 3140, 3141,
    3152, 3142, 135, 136, 3143, 15, 3144, 3151, 3145, 3146, 3149, 3147, 3148, 3150, 3153, 3157,
    3154, 3155, 3156, 3159, 3160, 3163, 3164, 3165, 3166, 3167, 3168, 3169, 3170, 3188, 3171, 3172,
    3173, 3184, 3187, 3174, 3175, 3176, 135, 136, 3177, 15, 3178, 3179, 3180, 3181, 3182, 3183,
    3185, 3186, 3190, 3191, 3192, 3204, 3193, 3194, 3195, 3196, 3197, 3198, 3201, 3199, 3200, 3202,
    135, 136, 3203, 15, 3205, 3206, 3207, 3208, 3209, 3210, 3211, 3212, 3215, 3216, 3217, 3228,
    3218, 3219, 3220, 3221, 3222, 3225, 3226, 3227, 3223, 135, 136, 3224, 15, 3229, 3230, 3231,
    3232, 3233, 3236, 3237, 3238, 3234, 135, 136, 3235, 15, 3240, 3241, 3242, 3243, 3244, 3245,
    3257, 3258, 3246, 3247, 3248, 135, 136, 3249, 15, 3250, 3251, 3252, 3253, 3254, 3255, 2097,
    2098, 3256, 15, 3260, 3261, 3262, 3264, 3263, 3265, 135, 136, 3266, 15, 3267, 3273, 3278,
    3283, 3292, 3293, 3268, 3271, 3272, 3269, 3270, 3274, 3275, 3276, 3277, 3279, 3280, 3281, 3282,
    3284, 3290, 3291, 3285, 3286, 3287, 3288, 3289, 3295, 3296, 3297, 3298, 3299, 3300, 3305, 3313,
    3322, 3326, 3301, 3302, 3303, 3304, 3306, 3307, 3308, 3309, 3310, 3311, 3312, 3314, 3315, 3316,
    15, 3317, 3318, 3319, 3320, 3321, 3323, 3324, 3325, 135, 136, 3327, 3328, 3329, 3330, 135,
    136, 3332, 3333, 3334, 3335, 3336, 3337, 3339, 3340, 3341, 3342, 3351, 3343, 3347, 3344, 3345,
    3346, 3348, 3349, 3350, 15, 3352, 3353, 3355, 3356, 3359, 3360, 3357, 3358, 3361, 3362, 3363,
    3366, 3369, 3372, 3375, 3378, 3381, 3384, 3388, 3394, 3398, 3402, 3406, 3409, 3412, 3415, 3364,
    3365, 3367, 3368, 3370, 3371, 3373, 3374, 3376, 3377, 3379, 3380, 3382, 3383, 3385, 3386, 3387,
    3389, 3390, 3391, 3392, 3393, 3395, 3396, 3397, 3399, 3400, 3401, 3403, 3404, 3405, 3407, 3408,
    3410, 3411, 3413, 3414, 3416, 3417, 3419, 3420, 3421, 3422, 3423, 3427, 3424, 3425, 3426, 3429,
    3430, 3431, 3432, 3433, 3434, 3436, 3439, 3444, 3452, 3460, 3465, 3468, 3473, 3437, 3438, 3440,
    3441, 3442, 3443, 3445, 3449, 3446, 3447, 3448, 3450, 3451, 3453, 3457, 3454, 3455, 3456, 3458,
    3459, 3461, 3462, 3463, 3464, 3466, 3467, 3469, 3470, 3471, 3472, 3474, 3475, 3476, 3477, 3478,
    3480, 3481, 3482, 3483, 3484, 3485, 3486, 3488, 3489, 3490, 3491, 3492, 3495, 3498, 3499, 3500,
    3493, 3494, 3496, 3497, 3502, 3503, 3504, 3505, 3506, 3507, 3508, 3509, 3510, 3511, 3516, 3519,
    3524, 3512, 3513, 3514, 135, 136, 3515, 15, 3517, 3518, 3520, 3521, 3522, 3523, 3525, 3526,
    3527, 135, 136, 3528, 15, 3530, 3531, 3532, 3533, 3534, 3535, 3538, 3539, 3540, 3541, 3542,
    3545, 3552, 3555, 3559, 3565, 3536, 135, 136, 3537, 15, 3543, 135, 136, 3544, 15, 3546,
    3547, 3548, 3549, 3550, 3551, 3553, 3554, 3556, 3557, 3558, 15, 3560, 3561, 3562, 3563, 3564,
    3566, 3567, 3568, 135, 136, 3569, 15, 3571, 3572, 3573, 3574, 3575, 3576, 3580, 3585, 3588,
    3592, 3577, 3578, 3579, 3581, 3582, 3583, 3584, 3586, 135, 136, 3587, 15, 3589, 3590, 3591,
    3593, 3594, 3595, 135, 136, 3596, 15, 3598, 3599, 3600, 3601, 3602, 3603, 3611, 3622, 3604,
    3605, 3606, 3607, 3608, 3609, 135, 136, 3610, 15, 3612, 3616, 3613, 3614, 3615, 3617, 3618,
    3619, 3620, 135, 136, 3621, 15, 3623, 3624, 3625, 135, 136, 3626, 15, 3628, 3629, 3630,
    3631, 3632, 3633, 3638, 3642, 3654, 3661, 3634, 3635, 3636, 3637, 3639, 3640, 3641, 3643, 3648,
    3644, 3645, 3646, 3647, 3649, 3650, 3651, 3652, 135, 136, 3653, 15, 3655, 3656, 3657, 3658,
    135, 136, 3659, 135, 136, 3660, 15, 3662, 3664, 3665, 3666, 3669, 3676, 3677, 3681, 3682,
    3667, 3668, 3670, 135, 136, 3671, 15, 3672, 3673, 3674, 3675, 3678, 15, 3679, 3680, 3684,
    3685, 3686, 3687, 3688, 3689, 3690, 3692, 3691, 15, 3693, 3694, 3696, 3697, 3704, 3705, 3706,
    3707, 3743, 3744, 3759, 3698, 3703, 3699, 3702, 3700, 3701, 3708, 3711, 3714, 3729, 3740, 3709,
    135, 136, 3710, 15, 3712, 3713, 3715, 3726, 3716, 3717, 3718, 135, 136, 3719, 15, 3720,
    3721, 3722, 3723, 3724, 3725, 3727, 3728, 3730, 3731, 3732, 3733, 3736, 3739, 3734, 3735, 3737,
    3738, 3741, 3742, 3745, 3748, 3753, 3754, 3746, 3747, 3747, 135, 136, 3749, 3750, 3750, 135,
    136, 3751, 3752, 3755, 3756, 3756, 135, 136, 3757, 3758, 3761, 3762, 3769, 3770, 3771, 3772,
    3824, 3763, 3768, 3764, 3767, 3765, 3766, 3773, 3789, 3803, 3774, 3784, 3775, 135, 136, 3776,
    15, 3777, 3778, 3779, 3780, 3781, 3782, 3783, 3785, 3786, 3787, 135, 136, 3788, 15, 3790,
    3791, 3792, 3793, 135, 136, 3794, 15, 3795, 3802, 3796, 3797, 3800, 3798, 3799, 3801, 3804,
    3805, 3806, 3807, 3818, 3808, 135, 136, 3809, 15, 3810, 3817, 3811, 3812, 3815, 3813, 3814,
    3816, 3819, 3823, 3820, 3821, 3822, 3825, 3844, 3847, 3854, 3865, 3826, 3827, 3828, 3832, 3829,
    3830, 3831, 3833, 135, 136, 3834, 3835, 15, 3836, 3839, 3837, 3838, 3840, 3843, 3841, 3842,
    3845, 3846, 3848, 3851, 3849, 3850, 3852, 3853, 3855, 3856, 3857, 3858, 3861, 3864, 3859, 3860,
    3862, 3863, 3866, 3867, 3869, 3870, 3871, 3872, 3873, 3874, 3875, 135, 136, 3876, 3880, 3877,
    3878, 3879, 3881, 3882, 3883, 3885, 3886, 3887, 3888, 3889, 3890, 3896, 3910, 3911, 3912, 3954,
    3891, 3892, 3893, 3894, 3895, 3897, 3909, 3898, 3899, 3900, 3901, 3902, 3903, 3904, 3905, 3906,
    15, 3913, 3916, 3929, 3944, 3950, 3914, 3915, 3917, 3920, 3918, 3919, 3921, 3922, 3923, 3926,
    3924, 3925, 3927, 3928, 3930, 3931, 3934, 3935, 3936, 3937, 3932, 3933, 3938, 3941, 3942, 3943,
    3939, 3940, 3945, 3946, 3947, 3948, 3949, 3951, 3952, 3953, 135, 136, 3955, 3956, 3959, 3957,
    3958, 3961, 3962, 3963, 3964, 3965, 3972, 3966, 3967, 3968, 135, 136, 3969, 15, 3970, 3971,
    3973, 3974, 3977, 3975, 3976, 3978, 135, 136, 3979, 15, 3980, 3981, 3982, 3984, 3985, 3986,
    3988, 3989, 3990, 3991, 3995, 3996, 3997, 3992, 3993, 3994, 3999, 4000, 4001, 4003, 4004, 4007,
    4008, 4009, 4010, 4013, 4014, 4005, 4006, 4011, 135, 136, 4012, 15, 4024, 4027, 4025, 4026,
    4028, 135, 136, 4041, 4042, 4046, 4047, 4048, 4050, 4153, 4051, 4054, 4148, 4052, 4053, 4055,
    4056, 4057, 4058, 4059, 4145, 4060, 4061, 4062, 4063, 4064, 4065, 4066, 4067, 4068, 4069, 4070,
    4071, 4074, 4075, 4076, 4077, 4078, 4079, 4080, 4084, 4089, 4090, 4105, 4106, 4109, 4117, 4122,
    4125, 4130, 4131, 4132, 4135, 4136, 4137, 4138, 4139, 4140, 4141, 4142, 4072, 4073, 4081, 4083,
    4082, 4085, 4088, 4086, 4087, 4091, 4104, 4092, 4103, 4093, 4094, 4095, 4098, 4099, 4102, 4096,
    4097, 4100, 4101, 4107, 4108, 4110, 4111, 4112, 4113, 4114, 4115, 4116, 4118, 4119, 4120, 4121,
    4123, 4124, 4126, 4127, 4129, 4128, 4133, 4134, 4143, 4144, 4146, 4147, 4149, 4151, 4152, 4150,
    4158, 4159, 135, 136, 4160, 15, 4161, 4162, 4164, 4165, 4167, 4168, 4169, 4170, 4174, 4171,
    4172, 4173, 4175, 135, 136, 4192, 4193, 4194, 4195, 4196, 4204, 4205, 4206, 4207, 4210, 4211,
    4208, 4209, 4212, 4213, 4216, 4220, 4214, 4215, 4217, 4218, 4219, 4221, 4225, 4226, 4232, 4233,
    4235, 4236, 4237, 4238, 4240, 135, 136, 4241, 15, 4246, 4247, 4248, 4249, 4250, 4251, 4252,
    4258, 4259, 4266, 4253, 128, 4254, 144, 4255, 15, 4256, 4257, 4260, 4265, 4261, 4262, 4263,
    4264, 4267, 128, 4268, 144, 4269, 4270, 4271, 15, 4272, 4273, 4285, 4286, 4287, 4298, 4299,
    4300, 4301, 4302, 4303, 4304, 4314, 4315, 4316, 4319, 4317, 4318, 4320, 4321, 4329, 4330, 4331,
    4332, 4339, 4333, 15, 4334, 4335, 4336, 4337, 4338, 4341, 4342, 4344, 4345, 4346, 4347, 4351,
    4348, 4349, 4350, 135, 136, 4353, 4354, 4355, 4356, 4357, 4359, 4360, 4361, 4362, 4363, 4364,
    4365, 4366, 135, 136, 4368, 4369, 4370, 4371, 4373, 4372, 15, 4375, 4376, 4377, 4378, 4379,
    4381, 4382, 4383, 4384, 4385, 4387, 4386, 15, 4389, 4390, 4391, 4392, 4396, 4393, 15, 4394,
    4395, 4398, 4399, 4400, 4401, 4402, 4404, 4403, 15, 4407, 4408, 4409, 4410, 4411, 4413, 4412,
    15, 4415, 4416, 4417, 4418, 4420, 4421, 4422, 4423, 4429, 4424, 15, 4425, 4426, 4427, 4428,
    4431, 4432, 4433, 4434, 4435, 4436, 4437, 4439, 4440, 4441, 4442, 4446, 4443, 15, 4444, 4445,
    4447, 4448, 4450, 4451, 4452, 4453, 4455, 4454, 15, 4457, 4458, 4459, 4460, 4462, 4463, 4464,
    4465, 4466, 4468, 4469, 4470, 4471, 4473, 4472, 15, 4475, 4476, 4496, 4497, 4498, 4477, 4478,
    4479, 4482, 4483, 4484, 4487, 4488, 4489, 4490, 4491, 4492, 4493, 4494, 4495, 4480, 4481, 4485,
    4486, 4499, 4501, 4500, 4502, 4504, 4505, 4506, 4507, 4509, 4508, 15, 4510, 4511, 4513, 4514,
    4515, 4516, 4517, 4518, 4519, 4521, 4522, 4523, 4524, 4525, 4527, 4526, 15, 4529, 4530, 4531,
    4532, 4533, 4534, 4535, 4537, 4538, 4539, 4540, 4541, 4542, 4543, 4545, 4546, 4547, 4548, 4550,
    4549, 15, 4552, 4553, 4554, 4555, 4557, 4558, 4559, 4560, 4562, 4561, 15, 4569, 2253, 4570,
    144, 135, 136, 4576, 4577, 4598, 4599, 4600, 4604, 4605, 4606, 4607, 135, 136, 4608, 15,
    4615, 4627, 4616, 4626, 4617, 4618, 4619, 4620, 4621, 4622, 4623, 4624, 4625, 4628, 4629, 4630,
    4631, 4632, 4633, 4635, 4636, 4645, 4637, 4642, 4638, 4641, 4639, 4640, 4643, 135, 136, 4644,
    15, 4646, 4647, 4648, 4649, 4653, 4654, 4655, 4656, 4657, 4660, 4661, 4662, 4666, 4667, 4668,
    4669, 4675, 4658, 4659, 4663, 4664, 4665, 4670, 4671, 4672, 4673, 4674, 4676, 4677, 4679, 4682,
    4683, 4684, 4680, 4681, 4686, 4687, 4688, 4689, 4690, 4694, 4691, 4692, 4693, 4696, 4699, 4697,
    4698, 4701, 4702, 4703, 4716, 4704, 4708, 4714, 4705, 4706, 4707, 135, 136, 4709, 4712, 4710,
    4711, 4713, 4717, 4718, 4720, 4755, 4721, 4722, 4723, 4735, 4736, 4737, 4744, 4752, 4753, 4724,
    135, 136, 4725, 4726, 4727, 4728, 4729, 15, 4730, 4731, 4732, 4733, 4734, 4738, 4741, 4739,
    4740, 4742, 135, 136, 4743, 15, 4745, 4746, 4747, 4748, 4749, 4750, 4751, 4757, 4758, 4759,
    4760, 4761, 4762, 4763, 4767, 4768, 4773, 4769, 4770, 4771, 4772, 4772, 135, 136, 4774, 4775,
    4784, 4785, 4788, 4792, 4795, 4786, 4787, 4789, 4790, 4791, 4793, 4794, 4796, 4797, 4798, 4799,
    4802, 4803, 4805, 4808, 4810, 4811, 4804, 4806, 4807, 4809, 4812, 4813, 4814, 135, 136, 4815,
    15, 4819, 4820, 4821, 4822, 4823, 4824, 4825, 4828, 4829, 4830, 135, 136, 4831, 4832, 4842,
    4843, 4853, 4857, 4844, 4845, 4849, 4850, 4846, 4847, 4848, 4851, 4852, 4854, 4855, 4856, 15,
    4858, 4859, 4860, 4861, 4868, 4869, 4870, 4872, 4873, 4876, 4879, 4877, 4878, 4880, 4883, 4890,
    4898, 4881, 4882, 4884, 4885, 4887, 4886, 135, 136, 4888, 4889, 4891, 4892, 4893, 4894, 4895,
    4896, 4897, 4899, 4900, 4901, 4902, 4903, 4905, 4908, 4906, 4907, 4909, 4910, 4912, 4911, 135,
    136, 4913, 4914, 4918, 4919, 4936, 4920, 4933, 4921, 4924, 4922, 4923, 4925, 4930, 4926, 4927,
    4928, 4929, 4931, 4932, 4934, 4920, 4935, 4920, 4937, 4938, 4939, 4942, 4943, 4944, 4940, 4941,
    4945, 4946, 4948, 4949, 4950, 4955, 4956, 4957, 15, 4959, 4960, 4961, 4964, 4967, 4968, 4969,
    4970, 4971, 4972, 4973, 4974, 4975, 4962, 4963, 4965, 4966, 4977, 4995, 4978, 4979, 4982, 4983,
    4989, 4993, 4994, 4980, 4981, 4981, 135, 136, 4984, 4985, 4986, 4987, 4988, 4990, 4991, 4992,
    4996, 135, 136, 4977, 4997, 4998, 5000, 5001, 5001, 135, 136, 2253, 5002, 144, 5013, 5014,
    5025, 5003, 5004, 5005, 5006, 135, 136, 5008, 5009, 5011, 5012, 5015, 5016, 5017, 5018, 5020,
    5021, 5023, 5024, 5032, 5033, 5034, 5035, 5036, 5037, 5038, 5040, 5041, 5044, 5048, 5054, 5065,
    5068, 5074, 5075, 5078, 5090, 5091, 5092, 5093, 5042, 5043, 5043, 135, 136, 5045, 5046, 5047,
    5049, 5050, 5052, 5053, 5051, 5055, 5058, 5056, 5057, 5059, 5062, 5060, 5061, 5063, 5064, 5066,
    5067, 5069, 5070, 5071, 5072, 5073, 15, 5076, 5077, 5079, 5084, 5085, 5080, 5081, 5082, 5083,
    5086, 5087, 5088, 5089, 5094, 15, 5095, 5096, 5099, 5097, 5098, 5101, 5102, 135, 136, 5104,
    5200, 5105, 5106, 5111, 5112, 5116, 5119, 5132, 5138, 5144, 5147, 5150, 5153, 5168, 5176, 5179,
    5107, 5108, 5109, 5110, 5113, 5114, 5115, 5117, 5118, 5120, 5125, 5131, 5121, 5122, 5123, 5124,
    5126, 5127, 5128, 5129, 5130, 5133, 5134, 5135, 5136, 5137, 5139, 5140, 5141, 5142, 5143, 5145,
    5146, 5148, 5149, 5151, 5152, 5154, 5155, 5156, 5157, 5165, 5158, 5161, 5159, 5160, 5162, 15,
    5163, 5164, 5166, 5167, 5169, 5170, 5171, 5172, 5173, 5174, 5175, 5177, 5178, 5180, 5181, 128,
    5182, 144, 5199, 5183, 5186, 5189, 5195, 5184, 5185, 5187, 5188, 5190, 5194, 5191, 5192, 5193,
    5196, 5197, 5198, 5201, 5202, 5203, 135, 136, 5204, 15, 5208, 5213, 5209, 5210, 5211, 5214,
    5215, 5218, 5223, 5235, 5219, 5220, 5221, 5222, 5224, 5225, 5226, 5227, 5232, 5228, 5229, 5230,
    5231, 5233, 5234, 5236, 5240, 5237, 5238, 5239, 5242, 135, 136, 5243, 15, 5245, 5248, 5249,
    5246, 5247, 5250, 5257, 5251, 5256, 5252, 5253, 5254, 5255, 5272, 5273, 5274, 5275, 5276, 5277,
    5282, 5283, 5287, 5288, 128, 5289, 144, 5290, 15, 5291, 5292, 5293, 5294, 5295, 5296, 135,
    136, 5298, 5299, 5306, 5309, 5310, 5311, 5312, 5313, 5307, 5308, 5316, 5317, 15, 5318, 5319,
    5320, 5321, 135, 136, 5325, 5326, 5327, 5328, 135, 136, 5329, 15, 5330, 5331, 5337, 2253,
    5338, 144, 5339, 5340, 5340, 135, 136, 5342, 5345, 5346, 5347, 5348, 5343, 5344, 5374, 5375,
    5378, 5379, 5380, 5383, 5386, 5384, 5385, 5392, 5393, 5394, 5396, 5397, 5398, 5407, 5408, 5409,
    5410, 5411, 5412, 5399, 5404, 5400, 5403, 5401, 5402, 5405, 135, 136, 5406, 15, 5416, 5417,
    5418, 5421, 5422, 5423, 5424, 5429, 5425, 135, 136, 5426, 5428, 5427, 15, 5435, 5436, 5437,
    5438, 5440, 5443, 5451, 5454, 5441, 5442, 5444, 5450, 5445, 5446, 5447, 5448, 5449, 5452, 5453,
    5455, 5456, 5457, 5458, 5460, 5465, 5461, 5462, 5463, 5464, 135, 136, 5468, 5471, 5474, 5469,
    5470, 5472, 5473, 5475, 5476, 5477, 5478, 5496, 5497, 5498, 5499, 5500, 5501, 5507, 5512, 5517,
    5502, 5503, 5506, 5504, 5505, 5508, 5511, 5509, 5510, 5513, 5514, 5515, 5516, 5518, 128, 5519,
    144, 5520, 5521, 5522, 15, 5523, 5524, 5527, 5528, 5543, 5544, 5545, 5550, 5551, 5546, 5547,
    5548, 5549, 5552, 5553, 5554, 5555, 5558, 5559, 5560, 5561, 5562, 5563, 5565, 5566, 5570, 5571,
    5567, 5568, 5569, 5576, 5577, 5578, 5579, 5580, 5581, 5583, 5584, 5585, 5586, 5596, 5609, 5614,
    5597, 5598, 128, 5599, 5600, 144, 5602, 5601, 5603, 5604, 5605, 5608, 5606, 5607, 5610, 5611,
    128, 5612, 5613, 144, 5615, 128, 5616, 144, 5619, 5622, 5623, 5624, 5627, 5625, 5626, 5626,
    135, 136, 5629, 5630, 5631, 5635, 5632, 5633, 5634, 5637, 128, 5638, 144, 5639, 135, 136,
    5640, 15, 5644, 5645, 5646, 5648, 5649, 5650, 5651, 5653, 5656, 5657, 5654, 5655, 5658, 5669,
    5670, 5671, 5672, 5690, 5691, 5696, 5699, 5700, 5697, 5698, 5701, 5702, 5704, 5705, 5706, 5707,
    5708, 5712, 5715, 5713, 5714, 5716, 5717, 5718, 5719, 5720, 5721, 5722, 5723, 5724, 5726, 128,
    5727, 5733, 144, 5728, 5729, 5729, 135, 136, 5730, 5731, 5732, 5734, 5739, 5735, 5736, 5737,
    5738, 5740, 5741, 5754, 5755, 5757, 5758, 5759, 5760, 5761, 5762, 5763, 5764, 5765, 5766, 5767,
    5772, 5773, 5782, 5783, 5784, 5785, 5789, 5815, 5786, 5788, 5787, 15, 5790, 5791, 5814, 5792,
    5795, 5798, 5801, 5805, 5806, 5810, 5811, 5793, 5794, 5796, 5797, 5799, 5800, 5802, 5803, 5804,
    5807, 5808, 5809, 5812, 5813, 5830, 5831, 5832, 5833, 5834, 5837, 135, 136, 5835, 5836, 5838,
    5839, 5840, 135, 136, 5856, 5857, 5865, 5870, 5875, 5877, 5858, 5859, 5864, 5860, 5861, 5862,
    5863, 5866, 5867, 5868, 135, 136, 5869, 15, 5871, 5872, 5873, 5874, 5876, 5878, 5879, 5882,
    5886, 5880, 5881, 5883, 5884, 5885, 5887, 5901, 128, 5902, 144, 5903, 5905, 5906, 5909, 5910,
    5911, 5913, 5914, 5915, 5918, 128, 5919, 144, 5916, 5917, 5920, 5921, 5922, 5923, 5924, 5926,
    5927, 5928, 5929, 5930, 5931, 5934, 128, 5935, 144, 5932, 5933, 5936, 5937, 5938, 5939, 5940,
    5942, 5943, 5944, 5945, 5948, 5951, 128, 5952, 144, 5946, 5947, 5949, 5950, 5953, 5954, 5955,
    5956, 5958, 128, 5959, 144, 5962, 128, 5963, 144, 5973, 5974, 5975, 5976, 5960, 5961, 5964,
    5965, 5966, 5970, 5967, 5969, 5968, 135, 136, 5971, 5972, 5978, 128, 5979, 144, 5983, 5984,
    5985, 5986, 6015, 6036, 6037, 6041, 6038, 6039, 6040, 6043, 6044, 6045, 6046, 6047, 6048, 135,
    136, 6050, 128, 6051, 144, 6052, 15, 6058, 6059, 6086, 6087, 6088, 6090, 6092, 6094, 6089,
    135, 136, 6091, 135, 136, 6093, 135, 136, 6098, 6099, 6102, 6103, 6104, 6105, 6100, 6101,
    6101, 135, 136, 6112, 6117, 6113, 6114, 6115, 6116, 6122, 6125, 6123, 6124, 6126, 6127, 6129,
    6130, 6132, 6133, 6142, 6143, 6144, 6145, 6146, 6158, 6159, 6160, 6162, 6164, 4583, 6163, 15,
    6166, 6167, 6168, 6169, 6171, 6172, 6173, 6174, 6175, 6176, 6177, 6178, 6179, 6180, 6181, 2195,
    6187, 128, 6188, 144, 6189, 6190, 6212, 6213, 6214, 135, 136, 6215, 6218, 6219, 15, 6220,
    6221, 6225, 6226, 6228, 6229, 128, 6230, 144, 6231, 15, 6232, 6236, 6242, 6247, 6233, 6234,
    6235, 6237, 6238, 6239, 6240, 6241, 6243, 6244, 6245, 6246, 6251, 6252, 6253, 6254, 6255, 6256,
    6257, 6260, 6258, 6259, 6263, 6264, 128, 6271, 6272, 6273, 144, 6274, 6275, 6276, 135, 136,
    6279, 6280, 6282, 6297, 6298, 6283, 6296, 6284, 6285, 6290, 6291, 135, 136, 6286, 6287, 6288,
    6289, 6292, 6293, 6294, 6295, 6299, 6312, 6300, 6301, 6306, 6307, 135, 136, 6302, 6303, 6304,
    6305, 6308, 6309, 6310, 6311, 6328, 6333, 6348, 6329, 6330, 6331, 135, 136, 6332, 15, 6334,
    6335, 6341, 6342, 6336, 135, 136, 6337, 15, 6338, 6339, 6340, 6343, 135, 136, 6344, 15,
    6345, 6346, 6347, 6349, 6350, 6351, 135, 136, 6352, 6353, 6354, 6355, 6356, 6358, 6359, 128,
    6360, 144, 6361, 6362, 6362, 135, 136, 6363, 15, 6369, 6374, 6370, 6371, 6372, 6373, 6403,
    6404, 6405, 6411, 6414, 6421, 6422, 6423, 6426, 6427, 6424, 135, 136, 6425, 15, 6428, 6429,
    6433, 6434, 6454, 6457, 6460, 6461, 6462, 6462, 135, 136, 6463, 6464, 6465, 6466, 6470, 6479,
    6471, 6472, 6473, 15, 6475, 6476, 6477, 6478, 6480, 6481, 6482, 6483, 6484, 15, 6485, 6486,
    6488, 6489, 6490, 6491, 6494, 6495, 6496, 6497, 6498, 6499, 135, 136, 6502, 6503, 6505, 6506,
    6507, 135, 136, 6508, 6509, 6510, 6511, 6513, 6514, 6515, 6516, 6519, 6520, 6524, 6521, 6522,
    6523, 6525, 135, 136, 6530, 6531, 6533, 6540, 6545, 6553, 6534, 6537, 6535, 6536, 6538, 6539,
    6541, 6542, 6543, 6544, 6546, 6547, 6550, 6548, 6549, 6551, 6552, 6554, 15, 6555, 6556, 6557,
    6558, 6573, 6574, 6575, 6576, 6582, 6583, 6577, 15, 6578, 6579, 6580, 6581, 6584, 6585, 6586,
    6587, 6593, 6594, 6595, 6596, 6597, 6598, 6602, 6599, 6600, 6601, 6603, 6604, 6605, 6607, 6608,
    6609, 6610, 6611, 6616, 6617, 6625, 6626, 6618, 6619, 6622, 6620, 6621, 6623, 135, 136, 6624,
    15, 6627, 6628, 6631, 6632, 6633, 6634, 6635, 6636, 6643, 6644, 6656, 6645, 135, 136, 6646,
    15, 6647, 6650, 6653, 6648, 6649, 6651, 6652, 6654, 6655, 6657, 6661, 6665, 6669, 6658, 6659,
    6660, 6662, 6663, 6664, 6666, 6667, 6668, 6670, 6673, 6674, 6671, 6672, 6676, 6677, 6680, 6678,
    6679, 6681, 6682, 6684, 135, 136, 6685, 15, 6701, 6702, 6703, 6705, 6706, 6707, 6708, 6709,
    6724, 6725, 6729, 6743, 6744, 6745, 6726, 6727, 6728, 6730, 6734, 6737, 6742, 6731, 6732, 6733,
    6735, 6736, 6738, 6739, 6740, 6741, 6768, 6769, 6772, 6774, 6770, 6771, 6773, 135, 136, 6775,
    6777, 6776, 6778, 135, 136, 6797, 6798, 6799, 6802, 6803, 6883, 6884, 6800, 6801, 6804, 6807,
    6810, 6815, 6818, 6821, 6825, 6829, 6835, 6839, 6843, 6847, 6853, 6856, 6859, 6865, 6868, 6871,
    6874, 6877, 6880, 6805, 6806, 6808, 6809, 6811, 6812, 6813, 6814, 135, 136, 6816, 6817, 6819,
    6820, 6822, 6823, 6824, 6826, 6827, 6828, 6830, 6831, 6832, 6833, 6834, 6834, 135, 136, 6836,
    6837, 6838, 6840, 6841, 6842, 6844, 6845, 6846, 6848, 6849, 6850, 6851, 6852, 6852, 135, 136,
    6854, 6855, 6857, 6858, 6860, 6861, 6862, 6863, 6864, 6864, 135, 136, 6866, 6867, 6869, 6870,
    6872, 6873, 6875, 6876, 6878, 6879, 6881, 6882, 6885, 6886, 6889, 6890, 6891, 6892, 6894, 6901,
    6895, 6896, 6897, 6898, 6899, 135, 136, 6900, 15, 6903, 6904, 128, 6905, 144, 6906, 15,
    6911, 6912, 6914, 6915, 6921, 6922, 6923, 6927, 6928, 6930, 6931, 6934, 6935, 6936, 6937, 6938,
    6929, 6932, 6933, 6941, 6942, 6943, 6944, 6945, 6946, 6947, 6948, 6949, 6950, 6951, 6952, 6953,
    6954, 6959, 6960, 6965, 6968, 6971, 6972, 6966, 6967, 6967, 135, 136, 6969, 6970, 6970, 135,
    136, 6973, 135, 136, 6975, 6976, 6977, 6978, 6979, 6982, 6980, 6981, 6987, 6990, 6988, 6989,
    6991, 6992, 6994, 6997, 6995, 6996, 6998, 6999, 7011, 7012, 7013, 7014, 7018, 128, 7019, 144,
    7020, 15, 7022, 7023, 7036, 7024, 7025, 7026, 7027, 7034, 7028, 7029, 7030, 7031, 7032, 7033,
    7035, 7037, 7040, 7041, 7038, 135, 136, 7039, 15, 7042, 135, 136, 7043, 7044, 7045, 15,
    7046, 7053, 7047, 7048, 7049, 7050, 7051, 7052, 7054, 7056, 7057, 7058, 7062, 7059, 7060, 7061,
    15, 7063, 7064, 7066, 7067, 7071, 7072, 7073, 7068, 7069, 7070, 7076, 7077, 7082, 7078, 7081,
    7079, 7080, 7084, 7085, 7086, 7087, 7089, 7090, 7097, 7098, 7091, 7094, 7092, 7093, 7095, 7096,
    7099, 7100, 7102, 7103, 7106, 7104, 7105, 7107, 7120, 7127, 7130, 7108, 7109, 7112, 7110, 7111,
    7113, 7114, 7115, 15, 7116, 7117, 7118, 7119, 7121, 7122, 7123, 7124, 7125, 7126, 7128, 7129,
    7131, 7132, 7133, 7134, 7142, 7145, 7143, 7144, 7146, 7147, 7148, 7149, 7151, 7152, 7153, 7169,
    7154, 7156, 7158, 7161, 7163, 7165, 7167, 7155, 7157, 7159, 7160, 7162, 7164, 7166, 7168, 7171,
    7172, 7173, 7176, 7179, 7180, 7174, 7175, 7177, 7178, 7181, 7184, 7182, 7183, 7185, 7186, 7189,
    7190, 7191, 7192, 7195, 7193, 7194, 7197, 7198, 7218, 7219, 7222, 7199, 7200, 7201, 7202, 7205,
    7206, 7207, 7210, 7211, 7212, 7213, 7214, 7215, 7216, 7217, 7203, 7204, 7208, 7209, 7220, 7221,
    7223, 7226, 7224, 7225, 7227, 7228, 7232, 7233, 7236, 7239, 7240, 7241, 7243, 7244, 7247, 15,
    7249, 7250, 7251, 7257, 7258, 7296, 7297, 7298, 7299, 7300, 7301, 7302, 7303, 7304, 7305, 7306,
    7307, 7308, 7309, 7310, 7311, 7312, 7313, 7314, 7315, 7316, 7317, 7318, 7319, 7320, 7321, 7322,
    7323, 7324, 7325, 7326, 7327, 7328, 7329, 7330, 7331, 7332, 7333, 7334, 7335, 7336, 7337, 7338,
    7339, 7340, 7341, 7342, 7343, 7344, 7345, 7346, 7347, 7348, 7349, 7350, 7351, 7352, 7353, 7354,
    7355, 7356, 7357, 7358, 7359, 7360, 7361, 7362, 7363, 7364, 7377, 7378, 7382, 135, 136, 7383,
    15, 7384, 7385, 7403, 7406, 7444, 7404, 7405, 7407, 7414, 7422, 7426, 7439, 7408, 7409, 7411,
    7410, 135, 136, 7412, 7413, 7415, 7416, 7420, 7421, 7417, 7418, 7419, 7423, 7424, 7425, 7427,
    7428, 7431, 7434, 7435, 7429, 7430, 7432, 135, 136, 7433, 15, 7436, 7437, 7438, 135, 136,
    7440, 7441, 7442, 7443, 7445, 7450, 7457, 7460, 7446, 7447, 7448, 7449, 7451, 7454, 7452, 7453,
    7455, 7456, 7458, 7459, 7461, 7462, 7464, 7467, 7474, 7479, 7465, 7466, 7468, 7471, 7472, 7473,
    7469, 7470, 7475, 7476, 7477, 7478, 7480, 7483, 7481, 7482, 7484, 7485, 7488, 7489, 7490, 7495,
    7496, 7498, 7491, 7492, 7493, 7494, 7497, 135, 136, 7499, 135, 136, 7506, 7509, 7507, 7511,
    7513, 7512, 7521, 7522, 7525, 7526, 7539, 7542, 7543, 7540, 7541, 7544, 7547, 7548, 7545, 7546,
    7550, 7551, 7552, 7553, 7554, 7576, 7582, 7585, 7588, 7577, 7578, 7579, 7580, 7581, 7583, 7584,
    7586, 7587, 7589, 7590, 7591, 7600, 7601, 7602, 7607, 7608, 7609, 7617, 7622, 7610, 15, 7611,
    7614, 7612, 7613, 7615, 7616, 7618, 7621, 7619, 7620, 7631, 135, 136, 7632, 15, 7635, 7640,
    7641, 7636, 7637, 7638, 7639, 7643, 7644, 7652, 7653, 7654, 7658, 7659, 7660, 7661, 7664, 7670,
    7671, 7672, 7673, 7677, 7678, 7679, 7680, 7681, 7682, 7683, 7684, 7691, 7692, 7693, 7695, 7696,
    7701, 7702, 7703, 7704, 7705, 7711, 7712, 128, 7713, 7714, 144, 7716, 7717, 7718, 7725, 7715,
    7719, 7724, 7720, 7721, 7722, 7723, 7726, 128, 7727, 144, 7728, 7729, 7730, 15, 7731, 7732,
    7739, 7740, 7753, 7754, 7781, 7755, 7762, 7756, 7757, 7758, 7759, 7760, 7761, 7763, 135, 136,
    7764, 15, 7765, 7777, 7766, 7767, 7768, 7769, 7770, 7771, 7772, 7773, 7774, 7775, 7776, 7778,
    7779, 7780, 7782, 15, 7783, 7784, 7790, 7791, 7797, 7798, 7799, 7792, 15, 7793, 135, 136,
    7794, 15, 7795, 7796, 7811, 7812, 7827, 7878, 7828, 7865, 7829, 7830, 7831, 7832, 7833, 7834,
    7835, 7836, 7837, 7838, 7839, 7840, 7841, 7842, 7843, 7844, 7845, 7846, 7847, 7848, 7849, 7850,
    7851, 7852, 7853, 7854, 7855, 7856, 7857, 7858, 7859, 7860, 7861, 7862, 7863, 7864, 7866, 135,
    136, 7867, 15, 7868, 7872, 7877, 7869, 7870, 7870, 135, 136, 7871, 15, 7873, 135, 136,
    7874, 15, 7875, 135, 136, 7876, 15, 7879, 7882, 7880, 7881, 7883, 135, 136, 7884, 7923,
    7885, 7886, 7887, 7888, 7889, 7890, 7891, 7892, 7893, 7894, 7895, 7896, 7897, 7898, 7899, 7900,
    7901, 7902, 7903, 7904, 7905, 7906, 7907, 7908, 7909, 7910, 7911, 7912, 7913, 7914, 7915, 7916,
    7917, 7918, 7919, 7920, 7921, 7922, 7924, 7925, 7927, 7928, 7930, 7931, 7932, 7933, 7934, 7935,
    2253, 7929, 144, 7938, 2253, 7939, 144, 7940, 7941, 7941, 135, 136, 7943, 7944, 7947, 7950,
    7951, 7952, 7953, 7954, 7955, 7958, 7945, 7946, 7948, 7949, 7956, 7957, 7964, 7966, 7974, 7967,
    7968, 7971, 7969, 7970, 7972, 7973, 7977, 7981, 7982, 7983, 7978, 7979, 7980, 7986, 7987, 7990,
    7991, 7988, 7989, 7993, 7994, 7995, 7996, 7999, 8000, 7997, 15, 8002, 8003, 8004, 8005, 8008,
    8009, 8006, 15, 8011, 8012, 8014, 8013, 8018, 8019, 8020, 8021, 8023, 8024, 8025, 8026, 135,
    136,
];

pub static TERMINATORS: &[u32] = &[
    59, 248, 269, 282, 283, 2146, 2147, 2256, 2257, 2259, 2266, 2267, 2269, 2271, 2272, 2273,
    2324, 2437, 2438, 2439, 2440, 2441, 2442, 2443, 2444, 2445, 2446, 2447, 2195, 3907, 3908, 4043,
    4715, 4754, 5007, 5010, 5019, 5022, 5212, 5216, 5297, 5300, 5301, 5302, 5303, 5304, 5322, 6248,
    6249, 6474, 6487, 6907, 6908, 6909, 6910, 6913, 6916, 6917, 6918, 6919, 6920, 6924, 6925, 6955,
    6956, 6957, 6958, 6961, 6962, 6963, 7245, 7365, 7508, 7685, 7686, 7687, 7688, 7689, 7690, 7694,
    7697, 7698, 7699, 7998, 8007,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABSOLUTE", // [6]
    "ACCESS", // [7]
    "CLUSTER", // [8]
    "CONNECTION", // [9]
    "DATABASE", // [10]
    "SCHEMA", // [11]
    "SECRET", // [12]
    "TABLE", // [13]
    "TYPE", // [14]
    "TableKeywordSegment", // [15]
    "TypeKeywordSegment", // [16]
    "SecretKeywordSegment", // [17]
    "ConnectionKeywordSegment", // [18]
    "DatabaseKeywordSegment", // [19]
    "SchemaKeywordSegment", // [20]
    "ClusterKeywordSegment", // [21]
    "ObjectReferenceSegment", // [22]
    "object_reference", // [23]
    "CommaSegment", // [24]
    ",", // [25]
    "SystemKeywordSegment", // [26]
    "SYSTEM", // [27]
    "ALL", // [28]
    "AllKeywordSegment", // [29]
    "CLUSTERS", // [30]
    "CONNECTIONS", // [31]
    "DATABASES", // [32]
    "SCHEMAS", // [33]
    "SECRETS", // [34]
    "TABLES", // [35]
    "TYPES", // [36]
    "TablesKeywordSegment", // [37]
    "TypesKeywordSegment", // [38]
    "SecretsKeywordSegment", // [39]
    "ConnectionsKeywordSegment", // [40]
    "InKeywordSegment", // [41]
    "IN", // [42]
    "SchemasKeywordSegment", // [43]
    "DatabasesKeywordSegment", // [44]
    "ClustersKeywordSegment", // [45]
    "CREATE", // [46]
    "CREATECLUSTER", // [47]
    "CREATEDB", // [48]
    "CREATEROLE", // [49]
    "DELETE", // [50]
    "INSERT", // [51]
    "SELECT", // [52]
    "UPDATE", // [53]
    "USAGE", // [54]
    "SelectKeywordSegment", // [55]
    "InsertKeywordSegment", // [56]
    "UpdateKeywordSegment", // [57]
    "DeleteKeywordSegment", // [58]
    "CreateKeywordSegment", // [59]
    "UsageKeywordSegment", // [60]
    "CreateroleKeywordSegment", // [61]
    "CreatedbKeywordSegment", // [62]
    "CreateclusterKeywordSegment", // [63]
    "PrivilegesKeywordSegment", // [64]
    "PRIVILEGES", // [65]
    "AccessPermissionSegment", // [66]
    "access_permission", // [67]
    "OnKeywordSegment", // [68]
    "ON", // [69]
    "EXTERNAL", // [70]
    "FILE", // [71]
    "FUNCTION", // [72]
    "MASKING", // [73]
    "MATERIALIZED", // [74]
    "PROCEDURE", // [75]
    "ROUTINE", // [76]
    "SEQUENCE", // [77]
    "STAGE", // [78]
    "STREAM", // [79]
    "TASK", // [80]
    "VIEW", // [81]
    "ViewKeywordSegment", // [82]
    "StageKeywordSegment", // [83]
    "FunctionKeywordSegment", // [84]
    "ProcedureKeywordSegment", // [85]
    "RoutineKeywordSegment", // [86]
    "SequenceKeywordSegment", // [87]
    "StreamKeywordSegment", // [88]
    "TaskKeywordSegment", // [89]
    "MaskingKeywordSegment", // [90]
    "PolicyKeywordSegment", // [91]
    "POLICY", // [92]
    "MaterializedKeywordSegment", // [93]
    "ExternalKeywordSegment", // [94]
    "FileKeywordSegment", // [95]
    "FormatKeywordSegment", // [96]
    "FORMAT", // [97]
    "FUNCTIONS", // [98]
    "PROCEDURES", // [99]
    "ROUTINES", // [100]
    "SEQUENCES", // [101]
    "STAGES", // [102]
    "STREAMS", // [103]
    "TASKS", // [104]
    "VIEWS", // [105]
    "ViewsKeywordSegment", // [106]
    "StagesKeywordSegment", // [107]
    "FunctionsKeywordSegment", // [108]
    "ProceduresKeywordSegment", // [109]
    "RoutinesKeywordSegment", // [110]
    "SequencesKeywordSegment", // [111]
    "StreamsKeywordSegment", // [112]
    "TasksKeywordSegment", // [113]
    "PoliciesKeywordSegment", // [114]
    "FormatsKeywordSegment", // [115]
    "GRANT", // [116]
    "REVOKE", // [117]
    "GrantStatementSegment", // [118]
    "grant_statement", // [119]
    "RevokeStatementSegment", // [120]
    "revoke_statement", // [121]
    ".", // [122]
    "[", // [123]
    "ArrayAccessorSegment", // [124]
    "array_accessor", // [125]
    "SemiStructuredAccessorSegment", // [126]
    "semi_structured_expression", // [127]
    "ACCOUNT", // [128]
    "ACCOUNTS", // [129]
    "ACKS", // [130]
    "ACTION", // [131]
    "ADD", // [132]
    "-", // [133]
    "MinusSegment", // [134]
    "PipeSegment", // [135]
    "|", // [136]
    "ADMIN", // [137]
    "AFTER", // [138]
    "AGGREGATE", // [139]
    "OrderByClauseSegment", // [140]
    "ORDER", // [141]
    "orderby_clause", // [142]
    "indent", // [143]
    "Indent", // [144]
    "meta", // [145]
    "AsAliasOperatorSegment", // [146]
    "AS", // [147]
    "alias_operator", // [148]
    "SingleIdentifierGrammar", // [149]
    "(", // [150]
    "SingleIdentifierListSegment", // [151]
    "identifier_list", // [152]
    "start_bracket", // [153]
    "SymbolSegment", // [154]
    "symbol", // [155]
    ")", // [156]
    "end_bracket", // [157]
    "ParameterNameSegment", // [158]
    "DatatypeSegment", // [159]
    "data_type", // [160]
    "dedent", // [161]
    "Dedent", // [162]
    "ALIAS", // [163]
    "TableReferenceSegment", // [164]
    "table_reference", // [165]
    "AliasExpressionSegment", // [166]
    "alias_expression", // [167]
    "ALLOW_CONNECTIONS", // [168]
    "ALSO", // [169]
    "ALTER", // [170]
    "AlterKeywordSegment", // [171]
    "AggregateKeywordSegment", // [172]
    "FunctionParameterListGrammar", // [173]
    "function_parameter_list", // [174]
    "StarSegment", // [175]
    "*", // [176]
    "OWNER", // [177]
    "RENAME", // [178]
    "SET", // [179]
    "RenameKeywordSegment", // [180]
    "ToKeywordSegment", // [181]
    "TO", // [182]
    "FunctionNameSegment", // [183]
    "function_name", // [184]
    "OwnerKeywordSegment", // [185]
    "Current_roleKeywordSegment", // [186]
    "CURRENT_ROLE", // [187]
    "Current_userKeywordSegment", // [188]
    "CURRENT_USER", // [189]
    "Session_userKeywordSegment", // [190]
    "SESSION_USER", // [191]
    "RoleReferenceSegment", // [192]
    "role_reference", // [193]
    "SetKeywordSegment", // [194]
    "SchemaReferenceSegment", // [195]
    "schema_reference", // [196]
    "IfExistsGrammar", // [197]
    "IF", // [198]
    "RotateKeywordSegment", // [199]
    "ROTATE", // [200]
    "KeysKeywordSegment", // [201]
    "KEYS", // [202]
    "DatabaseReferenceSegment", // [203]
    "database_reference", // [204]
    "IS_TEMPLATE", // [205]
    "RESET", // [206]
    "WITH", // [207]
    "WithKeywordSegment", // [208]
    "Allow_connectionsKeywordSegment", // [209]
    "BooleanLiteralGrammar", // [210]
    "FALSE", // [211]
    "TRUE", // [212]
    "LimitKeywordSegment", // [213]
    "LIMIT", // [214]
    "NumericLiteralSegment", // [215]
    "numeric_literal", // [216]
    "Is_templateKeywordSegment", // [217]
    "TablespaceKeywordSegment", // [218]
    "TABLESPACE", // [219]
    "TablespaceReferenceSegment", // [220]
    "tablespace_reference", // [221]
    "=", // [222]
    "FROM", // [223]
    "EqualsSegment", // [224]
    "comparison_operator", // [225]
    "DefaultKeywordSegment", // [226]
    "DEFAULT", // [227]
    "LiteralGrammar", // [228]
    "+", // [229]
    ":", // [230]
    "ARRAY", // [231]
    "DATE", // [232]
    "INTERVAL", // [233]
    "NULL", // [234]
    "TIME", // [235]
    "TIMESTAMP", // [236]
    "TIMESTAMPTZ", // [237]
    "TIMETZ", // [238]
    "VARIADIC", // [239]
    "{", // [240]
    "bit_string_literal", // [241]
    "dollar_numeric_literal", // [242]
    "dollar_quote", // [243]
    "escaped_single_quote", // [244]
    "newline", // [245]
    "single_quote", // [246]
    "unicode_single_quote", // [247]
    "NakedIdentifierSegment", // [248]
    "QuotedIdentifierSegment", // [249]
    "double_quote", // [250]
    "unicode_double_quote", // [251]
    "FromKeywordSegment", // [252]
    "CurrentKeywordSegment", // [253]
    "CURRENT", // [254]
    "ResetKeywordSegment", // [255]
    "GrantKeywordSegment", // [256]
    "AlterDefaultPrivilegesObjectPrivilegesSegment", // [257]
    "EXECUTE", // [258]
    "REFERENCES", // [259]
    "TRIGGER", // [260]
    "TRUNCATE", // [261]
    "alter_default_privileges_object_privilege", // [262]
    "AlterDefaultPrivilegesSchemaObjectsSegment", // [263]
    "alter_default_privileges_schema_object", // [264]
    "AlterDefaultPrivilegesToFromRolesSegment", // [265]
    "alter_default_privileges_to_from_roles", // [266]
    "OptionKeywordSegment", // [267]
    "OPTION", // [268]
    "ExecuteKeywordSegment", // [269]
    "ReferencesKeywordSegment", // [270]
    "TriggerKeywordSegment", // [271]
    "TruncateKeywordSegment", // [272]
    "RevokeKeywordSegment", // [273]
    "ForKeywordSegment", // [274]
    "FOR", // [275]
    "RestrictKeywordSegment", // [276]
    "RESTRICT", // [277]
    "CascadeKeywordSegment", // [278]
    "CASCADE", // [279]
    "DropBehaviorGrammar", // [280]
    "ROLE", // [281]
    "USER", // [282]
    "RoleKeywordSegment", // [283]
    "UserKeywordSegment", // [284]
    "UserReferenceSegment", // [285]
    "user_reference", // [286]
    "RolesKeywordSegment", // [287]
    "ROLES", // [288]
    "AccessPermissionsSegment", // [289]
    "access_permissions", // [290]
    "GroupKeywordSegment", // [291]
    "GROUP", // [292]
    "PublicKeywordSegment", // [293]
    "PUBLIC", // [294]
    "DomainKeywordSegment", // [295]
    "DOMAIN", // [296]
    "DROP", // [297]
    "VALIDATE", // [298]
    "ExpressionSegment", // [299]
    "expression", // [300]
    "DropKeywordSegment", // [301]
    "NotKeywordSegment", // [302]
    "NOT", // [303]
    "NullKeywordSegment", // [304]
    "AddKeywordSegment", // [305]
    "CONSTRAINT", // [306]
    "ConstraintKeywordSegment", // [307]
    "CHECK", // [308]
    "CheckKeywordSegment", // [309]
    "ValidKeywordSegment", // [310]
    "VALID", // [311]
    "ValidateKeywordSegment", // [312]
    "ExtensionKeywordSegment", // [313]
    "EXTENSION", // [314]
    "ExtensionReferenceSegment", // [315]
    "extension_reference", // [316]
    "Current_schemaKeywordSegment", // [317]
    "CAST", // [318]
    "COLLATION", // [319]
    "CONVERSION", // [320]
    "EVENT", // [321]
    "FOREIGN", // [322]
    "LANGUAGE", // [323]
    "OPERATOR", // [324]
    "PROCEDURAL", // [325]
    "SERVER", // [326]
    "TEXT", // [327]
    "TRANSFORM", // [328]
    "AccessKeywordSegment", // [329]
    "MethodKeywordSegment", // [330]
    "METHOD", // [331]
    "CollationKeywordSegment", // [332]
    "ConversionKeywordSegment", // [333]
    "EventKeywordSegment", // [334]
    "ForeignKeywordSegment", // [335]
    "DataKeywordSegment", // [336]
    "DATA", // [337]
    "WrapperKeywordSegment", // [338]
    "WRAPPER", // [339]
    "ProceduralKeywordSegment", // [340]
    "LanguageKeywordSegment", // [341]
    "ServerKeywordSegment", // [342]
    "TextKeywordSegment", // [343]
    "SearchKeywordSegment", // [344]
    "SEARCH", // [345]
    "CONFIGURATION", // [346]
    "DICTIONARY", // [347]
    "PARSER", // [348]
    "TEMPLATE", // [349]
    "ConfigurationKeywordSegment", // [350]
    "DictionaryKeywordSegment", // [351]
    "ParserKeywordSegment", // [352]
    "TemplateKeywordSegment", // [353]
    "CastKeywordSegment", // [354]
    "AsKeywordSegment", // [355]
    "OperatorKeywordSegment", // [356]
    "CLASS", // [357]
    "FAMILY", // [358]
    "ClassKeywordSegment", // [359]
    "FamilyKeywordSegment", // [360]
    "UsingKeywordSegment", // [361]
    "USING", // [362]
    "IndexAccessMethodSegment", // [363]
    "index_access_method", // [364]
    "TransformKeywordSegment", // [365]
    "DISABLE", // [366]
    "ENABLE", // [367]
    "FORCE", // [368]
    "INHERIT", // [369]
    "NO", // [370]
    "OF", // [371]
    "OPTIONS", // [372]
    "REPLICA", // [373]
    "ColumnKeywordSegment", // [374]
    "COLUMN", // [375]
    "IfNotExistsGrammar", // [376]
    "ColumnReferenceSegment", // [377]
    "column_reference", // [378]
    "COLLATE", // [379]
    "CollateKeywordSegment", // [380]
    "CollationReferenceSegment", // [381]
    "collation_reference", // [382]
    "GENERATED", // [383]
    "PRIMARY", // [384]
    "UNIQUE", // [385]
    "ColumnConstraintSegment", // [386]
    "column_constraint_segment", // [387]
    "RESTART", // [388]
    "FunctionSegment", // [389]
    "function", // [390]
    "BareFunctionSegment", // [391]
    "CURRENT_CATALOG", // [392]
    "CURRENT_DATE", // [393]
    "CURRENT_SCHEMA", // [394]
    "CURRENT_TIME", // [395]
    "CURRENT_TIMESTAMP", // [396]
    "LOCALTIME", // [397]
    "LOCALTIMESTAMP", // [398]
    "SYSTEM_USER", // [399]
    "ExpressionKeywordSegment", // [400]
    "EXPRESSION", // [401]
    "GeneratedKeywordSegment", // [402]
    "ALWAYS", // [403]
    "BY", // [404]
    "AlwaysKeywordSegment", // [405]
    "ByKeywordSegment", // [406]
    "IdentityKeywordSegment", // [407]
    "IDENTITY", // [408]
    "CACHE", // [409]
    "CYCLE", // [410]
    "INCREMENT", // [411]
    "MAXVALUE", // [412]
    "MINVALUE", // [413]
    "OWNED", // [414]
    "START", // [415]
    "AlterSequenceOptionsSegment", // [416]
    "alter_sequence_options_segment", // [417]
    "RestartKeywordSegment", // [418]
    "StatisticsKeywordSegment", // [419]
    "STATISTICS", // [420]
    "RelationOptionsSegment", // [421]
    "relation_options", // [422]
    "StorageKeywordSegment", // [423]
    "STORAGE", // [424]
    "EXTENDED", // [425]
    "MAIN", // [426]
    "PLAIN", // [427]
    "PlainKeywordSegment", // [428]
    "ExtendedKeywordSegment", // [429]
    "MainKeywordSegment", // [430]
    "CompressionKeywordSegment", // [431]
    "COMPRESSION", // [432]
    "TableConstraintSegment", // [433]
    "EXCLUDE", // [434]
    "table_constraint", // [435]
    "TableConstraintUsingIndexSegment", // [436]
    "DEFERRABLE", // [437]
    "DeferrableKeywordSegment", // [438]
    "INITIALLY", // [439]
    "InitiallyKeywordSegment", // [440]
    "DeferredKeywordSegment", // [441]
    "DEFERRED", // [442]
    "ImmediateKeywordSegment", // [443]
    "IMMEDIATE", // [444]
    "EnableKeywordSegment", // [445]
    "DisableKeywordSegment", // [446]
    "ReplicaKeywordSegment", // [447]
    "RuleKeywordSegment", // [448]
    "RULE", // [449]
    "ForceKeywordSegment", // [450]
    "NoKeywordSegment", // [451]
    "RowKeywordSegment", // [452]
    "ROW", // [453]
    "LevelKeywordSegment", // [454]
    "LEVEL", // [455]
    "SecurityKeywordSegment", // [456]
    "SECURITY", // [457]
    "WithoutKeywordSegment", // [458]
    "WITHOUT", // [459]
    "OIDS", // [460]
    "OidsKeywordSegment", // [461]
    "LOGGED", // [462]
    "UNLOGGED", // [463]
    "LoggedKeywordSegment", // [464]
    "UnloggedKeywordSegment", // [465]
    "InheritKeywordSegment", // [466]
    "OfKeywordSegment", // [467]
    "FULL", // [468]
    "NOTHING", // [469]
    "IndexKeywordSegment", // [470]
    "INDEX", // [471]
    "IndexReferenceSegment", // [472]
    "index_reference", // [473]
    "FullKeywordSegment", // [474]
    "NothingKeywordSegment", // [475]
    "OptionsKeywordSegment", // [476]
    "QuotedLiteralSegment", // [477]
    "OnlyKeywordSegment", // [478]
    "ONLY", // [479]
    "AlterForeignTableActionSegment", // [480]
    "alter_foreign_table_action_segment", // [481]
    "alter_table_action_segment", // [482]
    "CALLED", // [483]
    "COST", // [484]
    "IMMUTABLE", // [485]
    "LEAKPROOF", // [486]
    "PARALLEL", // [487]
    "RETURNS", // [488]
    "ROWS", // [489]
    "STABLE", // [490]
    "STRICT", // [491]
    "SUPPORT", // [492]
    "VOLATILE", // [493]
    "CalledKeywordSegment", // [494]
    "InputKeywordSegment", // [495]
    "INPUT", // [496]
    "ReturnsKeywordSegment", // [497]
    "StrictKeywordSegment", // [498]
    "ImmutableKeywordSegment", // [499]
    "StableKeywordSegment", // [500]
    "VolatileKeywordSegment", // [501]
    "LeakproofKeywordSegment", // [502]
    "DEFINER", // [503]
    "INVOKER", // [504]
    "DefinerKeywordSegment", // [505]
    "InvokerKeywordSegment", // [506]
    "ParallelKeywordSegment", // [507]
    "RESTRICTED", // [508]
    "SAFE", // [509]
    "UNSAFE", // [510]
    "UnsafeKeywordSegment", // [511]
    "RestrictedKeywordSegment", // [512]
    "SafeKeywordSegment", // [513]
    "CostKeywordSegment", // [514]
    "RowsKeywordSegment", // [515]
    "SupportKeywordSegment", // [516]
    "DEPENDS", // [517]
    "AlterFunctionActionSegment", // [518]
    "alter_function_action_segment", // [519]
    "DependsKeywordSegment", // [520]
    "EnabledKeywordSegment", // [521]
    "ENABLED", // [522]
    "QualifiedParameterNameGrammar", // [523]
    "AlterMaterializedViewActionSegment", // [524]
    "alter_materialized_view_action_segment", // [525]
    "OwnedKeywordSegment", // [526]
    "NOWAIT", // [527]
    "NowaitKeywordSegment", // [528]
    "SINK", // [529]
    "SOURCE", // [530]
    "SourceKeywordSegment", // [531]
    "SinkKeywordSegment", // [532]
    "AlterProcedureActionSegment", // [533]
    "alter_procedure_action_segment", // [534]
    "PublicationKeywordSegment", // [535]
    "PUBLICATION", // [536]
    "PublicationReferenceSegment", // [537]
    "publication_reference", // [538]
    "DefinitionParametersSegment", // [539]
    "definition_parameters", // [540]
    "PublicationObjectsSegment", // [541]
    "publication_objects", // [542]
    "BYPASSRLS", // [543]
    "ENCRYPTED", // [544]
    "LOGIN", // [545]
    "NOBYPASSRLS", // [546]
    "NOCREATEDB", // [547]
    "NOCREATEROLE", // [548]
    "NOINHERIT", // [549]
    "NOLOGIN", // [550]
    "NOREPLICATION", // [551]
    "NOSUPERUSER", // [552]
    "PASSWORD", // [553]
    "REPLICATION", // [554]
    "SUPERUSER", // [555]
    "SuperuserKeywordSegment", // [556]
    "NosuperuserKeywordSegment", // [557]
    "NocreatedbKeywordSegment", // [558]
    "NocreateroleKeywordSegment", // [559]
    "NoinheritKeywordSegment", // [560]
    "LoginKeywordSegment", // [561]
    "NologinKeywordSegment", // [562]
    "ReplicationKeywordSegment", // [563]
    "NoreplicationKeywordSegment", // [564]
    "BypassrlsKeywordSegment", // [565]
    "NobypassrlsKeywordSegment", // [566]
    "EncryptedKeywordSegment", // [567]
    "PasswordKeywordSegment", // [568]
    "UntilKeywordSegment", // [569]
    "UNTIL", // [570]
    "OnKeywordAsIdentifierSegment", // [571]
    "IncrementKeywordSegment", // [572]
    "SignedSegmentGrammar", // [573]
    "MinvalueKeywordSegment", // [574]
    "MaxvalueKeywordSegment", // [575]
    "NameKeywordSegment", // [576]
    "NAME", // [577]
    "SequenceReferenceSegment", // [578]
    "sequence_reference", // [579]
    "StartKeywordSegment", // [580]
    "CacheKeywordSegment", // [581]
    "CycleKeywordSegment", // [582]
    "NoneKeywordSegment", // [583]
    "NONE", // [584]
    "SizeKeywordSegment", // [585]
    "SIZE", // [586]
    "InstanceSizes", // [587]
    "'2XLARGE'", // [588]
    "'2XSMALL'", // [589]
    "'3XLARGE'", // [590]
    "'3XSMALL'", // [591]
    "'4XLARGE'", // [592]
    "'5XLARGE'", // [593]
    "'6XLARGE'", // [594]
    "'LARGE'", // [595]
    "'MEDIUM'", // [596]
    "'SMALL'", // [597]
    "'XLARGE'", // [598]
    "'XSMALL'", // [599]
    "2XLARGE", // [600]
    "2XSMALL", // [601]
    "3XLARGE", // [602]
    "3XSMALL", // [603]
    "4XLARGE", // [604]
    "5XLARGE", // [605]
    "6XLARGE", // [606]
    "LARGE", // [607]
    "MEDIUM", // [608]
    "SMALL", // [609]
    "XLARGE", // [610]
    "XSMALL", // [611]
    "StatisticsReferenceSegment", // [612]
    "statistics_reference", // [613]
    "SubscriptionKeywordSegment", // [614]
    "SUBSCRIPTION", // [615]
    "SubscriptionReferenceSegment", // [616]
    "subscription_reference", // [617]
    "REFRESH", // [618]
    "SKIP", // [619]
    "RefreshKeywordSegment", // [620]
    "SkipKeywordSegment", // [621]
    "RawEqualsSegment", // [622]
    "Current_sessionKeywordSegment", // [623]
    "MODIFY", // [624]
    "ModifyKeywordSegment", // [625]
    "ColumnDefinitionSegment", // [626]
    "column_definition", // [627]
    "FIRST", // [628]
    "FirstKeywordSegment", // [629]
    "AfterKeywordSegment", // [630]
    "BracketedColumnReferenceListGrammar", // [631]
    "AlterTableDropColumnGrammar", // [632]
    "AlterTableActionSegment", // [633]
    "ATTACH", // [634]
    "DETACH", // [635]
    "AttachKeywordSegment", // [636]
    "PartitionKeywordSegment", // [637]
    "PARTITION", // [638]
    "ValuesKeywordSegment", // [639]
    "VALUES", // [640]
    "PartitionBoundSpecSegment", // [641]
    "partition_bound_spec", // [642]
    "DetachKeywordSegment", // [643]
    "ConcurrentlyKeywordSegment", // [644]
    "FinalizeKeywordSegment", // [645]
    "FINALIZE", // [646]
    "MappingKeywordSegment", // [647]
    "MAPPING", // [648]
    "ReplaceKeywordSegment", // [649]
    "REPLACE", // [650]
    "TriggerReferenceSegment", // [651]
    "trigger_reference", // [652]
    "ValueKeywordSegment", // [653]
    "VALUE", // [654]
    "AttributeKeywordSegment", // [655]
    "ATTRIBUTE", // [656]
    "CascadeRestrictGrammar", // [657]
    "BEFORE", // [658]
    "BeforeKeywordSegment", // [659]
    "&", // [660]
    "ampersand", // [661]
    "ANALYSE", // [662]
    "ANALYZE", // [663]
    "AnalyzeKeywordSegment", // [664]
    "AnalyseKeywordSegment", // [665]
    "VERBOSE", // [666]
    "SKIP_LOCKED", // [667]
    "VerboseKeywordSegment", // [668]
    "Skip_lockedKeywordSegment", // [669]
    "AND", // [670]
    "binary_operator", // [671]
    "BinaryOperatorSegment", // [672]
    "ANY", // [673]
    "APPLY", // [674]
    "%", // [675]
    "/", // [676]
    "<", // [677]
    ">", // [678]
    "^", // [679]
    "PlusSegment", // [680]
    "DivideSegment", // [681]
    "MultiplySegment", // [682]
    "ModuloSegment", // [683]
    "BitwiseAndSegment", // [684]
    "BitwiseOrSegment", // [685]
    "BitwiseXorSegment", // [686]
    "BitwiseLShiftSegment", // [687]
    "BitwiseRShiftSegment", // [688]
    "ARN", // [689]
    "ARRANGEMENT", // [690]
    "QualifiedNumericLiteralSegment", // [691]
    "SliceSegment", // [692]
    "start_square_bracket", // [693]
    "]", // [694]
    "end_square_bracket", // [695]
    "BaseExpressionElementGrammar", // [696]
    "ArrayKeywordSegment", // [697]
    "ASC", // [698]
    "ASSERTION", // [699]
    "ASSIGNMENT", // [700]
    "ASYMMETRIC", // [701]
    "AT", // [702]
    "ATOMIC", // [703]
    "AUCTION", // [704]
    "AUDIT", // [705]
    "AUTHORITY", // [706]
    "AUTHORIZATION", // [707]
    "AUTO_INCREMENT", // [708]
    "Auto_incrementKeywordSegment", // [709]
    "AVAILABILITY", // [710]
    "AVG_ROW_LENGTH", // [711]
    "AVRO", // [712]
    "AWS", // [713]
    "BACKUP", // [714]
    "BACKWARD", // [715]
    "bare_function", // [716]
    "CodeSegment", // [717]
    "IntervalExpressionSegment", // [718]
    "interval_expression", // [719]
    "CompositeValueExpansionSegment", // [720]
    "composite_value_expansion", // [721]
    "file", // [722]
    "BaseFileSegment", // [723]
    "BaseSegment", // [724]
    "BEGIN", // [725]
    "BERNOULLI", // [726]
    "BETWEEN", // [727]
    "BIGINT", // [728]
    "BIGSERIAL", // [729]
    "BINARY", // [730]
    "!", // [731]
    "IS", // [732]
    "OR", // [733]
    "json_operator", // [734]
    "like_operator", // [735]
    "pg_trgm_operator", // [736]
    "pgvector_operator", // [737]
    "postgis_operator", // [738]
    "ArithmeticBinaryOperatorGrammar", // [739]
    "StringBinaryOperatorGrammar", // [740]
    "BooleanBinaryOperatorGrammar", // [741]
    "ComparisonOperatorGrammar", // [742]
    "JsonOperatorSegment", // [743]
    "BINDING", // [744]
    "BIT", // [745]
    "BITVAR", // [746]
    "AmpersandSegment", // [747]
    "RawLessThanSegment", // [748]
    "RawGreaterThanSegment", // [749]
    "BODY", // [750]
    "BOOL", // [751]
    "AndOperatorGrammar", // [752]
    "OrOperatorGrammar", // [753]
    "BOOLEAN", // [754]
    "TrueSegment", // [755]
    "FalseSegment", // [756]
    "BOTH", // [757]
    "BOX", // [758]
    "BPCHAR", // [759]
    "bracketed", // [760]
    "BracketedSegment", // [761]
    "SetExpressionSegment", // [762]
    "set_expression", // [763]
    "BREADTH", // [764]
    "BREAK", // [765]
    "BROKER", // [766]
    "BROKERS", // [767]
    "BROWSE", // [768]
    "BUCKET", // [769]
    "BUFFERS", // [770]
    "BULK", // [771]
    "BYTEA", // [772]
    "BYTES", // [773]
    "CTEColumnList", // [774]
    "cte_column_list", // [775]
    "SelectableGrammar", // [776]
    "DEPTH", // [777]
    "BreadthKeywordSegment", // [778]
    "DepthKeywordSegment", // [779]
    "CALL", // [780]
    "CallKeywordSegment", // [781]
    "CASCADED", // [782]
    "CASE", // [783]
    "CaseKeywordSegment", // [784]
    "implicit_indent", // [785]
    "ImplicitIndent", // [786]
    "WHEN", // [787]
    "WhenClauseSegment", // [788]
    "when_clause", // [789]
    "ElseKeywordSegment", // [790]
    "ELSE", // [791]
    "EndKeywordSegment", // [792]
    "END", // [793]
    "ElseClauseSegment", // [794]
    "else_clause", // [795]
    "BinaryOperatorGrammar", // [796]
    "::", // [797]
    "casting_operator", // [798]
    "CATALOG", // [799]
    "CERTIFICATE", // [800]
    "CHAIN", // [801]
    "CHANGE", // [802]
    "CHAR", // [803]
    "CHARACTER", // [804]
    "CHARACTERISTICS", // [805]
    "CHECKED", // [806]
    "CHECKPOINT", // [807]
    "CHECKSUM", // [808]
    "CIDR", // [809]
    "CIRCLE", // [810]
    "CIRCULARSTRING", // [811]
    "CLIENT", // [812]
    "CLOSE", // [813]
    "CloseKeywordSegment", // [814]
    "CLUSTERED", // [815]
    "COALESCE", // [816]
    "ObjectReferenceDelimiterGrammar", // [817]
    "ObjectReferenceTerminatorGrammar", // [818]
    "colon_delimiter", // [819]
    "colon_prefix", // [820]
    "colon", // [821]
    "ShorthandCastSegment", // [822]
    "cast_expression", // [823]
    "StoredKeywordSegment", // [824]
    "STORED", // [825]
    "UniqueKeywordSegment", // [826]
    "NULLS", // [827]
    "NullsKeywordSegment", // [828]
    "DistinctKeywordSegment", // [829]
    "DISTINCT", // [830]
    "PrimaryKeywordSegment", // [831]
    "KeyKeywordSegment", // [832]
    "KEY", // [833]
    "ReferenceDefinitionGrammar", // [834]
    "DotSegment", // [835]
    "SingleIdentifierFullGrammar", // [836]
    "CastOperatorSegment", // [837]
    "StartSquareBracketSegment", // [838]
    "StartBracketSegment", // [839]
    "ColonSegment", // [840]
    "DelimiterGrammar", // [841]
    ";", // [842]
    "JoinLikeClauseGrammar", // [843]
    "ColumnsExpressionNameGrammar", // [844]
    "COLUMNS", // [845]
    "comma", // [846]
    "COMMENT", // [847]
    "CommentKeywordSegment", // [848]
    "IsKeywordSegment", // [849]
    "comment", // [850]
    "CommentSegment", // [851]
    "COMMENTS", // [852]
    "COMMIT", // [853]
    "COMMITTED", // [854]
    "COMPACTION", // [855]
    "GreaterThanSegment", // [856]
    "LessThanSegment", // [857]
    "GreaterThanOrEqualToSegment", // [858]
    "LessThanOrEqualToSegment", // [859]
    "NotEqualToSegment", // [860]
    "LikeOperatorSegment", // [861]
    "OverlapSegment", // [862]
    "NotExtendRightSegment", // [863]
    "NotExtendLeftSegment", // [864]
    "AdjacentSegment", // [865]
    "PostgisOperatorSegment", // [866]
    "PgvectorOperatorSegment", // [867]
    "PgTrgmOperatorSegment", // [868]
    "QualifiedOperatorSegment", // [869]
    "qualified_operator", // [870]
    "ComparisonOperatorSegment", // [871]
    "COMPLETION", // [872]
    "CompositeBinaryOperatorSegment", // [873]
    "CompositeComparisonOperatorSegment", // [874]
    "COMPOUNDCURVE", // [875]
    "COMPRESS", // [876]
    "COMPUTE", // [877]
    "JoinTypeKeywordsGrammar", // [878]
    "INNER", // [879]
    "LEFT", // [880]
    "RIGHT", // [881]
    "ConditionalCrossJoinKeywordsGrammar", // [882]
    "NonStandardJoinTypeKeywordsGrammar", // [883]
    "DO", // [884]
    "DoKeywordSegment", // [885]
    "WHERE", // [886]
    "WhereKeywordSegment", // [887]
    "CONFLICT", // [888]
    "OperationClassReferenceSegment", // [889]
    "operation_class_reference", // [890]
    "CONFLUENT", // [891]
    "CONNECT", // [892]
    "CONSTRAINTS", // [893]
    "CONTAINSTABLE", // [894]
    "CONTENT", // [895]
    "CONTINUE", // [896]
    "COPY", // [897]
    "CopyKeywordSegment", // [898]
    "StdinKeywordSegment", // [899]
    "STDIN", // [900]
    "PROGRAM", // [901]
    "ProgramKeywordSegment", // [902]
    "PsqlVariableGrammar", // [903]
    "psql_variable", // [904]
    "DELIMITER", // [905]
    "ENCODING", // [906]
    "ESCAPE", // [907]
    "FORCE_NOT_NULL", // [908]
    "FORCE_NULL", // [909]
    "FORCE_QUOTE", // [910]
    "FREEZE", // [911]
    "HEADER", // [912]
    "QUOTE", // [913]
    "FreezeKeywordSegment", // [914]
    "DelimiterKeywordSegment", // [915]
    "HeaderKeywordSegment", // [916]
    "QuoteKeywordSegment", // [917]
    "EscapeKeywordSegment", // [918]
    "Force_quoteKeywordSegment", // [919]
    "Force_not_nullKeywordSegment", // [920]
    "Force_nullKeywordSegment", // [921]
    "EncodingKeywordSegment", // [922]
    "CSV", // [923]
    "BinaryKeywordSegment", // [924]
    "CsvKeywordSegment", // [925]
    "STDOUT", // [926]
    "StdoutKeywordSegment", // [927]
    "SUBSCRIBE", // [928]
    "SelectStatementSegment", // [929]
    "select_statement", // [930]
    "SubscribeKeywordSegment", // [931]
    "CORRELATION", // [932]
    "COSTS", // [933]
    "COUNT", // [934]
    "COUNTER", // [935]
    "OrReplaceGrammar", // [936]
    "InoutKeywordSegment", // [937]
    "INOUT", // [938]
    "AssignmentKeywordSegment", // [939]
    "ImplicitKeywordSegment", // [940]
    "IMPLICIT", // [941]
    "REPLICAS", // [942]
    "ReplicasKeywordSegment", // [943]
    "DETERMINISTIC", // [944]
    "LC_COLLATE", // [945]
    "LC_CTYPE", // [946]
    "LOCALE", // [947]
    "PROVIDER", // [948]
    "VERSION", // [949]
    "LocaleKeywordSegment", // [950]
    "Lc_collateKeywordSegment", // [951]
    "Lc_ctypeKeywordSegment", // [952]
    "ProviderKeywordSegment", // [953]
    "ICU", // [954]
    "LIBC", // [955]
    "IcuKeywordSegment", // [956]
    "LibcKeywordSegment", // [957]
    "DeterministicKeywordSegment", // [958]
    "VersionKeywordSegment", // [959]
    "KAFKA", // [960]
    "POSTGRES", // [961]
    "SSH", // [962]
    "AwsKeywordSegment", // [963]
    "PrivatelinkKeywordSegment", // [964]
    "PRIVATELINK", // [965]
    "ConfluentKeywordSegment", // [966]
    "RegistryKeywordSegment", // [967]
    "REGISTRY", // [968]
    "KafkaKeywordSegment", // [969]
    "PostgresKeywordSegment", // [970]
    "SshKeywordSegment", // [971]
    "TunnelKeywordSegment", // [972]
    "TUNNEL", // [973]
    "VersionIdentifierSegment", // [974]
    "version_identifier", // [975]
    "ForeignDataWrapperGrammar", // [976]
    "HandlerKeywordSegment", // [977]
    "HANDLER", // [978]
    "ValidatorKeywordSegment", // [979]
    "VALIDATOR", // [980]
    "CreateForeignTableGrammar", // [981]
    "OptionsGrammar", // [982]
    "ForeignTableColumnConstraintSegment", // [983]
    "ForeignTableTableConstraintSegment", // [984]
    "INHERITS", // [985]
    "InheritsKeywordSegment", // [986]
    "ServerReferenceSegment", // [987]
    "server_reference", // [988]
    "TemporaryGrammar", // [989]
    "TEMP", // [990]
    "TEMPORARY", // [991]
    "ColumnTypeReferenceSegment", // [992]
    "column_type_reference", // [993]
    "SETOF", // [994]
    "SetofKeywordSegment", // [995]
    "FunctionDefinitionGrammar", // [996]
    "RETURN", // [997]
    "WINDOW", // [998]
    "function_definition", // [999]
    "InCluster", // [1000]
    "ModelKeywordSegment", // [1001]
    "MODEL", // [1002]
    "commutator", // [1003]
    "LeftargKeywordSegment", // [1004]
    "LEFTARG", // [1005]
    "RightargKeywordSegment", // [1006]
    "CommutatorKeywordSegment", // [1007]
    "NegatorKeywordSegment", // [1008]
    "negator", // [1009]
    "JOIN", // [1010]
    "JoinKeywordSegment", // [1011]
    "HashesKeywordSegment", // [1012]
    "MergesKeywordSegment", // [1013]
    "PERMISSIVE", // [1014]
    "RESTRICTIVE", // [1015]
    "PermissiveKeywordSegment", // [1016]
    "RestrictiveKeywordSegment", // [1017]
    "SYSID", // [1018]
    "AdminKeywordSegment", // [1019]
    "SysidKeywordSegment", // [1020]
    "AuthorizationKeywordSegment", // [1021]
    "CreateSequenceOptionsSegment", // [1022]
    "create_sequence_options_segment", // [1023]
    "IntoKeywordSegment", // [1024]
    "INTO", // [1025]
    "ENVELOPE", // [1026]
    "EnvelopeKeywordSegment", // [1027]
    "DEBEZIUM", // [1028]
    "UPSERT", // [1029]
    "DebeziumKeywordSegment", // [1030]
    "UpsertKeywordSegment", // [1031]
    "INCLUDE", // [1032]
    "IncludeKeywordSegment", // [1033]
    "LoadKeywordSegment", // [1034]
    "LOAD", // [1035]
    "GeneratorKeywordSegment", // [1036]
    "GENERATOR", // [1037]
    "MARKETING", // [1038]
    "TPCH", // [1039]
    "AuctionKeywordSegment", // [1040]
    "CounterKeywordSegment", // [1041]
    "MarketingKeywordSegment", // [1042]
    "TpchKeywordSegment", // [1043]
    "WebhookKeywordSegment", // [1044]
    "WEBHOOK", // [1045]
    "BodyKeywordSegment", // [1046]
    "JSON", // [1047]
    "JsonKeywordSegment", // [1048]
    "BytesKeywordSegment", // [1049]
    "HeadersKeywordSegment", // [1050]
    "HEADERS", // [1051]
    "DEPENDENCIES", // [1052]
    "MCV", // [1053]
    "NDISTINCT", // [1054]
    "DependenciesKeywordSegment", // [1055]
    "McvKeywordSegment", // [1056]
    "NdistinctKeywordSegment", // [1057]
    "CorrelationKeywordSegment", // [1058]
    "GLOBAL", // [1059]
    "LOCAL", // [1060]
    "GlobalKeywordSegment", // [1061]
    "LocalKeywordSegment", // [1062]
    "CommitKeywordSegment", // [1063]
    "PRESERVE", // [1064]
    "PreserveKeywordSegment", // [1065]
    "ValuesClauseSegment", // [1066]
    "values_clause", // [1067]
    "WithDataClauseSegment", // [1068]
    "with_data_clause", // [1069]
    "LIKE", // [1070]
    "LikeKeywordSegment", // [1071]
    "EXCLUDING", // [1072]
    "INCLUDING", // [1073]
    "LikeOptionSegment", // [1074]
    "like_option_segment", // [1075]
    "HASH", // [1076]
    "LIST", // [1077]
    "RANGE", // [1078]
    "RangeKeywordSegment", // [1079]
    "ListKeywordSegment", // [1080]
    "HashKeywordSegment", // [1081]
    "INSTEAD", // [1082]
    "InsteadKeywordSegment", // [1083]
    "OrKeywordSegment", // [1084]
    "REFERENCING", // [1085]
    "ReferencingKeywordSegment", // [1086]
    "NEW", // [1087]
    "OLD", // [1088]
    "OldKeywordSegment", // [1089]
    "NewKeywordSegment", // [1090]
    "EachKeywordSegment", // [1091]
    "EACH", // [1092]
    "STATEMENT", // [1093]
    "StatementKeywordSegment", // [1094]
    "WhenKeywordSegment", // [1095]
    "MAP", // [1096]
    "MapKeywordSegment", // [1097]
    "CreateUserMappingGrammar", // [1098]
    "SessionInformationUserFunctionsGrammar", // [1099]
    "TempKeywordSegment", // [1100]
    "TemporaryKeywordSegment", // [1101]
    "CREATEUSER", // [1102]
    "CROSS", // [1103]
    "CUBE", // [1104]
    "function_name_identifier", // [1105]
    "ROLLUP", // [1106]
    "CubeFunctionNameSegment", // [1107]
    "RollupFunctionNameSegment", // [1108]
    "GroupingExpressionList", // [1109]
    "grouping_expression_list", // [1110]
    "CURSOR", // [1111]
    "CURVEPOLYGON", // [1112]
    "data_type_identifier", // [1113]
    "WellKnownTextGeometrySegment", // [1114]
    "GEOGRAPHY", // [1115]
    "GEOMETRY", // [1116]
    "GEOMETRYCOLLECTION", // [1117]
    "GEOMETRYCOLLECTIONM", // [1118]
    "GEOMETRYCOLLECTIONZ", // [1119]
    "GEOMETRYCOLLECTIONZM", // [1120]
    "LINESTRING", // [1121]
    "LINESTRINGM", // [1122]
    "LINESTRINGZ", // [1123]
    "LINESTRINGZM", // [1124]
    "MULTICURVE", // [1125]
    "MULTILINESTRING", // [1126]
    "MULTILINESTRINGM", // [1127]
    "MULTILINESTRINGZ", // [1128]
    "MULTILINESTRINGZM", // [1129]
    "MULTIPOINT", // [1130]
    "MULTIPOINTM", // [1131]
    "MULTIPOINTZ", // [1132]
    "MULTIPOINTZM", // [1133]
    "MULTIPOLYGON", // [1134]
    "MULTIPOLYGONM", // [1135]
    "MULTIPOLYGONZ", // [1136]
    "MULTIPOLYGONZM", // [1137]
    "MULTISURFACE", // [1138]
    "POINT", // [1139]
    "POINTM", // [1140]
    "POINTZ", // [1141]
    "POINTZM", // [1142]
    "POLYGON", // [1143]
    "POLYGONM", // [1144]
    "POLYGONZ", // [1145]
    "POLYGONZM", // [1146]
    "POLYHEDRALSURFACE", // [1147]
    "TIN", // [1148]
    "TRIANGLE", // [1149]
    "wkt_geometry_type", // [1150]
    "DateTimeTypeIdentifier", // [1151]
    "datetime_type_identifier", // [1152]
    "StructTypeSegment", // [1153]
    "struct_type", // [1154]
    "MapTypeSegment", // [1155]
    "map_type", // [1156]
    "DATERANGE", // [1157]
    "DECIMAL", // [1158]
    "DOUBLE", // [1159]
    "FLOAT", // [1160]
    "FLOAT4", // [1161]
    "FLOAT8", // [1162]
    "INET", // [1163]
    "INT", // [1164]
    "INT2", // [1165]
    "INT4", // [1166]
    "INT4RANGE", // [1167]
    "INT8", // [1168]
    "INT8RANGE", // [1169]
    "INTEGER", // [1170]
    "JSONB", // [1171]
    "LINE", // [1172]
    "LSEG", // [1173]
    "MACADDR", // [1174]
    "MACADDR8", // [1175]
    "MONEY", // [1176]
    "NUMERIC", // [1177]
    "NUMRANGE", // [1178]
    "PATH", // [1179]
    "PG_LSN", // [1180]
    "REAL", // [1181]
    "SERIAL", // [1182]
    "SERIAL2", // [1183]
    "SERIAL4", // [1184]
    "SERIAL8", // [1185]
    "SMALLINT", // [1186]
    "SMALLSERIAL", // [1187]
    "TSQUERY", // [1188]
    "TSRANGE", // [1189]
    "TSTZRANGE", // [1190]
    "TSVECTOR", // [1191]
    "UUID", // [1192]
    "VARCHAR", // [1193]
    "VECTOR", // [1194]
    "XML", // [1195]
    "SmallintKeywordSegment", // [1196]
    "IntegerKeywordSegment", // [1197]
    "IntKeywordSegment", // [1198]
    "Int2KeywordSegment", // [1199]
    "Int4KeywordSegment", // [1200]
    "Int8KeywordSegment", // [1201]
    "BigintKeywordSegment", // [1202]
    "Float4KeywordSegment", // [1203]
    "Float8KeywordSegment", // [1204]
    "RealKeywordSegment", // [1205]
    "DoubleKeywordSegment", // [1206]
    "PrecisionKeywordSegment", // [1207]
    "PRECISION", // [1208]
    "SmallserialKeywordSegment", // [1209]
    "SerialKeywordSegment", // [1210]
    "Serial2KeywordSegment", // [1211]
    "Serial4KeywordSegment", // [1212]
    "Serial8KeywordSegment", // [1213]
    "BigserialKeywordSegment", // [1214]
    "FloatKeywordSegment", // [1215]
    "BracketedArguments", // [1216]
    "bracketed_arguments", // [1217]
    "DecimalKeywordSegment", // [1218]
    "NumericKeywordSegment", // [1219]
    "MoneyKeywordSegment", // [1220]
    "BpcharKeywordSegment", // [1221]
    "CharKeywordSegment", // [1222]
    "VaryingKeywordSegment", // [1223]
    "VARYING", // [1224]
    "CharacterKeywordSegment", // [1225]
    "VarcharKeywordSegment", // [1226]
    "ByteaKeywordSegment", // [1227]
    "BooleanKeywordSegment", // [1228]
    "BoolKeywordSegment", // [1229]
    "PointKeywordSegment", // [1230]
    "LineKeywordSegment", // [1231]
    "LsegKeywordSegment", // [1232]
    "BoxKeywordSegment", // [1233]
    "PathKeywordSegment", // [1234]
    "PolygonKeywordSegment", // [1235]
    "CircleKeywordSegment", // [1236]
    "CidrKeywordSegment", // [1237]
    "InetKeywordSegment", // [1238]
    "MacaddrKeywordSegment", // [1239]
    "Macaddr8KeywordSegment", // [1240]
    "TsvectorKeywordSegment", // [1241]
    "TsqueryKeywordSegment", // [1242]
    "BitKeywordSegment", // [1243]
    "UuidKeywordSegment", // [1244]
    "XmlKeywordSegment", // [1245]
    "JsonbKeywordSegment", // [1246]
    "Int4rangeKeywordSegment", // [1247]
    "Int8rangeKeywordSegment", // [1248]
    "NumrangeKeywordSegment", // [1249]
    "TsrangeKeywordSegment", // [1250]
    "TstzrangeKeywordSegment", // [1251]
    "DaterangeKeywordSegment", // [1252]
    "Pg_lsnKeywordSegment", // [1253]
    "VectorKeywordSegment", // [1254]
    "DatatypeIdentifierSegment", // [1255]
    "ArrayTypeSuffixSegment", // [1256]
    "array_type_suffix", // [1257]
    "ArrayTypeSegment", // [1258]
    "array_type", // [1259]
    "SizedArrayTypeSegment", // [1260]
    "sized_array_type", // [1261]
    "DatePartFunctionName", // [1262]
    "DatetimeUnitSegment", // [1263]
    "CENTURY", // [1264]
    "DAY", // [1265]
    "DAYOFYEAR", // [1266]
    "DECADE", // [1267]
    "DOW", // [1268]
    "DOY", // [1269]
    "EPOCH", // [1270]
    "HOUR", // [1271]
    "ISODOW", // [1272]
    "ISOYEAR", // [1273]
    "MICROSECONDS", // [1274]
    "MILLENNIUM", // [1275]
    "MILLISECOND", // [1276]
    "MILLISECONDS", // [1277]
    "MINUTE", // [1278]
    "MONTH", // [1279]
    "QUARTER", // [1280]
    "SECOND", // [1281]
    "TIMEZONE", // [1282]
    "TIMEZONE_HOUR", // [1283]
    "TIMEZONE_MINUTE", // [1284]
    "WEEK", // [1285]
    "WEEKDAY", // [1286]
    "YEAR", // [1287]
    "FunctionContentsGrammar", // [1288]
    "DateKeywordSegment", // [1289]
    "TimeWithTZGrammar", // [1290]
    "IntervalKeywordSegment", // [1291]
    "TimetzKeywordSegment", // [1292]
    "TimestamptzKeywordSegment", // [1293]
    "DATETIME", // [1294]
    "date_part", // [1295]
    "DATUMS", // [1296]
    "DAY_HOUR", // [1297]
    "DAY_MICROSECOND", // [1298]
    "DAY_MINUTE", // [1299]
    "DAY_SECOND", // [1300]
    "DAYOFMONTH", // [1301]
    "DAYOFWEEK", // [1302]
    "DAYS", // [1303]
    "DBCC", // [1304]
    "DEALLOCATE", // [1305]
    "DeallocateKeywordSegment", // [1306]
    "PrepareKeywordSegment", // [1307]
    "PREPARE", // [1308]
    "DEBUG", // [1309]
    "DEBUGGING", // [1310]
    "DEC", // [1311]
    "DECLARE", // [1312]
    "DeclareKeywordSegment", // [1313]
    "CursorKeywordSegment", // [1314]
    "HoldKeywordSegment", // [1315]
    "HOLD", // [1316]
    "DECORRELATED", // [1317]
    "DEFAULTS", // [1318]
    "PropertiesNakedIdentifierSegment", // [1319]
    "DefinitionArgumentValueGrammar", // [1320]
    "DefinitionParameterSegment", // [1321]
    "definition_parameter", // [1322]
    "DELAY_KEY_WRITE", // [1323]
    "DELAYED", // [1324]
    "TableExpressionSegment", // [1325]
    "table_expression", // [1326]
    "JoinClauseSegment", // [1327]
    "join_clause", // [1328]
    "WhereClauseSegment", // [1329]
    "where_clause", // [1330]
    "RETURNING", // [1331]
    "ReturningKeywordSegment", // [1332]
    "DELIMITED", // [1333]
    "SemicolonSegment", // [1334]
    "DELIMITERS", // [1335]
    "DENY", // [1336]
    "DESC", // [1337]
    "DESCRIBE", // [1338]
    "DescribeKeywordSegment", // [1339]
    "DESTROY", // [1340]
    "DESTRUCTOR", // [1341]
    "DETAILS", // [1342]
    "DISABLE_PAGE_SKIPPING", // [1343]
    "DISCARD", // [1344]
    "DiscardKeywordSegment", // [1345]
    "PLANS", // [1346]
    "PlansKeywordSegment", // [1347]
    "DISCOVER", // [1348]
    "DISK", // [1349]
    "DISTINCTROW", // [1350]
    "DISTRIBUTED", // [1351]
    "DIV", // [1352]
    "LanguageClauseSegment", // [1353]
    "language_clause", // [1354]
    "DOCUMENT", // [1355]
    "literal", // [1356]
    "LiteralSegment", // [1357]
    "DOT", // [1358]
    "dot", // [1359]
    "DUMMY", // [1360]
    "DUMP", // [1361]
    "EFFORT", // [1362]
    "ELEMENT", // [1363]
    "ELSEIF", // [1364]
    "EMPTY", // [1365]
    "EmptyStructLiteralBracketsSegment", // [1366]
    "struct_literal", // [1367]
    "ENCLOSED", // [1368]
    "}", // [1369]
    "end_curly_bracket", // [1370]
    "ENDPOINT", // [1371]
    "ENFORCED", // [1372]
    "ENUM", // [1373]
    "ERRLVL", // [1374]
    "ESCAPED", // [1375]
    "EXCEPT", // [1376]
    "IndexElementSegment", // [1377]
    "index_element", // [1378]
    "EXCLUSIVE", // [1379]
    "EXISTING", // [1380]
    "EXISTS", // [1381]
    "EXIT", // [1382]
    "EXPECTED", // [1383]
    "EXPLAIN", // [1384]
    "SETTINGS", // [1385]
    "SUMMARY", // [1386]
    "TIMING", // [1387]
    "WAL", // [1388]
    "CostsKeywordSegment", // [1389]
    "SettingsKeywordSegment", // [1390]
    "BuffersKeywordSegment", // [1391]
    "WalKeywordSegment", // [1392]
    "TimingKeywordSegment", // [1393]
    "SummaryKeywordSegment", // [1394]
    "YAML", // [1395]
    "YamlKeywordSegment", // [1396]
    "ExplainKeywordSegment", // [1397]
    "ExplainOptionSegment", // [1398]
    "explain_option", // [1399]
    "InsertStatementSegment", // [1400]
    "insert_statement", // [1401]
    "UpdateStatementSegment", // [1402]
    "update_statement", // [1403]
    "DeleteStatementSegment", // [1404]
    "delete_statement", // [1405]
    "Expression_A_Grammar", // [1406]
    "Tail_Recurse_Expression_A_Grammar", // [1407]
    "LikeExpressionGrammar", // [1408]
    "ILIKE", // [1409]
    "SIMILAR", // [1410]
    "InOperatorGrammar", // [1411]
    "QuantifiedComparisonOperatorGrammar", // [1412]
    "IsClauseGrammar", // [1413]
    "NAN", // [1414]
    "NFC", // [1415]
    "NFD", // [1416]
    "NFKC", // [1417]
    "NFKD", // [1418]
    "NORMALIZED", // [1419]
    "UNKNOWN", // [1420]
    "IsNullGrammar", // [1421]
    "ISNULL", // [1422]
    "NotNullGrammar", // [1423]
    "NOTNULL", // [1424]
    "CollateGrammar", // [1425]
    "BetweenKeywordSegment", // [1426]
    "Expression_B_Grammar", // [1427]
    "AndKeywordSegment", // [1428]
    "PatternMatchingGrammar", // [1429]
    "PRIOR", // [1430]
    "~", // [1431]
    "TildeSegment", // [1432]
    "NotOperatorGrammar", // [1433]
    "PriorKeywordSegment", // [1434]
    "Tail_Recurse_Expression_B_Grammar", // [1435]
    "WalrusOperatorSegment", // [1436]
    ":=", // [1437]
    "ExistsKeywordSegment", // [1438]
    "Expression_D_Grammar", // [1439]
    "CaseExpressionSegment", // [1440]
    "case_expression", // [1441]
    "TimeZoneGrammar", // [1442]
    "time_zone_grammar", // [1443]
    "FullTextSearchOperatorSegment", // [1444]
    "full_text_search_operator", // [1445]
    "LateralColumnAliasExpressionGrammar", // [1446]
    "LocalAliasSegment", // [1447]
    "local_alias_segment", // [1448]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [1449]
    "SingleIdentifierWildcardSegment", // [1450]
    "wildcard_identifier", // [1451]
    "NullLiteralSegment", // [1452]
    "DateTimeLiteralGrammar", // [1453]
    "datetime_literal", // [1454]
    "ListComprehensionGrammar", // [1455]
    "AccessorGrammar", // [1456]
    "TypedStructLiteralSegment", // [1457]
    "typed_struct_literal", // [1458]
    "ArrayExpressionSegment", // [1459]
    "array_expression", // [1460]
    "OverlapsClauseSegment", // [1461]
    "overlaps_clause", // [1462]
    "EXTRACT", // [1463]
    "FACTOR", // [1464]
    "boolean_literal", // [1465]
    "LiteralKeywordSegment", // [1466]
    "FETCH", // [1467]
    "FetchKeywordSegment", // [1468]
    "NEXT", // [1469]
    "NextKeywordSegment", // [1470]
    "TiesKeywordSegment", // [1471]
    "TIES", // [1472]
    "FORWARD", // [1473]
    "ForwardKeywordSegment", // [1474]
    "FIELDS", // [1475]
    "MERGE", // [1476]
    "ROLLBACK", // [1477]
    "SHOW", // [1478]
    "USE", // [1479]
    "postgres_copy_stdin_data_block", // [1480]
    "psql_copy_command", // [1481]
    "psql_set_command", // [1482]
    "PostgresCopyStdinDataStatementSegment", // [1483]
    "postgres_copy_stdin_data_statement", // [1484]
    "PsqlCopyMetaCommandStatementSegment", // [1485]
    "psql_copy_meta_command_statement", // [1486]
    "PsqlSetMetaCommandStatementSegment", // [1487]
    "psql_set_meta_command_statement", // [1488]
    "StatementSegment", // [1489]
    "statement", // [1490]
    "FILLFACTOR", // [1491]
    "FILTER", // [1492]
    "FilterKeywordSegment", // [1493]
    "FLUSH", // [1494]
    "FOLLOWING", // [1495]
    "SHARE", // [1496]
    "ShareKeywordSegment", // [1497]
    "LockedKeywordSegment", // [1498]
    "LOCKED", // [1499]
    "GROUPS", // [1500]
    "FrameClauseUnitGrammar", // [1501]
    "UnboundedKeywordSegment", // [1502]
    "UNBOUNDED", // [1503]
    "PRECEDING", // [1504]
    "PrecedingKeywordSegment", // [1505]
    "FollowingKeywordSegment", // [1506]
    "ExcludeKeywordSegment", // [1507]
    "OthersKeywordSegment", // [1508]
    "OTHERS", // [1509]
    "GroupsKeywordSegment", // [1510]
    "FREETEXT", // [1511]
    "FREETEXTTABLE", // [1512]
    "FromExpressionSegment", // [1513]
    "from_expression", // [1514]
    "HAVING", // [1515]
    "INTERSECT", // [1516]
    "MINUS", // [1517]
    "OFFSET", // [1518]
    "QUALIFY", // [1519]
    "UNION", // [1520]
    "LimitClauseSegment", // [1521]
    "limit_clause", // [1522]
    "OrderKeywordSegment", // [1523]
    "HavingKeywordSegment", // [1524]
    "QualifyKeywordSegment", // [1525]
    "WindowKeywordSegment", // [1526]
    "SetOperatorSegment", // [1527]
    "set_operator", // [1528]
    "WithNoSchemaBindingClauseSegment", // [1529]
    "with_no_schema_binding_clause", // [1530]
    "OffsetKeywordSegment", // [1531]
    "ForClauseSegment", // [1532]
    "for_clause", // [1533]
    "PreTableFunctionKeywordsGrammar", // [1534]
    "LATERAL", // [1535]
    "TemporalQuerySegment", // [1536]
    "temporal_query", // [1537]
    "FromClauseTerminatorGrammar", // [1538]
    "SamplingExpressionSegment", // [1539]
    "TABLESAMPLE", // [1540]
    "sample_expression", // [1541]
    "PostTableExpressionGrammar", // [1542]
    "MLTableExpressionSegment", // [1543]
    "ML", // [1544]
    "ml_table_expression", // [1545]
    "FromExpressionElementSegment", // [1546]
    "from_expression_element", // [1547]
    "indented_joins", // [1548]
    "FULLNAME", // [1549]
    "FULLTEXT", // [1550]
    "NamedArgumentSegment", // [1551]
    "named_argument", // [1552]
    "VariadicKeywordSegment", // [1553]
    "LEADING", // [1554]
    "TRAILING", // [1555]
    "TrimParametersGrammar", // [1556]
    "FunctionContentsExpressionGrammar", // [1557]
    "AggregateOrderByClause", // [1558]
    "aggregate_order_by", // [1559]
    "SEPARATOR", // [1560]
    "SeparatorKeywordSegment", // [1561]
    "IgnoreRespectNullsGrammar", // [1562]
    "IGNORE", // [1563]
    "RESPECT", // [1564]
    "IndexColumnDefinitionSegment", // [1565]
    "index_column_definition", // [1566]
    "EmptyStructLiteralSegment", // [1567]
    "ReturnKeywordSegment", // [1568]
    "BeginKeywordSegment", // [1569]
    "AtomicKeywordSegment", // [1570]
    "WithCompoundStatementSegment", // [1571]
    "with_compound_statement", // [1572]
    "FunctionNameIdentifierSegment", // [1573]
    "OUT", // [1574]
    "OutKeywordSegment", // [1575]
    "FunctionParameterGrammar", // [1576]
    "DatePartFunctionNameSegment", // [1577]
    "DateTimeFunctionContentsSegment", // [1578]
    "function_contents", // [1579]
    "ColumnsExpressionGrammar", // [1580]
    "ColumnsExpressionFunctionNameSegment", // [1581]
    "FunctionContentsSegment", // [1582]
    "PostFunctionGrammar", // [1583]
    "FUTURE", // [1584]
    "glob_operator", // [1585]
    "AccessObjectSegment", // [1586]
    "access_object", // [1587]
    "AccessTargetSegment", // [1588]
    "access_target", // [1589]
    "GRANTED", // [1590]
    "GRANTS", // [1591]
    "GRAPH", // [1592]
    "GREATEST", // [1593]
    "CubeRollupClauseSegment", // [1594]
    "cube_rollup_clause", // [1595]
    "GroupingSetsClauseSegment", // [1596]
    "GROUPING", // [1597]
    "grouping_sets_clause", // [1598]
    "GroupByClauseTerminatorGrammar", // [1599]
    "GroupingKeywordSegment", // [1600]
    "SetsKeywordSegment", // [1601]
    "SETS", // [1602]
    "GZIP", // [1603]
    "HEAP", // [1604]
    "HIGH_PRIORITY", // [1605]
    "HOLDLOCK", // [1606]
    "HOST", // [1607]
    "HOSTS", // [1608]
    "HOUR_MICROSECOND", // [1609]
    "HOUR_MINUTE", // [1610]
    "HOUR_SECOND", // [1611]
    "HOURS", // [1612]
    "ID", // [1613]
    "IDEMPOTENCE", // [1614]
    "IDENTIFIED", // [1615]
    "identifier", // [1616]
    "IdentifierSegment", // [1617]
    "IDENTITY_INSERT", // [1618]
    "IDENTITYCOL", // [1619]
    "IDLE", // [1620]
    "IfKeywordSegment", // [1621]
    "IgnoreKeywordSegment", // [1622]
    "RespectKeywordSegment", // [1623]
    "IMPORT", // [1624]
    "ImportKeywordSegment", // [1625]
    "ImportForeignSchemaGrammar", // [1626]
    "ExceptKeywordSegment", // [1627]
    "NakedIdentifierFullSegment", // [1628]
    "IMPORTED", // [1629]
    "AscKeywordSegment", // [1630]
    "DescKeywordSegment", // [1631]
    "OperatorClassReferenceSegment", // [1632]
    "LAST", // [1633]
    "LastKeywordSegment", // [1634]
    "operator_class_reference", // [1635]
    "IndexElementOptionsSegment", // [1636]
    "index_element_options", // [1637]
    "INDEX_CLEANUP", // [1638]
    "INDEXES", // [1639]
    "INFILE", // [1640]
    "INFIX", // [1641]
    "INFO", // [1642]
    "INITIALIZE", // [1643]
    "INLINE", // [1644]
    "INSENSITIVE", // [1645]
    "AsAliasExpressionSegment", // [1646]
    "OVERRIDING", // [1647]
    "OverridingKeywordSegment", // [1648]
    "ConflictKeywordSegment", // [1649]
    "ConflictTargetSegment", // [1650]
    "conflict_target", // [1651]
    "ConflictActionSegment", // [1652]
    "conflict_action", // [1653]
    "INSERT_ID", // [1654]
    "materialize_size", // [1655]
    "compression_type", // [1656]
    "INT1", // [1657]
    "INT3", // [1658]
    "INTEGRATION", // [1659]
    "INTEGRATIONS", // [1660]
    "YearKeywordSegment", // [1661]
    "MonthKeywordSegment", // [1662]
    "DayKeywordSegment", // [1663]
    "HourKeywordSegment", // [1664]
    "MinuteKeywordSegment", // [1665]
    "SecondKeywordSegment", // [1666]
    "INTROSPECTION", // [1667]
    "NanLiteralSegment", // [1668]
    "UnknownLiteralSegment", // [1669]
    "NormalizedGrammar", // [1670]
    "JsonTestGrammar", // [1671]
    "IsnullKeywordSegment", // [1672]
    "IS_SUPERUSER", // [1673]
    "ISAM", // [1674]
    "ISOLATION", // [1675]
    "ITERATE", // [1676]
    "ConditionalJoinKeywordsGrammar", // [1677]
    "JoinKeywordsGrammar", // [1678]
    "NestedJoinGrammar", // [1679]
    "indented_using_on", // [1680]
    "MatchConditionSegment", // [1681]
    "match_condition", // [1682]
    "JoinOnConditionSegment", // [1683]
    "join_on_condition", // [1684]
    "JoinUsingConditionGrammar", // [1685]
    "UnconditionalJoinKeywordsGrammar", // [1686]
    "ExtendedNaturalJoinKeywordsGrammar", // [1687]
    "indented_on_contents", // [1688]
    "InnerKeywordSegment", // [1689]
    "LeftKeywordSegment", // [1690]
    "RightKeywordSegment", // [1691]
    "OuterKeywordSegment", // [1692]
    "OUTER", // [1693]
    "JsonTypeGrammar", // [1694]
    "OBJECT", // [1695]
    "SCALAR", // [1696]
    "JsonUniqueKeysGrammar", // [1697]
    "ScalarKeywordSegment", // [1698]
    "ObjectKeywordSegment", // [1699]
    "KILL", // [1700]
    "KINESIS", // [1701]
    "LABEL", // [1702]
    "LANCOMPILER", // [1703]
    "SingleQuotedIdentifierSegment", // [1704]
    "LAST_INSERT_ID", // [1705]
    "LATEST", // [1706]
    "LEAST", // [1707]
    "LEAVE", // [1708]
    "LESS", // [1709]
    "LikeGrammar", // [1710]
    "IlikeKeywordSegment", // [1711]
    "SimilarKeywordSegment", // [1712]
    "IncludingKeywordSegment", // [1713]
    "ExcludingKeywordSegment", // [1714]
    "CommentsKeywordSegment", // [1715]
    "ConstraintsKeywordSegment", // [1716]
    "DefaultsKeywordSegment", // [1717]
    "IndexesKeywordSegment", // [1718]
    "LINENO", // [1719]
    "LINES", // [1720]
    "LISTEN", // [1721]
    "ListenKeywordSegment", // [1722]
    "DollarNumericLiteralSegment", // [1723]
    "ArrayLiteralSegment", // [1724]
    "array_literal", // [1725]
    "TypedArrayLiteralSegment", // [1726]
    "typed_array_literal", // [1727]
    "ObjectLiteralSegment", // [1728]
    "object_literal", // [1729]
    "LOCATION", // [1730]
    "LOCK", // [1731]
    "LockKeywordSegment", // [1732]
    "ExclusiveKeywordSegment", // [1733]
    "ModeKeywordSegment", // [1734]
    "MODE", // [1735]
    "LOCKS", // [1736]
    "LOG", // [1737]
    "LOGICAL", // [1738]
    "LOGS", // [1739]
    "LONG", // [1740]
    "LONGBLOB", // [1741]
    "LONGTEXT", // [1742]
    "LOOP", // [1743]
    "LOW_PRIORITY", // [1744]
    "MlKeywordSegment", // [1745]
    "MANAGE", // [1746]
    "MANAGED", // [1747]
    "MATCH", // [1748]
    "MATCHED", // [1749]
    "MATCHING", // [1750]
    "OPTIMIZED", // [1751]
    "PHYSICAL", // [1752]
    "PLAN", // [1753]
    "RAW", // [1754]
    "RawKeywordSegment", // [1755]
    "DecorrelatedKeywordSegment", // [1756]
    "OptimizedKeywordSegment", // [1757]
    "PhysicalKeywordSegment", // [1758]
    "PlanKeywordSegment", // [1759]
    "MATERIALIZE", // [1760]
    "MAX", // [1761]
    "MAX_ROWS", // [1762]
    "MAXEXTENTS", // [1763]
    "MECHANISMS", // [1764]
    "MEDIUMBLOB", // [1765]
    "MEDIUMINT", // [1766]
    "MEDIUMTEXT", // [1767]
    "MergeKeywordSegment", // [1768]
    "MergeMatchedClauseSegment", // [1769]
    "merge_when_matched_clause", // [1770]
    "MergeNotMatchedClauseSegment", // [1771]
    "merge_when_not_matched_clause", // [1772]
    "MergeNotMatchedBySourceClauseSegment", // [1773]
    "merge_when_not_matched_by_source_clause", // [1774]
    "MatchedKeywordSegment", // [1775]
    "ThenKeywordSegment", // [1776]
    "THEN", // [1777]
    "MergeUpdateClauseSegment", // [1778]
    "merge_update_clause", // [1779]
    "MergeDeleteClauseSegment", // [1780]
    "merge_delete_clause", // [1781]
    "TargetKeywordSegment", // [1782]
    "TARGET", // [1783]
    "MergeInsertClauseSegment", // [1784]
    "merge_insert_clause", // [1785]
    "MergeIntoLiteralGrammar", // [1786]
    "AliasedTableReferenceGrammar", // [1787]
    "MergeMatchSegment", // [1788]
    "merge_match", // [1789]
    "SetClauseListSegment", // [1790]
    "set_clause_list", // [1791]
    "MESSAGE", // [1792]
    "meta_command_query_buffer", // [1793]
    "meta_command", // [1794]
    "MetaCommandQueryBufferSegment", // [1795]
    "METADATA", // [1796]
    "MIDDLEINT", // [1797]
    "MIN_ROWS", // [1798]
    "MINUTE_MICROSECOND", // [1799]
    "MINUTE_SECOND", // [1800]
    "MINUTES", // [1801]
    "MLSLABEL", // [1802]
    "MODULUS", // [1803]
    "MONITOR", // [1804]
    "MONTHNAME", // [1805]
    "MONTHS", // [1806]
    "MOVE", // [1807]
    "MS", // [1808]
    "MultilineConcatenateNewline", // [1809]
    "NewlineSegment", // [1810]
    "MYISAM", // [1811]
    "naked_identifier_all", // [1812]
    "naked_identifier", // [1813]
    "=>", // [1814]
    "RightArrowSegment", // [1815]
    "WindowSpecificationSegment", // [1816]
    "window_specification", // [1817]
    "NamedWindowExpressionSegment", // [1818]
    "named_window_expression", // [1819]
    "NAMES", // [1820]
    "null_literal", // [1821]
    "NATIONAL", // [1822]
    "NATURAL", // [1823]
    "NaturalKeywordSegment", // [1824]
    "NCHAR", // [1825]
    "sign_indicator", // [1826]
    "NO_WRITE_TO_BINLOG", // [1827]
    "NOAUDIT", // [1828]
    "NOCACHE", // [1829]
    "NOCHECK", // [1830]
    "NOCOMPRESS", // [1831]
    "NOCREATEUSER", // [1832]
    "NOCYCLE", // [1833]
    "UnorderedSelectStatementSegment", // [1834]
    "NonSetSelectableGrammar", // [1835]
    "BracketedSetExpressionGrammar", // [1836]
    "NONCLUSTERED", // [1837]
    "NOORDER", // [1838]
    "NORMALIZE", // [1839]
    "NfcKeywordSegment", // [1840]
    "NfdKeywordSegment", // [1841]
    "NfkcKeywordSegment", // [1842]
    "NfkdKeywordSegment", // [1843]
    "NormalizedKeywordSegment", // [1844]
    "RawNotSegment", // [1845]
    "NotnullKeywordSegment", // [1846]
    "NOTICE", // [1847]
    "NOTIFICATIONS", // [1848]
    "NOTIFY", // [1849]
    "NotifyKeywordSegment", // [1850]
    "NULLIF", // [1851]
    "ObjectLiteralElementSegment", // [1852]
    "object_literal_element", // [1853]
    "start_curly_bracket", // [1854]
    "OBJECTS", // [1855]
    "OFF", // [1856]
    "OFFLINE", // [1857]
    "OFFSETS", // [1858]
    "ONLINE", // [1859]
    "OPENDATASOURCE", // [1860]
    "OPENQUERY", // [1861]
    "OPENROWSET", // [1862]
    "OPENXML", // [1863]
    "OPERATE", // [1864]
    "OPERATION", // [1865]
    "OPTIMIZE", // [1866]
    "OPTIMIZER", // [1867]
    "OPTIONALLY", // [1868]
    "OptionsListGrammar", // [1869]
    "WithFillSegment", // [1870]
    "with_fill", // [1871]
    "NoorderKeywordSegment", // [1872]
    "ORDINALITY", // [1873]
    "OUTFILE", // [1874]
    "OverKeywordSegment", // [1875]
    "OVER", // [1876]
    "OverlapsKeywordSegment", // [1877]
    "OVERLAPS", // [1878]
    "OVERLAY", // [1879]
    "OVERWRITE", // [1880]
    "OWNERSHIP", // [1881]
    "PACK_KEYS", // [1882]
    "parameter", // [1883]
    "?", // [1884]
    "PARAMETERS", // [1885]
    "PARTIAL", // [1886]
    "ModulusKeywordSegment", // [1887]
    "RemainderKeywordSegment", // [1888]
    "REMAINDER", // [1889]
    "PASSING", // [1890]
    "SlashSegment", // [1891]
    "path_segment", // [1892]
    "WordSegment", // [1893]
    "PCTFREE", // [1894]
    "PIPE", // [1895]
    "pipe", // [1896]
    "PLACING", // [1897]
    "PORT", // [1898]
    "POSITION", // [1899]
    "WithinGroupClauseSegment", // [1900]
    "WITHIN", // [1901]
    "withingroup_clause", // [1902]
    "OverClauseSegment", // [1903]
    "over_clause", // [1904]
    "FilterClauseGrammar", // [1905]
    "POSTFIX", // [1906]
    "PostgresCopyStdinDataSegment", // [1907]
    "LateralKeywordSegment", // [1908]
    "PREFIX", // [1909]
    "PREORDER", // [1910]
    "MergeStatementSegment", // [1911]
    "merge_statement", // [1912]
    "PREPARED", // [1913]
    "PRINT", // [1914]
    "PROC", // [1915]
    "PROCESS", // [1916]
    "PROCESS_TOAST", // [1917]
    "PROCESSLIST", // [1918]
    "PROGRESS", // [1919]
    "properties_naked_identifier", // [1920]
    "PROTOBUF", // [1921]
    "PsqlCopyMetaCommandSegment", // [1922]
    "PsqlSetMetaCommandSegment", // [1923]
    "psql_variable_colon", // [1924]
    "PsqlVariableColonSegment", // [1925]
    "PublicationTableSegment", // [1926]
    "publication_table", // [1927]
    "ExtendedTableReferenceGrammar", // [1928]
    "PURGE", // [1929]
    "operator", // [1930]
    "SOME", // [1931]
    "AnyKeywordSegment", // [1932]
    "SomeKeywordSegment", // [1933]
    "QUERY", // [1934]
    "quoted_identifier", // [1935]
    "quoted_literal", // [1936]
    "MultilineConcatenateDelimiterGrammar", // [1937]
    "UESCAPE", // [1938]
    "UescapeKeywordSegment", // [1939]
    "unicode_escape_value", // [1940]
    "RAID0", // [1941]
    "RAISE", // [1942]
    "RAISERROR", // [1943]
    "raw_comparison_operator", // [1944]
    "READ", // [1945]
    "READTEXT", // [1946]
    "REASSIGN", // [1947]
    "ReassignKeywordSegment", // [1948]
    "RECHECK", // [1949]
    "RECONFIGURE", // [1950]
    "RECURSIVE", // [1951]
    "REF", // [1952]
    "ReferenceMatchGrammar", // [1953]
    "ReferentialActionGrammar", // [1954]
    "MatchKeywordSegment", // [1955]
    "SIMPLE", // [1956]
    "PartialKeywordSegment", // [1957]
    "SimpleKeywordSegment", // [1958]
    "REFERENCE_USAGE", // [1959]
    "ActionKeywordSegment", // [1960]
    "REGEX", // [1961]
    "REGEXP", // [1962]
    "REGION", // [1963]
    "REINDEX", // [1964]
    "ReindexKeywordSegment", // [1965]
    "RelationOptionSegment", // [1966]
    "relation_option", // [1967]
    "RELATIVE", // [1968]
    "RELEASE", // [1969]
    "RELOAD", // [1970]
    "REMOTE", // [1971]
    "REPEAT", // [1972]
    "REPEATABLE", // [1973]
    "REQUIRE", // [1974]
    "SessionKeywordSegment", // [1975]
    "SESSION", // [1976]
    "RESIGNAL", // [1977]
    "RESOURCE", // [1978]
    "RETENTION", // [1979]
    "RETRIEVE", // [1980]
    "OwnershipKeywordSegment", // [1981]
    "right_arrow", // [1982]
    "RLIKE", // [1983]
    "ROWCOUNT", // [1984]
    "ROWGUIDCOL", // [1985]
    "ROWID", // [1986]
    "ROWNUM", // [1987]
    "S3", // [1988]
    "TablesampleKeywordSegment", // [1989]
    "BernoulliKeywordSegment", // [1990]
    "RepeatableKeywordSegment", // [1991]
    "SASL", // [1992]
    "SAVE", // [1993]
    "SAVEPOINT", // [1994]
    "SCALE", // [1995]
    "SCAN", // [1996]
    "SCRIPT", // [1997]
    "SCROLL", // [1998]
    "SECOND_MICROSECOND", // [1999]
    "SECONDS", // [2000]
    "LabelKeywordSegment", // [2001]
    "LargeKeywordSegment", // [2002]
    "SEED", // [2003]
    "WildcardExpressionSegment", // [2004]
    "wildcard_expression", // [2005]
    "SelectClauseModifierSegment", // [2006]
    "select_clause_modifier", // [2007]
    "SelectClauseElementSegment", // [2008]
    "select_clause_element", // [2009]
    "WithCheckOptionSegment", // [2010]
    "with_check_option", // [2011]
    "SelectClauseSegment", // [2012]
    "select_clause", // [2013]
    "IntoClauseSegment", // [2014]
    "into_clause", // [2015]
    "FromClauseSegment", // [2016]
    "from_clause", // [2017]
    "GroupByClauseSegment", // [2018]
    "groupby_clause", // [2019]
    "HavingClauseSegment", // [2020]
    "having_clause", // [2021]
    "NamedWindowSegment", // [2022]
    "named_window", // [2023]
    "OffsetClauseSegment", // [2024]
    "offset_clause", // [2025]
    "FetchClauseSegment", // [2026]
    "fetch_clause", // [2027]
    "WithCompoundNonSelectStatementSegment", // [2028]
    "NonWithSelectableGrammar", // [2029]
    "statement_terminator", // [2030]
    "SERIALIZABLE", // [2031]
    "SERVER_ENCODING", // [2032]
    "SERVER_VERSION", // [2033]
    "SERVICE", // [2034]
    "SetClauseSegment", // [2035]
    "set_clause", // [2036]
    "UnionGrammar", // [2037]
    "IntersectKeywordSegment", // [2038]
    "MinusKeywordSegment", // [2039]
    "TimeKeywordSegment", // [2040]
    "ZoneKeywordSegment", // [2041]
    "ZONE", // [2042]
    "SETUSER", // [2043]
    "SHARES", // [2044]
    "ShowKeywordSegment", // [2045]
    "SINKS", // [2046]
    "SOURCES", // [2047]
    "ColumnsKeywordSegment", // [2048]
    "SinksKeywordSegment", // [2049]
    "SourcesKeywordSegment", // [2050]
    "ObjectsKeywordSegment", // [2051]
    "SHUTDOWN", // [2052]
    "SIGNAL", // [2053]
    "PositiveSegment", // [2054]
    "NegativeSegment", // [2055]
    "slash", // [2056]
    "slice", // [2057]
    "SNAPSHOT", // [2058]
    "SONAME", // [2059]
    "SPATIAL", // [2060]
    "SQL", // [2061]
    "SQL_BIG_RESULT", // [2062]
    "SQL_BIG_SELECTS", // [2063]
    "SQL_BIG_TABLES", // [2064]
    "SQL_CALC_FOUND_ROWS", // [2065]
    "SQL_LOG_OFF", // [2066]
    "SQL_LOG_UPDATE", // [2067]
    "SQL_LOW_PRIORITY_UPDATES", // [2068]
    "SQL_SELECT_LIMIT", // [2069]
    "SQL_SMALL_RESULT", // [2070]
    "SQL_WARNINGS", // [2071]
    "SQLCA", // [2072]
    "SQS", // [2073]
    "SSL", // [2074]
    "STANDALONE", // [2075]
    "star", // [2076]
    "STARTING", // [2077]
    "STARTS", // [2078]
    "TransactionStatementSegment", // [2079]
    "transaction_statement", // [2080]
    "DropTableStatementSegment", // [2081]
    "drop_table_statement", // [2082]
    "DropViewStatementSegment", // [2083]
    "drop_view_statement", // [2084]
    "CreateUserStatementSegment", // [2085]
    "create_user_statement", // [2086]
    "DropUserStatementSegment", // [2087]
    "drop_user_statement", // [2088]
    "TruncateStatementSegment", // [2089]
    "truncate_table", // [2090]
    "AccessStatementSegment", // [2091]
    "access_statement", // [2092]
    "CreateTableStatementSegment", // [2093]
    "create_table_statement", // [2094]
    "CreateRoleStatementSegment", // [2095]
    "create_role_statement", // [2096]
    "DropRoleStatementSegment", // [2097]
    "drop_role_statement", // [2098]
    "AlterTableStatementSegment", // [2099]
    "alter_table_statement", // [2100]
    "CreateSchemaStatementSegment", // [2101]
    "create_schema_statement", // [2102]
    "SetSchemaStatementSegment", // [2103]
    "set_schema_statement", // [2104]
    "DropSchemaStatementSegment", // [2105]
    "drop_schema_statement", // [2106]
    "DropTypeStatementSegment", // [2107]
    "drop_type_statement", // [2108]
    "CreateDatabaseStatementSegment", // [2109]
    "create_database_statement", // [2110]
    "DropDatabaseStatementSegment", // [2111]
    "drop_database_statement", // [2112]
    "CreateViewStatementSegment", // [2113]
    "create_view_statement", // [2114]
    "CreateCastStatementSegment", // [2115]
    "create_cast_statement", // [2116]
    "DropCastStatementSegment", // [2117]
    "drop_cast_statement", // [2118]
    "CreateFunctionStatementSegment", // [2119]
    "create_function_statement", // [2120]
    "DropFunctionStatementSegment", // [2121]
    "drop_function_statement", // [2122]
    "CreateModelStatementSegment", // [2123]
    "create_model_statement", // [2124]
    "DropModelStatementSegment", // [2125]
    "drop_MODELstatement", // [2126]
    "DescribeStatementSegment", // [2127]
    "describe_statement", // [2128]
    "UseStatementSegment", // [2129]
    "use_statement", // [2130]
    "ExplainStatementSegment", // [2131]
    "explain_statement", // [2132]
    "CreateSequenceStatementSegment", // [2133]
    "create_sequence_statement", // [2134]
    "AlterSequenceStatementSegment", // [2135]
    "alter_sequence_statement", // [2136]
    "DropSequenceStatementSegment", // [2137]
    "drop_sequence_statement", // [2138]
    "CreateTriggerStatementSegment", // [2139]
    "create_trigger", // [2140]
    "DropTriggerStatementSegment", // [2141]
    "drop_trigger", // [2142]
    "AlterOwnerStatementSegment", // [2143]
    "alter_owner_statement", // [2144]
    "AlterConnectionRotateKeys", // [2145]
    "alter_connection_rotate_keys", // [2146]
    "AlterDefaultPrivilegesStatementSegment", // [2147]
    "alter_default_privileges_statement", // [2148]
    "AlterIndexStatementSegment", // [2149]
    "alter_index_statement", // [2150]
    "AlterRenameStatementSegment", // [2151]
    "alter_rename_statement", // [2152]
    "AlterSecretStatementSegment", // [2153]
    "alter_secret_statement", // [2154]
    "AlterSetClusterStatementSegment", // [2155]
    "alter_set_cluster_statement", // [2156]
    "AlterSourceSinkSizeStatementSegment", // [2157]
    "alter_source_sink_size_statement", // [2158]
    "CloseStatementSegment", // [2159]
    "close_statement", // [2160]
    "CopyToStatementSegment", // [2161]
    "copy_to_statement", // [2162]
    "CopyFromStatementSegment", // [2163]
    "copy_from_statement", // [2164]
    "CreateClusterStatementSegment", // [2165]
    "create_cluster_statement", // [2166]
    "CreateClusterReplicaStatementSegment", // [2167]
    "create_cluster_replica_statement", // [2168]
    "CreateConnectionStatementSegment", // [2169]
    "create_connection_statement", // [2170]
    "CreateIndexStatementSegment", // [2171]
    "create_index_statement", // [2172]
    "CreateMaterializedViewStatementSegment", // [2173]
    "create_materialized_view_statement", // [2174]
    "CreateSecretStatementSegment", // [2175]
    "create_secret_statement", // [2176]
    "CreateSinkKafkaStatementSegment", // [2177]
    "create_sink_kafka_statement", // [2178]
    "CreateSourceKafkaStatementSegment", // [2179]
    "create_source_kafka_statement", // [2180]
    "CreateSourceLoadGeneratorStatementSegment", // [2181]
    "create_source_load_generator_statement", // [2182]
    "CreateSourcePostgresStatementSegment", // [2183]
    "create_source_postgres_statement", // [2184]
    "CreateSourceWebhookStatementSegment", // [2185]
    "CreateTypeStatementSegment", // [2186]
    "create_type_statement", // [2187]
    "DropStatementSegment", // [2188]
    "drop_statement", // [2189]
    "FetchStatementSegment", // [2190]
    "fetch_statement", // [2191]
    "MaterializeExplainStatementSegment", // [2192]
    "ShowStatementSegment", // [2193]
    "show_statement", // [2194]
    "ShowCreateStatementSegment", // [2195]
    "show_create_statement", // [2196]
    "ShowIndexesStatementSegment", // [2197]
    "show_indexes_statement", // [2198]
    "ShowMaterializedViewsStatementSegment", // [2199]
    "show_materialized_views_statement", // [2200]
    "DeclareStatementSegment", // [2201]
    "declare_statement", // [2202]
    "STRAIGHT_JOIN", // [2203]
    "STRATEGY", // [2204]
    "ConcatSegment", // [2205]
    "STRING", // [2206]
    "STRIP", // [2207]
    "SUBLIST", // [2208]
    "SUBSOURCE", // [2209]
    "SUBSTRING", // [2210]
    "SUCCESSFUL", // [2211]
    "SYMMETRIC", // [2212]
    "SYNONYM", // [2213]
    "SYSDATE", // [2214]
    "IndexParametersSegment", // [2215]
    "index_parameters", // [2216]
    "PrimaryKeyGrammar", // [2217]
    "ExclusionConstraintElementSegment", // [2218]
    "exclusion_constraint_element", // [2219]
    "OrdinalityKeywordSegment", // [2220]
    "TAIL", // [2221]
    "Expression_A_Unary_Operator_Grammar", // [2222]
    "Expression_C_Grammar", // [2223]
    "Expression_B_Unary_Operator_Grammar", // [2224]
    "TRANSIENT", // [2225]
    "TransientKeywordSegment", // [2226]
    "TERMINATE", // [2227]
    "TERMINATED", // [2228]
    "TEST", // [2229]
    "TEXTSIZE", // [2230]
    "THAN", // [2231]
    "TICK", // [2232]
    "tilde", // [2233]
    "TimestampKeywordSegment", // [2234]
    "AtKeywordSegment", // [2235]
    "TIMELINE", // [2236]
    "TIMEOUT", // [2237]
    "TINYBLOB", // [2238]
    "TINYINT", // [2239]
    "TINYTEXT", // [2240]
    "TOAST", // [2241]
    "TOKEN", // [2242]
    "TOP", // [2243]
    "TOPIC", // [2244]
    "TRACE", // [2245]
    "TRAN", // [2246]
    "TRANSACTION", // [2247]
    "RollbackKeywordSegment", // [2248]
    "WORK", // [2249]
    "TransactionKeywordSegment", // [2250]
    "WorkKeywordSegment", // [2251]
    "ChainKeywordSegment", // [2252]
    "TRANSACTIONS", // [2253]
    "TREAT", // [2254]
    "TRIM", // [2255]
    "BothKeywordSegment", // [2256]
    "LeadingKeywordSegment", // [2257]
    "TrailingKeywordSegment", // [2258]
    "ContinueKeywordSegment", // [2259]
    "TRUSTED", // [2260]
    "TSEQUAL", // [2261]
    "StructLiteralSegment", // [2262]
    "UID", // [2263]
    "UNCOMMITTED", // [2264]
    "CrossKeywordSegment", // [2265]
    "NaturalJoinKeywordsGrammar", // [2266]
    "UnconditionalCrossJoinKeywordsGrammar", // [2267]
    "HorizontalJoinKeywordsGrammar", // [2268]
    "UNDO", // [2269]
    "UNENCRYPTED", // [2270]
    "UnionKeywordSegment", // [2271]
    "UNLISTEN", // [2272]
    "UnlistenKeywordSegment", // [2273]
    "UNLOCK", // [2274]
    "UNSIGNED", // [2275]
    "UPDATETEXT", // [2276]
    "URL", // [2277]
    "UseKeywordSegment", // [2278]
    "USE_ANY_ROLE", // [2279]
    "USERNAME", // [2280]
    "USERS", // [2281]
    "UTC_DATE", // [2282]
    "UTC_TIME", // [2283]
    "UTC_TIMESTAMP", // [2284]
    "VACUUM", // [2285]
    "VacuumKeywordSegment", // [2286]
    "Disable_page_skippingKeywordSegment", // [2287]
    "Index_cleanupKeywordSegment", // [2288]
    "Process_toastKeywordSegment", // [2289]
    "VARCHAR2", // [2290]
    "VARCHARACTER", // [2291]
    "VARIABLE", // [2292]
    "VARIABLES", // [2293]
    "WAITFOR", // [2294]
    "assignment_operator", // [2295]
    "WAREHOUSE", // [2296]
    "WAREHOUSES", // [2297]
    "WARNING", // [2298]
    "LinestringKeywordSegment", // [2299]
    "MultipointKeywordSegment", // [2300]
    "MultilinestringKeywordSegment", // [2301]
    "MultipolygonKeywordSegment", // [2302]
    "GeometrycollectionKeywordSegment", // [2303]
    "PointzKeywordSegment", // [2304]
    "LinestringzKeywordSegment", // [2305]
    "PolygonzKeywordSegment", // [2306]
    "MultipointzKeywordSegment", // [2307]
    "MultilinestringzKeywordSegment", // [2308]
    "MultipolygonzKeywordSegment", // [2309]
    "GeometrycollectionzKeywordSegment", // [2310]
    "PointmKeywordSegment", // [2311]
    "LinestringmKeywordSegment", // [2312]
    "PolygonmKeywordSegment", // [2313]
    "MultipointmKeywordSegment", // [2314]
    "MultilinestringmKeywordSegment", // [2315]
    "MultipolygonmKeywordSegment", // [2316]
    "GeometrycollectionmKeywordSegment", // [2317]
    "PointzmKeywordSegment", // [2318]
    "LinestringzmKeywordSegment", // [2319]
    "PolygonzmKeywordSegment", // [2320]
    "MultipointzmKeywordSegment", // [2321]
    "MultilinestringzmKeywordSegment", // [2322]
    "MultipolygonzmKeywordSegment", // [2323]
    "GeometrycollectionzmKeywordSegment", // [2324]
    "CircularstringKeywordSegment", // [2325]
    "CompoundcurveKeywordSegment", // [2326]
    "CurvepolygonKeywordSegment", // [2327]
    "MulticurveKeywordSegment", // [2328]
    "MultisurfaceKeywordSegment", // [2329]
    "PolyhedralsurfaceKeywordSegment", // [2330]
    "TriangleKeywordSegment", // [2331]
    "TinKeywordSegment", // [2332]
    "SimpleGeometryGrammar", // [2333]
    "GeometryKeywordSegment", // [2334]
    "GeographyKeywordSegment", // [2335]
    "indented_then", // [2336]
    "indented_then_contents", // [2337]
    "WHILE", // [2338]
    "WHITESPACE", // [2339]
    "whitespace", // [2340]
    "WhitespaceSegment", // [2341]
    "WildcardIdentifierSegment", // [2342]
    "PartitionClauseSegment", // [2343]
    "partitionby_clause", // [2344]
    "FrameClauseSegment", // [2345]
    "frame_clause", // [2346]
    "WIRE", // [2347]
    "CascadedKeywordSegment", // [2348]
    "RecursiveKeywordSegment", // [2349]
    "indented_ctes", // [2350]
    "CTEDefinitionSegment", // [2351]
    "common_table_expression", // [2352]
    "NonWithNonSelectableGrammar", // [2353]
    "BindingKeywordSegment", // [2354]
    "WithinKeywordSegment", // [2355]
    "WORKERS", // [2356]
    "WRITE", // [2357]
    "WRITETEXT", // [2358]
    "X509", // [2359]
    "XMLATTRIBUTES", // [2360]
    "XMLCONCAT", // [2361]
    "XMLELEMENT", // [2362]
    "XMLEXISTS", // [2363]
    "XMLFOREST", // [2364]
    "XMLNAMESPACES", // [2365]
    "XMLPARSE", // [2366]
    "XMLPI", // [2367]
    "XMLROOT", // [2368]
    "XMLSERIALIZE", // [2369]
    "XMLTABLE", // [2370]
    "XOR", // [2371]
    "YEAR_MONTH", // [2372]
    "YEARS", // [2373]
    "YES", // [2374]
    "ZEROFILL", // [2375]
    "ZONES", // [2376]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1,
    0, 0, 4294967295, 4294967295, 0, 128, 1, 2, 1, 1, 4, 3, 1, 4, 5, 129,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 130, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 131, 1, 2, 1, 1, 4, 3, 1, 4, 5, 132, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 137, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 138, 1, 2, 1, 1, 4, 3, 1, 4, 5, 139, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 150, 153, 154, 1, 153, 3, 3, 4, 155, 156, 157,
    154, 1, 157, 3, 3, 4, 155, 1, 2, 1, 1, 0, 1, 2, 1, 0,
    163, 1, 2, 1, 1, 4, 3, 1, 4, 5, 28, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 168, 1, 2, 1, 1, 4, 3, 1, 4, 5, 169, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 0, 1, 4294967295,
    4294967295, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
    2, 1, 1, 2, 1, 0, 1, 0, 0, 1, 0, 1, 2, 1, 0, 0,
    0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 1,
    2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 1, 0, 1, 2, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 1, 0, 0, 0, 0, 170, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 0, 1,
    0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 0, 1, 0, 0, 1, 0,
    0, 0, 0, 1, 0, 1, 2, 1, 1, 2, 1, 1, 4294967295, 1, 0, 0,
    0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0,
    1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 4294967295, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 2, 3, 1,
    0, 0, 0, 0, 0, 1, 0, 3, 4, 1, 0, 0, 0, 0, 1, 0,
    0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    1, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0,
    0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0,
    0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 1, 0,
    1, 2, 1, 0, 403, 1, 2, 1, 1, 4, 3, 1, 4, 5, 660, 661,
    154, 1, 661, 3, 3, 4, 155, 662, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 663, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 1,
    2, 1, 0, 1, 0, 1, 2, 1, 1, 0, 670, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 670, 671, 672, 1, 671, 3, 3, 671, 4, 673, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 674, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 689, 1, 2, 1, 1, 4, 3, 1, 4, 5, 690, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 123, 693, 154, 1, 693, 3, 3,
    4, 155, 694, 695, 154, 1, 695, 3, 3, 4, 155, 1, 2, 0, 231, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 1, 2, 0,
    147, 1, 2, 1, 1, 4, 3, 1, 4, 5, 698, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 699, 1, 2, 1, 1, 4, 3, 1, 4, 5, 700, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 701, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 702, 1, 2, 1, 1, 4, 3, 1, 4, 5, 703, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 634, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    656, 1, 2, 1, 1, 4, 3, 1, 4, 5, 704, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 705, 1, 2, 1, 1, 4, 3, 1, 4, 5, 706, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 707, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 708, 1, 2, 1, 1, 4, 3, 1, 4, 5, 710, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 711, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    712, 1, 2, 1, 1, 4, 3, 1, 4, 5, 713, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 714, 1, 2, 1, 1, 4, 3, 1, 4, 5, 715, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 392, 393, 187, 394, 395, 396, 189, 397,
    398, 191, 399, 282, 888, 12, 716, 717, 1, 716, 2, 3, 4, 0, 0, 722,
    723, 0, 2, 3, 722, 3, 724, 0, 1, 3, 658, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 725, 1, 2, 1, 1, 4, 3, 1, 4, 5, 726, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 727, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 728, 1, 2, 1, 1, 4, 3, 1, 4, 5, 729, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 730, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 671, 672, 9, 3, 3, 671, 4, 744, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 745, 1, 2, 1, 1, 4, 3, 1, 4, 5, 746, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 679, 671, 154, 1, 671, 3, 3, 4, 155, 750,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 751, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 754, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 757,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 758, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 759, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1,
    2, 1, 1, 0, 1, 2, 1, 760, 761, 0, 2, 3, 760, 1, 2, 1,
    764, 1, 2, 1, 1, 4, 3, 1, 4, 5, 765, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 766, 1, 2, 1, 1, 4, 3, 1, 4, 5, 767, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 768, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 769, 1, 2, 1, 1, 4, 3, 1, 4, 5, 770, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 771, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    404, 1, 2, 1, 1, 4, 3, 1, 4, 5, 543, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 772, 1, 2, 1, 1, 4, 3, 1, 4, 5, 773, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 0, 0,
    409, 1, 2, 1, 1, 4, 3, 1, 4, 5, 780, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 483, 1, 2, 1, 1, 4, 3, 1, 4, 5, 279, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 782, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0, 783, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 318, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    797, 798, 154, 1, 798, 3, 3, 4, 155, 799, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 800, 1, 2, 1, 1, 4, 3, 1, 4, 5, 801, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 802, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 803, 1, 2, 1, 1, 4, 3, 1, 4, 5, 804, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 805, 1, 2, 1, 1, 4, 3, 1, 4, 5, 308,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 806, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 807, 1, 2, 1, 1, 4, 3, 1, 4, 5, 808, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 809, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 810, 1, 2, 1, 1, 4, 3, 1, 4, 5, 811, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 357, 1, 2, 1, 1, 4, 3, 1, 4, 5, 812,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 813, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 8, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 815, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 30, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 816, 1, 2, 1, 1, 4, 3, 1, 4, 5, 4, 717, 9, 2,
    3, 4, 379, 1, 2, 1, 1, 4, 3, 1, 4, 5, 319, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 0, 230, 819, 154, 1, 819, 3, 3,
    4, 155, 230, 820, 154, 1, 820, 3, 3, 4, 155, 230, 821, 154, 1, 821,
    3, 3, 4, 155, 0, 1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2,
    1, 0, 0, 0, 0, 4294967295, 4294967295, 0, 375, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 1, 0, 845, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    25, 846, 154, 1, 846, 3, 3, 4, 155, 847, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0, 850, 851,
    10, 3, 3, 850, 4, 852, 1, 2, 1, 1, 4, 3, 1, 4, 5, 853,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 854, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 855, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 225, 871,
    9, 3, 3, 225, 4, 872, 1, 2, 1, 1, 4, 3, 1, 4, 5, 671,
    873, 0, 2, 3, 671, 225, 874, 0, 2, 3, 225, 1, 2, 1, 875, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 876, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 432, 1, 2, 1, 1, 4, 3, 1, 4, 5, 877, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 346, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 0, 1,
    2, 1, 1, 2, 1, 0, 1, 0, 0, 888, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 891, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 892, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 9, 1, 2, 1, 1, 4, 3, 1, 4, 5, 31, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 306, 1, 2, 1, 1, 4, 3, 1, 4, 5, 893,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 894, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 895, 1, 2, 1, 1, 4, 3, 1, 4, 5, 896, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 320, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 2, 1, 1, 2, 1, 897, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 1, 2, 1, 0, 0, 0, 1, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 4294967295, 1, 0, 1, 0, 1,
    2, 1, 0, 1, 0, 0, 0, 4294967295, 1, 0, 1, 2, 1, 0, 0, 1,
    2, 1, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 4294967295, 1, 0, 0,
    1, 0, 0, 1, 2, 1, 1, 2, 1, 932, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 484, 1, 2, 1, 1, 4, 3, 1, 4, 5, 933, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 934, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 935, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 3, 4,
    1, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 1, 0, 1, 2, 1,
    0, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    0, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1, 1,
    0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1, 0, 0,
    0, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 46, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1, 0, 1,
    2, 1, 0, 1, 0, 1, 2, 0, 0, 1, 0, 1, 2, 1, 0, 4294967295,
    1003, 154, 1, 1003, 3, 3, 4, 155, 0, 4294967295, 4294967295, 0, 0, 0, 4294967295, 1003,
    154, 1, 1003, 3, 3, 4, 155, 0, 4294967295, 4294967295, 0, 0, 4294967295, 1009, 154, 1,
    1009, 3, 3, 4, 155, 0, 4294967295, 4294967295, 0, 0, 0, 0, 1, 0, 1, 2,
    1, 0, 0, 0, 1, 0, 1, 2, 1, 1, 2, 1, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 4294967295, 1, 0, 0, 0, 0, 0,
    0, 0, 0, 4294967295, 4294967295, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2,
    1, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 1, 0, 0, 1, 0,
    1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 1, 0,
    1, 2, 1, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 2, 3, 1, 0, 1, 0,
    1, 2, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 2, 1, 0,
    1, 0, 1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 1,
    2, 1, 0, 1, 2, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1,
    0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 1, 0,
    1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0,
    0, 0, 4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 0,
    0, 4294967295, 4294967295, 0, 0, 1, 2, 1, 0, 1, 0, 0, 1, 0, 0, 0,
    0, 0, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1,
    0, 1, 0, 1, 2, 1, 0, 0, 0, 1, 0, 1, 2, 1, 47, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 48, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 49, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1102, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1103, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    923, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1104, 1105, 717, 1, 1105, 2,
    3, 4, 1104, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1,
    254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 393, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 187, 1, 2, 1, 1, 4, 3, 1, 4, 5, 395, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 396, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 189, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1111, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1112, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    410, 1, 2, 1, 1, 4, 3, 1, 4, 5, 337, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 10, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    32, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1113, 717, 1, 1113,
    2, 3, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 4294967295, 4294967295, 0, 0, 232, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2751,
    0, 1105, 717, 1, 1105, 2, 3, 4, 0, 1, 0, 1, 2, 1, 0, 1,
    2, 1, 0, 1157, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1294, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1264, 1265, 1266, 1267, 1268, 1269, 1270, 1271, 1272,
    1273, 1274, 1275, 1276, 1277, 1278, 1279, 1280, 1281, 1282, 1283, 1284, 1285, 1286, 1287, 2791,
    24, 1295, 717, 1, 1295, 2, 3, 4, 1296, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1265, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1297, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1298, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1299, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1300, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1301, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1302, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1266, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1303, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1304, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1305, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1028, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1309, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1310, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1311,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1158, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1312, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1317, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 227, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1318, 1, 2, 1, 1, 4, 3, 1, 4, 5, 437, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 442, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    503, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1,
    1323, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1324, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 50, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    0, 1, 0, 0, 1333, 1, 2, 1, 1, 4, 3, 1, 4, 5, 905, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1335, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1336, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1052, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 517, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    777, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1337, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1338, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1340, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1341, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 635, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1342, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 944, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    347, 1, 2, 1, 1, 4, 3, 1, 4, 5, 366, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1343, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1344, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1348, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1349, 1, 2, 1, 1, 4, 3, 1, 4, 5, 830, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1350, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1351, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1352, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 676, 671, 154, 1, 671, 3, 3, 4, 155, 884, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1355, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 242, 242, 1357, 3, 242, 1356, 242, 3, 3, 1356, 4, 296, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1358, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 122, 1359, 154, 1, 1359, 3, 3, 4, 155, 1159, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 0, 3, 4, 1, 1, 2, 1, 1,
    0, 1, 0, 1, 0, 1, 0, 297, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1,
    0, 0, 0, 1, 0, 1, 0, 1, 0, 1360, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1361, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1092, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1362, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1363, 1, 2, 1, 1, 4, 3, 1, 4, 5, 791, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1364, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1365,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 1, 367, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 522, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1368, 1, 2, 1, 1, 4, 3, 1, 4, 5, 906, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 544, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1369, 1370,
    154, 1, 1370, 3, 3, 4, 155, 793, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1371, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1372, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1373, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1026,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1374, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 907, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1375, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 321, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1376, 1, 2, 1, 1, 4, 3, 1, 4, 5, 434, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1072, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1379,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 258, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 1, 2, 1, 1380, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1381, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1382, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1383, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1384, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0,
    1, 2, 1, 0, 0, 401, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2, 1, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 0, 1, 0, 0, 1, 2, 1, 0, 1, 0,
    1, 2, 1, 0, 0, 0, 425, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 0, 0, 314, 1, 2, 1, 1, 4, 3, 1, 4, 5, 70,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1463, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1464, 1, 2, 1, 1, 4, 3, 1, 4, 5, 211, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 211, 1465, 1466, 1, 1465, 5, 3, 1, 1356,
    4, 5, 358, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0,
    1467, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1,
    1475, 1, 2, 1, 1, 4, 3, 1, 4, 5, 71, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1491, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1492, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 646, 1, 2, 1, 1, 4, 3, 1, 4, 5, 628,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1161, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1162, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1160, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1494, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1495, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 275,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 368, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 908, 1, 2, 1, 1, 4, 3, 1, 4, 5, 909, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 910, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 322, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0,
    1, 2, 1, 97, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1473, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 0, 1511, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1512, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 911, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2,
    1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 1, 2, 1, 0, 143,
    1548, 1, 0, 0, 4294967295, 4294967295, 0, 161, 1548, 1, 1, 2, 1, 0, 223, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 468, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1445, 1445, 1357, 3, 1445, 1356, 1445, 3, 3, 1356, 4, 1549, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1550, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 1, 2, 1, 0, 0, 0, 4294967295, 1, 0, 1, 0, 0, 0,
    0, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 0, 0, 0, 0,
    0, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 1, 0,
    1, 2, 1, 72, 1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4294967295, 1105,
    717, 1, 1105, 2, 3, 4, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0,
    1, 0, 1, 2, 1, 0, 0, 98, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 383, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1037, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1115,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1116, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1117, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1118, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1119, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1585, 1585, 871, 3, 1585,
    225, 1585, 3, 3, 225, 4, 1059, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    116, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1590, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1591, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1592, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1593, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 1, 0, 1, 0, 0, 292, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 1, 0, 1, 0, 1597, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 0, 1, 2, 1, 1500, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1603, 1, 2, 1, 1, 4, 3, 1, 4, 5, 978, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1076, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 2, 1, 0, 0, 1515, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    912, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1051, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1604, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1605, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1316, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1606, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1607, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1608, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1271, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1609, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1610, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1611, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1612, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 954, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1613, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1614, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1615, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1616, 1617, 9, 3, 3, 1616,
    4, 408, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1618, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1619, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1620,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 198, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1563, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1409, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 444, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 485, 1, 2, 1, 1, 4, 3, 1, 4, 5, 941, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1624, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1629, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    42, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2, 1,
    0, 1032, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1073, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 411, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 0, 0, 1, 2, 1, 0, 471, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1638, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1639, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1163, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1640,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1641, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1642, 1, 2, 1, 1, 4, 3, 1, 4, 5, 369, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 985, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1643, 1, 2, 1, 1, 4, 3, 1, 4, 5, 439, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1644, 1, 2, 1, 1, 4, 3, 1, 4, 5, 879,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 938, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 496, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1645, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 51, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 1, 0, 0, 1654, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    600, 601, 602, 603, 604, 605, 606, 607, 608, 609, 610, 611, 5216, 12, 1655, 2,
    1, 1655, 4, 3, 1, 4, 5, 588, 589, 590, 591, 592, 593, 594, 595, 596,
    597, 598, 599, 5239, 12, 1656, 2, 1, 1656, 4, 3, 1, 4, 5, 0, 1082,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1657, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1165, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1658, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1166, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1167, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1168, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1169, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1164,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1170, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1659, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1660, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1516, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 233, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1025, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1667, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 504, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 732,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1673, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 205, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1674, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1422, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1675, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1676, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 143, 1680, 1, 0, 161, 1680, 1,
    0, 1010, 1, 2, 1, 1, 4, 3, 1, 4, 5, 785, 1688, 1, 1, 2,
    1, 0, 161, 1688, 1, 0, 0, 1, 0, 1, 2, 1, 1047, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 734, 671, 154, 3, 671, 155, 734, 3, 3, 4,
    155, 0, 0, 1171, 1, 2, 1, 1, 4, 3, 1, 4, 5, 960, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 833, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 202, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 9, 4, 3,
    1, 4, 5, 1700, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1701, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1702, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1703, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 323, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 607, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1633, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1705, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1535, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1706, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 945, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 946, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1554, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 486, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1707, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1708, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 880, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1005, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1709, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 455, 1, 2, 1, 1, 4, 3, 1, 4, 5, 955, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1070, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 735, 735, 871, 3, 735, 225, 735, 3, 3, 225, 4, 0, 0, 0, 1,
    2, 1, 0, 0, 0, 214, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1172,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1719, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1720, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1121, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1122, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1123, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1124, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1077, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1721,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1356, 1466, 9, 5, 3, 1,
    1356, 4, 5, 1356, 1357, 9, 3, 3, 1356, 4, 1035, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1060, 1, 2, 1, 1, 4, 3, 1, 4, 5, 947, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 397, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 398, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1730, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1731, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 0, 0, 1499, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1736, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1737, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 462, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1738, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 545, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1739, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1740, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1741, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1742, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1743, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1744, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1173, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 2, 1, 2, 3, 1, 1175, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1174, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    426, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1746, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1747, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1096, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 648, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1038, 1, 2, 1, 1, 4, 3, 1, 4, 5, 73, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1748, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1749, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1750, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 1, 0, 1, 2, 1, 0, 0, 1760, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 74, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1761, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1762, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1763, 1, 2, 1, 1, 4, 3, 1, 4, 5, 412,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1053, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1764, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1765, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1766, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1767, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1476, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 0, 0, 0, 0, 1, 2, 1,
    0, 143, 1680, 1, 161, 1680, 1, 1792, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1793, 1794, 154, 3, 1794, 155, 1793, 3, 3, 4, 155, 0, 4294967295, 4294967295, 0,
    1796, 1, 2, 1, 1, 4, 3, 1, 4, 5, 331, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1797, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1276, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1798, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1517, 1, 2, 1, 1, 4, 3, 1, 4, 5, 133, 671, 154, 1,
    671, 3, 3, 4, 155, 1278, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1799,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1800, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1801, 1, 2, 1, 1, 4, 3, 1, 4, 5, 413, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1544, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1802, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1735, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1002, 1, 2, 1, 1, 4, 3, 1, 4, 5, 624,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 675, 671, 154, 1, 671, 3, 3,
    4, 155, 1803, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1176, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1804, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1279, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1805, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1806, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1807, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1808, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1125, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0,
    245, 245, 1810, 1, 245, 3, 3, 245, 4, 1126, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1127, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1128, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1129, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 176, 671, 154, 1, 671, 3, 3, 4, 155, 1130, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1131, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1132, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1133, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1134, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1135, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1136, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1137, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1138, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1811, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 1812,
    1617, 3, 1812, 1616, 5, 3, 3, 1616, 4, 4, 5, 1813, 1617, 1, 1813, 3,
    3, 1616, 4, 577, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2,
    1, 0, 1, 0, 1820, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1414, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1414, 1821, 1466, 1, 1821, 5, 3, 1,
    1356, 4, 5, 1822, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1823, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1825, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1054, 1, 2, 1, 1, 4, 3, 1, 4, 5, 133, 1826, 154, 1, 1826,
    3, 3, 4, 155, 1087, 1, 2, 1, 1, 4, 3, 1, 4, 5, 245, 1810,
    12, 3, 3, 245, 4, 1469, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1415,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1416, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1417, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1418, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 370, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1827, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1828, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 546, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1829,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1830, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1831, 1, 2, 1, 1, 4, 3, 1, 4, 5, 547, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 548, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1832, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1833, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 549, 1, 2, 1, 1, 4, 3, 1, 4, 5, 550,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 1,
    2, 1, 0, 0, 1, 2, 1, 0, 0, 1837, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1838, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 551, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1839, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1419, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 552, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 303, 1, 2, 1, 1, 4, 3, 1, 4, 5, 303, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 469, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1847,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1848, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1849, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1424, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 527, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 234, 1, 2, 1, 1, 4, 3, 1, 4, 5, 234, 1821, 1466, 1, 1821,
    5, 3, 1, 1356, 4, 5, 1851, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    827, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1177, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 216, 216, 1357, 3, 216, 1356, 216, 3, 3, 1356, 4, 1178,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1695, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 240, 1854, 154, 1, 1854, 3, 3, 4, 155, 1, 2,
    0, 0, 0, 1855, 1, 2, 1, 1, 4, 3, 1, 4, 5, 371, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1856, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1857, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1518, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1858, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    460, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1088, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 69, 1813, 1617, 1, 1813, 3, 3, 1616, 4, 69, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1859, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 479, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1860, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1861, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1862,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1863, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1864, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1865, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 324, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1866, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1751, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1867, 1, 2, 1, 1, 4, 3, 1, 4, 5, 268,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1868, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 4294967295, 4294967295, 0, 1, 2, 1, 372, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 0, 733, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 733, 671, 672, 1, 671, 3, 3, 671, 4, 0, 0, 0, 1, 0, 0,
    141, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1873, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1509, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1574,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1693, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1874, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    0, 1876, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 3, 4, 1,
    0, 0, 0, 3, 4, 1, 0, 1878, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1879, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1647, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1880, 1, 2, 1, 1, 4, 3, 1, 4, 5, 414,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 177, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1881, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1882, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 487, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 6, 4294967295, 1883, 717, 1, 1883, 2, 3, 4, 1884, 1883, 154, 1, 1883, 3,
    3, 4, 155, 1885, 1, 2, 1, 1, 4, 3, 1, 4, 5, 348, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1886, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1,
    2, 1, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 638, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1890, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    553, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1179, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 5, 1892, 1893, 2, 1892, 5, 3, 3, 4, 5, 1, 0,
    0, 1894, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1014, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 736, 671, 154, 3, 671, 155, 736, 3, 3, 4, 155,
    1180, 1, 2, 1, 1, 4, 3, 1, 4, 5, 737, 671, 154, 3, 671, 155,
    737, 3, 3, 4, 155, 1752, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1895,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 136, 1896, 154, 1, 1896, 3, 3,
    4, 155, 1897, 1, 2, 1, 1, 4, 3, 1, 4, 5, 427, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1753, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1346, 1, 2, 1, 1, 4, 3, 1, 4, 5, 229, 671, 154, 1, 671, 3,
    3, 4, 155, 1139, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1140, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1141, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1142, 1, 2, 1, 1, 4, 3, 1, 4, 5, 92, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1143, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1144,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1145, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1146, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1147, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1898, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1899, 1, 2, 1, 1, 4, 3, 1, 4, 5, 229, 1826, 154, 1, 1826,
    3, 3, 4, 155, 0, 4294967295, 4294967295, 0, 1906, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 738, 671, 154, 3, 671, 155, 738, 3, 3, 4, 155, 1480, 1484, 717,
    3, 1484, 4, 1480, 2, 3, 4, 961, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1504, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1208, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1909, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1910, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1308, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 1913, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1064, 1, 2, 1, 1, 4, 3, 1, 4, 5, 384, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1914, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1430, 1, 2, 1, 1, 4, 3, 1, 4, 5, 965, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 65, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1915, 1, 2, 1, 1, 4, 3, 1, 4, 5, 325, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 75, 1, 2, 1, 1, 4, 3, 1, 4, 5, 99, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1916, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1917, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1918, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 901, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1919, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 1920, 717, 3, 1920, 4,
    5, 2, 3, 4, 1921, 1, 2, 1, 1, 4, 3, 1, 4, 5, 948, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1481, 1481, 717, 3, 1481, 4, 1481, 2,
    3, 4, 1482, 1482, 717, 3, 1482, 4, 1482, 2, 3, 4, 230, 1924, 154, 1,
    1924, 3, 3, 4, 155, 0, 1, 2, 1, 0, 294, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 536, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    0, 0, 1, 0, 0, 0, 1, 2, 1, 1929, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 7, 4294967295, 1930, 154, 1, 1930, 3, 3, 4, 155, 1, 4294967295, 4294967295,
    0, 1, 2, 1, 1519, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    2, 1, 1280, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1934, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 913, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    250, 1935, 1617, 3, 1935, 1616, 250, 3, 3, 1616, 4, 251, 1935, 1617, 3, 1935,
    1616, 251, 3, 3, 1616, 4, 0, 246, 1936, 1357, 3, 1936, 1356, 246, 3, 3,
    1356, 4, 244, 1936, 1357, 3, 1936, 1356, 244, 3, 3, 1356, 4, 0, 246, 1936,
    1357, 3, 1936, 1356, 246, 3, 3, 1356, 4, 0, 4294967295, 4294967295, 0, 241, 1936, 1357,
    3, 1936, 1356, 241, 3, 3, 1356, 4, 8, 4294967295, 1936, 1357, 1, 1936, 3, 3,
    1356, 4, 0, 4294967295, 4294967295, 0, 247, 1936, 1357, 3, 1936, 1356, 247, 3, 3, 1356,
    4, 9, 4294967295, 1936, 1357, 1, 1936, 3, 3, 1356, 4, 0, 4294967295, 4294967295, 0, 10,
    4294967295, 1940, 717, 1, 1940, 2, 3, 4, 243, 1936, 1357, 3, 1936, 1356, 243, 3,
    3, 1356, 4, 243, 1936, 1357, 3, 1936, 1356, 243, 3, 3, 1356, 4, 0, 4294967295,
    4294967295, 0, 0, 1, 0, 0, 1941, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1942, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1943, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1078, 1, 2, 1, 1, 4, 3, 1, 4, 5, 222, 1944,
    154, 1, 1944, 3, 3, 4, 155, 678, 1944, 154, 1, 1944, 3, 3, 4, 155,
    1754, 1, 2, 1, 1, 4, 3, 1, 4, 5, 677, 1944, 154, 1, 1944, 3,
    3, 4, 155, 731, 1944, 154, 1, 1944, 3, 3, 4, 155, 1945, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1946, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1181, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1947, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 0, 0, 1949, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1950, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1951, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1952, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 4294967295, 1, 0, 0, 1959, 1, 2, 1, 1, 4, 3, 1, 4, 5, 259,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1085, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 0, 1, 2, 1, 0, 618, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1961, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1962, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1963, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 968, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1964, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 0, 0, 1, 0,
    1, 2, 1, 1968, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1969, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1970, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1889, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1971, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 178, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1972,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1973, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 650, 1, 2, 1, 1, 4, 3, 1, 4, 5, 373, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 942, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 554, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1974, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 206, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1977, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1978, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1564, 1, 2, 1, 1, 4, 3, 1, 4, 5, 388, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 277, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 508, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1015, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1979, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1980, 1, 2, 1, 1, 4, 3, 1, 4, 5, 997, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1331, 1, 2, 1, 1, 4, 3, 1, 4, 5, 488, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 117, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1814, 1982, 154, 1, 1982, 3, 3, 4, 155, 881, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1983, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    281, 1, 2, 1, 1, 4, 3, 1, 4, 5, 288, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1477, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1106, 1105,
    717, 1, 1105, 2, 3, 4, 1106, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    200, 1, 2, 1, 1, 4, 3, 1, 4, 5, 76, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 100, 1, 2, 1, 1, 4, 3, 1, 4, 5, 453, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1984, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1985, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1986, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1987, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    489, 1, 2, 1, 1, 4, 3, 1, 4, 5, 449, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1988, 1, 2, 1, 1, 4, 3, 1, 4, 5, 509, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 1, 2, 1,
    1992, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1993, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1994, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1696, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1995, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1996, 1, 2, 1, 1, 4, 3, 1, 4, 5, 11, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 33, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1997, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1998, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 345, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1281, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1999, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2000, 1, 2, 1, 1, 4, 3, 1, 4, 5, 12, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 34, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1,
    0, 1, 2, 1, 0, 1, 2, 1, 0, 0, 0, 2003, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 0, 1, 0, 2, 9891,
    2, 9893, 3, 732, 830, 732, 303, 830, 0, 52, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 0,
    4294967295, 4294967295, 0, 842, 2030, 154, 1, 2030, 3, 3, 4, 155, 1560, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 77, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 101, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1183, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1184, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1185, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1182, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2031, 1, 2, 1, 1, 4, 3, 1, 4, 5, 326, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2032, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2033, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2034, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1976, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 191, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 0, 4294967295,
    4294967295, 0, 1, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 1, 0, 0, 1,
    2, 1, 0, 1, 0, 0, 0, 1, 4294967295, 4294967295, 0, 179, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0,
    0, 0, 994, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1602, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1385, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2043, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1496, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2044, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    4294967295, 4294967295, 0, 0, 0, 1478, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 2052, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2053, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1410, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 4294967295, 4294967295, 0, 1956, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1, 0, 246, 1935, 1617, 3, 1935, 1616, 246, 3, 3, 1616, 4, 529, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2046, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 586, 1, 2, 1, 1, 4, 3, 1, 4, 5, 619, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 667, 1, 2, 1, 1, 4, 3, 1, 4, 5, 676,
    2056, 154, 1, 2056, 3, 3, 4, 155, 230, 2057, 154, 1, 2057, 3, 3, 4,
    155, 1186, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1187, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2058, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1931,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2059, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 530, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2047, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2060, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2061, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2062, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2063, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2064,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2065, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2066, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2067, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2068, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2069, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2070, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2071, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2072,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2073, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 962, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2074, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 490, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 78, 1, 2, 1, 1, 4, 3, 1, 4, 5, 102, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2075, 1, 2, 1, 1, 4, 3, 1, 4, 5, 176,
    2076, 154, 1, 2076, 3, 3, 4, 155, 415, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2077, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2078, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1093, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 420, 1, 2, 1, 1, 4, 3, 1, 4, 5, 900, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 926, 1, 2, 1, 1, 4, 3, 1, 4, 5, 424,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 825, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2203, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2204, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 79, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 103, 1, 2, 1, 1, 4, 3, 1, 4, 5, 491, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 2206, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2207, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 2208,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 928, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 615, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2209, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2210, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2211, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1386, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 555, 1, 2, 1, 1, 4, 3, 1, 4, 5, 492,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 155, 154, 9, 3, 3, 4, 155,
    2212, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2213, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2214, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1018, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 27, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 2, 1, 1, 0, 1, 2, 1, 1, 2, 1, 0, 0, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 0, 1, 2, 1, 1, 2, 1, 0, 13, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 35, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1540, 1, 2, 1, 1, 4, 3, 1, 4, 5, 219, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2221, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 1783, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 80, 1, 2, 1, 1, 4, 3, 1, 4, 5, 104, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 990, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    349, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 991, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 2227, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2228, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2229, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 327, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2230, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2231, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1777, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2232, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1472, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1431, 2233, 154, 1, 2233, 3, 3, 4, 155, 235, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 2236, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2237, 1, 2, 1, 1, 4, 3, 1, 4, 5, 236, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 237, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 238, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1387, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1148, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2238,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2239, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2240, 1, 2, 1, 1, 4, 3, 1, 4, 5, 182, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2241, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2242, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2243, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2244, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1039,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2245, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1555, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2246, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2247, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 2253, 1, 2, 1, 1, 4, 3, 1, 4, 5, 328, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2225, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1149, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 260, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2255,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 212, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 212, 1465, 1466, 1, 1465, 5, 3, 1, 1356, 4, 5, 261,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 2260, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2261, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1188, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1189, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1190, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1191,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 973, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 1, 2, 1, 14, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 36, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1938, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2263, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1503, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2264, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 2269, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2270,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1520, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 385, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1420, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1420, 1821, 1466, 1, 1821, 5, 3, 1,
    1356, 4, 5, 2272, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2274, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 463, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 4294967295, 4294967295, 0, 510, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2275, 1, 2, 1, 1, 4, 3, 1, 4, 5, 570, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 53, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    0, 0, 2276, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1029, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2277, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    54, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1479, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2279, 1, 2, 1, 1, 4, 3, 1, 4, 5, 282, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2280, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2281, 1, 2, 1, 1, 4, 3, 1, 4, 5, 362, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2282, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2283, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2284, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1192, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2285, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 1, 0, 1, 2, 1,
    0, 1, 0, 311, 1, 2, 1, 1, 4, 3, 1, 4, 5, 298, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 980, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 654, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2,
    1, 1, 0, 640, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2290, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1193, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2291, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2292, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2293, 1, 2, 1, 1, 4, 3, 1, 4, 5, 239,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1224, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1194, 1, 2, 1, 1, 4, 3, 1, 4, 5, 666, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 949, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 81, 1, 2, 1, 1, 4, 3, 1, 4, 5, 105, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 493, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2294, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1388, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1437, 2295, 154, 1, 2295, 3, 3, 4, 155, 2296, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2297, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2298, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1045, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1285, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1286,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 0,
    1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1,
    0, 0, 1, 2, 1, 0, 143, 2336, 1, 785, 2337, 1, 161, 2337, 1, 161,
    2336, 1, 787, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0,
    0, 886, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2338, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2339, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2340,
    2341, 12, 3, 3, 4, 2340, 0, 0, 4294967295, 4294967295, 0, 998, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 2347, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 143, 2350, 1, 1, 0, 161, 2350, 1, 143, 2350, 1, 1, 0, 161, 2350,
    1, 207, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1901, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 459, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 5, 1893, 9, 3, 3, 4, 5, 2249, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2356, 1, 2, 1, 1, 4, 3, 1, 4, 5, 339, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2357, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2358, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2359, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1195, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2360,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2361, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2362, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2363, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2364, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2365, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2366, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2367, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2368,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2369, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2370, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2371, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1395, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1287, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2372, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2373, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2374,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2375, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2042, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2376, 1, 2,
    1, 1, 4, 3, 1, 4, 5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 40, 0, 30, 15, 16, 17, 18, 19, 20, 21, 31, 22, 24,
    26, 0, 29, 39, 0, 33, 37, 38, 39, 40, 41, 20, 34, 22, 0, 36,
    37, 38, 39, 40, 43, 41, 19, 37, 22, 44, 43, 45, 41, 55, 56, 57,
    58, 59, 60, 61, 62, 63, 0, 29, 64, 42, 66, 68, 44, 15, 82, 83,
    84, 85, 86, 87, 88, 89, 20, 0, 90, 91, 0, 93, 82, 0, 94, 15,
    0, 95, 96, 45, 37, 106, 107, 108, 109, 110, 111, 112, 113, 43, 0, 90,
    114, 0, 93, 106, 0, 94, 37, 0, 95, 115, 46, 118, 120, 47, 22, 49,
    124, 126, 53, 63, 73, 83, 93, 0, 134, 135, 134, 103, 113, 123, 140, 0,
    143, 146, 159, 0, 149, 151, 151, 133, 142, 0, 149, 156, 154, 0, 158, 159,
    161, 160, 0, 164, 166, 170, 180, 190, 0, 171, 172, 22, 201, 200, 173, 0,
    175, 205, 0, 180, 181, 183, 0, 185, 181, 204, 186, 188, 190, 192, 0, 194,
    20, 195, 0, 171, 18, 197, 22, 199, 201, 0, 171, 19, 203, 215, 0, 208,
    206, 0, 209, 210, 0, 18, 213, 215, 0, 217, 210, 0, 180, 181, 203, 0,
    185, 181, 210, 22, 186, 188, 190, 0, 194, 218, 220, 0, 194, 158, 213, 0,
    211, 181, 224, 212, 226, 228, 248, 249, 0, 252, 253, 0, 255, 214, 29, 158,
    0, 256, 257, 68, 263, 181, 216, 265, 208, 0, 208, 256, 267, 221, 0, 29,
    64, 219, 218, 59, 58, 269, 56, 270, 55, 271, 272, 57, 60, 68, 0, 273,
    0, 256, 267, 274, 257, 68, 263, 252, 222, 265, 276, 278, 280, 224, 37, 108,
    110, 111, 38, 43, 0, 0, 171, 226, 64, 274, 225, 0, 283, 192, 0, 284,
    285, 0, 29, 287, 0, 41, 226, 20, 19, 22, 256, 289, 68, 227, 37, 38,
    39, 40, 44, 43, 45, 181, 22, 228, 0, 291, 192, 293, 0, 171, 295, 22,
    233, 0, 194, 226, 299, 0, 301, 226, 0, 229, 194, 301, 302, 304, 0, 305,
    0, 307, 22, 230, 0, 302, 304, 0, 309, 299, 0, 302, 310, 0, 301, 307,
    197, 22, 231, 276, 278, 0, 180, 307, 22, 181, 22, 0, 312, 307, 22, 0,
    185, 181, 232, 22, 186, 188, 190, 0, 180, 181, 22, 0, 194, 20, 22, 0,
    171, 313, 315, 257, 0, 57, 0, 181, 228, 0, 194, 20, 234, 195, 317, 0,
    235, 305, 301, 256, 0, 237, 0, 329, 330, 332, 333, 295, 0, 334, 271, 0,
    335, 336, 338, 0, 335, 15, 0, 340, 341, 20, 87, 342, 0, 343, 344, 236,
    350, 351, 352, 353, 16, 22, 0, 238, 0, 93, 82, 15, 82, 164, 0, 172,
    22, 239, 0, 0, 0, 354, 242, 0, 22, 355, 22, 0, 245, 84, 85, 86,
    246, 0, 183, 173, 0, 356, 255, 0, 22, 251, 249, 248, 159, 24, 159, 0,
    254, 359, 360, 22, 361, 363, 0, 365, 274, 16, 158, 295, 0, 305, 374, 376,
    377, 159, 0, 380, 381, 258, 386, 0, 301, 374, 197, 377, 280, 0, 171, 374,
    377, 277, 0, 0, 194, 336, 16, 159, 0, 380, 381, 0, 361, 262, 299, 0,
    194, 226, 264, 263, 228, 389, 391, 299, 0, 301, 226, 0, 265, 194, 301, 302,
    304, 0, 301, 400, 197, 0, 305, 402, 266, 405, 0, 406, 226, 355, 407, 271,
    267, 416, 0, 275, 0, 194, 402, 274, 405, 0, 406, 226, 0, 194, 416, 0,
    418, 0, 208, 215, 0, 301, 407, 197, 0, 194, 419, 215, 0, 194, 421, 0,
    255, 421, 0, 194, 423, 276, 428, 94, 429, 430, 0, 194, 431, 158, 0, 305,
    433, 0, 305, 436, 0, 171, 307, 158, 278, 438, 0, 302, 438, 279, 0, 440,
    441, 0, 440, 443, 0, 312, 307, 158, 0, 301, 307, 197, 158, 280, 0, 280,
    445, 446, 271, 281, 158, 29, 284, 0, 445, 282, 447, 405, 271, 158, 0, 283,
    445, 446, 0, 445, 447, 0, 445, 448, 448, 158, 0, 284, 446, 445, 450, 0,
    451, 450, 452, 454, 456, 0, 21, 68, 158, 0, 194, 458, 285, 21, 461, 0,
    194, 218, 220, 0, 194, 286, 464, 465, 0, 194, 421, 0, 255, 421, 0, 451,
    466, 164, 0, 467, 158, 0, 302, 467, 0, 185, 181, 287, 158, 186, 188, 190,
    0, 447, 407, 288, 226, 0, 361, 470, 472, 474, 475, 0, 0, 171, 375, 377,
    476, 292, 290, 0, 289, 305, 194, 301, 149, 477, 0, 171, 335, 15, 0, 197,
    478, 164, 175, 298, 296, 480, 0, 180, 374, 377, 181, 377, 0, 180, 181, 164,
    0, 307, 299, 0, 494, 68, 304, 495, 0, 497, 304, 68, 304, 495, 498, 300,
    499, 500, 501, 0, 302, 502, 0, 94, 456, 301, 505, 506, 0, 507, 302, 511,
    512, 513, 0, 514, 215, 0, 515, 215, 0, 516, 158, 0, 194, 158, 305, 0,
    303, 181, 224, 304, 228, 248, 226, 0, 252, 253, 0, 255, 306, 29, 158, 276,
    0, 171, 84, 308, 0, 183, 173, 312, 518, 0, 180, 181, 183, 0, 194, 20,
    195, 0, 185, 181, 311, 310, 158, 249, 186, 188, 190, 0, 451, 520, 68, 313,
    315, 0, 171, 470, 22, 0, 194, 521, 313, 346, 0, 171, 374, 377, 334, 0,
    194, 419, 215, 0, 194, 325, 323, 0, 523, 224, 228, 0, 255, 330, 328, 523,
    0, 194, 423, 333, 428, 94, 429, 430, 0, 194, 431, 158, 0, 21, 68, 158,
    0, 194, 458, 21, 0, 194, 337, 335, 0, 523, 0, 224, 228, 0, 255, 342,
    340, 523, 0, 185, 181, 345, 22, 186, 188, 190, 0, 171, 93, 82, 352, 0,
    197, 164, 349, 347, 524, 0, 180, 0, 374, 377, 181, 377, 0, 180, 181, 164,
    0, 194, 20, 195, 0, 164, 451, 520, 68, 313, 315, 0, 29, 41, 218, 220,
    0, 526, 406, 350, 22, 194, 218, 220, 0, 528, 0, 171, 353, 18, 21, 0,
    21, 447, 470, 531, 532, 82, 0, 93, 82, 15, 17, 22, 0, 185, 181, 22,
    0, 171, 91, 22, 68, 164, 367, 0, 180, 181, 22, 363, 0, 181, 355, 354,
    192, 293, 186, 188, 190, 0, 361, 357, 299, 0, 208, 309, 360, 299, 0, 373,
    0, 94, 456, 368, 505, 506, 0, 194, 158, 371, 0, 369, 181, 224, 370, 228,
    248, 226, 0, 252, 253, 0, 255, 372, 29, 158, 276, 0, 171, 85, 374, 0,
    183, 173, 383, 533, 0, 180, 181, 183, 0, 194, 20, 195, 0, 194, 158, 380,
    0, 376, 181, 224, 378, 377, 158, 228, 0, 252, 253, 0, 185, 181, 382, 381,
    158, 249, 186, 188, 190, 0, 451, 520, 68, 313, 315, 0, 171, 535, 537, 391,
    0, 194, 539, 0, 305, 384, 541, 0, 194, 386, 541, 0, 301, 388, 541, 0,
    180, 181, 537, 0, 185, 181, 390, 186, 188, 190, 192, 0, 171, 392, 18, 0,
    21, 447, 470, 531, 532, 82, 0, 93, 82, 15, 17, 22, 0, 180, 181, 22,
    0, 171, 393, 283, 284, 416, 0, 394, 186, 188, 190, 192, 208, 403, 395, 556,
    557, 396, 62, 558, 397, 61, 559, 398, 466, 560, 399, 561, 562, 400, 563, 564,
    401, 565, 566, 0, 18, 213, 215, 0, 567, 568, 402, 477, 304, 0, 310, 569,
    477, 0, 192, 0, 180, 181, 192, 0, 407, 186, 188, 190, 29, 192, 0, 41,
    19, 203, 415, 0, 194, 158, 413, 0, 408, 181, 224, 412, 226, 410, 409, 228,
    248, 571, 0, 252, 253, 0, 255, 414, 158, 29, 0, 171, 20, 195, 417, 0,
    180, 181, 195, 0, 185, 181, 192, 0, 171, 17, 197, 22, 355, 0, 421, 0,
    355, 159, 0, 572, 406, 573, 215, 418, 0, 574, 573, 215, 0, 451, 574, 419,
    0, 575, 573, 215, 0, 451, 575, 0, 87, 576, 578, 0, 580, 208, 573, 215,
    0, 418, 208, 573, 215, 0, 581, 215, 0, 451, 582, 0, 526, 406, 420, 583,
    377, 0, 171, 87, 197, 578, 427, 422, 416, 0, 185, 181, 426, 158, 188, 190,
    0, 180, 181, 578, 0, 194, 20, 195, 0, 0, 171, 93, 82, 197, 22, 0,
    41, 21, 22, 0, 171, 428, 531, 532, 197, 22, 194, 429, 585, 587, 0, 171,
    419, 197, 612, 435, 0, 185, 181, 433, 432, 158, 249, 186, 188, 190, 0, 180,
    181, 612, 0, 194, 434, 0, 20, 195, 0, 419, 215, 0, 171, 614, 616, 443,
    0, 18, 477, 0, 436, 194, 305, 301, 535, 437, 537, 0, 208, 539, 0, 620,
    535, 0, 208, 539, 445, 446, 0, 194, 539, 0, 621, 439, 158, 622, 299, 0,
    185, 181, 442, 22, 186, 188, 623, 0, 180, 181, 616, 0, 171, 26, 450, 0,
    194, 158, 444, 181, 224, 448, 226, 446, 445, 228, 248, 249, 571, 0, 255, 449,
    29, 158, 482, 0, 305, 374, 376, 377, 159, 0, 380, 381, 451, 386, 0, 301,
    374, 197, 377, 280, 0, 171, 374, 377, 470, 0, 0, 194, 336, 16, 159, 0,
    380, 381, 0, 361, 455, 299, 0, 194, 226, 457, 456, 228, 389, 391, 299, 0,
    301, 226, 0, 458, 194, 301, 302, 304, 0, 301, 400, 197, 0, 305, 402, 459,
    405, 0, 406, 226, 355, 407, 464, 460, 416, 0, 468, 0, 194, 402, 467, 405,
    0, 406, 226, 0, 194, 416, 0, 418, 0, 208, 215, 0, 301, 407, 197, 0,
    194, 419, 215, 0, 194, 421, 0, 255, 421, 0, 194, 423, 469, 428, 94, 429,
    430, 0, 194, 431, 158, 0, 305, 433, 0, 305, 436, 0, 171, 307, 158, 471,
    438, 0, 302, 438, 472, 0, 440, 441, 0, 440, 443, 0, 312, 307, 158, 0,
    301, 307, 197, 158, 280, 0, 473, 445, 446, 271, 474, 158, 29, 284, 0, 445,
    475, 447, 405, 271, 158, 0, 476, 445, 446, 0, 445, 447, 0, 445, 448, 448,
    158, 0, 477, 446, 445, 450, 0, 451, 450, 452, 454, 456, 0, 21, 68, 158,
    0, 194, 458, 478, 21, 461, 0, 194, 218, 220, 0, 194, 479, 464, 465, 0,
    194, 421, 0, 255, 421, 0, 451, 466, 164, 0, 467, 158, 0, 302, 467, 0,
    185, 181, 480, 158, 186, 188, 190, 0, 447, 407, 481, 226, 0, 361, 470, 472,
    474, 475, 0, 301, 374, 197, 149, 488, 0, 158, 224, 483, 228, 248, 0, 484,
    305, 625, 374, 626, 486, 0, 485, 629, 630, 377, 631, 632, 0, 180, 487, 355,
    181, 164, 0, 171, 15, 496, 0, 197, 478, 164, 175, 491, 489, 633, 0, 180,
    374, 377, 181, 377, 0, 180, 307, 158, 181, 158, 0, 197, 164, 493, 0, 180,
    181, 164, 0, 194, 20, 195, 0, 636, 637, 158, 492, 0, 274, 639, 641, 226,
    0, 643, 637, 158, 644, 645, 0, 29, 41, 218, 220, 0, 526, 406, 494, 22,
    194, 218, 220, 528, 0, 171, 343, 344, 350, 22, 510, 0, 305, 647, 274, 497,
    22, 208, 499, 22, 0, 171, 647, 274, 501, 22, 208, 503, 22, 0, 171, 647,
    649, 22, 208, 22, 0, 171, 647, 274, 505, 22, 649, 22, 208, 22, 0, 301,
    647, 197, 274, 507, 22, 0, 180, 181, 22, 0, 185, 181, 509, 22, 186, 188,
    190, 0, 194, 20, 22, 0, 171, 271, 651, 68, 164, 511, 0, 180, 181, 651,
    0, 451, 520, 68, 313, 315, 0, 171, 16, 159, 517, 0, 185, 181, 512, 188,
    190, 186, 22, 0, 180, 653, 477, 181, 477, 0, 180, 181, 22, 0, 194, 20,
    195, 514, 513, 0, 305, 655, 377, 159, 0, 380, 381, 657, 0, 171, 655, 377,
    0, 194, 336, 16, 159, 0, 380, 381, 657, 0, 301, 655, 197, 377, 657, 0,
    180, 655, 377, 181, 377, 657, 0, 305, 653, 376, 477, 0, 516, 659, 630, 477,
    0, 171, 82, 197, 164, 531, 0, 171, 374, 377, 519, 0, 194, 226, 518, 228,
    389, 391, 299, 0, 301, 226, 0, 185, 181, 520, 22, 186, 188, 190, 0, 180,
    374, 377, 181, 377, 0, 180, 181, 164, 0, 194, 20, 195, 0, 194, 523, 521,
    0, 523, 0, 224, 228, 0, 255, 528, 526, 523, 532, 542, 551, 561, 0, 571,
    664, 665, 578, 575, 573, 0, 572, 668, 669, 210, 668, 584, 0, 164, 581, 579,
    377, 586, 596, 605, 615, 625, 680, 134, 681, 682, 683, 684, 685, 686, 687, 688,
    626, 636, 667, 648, 691, 215, 299, 0, 646, 691, 215, 299, 692, 647, 691, 215,
    299, 649, 658, 0, 670, 682, 680, 696, 697, 685, 299, 0, 146, 149, 0, 355,
    688, 698, 708, 718, 728, 738, 748, 758, 768, 778, 788, 798, 808, 0, 709, 818,
    828, 838, 848, 858, 868, 878, 900, 910, 909, 228, 391, 718, 389, 377, 299, 0,
    159, 228, 24, 355, 720, 911, 917, 922, 932, 942, 952, 962, 972, 982, 992, 739,
    740, 741, 742, 743, 993, 1000, 1010, 1020, 747, 0, 748, 748, 135, 0, 749, 749,
    1030, 1039, 1049, 1059, 752, 753, 1060, 1070, 755, 756, 1071, 1081, 1091, 1103, 1101, 228,
    1108, 1106, 377, 1111, 1117, 762, 1120, 1130, 1140, 1150, 1160, 1170, 1180, 1190, 1200, 1210,
    1220, 1230, 1240, 151, 0, 149, 774, 355, 0, 302, 93, 1243, 776, 1247, 0, 344,
    1246, 778, 779, 629, 406, 377, 194, 377, 0, 582, 377, 194, 377, 361, 377, 1248,
    1258, 0, 781, 389, 1268, 1278, 1288, 278, 276, 1289, 1307, 0, 784, 785, 1299, 788,
    790, 792, 794, 792, 792, 0, 784, 299, 1303, 788, 790, 792, 794, 792, 792, 742,
    24, 796, 1308, 1318, 1328, 1337, 1347, 1357, 1367, 0, 1377, 1387, 1397, 1407, 1417, 1427,
    1437, 1447, 1457, 1467, 1477, 1487, 1497, 0, 814, 22, 1507, 0, 21, 668, 1517, 0,
    164, 0, 361, 472, 0, 472, 68, 164, 1518, 1528, 1538, 1548, 0, 1554, 1564, 1576,
    477, 1574, 149, 817, 818, 1577, 1586, 1595, 1604, 822, 228, 389, 391, 0, 0, 307,
    22, 1617, 0, 302, 304, 0, 309, 1605, 299, 0, 451, 466, 0, 226, 1608, 822,
    228, 389, 391, 299, 0, 402, 405, 355, 299, 824, 0, 402, 1609, 405, 0, 406,
    226, 355, 407, 1614, 1610, 416, 0, 826, 0, 828, 302, 829, 0, 208, 539, 0,
    361, 470, 218, 220, 0, 831, 832, 0, 208, 539, 0, 361, 470, 218, 220, 834,
    1618, 438, 0, 302, 438, 1619, 0, 440, 441, 0, 440, 443, 0, 149, 159, 1620,
    386, 0, 380, 381, 0, 1624, 0, 149, 0, 1634, 835, 0, 835, 835, 1636, 836,
    1635, 835, 0, 835, 835, 68, 355, 361, 24, 837, 838, 839, 796, 840, 841, 843,
    0, 377, 683, 16, 0, 844, 0, 0, 1638, 1648, 0, 848, 477, 1657, 0, 848,
    68, 0, 1676, 0, 1667, 15, 82, 164, 0, 354, 1668, 0, 22, 355, 22, 0,
    374, 377, 0, 307, 22, 0, 68, 295, 22, 0, 19, 203, 0, 313, 315, 0,
    84, 183, 0, 173, 0, 470, 472, 0, 20, 195, 0, 1671, 332, 333, 295, 341,
    91, 535, 283, 448, 87, 342, 419, 614, 218, 271, 16, 0, 329, 330, 0, 334,
    271, 0, 335, 336, 338, 0, 335, 15, 0, 93, 82, 0, 343, 344, 350, 0,
    343, 344, 351, 0, 343, 344, 352, 0, 343, 344, 353, 22, 0, 68, 22, 0,
    1672, 172, 85, 86, 22, 1673, 0, 0, 0, 849, 1677, 477, 304, 1678, 1685, 1695,
    1705, 1715, 1725, 224, 856, 857, 858, 859, 860, 861, 0, 849, 829, 252, 0, 849,
    302, 829, 252, 862, 863, 864, 865, 866, 867, 868, 869, 1726, 1733, 1743, 1749, 0,
    1755, 299, 835, 175, 1758, 1768, 1778, 1788, 0, 135, 135, 0, 1798, 878, 882, 883,
    1799, 0, 885, 1832, 475, 0, 57, 194, 1830, 1829, 0, 377, 224, 1809, 299, 226,
    0, 1812, 1810, 377, 224, 452, 1818, 1816, 1815, 299, 226, 0, 1823, 1821, 377, 224,
    1826, 776, 0, 887, 299, 1833, 1852, 0, 1849, 1847, 0, 1846, 377, 1843, 299, 389,
    0, 380, 381, 889, 0, 887, 299, 0, 68, 307, 158, 1853, 1863, 1873, 1883, 1893,
    1903, 1913, 1923, 1933, 1943, 0, 898, 22, 1953, 0, 252, 899, 0, 0, 208, 1956,
    0, 1959, 0, 898, 2031, 0, 0, 164, 1971, 1969, 377, 252, 1976, 1975, 477, 0,
    902, 1974, 477, 903, 903, 0, 899, 0, 208, 1999, 1997, 1993, 0, 96, 149, 0,
    914, 210, 0, 915, 477, 0, 304, 477, 0, 916, 210, 0, 917, 477, 0, 918,
    477, 0, 919, 1982, 1979, 1977, 377, 175, 0, 920, 1985, 1983, 377, 0, 921, 1990,
    1988, 377, 0, 922, 477, 0, 887, 299, 0, 252, 2002, 477, 903, 0, 899, 0,
    208, 2006, 0, 924, 0, 915, 355, 477, 0, 304, 355, 477, 0, 925, 2005, 916,
    0, 917, 355, 477, 0, 918, 355, 477, 0, 450, 302, 304, 2003, 377, 0, 2013,
    2010, 776, 181, 2014, 0, 927, 0, 2018, 2015, 776, 181, 2019, 477, 903, 0, 927,
    0, 208, 2027, 0, 924, 0, 915, 355, 477, 0, 304, 355, 477, 0, 925, 2026,
    916, 0, 917, 355, 477, 0, 918, 355, 477, 0, 450, 917, 2025, 2022, 2020, 377,
    175, 0, 898, 2035, 2034, 929, 0, 931, 22, 0, 639, 2032, 0, 181, 927, 0,
    208, 2038, 0, 2041, 2051, 2061, 2071, 2081, 0, 59, 936, 172, 22, 2091, 0, 173,
    0, 59, 354, 2094, 159, 355, 159, 2097, 0, 208, 84, 183, 173, 0, 458, 84,
    0, 208, 937, 2098, 0, 355, 939, 0, 355, 940, 0, 59, 21, 447, 22, 0,
    0, 0, 59, 21, 22, 2104, 0, 943, 2101, 2099, 0, 0, 0, 0, 59, 332,
    376, 22, 2112, 2109, 2107, 2106, 0, 950, 224, 477, 0, 951, 224, 477, 0, 952,
    224, 477, 0, 953, 224, 2105, 956, 957, 0, 958, 224, 210, 0, 959, 224, 477,
    0, 252, 22, 0, 59, 18, 376, 22, 181, 2113, 0, 963, 964, 0, 966, 20,
    967, 969, 970, 0, 971, 972, 2114, 0, 0, 59, 19, 203, 208, 2124, 0, 185,
    224, 22, 0, 353, 224, 22, 0, 922, 224, 2117, 477, 226, 2122, 0, 950, 224,
    477, 2118, 0, 951, 224, 477, 0, 952, 224, 477, 0, 218, 224, 2123, 220, 226,
    0, 209, 224, 210, 0, 18, 213, 224, 215, 0, 217, 224, 210, 0, 59, 295,
    22, 0, 355, 159, 0, 380, 381, 0, 226, 299, 2129, 0, 0, 307, 22, 2128,
    0, 302, 304, 0, 309, 299, 0, 59, 313, 376, 315, 208, 0, 20, 195, 0,
    959, 974, 0, 252, 974, 278, 0, 59, 976, 149, 977, 149, 979, 149, 0, 59,
    335, 15, 2159, 0, 981, 376, 164, 2140, 2138, 2137, 0, 377, 159, 982, 0, 380,
    381, 2133, 983, 984, 0, 986, 2145, 2143, 164, 0, 342, 987, 982, 0, 981, 376,
    164, 0, 637, 467, 164, 2155, 2153, 2152, 0, 377, 0, 208, 476, 2148, 983, 984,
    2158, 0, 274, 639, 641, 226, 0, 342, 987, 982, 0, 59, 936, 989, 84, 376,
    183, 173, 0, 497, 2167, 0, 15, 2164, 2162, 2161, 159, 0, 377, 2160, 159, 992,
    0, 995, 159, 159, 996, 0, 59, 2173, 0, 470, 22, 1000, 68, 22, 0, 361,
    0, 2170, 2168, 0, 0, 226, 470, 1000, 68, 22, 0, 361, 0, 2174, 0, 59,
    2194, 0, 93, 82, 376, 22, 2186, 2184, 377, 1000, 355, 0, 0, 936, 93, 82,
    22, 2191, 2189, 377, 1000, 355, 0, 0, 59, 936, 1001, 376, 22, 0, 476, 2203,
    2201, 0, 158, 224, 2200, 228, 2197, 2195, 477, 355, 776, 0, 59, 356, 2216, 2206,
    2254, 2252, 2251, 0, 2220, 1004, 1006, 224, 22, 0, 1007, 224, 2231, 2221, 0, 1008,
    224, 2245, 2235, 0, 2250, 276, 1011, 2249, 85, 84, 224, 183, 1012, 1013, 0, 59,
    91, 22, 68, 164, 0, 355, 2257, 1016, 1017, 0, 274, 2258, 29, 55, 56, 57,
    58, 0, 181, 2260, 2259, 22, 293, 186, 188, 190, 0, 361, 2262, 299, 0, 208,
    309, 2265, 299, 0, 59, 936, 85, 183, 173, 996, 0, 59, 535, 537, 2270, 0,
    274, 29, 37, 0, 274, 2268, 541, 0, 208, 539, 0, 59, 2271, 283, 284, 192,
    0, 208, 2280, 2272, 556, 557, 2273, 62, 558, 2274, 61, 559, 2275, 466, 560, 2276,
    561, 562, 2277, 563, 564, 2278, 565, 566, 0, 18, 213, 215, 0, 567, 568, 2279,
    477, 304, 0, 310, 569, 477, 0, 41, 283, 192, 0, 41, 291, 192, 0, 283,
    192, 0, 1019, 192, 0, 284, 192, 0, 1020, 215, 0, 59, 20, 376, 2284, 0,
    195, 1021, 192, 195, 0, 59, 17, 376, 22, 355, 0, 2289, 0, 355, 159, 0,
    572, 406, 573, 215, 2285, 0, 574, 573, 215, 0, 451, 574, 2286, 0, 575, 573,
    215, 0, 451, 575, 0, 580, 208, 573, 215, 0, 581, 215, 2287, 582, 0, 451,
    582, 0, 526, 406, 2288, 583, 377, 0, 59, 989, 87, 376, 578, 2290, 1022, 0,
    59, 342, 376, 987, 0, 16, 477, 0, 959, 974, 976, 22, 982, 0, 59, 532,
    376, 22, 1000, 252, 22, 1024, 0, 0, 832, 2296, 2294, 377, 0, 96, 0, 0,
    1027, 2299, 1030, 1031, 0, 208, 2302, 2300, 0, 0, 59, 531, 376, 22, 1000, 2307,
    2305, 377, 252, 969, 18, 22, 2312, 2310, 0, 0, 832, 96, 0, 653, 96, 0,
    0, 96, 0, 0, 1033, 2315, 0, 0, 1027, 2317, 583, 1030, 1031, 0, 208, 2320,
    2318, 0, 0, 59, 531, 376, 22, 1000, 0, 252, 1034, 1036, 2323, 1040, 1041, 1042,
    1043, 2326, 2324, 0, 0, 274, 29, 37, 0, 208, 2331, 2329, 0, 0, 59, 531,
    376, 22, 1000, 0, 252, 970, 18, 22, 2336, 2334, 0, 2344, 0, 274, 29, 37,
    0, 274, 37, 2341, 2339, 0, 0, 208, 2347, 2345, 0, 0, 59, 531, 376, 22,
    1000, 0, 252, 1044, 1046, 96, 2350, 343, 1048, 1049, 2356, 0, 1033, 916, 0, 0,
    0, 1033, 1050, 2353, 2351, 0, 0, 309, 2362, 208, 2359, 2357, 0, 0, 0, 0,
    59, 419, 0, 376, 612, 2368, 2366, 2365, 1055, 1056, 1057, 1058, 68, 2372, 2371, 377,
    299, 252, 164, 0, 59, 614, 616, 18, 477, 535, 2374, 537, 0, 208, 539, 0,
    59, 2377, 0, 2376, 1061, 1062, 989, 465, 15, 376, 164, 2391, 2380, 2378, 377, 0,
    361, 158, 2389, 0, 208, 2386, 2384, 0, 158, 0, 224, 2383, 228, 248, 0, 458,
    461, 0, 68, 1063, 2390, 0, 1065, 515, 0, 58, 515, 301, 0, 218, 220, 355,
    2407, 2398, 2395, 776, 2402, 2399, 0, 15, 164, 1066, 2406, 2403, 0, 269, 389, 1068,
    0, 59, 2409, 0, 2408, 1061, 1062, 989, 465, 15, 376, 164, 2446, 0, 2417, 2415,
    2414, 626, 433, 0, 1071, 164, 2410, 1074, 0, 986, 2422, 2420, 164, 0, 467, 158,
    2432, 2430, 2429, 0, 377, 0, 208, 476, 2425, 386, 433, 0, 637, 467, 164, 2442,
    2440, 2439, 0, 377, 0, 208, 476, 2435, 386, 433, 2445, 0, 274, 639, 641, 226,
    2464, 0, 637, 406, 2447, 1079, 1080, 1081, 2459, 2455, 2453, 0, 2448, 377, 389, 2449,
    0, 380, 381, 158, 0, 361, 158, 2462, 0, 208, 421, 0, 458, 461, 0, 68,
    1063, 2463, 0, 1065, 515, 0, 58, 515, 301, 0, 218, 220, 0, 59, 343, 344,
    350, 22, 2469, 2468, 0, 352, 224, 22, 0, 898, 224, 22, 0, 59, 936, 307,
    271, 651, 2472, 659, 630, 0, 1083, 467, 2476, 2475, 56, 58, 272, 0, 57, 0,
    467, 2473, 377, 1084, 68, 1084, 68, 164, 2486, 0, 252, 164, 2479, 0, 302, 438,
    0, 438, 2478, 0, 440, 443, 0, 440, 441, 0, 1086, 2480, 1089, 1090, 15, 355,
    164, 0, 2481, 1089, 1090, 15, 355, 164, 0, 274, 1091, 2482, 452, 1094, 0, 1095,
    2483, 299, 0, 269, 2490, 84, 85, 389, 0, 59, 16, 22, 2502, 0, 355, 2493,
    2491, 0, 22, 159, 0, 355, 2496, 1080, 1097, 2499, 2497, 0, 22, 224, 0, 0,
    59, 284, 647, 0, 1098, 376, 274, 2503, 293, 149, 1099, 342, 987, 982, 0, 59,
    284, 192, 0, 59, 2504, 1100, 1101, 82, 376, 22, 2507, 2505, 377, 355, 776, 2510,
    2520, 2530, 2540, 2550, 2560, 2570, 2578, 0, 2588, 1107, 1108, 2589, 1109, 2592, 2602, 2612,
    2622, 2632, 2642, 2652, 2662, 2672, 2682, 2692, 2702, 149, 817, 818, 2704, 2723, 2714, 149,
    248, 0, 0, 0, 149, 835, 2735, 1114, 1151, 1153, 1155, 0, 2734, 1196, 1197, 1198,
    1199, 1200, 1201, 1202, 1203, 1204, 1205, 0, 1206, 1207, 1209, 1210, 1211, 1212, 1213, 1214,
    0, 2724, 1215, 1216, 0, 2725, 1218, 1219, 1216, 1220, 2727, 0, 2726, 1221, 1222, 0,
    1222, 1223, 1225, 0, 1225, 1223, 1226, 1216, 343, 1227, 2728, 1228, 1229, 2729, 1230, 1231,
    1232, 1233, 1234, 1235, 1236, 2730, 1237, 1238, 1239, 1240, 2731, 1241, 1242, 0, 1243, 2732,
    1223, 1216, 1244, 1245, 2733, 1048, 1246, 1247, 1248, 1249, 1250, 1251, 1252, 1253, 0, 1254,
    1216, 0, 1255, 1216, 2740, 2736, 1256, 1258, 1260, 1080, 2741, 2751, 1262, 0, 2763, 2761,
    2760, 1263, 1288, 0, 1151, 477, 2770, 1289, 1290, 0, 2766, 1291, 1292, 1293, 2767, 215,
    2771, 2781, 2815, 2824, 2834, 2844, 2854, 2864, 2874, 2884, 2894, 2904, 2914, 2924, 2934, 0,
    1306, 1307, 2944, 22, 29, 2945, 2955, 2965, 2975, 2985, 2995, 0, 1313, 22, 1314, 0,
    458, 1315, 274, 3005, 776, 0, 82, 22, 0, 93, 82, 22, 0, 0, 3006, 3016,
    0, 226, 639, 3026, 3036, 3046, 3056, 3066, 228, 1319, 0, 1319, 0, 224, 1320, 3069,
    3067, 1321, 3072, 3082, 3092, 0, 58, 252, 478, 164, 175, 166, 0, 361, 3102, 0,
    1325, 166, 1327, 3104, 0, 887, 253, 467, 22, 1329, 0, 1332, 3107, 175, 3105, 0,
    299, 166, 3108, 1334, 3118, 3128, 3138, 3148, 3158, 3168, 3178, 3188, 0, 1339, 248, 22,
    3198, 3208, 3218, 3228, 3238, 3248, 3258, 3268, 3278, 0, 1345, 3288, 29, 1347, 111, 1101,
    1100, 3289, 3299, 3309, 3319, 3329, 3339, 3349, 3358, 0, 885, 3368, 0, 1353, 477, 0,
    477, 1353, 3369, 3379, 3390, 3400, 3410, 3419, 0, 301, 172, 197, 3430, 0, 22, 3429,
    173, 0, 175, 280, 3432, 276, 278, 0, 301, 354, 197, 3433, 159, 355, 159, 280,
    0, 301, 332, 197, 22, 280, 0, 301, 19, 197, 203, 0, 208, 3436, 450, 0,
    301, 295, 197, 3439, 22, 280, 0, 301, 313, 197, 315, 280, 0, 301, 335, 15,
    197, 3441, 164, 657, 0, 301, 84, 197, 3443, 0, 22, 173, 280, 0, 301, 470,
    644, 197, 3445, 472, 280, 3447, 0, 301, 93, 82, 197, 3457, 164, 280, 0, 301,
    1001, 197, 22, 0, 301, 526, 406, 3460, 3459, 186, 188, 190, 192, 280, 0, 301,
    91, 197, 22, 68, 164, 280, 0, 301, 85, 197, 3462, 0, 183, 173, 3464, 278,
    276, 0, 301, 535, 197, 3465, 537, 280, 0, 301, 283, 197, 149, 0, 301, 20,
    197, 195, 280, 0, 301, 87, 197, 3467, 578, 280, 0, 301, 3469, 18, 21, 0,
    21, 447, 19, 470, 0, 93, 82, 283, 17, 20, 532, 531, 15, 16, 82, 284,
    197, 22, 3470, 0, 278, 0, 276, 0, 301, 419, 197, 3471, 612, 3473, 278, 276,
    0, 301, 614, 197, 616, 3474, 278, 276, 0, 301, 989, 15, 197, 3475, 164, 280,
    0, 301, 343, 344, 350, 197, 22, 280, 0, 301, 271, 197, 651, 68, 164, 280,
    0, 301, 16, 197, 3477, 159, 280, 0, 301, 284, 197, 192, 0, 301, 82, 197,
    3479, 164, 280, 3481, 3491, 3501, 3511, 3521, 0, 790, 299, 3531, 3541, 3551, 3561, 0,
    1153, 1366, 3564, 3574, 3584, 3594, 3604, 3614, 3623, 3633, 3643, 3653, 3663, 622, 3673, 3683,
    3693, 3703, 3713, 3723, 3733, 0, 1377, 208, 742, 3743, 3753, 0, 269, 22, 3765, 3763,
    299, 3768, 3778, 3788, 3798, 3808, 3820, 0, 3818, 664, 665, 668, 1389, 1390, 1391, 1392,
    1393, 1394, 210, 0, 96, 3819, 343, 1245, 1048, 1396, 0, 1397, 3827, 0, 3821, 664,
    665, 668, 3824, 3822, 1398, 3828, 776, 1400, 1402, 1404, 3829, 1406, 0, 1407, 3840, 3839,
    1408, 0, 796, 1407, 1411, 1412, 0, 849, 302, 1413, 1421, 1423, 1425, 0, 302, 1426,
    1427, 1428, 1407, 0, 1429, 1406, 3844, 573, 0, 691, 1432, 1433, 1434, 0, 1435, 3847,
    3846, 0, 3845, 739, 740, 742, 1435, 3851, 573, 0, 691, 1432, 0, 1436, 3861, 3860,
    0, 1438, 3852, 776, 0, 3855, 1439, 1440, 3856, 1442, 822, 24, 0, 1444, 822, 0,
    3876, 391, 389, 3866, 3865, 1446, 299, 776, 3863, 3862, 228, 377, 389, 1447, 299, 1449,
    1450, 0, 3869, 1153, 1155, 3872, 3870, 299, 0, 159, 3875, 477, 215, 210, 1452, 1453,
    1447, 1455, 24, 1456, 3877, 929, 228, 718, 1457, 1459, 377, 1461, 3878, 0, 3892, 164,
    0, 478, 3891, 3888, 164, 0, 164, 175, 3893, 3903, 3913, 3923, 3933, 3943, 3954, 0,
    1468, 3964, 629, 1470, 3965, 215, 299, 452, 3966, 452, 515, 3967, 478, 0, 208, 1471,
    3968, 0, 1468, 0, 1474, 3978, 29, 215, 0, 252, 22, 0, 208, 3981, 3979, 0,
    3984, 3994, 4010, 1483, 1485, 1487, 4008, 1489, 4004, 841, 4014, 0, 1493, 4024, 0, 887,
    299, 4027, 4037, 4047, 4057, 4067, 4077, 4087, 4097, 0, 274, 4107, 57, 0, 451, 832,
    57, 1497, 0, 832, 1497, 0, 467, 4108, 164, 4110, 528, 0, 621, 1498, 4111, 4121,
    4131, 4141, 4151, 0, 335, 336, 338, 0, 335, 832, 4161, 0, 0, 307, 22, 4175,
    0, 302, 304, 0, 309, 4171, 299, 0, 451, 466, 0, 226, 4174, 822, 228, 389,
    391, 299, 0, 402, 405, 355, 299, 824, 0, 0, 307, 22, 0, 309, 4176, 299,
    0, 451, 466, 4179, 4189, 0, 1501, 4202, 4201, 0, 253, 452, 0, 4199, 215, 718,
    1502, 377, 4200, 1505, 1506, 0, 1426, 1428, 0, 1507, 4203, 0, 253, 452, 291, 1471,
    0, 451, 1508, 4204, 1079, 515, 1510, 4205, 4215, 4225, 0, 252, 4235, 1513, 4237, 887,
    1521, 0, 291, 406, 0, 1523, 406, 1524, 1525, 1526, 1527, 1529, 1068, 1468, 1531, 1532,
    4250, 0, 1534, 4241, 4238, 1325, 1536, 166, 4242, 1538, 1539, 843, 1327, 0, 208, 1531,
    166, 1539, 1542, 4247, 0, 4243, 1327, 4269, 4266, 0, 4254, 1543, 1546, 4251, 1513, 0,
    1523, 406, 0, 291, 406, 4255, 4259, 0, 4258, 1327, 843, 0, 1523, 406, 0, 291,
    406, 4263, 4270, 4280, 4290, 4301, 4311, 4322, 299, 1551, 0, 1553, 4321, 299, 1551, 4340,
    299, 4326, 4323, 762, 0, 299, 355, 159, 0, 1556, 299, 252, 252, 299, 0, 4327,
    1263, 299, 4328, 0, 252, 299, 0, 274, 299, 0, 1332, 159, 0, 829, 4334, 175,
    4332, 1557, 1558, 0, 1561, 228, 0, 4335, 477, 149, 377, 299, 41, 4336, 477, 149,
    377, 299, 1562, 1565, 1567, 4338, 0, 299, 4337, 653, 840, 299, 0, 4344, 1288, 0,
    4362, 1353, 0, 365, 274, 16, 158, 1526, 4347, 499, 500, 501, 0, 302, 502, 4348,
    0, 494, 68, 304, 495, 0, 497, 304, 68, 304, 495, 498, 0, 94, 456, 4349,
    506, 505, 0, 507, 4350, 511, 512, 513, 0, 514, 215, 0, 515, 215, 0, 516,
    158, 0, 194, 158, 4355, 0, 4351, 181, 224, 4353, 4352, 158, 228, 0, 252, 253,
    0, 355, 4356, 477, 0, 477, 24, 477, 0, 1568, 299, 0, 1569, 1570, 4358, 0,
    1400, 1334, 0, 1402, 1334, 0, 4357, 1571, 762, 929, 1334, 0, 1568, 299, 1334, 792,
    0, 208, 4368, 4366, 158, 4371, 4381, 0, 4390, 0, 149, 835, 761, 4394, 1573, 249,
    761, 0, 4395, 41, 1575, 937, 1553, 4398, 159, 0, 158, 4396, 41, 1575, 937, 1553,
    4397, 159, 992, 0, 4399, 226, 224, 1436, 299, 4402, 4400, 1576, 4406, 0, 1577, 1578,
    1580, 0, 0, 183, 4405, 1577, 1581, 1066, 1582, 1583, 4407, 4417, 4427, 4437, 4447, 4457,
    4467, 4477, 4487, 4497, 4507, 4518, 4528, 0, 256, 289, 68, 1586, 181, 1588, 4538, 4548,
    4558, 0, 749, 622, 749, 4568, 0, 291, 406, 4582, 4581, 377, 215, 1594, 1596, 299,
    4578, 0, 1523, 406, 213, 1524, 1525, 1526, 1527, 4584, 0, 1523, 406, 1521, 1524, 1525,
    1526, 1468, 4585, 0, 4599, 4598, 377, 215, 299, 4595, 1599, 4601, 0, 1600, 1601, 4614,
    4612, 4611, 1594, 1109, 4617, 4627, 4637, 4647, 0, 1524, 4660, 4657, 299, 4661, 0, 1523,
    406, 1521, 1525, 1526, 1468, 4662, 4672, 4682, 4692, 4702, 4712, 4722, 0, 4732, 4742, 4752,
    4762, 4772, 4782, 4792, 4802, 4812, 4822, 4832, 4842, 4849, 4859, 4869, 4879, 0, 1621, 1438,
    4889, 0, 1621, 302, 1438, 4899, 0, 4909, 1622, 1623, 828, 4910, 4920, 4930, 4940, 0,
    1625, 335, 20, 0, 1626, 195, 0, 4950, 0, 213, 181, 1627, 4953, 4951, 1628, 252,
    342, 987, 1024, 195, 982, 4956, 4966, 0, 41, 21, 22, 4976, 0, 302, 41, 4992,
    4989, 4988, 4986, 1406, 776, 389, 4993, 5003, 5013, 149, 0, 149, 5023, 1630, 1631, 0,
    0, 380, 381, 0, 1632, 0, 828, 5024, 629, 1634, 421, 5025, 1630, 1631, 0, 828,
    5026, 629, 1634, 0, 5030, 377, 389, 5027, 299, 1636, 5031, 0, 0, 1033, 631, 0,
    208, 539, 0, 361, 470, 218, 220, 5041, 5051, 5061, 5071, 5081, 5091, 5101, 5111, 5121,
    5131, 5141, 5151, 5161, 5171, 5181, 5191, 0, 56, 1024, 164, 1646, 631, 0, 1648, 5201,
    26, 284, 653, 5202, 0, 226, 639, 776, 0, 68, 1649, 1650, 1652, 0, 1332, 5205,
    175, 5203, 0, 299, 1646, 5206, 5262, 5228, 5251, 5263, 5273, 5283, 5293, 5303, 5313, 5323,
    5333, 5343, 5353, 5363, 5373, 5383, 0, 1291, 5395, 0, 215, 5393, 477, 1263, 477, 0,
    477, 5394, 477, 1263, 5396, 5406, 1661, 1662, 1663, 1664, 1665, 1666, 0, 1024, 5407, 1101,
    1100, 465, 15, 164, 5408, 5418, 5428, 5438, 1452, 1668, 1669, 210, 1670, 1671, 0, 849,
    302, 829, 252, 5439, 1672, 5449, 5459, 5469, 5479, 5489, 5499, 5520, 0, 1677, 1678, 1546,
    5509, 1679, 0, 5513, 1681, 5516, 1683, 1685, 5517, 0, 1686, 1678, 1546, 1681, 0, 1687,
    1546, 5521, 0, 1011, 0, 0, 68, 5531, 5537, 5534, 299, 5538, 5542, 1689, 0, 5541,
    474, 1690, 1691, 1692, 0, 361, 5545, 5543, 149, 5548, 5558, 0, 1048, 1694, 1697, 5569,
    653, 1698, 697, 1699, 0, 5570, 208, 458, 826, 0, 201, 5571, 5581, 5591, 5601, 5611,
    5619, 5629, 5639, 5649, 0, 341, 5659, 248, 1704, 5660, 5670, 5680, 5690, 0, 5700, 5710,
    5720, 5730, 5740, 5750, 5760, 5770, 5780, 5790, 5800, 0, 748, 622, 748, 5810, 5820, 0,
    0, 302, 1710, 1406, 0, 918, 1407, 5830, 1071, 1711, 0, 1712, 181, 5831, 5841, 0,
    5852, 1713, 1714, 5853, 1715, 1716, 1717, 402, 407, 1718, 419, 423, 29, 0, 213, 5858,
    5855, 5854, 215, 299, 29, 5860, 0, 1531, 5859, 215, 299, 0, 24, 215, 5861, 5871,
    5881, 5891, 5901, 5911, 5921, 5931, 0, 5941, 5951, 0, 1722, 149, 5961, 477, 215, 210,
    691, 1452, 1453, 1723, 903, 1724, 1726, 1728, 5962, 5971, 5978, 0, 1034, 477, 0, 5988,
    5998, 6008, 6018, 6028, 6038, 0, 1732, 15, 478, 6050, 6048, 164, 175, 0, 41, 6051,
    0, 329, 1497, 0, 452, 1497, 0, 452, 1733, 0, 1497, 57, 1733, 1497, 0, 1497,
    452, 1733, 1733, 0, 329, 1733, 1734, 528, 6052, 6062, 6072, 6082, 6092, 6102, 6112, 6122,
    6132, 6142, 6152, 6162, 6172, 0, 1745, 835, 149, 6185, 0, 1001, 22, 0, 24, 6182,
    776, 6188, 6198, 6208, 6218, 6228, 6238, 0, 6248, 6258, 6268, 0, 6278, 6288, 6298, 0,
    1397, 6309, 0, 6308, 1755, 1756, 1757, 1758, 1759, 0, 208, 6312, 6310, 0, 0, 355,
    6315, 343, 1048, 0, 274, 6316, 776, 0, 82, 22, 0, 93, 82, 22, 0, 0,
    6317, 6327, 6337, 6347, 6357, 6367, 6377, 6387, 6397, 6407, 6417, 58, 0, 56, 631, 1066,
    0, 1768, 1024, 6427, 6437, 1769, 1771, 1773, 0, 1095, 1775, 0, 1428, 299, 1776, 6441,
    1778, 1780, 0, 885, 475, 0, 1095, 302, 1775, 406, 531, 0, 1428, 299, 1776, 6442,
    1778, 1780, 0, 885, 475, 0, 1095, 302, 1775, 0, 406, 1782, 0, 1428, 299, 1776,
    6443, 1784, 0, 885, 475, 0, 1786, 6444, 164, 1787, 361, 6448, 164, 1787, 0, 6445,
    776, 166, 0, 389, 166, 6449, 1683, 6452, 1788, 0, 57, 1790, 6455, 6465, 0, 6476,
    0, 776, 1795, 6480, 6490, 6500, 6510, 6520, 6530, 6540, 6549, 6559, 6569, 6579, 6589, 6599,
    6609, 6619, 6629, 6639, 6649, 6658, 6668, 6678, 6688, 6698, 6708, 6718, 6728, 6738, 6748, 1809,
    6752, 6761, 6771, 6781, 6791, 6801, 6810, 6820, 6830, 6840, 6850, 6860, 6870, 6880, 6890, 6900,
    6910, 6921, 6931, 0, 248, 6941, 1815, 1436, 224, 299, 0, 149, 355, 6945, 149, 6942,
    1816, 0, 1526, 6946, 1818, 6948, 6958, 6968, 6979, 0, 1824, 878, 6989, 6999, 7009, 7019,
    0, 7028, 7038, 7045, 7055, 7065, 7075, 7085, 7095, 7105, 7115, 7125, 7135, 7145, 7155, 7165,
    7175, 7185, 7195, 7205, 7215, 7234, 1066, 1834, 7225, 929, 7228, 1571, 7231, 1835, 1836, 0,
    7235, 7240, 762, 7239, 7236, 929, 1835, 1402, 1400, 1404, 7241, 7251, 7261, 7271, 7281, 0,
    7291, 1840, 1841, 1842, 1843, 1844, 7292, 7302, 0, 7312, 0, 1845, 622, 0, 748, 749,
    0, 747, 748, 0, 747, 749, 7313, 1846, 7323, 7333, 7343, 7353, 7363, 0, 1850, 149,
    0, 24, 477, 7373, 7383, 7393, 7403, 7414, 7424, 7434, 7444, 7455, 7465, 0, 477, 840,
    696, 7486, 7475, 1852, 7477, 7489, 835, 0, 835, 835, 7490, 68, 355, 361, 24, 837,
    838, 839, 796, 840, 841, 843, 7491, 7501, 7511, 7521, 0, 1531, 7531, 215, 299, 7532,
    7542, 7552, 7562, 7572, 7581, 7591, 7601, 7611, 7621, 7631, 7641, 7651, 7661, 7671, 7681, 7691,
    7701, 7711, 7721, 0, 476, 7735, 7731, 1869, 7738, 0, 7749, 7748, 1628, 477, 7751, 7761,
    0, 1084, 649, 0, 1523, 406, 7773, 0, 7770, 377, 215, 299, 7771, 1630, 1631, 0,
    361, 742, 0, 828, 7772, 629, 1634, 1870, 1521, 1501, 7775, 213, 1524, 1525, 1526, 1501,
    1561, 0, 208, 336, 1532, 7776, 7786, 1523, 1872, 7787, 7797, 7807, 7817, 7827, 0, 1562,
    1875, 7840, 149, 7837, 1816, 7841, 0, 747, 747, 0, 7856, 0, 7853, 7851, 377, 1453,
    822, 1557, 24, 7852, 377, 1453, 822, 1557, 377, 1877, 7862, 0, 7859, 7857, 377, 1453,
    822, 1557, 24, 7858, 377, 1453, 822, 1557, 377, 7863, 7873, 7883, 7893, 7903, 7913, 7923,
    7933, 7943, 7953, 7962, 7971, 7981, 7991, 8021, 0, 41, 8003, 8001, 299, 0, 252, 8009,
    8007, 8006, 299, 574, 575, 181, 8015, 8013, 8012, 299, 574, 575, 0, 208, 8018, 0,
    1887, 215, 24, 1888, 215, 0, 637, 406, 8027, 8024, 8022, 299, 8028, 8038, 8048, 8058,
    8080, 0, 1891, 8078, 8068, 1891, 477, 0, 8081, 8091, 8101, 8112, 8122, 8133, 8143, 8153,
    8162, 8172, 8182, 8192, 8202, 8211, 8221, 8231, 8241, 8251, 8261, 8271, 8281, 8291, 8301, 8311,
    8321, 8331, 8340, 1900, 1903, 1905, 0, 8344, 8354, 8365, 0, 1907, 8375, 8385, 1908, 8386,
    8396, 8406, 8416, 8426, 0, 1307, 22, 8438, 8436, 159, 355, 8441, 776, 1911, 8442, 8452,
    0, 831, 832, 8462, 8472, 8482, 8492, 8502, 8512, 8522, 8532, 8542, 8552, 8562, 8572, 8582,
    8592, 8602, 8612, 8622, 8632, 0, 1922, 8642, 0, 1923, 8652, 0, 8665, 8662, 0, 1925,
    8661, 158, 477, 8666, 8676, 8693, 0, 15, 8686, 1926, 0, 24, 8688, 15, 37, 0,
    37, 41, 20, 8690, 8689, 195, 317, 0, 24, 8692, 15, 37, 149, 0, 1928, 631,
    0, 887, 8694, 299, 8697, 0, 573, 215, 0, 356, 8721, 0, 248, 835, 8717, 8707,
    0, 158, 0, 835, 158, 8724, 0, 742, 8734, 29, 1932, 1933, 8735, 776, 8738, 8748,
    8758, 8790, 8768, 8779, 8917, 0, 8813, 8791, 8802, 8825, 1937, 8814, 0, 8829, 8850, 1937,
    8840, 0, 8854, 8875, 1937, 8865, 0, 1939, 8879, 8915, 8914, 8888, 8910, 1937, 8899, 8918,
    8928, 8938, 8948, 8958, 8967, 8976, 8986, 8995, 9004, 9014, 9024, 9034, 0, 1948, 526, 406,
    9045, 9044, 186, 188, 190, 192, 181, 9047, 186, 188, 190, 192, 9048, 9058, 9068, 9078,
    0, 270, 164, 631, 1953, 9088, 0, 68, 58, 1954, 0, 68, 57, 1954, 0, 1955,
    9092, 474, 1957, 1958, 9093, 9103, 9113, 9129, 278, 0, 194, 9123, 226, 304, 9126, 9124,
    377, 276, 0, 451, 1960, 9130, 0, 620, 93, 82, 644, 164, 1068, 9140, 9150, 9160,
    9170, 9180, 0, 1965, 9193, 9191, 9190, 0, 644, 210, 0, 218, 220, 0, 668, 210,
    9197, 0, 470, 644, 472, 0, 15, 644, 164, 0, 20, 644, 195, 0, 9196, 19,
    26, 644, 203, 0, 1319, 0, 835, 1319, 0, 224, 1320, 9200, 9198, 1966, 9203, 9213,
    9223, 9233, 9243, 9253, 9263, 9273, 9283, 9293, 9303, 9313, 9323, 9333, 0, 255, 1975, 1021,
    0, 255, 9343, 29, 283, 158, 9344, 9354, 9364, 9374, 9384, 9394, 9404, 9414, 9424, 9434,
    9444, 9454, 9464, 0, 273, 0, 256, 267, 274, 9474, 0, 289, 68, 1586, 0, 283,
    192, 0, 1981, 68, 284, 285, 285, 252, 1588, 280, 9475, 9484, 9494, 9504, 149, 9514,
    9524, 9534, 9542, 9552, 9562, 9572, 9582, 9592, 9602, 9612, 9622, 9632, 9642, 9652, 9662, 0,
    1989, 9672, 1990, 26, 9673, 215, 0, 9676, 1991, 9677, 215, 9680, 9690, 9700, 9710, 9720,
    9730, 9740, 9750, 9760, 9770, 9780, 9790, 9800, 9810, 9820, 9830, 9840, 0, 456, 2001, 0,
    274, 22, 68, 9865, 0, 15, 164, 0, 374, 377, 0, 172, 183, 9850, 173, 0,
    19, 203, 0, 295, 22, 0, 334, 271, 22, 0, 335, 15, 164, 0, 84, 183,
    9856, 9853, 1576, 0, 2002, 1699, 215, 0, 93, 82, 164, 0, 340, 341, 22, 0,
    85, 183, 9860, 9857, 1576, 0, 535, 22, 0, 283, 192, 0, 86, 183, 9864, 9861,
    1576, 0, 20, 195, 0, 87, 22, 0, 614, 22, 0, 218, 22, 0, 16, 22,
    0, 82, 164, 849, 9866, 477, 304, 9867, 9877, 2004, 0, 696, 166, 9883, 0, 829,
    0, 68, 9880, 9878, 299, 29, 0, 55, 2006, 9884, 2008, 1024, 252, 887, 0, 1523,
    406, 0, 68, 1649, 213, 1332, 1877, 1527, 0, 208, 451, 336, 2010, 1795, 9896, 1024,
    252, 9886, 887, 0, 1523, 406, 213, 1332, 24, 1527, 1795, 9897, 0, 2012, 2014, 2016,
    1329, 2018, 2020, 1461, 2022, 2022, 140, 1521, 2024, 2026, 1532, 1527, 1529, 1068, 0, 68,
    1649, 1332, 2010, 1795, 9918, 9910, 9907, 1571, 9914, 9911, 2028, 2029, 9915, 776, 0, 835,
    149, 124, 9919, 0, 835, 149, 124, 9923, 9932, 9942, 9952, 0, 575, 215, 0, 451,
    575, 9953, 0, 574, 215, 0, 451, 574, 9954, 9964, 9974, 9984, 9994, 10004, 10014, 10024,
    10034, 10044, 10054, 284, 186, 188, 190, 10055, 10065, 0, 194, 10075, 2035, 0, 10098, 0,
    377, 124, 224, 10077, 228, 391, 389, 377, 299, 226, 10078, 822, 0, 10084, 10082, 377,
    224, 10095, 10094, 776, 10092, 0, 10087, 228, 391, 389, 377, 299, 226, 10088, 822, 0,
    194, 1716, 10101, 29, 10099, 22, 10102, 441, 443, 0, 1835, 10103, 0, 1527, 1835, 140,
    1521, 2022, 10107, 10118, 2037, 0, 10117, 2038, 1627, 29, 2039, 0, 194, 20, 376, 195,
    0, 194, 10119, 0, 1062, 1975, 0, 1975, 1975, 1021, 10120, 192, 226, 0, 194, 10121,
    1975, 1062, 10129, 0, 158, 10122, 181, 224, 10126, 226, 10124, 10123, 228, 248, 249, 571,
    0, 2040, 2041, 10127, 477, 1062, 226, 0, 20, 477, 0, 283, 10128, 583, 192, 10130,
    10140, 10150, 10160, 10170, 10180, 0, 10190, 1439, 1440, 10191, 0, 837, 159, 1442, 0, 2045,
    59, 10195, 0, 18, 0, 470, 0, 93, 82, 0, 532, 0, 531, 0, 15, 0,
    82, 22, 0, 2045, 1718, 0, 68, 22, 0, 252, 22, 1000, 10196, 0, 1071, 477,
    0, 887, 299, 10197, 0, 2045, 93, 106, 0, 252, 22, 1000, 0, 2045, 10207, 2048,
    40, 45, 0, 21, 943, 44, 1718, 0, 93, 106, 39, 43, 2049, 2050, 37, 38,
    106, 2051, 22, 0, 252, 22, 10208, 0, 1071, 477, 0, 887, 299, 10209, 10219, 10229,
    2054, 2055, 10230, 10240, 215, 10244, 10254, 248, 249, 1628, 10255, 248, 249, 835, 10256, 149,
    0, 149, 817, 175, 10258, 10269, 10279, 10289, 0, 1258, 124, 10299, 10309, 10319, 10328, 10337,
    10347, 10357, 10367, 10377, 10387, 10397, 10407, 10417, 10427, 10437, 10447, 10457, 10467, 10477, 10487, 10497,
    10507, 10517, 10527, 10537, 10547, 10557, 10567, 10577, 10587, 10597, 10607, 10616, 10626, 10636, 10646, 10656,
    776, 1911, 1400, 2079, 2081, 2083, 2085, 2087, 2089, 2091, 2093, 2095, 2097, 2099, 2101, 2103,
    2105, 2107, 2109, 2111, 2113, 1404, 1402, 2115, 2117, 2119, 2121, 2123, 2125, 2127, 2129, 2131,
    2133, 2135, 2137, 2139, 2141, 2143, 2145, 2147, 2149, 2151, 2153, 2155, 2157, 2159, 2161, 2163,
    2165, 2167, 2169, 2171, 2173, 2175, 2177, 2179, 2181, 2183, 2185, 2186, 2113, 2188, 2190, 2192,
    2193, 2195, 2197, 2199, 2201, 841, 10657, 10667, 10677, 10687, 10697, 10707, 10717, 10727, 10737, 10747,
    10757, 2205, 380, 10758, 10768, 10780, 10778, 0, 696, 166, 0, 10783, 10793, 10803, 10813, 10823,
    10833, 10843, 10853, 10863, 10873, 10880, 10890, 10900, 10910, 10920, 0, 0, 307, 22, 10941, 0,
    309, 10930, 299, 0, 451, 466, 0, 826, 0, 828, 302, 829, 631, 2215, 0, 2217,
    631, 2215, 0, 1507, 0, 361, 363, 10935, 10933, 2218, 2215, 0, 887, 10938, 299, 0,
    335, 832, 631, 834, 10944, 10942, 438, 0, 302, 438, 10943, 0, 440, 441, 0, 440,
    443, 0, 302, 310, 0, 451, 466, 0, 0, 307, 22, 0, 10948, 826, 2217, 361,
    470, 472, 10949, 438, 0, 302, 438, 10950, 0, 440, 441, 0, 440, 443, 0, 10957,
    1066, 391, 0, 389, 0, 208, 2220, 164, 10951, 776, 10954, 1911, 10958, 10968, 10978, 10988,
    10998, 0, 11008, 2222, 796, 2223, 0, 11012, 2224, 2223, 11016, 11026, 11036, 11046, 11056, 0,
    11066, 1100, 1101, 11067, 11077, 2226, 989, 11078, 11088, 11098, 11108, 11118, 11128, 11138, 11148, 11158,
    11168, 11177, 0, 11187, 2040, 2234, 1216, 0, 11188, 208, 458, 2040, 2041, 11189, 0, 2235,
    2040, 2041, 299, 11193, 11203, 11213, 11223, 11233, 11243, 11253, 11263, 11273, 11283, 11293, 11303, 11313,
    11323, 11333, 11343, 11353, 11363, 11373, 11383, 0, 11393, 580, 1569, 1063, 2248, 792, 11394, 2250,
    2251, 0, 576, 149, 0, 1428, 451, 2252, 11395, 11405, 11415, 11425, 11435, 11445, 11455, 11465,
    2256, 2257, 2258, 11466, 11476, 11487, 0, 272, 15, 11498, 11497, 0, 478, 164, 0, 164,
    175, 0, 11500, 418, 2259, 407, 280, 11501, 11511, 11521, 11531, 11541, 11551, 11561, 11573, 11571,
    696, 11576, 0, 0, 1553, 0, 248, 1436, 1258, 1724, 0, 1153, 2262, 11586, 11596, 11606,
    11616, 11626, 2265, 11636, 2266, 2267, 2268, 11637, 11647, 0, 2271, 11657, 829, 29, 11658, 0,
    826, 11668, 11678, 11688, 11699, 0, 2273, 11709, 149, 175, 11710, 11720, 0, 2012, 2014, 2016,
    1329, 2018, 2020, 1461, 2022, 1527, 1529, 1068, 140, 1521, 0, 208, 451, 336, 0, 68,
    1649, 1332, 2010, 1795, 0, 1835, 11730, 0, 1527, 1835, 11734, 11744, 11754, 11764, 0, 57,
    478, 164, 166, 194, 1790, 2016, 11774, 0, 887, 253, 467, 22, 1329, 0, 1332, 11777,
    175, 11775, 0, 299, 166, 11778, 11788, 11798, 11808, 11818, 0, 2278, 203, 11828, 11838, 149,
    11848, 11858, 11868, 11878, 11888, 11898, 11908, 11918, 0, 2286, 11936, 0, 474, 914, 668, 11928,
    664, 665, 11933, 11931, 0, 11929, 474, 914, 668, 664, 665, 2287, 669, 2288, 2289, 272,
    507, 11930, 228, 248, 571, 11937, 0, 164, 631, 11939, 11949, 11959, 11969, 0, 639, 11985,
    11982, 11980, 11979, 299, 226, 166, 140, 1521, 11987, 11997, 12007, 12017, 12027, 12037, 12047, 12057,
    12067, 12077, 12087, 477, 248, 12088, 12098, 12108, 12118, 12128, 12138, 12148, 12157, 12167, 12177, 12187,
    12197, 12207, 12245, 0, 12217, 1230, 2299, 1235, 2300, 2301, 2302, 2303, 2304, 2305, 2306, 2307,
    2308, 2309, 2310, 2311, 2312, 2313, 2314, 2315, 2316, 2317, 2318, 2319, 2320, 2321, 2322, 2323,
    2324, 2325, 2326, 2327, 2328, 2329, 2330, 2331, 2332, 12237, 12235, 12234, 12223, 12220, 12218, 2333,
    12231, 12229, 12226, 12224, 2333, 1114, 0, 12240, 2334, 2335, 12242, 0, 12241, 1230, 2299, 1235,
    2300, 2301, 2302, 2303, 2304, 2305, 2306, 2307, 2308, 2309, 2310, 2311, 2312, 2313, 2314, 2315,
    2316, 2317, 2318, 2319, 2320, 2321, 2322, 2323, 2324, 2325, 2326, 2327, 2328, 2329, 2330, 2331,
    2332, 2334, 2335, 0, 24, 215, 0, 1095, 0, 299, 12246, 1776, 12249, 299, 12252, 12255,
    12258, 0, 887, 12271, 12268, 299, 12272, 213, 0, 291, 406, 0, 1523, 406, 1524, 1525,
    1526, 1877, 1332, 0, 68, 1649, 1532, 12273, 12283, 12293, 12303, 0, 2342, 0, 12311, 12310,
    0, 149, 817, 0, 175, 835, 175, 12315, 0, 149, 12325, 637, 1523, 2343, 140, 2345,
    12326, 0, 208, 12336, 2348, 1062, 309, 267, 0, 208, 2349, 12337, 12340, 2351, 55, 12342,
    2353, 0, 208, 2349, 12345, 12348, 2351, 55, 12350, 2029, 0, 208, 0, 451, 336, 0,
    12353, 0, 208, 451, 20, 2354, 0, 2355, 291, 12363, 140, 12366, 12376, 12386, 12393, 12403,
    12413, 12423, 12433, 12443, 12453, 12463, 12473, 12483, 12493, 12503, 12513, 12523, 12533, 12543, 12553, 12563,
    12573, 12583, 12593, 12603, 12613, 12623, 12633, 12643, 12653,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 67, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 119, 121, 4294967295, 23, 4294967295,
    125, 127, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 142, 4294967295,
    143, 148, 4294967295, 4294967295, 4294967295, 4294967295, 152, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 160,
    161, 4294967295, 4294967295, 165, 167, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 174, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295,
    4294967295, 196, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295,
    4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 262, 4294967295, 264, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 262, 4294967295, 264, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295,
    286, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 4294967295, 23,
    4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 23, 4294967295, 4294967295, 4294967295, 23, 4294967295,
    4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 23, 4294967295,
    4294967295, 4294967295, 316, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 196, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295,
    23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 184, 174, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 160, 4294967295, 160, 4294967295,
    4294967295, 4294967295, 4294967295, 23, 4294967295, 364, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    378, 160, 4294967295, 4294967295, 382, 4294967295, 387, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295,
    378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 160, 4294967295, 4294967295, 382, 4294967295, 4294967295, 4294967295, 300, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 390, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 422, 4294967295,
    4294967295, 422, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    435, 4294967295, 4294967295, 435, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 422, 4294967295, 4294967295, 422, 4294967295, 4294967295,
    4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 473, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 165, 4294967295, 4294967295, 4294967295, 481, 4294967295, 4294967295, 4294967295, 378, 4294967295, 378, 4294967295, 4294967295, 4294967295, 165,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 174, 4294967295, 519, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295,
    196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    316, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 165, 4294967295, 4294967295, 525, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 378, 4294967295, 4294967295, 4294967295, 165,
    4294967295, 4294967295, 4294967295, 196, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 4294967295, 4294967295, 221,
    4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 23,
    4294967295, 4294967295, 4294967295, 23, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    193, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    184, 174, 4294967295, 534, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 196, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 4294967295, 538, 4294967295,
    4294967295, 4294967295, 540, 4294967295, 4294967295, 4294967295, 542, 4294967295, 4294967295, 4294967295, 542, 4294967295, 4294967295, 4294967295, 542, 4294967295,
    4294967295, 4294967295, 538, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 23,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295,
    4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 196, 4294967295, 4294967295,
    4294967295, 4294967295, 196, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 160, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 579, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    378, 4294967295, 4294967295, 4294967295, 4294967295, 579, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 579, 4294967295, 4294967295, 4294967295, 196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295,
    4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 613, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 613, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 617, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 538, 4294967295, 4294967295, 540, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 540, 4294967295, 4294967295, 4294967295, 4294967295, 540, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295,
    4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 617, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 160, 4294967295, 4294967295, 382, 4294967295, 387, 4294967295, 4294967295,
    4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 160, 4294967295,
    4294967295, 382, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 390, 4294967295, 300, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 422, 4294967295, 4294967295, 422, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 435, 4294967295, 4294967295, 435, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 422, 4294967295, 4294967295, 422, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 473,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 627, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 482, 4294967295, 4294967295,
    4294967295, 378, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295,
    4294967295, 165, 4294967295, 4294967295, 4294967295, 196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 642, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 4294967295, 23,
    4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    23, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295,
    4294967295, 23, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 23, 4294967295, 23, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 652, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 652,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 4294967295, 160, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295,
    196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 160, 4294967295, 4294967295, 382, 4294967295, 4294967295, 4294967295, 4294967295, 378,
    4294967295, 4294967295, 4294967295, 4294967295, 160, 4294967295, 4294967295, 382, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295,
    4294967295, 4294967295, 378, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    390, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 378, 4294967295, 378, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 196, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295,
    378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 671, 671, 4294967295, 671, 671,
    4294967295, 4294967295, 4294967295, 4294967295, 216, 4294967295, 300, 4294967295, 4294967295, 216, 4294967295, 300, 4294967295, 4294967295, 216, 4294967295,
    300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 148, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 719, 390, 378, 300, 4294967295,
    160, 4294967295, 4294967295, 4294967295, 721, 722, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 671, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 378, 760, 4294967295, 763, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 152, 4294967295, 4294967295, 775, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 378, 4294967295, 4294967295, 378, 4294967295, 378, 4294967295, 378, 4294967295,
    4294967295, 4294967295, 4294967295, 390, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 143, 4294967295, 789,
    4294967295, 4294967295, 795, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 789, 4294967295, 4294967295, 795, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    165, 4294967295, 4294967295, 473, 4294967295, 473, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 823, 4294967295, 390, 4294967295, 4294967295, 4294967295, 4294967295,
    23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 823,
    4294967295, 390, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 540, 4294967295,
    4294967295, 4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 540, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 160, 4294967295,
    387, 4294967295, 4294967295, 382, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 23, 4294967295,
    4294967295, 378, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 204, 4294967295, 4294967295, 316, 4294967295,
    4294967295, 184, 4294967295, 174, 4294967295, 4294967295, 473, 4294967295, 4294967295, 196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 23, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 850, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 225, 225, 225, 225, 225, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 225, 225, 225, 225, 4294967295, 4294967295, 4294967295, 870, 225, 4294967295, 671, 225, 4294967295,
    4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 225, 4294967295, 300, 4294967295,
    4294967295, 4294967295, 4294967295, 378, 225, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 378, 225,
    4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 300, 390,
    4294967295, 4294967295, 382, 890, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 904, 904, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295,
    4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 904, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 904, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 930, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 174,
    4294967295, 4294967295, 4294967295, 4294967295, 160, 4294967295, 160, 4294967295, 4294967295, 4294967295, 4294967295, 184, 174, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295,
    225, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 225, 4294967295,
    4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295,
    225, 23, 4294967295, 4294967295, 225, 23, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225,
    4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 225, 4294967295, 221, 4294967295,
    4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295,
    23, 4294967295, 4294967295, 160, 4294967295, 4294967295, 382, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 4294967295, 196, 4294967295,
    4294967295, 975, 4294967295, 4294967295, 975, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 378, 160, 4294967295, 4294967295, 4294967295,
    382, 4294967295, 387, 435, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 988, 4294967295, 4294967295, 4294967295, 4294967295,
    165, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 387, 435,
    4294967295, 4294967295, 4294967295, 4294967295, 642, 4294967295, 4294967295, 4294967295, 988, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    184, 174, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 160, 4294967295, 378, 4294967295, 160, 993,
    4294967295, 4294967295, 160, 160, 999, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 23, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    23, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 23, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295,
    225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 184, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 23, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295,
    4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 184, 174, 999, 4294967295, 4294967295, 4294967295, 538, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 542, 4294967295, 4294967295, 540, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295,
    193, 4294967295, 4294967295, 193, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    196, 4294967295, 193, 196, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 160, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 579, 4294967295, 1023, 4294967295,
    4294967295, 4294967295, 4294967295, 988, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 975, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 23, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295,
    4294967295, 378, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 613, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378,
    300, 4294967295, 165, 4294967295, 4294967295, 4294967295, 617, 4294967295, 4294967295, 4294967295, 4294967295, 538, 4294967295, 4294967295, 540, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 378, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 1067, 4294967295, 4294967295, 4294967295, 4294967295, 390, 1069,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 627, 435, 4294967295, 4294967295, 165, 4294967295, 1075, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 387, 435, 4294967295, 4294967295, 4294967295, 165, 4294967295,
    4294967295, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 387, 435, 4294967295, 4294967295, 4294967295, 4294967295, 642, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 390, 4294967295,
    4294967295, 4294967295, 382, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 422, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 225, 23, 4294967295, 4294967295, 225, 23, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 652, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 390, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 23, 160, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 225, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 988, 4294967295, 4294967295, 4294967295,
    4294967295, 193, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 378, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 184, 4294967295, 1110, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1150, 1152, 1154, 1156, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1217, 4294967295, 4294967295, 4294967295, 4294967295, 1217, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1217, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1217, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1217, 4294967295, 4294967295, 1217, 4294967295, 4294967295, 1257, 1259, 1261, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1152, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295,
    4294967295, 1322, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 167, 4294967295, 4294967295, 4294967295, 4294967295,
    1326, 167, 1328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 1330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    300, 167, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1354, 4294967295, 4294967295,
    4294967295, 1354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295,
    174, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 160, 4294967295, 160, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 174, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 473, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 23, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 174, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 538, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 579, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 613, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 617, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 652, 4294967295, 165, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 160, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1154, 1367, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295,
    300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1399, 4294967295, 4294967295, 1401, 1403, 1405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 216, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 216, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1441, 4294967295, 1443, 823, 4294967295, 4294967295, 4294967295, 823, 4294967295,
    4294967295, 4294967295, 390, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 378, 390, 1448, 300, 4294967295,
    1451, 4294967295, 4294967295, 1154, 1156, 4294967295, 4294967295, 300, 4294967295, 160, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1454,
    1448, 4294967295, 4294967295, 4294967295, 4294967295, 930, 4294967295, 719, 1458, 1460, 378, 1462, 4294967295, 4294967295, 4294967295, 165,
    4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1484, 1486, 1488, 4294967295, 1490, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 823, 4294967295, 390,
    4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 300,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 719,
    4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1514, 4294967295, 4294967295,
    1522, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1528, 1530, 1069, 4294967295, 4294967295, 1533,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1326, 1537, 167, 4294967295, 4294967295, 1541, 4294967295, 1328, 4294967295, 4294967295, 4294967295,
    167, 1541, 4294967295, 4294967295, 4294967295, 4294967295, 1328, 4294967295, 4294967295, 4294967295, 4294967295, 1545, 1547, 4294967295, 1514, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 143, 4294967295, 4294967295, 4294967295, 1328, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 161, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 1552, 4294967295, 4294967295, 4294967295, 300, 1552, 4294967295,
    300, 4294967295, 4294967295, 763, 4294967295, 300, 4294967295, 160, 4294967295, 4294967295, 300, 4294967295, 4294967295, 300, 4294967295, 4294967295,
    4294967295, 300, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 300, 4294967295, 4294967295, 160, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1559, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 300, 4294967295, 4294967295, 4294967295, 4294967295,
    378, 300, 4294967295, 1566, 1458, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1401, 4294967295, 4294967295, 1403, 4294967295, 4294967295, 4294967295, 1572, 763, 930, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 760, 4294967295, 4294967295, 4294967295,
    760, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 160, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 160, 993, 4294967295, 4294967295, 4294967295, 225, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 1579,
    4294967295, 4294967295, 4294967295, 184, 4294967295, 184, 184, 1067, 1579, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 1587, 4294967295, 1589, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 1595, 1598, 300,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1528, 4294967295, 4294967295, 4294967295, 4294967295, 1522, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1595, 1110, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295,
    4294967295, 1522, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1616, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 988, 4294967295, 196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 390, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 382, 4294967295, 1635, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 422, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 390, 4294967295, 300, 1637, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 540, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 167, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1651, 1653, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 300, 167, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1547,
    4294967295, 4294967295, 4294967295, 143, 1682, 4294967295, 1684, 4294967295, 161, 4294967295, 4294967295, 4294967295, 1547, 1682, 4294967295, 4294967295,
    1547, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 143, 4294967295, 4294967295, 300, 161, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    216, 4294967295, 1454, 4294967295, 904, 1725, 1727, 1729, 1356, 1356, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1067,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1770, 1772, 1774, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295,
    1779, 1781, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295,
    1779, 1781, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295,
    4294967295, 1785, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295,
    4294967295, 167, 4294967295, 390, 167, 143, 1684, 161, 1789, 4294967295, 4294967295, 1791, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1817, 4294967295, 4294967295, 4294967295, 1819, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1067, 930, 4294967295, 930, 4294967295, 1572, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 763, 4294967295, 4294967295, 930, 4294967295, 1403, 1401, 1405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1853, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1871, 1522, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1533, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1817, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 1454,
    823, 4294967295, 4294967295, 4294967295, 378, 1454, 823, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 378, 1454,
    823, 4294967295, 4294967295, 4294967295, 378, 1454, 823, 4294967295, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1902, 1904, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 160, 4294967295, 4294967295, 4294967295, 1912, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1927, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 1069, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 473, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 196, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 1967, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 1587, 4294967295, 4294967295,
    193, 4294967295, 4294967295, 4294967295, 4294967295, 286, 286, 4294967295, 1589, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 378, 4294967295, 4294967295, 184, 4294967295, 174, 4294967295,
    4294967295, 204, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 184,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 23, 4294967295,
    4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 193, 4294967295, 4294967295, 184, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 196, 4294967295, 4294967295, 23, 4294967295, 4294967295, 23, 4294967295, 4294967295, 23, 4294967295, 4294967295, 23,
    4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2005, 4294967295, 4294967295, 167, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 2007, 4294967295, 2009, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1528, 4294967295, 4294967295, 4294967295, 4294967295, 2011, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1528, 4294967295, 4294967295, 4294967295, 2013, 2015, 2017,
    1330, 2019, 2021, 1462, 2023, 2023, 142, 1522, 2025, 2027, 1533, 1528, 1530, 1069, 4294967295, 4294967295,
    4294967295, 4294967295, 2011, 4294967295, 4294967295, 4294967295, 4294967295, 1572, 4294967295, 4294967295, 1572, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 125, 4294967295, 4294967295, 4294967295, 4294967295, 125, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2036, 4294967295, 4294967295, 4294967295,
    378, 125, 225, 4294967295, 4294967295, 4294967295, 390, 378, 300, 4294967295, 4294967295, 823, 4294967295, 4294967295, 4294967295, 378,
    225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 390, 378, 300, 4294967295, 4294967295, 823, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1528, 4294967295, 142,
    1522, 2023, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 196,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1441, 4294967295, 4294967295, 4294967295, 160, 1443, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 23, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1259, 125, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1912, 1401, 2080, 2082, 2084, 2086, 2088, 2090, 2092, 2094, 2096, 2098, 2100, 2102, 2104,
    2106, 2108, 2110, 2112, 2114, 1405, 1403, 2116, 2118, 2120, 2122, 2124, 2126, 2128, 2130, 2132,
    2134, 2136, 2138, 2140, 2142, 2144, 2146, 2148, 2150, 2152, 2154, 2156, 2158, 2160, 2162, 2164,
    2166, 2168, 2170, 2172, 2174, 2176, 2178, 2180, 2182, 2184, 2182, 2187, 2114, 2189, 2191, 2132,
    2194, 2196, 2198, 2200, 2202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 671, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 167, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 155, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295,
    4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2216, 4294967295, 4294967295,
    4294967295, 2216, 4294967295, 4294967295, 4294967295, 4294967295, 364, 4294967295, 4294967295, 2219, 2216, 4294967295, 4294967295, 4294967295, 300, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 473, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1067, 4294967295, 4294967295, 390, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 1912, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1217, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 165,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1259, 1725, 4294967295, 1154, 1367, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2013, 2015, 2017,
    1330, 2019, 2021, 1462, 2023, 1528, 1530, 1069, 142, 1522, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2011, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1528, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 165, 167, 4294967295, 1791, 2017, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 23, 1330, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 300, 167, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 300, 4294967295, 167, 142, 1522, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1150, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 143, 4294967295, 143, 300, 161, 161,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1533, 4294967295, 4294967295, 4294967295, 2340, 4294967295, 1451, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2344, 142, 2346,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 143, 4294967295, 2352, 4294967295, 161,
    4294967295, 4294967295, 4294967295, 4294967295, 143, 4294967295, 2352, 4294967295, 161, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 142, 4294967295, 4294967295, 5, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 66, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 118, 120, 4294967295, 22, 4294967295,
    124, 126, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 140, 4294967295,
    144, 146, 4294967295, 4294967295, 4294967295, 4294967295, 151, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 159,
    162, 4294967295, 4294967295, 164, 166, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 173, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295,
    4294967295, 195, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295,
    4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 257, 4294967295, 263, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 257, 4294967295, 263, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295,
    285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 4294967295, 22,
    4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 22, 4294967295, 4294967295, 4294967295, 22, 4294967295,
    4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 22, 4294967295,
    4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 195, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295,
    22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 183, 173, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 159, 4294967295, 159, 4294967295,
    4294967295, 4294967295, 4294967295, 22, 4294967295, 363, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    377, 159, 4294967295, 4294967295, 381, 4294967295, 386, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295,
    377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295, 4294967295, 381, 4294967295, 4294967295, 4294967295, 299, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 421, 4294967295,
    4294967295, 421, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    433, 4294967295, 4294967295, 436, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 421, 4294967295, 4294967295, 421, 4294967295, 4294967295,
    4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 472, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 164, 4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 377, 4294967295, 377, 4294967295, 4294967295, 4294967295, 164,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 173, 4294967295, 518, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295,
    195, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    315, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 164, 4294967295, 4294967295, 524, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 377, 4294967295, 4294967295, 4294967295, 164,
    4294967295, 4294967295, 4294967295, 195, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 220,
    4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 22,
    4294967295, 4294967295, 4294967295, 22, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    192, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    183, 173, 4294967295, 533, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 195, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 537, 4294967295,
    4294967295, 4294967295, 539, 4294967295, 4294967295, 4294967295, 541, 4294967295, 4294967295, 4294967295, 541, 4294967295, 4294967295, 4294967295, 541, 4294967295,
    4294967295, 4294967295, 537, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 22,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295,
    4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 195, 4294967295, 4294967295,
    4294967295, 4294967295, 195, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 159, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 578, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    377, 4294967295, 4294967295, 4294967295, 4294967295, 578, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 578, 4294967295, 4294967295, 4294967295, 195, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295,
    4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 612, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 612, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 195, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 616, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 537, 4294967295, 4294967295, 539, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 539, 4294967295, 4294967295, 4294967295, 4294967295, 539, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295,
    4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 616, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 159, 4294967295, 4294967295, 381, 4294967295, 386, 4294967295, 4294967295,
    4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295,
    4294967295, 381, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 299, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 421, 4294967295, 4294967295, 421, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 433, 4294967295, 4294967295, 436, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 421, 4294967295, 4294967295, 421, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 472,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 626, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 633, 4294967295, 4294967295,
    4294967295, 377, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295,
    4294967295, 164, 4294967295, 4294967295, 4294967295, 195, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 641, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295, 4294967295, 22,
    4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    22, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295,
    4294967295, 22, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 22, 4294967295, 22, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 651, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 651,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 159, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295,
    195, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 159, 4294967295, 4294967295, 381, 4294967295, 4294967295, 4294967295, 4294967295, 377,
    4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295, 4294967295, 381, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295,
    4294967295, 4294967295, 377, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    389, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 377, 4294967295, 377, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 195, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295,
    377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 684, 685, 4294967295, 687, 688,
    4294967295, 4294967295, 4294967295, 4294967295, 691, 4294967295, 299, 4294967295, 4294967295, 691, 4294967295, 299, 4294967295, 4294967295, 691, 4294967295,
    299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 146, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 718, 389, 377, 299, 4294967295,
    159, 4294967295, 4294967295, 4294967295, 720, 723, 724, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 672, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 377, 761, 4294967295, 762, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 151, 4294967295, 4294967295, 774, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 377, 4294967295, 4294967295, 377, 4294967295, 377, 4294967295, 377, 4294967295,
    4294967295, 4294967295, 4294967295, 389, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 786, 4294967295, 788,
    4294967295, 4294967295, 794, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 788, 4294967295, 4294967295, 794, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    164, 4294967295, 4294967295, 472, 4294967295, 472, 4294967295, 164, 4294967295, 4294967295, 4294967295, 717, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 822, 4294967295, 389, 4294967295, 4294967295, 4294967295, 4294967295,
    22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 822,
    4294967295, 389, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 539, 4294967295,
    4294967295, 4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 539, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295,
    386, 4294967295, 4294967295, 381, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 22, 4294967295,
    4294967295, 377, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 203, 4294967295, 4294967295, 315, 4294967295,
    4294967295, 183, 4294967295, 173, 4294967295, 4294967295, 472, 4294967295, 4294967295, 195, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 22, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 851, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 224, 856, 857, 858, 859, 860, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 862, 863, 864, 865, 4294967295, 4294967295, 4294967295, 869, 871, 4294967295, 873, 874, 4294967295,
    4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 224, 4294967295, 299, 4294967295,
    4294967295, 4294967295, 4294967295, 377, 224, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 377, 224,
    4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 299, 389,
    4294967295, 4294967295, 381, 889, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 903, 903, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295,
    4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 903, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 903, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 929, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 173,
    4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295, 159, 4294967295, 4294967295, 4294967295, 4294967295, 183, 173, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295,
    224, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 224, 4294967295,
    4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295,
    224, 22, 4294967295, 4294967295, 224, 22, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224,
    4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 224, 4294967295, 220, 4294967295,
    4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295,
    22, 4294967295, 4294967295, 159, 4294967295, 4294967295, 381, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 195, 4294967295,
    4294967295, 974, 4294967295, 4294967295, 974, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 377, 159, 4294967295, 4294967295, 4294967295,
    381, 4294967295, 983, 984, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 987, 4294967295, 4294967295, 4294967295, 4294967295,
    164, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 983, 984,
    4294967295, 4294967295, 4294967295, 4294967295, 641, 4294967295, 4294967295, 4294967295, 987, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    183, 173, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295, 377, 4294967295, 159, 992,
    4294967295, 4294967295, 159, 159, 996, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 22, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    22, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 22, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295,
    224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 183, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 22, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295,
    4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 183, 173, 996, 4294967295, 4294967295, 4294967295, 537, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 541, 4294967295, 4294967295, 539, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295,
    192, 4294967295, 4294967295, 192, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    195, 4294967295, 192, 195, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 578, 4294967295, 1022, 4294967295,
    4294967295, 4294967295, 4294967295, 987, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 974, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 22, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295,
    4294967295, 377, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 612, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377,
    299, 4294967295, 164, 4294967295, 4294967295, 4294967295, 616, 4294967295, 4294967295, 4294967295, 4294967295, 537, 4294967295, 4294967295, 539, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 377, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 1066, 4294967295, 4294967295, 4294967295, 4294967295, 389, 1068,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 626, 433, 4294967295, 4294967295, 164, 4294967295, 1074, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 386, 433, 4294967295, 4294967295, 4294967295, 164, 4294967295,
    4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 386, 433, 4294967295, 4294967295, 4294967295, 4294967295, 641, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 389, 4294967295,
    4294967295, 4294967295, 381, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 421, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 224, 22, 4294967295, 4294967295, 224, 22, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 651, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 22, 159, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 224, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 987, 4294967295, 4294967295, 4294967295,
    4294967295, 192, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1107, 1108, 4294967295, 1109, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1114, 1151, 1153, 1155, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1216, 4294967295, 4294967295, 4294967295, 4294967295, 1216, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1216, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1216, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1216, 4294967295, 4294967295, 1216, 4294967295, 4294967295, 1256, 1258, 1260, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1151, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295,
    4294967295, 1321, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 166, 4294967295, 4294967295, 4294967295, 4294967295,
    1325, 166, 1327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 1329, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    299, 166, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1353, 4294967295, 4294967295,
    4294967295, 1353, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295,
    173, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295, 159, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 173, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 472, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 22, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 173, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 537, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 195, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 578, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 612, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 616, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 651, 4294967295, 164, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1153, 1366, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295,
    299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1398, 4294967295, 4294967295, 1400, 1402, 1404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 691, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 691, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1440, 4294967295, 1442, 822, 4294967295, 4294967295, 4294967295, 822, 4294967295,
    4294967295, 4294967295, 389, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 377, 389, 1447, 299, 4294967295,
    1450, 4294967295, 4294967295, 1153, 1155, 4294967295, 4294967295, 299, 4294967295, 159, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1453,
    1447, 4294967295, 4294967295, 4294967295, 4294967295, 929, 4294967295, 718, 1457, 1459, 377, 1461, 4294967295, 4294967295, 4294967295, 164,
    4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1483, 1485, 1487, 4294967295, 1489, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 822, 4294967295, 389,
    4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 299,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 718,
    4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1513, 4294967295, 4294967295,
    1521, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1527, 1529, 1068, 4294967295, 4294967295, 1532,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1325, 1536, 166, 4294967295, 4294967295, 1539, 4294967295, 1327, 4294967295, 4294967295, 4294967295,
    166, 1539, 4294967295, 4294967295, 4294967295, 4294967295, 1327, 4294967295, 4294967295, 4294967295, 4294967295, 1543, 1546, 4294967295, 1513, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 144, 4294967295, 4294967295, 4294967295, 1327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 162, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 1551, 4294967295, 4294967295, 4294967295, 299, 1551, 4294967295,
    299, 4294967295, 4294967295, 762, 4294967295, 299, 4294967295, 159, 4294967295, 4294967295, 299, 4294967295, 4294967295, 299, 4294967295, 4294967295,
    4294967295, 299, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 299, 4294967295, 4294967295, 159, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1558, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 299, 4294967295, 4294967295, 4294967295, 4294967295,
    377, 299, 4294967295, 1565, 1567, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1353, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1400, 4294967295, 4294967295, 1402, 4294967295, 4294967295, 4294967295, 1571, 762, 929, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 761, 4294967295, 4294967295, 4294967295,
    761, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 159, 992, 4294967295, 4294967295, 4294967295, 224, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1577, 1578,
    4294967295, 4294967295, 4294967295, 183, 4294967295, 1577, 1581, 1066, 1582, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 1586, 4294967295, 1588, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 1594, 1596, 299,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1527, 4294967295, 4294967295, 4294967295, 4294967295, 1521, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1594, 1109, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295,
    4294967295, 1521, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1617, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 195, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 987, 4294967295, 195, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 381, 4294967295, 1632, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 421, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 389, 4294967295, 299, 1636, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 539, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 1646, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1650, 1652, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 299, 1646, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1546,
    4294967295, 4294967295, 4294967295, 144, 1681, 4294967295, 1683, 4294967295, 162, 4294967295, 4294967295, 4294967295, 1546, 1681, 4294967295, 4294967295,
    1546, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 786, 4294967295, 4294967295, 299, 162, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    691, 4294967295, 1453, 4294967295, 903, 1724, 1726, 1728, 1466, 1357, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1066,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1769, 1771, 1773, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295,
    1778, 1780, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295,
    1778, 1780, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295,
    4294967295, 1784, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295,
    4294967295, 166, 4294967295, 389, 166, 144, 1683, 162, 1788, 4294967295, 4294967295, 1790, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1816, 4294967295, 4294967295, 4294967295, 1818, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1810, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1066, 1834, 4294967295, 929, 4294967295, 1571, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 762, 4294967295, 4294967295, 929, 4294967295, 1402, 1400, 1404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1852, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1870, 1521, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1532, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1816, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 1453,
    822, 4294967295, 4294967295, 4294967295, 377, 1453, 822, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 1453,
    822, 4294967295, 4294967295, 4294967295, 377, 1453, 822, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1900, 1903, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 159, 4294967295, 4294967295, 4294967295, 1911, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1926, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 195, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 1068, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 472, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 195, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 1966, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 1586, 4294967295, 4294967295,
    192, 4294967295, 4294967295, 4294967295, 4294967295, 285, 285, 4294967295, 1588, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 377, 4294967295, 4294967295, 183, 4294967295, 173, 4294967295,
    4294967295, 203, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 183,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 22, 4294967295,
    4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 192, 4294967295, 4294967295, 183, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 195, 4294967295, 4294967295, 22, 4294967295, 4294967295, 22, 4294967295, 4294967295, 22, 4294967295, 4294967295, 22,
    4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2004, 4294967295, 4294967295, 166, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 2006, 4294967295, 2008, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1527, 4294967295, 4294967295, 4294967295, 4294967295, 2010, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1527, 4294967295, 4294967295, 4294967295, 2012, 2014, 2016,
    1329, 2018, 2020, 1461, 2022, 2022, 140, 1521, 2024, 2026, 1532, 1527, 1529, 1068, 4294967295, 4294967295,
    4294967295, 4294967295, 2010, 4294967295, 4294967295, 4294967295, 4294967295, 1571, 4294967295, 4294967295, 2028, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 124, 4294967295, 4294967295, 4294967295, 4294967295, 124, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2035, 4294967295, 4294967295, 4294967295,
    377, 124, 224, 4294967295, 4294967295, 4294967295, 389, 377, 299, 4294967295, 4294967295, 822, 4294967295, 4294967295, 4294967295, 377,
    224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 377, 299, 4294967295, 4294967295, 822, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1527, 4294967295, 140,
    1521, 2022, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 195,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1440, 4294967295, 4294967295, 4294967295, 159, 1442, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 22, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1258, 124, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1911, 1400, 2079, 2081, 2083, 2085, 2087, 2089, 2091, 2093, 2095, 2097, 2099, 2101, 2103,
    2105, 2107, 2109, 2111, 2113, 1404, 1402, 2115, 2117, 2119, 2121, 2123, 2125, 2127, 2129, 2131,
    2133, 2135, 2137, 2139, 2141, 2143, 2145, 2147, 2149, 2151, 2153, 2155, 2157, 2159, 2161, 2163,
    2165, 2167, 2169, 2171, 2173, 2175, 2177, 2179, 2181, 2183, 2185, 2186, 2113, 2188, 2190, 2192,
    2193, 2195, 2197, 2199, 2201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 166, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 154, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295,
    4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2215, 4294967295, 4294967295,
    4294967295, 2215, 4294967295, 4294967295, 4294967295, 4294967295, 363, 4294967295, 4294967295, 2218, 2215, 4294967295, 4294967295, 4294967295, 299, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 472, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1066, 4294967295, 4294967295, 389, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 1911, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1216, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 164,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1258, 1724, 4294967295, 1153, 2262, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2012, 2014, 2016,
    1329, 2018, 2020, 1461, 2022, 1527, 1529, 1068, 140, 1521, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2010, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1527, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 164, 166, 4294967295, 1790, 2016, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 22, 1329, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 299, 166, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 299, 4294967295, 166, 140, 1521, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1114, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 144, 4294967295, 786, 299, 162, 162,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1532, 4294967295, 4294967295, 4294967295, 2341, 4294967295, 2342, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2343, 140, 2345,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 144, 4294967295, 2351, 4294967295, 162,
    4294967295, 4294967295, 4294967295, 4294967295, 144, 4294967295, 2351, 4294967295, 162, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 140, 4294967295, 4294967295, 1893, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (6033, 2),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (14, 0, 2),
    (29, 2, 2),
    (40, 4, 2),
    (58, 6, 2),
    (107, 8, 2),
    (108, 10, 2),
    (110, 12, 2),
    (112, 14, 2),
    (113, 16, 2),
    (126, 18, 2),
    (128, 20, 4),
    (129, 24, 2),
    (134, 26, 2),
    (143, 28, 2),
    (144, 30, 5),
    (147, 35, 3),
    (148, 38, 2),
    (155, 40, 2),
    (158, 42, 2),
    (165, 44, 2),
    (173, 46, 3),
    (177, 49, 3),
    (182, 52, 2),
    (188, 54, 3),
    (206, 57, 3),
    (211, 60, 2),
    (218, 62, 3),
    (226, 65, 2),
    (242, 67, 2),
    (244, 69, 2),
    (247, 71, 2),
    (276, 73, 2),
    (278, 75, 2),
    (281, 77, 2),
    (301, 79, 3),
    (304, 82, 3),
    (313, 85, 2),
    (315, 87, 2),
    (326, 89, 2),
    (330, 91, 3),
    (335, 94, 2),
    (340, 96, 2),
    (354, 98, 2),
    (361, 100, 2),
    (369, 102, 2),
    (376, 104, 2),
    (378, 106, 2),
    (382, 108, 2),
    (387, 110, 2),
    (394, 112, 2),
    (398, 114, 2),
    (402, 116, 3),
    (413, 119, 3),
    (453, 122, 2),
    (461, 124, 3),
    (464, 127, 2),
    (472, 129, 2),
    (474, 131, 2),
    (482, 133, 2),
    (483, 135, 2),
    (488, 137, 2),
    (492, 139, 2),
    (494, 141, 2),
    (499, 143, 2),
    (501, 145, 2),
    (512, 147, 3),
    (513, 150, 2),
    (516, 152, 3),
    (518, 155, 2),
    (523, 157, 3),
    (528, 160, 3),
    (535, 163, 2),
    (538, 165, 3),
    (542, 168, 2),
    (549, 170, 2),
    (551, 172, 2),
    (577, 174, 2),
    (590, 176, 2),
    (606, 178, 2),
    (609, 180, 2),
    (624, 182, 2),
    (627, 184, 2),
    (706, 186, 3),
    (714, 189, 2),
    (717, 191, 2),
    (721, 193, 3),
    (744, 196, 3),
    (751, 199, 3),
    (769, 202, 3),
    (773, 205, 3),
    (777, 208, 3),
    (779, 211, 3),
    (783, 214, 3),
    (834, 217, 2),
    (853, 219, 2),
    (854, 221, 2),
    (856, 223, 2),
    (860, 225, 2),
    (864, 227, 3),
    (880, 230, 3),
    (884, 233, 2),
    (893, 235, 3),
    (905, 238, 2),
    (939, 240, 2),
    (950, 242, 2),
    (961, 244, 3),
    (964, 247, 2),
    (969, 249, 3),
    (971, 252, 3),
    (975, 255, 3),
    (979, 258, 3),
    (981, 261, 3),
    (986, 264, 3),
    (991, 267, 3),
    (996, 270, 2),
    (999, 272, 3),
    (1019, 275, 2),
    (1023, 277, 2),
    (1027, 279, 2),
    (1029, 281, 3),
    (1034, 284, 2),
    (1040, 286, 3),
    (1048, 289, 2),
    (1053, 291, 2),
    (1069, 293, 2),
    (1088, 295, 2),
    (1089, 297, 2),
    (1091, 299, 2),
    (1095, 301, 2),
    (1099, 303, 3),
    (1107, 306, 2),
    (1130, 308, 3),
    (1134, 311, 3),
    (1138, 314, 2),
    (1142, 316, 2),
    (1146, 318, 2),
    (1150, 320, 2),
    (1154, 322, 3),
    (1162, 325, 3),
    (1179, 328, 2),
    (1183, 330, 2),
    (1195, 332, 3),
    (1234, 335, 3),
    (1238, 338, 3),
    (1245, 341, 3),
    (1249, 344, 3),
    (1258, 347, 2),
    (1277, 349, 3),
    (1282, 352, 3),
    (1286, 355, 3),
    (1291, 358, 2),
    (1297, 360, 2),
    (1322, 362, 3),
    (1344, 365, 3),
    (1349, 368, 3),
    (1352, 371, 2),
    (1363, 373, 3),
    (1367, 376, 3),
    (1374, 379, 2),
    (1378, 381, 2),
    (1385, 383, 2),
    (1394, 385, 3),
    (1409, 388, 3),
    (1415, 391, 3),
    (1422, 394, 3),
    (1434, 397, 3),
    (1437, 400, 2),
    (1443, 402, 2),
    (1448, 404, 2),
    (1454, 406, 2),
    (1459, 408, 2),
    (1466, 410, 3),
    (1476, 413, 2),
    (1495, 415, 3),
    (1496, 418, 2),
    (1499, 420, 3),
    (1501, 423, 2),
    (1506, 425, 3),
    (1511, 428, 3),
    (1518, 431, 2),
    (1521, 433, 3),
    (1525, 436, 2),
    (1532, 438, 2),
    (1534, 440, 2),
    (1560, 442, 2),
    (1573, 444, 2),
    (1589, 446, 2),
    (1592, 448, 2),
    (1607, 450, 2),
    (1610, 452, 2),
    (1689, 454, 3),
    (1697, 457, 2),
    (1700, 459, 2),
    (1704, 461, 3),
    (1727, 464, 3),
    (1738, 467, 2),
    (1747, 469, 2),
    (1753, 471, 3),
    (1761, 474, 3),
    (1769, 477, 3),
    (1773, 480, 2),
    (1777, 482, 3),
    (1779, 485, 3),
    (1788, 488, 3),
    (1793, 491, 3),
    (1797, 494, 3),
    (1806, 497, 2),
    (1818, 499, 3),
    (1823, 502, 2),
    (1826, 504, 3),
    (1833, 507, 2),
    (1840, 509, 2),
    (1843, 511, 2),
    (1849, 513, 2),
    (1852, 515, 2),
    (1857, 517, 2),
    (1859, 519, 2),
    (1865, 521, 2),
    (1867, 523, 2),
    (1869, 525, 2),
    (1876, 527, 2),
    (1880, 529, 2),
    (1885, 531, 2),
    (1892, 533, 2),
    (1896, 535, 3),
    (1898, 538, 3),
    (1903, 541, 3),
    (1909, 544, 3),
    (1913, 547, 2),
    (1922, 549, 2),
    (1932, 551, 2),
    (1936, 553, 3),
    (1942, 556, 3),
    (1943, 559, 2),
    (1946, 561, 3),
    (1951, 564, 3),
    (1956, 567, 2),
    (1959, 569, 3),
    (1965, 572, 3),
    (1970, 575, 3),
    (1972, 578, 3),
    (1988, 581, 3),
    (1993, 584, 3),
    (2000, 587, 2),
    (2002, 589, 2),
    (2010, 591, 2),
    (2017, 593, 3),
    (2019, 596, 3),
    (2023, 599, 3),
    (2027, 602, 3),
    (2035, 605, 2),
    (2061, 607, 3),
    (2064, 610, 3),
    (2075, 613, 2),
    (2076, 615, 2),
    (2078, 617, 2),
    (2079, 619, 2),
    (2084, 621, 2),
    (2086, 623, 2),
    (2089, 625, 2),
    (2091, 627, 2),
    (2094, 629, 2),
    (2096, 631, 2),
    (2106, 633, 2),
    (2108, 635, 2),
    (2139, 637, 2),
    (2140, 639, 2),
    (2141, 641, 3),
    (2142, 644, 2),
    (2144, 646, 2),
    (2148, 648, 2),
    (2149, 650, 2),
    (2150, 652, 1),
    (2164, 653, 3),
    (2194, 656, 3),
    (2195, 659, 2),
    (2197, 661, 2),
    (2211, 663, 2),
    (2214, 665, 2),
    (2229, 667, 3),
    (2231, 670, 3),
    (2234, 673, 3),
    (2236, 676, 3),
    (2238, 679, 3),
    (2243, 682, 2),
    (2253, 684, 4),
    (2255, 688, 2),
    (2258, 690, 2),
    (2263, 692, 2),
    (2265, 694, 2),
    (2268, 696, 2),
    (2297, 698, 2),
    (2304, 700, 3),
    (2307, 703, 3),
    (2309, 706, 3),
    (2311, 709, 3),
    (2315, 712, 2),
    (2329, 714, 2),
    (2331, 716, 2),
    (2336, 718, 2),
    (2344, 720, 2),
    (2351, 722, 2),
    (2353, 724, 2),
    (2355, 726, 2),
    (2360, 728, 2),
    (2373, 730, 2),
    (2382, 732, 2),
    (2387, 734, 3),
    (2393, 737, 2),
    (2398, 739, 3),
    (2414, 742, 2),
    (2416, 744, 2),
    (2419, 746, 3),
    (2449, 749, 3),
    (2471, 752, 3),
    (2476, 755, 2),
    (2478, 757, 2),
    (2481, 759, 3),
    (2484, 762, 2),
    (2488, 764, 2),
    (2491, 766, 3),
    (2494, 769, 3),
    (2497, 772, 2),
    (2499, 774, 2),
    (2502, 776, 3),
    (2505, 779, 3),
    (2555, 782, 2),
    (2558, 784, 2),
    (2564, 786, 2),
    (2573, 788, 3),
    (2579, 791, 2),
    (2580, 793, 2),
    (2581, 795, 2),
    (2582, 797, 2),
    (2583, 799, 2),
    (2584, 801, 2),
    (2595, 803, 2),
    (2596, 805, 2),
    (2597, 807, 2),
    (2598, 809, 2),
    (2602, 811, 2),
    (2603, 813, 3),
    (2605, 816, 2),
    (2606, 818, 2),
    (2609, 820, 2),
    (2635, 822, 3),
    (2636, 825, 2),
    (2638, 827, 2),
    (2643, 829, 3),
    (2644, 832, 2),
    (2649, 834, 2),
    (2654, 836, 3),
    (2655, 839, 2),
    (2660, 841, 2),
    (2668, 843, 3),
    (2670, 846, 2),
    (2671, 848, 2),
    (2674, 850, 3),
    (2675, 853, 3),
    (2678, 856, 2),
    (2695, 858, 2),
    (2711, 860, 3),
    (2714, 863, 3),
    (2723, 866, 2),
    (2724, 868, 2),
    (2758, 870, 3),
    (2764, 873, 3),
    (2769, 876, 3),
    (2775, 879, 2),
    (2780, 881, 2),
    (2813, 883, 3),
    (2829, 886, 2),
    (2863, 888, 3),
    (2869, 891, 2),
    (2872, 893, 2),
    (2892, 895, 2),
    (2895, 897, 2),
    (2900, 899, 2),
    (2902, 901, 2),
    (2907, 903, 2),
    (2908, 905, 2),
    (2926, 907, 2),
    (2932, 909, 2),
    (2945, 911, 2),
    (2952, 913, 2),
    (2956, 915, 2),
    (2960, 917, 2),
    (2964, 919, 2),
    (2970, 921, 2),
    (2974, 923, 2),
    (2978, 925, 2),
    (2983, 927, 2),
    (3003, 929, 3),
    (3008, 932, 2),
    (3009, 934, 2),
    (3012, 936, 2),
    (3013, 938, 2),
    (3016, 940, 2),
    (3023, 942, 2),
    (3028, 944, 2),
    (3032, 946, 2),
    (3036, 948, 2),
    (3038, 950, 3),
    (3042, 953, 2),
    (3047, 955, 2),
    (3051, 957, 2),
    (3056, 959, 2),
    (3059, 961, 2),
    (3062, 963, 3),
    (3065, 966, 2),
    (3070, 968, 2),
    (3077, 970, 2),
    (3082, 972, 3),
    (3086, 975, 3),
    (3089, 978, 2),
    (3092, 980, 2),
    (3110, 982, 3),
    (3115, 985, 3),
    (3116, 988, 2),
    (3120, 990, 3),
    (3122, 993, 2),
    (3123, 995, 2),
    (3128, 997, 3),
    (3131, 1000, 3),
    (3136, 1003, 3),
    (3140, 1006, 3),
    (3145, 1009, 3),
    (3150, 1012, 2),
    (3151, 1014, 2),
    (3156, 1016, 2),
    (3160, 1018, 3),
    (3168, 1021, 2),
    (3169, 1023, 2),
    (3178, 1025, 2),
    (3180, 1027, 3),
    (3182, 1030, 2),
    (3183, 1032, 2),
    (3186, 1034, 2),
    (3187, 1036, 2),
    (3188, 1038, 2),
    (3194, 1040, 2),
    (3197, 1042, 2),
    (3209, 1044, 2),
    (3221, 1046, 2),
    (3224, 1048, 3),
    (3232, 1051, 2),
    (3235, 1053, 3),
    (3244, 1056, 2),
    (3251, 1058, 2),
    (3271, 1060, 2),
    (3272, 1062, 2),
    (3275, 1064, 2),
    (3280, 1066, 2),
    (3290, 1068, 2),
    (3291, 1070, 2),
    (3297, 1072, 2),
    (3299, 1074, 3),
    (3317, 1077, 2),
    (3325, 1079, 2),
    (3330, 1081, 2),
    (3335, 1083, 2),
    (3336, 1085, 2),
    (3337, 1087, 2),
    (3341, 1089, 3),
    (3350, 1092, 2),
    (3353, 1094, 2),
    (3359, 1096, 3),
    (3401, 1099, 3),
    (3405, 1102, 3),
    (3408, 1105, 3),
    (3411, 1108, 3),
    (3414, 1111, 3),
    (3424, 1114, 3),
    (3426, 1117, 3),
    (3427, 1120, 3),
    (3432, 1123, 2),
    (3438, 1125, 2),
    (3478, 1127, 3),
    (3484, 1130, 3),
    (3486, 1133, 2),
    (3491, 1135, 3),
    (3497, 1138, 2),
    (3499, 1140, 2),
    (3505, 1142, 2),
    (3508, 1144, 2),
    (3515, 1146, 3),
    (3533, 1149, 2),
    (3537, 1151, 3),
    (3541, 1154, 2),
    (3574, 1156, 2),
    (3601, 1158, 2),
    (3607, 1160, 2),
    (3631, 1162, 2),
    (3668, 1164, 3),
    (3679, 1167, 3),
    (3680, 1170, 2),
    (3682, 1172, 3),
    (3686, 1175, 3),
    (3691, 1178, 3),
    (3694, 1181, 2),
    (3706, 1183, 3),
    (3710, 1186, 3),
    (3722, 1189, 2),
    (3742, 1191, 3),
    (3752, 1194, 3),
    (3753, 1197, 2),
    (3758, 1199, 2),
    (3759, 1201, 2),
    (3771, 1203, 3),
    (3777, 1206, 2),
    (3778, 1208, 2),
    (3781, 1210, 3),
    (3783, 1213, 2),
    (3788, 1215, 3),
    (3796, 1218, 3),
    (3801, 1221, 2),
    (3802, 1223, 2),
    (3806, 1225, 3),
    (3811, 1228, 3),
    (3816, 1231, 2),
    (3817, 1233, 2),
    (3822, 1235, 2),
    (3837, 1237, 3),
    (3838, 1240, 2),
    (3842, 1242, 3),
    (3850, 1245, 2),
    (3867, 1247, 3),
    (3873, 1250, 2),
    (3878, 1252, 2),
    (3879, 1254, 2),
    (3882, 1256, 2),
    (3883, 1258, 2),
    (3889, 1260, 3),
    (3906, 1263, 3),
    (3911, 1266, 3),
    (3915, 1269, 3),
    (3936, 1272, 3),
    (3943, 1275, 3),
    (3953, 1278, 2),
    (3959, 1280, 2),
    (3963, 1282, 2),
    (3970, 1284, 2),
    (3971, 1286, 2),
    (3980, 1288, 2),
    (3981, 1290, 2),
    (3996, 1292, 3),
    (4001, 1295, 3),
    (4009, 1298, 2),
    (4012, 1300, 3),
    (4025, 1303, 2),
    (4026, 1305, 2),
    (4028, 1307, 2),
    (4055, 1309, 2),
    (4056, 1311, 2),
    (4057, 1313, 2),
    (4058, 1315, 2),
    (4083, 1317, 2),
    (4088, 1319, 2),
    (4103, 1321, 2),
    (4129, 1323, 2),
    (4144, 1325, 2),
    (4147, 1327, 2),
    (4150, 1329, 2),
    (4151, 1331, 2),
    (4152, 1333, 2),
    (4164, 1335, 2),
    (4195, 1337, 2),
    (4205, 1339, 2),
    (4215, 1341, 2),
    (4219, 1343, 2),
    (4237, 1345, 2),
    (4241, 1347, 2),
    (4249, 1349, 3),
    (4251, 1352, 2),
    (4256, 1354, 2),
    (4257, 1356, 2),
    (4258, 1358, 2),
    (4264, 1360, 2),
    (4265, 1362, 2),
    (4272, 1364, 2),
    (4273, 1366, 2),
    (4287, 1368, 2),
    (4317, 1370, 2),
    (4321, 1372, 2),
    (4334, 1374, 2),
    (4336, 1376, 2),
    (4348, 1378, 2),
    (4350, 1380, 2),
    (4356, 1382, 2),
    (4362, 1384, 3),
    (4372, 1387, 2),
    (4378, 1389, 3),
    (4386, 1392, 3),
    (4394, 1395, 2),
    (4395, 1397, 2),
    (4403, 1399, 3),
    (4412, 1402, 3),
    (4418, 1405, 2),
    (4428, 1407, 3),
    (4434, 1410, 2),
    (4436, 1412, 3),
    (4444, 1415, 2),
    (4445, 1417, 2),
    (4454, 1419, 3),
    (4465, 1422, 3),
    (4472, 1425, 3),
    (4497, 1428, 2),
    (4508, 1430, 3),
    (4516, 1433, 3),
    (4526, 1436, 3),
    (4534, 1439, 2),
    (4540, 1441, 3),
    (4542, 1444, 3),
    (4549, 1447, 2),
    (4555, 1449, 3),
    (4561, 1452, 3),
    (4570, 1455, 2),
    (4576, 1457, 2),
    (4577, 1459, 2),
    (4598, 1461, 2),
    (4605, 1463, 2),
    (4608, 1465, 2),
    (4644, 1467, 2),
    (4647, 1469, 2),
    (4648, 1471, 2),
    (4649, 1473, 2),
    (4681, 1475, 2),
    (4698, 1477, 2),
    (4711, 1479, 2),
    (4713, 1481, 2),
    (4714, 1483, 2),
    (4718, 1485, 2),
    (4722, 1487, 2),
    (4726, 1489, 2),
    (4731, 1491, 3),
    (4732, 1494, 2),
    (4733, 1496, 2),
    (4734, 1498, 2),
    (4736, 1500, 3),
    (4739, 1503, 2),
    (4740, 1505, 2),
    (4743, 1507, 2),
    (4745, 1509, 2),
    (4751, 1511, 2),
    (4752, 1513, 2),
    (4757, 1515, 2),
    (4759, 1517, 2),
    (4760, 1519, 2),
    (4761, 1521, 2),
    (4762, 1523, 3),
    (4763, 1526, 2),
    (4767, 1528, 3),
    (4772, 1531, 3),
    (4774, 1534, 3),
    (4790, 1537, 2),
    (4810, 1539, 2),
    (4819, 1541, 2),
    (4820, 1543, 2),
    (4821, 1545, 2),
    (4823, 1547, 2),
    (4832, 1549, 2),
    (4856, 1551, 3),
    (4878, 1554, 2),
    (4886, 1556, 2),
    (4893, 1558, 2),
    (4895, 1560, 2),
    (4897, 1562, 2),
    (4902, 1564, 2),
    (4907, 1566, 2),
    (4911, 1568, 2),
    (4927, 1570, 2),
    (4929, 1572, 3),
    (4957, 1575, 2),
    (4960, 1577, 2),
    (4970, 1579, 2),
    (4971, 1581, 2),
    (4972, 1583, 2),
    (4975, 1585, 2),
    (4981, 1587, 2),
    (4982, 1589, 2),
    (4983, 1595, 2),
    (4986, 1591, 2),
    (4988, 1593, 2),
    (4992, 1597, 2),
    (4993, 1599, 2),
    (4998, 1601, 2),
    (5003, 1603, 2),
    (5004, 1605, 2),
    (5006, 1607, 2),
    (5017, 1609, 2),
    (5032, 1611, 2),
    (5033, 1613, 2),
    (5037, 1615, 2),
    (5038, 1617, 2),
    (5040, 1619, 2),
    (5043, 1621, 2),
    (5045, 1623, 2),
    (5047, 1625, 2),
    (5050, 1627, 2),
    (5053, 1629, 2),
    (5057, 1631, 2),
    (5061, 1633, 2),
    (5064, 1635, 2),
    (5067, 1637, 2),
    (5074, 1639, 2),
    (5082, 1641, 3),
    (5083, 1644, 2),
    (5088, 1646, 3),
    (5089, 1649, 2),
    (5091, 1651, 2),
    (5092, 1653, 2),
    (5095, 1655, 2),
    (5099, 1657, 2),
    (5105, 1659, 2),
    (5160, 1661, 2),
    (5178, 1663, 2),
    (5184, 1665, 2),
    (5187, 1667, 2),
    (5191, 1669, 2),
    (5192, 1671, 2),
    (5193, 1673, 2),
    (5197, 1675, 2),
    (5212, 1677, 2),
    (5216, 1679, 2),
    (5224, 1681, 2),
    (5233, 1683, 2),
    (5234, 1685, 2),
    (5238, 1687, 2),
    (5240, 1689, 2),
    (5246, 1691, 2),
    (5247, 1693, 2),
    (5251, 1701, 2),
    (5253, 1695, 2),
    (5254, 1697, 2),
    (5255, 1699, 2),
    (5256, 1703, 2),
    (5273, 1705, 2),
    (5275, 1707, 2),
    (5277, 1709, 2),
    (5291, 1711, 3),
    (5293, 1714, 2),
    (5294, 1716, 2),
    (5295, 1718, 2),
    (5304, 1720, 2),
    (5309, 1722, 2),
    (5318, 1724, 3),
    (5320, 1727, 2),
    (5330, 1729, 2),
    (5331, 1731, 2),
    (5340, 1733, 2),
    (5345, 1735, 2),
    (5368, 1737, 3),
    (5397, 1740, 3),
    (5409, 1743, 3),
    (5411, 1746, 3),
    (5418, 1749, 2),
    (5429, 1751, 2),
    (5442, 1753, 3),
    (5444, 1756, 3),
    (5450, 1759, 2),
    (5461, 1761, 3),
    (5462, 1764, 2),
    (5464, 1766, 2),
    (5465, 1768, 2),
    (5473, 1770, 2),
    (5478, 1772, 3),
    (5498, 1775, 3),
    (5499, 1778, 2),
    (5515, 1780, 2),
    (5516, 1782, 2),
    (5523, 1784, 2),
    (5524, 1786, 2),
    (5571, 1788, 3),
    (5599, 1791, 2),
    (5604, 1793, 2),
    (5606, 1795, 2),
    (5612, 1797, 2),
    (5613, 1799, 2),
    (5616, 1801, 2),
    (5626, 1803, 2),
    (5663, 1805, 4),
    (5731, 1809, 2),
    (5738, 1811, 2),
    (5760, 1813, 2),
    (5762, 1815, 2),
    (5764, 1817, 2),
    (5765, 1819, 2),
    (5766, 1821, 2),
    (5767, 1823, 2),
    (5768, 1825, 5),
    (5769, 1830, 3),
    (5787, 1833, 3),
    (5836, 1836, 2),
    (5881, 1838, 2),
    (5885, 1840, 2),
    (5903, 1842, 2),
    (5909, 1844, 2),
    (5910, 1846, 2),
    (5911, 1848, 2),
    (5917, 1850, 2),
    (5920, 1852, 2),
    (5921, 1854, 2),
    (5933, 1856, 2),
    (5936, 1858, 2),
    (5937, 1860, 2),
    (5950, 1862, 2),
    (5953, 1864, 2),
    (5960, 1866, 3),
    (5964, 1869, 3),
    (5969, 1872, 2),
    (5971, 1874, 2),
    (5972, 1876, 2),
    (5974, 1878, 2),
    (5976, 1880, 2),
    (5979, 1882, 2),
    (6040, 1884, 2),
    (6041, 1886, 2),
    (6048, 1888, 2),
    (6052, 1890, 2),
    (6066, 1892, 3),
    (6086, 1895, 2),
    (6087, 1897, 2),
    (6089, 1899, 2),
    (6091, 1901, 2),
    (6098, 1903, 2),
    (6101, 1905, 2),
    (6103, 1907, 2),
    (6104, 1909, 2),
    (6105, 1911, 2),
    (6163, 1913, 2),
    (6190, 1915, 2),
    (6233, 1917, 3),
    (6235, 1920, 2),
    (6247, 1922, 2),
    (6248, 1924, 2),
    (6260, 1926, 2),
    (6276, 1928, 2),
    (6286, 1930, 3),
    (6287, 1933, 2),
    (6288, 1935, 2),
    (6292, 1937, 3),
    (6293, 1940, 2),
    (6294, 1942, 2),
    (6296, 1944, 3),
    (6302, 1947, 3),
    (6303, 1950, 2),
    (6304, 1952, 2),
    (6308, 1954, 3),
    (6309, 1957, 2),
    (6310, 1959, 2),
    (6312, 1961, 3),
    (6332, 1964, 2),
    (6338, 1966, 2),
    (6345, 1968, 2),
    (6363, 1970, 2),
    (6403, 1972, 2),
    (6404, 1974, 2),
    (6422, 1976, 2),
    (6425, 1978, 2),
    (6429, 1980, 2),
    (6473, 1982, 2),
    (6485, 1984, 3),
    (6499, 1987, 2),
    (6581, 1989, 3),
    (6587, 1992, 3),
    (6594, 1995, 3),
    (6624, 1998, 3),
    (6635, 2001, 3),
    (6636, 2004, 2),
    (6652, 2006, 3),
    (6660, 2009, 3),
    (6664, 2012, 3),
    (6668, 2015, 3),
    (6674, 2018, 3),
    (6681, 2021, 2),
    (6685, 2023, 2),
    (6731, 2025, 2),
    (6733, 2027, 2),
    (6736, 2029, 3),
    (6741, 2032, 3),
    (6742, 2035, 3),
    (6744, 2038, 2),
    (6801, 2040, 2),
    (6806, 2042, 3),
    (6809, 2045, 3),
    (6812, 2048, 2),
    (6814, 2050, 2),
    (6817, 2052, 3),
    (6820, 2055, 2),
    (6824, 2057, 2),
    (6828, 2059, 3),
    (6831, 2062, 2),
    (6842, 2064, 3),
    (6846, 2067, 2),
    (6849, 2069, 2),
    (6855, 2071, 2),
    (6858, 2073, 3),
    (6861, 2076, 2),
    (6867, 2078, 3),
    (6870, 2081, 2),
    (6873, 2083, 2),
    (6876, 2085, 2),
    (6879, 2087, 2),
    (6882, 2089, 3),
    (6889, 2092, 2),
    (6892, 2094, 2),
    (6900, 2096, 2),
    (6904, 2098, 2),
    (6906, 2100, 2),
    (6919, 2102, 2),
    (6924, 2104, 2),
    (6937, 2106, 2),
    (6941, 2108, 2),
    (6942, 2110, 2),
    (6943, 2112, 2),
    (6944, 2114, 2),
    (6945, 2116, 2),
    (6946, 2118, 2),
    (6947, 2120, 2),
    (6948, 2122, 2),
    (6949, 2124, 2),
    (6950, 2126, 2),
    (6951, 2128, 2),
    (6952, 2130, 2),
    (6953, 2132, 2),
    (6954, 2134, 2),
    (6955, 2136, 2),
    (6956, 2138, 2),
    (6957, 2140, 2),
    (6962, 2142, 2),
    (6967, 2144, 2),
    (6970, 2146, 2),
    (6977, 2148, 2),
    (6982, 2150, 2),
    (7020, 2152, 2),
    (7024, 2154, 3),
    (7025, 2157, 2),
    (7026, 2159, 2),
    (7030, 2161, 2),
    (7031, 2163, 3),
    (7032, 2166, 2),
    (7035, 2168, 2),
    (7039, 2170, 3),
    (7040, 2173, 2),
    (7049, 2175, 2),
    (7050, 2177, 3),
    (7051, 2180, 2),
    (7054, 2182, 2),
    (7061, 2184, 2),
    (7069, 2186, 2),
    (7071, 2188, 2),
    (7072, 2190, 2),
    (7073, 2192, 2),
    (7087, 2194, 3),
    (7099, 2197, 3),
    (7111, 2200, 2),
    (7134, 2202, 3),
    (7144, 2205, 2),
    (7148, 2207, 2),
    (7149, 2209, 2),
    (7169, 2211, 2),
    (7175, 2213, 2),
    (7178, 2215, 2),
    (7186, 2217, 2),
    (7194, 2219, 2),
    (7218, 2221, 2),
    (7221, 2223, 2),
    (7228, 2225, 2),
    (7257, 2227, 2),
    (7258, 2229, 2),
    (7297, 2231, 2),
    (7298, 2233, 2),
    (7299, 2235, 2),
    (7300, 2237, 2),
    (7301, 2239, 2),
    (7302, 2241, 2),
    (7303, 2243, 2),
    (7304, 2245, 2),
    (7305, 2247, 2),
    (7306, 2249, 2),
    (7307, 2251, 2),
    (7308, 2253, 2),
    (7309, 2255, 2),
    (7310, 2257, 2),
    (7311, 2259, 2),
    (7312, 2261, 2),
    (7313, 2263, 2),
    (7314, 2265, 2),
    (7315, 2267, 2),
    (7316, 2269, 2),
    (7317, 2271, 2),
    (7318, 2273, 2),
    (7319, 2275, 2),
    (7320, 2277, 2),
    (7321, 2279, 2),
    (7322, 2281, 2),
    (7323, 2283, 2),
    (7324, 2285, 2),
    (7325, 2287, 2),
    (7326, 2289, 2),
    (7327, 2291, 2),
    (7328, 2293, 2),
    (7329, 2295, 2),
    (7330, 2297, 2),
    (7331, 2299, 2),
    (7332, 2301, 2),
    (7333, 2303, 2),
    (7334, 2305, 2),
    (7335, 2307, 2),
    (7336, 2309, 2),
    (7337, 2311, 2),
    (7338, 2313, 2),
    (7339, 2315, 2),
    (7340, 2317, 2),
    (7341, 2319, 2),
    (7342, 2321, 2),
    (7343, 2323, 2),
    (7344, 2325, 2),
    (7345, 2327, 2),
    (7346, 2329, 2),
    (7347, 2331, 2),
    (7348, 2333, 2),
    (7349, 2335, 2),
    (7350, 2337, 2),
    (7351, 2339, 2),
    (7352, 2341, 2),
    (7353, 2343, 2),
    (7354, 2345, 2),
    (7355, 2347, 2),
    (7356, 2349, 2),
    (7357, 2351, 2),
    (7358, 2353, 2),
    (7359, 2355, 2),
    (7360, 2357, 2),
    (7361, 2359, 2),
    (7362, 2361, 2),
    (7363, 2363, 2),
    (7364, 2365, 2),
    (7377, 2367, 2),
    (7385, 2369, 2),
    (7396, 2371, 3),
    (7405, 2374, 2),
    (7410, 2376, 2),
    (7421, 2378, 2),
    (7425, 2380, 2),
    (7430, 2382, 2),
    (7433, 2384, 2),
    (7434, 2386, 2),
    (7438, 2388, 2),
    (7466, 2390, 2),
    (7473, 2392, 3),
    (7488, 2395, 2),
    (7491, 2397, 2),
    (7495, 2399, 3),
    (7499, 2402, 2),
    (7542, 2404, 2),
    (7554, 2406, 2),
    (7613, 2408, 3),
    (7615, 2411, 3),
    (7640, 2414, 2),
    (7641, 2416, 2),
    (7643, 2418, 2),
    (7644, 2420, 2),
    (7677, 2422, 2),
    (7678, 2424, 2),
    (7679, 2426, 2),
    (7680, 2428, 2),
    (7681, 2430, 2),
    (7682, 2432, 2),
    (7683, 2434, 2),
    (7684, 2436, 2),
    (7685, 2438, 2),
    (7686, 2440, 2),
    (7687, 2442, 2),
    (7688, 2444, 2),
    (7689, 2446, 2),
    (7698, 2448, 2),
    (7704, 2450, 2),
    (7713, 2452, 3),
    (7714, 2455, 2),
    (7716, 2457, 2),
    (7717, 2459, 2),
    (7723, 2461, 2),
    (7724, 2463, 2),
    (7731, 2465, 2),
    (7732, 2467, 2),
    (7740, 2469, 3),
    (7783, 2472, 3),
    (7795, 2475, 2),
    (7797, 2477, 2),
    (7798, 2479, 2),
    (7799, 2481, 2),
    (7877, 2483, 2),
    (7929, 2485, 2),
    (7933, 2487, 2),
    (7941, 2489, 2),
    (7958, 2491, 2),
    (7962, 2493, 3),
    (7964, 2496, 3),
    (7981, 2499, 2),
    (7982, 2501, 2),
    (7983, 2503, 2),
    (7997, 2505, 2),
    (8006, 2507, 2),
    (8026, 2509, 2),
    (8029, 2511, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    3, 23, 3, 23, 3, 23, 67, 3, 3, 119, 3, 121, 3, 23, 125, 3,
    3, 127, 3, 142, 3, 143, 145, 4, 148, 3, 3, 152, 3, 160, 3, 161,
    143, 145, 4, 3, 23, 165, 167, 3, 3, 23, 3, 174, 3, 184, 3, 23,
    193, 3, 23, 196, 3, 23, 3, 204, 23, 3, 204, 23, 3, 23, 3, 23,
    221, 3, 225, 262, 3, 264, 3, 266, 3, 262, 3, 264, 3, 266, 3, 3,
    23, 193, 3, 23, 286, 3, 23, 290, 3, 3, 23, 3, 23, 193, 3, 23,
    3, 300, 3, 23, 3, 300, 3, 23, 3, 23, 3, 23, 3, 23, 3, 23,
    3, 23, 3, 23, 3, 316, 23, 3, 23, 196, 3, 23, 3, 23, 165, 3,
    23, 3, 23, 3, 23, 3, 184, 3, 174, 3, 23, 3, 160, 3, 160, 3,
    23, 3, 364, 3, 378, 23, 3, 160, 3, 382, 23, 3, 387, 3, 378, 23,
    3, 378, 23, 3, 160, 3, 382, 23, 3, 300, 3, 390, 3, 300, 417, 3,
    417, 3, 3, 422, 3, 422, 3, 435, 3, 435, 3, 23, 221, 3, 422, 3,
    422, 3, 23, 165, 3, 473, 23, 3, 378, 23, 3, 23, 165, 481, 482, 3,
    3, 378, 23, 3, 378, 23, 3, 23, 165, 3, 225, 3, 184, 3, 174, 519,
    3, 3, 184, 3, 23, 196, 3, 316, 23, 3, 23, 3, 378, 23, 3, 225,
    3, 225, 3, 23, 3, 23, 165, 525, 3, 3, 378, 23, 3, 378, 23, 3,
    23, 165, 3, 23, 196, 3, 23, 165, 3, 316, 23, 3, 23, 221, 3, 23,
    3, 23, 221, 3, 23, 3, 23, 3, 23, 3, 23, 165, 3, 23, 3, 23,
    193, 3, 300, 3, 300, 3, 225, 3, 184, 3, 174, 534, 3, 3, 184, 3,
    23, 196, 3, 225, 3, 316, 23, 3, 23, 538, 3, 540, 3, 542, 3, 542,
    3, 542, 3, 23, 538, 3, 23, 193, 3, 23, 3, 23, 3, 23, 193, 3,
    23, 193, 3, 23, 193, 3, 23, 193, 3, 204, 23, 3, 225, 3, 23, 196,
    3, 23, 196, 3, 23, 193, 3, 23, 3, 160, 3, 23, 579, 3, 378, 23,
    3, 23, 579, 417, 3, 3, 23, 579, 3, 23, 196, 3, 23, 3, 23, 3,
    23, 3, 23, 613, 3, 23, 613, 3, 23, 196, 3, 23, 617, 3, 23, 538,
    3, 540, 3, 540, 3, 540, 3, 300, 3, 23, 3, 23, 617, 3, 225, 3,
    378, 23, 3, 160, 3, 382, 23, 3, 387, 3, 378, 23, 3, 378, 23, 3,
    160, 3, 382, 23, 3, 300, 3, 390, 3, 300, 417, 3, 417, 3, 3, 422,
    3, 422, 3, 435, 3, 435, 3, 23, 221, 3, 422, 3, 422, 3, 23, 165,
    3, 473, 23, 3, 225, 3, 627, 3, 378, 23, 3, 23, 165, 3, 23, 165,
    482, 3, 3, 378, 23, 3, 378, 23, 3, 23, 165, 3, 23, 165, 3, 23,
    196, 3, 642, 3, 23, 221, 3, 23, 3, 23, 221, 3, 23, 3, 23, 3,
    23, 3, 23, 3, 23, 3, 23, 3, 23, 3, 23, 3, 23, 3, 23, 3,
    23, 3, 23, 3, 23, 3, 23, 3, 23, 652, 3, 23, 165, 3, 23, 652,
    3, 316, 23, 3, 160, 3, 23, 3, 23, 3, 23, 196, 3, 378, 23, 3,
    160, 3, 382, 23, 3, 378, 23, 3, 160, 3, 382, 23, 3, 378, 23, 3,
    378, 23, 3, 378, 23, 3, 23, 165, 3, 378, 23, 3, 390, 3, 300, 3,
    23, 3, 378, 23, 3, 378, 23, 3, 23, 165, 3, 23, 196, 3, 225, 3,
    23, 165, 3, 378, 23, 3, 671, 3, 671, 3, 671, 3, 671, 3, 216, 3,
    300, 3, 216, 3, 300, 3, 216, 3, 300, 3, 300, 148, 3, 3, 719, 3,
    390, 3, 378, 23, 3, 300, 3, 160, 3, 721, 3, 722, 3, 3, 671, 4,
    3, 378, 23, 3, 760, 3, 763, 3, 152, 3, 775, 3, 378, 23, 3, 378,
    23, 3, 378, 23, 3, 378, 23, 3, 378, 23, 3, 390, 3, 143, 145, 4,
    3, 789, 3, 795, 3, 300, 3, 789, 3, 795, 3, 23, 3, 23, 165, 3,
    473, 23, 3, 473, 23, 3, 23, 165, 3, 4, 3, 823, 3, 390, 3, 23,
    3, 300, 3, 823, 3, 390, 3, 300, 3, 300, 417, 3, 3, 540, 3, 23,
    221, 3, 540, 3, 23, 221, 3, 160, 3, 387, 3, 382, 23, 3, 378, 23,
    3, 23, 165, 3, 23, 3, 23, 3, 378, 23, 3, 23, 3, 23, 3, 204,
    23, 3, 316, 23, 3, 184, 3, 174, 3, 473, 23, 3, 23, 196, 3, 23,
    3, 23, 3, 23, 3, 850, 4, 3, 225, 3, 225, 3, 225, 3, 225, 3,
    225, 3, 225, 3, 225, 3, 225, 3, 225, 3, 225, 3, 870, 3, 225, 4,
    3, 671, 3, 225, 3, 300, 3, 378, 23, 3, 225, 3, 300, 3, 378, 23,
    3, 225, 3, 300, 3, 378, 23, 3, 225, 3, 300, 3, 378, 23, 3, 300,
    3, 390, 3, 382, 23, 3, 23, 890, 3, 300, 3, 23, 3, 23, 165, 3,
    378, 23, 3, 904, 3, 904, 3, 378, 23, 3, 378, 23, 3, 378, 23, 3,
    300, 3, 904, 3, 378, 23, 3, 904, 3, 378, 23, 3, 930, 3, 23, 3,
    23, 3, 174, 3, 160, 3, 160, 3, 184, 3, 174, 3, 23, 3, 23, 3,
    23, 3, 225, 3, 225, 3, 225, 3, 225, 3, 225, 3, 225, 3, 23, 3,
    23, 3, 204, 23, 3, 225, 3, 23, 3, 225, 3, 23, 3, 225, 3, 225,
    3, 225, 3, 225, 3, 225, 3, 23, 221, 3, 225, 3, 225, 3, 225, 3,
    23, 3, 160, 3, 382, 23, 3, 300, 3, 23, 3, 300, 3, 316, 23, 3,
    23, 196, 3, 975, 3, 975, 3, 23, 165, 3, 378, 23, 3, 160, 3, 382,
    23, 3, 387, 3, 435, 3, 23, 165, 3, 23, 988, 3, 23, 165, 3, 23,
    165, 3, 378, 23, 3, 387, 3, 435, 3, 642, 3, 23, 988, 3, 184, 3,
    174, 3, 160, 3, 378, 23, 3, 160, 3, 993, 3, 160, 3, 160, 3, 999,
    3, 23, 3, 23, 3, 23, 3, 23, 3, 378, 23, 3, 23, 3, 378, 23,
    3, 23, 3, 225, 3, 225, 3, 23, 3, 225, 3, 225, 3, 225, 3, 184,
    3, 23, 3, 23, 165, 3, 23, 3, 300, 3, 300, 3, 184, 3, 174, 3,
    999, 3, 23, 538, 3, 542, 3, 540, 3, 23, 193, 3, 23, 193, 3, 23,
    193, 3, 23, 193, 3, 23, 193, 3, 23, 193, 3, 23, 196, 3, 23, 193,
    3, 23, 196, 3, 23, 3, 160, 3, 378, 23, 3, 23, 579, 3, 1023, 3,
    23, 988, 3, 975, 3, 23, 3, 23, 3, 23, 3, 378, 23, 3, 23, 3,
    378, 23, 3, 23, 3, 23, 3, 23, 3, 23, 3, 23, 3, 23, 613, 3,
    378, 23, 3, 300, 3, 23, 165, 3, 23, 617, 3, 23, 538, 3, 540, 3,
    23, 165, 3, 378, 23, 3, 225, 3, 23, 221, 3, 23, 165, 3, 1067, 3,
    390, 3, 1069, 3, 23, 165, 3, 627, 3, 435, 3, 23, 165, 3, 1075, 3,
    23, 165, 3, 378, 23, 3, 387, 3, 435, 3, 23, 165, 3, 378, 23, 3,
    387, 3, 435, 3, 642, 3, 378, 23, 3, 390, 3, 382, 23, 3, 422, 3,
    23, 221, 3, 23, 3, 225, 3, 23, 3, 225, 3, 23, 3, 23, 652, 3,
    378, 23, 3, 23, 165, 3, 23, 165, 3, 23, 165, 3, 23, 165, 3, 300,
    3, 390, 3, 23, 3, 23, 3, 160, 3, 23, 3, 225, 3, 23, 988, 3,
    23, 193, 3, 23, 3, 378, 23, 3, 184, 3, 184, 3, 1110, 3, 1150, 3,
    1152, 3, 1154, 3, 1156, 3, 1217, 3, 1217, 3, 1217, 3, 1217, 3, 1217, 3,
    1217, 1257, 3, 1259, 3, 3, 1261, 3, 1152, 3, 23, 3, 23, 3, 23, 3,
    23, 3, 225, 3, 1322, 3, 23, 165, 167, 3, 3, 1326, 167, 3, 3, 1328,
    3, 23, 3, 1330, 3, 300, 167, 3, 3, 23, 3, 1354, 3, 1354, 3, 23,
    3, 174, 3, 160, 3, 160, 3, 23, 3, 204, 23, 3, 23, 3, 316, 23,
    3, 23, 165, 3, 23, 3, 174, 3, 473, 23, 3, 23, 165, 3, 23, 3,
    23, 193, 3, 23, 3, 23, 165, 3, 184, 3, 174, 3, 23, 538, 3, 23,
    196, 3, 23, 579, 3, 23, 3, 23, 613, 3, 23, 617, 3, 23, 165, 3,
    23, 3, 23, 652, 3, 23, 165, 3, 160, 3, 23, 193, 3, 23, 165, 3,
    300, 3, 1154, 3, 1367, 3, 1378, 3, 23, 3, 300, 3, 1399, 3, 1401, 3,
    1403, 3, 1405, 3, 216, 3, 216, 3, 1441, 3, 1443, 3, 823, 3, 823, 3,
    390, 3, 300, 3, 378, 23, 3, 390, 3, 1448, 3, 300, 3, 23, 1451, 3,
    1154, 3, 1156, 3, 300, 3, 160, 3, 1454, 3, 1448, 3, 930, 3, 719, 3,
    1458, 1460, 3, 3, 378, 23, 3, 1462, 3, 23, 165, 3, 23, 165, 3, 23,
    165, 3, 300, 3, 23, 3, 1484, 3, 1486, 3, 1488, 3, 1490, 3, 300, 3,
    23, 165, 3, 23, 3, 300, 3, 823, 3, 390, 3, 300, 3, 300, 3, 23,
    3, 300, 3, 719, 3, 378, 23, 3, 1514, 3, 1522, 3, 1528, 3, 1530, 3,
    1069, 3, 1533, 3, 1326, 3, 1537, 3, 1541, 3, 1328, 167, 3, 167, 3, 3,
    1541, 3, 1328, 3, 1545, 3, 1547, 3, 1514, 3, 1328, 3, 300, 3, 1552, 3,
    300, 3, 1552, 3, 300, 3, 763, 3, 300, 3, 160, 3, 300, 3, 300, 3,
    300, 3, 300, 3, 300, 3, 160, 1559, 3, 3, 378, 23, 3, 300, 3, 378,
    23, 3, 300, 3, 1566, 3, 1458, 3, 300, 3, 300, 3, 1354, 3, 225, 3,
    300, 3, 1401, 3, 1403, 3, 1572, 3, 763, 3, 930, 3, 300, 3, 760, 3,
    760, 3, 160, 3, 160, 3, 993, 3, 225, 3, 300, 3, 184, 3, 1579, 3,
    184, 3, 184, 3, 1067, 3, 184, 3, 1579, 290, 3, 1587, 3, 1589, 3, 3,
    378, 23, 3, 1595, 3, 1598, 3, 300, 3, 1528, 3, 1522, 3, 378, 23, 3,
    300, 3, 1595, 3, 1110, 3, 300, 3, 1522, 3, 1616, 4, 3, 23, 196, 3,
    23, 988, 3, 23, 196, 3, 23, 3, 390, 3, 382, 23, 3, 23, 1635, 3,
    422, 3, 378, 23, 3, 390, 3, 300, 3, 1637, 3, 540, 3, 23, 221, 3,
    23, 165, 167, 3, 3, 1651, 3, 1653, 3, 300, 167, 3, 3, 23, 165, 3,
    1547, 3, 1682, 3, 1684, 3, 1547, 3, 1682, 3, 1547, 3, 300, 3, 1, 4,
    5, 3, 300, 3, 300, 3, 216, 3, 1454, 3, 904, 1725, 3, 3, 1727, 3,
    1729, 3, 1, 1356, 4, 5, 3, 1356, 4, 3, 23, 165, 3, 23, 3, 23,
    3, 23, 3, 1067, 3, 1770, 3, 1772, 3, 1774, 3, 300, 3, 1779, 3, 1781,
    3, 300, 3, 1779, 3, 1781, 3, 300, 3, 1785, 3, 23, 165, 3, 23, 165,
    167, 3, 3, 390, 167, 3, 3, 1684, 3, 1789, 3, 1791, 3, 225, 3, 300,
    3, 1817, 3, 1819, 3, 245, 4, 3, 1067, 3, 930, 3, 930, 3, 1572, 3,
    763, 3, 930, 3, 1403, 3, 1401, 3, 1405, 3, 1853, 3, 300, 3, 378, 23,
    3, 300, 3, 1871, 3, 1522, 3, 1533, 3, 1817, 3, 378, 23, 3, 1454, 3,
    823, 3, 378, 23, 3, 1454, 3, 823, 3, 378, 23, 3, 378, 23, 3, 1454,
    3, 823, 3, 378, 23, 3, 1454, 3, 823, 3, 378, 23, 3, 300, 3, 300,
    3, 300, 3, 300, 3, 1902, 3, 1904, 3, 23, 3, 160, 3, 1912, 3, 1927,
    3, 23, 196, 3, 300, 3, 23, 193, 3, 23, 193, 3, 23, 165, 3, 378,
    23, 3, 23, 165, 3, 1069, 3, 23, 221, 3, 473, 23, 3, 23, 165, 3,
    23, 196, 3, 204, 23, 3, 225, 3, 1967, 290, 3, 1587, 3, 3, 23, 193,
    3, 23, 286, 3, 23, 286, 1589, 3, 3, 23, 3, 23, 165, 3, 378, 23,
    3, 184, 3, 174, 3, 204, 23, 3, 23, 3, 23, 3, 23, 165, 3, 184,
    3, 23, 165, 3, 23, 3, 184, 3, 23, 3, 23, 193, 3, 184, 3, 23,
    196, 3, 23, 3, 23, 3, 23, 3, 23, 3, 23, 165, 3, 2005, 167, 3,
    3, 300, 3, 2007, 3, 2009, 3, 1528, 3, 2011, 3, 1528, 3, 2013, 3, 2015,
    3, 2017, 3, 1330, 3, 2019, 3, 2021, 3, 1462, 3, 2023, 3, 2023, 3, 142,
    3, 1522, 3, 2025, 3, 2027, 3, 1533, 3, 1528, 3, 1530, 3, 1069, 3, 2011,
    3, 1572, 3, 1572, 125, 3, 125, 3, 3, 2036, 3, 378, 23, 125, 3, 3,
    225, 3, 390, 3, 378, 23, 3, 300, 3, 823, 3, 378, 23, 3, 225, 3,
    390, 3, 378, 23, 3, 300, 3, 823, 3, 23, 3, 1528, 3, 142, 3, 1522,
    3, 2023, 3, 23, 196, 3, 23, 193, 3, 225, 3, 23, 193, 3, 1441, 3,
    160, 3, 1443, 3, 23, 3, 23, 3, 23, 3, 300, 3, 23, 3, 23, 3,
    23, 3, 300, 1259, 3, 125, 3, 3, 1912, 3, 1401, 3, 2080, 3, 2082, 3,
    2084, 3, 2086, 3, 2088, 3, 2090, 2092, 3, 3, 2094, 3, 2096, 3, 2098, 2100,
    3, 3, 2102, 3, 2104, 3, 2106, 3, 2108, 3, 2110, 3, 2112, 3, 2114, 3,
    1405, 3, 1403, 3, 2116, 3, 2118, 3, 2120, 3, 2122, 3, 2124, 3, 2126, 3,
    2128, 3, 2130, 3, 2132, 3, 2134, 2136, 3, 3, 2138, 3, 2140, 3, 2142, 2144,
    3, 2146, 3, 2148, 3, 2150, 3, 2152, 3, 2154, 3, 2156, 3, 2158, 3, 3,
    2160, 3, 2162, 3, 2164, 3, 2166, 3, 2168, 3, 2170, 3, 2172, 3, 2174, 3,
    2176, 3, 2178, 3, 2180, 3, 2182, 3, 2184, 3, 2182, 3, 2187, 3, 2114, 3,
    2189, 3, 2191, 3, 2132, 3, 2194, 3, 2196, 3, 2198, 3, 2200, 3, 2202, 3,
    671, 167, 3, 3, 4, 155, 3, 23, 3, 300, 3, 2216, 3, 2216, 3, 364,
    3, 2219, 3, 2216, 3, 300, 3, 23, 3, 473, 23, 3, 1067, 3, 390, 3,
    23, 165, 3, 1912, 3, 1217, 3, 300, 3, 23, 165, 3, 23, 165, 1259, 3,
    1725, 3, 3, 1154, 3, 1367, 3, 2013, 3, 2015, 3, 2017, 3, 1330, 3, 2019,
    3, 2021, 3, 1462, 3, 2023, 3, 1528, 3, 1530, 3, 1069, 3, 142, 3, 1522,
    3, 2011, 3, 1528, 3, 23, 165, 167, 3, 3, 1791, 3, 2017, 3, 23, 3,
    1330, 3, 300, 167, 3, 3, 204, 23, 3, 23, 165, 3, 300, 167, 3, 3,
    142, 3, 1522, 3, 1150, 3, 300, 3, 300, 3, 300, 3, 1533, 3, 4, 2340,
    3, 23, 1451, 3, 2344, 3, 142, 3, 2346, 3, 2352, 3, 2352, 3, 142, 3,
    4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"^[+\-*/<>=~!@#%^&|`?]+$"#, // [0]
    r#"[A-Z_][A-Z0-9_]*"#, // [1]
    r#"^(NOT)$"#, // [2]
    r#"[A-Z_][A-Z0-9_$]*"#, // [3]
    r#"[\p{L}_][\p{L}\p{N}_$]*"#, // [4]
    r#"^(ALL|ALTER|AND|ANY|AS|BY|CAST|CHECK|CLUSTER|CLUSTERS|CONNECTION|CONNECTIONS|CONSTRAINT|CREATE|CROSS|CURRENT|DELETE|DISTINCT|DROP|ELSE|EXISTS|FOLLOWING|FOR|FROM|FULL|GROUP|ILIKE|IN|INNER|INSERT|INTERSECT|INTO|IS|JOIN|LATERAL|LEFT|LIKE|NATURAL|NOT|NULL|NULLIF|OF|ON|OR|ORDER|RETURNING|ROW|ROWS|SELECT|SET|SINK|SINKS|TO|UNION|UNIQUE|UPDATE|USING|VALUES|WHEN|WHERE|WITH)$"#, // [5]
    r#"[A-Z_][A-Z0-9_$]*|"[^"]*""#, // [6]
    r#"[!<>=~@#%^&|`?+\-*/]+"#, // [7]
    r#"(?i)'[0-9a-f]*'"#, // [8]
    r#"'([^']|'')*'"#, // [9]
    r#"'[^0-9A-Fa-f'+\-\s)]'"#, // [10]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 9, 10, 11, 12, 13, 14,
    13, 14, 12, 9, 10, 11, 8, 25, 27, 28,
    30, 31, 32, 33, 34, 35, 36, 31, 34, 35,
    36, 35, 36, 34, 31, 42, 31, 33, 34, 35,
    36, 33, 32, 30, 28, 46, 47, 48, 49, 50,
    51, 52, 53, 54, 52, 51, 53, 50, 46, 54,
    49, 48, 47, 65, 69, 70, 71, 72, 73, 74,
    75, 76, 11, 77, 78, 79, 13, 80, 81, 81,
    78, 72, 75, 76, 77, 79, 80, 73, 92, 74,
    70, 71, 97, 70, 71, 98, 73, 74, 99, 100,
    33, 101, 102, 103, 35, 104, 105, 105, 102, 98,
    99, 100, 101, 103, 104, 116, 117, 116, 117, 122,
    123, 123, 122, 128, 129, 130, 131, 132, 133, 136,
    137, 138, 139, 141, 147, 150, 156, 163, 168, 169,
    170, 176, 177, 178, 179, 178, 182, 177, 187, 189,
    191, 179, 198, 200, 202, 168, 9, 205, 177, 178,
    206, 179, 207, 168, 9, 205, 207, 207, 168, 9,
    205, 211, 212, 214, 216, 205, 219, 222, 223, 182,
    222, 182, 222, 227, 150, 229, 133, 230, 231, 232,
    211, 233, 234, 235, 236, 237, 238, 212, 239, 123,
    240, 241, 242, 243, 244, 245, 216, 246, 247, 250,
    251, 223, 254, 206, 28, 46, 50, 258, 51, 259,
    52, 260, 261, 53, 54, 98, 100, 33, 101, 35,
    36, 268, 46, 50, 258, 51, 259, 52, 260, 261,
    53, 54, 258, 259, 260, 261, 275, 277, 279, 279,
    277, 28, 281, 282, 281, 282, 288, 10, 11, 292,
    294, 296, 132, 297, 177, 178, 179, 298, 297, 297,
    179, 303, 234, 306, 308, 303, 234, 303, 234, 308,
    311, 298, 314, 132, 297, 179, 53, 132, 297, 7,
    139, 318, 319, 320, 296, 321, 322, 72, 323, 74,
    324, 325, 75, 76, 11, 77, 326, 13, 327, 328,
    14, 81, 7, 319, 320, 296, 321, 322, 323, 325,
    11, 77, 326, 327, 14, 331, 319, 320, 321, 322,
    337, 339, 323, 325, 325, 323, 326, 327, 345, 346,
    347, 348, 349, 346, 347, 348, 349, 74, 13, 81,
    318, 72, 75, 76, 324, 357, 358, 357, 358, 362,
    328, 132, 170, 8, 366, 297, 367, 368, 369, 370,
    303, 371, 372, 177, 373, 206, 179, 298, 375, 379,
    308, 306, 227, 383, 303, 234, 384, 259, 385, 132,
    297, 303, 206, 388, 179, 14, 179, 14, 392, 393,
    187, 394, 395, 396, 189, 397, 398, 191, 399, 282,
    297, 303, 179, 401, 383, 403, 404, 403, 404, 408,
    147, 409, 410, 411, 412, 413, 370, 414, 388, 77,
    415, 388, 179, 388, 420, 424, 425, 70, 426, 427,
    427, 425, 426, 432, 308, 306, 434, 322, 384, 385,
    306, 384, 385, 437, 303, 437, 439, 442, 444, 366,
    367, 367, 366, 403, 373, 373, 449, 366, 367, 368,
    370, 368, 370, 453, 455, 457, 459, 8, 460, 460,
    462, 463, 462, 463, 369, 370, 369, 371, 227, 468,
    469, 362, 471, 468, 469, 170, 372, 372, 132, 297,
    179, 241, 243, 244, 245, 246, 247, 479, 132, 170,
    8, 366, 297, 367, 368, 369, 370, 303, 371, 372,
    177, 178, 373, 206, 179, 298, 483, 484, 70, 485,
    486, 303, 487, 206, 488, 489, 457, 179, 490, 491,
    492, 493, 483, 488, 491, 483, 496, 488, 491, 485,
    490, 493, 485, 490, 493, 486, 303, 486, 70, 457,
    503, 504, 503, 504, 487, 508, 509, 510, 510, 508,
    509, 484, 489, 492, 483, 484, 517, 70, 485, 486,
    370, 303, 177, 487, 178, 206, 488, 489, 457, 179,
    490, 491, 492, 493, 517, 370, 517, 522, 170, 8,
    177, 206, 179, 206, 179, 170, 8, 177, 178, 206,
    179, 414, 527, 8, 9, 471, 74, 12, 529, 530,
    13, 81, 530, 529, 178, 182, 362, 207, 182, 362,
    207, 70, 206, 457, 179, 517, 70, 370, 177, 178,
    206, 457, 179, 536, 132, 297, 177, 178, 179, 13,
    35, 281, 282, 543, 9, 48, 49, 544, 369, 545,
    546, 547, 548, 549, 550, 551, 552, 553, 554, 555,
    311, 552, 555, 555, 552, 48, 547, 547, 49, 548,
    548, 369, 549, 549, 545, 550, 545, 550, 551, 554,
    554, 551, 543, 546, 543, 546, 544, 553, 544, 553,
    234, 241, 243, 244, 245, 246, 247, 570, 177, 178,
    411, 229, 133, 413, 370, 413, 412, 370, 412, 577,
    415, 409, 410, 370, 410, 584, 147, 409, 410, 411,
    412, 413, 370, 414, 177, 178, 388, 77, 179, 415,
    529, 530, 586, 588, 589, 590, 591, 592, 593, 594,
    595, 596, 597, 598, 599, 600, 601, 602, 603, 604,
    605, 606, 607, 608, 609, 610, 611, 11, 420, 615,
    132, 9, 366, 297, 367, 177, 618, 178, 179, 619,
    618, 619, 132, 170, 8, 366, 297, 367, 368, 369,
    370, 303, 371, 177, 373, 206, 179, 298, 132, 624,
    624, 150, 138, 628, 138, 628, 628, 147, 182, 132,
    170, 8, 366, 297, 367, 368, 369, 370, 303, 371,
    177, 178, 373, 206, 179, 298, 634, 635, 178, 179,
    634, 638, 227, 275, 640, 223, 42, 207, 635, 646,
    132, 170, 297, 177, 178, 179, 648, 650, 517, 370,
    178, 654, 132, 170, 297, 178, 656, 138, 658, 658,
    170, 177, 178, 206, 179, 660, 662, 663, 662, 663,
    150, 666, 667, 666, 666, 667, 670, 673, 674, 675,
    660, 176, 229, 133, 676, 677, 678, 679, 136, 229,
    676, 675, 679, 677, 678, 689, 690, 230, 694, 231,
    698, 699, 700, 701, 702, 703, 704, 705, 706, 707,
    708, 710, 711, 712, 713, 714, 715, 233, 725, 726,
    727, 728, 729, 730, 731, 675, 660, 176, 229, 133,
    676, 677, 222, 678, 670, 379, 732, 324, 733, 679,
    136, 734, 735, 736, 737, 738, 379, 136, 670, 733,
    731, 660, 133, 677, 222, 678, 732, 324, 735, 736,
    737, 738, 734, 744, 745, 746, 750, 751, 733, 754,
    212, 211, 757, 758, 759, 150, 52, 640, 764, 765,
    766, 767, 768, 769, 770, 771, 772, 773, 74, 303,
    150, 50, 51, 52, 53, 640, 207, 410, 345, 764,
    777, 777, 780, 782, 783, 787, 791, 793, 797, 799,
    800, 801, 802, 803, 804, 805, 806, 807, 808, 809,
    810, 811, 812, 813, 815, 816, 308, 227, 383, 303,
    234, 384, 259, 385, 825, 385, 827, 830, 384, 833,
    308, 379, 306, 227, 383, 303, 234, 384, 259, 385,
    842, 845, 847, 7, 139, 318, 319, 375, 306, 320,
    10, 296, 321, 314, 322, 72, 471, 323, 74, 92,
    75, 536, 281, 76, 449, 11, 77, 326, 420, 615,
    13, 219, 327, 260, 14, 81, 13, 81, 7, 319,
    320, 296, 321, 322, 323, 74, 92, 536, 281, 449,
    77, 326, 420, 615, 219, 327, 260, 14, 139, 75,
    76, 732, 852, 853, 854, 855, 731, 677, 735, 738,
    737, 736, 872, 875, 876, 877, 468, 879, 880, 881,
    884, 469, 53, 886, 888, 150, 69, 891, 892, 893,
    894, 895, 896, 897, 900, 150, 207, 150, 230, 901,
    900, 241, 243, 244, 245, 246, 247, 150, 230, 901,
    241, 243, 244, 245, 246, 247, 901, 150, 230, 241,
    243, 244, 245, 246, 247, 150, 230, 905, 906, 907,
    908, 909, 910, 97, 911, 912, 234, 913, 911, 905,
    912, 913, 907, 910, 150, 176, 908, 909, 906, 150,
    230, 900, 241, 243, 244, 245, 246, 247, 730, 923,
    905, 234, 207, 730, 923, 905, 234, 923, 907, 368,
    912, 913, 150, 230, 901, 926, 241, 243, 244, 245,
    246, 247, 926, 150, 230, 926, 241, 243, 244, 245,
    246, 247, 52, 928, 640, 928, 932, 933, 934, 935,
    207, 459, 938, 941, 942, 150, 223, 944, 945, 946,
    947, 948, 949, 947, 945, 946, 948, 954, 955, 954,
    955, 944, 949, 713, 891, 960, 961, 962, 965, 968,
    960, 961, 962, 973, 168, 9, 906, 205, 945, 946,
    947, 177, 219, 349, 227, 241, 243, 244, 245, 246,
    247, 945, 946, 947, 945, 946, 308, 306, 303, 234,
    978, 980, 308, 306, 227, 383, 303, 234, 308, 306,
    985, 990, 991, 994, 147, 725, 483, 484, 70, 485,
    323, 486, 303, 487, 997, 488, 489, 457, 179, 490,
    491, 492, 328, 493, 998, 207, 227, 471, 74, 733,
    1002, 1005, 72, 1010, 75, 277, 1010, 72, 75, 1014,
    1015, 1014, 1015, 28, 50, 51, 52, 53, 137, 543,
    9, 48, 49, 544, 42, 369, 545, 546, 547, 548,
    549, 550, 551, 552, 553, 554, 281, 555, 1018, 282,
    311, 207, 137, 543, 9, 48, 49, 544, 42, 369,
    545, 546, 547, 548, 549, 550, 551, 552, 553, 554,
    281, 555, 1018, 282, 311, 1018, 147, 409, 410, 411,
    412, 413, 370, 414, 415, 1025, 1026, 1028, 1029, 1028,
    1029, 1032, 1028, 584, 1029, 1035, 1037, 704, 935, 1038,
    1039, 1038, 1039, 1045, 773, 1047, 327, 1047, 1051, 932,
    1052, 1053, 1054, 1052, 1053, 1054, 1059, 1060, 990, 991,
    463, 1059, 1060, 990, 991, 1059, 1060, 1059, 1060, 150,
    69, 219, 362, 207, 459, 50, 297, 1064, 1064, 150,
    50, 258, 51, 52, 13, 53, 640, 207, 150, 13,
    150, 258, 150, 371, 638, 1070, 1072, 1073, 69, 638,
    219, 362, 207, 459, 1076, 1077, 1078, 1078, 1077, 1076,
    897, 348, 138, 658, 1082, 1082, 50, 51, 261, 53,
    437, 275, 223, 439, 303, 1085, 787, 437, 439, 303,
    437, 439, 1085, 1087, 1088, 1088, 1087, 1092, 453, 1093,
    1093, 1077, 1096, 1096, 187, 189, 191, 282, 990, 991,
    1102, 1103, 1104, 1104, 1106, 1106, 393, 395, 396, 1111,
    1112, 811, 875, 1112, 1115, 1116, 1117, 1118, 1119, 1120,
    1121, 1122, 1123, 1124, 1125, 1126, 1127, 1128, 1129, 1130,
    1131, 1132, 1133, 1134, 1135, 1136, 1137, 1138, 1139, 1140,
    1141, 1142, 1143, 1144, 1145, 1146, 1147, 1148, 1149, 232,
    233, 235, 236, 237, 238, 728, 729, 745, 751, 754,
    758, 759, 772, 803, 804, 809, 810, 1157, 1158, 1159,
    1160, 1161, 1162, 1163, 1164, 1165, 1166, 1167, 1168, 1169,
    1170, 1047, 1171, 1172, 1173, 1174, 1175, 1176, 1177, 1178,
    1179, 1180, 1139, 1143, 1181, 1182, 1183, 1184, 1185, 1186,
    1187, 327, 1188, 1189, 1190, 1191, 1192, 1193, 1194, 1195,
    1186, 1170, 1164, 1165, 1166, 1168, 1161, 1162, 1181, 1159,
    1208, 1187, 1182, 1183, 1184, 1185, 1160, 1158, 1177, 1158,
    1177, 1176, 759, 803, 804, 327, 1193, 759, 803, 804,
    1193, 1224, 1193, 751, 754, 758, 810, 1172, 1173, 1179,
    1139, 1143, 1139, 1172, 1173, 1179, 1143, 809, 1163, 1174,
    1175, 1163, 1174, 1175, 1188, 1191, 1191, 1188, 1192, 1195,
    1047, 1171, 1171, 1167, 1169, 1178, 1189, 1190, 1157, 1180,
    1194, 231, 123, 232, 1264, 1265, 1266, 1267, 1268, 1269,
    1270, 1271, 1272, 1273, 1274, 1275, 1276, 1277, 1278, 1279,
    1280, 1281, 1282, 1283, 1284, 1285, 1286, 1287, 232, 233,
    235, 236, 237, 238, 241, 243, 244, 245, 246, 247,
    235, 236, 233, 237, 238, 238, 237, 1294, 1296, 1265,
    1297, 1298, 1299, 1300, 1301, 1302, 1266, 1303, 1304, 1305,
    1308, 1309, 1310, 1311, 1312, 1316, 1317, 1318, 150, 229,
    133, 230, 231, 232, 211, 233, 234, 235, 236, 237,
    238, 212, 239, 123, 240, 241, 242, 243, 244, 245,
    216, 246, 247, 5, 5, 1323, 1324, 1331, 1333, 1335,
    1336, 1337, 1338, 1340, 1341, 1342, 1343, 1344, 28, 1346,
    101, 990, 991, 1346, 1348, 1349, 1350, 1351, 1352, 323,
    241, 243, 244, 245, 246, 247, 1355, 242, 1358, 8,
    9, 10, 471, 74, 281, 11, 12, 529, 530, 13,
    14, 282, 81, 1360, 1361, 1362, 1363, 1364, 1365, 1368,
    1369, 1371, 1372, 1373, 1374, 1375, 1376, 434, 1072, 1379,
    1380, 1381, 1382, 1383, 1384, 662, 663, 770, 933, 97,
    1385, 1386, 1387, 666, 1388, 662, 663, 770, 933, 1385,
    1386, 1387, 666, 1388, 1385, 1388, 1387, 1386, 1047, 327,
    1195, 1395, 1395, 150, 662, 663, 666, 662, 663, 666,
    1409, 1070, 303, 1410, 42, 303, 211, 1047, 1414, 1415,
    1416, 1417, 1418, 1419, 234, 212, 1420, 1422, 1424, 727,
    303, 229, 133, 303, 1430, 1431, 1431, 1430, 731, 675,
    660, 176, 229, 133, 676, 677, 222, 678, 379, 732,
    324, 679, 136, 735, 736, 737, 738, 229, 133, 1431,
    1437, 1445, 232, 211, 233, 234, 235, 236, 237, 238,
    212, 241, 243, 244, 245, 216, 246, 247, 1463, 1464,
    1467, 628, 1469, 1469, 453, 489, 479, 207, 1472, 1473,
    28, 216, 1475, 150, 170, 725, 813, 853, 897, 46,
    1312, 50, 1338, 297, 793, 1384, 1467, 116, 51, 1476,
    117, 1477, 52, 179, 1478, 415, 261, 53, 1479, 640,
    207, 1480, 1481, 1482, 1480, 1481, 1482, 150, 170, 725,
    813, 853, 897, 46, 1312, 50, 1338, 297, 793, 1384,
    1467, 116, 51, 1476, 117, 1477, 52, 179, 1478, 415,
    261, 53, 1479, 640, 207, 1491, 1492, 1494, 1495, 833,
    370, 1496, 53, 1496, 527, 619, 1499, 308, 227, 383,
    303, 234, 1500, 1078, 489, 1503, 1495, 1504, 1504, 254,
    292, 370, 1472, 1509, 1500, 1511, 1512, 1376, 1467, 275,
    292, 1515, 1516, 214, 1517, 1518, 141, 1519, 1520, 886,
    998, 207, 1515, 1519, 998, 1376, 1516, 1517, 1520, 1518,
    1535, 1540, 1544, 1549, 1550, 239, 757, 1554, 1555, 275,
    223, 1560, 1563, 1564, 230, 654, 147, 725, 483, 484,
    70, 485, 323, 486, 303, 487, 997, 488, 489, 457,
    179, 490, 491, 492, 328, 493, 998, 997, 150, 51,
    997, 52, 53, 640, 207, 150, 52, 640, 207, 42,
    938, 1574, 239, 1574, 1437, 222, 227, 1584, 1115, 1116,
    1117, 1118, 1119, 1120, 1585, 1590, 1591, 1592, 1593, 1597,
    1467, 1515, 214, 141, 1519, 998, 1602, 1603, 1467, 214,
    141, 1519, 998, 1604, 1605, 1606, 1607, 1608, 1271, 1609,
    1610, 1611, 1612, 1613, 1614, 1615, 1618, 1619, 1620, 1563,
    1564, 1409, 1624, 1376, 214, 1629, 1073, 698, 1337, 628,
    1633, 1633, 1032, 362, 207, 1638, 1639, 1640, 1641, 1642,
    1643, 1644, 879, 1645, 1647, 27, 282, 150, 227, 50,
    51, 52, 53, 640, 207, 1654, 600, 601, 602, 603,
    604, 605, 606, 607, 608, 609, 610, 611, 588, 589,
    590, 591, 592, 593, 594, 595, 596, 597, 598, 599,
    1657, 1658, 1659, 1660, 1516, 241, 243, 244, 245, 216,
    246, 247, 1264, 1265, 1266, 1267, 1268, 1269, 1270, 1271,
    1272, 1273, 1274, 1275, 1276, 1277, 1278, 1279, 1280, 1281,
    1282, 1283, 1284, 1285, 1286, 1287, 241, 243, 244, 245,
    246, 247, 1265, 1271, 1278, 1279, 1281, 1287, 1287, 1279,
    1278, 1281, 990, 991, 463, 1667, 1414, 1420, 1415, 1416,
    1417, 1418, 1419, 1673, 1674, 1675, 1676, 69, 362, 468,
    880, 881, 880, 881, 1693, 231, 1695, 1696, 654, 1696,
    1695, 1700, 1701, 1702, 1703, 246, 607, 1705, 1706, 1554,
    1707, 1708, 1709, 1409, 1070, 1410, 1410, 28, 852, 893,
    1318, 383, 408, 1639, 420, 424, 25, 1518, 1719, 1720,
    1121, 1122, 1123, 1124, 1721, 231, 239, 240, 397, 398,
    1730, 1731, 7, 1379, 453, 1496, 1735, 1736, 1737, 1738,
    1739, 1740, 1741, 1742, 1743, 1744, 1746, 1747, 1748, 1749,
    1750, 1317, 1751, 1752, 1753, 1754, 1317, 1751, 1752, 1754,
    1754, 1751, 1752, 1753, 1047, 327, 1760, 1761, 1762, 1763,
    1764, 1765, 1766, 1767, 1476, 1777, 50, 884, 53, 1783,
    884, 51, 1792, 1793, 1796, 1797, 1276, 1798, 1517, 1799,
    1800, 1801, 1802, 1803, 1804, 1805, 1806, 1807, 1808, 1125,
    245, 1126, 1127, 1128, 1129, 1130, 1131, 1132, 1133, 1134,
    1135, 1136, 1137, 1138, 1811, 1437, 222, 1814, 1814, 1820,
    1822, 1823, 1825, 1415, 1416, 1417, 1418, 1827, 1828, 1829,
    1830, 1831, 1832, 1833, 150, 50, 51, 52, 53, 640,
    150, 52, 1837, 1838, 1839, 1415, 1416, 1417, 1418, 1419,
    731, 1847, 1848, 1849, 1851, 1855, 1856, 1857, 1858, 1859,
    1860, 1861, 1862, 1863, 1864, 1865, 1866, 1867, 1868, 241,
    243, 244, 245, 246, 247, 5, 698, 1337, 362, 275,
    1500, 1515, 214, 1519, 1078, 489, 1560, 998, 207, 1838,
    141, 1873, 1874, 1876, 1878, 1879, 1880, 1881, 1882, 1884,
    1885, 1886, 1889, 1890, 676, 241, 243, 244, 245, 246,
    247, 1894, 1895, 1897, 1140, 1141, 1142, 1144, 1145, 1146,
    1147, 1898, 1899, 1901, 1906, 1909, 1910, 150, 50, 51,
    1476, 52, 53, 640, 207, 1913, 1914, 1915, 1916, 1917,
    1918, 1919, 1921, 1929, 28, 673, 1931, 1931, 1280, 1934,
    250, 251, 244, 246, 244, 245, 246, 241, 247, 1938,
    243, 245, 243, 1941, 1942, 1943, 1945, 1946, 1947, 1949,
    1950, 1951, 1952, 279, 370, 277, 179, 468, 1886, 1956,
    1956, 1959, 227, 234, 1961, 1962, 1963, 1964, 10, 471,
    11, 27, 13, 10, 27, 1968, 1969, 1970, 1971, 1972,
    1973, 1974, 1976, 1977, 1978, 1979, 1980, 1983, 1477, 1984,
    1985, 1986, 1987, 1988, 726, 27, 1992, 1993, 1994, 1995,
    1996, 1997, 1998, 1999, 2000, 139, 375, 10, 296, 321,
    322, 72, 323, 607, 74, 325, 75, 536, 281, 76,
    11, 77, 615, 13, 219, 14, 81, 2003, 28, 830,
    25, 1376, 223, 1516, 1025, 214, 1517, 141, 1331, 1520,
    886, 1793, 2031, 2032, 2033, 2034, 442, 444, 1520, 1376,
    1516, 1060, 1976, 235, 2042, 227, 1060, 241, 243, 244,
    245, 246, 247, 2043, 2044, 1478, 9, 471, 74, 529,
    530, 13, 81, 1070, 886, 8, 30, 845, 31, 32,
    1639, 74, 1855, 33, 34, 2046, 2047, 35, 36, 105,
    2046, 2047, 2052, 2053, 2058, 2059, 2060, 2061, 2062, 2063,
    2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073,
    2074, 2075, 2077, 2078, 725, 853, 793, 1477, 415, 1479,
    2203, 2204, 2206, 2207, 2208, 2209, 2210, 2211, 2212, 2213,
    2214, 308, 434, 322, 384, 385, 437, 439, 370, 303,
    384, 385, 2221, 990, 991, 2225, 2225, 2227, 2228, 2229,
    2230, 2231, 2232, 236, 2236, 2237, 1148, 2238, 2239, 2240,
    2241, 2242, 2243, 2244, 2245, 1555, 2246, 2247, 2247, 2249,
    2249, 2253, 2254, 1149, 2255, 896, 388, 2260, 2261, 2263,
    2264, 2269, 2270, 2272, 2274, 2275, 2276, 2277, 2279, 2280,
    2281, 2282, 2283, 2284, 2285, 150, 662, 663, 911, 468,
    666, 662, 663, 911, 468, 666, 662, 663, 1343, 911,
    468, 1638, 487, 1917, 667, 261, 666, 2290, 2291, 2292,
    2293, 2294, 2296, 2297, 2298, 1285, 1286, 811, 875, 1112,
    1117, 1118, 1119, 1120, 1121, 1122, 1123, 1124, 1125, 1126,
    1127, 1128, 1129, 1130, 1131, 1132, 1133, 1134, 1135, 1136,
    1137, 1138, 1139, 1140, 1141, 1142, 1143, 1144, 1145, 1146,
    1147, 1148, 1149, 150, 811, 875, 1112, 1115, 1116, 1117,
    1118, 1119, 1120, 1121, 1122, 1123, 1124, 1125, 1126, 1127,
    1128, 1129, 1130, 1131, 1132, 1133, 1134, 1135, 1136, 1137,
    1138, 1139, 1140, 1141, 1142, 1143, 1144, 1145, 1146, 1147,
    1148, 1149, 216, 150, 216, 1115, 1116, 275, 292, 1515,
    214, 69, 141, 1878, 1519, 1331, 998, 2338, 2339, 141,
    638, 2347, 782, 1060, 2356, 2357, 2358, 2359, 2360, 2361,
    2362, 2363, 2364, 2365, 2366, 2367, 2368, 2369, 2370, 2371,
    2372, 2373, 2374, 2375, 2376,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 7, token_types_start: 10, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 10, raw_values_count: 1, token_types_start: 11, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 11, raw_values_count: 1, token_types_start: 12, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 12, raw_values_count: 1, token_types_start: 13, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 13, raw_values_count: 1, token_types_start: 14, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 14, raw_values_count: 1, token_types_start: 15, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 15, raw_values_count: 1, token_types_start: 16, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 16, raw_values_count: 1, token_types_start: 17, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 17, raw_values_count: 1, token_types_start: 18, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 18, raw_values_count: 1, token_types_start: 19, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 19, raw_values_count: 1, token_types_start: 20, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 20, raw_values_count: 7, token_types_start: 27, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 27, raw_values_count: 4, token_types_start: 31, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 31, raw_values_count: 1, token_types_start: 32, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 36, raw_values_count: 5, token_types_start: 41, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 44, raw_values_count: 10, token_types_start: 54, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 54, raw_values_count: 1, token_types_start: 55, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 55, raw_values_count: 1, token_types_start: 56, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 56, raw_values_count: 1, token_types_start: 57, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 57, raw_values_count: 1, token_types_start: 58, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 58, raw_values_count: 1, token_types_start: 59, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 59, raw_values_count: 1, token_types_start: 60, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 60, raw_values_count: 1, token_types_start: 61, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 61, raw_values_count: 1, token_types_start: 62, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 62, raw_values_count: 1, token_types_start: 63, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 63, raw_values_count: 1, token_types_start: 64, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 64, raw_values_count: 1, token_types_start: 65, token_types_count: 0 }, // [37]
    SimpleHintData { raw_values_start: 65, raw_values_count: 14, token_types_start: 79, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 79, raw_values_count: 1, token_types_start: 80, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 80, raw_values_count: 1, token_types_start: 81, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 81, raw_values_count: 1, token_types_start: 82, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 82, raw_values_count: 1, token_types_start: 83, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 83, raw_values_count: 1, token_types_start: 84, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 90, raw_values_count: 1, token_types_start: 91, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 91, raw_values_count: 1, token_types_start: 92, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 92, raw_values_count: 1, token_types_start: 93, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 93, raw_values_count: 14, token_types_start: 107, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 107, raw_values_count: 1, token_types_start: 108, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 108, raw_values_count: 1, token_types_start: 109, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 109, raw_values_count: 1, token_types_start: 110, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 110, raw_values_count: 1, token_types_start: 111, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 111, raw_values_count: 1, token_types_start: 112, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 112, raw_values_count: 1, token_types_start: 113, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 113, raw_values_count: 1, token_types_start: 114, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 114, raw_values_count: 1, token_types_start: 115, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 115, raw_values_count: 2, token_types_start: 117, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 117, raw_values_count: 1, token_types_start: 118, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 118, raw_values_count: 1, token_types_start: 119, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 119, raw_values_count: 2, token_types_start: 121, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 121, raw_values_count: 1, token_types_start: 122, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 122, raw_values_count: 1, token_types_start: 123, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 131, raw_values_count: 1, token_types_start: 132, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 132, raw_values_count: 1, token_types_start: 133, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 133, raw_values_count: 1, token_types_start: 134, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 134, raw_values_count: 1, token_types_start: 135, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 135, raw_values_count: 1, token_types_start: 136, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 136, raw_values_count: 1, token_types_start: 137, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 137, raw_values_count: 1, token_types_start: 138, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 138, raw_values_count: 1, token_types_start: 139, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 139, raw_values_count: 1, token_types_start: 140, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 140, raw_values_count: 1, token_types_start: 141, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 141, raw_values_count: 1, token_types_start: 142, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 142, raw_values_count: 3, token_types_start: 145, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 145, raw_values_count: 1, token_types_start: 146, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 146, raw_values_count: 1, token_types_start: 147, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 147, raw_values_count: 1, token_types_start: 148, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 148, raw_values_count: 1, token_types_start: 149, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 149, raw_values_count: 1, token_types_start: 150, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 150, raw_values_count: 1, token_types_start: 151, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 151, raw_values_count: 1, token_types_start: 152, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 152, raw_values_count: 1, token_types_start: 153, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 153, raw_values_count: 1, token_types_start: 154, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 154, raw_values_count: 1, token_types_start: 155, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 155, raw_values_count: 8, token_types_start: 163, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 163, raw_values_count: 4, token_types_start: 167, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 167, raw_values_count: 1, token_types_start: 168, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 168, raw_values_count: 3, token_types_start: 171, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 171, raw_values_count: 2, token_types_start: 173, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 173, raw_values_count: 1, token_types_start: 174, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 174, raw_values_count: 0, token_types_start: 174, token_types_count: 1 }, // [104]
    SimpleHintData { raw_values_start: 175, raw_values_count: 1, token_types_start: 176, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 176, raw_values_count: 1, token_types_start: 177, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 177, raw_values_count: 3, token_types_start: 180, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 180, raw_values_count: 2, token_types_start: 182, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 182, raw_values_count: 1, token_types_start: 183, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 184, raw_values_count: 17, token_types_start: 201, token_types_count: 8 }, // [111]
    SimpleHintData { raw_values_start: 209, raw_values_count: 0, token_types_start: 209, token_types_count: 2 }, // [112]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 214, raw_values_count: 11, token_types_start: 225, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 225, raw_values_count: 6, token_types_start: 231, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 232, raw_values_count: 10, token_types_start: 242, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 243, raw_values_count: 1, token_types_start: 244, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 244, raw_values_count: 1, token_types_start: 245, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 245, raw_values_count: 1, token_types_start: 246, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 246, raw_values_count: 1, token_types_start: 247, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 247, raw_values_count: 1, token_types_start: 248, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 248, raw_values_count: 1, token_types_start: 249, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 249, raw_values_count: 2, token_types_start: 251, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 251, raw_values_count: 3, token_types_start: 254, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 254, raw_values_count: 1, token_types_start: 255, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 255, raw_values_count: 1, token_types_start: 256, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 256, raw_values_count: 1, token_types_start: 257, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 257, raw_values_count: 2, token_types_start: 259, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 259, raw_values_count: 1, token_types_start: 260, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 260, raw_values_count: 1, token_types_start: 261, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 261, raw_values_count: 1, token_types_start: 262, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 262, raw_values_count: 6, token_types_start: 268, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 268, raw_values_count: 1, token_types_start: 269, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 269, raw_values_count: 2, token_types_start: 271, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 271, raw_values_count: 1, token_types_start: 272, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 272, raw_values_count: 1, token_types_start: 273, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 273, raw_values_count: 1, token_types_start: 274, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 274, raw_values_count: 3, token_types_start: 277, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 277, raw_values_count: 2, token_types_start: 279, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 279, raw_values_count: 1, token_types_start: 280, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 280, raw_values_count: 1, token_types_start: 281, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 281, raw_values_count: 1, token_types_start: 282, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 282, raw_values_count: 1, token_types_start: 283, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 283, raw_values_count: 4, token_types_start: 287, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 287, raw_values_count: 2, token_types_start: 289, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 289, raw_values_count: 23, token_types_start: 312, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 312, raw_values_count: 13, token_types_start: 325, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 325, raw_values_count: 1, token_types_start: 326, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 326, raw_values_count: 1, token_types_start: 327, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 327, raw_values_count: 1, token_types_start: 328, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 328, raw_values_count: 1, token_types_start: 329, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 329, raw_values_count: 1, token_types_start: 330, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 330, raw_values_count: 1, token_types_start: 331, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 331, raw_values_count: 1, token_types_start: 332, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 332, raw_values_count: 2, token_types_start: 334, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 334, raw_values_count: 1, token_types_start: 335, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 335, raw_values_count: 1, token_types_start: 336, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 336, raw_values_count: 1, token_types_start: 337, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 337, raw_values_count: 1, token_types_start: 338, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 338, raw_values_count: 1, token_types_start: 339, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 339, raw_values_count: 4, token_types_start: 343, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 343, raw_values_count: 1, token_types_start: 344, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 344, raw_values_count: 1, token_types_start: 345, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 345, raw_values_count: 1, token_types_start: 346, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 346, raw_values_count: 1, token_types_start: 347, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 347, raw_values_count: 3, token_types_start: 350, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 350, raw_values_count: 1, token_types_start: 351, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 351, raw_values_count: 3, token_types_start: 354, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 354, raw_values_count: 1, token_types_start: 355, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 355, raw_values_count: 2, token_types_start: 357, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 357, raw_values_count: 1, token_types_start: 358, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 358, raw_values_count: 1, token_types_start: 359, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 359, raw_values_count: 1, token_types_start: 360, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 360, raw_values_count: 1, token_types_start: 361, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 361, raw_values_count: 17, token_types_start: 378, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 378, raw_values_count: 1, token_types_start: 379, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 379, raw_values_count: 1, token_types_start: 380, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 380, raw_values_count: 9, token_types_start: 389, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 389, raw_values_count: 7, token_types_start: 396, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 396, raw_values_count: 2, token_types_start: 398, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 398, raw_values_count: 12, token_types_start: 410, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 410, raw_values_count: 3, token_types_start: 413, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 413, raw_values_count: 1, token_types_start: 414, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 414, raw_values_count: 1, token_types_start: 415, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 415, raw_values_count: 2, token_types_start: 417, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 417, raw_values_count: 1, token_types_start: 418, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 418, raw_values_count: 1, token_types_start: 419, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 419, raw_values_count: 1, token_types_start: 420, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 420, raw_values_count: 11, token_types_start: 431, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 431, raw_values_count: 2, token_types_start: 433, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 433, raw_values_count: 1, token_types_start: 434, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 434, raw_values_count: 1, token_types_start: 435, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 435, raw_values_count: 1, token_types_start: 436, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 436, raw_values_count: 4, token_types_start: 440, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 440, raw_values_count: 1, token_types_start: 441, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 441, raw_values_count: 1, token_types_start: 442, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 442, raw_values_count: 1, token_types_start: 443, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 443, raw_values_count: 1, token_types_start: 444, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 444, raw_values_count: 6, token_types_start: 450, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 450, raw_values_count: 3, token_types_start: 453, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 453, raw_values_count: 2, token_types_start: 455, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 455, raw_values_count: 1, token_types_start: 456, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 456, raw_values_count: 1, token_types_start: 457, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 457, raw_values_count: 1, token_types_start: 458, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 458, raw_values_count: 1, token_types_start: 459, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 459, raw_values_count: 2, token_types_start: 461, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 461, raw_values_count: 1, token_types_start: 462, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 462, raw_values_count: 1, token_types_start: 463, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 463, raw_values_count: 2, token_types_start: 465, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 465, raw_values_count: 1, token_types_start: 466, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 466, raw_values_count: 1, token_types_start: 467, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 467, raw_values_count: 4, token_types_start: 471, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 471, raw_values_count: 1, token_types_start: 472, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 472, raw_values_count: 1, token_types_start: 473, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 473, raw_values_count: 1, token_types_start: 474, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 474, raw_values_count: 1, token_types_start: 475, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 475, raw_values_count: 1, token_types_start: 476, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 476, raw_values_count: 1, token_types_start: 477, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 477, raw_values_count: 2, token_types_start: 479, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 479, raw_values_count: 1, token_types_start: 480, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 480, raw_values_count: 2, token_types_start: 482, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 482, raw_values_count: 1, token_types_start: 483, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 483, raw_values_count: 1, token_types_start: 484, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 484, raw_values_count: 2, token_types_start: 486, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 486, raw_values_count: 1, token_types_start: 487, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 487, raw_values_count: 1, token_types_start: 488, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 488, raw_values_count: 4, token_types_start: 492, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 492, raw_values_count: 1, token_types_start: 493, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 493, raw_values_count: 1, token_types_start: 494, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 494, raw_values_count: 1, token_types_start: 495, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 495, raw_values_count: 2, token_types_start: 497, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 497, raw_values_count: 1, token_types_start: 498, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 498, raw_values_count: 3, token_types_start: 501, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 501, raw_values_count: 0, token_types_start: 501, token_types_count: 6 }, // [238]
    SimpleHintData { raw_values_start: 507, raw_values_count: 1, token_types_start: 508, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 508, raw_values_count: 18, token_types_start: 526, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 526, raw_values_count: 16, token_types_start: 542, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 542, raw_values_count: 3, token_types_start: 545, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 545, raw_values_count: 1, token_types_start: 546, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 546, raw_values_count: 1, token_types_start: 547, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 547, raw_values_count: 1, token_types_start: 548, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 548, raw_values_count: 1, token_types_start: 549, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 549, raw_values_count: 3, token_types_start: 552, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 552, raw_values_count: 1, token_types_start: 553, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 553, raw_values_count: 1, token_types_start: 554, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 554, raw_values_count: 1, token_types_start: 555, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 555, raw_values_count: 2, token_types_start: 557, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 557, raw_values_count: 1, token_types_start: 558, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 558, raw_values_count: 2, token_types_start: 560, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 560, raw_values_count: 2, token_types_start: 562, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 562, raw_values_count: 1, token_types_start: 563, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 563, raw_values_count: 1, token_types_start: 564, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 564, raw_values_count: 1, token_types_start: 565, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 565, raw_values_count: 3, token_types_start: 568, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 568, raw_values_count: 1, token_types_start: 569, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 569, raw_values_count: 1, token_types_start: 570, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 570, raw_values_count: 1, token_types_start: 571, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 571, raw_values_count: 1, token_types_start: 572, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 572, raw_values_count: 1, token_types_start: 573, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 573, raw_values_count: 1, token_types_start: 574, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 574, raw_values_count: 20, token_types_start: 594, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 594, raw_values_count: 2, token_types_start: 596, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 596, raw_values_count: 1, token_types_start: 597, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 597, raw_values_count: 1, token_types_start: 598, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 598, raw_values_count: 5, token_types_start: 603, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 603, raw_values_count: 2, token_types_start: 605, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 605, raw_values_count: 6, token_types_start: 611, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 611, raw_values_count: 1, token_types_start: 612, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 612, raw_values_count: 1, token_types_start: 613, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 613, raw_values_count: 9, token_types_start: 622, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 622, raw_values_count: 1, token_types_start: 623, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 623, raw_values_count: 1, token_types_start: 624, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 624, raw_values_count: 4, token_types_start: 628, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 628, raw_values_count: 3, token_types_start: 631, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 631, raw_values_count: 4, token_types_start: 635, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 635, raw_values_count: 8, token_types_start: 643, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 643, raw_values_count: 1, token_types_start: 644, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 644, raw_values_count: 5, token_types_start: 649, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 649, raw_values_count: 2, token_types_start: 651, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 651, raw_values_count: 2, token_types_start: 653, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 653, raw_values_count: 18, token_types_start: 671, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 671, raw_values_count: 2, token_types_start: 673, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 673, raw_values_count: 1, token_types_start: 674, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 674, raw_values_count: 1, token_types_start: 675, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 675, raw_values_count: 2, token_types_start: 677, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 677, raw_values_count: 1, token_types_start: 678, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 678, raw_values_count: 2, token_types_start: 680, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 680, raw_values_count: 1, token_types_start: 681, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 681, raw_values_count: 2, token_types_start: 683, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 683, raw_values_count: 1, token_types_start: 684, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 684, raw_values_count: 2, token_types_start: 686, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 686, raw_values_count: 1, token_types_start: 687, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 687, raw_values_count: 1, token_types_start: 688, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 688, raw_values_count: 2, token_types_start: 690, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 690, raw_values_count: 1, token_types_start: 691, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 691, raw_values_count: 1, token_types_start: 692, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 692, raw_values_count: 2, token_types_start: 694, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 694, raw_values_count: 1, token_types_start: 695, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 695, raw_values_count: 1, token_types_start: 696, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 696, raw_values_count: 2, token_types_start: 698, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 698, raw_values_count: 1, token_types_start: 699, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 699, raw_values_count: 1, token_types_start: 700, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 700, raw_values_count: 1, token_types_start: 701, token_types_count: 6 }, // [307]
    SimpleHintData { raw_values_start: 707, raw_values_count: 1, token_types_start: 708, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 708, raw_values_count: 2, token_types_start: 710, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 710, raw_values_count: 1, token_types_start: 711, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 711, raw_values_count: 2, token_types_start: 713, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 713, raw_values_count: 2, token_types_start: 715, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 715, raw_values_count: 1, token_types_start: 716, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 716, raw_values_count: 2, token_types_start: 718, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 718, raw_values_count: 1, token_types_start: 719, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 719, raw_values_count: 1, token_types_start: 720, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 720, raw_values_count: 1, token_types_start: 721, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 721, raw_values_count: 1, token_types_start: 722, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 722, raw_values_count: 2, token_types_start: 724, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 724, raw_values_count: 1, token_types_start: 725, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 725, raw_values_count: 1, token_types_start: 726, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 726, raw_values_count: 14, token_types_start: 740, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 740, raw_values_count: 2, token_types_start: 742, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 742, raw_values_count: 1, token_types_start: 743, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 743, raw_values_count: 24, token_types_start: 767, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 767, raw_values_count: 2, token_types_start: 769, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 769, raw_values_count: 1, token_types_start: 770, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 770, raw_values_count: 10, token_types_start: 780, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 780, raw_values_count: 1, token_types_start: 781, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 781, raw_values_count: 1, token_types_start: 782, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 782, raw_values_count: 16, token_types_start: 798, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 798, raw_values_count: 2, token_types_start: 800, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 800, raw_values_count: 1, token_types_start: 801, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 801, raw_values_count: 3, token_types_start: 804, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 804, raw_values_count: 2, token_types_start: 806, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 806, raw_values_count: 1, token_types_start: 807, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 807, raw_values_count: 2, token_types_start: 809, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 809, raw_values_count: 17, token_types_start: 826, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 826, raw_values_count: 4, token_types_start: 830, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 830, raw_values_count: 1, token_types_start: 831, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 831, raw_values_count: 1, token_types_start: 832, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 832, raw_values_count: 2, token_types_start: 834, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 834, raw_values_count: 1, token_types_start: 835, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 835, raw_values_count: 3, token_types_start: 838, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 838, raw_values_count: 1, token_types_start: 839, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 839, raw_values_count: 1, token_types_start: 840, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 840, raw_values_count: 6, token_types_start: 846, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 846, raw_values_count: 1, token_types_start: 847, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 847, raw_values_count: 1, token_types_start: 848, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 848, raw_values_count: 3, token_types_start: 851, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 851, raw_values_count: 1, token_types_start: 852, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 852, raw_values_count: 4, token_types_start: 856, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 856, raw_values_count: 1, token_types_start: 857, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 857, raw_values_count: 2, token_types_start: 859, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 859, raw_values_count: 1, token_types_start: 860, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 860, raw_values_count: 5, token_types_start: 865, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 865, raw_values_count: 1, token_types_start: 866, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 866, raw_values_count: 1, token_types_start: 867, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 867, raw_values_count: 1, token_types_start: 868, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 868, raw_values_count: 2, token_types_start: 870, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 870, raw_values_count: 2, token_types_start: 872, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 872, raw_values_count: 2, token_types_start: 874, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 874, raw_values_count: 1, token_types_start: 875, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 875, raw_values_count: 1, token_types_start: 876, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 876, raw_values_count: 1, token_types_start: 877, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 877, raw_values_count: 1, token_types_start: 878, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 878, raw_values_count: 1, token_types_start: 879, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 879, raw_values_count: 10, token_types_start: 889, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 889, raw_values_count: 1, token_types_start: 890, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 890, raw_values_count: 1, token_types_start: 891, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 891, raw_values_count: 1, token_types_start: 892, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 892, raw_values_count: 1, token_types_start: 893, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 893, raw_values_count: 1, token_types_start: 894, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 894, raw_values_count: 1, token_types_start: 895, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 895, raw_values_count: 1, token_types_start: 896, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 896, raw_values_count: 1, token_types_start: 897, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 897, raw_values_count: 1, token_types_start: 898, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 898, raw_values_count: 1, token_types_start: 899, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 899, raw_values_count: 1, token_types_start: 900, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 900, raw_values_count: 1, token_types_start: 901, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 901, raw_values_count: 1, token_types_start: 902, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 902, raw_values_count: 1, token_types_start: 903, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 903, raw_values_count: 1, token_types_start: 904, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 904, raw_values_count: 1, token_types_start: 905, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 905, raw_values_count: 1, token_types_start: 906, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 906, raw_values_count: 1, token_types_start: 907, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 907, raw_values_count: 1, token_types_start: 908, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 908, raw_values_count: 1, token_types_start: 909, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 909, raw_values_count: 1, token_types_start: 910, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 910, raw_values_count: 1, token_types_start: 911, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 911, raw_values_count: 1, token_types_start: 912, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 912, raw_values_count: 1, token_types_start: 913, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 913, raw_values_count: 1, token_types_start: 914, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 914, raw_values_count: 1, token_types_start: 915, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 915, raw_values_count: 1, token_types_start: 916, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 916, raw_values_count: 1, token_types_start: 917, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 917, raw_values_count: 1, token_types_start: 918, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 918, raw_values_count: 1, token_types_start: 919, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 919, raw_values_count: 1, token_types_start: 920, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 920, raw_values_count: 1, token_types_start: 921, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 921, raw_values_count: 1, token_types_start: 922, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 922, raw_values_count: 1, token_types_start: 923, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 923, raw_values_count: 1, token_types_start: 924, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 924, raw_values_count: 17, token_types_start: 941, token_types_count: 5 }, // [404]
    SimpleHintData { raw_values_start: 946, raw_values_count: 2, token_types_start: 948, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 948, raw_values_count: 2, token_types_start: 950, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 950, raw_values_count: 8, token_types_start: 958, token_types_count: 4 }, // [407]
    SimpleHintData { raw_values_start: 962, raw_values_count: 0, token_types_start: 962, token_types_count: 1 }, // [408]
    SimpleHintData { raw_values_start: 963, raw_values_count: 1, token_types_start: 964, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 964, raw_values_count: 1, token_types_start: 965, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 965, raw_values_count: 1, token_types_start: 966, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 966, raw_values_count: 1, token_types_start: 967, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 967, raw_values_count: 1, token_types_start: 968, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 968, raw_values_count: 1, token_types_start: 969, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 969, raw_values_count: 1, token_types_start: 970, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 970, raw_values_count: 1, token_types_start: 971, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 971, raw_values_count: 1, token_types_start: 972, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 972, raw_values_count: 1, token_types_start: 973, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 973, raw_values_count: 1, token_types_start: 974, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 974, raw_values_count: 1, token_types_start: 975, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 975, raw_values_count: 3, token_types_start: 978, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 978, raw_values_count: 1, token_types_start: 979, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 979, raw_values_count: 1, token_types_start: 980, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 980, raw_values_count: 1, token_types_start: 981, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 981, raw_values_count: 1, token_types_start: 982, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 982, raw_values_count: 1, token_types_start: 983, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 983, raw_values_count: 1, token_types_start: 984, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 984, raw_values_count: 1, token_types_start: 985, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 985, raw_values_count: 1, token_types_start: 986, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 986, raw_values_count: 1, token_types_start: 987, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 987, raw_values_count: 1, token_types_start: 988, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 988, raw_values_count: 2, token_types_start: 990, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 990, raw_values_count: 7, token_types_start: 997, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 997, raw_values_count: 2, token_types_start: 999, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 999, raw_values_count: 2, token_types_start: 1001, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 1001, raw_values_count: 1, token_types_start: 1002, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 1002, raw_values_count: 1, token_types_start: 1003, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 1003, raw_values_count: 1, token_types_start: 1004, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 1004, raw_values_count: 1, token_types_start: 1005, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 1005, raw_values_count: 1, token_types_start: 1006, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 1006, raw_values_count: 1, token_types_start: 1007, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 1007, raw_values_count: 1, token_types_start: 1008, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 1008, raw_values_count: 1, token_types_start: 1009, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 1009, raw_values_count: 1, token_types_start: 1010, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 1010, raw_values_count: 1, token_types_start: 1011, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 1011, raw_values_count: 1, token_types_start: 1012, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 1012, raw_values_count: 1, token_types_start: 1013, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 1013, raw_values_count: 1, token_types_start: 1014, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 1014, raw_values_count: 1, token_types_start: 1015, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 1015, raw_values_count: 1, token_types_start: 1016, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 1016, raw_values_count: 1, token_types_start: 1017, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 1017, raw_values_count: 1, token_types_start: 1018, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 1018, raw_values_count: 1, token_types_start: 1019, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 1019, raw_values_count: 1, token_types_start: 1020, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 1020, raw_values_count: 1, token_types_start: 1021, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 1021, raw_values_count: 1, token_types_start: 1022, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 1022, raw_values_count: 1, token_types_start: 1023, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 1023, raw_values_count: 1, token_types_start: 1024, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 1024, raw_values_count: 1, token_types_start: 1025, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 1025, raw_values_count: 1, token_types_start: 1026, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 1026, raw_values_count: 8, token_types_start: 1034, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 1034, raw_values_count: 1, token_types_start: 1035, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 1035, raw_values_count: 1, token_types_start: 1036, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 1036, raw_values_count: 1, token_types_start: 1037, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 1037, raw_values_count: 1, token_types_start: 1038, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 1038, raw_values_count: 1, token_types_start: 1039, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 1039, raw_values_count: 1, token_types_start: 1040, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 1040, raw_values_count: 10, token_types_start: 1050, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 1050, raw_values_count: 1, token_types_start: 1051, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 1051, raw_values_count: 1, token_types_start: 1052, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 1052, raw_values_count: 1, token_types_start: 1053, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 1053, raw_values_count: 33, token_types_start: 1086, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 1086, raw_values_count: 2, token_types_start: 1088, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 1088, raw_values_count: 20, token_types_start: 1108, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 1108, raw_values_count: 3, token_types_start: 1111, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 1111, raw_values_count: 1, token_types_start: 1112, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 1112, raw_values_count: 1, token_types_start: 1113, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 1113, raw_values_count: 1, token_types_start: 1114, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 1114, raw_values_count: 1, token_types_start: 1115, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 1115, raw_values_count: 1, token_types_start: 1116, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 1116, raw_values_count: 2, token_types_start: 1118, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 1118, raw_values_count: 0, token_types_start: 1118, token_types_count: 1 }, // [482]
    SimpleHintData { raw_values_start: 1119, raw_values_count: 0, token_types_start: 1119, token_types_count: 1 }, // [483]
    SimpleHintData { raw_values_start: 1120, raw_values_count: 0, token_types_start: 1120, token_types_count: 1 }, // [484]
    SimpleHintData { raw_values_start: 1121, raw_values_count: 0, token_types_start: 1121, token_types_count: 1 }, // [485]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 1, token_types_start: 1123, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 1123, raw_values_count: 1, token_types_start: 1124, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 1124, raw_values_count: 1, token_types_start: 1125, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 1125, raw_values_count: 1, token_types_start: 1126, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 1126, raw_values_count: 4, token_types_start: 1130, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 1130, raw_values_count: 1, token_types_start: 1131, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 1131, raw_values_count: 2, token_types_start: 1133, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 1133, raw_values_count: 1, token_types_start: 1134, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 1134, raw_values_count: 1, token_types_start: 1135, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 2, token_types_start: 1137, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 1137, raw_values_count: 1, token_types_start: 1138, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 1138, raw_values_count: 1, token_types_start: 1139, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 1139, raw_values_count: 1, token_types_start: 1140, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 1140, raw_values_count: 1, token_types_start: 1141, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 1141, raw_values_count: 1, token_types_start: 1142, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 1142, raw_values_count: 1, token_types_start: 1143, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 1143, raw_values_count: 1, token_types_start: 1144, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 1144, raw_values_count: 1, token_types_start: 1145, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 1145, raw_values_count: 2, token_types_start: 1147, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 4, token_types_start: 1151, token_types_count: 6 }, // [505]
    SimpleHintData { raw_values_start: 1157, raw_values_count: 3, token_types_start: 1160, token_types_count: 6 }, // [506]
    SimpleHintData { raw_values_start: 1166, raw_values_count: 1, token_types_start: 1167, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 1167, raw_values_count: 2, token_types_start: 1169, token_types_count: 6 }, // [508]
    SimpleHintData { raw_values_start: 1175, raw_values_count: 2, token_types_start: 1177, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 1177, raw_values_count: 11, token_types_start: 1188, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 1, token_types_start: 1189, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 1189, raw_values_count: 1, token_types_start: 1190, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 1190, raw_values_count: 1, token_types_start: 1191, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 1191, raw_values_count: 1, token_types_start: 1192, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 1192, raw_values_count: 1, token_types_start: 1193, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 1, token_types_start: 1194, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 1194, raw_values_count: 2, token_types_start: 1196, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 1196, raw_values_count: 1, token_types_start: 1197, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 1197, raw_values_count: 1, token_types_start: 1198, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 1198, raw_values_count: 1, token_types_start: 1199, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 1199, raw_values_count: 3, token_types_start: 1202, token_types_count: 6 }, // [521]
    SimpleHintData { raw_values_start: 1208, raw_values_count: 5, token_types_start: 1213, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 1213, raw_values_count: 4, token_types_start: 1217, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 1217, raw_values_count: 1, token_types_start: 1218, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 1218, raw_values_count: 4, token_types_start: 1222, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 1222, raw_values_count: 4, token_types_start: 1226, token_types_count: 6 }, // [526]
    SimpleHintData { raw_values_start: 1232, raw_values_count: 1, token_types_start: 1233, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 1233, raw_values_count: 3, token_types_start: 1236, token_types_count: 6 }, // [528]
    SimpleHintData { raw_values_start: 1242, raw_values_count: 3, token_types_start: 1245, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 1245, raw_values_count: 1, token_types_start: 1246, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 1246, raw_values_count: 1, token_types_start: 1247, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 1247, raw_values_count: 1, token_types_start: 1248, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 1248, raw_values_count: 1, token_types_start: 1249, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 1249, raw_values_count: 1, token_types_start: 1250, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 1250, raw_values_count: 2, token_types_start: 1252, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 1, token_types_start: 1253, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 1253, raw_values_count: 1, token_types_start: 1254, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 1, token_types_start: 1255, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 1255, raw_values_count: 2, token_types_start: 1257, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 1257, raw_values_count: 6, token_types_start: 1263, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 1263, raw_values_count: 1, token_types_start: 1264, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 1264, raw_values_count: 1, token_types_start: 1265, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 1265, raw_values_count: 1, token_types_start: 1266, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 1266, raw_values_count: 1, token_types_start: 1267, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 1267, raw_values_count: 2, token_types_start: 1269, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 1269, raw_values_count: 1, token_types_start: 1270, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 1270, raw_values_count: 1, token_types_start: 1271, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 1271, raw_values_count: 1, token_types_start: 1272, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 1272, raw_values_count: 1, token_types_start: 1273, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 1273, raw_values_count: 5, token_types_start: 1278, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 1278, raw_values_count: 1, token_types_start: 1279, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 1279, raw_values_count: 1, token_types_start: 1280, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 1280, raw_values_count: 1, token_types_start: 1281, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 1281, raw_values_count: 1, token_types_start: 1282, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 1282, raw_values_count: 1, token_types_start: 1283, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 1283, raw_values_count: 1, token_types_start: 1284, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 1284, raw_values_count: 10, token_types_start: 1294, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 1294, raw_values_count: 1, token_types_start: 1295, token_types_count: 6 }, // [558]
    SimpleHintData { raw_values_start: 1301, raw_values_count: 3, token_types_start: 1304, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 1304, raw_values_count: 2, token_types_start: 1306, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 1306, raw_values_count: 4, token_types_start: 1310, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 1310, raw_values_count: 1, token_types_start: 1311, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 1311, raw_values_count: 1, token_types_start: 1312, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 1312, raw_values_count: 6, token_types_start: 1318, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 2, token_types_start: 1320, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 1, token_types_start: 1321, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 1321, raw_values_count: 2, token_types_start: 1323, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 1323, raw_values_count: 1, token_types_start: 1324, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 1324, raw_values_count: 22, token_types_start: 1346, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 1346, raw_values_count: 2, token_types_start: 1348, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 1348, raw_values_count: 2, token_types_start: 1350, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 1350, raw_values_count: 1, token_types_start: 1351, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 1351, raw_values_count: 1, token_types_start: 1352, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 1352, raw_values_count: 4, token_types_start: 1356, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 1356, raw_values_count: 1, token_types_start: 1357, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 1357, raw_values_count: 2, token_types_start: 1359, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 1359, raw_values_count: 2, token_types_start: 1361, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 1361, raw_values_count: 1, token_types_start: 1362, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 1362, raw_values_count: 1, token_types_start: 1363, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 1363, raw_values_count: 5, token_types_start: 1368, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 1368, raw_values_count: 24, token_types_start: 1392, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 1392, raw_values_count: 23, token_types_start: 1415, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 1415, raw_values_count: 1, token_types_start: 1416, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1416, raw_values_count: 9, token_types_start: 1425, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1425, raw_values_count: 1, token_types_start: 1426, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1426, raw_values_count: 1, token_types_start: 1427, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1427, raw_values_count: 2, token_types_start: 1429, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1429, raw_values_count: 1, token_types_start: 1430, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 1430, raw_values_count: 1, token_types_start: 1431, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1431, raw_values_count: 1, token_types_start: 1432, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1432, raw_values_count: 3, token_types_start: 1435, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1435, raw_values_count: 1, token_types_start: 1436, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1436, raw_values_count: 1, token_types_start: 1437, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1437, raw_values_count: 4, token_types_start: 1441, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1441, raw_values_count: 1, token_types_start: 1442, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1442, raw_values_count: 1, token_types_start: 1443, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1443, raw_values_count: 1, token_types_start: 1444, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1444, raw_values_count: 3, token_types_start: 1447, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1447, raw_values_count: 1, token_types_start: 1448, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1448, raw_values_count: 1, token_types_start: 1449, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1449, raw_values_count: 4, token_types_start: 1453, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1453, raw_values_count: 1, token_types_start: 1454, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1454, raw_values_count: 1, token_types_start: 1455, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1455, raw_values_count: 1, token_types_start: 1456, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1456, raw_values_count: 5, token_types_start: 1461, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1461, raw_values_count: 4, token_types_start: 1465, token_types_count: 0 }, // [606]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 2, token_types_start: 1467, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1467, raw_values_count: 1, token_types_start: 1468, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1468, raw_values_count: 1, token_types_start: 1469, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1469, raw_values_count: 6, token_types_start: 1475, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1475, raw_values_count: 3, token_types_start: 1478, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1478, raw_values_count: 1, token_types_start: 1479, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1479, raw_values_count: 9, token_types_start: 1488, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1488, raw_values_count: 2, token_types_start: 1490, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1490, raw_values_count: 2, token_types_start: 1492, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 3, token_types_start: 1495, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1495, raw_values_count: 1, token_types_start: 1496, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1496, raw_values_count: 2, token_types_start: 1498, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 6, token_types_start: 1504, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1504, raw_values_count: 3, token_types_start: 1507, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1507, raw_values_count: 1, token_types_start: 1508, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1508, raw_values_count: 1, token_types_start: 1509, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1509, raw_values_count: 1, token_types_start: 1510, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 2, token_types_start: 1512, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1512, raw_values_count: 3, token_types_start: 1515, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1515, raw_values_count: 1, token_types_start: 1516, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1516, raw_values_count: 4, token_types_start: 1520, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1520, raw_values_count: 7, token_types_start: 1527, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1527, raw_values_count: 3, token_types_start: 1530, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1530, raw_values_count: 2, token_types_start: 1532, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1532, raw_values_count: 1, token_types_start: 1533, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1533, raw_values_count: 2, token_types_start: 1535, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 1, token_types_start: 1537, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1537, raw_values_count: 1, token_types_start: 1538, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1538, raw_values_count: 2, token_types_start: 1540, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1540, raw_values_count: 1, token_types_start: 1541, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1541, raw_values_count: 2, token_types_start: 1543, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1543, raw_values_count: 1, token_types_start: 1544, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1544, raw_values_count: 4, token_types_start: 1548, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1548, raw_values_count: 1, token_types_start: 1549, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1549, raw_values_count: 1, token_types_start: 1550, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1550, raw_values_count: 1, token_types_start: 1551, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1551, raw_values_count: 1, token_types_start: 1552, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1552, raw_values_count: 1, token_types_start: 1553, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1553, raw_values_count: 2, token_types_start: 1555, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1555, raw_values_count: 1, token_types_start: 1556, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1556, raw_values_count: 1, token_types_start: 1557, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1557, raw_values_count: 1, token_types_start: 1558, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1558, raw_values_count: 1, token_types_start: 1559, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1559, raw_values_count: 1, token_types_start: 1560, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1560, raw_values_count: 1, token_types_start: 1561, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1561, raw_values_count: 38, token_types_start: 1599, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1599, raw_values_count: 6, token_types_start: 1605, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1605, raw_values_count: 55, token_types_start: 1660, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1660, raw_values_count: 1, token_types_start: 1661, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 1, token_types_start: 1662, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1662, raw_values_count: 1, token_types_start: 1663, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1663, raw_values_count: 1, token_types_start: 1664, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1664, raw_values_count: 1, token_types_start: 1665, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1665, raw_values_count: 1, token_types_start: 1666, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1666, raw_values_count: 1, token_types_start: 1667, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1667, raw_values_count: 1, token_types_start: 1668, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1668, raw_values_count: 1, token_types_start: 1669, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1669, raw_values_count: 1, token_types_start: 1670, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1670, raw_values_count: 1, token_types_start: 1671, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 1, token_types_start: 1672, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1672, raw_values_count: 1, token_types_start: 1673, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 1, token_types_start: 1674, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1674, raw_values_count: 1, token_types_start: 1675, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1675, raw_values_count: 1, token_types_start: 1676, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1676, raw_values_count: 1, token_types_start: 1677, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1677, raw_values_count: 2, token_types_start: 1679, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1679, raw_values_count: 1, token_types_start: 1680, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1680, raw_values_count: 1, token_types_start: 1681, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1681, raw_values_count: 1, token_types_start: 1682, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1682, raw_values_count: 5, token_types_start: 1687, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1687, raw_values_count: 4, token_types_start: 1691, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 1, token_types_start: 1692, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 1, token_types_start: 1693, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1693, raw_values_count: 2, token_types_start: 1695, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1695, raw_values_count: 7, token_types_start: 1702, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1702, raw_values_count: 1, token_types_start: 1703, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1703, raw_values_count: 1, token_types_start: 1704, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1704, raw_values_count: 1, token_types_start: 1705, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1705, raw_values_count: 1, token_types_start: 1706, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1706, raw_values_count: 1, token_types_start: 1707, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1707, raw_values_count: 4, token_types_start: 1711, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1711, raw_values_count: 1, token_types_start: 1712, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1712, raw_values_count: 1, token_types_start: 1713, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1713, raw_values_count: 1, token_types_start: 1714, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1714, raw_values_count: 2, token_types_start: 1716, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1716, raw_values_count: 1, token_types_start: 1717, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1717, raw_values_count: 1, token_types_start: 1718, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1718, raw_values_count: 1, token_types_start: 1719, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1719, raw_values_count: 1, token_types_start: 1720, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1720, raw_values_count: 2, token_types_start: 1722, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1722, raw_values_count: 1, token_types_start: 1723, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1723, raw_values_count: 1, token_types_start: 1724, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1724, raw_values_count: 1, token_types_start: 1725, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1725, raw_values_count: 1, token_types_start: 1726, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1726, raw_values_count: 1, token_types_start: 1727, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1727, raw_values_count: 1, token_types_start: 1728, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1728, raw_values_count: 1, token_types_start: 1729, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1729, raw_values_count: 1, token_types_start: 1730, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1730, raw_values_count: 1, token_types_start: 1731, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1731, raw_values_count: 2, token_types_start: 1733, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1733, raw_values_count: 1, token_types_start: 1734, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1734, raw_values_count: 0, token_types_start: 1734, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1734, raw_values_count: 24, token_types_start: 1758, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1758, raw_values_count: 6, token_types_start: 1764, token_types_count: 6 }, // [711]
    SimpleHintData { raw_values_start: 1770, raw_values_count: 2, token_types_start: 1772, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1772, raw_values_count: 3, token_types_start: 1775, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1775, raw_values_count: 1, token_types_start: 1776, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1776, raw_values_count: 1, token_types_start: 1777, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1777, raw_values_count: 1, token_types_start: 1778, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1778, raw_values_count: 1, token_types_start: 1779, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1779, raw_values_count: 1, token_types_start: 1780, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1780, raw_values_count: 1, token_types_start: 1781, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1781, raw_values_count: 1, token_types_start: 1782, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1782, raw_values_count: 1, token_types_start: 1783, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1783, raw_values_count: 1, token_types_start: 1784, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1784, raw_values_count: 1, token_types_start: 1785, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1785, raw_values_count: 1, token_types_start: 1786, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1786, raw_values_count: 1, token_types_start: 1787, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1787, raw_values_count: 1, token_types_start: 1788, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1788, raw_values_count: 1, token_types_start: 1789, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1789, raw_values_count: 1, token_types_start: 1790, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1790, raw_values_count: 1, token_types_start: 1791, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1791, raw_values_count: 1, token_types_start: 1792, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1792, raw_values_count: 1, token_types_start: 1793, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1793, raw_values_count: 1, token_types_start: 1794, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1794, raw_values_count: 1, token_types_start: 1795, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1795, raw_values_count: 1, token_types_start: 1796, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1796, raw_values_count: 1, token_types_start: 1797, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1797, raw_values_count: 1, token_types_start: 1798, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1798, raw_values_count: 17, token_types_start: 1815, token_types_count: 9 }, // [737]
    SimpleHintData { raw_values_start: 1824, raw_values_count: 0, token_types_start: 1824, token_types_count: 1 }, // [738]
    SimpleHintData { raw_values_start: 1825, raw_values_count: 1, token_types_start: 1826, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1826, raw_values_count: 1, token_types_start: 1827, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1827, raw_values_count: 1, token_types_start: 1828, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1828, raw_values_count: 1, token_types_start: 1829, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1829, raw_values_count: 1, token_types_start: 1830, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1830, raw_values_count: 1, token_types_start: 1831, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1831, raw_values_count: 1, token_types_start: 1832, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1832, raw_values_count: 1, token_types_start: 1833, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1833, raw_values_count: 1, token_types_start: 1834, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1834, raw_values_count: 1, token_types_start: 1835, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1835, raw_values_count: 1, token_types_start: 1836, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1836, raw_values_count: 1, token_types_start: 1837, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1837, raw_values_count: 1, token_types_start: 1838, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1838, raw_values_count: 5, token_types_start: 1843, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1843, raw_values_count: 1, token_types_start: 1844, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1844, raw_values_count: 1, token_types_start: 1845, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1845, raw_values_count: 1, token_types_start: 1846, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1846, raw_values_count: 1, token_types_start: 1847, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1847, raw_values_count: 1, token_types_start: 1848, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1848, raw_values_count: 1, token_types_start: 1849, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1849, raw_values_count: 1, token_types_start: 1850, token_types_count: 6 }, // [759]
    SimpleHintData { raw_values_start: 1856, raw_values_count: 1, token_types_start: 1857, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1857, raw_values_count: 0, token_types_start: 1857, token_types_count: 1 }, // [761]
    SimpleHintData { raw_values_start: 1858, raw_values_count: 1, token_types_start: 1859, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1859, raw_values_count: 14, token_types_start: 1873, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1873, raw_values_count: 1, token_types_start: 1874, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1874, raw_values_count: 1, token_types_start: 1875, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1875, raw_values_count: 1, token_types_start: 1876, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1876, raw_values_count: 1, token_types_start: 1877, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1877, raw_values_count: 1, token_types_start: 1878, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1878, raw_values_count: 1, token_types_start: 1879, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1879, raw_values_count: 1, token_types_start: 1880, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1880, raw_values_count: 1, token_types_start: 1881, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1881, raw_values_count: 1, token_types_start: 1882, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1882, raw_values_count: 1, token_types_start: 1883, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1883, raw_values_count: 1, token_types_start: 1884, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1884, raw_values_count: 1, token_types_start: 1885, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1885, raw_values_count: 1, token_types_start: 1886, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1886, raw_values_count: 1, token_types_start: 1887, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1887, raw_values_count: 1, token_types_start: 1888, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1888, raw_values_count: 1, token_types_start: 1889, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1889, raw_values_count: 1, token_types_start: 1890, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1890, raw_values_count: 1, token_types_start: 1891, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1891, raw_values_count: 1, token_types_start: 1892, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1892, raw_values_count: 1, token_types_start: 1893, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1893, raw_values_count: 1, token_types_start: 1894, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1894, raw_values_count: 1, token_types_start: 1895, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1895, raw_values_count: 10, token_types_start: 1905, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1905, raw_values_count: 9, token_types_start: 1914, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1914, raw_values_count: 1, token_types_start: 1915, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1915, raw_values_count: 1, token_types_start: 1916, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1916, raw_values_count: 1, token_types_start: 1917, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1917, raw_values_count: 1, token_types_start: 1918, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1918, raw_values_count: 4, token_types_start: 1922, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1922, raw_values_count: 1, token_types_start: 1923, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1923, raw_values_count: 4, token_types_start: 1927, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1927, raw_values_count: 3, token_types_start: 1930, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1930, raw_values_count: 4, token_types_start: 1934, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1934, raw_values_count: 2, token_types_start: 1936, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1936, raw_values_count: 11, token_types_start: 1947, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1947, raw_values_count: 1, token_types_start: 1948, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1948, raw_values_count: 1, token_types_start: 1949, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1949, raw_values_count: 2, token_types_start: 1951, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1951, raw_values_count: 5, token_types_start: 1956, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1956, raw_values_count: 1, token_types_start: 1957, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1957, raw_values_count: 1, token_types_start: 1958, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1958, raw_values_count: 15, token_types_start: 1973, token_types_count: 4 }, // [805]
    SimpleHintData { raw_values_start: 1977, raw_values_count: 3, token_types_start: 1980, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 1980, raw_values_count: 1, token_types_start: 1981, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1981, raw_values_count: 0, token_types_start: 1981, token_types_count: 1 }, // [808]
    SimpleHintData { raw_values_start: 1982, raw_values_count: 9, token_types_start: 1991, token_types_count: 7 }, // [809]
    SimpleHintData { raw_values_start: 1998, raw_values_count: 1, token_types_start: 1999, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1999, raw_values_count: 1, token_types_start: 2000, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 2000, raw_values_count: 1, token_types_start: 2001, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 2001, raw_values_count: 2, token_types_start: 2003, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 2003, raw_values_count: 1, token_types_start: 2004, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 2004, raw_values_count: 2, token_types_start: 2006, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 2006, raw_values_count: 2, token_types_start: 2008, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 2008, raw_values_count: 1, token_types_start: 2009, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 2009, raw_values_count: 1, token_types_start: 2010, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 2010, raw_values_count: 1, token_types_start: 2011, token_types_count: 1 }, // [819]
    SimpleHintData { raw_values_start: 2012, raw_values_count: 1, token_types_start: 2013, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 2013, raw_values_count: 28, token_types_start: 2041, token_types_count: 3 }, // [821]
    SimpleHintData { raw_values_start: 2044, raw_values_count: 0, token_types_start: 2044, token_types_count: 1 }, // [822]
    SimpleHintData { raw_values_start: 2045, raw_values_count: 0, token_types_start: 2045, token_types_count: 1 }, // [823]
    SimpleHintData { raw_values_start: 2046, raw_values_count: 0, token_types_start: 2046, token_types_count: 1 }, // [824]
    SimpleHintData { raw_values_start: 2047, raw_values_count: 28, token_types_start: 2075, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 2075, raw_values_count: 1, token_types_start: 2076, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 2076, raw_values_count: 1, token_types_start: 2077, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 2077, raw_values_count: 1, token_types_start: 2078, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 2078, raw_values_count: 1, token_types_start: 2079, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 2079, raw_values_count: 4, token_types_start: 2083, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 2083, raw_values_count: 1, token_types_start: 2084, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 2084, raw_values_count: 2, token_types_start: 2086, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 2086, raw_values_count: 1, token_types_start: 2087, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 2087, raw_values_count: 5, token_types_start: 2092, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 2092, raw_values_count: 3, token_types_start: 2095, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 2095, raw_values_count: 1, token_types_start: 2096, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 2096, raw_values_count: 2, token_types_start: 2098, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 2098, raw_values_count: 1, token_types_start: 2099, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 2099, raw_values_count: 4, token_types_start: 2103, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 2103, raw_values_count: 1, token_types_start: 2104, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 2104, raw_values_count: 1, token_types_start: 2105, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 2105, raw_values_count: 1, token_types_start: 2106, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 2106, raw_values_count: 1, token_types_start: 2107, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 2107, raw_values_count: 15, token_types_start: 2122, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 2122, raw_values_count: 1, token_types_start: 2123, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 2123, raw_values_count: 1, token_types_start: 2124, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 2124, raw_values_count: 1, token_types_start: 2125, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 2125, raw_values_count: 4, token_types_start: 2129, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 2129, raw_values_count: 1, token_types_start: 2130, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 2130, raw_values_count: 1, token_types_start: 2131, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 2131, raw_values_count: 1, token_types_start: 2132, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 2132, raw_values_count: 1, token_types_start: 2133, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 2133, raw_values_count: 1, token_types_start: 2134, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 2134, raw_values_count: 1, token_types_start: 2135, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 2135, raw_values_count: 1, token_types_start: 2136, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 2136, raw_values_count: 3, token_types_start: 2139, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 2139, raw_values_count: 2, token_types_start: 2141, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 2141, raw_values_count: 1, token_types_start: 2142, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 2142, raw_values_count: 2, token_types_start: 2144, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 2144, raw_values_count: 2, token_types_start: 2146, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 2146, raw_values_count: 21, token_types_start: 2167, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 2167, raw_values_count: 1, token_types_start: 2168, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 2168, raw_values_count: 7, token_types_start: 2175, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 2175, raw_values_count: 4, token_types_start: 2179, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 2179, raw_values_count: 4, token_types_start: 2183, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 2183, raw_values_count: 1, token_types_start: 2184, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 2184, raw_values_count: 3, token_types_start: 2187, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 2187, raw_values_count: 1, token_types_start: 2188, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 2188, raw_values_count: 1, token_types_start: 2189, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 2189, raw_values_count: 1, token_types_start: 2190, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 2190, raw_values_count: 1, token_types_start: 2191, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 2191, raw_values_count: 1, token_types_start: 2192, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 2192, raw_values_count: 1, token_types_start: 2193, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 2193, raw_values_count: 1, token_types_start: 2194, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 2194, raw_values_count: 0, token_types_start: 2194, token_types_count: 1 }, // [875]
    SimpleHintData { raw_values_start: 2195, raw_values_count: 1, token_types_start: 2196, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 2196, raw_values_count: 1, token_types_start: 2197, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 2197, raw_values_count: 1, token_types_start: 2198, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 2198, raw_values_count: 1, token_types_start: 2199, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 2199, raw_values_count: 1, token_types_start: 2200, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 2200, raw_values_count: 6, token_types_start: 2206, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 2206, raw_values_count: 1, token_types_start: 2207, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 2207, raw_values_count: 1, token_types_start: 2208, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 2208, raw_values_count: 5, token_types_start: 2213, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 2213, raw_values_count: 1, token_types_start: 2214, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 2214, raw_values_count: 1, token_types_start: 2215, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 2215, raw_values_count: 1, token_types_start: 2216, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 2216, raw_values_count: 1, token_types_start: 2217, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 2217, raw_values_count: 1, token_types_start: 2218, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 2218, raw_values_count: 1, token_types_start: 2219, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 2219, raw_values_count: 1, token_types_start: 2220, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 2220, raw_values_count: 1, token_types_start: 2221, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 2221, raw_values_count: 1, token_types_start: 2222, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 2222, raw_values_count: 1, token_types_start: 2223, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 2223, raw_values_count: 1, token_types_start: 2224, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 2224, raw_values_count: 1, token_types_start: 2225, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 2225, raw_values_count: 1, token_types_start: 2226, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 2226, raw_values_count: 1, token_types_start: 2227, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 2227, raw_values_count: 1, token_types_start: 2228, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 2228, raw_values_count: 1, token_types_start: 2229, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 2229, raw_values_count: 1, token_types_start: 2230, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 2230, raw_values_count: 1, token_types_start: 2231, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 2231, raw_values_count: 1, token_types_start: 2232, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 2232, raw_values_count: 1, token_types_start: 2233, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 2233, raw_values_count: 2, token_types_start: 2235, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 2235, raw_values_count: 1, token_types_start: 2236, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 2236, raw_values_count: 1, token_types_start: 2237, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 2237, raw_values_count: 2, token_types_start: 2239, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 2239, raw_values_count: 2, token_types_start: 2241, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 2241, raw_values_count: 1, token_types_start: 2242, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 2242, raw_values_count: 3, token_types_start: 2245, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 2245, raw_values_count: 1, token_types_start: 2246, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 2246, raw_values_count: 1, token_types_start: 2247, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 2247, raw_values_count: 1, token_types_start: 2248, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 2248, raw_values_count: 1, token_types_start: 2249, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 2249, raw_values_count: 1, token_types_start: 2250, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 2250, raw_values_count: 1, token_types_start: 2251, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 2251, raw_values_count: 1, token_types_start: 2252, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 2252, raw_values_count: 1, token_types_start: 2253, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 2253, raw_values_count: 1, token_types_start: 2254, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 2254, raw_values_count: 1, token_types_start: 2255, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 2255, raw_values_count: 2, token_types_start: 2257, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 2257, raw_values_count: 8, token_types_start: 2265, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 2265, raw_values_count: 1, token_types_start: 2266, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 2266, raw_values_count: 12, token_types_start: 2278, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 2278, raw_values_count: 12, token_types_start: 2290, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 2290, raw_values_count: 1, token_types_start: 2291, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 2291, raw_values_count: 1, token_types_start: 2292, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 2292, raw_values_count: 1, token_types_start: 2293, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 2293, raw_values_count: 1, token_types_start: 2294, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 2294, raw_values_count: 1, token_types_start: 2295, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 2295, raw_values_count: 0, token_types_start: 2295, token_types_count: 7 }, // [932]
    SimpleHintData { raw_values_start: 2302, raw_values_count: 24, token_types_start: 2326, token_types_count: 6 }, // [933]
    SimpleHintData { raw_values_start: 2332, raw_values_count: 6, token_types_start: 2338, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 2338, raw_values_count: 1, token_types_start: 2339, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 2339, raw_values_count: 1, token_types_start: 2340, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 2340, raw_values_count: 1, token_types_start: 2341, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 2341, raw_values_count: 1, token_types_start: 2342, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 2342, raw_values_count: 3, token_types_start: 2345, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 2345, raw_values_count: 1, token_types_start: 2346, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 2346, raw_values_count: 1, token_types_start: 2347, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 2347, raw_values_count: 1, token_types_start: 2348, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 2348, raw_values_count: 5, token_types_start: 2353, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 2353, raw_values_count: 1, token_types_start: 2354, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 2354, raw_values_count: 1, token_types_start: 2355, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 2355, raw_values_count: 1, token_types_start: 2356, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 2356, raw_values_count: 1, token_types_start: 2357, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 2357, raw_values_count: 2, token_types_start: 2359, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 2359, raw_values_count: 3, token_types_start: 2362, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 2362, raw_values_count: 1, token_types_start: 2363, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 2363, raw_values_count: 1, token_types_start: 2364, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 2364, raw_values_count: 1, token_types_start: 2365, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 2365, raw_values_count: 4, token_types_start: 2369, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 2369, raw_values_count: 1, token_types_start: 2370, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 2370, raw_values_count: 1, token_types_start: 2371, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 2371, raw_values_count: 1, token_types_start: 2372, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 2372, raw_values_count: 1, token_types_start: 2373, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 2373, raw_values_count: 1, token_types_start: 2374, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 2374, raw_values_count: 1, token_types_start: 2375, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 2375, raw_values_count: 0, token_types_start: 2375, token_types_count: 1 }, // [960]
    SimpleHintData { raw_values_start: 2376, raw_values_count: 1, token_types_start: 2377, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 2377, raw_values_count: 1, token_types_start: 2378, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 2378, raw_values_count: 1, token_types_start: 2379, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 2379, raw_values_count: 1, token_types_start: 2380, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 2380, raw_values_count: 1, token_types_start: 2381, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 2381, raw_values_count: 1, token_types_start: 2382, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 2382, raw_values_count: 1, token_types_start: 2383, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 2383, raw_values_count: 3, token_types_start: 2386, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 2386, raw_values_count: 1, token_types_start: 2387, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 2387, raw_values_count: 9, token_types_start: 2396, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 2396, raw_values_count: 2, token_types_start: 2398, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 2398, raw_values_count: 1, token_types_start: 2399, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 2399, raw_values_count: 1, token_types_start: 2400, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 2400, raw_values_count: 1, token_types_start: 2401, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 2401, raw_values_count: 1, token_types_start: 2402, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 2402, raw_values_count: 1, token_types_start: 2403, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 2403, raw_values_count: 1, token_types_start: 2404, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 2404, raw_values_count: 1, token_types_start: 2405, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 2405, raw_values_count: 2, token_types_start: 2407, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 2407, raw_values_count: 1, token_types_start: 2408, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 2408, raw_values_count: 1, token_types_start: 2409, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 2409, raw_values_count: 1, token_types_start: 2410, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 2410, raw_values_count: 1, token_types_start: 2411, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 2411, raw_values_count: 1, token_types_start: 2412, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 2412, raw_values_count: 4, token_types_start: 2416, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 2416, raw_values_count: 1, token_types_start: 2417, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 2417, raw_values_count: 1, token_types_start: 2418, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 2418, raw_values_count: 1, token_types_start: 2419, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 2419, raw_values_count: 1, token_types_start: 2420, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 2420, raw_values_count: 1, token_types_start: 2421, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 2421, raw_values_count: 1, token_types_start: 2422, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 2422, raw_values_count: 1, token_types_start: 2423, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 2423, raw_values_count: 1, token_types_start: 2424, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 2424, raw_values_count: 1, token_types_start: 2425, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 2425, raw_values_count: 1, token_types_start: 2426, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 2426, raw_values_count: 1, token_types_start: 2427, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 2427, raw_values_count: 1, token_types_start: 2428, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 2428, raw_values_count: 1, token_types_start: 2429, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 2429, raw_values_count: 1, token_types_start: 2430, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 2430, raw_values_count: 1, token_types_start: 2431, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 2431, raw_values_count: 5, token_types_start: 2436, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 2436, raw_values_count: 4, token_types_start: 2440, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 2440, raw_values_count: 1, token_types_start: 2441, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 2441, raw_values_count: 1, token_types_start: 2442, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 2442, raw_values_count: 1, token_types_start: 2443, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 2443, raw_values_count: 1, token_types_start: 2444, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 2444, raw_values_count: 2, token_types_start: 2446, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 2446, raw_values_count: 1, token_types_start: 2447, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 2447, raw_values_count: 1, token_types_start: 2448, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 2448, raw_values_count: 1, token_types_start: 2449, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 2449, raw_values_count: 1, token_types_start: 2450, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 2450, raw_values_count: 1, token_types_start: 2451, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 2451, raw_values_count: 1, token_types_start: 2452, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 2452, raw_values_count: 1, token_types_start: 2453, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 2453, raw_values_count: 1, token_types_start: 2454, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 2454, raw_values_count: 1, token_types_start: 2455, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 2455, raw_values_count: 1, token_types_start: 2456, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 2456, raw_values_count: 3, token_types_start: 2459, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 2459, raw_values_count: 1, token_types_start: 2460, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 2460, raw_values_count: 2, token_types_start: 2462, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 2462, raw_values_count: 1, token_types_start: 2463, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 2463, raw_values_count: 0, token_types_start: 2463, token_types_count: 1 }, // [1022]
    SimpleHintData { raw_values_start: 2464, raw_values_count: 1, token_types_start: 2465, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 2465, raw_values_count: 1, token_types_start: 2466, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 2466, raw_values_count: 1, token_types_start: 2467, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 2467, raw_values_count: 1, token_types_start: 2468, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 2468, raw_values_count: 1, token_types_start: 2469, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 2469, raw_values_count: 1, token_types_start: 2470, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 2470, raw_values_count: 1, token_types_start: 2471, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 2471, raw_values_count: 1, token_types_start: 2472, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 2472, raw_values_count: 1, token_types_start: 2473, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 2473, raw_values_count: 1, token_types_start: 2474, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 2474, raw_values_count: 1, token_types_start: 2475, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 2475, raw_values_count: 1, token_types_start: 2476, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 2476, raw_values_count: 1, token_types_start: 2477, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 2477, raw_values_count: 1, token_types_start: 2478, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 2478, raw_values_count: 1, token_types_start: 2479, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 2479, raw_values_count: 1, token_types_start: 2480, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 2480, raw_values_count: 0, token_types_start: 2480, token_types_count: 1 }, // [1039]
    SimpleHintData { raw_values_start: 2481, raw_values_count: 1, token_types_start: 2482, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 2482, raw_values_count: 1, token_types_start: 2483, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 2483, raw_values_count: 1, token_types_start: 2484, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 2484, raw_values_count: 1, token_types_start: 2485, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 2485, raw_values_count: 1, token_types_start: 2486, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 2486, raw_values_count: 1, token_types_start: 2487, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 2487, raw_values_count: 1, token_types_start: 2488, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 2488, raw_values_count: 1, token_types_start: 2489, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 2489, raw_values_count: 1, token_types_start: 2490, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 2490, raw_values_count: 1, token_types_start: 2491, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 2491, raw_values_count: 1, token_types_start: 2492, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 2492, raw_values_count: 1, token_types_start: 2493, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 2493, raw_values_count: 1, token_types_start: 2494, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 2494, raw_values_count: 1, token_types_start: 2495, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 2495, raw_values_count: 3, token_types_start: 2498, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 2498, raw_values_count: 1, token_types_start: 2499, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 2499, raw_values_count: 1, token_types_start: 2500, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 2500, raw_values_count: 1, token_types_start: 2501, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 2501, raw_values_count: 1, token_types_start: 2502, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 2502, raw_values_count: 1, token_types_start: 2503, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 2503, raw_values_count: 1, token_types_start: 2504, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 2504, raw_values_count: 1, token_types_start: 2505, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 2505, raw_values_count: 1, token_types_start: 2506, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 2506, raw_values_count: 1, token_types_start: 2507, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 2507, raw_values_count: 1, token_types_start: 2508, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 2508, raw_values_count: 1, token_types_start: 2509, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 2509, raw_values_count: 1, token_types_start: 2510, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 2510, raw_values_count: 1, token_types_start: 2511, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 2511, raw_values_count: 1, token_types_start: 2512, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 2512, raw_values_count: 1, token_types_start: 2513, token_types_count: 0 }, // [1069]
    SimpleHintData { raw_values_start: 2513, raw_values_count: 1, token_types_start: 2514, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 2514, raw_values_count: 6, token_types_start: 2520, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 2520, raw_values_count: 2, token_types_start: 2522, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 2522, raw_values_count: 1, token_types_start: 2523, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 2523, raw_values_count: 1, token_types_start: 2524, token_types_count: 0 }, // [1074]
    SimpleHintData { raw_values_start: 2524, raw_values_count: 1, token_types_start: 2525, token_types_count: 0 }, // [1075]
    SimpleHintData { raw_values_start: 2525, raw_values_count: 4, token_types_start: 2529, token_types_count: 0 }, // [1076]
    SimpleHintData { raw_values_start: 2529, raw_values_count: 1, token_types_start: 2530, token_types_count: 0 }, // [1077]
    SimpleHintData { raw_values_start: 2530, raw_values_count: 1, token_types_start: 2531, token_types_count: 0 }, // [1078]
    SimpleHintData { raw_values_start: 2531, raw_values_count: 1, token_types_start: 2532, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 2532, raw_values_count: 1, token_types_start: 2533, token_types_count: 0 }, // [1080]
    SimpleHintData { raw_values_start: 2533, raw_values_count: 1, token_types_start: 2534, token_types_count: 0 }, // [1081]
    SimpleHintData { raw_values_start: 2534, raw_values_count: 1, token_types_start: 2535, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 2535, raw_values_count: 1, token_types_start: 2536, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 2536, raw_values_count: 1, token_types_start: 2537, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 2537, raw_values_count: 1, token_types_start: 2538, token_types_count: 0 }, // [1085]
    SimpleHintData { raw_values_start: 2538, raw_values_count: 1, token_types_start: 2539, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 2539, raw_values_count: 1, token_types_start: 2540, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 2540, raw_values_count: 1, token_types_start: 2541, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 2541, raw_values_count: 1, token_types_start: 2542, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 2542, raw_values_count: 1, token_types_start: 2543, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 2543, raw_values_count: 1, token_types_start: 2544, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 2544, raw_values_count: 1, token_types_start: 2545, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 2545, raw_values_count: 1, token_types_start: 2546, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 2546, raw_values_count: 1, token_types_start: 2547, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 2547, raw_values_count: 1, token_types_start: 2548, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 2548, raw_values_count: 1, token_types_start: 2549, token_types_count: 0 }, // [1096]
    SimpleHintData { raw_values_start: 2549, raw_values_count: 0, token_types_start: 2549, token_types_count: 7 }, // [1097]
    SimpleHintData { raw_values_start: 2556, raw_values_count: 3, token_types_start: 2559, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 2559, raw_values_count: 10, token_types_start: 2569, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 2569, raw_values_count: 2, token_types_start: 2571, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 2571, raw_values_count: 1, token_types_start: 2572, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 2572, raw_values_count: 1, token_types_start: 2573, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 2573, raw_values_count: 1, token_types_start: 2574, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 2574, raw_values_count: 1, token_types_start: 2575, token_types_count: 0 }, // [1104]
    SimpleHintData { raw_values_start: 2575, raw_values_count: 1, token_types_start: 2576, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 2576, raw_values_count: 1, token_types_start: 2577, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 2577, raw_values_count: 1, token_types_start: 2578, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 2578, raw_values_count: 1, token_types_start: 2579, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 2579, raw_values_count: 1, token_types_start: 2580, token_types_count: 0 }, // [1109]
    SimpleHintData { raw_values_start: 2580, raw_values_count: 1, token_types_start: 2581, token_types_count: 0 }, // [1110]
    SimpleHintData { raw_values_start: 2581, raw_values_count: 1, token_types_start: 2582, token_types_count: 0 }, // [1111]
    SimpleHintData { raw_values_start: 2582, raw_values_count: 1, token_types_start: 2583, token_types_count: 0 }, // [1112]
    SimpleHintData { raw_values_start: 2583, raw_values_count: 1, token_types_start: 2584, token_types_count: 0 }, // [1113]
    SimpleHintData { raw_values_start: 2584, raw_values_count: 1, token_types_start: 2585, token_types_count: 6 }, // [1114]
    SimpleHintData { raw_values_start: 2591, raw_values_count: 1, token_types_start: 2592, token_types_count: 0 }, // [1115]
    SimpleHintData { raw_values_start: 2592, raw_values_count: 1, token_types_start: 2593, token_types_count: 0 }, // [1116]
    SimpleHintData { raw_values_start: 2593, raw_values_count: 1, token_types_start: 2594, token_types_count: 0 }, // [1117]
    SimpleHintData { raw_values_start: 2594, raw_values_count: 1, token_types_start: 2595, token_types_count: 0 }, // [1118]
    SimpleHintData { raw_values_start: 2595, raw_values_count: 1, token_types_start: 2596, token_types_count: 0 }, // [1119]
    SimpleHintData { raw_values_start: 2596, raw_values_count: 1, token_types_start: 2597, token_types_count: 0 }, // [1120]
    SimpleHintData { raw_values_start: 2597, raw_values_count: 1, token_types_start: 2598, token_types_count: 0 }, // [1121]
    SimpleHintData { raw_values_start: 2598, raw_values_count: 1, token_types_start: 2599, token_types_count: 0 }, // [1122]
    SimpleHintData { raw_values_start: 2599, raw_values_count: 1, token_types_start: 2600, token_types_count: 0 }, // [1123]
    SimpleHintData { raw_values_start: 2600, raw_values_count: 1, token_types_start: 2601, token_types_count: 0 }, // [1124]
    SimpleHintData { raw_values_start: 2601, raw_values_count: 1, token_types_start: 2602, token_types_count: 0 }, // [1125]
    SimpleHintData { raw_values_start: 2602, raw_values_count: 1, token_types_start: 2603, token_types_count: 0 }, // [1126]
    SimpleHintData { raw_values_start: 2603, raw_values_count: 1, token_types_start: 2604, token_types_count: 0 }, // [1127]
    SimpleHintData { raw_values_start: 2604, raw_values_count: 1, token_types_start: 2605, token_types_count: 0 }, // [1128]
    SimpleHintData { raw_values_start: 2605, raw_values_count: 1, token_types_start: 2606, token_types_count: 0 }, // [1129]
    SimpleHintData { raw_values_start: 2606, raw_values_count: 1, token_types_start: 2607, token_types_count: 0 }, // [1130]
    SimpleHintData { raw_values_start: 2607, raw_values_count: 8, token_types_start: 2615, token_types_count: 0 }, // [1131]
    SimpleHintData { raw_values_start: 2615, raw_values_count: 1, token_types_start: 2616, token_types_count: 0 }, // [1132]
    SimpleHintData { raw_values_start: 2616, raw_values_count: 1, token_types_start: 2617, token_types_count: 0 }, // [1133]
    SimpleHintData { raw_values_start: 2617, raw_values_count: 1, token_types_start: 2618, token_types_count: 0 }, // [1134]
    SimpleHintData { raw_values_start: 2618, raw_values_count: 1, token_types_start: 2619, token_types_count: 0 }, // [1135]
    SimpleHintData { raw_values_start: 2619, raw_values_count: 1, token_types_start: 2620, token_types_count: 0 }, // [1136]
    SimpleHintData { raw_values_start: 2620, raw_values_count: 1, token_types_start: 2621, token_types_count: 0 }, // [1137]
    SimpleHintData { raw_values_start: 2621, raw_values_count: 1, token_types_start: 2622, token_types_count: 0 }, // [1138]
    SimpleHintData { raw_values_start: 2622, raw_values_count: 1, token_types_start: 2623, token_types_count: 0 }, // [1139]
    SimpleHintData { raw_values_start: 2623, raw_values_count: 1, token_types_start: 2624, token_types_count: 0 }, // [1140]
    SimpleHintData { raw_values_start: 2624, raw_values_count: 3, token_types_start: 2627, token_types_count: 0 }, // [1141]
    SimpleHintData { raw_values_start: 2627, raw_values_count: 1, token_types_start: 2628, token_types_count: 0 }, // [1142]
    SimpleHintData { raw_values_start: 2628, raw_values_count: 1, token_types_start: 2629, token_types_count: 0 }, // [1143]
    SimpleHintData { raw_values_start: 2629, raw_values_count: 1, token_types_start: 2630, token_types_count: 0 }, // [1144]
    SimpleHintData { raw_values_start: 2630, raw_values_count: 0, token_types_start: 2630, token_types_count: 1 }, // [1145]
    SimpleHintData { raw_values_start: 2631, raw_values_count: 0, token_types_start: 2631, token_types_count: 1 }, // [1146]
    SimpleHintData { raw_values_start: 2632, raw_values_count: 0, token_types_start: 2632, token_types_count: 2 }, // [1147]
    SimpleHintData { raw_values_start: 2634, raw_values_count: 0, token_types_start: 2634, token_types_count: 1 }, // [1148]
    SimpleHintData { raw_values_start: 2635, raw_values_count: 0, token_types_start: 2635, token_types_count: 2 }, // [1149]
    SimpleHintData { raw_values_start: 2637, raw_values_count: 0, token_types_start: 2637, token_types_count: 1 }, // [1150]
    SimpleHintData { raw_values_start: 2638, raw_values_count: 0, token_types_start: 2638, token_types_count: 1 }, // [1151]
    SimpleHintData { raw_values_start: 2639, raw_values_count: 1, token_types_start: 2640, token_types_count: 0 }, // [1152]
    SimpleHintData { raw_values_start: 2640, raw_values_count: 0, token_types_start: 2640, token_types_count: 2 }, // [1153]
    SimpleHintData { raw_values_start: 2642, raw_values_count: 0, token_types_start: 2642, token_types_count: 1 }, // [1154]
    SimpleHintData { raw_values_start: 2643, raw_values_count: 1, token_types_start: 2644, token_types_count: 0 }, // [1155]
    SimpleHintData { raw_values_start: 2644, raw_values_count: 1, token_types_start: 2645, token_types_count: 0 }, // [1156]
    SimpleHintData { raw_values_start: 2645, raw_values_count: 1, token_types_start: 2646, token_types_count: 0 }, // [1157]
    SimpleHintData { raw_values_start: 2646, raw_values_count: 1, token_types_start: 2647, token_types_count: 0 }, // [1158]
    SimpleHintData { raw_values_start: 2647, raw_values_count: 1, token_types_start: 2648, token_types_count: 0 }, // [1159]
    SimpleHintData { raw_values_start: 2648, raw_values_count: 1, token_types_start: 2649, token_types_count: 0 }, // [1160]
    SimpleHintData { raw_values_start: 2649, raw_values_count: 1, token_types_start: 2650, token_types_count: 0 }, // [1161]
    SimpleHintData { raw_values_start: 2650, raw_values_count: 1, token_types_start: 2651, token_types_count: 0 }, // [1162]
    SimpleHintData { raw_values_start: 2651, raw_values_count: 1, token_types_start: 2652, token_types_count: 0 }, // [1163]
    SimpleHintData { raw_values_start: 2652, raw_values_count: 1, token_types_start: 2653, token_types_count: 0 }, // [1164]
    SimpleHintData { raw_values_start: 2653, raw_values_count: 4, token_types_start: 2657, token_types_count: 0 }, // [1165]
    SimpleHintData { raw_values_start: 2657, raw_values_count: 3, token_types_start: 2660, token_types_count: 0 }, // [1166]
    SimpleHintData { raw_values_start: 2660, raw_values_count: 1, token_types_start: 2661, token_types_count: 0 }, // [1167]
    SimpleHintData { raw_values_start: 2661, raw_values_count: 1, token_types_start: 2662, token_types_count: 0 }, // [1168]
    SimpleHintData { raw_values_start: 2662, raw_values_count: 2, token_types_start: 2664, token_types_count: 0 }, // [1169]
    SimpleHintData { raw_values_start: 2664, raw_values_count: 1, token_types_start: 2665, token_types_count: 0 }, // [1170]
    SimpleHintData { raw_values_start: 2665, raw_values_count: 1, token_types_start: 2666, token_types_count: 0 }, // [1171]
    SimpleHintData { raw_values_start: 2666, raw_values_count: 1, token_types_start: 2667, token_types_count: 0 }, // [1172]
    SimpleHintData { raw_values_start: 2667, raw_values_count: 1, token_types_start: 2668, token_types_count: 0 }, // [1173]
    SimpleHintData { raw_values_start: 2668, raw_values_count: 5, token_types_start: 2673, token_types_count: 0 }, // [1174]
    SimpleHintData { raw_values_start: 2673, raw_values_count: 2, token_types_start: 2675, token_types_count: 0 }, // [1175]
    SimpleHintData { raw_values_start: 2675, raw_values_count: 1, token_types_start: 2676, token_types_count: 0 }, // [1176]
    SimpleHintData { raw_values_start: 2676, raw_values_count: 1, token_types_start: 2677, token_types_count: 0 }, // [1177]
    SimpleHintData { raw_values_start: 2677, raw_values_count: 1, token_types_start: 2678, token_types_count: 0 }, // [1178]
    SimpleHintData { raw_values_start: 2678, raw_values_count: 1, token_types_start: 2679, token_types_count: 0 }, // [1179]
    SimpleHintData { raw_values_start: 2679, raw_values_count: 1, token_types_start: 2680, token_types_count: 0 }, // [1180]
    SimpleHintData { raw_values_start: 2680, raw_values_count: 1, token_types_start: 2681, token_types_count: 0 }, // [1181]
    SimpleHintData { raw_values_start: 2681, raw_values_count: 1, token_types_start: 2682, token_types_count: 0 }, // [1182]
    SimpleHintData { raw_values_start: 2682, raw_values_count: 1, token_types_start: 2683, token_types_count: 0 }, // [1183]
    SimpleHintData { raw_values_start: 2683, raw_values_count: 1, token_types_start: 2684, token_types_count: 0 }, // [1184]
    SimpleHintData { raw_values_start: 2684, raw_values_count: 1, token_types_start: 2685, token_types_count: 0 }, // [1185]
    SimpleHintData { raw_values_start: 2685, raw_values_count: 1, token_types_start: 2686, token_types_count: 0 }, // [1186]
    SimpleHintData { raw_values_start: 2686, raw_values_count: 1, token_types_start: 2687, token_types_count: 0 }, // [1187]
    SimpleHintData { raw_values_start: 2687, raw_values_count: 1, token_types_start: 2688, token_types_count: 0 }, // [1188]
    SimpleHintData { raw_values_start: 2688, raw_values_count: 1, token_types_start: 2689, token_types_count: 0 }, // [1189]
    SimpleHintData { raw_values_start: 2689, raw_values_count: 1, token_types_start: 2690, token_types_count: 0 }, // [1190]
    SimpleHintData { raw_values_start: 2690, raw_values_count: 1, token_types_start: 2691, token_types_count: 0 }, // [1191]
    SimpleHintData { raw_values_start: 2691, raw_values_count: 1, token_types_start: 2692, token_types_count: 0 }, // [1192]
    SimpleHintData { raw_values_start: 2692, raw_values_count: 1, token_types_start: 2693, token_types_count: 0 }, // [1193]
    SimpleHintData { raw_values_start: 2693, raw_values_count: 1, token_types_start: 2694, token_types_count: 0 }, // [1194]
    SimpleHintData { raw_values_start: 2694, raw_values_count: 2, token_types_start: 2696, token_types_count: 0 }, // [1195]
    SimpleHintData { raw_values_start: 2696, raw_values_count: 1, token_types_start: 2697, token_types_count: 0 }, // [1196]
    SimpleHintData { raw_values_start: 2697, raw_values_count: 1, token_types_start: 2698, token_types_count: 0 }, // [1197]
    SimpleHintData { raw_values_start: 2698, raw_values_count: 1, token_types_start: 2699, token_types_count: 0 }, // [1198]
    SimpleHintData { raw_values_start: 2699, raw_values_count: 1, token_types_start: 2700, token_types_count: 0 }, // [1199]
    SimpleHintData { raw_values_start: 2700, raw_values_count: 1, token_types_start: 2701, token_types_count: 0 }, // [1200]
    SimpleHintData { raw_values_start: 2701, raw_values_count: 1, token_types_start: 2702, token_types_count: 0 }, // [1201]
    SimpleHintData { raw_values_start: 2702, raw_values_count: 1, token_types_start: 2703, token_types_count: 0 }, // [1202]
    SimpleHintData { raw_values_start: 2703, raw_values_count: 1, token_types_start: 2704, token_types_count: 0 }, // [1203]
    SimpleHintData { raw_values_start: 2704, raw_values_count: 1, token_types_start: 2705, token_types_count: 0 }, // [1204]
    SimpleHintData { raw_values_start: 2705, raw_values_count: 22, token_types_start: 2727, token_types_count: 0 }, // [1205]
    SimpleHintData { raw_values_start: 2727, raw_values_count: 1, token_types_start: 2728, token_types_count: 0 }, // [1206]
    SimpleHintData { raw_values_start: 2728, raw_values_count: 2, token_types_start: 2730, token_types_count: 0 }, // [1207]
    SimpleHintData { raw_values_start: 2730, raw_values_count: 11, token_types_start: 2741, token_types_count: 1 }, // [1208]
    SimpleHintData { raw_values_start: 2742, raw_values_count: 1, token_types_start: 2743, token_types_count: 0 }, // [1209]
    SimpleHintData { raw_values_start: 2743, raw_values_count: 1, token_types_start: 2744, token_types_count: 0 }, // [1210]
    SimpleHintData { raw_values_start: 2744, raw_values_count: 1, token_types_start: 2745, token_types_count: 0 }, // [1211]
    SimpleHintData { raw_values_start: 2745, raw_values_count: 1, token_types_start: 2746, token_types_count: 0 }, // [1212]
    SimpleHintData { raw_values_start: 2746, raw_values_count: 2, token_types_start: 2748, token_types_count: 0 }, // [1213]
    SimpleHintData { raw_values_start: 2748, raw_values_count: 1, token_types_start: 2749, token_types_count: 0 }, // [1214]
    SimpleHintData { raw_values_start: 2749, raw_values_count: 2, token_types_start: 2751, token_types_count: 0 }, // [1215]
    SimpleHintData { raw_values_start: 2751, raw_values_count: 2, token_types_start: 2753, token_types_count: 0 }, // [1216]
    SimpleHintData { raw_values_start: 2753, raw_values_count: 1, token_types_start: 2754, token_types_count: 0 }, // [1217]
    SimpleHintData { raw_values_start: 2754, raw_values_count: 1, token_types_start: 2755, token_types_count: 0 }, // [1218]
    SimpleHintData { raw_values_start: 2755, raw_values_count: 2, token_types_start: 2757, token_types_count: 6 }, // [1219]
    SimpleHintData { raw_values_start: 2763, raw_values_count: 1, token_types_start: 2764, token_types_count: 0 }, // [1220]
    SimpleHintData { raw_values_start: 2764, raw_values_count: 1, token_types_start: 2765, token_types_count: 0 }, // [1221]
    SimpleHintData { raw_values_start: 2765, raw_values_count: 1, token_types_start: 2766, token_types_count: 0 }, // [1222]
    SimpleHintData { raw_values_start: 2766, raw_values_count: 7, token_types_start: 2773, token_types_count: 0 }, // [1223]
    SimpleHintData { raw_values_start: 2773, raw_values_count: 2, token_types_start: 2775, token_types_count: 0 }, // [1224]
    SimpleHintData { raw_values_start: 2775, raw_values_count: 15, token_types_start: 2790, token_types_count: 0 }, // [1225]
    SimpleHintData { raw_values_start: 2790, raw_values_count: 1, token_types_start: 2791, token_types_count: 0 }, // [1226]
    SimpleHintData { raw_values_start: 2791, raw_values_count: 1, token_types_start: 2792, token_types_count: 0 }, // [1227]
    SimpleHintData { raw_values_start: 2792, raw_values_count: 1, token_types_start: 2793, token_types_count: 0 }, // [1228]
    SimpleHintData { raw_values_start: 2793, raw_values_count: 1, token_types_start: 2794, token_types_count: 0 }, // [1229]
    SimpleHintData { raw_values_start: 2794, raw_values_count: 1, token_types_start: 2795, token_types_count: 0 }, // [1230]
    SimpleHintData { raw_values_start: 2795, raw_values_count: 1, token_types_start: 2796, token_types_count: 0 }, // [1231]
    SimpleHintData { raw_values_start: 2796, raw_values_count: 1, token_types_start: 2797, token_types_count: 0 }, // [1232]
    SimpleHintData { raw_values_start: 2797, raw_values_count: 1, token_types_start: 2798, token_types_count: 0 }, // [1233]
    SimpleHintData { raw_values_start: 2798, raw_values_count: 1, token_types_start: 2799, token_types_count: 0 }, // [1234]
    SimpleHintData { raw_values_start: 2799, raw_values_count: 1, token_types_start: 2800, token_types_count: 0 }, // [1235]
    SimpleHintData { raw_values_start: 2800, raw_values_count: 1, token_types_start: 2801, token_types_count: 0 }, // [1236]
    SimpleHintData { raw_values_start: 2801, raw_values_count: 1, token_types_start: 2802, token_types_count: 0 }, // [1237]
    SimpleHintData { raw_values_start: 2802, raw_values_count: 1, token_types_start: 2803, token_types_count: 0 }, // [1238]
    SimpleHintData { raw_values_start: 2803, raw_values_count: 1, token_types_start: 2804, token_types_count: 0 }, // [1239]
    SimpleHintData { raw_values_start: 2804, raw_values_count: 1, token_types_start: 2805, token_types_count: 0 }, // [1240]
    SimpleHintData { raw_values_start: 2805, raw_values_count: 1, token_types_start: 2806, token_types_count: 0 }, // [1241]
    SimpleHintData { raw_values_start: 2806, raw_values_count: 1, token_types_start: 2807, token_types_count: 0 }, // [1242]
    SimpleHintData { raw_values_start: 2807, raw_values_count: 1, token_types_start: 2808, token_types_count: 0 }, // [1243]
    SimpleHintData { raw_values_start: 2808, raw_values_count: 1, token_types_start: 2809, token_types_count: 0 }, // [1244]
    SimpleHintData { raw_values_start: 2809, raw_values_count: 1, token_types_start: 2810, token_types_count: 0 }, // [1245]
    SimpleHintData { raw_values_start: 2810, raw_values_count: 1, token_types_start: 2811, token_types_count: 0 }, // [1246]
    SimpleHintData { raw_values_start: 2811, raw_values_count: 1, token_types_start: 2812, token_types_count: 0 }, // [1247]
    SimpleHintData { raw_values_start: 2812, raw_values_count: 1, token_types_start: 2813, token_types_count: 0 }, // [1248]
    SimpleHintData { raw_values_start: 2813, raw_values_count: 1, token_types_start: 2814, token_types_count: 0 }, // [1249]
    SimpleHintData { raw_values_start: 2814, raw_values_count: 5, token_types_start: 2819, token_types_count: 0 }, // [1250]
    SimpleHintData { raw_values_start: 2819, raw_values_count: 1, token_types_start: 2820, token_types_count: 0 }, // [1251]
    SimpleHintData { raw_values_start: 2820, raw_values_count: 1, token_types_start: 2821, token_types_count: 0 }, // [1252]
    SimpleHintData { raw_values_start: 2821, raw_values_count: 1, token_types_start: 2822, token_types_count: 0 }, // [1253]
    SimpleHintData { raw_values_start: 2822, raw_values_count: 1, token_types_start: 2823, token_types_count: 0 }, // [1254]
    SimpleHintData { raw_values_start: 2823, raw_values_count: 1, token_types_start: 2824, token_types_count: 0 }, // [1255]
    SimpleHintData { raw_values_start: 2824, raw_values_count: 1, token_types_start: 2825, token_types_count: 0 }, // [1256]
    SimpleHintData { raw_values_start: 2825, raw_values_count: 1, token_types_start: 2826, token_types_count: 0 }, // [1257]
    SimpleHintData { raw_values_start: 2826, raw_values_count: 1, token_types_start: 2827, token_types_count: 0 }, // [1258]
    SimpleHintData { raw_values_start: 2827, raw_values_count: 1, token_types_start: 2828, token_types_count: 0 }, // [1259]
    SimpleHintData { raw_values_start: 2828, raw_values_count: 1, token_types_start: 2829, token_types_count: 0 }, // [1260]
    SimpleHintData { raw_values_start: 2829, raw_values_count: 1, token_types_start: 2830, token_types_count: 0 }, // [1261]
    SimpleHintData { raw_values_start: 2830, raw_values_count: 1, token_types_start: 2831, token_types_count: 0 }, // [1262]
    SimpleHintData { raw_values_start: 2831, raw_values_count: 5, token_types_start: 2836, token_types_count: 0 }, // [1263]
    SimpleHintData { raw_values_start: 2836, raw_values_count: 4, token_types_start: 2840, token_types_count: 0 }, // [1264]
    SimpleHintData { raw_values_start: 2840, raw_values_count: 2, token_types_start: 2842, token_types_count: 0 }, // [1265]
    SimpleHintData { raw_values_start: 2842, raw_values_count: 1, token_types_start: 2843, token_types_count: 0 }, // [1266]
    SimpleHintData { raw_values_start: 2843, raw_values_count: 3, token_types_start: 2846, token_types_count: 0 }, // [1267]
    SimpleHintData { raw_values_start: 2846, raw_values_count: 1, token_types_start: 2847, token_types_count: 0 }, // [1268]
    SimpleHintData { raw_values_start: 2847, raw_values_count: 1, token_types_start: 2848, token_types_count: 0 }, // [1269]
    SimpleHintData { raw_values_start: 2848, raw_values_count: 1, token_types_start: 2849, token_types_count: 0 }, // [1270]
    SimpleHintData { raw_values_start: 2849, raw_values_count: 1, token_types_start: 2850, token_types_count: 0 }, // [1271]
    SimpleHintData { raw_values_start: 2850, raw_values_count: 1, token_types_start: 2851, token_types_count: 0 }, // [1272]
    SimpleHintData { raw_values_start: 2851, raw_values_count: 1, token_types_start: 2852, token_types_count: 0 }, // [1273]
    SimpleHintData { raw_values_start: 2852, raw_values_count: 1, token_types_start: 2853, token_types_count: 0 }, // [1274]
    SimpleHintData { raw_values_start: 2853, raw_values_count: 1, token_types_start: 2854, token_types_count: 0 }, // [1275]
    SimpleHintData { raw_values_start: 2854, raw_values_count: 1, token_types_start: 2855, token_types_count: 0 }, // [1276]
    SimpleHintData { raw_values_start: 2855, raw_values_count: 1, token_types_start: 2856, token_types_count: 0 }, // [1277]
    SimpleHintData { raw_values_start: 2856, raw_values_count: 1, token_types_start: 2857, token_types_count: 0 }, // [1278]
    SimpleHintData { raw_values_start: 2857, raw_values_count: 1, token_types_start: 2858, token_types_count: 0 }, // [1279]
    SimpleHintData { raw_values_start: 2858, raw_values_count: 1, token_types_start: 2859, token_types_count: 0 }, // [1280]
    SimpleHintData { raw_values_start: 2859, raw_values_count: 1, token_types_start: 2860, token_types_count: 0 }, // [1281]
    SimpleHintData { raw_values_start: 2860, raw_values_count: 1, token_types_start: 2861, token_types_count: 0 }, // [1282]
    SimpleHintData { raw_values_start: 2861, raw_values_count: 1, token_types_start: 2862, token_types_count: 0 }, // [1283]
    SimpleHintData { raw_values_start: 2862, raw_values_count: 1, token_types_start: 2863, token_types_count: 0 }, // [1284]
    SimpleHintData { raw_values_start: 2863, raw_values_count: 1, token_types_start: 2864, token_types_count: 0 }, // [1285]
    SimpleHintData { raw_values_start: 2864, raw_values_count: 1, token_types_start: 2865, token_types_count: 0 }, // [1286]
    SimpleHintData { raw_values_start: 2865, raw_values_count: 1, token_types_start: 2866, token_types_count: 0 }, // [1287]
    SimpleHintData { raw_values_start: 2866, raw_values_count: 1, token_types_start: 2867, token_types_count: 0 }, // [1288]
    SimpleHintData { raw_values_start: 2867, raw_values_count: 1, token_types_start: 2868, token_types_count: 0 }, // [1289]
    SimpleHintData { raw_values_start: 2868, raw_values_count: 2, token_types_start: 2870, token_types_count: 0 }, // [1290]
    SimpleHintData { raw_values_start: 2870, raw_values_count: 1, token_types_start: 2871, token_types_count: 0 }, // [1291]
    SimpleHintData { raw_values_start: 2871, raw_values_count: 1, token_types_start: 2872, token_types_count: 0 }, // [1292]
    SimpleHintData { raw_values_start: 2872, raw_values_count: 1, token_types_start: 2873, token_types_count: 0 }, // [1293]
    SimpleHintData { raw_values_start: 2873, raw_values_count: 1, token_types_start: 2874, token_types_count: 0 }, // [1294]
    SimpleHintData { raw_values_start: 2874, raw_values_count: 1, token_types_start: 2875, token_types_count: 0 }, // [1295]
    SimpleHintData { raw_values_start: 2875, raw_values_count: 2, token_types_start: 2877, token_types_count: 0 }, // [1296]
    SimpleHintData { raw_values_start: 2877, raw_values_count: 1, token_types_start: 2878, token_types_count: 0 }, // [1297]
    SimpleHintData { raw_values_start: 2878, raw_values_count: 1, token_types_start: 2879, token_types_count: 0 }, // [1298]
    SimpleHintData { raw_values_start: 2879, raw_values_count: 1, token_types_start: 2880, token_types_count: 0 }, // [1299]
    SimpleHintData { raw_values_start: 2880, raw_values_count: 1, token_types_start: 2881, token_types_count: 0 }, // [1300]
    SimpleHintData { raw_values_start: 2881, raw_values_count: 1, token_types_start: 2882, token_types_count: 0 }, // [1301]
    SimpleHintData { raw_values_start: 2882, raw_values_count: 1, token_types_start: 2883, token_types_count: 0 }, // [1302]
    SimpleHintData { raw_values_start: 2883, raw_values_count: 1, token_types_start: 2884, token_types_count: 0 }, // [1303]
    SimpleHintData { raw_values_start: 2884, raw_values_count: 1, token_types_start: 2885, token_types_count: 0 }, // [1304]
    SimpleHintData { raw_values_start: 2885, raw_values_count: 1, token_types_start: 2886, token_types_count: 0 }, // [1305]
    SimpleHintData { raw_values_start: 2886, raw_values_count: 1, token_types_start: 2887, token_types_count: 0 }, // [1306]
    SimpleHintData { raw_values_start: 2887, raw_values_count: 1, token_types_start: 2888, token_types_count: 0 }, // [1307]
    SimpleHintData { raw_values_start: 2888, raw_values_count: 1, token_types_start: 2889, token_types_count: 0 }, // [1308]
    SimpleHintData { raw_values_start: 2889, raw_values_count: 1, token_types_start: 2890, token_types_count: 0 }, // [1309]
    SimpleHintData { raw_values_start: 2890, raw_values_count: 1, token_types_start: 2891, token_types_count: 0 }, // [1310]
    SimpleHintData { raw_values_start: 2891, raw_values_count: 1, token_types_start: 2892, token_types_count: 0 }, // [1311]
    SimpleHintData { raw_values_start: 2892, raw_values_count: 1, token_types_start: 2893, token_types_count: 0 }, // [1312]
    SimpleHintData { raw_values_start: 2893, raw_values_count: 1, token_types_start: 2894, token_types_count: 0 }, // [1313]
    SimpleHintData { raw_values_start: 2894, raw_values_count: 1, token_types_start: 2895, token_types_count: 0 }, // [1314]
    SimpleHintData { raw_values_start: 2895, raw_values_count: 6, token_types_start: 2901, token_types_count: 0 }, // [1315]
    SimpleHintData { raw_values_start: 2901, raw_values_count: 5, token_types_start: 2906, token_types_count: 0 }, // [1316]
    SimpleHintData { raw_values_start: 2906, raw_values_count: 11, token_types_start: 2917, token_types_count: 0 }, // [1317]
    SimpleHintData { raw_values_start: 2917, raw_values_count: 1, token_types_start: 2918, token_types_count: 0 }, // [1318]
    SimpleHintData { raw_values_start: 2918, raw_values_count: 1, token_types_start: 2919, token_types_count: 0 }, // [1319]
    SimpleHintData { raw_values_start: 2919, raw_values_count: 1, token_types_start: 2920, token_types_count: 0 }, // [1320]
    SimpleHintData { raw_values_start: 2920, raw_values_count: 1, token_types_start: 2921, token_types_count: 0 }, // [1321]
    SimpleHintData { raw_values_start: 2921, raw_values_count: 1, token_types_start: 2922, token_types_count: 0 }, // [1322]
    SimpleHintData { raw_values_start: 2922, raw_values_count: 1, token_types_start: 2923, token_types_count: 0 }, // [1323]
    SimpleHintData { raw_values_start: 2923, raw_values_count: 1, token_types_start: 2924, token_types_count: 0 }, // [1324]
    SimpleHintData { raw_values_start: 2924, raw_values_count: 1, token_types_start: 2925, token_types_count: 0 }, // [1325]
    SimpleHintData { raw_values_start: 2925, raw_values_count: 1, token_types_start: 2926, token_types_count: 0 }, // [1326]
    SimpleHintData { raw_values_start: 2926, raw_values_count: 1, token_types_start: 2927, token_types_count: 0 }, // [1327]
    SimpleHintData { raw_values_start: 2927, raw_values_count: 36, token_types_start: 2963, token_types_count: 0 }, // [1328]
    SimpleHintData { raw_values_start: 2963, raw_values_count: 39, token_types_start: 3002, token_types_count: 1 }, // [1329]
    SimpleHintData { raw_values_start: 3003, raw_values_count: 1, token_types_start: 3004, token_types_count: 1 }, // [1330]
    SimpleHintData { raw_values_start: 3005, raw_values_count: 2, token_types_start: 3007, token_types_count: 0 }, // [1331]
    SimpleHintData { raw_values_start: 3007, raw_values_count: 10, token_types_start: 3017, token_types_count: 0 }, // [1332]
    SimpleHintData { raw_values_start: 3017, raw_values_count: 1, token_types_start: 3018, token_types_count: 0 }, // [1333]
    SimpleHintData { raw_values_start: 3018, raw_values_count: 1, token_types_start: 3019, token_types_count: 0 }, // [1334]
    SimpleHintData { raw_values_start: 3019, raw_values_count: 2, token_types_start: 3021, token_types_count: 0 }, // [1335]
    SimpleHintData { raw_values_start: 3021, raw_values_count: 1, token_types_start: 3022, token_types_count: 0 }, // [1336]
    SimpleHintData { raw_values_start: 3022, raw_values_count: 2, token_types_start: 3024, token_types_count: 0 }, // [1337]
    SimpleHintData { raw_values_start: 3024, raw_values_count: 1, token_types_start: 3025, token_types_count: 0 }, // [1338]
    SimpleHintData { raw_values_start: 3025, raw_values_count: 1, token_types_start: 3026, token_types_count: 0 }, // [1339]
    SimpleHintData { raw_values_start: 3026, raw_values_count: 1, token_types_start: 3027, token_types_count: 0 }, // [1340]
    SimpleHintData { raw_values_start: 3027, raw_values_count: 1, token_types_start: 3028, token_types_count: 0 }, // [1341]
    SimpleHintData { raw_values_start: 3028, raw_values_count: 1, token_types_start: 3029, token_types_count: 0 }, // [1342]
    SimpleHintData { raw_values_start: 3029, raw_values_count: 1, token_types_start: 3030, token_types_count: 0 }, // [1343]
    SimpleHintData { raw_values_start: 3030, raw_values_count: 1, token_types_start: 3031, token_types_count: 0 }, // [1344]
    SimpleHintData { raw_values_start: 3031, raw_values_count: 1, token_types_start: 3032, token_types_count: 0 }, // [1345]
    SimpleHintData { raw_values_start: 3032, raw_values_count: 1, token_types_start: 3033, token_types_count: 0 }, // [1346]
    SimpleHintData { raw_values_start: 3033, raw_values_count: 1, token_types_start: 3034, token_types_count: 0 }, // [1347]
    SimpleHintData { raw_values_start: 3034, raw_values_count: 1, token_types_start: 3035, token_types_count: 0 }, // [1348]
    SimpleHintData { raw_values_start: 3035, raw_values_count: 1, token_types_start: 3036, token_types_count: 0 }, // [1349]
    SimpleHintData { raw_values_start: 3036, raw_values_count: 1, token_types_start: 3037, token_types_count: 0 }, // [1350]
    SimpleHintData { raw_values_start: 3037, raw_values_count: 1, token_types_start: 3038, token_types_count: 0 }, // [1351]
    SimpleHintData { raw_values_start: 3038, raw_values_count: 1, token_types_start: 3039, token_types_count: 0 }, // [1352]
    SimpleHintData { raw_values_start: 3039, raw_values_count: 1, token_types_start: 3040, token_types_count: 0 }, // [1353]
    SimpleHintData { raw_values_start: 3040, raw_values_count: 1, token_types_start: 3041, token_types_count: 0 }, // [1354]
    SimpleHintData { raw_values_start: 3041, raw_values_count: 1, token_types_start: 3042, token_types_count: 0 }, // [1355]
    SimpleHintData { raw_values_start: 3042, raw_values_count: 1, token_types_start: 3043, token_types_count: 0 }, // [1356]
    SimpleHintData { raw_values_start: 3043, raw_values_count: 1, token_types_start: 3044, token_types_count: 0 }, // [1357]
    SimpleHintData { raw_values_start: 3044, raw_values_count: 1, token_types_start: 3045, token_types_count: 0 }, // [1358]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 0, 0, 4, 5, 6, 7, 8, 9, 10, 11, 0, 0, 12,
    13, 14, 14, 15, 16, 16, 17, 18, 19, 20, 21, 10, 0, 0, 22, 22,
    17, 18, 19, 20, 23, 21, 9, 0, 0, 24, 23, 25, 26, 27, 28, 29,
    30, 31, 32, 33, 34, 35, 14, 14, 36, 26, 26, 37, 38, 5, 39, 40,
    41, 42, 43, 44, 45, 46, 10, 47, 47, 48, 49, 49, 39, 50, 50, 5,
    51, 51, 52, 53, 17, 54, 55, 56, 57, 58, 59, 60, 61, 23, 47, 47,
    0, 49, 49, 54, 50, 50, 17, 51, 51, 0, 62, 63, 64, 0, 0, 65,
    66, 67, 68, 69, 70, 71, 72, 73, 73, 74, 73, 75, 76, 77, 78, 0,
    0, 79, 0, 0, 0, 80, 0, 80, 81, 0, 0, 80, 0, 0, 0, 0,
    0, 82, 0, 0, 0, 14, 83, 84, 85, 85, 77, 0, 80, 0, 80, 0,
    86, 87, 88, 88, 89, 0, 90, 90, 89, 0, 91, 92, 93, 0, 94, 94,
    10, 0, 85, 85, 8, 95, 0, 96, 97, 85, 85, 9, 0, 98, 99, 100,
    101, 83, 83, 102, 8, 8, 103, 104, 105, 105, 102, 88, 88, 89, 0, 90,
    90, 89, 0, 0, 91, 92, 93, 94, 94, 106, 0, 94, 94, 0, 107, 108,
    108, 89, 109, 0, 110, 111, 0, 112, 113, 113, 114, 115, 115, 0, 14, 0,
    63, 63, 116, 37, 117, 89, 0, 0, 100, 100, 100, 63, 118, 116, 14, 14,
    36, 119, 119, 31, 30, 120, 28, 121, 27, 122, 123, 29, 32, 37, 64, 64,
    63, 63, 118, 124, 116, 37, 117, 113, 0, 0, 125, 126, 127, 117, 17, 56,
    58, 59, 18, 23, 85, 85, 85, 110, 36, 124, 128, 129, 129, 0, 130, 130,
    0, 14, 14, 131, 21, 21, 132, 10, 9, 0, 63, 26, 37, 15, 17, 18,
    19, 20, 24, 23, 25, 89, 0, 0, 0, 133, 0, 134, 85, 85, 135, 0,
    136, 94, 94, 110, 0, 137, 137, 110, 138, 138, 94, 137, 139, 140, 72, 72,
    141, 141, 0, 142, 143, 139, 140, 144, 144, 0, 139, 139, 145, 137, 137, 141,
    95, 0, 127, 125, 126, 88, 88, 141, 0, 89, 0, 146, 146, 141, 0, 90,
    90, 89, 0, 0, 91, 92, 93, 88, 88, 89, 0, 94, 94, 10, 0, 85,
    85, 147, 0, 148, 29, 29, 89, 89, 111, 94, 94, 10, 0, 0, 0, 149,
    149, 72, 137, 150, 151, 151, 3, 3, 152, 153, 154, 135, 155, 155, 122, 156,
    156, 157, 158, 156, 156, 5, 159, 160, 161, 10, 44, 162, 163, 163, 164, 165,
    166, 167, 168, 169, 6, 0, 170, 170, 49, 49, 39, 5, 39, 0, 77, 77,
    0, 80, 0, 0, 171, 171, 80, 0, 0, 79, 0, 172, 172, 41, 42, 43,
    0, 0, 0, 80, 173, 173, 0, 0, 0, 80, 0, 0, 0, 12, 0, 174,
    174, 175, 176, 0, 177, 0, 178, 178, 124, 6, 0, 179, 72, 72, 180, 95,
    0, 0, 181, 181, 0, 182, 182, 137, 137, 180, 95, 0, 127, 85, 85, 180,
    0, 183, 184, 94, 94, 157, 6, 0, 181, 181, 0, 177, 177, 0, 0, 94,
    94, 110, 0, 0, 111, 0, 185, 0, 137, 137, 110, 186, 138, 94, 137, 139,
    140, 137, 137, 187, 95, 72, 72, 188, 189, 190, 191, 191, 110, 79, 192, 80,
    193, 193, 194, 194, 94, 94, 188, 189, 190, 191, 191, 110, 94, 94, 193, 195,
    195, 100, 100, 104, 137, 137, 192, 95, 94, 94, 196, 104, 94, 94, 80, 115,
    115, 80, 94, 94, 197, 198, 199, 50, 200, 201, 94, 94, 202, 0, 72, 72,
    203, 72, 72, 204, 85, 85, 141, 0, 205, 206, 139, 139, 206, 207, 207, 207,
    208, 207, 207, 209, 146, 146, 141, 0, 137, 137, 141, 95, 0, 127, 210, 210,
    211, 212, 122, 0, 0, 14, 130, 211, 211, 213, 214, 190, 122, 0, 210, 210,
    211, 212, 211, 211, 214, 211, 211, 215, 215, 0, 216, 216, 212, 211, 217, 218,
    218, 217, 219, 220, 221, 11, 11, 37, 0, 94, 94, 222, 223, 11, 224, 94,
    94, 106, 0, 94, 94, 225, 226, 227, 94, 94, 80, 115, 115, 80, 228, 218,
    229, 0, 230, 230, 0, 139, 139, 230, 90, 90, 89, 0, 0, 91, 92, 93,
    214, 214, 192, 231, 110, 177, 177, 232, 0, 233, 234, 235, 85, 85, 0, 0,
    236, 80, 0, 0, 237, 72, 94, 137, 0, 238, 85, 85, 156, 5, 0, 95,
    239, 0, 86, 240, 179, 179, 88, 88, 180, 0, 89, 0, 88, 88, 89, 0,
    241, 241, 242, 243, 243, 37, 140, 244, 245, 245, 140, 37, 140, 244, 246, 247,
    248, 249, 250, 251, 139, 252, 253, 50, 221, 254, 255, 256, 257, 257, 258, 259,
    260, 261, 262, 262, 104, 263, 263, 104, 264, 264, 0, 94, 94, 0, 107, 108,
    108, 89, 109, 0, 111, 0, 110, 113, 113, 114, 115, 115, 0, 14, 0, 125,
    85, 85, 41, 0, 0, 0, 80, 265, 241, 88, 88, 89, 0, 94, 94, 10,
    0, 90, 90, 89, 0, 0, 0, 112, 91, 92, 93, 266, 218, 267, 37, 147,
    0, 85, 85, 232, 0, 94, 94, 268, 85, 269, 85, 85, 180, 0, 270, 94,
    94, 196, 104, 94, 94, 80, 0, 0, 0, 109, 111, 115, 115, 80, 0, 0,
    94, 94, 197, 198, 199, 50, 200, 201, 94, 94, 202, 0, 11, 11, 37, 0,
    94, 94, 222, 11, 94, 94, 80, 0, 0, 0, 109, 109, 111, 115, 115, 80,
    0, 0, 90, 90, 89, 0, 0, 91, 92, 93, 85, 85, 49, 39, 0, 0,
    95, 0, 271, 269, 269, 88, 88, 180, 180, 0, 89, 0, 88, 88, 89, 0,
    94, 94, 10, 0, 0, 0, 218, 267, 37, 147, 0, 14, 14, 21, 106, 0,
    272, 272, 191, 0, 0, 94, 106, 0, 273, 273, 85, 85, 274, 8, 11, 11,
    11, 214, 232, 275, 276, 39, 49, 49, 39, 5, 7, 0, 90, 90, 89, 0,
    85, 85, 48, 0, 37, 0, 277, 88, 88, 89, 0, 278, 89, 89, 0, 0,
    0, 134, 91, 92, 93, 177, 177, 80, 0, 100, 100, 144, 80, 0, 279, 279,
    253, 50, 221, 254, 255, 256, 94, 94, 0, 107, 108, 108, 89, 109, 0, 111,
    0, 110, 113, 113, 114, 115, 115, 0, 14, 0, 125, 85, 85, 42, 0, 0,
    0, 80, 280, 279, 88, 88, 89, 0, 94, 94, 10, 0, 94, 94, 0, 107,
    108, 108, 89, 109, 0, 0, 0, 111, 113, 113, 114, 90, 90, 89, 0, 0,
    0, 112, 91, 92, 93, 266, 218, 267, 37, 147, 0, 85, 85, 281, 0, 282,
    94, 94, 80, 72, 72, 283, 283, 94, 94, 283, 283, 137, 137, 283, 283, 88,
    88, 89, 0, 90, 90, 89, 0, 91, 92, 93, 0, 85, 85, 274, 8, 11,
    11, 214, 232, 275, 276, 39, 49, 49, 39, 5, 7, 0, 88, 88, 89, 0,
    85, 85, 284, 129, 130, 0, 0, 0, 91, 92, 93, 0, 100, 285, 286, 287,
    288, 289, 34, 290, 291, 33, 292, 293, 229, 294, 295, 296, 297, 298, 299, 300,
    301, 302, 303, 8, 8, 103, 104, 304, 305, 306, 307, 238, 140, 145, 145, 308,
    238, 0, 0, 88, 88, 89, 0, 0, 0, 91, 92, 93, 14, 0, 21, 21,
    9, 0, 270, 94, 94, 0, 107, 108, 108, 89, 109, 0, 110, 0, 0, 111,
    0, 37, 113, 113, 114, 115, 115, 0, 0, 14, 85, 85, 10, 0, 309, 88,
    88, 89, 0, 90, 90, 89, 0, 85, 85, 7, 95, 0, 79, 0, 193, 79,
    79, 0, 310, 310, 191, 311, 104, 312, 313, 313, 311, 104, 218, 218, 313, 314,
    315, 315, 311, 104, 218, 218, 315, 44, 44, 316, 0, 317, 317, 100, 311, 104,
    195, 195, 100, 311, 104, 318, 318, 104, 319, 218, 320, 272, 272, 191, 0, 321,
    0, 85, 85, 44, 95, 0, 322, 193, 193, 90, 90, 89, 0, 0, 92, 93,
    88, 88, 89, 0, 94, 94, 10, 0, 85, 85, 85, 49, 39, 95, 0, 21,
    21, 11, 0, 85, 85, 323, 275, 276, 95, 0, 94, 80, 324, 325, 85, 85,
    196, 95, 0, 87, 90, 90, 89, 0, 0, 0, 112, 91, 92, 93, 88, 88,
    89, 0, 94, 94, 326, 10, 10, 0, 196, 196, 104, 85, 85, 327, 0, 328,
    8, 8, 238, 237, 237, 94, 72, 137, 281, 0, 0, 100, 100, 80, 329, 329,
    281, 100, 100, 80, 211, 212, 94, 94, 80, 330, 330, 80, 0, 109, 0, 90,
    90, 89, 0, 0, 91, 92, 0, 88, 88, 89, 0, 85, 85, 13, 270, 94,
    94, 0, 108, 89, 109, 0, 110, 0, 0, 111, 0, 112, 37, 115, 115, 0,
    14, 0, 331, 72, 72, 180, 95, 0, 0, 181, 181, 0, 182, 182, 137, 137,
    180, 95, 0, 127, 85, 85, 180, 0, 183, 184, 94, 94, 157, 6, 0, 181,
    181, 0, 177, 177, 0, 0, 94, 94, 110, 0, 0, 111, 0, 185, 0, 137,
    137, 110, 186, 138, 94, 137, 139, 140, 137, 137, 187, 95, 72, 72, 188, 189,
    190, 191, 191, 110, 79, 192, 80, 193, 193, 194, 194, 94, 94, 188, 189, 190,
    191, 191, 110, 94, 94, 193, 195, 195, 100, 100, 104, 137, 137, 192, 95, 94,
    94, 196, 104, 94, 94, 80, 115, 115, 80, 94, 94, 197, 198, 199, 50, 200,
    201, 94, 94, 202, 0, 72, 72, 203, 72, 72, 204, 85, 85, 141, 0, 205,
    206, 139, 139, 206, 207, 207, 207, 208, 207, 207, 209, 146, 146, 141, 0, 137,
    137, 141, 95, 0, 127, 210, 210, 211, 212, 122, 0, 0, 14, 130, 211, 211,
    213, 214, 190, 122, 0, 210, 210, 211, 212, 211, 211, 214, 211, 211, 215, 215,
    0, 216, 216, 212, 211, 217, 218, 218, 217, 219, 220, 221, 11, 11, 37, 0,
    94, 94, 222, 223, 11, 224, 94, 94, 106, 0, 94, 94, 225, 226, 227, 94,
    94, 80, 115, 115, 80, 228, 218, 229, 0, 230, 230, 0, 139, 139, 230, 90,
    90, 89, 0, 0, 91, 92, 93, 214, 214, 192, 231, 110, 177, 177, 232, 0,
    233, 234, 137, 137, 180, 95, 0, 0, 0, 0, 109, 0, 111, 0, 332, 332,
    72, 333, 180, 0, 334, 335, 335, 336, 76, 0, 80, 137, 88, 88, 337, 79,
    89, 0, 85, 85, 5, 0, 0, 95, 239, 0, 86, 338, 331, 331, 88, 88,
    180, 0, 89, 0, 88, 88, 141, 0, 89, 0, 0, 95, 0, 339, 88, 88,
    89, 0, 94, 94, 10, 0, 340, 340, 341, 0, 342, 124, 124, 343, 344, 110,
    345, 345, 341, 0, 0, 346, 14, 14, 21, 106, 0, 272, 272, 191, 0, 0,
    94, 106, 0, 273, 85, 85, 163, 164, 166, 0, 347, 72, 72, 348, 124, 0,
    0, 100, 0, 0, 85, 85, 348, 124, 0, 0, 100, 0, 0, 85, 85, 348,
    349, 0, 100, 0, 85, 85, 348, 124, 0, 0, 349, 0, 100, 0, 137, 137,
    348, 95, 124, 0, 0, 88, 88, 89, 0, 90, 90, 89, 0, 0, 91, 92,
    93, 94, 94, 10, 0, 85, 85, 122, 0, 37, 0, 350, 88, 88, 89, 0,
    266, 218, 267, 37, 147, 0, 85, 85, 6, 0, 347, 90, 90, 89, 0, 92,
    93, 91, 0, 88, 88, 351, 238, 89, 238, 88, 88, 89, 0, 94, 94, 10,
    0, 352, 352, 72, 72, 353, 0, 0, 181, 181, 0, 127, 85, 85, 353, 0,
    94, 94, 157, 6, 0, 181, 181, 0, 127, 137, 137, 353, 95, 0, 127, 88,
    88, 353, 0, 89, 0, 127, 72, 72, 351, 95, 238, 354, 354, 355, 76, 238,
    85, 85, 39, 95, 0, 356, 85, 85, 180, 0, 138, 94, 94, 110, 0, 111,
    0, 185, 0, 137, 137, 110, 90, 90, 89, 0, 0, 91, 92, 93, 88, 88,
    180, 0, 89, 0, 88, 88, 89, 0, 94, 94, 10, 0, 94, 94, 80, 0,
    0, 0, 109, 109, 111, 115, 115, 80, 0, 0, 190, 357, 358, 359, 360, 360,
    359, 358, 361, 80, 362, 362, 362, 363, 364, 102, 363, 0, 0, 0, 80, 0,
    0, 365, 365, 366, 367, 368, 369, 73, 370, 86, 371, 357, 74, 372, 373, 374,
    375, 376, 66, 0, 311, 104, 0, 0, 0, 311, 104, 0, 377, 0, 311, 104,
    0, 66, 378, 0, 379, 66, 0, 0, 379, 66, 0, 0, 79, 0, 79, 79,
    79, 380, 381, 382, 383, 384, 385, 340, 353, 386, 387, 388, 389, 390, 390, 390,
    391, 392, 393, 394, 395, 396, 185, 0, 0, 111, 185, 397, 0, 0, 0, 0,
    0, 111, 12, 79, 80, 0, 0, 355, 398, 399, 400, 401, 402, 403, 404, 368,
    405, 406, 407, 408, 0, 409, 410, 411, 357, 373, 373, 373, 74, 374, 374, 374,
    372, 412, 413, 406, 365, 414, 415, 102, 416, 417, 418, 419, 420, 80, 111, 111,
    80, 0, 0, 0, 80, 421, 422, 423, 424, 425, 426, 427, 428, 429, 191, 302,
    430, 431, 80, 0, 0, 0, 80, 79, 432, 139, 49, 80, 433, 434, 164, 164,
    435, 422, 436, 336, 191, 0, 94, 0, 320, 320, 0, 94, 0, 177, 0, 318,
    437, 437, 437, 0, 243, 126, 127, 126, 125, 438, 439, 439, 439, 0, 440, 440,
    441, 442, 441, 442, 442, 439, 439, 0, 440, 440, 441, 442, 441, 442, 442, 407,
    12, 404, 439, 171, 443, 444, 445, 446, 447, 0, 448, 449, 450, 144, 451, 452,
    453, 454, 455, 456, 175, 457, 458, 458, 458, 0, 11, 11, 11, 363, 0, 0,
    0, 177, 177, 0, 0, 0, 37, 0, 459, 25, 460, 0, 0, 181, 153, 0,
    238, 0, 0, 67, 0, 377, 377, 377, 0, 0, 111, 0, 185, 182, 141, 141,
    0, 461, 143, 139, 140, 144, 144, 80, 0, 218, 218, 229, 110, 110, 0, 0,
    111, 0, 185, 0, 188, 188, 190, 79, 0, 462, 188, 188, 189, 190, 191, 191,
    110, 79, 192, 80, 193, 193, 463, 463, 464, 464, 139, 465, 100, 100, 80, 177,
    177, 232, 106, 0, 466, 466, 467, 100, 100, 80, 177, 177, 232, 106, 0, 121,
    205, 206, 139, 139, 206, 207, 207, 207, 208, 207, 207, 209, 0, 0, 0, 468,
    182, 181, 181, 0, 0, 180, 0, 0, 67, 67, 67, 67, 67, 67, 0, 0,
    67, 67, 67, 67, 67, 37, 79, 177, 12, 443, 66, 80, 404, 377, 469, 0,
    0, 0, 371, 6, 0, 0, 0, 0, 470, 12, 471, 471, 238, 471, 471, 471,
    37, 472, 472, 473, 473, 5, 39, 0, 171, 171, 80, 0, 0, 79, 0, 180,
    180, 0, 141, 141, 0, 37, 37, 135, 0, 9, 9, 0, 147, 147, 0, 41,
    41, 0, 80, 80, 232, 232, 0, 10, 10, 0, 474, 474, 153, 154, 135, 161,
    48, 281, 129, 215, 44, 162, 196, 327, 106, 122, 6, 3, 3, 152, 155, 155,
    122, 156, 156, 157, 158, 156, 156, 5, 49, 49, 39, 163, 163, 164, 166, 163,
    163, 164, 167, 163, 163, 164, 168, 163, 163, 164, 169, 0, 37, 37, 0, 475,
    475, 77, 42, 43, 0, 80, 0, 0, 476, 476, 307, 238, 140, 0, 477, 478,
    479, 480, 407, 109, 374, 373, 374, 373, 481, 482, 476, 476, 465, 113, 476, 476,
    139, 465, 113, 357, 357, 357, 73, 483, 484, 485, 173, 0, 486, 0, 0, 80,
    80, 0, 67, 86, 487, 488, 202, 489, 74, 74, 74, 0, 0, 490, 0, 0,
    166, 491, 491, 492, 234, 29, 29, 94, 0, 0, 0, 0, 109, 0, 0, 110,
    80, 80, 0, 0, 109, 219, 80, 0, 0, 0, 110, 80, 80, 0, 0, 109,
    80, 433, 493, 493, 0, 494, 495, 80, 80, 0, 0, 0, 0, 80, 0, 0,
    181, 181, 0, 0, 493, 493, 0, 37, 37, 141, 0, 496, 497, 8, 20, 141,
    498, 499, 500, 501, 154, 502, 502, 0, 80, 0, 113, 503, 504, 100, 100, 80,
    0, 502, 502, 502, 0, 0, 0, 0, 80, 0, 0, 113, 505, 506, 238, 507,
    507, 508, 238, 509, 509, 503, 503, 504, 100, 80, 510, 510, 52, 52, 0, 511,
    511, 102, 512, 512, 238, 140, 140, 238, 513, 513, 102, 514, 514, 238, 515, 515,
    238, 516, 516, 517, 80, 0, 0, 86, 518, 518, 80, 0, 0, 519, 519, 80,
    0, 0, 520, 520, 238, 493, 493, 0, 0, 113, 521, 238, 509, 503, 503, 522,
    100, 523, 403, 403, 512, 512, 79, 238, 140, 140, 79, 238, 524, 524, 525, 513,
    514, 514, 79, 238, 515, 515, 79, 238, 217, 217, 139, 140, 0, 0, 0, 0,
    80, 433, 89, 526, 527, 527, 0, 0, 80, 433, 89, 528, 238, 509, 527, 527,
    522, 100, 523, 403, 403, 512, 512, 79, 238, 140, 140, 79, 238, 524, 524, 525,
    513, 514, 514, 79, 238, 515, 515, 79, 238, 217, 217, 514, 517, 80, 0, 0,
    86, 502, 502, 80, 529, 27, 530, 530, 0, 343, 343, 0, 0, 89, 527, 100,
    100, 80, 0, 531, 262, 532, 533, 534, 31, 31, 414, 77, 0, 80, 0, 80,
    31, 31, 171, 80, 0, 79, 0, 535, 100, 100, 41, 0, 80, 222, 222, 41,
    100, 100, 536, 79, 79, 79, 382, 79, 79, 537, 31, 31, 11, 214, 0, 0,
    0, 31, 31, 11, 0, 0, 538, 538, 80, 0, 0, 0, 0, 31, 31, 153,
    95, 0, 539, 80, 540, 540, 541, 541, 109, 238, 542, 542, 109, 238, 543, 543,
    109, 238, 544, 544, 109, 545, 546, 547, 548, 548, 109, 102, 549, 549, 109, 238,
    113, 113, 0, 31, 31, 8, 95, 0, 89, 550, 394, 394, 551, 496, 496, 10,
    552, 553, 554, 555, 555, 556, 80, 0, 31, 31, 9, 0, 100, 557, 90, 90,
    109, 0, 169, 169, 109, 0, 520, 520, 109, 558, 238, 110, 559, 541, 541, 109,
    238, 560, 542, 542, 109, 238, 543, 543, 109, 238, 106, 106, 109, 0, 0, 110,
    83, 83, 109, 102, 8, 8, 103, 109, 104, 105, 105, 109, 102, 31, 31, 135,
    0, 79, 79, 0, 181, 181, 0, 110, 110, 0, 561, 561, 141, 141, 0, 142,
    143, 139, 140, 144, 144, 0, 31, 31, 147, 95, 0, 100, 10, 10, 0, 549,
    549, 0, 113, 113, 0, 126, 31, 31, 156, 0, 562, 0, 563, 0, 31, 31,
    156, 5, 31, 31, 31, 95, 0, 80, 0, 0, 0, 0, 0, 236, 181, 181,
    0, 564, 564, 565, 566, 566, 80, 0, 0, 162, 162, 0, 236, 31, 31, 95,
    0, 341, 341, 230, 0, 80, 0, 0, 0, 0, 100, 100, 236, 564, 564, 565,
    342, 124, 124, 343, 344, 110, 162, 162, 0, 236, 31, 31, 414, 567, 41, 95,
    0, 80, 245, 245, 0, 5, 5, 80, 0, 0, 0, 0, 0, 0, 0, 0,
    568, 568, 0, 0, 569, 31, 31, 570, 232, 232, 0, 21, 37, 0, 177, 177,
    0, 80, 0, 0, 110, 110, 232, 21, 37, 0, 177, 177, 0, 31, 31, 31,
    571, 49, 49, 39, 95, 0, 80, 0, 0, 21, 79, 0, 414, 414, 49, 39,
    0, 80, 0, 0, 21, 79, 0, 31, 31, 414, 572, 95, 0, 236, 236, 80,
    0, 0, 0, 109, 111, 111, 66, 238, 238, 79, 433, 31, 31, 173, 0, 0,
    80, 0, 0, 0, 0, 573, 0, 109, 0, 0, 0, 109, 0, 0, 0, 0,
    109, 0, 0, 574, 574, 125, 575, 576, 42, 41, 109, 0, 0, 0, 31, 31,
    48, 0, 37, 0, 79, 79, 577, 578, 579, 124, 124, 580, 14, 27, 28, 29,
    30, 89, 89, 0, 0, 0, 134, 91, 92, 93, 177, 177, 80, 0, 100, 100,
    144, 80, 0, 31, 31, 414, 42, 0, 80, 569, 31, 31, 281, 0, 124, 124,
    124, 14, 17, 124, 124, 283, 283, 100, 100, 80, 31, 31, 284, 129, 130, 0,
    581, 100, 582, 286, 287, 288, 289, 34, 290, 291, 33, 292, 293, 229, 294, 295,
    296, 297, 298, 299, 300, 301, 302, 303, 8, 8, 103, 104, 304, 305, 306, 307,
    238, 140, 145, 145, 308, 238, 21, 21, 129, 0, 21, 21, 133, 0, 129, 129,
    0, 75, 75, 0, 130, 130, 0, 583, 583, 104, 31, 31, 10, 95, 0, 0,
    0, 389, 0, 0, 31, 31, 7, 95, 0, 79, 0, 584, 79, 79, 0, 310,
    310, 191, 311, 104, 312, 313, 313, 311, 104, 218, 218, 313, 314, 315, 315, 311,
    104, 218, 218, 315, 317, 317, 100, 311, 104, 318, 318, 104, 319, 320, 218, 218,
    320, 272, 272, 191, 0, 321, 0, 31, 31, 567, 44, 95, 0, 584, 584, 31,
    31, 162, 95, 0, 6, 6, 238, 549, 549, 0, 156, 0, 236, 31, 31, 276,
    95, 0, 21, 113, 0, 585, 0, 467, 467, 80, 0, 0, 52, 52, 0, 586,
    586, 587, 588, 589, 100, 100, 80, 0, 0, 31, 31, 275, 95, 0, 21, 80,
    0, 0, 113, 553, 8, 0, 80, 0, 0, 467, 467, 52, 0, 351, 52, 0,
    52, 52, 0, 590, 590, 0, 0, 586, 586, 591, 321, 588, 589, 100, 100, 80,
    0, 0, 31, 31, 275, 95, 0, 21, 113, 113, 592, 593, 594, 386, 534, 595,
    596, 80, 0, 0, 124, 124, 14, 17, 100, 100, 80, 0, 0, 31, 31, 275,
    95, 0, 21, 113, 113, 554, 8, 0, 80, 0, 0, 124, 124, 124, 14, 17,
    124, 124, 17, 80, 0, 0, 100, 100, 80, 0, 0, 31, 31, 275, 95, 0,
    21, 113, 113, 597, 412, 52, 598, 163, 599, 431, 590, 590, 590, 513, 0, 0,
    590, 590, 600, 80, 0, 0, 144, 144, 80, 100, 80, 0, 0, 0, 0, 31,
    31, 196, 0, 95, 0, 80, 601, 601, 602, 603, 604, 531, 37, 0, 0, 0,
    0, 113, 0, 31, 31, 327, 0, 8, 238, 281, 0, 0, 100, 100, 80, 31,
    31, 605, 606, 607, 608, 609, 567, 227, 5, 95, 0, 610, 80, 0, 0, 177,
    177, 0, 535, 100, 100, 80, 0, 0, 0, 109, 109, 0, 111, 0, 222, 222,
    224, 37, 37, 478, 611, 612, 612, 263, 30, 30, 263, 137, 106, 106, 0, 79,
    613, 433, 80, 433, 614, 80, 5, 5, 0, 343, 615, 80, 120, 120, 0, 100,
    31, 31, 605, 606, 607, 608, 609, 567, 227, 5, 95, 0, 616, 80, 80, 0,
    0, 0, 203, 617, 617, 0, 618, 618, 566, 566, 80, 0, 0, 230, 230, 0,
    80, 0, 0, 0, 0, 100, 100, 236, 182, 182, 203, 341, 341, 230, 0, 80,
    0, 0, 0, 0, 100, 100, 236, 182, 182, 203, 342, 124, 124, 343, 344, 110,
    619, 341, 341, 191, 620, 621, 622, 623, 80, 0, 0, 0, 0, 0, 0, 0,
    181, 181, 0, 0, 177, 177, 0, 535, 100, 100, 80, 222, 222, 224, 37, 37,
    478, 611, 612, 612, 263, 30, 30, 263, 137, 106, 106, 0, 31, 31, 163, 164,
    166, 0, 80, 624, 168, 168, 109, 0, 502, 502, 109, 0, 31, 31, 414, 141,
    122, 0, 625, 355, 76, 626, 626, 230, 627, 627, 28, 30, 123, 29, 29, 230,
    230, 0, 0, 414, 37, 414, 37, 0, 628, 113, 113, 0, 629, 139, 139, 206,
    630, 206, 207, 207, 207, 209, 207, 207, 208, 631, 631, 632, 633, 634, 5, 79,
    0, 632, 632, 633, 634, 5, 79, 0, 124, 124, 635, 636, 219, 637, 440, 440,
    80, 0, 120, 120, 576, 41, 42, 0, 31, 31, 6, 0, 79, 79, 79, 80,
    0, 0, 0, 0, 79, 79, 638, 622, 639, 80, 0, 0, 0, 109, 0, 31,
    31, 130, 348, 31, 31, 95, 124, 0, 134, 0, 640, 162, 0, 236, 31, 31,
    130, 0, 31, 31, 567, 641, 642, 39, 95, 0, 80, 0, 0, 79, 433, 35,
    34, 33, 643, 644, 524, 645, 645, 646, 646, 645, 647, 80, 0, 114, 648, 91,
    649, 650, 92, 651, 652, 320, 157, 9, 0, 0, 67, 0, 24, 0, 0, 0,
    0, 0, 0, 0, 0, 67, 0, 653, 654, 0, 0, 655, 655, 656, 657, 658,
    659, 660, 661, 401, 662, 663, 664, 665, 665, 666, 667, 668, 669, 670, 671, 402,
    672, 672, 672, 80, 673, 673, 674, 675, 80, 676, 677, 678, 678, 420, 448, 448,
    448, 679, 449, 449, 449, 679, 680, 80, 163, 430, 681, 415, 413, 682, 683, 684,
    685, 419, 686, 687, 455, 688, 454, 689, 690, 691, 692, 693, 694, 410, 410, 679,
    679, 80, 695, 696, 697, 599, 698, 699, 700, 701, 702, 703, 704, 705, 706, 706,
    80, 0, 0, 80, 707, 66, 66, 379, 379, 622, 708, 709, 709, 80, 80, 0,
    0, 710, 0, 711, 654, 238, 654, 708, 712, 713, 713, 397, 714, 715, 80, 104,
    704, 716, 710, 717, 718, 719, 720, 721, 722, 723, 724, 725, 726, 727, 728, 728,
    728, 729, 0, 0, 14, 588, 730, 731, 732, 674, 733, 733, 733, 0, 651, 222,
    222, 734, 124, 0, 433, 39, 39, 0, 49, 49, 39, 0, 0, 0, 735, 110,
    110, 110, 343, 736, 206, 208, 255, 737, 111, 738, 738, 738, 109, 109, 737, 80,
    738, 738, 739, 740, 30, 30, 30, 113, 239, 0, 86, 0, 177, 177, 0, 0,
    0, 0, 0, 493, 493, 493, 114, 230, 0, 493, 741, 741, 0, 86, 0, 0,
    0, 0, 742, 469, 512, 743, 744, 602, 267, 436, 745, 746, 746, 746, 0, 0,
    747, 748, 345, 749, 548, 167, 212, 750, 751, 751, 751, 752, 14, 753, 59, 642,
    641, 754, 755, 465, 756, 757, 758, 370, 491, 491, 491, 759, 759, 161, 238, 238,
    238, 161, 760, 761, 135, 762, 67, 665, 137, 137, 77, 95, 0, 0, 0, 0,
    80, 0, 86, 127, 127, 125, 126, 137, 137, 171, 95, 80, 0, 79, 0, 127,
    137, 137, 153, 95, 0, 127, 137, 137, 9, 95, 0, 504, 100, 80, 217, 137,
    137, 135, 95, 0, 0, 127, 137, 137, 147, 95, 0, 127, 137, 137, 156, 5,
    95, 0, 0, 127, 137, 137, 41, 95, 0, 0, 0, 80, 127, 137, 137, 232,
    0, 95, 0, 0, 127, 137, 137, 137, 49, 39, 95, 0, 0, 127, 137, 137,
    572, 95, 0, 137, 137, 272, 191, 0, 0, 91, 92, 93, 0, 127, 137, 137,
    48, 95, 0, 37, 0, 127, 137, 137, 42, 95, 0, 0, 0, 80, 127, 126,
    125, 137, 137, 281, 95, 0, 0, 127, 137, 137, 129, 95, 0, 137, 137, 10,
    95, 0, 127, 137, 137, 44, 95, 0, 0, 127, 137, 137, 763, 8, 11, 11,
    11, 214, 9, 232, 49, 49, 39, 129, 7, 10, 276, 275, 5, 6, 39, 130,
    95, 0, 127, 126, 126, 125, 125, 137, 137, 196, 95, 0, 0, 127, 126, 125,
    137, 137, 327, 95, 0, 127, 126, 125, 137, 137, 567, 5, 95, 0, 0, 127,
    137, 137, 163, 164, 166, 95, 0, 127, 137, 137, 122, 95, 0, 37, 0, 127,
    137, 137, 6, 95, 0, 0, 127, 137, 137, 130, 95, 0, 137, 137, 39, 95,
    0, 0, 127, 764, 765, 635, 766, 767, 441, 441, 0, 441, 768, 769, 80, 0,
    0, 80, 211, 268, 770, 520, 305, 771, 442, 772, 773, 774, 586, 109, 775, 515,
    776, 155, 777, 778, 779, 0, 0, 100, 407, 780, 120, 120, 120, 0, 80, 0,
    0, 781, 782, 783, 784, 785, 786, 787, 787, 359, 358, 363, 532, 788, 428, 789,
    790, 791, 102, 52, 52, 792, 163, 696, 599, 793, 785, 785, 794, 795, 360, 359,
    358, 363, 80, 786, 786, 433, 433, 28, 29, 30, 187, 0, 0, 0, 0, 0,
    796, 404, 404, 0, 797, 407, 476, 476, 139, 798, 799, 800, 0, 801, 139, 400,
    0, 365, 0, 0, 0, 0, 802, 311, 311, 311, 803, 139, 804, 0, 0, 805,
    805, 805, 805, 368, 405, 407, 0, 806, 311, 311, 311, 803, 0, 807, 0, 0,
    782, 782, 80, 433, 0, 0, 0, 439, 384, 384, 0, 12, 0, 808, 0, 0,
    0, 185, 0, 80, 0, 0, 0, 433, 0, 0, 111, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 80, 0, 0, 0, 0, 809, 238, 104, 102, 140, 711,
    0, 0, 12, 65, 0, 27, 111, 397, 0, 0, 0, 0, 200, 0, 0, 0,
    239, 239, 0, 80, 0, 0, 0, 86, 147, 50, 810, 811, 417, 417, 176, 812,
    812, 813, 336, 814, 0, 104, 0, 219, 815, 219, 263, 816, 239, 100, 100, 817,
    812, 812, 812, 818, 818, 819, 14, 104, 113, 113, 0, 100, 100, 80, 0, 0,
    820, 51, 821, 822, 823, 824, 825, 825, 469, 469, 826, 827, 827, 80, 493, 493,
    0, 827, 346, 336, 662, 663, 672, 828, 829, 124, 124, 830, 29, 218, 218, 467,
    29, 831, 467, 467, 831, 230, 230, 0, 0, 832, 273, 330, 330, 833, 124, 217,
    518, 519, 516, 156, 156, 157, 158, 156, 156, 467, 156, 564, 141, 141, 0, 834,
    143, 139, 140, 144, 144, 80, 0, 218, 218, 229, 110, 110, 0, 0, 111, 0,
    185, 0, 188, 188, 190, 79, 0, 462, 565, 141, 141, 0, 144, 144, 80, 0,
    218, 218, 229, 52, 818, 835, 835, 0, 0, 114, 114, 219, 0, 0, 104, 397,
    836, 0, 837, 838, 829, 400, 400, 365, 778, 778, 839, 114, 114, 219, 133, 817,
    218, 218, 840, 835, 621, 263, 841, 842, 843, 511, 113, 113, 0, 0, 844, 493,
    103, 133, 133, 191, 78, 78, 191, 845, 846, 847, 848, 100, 100, 812, 849, 124,
    0, 0, 850, 0, 80, 0, 0, 0, 0, 844, 851, 0, 0, 100, 100, 849,
    0, 851, 0, 80, 0, 0, 0, 0, 80, 0, 0, 852, 0, 80, 0, 78,
    78, 191, 133, 133, 191, 0, 0, 0, 0, 0, 0, 78, 78, 191, 133, 133,
    191, 0, 113, 233, 808, 853, 854, 0, 0, 0, 855, 855, 0, 0, 0, 0,
    0, 421, 80, 421, 0, 0, 79, 0, 856, 856, 0, 113, 113, 0, 0, 0,
    710, 0, 857, 113, 113, 0, 124, 124, 0, 741, 741, 0, 0, 465, 0, 86,
    0, 0, 78, 858, 858, 111, 0, 0, 238, 0, 0, 0, 21, 0, 238, 0,
    0, 0, 859, 0, 0, 0, 0, 0, 860, 351, 377, 0, 80, 80, 0, 569,
    861, 161, 178, 178, 124, 6, 0, 847, 247, 248, 249, 250, 251, 139, 252, 242,
    243, 243, 37, 140, 244, 245, 245, 140, 37, 140, 244, 246, 253, 50, 221, 254,
    256, 255, 257, 257, 258, 259, 260, 261, 262, 262, 104, 263, 263, 104, 264, 264,
    0, 94, 94, 0, 107, 108, 108, 89, 109, 0, 0, 0, 111, 113, 113, 114,
    79, 79, 238, 238, 238, 238, 12, 238, 862, 862, 0, 398, 398, 385, 863, 28,
    28, 469, 29, 29, 469, 864, 864, 100, 421, 27, 469, 862, 862, 0, 469, 442,
    100, 100, 80, 0, 0, 41, 0, 0, 0, 0, 0, 67, 80, 0, 0, 112,
    80, 0, 865, 21, 866, 536, 855, 0, 0, 0, 0, 865, 21, 866, 536, 855,
    0, 0, 0, 867, 867, 110, 109, 807, 0, 80, 0, 0, 0, 709, 709, 80,
    0, 0, 0, 0, 0, 709, 0, 343, 80, 0, 56, 868, 188, 593, 869, 870,
    871, 872, 873, 874, 875, 608, 63, 63, 63, 26, 37, 0, 89, 0, 876, 877,
    878, 374, 374, 109, 374, 879, 133, 133, 191, 0, 0, 0, 104, 646, 880, 0,
    80, 78, 78, 191, 103, 845, 846, 847, 848, 881, 78, 78, 191, 103, 845, 846,
    847, 812, 133, 0, 0, 0, 0, 104, 0, 80, 881, 880, 880, 880, 882, 80,
    0, 0, 646, 0, 841, 883, 562, 623, 845, 845, 0, 80, 0, 884, 78, 78,
    191, 103, 846, 847, 812, 845, 513, 600, 885, 886, 734, 887, 0, 888, 889, 890,
    891, 892, 893, 894, 546, 895, 896, 897, 0, 192, 898, 899, 900, 95, 95, 782,
    95, 95, 95, 139, 782, 901, 859, 859, 901, 902, 464, 903, 209, 248, 537, 904,
    904, 156, 10, 904, 904, 0, 905, 905, 103, 103, 89, 777, 80, 738, 738, 113,
    162, 0, 585, 0, 236, 904, 906, 21, 21, 11, 0, 21, 797, 139, 21, 0,
    80, 0, 0, 0, 433, 0, 590, 907, 310, 0, 0, 0, 908, 380, 745, 0,
    181, 181, 0, 0, 0, 464, 464, 909, 336, 910, 80, 908, 380, 745, 464, 464,
    909, 336, 910, 0, 0, 0, 0, 80, 0, 0, 232, 911, 590, 590, 80, 100,
    100, 80, 177, 177, 232, 106, 0, 912, 913, 689, 914, 915, 916, 229, 566, 917,
    207, 918, 919, 536, 244, 920, 28, 28, 28, 585, 0, 0, 80, 921, 921, 922,
    13, 130, 351, 923, 110, 110, 343, 433, 37, 37, 494, 495, 491, 741, 741, 0,
    86, 0, 0, 0, 0, 924, 325, 925, 926, 626, 927, 659, 928, 660, 699, 661,
    700, 658, 657, 929, 930, 931, 397, 397, 932, 104, 104, 933, 238, 710, 238, 238,
    238, 933, 238, 710, 397, 934, 935, 936, 718, 890, 937, 938, 585, 585, 939, 642,
    641, 227, 5, 0, 585, 940, 256, 798, 140, 941, 942, 102, 943, 599, 476, 476,
    139, 465, 113, 476, 799, 944, 105, 945, 799, 946, 947, 0, 0, 0, 575, 0,
    0, 0, 0, 0, 0, 948, 37, 177, 0, 0, 0, 575, 0, 0, 0, 0,
    0, 575, 575, 575, 0, 37, 37, 0, 0, 80, 0, 0, 490, 919, 949, 949,
    233, 950, 951, 952, 177, 177, 80, 0, 0, 599, 408, 599, 599, 953, 535, 953,
    351, 954, 379, 955, 535, 535, 100, 222, 463, 97, 97, 698, 553, 467, 97, 0,
    956, 957, 958, 959, 161, 161, 0, 0, 960, 161, 961, 910, 962, 0, 850, 963,
    542, 543, 964, 252, 965, 966, 950, 573, 967, 373, 373, 109, 373, 220, 547, 796,
    796, 139, 968, 0, 515, 515, 0, 968, 617, 903, 969, 969, 89, 617, 482, 618,
    618, 907, 779, 970, 477, 498, 736, 188, 192, 913, 196, 197, 14, 103, 103, 0,
    80, 0, 104, 0, 14, 971, 849, 849, 0, 104, 0, 12, 12, 104, 103, 684,
    972, 973, 974, 975, 976, 977, 0, 622, 978, 978, 978, 0, 111, 238, 104, 102,
    311, 140, 711, 761, 509, 66, 979, 980, 0, 0, 592, 592, 592, 238, 0, 609,
    541, 981, 982, 983, 984, 984, 984, 5, 239, 0, 0, 0, 86, 21, 21, 985,
    3, 3, 831, 219, 219, 831, 219, 219, 780, 831, 831, 29, 780, 831, 831, 831,
    219, 780, 780, 3, 3, 780, 986, 273, 833, 987, 988, 226, 989, 296, 990, 991,
    992, 993, 994, 995, 685, 852, 852, 67, 0, 80, 572, 572, 0, 12, 12, 80,
    433, 691, 690, 201, 996, 997, 639, 0, 348, 595, 47, 0, 998, 999, 1000, 785,
    785, 1001, 1001, 1002, 1003, 735, 1004, 1005, 1006, 100, 100, 80, 0, 0, 79, 79,
    1007, 163, 599, 124, 124, 0, 433, 39, 39, 0, 49, 49, 39, 0, 0, 0,
    1008, 49, 1009, 1010, 1011, 315, 603, 1012, 1013, 1014, 1015, 30, 28, 28, 80, 343,
    1016, 1016, 585, 1016, 440, 440, 440, 440, 440, 440, 999, 365, 365, 0, 1017, 1018,
    29, 30, 491, 491, 234, 440, 440, 139, 999, 191, 275, 365, 365, 0, 1017, 1018,
    29, 30, 491, 491, 234, 440, 440, 139, 999, 191, 191, 1019, 365, 365, 0, 1017,
    1020, 28, 491, 491, 234, 1016, 1016, 0, 0, 0, 177, 0, 0, 0, 80, 80,
    433, 0, 0, 0, 0, 0, 37, 0, 440, 29, 29, 94, 1021, 1022, 433, 433,
    433, 433, 1022, 1023, 152, 1024, 1025, 1026, 1027, 73, 937, 1028, 1029, 1030, 313, 852,
    1031, 986, 572, 333, 371, 1032, 676, 1033, 936, 1034, 1035, 1036, 1037, 1038, 1039, 1039,
    1039, 1040, 1041, 1042, 1043, 86, 1044, 1045, 1046, 1047, 1048, 1049, 1050, 1051, 1052, 1053,
    738, 0, 316, 0, 0, 1054, 1055, 807, 109, 0, 0, 0, 79, 0, 0, 80,
    0, 847, 847, 0, 0, 1056, 941, 941, 1057, 1058, 1058, 490, 1058, 1059, 604, 73,
    0, 634, 0, 814, 1060, 1061, 1062, 1063, 218, 1064, 1065, 303, 1066, 1067, 1068, 290,
    292, 1069, 1070, 294, 297, 421, 343, 27, 80, 27, 80, 100, 80, 421, 80, 0,
    709, 1071, 421, 1072, 80, 27, 421, 29, 28, 30, 1073, 321, 1074, 300, 1075, 943,
    1076, 1060, 1061, 1062, 1063, 1077, 1077, 288, 0, 481, 1078, 1078, 109, 373, 373, 374,
    357, 357, 373, 357, 357, 374, 139, 800, 139, 234, 1079, 1080, 1081, 1081, 1081, 0,
    12, 12, 238, 800, 273, 140, 140, 1082, 464, 675, 104, 701, 955, 238, 238, 377,
    0, 980, 238, 238, 980, 67, 67, 67, 67, 67, 0, 37, 79, 177, 12, 443,
    66, 80, 404, 377, 469, 0, 1083, 230, 1084, 1085, 849, 849, 0, 104, 0, 849,
    1086, 224, 633, 37, 37, 1087, 239, 1088, 1089, 1090, 1091, 1092, 1093, 173, 1094, 1004,
    1095, 118, 1096, 236, 236, 80, 1097, 1097, 236, 1097, 1097, 1097, 738, 238, 414, 414,
    414, 414, 349, 78, 78, 191, 0, 0, 0, 0, 104, 0, 1098, 380, 745, 177,
    177, 407, 464, 464, 909, 336, 910, 0, 103, 835, 1099, 103, 845, 846, 847, 835,
    858, 100, 100, 157, 124, 78, 1100, 78, 1074, 1101, 840, 866, 952, 1102, 0, 859,
    1103, 0, 0, 80, 0, 1103, 357, 357, 357, 0, 0, 80, 80, 0, 0, 711,
    0, 0, 12, 0, 0, 711, 0, 0, 0, 1104, 0, 80, 80, 0, 0, 711,
    0, 0, 12, 0, 0, 711, 0, 0, 0, 1104, 1105, 921, 1106, 272, 90, 1107,
    1108, 257, 0, 1109, 1110, 168, 1111, 344, 21, 21, 80, 0, 0, 113, 113, 80,
    0, 0, 0, 313, 315, 89, 80, 0, 0, 0, 313, 315, 100, 100, 80, 1032,
    1032, 104, 12, 1112, 104, 341, 341, 191, 0, 80, 0, 0, 341, 1113, 306, 686,
    1114, 370, 370, 738, 738, 370, 238, 0, 1115, 578, 485, 705, 484, 1005, 1116, 74,
    1117, 199, 1006, 753, 369, 683, 1118, 1119, 1120, 48, 687, 1121, 1122, 1123, 1124, 1125,
    1126, 369, 0, 1127, 0, 827, 0, 1128, 483, 822, 822, 822, 554, 850, 850, 838,
    666, 1129, 1130, 729, 729, 729, 0, 80, 0, 0, 79, 1131, 433, 1016, 1132, 612,
    466, 466, 467, 466, 1133, 804, 551, 36, 1134, 160, 42, 57, 1135, 1136, 1137, 507,
    1138, 738, 1139, 544, 823, 823, 823, 824, 824, 824, 377, 509, 509, 80, 377, 377,
    0, 0, 238, 134, 281, 283, 5, 5, 0, 0, 12, 12, 283, 5, 17, 17,
    17, 21, 10, 0, 0, 0, 0, 12, 12, 283, 5, 17, 0, 0, 0, 80,
    493, 493, 80, 0, 1140, 311, 311, 104, 173, 173, 80, 0, 0, 67, 0, 0,
    0, 0, 67, 67, 0, 846, 407, 407, 1141, 14, 366, 1142, 80, 433, 1143, 1144,
    514, 112, 1145, 1146, 238, 1147, 1147, 960, 1148, 1149, 1039, 960, 1150, 1150, 0, 1039,
    0, 1151, 1151, 0, 1039, 0, 1152, 1152, 0, 1153, 1153, 1154, 1153, 1039, 1154, 1155,
    1156, 1157, 621, 109, 374, 1003, 373, 1078, 1158, 1159, 664, 1160, 1160, 1160, 272, 191,
    0, 0, 91, 92, 93, 0, 89, 0, 91, 92, 93, 0, 1161, 1162, 1163, 1164,
    121, 121, 0, 80, 998, 37, 37, 37, 30, 1165, 37, 37, 29, 1165, 998, 998,
    1166, 233, 1111, 1167, 1168, 121, 631, 1165, 126, 94, 94, 1169, 110, 140, 80, 0,
    0, 125, 218, 218, 71, 329, 329, 329, 49, 39, 0, 0, 100, 1170, 1171, 1172,
    552, 1173, 1173, 1173, 80, 0, 0, 0, 0, 102, 106, 106, 0, 363, 363, 102,
    1174, 232, 232, 0, 0, 5, 5, 0, 0, 10, 10, 0, 0, 1175, 1175, 9,
    13, 0, 0, 738, 738, 67, 67, 738, 109, 109, 737, 80, 738, 738, 1176, 1177,
    1178, 1112, 1179, 88, 1180, 1181, 349, 214, 538, 299, 1182, 115, 115, 115, 1183, 389,
    115, 115, 0, 14, 129, 0, 1184, 1185, 902, 195, 125, 260, 579, 1186, 1187, 862,
    741, 245, 64, 64, 64, 63, 63, 118, 124, 0, 26, 26, 37, 0, 129, 129,
    0, 1107, 1107, 37, 130, 0, 0, 113, 0, 127, 1055, 951, 1188, 129, 0, 131,
    1189, 647, 647, 96, 43, 58, 219, 1190, 1191, 1192, 1193, 263, 215, 1194, 261, 851,
    851, 1195, 399, 13, 80, 104, 1181, 1181, 1181, 80, 104, 1196, 1197, 1198, 954, 1199,
    1200, 10, 23, 1201, 1202, 164, 938, 1203, 1204, 7, 19, 221, 221, 221, 958, 124,
    124, 0, 37, 1205, 5, 5, 0, 180, 180, 0, 77, 77, 0, 80, 80, 9,
    9, 0, 135, 135, 0, 155, 155, 122, 0, 156, 156, 5, 0, 41, 41, 0,
    0, 80, 0, 961, 961, 955, 104, 49, 49, 39, 0, 159, 160, 161, 0, 42,
    42, 0, 0, 80, 0, 281, 281, 0, 129, 129, 0, 43, 43, 0, 0, 80,
    0, 10, 10, 0, 44, 44, 0, 327, 327, 0, 106, 106, 0, 6, 6, 0,
    39, 39, 0, 476, 307, 238, 140, 1206, 0, 0, 0, 0, 0, 1207, 465, 465,
    37, 37, 80, 0, 0, 14, 27, 27, 1207, 0, 0, 585, 113, 493, 78, 78,
    191, 37, 37, 494, 103, 741, 1104, 848, 100, 100, 218, 157, 100, 1022, 1208, 585,
    113, 0, 493, 78, 78, 191, 103, 741, 12, 848, 1022, 27, 27, 27, 585, 113,
    493, 133, 845, 0, 847, 847, 78, 103, 849, 812, 124, 848, 100, 100, 37, 37,
    494, 741, 100, 1022, 433, 504, 80, 100, 504, 80, 100, 1071, 80, 433, 67, 67,
    0, 66, 65, 67, 67, 0, 66, 469, 858, 44, 314, 315, 315, 104, 218, 218,
    315, 312, 313, 313, 104, 218, 218, 313, 59, 669, 670, 671, 668, 1209, 162, 1210,
    1211, 1212, 640, 130, 91, 92, 93, 1183, 93, 94, 94, 0, 0, 0, 0, 0,
    0, 66, 109, 0, 111, 185, 0, 0, 0, 110, 0, 0, 80, 80, 0, 0,
    109, 80, 0, 433, 0, 0, 0, 111, 185, 0, 0, 0, 110, 0, 0, 94,
    94, 498, 0, 14, 0, 0, 1213, 208, 209, 421, 421, 848, 848, 848, 421, 78,
    103, 847, 94, 848, 1214, 1215, 1215, 931, 777, 14, 1027, 94, 94, 10, 95, 0,
    94, 94, 1216, 1216, 609, 1183, 1183, 1183, 1183, 389, 0, 0, 110, 94, 94, 1216,
    1183, 609, 0, 0, 0, 108, 89, 109, 0, 110, 0, 0, 111, 0, 112, 37,
    1217, 1217, 1218, 1219, 238, 609, 110, 10, 10, 238, 129, 129, 0, 321, 0, 568,
    882, 788, 1220, 831, 1221, 0, 0, 0, 439, 443, 443, 443, 0, 384, 1222, 1222,
    31, 1223, 8, 8, 232, 232, 49, 49, 39, 276, 276, 275, 275, 5, 5, 39,
    39, 0, 1222, 1222, 913, 37, 37, 0, 113, 113, 0, 21, 1224, 617, 617, 238,
    493, 493, 0, 1222, 1222, 1222, 49, 54, 113, 113, 0, 21, 1222, 1222, 1225, 470,
    20, 25, 11, 11, 538, 24, 913, 49, 49, 54, 19, 23, 1226, 1227, 17, 18,
    54, 1083, 0, 113, 113, 0, 1224, 617, 617, 238, 493, 493, 0, 1228, 1229, 311,
    369, 73, 969, 104, 104, 1167, 0, 0, 112, 738, 0, 0, 112, 67, 0, 0,
    0, 0, 67, 86, 960, 276, 1226, 324, 379, 379, 66, 330, 364, 370, 377, 656,
    667, 1230, 1142, 1231, 275, 1227, 1232, 1233, 1234, 1235, 1236, 1237, 1238, 1239, 1240, 1241,
    1242, 1243, 1244, 1245, 555, 1246, 249, 40, 55, 1247, 86, 317, 1248, 1249, 637, 825,
    433, 1016, 28, 1250, 137, 137, 31, 137, 123, 62, 31, 31, 137, 85, 31, 94,
    137, 137, 31, 137, 31, 30, 29, 31, 137, 31, 137, 31, 137, 746, 1251, 785,
    31, 85, 137, 31, 137, 85, 85, 85, 85, 85, 85, 85, 85, 458, 502, 502,
    31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 31, 137, 812, 785,
    1222, 1222, 1222, 1222, 733, 469, 196, 503, 527, 197, 462, 1252, 1253, 45, 60, 246,
    405, 74, 181, 1254, 1255, 80, 0, 0, 0, 0, 0, 1256, 530, 327, 1257, 1258,
    1259, 791, 287, 264, 0, 1260, 1261, 1262, 583, 13, 203, 141, 141, 0, 1263, 144,
    144, 80, 0, 218, 218, 229, 463, 463, 464, 464, 139, 465, 80, 911, 466, 466,
    80, 911, 778, 778, 177, 177, 0, 80, 0, 0, 911, 493, 493, 80, 0, 156,
    156, 467, 80, 121, 1264, 205, 206, 139, 139, 206, 207, 207, 207, 208, 207, 207,
    209, 139, 139, 145, 218, 218, 229, 204, 141, 141, 0, 1265, 1265, 463, 466, 177,
    232, 0, 205, 206, 139, 139, 206, 207, 207, 207, 208, 207, 207, 209, 0, 0,
    343, 185, 0, 0, 100, 100, 1101, 0, 80, 433, 80, 1016, 5, 17, 851, 106,
    1266, 0, 802, 802, 404, 0, 0, 806, 806, 0, 1019, 46, 61, 641, 169, 0,
    567, 641, 642, 642, 1267, 1268, 567, 1269, 1270, 1271, 163, 1272, 1273, 1017, 1274, 817,
    803, 1217, 712, 712, 1217, 1275, 80, 535, 535, 100, 222, 1217, 1218, 384, 384, 384,
    1217, 1218, 0, 1276, 1277, 1275, 715, 714, 790, 1278, 1279, 1280, 1281, 89, 1282, 1283,
    1284, 1285, 596, 1286, 1287, 1288, 1289, 1250, 1250, 317, 398, 478, 1189, 442, 1290, 1289,
    1291, 316, 316, 0, 365, 365, 218, 446, 1292, 178, 1268, 1293, 1294, 122, 1295, 856,
    418, 964, 1287, 416, 416, 123, 123, 123, 5, 0, 0, 0, 239, 0, 0, 0,
    86, 1296, 1296, 195, 501, 192, 127, 1297, 1298, 694, 702, 703, 693, 556, 80, 0,
    0, 6, 979, 855, 855, 0, 0, 807, 379, 66, 0, 0, 80, 18, 1152, 1299,
    836, 1300, 644, 0, 1058, 644, 0, 1301, 1302, 1214, 1214, 1207, 465, 14, 1214, 463,
    463, 463, 942, 942, 1303, 1303, 1303, 0, 0, 86, 1304, 227, 27, 27, 585, 113,
    493, 133, 845, 0, 847, 848, 100, 100, 78, 103, 100, 100, 218, 157, 37, 37,
    494, 741, 100, 1022, 421, 421, 848, 848, 848, 421, 259, 1305, 308, 29, 29, 29,
    239, 0, 0, 94, 94, 113, 493, 493, 493, 114, 230, 0, 493, 741, 741, 0,
    86, 0, 0, 0, 0, 1306, 589, 1307, 32, 1251, 1251, 1251, 0, 1308, 130, 0,
    1309, 1310, 177, 1311, 1312, 1313, 695, 1314, 1314, 1314, 1315, 1316, 233, 511, 363, 360,
    359, 358, 80, 1317, 1317, 1317, 233, 511, 363, 359, 358, 750, 364, 912, 1136, 123,
    257, 0, 111, 0, 37, 0, 0, 0, 80, 145, 146, 563, 351, 343, 343, 80,
    80, 0, 0, 0, 110, 0, 78, 103, 343, 1318, 680, 1319, 1320, 1321, 855, 679,
    706, 363, 0, 238, 0, 549, 39, 54, 250, 1322, 789, 807, 1323, 1324, 1325, 597,
    1326, 1327, 653, 1328, 1328, 683, 974, 687, 1044, 1040, 1048, 871, 1119, 976, 1122, 1046,
    1042, 1050, 873, 1118, 975, 1121, 1045, 1041, 1049, 872, 1120, 977, 1123, 1047, 1043, 1051,
    874, 456, 487, 652, 1038, 1052, 1124, 1294, 1278, 80, 1329, 1329, 1330, 80, 104, 104,
    80, 80, 80, 104, 104, 653, 1331, 1331, 870, 869, 80, 653, 653, 683, 974, 687,
    1044, 1040, 1048, 871, 1119, 976, 1122, 1046, 1042, 1050, 873, 1118, 975, 1121, 1045, 1041,
    1049, 872, 1120, 977, 1123, 1047, 1043, 1051, 874, 456, 487, 652, 1038, 1052, 1124, 1294,
    1278, 870, 869, 12, 12, 104, 440, 440, 0, 0, 0, 1017, 0, 0, 0, 0,
    440, 493, 493, 0, 80, 0, 1332, 103, 133, 133, 191, 78, 78, 191, 845, 846,
    847, 1104, 741, 37, 37, 494, 124, 493, 1333, 1334, 0, 0, 0, 0, 0, 0,
    0, 0, 67, 86, 86, 67, 86, 847, 0, 0, 1335, 341, 78, 341, 78, 835,
    1336, 100, 100, 1337, 438, 609, 144, 118, 100, 100, 1163, 0, 0, 0, 27, 0,
    709, 100, 100, 1163, 0, 0, 0, 27, 0, 1071, 100, 100, 218, 218, 157, 0,
    100, 100, 100, 218, 10, 409, 1127, 1127, 133, 80, 78, 1127, 222, 0, 1291, 1338,
    158, 1339, 1340, 1341, 696, 1342, 1343, 1344, 1345, 1346, 1347, 1348, 1349, 1350, 1351, 1352,
    1353, 793, 935, 1354, 1355, 1356, 1357, 1218, 1358,
];

pub fn get_materialize_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &MATERIALIZE_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &MATERIALIZE_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &MATERIALIZE_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &MATERIALIZE_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(44), tables: &MATERIALIZE_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(57), tables: &MATERIALIZE_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(60), tables: &MATERIALIZE_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(83), tables: &MATERIALIZE_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(106), tables: &MATERIALIZE_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(109), tables: &MATERIALIZE_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(111), tables: &MATERIALIZE_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(114), tables: &MATERIALIZE_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(115), tables: &MATERIALIZE_TABLES }),
        "AcksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(116), tables: &MATERIALIZE_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(117), tables: &MATERIALIZE_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(118), tables: &MATERIALIZE_TABLES }),
        "AdjacentSegment" => Some(RootGrammar { grammar_id: GrammarId(119), tables: &MATERIALIZE_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(123), tables: &MATERIALIZE_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(124), tables: &MATERIALIZE_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(125), tables: &MATERIALIZE_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(126), tables: &MATERIALIZE_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(127), tables: &MATERIALIZE_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(145), tables: &MATERIALIZE_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(146), tables: &MATERIALIZE_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(149), tables: &MATERIALIZE_TABLES }),
        "Allow_connectionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(150), tables: &MATERIALIZE_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(151), tables: &MATERIALIZE_TABLES }),
        "AlterAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(152), tables: &MATERIALIZE_TABLES }),
        "AlterConnectionRotateKeys" => Some(RootGrammar { grammar_id: GrammarId(178), tables: &MATERIALIZE_TABLES }),
        "AlterDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(185), tables: &MATERIALIZE_TABLES }),
        "AlterDefaultPrivilegesGrantSegment" => Some(RootGrammar { grammar_id: GrammarId(240), tables: &MATERIALIZE_TABLES }),
        "AlterDefaultPrivilegesObjectPrivilegesSegment" => Some(RootGrammar { grammar_id: GrammarId(253), tables: &MATERIALIZE_TABLES }),
        "AlterDefaultPrivilegesRevokeSegment" => Some(RootGrammar { grammar_id: GrammarId(270), tables: &MATERIALIZE_TABLES }),
        "AlterDefaultPrivilegesSchemaObjectsSegment" => Some(RootGrammar { grammar_id: GrammarId(285), tables: &MATERIALIZE_TABLES }),
        "AlterDefaultPrivilegesStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(292), tables: &MATERIALIZE_TABLES }),
        "AlterDefaultPrivilegesToFromRolesSegment" => Some(RootGrammar { grammar_id: GrammarId(327), tables: &MATERIALIZE_TABLES }),
        "AlterDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(332), tables: &MATERIALIZE_TABLES }),
        "AlterExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(399), tables: &MATERIALIZE_TABLES }),
        "AlterForeignTableActionSegment" => Some(RootGrammar { grammar_id: GrammarId(507), tables: &MATERIALIZE_TABLES }),
        "AlterForeignTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(762), tables: &MATERIALIZE_TABLES }),
        "AlterFunctionActionSegment" => Some(RootGrammar { grammar_id: GrammarId(784), tables: &MATERIALIZE_TABLES }),
        "AlterFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(848), tables: &MATERIALIZE_TABLES }),
        "AlterIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(881), tables: &MATERIALIZE_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(888), tables: &MATERIALIZE_TABLES }),
        "AlterMaterializedViewActionSegment" => Some(RootGrammar { grammar_id: GrammarId(889), tables: &MATERIALIZE_TABLES }),
        "AlterMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(954), tables: &MATERIALIZE_TABLES }),
        "AlterOwnerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1002), tables: &MATERIALIZE_TABLES }),
        "AlterPolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1024), tables: &MATERIALIZE_TABLES }),
        "AlterProcedureActionSegment" => Some(RootGrammar { grammar_id: GrammarId(1054), tables: &MATERIALIZE_TABLES }),
        "AlterProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1083), tables: &MATERIALIZE_TABLES }),
        "AlterPublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1131), tables: &MATERIALIZE_TABLES }),
        "AlterRenameStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1163), tables: &MATERIALIZE_TABLES }),
        "AlterRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1184), tables: &MATERIALIZE_TABLES }),
        "AlterSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1274), tables: &MATERIALIZE_TABLES }),
        "AlterSecretStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1287), tables: &MATERIALIZE_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(1294), tables: &MATERIALIZE_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1345), tables: &MATERIALIZE_TABLES }),
        "AlterSetClusterStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1368), tables: &MATERIALIZE_TABLES }),
        "AlterSourceSinkSizeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1379), tables: &MATERIALIZE_TABLES }),
        "AlterStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1390), tables: &MATERIALIZE_TABLES }),
        "AlterSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1419), tables: &MATERIALIZE_TABLES }),
        "AlterSystemStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1467), tables: &MATERIALIZE_TABLES }),
        "AlterTableActionSegment" => Some(RootGrammar { grammar_id: GrammarId(1490), tables: &MATERIALIZE_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(1730), tables: &MATERIALIZE_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1735), tables: &MATERIALIZE_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1762), tables: &MATERIALIZE_TABLES }),
        "AlterTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1828), tables: &MATERIALIZE_TABLES }),
        "AlterTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1893), tables: &MATERIALIZE_TABLES }),
        "AlterTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1910), tables: &MATERIALIZE_TABLES }),
        "AlterViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1984), tables: &MATERIALIZE_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2042), tables: &MATERIALIZE_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(2043), tables: &MATERIALIZE_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2044), tables: &MATERIALIZE_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2045), tables: &MATERIALIZE_TABLES }),
        "AnalyzeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2046), tables: &MATERIALIZE_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2065), tables: &MATERIALIZE_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2066), tables: &MATERIALIZE_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2067), tables: &MATERIALIZE_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2068), tables: &MATERIALIZE_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2069), tables: &MATERIALIZE_TABLES }),
        "ArnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2080), tables: &MATERIALIZE_TABLES }),
        "ArrangementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2081), tables: &MATERIALIZE_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(2082), tables: &MATERIALIZE_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2099), tables: &MATERIALIZE_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2100), tables: &MATERIALIZE_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2101), tables: &MATERIALIZE_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2104), tables: &MATERIALIZE_TABLES }),
        "ArrayTypeSuffixSegment" => Some(RootGrammar { grammar_id: GrammarId(2105), tables: &MATERIALIZE_TABLES }),
        "AsAliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2107), tables: &MATERIALIZE_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2110), tables: &MATERIALIZE_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2112), tables: &MATERIALIZE_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2113), tables: &MATERIALIZE_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2114), tables: &MATERIALIZE_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2115), tables: &MATERIALIZE_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2116), tables: &MATERIALIZE_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2117), tables: &MATERIALIZE_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2118), tables: &MATERIALIZE_TABLES }),
        "AttachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2119), tables: &MATERIALIZE_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2120), tables: &MATERIALIZE_TABLES }),
        "AuctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2121), tables: &MATERIALIZE_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2122), tables: &MATERIALIZE_TABLES }),
        "AuthorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2123), tables: &MATERIALIZE_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2124), tables: &MATERIALIZE_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(2125), tables: &MATERIALIZE_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2127), tables: &MATERIALIZE_TABLES }),
        "AvailabilityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2128), tables: &MATERIALIZE_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2129), tables: &MATERIALIZE_TABLES }),
        "AvroKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2130), tables: &MATERIALIZE_TABLES }),
        "AwsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2131), tables: &MATERIALIZE_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2132), tables: &MATERIALIZE_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2133), tables: &MATERIALIZE_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(2134), tables: &MATERIALIZE_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(2135), tables: &MATERIALIZE_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(2149), tables: &MATERIALIZE_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(2150), tables: &MATERIALIZE_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2151), tables: &MATERIALIZE_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2152), tables: &MATERIALIZE_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2153), tables: &MATERIALIZE_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2154), tables: &MATERIALIZE_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2155), tables: &MATERIALIZE_TABLES }),
        "BigserialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2156), tables: &MATERIALIZE_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2157), tables: &MATERIALIZE_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2158), tables: &MATERIALIZE_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2164), tables: &MATERIALIZE_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2165), tables: &MATERIALIZE_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2166), tables: &MATERIALIZE_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2167), tables: &MATERIALIZE_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(2168), tables: &MATERIALIZE_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(2169), tables: &MATERIALIZE_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(2172), tables: &MATERIALIZE_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(2173), tables: &MATERIALIZE_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(2176), tables: &MATERIALIZE_TABLES }),
        "BodyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2177), tables: &MATERIALIZE_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2178), tables: &MATERIALIZE_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2179), tables: &MATERIALIZE_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2182), tables: &MATERIALIZE_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2183), tables: &MATERIALIZE_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2186), tables: &MATERIALIZE_TABLES }),
        "BoxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2187), tables: &MATERIALIZE_TABLES }),
        "BpcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2188), tables: &MATERIALIZE_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(2189), tables: &MATERIALIZE_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(2192), tables: &MATERIALIZE_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(2195), tables: &MATERIALIZE_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2196), tables: &MATERIALIZE_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2198), tables: &MATERIALIZE_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2199), tables: &MATERIALIZE_TABLES }),
        "BrokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2200), tables: &MATERIALIZE_TABLES }),
        "BrokersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2201), tables: &MATERIALIZE_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2202), tables: &MATERIALIZE_TABLES }),
        "BucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2203), tables: &MATERIALIZE_TABLES }),
        "BuffersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2204), tables: &MATERIALIZE_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2205), tables: &MATERIALIZE_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2206), tables: &MATERIALIZE_TABLES }),
        "BypassrlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2207), tables: &MATERIALIZE_TABLES }),
        "ByteaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2208), tables: &MATERIALIZE_TABLES }),
        "BytesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2209), tables: &MATERIALIZE_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(2210), tables: &MATERIALIZE_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2212), tables: &MATERIALIZE_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2239), tables: &MATERIALIZE_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2240), tables: &MATERIALIZE_TABLES }),
        "CallStoredProcedureSegment" => Some(RootGrammar { grammar_id: GrammarId(2241), tables: &MATERIALIZE_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2244), tables: &MATERIALIZE_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2245), tables: &MATERIALIZE_TABLES }),
        "CascadeRestrictGrammar" => Some(RootGrammar { grammar_id: GrammarId(2246), tables: &MATERIALIZE_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2249), tables: &MATERIALIZE_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2250), tables: &MATERIALIZE_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2274), tables: &MATERIALIZE_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2275), tables: &MATERIALIZE_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2276), tables: &MATERIALIZE_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2277), tables: &MATERIALIZE_TABLES }),
        "CertificateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2278), tables: &MATERIALIZE_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2279), tables: &MATERIALIZE_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2280), tables: &MATERIALIZE_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(2281), tables: &MATERIALIZE_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2282), tables: &MATERIALIZE_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2283), tables: &MATERIALIZE_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2284), tables: &MATERIALIZE_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2285), tables: &MATERIALIZE_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2286), tables: &MATERIALIZE_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2287), tables: &MATERIALIZE_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2288), tables: &MATERIALIZE_TABLES }),
        "CidrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2289), tables: &MATERIALIZE_TABLES }),
        "CircleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2290), tables: &MATERIALIZE_TABLES }),
        "CircularstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2291), tables: &MATERIALIZE_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2292), tables: &MATERIALIZE_TABLES }),
        "ClientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2293), tables: &MATERIALIZE_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2294), tables: &MATERIALIZE_TABLES }),
        "CloseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2295), tables: &MATERIALIZE_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2298), tables: &MATERIALIZE_TABLES }),
        "ClusterStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2299), tables: &MATERIALIZE_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2312), tables: &MATERIALIZE_TABLES }),
        "ClustersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2313), tables: &MATERIALIZE_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2314), tables: &MATERIALIZE_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(2315), tables: &MATERIALIZE_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(2316), tables: &MATERIALIZE_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2317), tables: &MATERIALIZE_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2318), tables: &MATERIALIZE_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2319), tables: &MATERIALIZE_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(2325), tables: &MATERIALIZE_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(2326), tables: &MATERIALIZE_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(2327), tables: &MATERIALIZE_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(2328), tables: &MATERIALIZE_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(2333), tables: &MATERIALIZE_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2412), tables: &MATERIALIZE_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2420), tables: &MATERIALIZE_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2421), tables: &MATERIALIZE_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2422), tables: &MATERIALIZE_TABLES }),
        "ColumnTypeReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2448), tables: &MATERIALIZE_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2452), tables: &MATERIALIZE_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2453), tables: &MATERIALIZE_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2454), tables: &MATERIALIZE_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(2455), tables: &MATERIALIZE_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2456), tables: &MATERIALIZE_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(2457), tables: &MATERIALIZE_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2458), tables: &MATERIALIZE_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2461), tables: &MATERIALIZE_TABLES }),
        "CommentOnStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2462), tables: &MATERIALIZE_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(2573), tables: &MATERIALIZE_TABLES }),
        "CommentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2574), tables: &MATERIALIZE_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2575), tables: &MATERIALIZE_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2576), tables: &MATERIALIZE_TABLES }),
        "CompactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2577), tables: &MATERIALIZE_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2578), tables: &MATERIALIZE_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2603), tables: &MATERIALIZE_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2604), tables: &MATERIALIZE_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2605), tables: &MATERIALIZE_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2606), tables: &MATERIALIZE_TABLES }),
        "CompositeValueExpansionSegment" => Some(RootGrammar { grammar_id: GrammarId(2607), tables: &MATERIALIZE_TABLES }),
        "CompoundcurveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2612), tables: &MATERIALIZE_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2613), tables: &MATERIALIZE_TABLES }),
        "CompressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2614), tables: &MATERIALIZE_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2615), tables: &MATERIALIZE_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(2616), tables: &MATERIALIZE_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2619), tables: &MATERIALIZE_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2620), tables: &MATERIALIZE_TABLES }),
        "ConfigurationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2624), tables: &MATERIALIZE_TABLES }),
        "ConflictActionSegment" => Some(RootGrammar { grammar_id: GrammarId(2625), tables: &MATERIALIZE_TABLES }),
        "ConflictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2661), tables: &MATERIALIZE_TABLES }),
        "ConflictTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(2662), tables: &MATERIALIZE_TABLES }),
        "ConfluentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2683), tables: &MATERIALIZE_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2684), tables: &MATERIALIZE_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2685), tables: &MATERIALIZE_TABLES }),
        "ConnectionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2686), tables: &MATERIALIZE_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2687), tables: &MATERIALIZE_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2688), tables: &MATERIALIZE_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2689), tables: &MATERIALIZE_TABLES }),
        "ContentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2690), tables: &MATERIALIZE_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2691), tables: &MATERIALIZE_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2692), tables: &MATERIALIZE_TABLES }),
        "CopyFromStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2693), tables: &MATERIALIZE_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2705), tables: &MATERIALIZE_TABLES }),
        "CopyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2706), tables: &MATERIALIZE_TABLES }),
        "CopyToStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2865), tables: &MATERIALIZE_TABLES }),
        "CorrelationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2883), tables: &MATERIALIZE_TABLES }),
        "CostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2884), tables: &MATERIALIZE_TABLES }),
        "CostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2885), tables: &MATERIALIZE_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2886), tables: &MATERIALIZE_TABLES }),
        "CounterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2887), tables: &MATERIALIZE_TABLES }),
        "CreateAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2888), tables: &MATERIALIZE_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2896), tables: &MATERIALIZE_TABLES }),
        "CreateClusterReplicaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2922), tables: &MATERIALIZE_TABLES }),
        "CreateClusterStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2929), tables: &MATERIALIZE_TABLES }),
        "CreateCollationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2941), tables: &MATERIALIZE_TABLES }),
        "CreateConnectionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2979), tables: &MATERIALIZE_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3000), tables: &MATERIALIZE_TABLES }),
        "CreateDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3053), tables: &MATERIALIZE_TABLES }),
        "CreateExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3078), tables: &MATERIALIZE_TABLES }),
        "CreateForeignDataWrapperStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3094), tables: &MATERIALIZE_TABLES }),
        "CreateForeignTableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3102), tables: &MATERIALIZE_TABLES }),
        "CreateForeignTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3106), tables: &MATERIALIZE_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3162), tables: &MATERIALIZE_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3189), tables: &MATERIALIZE_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3213), tables: &MATERIALIZE_TABLES }),
        "CreateMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3214), tables: &MATERIALIZE_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3239), tables: &MATERIALIZE_TABLES }),
        "CreateOperatorStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3259), tables: &MATERIALIZE_TABLES }),
        "CreatePolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3294), tables: &MATERIALIZE_TABLES }),
        "CreateProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3331), tables: &MATERIALIZE_TABLES }),
        "CreatePublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3338), tables: &MATERIALIZE_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3354), tables: &MATERIALIZE_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3418), tables: &MATERIALIZE_TABLES }),
        "CreateSecretStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3428), tables: &MATERIALIZE_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(3435), tables: &MATERIALIZE_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3479), tables: &MATERIALIZE_TABLES }),
        "CreateServerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3487), tables: &MATERIALIZE_TABLES }),
        "CreateSinkKafkaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3501), tables: &MATERIALIZE_TABLES }),
        "CreateSourceKafkaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3529), tables: &MATERIALIZE_TABLES }),
        "CreateSourceLoadGeneratorStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3570), tables: &MATERIALIZE_TABLES }),
        "CreateSourcePostgresStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &MATERIALIZE_TABLES }),
        "CreateSourceWebhookStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3627), tables: &MATERIALIZE_TABLES }),
        "CreateStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3663), tables: &MATERIALIZE_TABLES }),
        "CreateSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3683), tables: &MATERIALIZE_TABLES }),
        "CreateTableAsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3695), tables: &MATERIALIZE_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3760), tables: &MATERIALIZE_TABLES }),
        "CreateTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3868), tables: &MATERIALIZE_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3884), tables: &MATERIALIZE_TABLES }),
        "CreateTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3960), tables: &MATERIALIZE_TABLES }),
        "CreateUserMappingGrammar" => Some(RootGrammar { grammar_id: GrammarId(3983), tables: &MATERIALIZE_TABLES }),
        "CreateUserMappingStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3987), tables: &MATERIALIZE_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3998), tables: &MATERIALIZE_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4002), tables: &MATERIALIZE_TABLES }),
        "CreateclusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4015), tables: &MATERIALIZE_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4016), tables: &MATERIALIZE_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4017), tables: &MATERIALIZE_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4018), tables: &MATERIALIZE_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4019), tables: &MATERIALIZE_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4020), tables: &MATERIALIZE_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4021), tables: &MATERIALIZE_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4022), tables: &MATERIALIZE_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4023), tables: &MATERIALIZE_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4029), tables: &MATERIALIZE_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4030), tables: &MATERIALIZE_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4031), tables: &MATERIALIZE_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4032), tables: &MATERIALIZE_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4033), tables: &MATERIALIZE_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4034), tables: &MATERIALIZE_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4035), tables: &MATERIALIZE_TABLES }),
        "CurvepolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4036), tables: &MATERIALIZE_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4037), tables: &MATERIALIZE_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4038), tables: &MATERIALIZE_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4039), tables: &MATERIALIZE_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4044), tables: &MATERIALIZE_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4045), tables: &MATERIALIZE_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(4049), tables: &MATERIALIZE_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4154), tables: &MATERIALIZE_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(4155), tables: &MATERIALIZE_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4156), tables: &MATERIALIZE_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(4157), tables: &MATERIALIZE_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(4163), tables: &MATERIALIZE_TABLES }),
        "DateTimeTypeIdentifier" => Some(RootGrammar { grammar_id: GrammarId(4166), tables: &MATERIALIZE_TABLES }),
        "DaterangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4176), tables: &MATERIALIZE_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4177), tables: &MATERIALIZE_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(4178), tables: &MATERIALIZE_TABLES }),
        "DatumsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4179), tables: &MATERIALIZE_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4180), tables: &MATERIALIZE_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4181), tables: &MATERIALIZE_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4182), tables: &MATERIALIZE_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4183), tables: &MATERIALIZE_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4184), tables: &MATERIALIZE_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4185), tables: &MATERIALIZE_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4186), tables: &MATERIALIZE_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4187), tables: &MATERIALIZE_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4188), tables: &MATERIALIZE_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4189), tables: &MATERIALIZE_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4190), tables: &MATERIALIZE_TABLES }),
        "DeallocateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4191), tables: &MATERIALIZE_TABLES }),
        "DebeziumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4197), tables: &MATERIALIZE_TABLES }),
        "DebugKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4198), tables: &MATERIALIZE_TABLES }),
        "DebuggingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4199), tables: &MATERIALIZE_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4200), tables: &MATERIALIZE_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4201), tables: &MATERIALIZE_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4202), tables: &MATERIALIZE_TABLES }),
        "DeclareStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4203), tables: &MATERIALIZE_TABLES }),
        "DecorrelatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4222), tables: &MATERIALIZE_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(144), tables: &MATERIALIZE_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4223), tables: &MATERIALIZE_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(4224), tables: &MATERIALIZE_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4227), tables: &MATERIALIZE_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4228), tables: &MATERIALIZE_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4229), tables: &MATERIALIZE_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4230), tables: &MATERIALIZE_TABLES }),
        "DefinitionArgumentValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(4231), tables: &MATERIALIZE_TABLES }),
        "DefinitionParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(4234), tables: &MATERIALIZE_TABLES }),
        "DefinitionParametersSegment" => Some(RootGrammar { grammar_id: GrammarId(4239), tables: &MATERIALIZE_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4242), tables: &MATERIALIZE_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4243), tables: &MATERIALIZE_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4244), tables: &MATERIALIZE_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4245), tables: &MATERIALIZE_TABLES }),
        "DelimitedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4274), tables: &MATERIALIZE_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(4275), tables: &MATERIALIZE_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4276), tables: &MATERIALIZE_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4277), tables: &MATERIALIZE_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4278), tables: &MATERIALIZE_TABLES }),
        "DependenciesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4279), tables: &MATERIALIZE_TABLES }),
        "DependsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4280), tables: &MATERIALIZE_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4281), tables: &MATERIALIZE_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4282), tables: &MATERIALIZE_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4283), tables: &MATERIALIZE_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4284), tables: &MATERIALIZE_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4288), tables: &MATERIALIZE_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4289), tables: &MATERIALIZE_TABLES }),
        "DetachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4290), tables: &MATERIALIZE_TABLES }),
        "DetailsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4291), tables: &MATERIALIZE_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4292), tables: &MATERIALIZE_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4293), tables: &MATERIALIZE_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4294), tables: &MATERIALIZE_TABLES }),
        "Disable_page_skippingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4295), tables: &MATERIALIZE_TABLES }),
        "DiscardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4296), tables: &MATERIALIZE_TABLES }),
        "DiscardStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4297), tables: &MATERIALIZE_TABLES }),
        "DiscoverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4305), tables: &MATERIALIZE_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4306), tables: &MATERIALIZE_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4307), tables: &MATERIALIZE_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4308), tables: &MATERIALIZE_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4309), tables: &MATERIALIZE_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4310), tables: &MATERIALIZE_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(4311), tables: &MATERIALIZE_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4312), tables: &MATERIALIZE_TABLES }),
        "DoStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4313), tables: &MATERIALIZE_TABLES }),
        "DocumentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4322), tables: &MATERIALIZE_TABLES }),
        "DollarNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4323), tables: &MATERIALIZE_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4324), tables: &MATERIALIZE_TABLES }),
        "DotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4325), tables: &MATERIALIZE_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(4326), tables: &MATERIALIZE_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4327), tables: &MATERIALIZE_TABLES }),
        "DropAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4328), tables: &MATERIALIZE_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4340), tables: &MATERIALIZE_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4343), tables: &MATERIALIZE_TABLES }),
        "DropCollationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4352), tables: &MATERIALIZE_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4358), tables: &MATERIALIZE_TABLES }),
        "DropDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4367), tables: &MATERIALIZE_TABLES }),
        "DropExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4374), tables: &MATERIALIZE_TABLES }),
        "DropForeignTableStatement" => Some(RootGrammar { grammar_id: GrammarId(4380), tables: &MATERIALIZE_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4388), tables: &MATERIALIZE_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4397), tables: &MATERIALIZE_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4405), tables: &MATERIALIZE_TABLES }),
        "DropMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4406), tables: &MATERIALIZE_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4414), tables: &MATERIALIZE_TABLES }),
        "DropOwnedStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4419), tables: &MATERIALIZE_TABLES }),
        "DropPolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4430), tables: &MATERIALIZE_TABLES }),
        "DropProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4438), tables: &MATERIALIZE_TABLES }),
        "DropPublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4449), tables: &MATERIALIZE_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4456), tables: &MATERIALIZE_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4461), tables: &MATERIALIZE_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4467), tables: &MATERIALIZE_TABLES }),
        "DropStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4474), tables: &MATERIALIZE_TABLES }),
        "DropStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4503), tables: &MATERIALIZE_TABLES }),
        "DropSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4512), tables: &MATERIALIZE_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4520), tables: &MATERIALIZE_TABLES }),
        "DropTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4528), tables: &MATERIALIZE_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4536), tables: &MATERIALIZE_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4544), tables: &MATERIALIZE_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4551), tables: &MATERIALIZE_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4556), tables: &MATERIALIZE_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4563), tables: &MATERIALIZE_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4564), tables: &MATERIALIZE_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4565), tables: &MATERIALIZE_TABLES }),
        "EffortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4566), tables: &MATERIALIZE_TABLES }),
        "ElementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4567), tables: &MATERIALIZE_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4568), tables: &MATERIALIZE_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4571), tables: &MATERIALIZE_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4572), tables: &MATERIALIZE_TABLES }),
        "EmptyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4573), tables: &MATERIALIZE_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(4574), tables: &MATERIALIZE_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4575), tables: &MATERIALIZE_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4578), tables: &MATERIALIZE_TABLES }),
        "EnabledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4579), tables: &MATERIALIZE_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4580), tables: &MATERIALIZE_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4581), tables: &MATERIALIZE_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4582), tables: &MATERIALIZE_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(136), tables: &MATERIALIZE_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(4583), tables: &MATERIALIZE_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4584), tables: &MATERIALIZE_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2098), tables: &MATERIALIZE_TABLES }),
        "EndpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4585), tables: &MATERIALIZE_TABLES }),
        "EnforcedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4586), tables: &MATERIALIZE_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4587), tables: &MATERIALIZE_TABLES }),
        "EnvelopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4588), tables: &MATERIALIZE_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(4589), tables: &MATERIALIZE_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4590), tables: &MATERIALIZE_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4591), tables: &MATERIALIZE_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4592), tables: &MATERIALIZE_TABLES }),
        "EventKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4593), tables: &MATERIALIZE_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4594), tables: &MATERIALIZE_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4595), tables: &MATERIALIZE_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4596), tables: &MATERIALIZE_TABLES }),
        "ExclusionConstraintElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4597), tables: &MATERIALIZE_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4601), tables: &MATERIALIZE_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4602), tables: &MATERIALIZE_TABLES }),
        "ExecuteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4603), tables: &MATERIALIZE_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4609), tables: &MATERIALIZE_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4610), tables: &MATERIALIZE_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4611), tables: &MATERIALIZE_TABLES }),
        "ExpectedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4612), tables: &MATERIALIZE_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4613), tables: &MATERIALIZE_TABLES }),
        "ExplainOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(4614), tables: &MATERIALIZE_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4634), tables: &MATERIALIZE_TABLES }),
        "ExpressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4650), tables: &MATERIALIZE_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4651), tables: &MATERIALIZE_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4652), tables: &MATERIALIZE_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4678), tables: &MATERIALIZE_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4685), tables: &MATERIALIZE_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4695), tables: &MATERIALIZE_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4700), tables: &MATERIALIZE_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4719), tables: &MATERIALIZE_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(4756), tables: &MATERIALIZE_TABLES }),
        "ExtendedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4764), tables: &MATERIALIZE_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4765), tables: &MATERIALIZE_TABLES }),
        "ExtendedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(4766), tables: &MATERIALIZE_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4776), tables: &MATERIALIZE_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4777), tables: &MATERIALIZE_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4778), tables: &MATERIALIZE_TABLES }),
        "FactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4779), tables: &MATERIALIZE_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4780), tables: &MATERIALIZE_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(4781), tables: &MATERIALIZE_TABLES }),
        "FamilyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4782), tables: &MATERIALIZE_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4783), tables: &MATERIALIZE_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4800), tables: &MATERIALIZE_TABLES }),
        "FetchStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4801), tables: &MATERIALIZE_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4816), tables: &MATERIALIZE_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4817), tables: &MATERIALIZE_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(4818), tables: &MATERIALIZE_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4826), tables: &MATERIALIZE_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(4827), tables: &MATERIALIZE_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4833), tables: &MATERIALIZE_TABLES }),
        "FinalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4834), tables: &MATERIALIZE_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4835), tables: &MATERIALIZE_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4836), tables: &MATERIALIZE_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4837), tables: &MATERIALIZE_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4838), tables: &MATERIALIZE_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4839), tables: &MATERIALIZE_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4840), tables: &MATERIALIZE_TABLES }),
        "ForClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4841), tables: &MATERIALIZE_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4862), tables: &MATERIALIZE_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4863), tables: &MATERIALIZE_TABLES }),
        "Force_not_nullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4864), tables: &MATERIALIZE_TABLES }),
        "Force_nullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4865), tables: &MATERIALIZE_TABLES }),
        "Force_quoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4866), tables: &MATERIALIZE_TABLES }),
        "ForeignDataWrapperGrammar" => Some(RootGrammar { grammar_id: GrammarId(4867), tables: &MATERIALIZE_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(4871), tables: &MATERIALIZE_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4874), tables: &MATERIALIZE_TABLES }),
        "ForeignTableColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(4875), tables: &MATERIALIZE_TABLES }),
        "ForeignTableTableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(4904), tables: &MATERIALIZE_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4915), tables: &MATERIALIZE_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4916), tables: &MATERIALIZE_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4917), tables: &MATERIALIZE_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(4947), tables: &MATERIALIZE_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4951), tables: &MATERIALIZE_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4952), tables: &MATERIALIZE_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4953), tables: &MATERIALIZE_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4954), tables: &MATERIALIZE_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4958), tables: &MATERIALIZE_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4976), tables: &MATERIALIZE_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4999), tables: &MATERIALIZE_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5026), tables: &MATERIALIZE_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5027), tables: &MATERIALIZE_TABLES }),
        "FullTextSearchOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5028), tables: &MATERIALIZE_TABLES }),
        "FullnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5029), tables: &MATERIALIZE_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5030), tables: &MATERIALIZE_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5031), tables: &MATERIALIZE_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5039), tables: &MATERIALIZE_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(5100), tables: &MATERIALIZE_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5103), tables: &MATERIALIZE_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5205), tables: &MATERIALIZE_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5206), tables: &MATERIALIZE_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(5207), tables: &MATERIALIZE_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(5217), tables: &MATERIALIZE_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(5241), tables: &MATERIALIZE_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(5244), tables: &MATERIALIZE_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5258), tables: &MATERIALIZE_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5259), tables: &MATERIALIZE_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5260), tables: &MATERIALIZE_TABLES }),
        "GeneratorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5261), tables: &MATERIALIZE_TABLES }),
        "GeographyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5262), tables: &MATERIALIZE_TABLES }),
        "GeometryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5263), tables: &MATERIALIZE_TABLES }),
        "GeometrycollectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5264), tables: &MATERIALIZE_TABLES }),
        "GeometrycollectionmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5265), tables: &MATERIALIZE_TABLES }),
        "GeometrycollectionzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5266), tables: &MATERIALIZE_TABLES }),
        "GeometrycollectionzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5267), tables: &MATERIALIZE_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5268), tables: &MATERIALIZE_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5269), tables: &MATERIALIZE_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5270), tables: &MATERIALIZE_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5271), tables: &MATERIALIZE_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5278), tables: &MATERIALIZE_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5279), tables: &MATERIALIZE_TABLES }),
        "GraphKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5280), tables: &MATERIALIZE_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(5281), tables: &MATERIALIZE_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(5284), tables: &MATERIALIZE_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5285), tables: &MATERIALIZE_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5286), tables: &MATERIALIZE_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5305), tables: &MATERIALIZE_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5314), tables: &MATERIALIZE_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(5315), tables: &MATERIALIZE_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5323), tables: &MATERIALIZE_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5324), tables: &MATERIALIZE_TABLES }),
        "GroupsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5332), tables: &MATERIALIZE_TABLES }),
        "GzipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5333), tables: &MATERIALIZE_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5334), tables: &MATERIALIZE_TABLES }),
        "HashKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5335), tables: &MATERIALIZE_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5336), tables: &MATERIALIZE_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5341), tables: &MATERIALIZE_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5349), tables: &MATERIALIZE_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5350), tables: &MATERIALIZE_TABLES }),
        "HeadersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5351), tables: &MATERIALIZE_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5352), tables: &MATERIALIZE_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5353), tables: &MATERIALIZE_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5354), tables: &MATERIALIZE_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5355), tables: &MATERIALIZE_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5356), tables: &MATERIALIZE_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5357), tables: &MATERIALIZE_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5358), tables: &MATERIALIZE_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5359), tables: &MATERIALIZE_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5360), tables: &MATERIALIZE_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5361), tables: &MATERIALIZE_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5362), tables: &MATERIALIZE_TABLES }),
        "HoursKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5363), tables: &MATERIALIZE_TABLES }),
        "IcuKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5364), tables: &MATERIALIZE_TABLES }),
        "IdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5365), tables: &MATERIALIZE_TABLES }),
        "IdempotenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5366), tables: &MATERIALIZE_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5367), tables: &MATERIALIZE_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5368), tables: &MATERIALIZE_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5369), tables: &MATERIALIZE_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5370), tables: &MATERIALIZE_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5371), tables: &MATERIALIZE_TABLES }),
        "IdleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5372), tables: &MATERIALIZE_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5373), tables: &MATERIALIZE_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5376), tables: &MATERIALIZE_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5377), tables: &MATERIALIZE_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5381), tables: &MATERIALIZE_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5382), tables: &MATERIALIZE_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5387), tables: &MATERIALIZE_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5388), tables: &MATERIALIZE_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5389), tables: &MATERIALIZE_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(2253), tables: &MATERIALIZE_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5390), tables: &MATERIALIZE_TABLES }),
        "ImportForeignSchemaGrammar" => Some(RootGrammar { grammar_id: GrammarId(5391), tables: &MATERIALIZE_TABLES }),
        "ImportForeignSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5395), tables: &MATERIALIZE_TABLES }),
        "ImportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5413), tables: &MATERIALIZE_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5414), tables: &MATERIALIZE_TABLES }),
        "InCluster" => Some(RootGrammar { grammar_id: GrammarId(5415), tables: &MATERIALIZE_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5419), tables: &MATERIALIZE_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5420), tables: &MATERIALIZE_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5430), tables: &MATERIALIZE_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5431), tables: &MATERIALIZE_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5432), tables: &MATERIALIZE_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(128), tables: &MATERIALIZE_TABLES }),
        "IndexAccessMethodSegment" => Some(RootGrammar { grammar_id: GrammarId(5433), tables: &MATERIALIZE_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(5434), tables: &MATERIALIZE_TABLES }),
        "IndexElementOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(5439), tables: &MATERIALIZE_TABLES }),
        "IndexElementSegment" => Some(RootGrammar { grammar_id: GrammarId(5459), tables: &MATERIALIZE_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5466), tables: &MATERIALIZE_TABLES }),
        "IndexParametersSegment" => Some(RootGrammar { grammar_id: GrammarId(5467), tables: &MATERIALIZE_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "Index_cleanupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5479), tables: &MATERIALIZE_TABLES }),
        "IndexesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5480), tables: &MATERIALIZE_TABLES }),
        "InetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5481), tables: &MATERIALIZE_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5482), tables: &MATERIALIZE_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5483), tables: &MATERIALIZE_TABLES }),
        "InfoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5484), tables: &MATERIALIZE_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5485), tables: &MATERIALIZE_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5486), tables: &MATERIALIZE_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5487), tables: &MATERIALIZE_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5488), tables: &MATERIALIZE_TABLES }),
        "InlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5489), tables: &MATERIALIZE_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5490), tables: &MATERIALIZE_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5491), tables: &MATERIALIZE_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5492), tables: &MATERIALIZE_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5493), tables: &MATERIALIZE_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5494), tables: &MATERIALIZE_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5495), tables: &MATERIALIZE_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5525), tables: &MATERIALIZE_TABLES }),
        "InstanceSizes" => Some(RootGrammar { grammar_id: GrammarId(5526), tables: &MATERIALIZE_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5529), tables: &MATERIALIZE_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5530), tables: &MATERIALIZE_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5531), tables: &MATERIALIZE_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5532), tables: &MATERIALIZE_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5533), tables: &MATERIALIZE_TABLES }),
        "Int4rangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5534), tables: &MATERIALIZE_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5535), tables: &MATERIALIZE_TABLES }),
        "Int8rangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5536), tables: &MATERIALIZE_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5537), tables: &MATERIALIZE_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5538), tables: &MATERIALIZE_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5539), tables: &MATERIALIZE_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5540), tables: &MATERIALIZE_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5541), tables: &MATERIALIZE_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5542), tables: &MATERIALIZE_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5556), tables: &MATERIALIZE_TABLES }),
        "IntervalUnitsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5557), tables: &MATERIALIZE_TABLES }),
        "IntoClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5564), tables: &MATERIALIZE_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5572), tables: &MATERIALIZE_TABLES }),
        "IntrospectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5573), tables: &MATERIALIZE_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5574), tables: &MATERIALIZE_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(5575), tables: &MATERIALIZE_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(5582), tables: &MATERIALIZE_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5587), tables: &MATERIALIZE_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(5588), tables: &MATERIALIZE_TABLES }),
        "Is_superuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5589), tables: &MATERIALIZE_TABLES }),
        "Is_templateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5590), tables: &MATERIALIZE_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5591), tables: &MATERIALIZE_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5592), tables: &MATERIALIZE_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5593), tables: &MATERIALIZE_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5594), tables: &MATERIALIZE_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5595), tables: &MATERIALIZE_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5617), tables: &MATERIALIZE_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5618), tables: &MATERIALIZE_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(5620), tables: &MATERIALIZE_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(5621), tables: &MATERIALIZE_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5628), tables: &MATERIALIZE_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5636), tables: &MATERIALIZE_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5641), tables: &MATERIALIZE_TABLES }),
        "JsonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5642), tables: &MATERIALIZE_TABLES }),
        "JsonTestGrammar" => Some(RootGrammar { grammar_id: GrammarId(5643), tables: &MATERIALIZE_TABLES }),
        "JsonTypeGrammar" => Some(RootGrammar { grammar_id: GrammarId(5647), tables: &MATERIALIZE_TABLES }),
        "JsonUniqueKeysGrammar" => Some(RootGrammar { grammar_id: GrammarId(5652), tables: &MATERIALIZE_TABLES }),
        "JsonbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5659), tables: &MATERIALIZE_TABLES }),
        "KafkaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5660), tables: &MATERIALIZE_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5661), tables: &MATERIALIZE_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5662), tables: &MATERIALIZE_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5663), tables: &MATERIALIZE_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5664), tables: &MATERIALIZE_TABLES }),
        "KinesisKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5665), tables: &MATERIALIZE_TABLES }),
        "LabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5666), tables: &MATERIALIZE_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5667), tables: &MATERIALIZE_TABLES }),
        "LanguageClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5668), tables: &MATERIALIZE_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5673), tables: &MATERIALIZE_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5674), tables: &MATERIALIZE_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5675), tables: &MATERIALIZE_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5676), tables: &MATERIALIZE_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5677), tables: &MATERIALIZE_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5678), tables: &MATERIALIZE_TABLES }),
        "LatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5679), tables: &MATERIALIZE_TABLES }),
        "Lc_collateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5680), tables: &MATERIALIZE_TABLES }),
        "Lc_ctypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5681), tables: &MATERIALIZE_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5682), tables: &MATERIALIZE_TABLES }),
        "LeakproofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5683), tables: &MATERIALIZE_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5684), tables: &MATERIALIZE_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5685), tables: &MATERIALIZE_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5686), tables: &MATERIALIZE_TABLES }),
        "LeftargKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5687), tables: &MATERIALIZE_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5688), tables: &MATERIALIZE_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(5689), tables: &MATERIALIZE_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(5692), tables: &MATERIALIZE_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5693), tables: &MATERIALIZE_TABLES }),
        "LibcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5694), tables: &MATERIALIZE_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5695), tables: &MATERIALIZE_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(5703), tables: &MATERIALIZE_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5709), tables: &MATERIALIZE_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5710), tables: &MATERIALIZE_TABLES }),
        "LikeOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(5711), tables: &MATERIALIZE_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5725), tables: &MATERIALIZE_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5742), tables: &MATERIALIZE_TABLES }),
        "LineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5743), tables: &MATERIALIZE_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5744), tables: &MATERIALIZE_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5745), tables: &MATERIALIZE_TABLES }),
        "LinestringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5746), tables: &MATERIALIZE_TABLES }),
        "LinestringmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5747), tables: &MATERIALIZE_TABLES }),
        "LinestringzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5748), tables: &MATERIALIZE_TABLES }),
        "LinestringzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5749), tables: &MATERIALIZE_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5750), tables: &MATERIALIZE_TABLES }),
        "ListKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5751), tables: &MATERIALIZE_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5752), tables: &MATERIALIZE_TABLES }),
        "ListenStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5753), tables: &MATERIALIZE_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(5756), tables: &MATERIALIZE_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5768), tables: &MATERIALIZE_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5769), tables: &MATERIALIZE_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5770), tables: &MATERIALIZE_TABLES }),
        "LoadStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5771), tables: &MATERIALIZE_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(5774), tables: &MATERIALIZE_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5775), tables: &MATERIALIZE_TABLES }),
        "LocaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5776), tables: &MATERIALIZE_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5777), tables: &MATERIALIZE_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5778), tables: &MATERIALIZE_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5779), tables: &MATERIALIZE_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5780), tables: &MATERIALIZE_TABLES }),
        "LockTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5781), tables: &MATERIALIZE_TABLES }),
        "LockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5816), tables: &MATERIALIZE_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5817), tables: &MATERIALIZE_TABLES }),
        "LogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5818), tables: &MATERIALIZE_TABLES }),
        "LoggedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5819), tables: &MATERIALIZE_TABLES }),
        "LogicalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5820), tables: &MATERIALIZE_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5821), tables: &MATERIALIZE_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5822), tables: &MATERIALIZE_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5823), tables: &MATERIALIZE_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5824), tables: &MATERIALIZE_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5825), tables: &MATERIALIZE_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5826), tables: &MATERIALIZE_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5827), tables: &MATERIALIZE_TABLES }),
        "LsegKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5828), tables: &MATERIALIZE_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5829), tables: &MATERIALIZE_TABLES }),
        "Macaddr8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5841), tables: &MATERIALIZE_TABLES }),
        "MacaddrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5842), tables: &MATERIALIZE_TABLES }),
        "MainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5843), tables: &MATERIALIZE_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5844), tables: &MATERIALIZE_TABLES }),
        "ManagedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5845), tables: &MATERIALIZE_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5846), tables: &MATERIALIZE_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(5847), tables: &MATERIALIZE_TABLES }),
        "MappingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5848), tables: &MATERIALIZE_TABLES }),
        "MarketingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5849), tables: &MATERIALIZE_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5850), tables: &MATERIALIZE_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(5851), tables: &MATERIALIZE_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5852), tables: &MATERIALIZE_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5853), tables: &MATERIALIZE_TABLES }),
        "MatchingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5854), tables: &MATERIALIZE_TABLES }),
        "MaterializeExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5855), tables: &MATERIALIZE_TABLES }),
        "MaterializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5888), tables: &MATERIALIZE_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5889), tables: &MATERIALIZE_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5890), tables: &MATERIALIZE_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5891), tables: &MATERIALIZE_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5892), tables: &MATERIALIZE_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5893), tables: &MATERIALIZE_TABLES }),
        "McvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5894), tables: &MATERIALIZE_TABLES }),
        "MechanismsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5895), tables: &MATERIALIZE_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5896), tables: &MATERIALIZE_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5897), tables: &MATERIALIZE_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5898), tables: &MATERIALIZE_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5899), tables: &MATERIALIZE_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5900), tables: &MATERIALIZE_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(5904), tables: &MATERIALIZE_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5907), tables: &MATERIALIZE_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(5908), tables: &MATERIALIZE_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5912), tables: &MATERIALIZE_TABLES }),
        "MergeNotMatchedBySourceClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5925), tables: &MATERIALIZE_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5941), tables: &MATERIALIZE_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5957), tables: &MATERIALIZE_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5977), tables: &MATERIALIZE_TABLES }),
        "MessageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5980), tables: &MATERIALIZE_TABLES }),
        "MetaCommandQueryBufferSegment" => Some(RootGrammar { grammar_id: GrammarId(5981), tables: &MATERIALIZE_TABLES }),
        "MetaCommandQueryBufferStatement" => Some(RootGrammar { grammar_id: GrammarId(5982), tables: &MATERIALIZE_TABLES }),
        "MetadataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5987), tables: &MATERIALIZE_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5988), tables: &MATERIALIZE_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5989), tables: &MATERIALIZE_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5990), tables: &MATERIALIZE_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5991), tables: &MATERIALIZE_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5992), tables: &MATERIALIZE_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(5993), tables: &MATERIALIZE_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5994), tables: &MATERIALIZE_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5995), tables: &MATERIALIZE_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5996), tables: &MATERIALIZE_TABLES }),
        "MinutesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5997), tables: &MATERIALIZE_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5998), tables: &MATERIALIZE_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5999), tables: &MATERIALIZE_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6000), tables: &MATERIALIZE_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6001), tables: &MATERIALIZE_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6002), tables: &MATERIALIZE_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6003), tables: &MATERIALIZE_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(6004), tables: &MATERIALIZE_TABLES }),
        "ModulusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6005), tables: &MATERIALIZE_TABLES }),
        "MoneyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6006), tables: &MATERIALIZE_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6007), tables: &MATERIALIZE_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6008), tables: &MATERIALIZE_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6009), tables: &MATERIALIZE_TABLES }),
        "MonthsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6010), tables: &MATERIALIZE_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6011), tables: &MATERIALIZE_TABLES }),
        "MsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6012), tables: &MATERIALIZE_TABLES }),
        "MulticurveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6013), tables: &MATERIALIZE_TABLES }),
        "MultilineConcatenateDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(6014), tables: &MATERIALIZE_TABLES }),
        "MultilineConcatenateNewline" => Some(RootGrammar { grammar_id: GrammarId(6016), tables: &MATERIALIZE_TABLES }),
        "MultilinestringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6017), tables: &MATERIALIZE_TABLES }),
        "MultilinestringmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6018), tables: &MATERIALIZE_TABLES }),
        "MultilinestringzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6019), tables: &MATERIALIZE_TABLES }),
        "MultilinestringzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6020), tables: &MATERIALIZE_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(6021), tables: &MATERIALIZE_TABLES }),
        "MultipointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6022), tables: &MATERIALIZE_TABLES }),
        "MultipointmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6023), tables: &MATERIALIZE_TABLES }),
        "MultipointzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6024), tables: &MATERIALIZE_TABLES }),
        "MultipointzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6025), tables: &MATERIALIZE_TABLES }),
        "MultipolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6026), tables: &MATERIALIZE_TABLES }),
        "MultipolygonmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6027), tables: &MATERIALIZE_TABLES }),
        "MultipolygonzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6028), tables: &MATERIALIZE_TABLES }),
        "MultipolygonzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6029), tables: &MATERIALIZE_TABLES }),
        "MultisurfaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6030), tables: &MATERIALIZE_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6031), tables: &MATERIALIZE_TABLES }),
        "NakedIdentifierFullSegment" => Some(RootGrammar { grammar_id: GrammarId(6032), tables: &MATERIALIZE_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6033), tables: &MATERIALIZE_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6034), tables: &MATERIALIZE_TABLES }),
        "NamedArgumentSegment" => Some(RootGrammar { grammar_id: GrammarId(6035), tables: &MATERIALIZE_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(6042), tables: &MATERIALIZE_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(6049), tables: &MATERIALIZE_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6053), tables: &MATERIALIZE_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6054), tables: &MATERIALIZE_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6055), tables: &MATERIALIZE_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6056), tables: &MATERIALIZE_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(6057), tables: &MATERIALIZE_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6060), tables: &MATERIALIZE_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6061), tables: &MATERIALIZE_TABLES }),
        "NdistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6062), tables: &MATERIALIZE_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(6063), tables: &MATERIALIZE_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(6064), tables: &MATERIALIZE_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6065), tables: &MATERIALIZE_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(6066), tables: &MATERIALIZE_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6067), tables: &MATERIALIZE_TABLES }),
        "NfcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6068), tables: &MATERIALIZE_TABLES }),
        "NfdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6069), tables: &MATERIALIZE_TABLES }),
        "NfkcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6070), tables: &MATERIALIZE_TABLES }),
        "NfkdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6071), tables: &MATERIALIZE_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6072), tables: &MATERIALIZE_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6073), tables: &MATERIALIZE_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6074), tables: &MATERIALIZE_TABLES }),
        "NobypassrlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6075), tables: &MATERIALIZE_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6076), tables: &MATERIALIZE_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6077), tables: &MATERIALIZE_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6078), tables: &MATERIALIZE_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6079), tables: &MATERIALIZE_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6080), tables: &MATERIALIZE_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6081), tables: &MATERIALIZE_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6082), tables: &MATERIALIZE_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6083), tables: &MATERIALIZE_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6084), tables: &MATERIALIZE_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6085), tables: &MATERIALIZE_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(6095), tables: &MATERIALIZE_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6096), tables: &MATERIALIZE_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6097), tables: &MATERIALIZE_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6106), tables: &MATERIALIZE_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6107), tables: &MATERIALIZE_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6108), tables: &MATERIALIZE_TABLES }),
        "NoreplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6109), tables: &MATERIALIZE_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6110), tables: &MATERIALIZE_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(6111), tables: &MATERIALIZE_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6118), tables: &MATERIALIZE_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6119), tables: &MATERIALIZE_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(6120), tables: &MATERIALIZE_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(6121), tables: &MATERIALIZE_TABLES }),
        "NotExtendLeftSegment" => Some(RootGrammar { grammar_id: GrammarId(6128), tables: &MATERIALIZE_TABLES }),
        "NotExtendRightSegment" => Some(RootGrammar { grammar_id: GrammarId(6131), tables: &MATERIALIZE_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6134), tables: &MATERIALIZE_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(6135), tables: &MATERIALIZE_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6136), tables: &MATERIALIZE_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6137), tables: &MATERIALIZE_TABLES }),
        "NoticeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6138), tables: &MATERIALIZE_TABLES }),
        "NotificationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6139), tables: &MATERIALIZE_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6140), tables: &MATERIALIZE_TABLES }),
        "NotifyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6141), tables: &MATERIALIZE_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6147), tables: &MATERIALIZE_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6148), tables: &MATERIALIZE_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6149), tables: &MATERIALIZE_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6150), tables: &MATERIALIZE_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6151), tables: &MATERIALIZE_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6152), tables: &MATERIALIZE_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6153), tables: &MATERIALIZE_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6154), tables: &MATERIALIZE_TABLES }),
        "NumrangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6155), tables: &MATERIALIZE_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6156), tables: &MATERIALIZE_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(6157), tables: &MATERIALIZE_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6161), tables: &MATERIALIZE_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(6165), tables: &MATERIALIZE_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6170), tables: &MATERIALIZE_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6182), tables: &MATERIALIZE_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6183), tables: &MATERIALIZE_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6184), tables: &MATERIALIZE_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6185), tables: &MATERIALIZE_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6186), tables: &MATERIALIZE_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6191), tables: &MATERIALIZE_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6192), tables: &MATERIALIZE_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6193), tables: &MATERIALIZE_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6194), tables: &MATERIALIZE_TABLES }),
        "OnKeywordAsIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6195), tables: &MATERIALIZE_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6196), tables: &MATERIALIZE_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6197), tables: &MATERIALIZE_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6198), tables: &MATERIALIZE_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6199), tables: &MATERIALIZE_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6200), tables: &MATERIALIZE_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6201), tables: &MATERIALIZE_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6202), tables: &MATERIALIZE_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6203), tables: &MATERIALIZE_TABLES }),
        "OperationClassReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6204), tables: &MATERIALIZE_TABLES }),
        "OperatorClassReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6205), tables: &MATERIALIZE_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6206), tables: &MATERIALIZE_TABLES }),
        "OptimizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6207), tables: &MATERIALIZE_TABLES }),
        "OptimizerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6208), tables: &MATERIALIZE_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6209), tables: &MATERIALIZE_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6210), tables: &MATERIALIZE_TABLES }),
        "OptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(6211), tables: &MATERIALIZE_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6216), tables: &MATERIALIZE_TABLES }),
        "OptionsListGrammar" => Some(RootGrammar { grammar_id: GrammarId(6217), tables: &MATERIALIZE_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6222), tables: &MATERIALIZE_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6223), tables: &MATERIALIZE_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(6224), tables: &MATERIALIZE_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6227), tables: &MATERIALIZE_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(6250), tables: &MATERIALIZE_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6261), tables: &MATERIALIZE_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(6262), tables: &MATERIALIZE_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6265), tables: &MATERIALIZE_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6266), tables: &MATERIALIZE_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6267), tables: &MATERIALIZE_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6268), tables: &MATERIALIZE_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6269), tables: &MATERIALIZE_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6270), tables: &MATERIALIZE_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6277), tables: &MATERIALIZE_TABLES }),
        "OverlapSegment" => Some(RootGrammar { grammar_id: GrammarId(6278), tables: &MATERIALIZE_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6281), tables: &MATERIALIZE_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6313), tables: &MATERIALIZE_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6314), tables: &MATERIALIZE_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6315), tables: &MATERIALIZE_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6316), tables: &MATERIALIZE_TABLES }),
        "OwnedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6317), tables: &MATERIALIZE_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6318), tables: &MATERIALIZE_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6319), tables: &MATERIALIZE_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6320), tables: &MATERIALIZE_TABLES }),
        "ParallelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6321), tables: &MATERIALIZE_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(6322), tables: &MATERIALIZE_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(6323), tables: &MATERIALIZE_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6324), tables: &MATERIALIZE_TABLES }),
        "ParserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6325), tables: &MATERIALIZE_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6326), tables: &MATERIALIZE_TABLES }),
        "PartitionBoundSpecSegment" => Some(RootGrammar { grammar_id: GrammarId(6327), tables: &MATERIALIZE_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6357), tables: &MATERIALIZE_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6364), tables: &MATERIALIZE_TABLES }),
        "PassingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6365), tables: &MATERIALIZE_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6366), tables: &MATERIALIZE_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6367), tables: &MATERIALIZE_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(6368), tables: &MATERIALIZE_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(6375), tables: &MATERIALIZE_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6376), tables: &MATERIALIZE_TABLES }),
        "PermissiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6377), tables: &MATERIALIZE_TABLES }),
        "PgTrgmOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6378), tables: &MATERIALIZE_TABLES }),
        "Pg_lsnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6379), tables: &MATERIALIZE_TABLES }),
        "PgvectorOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6380), tables: &MATERIALIZE_TABLES }),
        "PhysicalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6381), tables: &MATERIALIZE_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6382), tables: &MATERIALIZE_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(6383), tables: &MATERIALIZE_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6384), tables: &MATERIALIZE_TABLES }),
        "PlainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6385), tables: &MATERIALIZE_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6386), tables: &MATERIALIZE_TABLES }),
        "PlansKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6387), tables: &MATERIALIZE_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(6388), tables: &MATERIALIZE_TABLES }),
        "PointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6389), tables: &MATERIALIZE_TABLES }),
        "PointmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6390), tables: &MATERIALIZE_TABLES }),
        "PointzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6391), tables: &MATERIALIZE_TABLES }),
        "PointzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6392), tables: &MATERIALIZE_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6393), tables: &MATERIALIZE_TABLES }),
        "PolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6394), tables: &MATERIALIZE_TABLES }),
        "PolygonmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6395), tables: &MATERIALIZE_TABLES }),
        "PolygonzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6396), tables: &MATERIALIZE_TABLES }),
        "PolygonzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6397), tables: &MATERIALIZE_TABLES }),
        "PolyhedralsurfaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6398), tables: &MATERIALIZE_TABLES }),
        "PortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6399), tables: &MATERIALIZE_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6400), tables: &MATERIALIZE_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(6401), tables: &MATERIALIZE_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6402), tables: &MATERIALIZE_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6406), tables: &MATERIALIZE_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6407), tables: &MATERIALIZE_TABLES }),
        "PostgisOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6408), tables: &MATERIALIZE_TABLES }),
        "PostgresCopyStdinDataSegment" => Some(RootGrammar { grammar_id: GrammarId(6409), tables: &MATERIALIZE_TABLES }),
        "PostgresCopyStdinDataStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6410), tables: &MATERIALIZE_TABLES }),
        "PostgresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6412), tables: &MATERIALIZE_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(6413), tables: &MATERIALIZE_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6415), tables: &MATERIALIZE_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6416), tables: &MATERIALIZE_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6417), tables: &MATERIALIZE_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6418), tables: &MATERIALIZE_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6419), tables: &MATERIALIZE_TABLES }),
        "PrepareStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6420), tables: &MATERIALIZE_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6430), tables: &MATERIALIZE_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6431), tables: &MATERIALIZE_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(6432), tables: &MATERIALIZE_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6435), tables: &MATERIALIZE_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6436), tables: &MATERIALIZE_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6437), tables: &MATERIALIZE_TABLES }),
        "PrivatelinkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6438), tables: &MATERIALIZE_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6439), tables: &MATERIALIZE_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6440), tables: &MATERIALIZE_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6441), tables: &MATERIALIZE_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6442), tables: &MATERIALIZE_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6443), tables: &MATERIALIZE_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6444), tables: &MATERIALIZE_TABLES }),
        "Process_toastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6445), tables: &MATERIALIZE_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6446), tables: &MATERIALIZE_TABLES }),
        "ProgramKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6447), tables: &MATERIALIZE_TABLES }),
        "ProgressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6448), tables: &MATERIALIZE_TABLES }),
        "PropertiesNakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6449), tables: &MATERIALIZE_TABLES }),
        "ProtobufKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6450), tables: &MATERIALIZE_TABLES }),
        "ProviderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6451), tables: &MATERIALIZE_TABLES }),
        "PsqlCopyMetaCommandSegment" => Some(RootGrammar { grammar_id: GrammarId(6452), tables: &MATERIALIZE_TABLES }),
        "PsqlCopyMetaCommandStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6453), tables: &MATERIALIZE_TABLES }),
        "PsqlSetMetaCommandSegment" => Some(RootGrammar { grammar_id: GrammarId(6455), tables: &MATERIALIZE_TABLES }),
        "PsqlSetMetaCommandStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6456), tables: &MATERIALIZE_TABLES }),
        "PsqlVariableColonSegment" => Some(RootGrammar { grammar_id: GrammarId(6458), tables: &MATERIALIZE_TABLES }),
        "PsqlVariableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6459), tables: &MATERIALIZE_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6467), tables: &MATERIALIZE_TABLES }),
        "PublicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6468), tables: &MATERIALIZE_TABLES }),
        "PublicationObjectsSegment" => Some(RootGrammar { grammar_id: GrammarId(6469), tables: &MATERIALIZE_TABLES }),
        "PublicationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(6492), tables: &MATERIALIZE_TABLES }),
        "PublicationTableSegment" => Some(RootGrammar { grammar_id: GrammarId(6493), tables: &MATERIALIZE_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6500), tables: &MATERIALIZE_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6501), tables: &MATERIALIZE_TABLES }),
        "QualifiedOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6504), tables: &MATERIALIZE_TABLES }),
        "QualifiedParameterNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(6512), tables: &MATERIALIZE_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6517), tables: &MATERIALIZE_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6518), tables: &MATERIALIZE_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6526), tables: &MATERIALIZE_TABLES }),
        "QueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6527), tables: &MATERIALIZE_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6528), tables: &MATERIALIZE_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6529), tables: &MATERIALIZE_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6532), tables: &MATERIALIZE_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6559), tables: &MATERIALIZE_TABLES }),
        "RaiseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6560), tables: &MATERIALIZE_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6561), tables: &MATERIALIZE_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6562), tables: &MATERIALIZE_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(6563), tables: &MATERIALIZE_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(6564), tables: &MATERIALIZE_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6565), tables: &MATERIALIZE_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(6566), tables: &MATERIALIZE_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(6567), tables: &MATERIALIZE_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6568), tables: &MATERIALIZE_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6569), tables: &MATERIALIZE_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6570), tables: &MATERIALIZE_TABLES }),
        "ReassignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6571), tables: &MATERIALIZE_TABLES }),
        "ReassignOwnedStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6572), tables: &MATERIALIZE_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6588), tables: &MATERIALIZE_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6589), tables: &MATERIALIZE_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6590), tables: &MATERIALIZE_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6591), tables: &MATERIALIZE_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6592), tables: &MATERIALIZE_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(6606), tables: &MATERIALIZE_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6612), tables: &MATERIALIZE_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6613), tables: &MATERIALIZE_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6614), tables: &MATERIALIZE_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6615), tables: &MATERIALIZE_TABLES }),
        "RefreshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6629), tables: &MATERIALIZE_TABLES }),
        "RefreshMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6630), tables: &MATERIALIZE_TABLES }),
        "RegexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6637), tables: &MATERIALIZE_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6638), tables: &MATERIALIZE_TABLES }),
        "RegionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6639), tables: &MATERIALIZE_TABLES }),
        "RegistryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6640), tables: &MATERIALIZE_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6641), tables: &MATERIALIZE_TABLES }),
        "ReindexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6642), tables: &MATERIALIZE_TABLES }),
        "RelationOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(6675), tables: &MATERIALIZE_TABLES }),
        "RelationOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(6683), tables: &MATERIALIZE_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6686), tables: &MATERIALIZE_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6687), tables: &MATERIALIZE_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6688), tables: &MATERIALIZE_TABLES }),
        "RemainderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6689), tables: &MATERIALIZE_TABLES }),
        "RemoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6690), tables: &MATERIALIZE_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6691), tables: &MATERIALIZE_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6692), tables: &MATERIALIZE_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6693), tables: &MATERIALIZE_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6694), tables: &MATERIALIZE_TABLES }),
        "ReplicaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6695), tables: &MATERIALIZE_TABLES }),
        "ReplicasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6696), tables: &MATERIALIZE_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6697), tables: &MATERIALIZE_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6698), tables: &MATERIALIZE_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6699), tables: &MATERIALIZE_TABLES }),
        "ResetSessionAuthorizationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6700), tables: &MATERIALIZE_TABLES }),
        "ResetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6704), tables: &MATERIALIZE_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6710), tables: &MATERIALIZE_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6711), tables: &MATERIALIZE_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6712), tables: &MATERIALIZE_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6713), tables: &MATERIALIZE_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6714), tables: &MATERIALIZE_TABLES }),
        "RestrictedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6715), tables: &MATERIALIZE_TABLES }),
        "RestrictiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6716), tables: &MATERIALIZE_TABLES }),
        "RetentionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6717), tables: &MATERIALIZE_TABLES }),
        "RetrieveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6718), tables: &MATERIALIZE_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6719), tables: &MATERIALIZE_TABLES }),
        "ReturningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6720), tables: &MATERIALIZE_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6721), tables: &MATERIALIZE_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6722), tables: &MATERIALIZE_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6723), tables: &MATERIALIZE_TABLES }),
        "RightArrowSegment" => Some(RootGrammar { grammar_id: GrammarId(6746), tables: &MATERIALIZE_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6747), tables: &MATERIALIZE_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6748), tables: &MATERIALIZE_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6749), tables: &MATERIALIZE_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(6750), tables: &MATERIALIZE_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6751), tables: &MATERIALIZE_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6752), tables: &MATERIALIZE_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(6753), tables: &MATERIALIZE_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6754), tables: &MATERIALIZE_TABLES }),
        "RotateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6755), tables: &MATERIALIZE_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6756), tables: &MATERIALIZE_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6757), tables: &MATERIALIZE_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6758), tables: &MATERIALIZE_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6759), tables: &MATERIALIZE_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6760), tables: &MATERIALIZE_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6761), tables: &MATERIALIZE_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6762), tables: &MATERIALIZE_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6763), tables: &MATERIALIZE_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6764), tables: &MATERIALIZE_TABLES }),
        "S3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6765), tables: &MATERIALIZE_TABLES }),
        "SafeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6766), tables: &MATERIALIZE_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(6767), tables: &MATERIALIZE_TABLES }),
        "SaslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6779), tables: &MATERIALIZE_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6780), tables: &MATERIALIZE_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6781), tables: &MATERIALIZE_TABLES }),
        "ScalarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6782), tables: &MATERIALIZE_TABLES }),
        "ScaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6783), tables: &MATERIALIZE_TABLES }),
        "ScanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6784), tables: &MATERIALIZE_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6785), tables: &MATERIALIZE_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6786), tables: &MATERIALIZE_TABLES }),
        "ScriptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6787), tables: &MATERIALIZE_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6788), tables: &MATERIALIZE_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6789), tables: &MATERIALIZE_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6790), tables: &MATERIALIZE_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6791), tables: &MATERIALIZE_TABLES }),
        "SecondsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6792), tables: &MATERIALIZE_TABLES }),
        "SecretKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6793), tables: &MATERIALIZE_TABLES }),
        "SecretsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6794), tables: &MATERIALIZE_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6795), tables: &MATERIALIZE_TABLES }),
        "SecurityLabelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6796), tables: &MATERIALIZE_TABLES }),
        "SeedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6887), tables: &MATERIALIZE_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(6888), tables: &MATERIALIZE_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6893), tables: &MATERIALIZE_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6902), tables: &MATERIALIZE_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6926), tables: &MATERIALIZE_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6939), tables: &MATERIALIZE_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6940), tables: &MATERIALIZE_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6964), tables: &MATERIALIZE_TABLES }),
        "SemiStructuredAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(6974), tables: &MATERIALIZE_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(6983), tables: &MATERIALIZE_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6984), tables: &MATERIALIZE_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6985), tables: &MATERIALIZE_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(6986), tables: &MATERIALIZE_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(6993), tables: &MATERIALIZE_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7000), tables: &MATERIALIZE_TABLES }),
        "Serial2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7001), tables: &MATERIALIZE_TABLES }),
        "Serial4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7002), tables: &MATERIALIZE_TABLES }),
        "Serial8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7003), tables: &MATERIALIZE_TABLES }),
        "SerialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7004), tables: &MATERIALIZE_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7005), tables: &MATERIALIZE_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7006), tables: &MATERIALIZE_TABLES }),
        "ServerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "Server_encodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7007), tables: &MATERIALIZE_TABLES }),
        "Server_versionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7008), tables: &MATERIALIZE_TABLES }),
        "ServiceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7009), tables: &MATERIALIZE_TABLES }),
        "SessionInformationUserFunctionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(7010), tables: &MATERIALIZE_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7015), tables: &MATERIALIZE_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7016), tables: &MATERIALIZE_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(7017), tables: &MATERIALIZE_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7021), tables: &MATERIALIZE_TABLES }),
        "SetConstraintsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7055), tables: &MATERIALIZE_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7065), tables: &MATERIALIZE_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7074), tables: &MATERIALIZE_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(7075), tables: &MATERIALIZE_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7083), tables: &MATERIALIZE_TABLES }),
        "SetSessionAuthorizationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7088), tables: &MATERIALIZE_TABLES }),
        "SetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7101), tables: &MATERIALIZE_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7135), tables: &MATERIALIZE_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7136), tables: &MATERIALIZE_TABLES }),
        "SettingsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7137), tables: &MATERIALIZE_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7138), tables: &MATERIALIZE_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7139), tables: &MATERIALIZE_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7140), tables: &MATERIALIZE_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(7141), tables: &MATERIALIZE_TABLES }),
        "ShowCreateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7150), tables: &MATERIALIZE_TABLES }),
        "ShowIndexesStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7170), tables: &MATERIALIZE_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7187), tables: &MATERIALIZE_TABLES }),
        "ShowMaterializedViewsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7188), tables: &MATERIALIZE_TABLES }),
        "ShowStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7196), tables: &MATERIALIZE_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7229), tables: &MATERIALIZE_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7230), tables: &MATERIALIZE_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(7231), tables: &MATERIALIZE_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7234), tables: &MATERIALIZE_TABLES }),
        "SimpleGeometryGrammar" => Some(RootGrammar { grammar_id: GrammarId(7235), tables: &MATERIALIZE_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7237), tables: &MATERIALIZE_TABLES }),
        "SingleIdentifierFullGrammar" => Some(RootGrammar { grammar_id: GrammarId(7238), tables: &MATERIALIZE_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(7242), tables: &MATERIALIZE_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(7246), tables: &MATERIALIZE_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(7248), tables: &MATERIALIZE_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(7252), tables: &MATERIALIZE_TABLES }),
        "SinkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7253), tables: &MATERIALIZE_TABLES }),
        "SinksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7254), tables: &MATERIALIZE_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7255), tables: &MATERIALIZE_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(7256), tables: &MATERIALIZE_TABLES }),
        "SkipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7259), tables: &MATERIALIZE_TABLES }),
        "Skip_lockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7260), tables: &MATERIALIZE_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(7261), tables: &MATERIALIZE_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(7262), tables: &MATERIALIZE_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7263), tables: &MATERIALIZE_TABLES }),
        "SmallserialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7264), tables: &MATERIALIZE_TABLES }),
        "SnapshotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7265), tables: &MATERIALIZE_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7266), tables: &MATERIALIZE_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7267), tables: &MATERIALIZE_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7268), tables: &MATERIALIZE_TABLES }),
        "SourcesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7269), tables: &MATERIALIZE_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7270), tables: &MATERIALIZE_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7271), tables: &MATERIALIZE_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7272), tables: &MATERIALIZE_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7273), tables: &MATERIALIZE_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7274), tables: &MATERIALIZE_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7275), tables: &MATERIALIZE_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7276), tables: &MATERIALIZE_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7277), tables: &MATERIALIZE_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7278), tables: &MATERIALIZE_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7279), tables: &MATERIALIZE_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7280), tables: &MATERIALIZE_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7281), tables: &MATERIALIZE_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7282), tables: &MATERIALIZE_TABLES }),
        "SqsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7283), tables: &MATERIALIZE_TABLES }),
        "SshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7284), tables: &MATERIALIZE_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7285), tables: &MATERIALIZE_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7286), tables: &MATERIALIZE_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7287), tables: &MATERIALIZE_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7288), tables: &MATERIALIZE_TABLES }),
        "StandaloneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7289), tables: &MATERIALIZE_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(7290), tables: &MATERIALIZE_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(135), tables: &MATERIALIZE_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(6164), tables: &MATERIALIZE_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7291), tables: &MATERIALIZE_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2097), tables: &MATERIALIZE_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7292), tables: &MATERIALIZE_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7293), tables: &MATERIALIZE_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7294), tables: &MATERIALIZE_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7295), tables: &MATERIALIZE_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7366), tables: &MATERIALIZE_TABLES }),
        "StatisticsReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7367), tables: &MATERIALIZE_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7368), tables: &MATERIALIZE_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7369), tables: &MATERIALIZE_TABLES }),
        "StoredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7370), tables: &MATERIALIZE_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7371), tables: &MATERIALIZE_TABLES }),
        "StrategyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7372), tables: &MATERIALIZE_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7373), tables: &MATERIALIZE_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7374), tables: &MATERIALIZE_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7375), tables: &MATERIALIZE_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(7376), tables: &MATERIALIZE_TABLES }),
        "StringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7379), tables: &MATERIALIZE_TABLES }),
        "StripKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7380), tables: &MATERIALIZE_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7381), tables: &MATERIALIZE_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(7386), tables: &MATERIALIZE_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7387), tables: &MATERIALIZE_TABLES }),
        "SubscribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7388), tables: &MATERIALIZE_TABLES }),
        "SubscriptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7389), tables: &MATERIALIZE_TABLES }),
        "SubscriptionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "SubsourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7390), tables: &MATERIALIZE_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7391), tables: &MATERIALIZE_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7392), tables: &MATERIALIZE_TABLES }),
        "SummaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7393), tables: &MATERIALIZE_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7394), tables: &MATERIALIZE_TABLES }),
        "SupportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7395), tables: &MATERIALIZE_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(7396), tables: &MATERIALIZE_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7397), tables: &MATERIALIZE_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7398), tables: &MATERIALIZE_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7399), tables: &MATERIALIZE_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7400), tables: &MATERIALIZE_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7401), tables: &MATERIALIZE_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(7402), tables: &MATERIALIZE_TABLES }),
        "TableConstraintUsingIndexSegment" => Some(RootGrammar { grammar_id: GrammarId(7463), tables: &MATERIALIZE_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7486), tables: &MATERIALIZE_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7487), tables: &MATERIALIZE_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7500), tables: &MATERIALIZE_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7501), tables: &MATERIALIZE_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7502), tables: &MATERIALIZE_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7503), tables: &MATERIALIZE_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "TailKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7504), tables: &MATERIALIZE_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(7505), tables: &MATERIALIZE_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(7510), tables: &MATERIALIZE_TABLES }),
        "TargetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7514), tables: &MATERIALIZE_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7515), tables: &MATERIALIZE_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7516), tables: &MATERIALIZE_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7517), tables: &MATERIALIZE_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7518), tables: &MATERIALIZE_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(7519), tables: &MATERIALIZE_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(7520), tables: &MATERIALIZE_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7523), tables: &MATERIALIZE_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(7524), tables: &MATERIALIZE_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7527), tables: &MATERIALIZE_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7528), tables: &MATERIALIZE_TABLES }),
        "TestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7529), tables: &MATERIALIZE_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7530), tables: &MATERIALIZE_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7531), tables: &MATERIALIZE_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7532), tables: &MATERIALIZE_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7533), tables: &MATERIALIZE_TABLES }),
        "TickKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7534), tables: &MATERIALIZE_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7535), tables: &MATERIALIZE_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(7536), tables: &MATERIALIZE_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7537), tables: &MATERIALIZE_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(7538), tables: &MATERIALIZE_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(7549), tables: &MATERIALIZE_TABLES }),
        "TimelineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7555), tables: &MATERIALIZE_TABLES }),
        "TimeoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7556), tables: &MATERIALIZE_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7557), tables: &MATERIALIZE_TABLES }),
        "TimestamptzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7558), tables: &MATERIALIZE_TABLES }),
        "TimetzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7559), tables: &MATERIALIZE_TABLES }),
        "TimingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7560), tables: &MATERIALIZE_TABLES }),
        "TinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7561), tables: &MATERIALIZE_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7562), tables: &MATERIALIZE_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7563), tables: &MATERIALIZE_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7564), tables: &MATERIALIZE_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7565), tables: &MATERIALIZE_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7566), tables: &MATERIALIZE_TABLES }),
        "TokenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7567), tables: &MATERIALIZE_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7568), tables: &MATERIALIZE_TABLES }),
        "TopicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7569), tables: &MATERIALIZE_TABLES }),
        "TpchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7570), tables: &MATERIALIZE_TABLES }),
        "TraceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7571), tables: &MATERIALIZE_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7572), tables: &MATERIALIZE_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7573), tables: &MATERIALIZE_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7574), tables: &MATERIALIZE_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7575), tables: &MATERIALIZE_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7592), tables: &MATERIALIZE_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7593), tables: &MATERIALIZE_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7594), tables: &MATERIALIZE_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7595), tables: &MATERIALIZE_TABLES }),
        "TriangleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7596), tables: &MATERIALIZE_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7597), tables: &MATERIALIZE_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MATERIALIZE_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7598), tables: &MATERIALIZE_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(7599), tables: &MATERIALIZE_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7603), tables: &MATERIALIZE_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(7604), tables: &MATERIALIZE_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7605), tables: &MATERIALIZE_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7606), tables: &MATERIALIZE_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7623), tables: &MATERIALIZE_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7624), tables: &MATERIALIZE_TABLES }),
        "TsqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7625), tables: &MATERIALIZE_TABLES }),
        "TsrangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7626), tables: &MATERIALIZE_TABLES }),
        "TstzrangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7627), tables: &MATERIALIZE_TABLES }),
        "TsvectorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7628), tables: &MATERIALIZE_TABLES }),
        "TunnelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7629), tables: &MATERIALIZE_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(7630), tables: &MATERIALIZE_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7633), tables: &MATERIALIZE_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7634), tables: &MATERIALIZE_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7642), tables: &MATERIALIZE_TABLES }),
        "TypesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7645), tables: &MATERIALIZE_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7646), tables: &MATERIALIZE_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7647), tables: &MATERIALIZE_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7648), tables: &MATERIALIZE_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7649), tables: &MATERIALIZE_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(7650), tables: &MATERIALIZE_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(7651), tables: &MATERIALIZE_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7655), tables: &MATERIALIZE_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7656), tables: &MATERIALIZE_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(7657), tables: &MATERIALIZE_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7662), tables: &MATERIALIZE_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(7663), tables: &MATERIALIZE_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7665), tables: &MATERIALIZE_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7666), tables: &MATERIALIZE_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7667), tables: &MATERIALIZE_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7668), tables: &MATERIALIZE_TABLES }),
        "UnlistenStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7669), tables: &MATERIALIZE_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7674), tables: &MATERIALIZE_TABLES }),
        "UnloggedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7675), tables: &MATERIALIZE_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7676), tables: &MATERIALIZE_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7700), tables: &MATERIALIZE_TABLES }),
        "UnsafeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7706), tables: &MATERIALIZE_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7707), tables: &MATERIALIZE_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7708), tables: &MATERIALIZE_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7709), tables: &MATERIALIZE_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7710), tables: &MATERIALIZE_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7733), tables: &MATERIALIZE_TABLES }),
        "UpsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7734), tables: &MATERIALIZE_TABLES }),
        "UrlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7735), tables: &MATERIALIZE_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7736), tables: &MATERIALIZE_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7737), tables: &MATERIALIZE_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7738), tables: &MATERIALIZE_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7741), tables: &MATERIALIZE_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7742), tables: &MATERIALIZE_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(7743), tables: &MATERIALIZE_TABLES }),
        "UsernameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7744), tables: &MATERIALIZE_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7745), tables: &MATERIALIZE_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7746), tables: &MATERIALIZE_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7747), tables: &MATERIALIZE_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7748), tables: &MATERIALIZE_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7749), tables: &MATERIALIZE_TABLES }),
        "UuidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7750), tables: &MATERIALIZE_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7751), tables: &MATERIALIZE_TABLES }),
        "VacuumStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7752), tables: &MATERIALIZE_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7785), tables: &MATERIALIZE_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7786), tables: &MATERIALIZE_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7787), tables: &MATERIALIZE_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7788), tables: &MATERIALIZE_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7789), tables: &MATERIALIZE_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7800), tables: &MATERIALIZE_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7801), tables: &MATERIALIZE_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7802), tables: &MATERIALIZE_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7803), tables: &MATERIALIZE_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7804), tables: &MATERIALIZE_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7805), tables: &MATERIALIZE_TABLES }),
        "VariadicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7806), tables: &MATERIALIZE_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7807), tables: &MATERIALIZE_TABLES }),
        "VectorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7808), tables: &MATERIALIZE_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7809), tables: &MATERIALIZE_TABLES }),
        "VersionIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(7810), tables: &MATERIALIZE_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7813), tables: &MATERIALIZE_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7814), tables: &MATERIALIZE_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7815), tables: &MATERIALIZE_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7816), tables: &MATERIALIZE_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7817), tables: &MATERIALIZE_TABLES }),
        "WalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7818), tables: &MATERIALIZE_TABLES }),
        "WalrusOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(7819), tables: &MATERIALIZE_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7820), tables: &MATERIALIZE_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7821), tables: &MATERIALIZE_TABLES }),
        "WarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7822), tables: &MATERIALIZE_TABLES }),
        "WebhookKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7823), tables: &MATERIALIZE_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7824), tables: &MATERIALIZE_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7825), tables: &MATERIALIZE_TABLES }),
        "WellKnownTextGeometrySegment" => Some(RootGrammar { grammar_id: GrammarId(7826), tables: &MATERIALIZE_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7926), tables: &MATERIALIZE_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7936), tables: &MATERIALIZE_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7937), tables: &MATERIALIZE_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(7942), tables: &MATERIALIZE_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7959), tables: &MATERIALIZE_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7960), tables: &MATERIALIZE_TABLES }),
        "WhitespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7961), tables: &MATERIALIZE_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(7962), tables: &MATERIALIZE_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7963), tables: &MATERIALIZE_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(7965), tables: &MATERIALIZE_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7975), tables: &MATERIALIZE_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(7976), tables: &MATERIALIZE_TABLES }),
        "WireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7984), tables: &MATERIALIZE_TABLES }),
        "WithCheckOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(7985), tables: &MATERIALIZE_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7992), tables: &MATERIALIZE_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(8001), tables: &MATERIALIZE_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(8010), tables: &MATERIALIZE_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(8015), tables: &MATERIALIZE_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8016), tables: &MATERIALIZE_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(8017), tables: &MATERIALIZE_TABLES }),
        "WithinGroupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(8022), tables: &MATERIALIZE_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8027), tables: &MATERIALIZE_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8028), tables: &MATERIALIZE_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(8029), tables: &MATERIALIZE_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8030), tables: &MATERIALIZE_TABLES }),
        "WorkersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8031), tables: &MATERIALIZE_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8032), tables: &MATERIALIZE_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8033), tables: &MATERIALIZE_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8034), tables: &MATERIALIZE_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8035), tables: &MATERIALIZE_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8036), tables: &MATERIALIZE_TABLES }),
        "XmlattributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8037), tables: &MATERIALIZE_TABLES }),
        "XmlconcatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8038), tables: &MATERIALIZE_TABLES }),
        "XmlelementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8039), tables: &MATERIALIZE_TABLES }),
        "XmlexistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8040), tables: &MATERIALIZE_TABLES }),
        "XmlforestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8041), tables: &MATERIALIZE_TABLES }),
        "XmlnamespacesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8042), tables: &MATERIALIZE_TABLES }),
        "XmlparseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8043), tables: &MATERIALIZE_TABLES }),
        "XmlpiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8044), tables: &MATERIALIZE_TABLES }),
        "XmlrootKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8045), tables: &MATERIALIZE_TABLES }),
        "XmlserializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8046), tables: &MATERIALIZE_TABLES }),
        "XmltableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8047), tables: &MATERIALIZE_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8048), tables: &MATERIALIZE_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8049), tables: &MATERIALIZE_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8050), tables: &MATERIALIZE_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8051), tables: &MATERIALIZE_TABLES }),
        "YearsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8052), tables: &MATERIALIZE_TABLES }),
        "YesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8053), tables: &MATERIALIZE_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8054), tables: &MATERIALIZE_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8055), tables: &MATERIALIZE_TABLES }),
        "ZonesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8056), tables: &MATERIALIZE_TABLES }),
        _ => None,
    }
}

pub fn get_materialize_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AdjacentSegment" => Some("comparison_operator"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterAggregateStatementSegment" => Some("alter_aggregate_statement"),
        "AlterConnectionRotateKeys" => Some("alter_connection_rotate_keys"),
        "AlterDatabaseStatementSegment" => Some("alter_database_statement"),
        "AlterDefaultPrivilegesGrantSegment" => Some("alter_default_privileges_grant"),
        "AlterDefaultPrivilegesObjectPrivilegesSegment" => Some("alter_default_privileges_object_privilege"),
        "AlterDefaultPrivilegesRevokeSegment" => Some("alter_default_privileges_revoke"),
        "AlterDefaultPrivilegesSchemaObjectsSegment" => Some("alter_default_privileges_schema_object"),
        "AlterDefaultPrivilegesStatementSegment" => Some("alter_default_privileges_statement"),
        "AlterDefaultPrivilegesToFromRolesSegment" => Some("alter_default_privileges_to_from_roles"),
        "AlterDomainStatementSegment" => Some("alter_domain_statement"),
        "AlterExtensionStatementSegment" => Some("alter_extension_statement"),
        "AlterForeignTableActionSegment" => Some("alter_foreign_table_action_segment"),
        "AlterForeignTableStatementSegment" => Some("alter_foreign_table_statement"),
        "AlterFunctionActionSegment" => Some("alter_function_action_segment"),
        "AlterFunctionStatementSegment" => Some("alter_function_statement"),
        "AlterIndexStatementSegment" => Some("alter_index_statement"),
        "AlterMaterializedViewActionSegment" => Some("alter_materialized_view_action_segment"),
        "AlterMaterializedViewStatementSegment" => Some("alter_materialized_view_statement"),
        "AlterOwnerStatementSegment" => Some("alter_owner_statement"),
        "AlterPolicyStatementSegment" => Some("alter_policy_statement"),
        "AlterProcedureActionSegment" => Some("alter_procedure_action_segment"),
        "AlterProcedureStatementSegment" => Some("alter_procedure_statement"),
        "AlterPublicationStatementSegment" => Some("alter_publication_statement"),
        "AlterRenameStatementSegment" => Some("alter_rename_statement"),
        "AlterRoleStatementSegment" => Some("alter_role_statement"),
        "AlterSchemaStatementSegment" => Some("alter_schema_statement"),
        "AlterSecretStatementSegment" => Some("alter_secret_statement"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterSetClusterStatementSegment" => Some("alter_set_cluster_statement"),
        "AlterSourceSinkSizeStatementSegment" => Some("alter_source_sink_size_statement"),
        "AlterStatisticsStatementSegment" => Some("alter_statistics_statement"),
        "AlterSubscriptionStatementSegment" => Some("alter_subscription"),
        "AlterSystemStatementSegment" => Some("alter_system_statement"),
        "AlterTableActionSegment" => Some("alter_table_action_segment"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AlterTextSearchConfigurationStatementSegment" => Some("alter_text_search_configuration_statement"),
        "AlterTriggerStatementSegment" => Some("alter_trigger"),
        "AlterTypeStatementSegment" => Some("alter_type_statement"),
        "AlterViewStatementSegment" => Some("alter_view_statement"),
        "AnalyzeStatementSegment" => Some("analyze_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "ArrayTypeSuffixSegment" => Some("array_type_suffix"),
        "AsAliasExpressionSegment" => Some("alias_expression"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CallStoredProcedureSegment" => Some("call_statement"),
        "CaseExpressionSegment" => Some("case_expression"),
        "CloseStatementSegment" => Some("close_statement"),
        "ClusterStatementSegment" => Some("cluster_statement"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnTypeReferenceSegment" => Some("column_type_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentOnStatementSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeValueExpansionSegment" => Some("composite_value_expansion"),
        "ConcatSegment" => Some("binary_operator"),
        "ConflictActionSegment" => Some("conflict_action"),
        "ConflictTargetSegment" => Some("conflict_target"),
        "CopyFromStatementSegment" => Some("copy_from_statement"),
        "CopyStatementSegment" => Some("copy_statement"),
        "CopyToStatementSegment" => Some("copy_to_statement"),
        "CreateAggregateStatementSegment" => Some("create_aggregate_statement"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateClusterReplicaStatementSegment" => Some("create_cluster_replica_statement"),
        "CreateClusterStatementSegment" => Some("create_cluster_statement"),
        "CreateCollationStatementSegment" => Some("create_collation_statement"),
        "CreateConnectionStatementSegment" => Some("create_connection_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateDomainStatementSegment" => Some("create_domain_statement"),
        "CreateExtensionStatementSegment" => Some("create_extension_statement"),
        "CreateForeignDataWrapperStatementSegment" => Some("create_foreign_data_wrapper"),
        "CreateForeignTableStatementSegment" => Some("create_foreign_table_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateMaterializedViewStatementSegment" => Some("create_materialized_view_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateOperatorStatementSegment" => Some("create_operator_statement"),
        "CreatePolicyStatementSegment" => Some("create_policy_statement"),
        "CreateProcedureStatementSegment" => Some("create_procedure_statement"),
        "CreatePublicationStatementSegment" => Some("create_publication_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSecretStatementSegment" => Some("create_secret_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateServerStatementSegment" => Some("create_server_statement"),
        "CreateSinkKafkaStatementSegment" => Some("create_sink_kafka_statement"),
        "CreateSourceKafkaStatementSegment" => Some("create_source_kafka_statement"),
        "CreateSourceLoadGeneratorStatementSegment" => Some("create_source_load_generator_statement"),
        "CreateSourcePostgresStatementSegment" => Some("create_source_postgres_statement"),
        "CreateSourceWebhookStatementSegment" => Some("create_source_load_generator_statement"),
        "CreateStatisticsStatementSegment" => Some("create_statistics_statement"),
        "CreateSubscriptionStatementSegment" => Some("create_subscription"),
        "CreateTableAsStatementSegment" => Some("create_table_as_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTextSearchConfigurationStatementSegment" => Some("create_text_search_configuration_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateTypeStatementSegment" => Some("create_type_statement"),
        "CreateUserMappingStatementSegment" => Some("create_user_mapping_statement"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "DateTimeLiteralGrammar" => Some("datetime_literal"),
        "DateTimeTypeIdentifier" => Some("datetime_type_identifier"),
        "DeallocateStatementSegment" => Some("deallocate_statement"),
        "DeclareStatementSegment" => Some("declare_statement"),
        "Dedent" => Some("dedent"),
        "DefinitionParameterSegment" => Some("definition_parameter"),
        "DefinitionParametersSegment" => Some("definition_parameters"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DiscardStatementSegment" => Some("discard_statement"),
        "DoStatementSegment" => Some("do_statement"),
        "DropAggregateStatementSegment" => Some("drop_aggregate_statement"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropCollationStatementSegment" => Some("drop_collation_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropDomainStatementSegment" => Some("drop_domain_statement"),
        "DropExtensionStatementSegment" => Some("drop_extension_statement"),
        "DropForeignTableStatement" => Some("drop_foreign_table_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropMaterializedViewStatementSegment" => Some("drop_materialized_view_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropOwnedStatementSegment" => Some("drop_owned_statement"),
        "DropPolicyStatementSegment" => Some("drop_policy_statement"),
        "DropProcedureStatementSegment" => Some("drop_procedure_statement"),
        "DropPublicationStatementSegment" => Some("drop_publication_statement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropStatementSegment" => Some("drop_statement"),
        "DropStatisticsStatementSegment" => Some("drop_statistics_statement"),
        "DropSubscriptionStatementSegment" => Some("drop_subscription"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTextSearchConfigurationStatementSegment" => Some("drop_text_search_configuration_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExclusionConstraintElementSegment" => Some("exclusion_constraint_element"),
        "ExecuteStatementSegment" => Some("execute_statement"),
        "ExplainOptionSegment" => Some("explain_option"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FetchStatementSegment" => Some("fetch_statement"),
        "FileSegment" => Some("file"),
        "ForClauseSegment" => Some("for_clause"),
        "ForeignTableColumnConstraintSegment" => Some("column_constraint_segment"),
        "ForeignTableTableConstraintSegment" => Some("table_constraint"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "IdentifierSegment" => Some("identifier"),
        "ImplicitIndent" => Some("indent"),
        "ImportForeignSchemaStatementSegment" => Some("import_foreign_schema_statement"),
        "Indent" => Some("indent"),
        "IndexAccessMethodSegment" => Some("index_access_method"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexElementOptionsSegment" => Some("index_element_options"),
        "IndexElementSegment" => Some("index_element"),
        "IndexParametersSegment" => Some("index_parameters"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "IntoClauseSegment" => Some("into_clause"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "LanguageClauseSegment" => Some("language_clause"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LikeOptionSegment" => Some("like_option_segment"),
        "LimitClauseSegment" => Some("limit_clause"),
        "ListenStatementSegment" => Some("listen_statement"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LoadStatementSegment" => Some("load_statement"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "LockTableStatementSegment" => Some("lock_table_statement"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MaterializeExplainStatementSegment" => Some("explain_statement"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedBySourceClauseSegment" => Some("merge_when_not_matched_by_source_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "MetaCommandQueryBufferStatement" => Some("meta_command_statement"),
        "NamedArgumentSegment" => Some("named_argument"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "NotExtendLeftSegment" => Some("comparison_operator"),
        "NotExtendRightSegment" => Some("comparison_operator"),
        "NotifyStatementSegment" => Some("notify_statement"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OperationClassReferenceSegment" => Some("operation_class_reference"),
        "OperatorClassReferenceSegment" => Some("operator_class_reference"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapSegment" => Some("comparison_operator"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionBoundSpecSegment" => Some("partition_bound_spec"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "PostgresCopyStdinDataStatementSegment" => Some("postgres_copy_stdin_data_statement"),
        "PrepareStatementSegment" => Some("prepare_statement"),
        "PsqlCopyMetaCommandStatementSegment" => Some("psql_copy_meta_command_statement"),
        "PsqlSetMetaCommandStatementSegment" => Some("psql_set_meta_command_statement"),
        "PsqlVariableGrammar" => Some("psql_variable"),
        "PublicationObjectsSegment" => Some("publication_objects"),
        "PublicationReferenceSegment" => Some("publication_reference"),
        "PublicationTableSegment" => Some("publication_table"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "QualifiedOperatorSegment" => Some("qualified_operator"),
        "ReassignOwnedStatementSegment" => Some("reassign_owned_statement"),
        "RefreshMaterializedViewStatementSegment" => Some("refresh_materialized_view_statement"),
        "ReindexStatementSegment" => Some("reindex_statement_segment"),
        "RelationOptionSegment" => Some("relation_option"),
        "RelationOptionsSegment" => Some("relation_options"),
        "ResetSessionAuthorizationStatementSegment" => Some("reset_session_authorization_statement"),
        "ResetStatementSegment" => Some("reset_statement"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SecurityLabelStatementSegment" => Some("security_label_statement"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SemiStructuredAccessorSegment" => Some("semi_structured_expression"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "ServerReferenceSegment" => Some("server_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetConstraintsStatementSegment" => Some("set_constraint_statement"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "SetSessionAuthorizationStatementSegment" => Some("set_session_authorization_statement"),
        "SetStatementSegment" => Some("set_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "ShowCreateStatementSegment" => Some("show_create_statement"),
        "ShowIndexesStatementSegment" => Some("show_indexes_statement"),
        "ShowMaterializedViewsStatementSegment" => Some("show_materialized_views_statement"),
        "ShowStatementSegment" => Some("show_statement"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StatisticsReferenceSegment" => Some("statistics_reference"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SubscriptionReferenceSegment" => Some("subscription_reference"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableConstraintUsingIndexSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnlistenStatementSegment" => Some("unlisten_statement"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "VacuumStatementSegment" => Some("vacuum_statement"),
        "ValuesClauseSegment" => Some("values_clause"),
        "VersionIdentifierSegment" => Some("version_identifier"),
        "WellKnownTextGeometrySegment" => Some("wkt_geometry_type"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCheckOptionSegment" => Some("with_check_option"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WithinGroupClauseSegment" => Some("withingroup_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static MATERIALIZE_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_materialize_root_grammar_id() -> GrammarId {
    GrammarId(4818)
}
pub fn get_materialize_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_materialize_root_grammar_id(),
        tables: &MATERIALIZE_TABLES,
    }
}
pub fn get_materialize_root_grammar() -> RootGrammar {
    get_materialize_root_grammar_table()
}
