/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      3079 × 20 bytes =    61580 bytes
//   Child IDs:         2010 ×  4 bytes =     8040 bytes
//   Terminators:         44 ×  4 bytes =      176 bytes
//   Strings:           1693 strings      =    21990 bytes
//   Aux Data:         10552 ×  4 bytes =    42208 bytes
//   Aux Offsets:       3079 ×  4 bytes =    12316 bytes
//   Regex Patterns:       5 patterns     =      720 bytes
//   Simple Hints:      1063 hints        =    17008 bytes
//   Hint Indices:      1511 ×  4 bytes =     6044 bytes
//   TOTAL:                                    ≈   170082 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [4] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [6] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [51] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [61] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [62] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [63] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [64] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [131] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [133] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [134] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [135] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [136] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [137] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [138] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [139] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [140] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] StringParser("ADA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 210, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] StringParser(")")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] Meta("dedent")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] StringParser("ALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 221, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 226, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 234, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 234, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 234, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 234, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 243, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 254, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 256, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 258, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 263, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] Ref(AlterTableOptionsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] StringParser("ARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [323] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [328] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [329] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 291, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 293, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [332] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [333] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [334] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [335] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [336] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [337] StringParser("ASENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [338] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [339] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [340] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [341] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [342] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [343] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [344] StringParser("ATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [345] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [346] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [347] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [348] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [349] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [350] StringParser("AVG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [351] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [352] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [353] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [354] MultiStringParser(26 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 295, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 2, _padding: 0 }, // [355] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [356] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [357] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [358] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [359] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [360] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [361] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [362] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [363] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [364] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [365] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [366] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [367] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [368] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [369] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [370] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [371] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [372] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [373] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [374] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [375] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [376] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [377] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [378] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [379] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [380] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [381] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [382] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [383] StringParser("BIT_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [384] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [385] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [386] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [387] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [388] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [389] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 310, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [390] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [391] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [392] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [393] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [394] StringParser("BLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [395] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [396] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 314, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [397] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 314, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [398] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 314, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [399] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 314, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [400] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [401] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [402] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [403] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [404] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 319, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [405] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [406] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [407] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 324, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [408] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 326, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [409] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 326, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [410] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 326, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [411] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [412] Ref(UnorderedSetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [413] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [414] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [415] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [416] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [417] StringParser("BY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 329, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [418] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 332, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [419] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 332, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [420] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 336, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [421] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 336, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [422] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 336, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [423] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 336, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [424] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [425] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [426] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [427] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [428] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [429] StringParser("CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [430] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 339, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [431] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 2, min_times: 0, first_terminator_idx: 11, terminator_count: 3, _padding: 0 }, // [432] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 6, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [433] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [434] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [435] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 347, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [436] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [437] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [438] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [439] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 1, _padding: 0 }, // [440] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [441] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [442] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 7, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [443] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 355, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [444] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 355, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [445] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 355, child_count: 1, min_times: 0, first_terminator_idx: 8, terminator_count: 2, _padding: 0 }, // [446] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [447] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [448] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [449] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 1, _padding: 0 }, // [450] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [451] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [452] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [453] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [454] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [455] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [456] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [457] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [458] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [459] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [460] StringParser("CATALOG_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [461] StringParser("CEIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [462] StringParser("CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [463] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [464] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [465] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [466] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [467] StringParser("CHAR_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [468] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [469] StringParser("CHARACTER_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [470] StringParser("CHARACTER_SET_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [471] StringParser("CHARACTER_SET_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [472] StringParser("CHARACTER_SET_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [473] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [474] StringParser("CHARACTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [475] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [476] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [477] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [478] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [479] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [480] StringParser("CLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [481] StringParser("CLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [482] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [483] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [484] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [485] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [486] StringParser("COBOL")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [487] Token("raw")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [488] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [489] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [490] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [491] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [492] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 358, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 1, _padding: 0 }, // [493] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [494] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [495] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [496] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [497] StringParser("COLLATION_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [498] StringParser("COLLATION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [499] StringParser("COLLATION_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [500] StringParser("COLLECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [501] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [502] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [503] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [504] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 364, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [505] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 364, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [506] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 364, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [507] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 364, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [508] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 364, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [509] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 366, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [510] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 368, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [511] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 368, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [512] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 368, child_count: 10, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [513] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [514] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [515] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [516] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [517] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [518] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [519] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 385, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [520] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 385, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [521] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [522] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [523] Ref(ColumnConstraintDefaultGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [524] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [525] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [526] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [527] Ref(UniqueKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [528] Ref(AutoIncrementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [529] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 391, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [530] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 391, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [531] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 391, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [532] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 391, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [533] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 393, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [534] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 393, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [535] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 393, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [536] Ref(ColumnGeneratedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 393, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [537] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [538] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [539] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 397, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [540] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 400, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [541] Anything
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 400, child_count: 1, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [542] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [543] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [544] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [545] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 1, _padding: 0 }, // [546] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [547] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [548] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [549] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [550] StringParser("COLUMN_NAME")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [551] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [552] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [553] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [554] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [555] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [556] StringParser(",")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [557] StringParser("COMMAND_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [558] StringParser("COMMAND_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [559] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [560] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [561] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [562] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [563] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [564] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 405, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [565] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 405, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [566] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [567] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [568] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [569] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [570] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [571] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [572] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [573] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [574] Ref(IsDistinctFromGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [575] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [576] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [577] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [578] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [579] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [580] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 413, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [581] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [582] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [583] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [584] StringParser("CONDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [585] StringParser("CONDITION_NUMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [586] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [587] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [588] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [589] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [590] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [591] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [592] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [593] StringParser("CONNECTION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [594] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [595] StringParser("CONSTRAINT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [596] StringParser("CONSTRAINT_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [597] StringParser("CONSTRAINT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [598] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [599] StringParser("CONSTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [600] StringParser("CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [601] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [602] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [603] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [604] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [605] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [606] StringParser("CORR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [607] StringParser("CORRESPONDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [608] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [609] StringParser("COVAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [610] StringParser("COVAR_SAMP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 10, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [611] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 428, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [612] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 428, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [613] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 428, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [614] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [615] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [616] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [617] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [618] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [619] Ref(SpecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [620] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [621] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [622] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [623] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [624] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 439, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [625] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [626] Ref(InstanceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [627] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [628] Ref(ConstructorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [629] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [630] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [631] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 442, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [632] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 444, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [633] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 444, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [634] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 444, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [635] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [636] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [637] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [638] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [639] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [640] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [641] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [642] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [643] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [644] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [645] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [646] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [647] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [648] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [649] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [650] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 459, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [651] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [652] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [653] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [654] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [655] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [656] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [657] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [658] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [659] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [660] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [661] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [662] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [663] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [664] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 471, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [665] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 474, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [666] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 476, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [667] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 476, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [668] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 476, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [669] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [670] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [671] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [672] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [673] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [674] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 484, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [675] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 486, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [676] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 486, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [677] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 489, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [678] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 491, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [679] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [680] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [681] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [682] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 496, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [683] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 496, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [684] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [685] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [686] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [687] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [688] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [689] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [690] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [691] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [692] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [693] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [694] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [695] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [696] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [697] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [698] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 509, child_count: 7, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [699] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [700] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [701] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [702] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [703] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [704] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [705] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [706] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [707] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [708] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [709] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [710] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [711] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 526, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [712] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 526, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [713] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 526, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [714] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 526, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [715] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [716] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [717] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [718] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [719] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [720] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [721] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [722] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 532, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [723] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 533, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [724] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 533, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [725] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 541, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [726] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 541, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [727] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 541, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [728] Ref(TemporaryTransientGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 541, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [729] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 541, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [730] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 541, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [731] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 541, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [732] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 544, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [733] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 546, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [734] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [735] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [736] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [737] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [738] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [739] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [740] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [741] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [742] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [743] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [744] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [745] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [746] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [747] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [748] Ref(TableEndClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [749] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 571, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [750] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 571, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [751] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 571, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [752] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 571, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [753] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 574, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [754] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 574, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [755] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 574, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [756] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [757] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [758] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 1, _padding: 0 }, // [759] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 578, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [760] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 581, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [761] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 581, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [762] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 581, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [763] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [764] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [765] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 2, _padding: 0 }, // [766] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [767] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [768] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [769] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [770] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [771] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [772] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [773] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 586, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [774] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [775] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [776] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [777] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [778] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [779] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [780] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [781] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [782] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [783] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [784] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [785] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [786] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [787] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [788] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [789] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [790] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [791] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [792] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [793] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 608, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [794] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 610, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [795] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 612, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [796] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 612, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [797] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 612, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [798] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 614, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [799] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 614, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [800] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 614, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [801] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [802] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 617, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [803] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [804] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [805] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [806] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [807] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [808] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [809] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [810] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 624, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [811] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [812] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [813] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [814] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [815] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [816] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 631, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [817] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 631, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [818] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 631, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [819] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 631, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [820] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [821] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [822] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [823] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [824] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [825] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [826] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [827] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [828] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 642, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [829] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [830] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [831] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [832] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [833] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [834] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [835] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [836] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [837] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [838] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 645, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [839] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 647, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [840] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [841] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [842] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [843] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [844] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [845] StringParser("CUME_DIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [846] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [847] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [848] StringParser("CURRENT_DEFAULT_TRANSFORM_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [849] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [850] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [851] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [852] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [853] StringParser("CURRENT_TRANSFORM_GROUP_FOR_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [854] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [855] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [856] StringParser("CURSOR_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [857] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [858] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [859] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [860] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [861] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 654, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [862] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 654, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [863] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [864] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [865] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [866] Ref(TimeWithTZGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [867] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [868] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [869] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [870] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 664, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [871] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 666, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [872] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [873] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [874] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [875] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [876] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [877] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [878] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [879] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [880] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 674, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [881] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 676, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [882] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [883] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [884] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [885] Ref(DatatypeIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [886] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 678, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [887] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [888] Ref(UnsignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [889] Ref(CharCharacterSetGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [890] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [891] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 680, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [892] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 14, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [893] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [894] Ref(Last_n_daysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [895] Ref(Last_n_fiscal_quartersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [896] Ref(Last_n_fiscal_yearsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [897] Ref(Last_n_monthsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [898] Ref(Last_n_quartersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [899] Ref(Last_n_weeksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [900] Ref(Last_n_yearsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [901] Ref(Next_n_daysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [902] Ref(Next_n_fiscal_quartersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [903] Ref(Next_n_fiscal_yearsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [904] Ref(Next_n_monthsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [905] Ref(Next_n_quartersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [906] Ref(Next_n_weeksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [907] Ref(Next_n_yearsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [908] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [909] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [910] MultiStringParser(1 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [911] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [912] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [913] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 701, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [914] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 703, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [915] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [916] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [917] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [918] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 708, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [919] TypedParser("date_literal")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 708, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [920] TypedParser("datetime_literal")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 708, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [921] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [922] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [923] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [924] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [925] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [926] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [927] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [928] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [929] MultiStringParser(11 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [930] StringParser("DATETIME_INTERVAL_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [931] StringParser("DATETIME_INTERVAL_PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [932] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [933] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [934] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [935] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [936] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [937] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [938] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [939] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [940] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [941] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [942] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [943] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [944] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [945] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 714, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [946] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 714, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [947] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [948] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [949] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [950] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [951] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [952] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [953] StringParser("DEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [954] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [955] StringParser("DEGREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [956] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [957] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [958] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [959] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [960] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [961] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [962] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [963] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [964] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [965] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [966] StringParser("DENSE_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [967] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [968] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [969] StringParser("DEREF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [970] StringParser("DERIVED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [971] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [972] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [973] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [974] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [975] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [976] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [977] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [978] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [979] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [980] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [981] StringParser("DIAGNOSTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [982] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [983] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [984] StringParser("DISCONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [985] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [986] StringParser("DISPATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [987] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [988] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [989] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [990] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [991] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [992] StringParser("DO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [993] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [994] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [995] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 722, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [996] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [997] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [998] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [999] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 728, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1000] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 728, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1001] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 728, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1002] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1003] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1004] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1005] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 733, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1006] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1007] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 738, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1008] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 738, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1009] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 738, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1010] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 738, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1011] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 738, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1012] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 738, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1013] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1014] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1015] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1016] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1017] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1018] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1019] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1020] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1021] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1022] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1023] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1024] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1025] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1026] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1027] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1028] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1029] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1030] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 755, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1031] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 755, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1032] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 755, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1033] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 755, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1034] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 755, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1035] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1036] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1037] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1038] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1039] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1040] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1041] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1042] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1043] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1044] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 6, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1045] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 769, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1046] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 769, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1047] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 769, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1048] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 769, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1049] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 769, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1050] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1051] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 771, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1052] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1053] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1054] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1055] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1056] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1057] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1058] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 780, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1059] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 780, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1060] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 780, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1061] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 780, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1062] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 780, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1063] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 780, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1064] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1065] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1066] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1067] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1068] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1069] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1070] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1071] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1072] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1073] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1074] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1075] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1076] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1077] StringParser("DYNAMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1078] StringParser("DYNAMIC_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1079] StringParser("DYNAMIC_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1080] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1081] StringParser("ELEMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1082] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1083] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1084] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1085] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1086] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1087] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 795, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1088] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1089] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1090] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1091] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1092] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1093] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1094] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1095] StringParser("END-EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1096] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1097] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1098] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1099] StringParser("EQUALS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1100] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1101] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1102] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1103] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1104] StringParser("EVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1105] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1106] StringParser("EXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1107] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1108] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1109] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1110] StringParser("EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1111] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1112] StringParser("EXECUTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1113] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1114] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1115] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1116] StringParser("EXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1117] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 797, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1118] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1119] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1120] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1121] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1122] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1123] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1124] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1125] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1126] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 805, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1127] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 805, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1128] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 10, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1129] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1130] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1131] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1132] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1133] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1134] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1135] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1136] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1137] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1138] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1139] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1140] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1141] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1142] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1143] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 826, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1144] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1145] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1146] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1147] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1148] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1149] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 828, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1150] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 828, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1151] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 828, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1152] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 832, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1153] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1154] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1155] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1156] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1157] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1158] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1159] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1160] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 836, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1161] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 837, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1162] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 838, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1163] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1164] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 843, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1165] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 843, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1166] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 843, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1167] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 843, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1168] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 843, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1169] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 845, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1170] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1171] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 847, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1172] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 847, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1173] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 847, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [1174] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 851, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1175] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1176] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1177] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1178] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1179] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1180] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1181] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1182] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 860, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1183] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1184] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1185] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1186] Ref(DateLiteralNSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1187] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1188] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 9, min_times: 0, first_terminator_idx: 20, terminator_count: 1, _padding: 0 }, // [1189] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1190] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1191] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1192] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 875, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1193] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 879, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1194] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 879, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1195] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 879, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1196] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 879, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1197] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 881, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1198] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1199] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1200] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1201] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1202] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1203] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1204] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1205] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1206] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 888, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1207] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1208] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1209] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1210] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1211] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1212] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1213] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1214] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1215] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1216] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1217] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1218] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1219] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1220] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1221] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1222] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1223] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1224] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 7, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1225] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1226] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1227] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1228] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1229] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1230] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1231] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1232] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1233] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1234] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1235] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1236] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1237] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1238] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 5, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1239] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 914, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1240] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 914, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1241] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1242] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1243] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 916, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1244] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1245] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 918, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1246] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 919, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1247] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 919, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1248] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1249] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1250] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1251] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1252] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1253] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1254] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1255] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1256] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1257] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1258] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1259] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 928, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1260] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1261] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 929, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1262] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1263] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 1, min_times: 1, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1264] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 932, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1265] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 932, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1266] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1267] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1268] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1269] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1270] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1271] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1272] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1273] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1274] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1275] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1276] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1277] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1278] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1279] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1280] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1281] StringParser("FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1282] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1283] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1284] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1285] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1286] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 942, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1287] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 942, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1288] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 942, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1289] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 942, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1290] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 942, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1291] StringParser("FORTRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 942, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1292] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 942, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1293] StringParser("FOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 942, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1294] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 944, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1295] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 944, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1296] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 946, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1297] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1298] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 950, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1299] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 950, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1300] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 950, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1301] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 952, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1302] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1303] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1304] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1305] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1306] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1307] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1308] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1309] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1310] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1311] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1312] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1313] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1314] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1315] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1316] StringParser("FREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1317] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1318] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1319] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1320] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1321] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1322] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1323] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 12, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1324] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1325] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1326] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1327] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1328] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1329] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1330] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1331] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1332] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1333] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1334] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1335] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1336] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1337] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1338] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1339] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1340] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1341] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 986, child_count: 7, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1342] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1343] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 993, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1344] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 995, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1345] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 998, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1346] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 998, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1347] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 998, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1348] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 999, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1349] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1350] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1351] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1352] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1353] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1003, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1354] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1355] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1356] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1357] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1358] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1359] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1360] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1361] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1011, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1362] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1012, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1363] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1012, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1364] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1365] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1017, child_count: 6, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1366] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 2, _padding: 0 }, // [1367] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1368] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1369] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1370] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1371] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1372] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1031, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1373] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1031, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1374] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1031, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1375] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1376] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1377] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1378] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1033, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 2, _padding: 0 }, // [1379] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1034, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1380] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1035, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1381] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1037, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1382] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1037, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1383] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1037, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1384] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1385] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1386] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1387] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1388] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1389] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1390] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1391] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1392] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1393] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1394] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1041, child_count: 11, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1395] AnyNumberOf(11 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1052, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1396] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1052, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1397] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1398] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1399] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1400] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 4, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1401] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1059, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1402] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1059, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1403] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1404] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1405] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1406] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1407] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1063, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1408] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1409] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1410] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1411] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1412] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1413] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1067, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1414] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1067, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1415] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1416] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1417] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1418] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1419] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1420] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1421] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1422] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1423] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1076, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1424] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1425] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1426] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1427] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1428] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1429] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1430] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1431] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1432] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1433] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1434] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1435] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1436] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1437] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1086, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1438] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1086, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1439] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1440] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1089, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1441] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1089, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1442] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1443] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1444] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1445] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1446] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1091, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1447] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1093, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 1, _padding: 0 }, // [1448] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1094, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1449] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1450] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1451] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1452] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 1, _padding: 0 }, // [1453] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1454] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1455] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1456] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1457] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1458] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1459] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1460] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1461] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1462] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1463] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1464] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1465] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1466] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1467] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1468] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1469] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1470] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1113, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1471] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1472] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1473] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1474] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1120, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1475] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1120, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1476] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1120, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1477] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1120, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1478] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1122, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1479] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1124, child_count: 1, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1480] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1125, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1481] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1482] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1483] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1484] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1485] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1486] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1487] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1488] StringParser("FUSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1489] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1490] StringParser("G")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1491] StringParser("GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1492] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1493] StringParser("GET")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1494] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1495] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1496] StringParser("GO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1497] StringParser("GOTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1498] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 6, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1499] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1500] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 4, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1501] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1138, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1502] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1503] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1504] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1505] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1506] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1143, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1507] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1143, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1508] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1143, child_count: 4, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1509] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1510] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1511] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1512] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1513] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1514] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1515] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1516] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1147, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1517] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1150, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1518] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1519] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1520] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1521] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1522] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1523] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1524] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1525] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1526] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1527] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1528] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1529] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1159, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1530] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1162, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1531] Ref(GrantedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1162, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1532] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1162, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1533] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1534] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1535] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1536] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1537] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1538] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1165, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1539] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1540] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1541] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1542] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1543] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1167, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1544] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1545] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1546] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1547] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1548] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1549] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1550] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1551] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 1, _padding: 0 }, // [1552] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1553] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1554] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1555] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1556] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1557] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 6, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1558] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1559] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1560] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1561] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1562] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1563] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1564] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1565] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1566] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1190, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1567] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1568] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 1, _padding: 0 }, // [1569] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1570] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1197, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1571] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1197, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1572] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1197, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1573] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1197, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1574] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1199, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1575] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1199, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1576] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1199, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1577] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1578] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1579] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1202, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1580] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1205, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1581] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1207, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1582] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1583] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1584] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1585] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1586] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1587] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1588] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1589] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1590] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1591] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1223, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1592] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1593] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1594] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1595] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1596] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1597] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1598] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1599] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1600] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1601] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1602] StringParser("HIERARCHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1603] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1604] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1605] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1606] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1607] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1608] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1609] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1610] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1611] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1612] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1613] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1614] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1615] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1616] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1617] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1618] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1619] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1620] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1621] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1622] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1623] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1624] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1625] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1626] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1627] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1628] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1629] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1630] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1631] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1632] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1633] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1634] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1635] StringParser("IMPLEMENTATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1636] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1637] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1638] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1639] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1237, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1640] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1237, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1641] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1237, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1642] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1643] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1644] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1645] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1646] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1647] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1648] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1649] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1650] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1651] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1652] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1653] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1248, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1654] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1655] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1656] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1657] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1658] StringParser("INDICATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1659] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1660] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1661] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1662] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1663] StringParser("INITIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1664] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1665] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1666] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1667] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1668] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1669] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1670] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1250, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1671] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1672] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1673] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1674] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1675] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1676] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1258, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1677] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1258, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1678] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1679] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1680] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1681] Ref(DefaultValuesGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1682] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1683] StringParser("INSTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1684] StringParser("INSTANTIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1685] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1686] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1687] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1688] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1689] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1690] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1691] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1692] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1693] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1694] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1695] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1696] StringParser("INTERSECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1697] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1698] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1699] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1700] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1701] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1702] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1703] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1704] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1705] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1706] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1707] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1708] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1709] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1710] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1711] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1712] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1713] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1714] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1715] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1716] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1717] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1718] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1719] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1720] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1721] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1722] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1723] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1724] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1725] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1726] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1727] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1728] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1729] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1730] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1731] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 7, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1732] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1292, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1733] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1292, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1734] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1292, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1735] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1292, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1736] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1737] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1293, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1738] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1739] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1740] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1741] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1742] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1743] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1744] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1745] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1305, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1746] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1305, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1747] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1305, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1748] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1305, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1749] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1305, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1750] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1751] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1752] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1309, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1753] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1754] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1310, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1755] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1310, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1756] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1310, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1757] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1758] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1759] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1760] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1761] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1319, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1762] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1319, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1763] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1319, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1764] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1765] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1766] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1323, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1767] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1326, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1768] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1326, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1769] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1326, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1770] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1326, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1771] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1326, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1772] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1773] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1774] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1333, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1775] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1776] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1777] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1778] StringParser("K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1779] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1780] StringParser("KEY_MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1781] StringParser("KEY_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1782] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1783] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1784] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1785] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1786] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1787] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1788] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1789] StringParser("LAST_90_DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1790] StringParser("LAST_FISCAL_QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1791] StringParser("LAST_FISCAL_YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1792] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1793] StringParser("LAST_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1794] StringParser("LAST_N_DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1795] StringParser("LAST_N_FISCAL_QUARTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1796] StringParser("LAST_N_FISCAL_YEARS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1797] StringParser("LAST_N_MONTHS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1798] StringParser("LAST_N_QUARTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1799] StringParser("LAST_N_WEEKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1800] StringParser("LAST_N_YEARS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1801] StringParser("LAST_QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1802] StringParser("LAST_WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1803] StringParser("LAST_YEAR")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1804] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1805] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1806] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1807] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1808] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1809] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1810] StringParser("LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1811] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1335, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1812] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1813] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1814] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1815] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1337, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1816] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1817] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1340, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1818] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1819] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1820] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1821] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1342, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1822] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1344, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1823] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1344, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1824] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1344, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1825] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1826] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1827] Ref(RlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1828] Ref(IlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1829] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1830] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1831] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1832] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1833] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1834] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1835] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1360, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1836] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1360, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1837] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1360, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1838] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1360, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1839] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1840] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1364, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1841] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1364, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1842] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1843] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1844] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1845] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1846] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1847] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1848] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1849] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1850] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1851] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1368, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1852] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1368, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1853] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1854] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1855] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1856] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1857] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1858] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1859] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1860] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1861] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1862] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1863] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1864] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1865] StringParser("LN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1866] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1867] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1868] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1869] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1870] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1871] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1872] StringParser("LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1873] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1874] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1875] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1876] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1877] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1878] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1879] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1880] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1881] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1882] StringParser("LOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1883] StringParser("M")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1884] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1885] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1886] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1887] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1888] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1385, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1889] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1890] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1891] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1387, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1892] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1389, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1893] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1389, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1894] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1895] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1896] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1897] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1898] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1899] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1900] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1901] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1902] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1903] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1904] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1905] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1906] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1907] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1908] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1909] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1910] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1911] StringParser("MEMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1392, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1912] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1392, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1913] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1914] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1915] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1916] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1917] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1918] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1919] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1920] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 2, min_times: 1, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1921] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1922] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1923] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 7, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1924] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1925] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1926] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1408, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1927] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1928] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1929] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1930] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1931] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1932] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1933] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1934] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1935] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1936] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1937] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1420, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1938] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1939] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1940] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1941] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1942] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 12, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1943] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1944] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1945] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1436, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1946] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1436, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1947] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1436, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1948] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1436, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1949] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1950] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1951] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1952] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1953] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1954] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1955] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1956] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1957] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1958] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1959] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1960] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1961] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1962] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1963] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1964] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1965] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1966] StringParser("MESSAGE_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1967] StringParser("MESSAGE_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1968] StringParser("MESSAGE_TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1969] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1970] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1971] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1972] StringParser("MIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1973] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1974] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1975] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1976] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1977] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1978] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1979] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1980] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1981] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1982] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1983] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1984] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1985] StringParser("MODIFIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1986] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1987] StringParser("MODULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1988] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1989] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1990] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1991] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1992] StringParser("MORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1993] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1994] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1995] StringParser("MULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1996] StringParser("MUMPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1997] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1998] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1999] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2000] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2001] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2002] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2003] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1456, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2004] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1456, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2005] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1459, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2006] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1459, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2007] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2008] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2009] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2010] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1465, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2011] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1465, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2012] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1465, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2013] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1465, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2014] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2015] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2016] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2017] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2018] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2019] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2020] StringParser("NCLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2021] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2022] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2023] StringParser("NESTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2024] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2025] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2026] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2027] StringParser("NEXT_90_DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2028] StringParser("NEXT_FISCAL_QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2029] StringParser("NEXT_FISCAL_YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2030] StringParser("NEXT_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2031] StringParser("NEXT_N_DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2032] StringParser("NEXT_N_FISCAL_QUARTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2033] StringParser("NEXT_N_FISCAL_YEARS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2034] StringParser("NEXT_N_MONTHS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2035] StringParser("NEXT_N_QUARTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2036] StringParser("NEXT_N_WEEKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2037] StringParser("NEXT_N_YEARS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2038] StringParser("NEXT_QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2039] StringParser("NEXT_WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2040] StringParser("NEXT_YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2041] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2042] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2043] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2044] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2045] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2046] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2047] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2048] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2049] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2050] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2051] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2052] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1467, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2053] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2054] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2055] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2056] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2057] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1476, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2058] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1479, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2059] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1479, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2060] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1482, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2061] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1482, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2062] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1482, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2063] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1482, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2064] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1486, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2065] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1486, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2066] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1486, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2067] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1486, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2068] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1486, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2069] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2070] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1489, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2071] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2072] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2073] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2074] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2075] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2076] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2077] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2078] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2079] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2080] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2081] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2082] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1494, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2083] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1496, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2084] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2085] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2086] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1498, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2087] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2088] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2089] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2090] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2091] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2092] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2093] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2094] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2095] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2096] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2097] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2098] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2099] StringParser("NULLABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2100] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2101] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2102] StringParser("NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2103] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2104] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2105] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2106] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1503, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2107] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1503, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2108] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1503, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2109] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1503, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2110] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1506, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2111] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2112] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2113] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2114] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2115] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2116] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2117] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2118] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 12, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2119] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2120] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2121] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2122] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2123] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2124] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2125] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2126] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2127] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2128] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2129] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2130] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2131] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2132] StringParser("OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2133] StringParser("OCTETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2134] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2135] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2136] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2137] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2138] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2139] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2140] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1529, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2141] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1530, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2142] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1530, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2143] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2144] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2145] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2146] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2147] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2148] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2149] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2150] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2151] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2152] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2153] StringParser("OPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2154] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2155] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2156] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2157] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2158] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2159] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2160] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2161] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2162] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2163] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2164] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2165] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2166] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1532, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2167] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2168] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2169] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2170] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2171] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2172] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 2, _padding: 0 }, // [2173] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1541, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2174] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1545, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2175] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2176] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2177] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2178] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1548, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2179] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1550, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2180] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1550, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2181] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1550, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2182] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2183] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1552, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2184] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2185] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2186] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2187] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2188] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2189] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 7, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2190] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2191] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2192] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2193] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2194] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2195] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2196] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2197] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1561, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2198] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1561, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2199] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2200] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2201] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2202] StringParser("ORDERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2203] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2204] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2205] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2206] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2207] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2208] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2209] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2210] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2211] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2212] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1570, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2213] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1570, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2214] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1573, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2215] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1573, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2216] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2217] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1575, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2218] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1575, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2219] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1577, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2220] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2221] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2222] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2223] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2224] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2225] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2226] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2227] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2228] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2229] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2230] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2231] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2232] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2233] StringParser("PAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2234] StringParser("PARAMETER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2235] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2236] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2237] StringParser("PARAMETER_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2238] StringParser("PARAMETER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2239] StringParser("PARAMETER_ORDINAL_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2240] StringParser("PARAMETER_SPECIFIC_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2241] StringParser("PARAMETER_SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2242] StringParser("PARAMETER_SPECIFIC_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2243] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2244] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2245] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2246] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2247] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2248] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1590, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2249] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2250] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1595, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2251] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1595, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2252] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1595, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2253] StringParser("PASCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1595, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2254] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1595, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2255] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1595, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2256] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2257] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2258] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1599, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2259] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2260] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2261] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2262] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2263] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2264] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2265] StringParser("PERCENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2266] StringParser("PERCENT_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2267] StringParser("PERCENTILE_CONT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2268] StringParser("PERCENTILE_DISC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2269] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2270] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2271] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2272] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2273] StringParser("PLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2274] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2275] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2276] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2277] StringParser("+")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1601, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2278] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2279] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2280] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2281] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2282] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2283] StringParser("POWER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2284] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2285] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2286] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2287] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2288] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2289] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2290] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2291] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2292] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2293] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2294] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2295] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2296] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2297] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2298] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2299] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2300] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2301] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2302] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2303] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2304] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2305] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2306] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2307] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2308] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2309] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2310] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2311] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1610, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2312] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1610, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2313] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2314] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2315] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2316] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2317] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2318] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2319] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2320] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2321] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2322] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2323] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2324] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2325] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2326] StringParser("RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2327] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2328] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2329] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2330] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2331] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2332] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2333] StringParser("READS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2334] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2335] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2336] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2337] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2338] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2339] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2340] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2341] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2342] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2343] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2344] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1621, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2345] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2346] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2347] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2348] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2349] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2350] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2351] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2352] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2353] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2354] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2355] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2356] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2357] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2358] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2359] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2360] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2361] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2362] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2363] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1639, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2364] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1639, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2365] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1639, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2366] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2367] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2368] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2369] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1643, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2370] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1643, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2371] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1643, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2372] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2373] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2374] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2375] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2376] StringParser("REGR_AVGX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2377] StringParser("REGR_AVGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2378] StringParser("REGR_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2379] StringParser("REGR_INTERCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2380] StringParser("REGR_R2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2381] StringParser("REGR_SLOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2382] StringParser("REGR_SXX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2383] StringParser("REGR_SXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2384] StringParser("REGR_SYY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2385] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2386] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2387] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2388] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2389] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2390] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2391] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2392] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2393] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2394] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2395] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2396] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2397] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2398] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2399] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2400] StringParser("RESTORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2401] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2402] StringParser("RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2403] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2404] StringParser("RETURNED_CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2405] StringParser("RETURNED_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2406] StringParser("RETURNED_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2407] StringParser("RETURNED_SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2408] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2409] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1645, child_count: 6, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2410] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2411] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1651, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2412] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2413] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2414] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2415] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2416] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2417] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1661, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2418] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1661, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2419] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1661, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2420] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1661, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2421] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2422] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2423] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2424] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2425] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2426] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2427] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2428] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2429] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2430] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2431] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2432] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2433] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2434] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2435] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2436] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2437] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2438] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2439] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2440] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2441] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2442] StringParser("ROUTINE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2443] StringParser("ROUTINE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2444] StringParser("ROUTINE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2445] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2446] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2447] StringParser("ROW_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2448] StringParser("ROW_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2449] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2450] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2451] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2452] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2453] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2454] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2455] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1671, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2456] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1671, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2457] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2458] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2459] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2460] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1676, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2461] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1676, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2462] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1678, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2463] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2464] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2465] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2466] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2467] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2468] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2469] StringParser("SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2470] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2471] StringParser("SCHEMA_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2472] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2473] StringParser("SCOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2474] StringParser("SCOPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2475] StringParser("SCOPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2476] StringParser("SCOPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2477] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2478] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2479] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2480] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2481] StringParser("SECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2482] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2483] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2484] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2485] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2486] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2487] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2488] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2489] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2490] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 1, _padding: 0 }, // [2491] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2492] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2493] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 1693, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2494] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2495] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2496] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 7, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2497] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1702, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2498] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2499] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2500] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2501] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2502] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2503] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2504] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2505] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2506] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2507] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2508] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 12, min_times: 0, first_terminator_idx: 32, terminator_count: 3, _padding: 0 }, // [2509] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2510] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2511] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2512] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2513] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2514] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2515] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2516] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2517] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2518] Ref(OffsetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2519] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2520] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2521] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2522] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2523] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [2524] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2525] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1721, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2526] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1723, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2527] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2528] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2529] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2530] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1731, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2531] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1731, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2532] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1731, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2533] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2534] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2535] StringParser("SELF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2536] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2537] StringParser("SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2538] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2539] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2540] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2541] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2542] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2543] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1738, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2544] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1740, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2545] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1740, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2546] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1740, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2547] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2548] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2549] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2550] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2551] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2552] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2553] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2554] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2555] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2556] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2557] StringParser("SERVER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2558] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2559] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2560] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2561] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2562] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2563] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2564] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2565] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2566] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 7, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2567] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1762, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2568] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1762, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2569] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1762, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2570] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1762, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2571] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1762, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2572] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1762, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2573] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1762, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2574] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1762, child_count: 5, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2575] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2576] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 1, min_times: 1, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2577] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2578] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2579] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2580] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2581] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2582] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2583] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2584] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2585] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2586] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2587] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1775, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2588] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1777, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2589] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1777, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2590] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1777, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2591] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1777, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2592] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1777, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2593] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2594] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2595] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2596] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2597] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2598] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2599] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2600] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2601] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2602] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2603] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1783, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2604] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2605] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2606] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 1, min_times: 1, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2607] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2608] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2609] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2610] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2611] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2612] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2613] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2614] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2615] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2616] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2617] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2618] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2619] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 1, _padding: 0 }, // [2620] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2621] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2622] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2623] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2624] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2625] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2626] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2627] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2628] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2629] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2630] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2631] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2632] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2633] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2634] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2635] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2636] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2637] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2638] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2639] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2640] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2641] StringParser("SPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2642] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2643] StringParser("SPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2644] StringParser("SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2645] StringParser("SPECIFICTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2646] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2647] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2648] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2649] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2650] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2651] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2652] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2653] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2654] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2655] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2656] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2657] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2658] StringParser("SQLCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2659] StringParser("SQLERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2660] StringParser("SQLEXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2661] StringParser("SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2662] StringParser("SQLWARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2663] StringParser("SQRT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2664] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2665] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2666] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2667] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2668] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2669] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2670] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2671] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2672] StringParser("STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2673] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2674] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2675] StringParser("STATIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2676] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2677] StringParser("STDDEV_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2678] StringParser("STDDEV_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2679] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2680] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2681] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2682] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2683] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2684] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2685] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2686] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2687] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2688] StringParser("STRING")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1801, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2689] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1804, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2690] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1806, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2691] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2692] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2693] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2694] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2695] StringParser("STRUCTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2696] StringParser("STYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2697] StringParser("SUBCLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2698] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2699] StringParser("SUBMULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2700] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2701] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2702] StringParser("SUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2703] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2704] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2705] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2706] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2707] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2708] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2709] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2710] StringParser("SYSTEM_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1808, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2711] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1810, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2712] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2713] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2714] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2715] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1815, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2716] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2717] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2718] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2719] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2720] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2721] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2722] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2723] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2724] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2725] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2726] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2727] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2728] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2729] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2730] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2731] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2732] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2733] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2734] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1834, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2735] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1834, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2736] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1834, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2737] StringParser("TABLE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1834, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2738] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1834, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2739] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1834, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2740] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1834, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2741] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1836, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 1, _padding: 0 }, // [2742] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2743] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2744] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2745] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2746] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1839, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2747] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1840, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2748] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1840, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2749] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1840, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2750] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1840, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2751] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1840, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2752] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1840, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2753] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1840, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2754] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1840, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2755] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2756] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2757] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2758] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2759] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2760] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2761] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2762] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2763] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2764] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2765] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2766] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2767] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2768] StringParser("THIS_FISCAL_QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2769] StringParser("THIS_FISCAL_YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2770] StringParser("THIS_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2771] StringParser("THIS_QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2772] StringParser("THIS_WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2773] StringParser("THIS_YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2774] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2775] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2776] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2777] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2778] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2779] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2780] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2781] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1849, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2782] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1852, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2783] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1854, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2784] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1854, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2785] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1854, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2786] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1854, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2787] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1854, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2788] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1855, child_count: 4, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2789] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2790] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2791] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2792] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2793] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2794] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2795] StringParser("TIMEZONE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2796] StringParser("TIMEZONE_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2797] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2798] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2799] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2800] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2801] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2802] StringParser("TODAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2803] StringParser("TOMORROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2804] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2805] StringParser("TOP_LEVEL_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2806] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2807] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1859, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2808] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 4, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2809] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1863, child_count: 5, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2810] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2811] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2812] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2813] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2814] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2815] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1868, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2816] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1870, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2817] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1870, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2818] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1870, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2819] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1872, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2820] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1872, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2821] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1872, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2822] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2823] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2824] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2825] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2826] StringParser("TRANSACTION_ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2827] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2828] StringParser("TRANSACTIONS_COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2829] StringParser("TRANSACTIONS_ROLLED_BACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2830] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2831] StringParser("TRANSFORMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2832] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2833] StringParser("TRANSLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2834] StringParser("TRANSLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2835] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2836] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2837] StringParser("TRIGGER_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2838] StringParser("TRIGGER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2839] StringParser("TRIGGER_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2840] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2841] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1878, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2842] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1878, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2843] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1878, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2844] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1878, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2845] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1878, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2846] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1878, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2847] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1878, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2848] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2849] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2850] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2851] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2852] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2853] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2854] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2855] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1886, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2856] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1886, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2857] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1886, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2858] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2859] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2860] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2861] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2862] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2863] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2864] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2865] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2866] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2867] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2868] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2869] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2870] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2871] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2872] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2873] StringParser("UNDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2874] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2875] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2876] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1895, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2877] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1895, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2878] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2879] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2880] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1897, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2881] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1897, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2882] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2883] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2884] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2885] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2886] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2887] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2888] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2889] StringParser("UNNAMED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2890] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 7, min_times: 0, first_terminator_idx: 37, terminator_count: 5, _padding: 0 }, // [2891] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2892] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2893] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2894] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2895] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2896] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2897] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2898] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2899] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2900] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2901] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2902] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2903] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2904] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2905] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 1, min_times: 1, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2906] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2907] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2908] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2909] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2910] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2911] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2912] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 8, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2913] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2914] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2915] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1918, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2916] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2917] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2918] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2919] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2920] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2921] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2922] StringParser("UPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2923] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2924] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2925] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2926] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2927] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2928] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2929] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2930] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2931] StringParser("USER_DEFINED_TYPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2932] StringParser("USER_DEFINED_TYPE_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2933] StringParser("USER_DEFINED_TYPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2934] StringParser("USER_DEFINED_TYPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2935] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2936] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2937] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2938] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2939] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2940] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2941] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2942] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2943] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1921, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2944] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1921, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2945] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2946] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1925, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2947] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1925, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2948] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1925, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2949] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2950] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1929, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2951] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2952] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2953] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1934, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2954] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2955] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2956] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2957] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2958] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2959] StringParser("VAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2960] StringParser("VAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2961] StringParser("VARBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2962] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2963] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2964] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2965] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2966] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2967] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2968] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2969] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2970] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2971] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2972] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2973] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2974] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2975] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2976] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2977] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1937, child_count: 8, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2978] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2979] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2980] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2981] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2982] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2983] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2984] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2985] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2986] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2987] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2988] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2989] StringParser("WHENEVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2990] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1952, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2991] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1952, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2992] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1954, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2993] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2994] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 8, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2995] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2996] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2997] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2998] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2999] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3000] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3001] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3002] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3003] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3004] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3005] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3006] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3007] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3008] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3009] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3010] Token("whitespace")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3011] StringParser("WIDTH_BUCKET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3012] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3013] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1970, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3014] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1972, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3015] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1973, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3016] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1975, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3017] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3018] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3019] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3020] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3021] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3022] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3023] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3024] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1979, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3025] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1983, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3026] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3027] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1986, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3028] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1986, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3029] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1986, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3030] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1986, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3031] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1986, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3032] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1986, child_count: 6, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3033] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3034] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3035] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3036] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 1, _padding: 0 }, // [3037] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1994, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3038] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1994, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3039] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1994, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3040] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1994, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3041] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1994, child_count: 6, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3042] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3043] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3044] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3045] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 1, _padding: 0 }, // [3046] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3047] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3048] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3049] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3050] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3051] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2005, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3052] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2005, child_count: 1, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3053] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3054] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3055] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2006, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3056] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2006, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3057] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 4, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3058] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3059] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3060] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3061] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3062] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3063] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3064] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3065] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3066] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3067] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3068] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3069] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3070] StringParser("X509")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3071] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3072] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3073] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3074] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3075] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3076] StringParser("YESTERDAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3077] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3078] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    5, 6, 61, 7, 51, 8, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 28, 33, 38, 39, 44, 9, 10, 22, 23, 24, 25, 26, 27, 29, 30,
    31, 32, 34, 35, 36, 37, 40, 41, 42, 43, 45, 46, 47, 48, 49, 50,
    52, 58, 53, 54, 57, 55, 56, 62, 63, 64, 66, 74, 78, 81, 84, 89,
    67, 68, 69, 70, 71, 72, 73, 75, 76, 77, 79, 80, 82, 83, 85, 86,
    87, 88, 90, 133, 91, 100, 105, 108, 109, 110, 111, 112, 113, 114, 115, 116,
    117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 92, 93,
    94, 95, 98, 99, 96, 97, 101, 102, 103, 104, 106, 107, 131, 132, 135, 58,
    138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 151, 154, 157, 149, 150,
    152, 153, 155, 156, 158, 159, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
    171, 174, 177, 180, 172, 173, 175, 176, 178, 179, 181, 182, 184, 185, 187, 191,
    195, 199, 203, 205, 207, 188, 189, 190, 58, 192, 193, 194, 58, 196, 197, 198,
    58, 200, 201, 202, 58, 204, 58, 206, 58, 209, 220, 221, 222, 230, 223, 229,
    224, 225, 226, 227, 228, 233, 234, 240, 244, 245, 246, 251, 254, 241, 242, 243,
    247, 250, 248, 249, 252, 253, 256, 257, 258, 259, 260, 262, 263, 264, 265, 267,
    273, 286, 287, 268, 269, 270, 271, 272, 274, 277, 278, 279, 275, 276, 280, 285,
    281, 284, 282, 283, 288, 289, 292, 290, 291, 294, 295, 296, 297, 298, 58, 309,
    310, 311, 312, 313, 314, 315, 316, 317, 318, 320, 325, 326, 321, 324, 322, 323,
    330, 325, 326, 331, 58, 334, 348, 356, 357, 358, 359, 360, 361, 362, 363, 364,
    376, 377, 378, 379, 387, 388, 391, 392, 397, 398, 401, 402, 405, 227, 228, 406,
    58, 408, 227, 228, 409, 58, 412, 227, 228, 419, 227, 228, 421, 422, 423, 424,
    425, 227, 228, 433, 443, 434, 435, 436, 440, 230, 442, 437, 444, 445, 435, 446,
    450, 230, 452, 447, 492, 493, 494, 495, 505, 506, 507, 508, 510, 513, 511, 512,
    514, 517, 521, 524, 527, 528, 529, 532, 533, 536, 515, 516, 518, 519, 520, 227,
    228, 522, 523, 525, 526, 530, 531, 534, 535, 538, 539, 540, 542, 541, 227, 228,
    543, 547, 548, 560, 561, 567, 568, 569, 570, 571, 572, 573, 574, 582, 583, 588,
    589, 590, 612, 613, 614, 618, 619, 620, 630, 631, 632, 635, 615, 616, 617, 227,
    228, 621, 622, 623, 624, 625, 629, 626, 627, 628, 633, 634, 636, 637, 639, 640,
    641, 642, 644, 645, 646, 647, 648, 649, 650, 651, 654, 652, 653, 656, 657, 658,
    659, 660, 661, 662, 663, 664, 665, 666, 227, 228, 667, 58, 670, 671, 672, 673,
    674, 675, 687, 688, 676, 677, 678, 227, 228, 679, 58, 680, 681, 682, 683, 684,
    685, 325, 326, 686, 58, 690, 691, 692, 693, 695, 696, 697, 698, 700, 704, 708,
    709, 710, 715, 718, 701, 702, 703, 705, 706, 707, 711, 714, 712, 713, 716, 717,
    720, 721, 722, 723, 724, 726, 727, 728, 729, 730, 731, 732, 748, 733, 740, 745,
    734, 739, 735, 227, 228, 736, 58, 737, 738, 741, 742, 743, 744, 744, 227, 228,
    746, 747, 750, 751, 752, 753, 759, 772, 773, 774, 811, 754, 755, 756, 757, 758,
    760, 770, 761, 762, 763, 764, 765, 766, 767, 58, 775, 785, 788, 801, 807, 776,
    777, 778, 779, 780, 781, 782, 783, 784, 786, 787, 789, 792, 790, 791, 793, 794,
    795, 798, 796, 797, 799, 800, 802, 803, 804, 805, 806, 808, 809, 810, 227, 228,
    812, 813, 814, 815, 817, 818, 819, 821, 822, 823, 824, 825, 826, 827, 828, 831,
    829, 830, 830, 227, 228, 840, 843, 841, 842, 844, 227, 228, 862, 863, 864, 866,
    867, 870, 890, 868, 869, 871, 886, 887, 872, 881, 873, 876, 874, 875, 877, 878,
    879, 880, 882, 885, 883, 884, 888, 889, 893, 908, 909, 894, 895, 896, 897, 898,
    899, 900, 901, 902, 903, 904, 905, 906, 907, 913, 914, 227, 228, 915, 58, 916,
    917, 919, 920, 921, 922, 927, 923, 924, 925, 926, 948, 949, 960, 961, 962, 974,
    975, 976, 997, 998, 1000, 1001, 1002, 1006, 1003, 1004, 1005, 227, 228, 1008, 1009, 1010,
    1011, 1012, 1014, 1015, 1016, 1017, 1019, 1020, 1021, 1022, 1023, 1026, 1027, 1028, 1029, 1031,
    1032, 1033, 1034, 1036, 1037, 1038, 1039, 1040, 1042, 1043, 1044, 1046, 1047, 1048, 1049, 1050,
    1052, 1051, 58, 1054, 1055, 1056, 1057, 1059, 1060, 1061, 1062, 1063, 1065, 1066, 1067, 1068,
    1070, 1071, 1072, 1073, 1074, 1083, 435, 1084, 230, 227, 228, 1089, 1090, 1119, 1120, 1121,
    1122, 1123, 1124, 1127, 1128, 1129, 1130, 1131, 1134, 1135, 1136, 1140, 1141, 1142, 1143, 1149,
    1132, 1133, 1137, 1138, 1139, 1144, 1145, 1146, 1147, 1148, 1150, 1151, 1153, 1156, 1157, 1158,
    1154, 1155, 1160, 1161, 1162, 1163, 1164, 1168, 1165, 1166, 1167, 1170, 1173, 1171, 1172, 1175,
    1179, 1185, 1186, 1176, 1177, 1178, 227, 228, 1180, 1183, 1181, 1182, 1184, 1189, 1224, 1190,
    1191, 1192, 1204, 1205, 1206, 1213, 1221, 1222, 1193, 227, 228, 1194, 1195, 1196, 1197, 1198,
    58, 1199, 1200, 1201, 1202, 1203, 1207, 1210, 1208, 1209, 1211, 227, 228, 1212, 58, 1214,
    1215, 1216, 1217, 1218, 1219, 1220, 1226, 1227, 1228, 1229, 1230, 1231, 1232, 1240, 1241, 1244,
    1248, 1251, 1242, 1243, 1245, 1246, 1247, 1249, 1250, 1252, 1253, 1254, 1255, 1260, 1262, 1266,
    1261, 1263, 1264, 1265, 1267, 1270, 1271, 1272, 227, 228, 1273, 1274, 1287, 1288, 1295, 1296,
    1297, 1310, 1298, 1301, 1299, 1300, 1302, 1307, 1303, 1304, 1305, 1306, 1308, 1309, 1311, 1297,
    1312, 1297, 1314, 1315, 1321, 1322, 1323, 58, 1325, 1326, 1327, 1330, 1333, 1334, 1335, 1336,
    1337, 1338, 1339, 1340, 1328, 1329, 1331, 1332, 1342, 1360, 1343, 1344, 1347, 1348, 1354, 1358,
    1359, 1345, 1346, 1346, 227, 228, 1349, 1350, 1351, 1352, 1353, 1355, 1356, 1357, 1361, 227,
    228, 1342, 1362, 1363, 1365, 1366, 1366, 227, 228, 435, 1367, 230, 1378, 1379, 1390, 1368,
    1369, 1370, 1371, 227, 228, 1373, 1374, 1376, 1377, 1380, 1381, 1382, 1383, 1385, 1386, 1388,
    1389, 1396, 1397, 1401, 1407, 1413, 1419, 1420, 1423, 1433, 1434, 1435, 1398, 1399, 1400, 1402,
    1403, 1405, 1406, 1404, 1408, 1411, 1412, 1409, 1410, 1414, 1415, 1416, 1417, 1418, 58, 1421,
    1422, 1424, 1428, 1429, 1425, 1426, 1427, 1430, 1431, 1432, 1437, 1438, 227, 228, 1440, 1441,
    1442, 1443, 1444, 1448, 1453, 1449, 1450, 1451, 1454, 1455, 1458, 1465, 1459, 1460, 1461, 1464,
    1462, 1463, 1466, 1469, 1467, 1468, 1471, 227, 228, 1472, 58, 1474, 1477, 1478, 1475, 1476,
    1479, 1486, 1480, 1485, 1481, 1482, 1483, 1484, 1500, 1501, 1515, 1516, 1517, 1530, 1502, 1506,
    1509, 1514, 1503, 1504, 1505, 1507, 1508, 1510, 1511, 1512, 1513, 1518, 1522, 1526, 1519, 1520,
    1521, 1523, 1524, 1525, 1527, 1528, 1529, 1531, 1532, 1533, 1534, 1535, 1536, 1540, 1541, 1545,
    1546, 220, 1547, 230, 1548, 1549, 1550, 1551, 1552, 1553, 58, 1554, 1555, 1556, 1559, 1562,
    1563, 1564, 1565, 1566, 1560, 1561, 1569, 1570, 58, 1571, 1572, 1573, 1574, 227, 228, 1578,
    1579, 1580, 1581, 227, 228, 1582, 58, 1583, 1584, 1587, 435, 1588, 230, 1589, 1590, 1590,
    227, 228, 1592, 1595, 1596, 1597, 1598, 1593, 1594, 1619, 1620, 1623, 1624, 1625, 1628, 1631,
    1629, 1630, 1640, 1641, 1642, 1643, 1648, 1644, 227, 228, 1645, 1647, 1646, 58, 1653, 1654,
    1655, 1656, 1672, 1673, 1674, 1675, 1676, 1677, 1678, 1681, 1679, 1680, 1698, 1699, 1700, 1705,
    1706, 1701, 1702, 1703, 1704, 1707, 1708, 1709, 1710, 1715, 1716, 1717, 1718, 1719, 1721, 1722,
    1723, 1724, 1732, 1745, 1750, 1733, 1734, 220, 1735, 1736, 230, 1738, 1737, 1739, 1740, 1741,
    1744, 1742, 1743, 1746, 1747, 220, 1748, 1749, 230, 1751, 220, 1752, 230, 1755, 1758, 1759,
    1760, 1763, 1761, 1762, 1762, 227, 228, 1765, 1766, 1767, 1771, 1768, 1769, 1770, 1773, 220,
    1774, 230, 1775, 227, 228, 1776, 58, 1813, 1814, 1818, 1821, 1822, 1819, 1820, 1823, 1824,
    1826, 1827, 1828, 1832, 220, 1833, 1839, 230, 1834, 1835, 1835, 227, 228, 1836, 1837, 1838,
    1840, 1845, 1841, 1842, 1843, 1844, 1846, 1847, 1854, 1855, 1856, 1857, 1858, 1859, 1860, 1861,
    1862, 1885, 1886, 1887, 1888, 1889, 1892, 227, 228, 1890, 1891, 1893, 1894, 1895, 227, 228,
    1914, 220, 1915, 230, 1916, 1918, 1919, 1922, 1923, 1925, 1926, 1927, 1930, 220, 1931, 230,
    1928, 1929, 1932, 1933, 1935, 1936, 1937, 1938, 1941, 220, 1942, 230, 1939, 1940, 1944, 220,
    1945, 230, 1948, 220, 1949, 230, 1959, 1960, 1961, 1962, 1946, 1947, 1950, 1951, 1952, 1956,
    1953, 1955, 1954, 227, 228, 1957, 1958, 1964, 220, 1965, 230, 2001, 2002, 2003, 2004, 2005,
    2006, 227, 228, 2008, 220, 2009, 230, 2010, 58, 2016, 2017, 2054, 2055, 2056, 2058, 2060,
    2062, 2057, 227, 228, 2059, 227, 228, 2061, 227, 228, 2065, 2066, 2067, 2068, 2070, 2071,
    2074, 2072, 2073, 2073, 227, 228, 2084, 2087, 2085, 2086, 2088, 2089, 2107, 2108, 2109, 2111,
    2113, 1096, 2112, 58, 2115, 2116, 2117, 2118, 2120, 2121, 2122, 2123, 2124, 2125, 2126, 2127,
    2128, 2129, 2130, 410, 2138, 2139, 2143, 2140, 2141, 2142, 2144, 2145, 2168, 2169, 2171, 2172,
    220, 2173, 230, 2174, 58, 2175, 2179, 2182, 2187, 2176, 2177, 2178, 2180, 2181, 2183, 2184,
    2185, 2186, 2191, 2192, 2193, 2194, 2195, 2196, 2197, 2200, 2201, 220, 2210, 2211, 2212, 230,
    2213, 2214, 2215, 227, 228, 2218, 2219, 2220, 2225, 2221, 2222, 2223, 2224, 227, 228, 2246,
    2247, 220, 2248, 230, 2249, 2250, 2250, 227, 228, 2251, 58, 2257, 2262, 2258, 2259, 2260,
    2261, 2279, 2280, 2293, 2294, 2308, 2309, 2312, 2313, 2317, 2314, 2315, 2316, 2318, 227, 228,
    2341, 2342, 2343, 2344, 2345, 2346, 2350, 2347, 2348, 2349, 2351, 2352, 2353, 2355, 2356, 2357,
    2358, 2359, 2364, 2365, 2366, 2369, 2372, 2367, 2368, 2370, 2371, 2373, 2374, 2411, 2412, 2416,
    2430, 2431, 2432, 2413, 2414, 2415, 2417, 2421, 2424, 2429, 2418, 2419, 2420, 2422, 2423, 2425,
    2426, 2427, 2428, 2456, 2457, 2460, 2462, 2458, 2459, 2461, 227, 228, 2463, 2465, 2464, 2466,
    227, 228, 2484, 2485, 2486, 2487, 2489, 2490, 2492, 2493, 220, 2494, 230, 2495, 58, 2498,
    2500, 2501, 2504, 2505, 2506, 2507, 2499, 2502, 2503, 2510, 2511, 2512, 2513, 2514, 2515, 2516,
    2517, 2518, 2519, 2520, 2521, 2526, 2529, 2532, 2533, 2527, 2528, 2528, 227, 228, 2530, 2531,
    2531, 227, 228, 2534, 227, 228, 2541, 2544, 2542, 2543, 2545, 2546, 2548, 2551, 2549, 2550,
    2552, 2553, 2561, 220, 2562, 230, 2563, 58, 2565, 2566, 2567, 2568, 2569, 2570, 2571, 2572,
    2573, 2574, 2576, 2577, 2581, 2582, 2583, 2578, 2579, 2580, 2586, 2587, 2592, 2588, 2591, 2589,
    2590, 2594, 2595, 2596, 2597, 2604, 2607, 2605, 2606, 2608, 2609, 2610, 2611, 2616, 2617, 2621,
    2622, 2625, 58, 2627, 2628, 2629, 2633, 2634, 2687, 2690, 227, 228, 2691, 58, 2692, 2693,
    2712, 2715, 2713, 2714, 2716, 2719, 2722, 2717, 2718, 2720, 2721, 2723, 2724, 2725, 2728, 2729,
    2730, 2731, 2732, 2734, 2733, 227, 228, 2735, 227, 228, 2742, 2745, 2743, 2747, 2749, 2748,
    2756, 2757, 2760, 2761, 2778, 2781, 2782, 2779, 2780, 2783, 2786, 2787, 2784, 2785, 2789, 2790,
    2791, 2792, 2793, 2810, 2816, 2819, 2822, 2811, 2812, 2813, 2814, 2815, 2817, 2818, 2820, 2821,
    2823, 2824, 2825, 2842, 2843, 2844, 2849, 2850, 2851, 2855, 227, 228, 2856, 58, 2859, 2860,
    2862, 2863, 2870, 2871, 2872, 2877, 2878, 2879, 2880, 2883, 2892, 2893, 2894, 2895, 2896, 2897,
    2898, 2905, 2906, 2907, 2908, 2909, 2914, 220, 2915, 2916, 230, 2918, 2919, 2920, 2917, 2926,
    2927, 2946, 2949, 2947, 2948, 2950, 58, 2951, 2952, 2953, 227, 228, 2954, 58, 2955, 2956,
    2957, 2979, 2980, 2982, 2983, 2984, 2985, 2986, 2987, 435, 2981, 230, 2991, 435, 2992, 230,
    2993, 2994, 2994, 227, 228, 2996, 2997, 3000, 3003, 3004, 3005, 3006, 3007, 2998, 2999, 3001,
    3002, 3013, 3015, 3023, 3016, 3017, 3020, 3018, 3019, 3021, 3022, 3026, 3030, 3031, 3032, 3027,
    3028, 3029, 3034, 3035, 3036, 3037, 3040, 3041, 3038, 58, 3043, 3044, 3045, 3046, 3049, 3050,
    3047, 58, 3052, 3053, 3055, 3054, 3059, 3060, 3061, 3062,
];

pub static TERMINATORS: &[u32] = &[
    59, 60, 136, 365, 366, 438, 439, 441, 448, 449, 451, 453, 454, 455, 496, 549,
    768, 769, 771, 1187, 1223, 1372, 1375, 1384, 1387, 1452, 1456, 1557, 1575, 2188, 2189, 2496,
    2522, 2523, 2524, 2623, 2744, 2899, 2900, 2901, 2902, 2903, 3039, 3048,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABS", // [6]
    "ABSOLUTE", // [7]
    "ACCESS", // [8]
    "AccountKeywordSegment", // [9]
    "ACCOUNT", // [10]
    "ALL", // [11]
    "CATALOG", // [12]
    "DATABASE", // [13]
    "DOMAIN", // [14]
    "EXTERNAL", // [15]
    "FILE", // [16]
    "FOREIGN", // [17]
    "FUNCTION", // [18]
    "FUTURE", // [19]
    "INTEGRATION", // [20]
    "LANGUAGE", // [21]
    "MASKING", // [22]
    "MATERIALIZED", // [23]
    "PROCEDURE", // [24]
    "RESOURCE", // [25]
    "ROLE", // [26]
    "ROUTINE", // [27]
    "SCHEMA", // [28]
    "SEQUENCE", // [29]
    "STAGE", // [30]
    "STREAM", // [31]
    "TABLE", // [32]
    "TABLESPACE", // [33]
    "TASK", // [34]
    "TYPE", // [35]
    "VIEW", // [36]
    "WAREHOUSE", // [37]
    "ResourceKeywordSegment", // [38]
    "MonitorKeywordSegment", // [39]
    "MONITOR", // [40]
    "WarehouseKeywordSegment", // [41]
    "DatabaseKeywordSegment", // [42]
    "DomainKeywordSegment", // [43]
    "IntegrationKeywordSegment", // [44]
    "LanguageKeywordSegment", // [45]
    "SchemaKeywordSegment", // [46]
    "CatalogKeywordSegment", // [47]
    "RoleKeywordSegment", // [48]
    "TablespaceKeywordSegment", // [49]
    "TypeKeywordSegment", // [50]
    "ForeignKeywordSegment", // [51]
    "DATA", // [52]
    "SERVER", // [53]
    "ServerKeywordSegment", // [54]
    "DataKeywordSegment", // [55]
    "WrapperKeywordSegment", // [56]
    "WRAPPER", // [57]
    "AllKeywordSegment", // [58]
    "SchemasKeywordSegment", // [59]
    "SCHEMAS", // [60]
    "InKeywordSegment", // [61]
    "IN", // [62]
    "FutureKeywordSegment", // [63]
    "AccessSchemaObjectSegment", // [64]
    "access_schema_object", // [65]
    "AccessSchemaPluralObjectSegment", // [66]
    "FUNCTIONS", // [67]
    "PROCEDURES", // [68]
    "ROUTINES", // [69]
    "SEQUENCES", // [70]
    "STAGES", // [71]
    "STREAMS", // [72]
    "TABLES", // [73]
    "TASKS", // [74]
    "VIEWS", // [75]
    "access_schema_plural_object", // [76]
    "ObjectReferenceSegment", // [77]
    "object_reference", // [78]
    "FunctionNameSegment", // [79]
    "function_name", // [80]
    "FunctionParameterListGrammar", // [81]
    "(", // [82]
    "function_parameter_list", // [83]
    "WildcardIdentifierSegment", // [84]
    "wildcard_identifier", // [85]
    "CommaSegment", // [86]
    ",", // [87]
    "ToKeywordSegment", // [88]
    "TO", // [89]
    "FromKeywordSegment", // [90]
    "FROM", // [91]
    "LARGE", // [92]
    "LargeKeywordSegment", // [93]
    "ObjectKeywordSegment", // [94]
    "OBJECT", // [95]
    "NumericLiteralSegment", // [96]
    "numeric_literal", // [97]
    "APPLY", // [98]
    "CONNECT", // [99]
    "CREATE", // [100]
    "DELETE", // [101]
    "EXECUTE", // [102]
    "IMPORTED", // [103]
    "INSERT", // [104]
    "MANAGE", // [105]
    "MODIFY", // [106]
    "OPERATE", // [107]
    "OWNERSHIP", // [108]
    "READ", // [109]
    "REFERENCES", // [110]
    "REFERENCE_USAGE", // [111]
    "SELECT", // [112]
    "TEMP", // [113]
    "TEMPORARY", // [114]
    "TRIGGER", // [115]
    "TRUNCATE", // [116]
    "UPDATE", // [117]
    "USAGE", // [118]
    "USE", // [119]
    "USE_ANY_ROLE", // [120]
    "WRITE", // [121]
    "CreateKeywordSegment", // [122]
    "USER", // [123]
    "UserKeywordSegment", // [124]
    "ApplyKeywordSegment", // [125]
    "MaskingKeywordSegment", // [126]
    "PolicyKeywordSegment", // [127]
    "POLICY", // [128]
    "ExecuteKeywordSegment", // [129]
    "TaskKeywordSegment", // [130]
    "ManageKeywordSegment", // [131]
    "GrantsKeywordSegment", // [132]
    "GRANTS", // [133]
    "EXECUTION", // [134]
    "ExecutionKeywordSegment", // [135]
    "UsageKeywordSegment", // [136]
    "PIPE", // [137]
    "PipeKeywordSegment", // [138]
    "UseKeywordSegment", // [139]
    "ImportedKeywordSegment", // [140]
    "PrivilegesKeywordSegment", // [141]
    "PRIVILEGES", // [142]
    "ConnectKeywordSegment", // [143]
    "DeleteKeywordSegment", // [144]
    "InsertKeywordSegment", // [145]
    "ModifyKeywordSegment", // [146]
    "OperateKeywordSegment", // [147]
    "OwnershipKeywordSegment", // [148]
    "ReadKeywordSegment", // [149]
    "Reference_usageKeywordSegment", // [150]
    "ReferencesKeywordSegment", // [151]
    "SelectKeywordSegment", // [152]
    "TempKeywordSegment", // [153]
    "TemporaryKeywordSegment", // [154]
    "TriggerKeywordSegment", // [155]
    "TruncateKeywordSegment", // [156]
    "UpdateKeywordSegment", // [157]
    "Use_any_roleKeywordSegment", // [158]
    "WriteKeywordSegment", // [159]
    "BracketedColumnReferenceListGrammar", // [160]
    "AccessPermissionSegment", // [161]
    "access_permission", // [162]
    "OnKeywordSegment", // [163]
    "ON", // [164]
    "TableKeywordSegment", // [165]
    "ViewKeywordSegment", // [166]
    "StageKeywordSegment", // [167]
    "FunctionKeywordSegment", // [168]
    "ProcedureKeywordSegment", // [169]
    "RoutineKeywordSegment", // [170]
    "SequenceKeywordSegment", // [171]
    "StreamKeywordSegment", // [172]
    "MaterializedKeywordSegment", // [173]
    "ExternalKeywordSegment", // [174]
    "FileKeywordSegment", // [175]
    "FormatKeywordSegment", // [176]
    "FORMAT", // [177]
    "TablesKeywordSegment", // [178]
    "ViewsKeywordSegment", // [179]
    "StagesKeywordSegment", // [180]
    "FunctionsKeywordSegment", // [181]
    "ProceduresKeywordSegment", // [182]
    "RoutinesKeywordSegment", // [183]
    "SequencesKeywordSegment", // [184]
    "StreamsKeywordSegment", // [185]
    "TasksKeywordSegment", // [186]
    "PoliciesKeywordSegment", // [187]
    "FormatsKeywordSegment", // [188]
    "GRANT", // [189]
    "REVOKE", // [190]
    "GrantStatementSegment", // [191]
    "grant_statement", // [192]
    "RevokeStatementSegment", // [193]
    "revoke_statement", // [194]
    "GROUP", // [195]
    "GroupKeywordSegment", // [196]
    "UserReferenceSegment", // [197]
    "user_reference", // [198]
    "RoleReferenceSegment", // [199]
    "role_reference", // [200]
    "SHARE", // [201]
    "ShareKeywordSegment", // [202]
    "FunctionSegment", // [203]
    "function", // [204]
    "PublicKeywordSegment", // [205]
    "PUBLIC", // [206]
    "[", // [207]
    "ArrayAccessorSegment", // [208]
    "array_accessor", // [209]
    "ACCOUNTS", // [210]
    "ACTION", // [211]
    "ADA", // [212]
    "ADD", // [213]
    "ADMIN", // [214]
    "AFTER", // [215]
    "AGGREGATE", // [216]
    "OrderByClauseSegment", // [217]
    "ORDER", // [218]
    "orderby_clause", // [219]
    "indent", // [220]
    "Indent", // [221]
    "meta", // [222]
    "AsAliasOperatorSegment", // [223]
    "AS", // [224]
    "alias_operator", // [225]
    "SingleIdentifierGrammar", // [226]
    "SingleIdentifierListSegment", // [227]
    "identifier_list", // [228]
    "start_bracket", // [229]
    "SymbolSegment", // [230]
    "symbol", // [231]
    ")", // [232]
    "end_bracket", // [233]
    "SingleQuotedIdentifierSegment", // [234]
    "single_quote", // [235]
    "dedent", // [236]
    "Dedent", // [237]
    "ALIAS", // [238]
    "TableReferenceSegment", // [239]
    "table_reference", // [240]
    "AliasExpressionSegment", // [241]
    "alias_expression", // [242]
    "ALLOCATE", // [243]
    "ALSO", // [244]
    "ALTER", // [245]
    "CACHE", // [246]
    "CYCLE", // [247]
    "INCREMENT", // [248]
    "MAXVALUE", // [249]
    "MINVALUE", // [250]
    "NO", // [251]
    "NOCACHE", // [252]
    "NOCYCLE", // [253]
    "NOORDER", // [254]
    "IncrementKeywordSegment", // [255]
    "ByKeywordSegment", // [256]
    "BY", // [257]
    "SequenceMinValueGrammar", // [258]
    "SequenceMaxValueGrammar", // [259]
    "CacheKeywordSegment", // [260]
    "NocacheKeywordSegment", // [261]
    "CycleKeywordSegment", // [262]
    "NocycleKeywordSegment", // [263]
    "OrderNoOrderGrammar", // [264]
    "AlterKeywordSegment", // [265]
    "SequenceReferenceSegment", // [266]
    "sequence_reference", // [267]
    "AlterSequenceOptionsSegment", // [268]
    "alter_sequence_options_segment", // [269]
    "DROP", // [270]
    "DropKeywordSegment", // [271]
    "ColumnKeywordSegment", // [272]
    "COLUMN", // [273]
    "IfExistsGrammar", // [274]
    "IF", // [275]
    "ParameterNameSegment", // [276]
    "EqualsSegment", // [277]
    "=", // [278]
    "comparison_operator", // [279]
    "LiteralGrammar", // [280]
    "NakedIdentifierSegment", // [281]
    "AddKeywordSegment", // [282]
    "ColumnDefinitionSegment", // [283]
    "column_definition", // [284]
    "FIRST", // [285]
    "FirstKeywordSegment", // [286]
    "AfterKeywordSegment", // [287]
    "ColumnReferenceSegment", // [288]
    "column_reference", // [289]
    "AlterTableDropColumnGrammar", // [290]
    "RENAME", // [291]
    "RenameKeywordSegment", // [292]
    "AsKeywordSegment", // [293]
    "AlterTableOptionsGrammar", // [294]
    "ALWAYS", // [295]
    "&", // [296]
    "ampersand", // [297]
    "ANALYSE", // [298]
    "ANALYZE", // [299]
    "AND", // [300]
    "binary_operator", // [301]
    "BinaryOperatorSegment", // [302]
    "ANY", // [303]
    "ARE", // [304]
    "%", // [305]
    "*", // [306]
    "+", // [307]
    "-", // [308]
    "/", // [309]
    "<", // [310]
    ">", // [311]
    "^", // [312]
    "|", // [313]
    "PlusSegment", // [314]
    "MinusSegment", // [315]
    "DivideSegment", // [316]
    "MultiplySegment", // [317]
    "ModuloSegment", // [318]
    "BitwiseAndSegment", // [319]
    "BitwiseOrSegment", // [320]
    "BitwiseXorSegment", // [321]
    "BitwiseLShiftSegment", // [322]
    "BitwiseRShiftSegment", // [323]
    "ExpressionSegment", // [324]
    "expression", // [325]
    "SliceSegment", // [326]
    ":", // [327]
    "start_square_bracket", // [328]
    "]", // [329]
    "end_square_bracket", // [330]
    "ARRAY", // [331]
    "BaseExpressionElementGrammar", // [332]
    "ASC", // [333]
    "ASENSITIVE", // [334]
    "ASSERTION", // [335]
    "ASSIGNMENT", // [336]
    "ASYMMETRIC", // [337]
    "AT", // [338]
    "ATOMIC", // [339]
    "ATTRIBUTE", // [340]
    "ATTRIBUTES", // [341]
    "AUDIT", // [342]
    "AUTHORIZATION", // [343]
    "AUTO_INCREMENT", // [344]
    "Auto_incrementKeywordSegment", // [345]
    "AVG", // [346]
    "AVG_ROW_LENGTH", // [347]
    "BACKUP", // [348]
    "BACKWARD", // [349]
    "CURRENT_DATE", // [350]
    "CURRENT_TIME", // [351]
    "CURRENT_TIMESTAMP", // [352]
    "LAST_90_DAYS", // [353]
    "LAST_FISCAL_QUARTER", // [354]
    "LAST_FISCAL_YEAR", // [355]
    "LAST_MONTH", // [356]
    "LAST_QUARTER", // [357]
    "LAST_WEEK", // [358]
    "LAST_YEAR", // [359]
    "NEXT_90_DAYS", // [360]
    "NEXT_FISCAL_QUARTER", // [361]
    "NEXT_FISCAL_YEAR", // [362]
    "NEXT_MONTH", // [363]
    "NEXT_QUARTER", // [364]
    "NEXT_WEEK", // [365]
    "NEXT_YEAR", // [366]
    "THIS_FISCAL_QUARTER", // [367]
    "THIS_FISCAL_YEAR", // [368]
    "THIS_MONTH", // [369]
    "THIS_QUARTER", // [370]
    "THIS_WEEK", // [371]
    "THIS_YEAR", // [372]
    "TODAY", // [373]
    "TOMORROW", // [374]
    "YESTERDAY", // [375]
    "bare_function", // [376]
    "CodeSegment", // [377]
    "BareFunctionSegment", // [378]
    "IntervalExpressionSegment", // [379]
    "INTERVAL", // [380]
    "interval_expression", // [381]
    "DatatypeSegment", // [382]
    "data_type", // [383]
    "file", // [384]
    "BaseFileSegment", // [385]
    "BaseSegment", // [386]
    "BEFORE", // [387]
    "BEGIN", // [388]
    "BERNOULLI", // [389]
    "BETWEEN", // [390]
    "BIGINT", // [391]
    "BINARY", // [392]
    "!", // [393]
    "IS", // [394]
    "OR", // [395]
    "like_operator", // [396]
    "ArithmeticBinaryOperatorGrammar", // [397]
    "StringBinaryOperatorGrammar", // [398]
    "BooleanBinaryOperatorGrammar", // [399]
    "ComparisonOperatorGrammar", // [400]
    "BINDING", // [401]
    "BIT", // [402]
    "BIT_LENGTH", // [403]
    "BITVAR", // [404]
    "AmpersandSegment", // [405]
    "RawLessThanSegment", // [406]
    "PipeSegment", // [407]
    "RawGreaterThanSegment", // [408]
    "BLOB", // [409]
    "BOOL", // [410]
    "AndOperatorGrammar", // [411]
    "OrOperatorGrammar", // [412]
    "BOOLEAN", // [413]
    "FALSE", // [414]
    "TRUE", // [415]
    "TrueSegment", // [416]
    "FalseSegment", // [417]
    "BOTH", // [418]
    "bracketed", // [419]
    "BracketedSegment", // [420]
    "UnorderedSetExpressionSegment", // [421]
    "VALUE", // [422]
    "VALUES", // [423]
    "set_expression", // [424]
    "BREADTH", // [425]
    "BREAK", // [426]
    "BROWSE", // [427]
    "BULK", // [428]
    "CTEColumnList", // [429]
    "cte_column_list", // [430]
    "SelectableGrammar", // [431]
    "WITH", // [432]
    "CALL", // [433]
    "CALLED", // [434]
    "CARDINALITY", // [435]
    "CASCADE", // [436]
    "CASCADED", // [437]
    "CASE", // [438]
    "CaseKeywordSegment", // [439]
    "implicit_indent", // [440]
    "ImplicitIndent", // [441]
    "WHEN", // [442]
    "WhenClauseSegment", // [443]
    "when_clause", // [444]
    "ElseKeywordSegment", // [445]
    "ELSE", // [446]
    "EndKeywordSegment", // [447]
    "END", // [448]
    "ElseClauseSegment", // [449]
    "else_clause", // [450]
    "BinaryOperatorGrammar", // [451]
    "CAST", // [452]
    "::", // [453]
    "casting_operator", // [454]
    "CATALOG_NAME", // [455]
    "CEIL", // [456]
    "CEILING", // [457]
    "CHAIN", // [458]
    "CHANGE", // [459]
    "CHAR", // [460]
    "CHAR_LENGTH", // [461]
    "CHARACTER", // [462]
    "CHARACTER_LENGTH", // [463]
    "CHARACTER_SET_CATALOG", // [464]
    "CHARACTER_SET_NAME", // [465]
    "CHARACTER_SET_SCHEMA", // [466]
    "CHARACTERISTICS", // [467]
    "CHARACTERS", // [468]
    "CHECK", // [469]
    "CHECKED", // [470]
    "CHECKPOINT", // [471]
    "CHECKSUM", // [472]
    "CLASS", // [473]
    "CLASS_ORIGIN", // [474]
    "CLOB", // [475]
    "CLOSE", // [476]
    "CLUSTER", // [477]
    "CLUSTERED", // [478]
    "COALESCE", // [479]
    "COBOL", // [480]
    "COLLATE", // [481]
    "COLLATION", // [482]
    "QuotedLiteralSegment", // [483]
    "ObjectReferenceDelimiterGrammar", // [484]
    ".", // [485]
    "ObjectReferenceTerminatorGrammar", // [486]
    "COLLATION_CATALOG", // [487]
    "COLLATION_NAME", // [488]
    "COLLATION_SCHEMA", // [489]
    "COLLECT", // [490]
    "colon_delimiter", // [491]
    "colon_prefix", // [492]
    "colon", // [493]
    "ShorthandCastSegment", // [494]
    "cast_expression", // [495]
    "CONSTRAINT", // [496]
    "ConstraintKeywordSegment", // [497]
    "NOT", // [498]
    "NULL", // [499]
    "NotKeywordSegment", // [500]
    "NullKeywordSegment", // [501]
    "CheckKeywordSegment", // [502]
    "DEFAULT", // [503]
    "DefaultKeywordSegment", // [504]
    "ColumnConstraintDefaultGrammar", // [505]
    "PRIMARY", // [506]
    "PrimaryKeyGrammar", // [507]
    "NotEnforcedGrammar", // [508]
    "UniqueKeyGrammar", // [509]
    "UNIQUE", // [510]
    "AutoIncrementGrammar", // [511]
    "ReferenceDefinitionGrammar", // [512]
    "CommentClauseSegment", // [513]
    "COMMENT", // [514]
    "comment_clause", // [515]
    "CollateKeywordSegment", // [516]
    "CollationReferenceSegment", // [517]
    "collation_reference", // [518]
    "ColumnGeneratedGrammar", // [519]
    "ColumnConstraintSegment", // [520]
    "column_constraint_segment", // [521]
    "COLUMN_NAME", // [522]
    "ColumnsExpressionNameGrammar", // [523]
    "COLUMNS", // [524]
    "comma", // [525]
    "COMMAND_FUNCTION", // [526]
    "COMMAND_FUNCTION_CODE", // [527]
    "CommentKeywordSegment", // [528]
    "comment", // [529]
    "CommentSegment", // [530]
    "COMMIT", // [531]
    "COMMITTED", // [532]
    "GreaterThanSegment", // [533]
    "LessThanSegment", // [534]
    "GreaterThanOrEqualToSegment", // [535]
    "LessThanOrEqualToSegment", // [536]
    "NotEqualToSegment", // [537]
    "LikeOperatorSegment", // [538]
    "IsDistinctFromGrammar", // [539]
    "ComparisonOperatorSegment", // [540]
    "COMPLETION", // [541]
    "CompositeBinaryOperatorSegment", // [542]
    "CompositeComparisonOperatorSegment", // [543]
    "COMPRESS", // [544]
    "COMPUTE", // [545]
    "CONDITION", // [546]
    "CONDITION_NUMBER", // [547]
    "CrossKeywordSegment", // [548]
    "CROSS", // [549]
    "JoinTypeKeywordsGrammar", // [550]
    "FULL", // [551]
    "INNER", // [552]
    "LEFT", // [553]
    "RIGHT", // [554]
    "ConditionalCrossJoinKeywordsGrammar", // [555]
    "NonStandardJoinTypeKeywordsGrammar", // [556]
    "CONNECTION", // [557]
    "CONNECTION_NAME", // [558]
    "CONSTRAINT_CATALOG", // [559]
    "CONSTRAINT_NAME", // [560]
    "CONSTRAINT_SCHEMA", // [561]
    "CONSTRAINTS", // [562]
    "CONSTRUCTOR", // [563]
    "CONTAINS", // [564]
    "CONTAINSTABLE", // [565]
    "CONTINUE", // [566]
    "CONVERSION", // [567]
    "CONVERT", // [568]
    "COPY", // [569]
    "CORR", // [570]
    "CORRESPONDING", // [571]
    "COUNT", // [572]
    "COVAR_POP", // [573]
    "COVAR_SAMP", // [574]
    "CastKeywordSegment", // [575]
    "WithKeywordSegment", // [576]
    "SpecificKeywordSegment", // [577]
    "SPECIFIC", // [578]
    "INSTANCE", // [579]
    "METHOD", // [580]
    "STATIC", // [581]
    "InstanceKeywordSegment", // [582]
    "StaticKeywordSegment", // [583]
    "ConstructorKeywordSegment", // [584]
    "MethodKeywordSegment", // [585]
    "FOR", // [586]
    "ForKeywordSegment", // [587]
    "AssignmentKeywordSegment", // [588]
    "IfNotExistsGrammar", // [589]
    "DatabaseReferenceSegment", // [590]
    "database_reference", // [591]
    "OrReplaceGrammar", // [592]
    "TemporaryGrammar", // [593]
    "RETURNS", // [594]
    "ReturnsKeywordSegment", // [595]
    "FunctionDefinitionGrammar", // [596]
    "function_definition", // [597]
    "UniqueKeywordSegment", // [598]
    "IndexKeywordSegment", // [599]
    "INDEX", // [600]
    "IndexReferenceSegment", // [601]
    "index_reference", // [602]
    "IndexColumnDefinitionSegment", // [603]
    "index_column_definition", // [604]
    "ModelKeywordSegment", // [605]
    "MODEL", // [606]
    "OPTIONS", // [607]
    "OptionsKeywordSegment", // [608]
    "SchemaReferenceSegment", // [609]
    "schema_reference", // [610]
    "START", // [611]
    "StartKeywordSegment", // [612]
    "CreateSequenceOptionsSegment", // [613]
    "create_sequence_options_segment", // [614]
    "TemporaryTransientGrammar", // [615]
    "TRANSIENT", // [616]
    "LIKE", // [617]
    "TableConstraintSegment", // [618]
    "table_constraint", // [619]
    "LikeKeywordSegment", // [620]
    "TableEndClauseSegment", // [621]
    "table_end_clause_segment", // [622]
    "TriggerReferenceSegment", // [623]
    "trigger_reference", // [624]
    "INSTEAD", // [625]
    "BeforeKeywordSegment", // [626]
    "InsteadKeywordSegment", // [627]
    "OfKeywordSegment", // [628]
    "OF", // [629]
    "OrKeywordSegment", // [630]
    "DEFERRABLE", // [631]
    "INITIALLY", // [632]
    "REFERENCING", // [633]
    "ReferencingKeywordSegment", // [634]
    "OldKeywordSegment", // [635]
    "OLD", // [636]
    "RowKeywordSegment", // [637]
    "ROW", // [638]
    "NewKeywordSegment", // [639]
    "NEW", // [640]
    "DeferrableKeywordSegment", // [641]
    "InitiallyKeywordSegment", // [642]
    "ImmediateKeywordSegment", // [643]
    "IMMEDIATE", // [644]
    "DeferredKeywordSegment", // [645]
    "DEFERRED", // [646]
    "EachKeywordSegment", // [647]
    "EACH", // [648]
    "STATEMENT", // [649]
    "StatementKeywordSegment", // [650]
    "WhenKeywordSegment", // [651]
    "FunctionNameIdentifierSegment", // [652]
    "FunctionContentsSegment", // [653]
    "function_contents", // [654]
    "WithNoSchemaBindingClauseSegment", // [655]
    "with_no_schema_binding_clause", // [656]
    "CREATEDB", // [657]
    "CREATEROLE", // [658]
    "CREATEUSER", // [659]
    "CSV", // [660]
    "CUBE", // [661]
    "function_name_identifier", // [662]
    "ROLLUP", // [663]
    "CubeFunctionNameSegment", // [664]
    "RollupFunctionNameSegment", // [665]
    "GroupingExpressionList", // [666]
    "grouping_expression_list", // [667]
    "CUME_DIST", // [668]
    "CURRENT", // [669]
    "CURRENT_DEFAULT_TRANSFORM_GROUP", // [670]
    "CURRENT_PATH", // [671]
    "CURRENT_ROLE", // [672]
    "CURRENT_TRANSFORM_GROUP_FOR_TYPE", // [673]
    "CURRENT_USER", // [674]
    "CURSOR", // [675]
    "CURSOR_NAME", // [676]
    "DATABASES", // [677]
    "data_type_identifier", // [678]
    "TimeWithTZGrammar", // [679]
    "TIME", // [680]
    "TIMESTAMP", // [681]
    "DOUBLE", // [682]
    "DoubleKeywordSegment", // [683]
    "PrecisionKeywordSegment", // [684]
    "PRECISION", // [685]
    "CharacterKeywordSegment", // [686]
    "BinaryKeywordSegment", // [687]
    "VARYING", // [688]
    "VaryingKeywordSegment", // [689]
    "DotSegment", // [690]
    "DatatypeIdentifierSegment", // [691]
    "BracketedArguments", // [692]
    "bracketed_arguments", // [693]
    "UnsignedKeywordSegment", // [694]
    "UNSIGNED", // [695]
    "CharCharacterSetGrammar", // [696]
    "ArrayTypeSegment", // [697]
    "array_type", // [698]
    "DATE", // [699]
    "LAST_N_DAYS", // [700]
    "LAST_N_FISCAL_QUARTERS", // [701]
    "LAST_N_FISCAL_YEARS", // [702]
    "LAST_N_MONTHS", // [703]
    "LAST_N_QUARTERS", // [704]
    "LAST_N_WEEKS", // [705]
    "LAST_N_YEARS", // [706]
    "NEXT_N_DAYS", // [707]
    "NEXT_N_FISCAL_QUARTERS", // [708]
    "NEXT_N_FISCAL_YEARS", // [709]
    "NEXT_N_MONTHS", // [710]
    "NEXT_N_QUARTERS", // [711]
    "NEXT_N_WEEKS", // [712]
    "NEXT_N_YEARS", // [713]
    "Last_n_daysKeywordSegment", // [714]
    "Last_n_fiscal_quartersKeywordSegment", // [715]
    "Last_n_fiscal_yearsKeywordSegment", // [716]
    "Last_n_monthsKeywordSegment", // [717]
    "Last_n_quartersKeywordSegment", // [718]
    "Last_n_weeksKeywordSegment", // [719]
    "Last_n_yearsKeywordSegment", // [720]
    "Next_n_daysKeywordSegment", // [721]
    "Next_n_fiscal_quartersKeywordSegment", // [722]
    "Next_n_fiscal_yearsKeywordSegment", // [723]
    "Next_n_monthsKeywordSegment", // [724]
    "Next_n_quartersKeywordSegment", // [725]
    "Next_n_weeksKeywordSegment", // [726]
    "Next_n_yearsKeywordSegment", // [727]
    "ColonSegment", // [728]
    "DATEADD", // [729]
    "DatePartFunctionName", // [730]
    "DatetimeUnitSegment", // [731]
    "DAY", // [732]
    "DAYOFYEAR", // [733]
    "HOUR", // [734]
    "MILLISECOND", // [735]
    "MINUTE", // [736]
    "MONTH", // [737]
    "QUARTER", // [738]
    "SECOND", // [739]
    "WEEK", // [740]
    "WEEKDAY", // [741]
    "YEAR", // [742]
    "FunctionContentsGrammar", // [743]
    "date_literal", // [744]
    "datetime_literal", // [745]
    "literal", // [746]
    "LiteralSegment", // [747]
    "DateKeywordSegment", // [748]
    "TimeKeywordSegment", // [749]
    "TimestampKeywordSegment", // [750]
    "IntervalKeywordSegment", // [751]
    "date_constructor_literal", // [752]
    "DATETIME", // [753]
    "date_part", // [754]
    "DATETIME_INTERVAL_CODE", // [755]
    "DATETIME_INTERVAL_PRECISION", // [756]
    "DAY_HOUR", // [757]
    "DAY_MICROSECOND", // [758]
    "DAY_MINUTE", // [759]
    "DAY_SECOND", // [760]
    "DAYOFMONTH", // [761]
    "DAYOFWEEK", // [762]
    "DAYS", // [763]
    "DBCC", // [764]
    "DEALLOCATE", // [765]
    "DEC", // [766]
    "DECIMAL", // [767]
    "DECLARE", // [768]
    "ValuesKeywordSegment", // [769]
    "DEFAULTS", // [770]
    "DEFINED", // [771]
    "DEFINER", // [772]
    "DEGREE", // [773]
    "DELAY_KEY_WRITE", // [774]
    "DELAYED", // [775]
    "FromClauseSegment", // [776]
    "from_clause", // [777]
    "WhereClauseSegment", // [778]
    "WHERE", // [779]
    "where_clause", // [780]
    "SemicolonSegment", // [781]
    ";", // [782]
    "DELIMITER", // [783]
    "DELIMITERS", // [784]
    "DENSE_RANK", // [785]
    "DENY", // [786]
    "DEPTH", // [787]
    "DEREF", // [788]
    "DERIVED", // [789]
    "DESC", // [790]
    "DESCRIBE", // [791]
    "DescribeKeywordSegment", // [792]
    "DESCRIPTOR", // [793]
    "DESTROY", // [794]
    "DESTRUCTOR", // [795]
    "DETERMINISTIC", // [796]
    "DIAGNOSTICS", // [797]
    "DICTIONARY", // [798]
    "DISABLE", // [799]
    "DISCONNECT", // [800]
    "DISK", // [801]
    "DISPATCH", // [802]
    "DISTINCT", // [803]
    "DISTINCTROW", // [804]
    "DISTRIBUTED", // [805]
    "DIV", // [806]
    "DO", // [807]
    "dot", // [808]
    "RESTRICT", // [809]
    "RestrictKeywordSegment", // [810]
    "CascadeKeywordSegment", // [811]
    "DropBehaviorGrammar", // [812]
    "DUMMY", // [813]
    "DUMP", // [814]
    "DYNAMIC", // [815]
    "DYNAMIC_FUNCTION", // [816]
    "DYNAMIC_FUNCTION_CODE", // [817]
    "ELEMENT", // [818]
    "ELSEIF", // [819]
    "StructTypeSegment", // [820]
    "struct_type", // [821]
    "EmptyStructLiteralBracketsSegment", // [822]
    "struct_literal", // [823]
    "ENABLE", // [824]
    "ENCLOSED", // [825]
    "ENCODING", // [826]
    "ENCRYPTED", // [827]
    "END-EXEC", // [828]
    "}", // [829]
    "end_curly_bracket", // [830]
    "ENUM", // [831]
    "EQUALS", // [832]
    "RawEqualsSegment", // [833]
    "ERRLVL", // [834]
    "ESCAPE", // [835]
    "ESCAPED", // [836]
    "EVERY", // [837]
    "EXCEPT", // [838]
    "EXCEPTION", // [839]
    "EXCLUDE", // [840]
    "EXCLUDING", // [841]
    "EXCLUSIVE", // [842]
    "EXEC", // [843]
    "EXISTING", // [844]
    "EXISTS", // [845]
    "EXIT", // [846]
    "EXP", // [847]
    "EXPLAIN", // [848]
    "ExplainKeywordSegment", // [849]
    "InsertStatementSegment", // [850]
    "insert_statement", // [851]
    "UpdateStatementSegment", // [852]
    "update_statement", // [853]
    "DeleteStatementSegment", // [854]
    "delete_statement", // [855]
    "Expression_A_Grammar", // [856]
    "Tail_Recurse_Expression_A_Grammar", // [857]
    "LikeExpressionGrammar", // [858]
    "ILIKE", // [859]
    "RLIKE", // [860]
    "InOperatorGrammar", // [861]
    "QuantifiedComparisonOperatorGrammar", // [862]
    "IsKeywordSegment", // [863]
    "IsClauseGrammar", // [864]
    "IsNullGrammar", // [865]
    "NotNullGrammar", // [866]
    "CollateGrammar", // [867]
    "BetweenKeywordSegment", // [868]
    "Expression_B_Grammar", // [869]
    "AndKeywordSegment", // [870]
    "PatternMatchingGrammar", // [871]
    "PRIOR", // [872]
    "~", // [873]
    "SignedSegmentGrammar", // [874]
    "QualifiedNumericLiteralSegment", // [875]
    "TildeSegment", // [876]
    "NotOperatorGrammar", // [877]
    "PriorKeywordSegment", // [878]
    "Tail_Recurse_Expression_B_Grammar", // [879]
    "ExistsKeywordSegment", // [880]
    "Expression_D_Grammar", // [881]
    "CaseExpressionSegment", // [882]
    "case_expression", // [883]
    "TimeZoneGrammar", // [884]
    "time_zone_grammar", // [885]
    "DateLiteralNSegment", // [886]
    "date_n_literal", // [887]
    "LateralColumnAliasExpressionGrammar", // [888]
    "LocalAliasSegment", // [889]
    "local_alias_segment", // [890]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [891]
    "SingleIdentifierWildcardSegment", // [892]
    "MapTypeSegment", // [893]
    "map_type", // [894]
    "BooleanLiteralGrammar", // [895]
    "NullLiteralSegment", // [896]
    "DateTimeLiteralGrammar", // [897]
    "ListComprehensionGrammar", // [898]
    "AccessorGrammar", // [899]
    "SelectStatementSegment", // [900]
    "select_statement", // [901]
    "TypedStructLiteralSegment", // [902]
    "typed_struct_literal", // [903]
    "ArrayExpressionSegment", // [904]
    "array_expression", // [905]
    "OverlapsClauseSegment", // [906]
    "OVERLAPS", // [907]
    "overlaps_clause", // [908]
    "EXTENSION", // [909]
    "EXTRACT", // [910]
    "boolean_literal", // [911]
    "LiteralKeywordSegment", // [912]
    "FETCH", // [913]
    "FetchKeywordSegment", // [914]
    "NEXT", // [915]
    "NextKeywordSegment", // [916]
    "ROWS", // [917]
    "RowsKeywordSegment", // [918]
    "ONLY", // [919]
    "OnlyKeywordSegment", // [920]
    "TiesKeywordSegment", // [921]
    "TIES", // [922]
    "FIELDS", // [923]
    "DelimiterGrammar", // [924]
    "StatementSegment", // [925]
    "statement", // [926]
    "FILLFACTOR", // [927]
    "FILTER", // [928]
    "FilterKeywordSegment", // [929]
    "WhereKeywordSegment", // [930]
    "FINAL", // [931]
    "FLOAT4", // [932]
    "FLOAT8", // [933]
    "FLOAT", // [934]
    "FLOOR", // [935]
    "FLUSH", // [936]
    "FOLLOWING", // [937]
    "FORCE", // [938]
    "KeyKeywordSegment", // [939]
    "KEY", // [940]
    "FORTRAN", // [941]
    "FORWARD", // [942]
    "FOUND", // [943]
    "RANGE", // [944]
    "FrameClauseUnitGrammar", // [945]
    "CurrentKeywordSegment", // [946]
    "UnboundedKeywordSegment", // [947]
    "UNBOUNDED", // [948]
    "PRECEDING", // [949]
    "PrecedingKeywordSegment", // [950]
    "FollowingKeywordSegment", // [951]
    "RangeKeywordSegment", // [952]
    "FREE", // [953]
    "FREETEXT", // [954]
    "FREETEXTTABLE", // [955]
    "FREEZE", // [956]
    "FromExpressionSegment", // [957]
    "from_expression", // [958]
    "HAVING", // [959]
    "INTERSECT", // [960]
    "LIMIT", // [961]
    "MINUS", // [962]
    "OFFSET", // [963]
    "QUALIFY", // [964]
    "UNION", // [965]
    "WINDOW", // [966]
    "LimitClauseSegment", // [967]
    "limit_clause", // [968]
    "OrderKeywordSegment", // [969]
    "HavingKeywordSegment", // [970]
    "QualifyKeywordSegment", // [971]
    "WindowKeywordSegment", // [972]
    "SetOperatorSegment", // [973]
    "set_operator", // [974]
    "WithDataClauseSegment", // [975]
    "with_data_clause", // [976]
    "OffsetKeywordSegment", // [977]
    "PreTableFunctionKeywordsGrammar", // [978]
    "TableExpressionSegment", // [979]
    "table_expression", // [980]
    "TemporalQuerySegment", // [981]
    "temporal_query", // [982]
    "FromClauseTerminatorGrammar", // [983]
    "SamplingExpressionSegment", // [984]
    "TABLESAMPLE", // [985]
    "sample_expression", // [986]
    "JoinLikeClauseGrammar", // [987]
    "JoinClauseSegment", // [988]
    "join_clause", // [989]
    "PostTableExpressionGrammar", // [990]
    "MLTableExpressionSegment", // [991]
    "ML", // [992]
    "ml_table_expression", // [993]
    "FromExpressionElementSegment", // [994]
    "from_expression_element", // [995]
    "indented_joins", // [996]
    "FULLTEXT", // [997]
    "LEADING", // [998]
    "TRAILING", // [999]
    "TrimParametersGrammar", // [1000]
    "DistinctKeywordSegment", // [1001]
    "StarSegment", // [1002]
    "FunctionContentsExpressionGrammar", // [1003]
    "AggregateOrderByClause", // [1004]
    "aggregate_order_by", // [1005]
    "SEPARATOR", // [1006]
    "SeparatorKeywordSegment", // [1007]
    "IgnoreRespectNullsGrammar", // [1008]
    "IGNORE", // [1009]
    "RESPECT", // [1010]
    "EmptyStructLiteralSegment", // [1011]
    "WordSegment", // [1012]
    "double_quote", // [1013]
    "QuotedIdentifierSegment", // [1014]
    "AnyKeywordSegment", // [1015]
    "FunctionParameterGrammar", // [1016]
    "DatePartFunctionNameSegment", // [1017]
    "DateTimeFunctionContentsSegment", // [1018]
    "ColumnsExpressionGrammar", // [1019]
    "ColumnsExpressionFunctionNameSegment", // [1020]
    "ValuesClauseSegment", // [1021]
    "values_clause", // [1022]
    "PostFunctionGrammar", // [1023]
    "FUSION", // [1024]
    "G", // [1025]
    "GENERAL", // [1026]
    "GENERATED", // [1027]
    "GET", // [1028]
    "glob_operator", // [1029]
    "GLOBAL", // [1030]
    "GO", // [1031]
    "GOTO", // [1032]
    "GrantKeywordSegment", // [1033]
    "AccessPermissionsSegment", // [1034]
    "access_permissions", // [1035]
    "AccessObjectSegment", // [1036]
    "access_object", // [1037]
    "AccessTargetSegment", // [1038]
    "access_target", // [1039]
    "OptionKeywordSegment", // [1040]
    "OPTION", // [1041]
    "AdminKeywordSegment", // [1042]
    "CopyKeywordSegment", // [1043]
    "GRANTED", // [1044]
    "GrantedKeywordSegment", // [1045]
    "Current_userKeywordSegment", // [1046]
    "Session_userKeywordSegment", // [1047]
    "SESSION_USER", // [1048]
    "GREATEST", // [1049]
    "GroupingSetsClauseSegment", // [1050]
    "GROUPING", // [1051]
    "grouping_sets_clause", // [1052]
    "CubeRollupClauseSegment", // [1053]
    "cube_rollup_clause", // [1054]
    "GroupByClauseTerminatorGrammar", // [1055]
    "GroupingKeywordSegment", // [1056]
    "SetsKeywordSegment", // [1057]
    "SETS", // [1058]
    "HANDLER", // [1059]
    "HEADER", // [1060]
    "HEAP", // [1061]
    "HIERARCHY", // [1062]
    "HIGH_PRIORITY", // [1063]
    "HOLD", // [1064]
    "HOLDLOCK", // [1065]
    "HOST", // [1066]
    "HOSTS", // [1067]
    "HOUR_MICROSECOND", // [1068]
    "HOUR_MINUTE", // [1069]
    "HOUR_SECOND", // [1070]
    "IDENTIFIED", // [1071]
    "identifier", // [1072]
    "IdentifierSegment", // [1073]
    "IDENTITY", // [1074]
    "IDENTITY_INSERT", // [1075]
    "IDENTITYCOL", // [1076]
    "IfKeywordSegment", // [1077]
    "IgnoreKeywordSegment", // [1078]
    "RespectKeywordSegment", // [1079]
    "NullsKeywordSegment", // [1080]
    "NULLS", // [1081]
    "IMMUTABLE", // [1082]
    "IMPLEMENTATION", // [1083]
    "IMPLICIT", // [1084]
    "INCLUDE", // [1085]
    "INCLUDING", // [1086]
    "AscKeywordSegment", // [1087]
    "DescKeywordSegment", // [1088]
    "INDICATOR", // [1089]
    "INFILE", // [1090]
    "INFIX", // [1091]
    "INHERIT", // [1092]
    "INHERITS", // [1093]
    "INITIAL", // [1094]
    "INITIALIZE", // [1095]
    "INOUT", // [1096]
    "INPUT", // [1097]
    "INSENSITIVE", // [1098]
    "OverwriteKeywordSegment", // [1099]
    "OVERWRITE", // [1100]
    "IntoKeywordSegment", // [1101]
    "INTO", // [1102]
    "DefaultValuesGrammar", // [1103]
    "INSERT_ID", // [1104]
    "INSTANTIABLE", // [1105]
    "INT1", // [1106]
    "INT2", // [1107]
    "INT3", // [1108]
    "INT4", // [1109]
    "INT8", // [1110]
    "INT", // [1111]
    "INTEGER", // [1112]
    "INTEGRATIONS", // [1113]
    "INTERSECTION", // [1114]
    "INVOKER", // [1115]
    "NanLiteralSegment", // [1116]
    "NAN", // [1117]
    "UnknownLiteralSegment", // [1118]
    "NormalizedGrammar", // [1119]
    "ISAM", // [1120]
    "ISNULL", // [1121]
    "ISOLATION", // [1122]
    "ITERATE", // [1123]
    "ConditionalJoinKeywordsGrammar", // [1124]
    "JoinKeywordsGrammar", // [1125]
    "JOIN", // [1126]
    "NestedJoinGrammar", // [1127]
    "indented_using_on", // [1128]
    "MatchConditionSegment", // [1129]
    "match_condition", // [1130]
    "USING", // [1131]
    "JoinOnConditionSegment", // [1132]
    "join_on_condition", // [1133]
    "JoinUsingConditionGrammar", // [1134]
    "UnconditionalJoinKeywordsGrammar", // [1135]
    "ExtendedNaturalJoinKeywordsGrammar", // [1136]
    "JoinKeywordSegment", // [1137]
    "indented_on_contents", // [1138]
    "InnerKeywordSegment", // [1139]
    "FullKeywordSegment", // [1140]
    "LeftKeywordSegment", // [1141]
    "RightKeywordSegment", // [1142]
    "OuterKeywordSegment", // [1143]
    "OUTER", // [1144]
    "UsingKeywordSegment", // [1145]
    "JSON", // [1146]
    "K", // [1147]
    "KEY_MEMBER", // [1148]
    "KEY_TYPE", // [1149]
    "KEYS", // [1150]
    "KILL", // [1151]
    "LANCOMPILER", // [1152]
    "LAST", // [1153]
    "LAST_INSERT_ID", // [1154]
    "LATERAL", // [1155]
    "LEAST", // [1156]
    "LEAVE", // [1157]
    "LENGTH", // [1158]
    "LESS", // [1159]
    "LEVEL", // [1160]
    "LikeGrammar", // [1161]
    "EscapeKeywordSegment", // [1162]
    "RlikeKeywordSegment", // [1163]
    "IlikeKeywordSegment", // [1164]
    "LimitKeywordSegment", // [1165]
    "LINENO", // [1166]
    "LINES", // [1167]
    "LISTEN", // [1168]
    "ArrayLiteralSegment", // [1169]
    "array_literal", // [1170]
    "TypedArrayLiteralSegment", // [1171]
    "typed_array_literal", // [1172]
    "ObjectLiteralSegment", // [1173]
    "{", // [1174]
    "object_literal", // [1175]
    "LN", // [1176]
    "LOAD", // [1177]
    "LOCAL", // [1178]
    "LOCALTIME", // [1179]
    "LOCALTIMESTAMP", // [1180]
    "LOCATION", // [1181]
    "LOCATOR", // [1182]
    "LOCK", // [1183]
    "LOCKS", // [1184]
    "LOGIN", // [1185]
    "LOGS", // [1186]
    "LONG", // [1187]
    "LONGBLOB", // [1188]
    "LONGTEXT", // [1189]
    "LOOP", // [1190]
    "LOW_PRIORITY", // [1191]
    "LOWER", // [1192]
    "M", // [1193]
    "MlKeywordSegment", // [1194]
    "MAP", // [1195]
    "MATCH", // [1196]
    "MATCHED", // [1197]
    "MAX", // [1198]
    "MAX_ROWS", // [1199]
    "MAXEXTENTS", // [1200]
    "MEDIUMBLOB", // [1201]
    "MEDIUMINT", // [1202]
    "MEDIUMTEXT", // [1203]
    "MEMBER", // [1204]
    "MERGE", // [1205]
    "MergeKeywordSegment", // [1206]
    "MergeMatchedClauseSegment", // [1207]
    "merge_when_matched_clause", // [1208]
    "MergeNotMatchedClauseSegment", // [1209]
    "merge_when_not_matched_clause", // [1210]
    "MatchedKeywordSegment", // [1211]
    "ThenKeywordSegment", // [1212]
    "THEN", // [1213]
    "MergeUpdateClauseSegment", // [1214]
    "merge_update_clause", // [1215]
    "MergeDeleteClauseSegment", // [1216]
    "merge_delete_clause", // [1217]
    "MergeInsertClauseSegment", // [1218]
    "merge_insert_clause", // [1219]
    "MergeIntoLiteralGrammar", // [1220]
    "AliasedTableReferenceGrammar", // [1221]
    "MergeMatchSegment", // [1222]
    "merge_match", // [1223]
    "SetClauseListSegment", // [1224]
    "SET", // [1225]
    "set_clause_list", // [1226]
    "MESSAGE_LENGTH", // [1227]
    "MESSAGE_OCTET_LENGTH", // [1228]
    "MESSAGE_TEXT", // [1229]
    "MIDDLEINT", // [1230]
    "MIN", // [1231]
    "MIN_ROWS", // [1232]
    "MINUTE_MICROSECOND", // [1233]
    "MINUTE_SECOND", // [1234]
    "MLSLABEL", // [1235]
    "MOD", // [1236]
    "MODE", // [1237]
    "MODIFIES", // [1238]
    "MODULE", // [1239]
    "MONTHNAME", // [1240]
    "MORE", // [1241]
    "MOVE", // [1242]
    "MULTISET", // [1243]
    "MUMPS", // [1244]
    "MYISAM", // [1245]
    "naked_identifier", // [1246]
    "NAME", // [1247]
    "WindowSpecificationSegment", // [1248]
    "window_specification", // [1249]
    "NamedWindowExpressionSegment", // [1250]
    "named_window_expression", // [1251]
    "NAMES", // [1252]
    "null_literal", // [1253]
    "NATIONAL", // [1254]
    "NATURAL", // [1255]
    "NaturalKeywordSegment", // [1256]
    "NCHAR", // [1257]
    "NCLOB", // [1258]
    "sign_indicator", // [1259]
    "NESTING", // [1260]
    "newline", // [1261]
    "NewlineSegment", // [1262]
    "NO_WRITE_TO_BINLOG", // [1263]
    "NOAUDIT", // [1264]
    "NOCHECK", // [1265]
    "NOCOMPRESS", // [1266]
    "NOCREATEDB", // [1267]
    "NOCREATEROLE", // [1268]
    "NOCREATEUSER", // [1269]
    "NOINHERIT", // [1270]
    "NOLOGIN", // [1271]
    "UnorderedSelectStatementSegment", // [1272]
    "WithCompoundStatementSegment", // [1273]
    "with_compound_statement", // [1274]
    "NonSetSelectableGrammar", // [1275]
    "BracketedSetExpressionGrammar", // [1276]
    "MergeStatementSegment", // [1277]
    "merge_statement", // [1278]
    "SetExpressionSegment", // [1279]
    "NONCLUSTERED", // [1280]
    "NONE", // [1281]
    "NORMALIZE", // [1282]
    "NORMALIZED", // [1283]
    "NOSUPERUSER", // [1284]
    "RawNotSegment", // [1285]
    "NOTHING", // [1286]
    "NOTIFY", // [1287]
    "NOTNULL", // [1288]
    "NOWAIT", // [1289]
    "NULLABLE", // [1290]
    "NULLIF", // [1291]
    "NUMBER", // [1292]
    "NUMERIC", // [1293]
    "ObjectLiteralElementSegment", // [1294]
    "object_literal_element", // [1295]
    "start_curly_bracket", // [1296]
    "CastOperatorSegment", // [1297]
    "StartSquareBracketSegment", // [1298]
    "StartBracketSegment", // [1299]
    "OBJECTS", // [1300]
    "OCTET_LENGTH", // [1301]
    "OCTETS", // [1302]
    "OFF", // [1303]
    "OFFLINE", // [1304]
    "OFFSETS", // [1305]
    "OIDS", // [1306]
    "ONLINE", // [1307]
    "OPEN", // [1308]
    "OPENDATASOURCE", // [1309]
    "OPENQUERY", // [1310]
    "OPENROWSET", // [1311]
    "OPENXML", // [1312]
    "OPERATION", // [1313]
    "OPERATOR", // [1314]
    "OPTIMIZE", // [1315]
    "OPTIONALLY", // [1316]
    "ReplaceKeywordSegment", // [1317]
    "REPLACE", // [1318]
    "LastKeywordSegment", // [1319]
    "WithFillSegment", // [1320]
    "with_fill", // [1321]
    "NoorderKeywordSegment", // [1322]
    "ORDERING", // [1323]
    "ORDINALITY", // [1324]
    "OTHERS", // [1325]
    "OUT", // [1326]
    "OUTFILE", // [1327]
    "OUTPUT", // [1328]
    "OverKeywordSegment", // [1329]
    "OVER", // [1330]
    "OverlapsKeywordSegment", // [1331]
    "OVERLAY", // [1332]
    "OVERRIDING", // [1333]
    "OWNER", // [1334]
    "PACK_KEYS", // [1335]
    "PAD", // [1336]
    "PARAMETER", // [1337]
    "parameter", // [1338]
    "?", // [1339]
    "PARAMETER_MODE", // [1340]
    "PARAMETER_NAME", // [1341]
    "PARAMETER_ORDINAL_POSITION", // [1342]
    "PARAMETER_SPECIFIC_CATALOG", // [1343]
    "PARAMETER_SPECIFIC_NAME", // [1344]
    "PARAMETER_SPECIFIC_SCHEMA", // [1345]
    "PARAMETERS", // [1346]
    "PARTIAL", // [1347]
    "PARTITION", // [1348]
    "PartitionKeywordSegment", // [1349]
    "PASCAL", // [1350]
    "PASSWORD", // [1351]
    "PATH", // [1352]
    "SlashSegment", // [1353]
    "path_segment", // [1354]
    "PCTFREE", // [1355]
    "PERCENT", // [1356]
    "PERCENT_RANK", // [1357]
    "PERCENTILE_CONT", // [1358]
    "PERCENTILE_DISC", // [1359]
    "pipe", // [1360]
    "PLACING", // [1361]
    "PLAN", // [1362]
    "PLI", // [1363]
    "POSITION", // [1364]
    "OverClauseSegment", // [1365]
    "over_clause", // [1366]
    "FilterClauseGrammar", // [1367]
    "POSTFIX", // [1368]
    "POWER", // [1369]
    "PREFIX", // [1370]
    "PREORDER", // [1371]
    "PREPARE", // [1372]
    "PREPARED", // [1373]
    "PRESERVE", // [1374]
    "PrimaryKeywordSegment", // [1375]
    "PRINT", // [1376]
    "PROC", // [1377]
    "PROCEDURAL", // [1378]
    "PROCESS", // [1379]
    "PROCESSLIST", // [1380]
    "PURGE", // [1381]
    "SOME", // [1382]
    "SomeKeywordSegment", // [1383]
    "QUOTE", // [1384]
    "quoted_identifier", // [1385]
    "quoted_literal", // [1386]
    "RAID0", // [1387]
    "RAISERROR", // [1388]
    "RANK", // [1389]
    "raw_comparison_operator", // [1390]
    "RAW", // [1391]
    "READS", // [1392]
    "READTEXT", // [1393]
    "REAL", // [1394]
    "RECHECK", // [1395]
    "RECONFIGURE", // [1396]
    "RECURSIVE", // [1397]
    "REF", // [1398]
    "ReferenceMatchGrammar", // [1399]
    "ReferentialActionGrammar", // [1400]
    "MatchKeywordSegment", // [1401]
    "SIMPLE", // [1402]
    "PartialKeywordSegment", // [1403]
    "SimpleKeywordSegment", // [1404]
    "SetKeywordSegment", // [1405]
    "NoKeywordSegment", // [1406]
    "ActionKeywordSegment", // [1407]
    "REGEXP", // [1408]
    "REGR_AVGX", // [1409]
    "REGR_AVGY", // [1410]
    "REGR_COUNT", // [1411]
    "REGR_INTERCEPT", // [1412]
    "REGR_R2", // [1413]
    "REGR_SLOPE", // [1414]
    "REGR_SXX", // [1415]
    "REGR_SXY", // [1416]
    "REGR_SYY", // [1417]
    "REINDEX", // [1418]
    "RELATIVE", // [1419]
    "RELEASE", // [1420]
    "RELOAD", // [1421]
    "REPEAT", // [1422]
    "REPEATABLE", // [1423]
    "REPLICATION", // [1424]
    "REQUIRE", // [1425]
    "RESET", // [1426]
    "RESIGNAL", // [1427]
    "RESTART", // [1428]
    "RESTORE", // [1429]
    "RESULT", // [1430]
    "RETURN", // [1431]
    "RETURNED_CARDINALITY", // [1432]
    "RETURNED_LENGTH", // [1433]
    "RETURNED_OCTET_LENGTH", // [1434]
    "RETURNED_SQLSTATE", // [1435]
    "RevokeKeywordSegment", // [1436]
    "ROLES", // [1437]
    "ROLLBACK", // [1438]
    "ROUTINE_CATALOG", // [1439]
    "ROUTINE_NAME", // [1440]
    "ROUTINE_SCHEMA", // [1441]
    "ROW_COUNT", // [1442]
    "ROW_NUMBER", // [1443]
    "ROWCOUNT", // [1444]
    "ROWGUIDCOL", // [1445]
    "ROWID", // [1446]
    "ROWNUM", // [1447]
    "RULE", // [1448]
    "TablesampleKeywordSegment", // [1449]
    "SYSTEM", // [1450]
    "BernoulliKeywordSegment", // [1451]
    "SystemKeywordSegment", // [1452]
    "RepeatableKeywordSegment", // [1453]
    "SAVE", // [1454]
    "SAVEPOINT", // [1455]
    "SCALE", // [1456]
    "SCHEMA_NAME", // [1457]
    "SCOPE", // [1458]
    "SCOPE_CATALOG", // [1459]
    "SCOPE_NAME", // [1460]
    "SCOPE_SCHEMA", // [1461]
    "SCROLL", // [1462]
    "SEARCH", // [1463]
    "SECOND_MICROSECOND", // [1464]
    "SECTION", // [1465]
    "SECURITY", // [1466]
    "WildcardExpressionSegment", // [1467]
    "wildcard_expression", // [1468]
    "SelectClauseModifierSegment", // [1469]
    "select_clause_modifier", // [1470]
    "SelectClauseElementSegment", // [1471]
    "select_clause_element", // [1472]
    "SelectClauseTerminatorGrammar", // [1473]
    "SelectClauseSegment", // [1474]
    "select_clause", // [1475]
    "GroupByClauseSegment", // [1476]
    "groupby_clause", // [1477]
    "HavingClauseSegment", // [1478]
    "having_clause", // [1479]
    "NamedWindowSegment", // [1480]
    "named_window", // [1481]
    "OffsetClauseSegment", // [1482]
    "offset_clause", // [1483]
    "FetchClauseSegment", // [1484]
    "fetch_clause", // [1485]
    "WithCompoundNonSelectStatementSegment", // [1486]
    "NonWithSelectableGrammar", // [1487]
    "SELF", // [1488]
    "statement_terminator", // [1489]
    "SENSITIVE", // [1490]
    "MaxvalueKeywordSegment", // [1491]
    "MinvalueKeywordSegment", // [1492]
    "SERIALIZABLE", // [1493]
    "SERVER_NAME", // [1494]
    "SESSION", // [1495]
    "SetClauseSegment", // [1496]
    "set_clause", // [1497]
    "UnionGrammar", // [1498]
    "IntersectKeywordSegment", // [1499]
    "ExceptKeywordSegment", // [1500]
    "MinusKeywordSegment", // [1501]
    "SETOF", // [1502]
    "SETUSER", // [1503]
    "SHARES", // [1504]
    "SHOW", // [1505]
    "SHUTDOWN", // [1506]
    "SIGNAL", // [1507]
    "PositiveSegment", // [1508]
    "NegativeSegment", // [1509]
    "SIMILAR", // [1510]
    "SIZE", // [1511]
    "slash", // [1512]
    "slice", // [1513]
    "SMALLINT", // [1514]
    "SONAME", // [1515]
    "SOURCE", // [1516]
    "SPACE", // [1517]
    "SPATIAL", // [1518]
    "SPECIFIC_NAME", // [1519]
    "SPECIFICTYPE", // [1520]
    "SQL", // [1521]
    "SQL_BIG_RESULT", // [1522]
    "SQL_BIG_SELECTS", // [1523]
    "SQL_BIG_TABLES", // [1524]
    "SQL_CALC_FOUND_ROWS", // [1525]
    "SQL_LOG_OFF", // [1526]
    "SQL_LOG_UPDATE", // [1527]
    "SQL_LOW_PRIORITY_UPDATES", // [1528]
    "SQL_SELECT_LIMIT", // [1529]
    "SQL_SMALL_RESULT", // [1530]
    "SQL_WARNINGS", // [1531]
    "SQLCA", // [1532]
    "SQLCODE", // [1533]
    "SQLERROR", // [1534]
    "SQLEXCEPTION", // [1535]
    "SQLSTATE", // [1536]
    "SQLWARNING", // [1537]
    "SQRT", // [1538]
    "SSL", // [1539]
    "STABLE", // [1540]
    "star", // [1541]
    "STARTING", // [1542]
    "STARTS", // [1543]
    "STATE", // [1544]
    "STATISTICS", // [1545]
    "STDDEV_POP", // [1546]
    "STDDEV_SAMP", // [1547]
    "STDIN", // [1548]
    "STDOUT", // [1549]
    "STORAGE", // [1550]
    "STRAIGHT_JOIN", // [1551]
    "STRICT", // [1552]
    "ConcatSegment", // [1553]
    "STRING", // [1554]
    "STRUCTURE", // [1555]
    "STYLE", // [1556]
    "SUBCLASS_ORIGIN", // [1557]
    "SUBLIST", // [1558]
    "SUBMULTISET", // [1559]
    "SUBSTRING", // [1560]
    "SUCCESSFUL", // [1561]
    "SUM", // [1562]
    "SUPERUSER", // [1563]
    "SYMMETRIC", // [1564]
    "SYNONYM", // [1565]
    "SYSDATE", // [1566]
    "SYSID", // [1567]
    "SYSTEM_USER", // [1568]
    "ForeignKeyGrammar", // [1569]
    "TABLE_NAME", // [1570]
    "Expression_A_Unary_Operator_Grammar", // [1571]
    "Expression_C_Grammar", // [1572]
    "Expression_B_Unary_Operator_Grammar", // [1573]
    "TEMPLATE", // [1574]
    "TransientKeywordSegment", // [1575]
    "TERMINATE", // [1576]
    "TERMINATED", // [1577]
    "TEXT", // [1578]
    "TEXTSIZE", // [1579]
    "THAN", // [1580]
    "tilde", // [1581]
    "WITHOUT", // [1582]
    "WithoutKeywordSegment", // [1583]
    "ZoneKeywordSegment", // [1584]
    "ZONE", // [1585]
    "AtKeywordSegment", // [1586]
    "TIMEZONE_HOUR", // [1587]
    "TIMEZONE_MINUTE", // [1588]
    "TINYBLOB", // [1589]
    "TINYINT", // [1590]
    "TINYTEXT", // [1591]
    "TOAST", // [1592]
    "TOP", // [1593]
    "TOP_LEVEL_COUNT", // [1594]
    "TRAN", // [1595]
    "TRANSACTION", // [1596]
    "BeginKeywordSegment", // [1597]
    "CommitKeywordSegment", // [1598]
    "RollbackKeywordSegment", // [1599]
    "WORK", // [1600]
    "TransactionKeywordSegment", // [1601]
    "WorkKeywordSegment", // [1602]
    "NameKeywordSegment", // [1603]
    "ChainKeywordSegment", // [1604]
    "TRANSACTION_ACTIVE", // [1605]
    "TRANSACTIONS", // [1606]
    "TRANSACTIONS_COMMITTED", // [1607]
    "TRANSACTIONS_ROLLED_BACK", // [1608]
    "TRANSFORM", // [1609]
    "TRANSFORMS", // [1610]
    "TRANSLATE", // [1611]
    "TRANSLATION", // [1612]
    "TREAT", // [1613]
    "TRIGGER_CATALOG", // [1614]
    "TRIGGER_NAME", // [1615]
    "TRIGGER_SCHEMA", // [1616]
    "TRIM", // [1617]
    "BothKeywordSegment", // [1618]
    "LeadingKeywordSegment", // [1619]
    "TrailingKeywordSegment", // [1620]
    "TRUSTED", // [1621]
    "TSEQUAL", // [1622]
    "StructLiteralSegment", // [1623]
    "UESCAPE", // [1624]
    "UID", // [1625]
    "UNCOMMITTED", // [1626]
    "NaturalJoinKeywordsGrammar", // [1627]
    "UnconditionalCrossJoinKeywordsGrammar", // [1628]
    "HorizontalJoinKeywordsGrammar", // [1629]
    "UNDER", // [1630]
    "UNDO", // [1631]
    "UNENCRYPTED", // [1632]
    "UnionKeywordSegment", // [1633]
    "UNKNOWN", // [1634]
    "UNLISTEN", // [1635]
    "UNLOCK", // [1636]
    "UNNAMED", // [1637]
    "UNNEST", // [1638]
    "UNTIL", // [1639]
    "UPDATETEXT", // [1640]
    "UPPER", // [1641]
    "USER_DEFINED_TYPE_CATALOG", // [1642]
    "USER_DEFINED_TYPE_CODE", // [1643]
    "USER_DEFINED_TYPE_NAME", // [1644]
    "USER_DEFINED_TYPE_SCHEMA", // [1645]
    "USERS", // [1646]
    "UTC_DATE", // [1647]
    "UTC_TIME", // [1648]
    "UTC_TIMESTAMP", // [1649]
    "VACUUM", // [1650]
    "VALID", // [1651]
    "VALIDATE", // [1652]
    "VALIDATOR", // [1653]
    "ValueKeywordSegment", // [1654]
    "VAR_POP", // [1655]
    "VAR_SAMP", // [1656]
    "VARBINARY", // [1657]
    "VARCHAR2", // [1658]
    "VARCHAR", // [1659]
    "VARCHARACTER", // [1660]
    "VARIABLE", // [1661]
    "VARIABLES", // [1662]
    "VERBOSE", // [1663]
    "VERSION", // [1664]
    "VOLATILE", // [1665]
    "WAITFOR", // [1666]
    "WAREHOUSES", // [1667]
    "indented_then", // [1668]
    "indented_then_contents", // [1669]
    "WHENEVER", // [1670]
    "WHILE", // [1671]
    "whitespace", // [1672]
    "WhitespaceSegment", // [1673]
    "WIDTH_BUCKET", // [1674]
    "PartitionClauseSegment", // [1675]
    "partitionby_clause", // [1676]
    "FrameClauseSegment", // [1677]
    "frame_clause", // [1678]
    "RecursiveKeywordSegment", // [1679]
    "indented_ctes", // [1680]
    "CTEDefinitionSegment", // [1681]
    "common_table_expression", // [1682]
    "NonWithNonSelectableGrammar", // [1683]
    "BindingKeywordSegment", // [1684]
    "WITHIN", // [1685]
    "WRITETEXT", // [1686]
    "X509", // [1687]
    "XML", // [1688]
    "XOR", // [1689]
    "YAML", // [1690]
    "YEAR_MONTH", // [1691]
    "ZEROFILL", // [1692]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 8, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0,
    1, 0, 1, 0, 1, 0, 0, 0, 4294967295, 4294967295, 0, 10, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 210, 1, 2, 1, 1, 4, 3, 1, 4, 5, 211,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 212, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 214, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 215, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 216, 1, 2, 1, 1, 4, 3, 1, 4, 5, 82, 229, 230, 1, 229,
    3, 3, 4, 231, 232, 233, 230, 1, 233, 3, 3, 4, 231, 1, 2, 1,
    0, 238, 1, 2, 1, 1, 4, 3, 1, 4, 5, 11, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 243, 1, 2, 1, 1, 4, 3, 1, 4, 5, 244,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 245, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0,
    1, 0, 295, 1, 2, 1, 1, 4, 3, 1, 4, 5, 296, 297, 230, 1,
    297, 3, 3, 4, 231, 298, 1, 2, 1, 1, 4, 3, 1, 4, 5, 299,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 300, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 300, 301, 302, 1, 301, 3, 3, 301, 4, 303, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 98, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    304, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 207, 328,
    230, 1, 328, 3, 3, 4, 231, 329, 330, 230, 1, 330, 3, 3, 4, 231,
    1, 2, 0, 331, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1,
    2, 0, 224, 1, 2, 1, 1, 4, 3, 1, 4, 5, 333, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 334, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    335, 1, 2, 1, 1, 4, 3, 1, 4, 5, 336, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 337, 1, 2, 1, 1, 4, 3, 1, 4, 5, 338, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 339, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 340, 1, 2, 1, 1, 4, 3, 1, 4, 5, 341, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 342, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    343, 1, 2, 1, 1, 4, 3, 1, 4, 5, 344, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 346, 1, 2, 1, 1, 4, 3, 1, 4, 5, 347, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 348, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 349, 1, 2, 1, 1, 4, 3, 1, 4, 5, 350, 351, 352, 353,
    354, 355, 356, 357, 358, 359, 360, 361, 362, 363, 364, 365, 366, 367, 368, 369,
    370, 371, 372, 373, 374, 375, 540, 26, 376, 377, 1, 376, 2, 3, 4, 0,
    384, 385, 0, 2, 3, 384, 3, 386, 0, 1, 3, 387, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 388, 1, 2, 1, 1, 4, 3, 1, 4, 5, 389,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 390, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 391, 1, 2, 1, 1, 4, 3, 1, 4, 5, 392, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 301, 302, 9, 3, 3, 301, 4, 401,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 402, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 403, 1, 2, 1, 1, 4, 3, 1, 4, 5, 404, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 312, 301, 230, 1, 301, 3, 3, 4, 231,
    409, 1, 2, 1, 1, 4, 3, 1, 4, 5, 410, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 413, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    418, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1,
    0, 1, 2, 1, 419, 420, 0, 2, 3, 419, 1, 2, 1, 425, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 426, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 427, 1, 2, 1, 1, 4, 3, 1, 4, 5, 428, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 257, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    2, 1, 1, 2, 1, 246, 1, 2, 1, 1, 4, 3, 1, 4, 5, 433,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 434, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 435, 1, 2, 1, 1, 4, 3, 1, 4, 5, 436, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 437, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0, 438, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 452, 1, 2, 1, 1, 4, 3, 1, 4, 5, 453, 454,
    230, 1, 454, 3, 3, 4, 231, 12, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 455, 1, 2, 1, 1, 4, 3, 1, 4, 5, 456, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 458,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 459, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 460, 1, 2, 1, 1, 4, 3, 1, 4, 5, 461, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 462, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 463, 1, 2, 1, 1, 4, 3, 1, 4, 5, 464, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 465, 1, 2, 1, 1, 4, 3, 1, 4, 5, 466,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 467, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 468, 1, 2, 1, 1, 4, 3, 1, 4, 5, 469, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 470, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 471, 1, 2, 1, 1, 4, 3, 1, 4, 5, 472, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 473, 1, 2, 1, 1, 4, 3, 1, 4, 5, 474,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 475, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 476, 1, 2, 1, 1, 4, 3, 1, 4, 5, 477, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 478, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 479, 1, 2, 1, 1, 4, 3, 1, 4, 5, 480, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 4, 377, 9, 2, 3, 4, 481, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 482, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 0, 487, 1, 2, 1, 1, 4, 3, 1, 4, 5, 488, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 489, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    490, 1, 2, 1, 1, 4, 3, 1, 4, 5, 327, 491, 230, 1, 491, 3,
    3, 4, 231, 327, 492, 230, 1, 492, 3, 3, 4, 231, 327, 493, 230, 1,
    493, 3, 3, 4, 231, 0, 1, 2, 1, 0, 1, 2, 1, 0, 4294967295, 4294967295,
    0, 273, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 522, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 524, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 87, 525, 230, 1, 525, 3, 3, 4, 231, 526, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 527, 1, 2, 1, 1, 4, 3, 1, 4, 5, 514, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 529, 530, 10, 3, 3, 529, 4, 531,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 532, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 279, 540, 9, 3, 3, 279, 4, 541, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 301, 542, 0, 2, 3, 301, 279, 543, 0, 2, 3,
    279, 544, 1, 2, 1, 1, 4, 3, 1, 4, 5, 545, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 546, 1, 2, 1, 1, 4, 3, 1, 4, 5, 547,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 99, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 557, 1, 2, 1, 1, 4, 3, 1, 4, 5, 558, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 496, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 560, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 561, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    562, 1, 2, 1, 1, 4, 3, 1, 4, 5, 563, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 564, 1, 2, 1, 1, 4, 3, 1, 4, 5, 565, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 566, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 567, 1, 2, 1, 1, 4, 3, 1, 4, 5, 568, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 569, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    570, 1, 2, 1, 1, 4, 3, 1, 4, 5, 571, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 572, 1, 2, 1, 1, 4, 3, 1, 4, 5, 573, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 574, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 3, 4, 1, 0, 0, 1, 0, 1, 2, 1, 100, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 0, 1, 0, 1, 2,
    1, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1, 1, 2,
    1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 2, 1, 0,
    4294967295, 4294967295, 0, 1, 2, 1, 0, 657, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 658, 1, 2, 1, 1, 4, 3, 1, 4, 5, 659, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 549, 1, 2, 1, 1, 4, 3, 1, 4, 5, 660,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 661, 662, 377, 1, 662, 2, 3,
    4, 661, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 668,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 669, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 350, 1, 2, 1, 1, 4, 3, 1, 4, 5, 670, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 671, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 672, 1, 2, 1, 1, 4, 3, 1, 4, 5, 351, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 352, 1, 2, 1, 1, 4, 3, 1, 4, 5, 673,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 674, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 675, 1, 2, 1, 1, 4, 3, 1, 4, 5, 676, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 247, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 52, 1, 2, 1, 1, 4, 3, 1, 4, 5, 13, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 677, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 678, 377, 1, 678, 2, 3, 4, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    0, 699, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 729, 1996, 1, 662,
    377, 1, 662, 2, 3, 4, 0, 1, 0, 1, 2, 1, 744, 744, 747, 3,
    744, 746, 744, 3, 3, 746, 4, 745, 745, 747, 3, 745, 746, 745, 3, 3,
    746, 4, 0, 235, 752, 747, 3, 752, 746, 235, 3, 3, 746, 4, 0, 753,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 732, 733, 734, 735, 736, 737, 738,
    739, 740, 741, 742, 2057, 11, 754, 377, 1, 754, 2, 3, 4, 755, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 756, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 732, 1, 2, 1, 1, 4, 3, 1, 4, 5, 757, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 758, 1, 2, 1, 1, 4, 3, 1, 4, 5, 759,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 760, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 761, 1, 2, 1, 1, 4, 3, 1, 4, 5, 762, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 733, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 763, 1, 2, 1, 1, 4, 3, 1, 4, 5, 764, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 765, 1, 2, 1, 1, 4, 3, 1, 4, 5, 766,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 767, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 768, 1, 2, 1, 1, 4, 3, 1, 4, 5, 503, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 770, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 631, 1, 2, 1, 1, 4, 3, 1, 4, 5, 646, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 771, 1, 2, 1, 1, 4, 3, 1, 4, 5, 772,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 773, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 774, 1, 2, 1, 1, 4, 3, 1, 4, 5, 775, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 101, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 783, 1, 2, 1, 1, 4, 3, 1, 4, 5, 784, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 785, 1, 2, 1, 1, 4, 3, 1, 4, 5, 786,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 787, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 788, 1, 2, 1, 1, 4, 3, 1, 4, 5, 789, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 790, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 791, 1, 2, 1, 1, 4, 3, 1, 4, 5, 793, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 794, 1, 2, 1, 1, 4, 3, 1, 4, 5, 795,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 796, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 797, 1, 2, 1, 1, 4, 3, 1, 4, 5, 798, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 799, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 800, 1, 2, 1, 1, 4, 3, 1, 4, 5, 801, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 802, 1, 2, 1, 1, 4, 3, 1, 4, 5, 803,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 804, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 805, 1, 2, 1, 1, 4, 3, 1, 4, 5, 806, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 309, 301, 230, 1, 301, 3, 3, 4, 231,
    807, 1, 2, 1, 1, 4, 3, 1, 4, 5, 14, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 485, 808, 230, 1, 808, 3, 3, 4, 231, 682, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 3, 4, 1, 270, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 813, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 814, 1, 2, 1, 1, 4, 3, 1, 4, 5, 815, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 816, 1, 2, 1, 1, 4, 3, 1, 4, 5, 817,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 648, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 818, 1, 2, 1, 1, 4, 3, 1, 4, 5, 446, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 819, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 1, 824, 1, 2, 1, 1, 4, 3, 1, 4, 5, 825, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 826, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 827, 1, 2, 1, 1, 4, 3, 1, 4, 5, 828, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 829, 830, 230, 1, 830, 3, 3, 4, 231, 448,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 831, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 832, 1, 2, 1, 1, 4, 3, 1, 4, 5, 834, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 835, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 836, 1, 2, 1, 1, 4, 3, 1, 4, 5, 837, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 838, 1, 2, 1, 1, 4, 3, 1, 4, 5, 839,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 840, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 841, 1, 2, 1, 1, 4, 3, 1, 4, 5, 842, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 843, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 102, 1, 2, 1, 1, 4, 3, 1, 4, 5, 134, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 844, 1, 2, 1, 1, 4, 3, 1, 4, 5, 845,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 846, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 847, 1, 2, 1, 1, 4, 3, 1, 4, 5, 848, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0,
    0, 4294967295, 4294967295, 0, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1,
    0, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 0, 0, 909, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 15, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 910, 1, 2, 1, 1, 4, 3, 1, 4, 5, 414, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 414, 911, 912, 1, 911, 5, 3, 1, 746, 4,
    5, 0, 0, 0, 0, 913, 1, 2, 1, 1, 4, 3, 1, 4, 5, 923,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 16, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 4294967295, 4294967295, 0, 1, 4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295,
    0, 927, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 928, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 931, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 285, 1, 2, 1, 1, 4, 3, 1, 4, 5, 932, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 933, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    934, 1, 2, 1, 1, 4, 3, 1, 4, 5, 935, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 936, 1, 2, 1, 1, 4, 3, 1, 4, 5, 937, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 586, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 938, 1, 2, 1, 1, 4, 3, 1, 4, 5, 17, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 177, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    941, 1, 2, 1, 1, 4, 3, 1, 4, 5, 942, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 943, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 0, 0, 953, 1, 2, 1, 1, 4, 3, 1, 4, 5, 954, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 955, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 956, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2,
    1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 1, 2, 1, 0, 220,
    996, 1, 0, 0, 4294967295, 4294967295, 0, 236, 996, 1, 1, 2, 1, 0, 91, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 551, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 997, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0,
    0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 18, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 5, 662, 1012, 2, 662, 5, 3, 3, 4, 5, 0, 4294967295, 4294967295,
    0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 67, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1024, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    19, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1025, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1026, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1027, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1028, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1029, 1029, 540, 3, 1029, 279, 1029, 3, 3, 279, 4, 1030, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1031, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1032, 1, 2, 1, 1, 4, 3, 1, 4, 5, 189, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 1044, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 133, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1049, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 0, 195, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 1, 0, 1, 0, 1051, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1059, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 959, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1060, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1061, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1062, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1063, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1064, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1065, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1066, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1067, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 734, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1068, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1069, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1070, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1071, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1072, 1073, 9, 3, 3, 1072, 4, 1074, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1075, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1076, 1, 2, 1, 1, 4, 3, 1, 4, 5, 275, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1009, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    859, 1, 2, 1, 1, 4, 3, 1, 4, 5, 644, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1082, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1083, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1084, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 103, 1, 2, 1, 1, 4, 3, 1, 4, 5, 62, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2, 1, 0, 1085, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1086, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 248, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 600, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1089, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1090, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1091, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1092, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1093, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1094, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1095, 1, 2, 1, 1, 4, 3, 1, 4, 5, 632, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 552, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1096, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1097, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1098, 1, 2, 1, 1, 4, 3, 1, 4, 5, 104, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1104, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 579, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1105, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 625, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1106, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1107, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1108, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1109,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1110, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1111, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1112, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 20, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1113, 1, 2, 1, 1, 4, 3, 1, 4, 5, 960, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1114, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 0, 380, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1102, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1115, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 394, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1120, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1121, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1122,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1123, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 4294967295, 4294967295, 0, 220, 1128, 1, 0, 236, 1128, 1, 0, 1126,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 440, 1138, 1, 1, 2, 1, 0,
    236, 1138, 1, 0, 0, 1, 0, 1, 2, 1, 1146, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1147, 1, 2, 1, 1, 4, 3, 1, 4, 5, 940, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1148, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1149, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1150, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 2, 9, 4, 3, 1, 4, 5, 1151, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1152, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 21, 1, 2, 1, 1, 4, 3, 1, 4, 5, 92, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1153, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    353, 1, 2, 1, 1, 4, 3, 1, 4, 5, 354, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 355, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1154, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 356, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 700, 1, 2, 1, 1, 4, 3, 1, 4, 5, 701, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 702, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    703, 1, 2, 1, 1, 4, 3, 1, 4, 5, 704, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 705, 1, 2, 1, 1, 4, 3, 1, 4, 5, 706, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 357, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 358, 1, 2, 1, 1, 4, 3, 1, 4, 5, 359, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1155, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    998, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1156, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1157, 1, 2, 1, 1, 4, 3, 1, 4, 5, 553, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1158, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1159, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1160, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 617, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 396, 396, 540, 3, 396, 279, 396, 3, 3, 279, 4, 0, 1, 2, 1,
    0, 0, 0, 961, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1166, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1167, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1168, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 746, 912, 9, 5,
    3, 1, 746, 4, 5, 746, 747, 9, 3, 3, 746, 4, 1176, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1177, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1178, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1179, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1180, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1181, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1182, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1183, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1184, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1185, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1186, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1187, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1188, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1189, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1190, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1191, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1192, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1193, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 2, 3, 1, 105, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1195, 1, 2, 1, 1, 4, 3, 1, 4, 5, 22, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1196, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1197, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 23, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1198, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1199, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1200, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    249, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1201, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1202, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1203, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1204, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1205, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0,
    0, 0, 1, 2, 1, 0, 220, 1128, 1, 236, 1128, 1, 1227, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1228, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1229, 1, 2, 1, 1, 4, 3, 1, 4, 5, 580, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1230, 1, 2, 1, 1, 4, 3, 1, 4, 5, 735, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1231, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1232, 1, 2, 1, 1, 4, 3, 1, 4, 5, 962, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 308, 301, 230, 1, 301, 3, 3, 4, 231, 736,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1233, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1234, 1, 2, 1, 1, 4, 3, 1, 4, 5, 250, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 992, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1235, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1236, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1237, 1, 2, 1, 1, 4, 3, 1, 4, 5, 606,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1238, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 106, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1239, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 305, 301, 230, 1, 301, 3, 3, 4, 231,
    40, 1, 2, 1, 1, 4, 3, 1, 4, 5, 737, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1240, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1241, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1242, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 306, 301, 230, 1, 301, 3, 3, 4, 231, 1243, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1244, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1245,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2, 3, 1246, 1073, 1, 1246, 3,
    3, 1072, 4, 1247, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    0, 1, 0, 1252, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1117, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1117, 1253, 912, 1, 1253, 5, 3, 1, 746,
    4, 5, 1254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1255, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1257, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1258, 1, 2, 1, 1, 4, 3, 1, 4, 5, 308, 1259, 230, 1, 1259, 3,
    3, 4, 231, 1260, 1, 2, 1, 1, 4, 3, 1, 4, 5, 640, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1261, 1262, 12, 3, 3, 1261, 4, 915, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 360, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 361, 1, 2, 1, 1, 4, 3, 1, 4, 5, 362, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 363, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    707, 1, 2, 1, 1, 4, 3, 1, 4, 5, 708, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 709, 1, 2, 1, 1, 4, 3, 1, 4, 5, 710, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 711, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 712, 1, 2, 1, 1, 4, 3, 1, 4, 5, 713, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 364, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    365, 1, 2, 1, 1, 4, 3, 1, 4, 5, 366, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 251, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1263, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1264, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 252, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1265, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1266, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1267, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1268, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1269, 1, 2, 1, 1, 4, 3, 1, 4, 5, 253, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1270, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1271, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1,
    2, 1, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0, 1280, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1281, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1282, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1283, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1284, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 498, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 498, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1286, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1287, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1288, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1289, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 499, 1, 2, 1, 1, 4, 3, 1, 4, 5, 499,
    1253, 912, 1, 1253, 5, 3, 1, 746, 4, 5, 1290, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1291, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1081, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1292, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1293, 1, 2, 1, 1, 4, 3, 1, 4, 5, 97, 97, 747, 3,
    97, 746, 97, 3, 3, 746, 4, 95, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 1174, 1296, 230, 1, 1296, 3, 3, 4, 231, 1, 2, 0, 0,
    0, 1300, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1301, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1302, 1, 2, 1, 1, 4, 3, 1, 4, 5, 629,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1303, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1304, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 963,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1305, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1306, 1, 2, 1, 1, 4, 3, 1, 4, 5, 636, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 164, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1307, 1, 2, 1, 1, 4, 3, 1, 4, 5, 919, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1308, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1309,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1310, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1311, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1312, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 107, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1313, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1314, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1315, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1041,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1316, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 607, 1, 2, 1, 1, 4, 3, 1, 4, 5, 395, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 395, 301, 302, 1, 301, 3, 3, 301, 4,
    0, 0, 0, 1, 0, 0, 218, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1323, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1324, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1325, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1326,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1144, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1327, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1328, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1330, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 3, 4, 1, 0, 907, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1332, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1333, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1100, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1334, 1, 2, 1, 1, 4, 3, 1, 4, 5, 108, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1335, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1336,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1337, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 4, 4294967295, 1338, 377, 1, 1338, 2, 3, 4, 1339, 1338, 230, 1,
    1338, 3, 3, 4, 231, 1340, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1341,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1342, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1343, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1344, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1345, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1346, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1347, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 1348, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1350, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1351,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1352, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 5, 1354, 1012, 2, 1354, 5, 3, 3, 4, 5, 1, 0, 0,
    1355, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1356, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1357, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1358, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1359, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 137, 1, 2, 1, 1, 4, 3, 1, 4, 5, 313, 1360, 230, 1,
    1360, 3, 3, 4, 231, 1361, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1362,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1363, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 307, 301, 230, 1, 301, 3, 3, 4, 231, 128, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1364, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    307, 1259, 230, 1, 1259, 3, 3, 4, 231, 0, 1368, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1369, 1, 2, 1, 1, 4, 3, 1, 4, 5, 949, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 685, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1370, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1371, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1372, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1373, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1374, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 506, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1376, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 872, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 142, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1377, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1378, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    24, 1, 2, 1, 1, 4, 3, 1, 4, 5, 68, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1379, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1380, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 206, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1381, 1, 2, 1, 1, 4, 3, 1, 4, 5, 964, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 738, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1384, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1013, 1385,
    1073, 3, 1385, 1072, 1013, 3, 3, 1072, 4, 235, 1386, 747, 3, 1386, 746, 235,
    3, 3, 746, 4, 1387, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1388, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 944, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1389, 1, 2, 1, 1, 4, 3, 1, 4, 5, 278, 1390, 230, 1,
    1390, 3, 3, 4, 231, 311, 1390, 230, 1, 1390, 3, 3, 4, 231, 1391, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 310, 1390, 230, 1, 1390, 3, 3, 4,
    231, 393, 1390, 230, 1, 1390, 3, 3, 4, 231, 109, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1392, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1393, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1394, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1395, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1396, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1397, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1398, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 1, 0, 0, 111,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 110, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 633, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1408, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1409, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1410, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1411, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1412, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1413, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1414, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1415, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1416, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1417, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1418, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1419, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1420, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1421, 1, 2, 1, 1, 4, 3, 1, 4, 5, 291, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1422, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1423, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1318, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1424, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1425, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1426, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1427, 1, 2, 1, 1, 4, 3, 1, 4, 5, 25, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1010, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1428, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1429, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 809, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1430, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1431, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1432, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1433, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1434, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1435, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 594, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 190, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 554, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 860, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 26, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1437, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1438, 1, 2, 1, 1, 4, 3, 1, 4, 5, 663,
    662, 377, 1, 662, 2, 3, 4, 663, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 27, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1439, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1440, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1441,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 69, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 638, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1442, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1443, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1444, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1445, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1446, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1447,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 917, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1448, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2,
    1, 0, 1, 2, 1, 1454, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1455,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1456, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 28, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1457, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 60, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1458, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1459, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1460, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1461,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1462, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1463, 1, 2, 1, 1, 4, 3, 1, 4, 5, 739, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1464, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1465, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1466, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 1, 0, 2, 8094, 2, 8096, 3, 394, 803,
    394, 498, 803, 0, 112, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2,
    1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1488, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 782, 1489, 230, 1, 1489, 3, 3, 4, 231, 1490, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1006, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 29, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 70, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1493, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 53, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1494, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1495, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1048,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 4294967295, 4294967295, 0,
    1225, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1502, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1058, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1503, 1, 2, 1, 1, 4, 3, 1, 4, 5, 201, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1504, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    4294967295, 4294967295, 0, 1505, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1506, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1507, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1510, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1402, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 235, 1385, 1073, 3, 1385, 1072, 235,
    3, 3, 1072, 4, 1511, 1, 2, 1, 1, 4, 3, 1, 4, 5, 309, 1512,
    230, 1, 1512, 3, 3, 4, 231, 327, 1513, 230, 1, 1513, 3, 3, 4, 231,
    1514, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1382, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1515, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1516, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1517, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1518, 1, 2, 1, 1, 4, 3, 1, 4, 5, 578, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1519, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1520, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1521, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1522, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1523, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1524, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1525, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1526, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1527, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1528, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1529, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1530, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1531, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1532, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1533, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1534, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1535, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1536, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1537, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1538, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1539, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1540, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 30, 1, 2, 1, 1, 4, 3, 1, 4, 5, 71, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 306, 1541, 230, 1, 1541, 3, 3, 4, 231, 611,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1542, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1543, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1544, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 649, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 581, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1545, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1546, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1547,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1548, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1549, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1550, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1551, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 31, 1, 2, 1, 1, 4, 3, 1, 4, 5, 72, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1552, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1554, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1555,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1556, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1557, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1558, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1559, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1560, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1561, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1562, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1563,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 231, 230, 9, 3, 3, 4, 231,
    1564, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1565, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1566, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1567, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1450, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1568, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1,
    1, 2, 1, 0, 32, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1570, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 73, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 985, 1, 2, 1, 1, 4, 3, 1, 4, 5, 33, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 34, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 74, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 113, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1574, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 114, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1576, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1577, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1578, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1579, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1580, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 367, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 368, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 369, 1, 2, 1, 1, 4, 3, 1, 4, 5, 370, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 371, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    372, 1, 2, 1, 1, 4, 3, 1, 4, 5, 922, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 873, 1581, 230, 1, 1581, 3, 3, 4, 231, 680, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 681, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1587, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1588, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1589, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1590, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1591,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 89, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1592, 1, 2, 1, 1, 4, 3, 1, 4, 5, 373, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 374, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1593, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1594, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 999, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1595,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1596, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 1605, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1606,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1607, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1608, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1609, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1610, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 616, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1611, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1612, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1613,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 115, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1614, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1615, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1616, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1617, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 415, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 415, 911, 912, 1, 911, 5, 3, 1, 746, 4,
    5, 116, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1621, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1622, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 2, 1, 35, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1624, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1625, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 948, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1626, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1630, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1631, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1632, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 965, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    510, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1634, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1635, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1636, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1637, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1638, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0,
    695, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1639, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 117, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1640, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1641, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 118, 1, 2, 1, 1, 4, 3, 1, 4, 5, 119, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 120, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    123, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1642, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1643, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1644, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1645, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1646, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1131, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1647, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1648, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1649, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1650, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1651, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1652, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1653, 1, 2, 1, 1, 4, 3, 1, 4, 5, 422, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 1, 2, 1, 1, 0, 423,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1655, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1656, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1657, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1658, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1659, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1660, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1661, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1662,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 688, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1663, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1664, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 36, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 75, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1665, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1666, 1, 2, 1, 1, 4, 3, 1, 4, 5, 37,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1667, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 740, 1, 2, 1, 1, 4, 3, 1, 4, 5, 741, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 220, 1668, 1, 440, 1669, 1, 236, 1669, 1,
    236, 1668, 1, 442, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1670, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 779, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1671, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1672, 1673, 12, 3, 3, 4, 1672, 1674, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 4294967295, 4294967295, 0, 966, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 220, 1680, 1, 1, 0, 236, 1680, 1, 220, 1680, 1, 1, 0, 236, 1680,
    1, 432, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1685, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5,
    1012, 9, 3, 3, 4, 5, 1600, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    57, 1, 2, 1, 1, 4, 3, 1, 4, 5, 121, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1686, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1687, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1688, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1689, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1690, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 742, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1691, 1, 2, 1, 1, 4, 3, 1, 4, 5, 375, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1692, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1585, 1,
    2, 1, 1, 4, 3, 1, 4, 5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 30, 46, 9, 0, 42, 0, 38, 39, 41, 42, 43, 44, 45,
    46, 47, 48, 49, 50, 0, 51, 40, 54, 0, 55, 56, 0, 58, 59, 61,
    42, 0, 63, 59, 61, 42, 64, 0, 58, 66, 61, 46, 0, 63, 66, 61,
    41, 42, 46, 44, 43, 77, 0, 79, 81, 84, 86, 88, 90, 0, 93, 94,
    96, 52, 0, 122, 47, 48, 124, 41, 42, 44, 0, 125, 126, 127, 0, 129,
    130, 0, 131, 132, 0, 39, 48, 135, 136, 0, 51, 0, 122, 49, 46, 0,
    126, 127, 138, 64, 0, 139, 50, 46, 47, 0, 140, 141, 125, 143, 122, 144,
    129, 145, 146, 39, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 136,
    158, 159, 0, 58, 141, 160, 53, 161, 163, 55, 165, 166, 167, 168, 169, 170,
    171, 172, 130, 46, 0, 126, 127, 0, 173, 166, 0, 174, 165, 0, 175, 176,
    56, 178, 179, 180, 181, 182, 183, 184, 185, 186, 59, 0, 126, 187, 0, 173,
    179, 0, 174, 178, 0, 175, 188, 57, 191, 193, 70, 0, 196, 58, 77, 0,
    124, 60, 197, 0, 48, 62, 199, 0, 202, 64, 77, 66, 199, 68, 203, 205,
    71, 208, 75, 85, 95, 105, 115, 125, 135, 145, 217, 0, 220, 223, 176, 0,
    226, 173, 227, 155, 164, 234, 236, 177, 0, 239, 241, 187, 197, 207, 217, 229,
    0, 255, 256, 96, 258, 259, 227, 0, 260, 96, 261, 228, 262, 263, 264, 0,
    265, 171, 266, 230, 268, 0, 271, 272, 274, 226, 239, 0, 276, 277, 234, 280,
    281, 0, 235, 282, 146, 272, 283, 237, 0, 236, 286, 287, 288, 160, 290, 0,
    292, 238, 293, 88, 239, 0, 265, 165, 239, 240, 294, 242, 252, 261, 271, 281,
    291, 300, 310, 320, 330, 314, 315, 316, 317, 318, 319, 320, 321, 322, 323, 352,
    332, 331, 96, 324, 326, 334, 343, 0, 355, 367, 365, 332, 0, 0, 293, 370,
    380, 390, 400, 410, 420, 430, 440, 450, 460, 470, 480, 0, 345, 490, 500, 510,
    520, 530, 566, 575, 280, 378, 379, 203, 288, 324, 0, 382, 280, 86, 293, 576,
    582, 587, 597, 607, 617, 627, 637, 647, 397, 398, 399, 400, 648, 655, 665, 675,
    685, 405, 0, 406, 406, 407, 0, 408, 408, 695, 704, 714, 724, 411, 412, 725,
    735, 416, 417, 736, 748, 746, 280, 753, 751, 288, 756, 762, 421, 765, 775, 785,
    795, 805, 815, 227, 0, 226, 429, 293, 818, 431, 821, 831, 841, 851, 861, 871,
    889, 0, 439, 440, 881, 443, 445, 447, 449, 447, 447, 0, 439, 324, 885, 443,
    445, 447, 449, 447, 447, 400, 86, 451, 890, 900, 910, 919, 929, 939, 949, 959,
    969, 0, 979, 989, 999, 1009, 1019, 1029, 1039, 1049, 1059, 1069, 1079, 1089, 1099, 1109,
    1119, 1129, 1139, 1149, 1159, 1169, 1179, 1189, 0, 1195, 1205, 1217, 483, 1215, 226, 484,
    486, 1218, 1228, 1238, 1248, 1258, 1267, 1276, 1285, 494, 280, 203, 378, 0, 0, 497,
    77, 1289, 0, 500, 501, 0, 502, 1286, 324, 0, 504, 505, 0, 507, 508, 509,
    511, 0, 512, 508, 513, 0, 516, 517, 519, 0, 226, 382, 1290, 0, 1293, 520,
    0, 1297, 1307, 226, 484, 486, 1309, 0, 523, 0, 0, 1319, 1329, 1338, 1348, 0,
    528, 483, 1358, 1368, 1375, 1385, 1395, 277, 533, 534, 535, 536, 537, 538, 539, 1396,
    1403, 1413, 1419, 1425, 1435, 0, 407, 407, 1445, 1455, 548, 1465, 550, 555, 556, 1466,
    1476, 1486, 1496, 1506, 1516, 1526, 1536, 1546, 1556, 1566, 1576, 1586, 1596, 1606, 1616, 1626,
    1636, 1646, 1656, 0, 122, 575, 1666, 382, 293, 382, 576, 577, 1670, 170, 168, 169,
    0, 1669, 582, 583, 584, 585, 79, 81, 0, 587, 77, 0, 293, 588, 0, 122,
    42, 589, 590, 0, 122, 592, 593, 168, 589, 79, 81, 0, 595, 382, 596, 0,
    122, 592, 598, 599, 589, 601, 163, 239, 0, 1673, 1671, 603, 1676, 0, 122, 592,
    605, 589, 77, 0, 608, 1694, 1692, 0, 276, 277, 1691, 280, 1688, 1686, 483, 293,
    431, 0, 122, 48, 589, 199, 0, 122, 46, 589, 609, 1699, 0, 255, 256, 96,
    0, 612, 576, 96, 258, 259, 1697, 0, 260, 96, 261, 1698, 262, 263, 264, 0,
    122, 171, 266, 1700, 613, 0, 122, 592, 615, 165, 589, 239, 1714, 0, 1707, 1705,
    1704, 618, 283, 513, 0, 293, 1713, 1710, 431, 0, 620, 239, 621, 0, 122, 155,
    623, 1715, 626, 287, 0, 627, 628, 1719, 1718, 145, 144, 0, 157, 628, 1716, 288,
    630, 163, 630, 163, 163, 239, 1727, 0, 634, 635, 637, 293, 276, 639, 637, 293,
    276, 0, 90, 239, 1722, 0, 500, 641, 0, 641, 1721, 0, 642, 643, 0, 642,
    645, 0, 587, 647, 1723, 637, 650, 0, 651, 1724, 324, 0, 129, 169, 652, 653,
    0, 122, 124, 199, 0, 122, 592, 166, 589, 239, 160, 293, 1734, 1731, 431, 655,
    1735, 1745, 1755, 1765, 1775, 1785, 1793, 0, 1803, 664, 665, 1804, 666, 1807, 1817, 1827,
    1837, 1847, 1857, 1867, 1877, 1887, 1897, 1907, 1917, 1927, 1937, 1947, 1957, 1976, 1967, 226,
    281, 1984, 679, 0, 683, 684, 0, 1979, 0, 1977, 686, 687, 1978, 689, 0, 93,
    94, 0, 0, 226, 690, 691, 692, 1980, 694, 696, 697, 1985, 0, 1995, 714, 715,
    716, 717, 718, 719, 720, 721, 722, 723, 724, 725, 726, 727, 728, 96, 1997, 730,
    0, 2009, 2007, 2006, 731, 743, 2046, 2012, 2023, 0, 2034, 748, 749, 750, 751, 2035,
    2047, 2068, 2077, 2087, 2097, 2107, 2117, 2127, 2137, 2147, 2157, 2167, 2177, 2187, 2197, 2207,
    2217, 2227, 2237, 0, 504, 769, 2247, 2257, 2267, 2277, 2287, 2297, 2307, 2317, 2327, 0,
    144, 776, 778, 781, 2337, 2347, 2357, 2367, 2377, 2387, 2397, 2407, 2417, 0, 792, 281,
    77, 2427, 2437, 2447, 2457, 2467, 2477, 2487, 2497, 2507, 2517, 2527, 2537, 2547, 2557, 2567,
    2576, 2586, 2596, 2605, 2615, 810, 811, 0, 271, 575, 2616, 382, 293, 382, 812, 0,
    271, 42, 274, 590, 812, 0, 271, 168, 274, 79, 0, 271, 599, 274, 601, 812,
    2619, 0, 271, 605, 274, 77, 0, 271, 48, 274, 226, 0, 271, 46, 274, 609,
    812, 0, 271, 171, 266, 0, 271, 593, 165, 274, 2629, 239, 812, 0, 271, 155,
    274, 623, 0, 271, 50, 274, 77, 812, 0, 271, 124, 274, 199, 0, 271, 166,
    274, 239, 812, 2631, 2641, 2651, 2661, 2671, 2681, 2691, 0, 445, 324, 2701, 2711, 2721,
    0, 820, 822, 2724, 2734, 2744, 2754, 2764, 2774, 2783, 2793, 2803, 833, 2813, 2823, 2833,
    2843, 2853, 2863, 2873, 2883, 2893, 2903, 2913, 2923, 2933, 2943, 2953, 2963, 2973, 0, 849,
    2983, 431, 850, 852, 854, 856, 0, 857, 2985, 2984, 858, 0, 451, 857, 861, 862,
    0, 863, 500, 864, 865, 866, 867, 0, 500, 868, 869, 870, 857, 0, 871, 856,
    2989, 874, 0, 875, 876, 877, 878, 0, 879, 2992, 2991, 0, 2990, 397, 398, 400,
    879, 2996, 874, 0, 875, 876, 3005, 0, 880, 2997, 431, 0, 3000, 881, 882, 3001,
    884, 494, 886, 86, 0, 3020, 378, 203, 3010, 3009, 888, 324, 431, 3007, 3006, 280,
    288, 203, 889, 324, 891, 892, 0, 3013, 820, 893, 3016, 3014, 324, 0, 382, 3019,
    483, 96, 895, 896, 897, 889, 898, 86, 899, 3021, 900, 280, 379, 902, 904, 288,
    906, 0, 3022, 3032, 3042, 3052, 3062, 0, 914, 3073, 286, 916, 3074, 96, 324, 637,
    3075, 637, 918, 3076, 920, 0, 576, 921, 3077, 3087, 3097, 0, 3107, 924, 3115, 925,
    3111, 924, 3117, 924, 3121, 0, 929, 3131, 0, 930, 324, 3134, 3144, 3154, 3164, 3174,
    3184, 3194, 3204, 3214, 3224, 3234, 0, 51, 939, 3244, 3254, 3264, 3274, 3284, 0, 945,
    3297, 3296, 0, 946, 637, 0, 3294, 96, 379, 947, 288, 3295, 950, 951, 0, 868,
    870, 3298, 918, 952, 3299, 3309, 3319, 3329, 0, 90, 3339, 957, 3341, 930, 967, 0,
    196, 256, 0, 969, 256, 970, 971, 972, 973, 655, 975, 914, 977, 3354, 0, 978,
    3345, 3342, 979, 981, 241, 3346, 983, 984, 987, 988, 0, 576, 977, 241, 984, 990,
    3351, 0, 3347, 988, 3373, 3370, 0, 3358, 991, 994, 3355, 957, 0, 969, 256, 0,
    196, 256, 3359, 3363, 0, 3362, 988, 987, 0, 969, 256, 0, 196, 256, 3367, 3374,
    3384, 3394, 324, 3410, 324, 0, 324, 293, 382, 0, 1000, 324, 90, 90, 324, 0,
    3404, 731, 324, 90, 324, 0, 1001, 3407, 1002, 3405, 1003, 1004, 0, 1007, 280, 0,
    3408, 483, 226, 288, 61, 3409, 483, 226, 288, 1008, 603, 1011, 0, 3414, 743, 0,
    293, 483, 0, 45, 281, 3417, 3427, 0, 3437, 0, 226, 690, 420, 3441, 652, 1014,
    420, 3444, 0, 276, 3442, 0, 1015, 50, 382, 3443, 0, 1015, 50, 382, 3447, 3445,
    1016, 3451, 0, 1017, 1018, 1019, 0, 0, 79, 3450, 1017, 1020, 1021, 653, 1023, 3452,
    3462, 3472, 3482, 3492, 3502, 3512, 3522, 3533, 3543, 3553, 3563, 0, 1033, 3573, 0, 1034,
    163, 1036, 0, 48, 199, 0, 148, 163, 124, 197, 77, 88, 1038, 3574, 0, 576,
    1033, 1040, 0, 576, 1042, 1040, 0, 1043, 946, 132, 0, 1045, 256, 3575, 1046, 1047,
    197, 3576, 3586, 0, 408, 833, 408, 3596, 0, 196, 256, 3609, 58, 1050, 1053, 0,
    3607, 3606, 288, 96, 324, 1055, 3610, 0, 969, 256, 967, 970, 971, 972, 914, 3611,
    0, 3625, 3624, 288, 96, 324, 3621, 1055, 3627, 0, 1056, 1057, 3640, 3638, 3637, 1053,
    666, 3643, 0, 970, 3656, 3653, 324, 3657, 0, 969, 256, 967, 971, 972, 914, 3658,
    3668, 3678, 3688, 3698, 3708, 3718, 0, 3728, 3738, 3748, 3758, 3768, 3778, 3788, 3798, 3805,
    3815, 3825, 0, 1077, 880, 3835, 0, 1077, 500, 880, 3845, 0, 3855, 1078, 1079, 1080,
    3856, 3866, 3876, 3886, 3896, 3906, 3916, 0, 500, 61, 3932, 3929, 3928, 3926, 856, 431,
    203, 3933, 3943, 3953, 0, 226, 3963, 1087, 1088, 3964, 3974, 3984, 3994, 4004, 4014, 4024,
    4034, 4044, 4054, 4064, 4074, 4084, 4094, 0, 145, 1099, 1101, 239, 4104, 431, 0, 160,
    431, 1103, 4105, 4115, 4125, 4135, 4145, 4155, 4165, 4175, 4185, 4195, 4205, 4215, 4225, 4235,
    4245, 0, 751, 4257, 0, 96, 4255, 483, 731, 483, 0, 483, 4256, 483, 731, 4258,
    4268, 4278, 4288, 896, 1116, 1118, 895, 1119, 0, 863, 500, 1001, 90, 4289, 0, 4299,
    4309, 4319, 4329, 4350, 0, 1124, 1125, 994, 4339, 1127, 0, 4343, 1129, 4346, 1132, 1134,
    4347, 0, 1135, 1125, 994, 1129, 0, 1136, 994, 4351, 0, 1137, 0, 0, 163, 4361,
    4367, 4364, 324, 4368, 4372, 1139, 0, 4371, 1140, 1141, 1142, 1143, 0, 1145, 4375, 4373,
    226, 4378, 4388, 4398, 4408, 4418, 4428, 4438, 4446, 4456, 4466, 4476, 4486, 4496, 4506, 4516,
    4526, 4536, 4546, 4556, 4566, 4576, 4586, 4596, 4606, 4616, 4626, 4636, 0, 4646, 4656, 4666,
    4676, 4686, 4696, 4706, 0, 406, 833, 406, 4716, 0, 0, 500, 1161, 856, 0, 1162,
    857, 4726, 620, 1163, 1164, 4727, 4737, 0, 1165, 4752, 4749, 4748, 96, 324, 58, 4754,
    0, 977, 4753, 96, 324, 0, 86, 96, 4755, 4765, 4775, 0, 4785, 4795, 483, 96,
    895, 875, 896, 897, 1169, 1171, 1173, 4796, 4805, 4812, 4822, 0, 4832, 4842, 4852, 4862,
    4872, 4882, 4892, 4902, 4912, 4922, 4932, 4942, 4952, 4962, 4972, 4982, 0, 1194, 690, 226,
    4995, 0, 605, 77, 0, 86, 4992, 431, 4998, 5008, 0, 5018, 0, 5028, 5038, 5048,
    5058, 5068, 5078, 5088, 5098, 5108, 5118, 5128, 144, 0, 145, 160, 1021, 0, 1206, 1101,
    5138, 5148, 1207, 1209, 0, 651, 1211, 0, 870, 324, 1212, 5152, 1214, 1216, 0, 651,
    500, 1211, 0, 870, 324, 1212, 1218, 0, 1220, 5153, 239, 1221, 1145, 5157, 239, 1221,
    0, 5154, 431, 241, 0, 203, 241, 5158, 1132, 5161, 1222, 0, 157, 1224, 5164, 5174,
    5184, 5194, 5204, 5214, 5224, 5234, 5244, 5254, 5263, 5273, 5283, 5293, 5303, 5313, 5323, 5333,
    5343, 5353, 5363, 5373, 5383, 5392, 5402, 5412, 5422, 5432, 5442, 5451, 5461, 5471, 5481, 5491,
    0, 226, 293, 5504, 226, 5501, 1248, 0, 972, 5505, 1250, 5507, 5517, 5527, 5538, 0,
    1256, 550, 5548, 5558, 5568, 5578, 0, 5587, 5597, 5607, 5614, 5624, 5634, 5644, 5654, 5664,
    5674, 5684, 5694, 5704, 5714, 5724, 5734, 5744, 5754, 5764, 5774, 5784, 5794, 5804, 5814, 5824,
    5834, 5844, 5854, 5864, 5874, 5893, 1021, 1272, 5884, 900, 5887, 1273, 5890, 1275, 1276, 0,
    5894, 852, 850, 854, 1277, 5899, 1279, 5898, 5895, 900, 1275, 5900, 5910, 5920, 5930, 0,
    5940, 5950, 0, 5960, 0, 1285, 833, 0, 406, 408, 5961, 0, 5971, 5981, 5991, 6001,
    6011, 6021, 6031, 6042, 6052, 6062, 6072, 6082, 6092, 6103, 0, 483, 728, 332, 6124, 6113,
    1294, 6115, 6127, 690, 0, 690, 690, 6128, 163, 293, 1145, 86, 1297, 1298, 1299, 451,
    728, 924, 987, 6129, 6139, 6149, 6159, 6169, 6179, 0, 977, 6189, 96, 324, 637, 6190,
    637, 918, 6191, 6201, 6211, 6221, 6231, 6241, 6251, 6261, 6271, 6281, 6291, 6301, 6311, 6321,
    6331, 6341, 6351, 6361, 6371, 6381, 6391, 0, 630, 1317, 0, 969, 256, 6403, 0, 6400,
    288, 96, 324, 6401, 1087, 1088, 0, 1080, 6402, 286, 1319, 1320, 967, 945, 6405, 967,
    970, 971, 972, 945, 1007, 914, 6406, 6416, 969, 1322, 6417, 6427, 6437, 6447, 6457, 6467,
    6477, 0, 1008, 1329, 6490, 226, 6487, 1248, 6491, 0, 1331, 6504, 0, 6501, 897, 86,
    897, 288, 6505, 6515, 6525, 6535, 6545, 6555, 6565, 6575, 6585, 6595, 6604, 6613, 6623, 6633,
    6643, 6653, 6663, 6673, 6683, 0, 1349, 256, 6698, 6695, 6693, 324, 6699, 6709, 6719, 6729,
    6751, 0, 1353, 6749, 6739, 1353, 483, 0, 6752, 6762, 6772, 6782, 6792, 6802, 6812, 6821,
    6831, 6841, 6851, 6860, 6870, 6880, 6889, 1365, 1367, 0, 6890, 6900, 0, 6910, 6920, 6930,
    6940, 6950, 6960, 6970, 0, 1375, 939, 6980, 6990, 7000, 7010, 7020, 7030, 7040, 7050, 7060,
    7070, 7080, 7090, 0, 874, 96, 7100, 0, 400, 7110, 58, 1015, 1383, 7111, 431, 7114,
    7124, 7134, 7145, 7156, 7166, 7176, 7186, 7196, 7205, 7214, 7224, 7233, 7242, 7252, 7262, 7272,
    7282, 7292, 7302, 7312, 0, 151, 239, 160, 1399, 7322, 0, 163, 144, 1400, 0, 163,
    157, 1400, 0, 1401, 7326, 1140, 1403, 1404, 7327, 7337, 7347, 7357, 810, 811, 0, 1405,
    501, 0, 1406, 1407, 0, 1405, 504, 7358, 7368, 7378, 7388, 7398, 7408, 7418, 7428, 7438,
    7448, 7458, 7468, 7478, 7488, 7498, 7508, 7518, 7528, 7538, 7548, 7558, 7568, 7578, 7588, 7598,
    7608, 7618, 7628, 7638, 7648, 7658, 7668, 7678, 7688, 7698, 0, 1436, 0, 1033, 1040, 587,
    7708, 0, 1034, 163, 1036, 0, 48, 199, 0, 148, 163, 124, 197, 197, 90, 1038,
    812, 7709, 7719, 7729, 226, 7739, 7749, 7759, 7767, 7777, 7787, 7797, 7807, 7817, 7827, 7837,
    7847, 7857, 7867, 7877, 7887, 7897, 7907, 0, 1449, 7917, 1451, 1452, 7918, 96, 0, 7921,
    1453, 7922, 96, 7925, 7935, 7945, 7955, 7965, 7975, 7985, 7995, 8005, 8015, 8025, 8035, 8045,
    8055, 8065, 8075, 8085, 1467, 0, 332, 241, 8086, 1001, 58, 0, 152, 1469, 8087, 1471,
    1473, 8099, 90, 8089, 930, 0, 969, 256, 967, 1331, 973, 914, 8100, 0, 1474, 776,
    778, 1476, 1478, 906, 1480, 217, 1482, 1484, 967, 1480, 973, 655, 975, 8121, 8113, 8110,
    1273, 8117, 8114, 1486, 1487, 8118, 431, 8122, 8132, 8141, 8151, 8161, 8171, 0, 1491, 96,
    0, 1406, 1491, 8172, 0, 1492, 96, 0, 1406, 1492, 8173, 8183, 8193, 8203, 8213, 8223,
    0, 1405, 8233, 1496, 0, 288, 277, 8235, 280, 378, 203, 288, 324, 1021, 504, 0,
    1275, 8236, 0, 973, 1275, 217, 967, 1480, 8240, 8251, 1498, 0, 8250, 1499, 1500, 58,
    1501, 0, 1405, 46, 589, 609, 8252, 8262, 8272, 8282, 8292, 0, 8302, 881, 882, 8303,
    0, 1297, 382, 884, 8307, 8317, 8327, 8337, 1508, 1509, 8338, 8348, 8358, 281, 1014, 690,
    8359, 226, 0, 226, 484, 1002, 8361, 8372, 0, 697, 208, 8382, 8391, 8400, 8410, 8420,
    8430, 8440, 8450, 8460, 8470, 8480, 8490, 8500, 8510, 8520, 8530, 8540, 8550, 8560, 8570, 8580,
    8590, 8600, 8610, 8620, 8630, 8640, 8650, 8660, 8670, 8680, 8690, 8700, 8710, 8719, 8729, 8739,
    8749, 8759, 431, 8769, 8779, 8789, 8799, 8809, 8819, 8829, 8839, 8849, 8859, 8869, 8879, 1553,
    8880, 8892, 8890, 0, 332, 241, 0, 8895, 8905, 8915, 8925, 8935, 8945, 8955, 8965, 8975,
    8985, 8992, 9002, 9012, 9022, 9032, 9042, 0, 0, 497, 77, 9052, 0, 598, 160, 0,
    507, 160, 0, 1569, 160, 512, 0, 9059, 1021, 378, 203, 239, 9053, 431, 9056, 1277,
    9060, 9070, 9080, 9090, 9100, 0, 9110, 1571, 451, 1572, 0, 9114, 1573, 1572, 9118, 9128,
    9138, 9148, 0, 9158, 153, 154, 9159, 9169, 1575, 593, 9170, 9180, 9190, 9200, 9210, 9220,
    9230, 9240, 9250, 9260, 9270, 9280, 9290, 9300, 9309, 0, 9319, 749, 750, 692, 0, 9320,
    576, 1583, 749, 1584, 9321, 0, 1586, 749, 1584, 324, 9325, 9335, 9345, 9355, 9365, 9375,
    9385, 9395, 9405, 9415, 9425, 9435, 9445, 9455, 9465, 0, 9475, 612, 1597, 1598, 1599, 447,
    9476, 1601, 1602, 0, 1603, 226, 0, 870, 1406, 1604, 9477, 9487, 9497, 9507, 9517, 9527,
    9537, 9547, 9557, 9567, 9577, 9587, 9597, 9607, 9617, 9627, 1618, 1619, 1620, 9628, 9638, 9649,
    0, 156, 165, 239, 9659, 9669, 9681, 9679, 332, 9684, 0, 697, 1169, 0, 820, 1623,
    9694, 9704, 9714, 9724, 0, 9734, 1627, 1628, 1629, 9735, 9745, 9755, 0, 1633, 9765, 1001,
    58, 9766, 0, 598, 9776, 9786, 0, 9796, 9806, 9816, 9826, 0, 1474, 776, 778, 1476,
    1478, 906, 1480, 973, 655, 975, 217, 967, 0, 1275, 9836, 0, 973, 1275, 9840, 9850,
    9860, 0, 157, 239, 241, 1405, 1224, 776, 778, 9870, 9880, 9890, 9900, 0, 139, 590,
    9910, 9920, 226, 9930, 9940, 9950, 9960, 9970, 9980, 9990, 10000, 10010, 10020, 10030, 10040, 10050,
    10060, 0, 10070, 1654, 769, 10077, 0, 637, 10074, 10072, 10071, 504, 280, 324, 10079, 10089,
    10099, 10109, 10119, 10129, 10139, 10149, 10159, 10169, 10179, 10189, 10199, 10209, 10219, 10229, 10239, 10249,
    10259, 10269, 0, 651, 0, 324, 10279, 1212, 10282, 324, 10285, 10288, 10291, 10301, 0, 930,
    10314, 10311, 324, 10315, 967, 0, 196, 256, 0, 969, 256, 970, 971, 972, 1331, 914,
    10316, 10326, 10336, 10343, 0, 84, 0, 10354, 10353, 0, 226, 484, 0, 1002, 690, 1002,
    10358, 0, 226, 10368, 1349, 969, 1675, 217, 1677, 0, 576, 1679, 10369, 10372, 1681, 152,
    10374, 1683, 0, 576, 1679, 10377, 10380, 1681, 152, 10382, 1487, 0, 576, 0, 1406, 55,
    0, 10385, 0, 576, 1406, 46, 1684, 10395, 10405, 10415, 10422, 10432, 10442, 10452, 10462, 10472,
    10482, 10492, 10502, 10512, 10522, 10532, 10542,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 80, 83, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 162, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 194, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 78, 4294967295, 200, 4294967295, 204, 4294967295,
    4294967295, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 219, 4294967295, 220, 225, 4294967295, 4294967295,
    4294967295, 4294967295, 228, 4294967295, 4294967295, 4294967295, 236, 4294967295, 4294967295, 240, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 267, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 301, 301, 4294967295, 301, 301, 4294967295,
    4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 381, 204, 289, 325, 4294967295, 383, 4294967295, 4294967295, 4294967295, 384,
    3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 301, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 419, 4294967295, 424, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 228, 4294967295, 4294967295, 430, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 220, 4294967295, 444, 4294967295, 4294967295, 450, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 444,
    4294967295, 4294967295, 450, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 495, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295,
    78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 515, 4294967295, 4294967295, 518, 4294967295, 4294967295, 4294967295, 383, 4294967295, 4294967295, 4294967295, 521,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 529, 4294967295, 4294967295, 4294967295, 279, 279, 279, 279, 279, 279, 4294967295, 4294967295, 279,
    4294967295, 301, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 383, 4294967295, 383, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 83, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 591, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 83, 4294967295, 4294967295, 383, 597, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 602, 4294967295, 240, 4294967295, 4294967295, 4294967295, 604, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 610, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 267, 4294967295, 614, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 619, 284, 515, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 622, 4294967295, 4294967295, 4294967295,
    624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 654,
    4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 656,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 80, 4294967295, 667, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 693, 4294967295, 4294967295, 4294967295, 698, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 777, 780, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 383, 4294967295, 383, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 591, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 602, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 610,
    4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 624, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295,
    4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295,
    4294967295, 821, 823, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 851, 853, 855, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 883, 4294967295,
    885, 495, 887, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295,
    289, 204, 890, 325, 4294967295, 85, 4294967295, 4294967295, 821, 894, 4294967295, 4294967295, 325, 4294967295, 383, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 890, 4294967295, 4294967295, 4294967295, 4294967295, 901, 4294967295, 381, 903, 905, 289,
    908, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 926,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 381, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 958, 4294967295, 4294967295, 968, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 974, 656, 976, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 980, 982, 242, 4294967295, 4294967295, 986, 4294967295, 989, 4294967295, 4294967295, 4294967295, 242, 986, 4294967295,
    4294967295, 4294967295, 4294967295, 989, 4294967295, 4294967295, 4294967295, 4294967295, 993, 995, 4294967295, 958, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 220, 4294967295, 4294967295, 4294967295, 989, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295,
    4294967295, 4294967295, 325, 4294967295, 325, 4294967295, 325, 4294967295, 383, 4294967295, 4294967295, 325, 4294967295, 4294967295, 325, 4294967295,
    4294967295, 4294967295, 325, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1005, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 604, 903, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 419, 4294967295, 4294967295, 4294967295,
    419, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 383, 4294967295, 4294967295, 4294967295, 4294967295, 383, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 80, 654, 4294967295, 4294967295, 4294967295, 80, 4294967295, 80, 80, 1022, 654, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1035,
    4294967295, 1037, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 198, 78, 4294967295, 1039, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1052, 1054, 4294967295,
    4294967295, 4294967295, 289, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 968, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 289, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1054,
    667, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 968, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1072, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 995, 4294967295, 4294967295, 4294967295, 220, 1130, 4294967295, 1133, 4294967295,
    236, 4294967295, 4294967295, 4294967295, 995, 1130, 4294967295, 4294967295, 995, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 220,
    4294967295, 4294967295, 325, 236, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 97, 4294967295, 4294967295, 1170, 1172, 1175, 746, 746, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1022, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1208, 1210, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 1215, 1217, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 1219, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 240, 4294967295,
    4294967295, 4294967295, 4294967295, 242, 4294967295, 204, 242, 220, 1133, 236, 1223, 4294967295, 4294967295, 1226, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1249, 4294967295, 4294967295, 4294967295, 1251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1261, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1022, 901, 4294967295, 901, 4294967295, 1274, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 853, 851, 855, 1278, 4294967295, 424, 4294967295, 4294967295, 901, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    289, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1321, 968, 4294967295, 4294967295, 968,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 289, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1366, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1035, 4294967295, 1037, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 198, 198, 4294967295, 1039,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1468, 4294967295, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1470, 4294967295, 1472,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 968, 4294967295, 974, 4294967295, 4294967295, 4294967295, 1475, 777,
    780, 1477, 1479, 908, 1481, 219, 1483, 1485, 968, 1481, 974, 656, 976, 4294967295, 4294967295, 4294967295,
    1274, 4294967295, 4294967295, 1274, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1497, 4294967295, 289, 279, 4294967295, 4294967295, 4294967295, 204, 289, 325, 1022, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 974, 4294967295, 219, 968, 1481, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 610, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 883, 4294967295,
    4294967295, 4294967295, 383, 885, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 698, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 301,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    231, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1022, 4294967295, 204, 240, 4294967295, 4294967295, 4294967295, 1278,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 693, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 698, 1170, 4294967295, 821, 823,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1475, 777, 780, 1477,
    1479, 908, 1481, 974, 656, 976, 219, 968, 4294967295, 4294967295, 4294967295, 4294967295, 974, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 240, 242, 4294967295, 1226, 777, 780, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 591,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 220, 4294967295, 220, 325, 236, 236, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 325, 4294967295, 968, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1672, 4294967295, 4294967295, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1676, 219, 1678, 4294967295, 4294967295, 4294967295, 220, 4294967295, 1682, 4294967295,
    236, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 1682, 4294967295, 236, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 5, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 66, 4294967295, 4294967295, 4294967295, 4294967295, 66, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 79, 81, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 161, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 191, 193, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 77, 4294967295, 199, 4294967295, 203, 4294967295,
    4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 217, 4294967295, 221, 223, 4294967295, 4294967295,
    4294967295, 4294967295, 227, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 239, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 266, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 283, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 319, 320, 4294967295, 322, 323, 4294967295,
    4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 379, 203, 288, 324, 4294967295, 382, 4294967295, 4294967295, 4294967295, 385,
    386, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 302, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 420, 4294967295, 421, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 227, 4294967295, 4294967295, 429, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 441, 4294967295, 443, 4294967295, 4294967295, 449, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 443,
    4294967295, 4294967295, 449, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 494, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 513, 4294967295, 4294967295, 517, 4294967295, 4294967295, 4294967295, 382, 4294967295, 4294967295, 4294967295, 520,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 530, 4294967295, 4294967295, 4294967295, 277, 533, 534, 535, 536, 537, 4294967295, 4294967295, 540,
    4294967295, 542, 543, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 382, 4294967295, 382, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 81, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 590, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 81, 4294967295, 4294967295, 382, 596, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 601, 4294967295, 239, 4294967295, 4294967295, 4294967295, 603, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 609, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 266, 4294967295, 613, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 618, 283, 513, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 621, 4294967295, 4294967295, 4294967295,
    623, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 653,
    4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 655,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 664, 665, 4294967295, 666, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 692, 4294967295, 4294967295, 4294967295, 697, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 776, 778, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 382, 4294967295, 382, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 590, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 601, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 609,
    4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 623, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295,
    4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295,
    4294967295, 820, 822, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 850, 852, 854, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 875, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 875, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 882, 4294967295,
    884, 494, 886, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295,
    288, 203, 889, 324, 4294967295, 892, 4294967295, 4294967295, 820, 893, 4294967295, 4294967295, 324, 4294967295, 382, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 889, 4294967295, 4294967295, 4294967295, 4294967295, 900, 4294967295, 379, 902, 904, 288,
    906, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 925,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 957, 4294967295, 4294967295, 967, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 973, 655, 975, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 979, 981, 241, 4294967295, 4294967295, 984, 4294967295, 988, 4294967295, 4294967295, 4294967295, 241, 984, 4294967295,
    4294967295, 4294967295, 4294967295, 988, 4294967295, 4294967295, 4294967295, 4294967295, 991, 994, 4294967295, 957, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 988, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295,
    4294967295, 4294967295, 324, 4294967295, 324, 4294967295, 324, 4294967295, 382, 4294967295, 4294967295, 324, 4294967295, 4294967295, 324, 4294967295,
    4294967295, 4294967295, 324, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1004, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 603, 1011, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 420, 4294967295, 4294967295, 4294967295,
    420, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 382, 4294967295, 4294967295, 4294967295, 4294967295, 382, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1017, 1018, 4294967295, 4294967295, 4294967295, 79, 4294967295, 1017, 1020, 1021, 653, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1034,
    4294967295, 1036, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 197, 77, 4294967295, 1038, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1050, 1053, 4294967295,
    4294967295, 4294967295, 288, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 967, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 288, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1053,
    666, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 967, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1073, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 994, 4294967295, 4294967295, 4294967295, 221, 1129, 4294967295, 1132, 4294967295,
    237, 4294967295, 4294967295, 4294967295, 994, 1129, 4294967295, 4294967295, 994, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441,
    4294967295, 4294967295, 324, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 875, 4294967295, 4294967295, 1169, 1171, 1173, 912, 747, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1021, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1207, 1209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 1214, 1216, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 1218, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 239, 4294967295,
    4294967295, 4294967295, 4294967295, 241, 4294967295, 203, 241, 221, 1132, 237, 1222, 4294967295, 4294967295, 1224, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1248, 4294967295, 4294967295, 4294967295, 1250, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1262, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1021, 1272, 4294967295, 900, 4294967295, 1273, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 852, 850, 854, 1277, 4294967295, 1279, 4294967295, 4294967295, 900, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    288, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1320, 967, 4294967295, 4294967295, 967,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1365, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1034, 4294967295, 1036, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 197, 197, 4294967295, 1038,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1467, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1469, 4294967295, 1471,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 967, 4294967295, 973, 4294967295, 4294967295, 4294967295, 1474, 776,
    778, 1476, 1478, 906, 1480, 217, 1482, 1484, 967, 1480, 973, 655, 975, 4294967295, 4294967295, 4294967295,
    1273, 4294967295, 4294967295, 1486, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1496, 4294967295, 288, 277, 4294967295, 4294967295, 4294967295, 203, 288, 324, 1021, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 973, 4294967295, 217, 967, 1480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 609, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 882, 4294967295,
    4294967295, 4294967295, 382, 884, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 697, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1553,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    230, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1021, 4294967295, 203, 239, 4294967295, 4294967295, 4294967295, 1277,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 692, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 697, 1169, 4294967295, 820, 1623,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1474, 776, 778, 1476,
    1478, 906, 1480, 973, 655, 975, 217, 967, 4294967295, 4294967295, 4294967295, 4294967295, 973, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 239, 241, 4294967295, 1224, 776, 778, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 590,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 324, 221, 4294967295, 441, 324, 237, 237, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 324, 4294967295, 967, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1673, 4294967295, 4294967295, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1675, 217, 1677, 4294967295, 4294967295, 4294967295, 221, 4294967295, 1681, 4294967295,
    237, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 1681, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1012, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (1998, 1),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (38, 0, 2),
    (41, 2, 2),
    (46, 4, 2),
    (53, 6, 2),
    (55, 8, 2),
    (56, 10, 2),
    (57, 12, 3),
    (99, 15, 2),
    (135, 17, 2),
    (184, 19, 2),
    (185, 21, 2),
    (190, 23, 2),
    (194, 25, 3),
    (198, 28, 3),
    (202, 31, 2),
    (204, 33, 3),
    (206, 36, 2),
    (209, 38, 2),
    (218, 40, 2),
    (220, 42, 4),
    (221, 46, 2),
    (226, 48, 2),
    (230, 50, 5),
    (233, 55, 3),
    (234, 58, 2),
    (258, 60, 3),
    (260, 63, 2),
    (269, 65, 2),
    (278, 67, 2),
    (284, 69, 3),
    (292, 72, 3),
    (296, 75, 3),
    (314, 78, 2),
    (315, 80, 2),
    (317, 82, 2),
    (318, 84, 2),
    (323, 86, 2),
    (358, 88, 2),
    (359, 90, 2),
    (360, 92, 3),
    (361, 95, 2),
    (363, 97, 2),
    (367, 99, 2),
    (368, 101, 1),
    (380, 102, 3),
    (409, 105, 3),
    (410, 108, 2),
    (412, 110, 2),
    (419, 112, 2),
    (422, 114, 2),
    (435, 116, 4),
    (437, 120, 2),
    (440, 122, 2),
    (445, 124, 2),
    (447, 126, 2),
    (450, 128, 2),
    (487, 130, 2),
    (505, 132, 2),
    (507, 134, 2),
    (512, 136, 2),
    (520, 138, 2),
    (532, 140, 2),
    (535, 142, 3),
    (539, 145, 2),
    (543, 147, 2),
    (563, 149, 3),
    (567, 152, 2),
    (568, 154, 2),
    (569, 156, 2),
    (570, 158, 2),
    (571, 160, 2),
    (572, 162, 2),
    (575, 164, 3),
    (577, 167, 2),
    (578, 169, 2),
    (615, 171, 2),
    (617, 173, 2),
    (630, 175, 2),
    (631, 177, 2),
    (634, 179, 2),
    (642, 181, 3),
    (649, 184, 2),
    (650, 186, 2),
    (653, 188, 2),
    (654, 190, 2),
    (661, 192, 3),
    (663, 195, 3),
    (667, 198, 2),
    (674, 200, 2),
    (681, 202, 2),
    (693, 204, 3),
    (698, 207, 3),
    (722, 210, 3),
    (724, 213, 2),
    (731, 215, 3),
    (737, 218, 2),
    (738, 220, 2),
    (739, 222, 2),
    (747, 224, 3),
    (748, 227, 2),
    (752, 229, 3),
    (767, 232, 3),
    (773, 235, 3),
    (787, 238, 3),
    (810, 241, 2),
    (815, 243, 2),
    (819, 245, 3),
    (825, 248, 3),
    (831, 251, 2),
    (841, 253, 2),
    (842, 255, 2),
    (844, 257, 2),
    (886, 259, 2),
    (890, 261, 2),
    (961, 263, 2),
    (962, 265, 2),
    (976, 267, 2),
    (1003, 269, 2),
    (1005, 271, 2),
    (1011, 273, 3),
    (1017, 276, 2),
    (1022, 278, 3),
    (1029, 281, 2),
    (1039, 283, 3),
    (1044, 286, 3),
    (1051, 289, 3),
    (1057, 292, 3),
    (1062, 295, 2),
    (1068, 297, 3),
    (1073, 300, 3),
    (1084, 303, 2),
    (1089, 305, 2),
    (1090, 307, 2),
    (1122, 309, 2),
    (1123, 311, 2),
    (1124, 313, 2),
    (1155, 315, 2),
    (1172, 317, 2),
    (1182, 319, 2),
    (1184, 321, 2),
    (1185, 323, 2),
    (1186, 325, 2),
    (1191, 327, 2),
    (1195, 329, 2),
    (1200, 331, 3),
    (1201, 334, 2),
    (1202, 336, 2),
    (1203, 338, 2),
    (1205, 340, 3),
    (1208, 343, 2),
    (1209, 345, 2),
    (1212, 347, 2),
    (1214, 349, 2),
    (1221, 351, 2),
    (1226, 353, 2),
    (1228, 355, 2),
    (1229, 357, 2),
    (1230, 359, 2),
    (1231, 361, 3),
    (1232, 364, 2),
    (1246, 366, 2),
    (1263, 368, 2),
    (1274, 370, 2),
    (1304, 372, 2),
    (1306, 374, 3),
    (1323, 377, 2),
    (1326, 379, 2),
    (1336, 381, 2),
    (1337, 383, 2),
    (1338, 385, 2),
    (1346, 387, 2),
    (1347, 389, 2),
    (1348, 395, 2),
    (1351, 391, 2),
    (1353, 393, 2),
    (1357, 397, 2),
    (1358, 399, 2),
    (1363, 401, 2),
    (1368, 403, 2),
    (1369, 405, 2),
    (1371, 407, 2),
    (1382, 409, 2),
    (1394, 411, 2),
    (1396, 413, 2),
    (1398, 415, 2),
    (1400, 417, 2),
    (1403, 419, 2),
    (1406, 421, 2),
    (1410, 423, 2),
    (1412, 425, 2),
    (1419, 427, 2),
    (1427, 429, 3),
    (1432, 432, 3),
    (1434, 435, 2),
    (1435, 437, 2),
    (1452, 439, 2),
    (1456, 441, 2),
    (1464, 443, 2),
    (1469, 445, 2),
    (1475, 447, 2),
    (1476, 449, 2),
    (1480, 457, 2),
    (1482, 451, 2),
    (1483, 453, 2),
    (1484, 455, 2),
    (1485, 459, 2),
    (1503, 461, 2),
    (1505, 463, 2),
    (1508, 465, 3),
    (1513, 468, 3),
    (1514, 471, 2),
    (1516, 473, 2),
    (1536, 475, 3),
    (1549, 478, 2),
    (1550, 480, 2),
    (1554, 482, 3),
    (1556, 485, 2),
    (1562, 487, 2),
    (1571, 489, 3),
    (1573, 492, 2),
    (1583, 494, 2),
    (1584, 496, 2),
    (1590, 498, 2),
    (1595, 500, 2),
    (1614, 502, 3),
    (1648, 505, 2),
    (1675, 507, 3),
    (1735, 510, 2),
    (1740, 512, 2),
    (1742, 514, 2),
    (1748, 516, 2),
    (1749, 518, 2),
    (1752, 520, 2),
    (1762, 522, 2),
    (1783, 524, 4),
    (1837, 528, 2),
    (1844, 530, 2),
    (1857, 532, 2),
    (1860, 534, 2),
    (1861, 536, 2),
    (1862, 538, 2),
    (1863, 540, 5),
    (1864, 545, 3),
    (1891, 548, 2),
    (1916, 550, 2),
    (1922, 552, 2),
    (1923, 554, 2),
    (1929, 556, 2),
    (1932, 558, 2),
    (1933, 560, 2),
    (1940, 562, 2),
    (1942, 564, 2),
    (1946, 566, 3),
    (1950, 569, 3),
    (1955, 572, 2),
    (1957, 574, 2),
    (1958, 576, 2),
    (1960, 578, 2),
    (1962, 580, 2),
    (1965, 582, 2),
    (2006, 584, 2),
    (2010, 586, 2),
    (2025, 588, 3),
    (2054, 591, 2),
    (2055, 593, 2),
    (2057, 595, 2),
    (2059, 597, 2),
    (2065, 599, 2),
    (2066, 601, 2),
    (2067, 603, 2),
    (2068, 605, 2),
    (2070, 607, 2),
    (2073, 609, 2),
    (2112, 611, 2),
    (2141, 613, 2),
    (2176, 615, 3),
    (2178, 618, 2),
    (2187, 620, 2),
    (2188, 622, 2),
    (2191, 624, 2),
    (2215, 626, 2),
    (2225, 628, 3),
    (2251, 631, 2),
    (2279, 633, 2),
    (2342, 635, 3),
    (2418, 638, 2),
    (2420, 640, 2),
    (2423, 642, 3),
    (2428, 645, 3),
    (2429, 648, 3),
    (2431, 651, 2),
    (2484, 653, 2),
    (2487, 655, 2),
    (2493, 657, 2),
    (2495, 659, 2),
    (2504, 661, 2),
    (2506, 663, 2),
    (2510, 665, 2),
    (2511, 667, 2),
    (2512, 669, 2),
    (2513, 671, 2),
    (2514, 673, 2),
    (2515, 675, 2),
    (2516, 677, 2),
    (2517, 679, 2),
    (2518, 681, 2),
    (2519, 683, 2),
    (2520, 685, 2),
    (2521, 687, 2),
    (2522, 689, 2),
    (2523, 691, 2),
    (2524, 693, 2),
    (2528, 695, 2),
    (2531, 697, 2),
    (2563, 699, 2),
    (2565, 701, 3),
    (2566, 704, 2),
    (2570, 706, 2),
    (2571, 708, 3),
    (2572, 711, 2),
    (2573, 713, 2),
    (2579, 715, 2),
    (2581, 717, 2),
    (2582, 719, 2),
    (2583, 721, 2),
    (2597, 723, 3),
    (2606, 726, 2),
    (2610, 728, 2),
    (2611, 730, 2),
    (2633, 732, 2),
    (2634, 734, 2),
    (2687, 736, 2),
    (2693, 738, 2),
    (2704, 740, 3),
    (2714, 743, 2),
    (2728, 745, 2),
    (2730, 747, 2),
    (2731, 749, 3),
    (2735, 752, 2),
    (2781, 754, 2),
    (2793, 756, 2),
    (2851, 758, 3),
    (2859, 761, 2),
    (2860, 763, 2),
    (2862, 765, 2),
    (2863, 767, 2),
    (2892, 769, 2),
    (2893, 771, 2),
    (2894, 773, 2),
    (2895, 775, 2),
    (2896, 777, 2),
    (2897, 779, 2),
    (2898, 781, 2),
    (2899, 783, 2),
    (2900, 785, 2),
    (2901, 787, 2),
    (2902, 789, 2),
    (2903, 791, 2),
    (2908, 793, 2),
    (2915, 795, 3),
    (2916, 798, 2),
    (2918, 800, 2),
    (2919, 802, 2),
    (2920, 804, 2),
    (2927, 806, 3),
    (2957, 809, 2),
    (2981, 811, 2),
    (2985, 813, 2),
    (2994, 815, 2),
    (2996, 817, 2),
    (3010, 819, 3),
    (3013, 822, 3),
    (3030, 825, 2),
    (3031, 827, 2),
    (3032, 829, 2),
    (3038, 831, 2),
    (3047, 833, 2),
    (3065, 835, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    65, 3, 76, 3, 76, 3, 3, 78, 3, 80, 3, 83, 3, 78, 85, 65,
    3, 162, 3, 3, 192, 3, 194, 3, 78, 3, 78, 198, 3, 78, 200, 3,
    78, 3, 78, 200, 3, 204, 209, 3, 3, 219, 3, 220, 222, 4, 225, 3,
    3, 228, 3, 236, 220, 222, 4, 3, 78, 240, 242, 3, 3, 78, 267, 269,
    3, 3, 279, 3, 284, 3, 289, 78, 3, 78, 240, 3, 78, 240, 3, 301,
    3, 301, 3, 301, 3, 301, 3, 325, 3, 381, 3, 204, 3, 289, 78, 3,
    325, 3, 383, 3, 384, 3, 3, 301, 4, 3, 289, 78, 3, 419, 3, 424,
    3, 228, 3, 430, 3, 220, 222, 4, 3, 444, 3, 450, 3, 325, 3, 444,
    3, 450, 3, 4, 3, 495, 3, 204, 3, 78, 3, 325, 3, 515, 3, 518,
    78, 3, 383, 3, 521, 3, 529, 4, 3, 279, 3, 279, 3, 279, 3, 279,
    3, 279, 3, 279, 3, 279, 4, 3, 301, 3, 279, 3, 383, 3, 383, 3,
    80, 3, 83, 3, 78, 3, 591, 78, 3, 80, 3, 83, 3, 383, 3, 597,
    3, 602, 78, 3, 78, 240, 3, 604, 3, 78, 3, 279, 3, 78, 200, 3,
    78, 610, 3, 78, 267, 3, 614, 3, 78, 240, 3, 619, 3, 284, 3, 515,
    3, 78, 240, 3, 622, 3, 78, 624, 3, 289, 78, 3, 78, 240, 3, 78,
    240, 3, 325, 3, 654, 3, 78, 200, 3, 78, 240, 3, 656, 3, 80, 3,
    80, 3, 667, 3, 693, 698, 3, 3, 777, 3, 780, 3, 78, 3, 383, 3,
    383, 3, 591, 78, 3, 80, 3, 602, 78, 3, 78, 3, 78, 610, 3, 78,
    267, 3, 78, 240, 3, 78, 624, 3, 78, 3, 78, 200, 3, 78, 240, 3,
    325, 3, 821, 3, 823, 3, 851, 3, 853, 3, 855, 3, 97, 3, 97, 3,
    883, 3, 885, 3, 495, 3, 887, 3, 204, 3, 325, 3, 289, 78, 3, 204,
    3, 890, 3, 325, 3, 78, 85, 3, 821, 3, 894, 3, 325, 3, 383, 3,
    890, 3, 901, 3, 381, 3, 903, 905, 3, 3, 289, 78, 3, 908, 3, 325,
    3, 926, 3, 325, 3, 381, 3, 289, 78, 3, 958, 3, 968, 3, 974, 3,
    656, 3, 976, 3, 980, 3, 982, 3, 986, 3, 989, 242, 3, 242, 3, 3,
    986, 3, 989, 3, 993, 3, 995, 3, 958, 3, 989, 3, 325, 3, 325, 3,
    325, 3, 383, 3, 325, 3, 325, 3, 325, 3, 325, 1005, 3, 3, 289, 78,
    3, 289, 78, 3, 604, 3, 903, 3, 419, 3, 419, 3, 383, 3, 383, 3,
    80, 3, 654, 3, 80, 3, 80, 3, 1022, 3, 80, 3, 654, 1035, 3, 1037,
    3, 3, 78, 200, 3, 78, 198, 3, 78, 1039, 3, 3, 78, 198, 3, 1052,
    3, 1054, 3, 289, 78, 3, 325, 3, 968, 3, 289, 78, 3, 325, 3, 1054,
    3, 667, 3, 325, 3, 968, 3, 1072, 4, 3, 204, 3, 78, 240, 3, 995,
    3, 1130, 3, 1133, 3, 995, 3, 1130, 3, 995, 3, 325, 3, 1, 4, 5,
    3, 325, 3, 325, 3, 97, 1170, 3, 3, 1172, 3, 1175, 3, 1, 746, 4,
    5, 3, 746, 4, 3, 78, 3, 1022, 3, 1208, 3, 1210, 3, 325, 3, 1215,
    3, 1217, 3, 325, 3, 1219, 3, 78, 240, 3, 78, 240, 242, 3, 3, 204,
    242, 3, 3, 1133, 3, 1223, 3, 1226, 3, 1249, 3, 1251, 3, 1261, 4, 3,
    1022, 3, 901, 3, 901, 3, 1274, 3, 853, 3, 851, 3, 855, 3, 1278, 3,
    424, 3, 901, 3, 1295, 3, 325, 3, 289, 78, 3, 325, 3, 1321, 3, 968,
    3, 968, 3, 1249, 3, 289, 78, 3, 325, 3, 1366, 3, 78, 240, 1035, 3,
    1037, 3, 3, 78, 200, 3, 78, 198, 3, 78, 198, 1039, 3, 3, 1468, 242,
    3, 3, 1470, 3, 1472, 3, 968, 3, 974, 3, 1475, 3, 777, 3, 780, 3,
    1477, 3, 1479, 3, 908, 3, 1481, 3, 219, 3, 1483, 3, 1485, 3, 968, 3,
    1481, 3, 974, 3, 656, 3, 976, 3, 1274, 3, 1274, 3, 1497, 3, 289, 78,
    3, 279, 3, 204, 3, 289, 78, 3, 325, 3, 1022, 3, 974, 3, 219, 3,
    968, 3, 1481, 3, 78, 610, 3, 883, 3, 383, 3, 885, 698, 3, 209, 3,
    3, 301, 242, 3, 3, 4, 231, 3, 78, 3, 1022, 3, 204, 3, 78, 240,
    3, 1278, 3, 693, 3, 325, 3, 78, 240, 698, 3, 1170, 3, 3, 821, 3,
    823, 3, 1475, 3, 777, 3, 780, 3, 1477, 3, 1479, 3, 908, 3, 1481, 3,
    974, 3, 656, 3, 976, 3, 219, 3, 968, 3, 974, 3, 78, 240, 242, 3,
    3, 1226, 3, 777, 3, 780, 3, 591, 78, 3, 325, 3, 325, 3, 325, 3,
    325, 3, 968, 3, 4, 1672, 3, 78, 85, 3, 1676, 3, 219, 3, 1678, 3,
    1682, 3, 1682, 3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"[A-Z_][A-Z0-9_]*"#, // [0]
    r#"^(NOT)$"#, // [1]
    r#"[A-Z0-9_]*[A-Z][A-Z0-9_]*"#, // [2]
    r#"^(CASE|CROSS|FULL|IGNORE|INNER|INTERVAL|JOIN|LAST_90_DAYS|LAST_FISCAL_QUARTER|LAST_FISCAL_YEAR|LAST_MONTH|LAST_N_DAYS|LAST_N_FISCAL_QUARTERS|LAST_N_FISCAL_YEARS|LAST_N_MONTHS|LAST_N_QUARTERS|LAST_N_WEEKS|LAST_N_YEARS|LAST_QUARTER|LAST_WEEK|LAST_YEAR|LEFT|NATURAL|NEXT_90_DAYS|NEXT_FISCAL_QUARTER|NEXT_FISCAL_YEAR|NEXT_MONTH|NEXT_N_DAYS|NEXT_N_FISCAL_QUARTERS|NEXT_N_FISCAL_YEARS|NEXT_N_MONTHS|NEXT_N_QUARTERS|NEXT_N_WEEKS|NEXT_N_YEARS|NEXT_QUARTER|NEXT_WEEK|NEXT_YEAR|NOT|NULL|ON|ORDER|OUTER|PARTITION|RESPECT|RIGHT|ROWS|SELECT|SET|THIS_FISCAL_QUARTER|THIS_FISCAL_YEAR|THIS_MONTH|THIS_QUARTER|THIS_WEEK|THIS_YEAR|TODAY|TOMORROW|UNION|USING|YESTERDAY)$"#, // [3]
    r#"\"?[A-Z][A-Z0-9_]*\"?"#, // [4]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 10, 11, 12, 13, 14, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
    26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
    36, 37, 25, 40, 37, 13, 14, 20, 21, 28,
    12, 26, 33, 35, 17, 52, 53, 53, 52, 57,
    11, 60, 62, 19, 15, 16, 18, 22, 23, 24,
    27, 28, 29, 30, 31, 32, 34, 36, 15, 16,
    67, 22, 23, 68, 69, 60, 70, 71, 72, 73,
    74, 75, 13, 28, 82, 87, 89, 91, 92, 95,
    97, 11, 98, 99, 100, 101, 102, 103, 104, 105,
    106, 40, 107, 108, 109, 110, 111, 112, 113, 114,
    115, 116, 117, 118, 119, 120, 121, 100, 13, 20,
    26, 123, 37, 123, 98, 22, 128, 102, 34, 105,
    133, 134, 118, 134, 118, 11, 98, 99, 100, 101,
    102, 103, 104, 106, 40, 107, 108, 109, 110, 111,
    112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
    15, 16, 18, 22, 23, 137, 24, 27, 28, 29,
    30, 31, 32, 34, 36, 137, 119, 12, 28, 103,
    142, 99, 101, 104, 106, 107, 108, 109, 111, 110,
    112, 113, 114, 115, 116, 117, 120, 121, 164, 32,
    36, 30, 18, 24, 27, 29, 31, 23, 15, 16,
    177, 73, 75, 71, 67, 68, 69, 70, 72, 74,
    189, 190, 189, 190, 195, 201, 206, 207, 210, 211,
    212, 213, 214, 215, 216, 218, 224, 232, 235, 238,
    243, 244, 245, 246, 247, 248, 249, 250, 251, 252,
    253, 254, 218, 248, 257, 250, 251, 249, 251, 246,
    252, 246, 252, 247, 253, 247, 253, 254, 218, 270,
    273, 275, 278, 213, 106, 82, 215, 285, 215, 285,
    285, 291, 224, 89, 295, 296, 298, 299, 300, 303,
    304, 305, 296, 306, 307, 308, 309, 310, 311, 312,
    313, 307, 308, 309, 306, 305, 313, 312, 310, 311,
    327, 329, 331, 333, 334, 335, 336, 337, 338, 339,
    340, 341, 342, 343, 344, 346, 347, 348, 349, 350,
    351, 352, 353, 354, 355, 356, 357, 358, 359, 360,
    361, 362, 363, 364, 365, 366, 367, 368, 369, 370,
    371, 372, 373, 374, 375, 380, 387, 388, 389, 390,
    391, 392, 393, 305, 296, 306, 307, 308, 309, 310,
    278, 311, 300, 394, 395, 312, 313, 396, 300, 395,
    393, 310, 278, 311, 394, 396, 401, 402, 403, 404,
    409, 410, 395, 413, 414, 415, 415, 414, 418, 82,
    112, 422, 423, 425, 426, 427, 428, 82, 112, 422,
    423, 432, 433, 434, 435, 436, 437, 438, 442, 446,
    448, 452, 453, 455, 456, 457, 458, 459, 460, 461,
    462, 463, 464, 465, 466, 467, 468, 469, 470, 471,
    472, 473, 474, 475, 476, 477, 478, 479, 480, 481,
    482, 485, 487, 488, 489, 490, 496, 498, 499, 498,
    499, 503, 506, 510, 514, 522, 524, 526, 527, 531,
    532, 393, 310, 396, 394, 541, 544, 545, 546, 547,
    549, 551, 552, 553, 554, 557, 558, 559, 560, 561,
    562, 563, 564, 565, 566, 567, 568, 569, 570, 571,
    572, 573, 574, 432, 578, 563, 18, 579, 580, 24,
    27, 581, 563, 579, 580, 581, 563, 579, 581, 579,
    581, 580, 586, 113, 114, 594, 600, 606, 607, 246,
    247, 248, 249, 250, 251, 252, 253, 254, 218, 611,
    611, 113, 114, 616, 82, 224, 617, 496, 17, 506,
    510, 617, 215, 387, 625, 625, 629, 101, 104, 117,
    631, 586, 91, 632, 498, 633, 442, 633, 636, 638,
    640, 631, 632, 498, 631, 631, 632, 632, 644, 646,
    648, 638, 649, 649, 5, 657, 658, 659, 660, 661,
    661, 663, 663, 668, 669, 350, 670, 671, 672, 351,
    352, 673, 674, 675, 676, 677, 680, 681, 682, 685,
    392, 462, 92, 688, 688, 695, 699, 700, 701, 702,
    703, 704, 705, 706, 707, 708, 709, 710, 711, 712,
    713, 700, 701, 702, 703, 704, 705, 706, 707, 708,
    709, 710, 711, 712, 713, 729, 732, 733, 734, 735,
    736, 737, 738, 739, 740, 741, 742, 699, 380, 680,
    681, 744, 745, 744, 745, 699, 380, 680, 681, 680,
    681, 753, 755, 756, 732, 757, 758, 759, 760, 761,
    762, 733, 763, 764, 765, 766, 767, 768, 423, 770,
    771, 772, 773, 774, 775, 779, 782, 783, 784, 785,
    786, 787, 788, 789, 790, 791, 793, 794, 795, 796,
    797, 798, 799, 800, 801, 802, 803, 804, 805, 806,
    807, 436, 809, 809, 813, 814, 815, 816, 817, 818,
    819, 824, 825, 826, 827, 828, 829, 831, 832, 834,
    835, 836, 837, 838, 839, 840, 841, 842, 843, 844,
    845, 846, 847, 848, 82, 101, 104, 112, 117, 422,
    423, 432, 859, 617, 498, 860, 62, 498, 390, 498,
    307, 308, 498, 872, 873, 307, 308, 873, 872, 393,
    305, 296, 306, 307, 308, 309, 310, 278, 311, 394,
    312, 313, 396, 307, 308, 873, 699, 414, 380, 499,
    680, 681, 415, 744, 745, 97, 235, 907, 909, 910,
    913, 285, 915, 915, 638, 917, 917, 919, 432, 919,
    922, 923, 82, 782, 112, 422, 423, 432, 927, 928,
    931, 932, 933, 934, 935, 936, 937, 938, 940, 941,
    942, 943, 944, 917, 948, 937, 949, 949, 944, 953,
    954, 955, 956, 838, 913, 195, 959, 960, 961, 962,
    963, 218, 964, 965, 779, 966, 432, 961, 959, 964,
    966, 838, 960, 962, 965, 963, 985, 992, 551, 997,
    418, 998, 999, 1006, 1009, 1010, 1013, 5, 1013, 422,
    423, 1024, 1025, 1026, 1027, 1028, 1029, 1030, 1031, 1032,
    569, 432, 1041, 1044, 1048, 1049, 1051, 913, 959, 961,
    218, 964, 966, 1058, 1059, 913, 961, 218, 964, 966,
    1060, 1061, 1062, 1063, 1064, 1065, 1066, 1067, 734, 1068,
    1069, 1070, 1071, 1074, 1075, 1076, 1009, 1010, 1081, 859,
    1082, 1083, 1084, 1085, 1086, 333, 790, 1089, 1090, 1091,
    1092, 1093, 1094, 1095, 552, 1096, 1097, 1098, 1100, 1102,
    82, 503, 112, 422, 423, 432, 1104, 1105, 1106, 1107,
    1108, 1109, 1110, 1111, 1112, 1113, 960, 1114, 97, 235,
    732, 733, 734, 735, 736, 737, 738, 739, 740, 741,
    742, 235, 1115, 1117, 1120, 1121, 1122, 1123, 1126, 164,
    1131, 1131, 551, 553, 554, 553, 554, 1144, 1146, 1147,
    1148, 1149, 1150, 1151, 1152, 1153, 353, 354, 355, 1154,
    356, 357, 358, 359, 1155, 998, 1156, 1157, 1158, 1159,
    1160, 859, 617, 860, 860, 87, 963, 1166, 1167, 1168,
    1174, 1176, 1177, 1178, 1179, 1180, 1181, 1182, 1183, 1184,
    1185, 1186, 1187, 1188, 1189, 1190, 1191, 1192, 1193, 1195,
    1196, 1197, 1198, 1199, 1200, 249, 1201, 1202, 1203, 1204,
    1205, 1213, 101, 117, 1225, 1227, 1228, 1229, 1230, 735,
    1231, 1232, 962, 736, 1233, 1234, 250, 1235, 1236, 1237,
    1238, 1239, 737, 1240, 1241, 1242, 1243, 1244, 1245, 1247,
    1252, 1254, 1255, 1257, 1258, 1260, 360, 361, 362, 363,
    364, 365, 366, 251, 1263, 1264, 1265, 1266, 1267, 1268,
    1269, 1270, 1271, 101, 104, 1205, 117, 82, 112, 1280,
    1281, 254, 1282, 1283, 1284, 393, 1286, 1287, 1288, 1289,
    1290, 1291, 1292, 1293, 1300, 1301, 1302, 1303, 1304, 1305,
    1306, 1307, 1308, 1309, 1310, 1311, 1312, 1313, 1314, 1315,
    1316, 1318, 285, 1153, 913, 959, 961, 964, 944, 917,
    1006, 966, 1323, 1324, 1325, 1326, 1327, 1328, 1330, 1332,
    1333, 1334, 1335, 1336, 1337, 1339, 1340, 1341, 1342, 1343,
    1344, 1345, 1346, 1347, 1348, 1350, 1351, 1352, 309, 235,
    1355, 1356, 1357, 1358, 1359, 1361, 1362, 1363, 1364, 1368,
    1369, 1370, 1371, 1372, 1373, 1374, 1376, 1377, 1378, 1379,
    1380, 1381, 11, 303, 1382, 1382, 738, 1384, 1387, 1388,
    1389, 1391, 1392, 1393, 1394, 1395, 1396, 1397, 1398, 436,
    251, 809, 1225, 551, 1347, 1402, 1402, 1408, 1409, 1410,
    1411, 1412, 1413, 1414, 1415, 1416, 1417, 1418, 1419, 1420,
    1421, 1422, 1423, 1424, 1425, 1426, 1427, 1428, 1429, 1430,
    1431, 1432, 1433, 1434, 1435, 1437, 1438, 1439, 1440, 1441,
    1442, 1443, 1444, 1445, 1446, 1447, 1448, 389, 1450, 1450,
    1454, 1455, 1456, 1457, 1458, 1459, 1460, 1461, 1462, 1463,
    739, 1464, 1465, 1466, 11, 803, 838, 913, 91, 960,
    961, 962, 218, 907, 965, 779, 82, 432, 1488, 1490,
    1493, 1494, 1495, 965, 838, 960, 1502, 1503, 1504, 1505,
    1506, 1507, 1510, 1511, 1514, 1515, 1516, 1517, 1518, 1519,
    1520, 1521, 1522, 1523, 1524, 1525, 1526, 1527, 1528, 1529,
    1530, 1531, 1532, 1533, 1534, 1535, 1536, 1537, 1538, 1539,
    1540, 1542, 1543, 1544, 1545, 1546, 1547, 1548, 1549, 1550,
    1551, 1552, 1554, 1555, 1556, 1557, 1558, 1559, 1560, 1561,
    1562, 1563, 1564, 1565, 1566, 1567, 1568, 17, 506, 510,
    1570, 1574, 616, 1576, 1577, 1578, 1579, 1580, 367, 368,
    369, 370, 371, 372, 432, 1582, 1582, 1585, 1587, 1588,
    1589, 1590, 1591, 1592, 373, 374, 1593, 1594, 999, 1595,
    1596, 388, 531, 448, 1438, 611, 1596, 1600, 1600, 1605,
    1606, 1607, 1608, 1609, 1610, 1611, 1612, 1613, 1614, 1615,
    1616, 1617, 1621, 1622, 1624, 1625, 1626, 1630, 1631, 1632,
    1634, 1635, 1636, 1637, 1638, 1639, 1640, 1641, 1642, 1643,
    1644, 1645, 1646, 1647, 1648, 1649, 1650, 1651, 1652, 1653,
    422, 82, 638, 1655, 1656, 1657, 1658, 1659, 1660, 1661,
    1662, 1663, 1664, 1665, 1666, 1667, 740, 741, 1670, 913,
    195, 959, 961, 218, 907, 964, 966, 1671, 1674, 218,
    1348, 1685, 1686, 1687, 1688, 1689, 1690, 742, 1691, 375,
    1692,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 27, token_types_start: 32, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 44, raw_values_count: 1, token_types_start: 45, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 45, raw_values_count: 2, token_types_start: 47, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 47, raw_values_count: 1, token_types_start: 48, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 49, raw_values_count: 1, token_types_start: 50, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 50, raw_values_count: 1, token_types_start: 51, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 54, raw_values_count: 14, token_types_start: 68, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 68, raw_values_count: 14, token_types_start: 82, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 82, raw_values_count: 2, token_types_start: 84, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 90, raw_values_count: 0, token_types_start: 90, token_types_count: 1 }, // [37]
    SimpleHintData { raw_values_start: 91, raw_values_count: 26, token_types_start: 117, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 117, raw_values_count: 1, token_types_start: 118, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 118, raw_values_count: 5, token_types_start: 123, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 131, raw_values_count: 2, token_types_start: 133, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 133, raw_values_count: 1, token_types_start: 134, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 134, raw_values_count: 1, token_types_start: 135, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 135, raw_values_count: 25, token_types_start: 160, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 160, raw_values_count: 15, token_types_start: 175, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 175, raw_values_count: 1, token_types_start: 176, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 176, raw_values_count: 1, token_types_start: 177, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 177, raw_values_count: 2, token_types_start: 179, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 179, raw_values_count: 1, token_types_start: 180, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 180, raw_values_count: 1, token_types_start: 181, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 181, raw_values_count: 1, token_types_start: 182, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 182, raw_values_count: 1, token_types_start: 183, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 184, raw_values_count: 1, token_types_start: 185, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 185, raw_values_count: 1, token_types_start: 186, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 186, raw_values_count: 1, token_types_start: 187, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 187, raw_values_count: 1, token_types_start: 188, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 188, raw_values_count: 1, token_types_start: 189, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 216, raw_values_count: 1, token_types_start: 217, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 217, raw_values_count: 1, token_types_start: 218, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 220, raw_values_count: 2, token_types_start: 222, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 223, raw_values_count: 1, token_types_start: 224, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 224, raw_values_count: 1, token_types_start: 225, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 225, raw_values_count: 1, token_types_start: 226, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 226, raw_values_count: 1, token_types_start: 227, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 233, raw_values_count: 1, token_types_start: 234, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 234, raw_values_count: 1, token_types_start: 235, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 237, raw_values_count: 1, token_types_start: 238, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 238, raw_values_count: 0, token_types_start: 238, token_types_count: 1 }, // [115]
    SimpleHintData { raw_values_start: 239, raw_values_count: 1, token_types_start: 240, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 243, raw_values_count: 10, token_types_start: 253, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 253, raw_values_count: 1, token_types_start: 254, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 254, raw_values_count: 1, token_types_start: 255, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 255, raw_values_count: 2, token_types_start: 257, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 257, raw_values_count: 2, token_types_start: 259, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 259, raw_values_count: 2, token_types_start: 261, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 261, raw_values_count: 1, token_types_start: 262, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 262, raw_values_count: 1, token_types_start: 263, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 263, raw_values_count: 2, token_types_start: 265, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 265, raw_values_count: 1, token_types_start: 266, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 266, raw_values_count: 1, token_types_start: 267, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 267, raw_values_count: 2, token_types_start: 269, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 269, raw_values_count: 1, token_types_start: 270, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 270, raw_values_count: 1, token_types_start: 271, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 271, raw_values_count: 1, token_types_start: 272, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 272, raw_values_count: 1, token_types_start: 273, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 273, raw_values_count: 2, token_types_start: 275, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 275, raw_values_count: 3, token_types_start: 278, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 278, raw_values_count: 2, token_types_start: 280, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 280, raw_values_count: 1, token_types_start: 281, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 281, raw_values_count: 1, token_types_start: 282, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 282, raw_values_count: 2, token_types_start: 284, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 284, raw_values_count: 1, token_types_start: 285, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 285, raw_values_count: 1, token_types_start: 286, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 286, raw_values_count: 1, token_types_start: 287, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 287, raw_values_count: 1, token_types_start: 288, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 288, raw_values_count: 1, token_types_start: 289, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 289, raw_values_count: 1, token_types_start: 290, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 290, raw_values_count: 1, token_types_start: 291, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 291, raw_values_count: 10, token_types_start: 301, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 301, raw_values_count: 1, token_types_start: 302, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 302, raw_values_count: 1, token_types_start: 303, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 303, raw_values_count: 1, token_types_start: 304, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 304, raw_values_count: 1, token_types_start: 305, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 305, raw_values_count: 1, token_types_start: 306, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 306, raw_values_count: 1, token_types_start: 307, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 307, raw_values_count: 1, token_types_start: 308, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 308, raw_values_count: 1, token_types_start: 309, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 309, raw_values_count: 1, token_types_start: 310, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 310, raw_values_count: 1, token_types_start: 311, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 311, raw_values_count: 1, token_types_start: 312, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 312, raw_values_count: 1, token_types_start: 313, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 313, raw_values_count: 1, token_types_start: 314, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 314, raw_values_count: 1, token_types_start: 315, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 315, raw_values_count: 1, token_types_start: 316, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 316, raw_values_count: 1, token_types_start: 317, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 317, raw_values_count: 1, token_types_start: 318, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 318, raw_values_count: 1, token_types_start: 319, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 319, raw_values_count: 1, token_types_start: 320, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 320, raw_values_count: 1, token_types_start: 321, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 321, raw_values_count: 1, token_types_start: 322, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 322, raw_values_count: 1, token_types_start: 323, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 323, raw_values_count: 1, token_types_start: 324, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 324, raw_values_count: 1, token_types_start: 325, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 325, raw_values_count: 1, token_types_start: 326, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 326, raw_values_count: 1, token_types_start: 327, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 327, raw_values_count: 1, token_types_start: 328, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 328, raw_values_count: 1, token_types_start: 329, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 329, raw_values_count: 26, token_types_start: 355, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 355, raw_values_count: 1, token_types_start: 356, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 356, raw_values_count: 1, token_types_start: 357, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 357, raw_values_count: 1, token_types_start: 358, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 358, raw_values_count: 1, token_types_start: 359, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 359, raw_values_count: 1, token_types_start: 360, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 360, raw_values_count: 1, token_types_start: 361, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 361, raw_values_count: 1, token_types_start: 362, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 362, raw_values_count: 15, token_types_start: 377, token_types_count: 1 }, // [186]
    SimpleHintData { raw_values_start: 378, raw_values_count: 2, token_types_start: 380, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 380, raw_values_count: 5, token_types_start: 385, token_types_count: 1 }, // [188]
    SimpleHintData { raw_values_start: 386, raw_values_count: 1, token_types_start: 387, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 387, raw_values_count: 1, token_types_start: 388, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 388, raw_values_count: 1, token_types_start: 389, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 389, raw_values_count: 1, token_types_start: 390, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 390, raw_values_count: 1, token_types_start: 391, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 391, raw_values_count: 1, token_types_start: 392, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 392, raw_values_count: 1, token_types_start: 393, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 393, raw_values_count: 1, token_types_start: 394, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 394, raw_values_count: 2, token_types_start: 396, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 396, raw_values_count: 1, token_types_start: 397, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 397, raw_values_count: 1, token_types_start: 398, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 398, raw_values_count: 1, token_types_start: 399, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 399, raw_values_count: 4, token_types_start: 403, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 403, raw_values_count: 1, token_types_start: 404, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 404, raw_values_count: 1, token_types_start: 405, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 405, raw_values_count: 1, token_types_start: 406, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 406, raw_values_count: 1, token_types_start: 407, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 407, raw_values_count: 5, token_types_start: 412, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 412, raw_values_count: 1, token_types_start: 413, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 413, raw_values_count: 1, token_types_start: 414, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 414, raw_values_count: 1, token_types_start: 415, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 415, raw_values_count: 1, token_types_start: 416, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 416, raw_values_count: 1, token_types_start: 417, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 417, raw_values_count: 1, token_types_start: 418, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 418, raw_values_count: 1, token_types_start: 419, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 419, raw_values_count: 1, token_types_start: 420, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 420, raw_values_count: 1, token_types_start: 421, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 421, raw_values_count: 1, token_types_start: 422, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 422, raw_values_count: 1, token_types_start: 423, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 423, raw_values_count: 1, token_types_start: 424, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 424, raw_values_count: 1, token_types_start: 425, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 425, raw_values_count: 1, token_types_start: 426, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 426, raw_values_count: 1, token_types_start: 427, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 427, raw_values_count: 1, token_types_start: 428, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 428, raw_values_count: 1, token_types_start: 429, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 429, raw_values_count: 1, token_types_start: 430, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 430, raw_values_count: 1, token_types_start: 431, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 431, raw_values_count: 1, token_types_start: 432, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 432, raw_values_count: 1, token_types_start: 433, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 433, raw_values_count: 1, token_types_start: 434, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 434, raw_values_count: 1, token_types_start: 435, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 435, raw_values_count: 1, token_types_start: 436, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 436, raw_values_count: 1, token_types_start: 437, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 437, raw_values_count: 1, token_types_start: 438, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 438, raw_values_count: 1, token_types_start: 439, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 439, raw_values_count: 1, token_types_start: 440, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 440, raw_values_count: 1, token_types_start: 441, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 441, raw_values_count: 1, token_types_start: 442, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 442, raw_values_count: 1, token_types_start: 443, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 443, raw_values_count: 1, token_types_start: 444, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 444, raw_values_count: 1, token_types_start: 445, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 445, raw_values_count: 1, token_types_start: 446, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 446, raw_values_count: 1, token_types_start: 447, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 447, raw_values_count: 1, token_types_start: 448, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 448, raw_values_count: 1, token_types_start: 449, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 449, raw_values_count: 1, token_types_start: 450, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 450, raw_values_count: 1, token_types_start: 451, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 451, raw_values_count: 1, token_types_start: 452, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 452, raw_values_count: 1, token_types_start: 453, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 453, raw_values_count: 1, token_types_start: 454, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 454, raw_values_count: 1, token_types_start: 455, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 455, raw_values_count: 1, token_types_start: 456, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 456, raw_values_count: 1, token_types_start: 457, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 457, raw_values_count: 2, token_types_start: 459, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 459, raw_values_count: 1, token_types_start: 460, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 460, raw_values_count: 1, token_types_start: 461, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 461, raw_values_count: 1, token_types_start: 462, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 462, raw_values_count: 1, token_types_start: 463, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 463, raw_values_count: 1, token_types_start: 464, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 464, raw_values_count: 1, token_types_start: 465, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 465, raw_values_count: 1, token_types_start: 466, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 466, raw_values_count: 1, token_types_start: 467, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 467, raw_values_count: 1, token_types_start: 468, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 468, raw_values_count: 1, token_types_start: 469, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 469, raw_values_count: 1, token_types_start: 470, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 470, raw_values_count: 1, token_types_start: 471, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 471, raw_values_count: 2, token_types_start: 473, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 473, raw_values_count: 0, token_types_start: 473, token_types_count: 1 }, // [266]
    SimpleHintData { raw_values_start: 474, raw_values_count: 1, token_types_start: 475, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 475, raw_values_count: 1, token_types_start: 476, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 476, raw_values_count: 1, token_types_start: 477, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 477, raw_values_count: 1, token_types_start: 478, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 478, raw_values_count: 1, token_types_start: 479, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 479, raw_values_count: 1, token_types_start: 480, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 480, raw_values_count: 1, token_types_start: 481, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 481, raw_values_count: 4, token_types_start: 485, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 485, raw_values_count: 1, token_types_start: 486, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 486, raw_values_count: 1, token_types_start: 487, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 487, raw_values_count: 1, token_types_start: 488, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 488, raw_values_count: 1, token_types_start: 489, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 489, raw_values_count: 1, token_types_start: 490, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 490, raw_values_count: 1, token_types_start: 491, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 491, raw_values_count: 1, token_types_start: 492, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 492, raw_values_count: 1, token_types_start: 493, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 493, raw_values_count: 1, token_types_start: 494, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 494, raw_values_count: 1, token_types_start: 495, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 495, raw_values_count: 1, token_types_start: 496, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 496, raw_values_count: 1, token_types_start: 497, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 497, raw_values_count: 1, token_types_start: 498, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 498, raw_values_count: 1, token_types_start: 499, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 499, raw_values_count: 1, token_types_start: 500, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 500, raw_values_count: 1, token_types_start: 501, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 501, raw_values_count: 1, token_types_start: 502, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 502, raw_values_count: 1, token_types_start: 503, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 503, raw_values_count: 1, token_types_start: 504, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 504, raw_values_count: 1, token_types_start: 505, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 505, raw_values_count: 7, token_types_start: 512, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 512, raw_values_count: 4, token_types_start: 516, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 516, raw_values_count: 3, token_types_start: 519, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 519, raw_values_count: 1, token_types_start: 520, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 520, raw_values_count: 1, token_types_start: 521, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 521, raw_values_count: 1, token_types_start: 522, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 522, raw_values_count: 1, token_types_start: 523, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 523, raw_values_count: 2, token_types_start: 525, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 525, raw_values_count: 1, token_types_start: 526, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 526, raw_values_count: 1, token_types_start: 527, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 527, raw_values_count: 1, token_types_start: 528, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 528, raw_values_count: 1, token_types_start: 529, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 529, raw_values_count: 11, token_types_start: 540, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 540, raw_values_count: 1, token_types_start: 541, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 541, raw_values_count: 3, token_types_start: 544, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 544, raw_values_count: 3, token_types_start: 547, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 547, raw_values_count: 4, token_types_start: 551, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 551, raw_values_count: 1, token_types_start: 552, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 552, raw_values_count: 3, token_types_start: 555, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 555, raw_values_count: 1, token_types_start: 556, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 556, raw_values_count: 1, token_types_start: 557, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 557, raw_values_count: 3, token_types_start: 560, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 560, raw_values_count: 7, token_types_start: 567, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 567, raw_values_count: 1, token_types_start: 568, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 568, raw_values_count: 1, token_types_start: 569, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 569, raw_values_count: 1, token_types_start: 570, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 570, raw_values_count: 1, token_types_start: 571, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 571, raw_values_count: 3, token_types_start: 574, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 574, raw_values_count: 1, token_types_start: 575, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 575, raw_values_count: 2, token_types_start: 577, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 577, raw_values_count: 1, token_types_start: 578, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 578, raw_values_count: 1, token_types_start: 579, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 579, raw_values_count: 1, token_types_start: 580, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 580, raw_values_count: 1, token_types_start: 581, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 581, raw_values_count: 2, token_types_start: 583, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 583, raw_values_count: 1, token_types_start: 584, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 584, raw_values_count: 0, token_types_start: 584, token_types_count: 1 }, // [331]
    SimpleHintData { raw_values_start: 585, raw_values_count: 1, token_types_start: 586, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 586, raw_values_count: 1, token_types_start: 587, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 587, raw_values_count: 1, token_types_start: 588, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 588, raw_values_count: 1, token_types_start: 589, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 589, raw_values_count: 1, token_types_start: 590, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 590, raw_values_count: 2, token_types_start: 592, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 592, raw_values_count: 1, token_types_start: 593, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 593, raw_values_count: 1, token_types_start: 594, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 594, raw_values_count: 1, token_types_start: 595, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 595, raw_values_count: 1, token_types_start: 596, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 596, raw_values_count: 1, token_types_start: 597, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 597, raw_values_count: 1, token_types_start: 598, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 598, raw_values_count: 1, token_types_start: 599, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 599, raw_values_count: 1, token_types_start: 600, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 600, raw_values_count: 1, token_types_start: 601, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 601, raw_values_count: 1, token_types_start: 602, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 602, raw_values_count: 1, token_types_start: 603, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 603, raw_values_count: 1, token_types_start: 604, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 604, raw_values_count: 1, token_types_start: 605, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 605, raw_values_count: 1, token_types_start: 606, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 606, raw_values_count: 2, token_types_start: 608, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 608, raw_values_count: 1, token_types_start: 609, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 609, raw_values_count: 1, token_types_start: 610, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 610, raw_values_count: 2, token_types_start: 612, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 612, raw_values_count: 2, token_types_start: 614, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 614, raw_values_count: 1, token_types_start: 615, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 615, raw_values_count: 1, token_types_start: 616, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 616, raw_values_count: 1, token_types_start: 617, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 617, raw_values_count: 14, token_types_start: 631, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 631, raw_values_count: 1, token_types_start: 632, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 632, raw_values_count: 1, token_types_start: 633, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 633, raw_values_count: 1, token_types_start: 634, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 634, raw_values_count: 1, token_types_start: 635, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 635, raw_values_count: 1, token_types_start: 636, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 636, raw_values_count: 1, token_types_start: 637, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 637, raw_values_count: 1, token_types_start: 638, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 638, raw_values_count: 1, token_types_start: 639, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 639, raw_values_count: 1, token_types_start: 640, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 640, raw_values_count: 1, token_types_start: 641, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 641, raw_values_count: 1, token_types_start: 642, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 642, raw_values_count: 1, token_types_start: 643, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 643, raw_values_count: 1, token_types_start: 644, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 644, raw_values_count: 1, token_types_start: 645, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 645, raw_values_count: 1, token_types_start: 646, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 646, raw_values_count: 11, token_types_start: 657, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 657, raw_values_count: 4, token_types_start: 661, token_types_count: 2 }, // [377]
    SimpleHintData { raw_values_start: 663, raw_values_count: 0, token_types_start: 663, token_types_count: 1 }, // [378]
    SimpleHintData { raw_values_start: 664, raw_values_count: 0, token_types_start: 664, token_types_count: 1 }, // [379]
    SimpleHintData { raw_values_start: 665, raw_values_count: 4, token_types_start: 669, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 669, raw_values_count: 1, token_types_start: 670, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 670, raw_values_count: 1, token_types_start: 671, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 671, raw_values_count: 1, token_types_start: 672, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 672, raw_values_count: 1, token_types_start: 673, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 673, raw_values_count: 1, token_types_start: 674, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 674, raw_values_count: 1, token_types_start: 675, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 675, raw_values_count: 1, token_types_start: 676, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 676, raw_values_count: 1, token_types_start: 677, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 677, raw_values_count: 1, token_types_start: 678, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 678, raw_values_count: 1, token_types_start: 679, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 679, raw_values_count: 1, token_types_start: 680, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 680, raw_values_count: 1, token_types_start: 681, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 681, raw_values_count: 1, token_types_start: 682, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 682, raw_values_count: 1, token_types_start: 683, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 683, raw_values_count: 1, token_types_start: 684, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 684, raw_values_count: 1, token_types_start: 685, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 685, raw_values_count: 1, token_types_start: 686, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 686, raw_values_count: 1, token_types_start: 687, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 687, raw_values_count: 1, token_types_start: 688, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 688, raw_values_count: 1, token_types_start: 689, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 689, raw_values_count: 1, token_types_start: 690, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 690, raw_values_count: 1, token_types_start: 691, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 691, raw_values_count: 1, token_types_start: 692, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 692, raw_values_count: 1, token_types_start: 693, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 693, raw_values_count: 1, token_types_start: 694, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 694, raw_values_count: 1, token_types_start: 695, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 695, raw_values_count: 1, token_types_start: 696, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 696, raw_values_count: 1, token_types_start: 697, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 697, raw_values_count: 1, token_types_start: 698, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 698, raw_values_count: 1, token_types_start: 699, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 699, raw_values_count: 1, token_types_start: 700, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 700, raw_values_count: 1, token_types_start: 701, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 701, raw_values_count: 1, token_types_start: 702, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 702, raw_values_count: 1, token_types_start: 703, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 703, raw_values_count: 1, token_types_start: 704, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 704, raw_values_count: 1, token_types_start: 705, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 705, raw_values_count: 1, token_types_start: 706, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 706, raw_values_count: 1, token_types_start: 707, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 707, raw_values_count: 1, token_types_start: 708, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 708, raw_values_count: 1, token_types_start: 709, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 709, raw_values_count: 1, token_types_start: 710, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 710, raw_values_count: 1, token_types_start: 711, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 711, raw_values_count: 1, token_types_start: 712, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 712, raw_values_count: 1, token_types_start: 713, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 713, raw_values_count: 1, token_types_start: 714, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 714, raw_values_count: 1, token_types_start: 715, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 715, raw_values_count: 1, token_types_start: 716, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 716, raw_values_count: 1, token_types_start: 717, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 717, raw_values_count: 1, token_types_start: 718, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 718, raw_values_count: 1, token_types_start: 719, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 719, raw_values_count: 1, token_types_start: 720, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 720, raw_values_count: 1, token_types_start: 721, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 721, raw_values_count: 2, token_types_start: 723, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 723, raw_values_count: 1, token_types_start: 724, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 724, raw_values_count: 1, token_types_start: 725, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 725, raw_values_count: 1, token_types_start: 726, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 726, raw_values_count: 1, token_types_start: 727, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 727, raw_values_count: 1, token_types_start: 728, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 728, raw_values_count: 1, token_types_start: 729, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 729, raw_values_count: 1, token_types_start: 730, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 730, raw_values_count: 1, token_types_start: 731, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 731, raw_values_count: 1, token_types_start: 732, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 732, raw_values_count: 1, token_types_start: 733, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 733, raw_values_count: 1, token_types_start: 734, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 734, raw_values_count: 1, token_types_start: 735, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 735, raw_values_count: 1, token_types_start: 736, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 736, raw_values_count: 1, token_types_start: 737, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 737, raw_values_count: 1, token_types_start: 738, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 738, raw_values_count: 1, token_types_start: 739, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 739, raw_values_count: 1, token_types_start: 740, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 740, raw_values_count: 1, token_types_start: 741, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 741, raw_values_count: 1, token_types_start: 742, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 742, raw_values_count: 1, token_types_start: 743, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 743, raw_values_count: 1, token_types_start: 744, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 744, raw_values_count: 1, token_types_start: 745, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 745, raw_values_count: 1, token_types_start: 746, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 746, raw_values_count: 1, token_types_start: 747, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 747, raw_values_count: 1, token_types_start: 748, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 748, raw_values_count: 1, token_types_start: 749, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 751, raw_values_count: 1, token_types_start: 752, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 752, raw_values_count: 1, token_types_start: 753, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 753, raw_values_count: 1, token_types_start: 754, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 754, raw_values_count: 8, token_types_start: 762, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 762, raw_values_count: 4, token_types_start: 766, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 766, raw_values_count: 2, token_types_start: 768, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 768, raw_values_count: 2, token_types_start: 770, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 770, raw_values_count: 5, token_types_start: 775, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 775, raw_values_count: 2, token_types_start: 777, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 777, raw_values_count: 1, token_types_start: 778, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 778, raw_values_count: 1, token_types_start: 779, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 779, raw_values_count: 13, token_types_start: 792, token_types_count: 1 }, // [473]
    SimpleHintData { raw_values_start: 793, raw_values_count: 3, token_types_start: 796, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 796, raw_values_count: 7, token_types_start: 803, token_types_count: 4 }, // [475]
    SimpleHintData { raw_values_start: 807, raw_values_count: 1, token_types_start: 808, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 808, raw_values_count: 1, token_types_start: 809, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 809, raw_values_count: 1, token_types_start: 810, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 810, raw_values_count: 1, token_types_start: 811, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 811, raw_values_count: 2, token_types_start: 813, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 813, raw_values_count: 1, token_types_start: 814, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 814, raw_values_count: 2, token_types_start: 816, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 816, raw_values_count: 1, token_types_start: 817, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 817, raw_values_count: 2, token_types_start: 819, token_types_count: 0 }, // [484]
    SimpleHintData { raw_values_start: 819, raw_values_count: 1, token_types_start: 820, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 820, raw_values_count: 1, token_types_start: 821, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 821, raw_values_count: 1, token_types_start: 822, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 822, raw_values_count: 6, token_types_start: 828, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 828, raw_values_count: 1, token_types_start: 829, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 829, raw_values_count: 1, token_types_start: 830, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 830, raw_values_count: 1, token_types_start: 831, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 831, raw_values_count: 1, token_types_start: 832, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 832, raw_values_count: 1, token_types_start: 833, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 833, raw_values_count: 1, token_types_start: 834, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 834, raw_values_count: 1, token_types_start: 835, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 835, raw_values_count: 1, token_types_start: 836, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 836, raw_values_count: 1, token_types_start: 837, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 837, raw_values_count: 1, token_types_start: 838, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 838, raw_values_count: 1, token_types_start: 839, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 839, raw_values_count: 1, token_types_start: 840, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 840, raw_values_count: 1, token_types_start: 841, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 841, raw_values_count: 1, token_types_start: 842, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 842, raw_values_count: 2, token_types_start: 844, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 844, raw_values_count: 1, token_types_start: 845, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 845, raw_values_count: 2, token_types_start: 847, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 847, raw_values_count: 1, token_types_start: 848, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 848, raw_values_count: 1, token_types_start: 849, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 849, raw_values_count: 1, token_types_start: 850, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 850, raw_values_count: 1, token_types_start: 851, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 851, raw_values_count: 1, token_types_start: 852, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 852, raw_values_count: 1, token_types_start: 853, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 853, raw_values_count: 14, token_types_start: 867, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 867, raw_values_count: 1, token_types_start: 868, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 868, raw_values_count: 1, token_types_start: 869, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 869, raw_values_count: 1, token_types_start: 870, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 870, raw_values_count: 1, token_types_start: 871, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 871, raw_values_count: 4, token_types_start: 875, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 875, raw_values_count: 1, token_types_start: 876, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 876, raw_values_count: 1, token_types_start: 877, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 877, raw_values_count: 1, token_types_start: 878, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 878, raw_values_count: 1, token_types_start: 879, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 879, raw_values_count: 1, token_types_start: 880, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 880, raw_values_count: 3, token_types_start: 883, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 883, raw_values_count: 1, token_types_start: 884, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 884, raw_values_count: 2, token_types_start: 886, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 886, raw_values_count: 0, token_types_start: 886, token_types_count: 2 }, // [526]
    SimpleHintData { raw_values_start: 888, raw_values_count: 0, token_types_start: 888, token_types_count: 1 }, // [527]
    SimpleHintData { raw_values_start: 889, raw_values_count: 2, token_types_start: 891, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 891, raw_values_count: 1, token_types_start: 892, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 892, raw_values_count: 1, token_types_start: 893, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 893, raw_values_count: 1, token_types_start: 894, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 894, raw_values_count: 1, token_types_start: 895, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 895, raw_values_count: 1, token_types_start: 896, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 896, raw_values_count: 0, token_types_start: 896, token_types_count: 1 }, // [534]
    SimpleHintData { raw_values_start: 897, raw_values_count: 1, token_types_start: 898, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 898, raw_values_count: 1, token_types_start: 899, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 899, raw_values_count: 1, token_types_start: 900, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 900, raw_values_count: 2, token_types_start: 902, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 902, raw_values_count: 1, token_types_start: 903, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 903, raw_values_count: 1, token_types_start: 904, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 904, raw_values_count: 1, token_types_start: 905, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 905, raw_values_count: 1, token_types_start: 906, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 906, raw_values_count: 1, token_types_start: 907, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 907, raw_values_count: 6, token_types_start: 913, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 913, raw_values_count: 1, token_types_start: 914, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 914, raw_values_count: 1, token_types_start: 915, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 915, raw_values_count: 5, token_types_start: 920, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 920, raw_values_count: 1, token_types_start: 921, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 921, raw_values_count: 1, token_types_start: 922, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 922, raw_values_count: 1, token_types_start: 923, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 923, raw_values_count: 1, token_types_start: 924, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 924, raw_values_count: 1, token_types_start: 925, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 925, raw_values_count: 1, token_types_start: 926, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 926, raw_values_count: 1, token_types_start: 927, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 927, raw_values_count: 1, token_types_start: 928, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 928, raw_values_count: 1, token_types_start: 929, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 929, raw_values_count: 1, token_types_start: 930, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 930, raw_values_count: 1, token_types_start: 931, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 931, raw_values_count: 1, token_types_start: 932, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 932, raw_values_count: 1, token_types_start: 933, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 933, raw_values_count: 1, token_types_start: 934, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 934, raw_values_count: 1, token_types_start: 935, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 935, raw_values_count: 1, token_types_start: 936, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 936, raw_values_count: 1, token_types_start: 937, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 937, raw_values_count: 1, token_types_start: 938, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 938, raw_values_count: 1, token_types_start: 939, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 939, raw_values_count: 1, token_types_start: 940, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 940, raw_values_count: 1, token_types_start: 941, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 941, raw_values_count: 1, token_types_start: 942, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 942, raw_values_count: 1, token_types_start: 943, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 943, raw_values_count: 1, token_types_start: 944, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 944, raw_values_count: 1, token_types_start: 945, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 945, raw_values_count: 2, token_types_start: 947, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 947, raw_values_count: 1, token_types_start: 948, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 948, raw_values_count: 1, token_types_start: 949, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 949, raw_values_count: 1, token_types_start: 950, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 950, raw_values_count: 1, token_types_start: 951, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 951, raw_values_count: 1, token_types_start: 952, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 952, raw_values_count: 1, token_types_start: 953, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 953, raw_values_count: 1, token_types_start: 954, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 954, raw_values_count: 1, token_types_start: 955, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 955, raw_values_count: 1, token_types_start: 956, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 956, raw_values_count: 1, token_types_start: 957, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 957, raw_values_count: 1, token_types_start: 958, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 958, raw_values_count: 1, token_types_start: 959, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 959, raw_values_count: 1, token_types_start: 960, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 960, raw_values_count: 6, token_types_start: 966, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 966, raw_values_count: 1, token_types_start: 967, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 967, raw_values_count: 1, token_types_start: 968, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 968, raw_values_count: 1, token_types_start: 969, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 969, raw_values_count: 1, token_types_start: 970, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 970, raw_values_count: 1, token_types_start: 971, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 971, raw_values_count: 1, token_types_start: 972, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 972, raw_values_count: 1, token_types_start: 973, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 973, raw_values_count: 1, token_types_start: 974, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 974, raw_values_count: 1, token_types_start: 975, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 975, raw_values_count: 1, token_types_start: 976, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 976, raw_values_count: 1, token_types_start: 977, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 977, raw_values_count: 1, token_types_start: 978, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 978, raw_values_count: 0, token_types_start: 978, token_types_count: 2 }, // [600]
    SimpleHintData { raw_values_start: 980, raw_values_count: 11, token_types_start: 991, token_types_count: 1 }, // [601]
    SimpleHintData { raw_values_start: 992, raw_values_count: 1, token_types_start: 993, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 993, raw_values_count: 1, token_types_start: 994, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 994, raw_values_count: 1, token_types_start: 995, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 995, raw_values_count: 1, token_types_start: 996, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 996, raw_values_count: 1, token_types_start: 997, token_types_count: 0 }, // [606]
    SimpleHintData { raw_values_start: 997, raw_values_count: 1, token_types_start: 998, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 998, raw_values_count: 1, token_types_start: 999, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 999, raw_values_count: 2, token_types_start: 1001, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1001, raw_values_count: 1, token_types_start: 1002, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1002, raw_values_count: 3, token_types_start: 1005, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1005, raw_values_count: 1, token_types_start: 1006, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1006, raw_values_count: 1, token_types_start: 1007, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1007, raw_values_count: 1, token_types_start: 1008, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1008, raw_values_count: 1, token_types_start: 1009, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1009, raw_values_count: 1, token_types_start: 1010, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1010, raw_values_count: 1, token_types_start: 1011, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1011, raw_values_count: 1, token_types_start: 1012, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1012, raw_values_count: 1, token_types_start: 1013, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1013, raw_values_count: 1, token_types_start: 1014, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1014, raw_values_count: 1, token_types_start: 1015, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1015, raw_values_count: 1, token_types_start: 1016, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1016, raw_values_count: 1, token_types_start: 1017, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1017, raw_values_count: 1, token_types_start: 1018, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1018, raw_values_count: 1, token_types_start: 1019, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1019, raw_values_count: 1, token_types_start: 1020, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1020, raw_values_count: 1, token_types_start: 1021, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1021, raw_values_count: 1, token_types_start: 1022, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1022, raw_values_count: 1, token_types_start: 1023, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1023, raw_values_count: 1, token_types_start: 1024, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1024, raw_values_count: 1, token_types_start: 1025, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1025, raw_values_count: 1, token_types_start: 1026, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1026, raw_values_count: 1, token_types_start: 1027, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1027, raw_values_count: 1, token_types_start: 1028, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1028, raw_values_count: 1, token_types_start: 1029, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1029, raw_values_count: 1, token_types_start: 1030, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1030, raw_values_count: 1, token_types_start: 1031, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1031, raw_values_count: 3, token_types_start: 1034, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1034, raw_values_count: 1, token_types_start: 1035, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1035, raw_values_count: 2, token_types_start: 1037, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1037, raw_values_count: 1, token_types_start: 1038, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1038, raw_values_count: 1, token_types_start: 1039, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1039, raw_values_count: 1, token_types_start: 1040, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1040, raw_values_count: 1, token_types_start: 1041, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1041, raw_values_count: 1, token_types_start: 1042, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1042, raw_values_count: 1, token_types_start: 1043, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1043, raw_values_count: 1, token_types_start: 1044, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1044, raw_values_count: 1, token_types_start: 1045, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1045, raw_values_count: 1, token_types_start: 1046, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1046, raw_values_count: 1, token_types_start: 1047, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1047, raw_values_count: 1, token_types_start: 1048, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1048, raw_values_count: 1, token_types_start: 1049, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1049, raw_values_count: 1, token_types_start: 1050, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1050, raw_values_count: 1, token_types_start: 1051, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1051, raw_values_count: 1, token_types_start: 1052, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1052, raw_values_count: 1, token_types_start: 1053, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1053, raw_values_count: 1, token_types_start: 1054, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1054, raw_values_count: 1, token_types_start: 1055, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1055, raw_values_count: 1, token_types_start: 1056, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1056, raw_values_count: 1, token_types_start: 1057, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1057, raw_values_count: 1, token_types_start: 1058, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1058, raw_values_count: 1, token_types_start: 1059, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1059, raw_values_count: 1, token_types_start: 1060, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1060, raw_values_count: 1, token_types_start: 1061, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1061, raw_values_count: 1, token_types_start: 1062, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1062, raw_values_count: 1, token_types_start: 1063, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1063, raw_values_count: 1, token_types_start: 1064, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1064, raw_values_count: 1, token_types_start: 1065, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1065, raw_values_count: 1, token_types_start: 1066, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1066, raw_values_count: 1, token_types_start: 1067, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1067, raw_values_count: 1, token_types_start: 1068, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1068, raw_values_count: 1, token_types_start: 1069, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1069, raw_values_count: 1, token_types_start: 1070, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1070, raw_values_count: 1, token_types_start: 1071, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1071, raw_values_count: 1, token_types_start: 1072, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1072, raw_values_count: 2, token_types_start: 1074, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1074, raw_values_count: 1, token_types_start: 1075, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1075, raw_values_count: 1, token_types_start: 1076, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1076, raw_values_count: 1, token_types_start: 1077, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1077, raw_values_count: 1, token_types_start: 1078, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1078, raw_values_count: 1, token_types_start: 1079, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1079, raw_values_count: 1, token_types_start: 1080, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1080, raw_values_count: 1, token_types_start: 1081, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1081, raw_values_count: 1, token_types_start: 1082, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1082, raw_values_count: 1, token_types_start: 1083, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1083, raw_values_count: 1, token_types_start: 1084, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1084, raw_values_count: 1, token_types_start: 1085, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1085, raw_values_count: 1, token_types_start: 1086, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1086, raw_values_count: 1, token_types_start: 1087, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1087, raw_values_count: 1, token_types_start: 1088, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1088, raw_values_count: 1, token_types_start: 1089, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1089, raw_values_count: 1, token_types_start: 1090, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1090, raw_values_count: 1, token_types_start: 1091, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1091, raw_values_count: 1, token_types_start: 1092, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1092, raw_values_count: 1, token_types_start: 1093, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1093, raw_values_count: 1, token_types_start: 1094, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1094, raw_values_count: 1, token_types_start: 1095, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1095, raw_values_count: 1, token_types_start: 1096, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1096, raw_values_count: 1, token_types_start: 1097, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1097, raw_values_count: 1, token_types_start: 1098, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1098, raw_values_count: 1, token_types_start: 1099, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1099, raw_values_count: 1, token_types_start: 1100, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1100, raw_values_count: 1, token_types_start: 1101, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1101, raw_values_count: 1, token_types_start: 1102, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1102, raw_values_count: 1, token_types_start: 1103, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1103, raw_values_count: 1, token_types_start: 1104, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1104, raw_values_count: 1, token_types_start: 1105, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1105, raw_values_count: 1, token_types_start: 1106, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1106, raw_values_count: 1, token_types_start: 1107, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1107, raw_values_count: 1, token_types_start: 1108, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1108, raw_values_count: 1, token_types_start: 1109, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1109, raw_values_count: 1, token_types_start: 1110, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1110, raw_values_count: 1, token_types_start: 1111, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1111, raw_values_count: 1, token_types_start: 1112, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1112, raw_values_count: 1, token_types_start: 1113, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1113, raw_values_count: 1, token_types_start: 1114, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1114, raw_values_count: 1, token_types_start: 1115, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1115, raw_values_count: 1, token_types_start: 1116, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1116, raw_values_count: 1, token_types_start: 1117, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1117, raw_values_count: 1, token_types_start: 1118, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1118, raw_values_count: 1, token_types_start: 1119, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1119, raw_values_count: 1, token_types_start: 1120, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1120, raw_values_count: 1, token_types_start: 1121, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1121, raw_values_count: 1, token_types_start: 1122, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 1, token_types_start: 1123, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1123, raw_values_count: 4, token_types_start: 1127, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1127, raw_values_count: 2, token_types_start: 1129, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1129, raw_values_count: 1, token_types_start: 1130, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1130, raw_values_count: 1, token_types_start: 1131, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1131, raw_values_count: 1, token_types_start: 1132, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1132, raw_values_count: 1, token_types_start: 1133, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1133, raw_values_count: 1, token_types_start: 1134, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1134, raw_values_count: 1, token_types_start: 1135, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 1, token_types_start: 1136, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1136, raw_values_count: 1, token_types_start: 1137, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1137, raw_values_count: 1, token_types_start: 1138, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1138, raw_values_count: 1, token_types_start: 1139, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1139, raw_values_count: 1, token_types_start: 1140, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1140, raw_values_count: 1, token_types_start: 1141, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1141, raw_values_count: 1, token_types_start: 1142, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1142, raw_values_count: 1, token_types_start: 1143, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1143, raw_values_count: 1, token_types_start: 1144, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1144, raw_values_count: 1, token_types_start: 1145, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1145, raw_values_count: 1, token_types_start: 1146, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1146, raw_values_count: 1, token_types_start: 1147, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 1, token_types_start: 1148, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1148, raw_values_count: 1, token_types_start: 1149, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1149, raw_values_count: 1, token_types_start: 1150, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1150, raw_values_count: 1, token_types_start: 1151, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1151, raw_values_count: 1, token_types_start: 1152, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1152, raw_values_count: 1, token_types_start: 1153, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 1, token_types_start: 1154, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1154, raw_values_count: 1, token_types_start: 1155, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1155, raw_values_count: 1, token_types_start: 1156, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1156, raw_values_count: 1, token_types_start: 1157, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1157, raw_values_count: 1, token_types_start: 1158, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1158, raw_values_count: 1, token_types_start: 1159, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1159, raw_values_count: 1, token_types_start: 1160, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1160, raw_values_count: 1, token_types_start: 1161, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1161, raw_values_count: 1, token_types_start: 1162, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1162, raw_values_count: 2, token_types_start: 1164, token_types_count: 0 }, // [761]
    SimpleHintData { raw_values_start: 1164, raw_values_count: 8, token_types_start: 1172, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 1, token_types_start: 1173, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1173, raw_values_count: 1, token_types_start: 1174, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1174, raw_values_count: 1, token_types_start: 1175, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1175, raw_values_count: 1, token_types_start: 1176, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1176, raw_values_count: 1, token_types_start: 1177, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1177, raw_values_count: 1, token_types_start: 1178, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1178, raw_values_count: 1, token_types_start: 1179, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 1, token_types_start: 1180, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1180, raw_values_count: 1, token_types_start: 1181, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1181, raw_values_count: 1, token_types_start: 1182, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1182, raw_values_count: 1, token_types_start: 1183, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1183, raw_values_count: 1, token_types_start: 1184, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1184, raw_values_count: 1, token_types_start: 1185, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1185, raw_values_count: 1, token_types_start: 1186, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 1, token_types_start: 1187, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1187, raw_values_count: 1, token_types_start: 1188, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 1, token_types_start: 1189, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1189, raw_values_count: 1, token_types_start: 1190, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1190, raw_values_count: 1, token_types_start: 1191, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1191, raw_values_count: 1, token_types_start: 1192, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1192, raw_values_count: 1, token_types_start: 1193, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 1, token_types_start: 1194, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1194, raw_values_count: 1, token_types_start: 1195, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1195, raw_values_count: 1, token_types_start: 1196, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1196, raw_values_count: 1, token_types_start: 1197, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1197, raw_values_count: 1, token_types_start: 1198, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1198, raw_values_count: 1, token_types_start: 1199, token_types_count: 1 }, // [789]
    SimpleHintData { raw_values_start: 1200, raw_values_count: 1, token_types_start: 1201, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1201, raw_values_count: 1, token_types_start: 1202, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1202, raw_values_count: 1, token_types_start: 1203, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1203, raw_values_count: 1, token_types_start: 1204, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1204, raw_values_count: 1, token_types_start: 1205, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1205, raw_values_count: 1, token_types_start: 1206, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1206, raw_values_count: 1, token_types_start: 1207, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1207, raw_values_count: 1, token_types_start: 1208, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1208, raw_values_count: 1, token_types_start: 1209, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1209, raw_values_count: 1, token_types_start: 1210, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1210, raw_values_count: 1, token_types_start: 1211, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1211, raw_values_count: 1, token_types_start: 1212, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1212, raw_values_count: 1, token_types_start: 1213, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1213, raw_values_count: 1, token_types_start: 1214, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1214, raw_values_count: 1, token_types_start: 1215, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1215, raw_values_count: 1, token_types_start: 1216, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 1216, raw_values_count: 1, token_types_start: 1217, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 1217, raw_values_count: 1, token_types_start: 1218, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1218, raw_values_count: 1, token_types_start: 1219, token_types_count: 0 }, // [808]
    SimpleHintData { raw_values_start: 1219, raw_values_count: 1, token_types_start: 1220, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 1220, raw_values_count: 1, token_types_start: 1221, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1221, raw_values_count: 1, token_types_start: 1222, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 1222, raw_values_count: 3, token_types_start: 1225, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 1225, raw_values_count: 1, token_types_start: 1226, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 1226, raw_values_count: 1, token_types_start: 1227, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 1227, raw_values_count: 1, token_types_start: 1228, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 1228, raw_values_count: 1, token_types_start: 1229, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 1229, raw_values_count: 1, token_types_start: 1230, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 1230, raw_values_count: 1, token_types_start: 1231, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 1231, raw_values_count: 1, token_types_start: 1232, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 1232, raw_values_count: 1, token_types_start: 1233, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 1233, raw_values_count: 1, token_types_start: 1234, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 1234, raw_values_count: 1, token_types_start: 1235, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 1235, raw_values_count: 1, token_types_start: 1236, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 1236, raw_values_count: 1, token_types_start: 1237, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 1237, raw_values_count: 1, token_types_start: 1238, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 1238, raw_values_count: 1, token_types_start: 1239, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 1239, raw_values_count: 4, token_types_start: 1243, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 1243, raw_values_count: 3, token_types_start: 1246, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 1246, raw_values_count: 1, token_types_start: 1247, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 1247, raw_values_count: 1, token_types_start: 1248, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 1248, raw_values_count: 1, token_types_start: 1249, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 1249, raw_values_count: 1, token_types_start: 1250, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 1250, raw_values_count: 1, token_types_start: 1251, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 1251, raw_values_count: 1, token_types_start: 1252, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 1, token_types_start: 1253, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 1253, raw_values_count: 1, token_types_start: 1254, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 1, token_types_start: 1255, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 1255, raw_values_count: 1, token_types_start: 1256, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 1256, raw_values_count: 1, token_types_start: 1257, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 1257, raw_values_count: 1, token_types_start: 1258, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 1258, raw_values_count: 1, token_types_start: 1259, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 1259, raw_values_count: 1, token_types_start: 1260, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 1260, raw_values_count: 1, token_types_start: 1261, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 1261, raw_values_count: 1, token_types_start: 1262, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 1262, raw_values_count: 1, token_types_start: 1263, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 1263, raw_values_count: 1, token_types_start: 1264, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 1264, raw_values_count: 1, token_types_start: 1265, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 1265, raw_values_count: 1, token_types_start: 1266, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 1266, raw_values_count: 1, token_types_start: 1267, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 1267, raw_values_count: 1, token_types_start: 1268, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 1268, raw_values_count: 1, token_types_start: 1269, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 1269, raw_values_count: 1, token_types_start: 1270, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 1270, raw_values_count: 1, token_types_start: 1271, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 1271, raw_values_count: 1, token_types_start: 1272, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 1272, raw_values_count: 1, token_types_start: 1273, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 1273, raw_values_count: 1, token_types_start: 1274, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 1274, raw_values_count: 1, token_types_start: 1275, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 1275, raw_values_count: 1, token_types_start: 1276, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 1276, raw_values_count: 1, token_types_start: 1277, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 1277, raw_values_count: 1, token_types_start: 1278, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 1278, raw_values_count: 1, token_types_start: 1279, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 1279, raw_values_count: 1, token_types_start: 1280, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 1280, raw_values_count: 1, token_types_start: 1281, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 1281, raw_values_count: 1, token_types_start: 1282, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 1282, raw_values_count: 1, token_types_start: 1283, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 1283, raw_values_count: 1, token_types_start: 1284, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 1284, raw_values_count: 1, token_types_start: 1285, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 1285, raw_values_count: 1, token_types_start: 1286, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 1286, raw_values_count: 1, token_types_start: 1287, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 1287, raw_values_count: 2, token_types_start: 1289, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 1289, raw_values_count: 1, token_types_start: 1290, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 1290, raw_values_count: 1, token_types_start: 1291, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 1291, raw_values_count: 1, token_types_start: 1292, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 1292, raw_values_count: 1, token_types_start: 1293, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 1293, raw_values_count: 1, token_types_start: 1294, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 1294, raw_values_count: 1, token_types_start: 1295, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 1295, raw_values_count: 1, token_types_start: 1296, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 1296, raw_values_count: 1, token_types_start: 1297, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 1297, raw_values_count: 1, token_types_start: 1298, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 1298, raw_values_count: 1, token_types_start: 1299, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 1299, raw_values_count: 1, token_types_start: 1300, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 1300, raw_values_count: 1, token_types_start: 1301, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 1301, raw_values_count: 1, token_types_start: 1302, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 1302, raw_values_count: 1, token_types_start: 1303, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 1303, raw_values_count: 1, token_types_start: 1304, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 1304, raw_values_count: 2, token_types_start: 1306, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 1306, raw_values_count: 10, token_types_start: 1316, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 1316, raw_values_count: 2, token_types_start: 1318, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 1, token_types_start: 1319, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 1319, raw_values_count: 1, token_types_start: 1320, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 1, token_types_start: 1321, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 1321, raw_values_count: 1, token_types_start: 1322, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 1322, raw_values_count: 1, token_types_start: 1323, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 1323, raw_values_count: 1, token_types_start: 1324, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 1324, raw_values_count: 2, token_types_start: 1326, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 1326, raw_values_count: 1, token_types_start: 1327, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 1327, raw_values_count: 1, token_types_start: 1328, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 1328, raw_values_count: 1, token_types_start: 1329, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 1329, raw_values_count: 1, token_types_start: 1330, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 1330, raw_values_count: 1, token_types_start: 1331, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 1331, raw_values_count: 1, token_types_start: 1332, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 1332, raw_values_count: 1, token_types_start: 1333, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 1333, raw_values_count: 1, token_types_start: 1334, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 1334, raw_values_count: 1, token_types_start: 1335, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 1335, raw_values_count: 1, token_types_start: 1336, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 1336, raw_values_count: 1, token_types_start: 1337, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 1337, raw_values_count: 1, token_types_start: 1338, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 1338, raw_values_count: 1, token_types_start: 1339, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 1339, raw_values_count: 1, token_types_start: 1340, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 1340, raw_values_count: 1, token_types_start: 1341, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 1341, raw_values_count: 1, token_types_start: 1342, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 1342, raw_values_count: 1, token_types_start: 1343, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 1343, raw_values_count: 1, token_types_start: 1344, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 1344, raw_values_count: 1, token_types_start: 1345, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 1345, raw_values_count: 1, token_types_start: 1346, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 1346, raw_values_count: 1, token_types_start: 1347, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 1347, raw_values_count: 1, token_types_start: 1348, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 1348, raw_values_count: 1, token_types_start: 1349, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 1349, raw_values_count: 1, token_types_start: 1350, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 1350, raw_values_count: 1, token_types_start: 1351, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 1351, raw_values_count: 1, token_types_start: 1352, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 1352, raw_values_count: 1, token_types_start: 1353, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 1353, raw_values_count: 1, token_types_start: 1354, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 1354, raw_values_count: 1, token_types_start: 1355, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 1355, raw_values_count: 1, token_types_start: 1356, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 1356, raw_values_count: 1, token_types_start: 1357, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 1357, raw_values_count: 1, token_types_start: 1358, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 1358, raw_values_count: 1, token_types_start: 1359, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 1359, raw_values_count: 1, token_types_start: 1360, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 1360, raw_values_count: 1, token_types_start: 1361, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 1361, raw_values_count: 1, token_types_start: 1362, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 1362, raw_values_count: 1, token_types_start: 1363, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 1363, raw_values_count: 1, token_types_start: 1364, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 1364, raw_values_count: 1, token_types_start: 1365, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 1365, raw_values_count: 1, token_types_start: 1366, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 1366, raw_values_count: 1, token_types_start: 1367, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 1367, raw_values_count: 1, token_types_start: 1368, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 1368, raw_values_count: 1, token_types_start: 1369, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 1369, raw_values_count: 1, token_types_start: 1370, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 1370, raw_values_count: 1, token_types_start: 1371, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 1371, raw_values_count: 1, token_types_start: 1372, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 1372, raw_values_count: 1, token_types_start: 1373, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 1373, raw_values_count: 1, token_types_start: 1374, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 1374, raw_values_count: 1, token_types_start: 1375, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 1375, raw_values_count: 1, token_types_start: 1376, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 1376, raw_values_count: 1, token_types_start: 1377, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 1377, raw_values_count: 1, token_types_start: 1378, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 1378, raw_values_count: 1, token_types_start: 1379, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 1379, raw_values_count: 1, token_types_start: 1380, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 1380, raw_values_count: 1, token_types_start: 1381, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 1381, raw_values_count: 1, token_types_start: 1382, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 1382, raw_values_count: 1, token_types_start: 1383, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 1383, raw_values_count: 1, token_types_start: 1384, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 1384, raw_values_count: 1, token_types_start: 1385, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 1385, raw_values_count: 1, token_types_start: 1386, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 1386, raw_values_count: 1, token_types_start: 1387, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 1387, raw_values_count: 3, token_types_start: 1390, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 1390, raw_values_count: 1, token_types_start: 1391, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 1391, raw_values_count: 1, token_types_start: 1392, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 1392, raw_values_count: 1, token_types_start: 1393, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 1393, raw_values_count: 1, token_types_start: 1394, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 1394, raw_values_count: 1, token_types_start: 1395, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 1395, raw_values_count: 1, token_types_start: 1396, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 1396, raw_values_count: 1, token_types_start: 1397, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 1397, raw_values_count: 1, token_types_start: 1398, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 1398, raw_values_count: 1, token_types_start: 1399, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 1399, raw_values_count: 1, token_types_start: 1400, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 1400, raw_values_count: 1, token_types_start: 1401, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 1401, raw_values_count: 1, token_types_start: 1402, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 1402, raw_values_count: 1, token_types_start: 1403, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 1403, raw_values_count: 1, token_types_start: 1404, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 1404, raw_values_count: 2, token_types_start: 1406, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 1406, raw_values_count: 1, token_types_start: 1407, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 1407, raw_values_count: 1, token_types_start: 1408, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 1408, raw_values_count: 1, token_types_start: 1409, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 1409, raw_values_count: 1, token_types_start: 1410, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 1410, raw_values_count: 1, token_types_start: 1411, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 1411, raw_values_count: 1, token_types_start: 1412, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 1412, raw_values_count: 1, token_types_start: 1413, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 1413, raw_values_count: 1, token_types_start: 1414, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 1414, raw_values_count: 1, token_types_start: 1415, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 1415, raw_values_count: 1, token_types_start: 1416, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 1416, raw_values_count: 1, token_types_start: 1417, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 1417, raw_values_count: 1, token_types_start: 1418, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 1418, raw_values_count: 1, token_types_start: 1419, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 1419, raw_values_count: 1, token_types_start: 1420, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 1420, raw_values_count: 1, token_types_start: 1421, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 1421, raw_values_count: 5, token_types_start: 1426, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 1426, raw_values_count: 2, token_types_start: 1428, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 1428, raw_values_count: 1, token_types_start: 1429, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 1429, raw_values_count: 1, token_types_start: 1430, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 1430, raw_values_count: 1, token_types_start: 1431, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 1431, raw_values_count: 1, token_types_start: 1432, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 1432, raw_values_count: 1, token_types_start: 1433, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 1433, raw_values_count: 1, token_types_start: 1434, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 1434, raw_values_count: 1, token_types_start: 1435, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 1435, raw_values_count: 1, token_types_start: 1436, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 1436, raw_values_count: 1, token_types_start: 1437, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 1437, raw_values_count: 1, token_types_start: 1438, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 1438, raw_values_count: 1, token_types_start: 1439, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 1439, raw_values_count: 1, token_types_start: 1440, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 1440, raw_values_count: 1, token_types_start: 1441, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 1441, raw_values_count: 1, token_types_start: 1442, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 1442, raw_values_count: 1, token_types_start: 1443, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 1443, raw_values_count: 1, token_types_start: 1444, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 1444, raw_values_count: 1, token_types_start: 1445, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 1445, raw_values_count: 1, token_types_start: 1446, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 1446, raw_values_count: 1, token_types_start: 1447, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 1447, raw_values_count: 1, token_types_start: 1448, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 1448, raw_values_count: 1, token_types_start: 1449, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 1449, raw_values_count: 1, token_types_start: 1450, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 1450, raw_values_count: 1, token_types_start: 1451, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 1451, raw_values_count: 1, token_types_start: 1452, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 1452, raw_values_count: 1, token_types_start: 1453, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 1453, raw_values_count: 1, token_types_start: 1454, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 1454, raw_values_count: 1, token_types_start: 1455, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 1455, raw_values_count: 1, token_types_start: 1456, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 1456, raw_values_count: 1, token_types_start: 1457, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 1457, raw_values_count: 1, token_types_start: 1458, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 1458, raw_values_count: 1, token_types_start: 1459, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 1459, raw_values_count: 1, token_types_start: 1460, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 1460, raw_values_count: 1, token_types_start: 1461, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 1461, raw_values_count: 1, token_types_start: 1462, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 1462, raw_values_count: 1, token_types_start: 1463, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 1463, raw_values_count: 1, token_types_start: 1464, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 1464, raw_values_count: 1, token_types_start: 1465, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 1, token_types_start: 1466, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 1466, raw_values_count: 1, token_types_start: 1467, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 1467, raw_values_count: 1, token_types_start: 1468, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 1468, raw_values_count: 1, token_types_start: 1469, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 1469, raw_values_count: 1, token_types_start: 1470, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 1470, raw_values_count: 1, token_types_start: 1471, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 1471, raw_values_count: 2, token_types_start: 1473, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 1473, raw_values_count: 1, token_types_start: 1474, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 1474, raw_values_count: 1, token_types_start: 1475, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 1475, raw_values_count: 1, token_types_start: 1476, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 1476, raw_values_count: 1, token_types_start: 1477, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 1477, raw_values_count: 1, token_types_start: 1478, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 1478, raw_values_count: 1, token_types_start: 1479, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 1479, raw_values_count: 1, token_types_start: 1480, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 1480, raw_values_count: 1, token_types_start: 1481, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 1481, raw_values_count: 1, token_types_start: 1482, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 1482, raw_values_count: 1, token_types_start: 1483, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 1483, raw_values_count: 1, token_types_start: 1484, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 1484, raw_values_count: 1, token_types_start: 1485, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 1485, raw_values_count: 1, token_types_start: 1486, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 1486, raw_values_count: 1, token_types_start: 1487, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 1487, raw_values_count: 1, token_types_start: 1488, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 1488, raw_values_count: 1, token_types_start: 1489, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 1489, raw_values_count: 8, token_types_start: 1497, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 1497, raw_values_count: 1, token_types_start: 1498, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 1, token_types_start: 1499, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 1499, raw_values_count: 2, token_types_start: 1501, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 1501, raw_values_count: 1, token_types_start: 1502, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 1502, raw_values_count: 1, token_types_start: 1503, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 1503, raw_values_count: 1, token_types_start: 1504, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 1504, raw_values_count: 1, token_types_start: 1505, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 1505, raw_values_count: 1, token_types_start: 1506, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 1506, raw_values_count: 1, token_types_start: 1507, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 1507, raw_values_count: 1, token_types_start: 1508, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 1508, raw_values_count: 1, token_types_start: 1509, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 1509, raw_values_count: 1, token_types_start: 1510, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 1, token_types_start: 1511, token_types_count: 0 }, // [1063]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 0, 5, 0, 6, 7, 7, 8, 9, 10, 11, 12, 13,
    14, 15, 16, 17, 18, 19, 19, 20, 21, 22, 22, 23, 24, 24, 25, 26,
    10, 27, 27, 25, 26, 10, 28, 24, 24, 29, 26, 14, 27, 27, 29, 26,
    30, 10, 14, 0, 0, 0, 0, 0, 31, 0, 32, 33, 34, 35, 35, 36,
    37, 38, 39, 39, 40, 16, 41, 9, 10, 12, 42, 42, 43, 44, 45, 45,
    46, 47, 47, 48, 8, 8, 49, 50, 51, 52, 52, 39, 39, 53, 14, 43,
    43, 44, 54, 28, 55, 55, 56, 14, 15, 57, 57, 58, 42, 59, 39, 60,
    45, 61, 62, 8, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 51,
    74, 75, 24, 24, 58, 31, 38, 38, 76, 28, 77, 78, 79, 80, 81, 82,
    83, 84, 46, 14, 43, 43, 44, 85, 85, 78, 86, 86, 77, 87, 87, 88,
    29, 89, 90, 91, 92, 93, 94, 95, 96, 97, 25, 43, 43, 0, 85, 85,
    90, 86, 86, 89, 87, 87, 0, 98, 99, 100, 0, 101, 101, 0, 0, 41,
    41, 0, 0, 16, 16, 0, 0, 102, 102, 0, 0, 0, 0, 0, 0, 103,
    104, 104, 5, 105, 106, 107, 108, 109, 110, 111, 112, 0, 0, 113, 0, 0,
    0, 31, 0, 31, 114, 115, 0, 116, 0, 0, 0, 24, 117, 118, 119, 120,
    121, 121, 122, 37, 123, 124, 125, 126, 126, 37, 127, 128, 129, 130, 131, 119,
    119, 83, 0, 120, 120, 132, 132, 133, 134, 0, 0, 0, 0, 135, 0, 0,
    0, 136, 136, 108, 62, 133, 0, 137, 138, 138, 139, 110, 0, 31, 132, 140,
    140, 141, 113, 33, 0, 119, 119, 77, 0, 0, 0, 142, 143, 144, 145, 146,
    146, 147, 42, 148, 149, 150, 151, 152, 153, 154, 143, 155, 156, 157, 158, 104,
    0, 0, 37, 0, 159, 104, 160, 0, 161, 104, 0, 0, 0, 113, 113, 113,
    162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 173, 173, 174, 175,
    176, 177, 178, 0, 0, 178, 179, 0, 0, 0, 0, 0, 0, 32, 113, 0,
    0, 180, 181, 182, 183, 184, 185, 186, 149, 155, 187, 188, 0, 189, 190, 191,
    192, 143, 157, 157, 157, 155, 158, 158, 158, 156, 193, 194, 187, 146, 195, 196,
    197, 198, 199, 200, 31, 0, 0, 31, 0, 0, 0, 31, 201, 202, 203, 204,
    205, 122, 31, 0, 0, 0, 31, 113, 31, 206, 126, 207, 208, 209, 210, 211,
    212, 212, 212, 0, 213, 213, 214, 215, 214, 215, 215, 212, 212, 0, 213, 213,
    214, 215, 214, 215, 215, 188, 32, 186, 212, 216, 217, 15, 218, 219, 220, 221,
    222, 0, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236,
    237, 238, 239, 240, 241, 242, 243, 0, 0, 244, 245, 0, 115, 0, 0, 246,
    0, 247, 248, 249, 250, 159, 159, 159, 0, 0, 0, 0, 178, 0, 251, 251,
    0, 0, 252, 253, 254, 232, 232, 31, 0, 255, 255, 0, 256, 256, 0, 257,
    173, 67, 67, 0, 258, 244, 244, 0, 0, 0, 0, 0, 31, 0, 0, 0,
    0, 133, 0, 0, 246, 0, 259, 0, 0, 0, 0, 260, 32, 261, 262, 258,
    258, 115, 258, 0, 263, 264, 188, 135, 158, 157, 158, 157, 265, 266, 267, 0,
    268, 0, 0, 269, 270, 155, 155, 155, 271, 272, 273, 0, 274, 273, 0, 59,
    275, 276, 251, 277, 278, 279, 280, 281, 282, 283, 284, 285, 286, 287, 288, 289,
    290, 291, 292, 39, 39, 216, 31, 0, 113, 0, 293, 294, 295, 82, 80, 81,
    296, 297, 298, 299, 281, 300, 0, 31, 301, 301, 0, 113, 113, 165, 39, 39,
    10, 134, 0, 39, 39, 195, 302, 80, 134, 0, 31, 303, 303, 0, 113, 39,
    39, 195, 257, 304, 134, 0, 76, 0, 31, 31, 0, 0, 39, 39, 39, 195,
    305, 134, 0, 306, 306, 31, 0, 0, 0, 135, 0, 0, 104, 115, 115, 113,
    206, 39, 39, 16, 134, 0, 39, 39, 14, 134, 0, 307, 121, 121, 122, 37,
    308, 308, 293, 37, 123, 124, 125, 126, 126, 37, 127, 128, 129, 130, 131, 39,
    39, 83, 0, 307, 307, 39, 39, 195, 309, 77, 134, 0, 310, 31, 31, 0,
    0, 311, 0, 258, 113, 113, 206, 31, 206, 312, 312, 0, 0, 39, 39, 71,
    0, 313, 180, 110, 314, 314, 315, 316, 316, 61, 60, 73, 73, 315, 0, 0,
    195, 76, 195, 76, 76, 0, 317, 318, 318, 319, 320, 113, 0, 321, 320, 113,
    0, 34, 34, 0, 322, 253, 253, 323, 324, 323, 325, 325, 325, 326, 325, 325,
    327, 301, 301, 328, 329, 320, 330, 213, 213, 31, 0, 45, 45, 81, 331, 31,
    39, 39, 41, 0, 39, 39, 195, 78, 134, 0, 31, 113, 206, 31, 206, 293,
    332, 333, 334, 273, 335, 336, 336, 337, 337, 336, 338, 31, 0, 339, 340, 341,
    342, 343, 344, 345, 346, 347, 348, 349, 350, 129, 22, 10, 351, 0, 0, 0,
    0, 0, 352, 353, 353, 354, 0, 0, 355, 355, 225, 185, 356, 357, 35, 35,
    36, 0, 0, 0, 246, 0, 31, 0, 358, 0, 0, 359, 360, 360, 361, 362,
    363, 364, 365, 366, 367, 368, 369, 370, 371, 372, 373, 374, 159, 37, 375, 375,
    31, 31, 0, 0, 376, 0, 377, 378, 379, 380, 380, 359, 381, 382, 179, 115,
    383, 376, 384, 385, 386, 387, 388, 389, 390, 391, 392, 393, 394, 395, 396, 397,
    398, 399, 255, 255, 255, 400, 401, 323, 327, 402, 403, 404, 405, 406, 60, 60,
    60, 34, 407, 408, 409, 410, 411, 412, 413, 414, 415, 416, 417, 417, 417, 0,
    0, 418, 419, 420, 421, 422, 423, 424, 425, 426, 427, 428, 429, 430, 431, 152,
    432, 11, 246, 353, 433, 434, 210, 132, 132, 216, 31, 0, 113, 0, 433, 132,
    132, 10, 134, 0, 433, 132, 132, 80, 134, 0, 132, 132, 304, 134, 0, 433,
    132, 132, 132, 305, 134, 0, 132, 132, 16, 134, 0, 132, 132, 14, 134, 0,
    433, 132, 132, 83, 0, 132, 132, 302, 77, 134, 0, 0, 433, 132, 132, 71,
    134, 0, 132, 132, 18, 134, 0, 433, 132, 132, 41, 134, 0, 132, 132, 78,
    134, 0, 433, 435, 436, 437, 438, 439, 328, 440, 214, 214, 0, 214, 441, 31,
    0, 0, 31, 442, 443, 444, 445, 446, 447, 215, 448, 449, 135, 450, 451, 452,
    453, 454, 455, 456, 457, 458, 459, 45, 50, 460, 461, 462, 463, 464, 464, 464,
    465, 206, 61, 73, 60, 0, 0, 0, 0, 0, 466, 186, 186, 0, 467, 188,
    267, 267, 253, 0, 0, 0, 0, 468, 253, 183, 0, 146, 0, 0, 0, 0,
    469, 470, 470, 470, 471, 253, 472, 0, 0, 473, 473, 473, 473, 149, 155, 188,
    0, 474, 470, 470, 470, 471, 0, 461, 461, 31, 206, 0, 0, 0, 212, 167,
    167, 0, 360, 32, 0, 0, 178, 0, 31, 0, 0, 0, 206, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 31, 0, 0, 0, 0, 475,
    115, 37, 197, 254, 377, 0, 0, 32, 104, 0, 68, 0, 179, 0, 0, 0,
    476, 0, 477, 86, 478, 199, 199, 479, 479, 480, 139, 481, 0, 37, 0, 320,
    482, 320, 483, 484, 485, 293, 293, 486, 479, 487, 87, 488, 408, 408, 206, 206,
    408, 408, 408, 408, 489, 490, 490, 31, 407, 407, 0, 490, 491, 139, 492, 493,
    494, 495, 496, 497, 301, 498, 19, 19, 499, 19, 88, 500, 501, 502, 503, 503,
    0, 0, 340, 340, 320, 0, 0, 37, 179, 504, 0, 505, 506, 497, 183, 183,
    146, 503, 483, 507, 508, 509, 510, 511, 34, 34, 0, 0, 512, 407, 513, 101,
    101, 122, 112, 112, 122, 514, 515, 516, 517, 293, 293, 479, 518, 0, 0, 0,
    0, 31, 0, 0, 0, 0, 512, 519, 0, 0, 293, 293, 518, 0, 519, 0,
    31, 0, 0, 0, 0, 31, 0, 0, 520, 0, 31, 0, 112, 112, 122, 101,
    101, 122, 0, 0, 0, 0, 0, 0, 112, 112, 122, 101, 101, 122, 0, 34,
    521, 522, 0, 0, 0, 0, 0, 113, 0, 523, 523, 0, 34, 34, 0, 0,
    0, 376, 0, 34, 0, 0, 428, 0, 153, 0, 0, 112, 524, 524, 0, 0,
    0, 115, 0, 0, 26, 0, 115, 0, 0, 525, 0, 0, 31, 31, 0, 113,
    113, 115, 13, 13, 0, 80, 331, 0, 0, 0, 0, 246, 31, 526, 331, 527,
    31, 0, 0, 0, 0, 147, 147, 18, 0, 0, 147, 147, 18, 0, 31, 0,
    0, 0, 375, 375, 31, 0, 0, 0, 0, 0, 375, 0, 528, 31, 0, 92,
    529, 27, 530, 531, 532, 533, 534, 535, 536, 537, 99, 99, 99, 0, 38, 38,
    76, 0, 16, 16, 0, 64, 64, 76, 41, 0, 0, 33, 0, 538, 293, 293,
    99, 539, 293, 293, 109, 539, 287, 287, 340, 48, 540, 540, 122, 0, 348, 541,
    0, 540, 48, 158, 158, 135, 158, 542, 101, 101, 122, 0, 24, 543, 337, 0,
    0, 0, 0, 37, 0, 544, 544, 112, 112, 122, 513, 514, 515, 516, 479, 101,
    0, 0, 0, 0, 37, 0, 31, 544, 543, 543, 543, 545, 31, 0, 0, 337,
    0, 546, 514, 514, 0, 31, 0, 547, 112, 112, 122, 513, 515, 516, 479, 514,
    548, 549, 550, 551, 552, 553, 0, 554, 555, 556, 557, 558, 559, 560, 0, 561,
    562, 563, 134, 134, 461, 134, 134, 134, 253, 461, 564, 525, 525, 564, 565, 566,
    567, 326, 568, 569, 570, 57, 26, 467, 253, 26, 0, 31, 0, 0, 0, 206,
    0, 571, 572, 121, 0, 0, 573, 162, 416, 304, 574, 575, 576, 577, 578, 579,
    580, 325, 581, 582, 583, 584, 61, 61, 61, 585, 586, 0, 587, 206, 31, 31,
    206, 255, 588, 298, 589, 314, 590, 591, 592, 593, 594, 595, 596, 12, 597, 598,
    599, 179, 179, 600, 37, 37, 601, 115, 376, 115, 115, 115, 601, 115, 376, 179,
    586, 602, 0, 254, 603, 0, 197, 0, 267, 267, 253, 428, 34, 267, 0, 604,
    605, 606, 607, 0, 0, 0, 608, 0, 0, 0, 0, 0, 0, 609, 76, 610,
    0, 0, 0, 608, 0, 0, 0, 0, 0, 608, 608, 608, 0, 76, 76, 0,
    0, 31, 0, 0, 274, 581, 611, 611, 521, 612, 613, 614, 610, 610, 31, 0,
    0, 615, 616, 499, 617, 618, 619, 0, 620, 621, 13, 35, 622, 623, 624, 625,
    626, 627, 361, 362, 363, 364, 365, 366, 367, 628, 629, 630, 0, 631, 632, 633,
    634, 612, 635, 636, 157, 157, 135, 157, 637, 466, 466, 253, 638, 0, 451, 451,
    0, 638, 312, 639, 567, 312, 266, 513, 513, 0, 31, 0, 37, 0, 24, 640,
    518, 518, 0, 37, 0, 32, 32, 37, 513, 641, 642, 0, 643, 0, 115, 37,
    197, 470, 254, 377, 104, 0, 644, 0, 0, 645, 646, 0, 647, 648, 649, 650,
    651, 652, 653, 654, 655, 656, 657, 658, 659, 660, 661, 662, 520, 520, 246, 0,
    31, 305, 305, 0, 32, 32, 31, 206, 47, 663, 0, 43, 0, 664, 665, 85,
    666, 667, 668, 669, 670, 671, 672, 673, 60, 61, 61, 31, 528, 674, 674, 586,
    674, 213, 213, 213, 213, 213, 665, 146, 146, 0, 675, 676, 73, 60, 213, 213,
    253, 665, 146, 146, 0, 675, 61, 674, 674, 0, 0, 0, 610, 0, 0, 0,
    31, 31, 206, 0, 0, 0, 0, 0, 76, 0, 213, 73, 73, 677, 678, 679,
    680, 300, 681, 682, 683, 684, 685, 151, 686, 687, 688, 689, 520, 690, 691, 692,
    305, 693, 62, 694, 154, 8, 695, 696, 697, 698, 153, 699, 700, 701, 0, 702,
    0, 0, 113, 0, 0, 31, 0, 516, 516, 0, 0, 703, 603, 603, 704, 705,
    705, 274, 705, 706, 707, 151, 0, 708, 321, 0, 481, 709, 710, 711, 712, 368,
    369, 370, 371, 372, 373, 374, 713, 714, 715, 716, 717, 718, 127, 719, 720, 721,
    722, 723, 130, 724, 725, 201, 528, 68, 31, 68, 31, 293, 31, 201, 31, 0,
    726, 73, 61, 60, 674, 201, 201, 727, 31, 68, 201, 728, 729, 730, 731, 0,
    732, 733, 0, 265, 734, 734, 135, 157, 157, 158, 253, 0, 253, 735, 736, 737,
    738, 254, 254, 739, 740, 566, 741, 742, 37, 36, 115, 115, 159, 0, 644, 115,
    115, 644, 246, 246, 246, 246, 246, 0, 76, 113, 610, 32, 217, 104, 31, 186,
    159, 408, 0, 743, 744, 745, 315, 746, 747, 518, 518, 0, 37, 0, 320, 482,
    320, 483, 518, 748, 749, 319, 76, 750, 485, 751, 752, 753, 754, 755, 63, 756,
    757, 758, 539, 759, 306, 195, 195, 195, 195, 760, 112, 112, 122, 0, 0, 0,
    0, 37, 0, 573, 162, 416, 566, 566, 761, 139, 622, 0, 513, 503, 762, 513,
    514, 515, 516, 503, 524, 479, 112, 131, 112, 730, 763, 764, 765, 766, 614, 767,
    768, 0, 525, 769, 0, 0, 31, 0, 769, 476, 476, 0, 31, 31, 377, 32,
    377, 0, 476, 770, 771, 585, 772, 64, 773, 774, 775, 0, 776, 777, 778, 779,
    780, 781, 782, 783, 784, 785, 785, 122, 0, 31, 0, 0, 785, 786, 787, 788,
    789, 152, 152, 331, 331, 152, 115, 0, 790, 791, 792, 793, 794, 54, 155, 795,
    796, 797, 150, 44, 798, 150, 0, 0, 490, 0, 799, 800, 0, 506, 354, 801,
    802, 803, 804, 805, 256, 256, 499, 256, 806, 472, 58, 807, 808, 81, 93, 809,
    810, 103, 811, 470, 470, 37, 515, 188, 188, 812, 24, 147, 813, 31, 206, 814,
    815, 527, 115, 816, 817, 507, 818, 135, 158, 819, 157, 734, 65, 820, 821, 822,
    823, 824, 825, 826, 67, 67, 0, 31, 664, 76, 76, 76, 60, 827, 76, 76,
    73, 827, 664, 664, 828, 521, 784, 829, 66, 67, 318, 827, 434, 210, 677, 677,
    254, 716, 716, 106, 677, 677, 255, 830, 831, 832, 833, 834, 835, 836, 837, 838,
    839, 840, 841, 842, 843, 140, 844, 845, 760, 846, 847, 848, 849, 7, 565, 850,
    851, 434, 852, 853, 854, 855, 856, 857, 303, 100, 100, 100, 99, 99, 539, 301,
    0, 38, 38, 76, 0, 16, 16, 0, 64, 64, 76, 41, 0, 0, 34, 0,
    433, 613, 639, 16, 0, 858, 859, 338, 338, 82, 860, 861, 862, 94, 320, 863,
    864, 865, 866, 867, 868, 483, 869, 519, 519, 870, 182, 871, 31, 37, 845, 845,
    845, 31, 37, 872, 873, 874, 14, 875, 25, 876, 877, 878, 879, 880, 881, 882,
    883, 884, 885, 0, 0, 0, 0, 0, 886, 428, 24, 68, 68, 886, 0, 0,
    887, 887, 34, 0, 407, 112, 112, 122, 513, 476, 517, 479, 68, 68, 68, 34,
    407, 101, 514, 476, 516, 112, 518, 479, 513, 516, 517, 293, 293, 206, 888, 31,
    293, 888, 31, 293, 201, 31, 206, 889, 408, 890, 524, 83, 124, 669, 669, 37,
    716, 716, 669, 123, 689, 689, 37, 716, 716, 689, 95, 891, 21, 892, 893, 541,
    677, 677, 0, 0, 0, 0, 135, 0, 0, 178, 0, 0, 0, 528, 255, 201,
    201, 517, 517, 517, 201, 112, 513, 516, 677, 517, 894, 895, 895, 598, 454, 24,
    685, 677, 677, 14, 134, 0, 896, 545, 897, 102, 898, 0, 0, 0, 212, 217,
    217, 217, 0, 167, 899, 900, 901, 470, 150, 151, 902, 829, 0, 0, 527, 246,
    0, 0, 0, 0, 246, 153, 115, 903, 0, 0, 104, 152, 159, 904, 813, 905,
    906, 907, 908, 294, 909, 910, 911, 912, 913, 914, 915, 916, 917, 918, 919, 920,
    921, 922, 923, 924, 925, 926, 927, 928, 929, 930, 79, 91, 153, 308, 931, 932,
    933, 330, 206, 299, 934, 935, 936, 937, 938, 939, 940, 84, 96, 941, 155, 155,
    942, 31, 0, 0, 0, 0, 0, 943, 944, 945, 946, 947, 948, 949, 950, 951,
    0, 952, 953, 954, 955, 871, 956, 311, 251, 251, 0, 957, 257, 257, 31, 256,
    256, 31, 19, 19, 31, 67, 0, 0, 528, 178, 0, 0, 31, 206, 31, 674,
    77, 958, 89, 519, 17, 0, 469, 469, 186, 0, 0, 474, 474, 0, 46, 97,
    69, 959, 0, 302, 69, 70, 70, 309, 960, 302, 961, 962, 963, 964, 965, 675,
    966, 967, 968, 969, 970, 971, 486, 471, 381, 352, 352, 381, 382, 31, 972, 972,
    293, 973, 381, 974, 167, 167, 167, 381, 974, 0, 382, 975, 976, 977, 978, 979,
    33, 980, 981, 982, 983, 984, 985, 986, 987, 988, 988, 308, 181, 263, 859, 215,
    989, 987, 990, 702, 702, 0, 146, 146, 716, 221, 991, 992, 993, 994, 995, 996,
    960, 997, 998, 999, 71, 1000, 1001, 1002, 1003, 523, 200, 632, 985, 198, 198, 72,
    72, 72, 77, 0, 1004, 1005, 31, 0, 0, 18, 0, 0, 104, 0, 0, 31,
    1006, 1007, 504, 1008, 0, 0, 705, 0, 0, 1009, 1010, 1011, 894, 894, 886, 428,
    24, 894, 257, 257, 257, 1012, 0, 1013, 1014, 1015, 1016, 68, 68, 34, 407, 101,
    514, 476, 516, 517, 293, 293, 112, 513, 201, 201, 517, 517, 517, 201, 358, 1017,
    73, 73, 73, 0, 0, 677, 677, 34, 407, 1018, 1019, 51, 55, 55, 55, 0,
    74, 41, 0, 1020, 1021, 1022, 1023, 1024, 610, 1025, 1026, 1027, 1028, 1029, 1030, 1031,
    1032, 528, 528, 1032, 400, 1033, 1033, 320, 31, 0, 0, 255, 0, 0, 400, 1034,
    1035, 1036, 1037, 1038, 1039, 1040, 1041, 357, 1042, 1043, 78, 90, 1044, 1045, 9, 1046,
    1047, 1048, 213, 213, 0, 0, 0, 675, 0, 0, 0, 0, 213, 1049, 407, 407,
    0, 31, 0, 1050, 513, 101, 101, 122, 112, 112, 122, 514, 515, 516, 476, 479,
    407, 1051, 0, 1052, 0, 0, 0, 0, 0, 0, 0, 246, 153, 153, 246, 153,
    516, 0, 0, 1053, 785, 112, 785, 112, 503, 293, 293, 825, 0, 0, 0, 68,
    0, 726, 293, 293, 825, 0, 0, 0, 68, 0, 201, 293, 293, 716, 716, 22,
    0, 293, 293, 293, 716, 14, 189, 1054, 973, 0, 990, 23, 75, 1055, 1056, 1057,
    1058, 1059, 1060, 1061, 1062, 1063, 974,
];

pub fn get_soql_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &SOQL_TABLES }),
        "AbsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &SOQL_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &SOQL_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &SOQL_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(4), tables: &SOQL_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(65), tables: &SOQL_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(134), tables: &SOQL_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(137), tables: &SOQL_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(160), tables: &SOQL_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(183), tables: &SOQL_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(186), tables: &SOQL_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(208), tables: &SOQL_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(210), tables: &SOQL_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &SOQL_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &SOQL_TABLES }),
        "AdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &SOQL_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(214), tables: &SOQL_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(215), tables: &SOQL_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(216), tables: &SOQL_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(217), tables: &SOQL_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(218), tables: &SOQL_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(219), tables: &SOQL_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(231), tables: &SOQL_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(232), tables: &SOQL_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(235), tables: &SOQL_TABLES }),
        "AllocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(236), tables: &SOQL_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(237), tables: &SOQL_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(238), tables: &SOQL_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(239), tables: &SOQL_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(255), tables: &SOQL_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(261), tables: &SOQL_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(266), tables: &SOQL_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(293), tables: &SOQL_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(299), tables: &SOQL_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(300), tables: &SOQL_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(301), tables: &SOQL_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(302), tables: &SOQL_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(303), tables: &SOQL_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(304), tables: &SOQL_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(305), tables: &SOQL_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(306), tables: &SOQL_TABLES }),
        "AreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(307), tables: &SOQL_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(308), tables: &SOQL_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(319), tables: &SOQL_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(327), tables: &SOQL_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(328), tables: &SOQL_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(329), tables: &SOQL_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(332), tables: &SOQL_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(333), tables: &SOQL_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(335), tables: &SOQL_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(336), tables: &SOQL_TABLES }),
        "AsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(337), tables: &SOQL_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(338), tables: &SOQL_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(339), tables: &SOQL_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(340), tables: &SOQL_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(341), tables: &SOQL_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(342), tables: &SOQL_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(343), tables: &SOQL_TABLES }),
        "AttributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(344), tables: &SOQL_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(345), tables: &SOQL_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(346), tables: &SOQL_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(347), tables: &SOQL_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(349), tables: &SOQL_TABLES }),
        "AvgKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(350), tables: &SOQL_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(351), tables: &SOQL_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(352), tables: &SOQL_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(353), tables: &SOQL_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(354), tables: &SOQL_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(355), tables: &SOQL_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(367), tables: &SOQL_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(368), tables: &SOQL_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(369), tables: &SOQL_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(370), tables: &SOQL_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(371), tables: &SOQL_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(372), tables: &SOQL_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(373), tables: &SOQL_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(374), tables: &SOQL_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(375), tables: &SOQL_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(380), tables: &SOQL_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(381), tables: &SOQL_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(382), tables: &SOQL_TABLES }),
        "Bit_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(383), tables: &SOQL_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(384), tables: &SOQL_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(385), tables: &SOQL_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(386), tables: &SOQL_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(389), tables: &SOQL_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(390), tables: &SOQL_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(393), tables: &SOQL_TABLES }),
        "BlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(394), tables: &SOQL_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(395), tables: &SOQL_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(396), tables: &SOQL_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(399), tables: &SOQL_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(400), tables: &SOQL_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(403), tables: &SOQL_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(404), tables: &SOQL_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(407), tables: &SOQL_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(410), tables: &SOQL_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(411), tables: &SOQL_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(413), tables: &SOQL_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(414), tables: &SOQL_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(415), tables: &SOQL_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(416), tables: &SOQL_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(417), tables: &SOQL_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(418), tables: &SOQL_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(420), tables: &SOQL_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(426), tables: &SOQL_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(427), tables: &SOQL_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(428), tables: &SOQL_TABLES }),
        "CardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(429), tables: &SOQL_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(430), tables: &SOQL_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(431), tables: &SOQL_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(432), tables: &SOQL_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(456), tables: &SOQL_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(457), tables: &SOQL_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(458), tables: &SOQL_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(459), tables: &SOQL_TABLES }),
        "Catalog_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(460), tables: &SOQL_TABLES }),
        "CeilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(461), tables: &SOQL_TABLES }),
        "CeilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(462), tables: &SOQL_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(463), tables: &SOQL_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(464), tables: &SOQL_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(465), tables: &SOQL_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(466), tables: &SOQL_TABLES }),
        "Char_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(467), tables: &SOQL_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(468), tables: &SOQL_TABLES }),
        "Character_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(469), tables: &SOQL_TABLES }),
        "Character_set_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(470), tables: &SOQL_TABLES }),
        "Character_set_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(471), tables: &SOQL_TABLES }),
        "Character_set_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(472), tables: &SOQL_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(473), tables: &SOQL_TABLES }),
        "CharactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(474), tables: &SOQL_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(475), tables: &SOQL_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(476), tables: &SOQL_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(477), tables: &SOQL_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(478), tables: &SOQL_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(479), tables: &SOQL_TABLES }),
        "Class_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(480), tables: &SOQL_TABLES }),
        "ClobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(481), tables: &SOQL_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(482), tables: &SOQL_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(483), tables: &SOQL_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(484), tables: &SOQL_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(485), tables: &SOQL_TABLES }),
        "CobolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(486), tables: &SOQL_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(487), tables: &SOQL_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(488), tables: &SOQL_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(489), tables: &SOQL_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(490), tables: &SOQL_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(491), tables: &SOQL_TABLES }),
        "Collation_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(497), tables: &SOQL_TABLES }),
        "Collation_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(498), tables: &SOQL_TABLES }),
        "Collation_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(499), tables: &SOQL_TABLES }),
        "CollectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(500), tables: &SOQL_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(501), tables: &SOQL_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(502), tables: &SOQL_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(503), tables: &SOQL_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(504), tables: &SOQL_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(509), tables: &SOQL_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(537), tables: &SOQL_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(544), tables: &SOQL_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(545), tables: &SOQL_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SOQL_TABLES }),
        "Column_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(550), tables: &SOQL_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(551), tables: &SOQL_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(552), tables: &SOQL_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(553), tables: &SOQL_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(554), tables: &SOQL_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(555), tables: &SOQL_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(556), tables: &SOQL_TABLES }),
        "Command_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(557), tables: &SOQL_TABLES }),
        "Command_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(558), tables: &SOQL_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(559), tables: &SOQL_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(562), tables: &SOQL_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(563), tables: &SOQL_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &SOQL_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(565), tables: &SOQL_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(566), tables: &SOQL_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(575), tables: &SOQL_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(576), tables: &SOQL_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(577), tables: &SOQL_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(578), tables: &SOQL_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(579), tables: &SOQL_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(580), tables: &SOQL_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(581), tables: &SOQL_TABLES }),
        "ConditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(584), tables: &SOQL_TABLES }),
        "Condition_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(585), tables: &SOQL_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(586), tables: &SOQL_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(587), tables: &SOQL_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(591), tables: &SOQL_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(592), tables: &SOQL_TABLES }),
        "Connection_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(593), tables: &SOQL_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(594), tables: &SOQL_TABLES }),
        "Constraint_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(595), tables: &SOQL_TABLES }),
        "Constraint_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(596), tables: &SOQL_TABLES }),
        "Constraint_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(597), tables: &SOQL_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(598), tables: &SOQL_TABLES }),
        "ConstructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &SOQL_TABLES }),
        "ContainsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(600), tables: &SOQL_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(601), tables: &SOQL_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(602), tables: &SOQL_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(603), tables: &SOQL_TABLES }),
        "ConvertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(604), tables: &SOQL_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(605), tables: &SOQL_TABLES }),
        "CorrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(606), tables: &SOQL_TABLES }),
        "CorrespondingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(607), tables: &SOQL_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(608), tables: &SOQL_TABLES }),
        "Covar_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(609), tables: &SOQL_TABLES }),
        "Covar_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(610), tables: &SOQL_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(611), tables: &SOQL_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(638), tables: &SOQL_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(643), tables: &SOQL_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(655), tables: &SOQL_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(668), tables: &SOQL_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(669), tables: &SOQL_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(689), tables: &SOQL_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(694), tables: &SOQL_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(699), tables: &SOQL_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(719), tables: &SOQL_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(725), tables: &SOQL_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(749), tables: &SOQL_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(816), tables: &SOQL_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(820), tables: &SOQL_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(832), tables: &SOQL_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(833), tables: &SOQL_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(834), tables: &SOQL_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(835), tables: &SOQL_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(836), tables: &SOQL_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(837), tables: &SOQL_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(838), tables: &SOQL_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(839), tables: &SOQL_TABLES }),
        "Cume_distKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(845), tables: &SOQL_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(846), tables: &SOQL_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(847), tables: &SOQL_TABLES }),
        "Current_default_transform_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(848), tables: &SOQL_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(849), tables: &SOQL_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(850), tables: &SOQL_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(851), tables: &SOQL_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(852), tables: &SOQL_TABLES }),
        "Current_transform_group_for_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(853), tables: &SOQL_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(854), tables: &SOQL_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(855), tables: &SOQL_TABLES }),
        "Cursor_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &SOQL_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(857), tables: &SOQL_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(858), tables: &SOQL_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(859), tables: &SOQL_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SOQL_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(860), tables: &SOQL_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SOQL_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(865), tables: &SOQL_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(891), tables: &SOQL_TABLES }),
        "DateLiteralNSegment" => Some(RootGrammar { grammar_id: GrammarId(892), tables: &SOQL_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(910), tables: &SOQL_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(911), tables: &SOQL_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(912), tables: &SOQL_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(918), tables: &SOQL_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(928), tables: &SOQL_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(929), tables: &SOQL_TABLES }),
        "Datetime_interval_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(930), tables: &SOQL_TABLES }),
        "Datetime_interval_precisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(931), tables: &SOQL_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(932), tables: &SOQL_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(933), tables: &SOQL_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(934), tables: &SOQL_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(935), tables: &SOQL_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(936), tables: &SOQL_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(937), tables: &SOQL_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(938), tables: &SOQL_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(939), tables: &SOQL_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(940), tables: &SOQL_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(941), tables: &SOQL_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(942), tables: &SOQL_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(943), tables: &SOQL_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(944), tables: &SOQL_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(945), tables: &SOQL_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(230), tables: &SOQL_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(946), tables: &SOQL_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(947), tables: &SOQL_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(950), tables: &SOQL_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(951), tables: &SOQL_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(952), tables: &SOQL_TABLES }),
        "DefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(953), tables: &SOQL_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(954), tables: &SOQL_TABLES }),
        "DegreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(955), tables: &SOQL_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(956), tables: &SOQL_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(957), tables: &SOQL_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(958), tables: &SOQL_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(959), tables: &SOQL_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(963), tables: &SOQL_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(964), tables: &SOQL_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(965), tables: &SOQL_TABLES }),
        "Dense_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(966), tables: &SOQL_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(967), tables: &SOQL_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(968), tables: &SOQL_TABLES }),
        "DerefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(969), tables: &SOQL_TABLES }),
        "DerivedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(970), tables: &SOQL_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(971), tables: &SOQL_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(972), tables: &SOQL_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(973), tables: &SOQL_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(977), tables: &SOQL_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(978), tables: &SOQL_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(979), tables: &SOQL_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(980), tables: &SOQL_TABLES }),
        "DiagnosticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(981), tables: &SOQL_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(982), tables: &SOQL_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(983), tables: &SOQL_TABLES }),
        "DisconnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(984), tables: &SOQL_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(985), tables: &SOQL_TABLES }),
        "DispatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(986), tables: &SOQL_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(987), tables: &SOQL_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(988), tables: &SOQL_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(989), tables: &SOQL_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(990), tables: &SOQL_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(991), tables: &SOQL_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(992), tables: &SOQL_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(993), tables: &SOQL_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(994), tables: &SOQL_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(995), tables: &SOQL_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(996), tables: &SOQL_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(999), tables: &SOQL_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1007), tables: &SOQL_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1013), tables: &SOQL_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1018), tables: &SOQL_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1024), tables: &SOQL_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1025), tables: &SOQL_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1030), tables: &SOQL_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1035), tables: &SOQL_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1041), tables: &SOQL_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1045), tables: &SOQL_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1053), tables: &SOQL_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1058), tables: &SOQL_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1064), tables: &SOQL_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1069), tables: &SOQL_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1075), tables: &SOQL_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1076), tables: &SOQL_TABLES }),
        "DynamicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1077), tables: &SOQL_TABLES }),
        "Dynamic_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1078), tables: &SOQL_TABLES }),
        "Dynamic_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &SOQL_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1080), tables: &SOQL_TABLES }),
        "ElementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1081), tables: &SOQL_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1082), tables: &SOQL_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1085), tables: &SOQL_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1086), tables: &SOQL_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(1087), tables: &SOQL_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1088), tables: &SOQL_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1091), tables: &SOQL_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1092), tables: &SOQL_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1093), tables: &SOQL_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1094), tables: &SOQL_TABLES }),
        "End-execKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1095), tables: &SOQL_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(228), tables: &SOQL_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(1096), tables: &SOQL_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1097), tables: &SOQL_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(326), tables: &SOQL_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1098), tables: &SOQL_TABLES }),
        "EqualsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1099), tables: &SOQL_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(1100), tables: &SOQL_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1101), tables: &SOQL_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1102), tables: &SOQL_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1103), tables: &SOQL_TABLES }),
        "EveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1104), tables: &SOQL_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1105), tables: &SOQL_TABLES }),
        "ExceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1106), tables: &SOQL_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1107), tables: &SOQL_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1108), tables: &SOQL_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1109), tables: &SOQL_TABLES }),
        "ExecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1110), tables: &SOQL_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1111), tables: &SOQL_TABLES }),
        "ExecutionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1112), tables: &SOQL_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1113), tables: &SOQL_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1114), tables: &SOQL_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1115), tables: &SOQL_TABLES }),
        "ExpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1116), tables: &SOQL_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1117), tables: &SOQL_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1118), tables: &SOQL_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1125), tables: &SOQL_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1126), tables: &SOQL_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &SOQL_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1159), tables: &SOQL_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1169), tables: &SOQL_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1174), tables: &SOQL_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1188), tables: &SOQL_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(1225), tables: &SOQL_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1233), tables: &SOQL_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1234), tables: &SOQL_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SOQL_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1235), tables: &SOQL_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1236), tables: &SOQL_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1237), tables: &SOQL_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(1238), tables: &SOQL_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1239), tables: &SOQL_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1256), tables: &SOQL_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1257), tables: &SOQL_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1258), tables: &SOQL_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(1259), tables: &SOQL_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1268), tables: &SOQL_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1269), tables: &SOQL_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1275), tables: &SOQL_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1276), tables: &SOQL_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1277), tables: &SOQL_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1278), tables: &SOQL_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1279), tables: &SOQL_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1280), tables: &SOQL_TABLES }),
        "FloorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1281), tables: &SOQL_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1282), tables: &SOQL_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1283), tables: &SOQL_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1284), tables: &SOQL_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1285), tables: &SOQL_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(1286), tables: &SOQL_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1289), tables: &SOQL_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1290), tables: &SOQL_TABLES }),
        "FortranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1291), tables: &SOQL_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1292), tables: &SOQL_TABLES }),
        "FoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1293), tables: &SOQL_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1294), tables: &SOQL_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(1313), tables: &SOQL_TABLES }),
        "FreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1316), tables: &SOQL_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1317), tables: &SOQL_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1318), tables: &SOQL_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1319), tables: &SOQL_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1320), tables: &SOQL_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1324), tables: &SOQL_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(1341), tables: &SOQL_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1364), tables: &SOQL_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1391), tables: &SOQL_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1392), tables: &SOQL_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1393), tables: &SOQL_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1394), tables: &SOQL_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1395), tables: &SOQL_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1436), tables: &SOQL_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1439), tables: &SOQL_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1445), tables: &SOQL_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1446), tables: &SOQL_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1447), tables: &SOQL_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1457), tables: &SOQL_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(1470), tables: &SOQL_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(1473), tables: &SOQL_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1487), tables: &SOQL_TABLES }),
        "FusionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1488), tables: &SOQL_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1489), tables: &SOQL_TABLES }),
        "GKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1490), tables: &SOQL_TABLES }),
        "GeneralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1491), tables: &SOQL_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1492), tables: &SOQL_TABLES }),
        "GetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1493), tables: &SOQL_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1494), tables: &SOQL_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1495), tables: &SOQL_TABLES }),
        "GoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1496), tables: &SOQL_TABLES }),
        "GotoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1497), tables: &SOQL_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1498), tables: &SOQL_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1499), tables: &SOQL_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1537), tables: &SOQL_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1538), tables: &SOQL_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(1539), tables: &SOQL_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(1542), tables: &SOQL_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1543), tables: &SOQL_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1544), tables: &SOQL_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1558), tables: &SOQL_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1567), tables: &SOQL_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(1568), tables: &SOQL_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1576), tables: &SOQL_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1577), tables: &SOQL_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1585), tables: &SOQL_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1586), tables: &SOQL_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1591), tables: &SOQL_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1599), tables: &SOQL_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1600), tables: &SOQL_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1601), tables: &SOQL_TABLES }),
        "HierarchyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1602), tables: &SOQL_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1603), tables: &SOQL_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1604), tables: &SOQL_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1605), tables: &SOQL_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1606), tables: &SOQL_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1607), tables: &SOQL_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1608), tables: &SOQL_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1609), tables: &SOQL_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1610), tables: &SOQL_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1611), tables: &SOQL_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1612), tables: &SOQL_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1613), tables: &SOQL_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1614), tables: &SOQL_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1615), tables: &SOQL_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1616), tables: &SOQL_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1617), tables: &SOQL_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1618), tables: &SOQL_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1621), tables: &SOQL_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1622), tables: &SOQL_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1626), tables: &SOQL_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1627), tables: &SOQL_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1632), tables: &SOQL_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1633), tables: &SOQL_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1634), tables: &SOQL_TABLES }),
        "ImplementationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1635), tables: &SOQL_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(435), tables: &SOQL_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1636), tables: &SOQL_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1637), tables: &SOQL_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1638), tables: &SOQL_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1639), tables: &SOQL_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1649), tables: &SOQL_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1650), tables: &SOQL_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1651), tables: &SOQL_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(220), tables: &SOQL_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(1652), tables: &SOQL_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1657), tables: &SOQL_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SOQL_TABLES }),
        "IndicatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1658), tables: &SOQL_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1659), tables: &SOQL_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1660), tables: &SOQL_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1661), tables: &SOQL_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1662), tables: &SOQL_TABLES }),
        "InitialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1663), tables: &SOQL_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1664), tables: &SOQL_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1665), tables: &SOQL_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1666), tables: &SOQL_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1667), tables: &SOQL_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1668), tables: &SOQL_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1669), tables: &SOQL_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1670), tables: &SOQL_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1671), tables: &SOQL_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1682), tables: &SOQL_TABLES }),
        "InstanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1683), tables: &SOQL_TABLES }),
        "InstantiableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1684), tables: &SOQL_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1685), tables: &SOQL_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1686), tables: &SOQL_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1687), tables: &SOQL_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1688), tables: &SOQL_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1689), tables: &SOQL_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1690), tables: &SOQL_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1691), tables: &SOQL_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1692), tables: &SOQL_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1693), tables: &SOQL_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1694), tables: &SOQL_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1695), tables: &SOQL_TABLES }),
        "IntersectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1696), tables: &SOQL_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1697), tables: &SOQL_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1711), tables: &SOQL_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1712), tables: &SOQL_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1713), tables: &SOQL_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1714), tables: &SOQL_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(1720), tables: &SOQL_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1725), tables: &SOQL_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(1726), tables: &SOQL_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1727), tables: &SOQL_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1728), tables: &SOQL_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1729), tables: &SOQL_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1730), tables: &SOQL_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1731), tables: &SOQL_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1753), tables: &SOQL_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1754), tables: &SOQL_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1756), tables: &SOQL_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(1757), tables: &SOQL_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &SOQL_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1772), tables: &SOQL_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1777), tables: &SOQL_TABLES }),
        "KKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1778), tables: &SOQL_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1779), tables: &SOQL_TABLES }),
        "Key_memberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1780), tables: &SOQL_TABLES }),
        "Key_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1781), tables: &SOQL_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1782), tables: &SOQL_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1783), tables: &SOQL_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1784), tables: &SOQL_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1785), tables: &SOQL_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1786), tables: &SOQL_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1787), tables: &SOQL_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1788), tables: &SOQL_TABLES }),
        "Last_90_daysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1789), tables: &SOQL_TABLES }),
        "Last_fiscal_quarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1790), tables: &SOQL_TABLES }),
        "Last_fiscal_yearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1791), tables: &SOQL_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1792), tables: &SOQL_TABLES }),
        "Last_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1793), tables: &SOQL_TABLES }),
        "Last_n_daysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1794), tables: &SOQL_TABLES }),
        "Last_n_fiscal_quartersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1795), tables: &SOQL_TABLES }),
        "Last_n_fiscal_yearsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1796), tables: &SOQL_TABLES }),
        "Last_n_monthsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1797), tables: &SOQL_TABLES }),
        "Last_n_quartersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1798), tables: &SOQL_TABLES }),
        "Last_n_weeksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1799), tables: &SOQL_TABLES }),
        "Last_n_yearsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1800), tables: &SOQL_TABLES }),
        "Last_quarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1801), tables: &SOQL_TABLES }),
        "Last_weekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1802), tables: &SOQL_TABLES }),
        "Last_yearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1803), tables: &SOQL_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1804), tables: &SOQL_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1805), tables: &SOQL_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1806), tables: &SOQL_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1807), tables: &SOQL_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1808), tables: &SOQL_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1809), tables: &SOQL_TABLES }),
        "LengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1810), tables: &SOQL_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1811), tables: &SOQL_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(1812), tables: &SOQL_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(1815), tables: &SOQL_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1816), tables: &SOQL_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1817), tables: &SOQL_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(1825), tables: &SOQL_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1829), tables: &SOQL_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1830), tables: &SOQL_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1831), tables: &SOQL_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1848), tables: &SOQL_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1849), tables: &SOQL_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1850), tables: &SOQL_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1851), tables: &SOQL_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1852), tables: &SOQL_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1853), tables: &SOQL_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1863), tables: &SOQL_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1864), tables: &SOQL_TABLES }),
        "LnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1865), tables: &SOQL_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1866), tables: &SOQL_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(1867), tables: &SOQL_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1868), tables: &SOQL_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1869), tables: &SOQL_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1870), tables: &SOQL_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1871), tables: &SOQL_TABLES }),
        "LocatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1872), tables: &SOQL_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1873), tables: &SOQL_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1874), tables: &SOQL_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1875), tables: &SOQL_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1876), tables: &SOQL_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1877), tables: &SOQL_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1878), tables: &SOQL_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1879), tables: &SOQL_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1880), tables: &SOQL_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1881), tables: &SOQL_TABLES }),
        "LowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1882), tables: &SOQL_TABLES }),
        "MKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1883), tables: &SOQL_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1884), tables: &SOQL_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1896), tables: &SOQL_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1897), tables: &SOQL_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(1898), tables: &SOQL_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1899), tables: &SOQL_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(1900), tables: &SOQL_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1901), tables: &SOQL_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1902), tables: &SOQL_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1903), tables: &SOQL_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1904), tables: &SOQL_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1905), tables: &SOQL_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1906), tables: &SOQL_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1907), tables: &SOQL_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1908), tables: &SOQL_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1909), tables: &SOQL_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1910), tables: &SOQL_TABLES }),
        "MemberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1911), tables: &SOQL_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1912), tables: &SOQL_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1913), tables: &SOQL_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1917), tables: &SOQL_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1920), tables: &SOQL_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(1921), tables: &SOQL_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1924), tables: &SOQL_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1934), tables: &SOQL_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1943), tables: &SOQL_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1963), tables: &SOQL_TABLES }),
        "Message_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1966), tables: &SOQL_TABLES }),
        "Message_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1967), tables: &SOQL_TABLES }),
        "Message_textKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1968), tables: &SOQL_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1969), tables: &SOQL_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1970), tables: &SOQL_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1971), tables: &SOQL_TABLES }),
        "MinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1972), tables: &SOQL_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1973), tables: &SOQL_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1974), tables: &SOQL_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(1975), tables: &SOQL_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1976), tables: &SOQL_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1977), tables: &SOQL_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1978), tables: &SOQL_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1979), tables: &SOQL_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1980), tables: &SOQL_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1981), tables: &SOQL_TABLES }),
        "ModKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1982), tables: &SOQL_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1983), tables: &SOQL_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1984), tables: &SOQL_TABLES }),
        "ModifiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1985), tables: &SOQL_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1986), tables: &SOQL_TABLES }),
        "ModuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1987), tables: &SOQL_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(1988), tables: &SOQL_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1989), tables: &SOQL_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1990), tables: &SOQL_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1991), tables: &SOQL_TABLES }),
        "MoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1992), tables: &SOQL_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1993), tables: &SOQL_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(1994), tables: &SOQL_TABLES }),
        "MultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1995), tables: &SOQL_TABLES }),
        "MumpsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1996), tables: &SOQL_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1997), tables: &SOQL_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1998), tables: &SOQL_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1999), tables: &SOQL_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2000), tables: &SOQL_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(2007), tables: &SOQL_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2011), tables: &SOQL_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2012), tables: &SOQL_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2013), tables: &SOQL_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2014), tables: &SOQL_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2015), tables: &SOQL_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2018), tables: &SOQL_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2019), tables: &SOQL_TABLES }),
        "NclobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2020), tables: &SOQL_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(2021), tables: &SOQL_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(2022), tables: &SOQL_TABLES }),
        "NestingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2023), tables: &SOQL_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2024), tables: &SOQL_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(2025), tables: &SOQL_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2026), tables: &SOQL_TABLES }),
        "Next_90_daysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2027), tables: &SOQL_TABLES }),
        "Next_fiscal_quarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2028), tables: &SOQL_TABLES }),
        "Next_fiscal_yearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2029), tables: &SOQL_TABLES }),
        "Next_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2030), tables: &SOQL_TABLES }),
        "Next_n_daysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2031), tables: &SOQL_TABLES }),
        "Next_n_fiscal_quartersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2032), tables: &SOQL_TABLES }),
        "Next_n_fiscal_yearsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2033), tables: &SOQL_TABLES }),
        "Next_n_monthsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2034), tables: &SOQL_TABLES }),
        "Next_n_quartersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2035), tables: &SOQL_TABLES }),
        "Next_n_weeksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2036), tables: &SOQL_TABLES }),
        "Next_n_yearsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2037), tables: &SOQL_TABLES }),
        "Next_quarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2038), tables: &SOQL_TABLES }),
        "Next_weekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2039), tables: &SOQL_TABLES }),
        "Next_yearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2040), tables: &SOQL_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2041), tables: &SOQL_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2042), tables: &SOQL_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2043), tables: &SOQL_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2044), tables: &SOQL_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2045), tables: &SOQL_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2046), tables: &SOQL_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2047), tables: &SOQL_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2048), tables: &SOQL_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2049), tables: &SOQL_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2050), tables: &SOQL_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2051), tables: &SOQL_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2052), tables: &SOQL_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2053), tables: &SOQL_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2063), tables: &SOQL_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2064), tables: &SOQL_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2069), tables: &SOQL_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2075), tables: &SOQL_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2076), tables: &SOQL_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2077), tables: &SOQL_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2078), tables: &SOQL_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2079), tables: &SOQL_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2080), tables: &SOQL_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2081), tables: &SOQL_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2082), tables: &SOQL_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2083), tables: &SOQL_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2090), tables: &SOQL_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(2091), tables: &SOQL_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2092), tables: &SOQL_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2093), tables: &SOQL_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2094), tables: &SOQL_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2095), tables: &SOQL_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2096), tables: &SOQL_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2097), tables: &SOQL_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2098), tables: &SOQL_TABLES }),
        "NullableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2099), tables: &SOQL_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2100), tables: &SOQL_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2101), tables: &SOQL_TABLES }),
        "NumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2102), tables: &SOQL_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2103), tables: &SOQL_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2104), tables: &SOQL_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2105), tables: &SOQL_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2106), tables: &SOQL_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2110), tables: &SOQL_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2114), tables: &SOQL_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SOQL_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2119), tables: &SOQL_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2131), tables: &SOQL_TABLES }),
        "Octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2132), tables: &SOQL_TABLES }),
        "OctetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2133), tables: &SOQL_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2134), tables: &SOQL_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2135), tables: &SOQL_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2136), tables: &SOQL_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2137), tables: &SOQL_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2146), tables: &SOQL_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2147), tables: &SOQL_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2148), tables: &SOQL_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2149), tables: &SOQL_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2150), tables: &SOQL_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2151), tables: &SOQL_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2152), tables: &SOQL_TABLES }),
        "OpenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2153), tables: &SOQL_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2154), tables: &SOQL_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2155), tables: &SOQL_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2156), tables: &SOQL_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2157), tables: &SOQL_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2158), tables: &SOQL_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2159), tables: &SOQL_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2160), tables: &SOQL_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2161), tables: &SOQL_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2162), tables: &SOQL_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2163), tables: &SOQL_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2164), tables: &SOQL_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2165), tables: &SOQL_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2166), tables: &SOQL_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(2167), tables: &SOQL_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2170), tables: &SOQL_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(2190), tables: &SOQL_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2198), tables: &SOQL_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(2199), tables: &SOQL_TABLES }),
        "OrderingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2202), tables: &SOQL_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2203), tables: &SOQL_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2204), tables: &SOQL_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2205), tables: &SOQL_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2206), tables: &SOQL_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2207), tables: &SOQL_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2208), tables: &SOQL_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2209), tables: &SOQL_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2216), tables: &SOQL_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2217), tables: &SOQL_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2226), tables: &SOQL_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2227), tables: &SOQL_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2228), tables: &SOQL_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2229), tables: &SOQL_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2230), tables: &SOQL_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2231), tables: &SOQL_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2232), tables: &SOQL_TABLES }),
        "PadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2233), tables: &SOQL_TABLES }),
        "ParameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2234), tables: &SOQL_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2235), tables: &SOQL_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(2236), tables: &SOQL_TABLES }),
        "Parameter_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2237), tables: &SOQL_TABLES }),
        "Parameter_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2238), tables: &SOQL_TABLES }),
        "Parameter_ordinal_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2239), tables: &SOQL_TABLES }),
        "Parameter_specific_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2240), tables: &SOQL_TABLES }),
        "Parameter_specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2241), tables: &SOQL_TABLES }),
        "Parameter_specific_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2242), tables: &SOQL_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2243), tables: &SOQL_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2244), tables: &SOQL_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2245), tables: &SOQL_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2252), tables: &SOQL_TABLES }),
        "PascalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2253), tables: &SOQL_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2254), tables: &SOQL_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2255), tables: &SOQL_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(2256), tables: &SOQL_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(2263), tables: &SOQL_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2264), tables: &SOQL_TABLES }),
        "PercentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2265), tables: &SOQL_TABLES }),
        "Percent_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2266), tables: &SOQL_TABLES }),
        "Percentile_contKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2267), tables: &SOQL_TABLES }),
        "Percentile_discKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2268), tables: &SOQL_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2269), tables: &SOQL_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(2270), tables: &SOQL_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2271), tables: &SOQL_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2272), tables: &SOQL_TABLES }),
        "PliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2273), tables: &SOQL_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(2274), tables: &SOQL_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2275), tables: &SOQL_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2276), tables: &SOQL_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(2277), tables: &SOQL_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2278), tables: &SOQL_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2281), tables: &SOQL_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2282), tables: &SOQL_TABLES }),
        "PowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2283), tables: &SOQL_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2284), tables: &SOQL_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2285), tables: &SOQL_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2286), tables: &SOQL_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2287), tables: &SOQL_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2288), tables: &SOQL_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2289), tables: &SOQL_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2290), tables: &SOQL_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2291), tables: &SOQL_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2292), tables: &SOQL_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2295), tables: &SOQL_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2296), tables: &SOQL_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2297), tables: &SOQL_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2298), tables: &SOQL_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2299), tables: &SOQL_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2300), tables: &SOQL_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2301), tables: &SOQL_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2302), tables: &SOQL_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2303), tables: &SOQL_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2304), tables: &SOQL_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2305), tables: &SOQL_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2306), tables: &SOQL_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2307), tables: &SOQL_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2310), tables: &SOQL_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2311), tables: &SOQL_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2319), tables: &SOQL_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2320), tables: &SOQL_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2321), tables: &SOQL_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2322), tables: &SOQL_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2323), tables: &SOQL_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2324), tables: &SOQL_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2325), tables: &SOQL_TABLES }),
        "RankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2326), tables: &SOQL_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(2327), tables: &SOQL_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2328), tables: &SOQL_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2329), tables: &SOQL_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2330), tables: &SOQL_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(2331), tables: &SOQL_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2332), tables: &SOQL_TABLES }),
        "ReadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2333), tables: &SOQL_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2334), tables: &SOQL_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2335), tables: &SOQL_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2336), tables: &SOQL_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2337), tables: &SOQL_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2338), tables: &SOQL_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2339), tables: &SOQL_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2340), tables: &SOQL_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(2354), tables: &SOQL_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2360), tables: &SOQL_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2361), tables: &SOQL_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2362), tables: &SOQL_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2363), tables: &SOQL_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2375), tables: &SOQL_TABLES }),
        "Regr_avgxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2376), tables: &SOQL_TABLES }),
        "Regr_avgyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2377), tables: &SOQL_TABLES }),
        "Regr_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2378), tables: &SOQL_TABLES }),
        "Regr_interceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2379), tables: &SOQL_TABLES }),
        "Regr_r2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2380), tables: &SOQL_TABLES }),
        "Regr_slopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2381), tables: &SOQL_TABLES }),
        "Regr_sxxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2382), tables: &SOQL_TABLES }),
        "Regr_sxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2383), tables: &SOQL_TABLES }),
        "Regr_syyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2384), tables: &SOQL_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2385), tables: &SOQL_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2386), tables: &SOQL_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2387), tables: &SOQL_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2388), tables: &SOQL_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2389), tables: &SOQL_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2390), tables: &SOQL_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2391), tables: &SOQL_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2392), tables: &SOQL_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2393), tables: &SOQL_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2394), tables: &SOQL_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2395), tables: &SOQL_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2396), tables: &SOQL_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2397), tables: &SOQL_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2398), tables: &SOQL_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2399), tables: &SOQL_TABLES }),
        "RestoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2400), tables: &SOQL_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2401), tables: &SOQL_TABLES }),
        "ResultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2402), tables: &SOQL_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2403), tables: &SOQL_TABLES }),
        "Returned_cardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2404), tables: &SOQL_TABLES }),
        "Returned_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2405), tables: &SOQL_TABLES }),
        "Returned_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2406), tables: &SOQL_TABLES }),
        "Returned_sqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2407), tables: &SOQL_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2408), tables: &SOQL_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2409), tables: &SOQL_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2410), tables: &SOQL_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2433), tables: &SOQL_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2434), tables: &SOQL_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2435), tables: &SOQL_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2436), tables: &SOQL_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2437), tables: &SOQL_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2438), tables: &SOQL_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2439), tables: &SOQL_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2440), tables: &SOQL_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2441), tables: &SOQL_TABLES }),
        "Routine_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2442), tables: &SOQL_TABLES }),
        "Routine_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2443), tables: &SOQL_TABLES }),
        "Routine_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2444), tables: &SOQL_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2445), tables: &SOQL_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2446), tables: &SOQL_TABLES }),
        "Row_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2447), tables: &SOQL_TABLES }),
        "Row_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2448), tables: &SOQL_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2449), tables: &SOQL_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2450), tables: &SOQL_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2451), tables: &SOQL_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2452), tables: &SOQL_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2453), tables: &SOQL_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2454), tables: &SOQL_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2455), tables: &SOQL_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2467), tables: &SOQL_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2468), tables: &SOQL_TABLES }),
        "ScaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2469), tables: &SOQL_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2470), tables: &SOQL_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SOQL_TABLES }),
        "Schema_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2471), tables: &SOQL_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2472), tables: &SOQL_TABLES }),
        "ScopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2473), tables: &SOQL_TABLES }),
        "Scope_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2474), tables: &SOQL_TABLES }),
        "Scope_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2475), tables: &SOQL_TABLES }),
        "Scope_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2476), tables: &SOQL_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2477), tables: &SOQL_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2478), tables: &SOQL_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2479), tables: &SOQL_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2480), tables: &SOQL_TABLES }),
        "SectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2481), tables: &SOQL_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2482), tables: &SOQL_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2483), tables: &SOQL_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2488), tables: &SOQL_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2491), tables: &SOQL_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2497), tables: &SOQL_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2508), tables: &SOQL_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2509), tables: &SOQL_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2525), tables: &SOQL_TABLES }),
        "SelfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2535), tables: &SOQL_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(2536), tables: &SOQL_TABLES }),
        "SensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2537), tables: &SOQL_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2538), tables: &SOQL_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2539), tables: &SOQL_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(2540), tables: &SOQL_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(2547), tables: &SOQL_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SOQL_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2554), tables: &SOQL_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2555), tables: &SOQL_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2556), tables: &SOQL_TABLES }),
        "Server_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2557), tables: &SOQL_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2558), tables: &SOQL_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2559), tables: &SOQL_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(2560), tables: &SOQL_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2564), tables: &SOQL_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2575), tables: &SOQL_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2584), tables: &SOQL_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2585), tables: &SOQL_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2593), tables: &SOQL_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2598), tables: &SOQL_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2599), tables: &SOQL_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2600), tables: &SOQL_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2601), tables: &SOQL_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2602), tables: &SOQL_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(2603), tables: &SOQL_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2612), tables: &SOQL_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2613), tables: &SOQL_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2614), tables: &SOQL_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(2615), tables: &SOQL_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2618), tables: &SOQL_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2619), tables: &SOQL_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(2620), tables: &SOQL_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(2624), tables: &SOQL_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(2626), tables: &SOQL_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2630), tables: &SOQL_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2631), tables: &SOQL_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2632), tables: &SOQL_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(2635), tables: &SOQL_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(2636), tables: &SOQL_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2637), tables: &SOQL_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2638), tables: &SOQL_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2639), tables: &SOQL_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2640), tables: &SOQL_TABLES }),
        "SpaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2641), tables: &SOQL_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2642), tables: &SOQL_TABLES }),
        "SpecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2643), tables: &SOQL_TABLES }),
        "Specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2644), tables: &SOQL_TABLES }),
        "SpecifictypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2645), tables: &SOQL_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2646), tables: &SOQL_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2647), tables: &SOQL_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2648), tables: &SOQL_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2649), tables: &SOQL_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2650), tables: &SOQL_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2651), tables: &SOQL_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2652), tables: &SOQL_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2653), tables: &SOQL_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2654), tables: &SOQL_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2655), tables: &SOQL_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2656), tables: &SOQL_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2657), tables: &SOQL_TABLES }),
        "SqlcodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2658), tables: &SOQL_TABLES }),
        "SqlerrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2659), tables: &SOQL_TABLES }),
        "SqlexceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2660), tables: &SOQL_TABLES }),
        "SqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2661), tables: &SOQL_TABLES }),
        "SqlwarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2662), tables: &SOQL_TABLES }),
        "SqrtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2663), tables: &SOQL_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2664), tables: &SOQL_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2665), tables: &SOQL_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2666), tables: &SOQL_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2667), tables: &SOQL_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(2668), tables: &SOQL_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(227), tables: &SOQL_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2113), tables: &SOQL_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2669), tables: &SOQL_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(325), tables: &SOQL_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2670), tables: &SOQL_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2671), tables: &SOQL_TABLES }),
        "StateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2672), tables: &SOQL_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2673), tables: &SOQL_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2674), tables: &SOQL_TABLES }),
        "StaticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2675), tables: &SOQL_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2676), tables: &SOQL_TABLES }),
        "Stddev_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2677), tables: &SOQL_TABLES }),
        "Stddev_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2678), tables: &SOQL_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2679), tables: &SOQL_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2680), tables: &SOQL_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2681), tables: &SOQL_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2682), tables: &SOQL_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2683), tables: &SOQL_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2684), tables: &SOQL_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2685), tables: &SOQL_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2686), tables: &SOQL_TABLES }),
        "StringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2688), tables: &SOQL_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2689), tables: &SOQL_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2694), tables: &SOQL_TABLES }),
        "StructureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2695), tables: &SOQL_TABLES }),
        "StyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2696), tables: &SOQL_TABLES }),
        "Subclass_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2697), tables: &SOQL_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2698), tables: &SOQL_TABLES }),
        "SubmultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2699), tables: &SOQL_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2700), tables: &SOQL_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2701), tables: &SOQL_TABLES }),
        "SumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2702), tables: &SOQL_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2703), tables: &SOQL_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(2704), tables: &SOQL_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2705), tables: &SOQL_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2706), tables: &SOQL_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2707), tables: &SOQL_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2708), tables: &SOQL_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2709), tables: &SOQL_TABLES }),
        "System_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2710), tables: &SOQL_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(2711), tables: &SOQL_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2726), tables: &SOQL_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2727), tables: &SOQL_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2736), tables: &SOQL_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SOQL_TABLES }),
        "Table_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2737), tables: &SOQL_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2738), tables: &SOQL_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2739), tables: &SOQL_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2740), tables: &SOQL_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SOQL_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SOQL_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2741), tables: &SOQL_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2746), tables: &SOQL_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2750), tables: &SOQL_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2751), tables: &SOQL_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2752), tables: &SOQL_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2753), tables: &SOQL_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(2754), tables: &SOQL_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(2755), tables: &SOQL_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2758), tables: &SOQL_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(2759), tables: &SOQL_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2762), tables: &SOQL_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2763), tables: &SOQL_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2764), tables: &SOQL_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2765), tables: &SOQL_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2766), tables: &SOQL_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2767), tables: &SOQL_TABLES }),
        "This_fiscal_quarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2768), tables: &SOQL_TABLES }),
        "This_fiscal_yearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2769), tables: &SOQL_TABLES }),
        "This_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2770), tables: &SOQL_TABLES }),
        "This_quarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2771), tables: &SOQL_TABLES }),
        "This_weekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2772), tables: &SOQL_TABLES }),
        "This_yearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2773), tables: &SOQL_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2774), tables: &SOQL_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(2775), tables: &SOQL_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2776), tables: &SOQL_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(2777), tables: &SOQL_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(2788), tables: &SOQL_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2794), tables: &SOQL_TABLES }),
        "Timezone_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2795), tables: &SOQL_TABLES }),
        "Timezone_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2796), tables: &SOQL_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2797), tables: &SOQL_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2798), tables: &SOQL_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2799), tables: &SOQL_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2800), tables: &SOQL_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2801), tables: &SOQL_TABLES }),
        "TodayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2802), tables: &SOQL_TABLES }),
        "TomorrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2803), tables: &SOQL_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2804), tables: &SOQL_TABLES }),
        "Top_level_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2805), tables: &SOQL_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2806), tables: &SOQL_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2807), tables: &SOQL_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2808), tables: &SOQL_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2809), tables: &SOQL_TABLES }),
        "Transaction_activeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2826), tables: &SOQL_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2827), tables: &SOQL_TABLES }),
        "Transactions_committedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2828), tables: &SOQL_TABLES }),
        "Transactions_rolled_backKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2829), tables: &SOQL_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2830), tables: &SOQL_TABLES }),
        "TransformsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2831), tables: &SOQL_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2832), tables: &SOQL_TABLES }),
        "TranslateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2833), tables: &SOQL_TABLES }),
        "TranslationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2834), tables: &SOQL_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2835), tables: &SOQL_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2836), tables: &SOQL_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SOQL_TABLES }),
        "Trigger_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2837), tables: &SOQL_TABLES }),
        "Trigger_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2838), tables: &SOQL_TABLES }),
        "Trigger_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2839), tables: &SOQL_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2840), tables: &SOQL_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(2841), tables: &SOQL_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2845), tables: &SOQL_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(2846), tables: &SOQL_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2847), tables: &SOQL_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2848), tables: &SOQL_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2852), tables: &SOQL_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2853), tables: &SOQL_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(2854), tables: &SOQL_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2857), tables: &SOQL_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2858), tables: &SOQL_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2861), tables: &SOQL_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2864), tables: &SOQL_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2865), tables: &SOQL_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2866), tables: &SOQL_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2867), tables: &SOQL_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2868), tables: &SOQL_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2869), tables: &SOQL_TABLES }),
        "UnderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2873), tables: &SOQL_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2874), tables: &SOQL_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2875), tables: &SOQL_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2876), tables: &SOQL_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2881), tables: &SOQL_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2882), tables: &SOQL_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2884), tables: &SOQL_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2885), tables: &SOQL_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2886), tables: &SOQL_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2887), tables: &SOQL_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2888), tables: &SOQL_TABLES }),
        "UnnamedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2889), tables: &SOQL_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2890), tables: &SOQL_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2891), tables: &SOQL_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2904), tables: &SOQL_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2910), tables: &SOQL_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2911), tables: &SOQL_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2912), tables: &SOQL_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2913), tables: &SOQL_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2921), tables: &SOQL_TABLES }),
        "UpperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2922), tables: &SOQL_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2923), tables: &SOQL_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2924), tables: &SOQL_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2925), tables: &SOQL_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2928), tables: &SOQL_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2929), tables: &SOQL_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2930), tables: &SOQL_TABLES }),
        "User_defined_type_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2931), tables: &SOQL_TABLES }),
        "User_defined_type_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2932), tables: &SOQL_TABLES }),
        "User_defined_type_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2933), tables: &SOQL_TABLES }),
        "User_defined_type_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2934), tables: &SOQL_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2935), tables: &SOQL_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2936), tables: &SOQL_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2937), tables: &SOQL_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2938), tables: &SOQL_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2939), tables: &SOQL_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2940), tables: &SOQL_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2941), tables: &SOQL_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2942), tables: &SOQL_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2943), tables: &SOQL_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2944), tables: &SOQL_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2945), tables: &SOQL_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2958), tables: &SOQL_TABLES }),
        "Var_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2959), tables: &SOQL_TABLES }),
        "Var_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2960), tables: &SOQL_TABLES }),
        "VarbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2961), tables: &SOQL_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2962), tables: &SOQL_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2963), tables: &SOQL_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2964), tables: &SOQL_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2965), tables: &SOQL_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2966), tables: &SOQL_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2967), tables: &SOQL_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2968), tables: &SOQL_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2969), tables: &SOQL_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2970), tables: &SOQL_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2971), tables: &SOQL_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2972), tables: &SOQL_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2973), tables: &SOQL_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2974), tables: &SOQL_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2975), tables: &SOQL_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2976), tables: &SOQL_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2977), tables: &SOQL_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2978), tables: &SOQL_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2988), tables: &SOQL_TABLES }),
        "WheneverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2989), tables: &SOQL_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2990), tables: &SOQL_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2995), tables: &SOQL_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3008), tables: &SOQL_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3009), tables: &SOQL_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(3010), tables: &SOQL_TABLES }),
        "Width_bucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3011), tables: &SOQL_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3012), tables: &SOQL_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3014), tables: &SOQL_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3024), tables: &SOQL_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(3025), tables: &SOQL_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3033), tables: &SOQL_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3042), tables: &SOQL_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3051), tables: &SOQL_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(3056), tables: &SOQL_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3057), tables: &SOQL_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3058), tables: &SOQL_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3063), tables: &SOQL_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3064), tables: &SOQL_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(3065), tables: &SOQL_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3066), tables: &SOQL_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3067), tables: &SOQL_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3068), tables: &SOQL_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3069), tables: &SOQL_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3070), tables: &SOQL_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3071), tables: &SOQL_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3072), tables: &SOQL_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3073), tables: &SOQL_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3074), tables: &SOQL_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3075), tables: &SOQL_TABLES }),
        "YesterdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3076), tables: &SOQL_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3077), tables: &SOQL_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3078), tables: &SOQL_TABLES }),
        _ => None,
    }
}

pub fn get_soql_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CaseExpressionSegment" => Some("case_expression"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "ConcatSegment" => Some("binary_operator"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DateLiteralNSegment" => Some("date_n_literal"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "Dedent" => Some("dedent"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileSegment" => Some("file"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "IdentifierSegment" => Some("identifier"),
        "ImplicitIndent" => Some("indent"),
        "Indent" => Some("indent"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LimitClauseSegment" => Some("limit_clause"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "ValuesClauseSegment" => Some("values_clause"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static SOQL_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_soql_root_grammar_id() -> GrammarId {
    GrammarId(1259)
}
pub fn get_soql_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_soql_root_grammar_id(),
        tables: &SOQL_TABLES,
    }
}
pub fn get_soql_root_grammar() -> RootGrammar {
    get_soql_root_grammar_table()
}
