/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      4352 × 20 bytes =    87040 bytes
//   Child IDs:         3066 ×  4 bytes =    12264 bytes
//   Terminators:         58 ×  4 bytes =      232 bytes
//   Strings:           2185 strings      =    29365 bytes
//   Aux Data:         13425 ×  4 bytes =    53700 bytes
//   Aux Offsets:       4352 ×  4 bytes =    17408 bytes
//   Regex Patterns:       5 patterns     =      771 bytes
//   Simple Hints:      1392 hints        =    22272 bytes
//   Hint Indices:      2239 ×  4 bytes =     8956 bytes
//   TOTAL:                                    ≈   232008 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [4] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [6] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [51] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [61] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [62] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [63] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [64] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [131] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [133] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [134] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [135] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [136] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [137] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [138] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [139] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [140] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] StringParser("ACCESSRANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] StringParser("ACTIVATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] StringParser("ACTIVEPARTITIONCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] StringParser("ADA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] StringParser("ADDRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 204, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 208, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 208, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 211, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 213, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 216, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] StringParser(")")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 216, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 217, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 220, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] Meta("dedent")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 227, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] StringParser("ALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 7, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [251] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 238, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 238, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Ref(AlterDefaultPrivilegesObjectPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 238, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 238, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 1, _padding: 0 }, // [255] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 240, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 242, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 244, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [268] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 2, min_times: 0, first_terminator_idx: 4, terminator_count: 1, _padding: 0 }, // [269] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [270] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [271] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 253, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [272] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 256, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [273] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 256, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [274] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 256, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [275] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 256, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [276] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 258, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [277] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [278] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [279] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [280] Ref(ExtendKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 1, _padding: 0 }, // [281] Delimited(8 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 263, child_count: 8, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [282] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [283] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [284] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [285] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [286] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [287] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [288] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [289] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [290] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [291] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [292] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [293] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [294] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [295] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [296] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [297] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [298] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [299] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [300] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [301] Ref(SchemaPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [302] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [303] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [304] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [305] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [306] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [307] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [308] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [309] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [310] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [311] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [312] Ref(DiskQuotaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 6, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [313] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [314] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [315] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [316] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [317] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [318] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [319] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [320] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [321] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [322] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [323] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [324] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [325] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [326] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [327] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [328] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [329] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 311, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [330] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 311, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [331] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 311, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [332] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 311, child_count: 1, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [333] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [334] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [335] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 315, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [336] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 315, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [337] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 315, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [338] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 319, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [339] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [340] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [341] Ref(ParameterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [342] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 324, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [343] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [344] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 330, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [345] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 330, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [346] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 330, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [347] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 330, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [348] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [349] Ref(ClearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [350] Ref(ParameterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [351] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [352] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 338, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [353] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [354] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [355] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [356] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [357] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 343, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [358] Ref(ParameterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 343, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [359] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 343, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [360] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [361] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 347, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [362] Ref(UdparameterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 347, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [363] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [364] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [365] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [366] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [367] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 355, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [368] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 355, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [369] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 355, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [370] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 355, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [371] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 359, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [372] Ref(ClearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 359, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [373] Ref(UdparameterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 359, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [374] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 361, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [375] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 361, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [376] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 361, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [377] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [378] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [379] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 366, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [380] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 366, child_count: 7, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [381] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 373, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [382] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [383] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [384] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [385] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 379, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [386] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 379, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [387] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 379, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [388] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 379, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [389] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [390] Ref(EnabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [391] Ref(DisabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [392] Ref(DiskQuotaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [393] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [394] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [395] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [396] Ref(IntegerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [397] Ref(SchemaPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [398] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [399] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [400] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [401] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [402] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [403] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [404] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [405] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 394, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [406] Ref(ActivepartitioncountKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 394, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [407] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 396, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [408] Ref(IntegerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 396, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [409] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 396, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [410] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 398, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [411] Ref(ImmutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 398, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [412] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 398, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [413] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 400, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [414] Ref(MergeoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 400, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [415] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 402, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [416] Ref(1KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 402, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [417] Ref(2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 402, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [418] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [419] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [420] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [421] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [422] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [423] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 410, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [424] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [425] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [426] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [427] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [428] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [429] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [430] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 419, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [431] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [432] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [433] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 421, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [434] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [435] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 421, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [436] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [437] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 425, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [438] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [439] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [440] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [441] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [442] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [443] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [444] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [445] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 430, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [446] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [447] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [448] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [449] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [450] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [451] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [452] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [453] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 438, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [454] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 438, child_count: 11, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [455] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 1, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [456] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [457] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [458] Ref(AlterTableActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 9, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [459] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [460] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [461] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [462] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [463] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [464] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 461, child_count: 1, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [465] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [466] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [467] Ref(ColumnEncodingSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 462, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [468] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [469] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [470] Ref(ProjectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [471] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 469, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [472] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 471, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [473] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 471, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [474] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 473, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [475] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 473, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [476] Ref(ProjectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 473, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [477] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 473, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [478] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [479] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [480] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [481] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 5, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [482] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 482, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [483] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [484] Ref(ColumnEncodingSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [485] Ref(ProjectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [486] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 488, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [487] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [488] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [489] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [490] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [491] Ref(ColumnSetSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [492] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 495, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [493] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 495, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [494] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 495, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [495] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 495, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [496] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [497] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [498] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [499] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [500] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [501] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [502] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [503] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [504] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [505] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 507, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [506] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 507, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [507] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 507, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [508] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [509] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [510] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 509, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [511] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [512] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [513] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [514] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [515] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [516] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [517] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 515, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [518] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 517, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [519] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 517, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [520] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 517, child_count: 5, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [521] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [522] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [523] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [524] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [525] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [526] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [527] Ref(PartitionByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [528] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [529] Ref(RemoveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [530] Ref(PartitioningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 5, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [531] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [532] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [533] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [534] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [535] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [536] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [537] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [538] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [539] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [540] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [541] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [542] Ref(ReorganizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [543] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [544] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [545] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [546] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [547] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 541, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [548] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 541, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [549] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 541, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [550] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 543, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [551] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 543, child_count: 4, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [552] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 547, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [553] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [554] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [555] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [556] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [557] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [558] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [559] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [560] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [561] Ref(SchemaPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [562] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 556, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [563] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 556, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [564] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 556, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [565] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [566] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [567] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [568] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [569] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [570] StringParser("ANALYTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [571] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [572] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [573] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [574] StringParser("ANNOTATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [575] StringParser("ANTI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [576] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [577] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [578] StringParser("ARCHIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [579] StringParser("ARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 558, child_count: 11, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [580] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [581] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [582] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [583] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [584] Ref(IntegerDivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [585] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [586] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [587] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [588] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [589] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [590] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [591] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [592] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 572, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [593] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 574, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [594] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [595] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [596] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [597] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [598] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [599] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [600] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [601] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [602] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 579, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [603] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 581, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [604] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 581, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [605] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [606] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 583, child_count: 3, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [607] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [608] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 586, child_count: 1, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [609] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [610] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [611] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [612] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [613] StringParser("ASENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [614] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [615] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [616] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [617] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [618] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [619] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [620] StringParser("ATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [621] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [622] StringParser("AUTHENTICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [623] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 1, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [624] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [625] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [626] StringParser("AUTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [627] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [628] StringParser("AUTOCOMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [629] StringParser("AVAILABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [630] StringParser("AVG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [631] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [632] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [633] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [634] StringParser("BALANCE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [635] MultiStringParser(6 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 7, min_times: 0, first_terminator_idx: 6, terminator_count: 2, _padding: 0 }, // [636] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [637] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [638] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [639] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [640] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [641] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [642] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 2, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [643] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [644] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [645] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [646] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 6, terminator_count: 0, _padding: 0 }, // [647] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [648] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [649] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [650] StringParser("BASENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [651] StringParser("BATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [652] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [653] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [654] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [655] StringParser("BEST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [656] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [657] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [658] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 4, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [659] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [660] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [661] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [662] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [663] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [664] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [665] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [666] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [667] StringParser("BIT_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [668] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [669] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 601, child_count: 2, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [670] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 603, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [671] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 603, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [672] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 603, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [673] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 603, child_count: 2, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [674] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 605, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [675] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 605, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [676] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 605, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [677] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 605, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [678] StringParser("BLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 605, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [679] StringParser("BLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 605, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [680] StringParser("BLOCK_DICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 605, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [681] StringParser("BLOCKDICT_COMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 605, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [682] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 605, child_count: 2, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [683] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [684] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [685] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [686] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 2, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [687] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 609, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [688] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 609, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [689] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 609, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [690] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 609, child_count: 3, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [691] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 612, child_count: 2, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [692] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 614, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [693] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 614, child_count: 3, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [694] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 2, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [695] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [696] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [697] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 3, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [698] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [699] Ref(UnorderedSetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [700] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [701] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [702] StringParser("BROADCAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [703] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [704] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [705] StringParser("BUNDLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [706] StringParser("BY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [707] StringParser("BYTEA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [708] StringParser("BYTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [709] StringParser("BZIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 622, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [710] StringParser("BZIP_COMP")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 622, child_count: 3, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [711] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 625, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [712] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 625, child_count: 4, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [713] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 629, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [714] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 629, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [715] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 629, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [716] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 629, child_count: 3, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [717] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [718] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [719] StringParser("CA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [720] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [721] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [722] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [723] StringParser("CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [724] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [725] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 3, _padding: 0 }, // [726] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 634, child_count: 6, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [727] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [728] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [729] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 640, child_count: 1, min_times: 0, first_terminator_idx: 8, terminator_count: 2, _padding: 0 }, // [730] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [731] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [732] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [733] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 1, _padding: 0 }, // [734] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [735] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [736] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 7, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [737] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [738] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [739] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 648, child_count: 1, min_times: 0, first_terminator_idx: 11, terminator_count: 2, _padding: 0 }, // [740] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [741] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [742] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [743] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 13, terminator_count: 1, _padding: 0 }, // [744] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 13, terminator_count: 0, _padding: 0 }, // [745] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [746] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [747] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [748] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [749] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [750] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [751] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [752] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [753] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [754] StringParser("CATALOG_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [755] StringParser("CATALOGPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [756] StringParser("CEIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [757] StringParser("CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [758] StringParser("CERTIFICATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [759] StringParser("CERTIFICATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [760] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [761] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [762] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [763] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [764] StringParser("CHAR_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [765] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [766] StringParser("CHARACTER_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [767] StringParser("CHARACTER_SET_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [768] StringParser("CHARACTER_SET_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [769] StringParser("CHARACTER_SET_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [770] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [771] StringParser("CHARACTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [772] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [773] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [774] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [775] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [776] StringParser("CIPHER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [777] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [778] StringParser("CLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [779] StringParser("CLEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [780] StringParser("CLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [781] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [782] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [783] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [784] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [785] StringParser("COBOL")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [786] Token("raw")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [787] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [788] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [789] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [790] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 651, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [791] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 651, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 1, _padding: 0 }, // [792] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [793] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [794] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [795] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [796] StringParser("COLLATION_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [797] StringParser("COLLATION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [798] StringParser("COLLATION_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [799] StringParser("COLLECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [800] StringParser("COLLECTIONCLOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [801] StringParser("COLLECTIONDELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [802] StringParser("COLLECTIONENCLOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [803] StringParser("COLLECTIONNULLELEMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [804] StringParser("COLLECTIONOPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [805] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [806] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [807] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [808] StringParser("COLSIZES")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [809] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [810] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [811] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [812] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [813] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [814] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 658, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [815] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 660, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [816] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 662, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [817] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 664, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [818] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 664, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [819] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 664, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [820] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 669, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [821] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [822] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [823] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [824] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [825] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [826] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 677, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [827] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 677, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [828] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 679, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [829] Ref(EnabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 679, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [830] Ref(DisabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 679, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [831] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [832] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 681, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [833] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [834] Ref(EnabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [835] Ref(DisabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [836] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [837] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [838] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 686, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [839] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [840] Ref(EnabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [841] Ref(DisabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [842] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [843] Ref(ColumnSetSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [844] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [845] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [846] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 692, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [847] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [848] Anything
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 695, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [849] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 696, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [850] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 696, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [851] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [852] Ref(EncodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [853] Ref(EncodingType)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [854] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [855] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 698, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 1, _padding: 0 }, // [856] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [857] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [858] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [859] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [860] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [861] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [862] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [863] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 707, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [864] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 707, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [865] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 707, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [866] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [867] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [868] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 6, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [869] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [870] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [871] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [872] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [873] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [874] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [875] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [876] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [877] StringParser("COLUMN_NAME")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [878] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [879] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [880] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [881] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [882] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [883] StringParser("COLUMNS_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [884] StringParser(",")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [885] StringParser("COMMAND_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 718, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [886] StringParser("COMMAND_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 718, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [887] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 720, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [888] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 720, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [889] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 720, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [890] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 720, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [891] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [892] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [893] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [894] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 7, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [895] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [896] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [897] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [898] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [899] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [900] Ref(ProjectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [901] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [902] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [903] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [904] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [905] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [906] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [907] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [908] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [909] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [910] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [911] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 748, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [912] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [913] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [914] Ref(AnalyticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [915] Ref(TransformKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [916] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [917] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 751, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [918] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [919] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [920] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [921] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [922] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [923] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [924] Ref(NodeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [925] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [926] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [927] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [928] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [929] Ref(LibraryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [930] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [931] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [932] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [933] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [934] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [935] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [936] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [937] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [938] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [939] StringParser("COMMONDELTA_COMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 764, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [940] StringParser("COMMUNAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [941] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [942] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [943] Ref(NullEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [944] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [945] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [946] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [947] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [948] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [949] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [950] Ref(IsDistinctFromGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [951] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [952] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [953] StringParser("COMPLEX")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [954] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [955] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [956] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [957] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 774, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [958] MultiStringParser(5 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 774, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [959] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 774, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [960] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [961] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [962] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [963] StringParser("CONDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [964] StringParser("CONDITION_NUMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [965] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [966] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [967] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [968] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [969] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [970] StringParser("CONFIGURATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [971] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [972] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [973] StringParser("CONNECTION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [974] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [975] StringParser("CONSTRAINT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [976] StringParser("CONSTRAINT_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [977] StringParser("CONSTRAINT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [978] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [979] StringParser("CONSTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [980] StringParser("CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [981] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [982] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [983] StringParser("CONTROL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [984] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [985] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [986] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [987] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [988] Ref(CopyOptionsForColumnsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [989] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [990] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 782, child_count: 8, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [991] AnySetOf(8 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 790, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [992] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [993] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 793, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [994] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [995] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [996] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [997] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [998] Ref(EnclosedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 797, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [999] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1000] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1001] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1002] Ref(EnforcelengthKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1003] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 800, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1004] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1005] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 803, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1006] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 804, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1007] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 804, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1008] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 804, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1009] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1010] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1011] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1012] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1013] Ref(FillerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1014] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1015] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1016] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1017] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1018] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 813, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1019] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 813, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1020] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1021] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1022] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1023] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1024] Ref(TrimKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1025] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1026] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 817, child_count: 10, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1027] AnyNumberOf(10 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 827, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1028] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 830, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1029] Ref(AbortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 830, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1030] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 830, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1031] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 830, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1032] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1033] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1034] Ref(ToleranceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1035] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1036] Ref(ExceptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1037] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1038] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 837, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1039] Ref(RecordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 837, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1040] Ref(TerminatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 837, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1041] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 837, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1042] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1043] Ref(RejectedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1044] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1045] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1046] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1047] Ref(RejectmaxKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1048] Ref(IntegerSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1049] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1050] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1051] Ref(IntegerSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1052] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 847, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1053] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 847, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1054] Ref(BytesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 847, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1055] Ref(IntegerSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 847, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1056] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 849, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1057] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 849, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1058] Ref(NullcolsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 849, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1059] Ref(CopyOptionsForColumnsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 849, child_count: 7, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1060] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1061] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1062] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 856, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1063] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1064] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1065] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1066] Ref(CopyColumnOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1067] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1068] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1069] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1070] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 869, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1071] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1072] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 873, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1073] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 873, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1074] Ref(CopyColumnOptionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 873, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1075] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 873, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1076] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1077] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1078] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1079] Ref(StdinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1080] Ref(CompressionType)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1081] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1082] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1083] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1084] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1085] Ref(CompressionType)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1086] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1087] Ref(VerticaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1088] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 887, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1089] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1090] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1091] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1092] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1093] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1094] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 895, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1095] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1096] Ref(NativeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 899, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1097] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1098] Ref(NativeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1099] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1100] Ref(OrcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1101] Ref(ParquetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1102] Ref(CopyOptionsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1103] StringParser("CORR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1104] StringParser("CORRELATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1105] StringParser("CORRESPONDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1106] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1107] StringParser("COVAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1108] StringParser("COVAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1109] StringParser("CPUAFFINITYMODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1110] StringParser("CPUAFFINITYSET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 10, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1111] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1112] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1113] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1114] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1115] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1116] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1117] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1118] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1119] Ref(SpecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1120] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1121] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1122] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1123] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1124] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 922, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1125] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1126] Ref(InstanceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1127] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1128] Ref(ConstructorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1129] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1130] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 925, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1131] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 925, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1132] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 927, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1133] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 927, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1134] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 927, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1135] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1136] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1137] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1138] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1139] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1140] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1141] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1142] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 14, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1143] Sequence(14 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1144] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1145] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1146] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1147] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1148] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1149] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1150] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1151] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 953, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1152] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 955, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1153] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1154] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1155] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 958, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1156] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 961, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1157] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 961, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1158] Ref(ColumnEncodingSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 961, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1159] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 963, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1160] Ref(AccessrankKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 963, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1161] Ref(IntegerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 963, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1162] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 963, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1163] Ref(SchemaPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 963, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1164] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 963, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1165] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 963, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1166] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 965, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1167] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1168] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 970, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1169] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 972, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1170] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 972, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1171] Ref(CopyColumnOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 972, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1172] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1173] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1174] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1175] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 978, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1176] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1178] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1179] Ref(CopyColumnOptionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1180] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1181] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 982, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1182] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 986, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1183] Ref(NativeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 986, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1184] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 988, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1185] Ref(NativeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 988, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1186] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 988, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1187] Ref(OrcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 988, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1188] Ref(ParquetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 988, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1189] Ref(CopyOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 988, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1190] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1191] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1192] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1193] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1194] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1195] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1196] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1197] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 997, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1198] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 999, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1199] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 999, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1200] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 999, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1201] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 999, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1202] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1008, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1203] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1008, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1204] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1008, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1205] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1008, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1206] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1008, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1207] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1008, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1208] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1008, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1209] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1008, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1210] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1008, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1211] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1212] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1012, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1213] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1214] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1014, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1215] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 8, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1216] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1217] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1022, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1218] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1219] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1022, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1220] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1221] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1022, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1222] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1223] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1024, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1224] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1027, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1225] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1226] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1032, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1227] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1032, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1228] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1032, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1229] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1034, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1230] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1034, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1231] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1037, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1232] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1233] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1234] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1235] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 9, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1236] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1048, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1237] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1048, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1238] Ref(ProjectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1048, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1239] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1048, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1240] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1048, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1241] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1242] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1243] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1244] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1056, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1245] Ref(ColumnEncodingSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1056, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1246] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1247] Ref(AccessrankKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1248] Ref(IntegerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1249] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 5, _padding: 0 }, // [1250] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1251] Ref(SegmentedByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1252] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1253] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1254] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1255] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1256] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1061, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1257] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1064, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1258] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1064, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1259] Ref(SegmentedByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1064, child_count: 7, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1260] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1261] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1262] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1263] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1264] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1265] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1266] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1267] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1268] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1269] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1270] Ref(KsafeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1271] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1272] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1273] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1274] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1275] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 5, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1276] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1080, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1277] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1080, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1278] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1080, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1279] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1080, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1280] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1080, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1281] AnySetOf(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1282] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1283] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1284] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1285] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1286] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1287] Ref(SchemaPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1288] Ref(DiskQuotaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 7, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1289] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1094, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1290] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1291] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1292] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1293] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1294] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1295] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1296] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1297] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1298] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1299] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1300] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1301] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1302] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1303] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1304] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1305] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1306] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1307] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1308] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1309] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1310] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1311] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1312] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1110, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1313] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1314] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1111, child_count: 12, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1315] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1123, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1316] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1123, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1317] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1318] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1126, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1319] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1320] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1321] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1322] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1323] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1324] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1325] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1128, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1326] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1130, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1327] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1328] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1329] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1330] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1331] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1332] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1333] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1138, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1335] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1138, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1336] Ref(ProjectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1138, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1337] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1140, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1338] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1143, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1339] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1145, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1340] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1341] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1148, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1342] Ref(ColumnEncodingSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1148, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1343] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1150, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1344] Ref(AccessrankKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1150, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1345] Ref(IntegerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1150, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1346] Ref(SchemaPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1150, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1347] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1150, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1348] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1152, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1349] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1152, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1350] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1351] Ref(LatestKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1352] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1353] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 2, _padding: 0 }, // [1354] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1355] Ref(SegmentedByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1356] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1357] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1358] Ref(SegmentedByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 6, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1359] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1360] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1361] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1362] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1363] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1364] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1164, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1365] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1164, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1366] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1164, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1367] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1368] Ref(LikeOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1369] Ref(DiskQuotaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 8, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1370] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1173, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1371] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1173, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1372] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1373] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1176, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1374] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1375] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1376] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1377] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1378] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1379] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1380] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1381] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1382] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1383] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1184, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1384] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1385] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1189, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1386] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1189, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1387] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1189, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1388] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1192, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1389] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1192, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1390] Ref(ColumnEncodingSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1192, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1391] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1392] Ref(AccessrankKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1393] Ref(IntegerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1194, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1394] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1194, child_count: 6, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1395] AnySetOf(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1396] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1397] Ref(SegmentedByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1398] Ref(KsafeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1399] Ref(SchemaPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1400] Ref(DiskQuotaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1401] Ref(PartitionByClauseSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1200, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1402] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1202, child_count: 4, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1403] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1404] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1405] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1406] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1407] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1408] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1409] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1410] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1411] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1412] Ref(ProjectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1413] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1219, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1414] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1219, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1415] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1219, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1416] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1219, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1417] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1222, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1418] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1222, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1419] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1222, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1420] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1421] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1422] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 1, _padding: 0 }, // [1423] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1226, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1424] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1425] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1426] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1427] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1428] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1429] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 2, _padding: 0 }, // [1430] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1431] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1432] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1433] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1434] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1435] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1436] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1437] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1234, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1438] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1439] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1440] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1441] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1442] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1443] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1444] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1445] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1446] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1447] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1448] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1449] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1450] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1451] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1250, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1452] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1453] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1254, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1454] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1254, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1455] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1254, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1456] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1256, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1457] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1256, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1458] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1258, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1459] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1460] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1461] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1462] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1463] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1464] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1465] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1466] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1265, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1467] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1468] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1469] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1470] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1471] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1472] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1473] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1272, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1474] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1272, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1475] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1276, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1476] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1276, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1477] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1276, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1478] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1276, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1479] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1276, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1480] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1481] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1482] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1483] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1484] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1288, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1485] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1288, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1486] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1288, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1487] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1288, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1488] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1288, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1489] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1288, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1490] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1288, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1491] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1288, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1492] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1493] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1494] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1495] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1496] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1497] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1498] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1499] StringParser("CRON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1500] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1501] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1502] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1503] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1504] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1505] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1506] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1507] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1508] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1509] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1510] StringParser("CUME_DIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1511] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1512] StringParser("CURRENT_DATABASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1513] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1514] StringParser("CURRENT_DEFAULT_TRANSFORM_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1515] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1516] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1517] StringParser("CURRENT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1518] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1519] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1520] StringParser("CURRENT_TRANSFORM_GROUP_FOR_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1521] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1522] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1523] StringParser("CURSOR_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1524] StringParser("CUSTOM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1525] StringParser("CUSTOM_PARTITIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1526] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1527] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1528] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1529] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1530] StringParser("DATAPATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1531] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1532] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1302, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1533] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1303, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1534] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1303, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1535] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 24, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1536] OneOf(24 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1537] Ref(TimeWithTZGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1538] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1539] Ref(DatetimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1540] Ref(SmalldatetimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1541] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1542] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1543] Ref(IntervalLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1544] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1545] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1546] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1547] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1548] Ref(FloatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1549] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1550] Ref(Float8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1551] Ref(RealKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1552] Ref(IntegerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1553] Ref(IntKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1554] Ref(BigintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1555] Ref(Int8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1556] Ref(SmallintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1557] Ref(TinyintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1558] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1336, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1559] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1340, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1560] Ref(DecimalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1340, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1561] Ref(NumericKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1340, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1562] Ref(NumberKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1340, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1563] Ref(MoneyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1340, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1564] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1340, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1565] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1566] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1344, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1567] Ref(GeometryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1344, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1568] Ref(GeographyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1344, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1569] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1570] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1571] Ref(UuidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1572] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1350, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1573] Ref(LongKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1574] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1350, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1575] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1576] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1577] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1578] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1579] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1580] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1581] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1582] Ref(LongKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1583] Ref(VarbinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1584] Ref(ByteaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1585] Ref(RawKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1586] Ref(BooleanKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1587] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1361, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1588] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1589] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1590] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1591] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1592] Ref(SizedArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1593] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1367, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1594] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1369, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1595] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1596] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1597] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1598] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1599] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1600] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1601] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1602] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1603] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1377, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1604] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 1379, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1605] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1606] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1607] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1608] Ref(DatatypeIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1609] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1610] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1611] MultiStringParser(4 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1612] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1613] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1382, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1614] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1385, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1615] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1387, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1616] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1389, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1617] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1389, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1618] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1389, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1619] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1392, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1620] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1621] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1622] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1623] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1624] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1625] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1626] Ref(IntervalLiteralGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1627] StringParser("DATEDIFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1628] StringParser("DATESTYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1629] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1630] MultiStringParser(25 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1631] StringParser("DATETIME_INTERVAL_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1632] StringParser("DATETIME_INTERVAL_PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1633] StringParser("DATETIMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1634] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1635] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1636] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1637] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1638] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1639] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1640] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1641] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1642] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1643] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1644] StringParser("DEACTIVATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1645] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1646] StringParser("DEBUG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1647] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1648] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1649] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1650] StringParser("DECODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1396, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1651] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1652] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1653] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1654] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1655] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1656] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1657] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1658] StringParser("DEFINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1659] StringParser("DEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1660] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1661] StringParser("DEGREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1662] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1663] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1664] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1665] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1666] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1667] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1668] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1669] StringParser("DELIMITED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1670] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1671] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1672] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1673] StringParser("DELTARANGE_COMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1674] StringParser("DELTARANGE_COMP_SP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1675] StringParser("DELTAVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1676] StringParser("DENSE_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1677] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1678] StringParser("DEPENDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1679] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1680] StringParser("DEREF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1681] StringParser("DERIVED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1682] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1683] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1684] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1685] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1686] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1687] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1688] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1689] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1690] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1691] StringParser("DETERMINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1692] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1693] StringParser("DIAGNOSTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1694] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1695] StringParser("DIRECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1696] StringParser("DIRECTCOLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1697] StringParser("DIRECTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1698] StringParser("DIRECTGROUPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1699] StringParser("DIRECTPROJ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1700] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1701] StringParser("DISABLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1702] StringParser("DISCONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1703] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1404, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1704] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1705] Ref(Disk_quotaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1706] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1707] StringParser("DISK_QUOTA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1708] StringParser("DISPATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1709] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1710] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1711] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1712] StringParser("DISTVALINDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1713] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1714] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1715] StringParser("DO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1716] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1717] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1718] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1406, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1719] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1720] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1721] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1722] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1723] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1724] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1725] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1726] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1727] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1728] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1729] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1730] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1731] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1732] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1733] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1734] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1735] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1736] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1737] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1738] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1739] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1740] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1741] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1742] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1743] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1744] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1745] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1746] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1747] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1748] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1749] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1750] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1751] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1752] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1753] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1754] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1755] Ref(ProjectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1756] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1757] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1758] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1759] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1760] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1446, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1761] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1446, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1762] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1446, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1763] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1446, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1764] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1446, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1765] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1766] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1767] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1768] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1769] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1451, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1770] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1451, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1771] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1772] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1773] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1774] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1775] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1776] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1777] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1778] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1779] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1780] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1781] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1782] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1783] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1784] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1785] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1786] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1787] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1788] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1789] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1790] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1471, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1791] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1792] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1471, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1793] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1794] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1795] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1796] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1797] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1798] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1799] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1800] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1801] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1802] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1803] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1804] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1805] StringParser("DRYRUN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1806] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1807] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1808] StringParser("DURABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1809] StringParser("DYNAMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1810] StringParser("DYNAMIC_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1811] StringParser("DYNAMIC_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1812] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1813] StringParser("ELEMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1814] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1484, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1815] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1484, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1816] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1484, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1817] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1484, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1818] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1484, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1819] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1486, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1820] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1488, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1821] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1488, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1822] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1488, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1823] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1488, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1824] StringParser("ENABLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1488, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1825] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1488, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1826] StringParser("ENCODED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1488, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1827] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1488, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1828] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1829] MultiStringParser(13 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1830] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1831] StringParser("END-EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1832] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1833] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1834] StringParser("ENFORCELENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1835] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1836] StringParser("EPHEMERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1837] StringParser("EPOCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1838] StringParser("EQUALS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1839] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1840] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1841] StringParser("ERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1842] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1843] StringParser("ESCAPE_STRING_WARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1844] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1845] StringParser("EVENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1846] StringParser("EVENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1847] StringParser("EVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1848] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1849] StringParser("EXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1850] StringParser("EXCEPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1851] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1852] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1853] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1854] StringParser("EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1855] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1856] StringParser("EXECUTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1857] StringParser("EXECUTIONPARALLELISM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1858] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1859] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1860] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1861] StringParser("EXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1862] StringParser("EXPIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1863] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1489, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1864] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1865] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1866] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1867] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1868] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1869] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1870] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1871] StringParser("EXPORT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1872] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1873] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1497, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1874] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1497, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1875] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1876] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1507, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1877] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1507, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1878] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1509, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1879] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1509, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1880] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1509, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1881] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1509, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1882] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1883] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1884] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1885] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1886] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1887] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1514, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1888] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1889] Ref(InterpolateGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1890] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1891] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1892] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1893] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1894] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1895] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1896] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1897] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1898] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1899] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1524, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1900] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1525, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1901] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1902] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1903] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1904] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1905] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1906] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1528, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1907] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1528, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1908] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1909] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1530, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1910] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1532, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1911] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1535, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1912] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1535, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1913] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1535, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1914] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1535, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1915] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1535, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1916] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1537, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1917] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1918] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1919] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1920] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 1, _padding: 0 }, // [1921] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1542, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1922] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1544, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1923] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1544, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1924] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1925] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1926] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1927] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1928] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1929] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1551, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1930] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1931] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1932] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1933] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1552, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1934] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 9, min_times: 0, first_terminator_idx: 30, terminator_count: 1, _padding: 0 }, // [1935] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1936] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1937] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1938] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1939] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1570, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1940] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1570, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1941] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1570, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1942] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1570, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1943] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1572, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1944] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1577, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1945] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1577, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1946] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1577, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1947] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1577, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1948] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1577, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1949] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1577, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1950] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1577, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1951] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1577, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1952] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1953] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1581, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1954] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1581, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1955] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1581, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1956] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1957] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1958] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1959] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1960] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1961] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1962] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1963] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1964] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1965] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1966] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1967] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1968] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1969] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1970] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 7, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1971] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1972] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1973] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1974] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1975] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1976] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1977] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1978] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1979] StringParser("EXTEND")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1980] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1981] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1982] StringParser("EXTENSIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1983] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1984] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1985] StringParser("FAILED_LOGIN_ATTEMPTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1986] StringParser("FALLTHROUGH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1987] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1988] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1989] StringParser("FAULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1990] StringParser("FENCED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1991] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1992] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1993] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1994] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1995] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1607, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1996] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1997] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1609, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1998] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1610, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1999] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1610, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2000] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2001] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2002] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2003] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2004] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2005] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2006] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2007] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2008] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2009] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2010] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2011] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1619, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2012] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2013] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 1620, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2014] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2015] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 1, min_times: 1, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2016] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2017] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1623, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2018] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2019] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2020] StringParser("FILESYSTEM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2021] StringParser("FILLER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2022] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2023] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2024] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2025] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2026] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2027] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2028] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2029] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2030] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2031] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2032] StringParser("FIXEDWIDTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2033] StringParser("FLEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2034] StringParser("FLEXIBLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2035] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2036] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2037] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2038] StringParser("FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2039] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2040] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2041] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2042] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2043] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2044] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2045] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2046] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2047] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2048] StringParser("FORTRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2049] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2050] StringParser("FOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1633, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2051] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2052] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2053] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2054] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1639, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2055] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2056] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2057] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2058] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1643, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2059] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1646, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2060] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1646, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2061] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1648, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2062] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2063] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2064] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2065] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1651, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2066] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2067] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2068] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2069] Ref(IntervalLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2070] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2071] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2072] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2073] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2074] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2075] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2076] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2077] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2078] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2079] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2080] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2081] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2082] StringParser("FREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2083] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2084] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2085] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1664, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2086] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1666, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2087] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1666, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2088] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1668, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2089] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1668, child_count: 12, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2090] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1680, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2091] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1680, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2092] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1680, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2093] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2094] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2095] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2096] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2097] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2098] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2099] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2100] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2101] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2102] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2103] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2104] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2105] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2106] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1684, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2107] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 7, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2108] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2109] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1693, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2110] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2111] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2112] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2113] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1698, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2114] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1699, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2115] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2116] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2117] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2118] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2119] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1703, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2120] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1706, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2121] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1706, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2122] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1706, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2123] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1706, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2124] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1706, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2125] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1706, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2126] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1709, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2127] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1711, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2128] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2129] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2130] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2131] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 6, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2132] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1723, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 2, _padding: 0 }, // [2133] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2134] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2135] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2136] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2137] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2138] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1731, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2139] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1731, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2140] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1731, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2141] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1733, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2142] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1733, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2143] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1733, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2144] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1733, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 2, _padding: 0 }, // [2145] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 1, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2146] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2147] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2148] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2149] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2150] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2151] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2152] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2153] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2154] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2155] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2156] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2157] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2158] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1741, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2159] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1741, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2160] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1743, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2161] Ref(LambdaExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1743, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2162] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1743, child_count: 12, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2163] AnyNumberOf(12 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2164] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2165] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1757, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2166] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2167] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2168] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2169] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2170] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2171] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2172] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2173] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2174] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1769, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2175] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1769, child_count: 1, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2176] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2177] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2178] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2179] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2181] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2182] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2183] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1774, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2184] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2185] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2186] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2187] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2188] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2189] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2190] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1782, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2192] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1782, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2193] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2194] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2195] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2196] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2197] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2198] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2199] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2200] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 5, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2201] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2202] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2203] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1796, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2204] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1796, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2205] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1796, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2206] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1796, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2207] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1796, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2208] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1796, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2209] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2210] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2211] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2212] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2213] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2214] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2215] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1806, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2216] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1806, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2217] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1806, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2218] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1806, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2219] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2220] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2221] Ref(ParametersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2222] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2223] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2224] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2225] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2226] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2227] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2228] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2229] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2230] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2231] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2232] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 1, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2233] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1818, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2234] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1821, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2235] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2236] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2237] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2238] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1824, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2239] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2240] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2241] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2242] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2243] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2244] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1828, child_count: 1, min_times: 0, first_terminator_idx: 35, terminator_count: 1, _padding: 0 }, // [2245] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1829, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2246] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2247] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2248] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [2249] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 1, _padding: 0 }, // [2250] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2251] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2252] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2253] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2254] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1835, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2255] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2256] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2257] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2258] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2259] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2260] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2261] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2262] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1843, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2263] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1845, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2264] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1845, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2265] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1845, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2266] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1845, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2267] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1848, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2268] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2269] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2270] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2271] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1854, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2272] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2273] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2274] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2275] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2276] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1858, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2277] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1860, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2278] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1861, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2279] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2280] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2281] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2282] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2283] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1864, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2284] AnySetOf(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2285] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2286] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2287] StringParser("FUSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2288] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2289] StringParser("G")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2290] StringParser("GCDDELTA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2291] StringParser("GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2292] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2293] StringParser("GEOGRAPHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2294] StringParser("GEOMETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2295] StringParser("GET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2296] StringParser("GLOB")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2297] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2298] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2299] StringParser("GO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2300] StringParser("GOTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2301] StringParser("GRACEPERIOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1865, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2302] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1865, child_count: 6, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2303] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2304] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 4, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2305] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2306] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1878, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2307] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1878, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2308] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1878, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2309] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1878, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2310] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2311] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2312] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 4, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2313] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2314] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2315] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2316] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2317] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2318] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2319] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2320] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1884, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2321] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2322] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2323] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2324] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2325] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2326] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2327] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2328] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2329] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2330] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2331] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2332] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2333] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1896, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2334] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2335] Ref(GrantedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2336] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2337] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1902, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2338] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1902, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2339] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1902, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2340] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1902, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2341] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1902, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2342] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1902, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2343] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2344] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2345] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2346] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1904, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2347] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2348] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2349] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2350] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2351] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 1, _padding: 0 }, // [2352] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2353] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2354] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2355] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2356] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [2357] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 6, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2358] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1921, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2359] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2360] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2361] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2362] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2363] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2364] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2365] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2366] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2367] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2368] StringParser("GROUPED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2369] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1924, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 1, _padding: 0 }, // [2370] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2371] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2372] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2373] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2374] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2375] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2376] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1932, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2377] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2378] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2379] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2380] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2381] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2382] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1940, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2383] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2384] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2385] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2386] StringParser("GZIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2387] StringParser("GZIP_COMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2388] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2389] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2390] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2391] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2392] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2393] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2394] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2395] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2396] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2397] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2398] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2399] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2400] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2401] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2402] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2403] StringParser("HCATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2404] StringParser("HCATALOG_CONNECTION_TIMEOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2405] StringParser("HCATALOG_DB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2406] StringParser("HCATALOG_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2407] StringParser("HCATALOG_SLOW_TRANSFER_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2408] StringParser("HCATALOG_SLOW_TRANSFER_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2409] StringParser("HCATALOG_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2410] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2411] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2412] StringParser("HIERARCHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2413] StringParser("HIGH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2414] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2415] StringParser("HIVESERVER2_HOSTNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2416] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2417] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2418] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2419] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2420] StringParser("HOSTNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2421] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2422] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2423] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2424] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2425] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2426] StringParser("HOURS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2427] StringParser("ICEBERG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2428] StringParser("ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2429] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2430] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2431] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2432] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2433] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2434] StringParser("IDLESESSIONTIMEOUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2435] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2436] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2437] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2438] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2439] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1963, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2440] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1963, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2441] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1963, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2442] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1963, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2443] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1963, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2444] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2445] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2446] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2447] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2448] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2449] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2450] StringParser("ILIKEB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2451] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2452] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2453] StringParser("IMPLEMENTATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2454] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2455] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2456] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2457] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2458] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2459] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2460] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1972, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2461] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1975, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2462] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2463] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2464] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2465] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2466] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2467] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2468] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1979, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2469] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1979, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2470] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1981, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2471] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1981, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2472] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2473] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2474] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2475] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2476] StringParser("INDICATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2477] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2478] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2479] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2480] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2481] StringParser("INITIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2482] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2483] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2484] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2485] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2486] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2487] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2488] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1983, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2489] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2490] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2491] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2492] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2493] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2494] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2495] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2496] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2497] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2498] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2499] Ref(DefaultValuesGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2500] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2501] StringParser("INSTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2502] StringParser("INSTANTIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2503] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2504] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2505] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2506] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2507] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2508] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2509] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2510] StringParser("//")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2511] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2512] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2513] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2514] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2515] StringParser("INTERFACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2516] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2517] Ref(InterpolateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2518] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1998, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2519] Ref(PreviousKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1998, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2520] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1998, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2521] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1998, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2522] StringParser("INTERPOLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1998, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2523] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1998, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2524] StringParser("INTERSECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1998, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2525] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2526] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2527] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2003, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2528] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2005, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2529] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2005, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2530] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2531] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2532] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2533] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2534] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2535] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2536] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2537] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2538] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2539] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2540] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2013, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2541] Ref(IntervalUnitsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2013, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2542] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2543] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2544] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2545] Ref(IntervalUnitsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2017, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2546] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2547] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2548] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2549] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2550] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2551] Ref(HourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2552] Ref(MinuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2553] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2554] StringParser("INTERVALSTYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2555] StringParser("INTERVALYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2556] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2557] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2558] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2559] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2560] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2561] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2562] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2563] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2564] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2565] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2566] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2567] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2568] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2569] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2570] Ref(IsnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2571] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2572] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2573] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2574] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2032, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2575] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2576] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2577] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2578] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2579] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2042, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2580] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2043, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2581] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2043, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2582] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2047, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2583] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2047, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2584] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2047, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2585] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2586] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2587] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2588] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2589] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2055, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2590] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2055, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2591] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2055, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2592] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2055, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2593] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2055, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2594] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2059, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2595] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2059, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2596] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2059, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2597] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2059, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2598] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2060, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2599] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2060, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2600] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2060, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2601] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2602] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2603] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2604] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2066, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2605] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2606] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2607] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2608] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2076, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2609] Ref(AntiKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2076, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2610] Ref(SemiallKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2076, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2611] Ref(SemiKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2076, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2612] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2613] Ref(NullawareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2614] Ref(AntiKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2615] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2616] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2080, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2617] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2083, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2618] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2083, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2619] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2083, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2620] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2083, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2621] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2083, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2622] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2085, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2623] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2085, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2624] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2625] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2626] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2089, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2627] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2628] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2629] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2630] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2631] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2632] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2096, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2633] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2096, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2634] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2099, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2635] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2636] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2637] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2638] StringParser("K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2639] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2640] StringParser("KEY_MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2641] StringParser("KEY_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2642] StringParser("KEYMANAGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2643] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2644] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2645] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2646] StringParser("KSAFE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2101, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2647] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2103, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2648] Ref(KsafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2103, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2649] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2103, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2650] StringParser("LABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2103, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2651] StringParser("->")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2103, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2652] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2106, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2653] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2108, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2654] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2108, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2655] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2656] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2657] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2658] Ref(LambdaArrowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2659] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2660] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2661] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2662] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2663] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2664] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2665] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2666] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2667] StringParser("LATEST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2668] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2669] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2670] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2671] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2672] StringParser("LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2673] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2674] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2675] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2676] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2677] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2678] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2679] StringParser("LIBRARY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2680] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2118, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2681] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2120, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2682] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2120, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2683] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2120, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2684] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2120, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2685] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2686] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2122, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2687] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2122, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2688] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2689] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2690] Ref(RlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2691] Ref(IlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2125, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2692] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2125, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2693] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2694] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2695] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2128, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2696] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2130, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2697] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2132, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2698] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2132, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2699] Ref(ExcludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2132, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2700] Ref(ProjectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2132, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2701] Ref(SchemaPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2132, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2702] StringParser("LIKEB")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2132, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2703] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2704] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2705] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2706] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2142, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2707] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2708] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2709] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2710] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2145, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2711] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2148, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2712] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2150, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2713] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2150, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2714] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2715] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2716] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2717] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2718] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2719] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2720] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2721] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2722] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2723] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2724] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2725] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 9, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2726] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2727] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2728] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2729] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2730] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2731] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2732] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2733] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2734] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2735] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2736] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2737] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2738] StringParser("LN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2739] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2740] StringParser("LOADER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2741] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2742] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2743] StringParser("LOCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2744] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2745] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2746] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2747] StringParser("LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2748] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2749] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2750] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2751] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2752] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2753] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2754] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2755] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2756] StringParser("LOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2757] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2758] StringParser("LOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2759] StringParser("LZO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2760] StringParser("M")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2761] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2762] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2763] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2764] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2765] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2171, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2766] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2173, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2767] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2173, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2768] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2173, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2769] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2770] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2771] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2772] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2773] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2774] StringParser("MANAGED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2775] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2776] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2777] StringParser("MASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2778] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2779] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2780] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2781] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2782] StringParser("MATERIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2783] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2784] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2785] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2786] StringParser("MAXCONCURRENCY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2787] StringParser("MAXCONCURRENCYGRACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2788] StringParser("MAXCONNECTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2789] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2790] StringParser("MAXMEMORYSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2791] StringParser("MAXPAYLOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2792] StringParser("MAXQUERYMEMORYSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2793] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2794] StringParser("MEDIUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2795] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2796] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2797] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2798] StringParser("MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2799] StringParser("MEMORYCAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2800] StringParser("MEMORYSIZE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2801] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2178, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2802] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2183, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2803] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2183, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2804] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2183, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2805] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2183, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2806] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2807] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2808] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2809] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 2, min_times: 1, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2810] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2811] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2812] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2813] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2814] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2815] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2194, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2816] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2817] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2818] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2819] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2820] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2821] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2822] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 8, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2823] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2824] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2825] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2826] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2206, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2827] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2828] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2829] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2830] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2831] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 12, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2832] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2220, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2833] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2220, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2834] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2835] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2836] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2837] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2838] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2226, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2839] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2226, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2840] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2226, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2841] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2842] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2231, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2843] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2231, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2844] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2231, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2845] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2846] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2847] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2848] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2849] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2850] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2851] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2852] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2853] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2854] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2855] StringParser("MERGEOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2856] StringParser("MESSAGE_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2857] StringParser("MESSAGE_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2858] StringParser("MESSAGE_TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2859] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2860] StringParser("MICROSECONDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2861] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2862] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2863] StringParser("MILLISECONDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2864] StringParser("MIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2865] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2866] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2867] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2868] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2869] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2870] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2871] StringParser("MINUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2872] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2873] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2874] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2875] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2876] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2877] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2878] StringParser("MODIFIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2879] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2880] StringParser("MODULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2881] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2882] StringParser("MONEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2883] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2884] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2885] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2886] StringParser("MORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2887] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2888] StringParser("MOVEOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2889] StringParser("MULTIPLEACTIVERESULTSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2890] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2891] StringParser("MULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2892] StringParser("MUMPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2893] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2894] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2895] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2896] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2240, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2897] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2240, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2898] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2240, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2899] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2900] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2901] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2245, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2902] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2245, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2903] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2904] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2905] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2906] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2907] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2908] StringParser("NAMESPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2909] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2910] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2911] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2912] StringParser("NATIVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2913] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2914] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2915] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2916] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2917] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2918] StringParser("NCLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2919] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2920] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2921] StringParser("NESTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2922] StringParser("NETWORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2923] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2924] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2925] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2926] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2927] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2928] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2929] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2930] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2931] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2932] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2933] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2934] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2935] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2936] StringParser("NODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2937] StringParser("NODES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2938] StringParser("NOGLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2939] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2940] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2253, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2941] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2259, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2942] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2259, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2943] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2259, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2944] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2945] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2262, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2946] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2947] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2948] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2268, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2949] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2268, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2950] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2268, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2951] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2268, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2952] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2953] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2954] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2955] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2956] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2957] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2958] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2959] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2960] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2961] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2962] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2963] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2964] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2965] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2966] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2967] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2968] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2969] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2970] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2971] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2282, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2972] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2284, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2973] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2284, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2974] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2284, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2975] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2976] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2977] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2978] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2979] Ref(NotnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2980] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2981] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2982] StringParser("NOTIFIER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2983] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2984] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2985] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2986] StringParser("::!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2987] StringParser("<=>")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2988] Ref(NullEqualsOperatorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2989] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2990] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2991] StringParser("NULLABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2992] StringParser("NULLAWARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2993] StringParser("NULLCOLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2994] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2995] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2996] StringParser("NULLSEQUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2997] StringParser("NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2998] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2999] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3000] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3001] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3002] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3003] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3004] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3005] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2292, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3006] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3007] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3008] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2294, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3009] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2296, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3010] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2296, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3011] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3012] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3013] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 13, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3014] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3015] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3016] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3017] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3018] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3019] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3020] Ref(NullCastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3021] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3022] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3023] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3024] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3025] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3026] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3027] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3028] StringParser("OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3029] StringParser("OCTETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3030] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3031] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3032] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3033] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2314, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3034] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2314, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3035] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3036] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2316, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3037] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2317, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3038] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2317, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3039] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3040] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3041] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3042] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3043] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3044] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3045] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3046] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3047] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3048] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3049] StringParser("OPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3050] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3051] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3052] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3053] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3054] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3055] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3056] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3057] StringParser("OPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3058] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3059] StringParser("OPTIMIZER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3060] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3061] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3062] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3063] StringParser("OPTVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3064] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3065] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3067] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3068] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3069] StringParser("ORC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3070] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3071] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3072] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 2, _padding: 0 }, // [3073] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3074] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3075] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3076] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3077] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3078] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2335, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3079] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2337, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3080] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2337, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3081] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2337, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3082] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3083] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3084] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3085] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3086] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3087] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3088] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3089] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 7, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3090] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3091] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3092] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3093] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3094] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3095] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3096] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3097] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3098] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3099] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3100] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3101] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3102] StringParser("ORDERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3103] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3104] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3105] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3106] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3107] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3108] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3109] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2355, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3110] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3111] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3112] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3113] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3114] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2360, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3115] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2360, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3116] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2360, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3117] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3118] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3119] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3120] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3121] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3122] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3123] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3124] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3125] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3126] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3127] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3128] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3129] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3130] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3131] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3132] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3133] StringParser("PAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3134] StringParser("PARAMETER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3135] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3136] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3137] StringParser("PARAMETER_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3138] StringParser("PARAMETER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3139] StringParser("PARAMETER_ORDINAL_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3140] StringParser("PARAMETER_SPECIFIC_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3141] StringParser("PARAMETER_SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3142] StringParser("PARAMETER_SPECIFIC_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3143] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3144] StringParser("PARQUET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3145] StringParser("PARSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3146] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3147] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2375, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3148] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2375, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3149] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2375, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 2, _padding: 0 }, // [3150] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2377, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3151] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2379, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3152] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2380, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3153] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2384, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3154] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2384, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3155] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2384, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3156] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2384, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3157] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2384, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3158] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2387, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3159] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3160] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2390, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3161] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2393, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3162] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2393, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3163] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2393, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3164] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2393, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3165] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3166] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3167] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3168] Ref(ReorganizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3169] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3170] Ref(ReorganizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2397, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3172] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2397, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3173] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2402, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3174] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3175] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3176] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2408, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3177] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2411, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3178] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3179] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3180] Ref(BestKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3181] Ref(NodesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3182] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3183] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3184] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3185] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3186] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3187] StringParser("PARTITIONING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3188] StringParser("PASCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3189] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3190] StringParser("PASSWORD_GRACE_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3191] StringParser("PASSWORD_LIFE_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3192] StringParser("PASSWORD_LOCK_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3193] StringParser("PASSWORD_MAX_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3194] StringParser("PASSWORD_MIN_CHAR_CHANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3195] StringParser("PASSWORD_MIN_DIGITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3196] StringParser("PASSWORD_MIN_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3197] StringParser("PASSWORD_MIN_LETTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3198] StringParser("PASSWORD_MIN_LIFE_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3199] StringParser("PASSWORD_MIN_LOWERCASE_LETTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3200] StringParser("PASSWORD_MIN_SYMBOLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3201] StringParser("PASSWORD_MIN_UPPERCASE_LETTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3202] StringParser("PASSWORD_REUSE_MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3203] StringParser("PASSWORD_REUSE_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3204] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3205] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2417, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3206] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3207] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2419, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3208] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3209] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3210] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3211] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3212] StringParser("PATTERN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3213] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3214] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3215] StringParser("PERCENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3216] StringParser("PERCENT_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3217] StringParser("PERCENTILE_CONT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3218] StringParser("PERCENTILE_DISC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3219] StringParser("PERMANENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3220] StringParser("PINNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3221] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3222] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3223] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3224] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3225] StringParser("PLANNEDCONCURRENCY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3226] StringParser("PLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3227] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3228] StringParser("POINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3229] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3230] StringParser("POOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3231] StringParser("PORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3232] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3233] StringParser("+")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2421, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3234] AnySetOf(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3235] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3236] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3237] Ref(WithinGroupClauseSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3238] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3239] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3240] StringParser("POWER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3241] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3242] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3243] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3244] StringParser("PREFER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3245] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3246] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3247] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3248] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3249] StringParser("PREPASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3250] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3251] StringParser("PREVIOUS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3252] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3253] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3254] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3255] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3256] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3257] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3258] StringParser("PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3259] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3260] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3261] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3262] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3263] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3264] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3265] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3266] StringParser("PROFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3267] StringParser("PROJECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3268] StringParser("PROJECTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3269] StringParser("PSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3270] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3271] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3272] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2428, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3273] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2428, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3274] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2428, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3275] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2428, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3276] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3277] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3278] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3279] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3280] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2434, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3281] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2434, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3282] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3283] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3284] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3285] StringParser("QUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3286] StringParser("QUEUETIMEOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3287] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3288] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3289] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3290] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2440, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3291] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2440, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3292] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3293] TypedParser("escaped_single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3294] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3295] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3296] StringParser("RANDOM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3297] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3298] StringParser("RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3299] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3300] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3301] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3302] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3303] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3304] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3305] StringParser("READS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3306] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3307] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3308] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3309] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3310] StringParser("RECORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3311] StringParser("RECOVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3312] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3313] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3314] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2446, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3315] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2446, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3316] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2446, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3317] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2446, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3318] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2446, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3319] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2448, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3320] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2451, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3321] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2451, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3322] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2451, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3323] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2451, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3324] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2454, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3325] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2454, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3326] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2454, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3327] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2454, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3328] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2456, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3329] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2456, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3330] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3331] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3332] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3333] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3334] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3335] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3336] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3337] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3338] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3339] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3340] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2466, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3341] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2466, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3342] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2466, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3343] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3344] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3345] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3346] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3347] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3348] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3349] StringParser("REFRESH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3350] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3351] StringParser("REGR_AVGX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3352] StringParser("REGR_AVGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3353] StringParser("REGR_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3354] StringParser("REGR_INTERCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3355] StringParser("REGR_R2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3356] StringParser("REGR_SLOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3357] StringParser("REGR_SXX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3358] StringParser("REGR_SXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3359] StringParser("REGR_SYY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3360] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3361] StringParser("REJECTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3362] StringParser("REJECTMAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3363] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3364] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3365] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3366] StringParser("REMOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3367] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3368] StringParser("REORGANIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3369] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3370] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3371] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3372] StringParser("REPLICATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3373] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3374] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3375] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3376] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3377] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3378] StringParser("RESOURCE_POOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3379] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3380] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3381] StringParser("RESTORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3382] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3383] StringParser("RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3384] StringParser("RESULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3385] StringParser("RETENTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3386] StringParser("RETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3387] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3388] StringParser("RETURNED_CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3389] StringParser("RETURNED_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3390] StringParser("RETURNED_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3391] StringParser("RETURNED_SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3392] StringParser("RETURNREJECTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3393] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3394] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 6, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3395] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3396] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2476, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3397] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3398] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3399] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3400] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3401] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2483, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3402] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3403] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3404] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3405] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2486, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3406] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3407] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3408] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3409] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3410] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3411] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3412] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3413] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3414] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3415] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3416] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3417] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3418] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3419] StringParser("RLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3420] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3421] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3422] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3423] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3424] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3425] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3426] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3427] StringParser("ROUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3428] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3429] StringParser("ROUTINE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3430] StringParser("ROUTINE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3431] StringParser("ROUTINE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3432] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3433] StringParser("ROUTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3434] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3435] StringParser("ROW_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3436] StringParser("ROW_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3437] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3438] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3439] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3440] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3441] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3442] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3443] StringParser("RUNTIMECAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3444] StringParser("RUNTIMEPRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3445] StringParser("RUNTIMEPRIORITYTHRESHOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3446] StringParser("RWITH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3447] StringParser("SALT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3448] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3449] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3450] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2498, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3451] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2498, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3452] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2498, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3453] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2501, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3454] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2501, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3455] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2503, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3456] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3457] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3458] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3459] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3460] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3461] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3462] StringParser("SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3463] StringParser("SCHEDULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3464] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2507, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3465] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2510, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3466] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3467] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3468] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3469] Ref(MaterializeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3470] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3471] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3472] StringParser("SCHEMA_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3473] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3474] StringParser("SCHEMATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3475] StringParser("SCOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3476] StringParser("SCOPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3477] StringParser("SCOPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3478] StringParser("SCOPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3479] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3480] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3481] StringParser("SEARCH_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3482] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3483] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3484] StringParser("SECONDARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3485] StringParser("SECONDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3486] StringParser("SECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3487] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3488] StringParser("SECURITY_ALGORITHM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3489] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2514, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3490] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2516, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3491] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2519, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3492] Ref(UnsegmentedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2519, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3493] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2519, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3494] Ref(NodesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2519, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3495] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3496] Ref(SegmentedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3497] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3498] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3499] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3500] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2529, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3501] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3502] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3503] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3504] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3505] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3506] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3507] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3508] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3509] Ref(NodesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2536, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3510] StringParser("SEGMENTED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2536, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3511] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2538, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3512] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2538, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3513] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3514] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2540, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3515] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2540, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3516] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3517] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3518] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 1, _padding: 0 }, // [3519] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2547, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3520] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2547, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3521] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 2547, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3522] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2549, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3523] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2549, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3524] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2549, child_count: 7, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3525] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2556, child_count: 1, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3526] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2557, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3527] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2557, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3528] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2557, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3529] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3530] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3531] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3532] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3533] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3534] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3535] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2559, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3536] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 12, min_times: 0, first_terminator_idx: 44, terminator_count: 3, _padding: 0 }, // [3537] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3538] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3539] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3540] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3541] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3542] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3543] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3544] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3545] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3546] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3547] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3548] Ref(TimeseriesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3549] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3550] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3551] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [3552] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3553] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2575, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3554] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2577, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3555] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2580, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3556] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2580, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3557] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2582, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3558] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2585, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3559] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2585, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3560] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2585, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3561] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3562] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3563] StringParser("SELF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3564] StringParser("SEMI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3565] StringParser("SEMIALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3566] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3567] StringParser("SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3568] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2588, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3569] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2588, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3570] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2590, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3571] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2592, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3572] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2592, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3573] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2592, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3574] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2594, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3575] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2594, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3576] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2594, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3577] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2596, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3578] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3579] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3580] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2598, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3581] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3582] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3583] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3584] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3585] StringParser("SERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3586] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3587] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3588] StringParser("SERVER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3589] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3590] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3591] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2604, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3592] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2604, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3593] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3594] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3595] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3596] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3597] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 7, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3598] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3599] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3600] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3601] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3602] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3603] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3604] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3605] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3606] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2621, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3607] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2621, child_count: 1, min_times: 1, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3608] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2622, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3609] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3610] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3611] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3612] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3613] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3614] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3615] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3616] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2627, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3617] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2627, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3618] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2629, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3619] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3620] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3621] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3622] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3623] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2631, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3624] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3625] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3626] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3627] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3628] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2635, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3629] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2637, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3630] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2637, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3631] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2643, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3632] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2646, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3633] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3634] Ref(DatestyleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3635] Ref(Escape_string_warningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3636] Ref(IntervalstyleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3637] Ref(LocaleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3638] Ref(Standard_conforming_stringsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3639] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3640] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3641] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2654, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3642] Ref(Search_pathKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2654, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3643] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2656, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3644] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2656, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3645] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2656, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3646] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2658, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3647] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3648] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3649] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3650] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2662, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3651] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2662, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3652] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2666, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3653] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2666, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3654] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2666, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3655] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2668, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3656] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2668, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3657] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2670, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3658] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2670, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3659] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2672, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3660] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2672, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3661] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2674, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3662] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2674, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3663] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2678, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3664] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2678, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3665] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2678, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3666] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2678, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3667] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3668] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3669] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3670] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3671] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3672] Ref(Resource_poolKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3673] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2684, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3674] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3675] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3676] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3677] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3678] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2689, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3679] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2689, child_count: 7, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3680] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2696, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3681] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2698, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3682] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2698, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3683] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3684] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3685] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2700, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3686] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3687] Ref(AutocommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3688] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3689] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2705, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3690] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2705, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3691] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2705, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3692] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3693] Ref(CharacteristicsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3694] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3695] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3696] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3697] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2711, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3698] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2714, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3699] Ref(GraceperiodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2714, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3700] Ref(IdlesessiontimeoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2714, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3701] Ref(RuntimecapKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2714, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3702] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2717, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3703] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2717, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3704] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2719, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3705] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2719, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3706] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2719, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3707] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2719, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3708] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2721, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3709] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2723, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3710] Ref(MemorycapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2723, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3711] Ref(TempspacecapKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2723, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3712] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2726, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3713] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2726, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3714] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2728, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3715] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2728, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3716] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2728, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3717] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2728, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3718] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2731, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3719] Ref(MultipleactiveresultsetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2731, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3720] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2731, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3721] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2733, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3722] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2733, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3723] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2733, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3724] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3725] Ref(WorkloadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3726] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2736, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3727] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3728] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3729] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3730] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3731] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3732] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3733] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3734] StringParser("SHARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3735] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3736] StringParser("SHARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3737] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3738] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2741, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3739] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2743, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3740] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2743, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3741] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2743, child_count: 1, min_times: 1, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3742] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2744, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3743] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2747, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3744] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2749, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3745] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2749, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3746] Ref(NullCastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2749, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3747] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2749, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3748] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2749, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3749] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2749, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3750] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2749, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3751] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2749, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3752] StringParser("SIGNED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2749, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3753] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3754] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3755] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3756] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2751, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3757] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2751, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 1, _padding: 0 }, // [3758] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3759] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3760] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3761] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3762] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3763] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 3, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3764] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3765] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3766] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3767] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3768] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3769] StringParser("SINGLEINITIATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3770] StringParser("SITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3771] StringParser("SITES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3772] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2758, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3773] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3774] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3775] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3776] StringParser("SKIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3777] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3778] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3779] StringParser("SMALLDATETIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3780] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3781] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3782] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3783] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3784] StringParser("SPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3785] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3786] StringParser("SPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3787] StringParser("SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3788] StringParser("SPECIFICTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3789] StringParser("SPLIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3790] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3791] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3792] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3793] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3794] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3795] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3796] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3797] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3798] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3799] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3800] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3801] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3802] StringParser("SQLCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3803] StringParser("SQLERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3804] StringParser("SQLEXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3805] StringParser("SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3806] StringParser("SQLWARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3807] StringParser("SQRT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3808] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3809] StringParser("SSL_CONFIG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3810] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3811] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3812] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3813] StringParser("STANDARD_CONFORMING_STRINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3814] StringParser("STANDBY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3815] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3816] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3817] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3818] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3819] StringParser("STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2760, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3820] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2760, child_count: 52, min_times: 0, first_terminator_idx: 48, terminator_count: 1, _padding: 0 }, // [3821] OneOf(52 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3822] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3823] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3824] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3825] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3826] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3827] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3828] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3829] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3830] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3831] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3832] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3833] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3834] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3835] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3836] Ref(CreateSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3837] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3838] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3839] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3840] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3841] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3842] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3843] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3844] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3845] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3846] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3847] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3848] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3849] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3850] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3851] Ref(CreateModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3852] Ref(DropModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3853] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3854] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3855] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3856] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3857] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3858] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3859] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3860] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3861] Ref(CreateExternalTableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3862] Ref(CreateTableLikeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3863] Ref(CreateTableAsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3864] Ref(CreateProjectionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3865] Ref(AlterDefaultPrivilegesGrantSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3866] Ref(DropProjectionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3867] Ref(AlterViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3868] Ref(SetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3869] Ref(CommentOnStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3870] Ref(TransactionalStatements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3871] Ref(AlterSessionStatements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3872] Ref(CopyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3873] Ref(AlterSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3874] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3875] StringParser("STATIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3876] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3877] StringParser("STDDEV_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3878] StringParser("STDDEV_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3879] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3880] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3881] StringParser("STEMMER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3882] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3883] StringParser("STORED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3884] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3885] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3886] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3887] StringParser("STRENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2812, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3888] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2812, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3889] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3890] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2813, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3891] StringParser("STRING")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3892] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2816, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3893] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2818, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3894] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3895] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3896] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3897] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3898] StringParser("STRUCTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3899] StringParser("STYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3900] StringParser("SUBCLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3901] StringParser("SUBCLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3902] StringParser("SUBJECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3903] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3904] StringParser("SUBMULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3905] StringParser("SUBNET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3906] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3907] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3908] StringParser("SUITES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3909] StringParser("SUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3910] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3911] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3912] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3913] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3914] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3915] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3916] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2820, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3917] StringParser("SYSTEM_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2820, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3918] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2822, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3919] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3920] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3921] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3922] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2828, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3923] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2831, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3924] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2831, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3925] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2831, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3926] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3927] Ref(EnabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3928] Ref(DisabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3929] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2836, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3930] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2836, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3931] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2839, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3932] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2839, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3933] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2841, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3934] Ref(EnabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2841, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3935] Ref(DisabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2841, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3936] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2844, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3937] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2844, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3938] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2844, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3939] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2846, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3940] Ref(EnabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2846, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3941] Ref(DisabledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2846, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3942] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2850, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3943] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2850, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3944] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2850, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3945] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2850, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3946] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2850, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3947] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2850, child_count: 6, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3948] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2856, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3949] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2856, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3950] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2856, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3951] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2856, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3952] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2856, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3953] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3954] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3955] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3956] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3957] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3958] StringParser("TABLE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3959] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3960] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2862, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3961] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3962] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2864, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 1, _padding: 0 }, // [3963] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2865, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3964] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2865, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3965] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2865, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3966] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2865, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3967] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2867, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3968] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3969] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3970] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3971] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3972] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3973] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3974] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3975] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3976] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2870, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3977] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2870, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3978] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2870, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3979] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2870, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3980] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3981] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3982] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3983] StringParser("TEMPSPACECAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3984] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3985] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3986] StringParser("TERMINATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3987] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3988] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3989] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3990] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3991] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3992] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3993] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2872, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3994] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2875, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3995] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2877, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3996] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2877, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3997] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2877, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3998] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2877, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3999] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2880, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4000] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2882, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4001] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2882, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4002] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2882, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4003] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2882, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4004] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2882, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4005] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2883, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4006] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4007] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4008] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4009] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4010] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 7, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4011] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4012] Ref(TimeseriesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4013] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4014] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4015] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4016] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4017] StringParser("TIMESERIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4018] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4019] StringParser("TIMESTAMPADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4020] StringParser("TIMESTAMPDIFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4021] StringParser("TIMESTAMPTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4022] StringParser("TIMETZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4023] StringParser("TIMEZONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4024] StringParser("TIMEZONE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4025] StringParser("TIMEZONE_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4026] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4027] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4028] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4029] StringParser("TLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4030] StringParser("TLSMODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4031] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4032] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4033] StringParser("TOKENIZER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4034] StringParser("TOLERANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4035] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4036] StringParser("TOP_LEVEL_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4037] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4038] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4039] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4040] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2898, child_count: 5, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4041] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2903, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4042] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2903, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4043] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2903, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4044] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2903, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4045] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2903, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4046] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2903, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4047] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2905, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4048] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2905, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4049] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2905, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4050] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2907, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4051] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2907, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4052] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2907, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4053] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4054] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4055] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4056] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4057] StringParser("TRANSACTION_ACTIVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2910, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4058] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2913, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4059] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2913, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4060] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2917, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4061] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2919, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4062] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2919, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4063] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2919, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4064] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2921, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4065] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2921, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4066] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2921, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4067] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2923, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4068] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2923, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4069] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2923, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4070] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4071] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4072] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4073] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4074] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4075] StringParser("TRANSACTIONS_COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4076] StringParser("TRANSACTIONS_ROLLED_BACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4077] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4078] StringParser("TRANSFORMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4079] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4080] StringParser("TRANSLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4081] StringParser("TRANSLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4082] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4083] StringParser("TRICKLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4084] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4085] StringParser("TRIGGER_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4086] StringParser("TRIGGER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4087] StringParser("TRIGGER_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4088] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2925, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4089] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2928, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4090] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2928, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4091] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2928, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4092] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2928, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4093] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2928, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4094] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2928, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4095] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2928, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4096] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2931, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4097] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2931, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4098] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2931, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4099] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2931, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4100] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2931, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4101] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2931, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4102] StringParser("TUNING")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2931, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4103] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2934, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4104] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4105] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4106] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4107] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4108] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4109] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2938, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4110] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4111] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4112] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4113] StringParser("TYPES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4114] StringParser("UDPARAMETER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4115] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4116] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4117] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4118] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4119] StringParser("UNCOMPRESSED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4120] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2940, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4121] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4122] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4123] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4124] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4125] StringParser("UNDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4126] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4127] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4128] StringParser("UNI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4129] StringParser("UNINDEXED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4131] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2945, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4132] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2947, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4133] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2947, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4134] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2947, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4135] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2947, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4136] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4137] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4138] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4139] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4140] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4141] StringParser("UNLIMITED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4142] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4143] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4144] StringParser("UNNAMED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4145] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2948, child_count: 7, min_times: 0, first_terminator_idx: 50, terminator_count: 6, _padding: 0 }, // [4146] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4147] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4148] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4149] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4150] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4151] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4152] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4153] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4154] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4155] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4156] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4157] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4158] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [4159] Ref(TimeseriesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2955, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4160] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4161] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 1, min_times: 1, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4162] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2958, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4163] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2960, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4164] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2960, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4165] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2960, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4166] StringParser("UNSEGMENTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2960, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4167] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2960, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4168] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2960, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4169] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2960, child_count: 8, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4170] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4171] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4172] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2968, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4173] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2969, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4174] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2969, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4175] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2969, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4176] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2969, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4177] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2969, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4178] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2969, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4179] StringParser("UPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2969, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4180] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2969, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4181] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2969, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4182] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4183] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4184] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4185] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4186] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4187] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4188] StringParser("USER_DEFINED_TYPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4189] StringParser("USER_DEFINED_TYPE_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4190] StringParser("USER_DEFINED_TYPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4191] StringParser("USER_DEFINED_TYPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4192] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4193] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4194] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4195] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4196] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4197] StringParser("UUID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4198] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4199] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4200] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4201] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4202] StringParser("VALINDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4203] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4204] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2973, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4205] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2975, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4206] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2975, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4207] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2975, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4208] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2977, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4209] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2979, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4210] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2979, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4211] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4212] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2984, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4213] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4214] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4215] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4216] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4217] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4218] StringParser("VAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4219] StringParser("VAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4220] StringParser("VARBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4221] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4222] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4223] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4224] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4225] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4226] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4227] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4228] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4229] StringParser("VERTICA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4230] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4231] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4232] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4233] StringParser("WAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4234] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4235] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4236] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4237] StringParser("WEBHDFS_ADDRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4238] StringParser("WEBSERVICE_HOSTNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4239] StringParser("WEBSERVICE_PORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4240] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4241] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2987, child_count: 8, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4242] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2995, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4243] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2995, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4244] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2998, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4245] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2998, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4246] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2998, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4247] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2998, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4248] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2998, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4249] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2998, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4250] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2998, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4251] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2998, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4252] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2998, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4253] StringParser("WHENEVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2998, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4254] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3002, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4255] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3002, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4256] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3004, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4257] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4258] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 8, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4259] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3015, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4260] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3015, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4261] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4262] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3017, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4263] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3017, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4264] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4265] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4266] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4267] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4268] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4269] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4270] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4271] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4272] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4273] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4274] Token("whitespace")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4275] StringParser("WIDTH_BUCKET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4276] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3020, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4277] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3020, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4278] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3022, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4279] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3023, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4280] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3025, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4281] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3027, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4282] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3027, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4283] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3027, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4284] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3029, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4285] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3029, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4286] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3029, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4287] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3029, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4288] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3029, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4289] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 3033, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4290] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3034, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4291] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4292] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4293] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4294] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4295] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4296] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 6, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [4297] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4298] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3042, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4299] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3042, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4300] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 1, _padding: 0 }, // [4301] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4302] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [4303] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3044, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [4304] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [4305] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 6, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4306] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [4307] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [4308] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [4309] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3050, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 1, _padding: 0 }, // [4310] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [4311] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [4312] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4313] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4314] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3052, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4315] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3055, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4316] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3055, child_count: 1, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4317] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3056, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4318] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3056, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4319] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3056, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4320] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3056, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4321] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3056, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4322] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4323] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4324] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4325] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4326] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3060, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4327] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4328] Ref(WithinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3063, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4329] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3063, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4330] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4331] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4332] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4333] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4334] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4335] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4336] StringParser("WORKLOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4337] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4338] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4339] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4340] StringParser("X509")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4341] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4342] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4343] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4344] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4345] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4346] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4347] StringParser("ZONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4348] StringParser("ZSTD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4349] StringParser("ZSTD_COMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4350] StringParser("ZSTD_FAST_COMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3066, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [4351] StringParser("ZSTD_HIGH_COMP")
];

pub static CHILD_IDS: &[u32] = &[
    5, 6, 61, 7, 51, 8, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 28, 33, 38, 39, 44, 9, 10, 22, 23, 24, 25, 26, 27, 29, 30,
    31, 32, 34, 35, 36, 37, 40, 41, 42, 43, 45, 46, 47, 48, 49, 50,
    52, 58, 53, 54, 57, 55, 56, 62, 63, 64, 66, 74, 78, 81, 84, 89,
    67, 68, 69, 70, 71, 72, 73, 75, 76, 77, 79, 80, 82, 83, 85, 86,
    87, 88, 90, 133, 91, 100, 105, 108, 109, 110, 111, 112, 113, 114, 115, 116,
    117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 92, 93,
    94, 95, 98, 99, 96, 97, 101, 102, 103, 104, 106, 107, 131, 132, 135, 58,
    138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 151, 154, 157, 149, 150,
    152, 153, 155, 156, 158, 159, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
    171, 174, 177, 180, 172, 173, 175, 176, 178, 179, 181, 182, 184, 185, 187, 191,
    195, 199, 203, 205, 207, 188, 189, 190, 58, 192, 193, 194, 58, 196, 197, 198,
    58, 200, 201, 202, 58, 204, 58, 206, 58, 209, 224, 239, 225, 226, 227, 238,
    228, 234, 237, 229, 230, 231, 232, 233, 235, 236, 232, 233, 225, 240, 241, 238,
    242, 232, 233, 243, 58, 246, 247, 252, 253, 254, 255, 268, 269, 272, 256, 260,
    257, 58, 258, 259, 261, 58, 262, 263, 264, 265, 266, 270, 58, 273, 274, 275,
    277, 281, 278, 279, 280, 282, 58, 283, 284, 285, 286, 287, 288, 289, 290, 294,
    295, 296, 298, 297, 58, 299, 302, 307, 312, 300, 301, 303, 304, 305, 306, 58,
    308, 309, 310, 311, 314, 318, 319, 320, 325, 328, 315, 316, 317, 321, 324, 322,
    323, 326, 327, 330, 331, 332, 333, 334, 336, 337, 338, 339, 348, 360, 371, 340,
    341, 342, 343, 344, 344, 232, 233, 345, 346, 347, 349, 350, 351, 352, 357, 353,
    232, 233, 354, 355, 356, 358, 359, 361, 362, 363, 366, 364, 365, 367, 232, 233,
    368, 369, 370, 372, 373, 374, 377, 375, 376, 378, 379, 380, 232, 233, 382, 385,
    392, 393, 397, 398, 402, 383, 384, 386, 387, 388, 389, 390, 391, 394, 395, 396,
    399, 400, 401, 403, 404, 405, 410, 413, 406, 407, 408, 409, 411, 412, 414, 415,
    416, 417, 419, 420, 421, 422, 424, 430, 443, 444, 425, 426, 427, 428, 429, 431,
    434, 435, 436, 432, 433, 437, 442, 438, 441, 439, 440, 445, 446, 449, 447, 448,
    451, 452, 453, 455, 454, 58, 456, 459, 478, 514, 521, 527, 528, 531, 537, 542,
    543, 457, 458, 58, 460, 461, 462, 463, 464, 465, 467, 468, 477, 466, 469, 474,
    470, 471, 472, 232, 233, 473, 58, 475, 476, 479, 480, 481, 482, 483, 489, 492,
    496, 501, 484, 485, 486, 487, 232, 233, 488, 58, 490, 491, 493, 494, 495, 497,
    498, 499, 500, 502, 503, 504, 505, 508, 511, 506, 507, 509, 510, 512, 513, 515,
    516, 517, 518, 519, 520, 522, 523, 524, 525, 526, 529, 530, 532, 533, 534, 535,
    536, 538, 539, 540, 541, 58, 544, 545, 546, 548, 549, 550, 552, 551, 58, 553,
    557, 561, 562, 554, 555, 556, 558, 559, 560, 563, 564, 565, 566, 58, 581, 582,
    583, 584, 585, 586, 587, 588, 589, 590, 591, 593, 598, 599, 594, 597, 595, 596,
    603, 598, 599, 604, 58, 606, 607, 608, 598, 599, 610, 625, 637, 638, 639, 640,
    641, 642, 643, 644, 645, 660, 661, 662, 663, 671, 672, 675, 676, 684, 685, 688,
    689, 692, 232, 233, 693, 58, 695, 232, 233, 696, 58, 699, 232, 233, 712, 232,
    233, 714, 715, 716, 717, 718, 232, 233, 727, 737, 728, 729, 730, 734, 238, 736,
    731, 738, 739, 729, 740, 744, 238, 746, 741, 791, 792, 793, 794, 810, 811, 812,
    813, 815, 816, 843, 817, 820, 818, 819, 821, 824, 831, 836, 842, 822, 823, 825,
    826, 828, 827, 232, 233, 829, 830, 832, 833, 834, 835, 837, 838, 839, 840, 841,
    845, 846, 847, 849, 848, 232, 233, 850, 852, 853, 857, 858, 861, 869, 862, 863,
    866, 864, 865, 867, 868, 870, 871, 872, 873, 874, 875, 876, 232, 233, 888, 889,
    892, 893, 894, 895, 931, 896, 902, 905, 911, 920, 923, 926, 897, 901, 898, 899,
    900, 903, 904, 906, 907, 908, 909, 910, 912, 916, 917, 918, 913, 914, 915, 919,
    921, 922, 924, 925, 927, 930, 928, 929, 932, 933, 934, 935, 942, 943, 944, 945,
    946, 947, 948, 949, 950, 958, 961, 962, 967, 968, 969, 987, 988, 991, 992, 997,
    1002, 1003, 1012, 1015, 1018, 1023, 993, 994, 996, 995, 998, 999, 1001, 1000, 1004, 1009,
    1005, 1006, 1008, 1007, 1010, 1011, 1013, 1014, 1016, 1017, 1019, 1020, 1022, 1021, 1024, 1025,
    1027, 1028, 1032, 1035, 1038, 1042, 1046, 1049, 1052, 1056, 1059, 1029, 1030, 1031, 1033, 1034,
    1036, 1037, 1039, 1040, 1041, 1043, 1044, 1045, 1047, 1048, 1050, 1051, 1053, 1054, 1055, 1057,
    1058, 1061, 1062, 1063, 1075, 1076, 1095, 1102, 1064, 1067, 1065, 232, 233, 1066, 58, 1068,
    1069, 1070, 1071, 232, 233, 1072, 58, 1073, 1074, 1077, 1082, 1086, 1092, 1078, 1079, 1080,
    1081, 1083, 1084, 1085, 1087, 1088, 1089, 1090, 232, 233, 1091, 58, 1093, 1094, 58, 1096,
    1097, 1100, 1101, 1098, 1099, 1112, 1113, 1114, 1118, 1119, 1120, 1130, 1131, 1132, 1135, 1115,
    1116, 1117, 232, 233, 1121, 1122, 1123, 1124, 1125, 1129, 1126, 1127, 1128, 1133, 1134, 1136,
    1137, 1139, 1140, 1141, 1142, 1144, 1145, 1146, 1147, 1148, 1149, 1163, 1164, 1165, 1166, 1180,
    1181, 1182, 1189, 1150, 1151, 232, 233, 1152, 58, 1153, 1162, 1154, 1155, 1156, 1157, 1158,
    1159, 1160, 1161, 1167, 1172, 1168, 232, 233, 1169, 58, 1170, 1171, 1173, 1174, 1175, 1176,
    232, 233, 1177, 58, 1178, 1179, 1183, 1184, 1187, 1188, 1185, 1186, 1191, 1192, 1193, 1194,
    1195, 1196, 1197, 1198, 1201, 1199, 1200, 1203, 1204, 1205, 1206, 1207, 1208, 1209, 1210, 1211,
    1212, 1213, 232, 233, 1214, 58, 1217, 1218, 1219, 1220, 1221, 1222, 1234, 1235, 1223, 1224,
    1225, 232, 233, 1226, 58, 1227, 1228, 1229, 1230, 1231, 1232, 598, 599, 1233, 58, 1237,
    1238, 1239, 1240, 1241, 1249, 1250, 1256, 1270, 1242, 232, 233, 1243, 58, 1244, 1245, 1246,
    1247, 1248, 1257, 1268, 1269, 1258, 1259, 1260, 1261, 1262, 1263, 1264, 1265, 1266, 1267, 1272,
    1273, 1274, 1275, 1277, 1278, 1279, 1280, 1281, 1282, 1285, 1288, 1283, 1284, 1286, 1287, 1290,
    1294, 1298, 1299, 1300, 1305, 1308, 1291, 1292, 1293, 1295, 1296, 1297, 1301, 1304, 1302, 1303,
    1306, 1307, 1310, 1311, 1312, 1313, 1314, 1316, 1317, 1323, 1324, 1325, 1326, 1337, 1347, 1348,
    1354, 1357, 1358, 1318, 1319, 1322, 1320, 1321, 1327, 1334, 1328, 1329, 1330, 1333, 1331, 1332,
    1335, 1336, 1338, 1346, 1339, 232, 233, 1340, 58, 1341, 1342, 1343, 1344, 1345, 1349, 1350,
    1351, 1352, 1353, 1360, 1361, 1362, 1363, 1364, 1369, 1365, 1366, 1367, 1368, 1371, 1372, 1378,
    1379, 1380, 1381, 1395, 1402, 1373, 1374, 1377, 1375, 1376, 1382, 1383, 232, 233, 1384, 58,
    1385, 1394, 1386, 1387, 1388, 1389, 1390, 1391, 1392, 1393, 1396, 1397, 1398, 1399, 1400, 1401,
    1403, 1410, 1404, 1405, 1406, 1409, 1407, 1408, 1411, 1412, 1414, 1415, 1416, 1417, 1423, 1436,
    1437, 1438, 1475, 1418, 1419, 1420, 1421, 1422, 1424, 1434, 1425, 1426, 1427, 1428, 1429, 1430,
    1431, 58, 1439, 1449, 1452, 1465, 1471, 1440, 1441, 1442, 1443, 1444, 1445, 1446, 1447, 1448,
    1450, 1451, 1453, 1456, 1454, 1455, 1457, 1458, 1459, 1462, 1460, 1461, 1463, 1464, 1466, 1467,
    1468, 1469, 1470, 1472, 1473, 1474, 232, 233, 1476, 1477, 1478, 1479, 1481, 1482, 1483, 1485,
    1486, 1487, 1488, 1489, 1490, 1491, 1492, 1495, 1493, 1494, 1494, 232, 233, 1505, 1508, 1506,
    1507, 1509, 232, 233, 1532, 1533, 1534, 1536, 1537, 1538, 1539, 1540, 1541, 1544, 1547, 1550,
    1551, 1552, 1553, 1554, 1555, 1556, 1557, 1558, 1565, 1571, 1572, 1576, 1579, 1586, 1587, 1593,
    1542, 1543, 1545, 1546, 1548, 1549, 1559, 1564, 1560, 1561, 1562, 1563, 1566, 1569, 1567, 1568,
    1570, 232, 233, 1573, 1574, 1575, 1577, 1578, 1580, 1581, 1584, 1585, 1582, 1583, 1588, 1591,
    1592, 1589, 1590, 598, 599, 1594, 1609, 1595, 1604, 1596, 1599, 1597, 1598, 1600, 1601, 1602,
    1603, 1605, 1608, 1606, 1607, 1614, 1615, 232, 233, 1616, 58, 1617, 1618, 1620, 1625, 1626,
    1621, 1622, 1623, 1624, 1653, 1654, 1666, 1667, 1668, 1685, 1686, 1687, 1705, 1706, 1720, 1721,
    1723, 1724, 1725, 1729, 1726, 1727, 1728, 232, 233, 1731, 1732, 1733, 1734, 1735, 1737, 1738,
    1739, 1740, 1742, 1743, 1744, 1745, 1746, 1749, 1750, 1751, 1752, 1754, 1755, 1756, 1757, 1759,
    1758, 58, 1761, 1762, 1763, 1764, 1766, 1767, 1768, 1769, 1770, 1772, 1773, 1774, 1776, 1777,
    1778, 1779, 1780, 1782, 1781, 58, 1784, 1785, 1786, 1787, 1789, 1790, 1791, 1792, 1793, 1795,
    1796, 1797, 1798, 1800, 1801, 1802, 1803, 1804, 1815, 729, 1816, 238, 232, 233, 1821, 1822,
    1829, 1865, 1866, 1867, 1868, 1869, 1870, 1874, 1875, 1876, 1877, 1878, 1881, 1882, 1886, 1887,
    1891, 1892, 1893, 1879, 1880, 1883, 1884, 1885, 1888, 1890, 1889, 1894, 1895, 1896, 1897, 1898,
    1900, 1903, 1904, 1905, 1901, 1902, 1907, 1908, 1909, 1910, 1911, 1915, 1912, 1913, 1914, 1917,
    1920, 1918, 1919, 1922, 1926, 1932, 1923, 1924, 1925, 232, 233, 1927, 1930, 1928, 1929, 1931,
    1935, 1970, 1936, 1937, 1938, 1950, 1951, 1952, 1959, 1967, 1968, 1939, 232, 233, 1940, 1941,
    1942, 1943, 1944, 58, 1945, 1946, 1947, 1948, 1949, 1953, 1956, 1954, 1955, 1957, 232, 233,
    1958, 58, 1960, 1961, 1962, 1963, 1964, 1965, 1966, 1972, 1973, 1974, 1975, 1976, 1977, 1978,
    1992, 1993, 1996, 2000, 2003, 1994, 1995, 1997, 1998, 1999, 2001, 2002, 2004, 2005, 2006, 2007,
    2012, 2014, 2018, 2013, 2015, 2016, 2017, 2019, 2024, 2025, 2026, 232, 233, 2027, 2028, 2044,
    2045, 2052, 2053, 2054, 2076, 2055, 2058, 2056, 2057, 2059, 2073, 2060, 2061, 2072, 2062, 2066,
    2063, 2064, 2065, 2067, 2068, 2071, 2069, 2070, 2074, 2075, 2077, 2054, 2078, 2054, 2080, 2081,
    2087, 2088, 2089, 58, 2091, 2092, 2093, 2096, 2099, 2100, 2101, 2102, 2103, 2104, 2105, 2106,
    2094, 2095, 2097, 2098, 2108, 2126, 2109, 2110, 2113, 2114, 2120, 2124, 2125, 2111, 2112, 2112,
    232, 233, 2115, 2116, 2117, 2118, 2119, 2121, 2122, 2123, 2127, 232, 233, 2108, 2128, 2129,
    2131, 2132, 2132, 232, 233, 729, 2133, 238, 2144, 2145, 2156, 2134, 2135, 2136, 2137, 232,
    233, 2139, 2140, 2142, 2143, 2146, 2147, 2148, 2149, 2151, 2152, 2154, 2155, 2161, 2162, 2164,
    2165, 2168, 2174, 2180, 2191, 2197, 2198, 2201, 2213, 2231, 2232, 2166, 2167, 2167, 232, 233,
    2169, 2170, 2171, 2172, 2173, 2175, 2176, 2178, 2179, 2177, 2181, 2184, 2182, 2183, 2185, 2188,
    2186, 2187, 2189, 2190, 2192, 2193, 2194, 2195, 2196, 58, 2199, 2200, 2202, 2203, 2207, 2208,
    2209, 2204, 2205, 2206, 2210, 2211, 2212, 2214, 2215, 2219, 2230, 2216, 2217, 2218, 2220, 2221,
    2222, 2223, 58, 2224, 2225, 2226, 2227, 2228, 2229, 2234, 2235, 232, 233, 2237, 2238, 2239,
    2240, 2241, 2245, 2250, 2246, 2247, 2248, 2251, 2252, 2255, 2262, 2256, 2257, 2258, 2261, 2259,
    2260, 2263, 2266, 2264, 2265, 2268, 232, 233, 2269, 58, 2271, 2275, 2276, 2272, 2273, 2274,
    2277, 2284, 2278, 2283, 2279, 2280, 2281, 2282, 2285, 2304, 2305, 2319, 2320, 2321, 2334, 2306,
    2310, 2313, 2318, 2307, 2308, 2309, 2311, 2312, 2314, 2315, 2316, 2317, 2322, 2326, 2330, 2323,
    2324, 2325, 2327, 2328, 2329, 2331, 2332, 2333, 2335, 2336, 2337, 2338, 2339, 2340, 2344, 2345,
    2349, 2350, 225, 2351, 238, 2352, 2353, 58, 2354, 2355, 2356, 2359, 2362, 2363, 2364, 2365,
    2366, 2360, 2361, 2370, 2371, 58, 2372, 2373, 2374, 2375, 232, 233, 2379, 2380, 2381, 2382,
    232, 233, 2383, 58, 2384, 2385, 2390, 729, 2391, 238, 2392, 2393, 2393, 232, 233, 2395,
    2398, 2399, 2400, 2401, 2396, 2397, 2436, 2437, 2440, 2441, 2442, 2445, 2448, 2446, 2447, 2458,
    2459, 2460, 2461, 2466, 2462, 232, 233, 2463, 2465, 2464, 58, 2471, 2472, 2473, 2474, 2490,
    2491, 2492, 2493, 2494, 2495, 2496, 2499, 2497, 2498, 2517, 2518, 2521, 2519, 2520, 2526, 2527,
    2528, 2533, 2534, 2529, 2530, 2531, 2532, 2535, 2536, 2537, 2538, 2541, 2542, 2543, 2544, 2545,
    2546, 2548, 2549, 2550, 2551, 2552, 2553, 2559, 2560, 2561, 2562, 2563, 2565, 2566, 2567, 2568,
    2576, 2589, 2594, 2577, 2578, 225, 2579, 2580, 238, 2582, 2581, 2583, 2584, 2585, 2588, 2586,
    2587, 2590, 2591, 225, 2592, 2593, 238, 2595, 225, 2596, 238, 2599, 2602, 2603, 2604, 2607,
    2605, 2606, 2606, 232, 233, 2609, 2610, 2611, 2612, 2615, 2616, 2622, 2613, 2614, 2617, 2621,
    2618, 2619, 2620, 2623, 2624, 2625, 2626, 2627, 2631, 2628, 2629, 2630, 2633, 225, 2634, 238,
    2635, 232, 233, 2636, 58, 2648, 2649, 2653, 2658, 2659, 2654, 2655, 2656, 232, 233, 2657,
    58, 2675, 2676, 2681, 2684, 2685, 2682, 2683, 2686, 2687, 2689, 2690, 2691, 2695, 2696, 2701,
    2697, 2700, 2698, 2699, 2704, 225, 2705, 2711, 238, 2706, 2707, 2707, 232, 233, 2708, 2709,
    2710, 2712, 2717, 2720, 2713, 2714, 2715, 2716, 2718, 2719, 2727, 2728, 2729, 2730, 2731, 2732,
    2733, 2734, 2735, 2762, 2763, 2764, 2765, 2766, 2769, 232, 233, 2767, 2768, 2770, 2771, 2772,
    232, 233, 2803, 225, 2804, 238, 2805, 2807, 2808, 2811, 2812, 2814, 2815, 2816, 2819, 225,
    2820, 238, 2817, 2818, 2821, 2822, 2824, 2825, 2826, 2827, 2830, 225, 2831, 238, 2828, 2829,
    2833, 225, 2834, 238, 2837, 225, 2838, 238, 2848, 2849, 2850, 2851, 2835, 2836, 2839, 2840,
    2841, 2845, 2842, 2844, 2843, 232, 233, 2846, 2847, 2853, 225, 2854, 238, 2897, 2898, 2899,
    2900, 2901, 2902, 232, 233, 2904, 225, 2905, 238, 2906, 58, 2914, 2915, 2942, 2943, 2944,
    2946, 2948, 2950, 2945, 232, 233, 2947, 232, 233, 2949, 232, 233, 2953, 2954, 2955, 2956,
    2958, 2959, 2962, 2960, 2961, 2961, 232, 233, 2972, 2975, 2973, 2974, 2976, 2977, 3002, 3003,
    3004, 3006, 3008, 1832, 3007, 58, 3010, 3011, 3012, 3013, 3015, 3016, 3017, 3018, 3019, 3020,
    3021, 3022, 3023, 3024, 3025, 3026, 697, 3034, 3035, 3039, 3036, 3037, 3038, 3040, 3041, 3067,
    3068, 3071, 3072, 225, 3073, 238, 3074, 58, 3075, 3079, 3082, 3087, 3076, 3077, 3078, 3080,
    3081, 3083, 3084, 3085, 3086, 3091, 3092, 3093, 3094, 3095, 3096, 3097, 3100, 3101, 225, 3110,
    3111, 3112, 238, 3113, 3114, 3115, 232, 233, 3118, 3119, 3120, 3125, 3121, 3122, 3123, 3124,
    232, 233, 3148, 3149, 3150, 3169, 3170, 3151, 3158, 3152, 58, 3153, 3154, 3155, 3156, 3157,
    3159, 232, 233, 3160, 58, 3161, 3162, 3163, 3164, 3166, 3167, 3172, 3173, 3174, 3180, 3181,
    3182, 3183, 3175, 225, 3176, 238, 3177, 3178, 3178, 232, 233, 3179, 58, 3184, 3185, 3206,
    3211, 3207, 3208, 3209, 3210, 3235, 3236, 3237, 3253, 3254, 3273, 3274, 3277, 3278, 3282, 3279,
    3280, 3281, 3283, 232, 233, 3290, 3292, 3291, 3293, 3315, 3316, 3317, 3318, 3319, 3320, 3324,
    3321, 3322, 3323, 3325, 3326, 3327, 3329, 3330, 3331, 3332, 3333, 3338, 3339, 3340, 3343, 3346,
    3341, 3342, 3344, 3345, 3347, 3348, 3396, 3397, 3401, 3415, 3416, 3417, 3398, 3399, 3400, 3402,
    3406, 3409, 3414, 3403, 3404, 3405, 3407, 3408, 3410, 3411, 3412, 3413, 3449, 3450, 3453, 3455,
    3451, 3452, 3454, 232, 233, 3456, 3458, 3457, 3459, 232, 233, 3466, 3470, 3471, 3467, 3468,
    3469, 3490, 3491, 3495, 3492, 3493, 3494, 3496, 3497, 3498, 3508, 3509, 3499, 3500, 3501, 232,
    233, 3502, 58, 3503, 3504, 3505, 3506, 3507, 3512, 3513, 3514, 3515, 3517, 3518, 3520, 3521,
    225, 3522, 238, 3523, 58, 3526, 3528, 3529, 3532, 3533, 3534, 3535, 3527, 3530, 3531, 3538,
    3539, 3540, 3541, 3542, 3543, 3544, 3545, 3546, 3547, 3548, 3549, 3554, 3557, 3560, 3561, 3555,
    3556, 3556, 232, 233, 3558, 3559, 3559, 232, 233, 3562, 232, 233, 3571, 3574, 3572, 3573,
    3575, 3576, 3578, 3581, 3579, 3580, 3582, 3583, 3592, 225, 3593, 238, 3594, 58, 3596, 3597,
    3598, 3599, 3600, 3601, 3602, 3603, 3604, 3605, 3607, 3608, 3612, 3613, 3614, 3609, 3610, 3611,
    3617, 3618, 3623, 3619, 3622, 3620, 3621, 3625, 3626, 3627, 3628, 3630, 3631, 3632, 3641, 3650,
    3663, 3670, 3678, 3633, 3639, 3640, 3634, 3635, 3636, 3637, 3638, 3642, 3643, 3646, 3644, 3645,
    3647, 3649, 3648, 58, 3651, 3652, 3653, 3654, 3655, 3661, 3656, 3657, 3658, 3659, 3660, 58,
    3662, 58, 3664, 3665, 3666, 3667, 3668, 3669, 3671, 3672, 3673, 3674, 3675, 3676, 3677, 3679,
    3680, 3681, 3686, 3692, 3697, 3708, 3718, 3724, 3682, 3683, 3684, 3685, 3687, 3688, 3689, 3690,
    3691, 3693, 3694, 3695, 3696, 3698, 3702, 3699, 3700, 3701, 3703, 3704, 3707, 3705, 3706, 3709,
    3712, 3710, 3711, 3713, 3714, 3717, 3715, 3716, 3719, 3720, 3721, 3722, 3723, 3725, 3726, 3727,
    3728, 3729, 3730, 3739, 3742, 3740, 3741, 3743, 3744, 3747, 3748, 3745, 3746, 3754, 3755, 3759,
    3760, 3763, 58, 3765, 3766, 3767, 3774, 3775, 3822, 3823, 3824, 3825, 3826, 3827, 3828, 3829,
    3830, 3831, 3832, 3833, 3834, 3835, 3836, 3837, 3838, 3839, 3840, 3841, 3842, 3843, 3844, 3845,
    3846, 3847, 3848, 3849, 3850, 3851, 3852, 3853, 3854, 3855, 3856, 3857, 3858, 3859, 3860, 3861,
    3862, 3863, 3864, 3865, 3866, 3867, 3868, 3869, 3870, 3871, 3872, 3873, 3890, 3893, 232, 233,
    3894, 58, 3895, 3896, 3919, 3922, 3920, 3921, 3923, 3929, 3936, 3942, 3924, 3925, 3926, 3927,
    3928, 3930, 3931, 3933, 3932, 232, 233, 3934, 3935, 3937, 3938, 3939, 3940, 3941, 3943, 3944,
    3945, 3946, 3949, 3950, 3951, 3952, 3953, 3955, 3954, 232, 233, 3956, 232, 233, 3963, 3966,
    3964, 3968, 3970, 3969, 3977, 3978, 3981, 3982, 3995, 3998, 3999, 3996, 3997, 4000, 4003, 4004,
    4001, 4002, 4006, 4007, 4008, 4009, 4010, 4012, 4013, 4014, 4015, 225, 4016, 238, 4041, 4047,
    4050, 4053, 4042, 4043, 4044, 4045, 4046, 4048, 4049, 4051, 4052, 4054, 4055, 4056, 4059, 4060,
    4073, 4061, 4064, 4067, 4070, 4062, 4063, 4065, 4066, 4068, 4069, 4071, 4072, 4090, 4091, 4092,
    4097, 4098, 4099, 4104, 232, 233, 4105, 58, 4108, 4109, 4111, 4112, 4122, 4123, 4124, 4131,
    4132, 4133, 4134, 4137, 4147, 4148, 4149, 4150, 4151, 4152, 4153, 4161, 4162, 4163, 4164, 4165,
    4171, 225, 4172, 4173, 238, 4175, 4176, 4177, 4174, 4183, 4184, 4205, 4208, 4206, 4207, 4209,
    58, 4210, 4211, 4212, 232, 233, 4213, 58, 4214, 4215, 4216, 4243, 4244, 4246, 4247, 4248,
    4249, 4250, 4251, 729, 4245, 238, 4255, 729, 4256, 238, 4257, 4258, 4258, 232, 233, 4260,
    4261, 4264, 4267, 4268, 4269, 4270, 4271, 4262, 4263, 4265, 4266, 4277, 4279, 4287, 4280, 4281,
    4284, 4282, 4283, 4285, 4286, 4290, 4294, 4295, 4296, 4291, 4292, 4293, 4298, 4299, 4300, 4301,
    4304, 4305, 4302, 58, 4307, 4308, 4309, 4310, 4313, 4314, 4311, 58, 4316, 4317, 4319, 4318,
    4323, 4324, 4325, 4326, 4328, 4329, 4330, 4331, 232, 233,
];

pub static TERMINATORS: &[u32] = &[
    59, 60, 136, 267, 271, 291, 646, 647, 732, 733, 735, 742, 743, 745, 747, 748,
    749, 795, 859, 1251, 1252, 1253, 1254, 1255, 1355, 1356, 1432, 1433, 1435, 1933, 1969, 2138,
    2141, 2150, 2153, 2249, 2253, 2357, 2376, 3088, 3089, 3165, 3168, 3524, 3550, 3551, 3552, 3761,
    3874, 3965, 4154, 4155, 4156, 4157, 4158, 4159, 4303, 4312,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABS", // [6]
    "ABSOLUTE", // [7]
    "ACCESS", // [8]
    "AccountKeywordSegment", // [9]
    "ACCOUNT", // [10]
    "ALL", // [11]
    "CATALOG", // [12]
    "DATABASE", // [13]
    "DOMAIN", // [14]
    "EXTERNAL", // [15]
    "FILE", // [16]
    "FOREIGN", // [17]
    "FUNCTION", // [18]
    "FUTURE", // [19]
    "INTEGRATION", // [20]
    "LANGUAGE", // [21]
    "MASKING", // [22]
    "MATERIALIZED", // [23]
    "PROCEDURE", // [24]
    "RESOURCE", // [25]
    "ROLE", // [26]
    "ROUTINE", // [27]
    "SCHEMA", // [28]
    "SEQUENCE", // [29]
    "STAGE", // [30]
    "STREAM", // [31]
    "TABLE", // [32]
    "TABLESPACE", // [33]
    "TASK", // [34]
    "TYPE", // [35]
    "VIEW", // [36]
    "WAREHOUSE", // [37]
    "ResourceKeywordSegment", // [38]
    "MonitorKeywordSegment", // [39]
    "MONITOR", // [40]
    "WarehouseKeywordSegment", // [41]
    "DatabaseKeywordSegment", // [42]
    "DomainKeywordSegment", // [43]
    "IntegrationKeywordSegment", // [44]
    "LanguageKeywordSegment", // [45]
    "SchemaKeywordSegment", // [46]
    "CatalogKeywordSegment", // [47]
    "RoleKeywordSegment", // [48]
    "TablespaceKeywordSegment", // [49]
    "TypeKeywordSegment", // [50]
    "ForeignKeywordSegment", // [51]
    "DATA", // [52]
    "SERVER", // [53]
    "ServerKeywordSegment", // [54]
    "DataKeywordSegment", // [55]
    "WrapperKeywordSegment", // [56]
    "WRAPPER", // [57]
    "AllKeywordSegment", // [58]
    "SchemasKeywordSegment", // [59]
    "SCHEMAS", // [60]
    "InKeywordSegment", // [61]
    "IN", // [62]
    "FutureKeywordSegment", // [63]
    "AccessSchemaObjectSegment", // [64]
    "access_schema_object", // [65]
    "AccessSchemaPluralObjectSegment", // [66]
    "FUNCTIONS", // [67]
    "PROCEDURES", // [68]
    "ROUTINES", // [69]
    "SEQUENCES", // [70]
    "STAGES", // [71]
    "STREAMS", // [72]
    "TABLES", // [73]
    "TASKS", // [74]
    "VIEWS", // [75]
    "access_schema_plural_object", // [76]
    "ObjectReferenceSegment", // [77]
    "object_reference", // [78]
    "FunctionNameSegment", // [79]
    "function_name", // [80]
    "FunctionParameterListGrammar", // [81]
    "(", // [82]
    "function_parameter_list", // [83]
    "WildcardIdentifierSegment", // [84]
    "wildcard_identifier", // [85]
    "CommaSegment", // [86]
    ",", // [87]
    "ToKeywordSegment", // [88]
    "TO", // [89]
    "FromKeywordSegment", // [90]
    "FROM", // [91]
    "LARGE", // [92]
    "LargeKeywordSegment", // [93]
    "ObjectKeywordSegment", // [94]
    "OBJECT", // [95]
    "NumericLiteralSegment", // [96]
    "numeric_literal", // [97]
    "APPLY", // [98]
    "CONNECT", // [99]
    "CREATE", // [100]
    "DELETE", // [101]
    "EXECUTE", // [102]
    "IMPORTED", // [103]
    "INSERT", // [104]
    "MANAGE", // [105]
    "MODIFY", // [106]
    "OPERATE", // [107]
    "OWNERSHIP", // [108]
    "READ", // [109]
    "REFERENCES", // [110]
    "REFERENCE_USAGE", // [111]
    "SELECT", // [112]
    "TEMP", // [113]
    "TEMPORARY", // [114]
    "TRIGGER", // [115]
    "TRUNCATE", // [116]
    "UPDATE", // [117]
    "USAGE", // [118]
    "USE", // [119]
    "USE_ANY_ROLE", // [120]
    "WRITE", // [121]
    "CreateKeywordSegment", // [122]
    "USER", // [123]
    "UserKeywordSegment", // [124]
    "ApplyKeywordSegment", // [125]
    "MaskingKeywordSegment", // [126]
    "PolicyKeywordSegment", // [127]
    "POLICY", // [128]
    "ExecuteKeywordSegment", // [129]
    "TaskKeywordSegment", // [130]
    "ManageKeywordSegment", // [131]
    "GrantsKeywordSegment", // [132]
    "GRANTS", // [133]
    "EXECUTION", // [134]
    "ExecutionKeywordSegment", // [135]
    "UsageKeywordSegment", // [136]
    "PIPE", // [137]
    "PipeKeywordSegment", // [138]
    "UseKeywordSegment", // [139]
    "ImportedKeywordSegment", // [140]
    "PrivilegesKeywordSegment", // [141]
    "PRIVILEGES", // [142]
    "ConnectKeywordSegment", // [143]
    "DeleteKeywordSegment", // [144]
    "InsertKeywordSegment", // [145]
    "ModifyKeywordSegment", // [146]
    "OperateKeywordSegment", // [147]
    "OwnershipKeywordSegment", // [148]
    "ReadKeywordSegment", // [149]
    "Reference_usageKeywordSegment", // [150]
    "ReferencesKeywordSegment", // [151]
    "SelectKeywordSegment", // [152]
    "TempKeywordSegment", // [153]
    "TemporaryKeywordSegment", // [154]
    "TriggerKeywordSegment", // [155]
    "TruncateKeywordSegment", // [156]
    "UpdateKeywordSegment", // [157]
    "Use_any_roleKeywordSegment", // [158]
    "WriteKeywordSegment", // [159]
    "BracketedColumnReferenceListGrammar", // [160]
    "AccessPermissionSegment", // [161]
    "access_permission", // [162]
    "OnKeywordSegment", // [163]
    "ON", // [164]
    "TableKeywordSegment", // [165]
    "ViewKeywordSegment", // [166]
    "StageKeywordSegment", // [167]
    "FunctionKeywordSegment", // [168]
    "ProcedureKeywordSegment", // [169]
    "RoutineKeywordSegment", // [170]
    "SequenceKeywordSegment", // [171]
    "StreamKeywordSegment", // [172]
    "MaterializedKeywordSegment", // [173]
    "ExternalKeywordSegment", // [174]
    "FileKeywordSegment", // [175]
    "FormatKeywordSegment", // [176]
    "FORMAT", // [177]
    "TablesKeywordSegment", // [178]
    "ViewsKeywordSegment", // [179]
    "StagesKeywordSegment", // [180]
    "FunctionsKeywordSegment", // [181]
    "ProceduresKeywordSegment", // [182]
    "RoutinesKeywordSegment", // [183]
    "SequencesKeywordSegment", // [184]
    "StreamsKeywordSegment", // [185]
    "TasksKeywordSegment", // [186]
    "PoliciesKeywordSegment", // [187]
    "FormatsKeywordSegment", // [188]
    "GRANT", // [189]
    "REVOKE", // [190]
    "GrantStatementSegment", // [191]
    "grant_statement", // [192]
    "RevokeStatementSegment", // [193]
    "revoke_statement", // [194]
    "GROUP", // [195]
    "GroupKeywordSegment", // [196]
    "UserReferenceSegment", // [197]
    "user_reference", // [198]
    "RoleReferenceSegment", // [199]
    "role_reference", // [200]
    "SHARE", // [201]
    "ShareKeywordSegment", // [202]
    "FunctionSegment", // [203]
    "function", // [204]
    "PublicKeywordSegment", // [205]
    "PUBLIC", // [206]
    "[", // [207]
    "ArrayAccessorSegment", // [208]
    "array_accessor", // [209]
    "ACCESSRANK", // [210]
    "ACCOUNTS", // [211]
    "ACTION", // [212]
    "ACTIVATE", // [213]
    "ACTIVEPARTITIONCOUNT", // [214]
    "ADA", // [215]
    "ADD", // [216]
    "ADDRESS", // [217]
    "ADMIN", // [218]
    "AFTER", // [219]
    "AGGREGATE", // [220]
    "OrderByClauseSegment", // [221]
    "ORDER", // [222]
    "orderby_clause", // [223]
    "indent", // [224]
    "Indent", // [225]
    "meta", // [226]
    "AsAliasOperatorSegment", // [227]
    "AS", // [228]
    "alias_operator", // [229]
    "SingleIdentifierGrammar", // [230]
    "SingleIdentifierListSegment", // [231]
    "identifier_list", // [232]
    "start_bracket", // [233]
    "SymbolSegment", // [234]
    "symbol", // [235]
    ")", // [236]
    "end_bracket", // [237]
    "SingleQuotedIdentifierSegment", // [238]
    "single_quote", // [239]
    "dedent", // [240]
    "Dedent", // [241]
    "ColumnReferenceSegment", // [242]
    "column_reference", // [243]
    "ALIAS", // [244]
    "TableReferenceSegment", // [245]
    "table_reference", // [246]
    "AliasExpressionSegment", // [247]
    "alias_expression", // [248]
    "ALLOCATE", // [249]
    "ALSO", // [250]
    "GrantKeywordSegment", // [251]
    "AlterDefaultPrivilegesObjectPrivilegesSegment", // [252]
    "ALTER", // [253]
    "DROP", // [254]
    "alter_default_privileges_object_privilege", // [255]
    "SchemaReferenceSegment", // [256]
    "schema_reference", // [257]
    "WithKeywordSegment", // [258]
    "WITH", // [259]
    "OptionKeywordSegment", // [260]
    "OPTION", // [261]
    "ExtendKeywordSegment", // [262]
    "EXTEND", // [263]
    "AlterKeywordSegment", // [264]
    "DropKeywordSegment", // [265]
    "DEFAULT", // [266]
    "DISK_QUOTA", // [267]
    "OWNER", // [268]
    "RENAME", // [269]
    "DefaultKeywordSegment", // [270]
    "SchemaPrivilegesSegment", // [271]
    "EXCLUDE", // [272]
    "INCLUDE", // [273]
    "MATERIALIZE", // [274]
    "schema_privileges_segment", // [275]
    "RenameKeywordSegment", // [276]
    "OwnerKeywordSegment", // [277]
    "CascadeKeywordSegment", // [278]
    "CASCADE", // [279]
    "DiskQuotaSegment", // [280]
    "disk_quota_segment", // [281]
    "CACHE", // [282]
    "CYCLE", // [283]
    "INCREMENT", // [284]
    "MAXVALUE", // [285]
    "MINVALUE", // [286]
    "NO", // [287]
    "NOCACHE", // [288]
    "NOCYCLE", // [289]
    "NOORDER", // [290]
    "IncrementKeywordSegment", // [291]
    "ByKeywordSegment", // [292]
    "BY", // [293]
    "SequenceMinValueGrammar", // [294]
    "SequenceMaxValueGrammar", // [295]
    "CacheKeywordSegment", // [296]
    "NocacheKeywordSegment", // [297]
    "CycleKeywordSegment", // [298]
    "NocycleKeywordSegment", // [299]
    "OrderNoOrderGrammar", // [300]
    "SequenceReferenceSegment", // [301]
    "sequence_reference", // [302]
    "AlterSequenceOptionsSegment", // [303]
    "alter_sequence_options_segment", // [304]
    "SessionKeywordSegment", // [305]
    "SESSION", // [306]
    "CLEAR", // [307]
    "SET", // [308]
    "SetKeywordSegment", // [309]
    "ParameterKeywordSegment", // [310]
    "PARAMETER", // [311]
    "ParameterNameSegment", // [312]
    "EqualsSegment", // [313]
    "=", // [314]
    "comparison_operator", // [315]
    "QuotedLiteralSegment", // [316]
    "escaped_single_quote", // [317]
    "ClearKeywordSegment", // [318]
    "UdparameterKeywordSegment", // [319]
    "UDPARAMETER", // [320]
    "FOR", // [321]
    "ForKeywordSegment", // [322]
    "FORCE", // [323]
    "AddKeywordSegment", // [324]
    "TableConstraintSegment", // [325]
    "CHECK", // [326]
    "CONSTRAINT", // [327]
    "PRIMARY", // [328]
    "UNIQUE", // [329]
    "table_constraint", // [330]
    "ConstraintKeywordSegment", // [331]
    "DISABLED", // [332]
    "ENABLED", // [333]
    "EnabledKeywordSegment", // [334]
    "DisabledKeywordSegment", // [335]
    "ForceKeywordSegment", // [336]
    "OuterKeywordSegment", // [337]
    "OUTER", // [338]
    "IntegerSegment", // [339]
    "IMMUTABLE", // [340]
    "MERGEOUT", // [341]
    "ActivepartitioncountKeywordSegment", // [342]
    "ImmutableKeywordSegment", // [343]
    "RowsKeywordSegment", // [344]
    "ROWS", // [345]
    "MergeoutKeywordSegment", // [346]
    "1KeywordSegment", // [347]
    "2KeywordSegment", // [348]
    "ColumnKeywordSegment", // [349]
    "COLUMN", // [350]
    "IfExistsGrammar", // [351]
    "IF", // [352]
    "LiteralGrammar", // [353]
    "+", // [354]
    "-", // [355]
    "ARRAY", // [356]
    "DATE", // [357]
    "FALSE", // [358]
    "INTERVAL", // [359]
    "NULL", // [360]
    "TIME", // [361]
    "TIMESTAMP", // [362]
    "TRUE", // [363]
    "{", // [364]
    "NakedIdentifierSegment", // [365]
    "ColumnDefinitionSegment", // [366]
    "column_definition", // [367]
    "FIRST", // [368]
    "FirstKeywordSegment", // [369]
    "AfterKeywordSegment", // [370]
    "AlterTableDropColumnGrammar", // [371]
    "AsKeywordSegment", // [372]
    "PARTITION", // [373]
    "REMOVE", // [374]
    "REORGANIZE", // [375]
    "AlterTableActionSegment", // [376]
    "alter_table_action_segment", // [377]
    "IfNotExistsGrammar", // [378]
    "DatatypeSegment", // [379]
    "data_type", // [380]
    "NOT", // [381]
    "USING", // [382]
    "ColumnConstraintSegment", // [383]
    "column_constraint_segment", // [384]
    "ColumnEncodingSegment", // [385]
    "ENCODING", // [386]
    "column_encoding", // [387]
    "PROJECTIONS", // [388]
    "ProjectionsKeywordSegment", // [389]
    "ColumnSetSegment", // [390]
    "column_set_segment", // [391]
    "NotKeywordSegment", // [392]
    "NullKeywordSegment", // [393]
    "UsingKeywordSegment", // [394]
    "RESTRICT", // [395]
    "RestrictKeywordSegment", // [396]
    "DropBehaviorGrammar", // [397]
    "PartitionByClauseSegment", // [398]
    "partitionby_clause", // [399]
    "RemoveKeywordSegment", // [400]
    "PartitioningKeywordSegment", // [401]
    "PARTITIONING", // [402]
    "ReorganizeKeywordSegment", // [403]
    "ALWAYS", // [404]
    "&", // [405]
    "ampersand", // [406]
    "ANALYSE", // [407]
    "ANALYTIC", // [408]
    "ANALYZE", // [409]
    "AND", // [410]
    "binary_operator", // [411]
    "BinaryOperatorSegment", // [412]
    "ANNOTATED", // [413]
    "ANTI", // [414]
    "ANY", // [415]
    "ARCHIVE", // [416]
    "ARE", // [417]
    "%", // [418]
    "*", // [419]
    "/", // [420]
    "//", // [421]
    "<", // [422]
    ">", // [423]
    "^", // [424]
    "|", // [425]
    "PlusSegment", // [426]
    "MinusSegment", // [427]
    "DivideSegment", // [428]
    "IntegerDivideSegment", // [429]
    "MultiplySegment", // [430]
    "ModuloSegment", // [431]
    "BitwiseAndSegment", // [432]
    "BitwiseOrSegment", // [433]
    "BitwiseXorSegment", // [434]
    "BitwiseLShiftSegment", // [435]
    "BitwiseRShiftSegment", // [436]
    "ExpressionSegment", // [437]
    "expression", // [438]
    "SliceSegment", // [439]
    ":", // [440]
    "start_square_bracket", // [441]
    "]", // [442]
    "end_square_bracket", // [443]
    "BaseExpressionElementGrammar", // [444]
    "ArrayKeywordSegment", // [445]
    "ASC", // [446]
    "ASENSITIVE", // [447]
    "ASSERTION", // [448]
    "ASSIGNMENT", // [449]
    "ASYMMETRIC", // [450]
    "AT", // [451]
    "ATOMIC", // [452]
    "ATTRIBUTE", // [453]
    "ATTRIBUTES", // [454]
    "AUDIT", // [455]
    "AUTHENTICATION", // [456]
    "AUTHORIZATION", // [457]
    "AUTO_INCREMENT", // [458]
    "Auto_incrementKeywordSegment", // [459]
    "AUTO", // [460]
    "AUTOCOMMIT", // [461]
    "AVAILABLE", // [462]
    "AVG", // [463]
    "AVG_ROW_LENGTH", // [464]
    "BACKUP", // [465]
    "BACKWARD", // [466]
    "BALANCE", // [467]
    "CURRENT_DATE", // [468]
    "CURRENT_TIME", // [469]
    "CURRENT_TIMESTAMP", // [470]
    "LOCALTIME", // [471]
    "LOCALTIMESTAMP", // [472]
    "SYSDATE", // [473]
    "bare_function", // [474]
    "CodeSegment", // [475]
    "BareFunctionSegment", // [476]
    "IntervalExpressionSegment", // [477]
    "interval_expression", // [478]
    "file", // [479]
    "BaseFileSegment", // [480]
    "BaseSegment", // [481]
    "BASENAME", // [482]
    "BATCH", // [483]
    "BEFORE", // [484]
    "BEGIN", // [485]
    "BERNOULLI", // [486]
    "BEST", // [487]
    "BETWEEN", // [488]
    "BIGINT", // [489]
    "BINARY", // [490]
    "!", // [491]
    "<=>", // [492]
    "IS", // [493]
    "OR", // [494]
    "like_operator", // [495]
    "ArithmeticBinaryOperatorGrammar", // [496]
    "StringBinaryOperatorGrammar", // [497]
    "BooleanBinaryOperatorGrammar", // [498]
    "ComparisonOperatorGrammar", // [499]
    "BINDING", // [500]
    "BIT", // [501]
    "BIT_LENGTH", // [502]
    "BITVAR", // [503]
    "AmpersandSegment", // [504]
    "RawLessThanSegment", // [505]
    "PipeSegment", // [506]
    "RawGreaterThanSegment", // [507]
    "BLOB", // [508]
    "BLOCK", // [509]
    "BLOCK_DICT", // [510]
    "BLOCKDICT_COMP", // [511]
    "BOOL", // [512]
    "AndOperatorGrammar", // [513]
    "OrOperatorGrammar", // [514]
    "BOOLEAN", // [515]
    "TrueSegment", // [516]
    "FalseSegment", // [517]
    "BOTH", // [518]
    "bracketed", // [519]
    "BracketedSegment", // [520]
    "UnorderedSetExpressionSegment", // [521]
    "VALUE", // [522]
    "VALUES", // [523]
    "set_expression", // [524]
    "BREADTH", // [525]
    "BREAK", // [526]
    "BROADCAST", // [527]
    "BROWSE", // [528]
    "BULK", // [529]
    "BUNDLE", // [530]
    "BYTEA", // [531]
    "BYTES", // [532]
    "BZIP", // [533]
    "BZIP_COMP", // [534]
    "CTEColumnList", // [535]
    "cte_column_list", // [536]
    "SelectableGrammar", // [537]
    "CA", // [538]
    "CALL", // [539]
    "CALLED", // [540]
    "CARDINALITY", // [541]
    "CASCADED", // [542]
    "CASE", // [543]
    "CaseKeywordSegment", // [544]
    "implicit_indent", // [545]
    "ImplicitIndent", // [546]
    "WHEN", // [547]
    "WhenClauseSegment", // [548]
    "when_clause", // [549]
    "ElseKeywordSegment", // [550]
    "ELSE", // [551]
    "EndKeywordSegment", // [552]
    "END", // [553]
    "ElseClauseSegment", // [554]
    "else_clause", // [555]
    "BinaryOperatorGrammar", // [556]
    "CAST", // [557]
    "::", // [558]
    "casting_operator", // [559]
    "CATALOG_NAME", // [560]
    "CATALOGPATH", // [561]
    "CEIL", // [562]
    "CEILING", // [563]
    "CERTIFICATE", // [564]
    "CERTIFICATES", // [565]
    "CHAIN", // [566]
    "CHANGE", // [567]
    "CHAR", // [568]
    "CHAR_LENGTH", // [569]
    "CHARACTER", // [570]
    "CHARACTER_LENGTH", // [571]
    "CHARACTER_SET_CATALOG", // [572]
    "CHARACTER_SET_NAME", // [573]
    "CHARACTER_SET_SCHEMA", // [574]
    "CHARACTERISTICS", // [575]
    "CHARACTERS", // [576]
    "CHECKED", // [577]
    "CHECKPOINT", // [578]
    "CHECKSUM", // [579]
    "CIPHER", // [580]
    "CLASS", // [581]
    "CLASS_ORIGIN", // [582]
    "CLOB", // [583]
    "CLOSE", // [584]
    "CLUSTER", // [585]
    "CLUSTERED", // [586]
    "COALESCE", // [587]
    "COBOL", // [588]
    "COLLATE", // [589]
    "COLLATION", // [590]
    "ObjectReferenceDelimiterGrammar", // [591]
    ".", // [592]
    "ObjectReferenceTerminatorGrammar", // [593]
    "COLLATION_CATALOG", // [594]
    "COLLATION_NAME", // [595]
    "COLLATION_SCHEMA", // [596]
    "COLLECT", // [597]
    "COLLECTIONCLOSE", // [598]
    "COLLECTIONDELIMITER", // [599]
    "COLLECTIONENCLOSE", // [600]
    "COLLECTIONNULLELEMENT", // [601]
    "COLLECTIONOPEN", // [602]
    "colon_delimiter", // [603]
    "colon_prefix", // [604]
    "colon", // [605]
    "COLSIZES", // [606]
    "ShorthandCastSegment", // [607]
    "cast_expression", // [608]
    "CheckKeywordSegment", // [609]
    "UniqueKeywordSegment", // [610]
    "PrimaryKeywordSegment", // [611]
    "KeyKeywordSegment", // [612]
    "KEY", // [613]
    "ReferenceDefinitionGrammar", // [614]
    "EncodingKeywordSegment", // [615]
    "EncodingType", // [616]
    "COMMONDELTA_COMP", // [617]
    "DELTARANGE_COMP", // [618]
    "DELTAVAL", // [619]
    "GCDDELTA", // [620]
    "GZIP_COMP", // [621]
    "RLE", // [622]
    "ZSTD_COMP", // [623]
    "ZSTD_FAST_COMP", // [624]
    "ZSTD_HIGH_COMP", // [625]
    "COLUMN_NAME", // [626]
    "ColumnsExpressionNameGrammar", // [627]
    "COLUMNS", // [628]
    "COLUMNS_COUNT", // [629]
    "comma", // [630]
    "COMMAND_FUNCTION", // [631]
    "COMMAND_FUNCTION_CODE", // [632]
    "COMMENT", // [633]
    "CommentKeywordSegment", // [634]
    "LIBRARY", // [635]
    "NODE", // [636]
    "PROJECTION", // [637]
    "TRANSFORM", // [638]
    "ProjectionKeywordSegment", // [639]
    "AggregateKeywordSegment", // [640]
    "AnalyticKeywordSegment", // [641]
    "TransformKeywordSegment", // [642]
    "NodeKeywordSegment", // [643]
    "LibraryKeywordSegment", // [644]
    "IsKeywordSegment", // [645]
    "comment", // [646]
    "CommentSegment", // [647]
    "COMMIT", // [648]
    "COMMITTED", // [649]
    "COMMUNAL", // [650]
    "NullEqualsSegment", // [651]
    "GreaterThanSegment", // [652]
    "LessThanSegment", // [653]
    "GreaterThanOrEqualToSegment", // [654]
    "LessThanOrEqualToSegment", // [655]
    "NotEqualToSegment", // [656]
    "LikeOperatorSegment", // [657]
    "IsDistinctFromGrammar", // [658]
    "ComparisonOperatorSegment", // [659]
    "COMPLETION", // [660]
    "COMPLEX", // [661]
    "CompositeBinaryOperatorSegment", // [662]
    "CompositeComparisonOperatorSegment", // [663]
    "COMPRESS", // [664]
    "GZIP", // [665]
    "LZO", // [666]
    "UNCOMPRESSED", // [667]
    "ZSTD", // [668]
    "compression_type", // [669]
    "COMPUTE", // [670]
    "CONDITION", // [671]
    "CONDITION_NUMBER", // [672]
    "CrossKeywordSegment", // [673]
    "CROSS", // [674]
    "JoinTypeKeywordsGrammar", // [675]
    "FULL", // [676]
    "INNER", // [677]
    "LEFT", // [678]
    "NATURAL", // [679]
    "NULLAWARE", // [680]
    "RIGHT", // [681]
    "SEMI", // [682]
    "SEMIALL", // [683]
    "ConditionalCrossJoinKeywordsGrammar", // [684]
    "NonStandardJoinTypeKeywordsGrammar", // [685]
    "CONFIGURATION", // [686]
    "CONNECTION", // [687]
    "CONNECTION_NAME", // [688]
    "CONSTRAINT_CATALOG", // [689]
    "CONSTRAINT_NAME", // [690]
    "CONSTRAINT_SCHEMA", // [691]
    "CONSTRAINTS", // [692]
    "CONSTRUCTOR", // [693]
    "CONTAINS", // [694]
    "CONTAINSTABLE", // [695]
    "CONTINUE", // [696]
    "CONTROL", // [697]
    "CONVERSION", // [698]
    "CONVERT", // [699]
    "CopyOptionsForColumnsSegment", // [700]
    "DELIMITER", // [701]
    "ENCLOSED", // [702]
    "ENFORCELENGTH", // [703]
    "ESCAPE", // [704]
    "FILLER", // [705]
    "TRIM", // [706]
    "copy_options_for_columns", // [707]
    "COPY", // [708]
    "DelimiterKeywordSegment", // [709]
    "EnclosedKeywordSegment", // [710]
    "EnforcelengthKeywordSegment", // [711]
    "EscapeKeywordSegment", // [712]
    "NoKeywordSegment", // [713]
    "FillerKeywordSegment", // [714]
    "TrimKeywordSegment", // [715]
    "ERROR", // [716]
    "EXCEPTION", // [717]
    "RECORD", // [718]
    "REJECTED", // [719]
    "REJECTMAX", // [720]
    "SKIP", // [721]
    "TRAILING", // [722]
    "AbortKeywordSegment", // [723]
    "ErrorKeywordSegment", // [724]
    "ToleranceKeywordSegment", // [725]
    "TOLERANCE", // [726]
    "ExceptionKeywordSegment", // [727]
    "RecordKeywordSegment", // [728]
    "TerminatorKeywordSegment", // [729]
    "TERMINATOR", // [730]
    "RejectedKeywordSegment", // [731]
    "RejectmaxKeywordSegment", // [732]
    "SkipKeywordSegment", // [733]
    "BytesKeywordSegment", // [734]
    "TrailingKeywordSegment", // [735]
    "NullcolsKeywordSegment", // [736]
    "NULLCOLS", // [737]
    "CopyKeywordSegment", // [738]
    "CopyColumnOptionsSegment", // [739]
    "copy_column_options", // [740]
    "LOCAL", // [741]
    "STDIN", // [742]
    "VERTICA", // [743]
    "LocalKeywordSegment", // [744]
    "StdinKeywordSegment", // [745]
    "CompressionType", // [746]
    "VerticaKeywordSegment", // [747]
    "NATIVE", // [748]
    "ORC", // [749]
    "PARQUET", // [750]
    "NativeKeywordSegment", // [751]
    "VarcharKeywordSegment", // [752]
    "VARCHAR", // [753]
    "OrcKeywordSegment", // [754]
    "ParquetKeywordSegment", // [755]
    "CopyOptionsSegment", // [756]
    "copy_options", // [757]
    "CORR", // [758]
    "CORRELATION", // [759]
    "CORRESPONDING", // [760]
    "COUNT", // [761]
    "COVAR_POP", // [762]
    "COVAR_SAMP", // [763]
    "CPUAFFINITYMODE", // [764]
    "CPUAFFINITYSET", // [765]
    "CastKeywordSegment", // [766]
    "SpecificKeywordSegment", // [767]
    "SPECIFIC", // [768]
    "INSTANCE", // [769]
    "METHOD", // [770]
    "STATIC", // [771]
    "InstanceKeywordSegment", // [772]
    "StaticKeywordSegment", // [773]
    "ConstructorKeywordSegment", // [774]
    "MethodKeywordSegment", // [775]
    "AssignmentKeywordSegment", // [776]
    "DatabaseReferenceSegment", // [777]
    "database_reference", // [778]
    "AccessrankKeywordSegment", // [779]
    "OrReplaceGrammar", // [780]
    "TemporaryGrammar", // [781]
    "RETURNS", // [782]
    "ReturnsKeywordSegment", // [783]
    "FunctionDefinitionGrammar", // [784]
    "function_definition", // [785]
    "IndexKeywordSegment", // [786]
    "INDEX", // [787]
    "IndexReferenceSegment", // [788]
    "index_reference", // [789]
    "IndexColumnDefinitionSegment", // [790]
    "index_column_definition", // [791]
    "ModelKeywordSegment", // [792]
    "MODEL", // [793]
    "OPTIONS", // [794]
    "OptionsKeywordSegment", // [795]
    "SegmentedByClauseSegment", // [796]
    "SEGMENTED", // [797]
    "UNSEGMENTED", // [798]
    "segmentedby_clause", // [799]
    "LimitClauseSegment", // [800]
    "LIMIT", // [801]
    "limit_clause", // [802]
    "GroupByClauseSegment", // [803]
    "groupby_clause", // [804]
    "PartitionKeywordSegment", // [805]
    "RangeKeywordSegment", // [806]
    "RANGE", // [807]
    "BetweenKeywordSegment", // [808]
    "AndKeywordSegment", // [809]
    "KsafeSegment", // [810]
    "KSAFE", // [811]
    "ksafe_segment", // [812]
    "AuthorizationKeywordSegment", // [813]
    "START", // [814]
    "StartKeywordSegment", // [815]
    "CreateSequenceOptionsSegment", // [816]
    "create_sequence_options_segment", // [817]
    "GLOBAL", // [818]
    "GlobalKeywordSegment", // [819]
    "CommitKeywordSegment", // [820]
    "PRESERVE", // [821]
    "PreserveKeywordSegment", // [822]
    "AtKeywordSegment", // [823]
    "CENTURY", // [824]
    "DAY", // [825]
    "DAYOFYEAR", // [826]
    "DECADE", // [827]
    "DOW", // [828]
    "DOY", // [829]
    "EPOCH", // [830]
    "HOUR", // [831]
    "ISODOW", // [832]
    "ISOWEEK", // [833]
    "ISOYEAR", // [834]
    "LATEST", // [835]
    "MICROSECONDS", // [836]
    "MILLENNIUM", // [837]
    "MILLISECOND", // [838]
    "MILLISECONDS", // [839]
    "MINUTE", // [840]
    "MONTH", // [841]
    "QUARTER", // [842]
    "SECOND", // [843]
    "TIME ZONE", // [844]
    "TIMEZONE_HOUR", // [845]
    "TIMEZONE_MINUTE", // [846]
    "WEEK", // [847]
    "WEEKDAY", // [848]
    "YEAR", // [849]
    "LatestKeywordSegment", // [850]
    "DatetimeUnitSegment", // [851]
    "LIKE", // [852]
    "LikeKeywordSegment", // [853]
    "EXCLUDING", // [854]
    "INCLUDING", // [855]
    "LikeOptionSegment", // [856]
    "like_option_segment", // [857]
    "TriggerReferenceSegment", // [858]
    "trigger_reference", // [859]
    "INSTEAD", // [860]
    "BeforeKeywordSegment", // [861]
    "InsteadKeywordSegment", // [862]
    "OfKeywordSegment", // [863]
    "OF", // [864]
    "OrKeywordSegment", // [865]
    "DEFERRABLE", // [866]
    "INITIALLY", // [867]
    "REFERENCING", // [868]
    "ReferencingKeywordSegment", // [869]
    "OldKeywordSegment", // [870]
    "OLD", // [871]
    "RowKeywordSegment", // [872]
    "ROW", // [873]
    "NewKeywordSegment", // [874]
    "NEW", // [875]
    "DeferrableKeywordSegment", // [876]
    "InitiallyKeywordSegment", // [877]
    "ImmediateKeywordSegment", // [878]
    "IMMEDIATE", // [879]
    "DeferredKeywordSegment", // [880]
    "DEFERRED", // [881]
    "EachKeywordSegment", // [882]
    "EACH", // [883]
    "STATEMENT", // [884]
    "StatementKeywordSegment", // [885]
    "WhenKeywordSegment", // [886]
    "FunctionNameIdentifierSegment", // [887]
    "FunctionContentsSegment", // [888]
    "function_contents", // [889]
    "WithNoSchemaBindingClauseSegment", // [890]
    "with_no_schema_binding_clause", // [891]
    "CREATEDB", // [892]
    "CREATEROLE", // [893]
    "CREATEUSER", // [894]
    "CRON", // [895]
    "CSV", // [896]
    "CUBE", // [897]
    "function_name_identifier", // [898]
    "ROLLUP", // [899]
    "CubeFunctionNameSegment", // [900]
    "RollupFunctionNameSegment", // [901]
    "GroupingExpressionList", // [902]
    "grouping_expression_list", // [903]
    "CUME_DIST", // [904]
    "CURRENT", // [905]
    "CURRENT_DATABASE", // [906]
    "CURRENT_DEFAULT_TRANSFORM_GROUP", // [907]
    "CURRENT_PATH", // [908]
    "CURRENT_ROLE", // [909]
    "CURRENT_SCHEMA", // [910]
    "CURRENT_TRANSFORM_GROUP_FOR_TYPE", // [911]
    "CURRENT_USER", // [912]
    "CURSOR", // [913]
    "CURSOR_NAME", // [914]
    "CUSTOM", // [915]
    "CUSTOM_PARTITIONS", // [916]
    "DATABASES", // [917]
    "DATAPATH", // [918]
    "data_type_identifier", // [919]
    "TimeWithTZGrammar", // [920]
    "DateKeywordSegment", // [921]
    "DatetimeKeywordSegment", // [922]
    "DATETIME", // [923]
    "SmalldatetimeKeywordSegment", // [924]
    "SMALLDATETIME", // [925]
    "IntervalKeywordSegment", // [926]
    "IntervalLiteralGrammar", // [927]
    "DOUBLE", // [928]
    "DoubleKeywordSegment", // [929]
    "PrecisionKeywordSegment", // [930]
    "PRECISION", // [931]
    "FLOAT", // [932]
    "FloatKeywordSegment", // [933]
    "BracketedArguments", // [934]
    "bracketed_arguments", // [935]
    "Float8KeywordSegment", // [936]
    "FLOAT8", // [937]
    "RealKeywordSegment", // [938]
    "REAL", // [939]
    "IntegerKeywordSegment", // [940]
    "INTEGER", // [941]
    "IntKeywordSegment", // [942]
    "INT", // [943]
    "BigintKeywordSegment", // [944]
    "Int8KeywordSegment", // [945]
    "INT8", // [946]
    "SmallintKeywordSegment", // [947]
    "SMALLINT", // [948]
    "TinyintKeywordSegment", // [949]
    "TINYINT", // [950]
    "DECIMAL", // [951]
    "MONEY", // [952]
    "NUMBER", // [953]
    "NUMERIC", // [954]
    "DecimalKeywordSegment", // [955]
    "NumericKeywordSegment", // [956]
    "NumberKeywordSegment", // [957]
    "MoneyKeywordSegment", // [958]
    "GEOGRAPHY", // [959]
    "GEOMETRY", // [960]
    "GeometryKeywordSegment", // [961]
    "GeographyKeywordSegment", // [962]
    "UuidKeywordSegment", // [963]
    "UUID", // [964]
    "LONG", // [965]
    "LongKeywordSegment", // [966]
    "CharKeywordSegment", // [967]
    "RAW", // [968]
    "VARBINARY", // [969]
    "BinaryKeywordSegment", // [970]
    "VarbinaryKeywordSegment", // [971]
    "ByteaKeywordSegment", // [972]
    "RawKeywordSegment", // [973]
    "BooleanKeywordSegment", // [974]
    "ArrayTypeSegment", // [975]
    "array_type", // [976]
    "SizedArrayTypeSegment", // [977]
    "sized_array_type", // [978]
    "CharacterKeywordSegment", // [979]
    "VARYING", // [980]
    "VaryingKeywordSegment", // [981]
    "DotSegment", // [982]
    "DatatypeIdentifierSegment", // [983]
    "DATEADD", // [984]
    "DATEDIFF", // [985]
    "DATE_PART", // [986]
    "EXTRACT", // [987]
    "DatePartFunctionName", // [988]
    "FunctionContentsGrammar", // [989]
    "TimeKeywordSegment", // [990]
    "TimestampKeywordSegment", // [991]
    "date_constructor_literal", // [992]
    "literal", // [993]
    "LiteralSegment", // [994]
    "DATESTYLE", // [995]
    "date_part", // [996]
    "DATETIME_INTERVAL_CODE", // [997]
    "DATETIME_INTERVAL_PRECISION", // [998]
    "DATETIMES", // [999]
    "DAY_HOUR", // [1000]
    "DAY_MICROSECOND", // [1001]
    "DAY_MINUTE", // [1002]
    "DAY_SECOND", // [1003]
    "DAYOFMONTH", // [1004]
    "DAYOFWEEK", // [1005]
    "DAYS", // [1006]
    "DBCC", // [1007]
    "DEACTIVATE", // [1008]
    "DEALLOCATE", // [1009]
    "DEBUG", // [1010]
    "DEC", // [1011]
    "DECLARE", // [1012]
    "DECODE", // [1013]
    "ValuesKeywordSegment", // [1014]
    "DEFAULTS", // [1015]
    "DEFINE", // [1016]
    "DEFINED", // [1017]
    "DEFINER", // [1018]
    "DEGREE", // [1019]
    "DELAY_KEY_WRITE", // [1020]
    "DELAYED", // [1021]
    "FromClauseSegment", // [1022]
    "from_clause", // [1023]
    "WhereClauseSegment", // [1024]
    "WHERE", // [1025]
    "where_clause", // [1026]
    "DELIMITED", // [1027]
    "SemicolonSegment", // [1028]
    ";", // [1029]
    "DELIMITERS", // [1030]
    "DELTARANGE_COMP_SP", // [1031]
    "DENSE_RANK", // [1032]
    "DENY", // [1033]
    "DEPENDS", // [1034]
    "DEPTH", // [1035]
    "DEREF", // [1036]
    "DERIVED", // [1037]
    "DESC", // [1038]
    "DESCRIBE", // [1039]
    "DescribeKeywordSegment", // [1040]
    "DESCRIPTOR", // [1041]
    "DESTROY", // [1042]
    "DESTRUCTOR", // [1043]
    "DETERMINES", // [1044]
    "DETERMINISTIC", // [1045]
    "DIAGNOSTICS", // [1046]
    "DICTIONARY", // [1047]
    "DIRECT", // [1048]
    "DIRECTCOLS", // [1049]
    "DIRECTED", // [1050]
    "DIRECTGROUPED", // [1051]
    "DIRECTPROJ", // [1052]
    "DISABLE", // [1053]
    "DISCONNECT", // [1054]
    "DISK", // [1055]
    "Disk_quotaKeywordSegment", // [1056]
    "DISPATCH", // [1057]
    "DISTINCT", // [1058]
    "DISTINCTROW", // [1059]
    "DISTRIBUTED", // [1060]
    "DISTVALINDEX", // [1061]
    "DIV", // [1062]
    "DO", // [1063]
    "dot", // [1064]
    "DRYRUN", // [1065]
    "DUMMY", // [1066]
    "DUMP", // [1067]
    "DURABLE", // [1068]
    "DYNAMIC", // [1069]
    "DYNAMIC_FUNCTION", // [1070]
    "DYNAMIC_FUNCTION_CODE", // [1071]
    "ELEMENT", // [1072]
    "ELSEIF", // [1073]
    "StructTypeSegment", // [1074]
    "struct_type", // [1075]
    "EmptyStructLiteralBracketsSegment", // [1076]
    "struct_literal", // [1077]
    "ENABLE", // [1078]
    "ENCODED", // [1079]
    "encoding_type", // [1080]
    "ENCRYPTED", // [1081]
    "END-EXEC", // [1082]
    "}", // [1083]
    "end_curly_bracket", // [1084]
    "ENUM", // [1085]
    "EPHEMERAL", // [1086]
    "EQUALS", // [1087]
    "RawEqualsSegment", // [1088]
    "ERRLVL", // [1089]
    "ESCAPE_STRING_WARNING", // [1090]
    "ESCAPED", // [1091]
    "EVENT", // [1092]
    "EVENTS", // [1093]
    "EVERY", // [1094]
    "EXCEPT", // [1095]
    "EXCEPTIONS", // [1096]
    "EXCLUSIVE", // [1097]
    "EXEC", // [1098]
    "EXECUTIONPARALLELISM", // [1099]
    "EXISTING", // [1100]
    "EXISTS", // [1101]
    "EXIT", // [1102]
    "EXP", // [1103]
    "EXPIRE", // [1104]
    "EXPLAIN", // [1105]
    "ExplainKeywordSegment", // [1106]
    "InsertStatementSegment", // [1107]
    "insert_statement", // [1108]
    "UpdateStatementSegment", // [1109]
    "update_statement", // [1110]
    "DeleteStatementSegment", // [1111]
    "delete_statement", // [1112]
    "EXPORT", // [1113]
    "Expression_A_Grammar", // [1114]
    "Tail_Recurse_Expression_A_Grammar", // [1115]
    "LikeExpressionGrammar", // [1116]
    "ILIKE", // [1117]
    "RLIKE", // [1118]
    "InOperatorGrammar", // [1119]
    "IsClauseGrammar", // [1120]
    "IsNullGrammar", // [1121]
    "ISNULL", // [1122]
    "INTERPOLATE", // [1123]
    "InterpolateGrammar", // [1124]
    "NotNullGrammar", // [1125]
    "NOTNULL", // [1126]
    "CollateGrammar", // [1127]
    "Expression_B_Grammar", // [1128]
    "PRIOR", // [1129]
    "~", // [1130]
    "SignedSegmentGrammar", // [1131]
    "QualifiedNumericLiteralSegment", // [1132]
    "TildeSegment", // [1133]
    "NotOperatorGrammar", // [1134]
    "PriorKeywordSegment", // [1135]
    "Tail_Recurse_Expression_B_Grammar", // [1136]
    "ExistsKeywordSegment", // [1137]
    "Expression_D_Grammar", // [1138]
    "CaseExpressionSegment", // [1139]
    "case_expression", // [1140]
    "TimeZoneGrammar", // [1141]
    "time_zone_grammar", // [1142]
    "LateralColumnAliasExpressionGrammar", // [1143]
    "LocalAliasSegment", // [1144]
    "local_alias_segment", // [1145]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [1146]
    "SingleIdentifierWildcardSegment", // [1147]
    "MapTypeSegment", // [1148]
    "map_type", // [1149]
    "BooleanLiteralGrammar", // [1150]
    "NullLiteralSegment", // [1151]
    "DateTimeLiteralGrammar", // [1152]
    "ListComprehensionGrammar", // [1153]
    "AccessorGrammar", // [1154]
    "SelectStatementSegment", // [1155]
    "select_statement", // [1156]
    "TypedStructLiteralSegment", // [1157]
    "typed_struct_literal", // [1158]
    "ArrayExpressionSegment", // [1159]
    "array_expression", // [1160]
    "OverlapsClauseSegment", // [1161]
    "OVERLAPS", // [1162]
    "overlaps_clause", // [1163]
    "EXTENSION", // [1164]
    "EXTENSIONS", // [1165]
    "FAILED_LOGIN_ATTEMPTS", // [1166]
    "FALLTHROUGH", // [1167]
    "boolean_literal", // [1168]
    "LiteralKeywordSegment", // [1169]
    "FAULT", // [1170]
    "FENCED", // [1171]
    "FETCH", // [1172]
    "FetchKeywordSegment", // [1173]
    "NEXT", // [1174]
    "NextKeywordSegment", // [1175]
    "ONLY", // [1176]
    "OnlyKeywordSegment", // [1177]
    "TiesKeywordSegment", // [1178]
    "TIES", // [1179]
    "FIELDS", // [1180]
    "MERGE", // [1181]
    "ROLLBACK", // [1182]
    "DelimiterGrammar", // [1183]
    "StatementSegment", // [1184]
    "statement", // [1185]
    "FILESYSTEM", // [1186]
    "FILLFACTOR", // [1187]
    "FILTER", // [1188]
    "FilterKeywordSegment", // [1189]
    "WhereKeywordSegment", // [1190]
    "FINAL", // [1191]
    "FIXEDWIDTH", // [1192]
    "FLEX", // [1193]
    "FLEXIBLE", // [1194]
    "FLOAT4", // [1195]
    "FLOOR", // [1196]
    "FLUSH", // [1197]
    "FOLLOWING", // [1198]
    "FORTRAN", // [1199]
    "FORWARD", // [1200]
    "FOUND", // [1201]
    "FrameClauseUnitGrammar", // [1202]
    "UNBOUNDED", // [1203]
    "CurrentKeywordSegment", // [1204]
    "CastOperatorSegment", // [1205]
    "UnboundedKeywordSegment", // [1206]
    "PRECEDING", // [1207]
    "PrecedingKeywordSegment", // [1208]
    "FollowingKeywordSegment", // [1209]
    "FREE", // [1210]
    "FREETEXT", // [1211]
    "FREETEXTTABLE", // [1212]
    "FREEZE", // [1213]
    "FromExpressionSegment", // [1214]
    "from_expression", // [1215]
    "HAVING", // [1216]
    "INTERSECT", // [1217]
    "MINUS", // [1218]
    "OFFSET", // [1219]
    "QUALIFY", // [1220]
    "UNION", // [1221]
    "WINDOW", // [1222]
    "OrderKeywordSegment", // [1223]
    "HavingKeywordSegment", // [1224]
    "QualifyKeywordSegment", // [1225]
    "WindowKeywordSegment", // [1226]
    "SetOperatorSegment", // [1227]
    "set_operator", // [1228]
    "WithDataClauseSegment", // [1229]
    "with_data_clause", // [1230]
    "OffsetKeywordSegment", // [1231]
    "PreTableFunctionKeywordsGrammar", // [1232]
    "TableExpressionSegment", // [1233]
    "table_expression", // [1234]
    "TemporalQuerySegment", // [1235]
    "temporal_query", // [1236]
    "FromClauseTerminatorGrammar", // [1237]
    "SamplingExpressionSegment", // [1238]
    "TABLESAMPLE", // [1239]
    "sample_expression", // [1240]
    "JoinLikeClauseGrammar", // [1241]
    "JoinClauseSegment", // [1242]
    "join_clause", // [1243]
    "PostTableExpressionGrammar", // [1244]
    "MLTableExpressionSegment", // [1245]
    "ML", // [1246]
    "ml_table_expression", // [1247]
    "FromExpressionElementSegment", // [1248]
    "from_expression_element", // [1249]
    "indented_joins", // [1250]
    "FULLTEXT", // [1251]
    "LambdaExpressionSegment", // [1252]
    "lambda_function", // [1253]
    "SetExpressionSegment", // [1254]
    "LEADING", // [1255]
    "TrimParametersGrammar", // [1256]
    "DistinctKeywordSegment", // [1257]
    "StarSegment", // [1258]
    "FunctionContentsExpressionGrammar", // [1259]
    "AggregateOrderByClause", // [1260]
    "aggregate_order_by", // [1261]
    "SEPARATOR", // [1262]
    "SeparatorKeywordSegment", // [1263]
    "ParametersKeywordSegment", // [1264]
    "PARAMETERS", // [1265]
    "OverClauseSegment", // [1266]
    "over_clause", // [1267]
    "IgnoreRespectNullsGrammar", // [1268]
    "IGNORE", // [1269]
    "RESPECT", // [1270]
    "EmptyStructLiteralSegment", // [1271]
    "WordSegment", // [1272]
    "double_quote", // [1273]
    "QuotedIdentifierSegment", // [1274]
    "AnyKeywordSegment", // [1275]
    "FunctionParameterGrammar", // [1276]
    "DatePartFunctionNameSegment", // [1277]
    "DateTimeFunctionContentsSegment", // [1278]
    "ColumnsExpressionGrammar", // [1279]
    "ColumnsExpressionFunctionNameSegment", // [1280]
    "ValuesClauseSegment", // [1281]
    "values_clause", // [1282]
    "PostFunctionGrammar", // [1283]
    "FUSION", // [1284]
    "G", // [1285]
    "GENERAL", // [1286]
    "GENERATED", // [1287]
    "GET", // [1288]
    "GLOB", // [1289]
    "glob_operator", // [1290]
    "GO", // [1291]
    "GOTO", // [1292]
    "GRACEPERIOD", // [1293]
    "AccessPermissionsSegment", // [1294]
    "access_permissions", // [1295]
    "AccessObjectSegment", // [1296]
    "access_object", // [1297]
    "AccessTargetSegment", // [1298]
    "access_target", // [1299]
    "AdminKeywordSegment", // [1300]
    "GRANTED", // [1301]
    "GrantedKeywordSegment", // [1302]
    "Current_userKeywordSegment", // [1303]
    "Session_userKeywordSegment", // [1304]
    "SESSION_USER", // [1305]
    "GREATEST", // [1306]
    "GroupByClauseTerminatorGrammar", // [1307]
    "GROUPED", // [1308]
    "GROUPING", // [1309]
    "GroupingKeywordSegment", // [1310]
    "SetsKeywordSegment", // [1311]
    "SETS", // [1312]
    "CubeRollupClauseSegment", // [1313]
    "cube_rollup_clause", // [1314]
    "HANDLER", // [1315]
    "HCATALOG", // [1316]
    "HCATALOG_CONNECTION_TIMEOUT", // [1317]
    "HCATALOG_DB", // [1318]
    "HCATALOG_SCHEMA", // [1319]
    "HCATALOG_SLOW_TRANSFER_LIMIT", // [1320]
    "HCATALOG_SLOW_TRANSFER_TIME", // [1321]
    "HCATALOG_USER", // [1322]
    "HEADER", // [1323]
    "HEAP", // [1324]
    "HIERARCHY", // [1325]
    "HIGH", // [1326]
    "HIGH_PRIORITY", // [1327]
    "HIVESERVER2_HOSTNAME", // [1328]
    "HOLD", // [1329]
    "HOLDLOCK", // [1330]
    "HOST", // [1331]
    "HOSTNAME", // [1332]
    "HOSTS", // [1333]
    "HOUR_MICROSECOND", // [1334]
    "HOUR_MINUTE", // [1335]
    "HOUR_SECOND", // [1336]
    "HOURS", // [1337]
    "ICEBERG", // [1338]
    "ID", // [1339]
    "IDENTIFIED", // [1340]
    "identifier", // [1341]
    "IdentifierSegment", // [1342]
    "IDENTITY", // [1343]
    "IDENTITY_INSERT", // [1344]
    "IDENTITYCOL", // [1345]
    "IDLESESSIONTIMEOUT", // [1346]
    "IfKeywordSegment", // [1347]
    "IgnoreKeywordSegment", // [1348]
    "RespectKeywordSegment", // [1349]
    "NullsKeywordSegment", // [1350]
    "NULLS", // [1351]
    "ILIKEB", // [1352]
    "IMPLEMENTATION", // [1353]
    "IMPLICIT", // [1354]
    "AscKeywordSegment", // [1355]
    "DescKeywordSegment", // [1356]
    "INDICATOR", // [1357]
    "INFILE", // [1358]
    "INFIX", // [1359]
    "INHERIT", // [1360]
    "INHERITS", // [1361]
    "INITIAL", // [1362]
    "INITIALIZE", // [1363]
    "INOUT", // [1364]
    "INPUT", // [1365]
    "INSENSITIVE", // [1366]
    "OverwriteKeywordSegment", // [1367]
    "OVERWRITE", // [1368]
    "IntoKeywordSegment", // [1369]
    "INTO", // [1370]
    "DefaultValuesGrammar", // [1371]
    "INSERT_ID", // [1372]
    "INSTANTIABLE", // [1373]
    "INT1", // [1374]
    "INT2", // [1375]
    "INT3", // [1376]
    "INT4", // [1377]
    "integer_literal", // [1378]
    "INTEGRATIONS", // [1379]
    "INTERFACE", // [1380]
    "InterpolateKeywordSegment", // [1381]
    "PREVIOUS", // [1382]
    "PreviousKeywordSegment", // [1383]
    "ValueKeywordSegment", // [1384]
    "INTERSECTION", // [1385]
    "IntervalUnitsGrammar", // [1386]
    "YearKeywordSegment", // [1387]
    "MonthKeywordSegment", // [1388]
    "DayKeywordSegment", // [1389]
    "HourKeywordSegment", // [1390]
    "MinuteKeywordSegment", // [1391]
    "SecondKeywordSegment", // [1392]
    "INTERVALSTYLE", // [1393]
    "INTERVALYM", // [1394]
    "INVOKER", // [1395]
    "NanLiteralSegment", // [1396]
    "NAN", // [1397]
    "UnknownLiteralSegment", // [1398]
    "NormalizedGrammar", // [1399]
    "IsnullKeywordSegment", // [1400]
    "ISAM", // [1401]
    "ISOLATION", // [1402]
    "ITERATE", // [1403]
    "ConditionalJoinKeywordsGrammar", // [1404]
    "JoinKeywordsGrammar", // [1405]
    "JOIN", // [1406]
    "NestedJoinGrammar", // [1407]
    "indented_using_on", // [1408]
    "MatchConditionSegment", // [1409]
    "match_condition", // [1410]
    "JoinOnConditionSegment", // [1411]
    "join_on_condition", // [1412]
    "JoinUsingConditionGrammar", // [1413]
    "UnconditionalJoinKeywordsGrammar", // [1414]
    "ExtendedNaturalJoinKeywordsGrammar", // [1415]
    "JoinKeywordSegment", // [1416]
    "indented_on_contents", // [1417]
    "AntiKeywordSegment", // [1418]
    "SemiallKeywordSegment", // [1419]
    "SemiKeywordSegment", // [1420]
    "NullawareKeywordSegment", // [1421]
    "InnerKeywordSegment", // [1422]
    "FullKeywordSegment", // [1423]
    "LeftKeywordSegment", // [1424]
    "RightKeywordSegment", // [1425]
    "NaturalKeywordSegment", // [1426]
    "JSON", // [1427]
    "K", // [1428]
    "KEY_MEMBER", // [1429]
    "KEY_TYPE", // [1430]
    "KEYMANAGER", // [1431]
    "KEYS", // [1432]
    "KILL", // [1433]
    "KsafeKeywordSegment", // [1434]
    "LABEL", // [1435]
    "->", // [1436]
    "lambda_arrow", // [1437]
    "LambdaArrowSegment", // [1438]
    "LANCOMPILER", // [1439]
    "LAST", // [1440]
    "LAST_INSERT_ID", // [1441]
    "LATERAL", // [1442]
    "LEAST", // [1443]
    "LEAVE", // [1444]
    "LENGTH", // [1445]
    "LESS", // [1446]
    "LEVEL", // [1447]
    "LikeGrammar", // [1448]
    "RlikeKeywordSegment", // [1449]
    "IlikeKeywordSegment", // [1450]
    "IncludingKeywordSegment", // [1451]
    "ExcludingKeywordSegment", // [1452]
    "LIKEB", // [1453]
    "LimitKeywordSegment", // [1454]
    "LINENO", // [1455]
    "LINES", // [1456]
    "LISTEN", // [1457]
    "ArrayLiteralSegment", // [1458]
    "array_literal", // [1459]
    "TypedArrayLiteralSegment", // [1460]
    "typed_array_literal", // [1461]
    "ObjectLiteralSegment", // [1462]
    "object_literal", // [1463]
    "LN", // [1464]
    "LOAD", // [1465]
    "LOADER", // [1466]
    "LOCALE", // [1467]
    "LOCATION", // [1468]
    "LOCATOR", // [1469]
    "LOCK", // [1470]
    "LOCKS", // [1471]
    "LOGIN", // [1472]
    "LOGS", // [1473]
    "LONGBLOB", // [1474]
    "LONGTEXT", // [1475]
    "LOOP", // [1476]
    "LOW", // [1477]
    "LOW_PRIORITY", // [1478]
    "LOWER", // [1479]
    "M", // [1480]
    "MlKeywordSegment", // [1481]
    "MANAGED", // [1482]
    "MAP", // [1483]
    "MASK", // [1484]
    "MATCH", // [1485]
    "MATCHED", // [1486]
    "MAX", // [1487]
    "MAX_ROWS", // [1488]
    "MAXCONCURRENCY", // [1489]
    "MAXCONCURRENCYGRACE", // [1490]
    "MAXCONNECTIONS", // [1491]
    "MAXEXTENTS", // [1492]
    "MAXMEMORYSIZE", // [1493]
    "MAXPAYLOAD", // [1494]
    "MAXQUERYMEMORYSIZE", // [1495]
    "MEDIUM", // [1496]
    "MEDIUMBLOB", // [1497]
    "MEDIUMINT", // [1498]
    "MEDIUMTEXT", // [1499]
    "MEMBER", // [1500]
    "MEMORYCAP", // [1501]
    "MEMORYSIZE", // [1502]
    "MergeKeywordSegment", // [1503]
    "MergeMatchedClauseSegment", // [1504]
    "merge_when_matched_clause", // [1505]
    "MergeNotMatchedClauseSegment", // [1506]
    "merge_when_not_matched_clause", // [1507]
    "MatchedKeywordSegment", // [1508]
    "ThenKeywordSegment", // [1509]
    "THEN", // [1510]
    "MergeUpdateClauseSegment", // [1511]
    "merge_update_clause", // [1512]
    "MergeDeleteClauseSegment", // [1513]
    "merge_delete_clause", // [1514]
    "MergeInsertClauseSegment", // [1515]
    "merge_insert_clause", // [1516]
    "MergeIntoLiteralGrammar", // [1517]
    "AliasedTableReferenceGrammar", // [1518]
    "MergeMatchSegment", // [1519]
    "merge_match", // [1520]
    "SetClauseListSegment", // [1521]
    "set_clause_list", // [1522]
    "MESSAGE_LENGTH", // [1523]
    "MESSAGE_OCTET_LENGTH", // [1524]
    "MESSAGE_TEXT", // [1525]
    "MIDDLEINT", // [1526]
    "MIN", // [1527]
    "MIN_ROWS", // [1528]
    "MINUTE_MICROSECOND", // [1529]
    "MINUTE_SECOND", // [1530]
    "MINUTES", // [1531]
    "MLSLABEL", // [1532]
    "MOD", // [1533]
    "MODE", // [1534]
    "MODIFIES", // [1535]
    "MODULE", // [1536]
    "MONTHNAME", // [1537]
    "MORE", // [1538]
    "MOVE", // [1539]
    "MOVEOUT", // [1540]
    "MULTIPLEACTIVERESULTSETS", // [1541]
    "MULTISET", // [1542]
    "MUMPS", // [1543]
    "MYISAM", // [1544]
    "naked_identifier", // [1545]
    "NAME", // [1546]
    "WindowSpecificationSegment", // [1547]
    "window_specification", // [1548]
    "NamedWindowExpressionSegment", // [1549]
    "named_window_expression", // [1550]
    "NAMES", // [1551]
    "NAMESPACE", // [1552]
    "null_literal", // [1553]
    "NATIONAL", // [1554]
    "NCHAR", // [1555]
    "NCLOB", // [1556]
    "sign_indicator", // [1557]
    "NESTING", // [1558]
    "NETWORK", // [1559]
    "newline", // [1560]
    "NewlineSegment", // [1561]
    "NO_WRITE_TO_BINLOG", // [1562]
    "NOAUDIT", // [1563]
    "NOCHECK", // [1564]
    "NOCOMPRESS", // [1565]
    "NOCREATEDB", // [1566]
    "NOCREATEROLE", // [1567]
    "NOCREATEUSER", // [1568]
    "NODES", // [1569]
    "NOGLOB", // [1570]
    "NOINHERIT", // [1571]
    "NOLOGIN", // [1572]
    "UnorderedSelectStatementSegment", // [1573]
    "WithCompoundStatementSegment", // [1574]
    "with_compound_statement", // [1575]
    "NonSetSelectableGrammar", // [1576]
    "BracketedSetExpressionGrammar", // [1577]
    "MergeStatementSegment", // [1578]
    "merge_statement", // [1579]
    "NONCLUSTERED", // [1580]
    "NONE", // [1581]
    "NORMALIZE", // [1582]
    "NORMALIZED", // [1583]
    "NOSUPERUSER", // [1584]
    "RawNotSegment", // [1585]
    "NotnullKeywordSegment", // [1586]
    "NOTHING", // [1587]
    "NOTIFIER", // [1588]
    "NOTIFY", // [1589]
    "NOWAIT", // [1590]
    "::!", // [1591]
    "null_casting_operator", // [1592]
    "null_equals_operator", // [1593]
    "NullEqualsOperatorSegment", // [1594]
    "NULLABLE", // [1595]
    "NULLIF", // [1596]
    "NULLSEQUAL", // [1597]
    "ColonSegment", // [1598]
    "ObjectLiteralElementSegment", // [1599]
    "object_literal_element", // [1600]
    "start_curly_bracket", // [1601]
    "NullCastOperatorSegment", // [1602]
    "StartSquareBracketSegment", // [1603]
    "StartBracketSegment", // [1604]
    "OBJECTS", // [1605]
    "OCTET_LENGTH", // [1606]
    "OCTETS", // [1607]
    "OFF", // [1608]
    "OFFLINE", // [1609]
    "OFFSETS", // [1610]
    "OIDS", // [1611]
    "ONLINE", // [1612]
    "OPEN", // [1613]
    "OPENDATASOURCE", // [1614]
    "OPENQUERY", // [1615]
    "OPENROWSET", // [1616]
    "OPENXML", // [1617]
    "OPERATION", // [1618]
    "OPERATOR", // [1619]
    "OPT", // [1620]
    "OPTIMIZE", // [1621]
    "OPTIMIZER", // [1622]
    "OPTIONALLY", // [1623]
    "OPTVER", // [1624]
    "ReplaceKeywordSegment", // [1625]
    "REPLACE", // [1626]
    "LastKeywordSegment", // [1627]
    "WithFillSegment", // [1628]
    "with_fill", // [1629]
    "NoorderKeywordSegment", // [1630]
    "ORDERING", // [1631]
    "ORDINALITY", // [1632]
    "OTHERS", // [1633]
    "OUT", // [1634]
    "OUTFILE", // [1635]
    "OUTPUT", // [1636]
    "OverKeywordSegment", // [1637]
    "OVER", // [1638]
    "OverlapsKeywordSegment", // [1639]
    "OVERLAY", // [1640]
    "OVERRIDING", // [1641]
    "PACK_KEYS", // [1642]
    "PAD", // [1643]
    "parameter", // [1644]
    "?", // [1645]
    "PARAMETER_MODE", // [1646]
    "PARAMETER_NAME", // [1647]
    "PARAMETER_ORDINAL_POSITION", // [1648]
    "PARAMETER_SPECIFIC_CATALOG", // [1649]
    "PARAMETER_SPECIFIC_NAME", // [1650]
    "PARAMETER_SPECIFIC_SCHEMA", // [1651]
    "PARSER", // [1652]
    "PARTIAL", // [1653]
    "BestKeywordSegment", // [1654]
    "NodesKeywordSegment", // [1655]
    "PASCAL", // [1656]
    "PASSWORD", // [1657]
    "PASSWORD_GRACE_TIME", // [1658]
    "PASSWORD_LIFE_TIME", // [1659]
    "PASSWORD_LOCK_TIME", // [1660]
    "PASSWORD_MAX_LENGTH", // [1661]
    "PASSWORD_MIN_CHAR_CHANGE", // [1662]
    "PASSWORD_MIN_DIGITS", // [1663]
    "PASSWORD_MIN_LENGTH", // [1664]
    "PASSWORD_MIN_LETTERS", // [1665]
    "PASSWORD_MIN_LIFE_TIME", // [1666]
    "PASSWORD_MIN_LOWERCASE_LETTERS", // [1667]
    "PASSWORD_MIN_SYMBOLS", // [1668]
    "PASSWORD_MIN_UPPERCASE_LETTERS", // [1669]
    "PASSWORD_REUSE_MAX", // [1670]
    "PASSWORD_REUSE_TIME", // [1671]
    "PATH", // [1672]
    "SlashSegment", // [1673]
    "path_segment", // [1674]
    "PATTERN", // [1675]
    "PCTFREE", // [1676]
    "PERCENT", // [1677]
    "PERCENT_RANK", // [1678]
    "PERCENTILE_CONT", // [1679]
    "PERCENTILE_DISC", // [1680]
    "PERMANENT", // [1681]
    "PINNED", // [1682]
    "pipe", // [1683]
    "PLACING", // [1684]
    "PLAN", // [1685]
    "PLANNEDCONCURRENCY", // [1686]
    "PLI", // [1687]
    "POINT", // [1688]
    "POOL", // [1689]
    "PORT", // [1690]
    "POSITION", // [1691]
    "FilterClauseGrammar", // [1692]
    "WithinGroupClauseSegment", // [1693]
    "WITHIN", // [1694]
    "within_group_clause_statement", // [1695]
    "POSTFIX", // [1696]
    "POWER", // [1697]
    "PREFER", // [1698]
    "PREFIX", // [1699]
    "PREORDER", // [1700]
    "PREPARE", // [1701]
    "PREPARED", // [1702]
    "PREPASS", // [1703]
    "PRINT", // [1704]
    "PRIORITY", // [1705]
    "PROC", // [1706]
    "PROCEDURAL", // [1707]
    "PROCESS", // [1708]
    "PROCESSLIST", // [1709]
    "PROFILE", // [1710]
    "PSDATE", // [1711]
    "PURGE", // [1712]
    "SOME", // [1713]
    "SomeKeywordSegment", // [1714]
    "QUERY", // [1715]
    "QUEUETIMEOUT", // [1716]
    "QUOTE", // [1717]
    "quoted_identifier", // [1718]
    "quoted_literal", // [1719]
    "RAID0", // [1720]
    "RAISERROR", // [1721]
    "RANDOM", // [1722]
    "RANK", // [1723]
    "raw_comparison_operator", // [1724]
    "READS", // [1725]
    "READTEXT", // [1726]
    "RECHECK", // [1727]
    "RECONFIGURE", // [1728]
    "RECOVER", // [1729]
    "RECURSIVE", // [1730]
    "REF", // [1731]
    "ReferenceMatchGrammar", // [1732]
    "ReferentialActionGrammar", // [1733]
    "MatchKeywordSegment", // [1734]
    "SIMPLE", // [1735]
    "PartialKeywordSegment", // [1736]
    "SimpleKeywordSegment", // [1737]
    "ActionKeywordSegment", // [1738]
    "REFRESH", // [1739]
    "REGEXP", // [1740]
    "REGR_AVGX", // [1741]
    "REGR_AVGY", // [1742]
    "REGR_COUNT", // [1743]
    "REGR_INTERCEPT", // [1744]
    "REGR_R2", // [1745]
    "REGR_SLOPE", // [1746]
    "REGR_SXX", // [1747]
    "REGR_SXY", // [1748]
    "REGR_SYY", // [1749]
    "REINDEX", // [1750]
    "RELATIVE", // [1751]
    "RELEASE", // [1752]
    "RELOAD", // [1753]
    "REPEAT", // [1754]
    "REPEATABLE", // [1755]
    "REPLICATE", // [1756]
    "REPLICATION", // [1757]
    "REQUIRE", // [1758]
    "RESET", // [1759]
    "RESIGNAL", // [1760]
    "RESOURCE_POOL", // [1761]
    "RESTART", // [1762]
    "RESTORE", // [1763]
    "RESULT", // [1764]
    "RESULTS", // [1765]
    "RETENTION", // [1766]
    "RETRY", // [1767]
    "RETURN", // [1768]
    "RETURNED_CARDINALITY", // [1769]
    "RETURNED_LENGTH", // [1770]
    "RETURNED_OCTET_LENGTH", // [1771]
    "RETURNED_SQLSTATE", // [1772]
    "RETURNREJECTED", // [1773]
    "RevokeKeywordSegment", // [1774]
    "ROLES", // [1775]
    "ROUTE", // [1776]
    "ROUTINE_CATALOG", // [1777]
    "ROUTINE_NAME", // [1778]
    "ROUTINE_SCHEMA", // [1779]
    "ROUTING", // [1780]
    "ROW_COUNT", // [1781]
    "ROW_NUMBER", // [1782]
    "ROWCOUNT", // [1783]
    "ROWGUIDCOL", // [1784]
    "ROWID", // [1785]
    "ROWNUM", // [1786]
    "RULE", // [1787]
    "RUNTIMECAP", // [1788]
    "RUNTIMEPRIORITY", // [1789]
    "RUNTIMEPRIORITYTHRESHOLD", // [1790]
    "RWITH", // [1791]
    "SALT", // [1792]
    "TablesampleKeywordSegment", // [1793]
    "SYSTEM", // [1794]
    "BernoulliKeywordSegment", // [1795]
    "SystemKeywordSegment", // [1796]
    "RepeatableKeywordSegment", // [1797]
    "SAVE", // [1798]
    "SAVEPOINT", // [1799]
    "SCALE", // [1800]
    "SCHEDULE", // [1801]
    "IncludeKeywordSegment", // [1802]
    "ExcludeKeywordSegment", // [1803]
    "MaterializeKeywordSegment", // [1804]
    "SCHEMA_NAME", // [1805]
    "SCHEMATA", // [1806]
    "SCOPE", // [1807]
    "SCOPE_CATALOG", // [1808]
    "SCOPE_NAME", // [1809]
    "SCOPE_SCHEMA", // [1810]
    "SCROLL", // [1811]
    "SEARCH", // [1812]
    "SEARCH_PATH", // [1813]
    "SECOND_MICROSECOND", // [1814]
    "SECONDARY", // [1815]
    "SECONDS", // [1816]
    "SECTION", // [1817]
    "SECURITY", // [1818]
    "SECURITY_ALGORITHM", // [1819]
    "UnsegmentedKeywordSegment", // [1820]
    "SegmentedKeywordSegment", // [1821]
    "WildcardExpressionSegment", // [1822]
    "wildcard_expression", // [1823]
    "SelectClauseModifierSegment", // [1824]
    "select_clause_modifier", // [1825]
    "SelectClauseElementSegment", // [1826]
    "select_clause_element", // [1827]
    "SelectClauseTerminatorGrammar", // [1828]
    "SelectClauseSegment", // [1829]
    "select_clause", // [1830]
    "HavingClauseSegment", // [1831]
    "having_clause", // [1832]
    "NamedWindowSegment", // [1833]
    "named_window", // [1834]
    "FetchClauseSegment", // [1835]
    "fetch_clause", // [1836]
    "TimeseriesClauseSegment", // [1837]
    "TIMESERIES", // [1838]
    "timeseries_clause_statement", // [1839]
    "WithCompoundNonSelectStatementSegment", // [1840]
    "NonWithSelectableGrammar", // [1841]
    "SELF", // [1842]
    "statement_terminator", // [1843]
    "SENSITIVE", // [1844]
    "MaxvalueKeywordSegment", // [1845]
    "MinvalueKeywordSegment", // [1846]
    "SERIAL", // [1847]
    "SERIALIZABLE", // [1848]
    "SERVER_NAME", // [1849]
    "SetClauseSegment", // [1850]
    "set_clause", // [1851]
    "UnionGrammar", // [1852]
    "IntersectKeywordSegment", // [1853]
    "ExceptKeywordSegment", // [1854]
    "MinusKeywordSegment", // [1855]
    "STANDARD_CONFORMING_STRINGS", // [1856]
    "DatestyleKeywordSegment", // [1857]
    "Escape_string_warningKeywordSegment", // [1858]
    "IntervalstyleKeywordSegment", // [1859]
    "LocaleKeywordSegment", // [1860]
    "Standard_conforming_stringsKeywordSegment", // [1861]
    "Search_pathKeywordSegment", // [1862]
    "NoneKeywordSegment", // [1863]
    "ZoneKeywordSegment", // [1864]
    "ZONE", // [1865]
    "Resource_poolKeywordSegment", // [1866]
    "TEMPSPACECAP", // [1867]
    "WORKLOAD", // [1868]
    "AutocommitKeywordSegment", // [1869]
    "OffKeywordSegment", // [1870]
    "CharacteristicsKeywordSegment", // [1871]
    "TransactionKeywordSegment", // [1872]
    "TRANSACTION", // [1873]
    "GraceperiodKeywordSegment", // [1874]
    "IdlesessiontimeoutKeywordSegment", // [1875]
    "RuntimecapKeywordSegment", // [1876]
    "MemorycapKeywordSegment", // [1877]
    "TempspacecapKeywordSegment", // [1878]
    "MultipleactiveresultsetsKeywordSegment", // [1879]
    "WorkloadKeywordSegment", // [1880]
    "SETOF", // [1881]
    "SETUSER", // [1882]
    "SHARD", // [1883]
    "SHARED", // [1884]
    "SHARES", // [1885]
    "SHOW", // [1886]
    "SHUTDOWN", // [1887]
    "SIGNAL", // [1888]
    "SIGNED", // [1889]
    "PositiveSegment", // [1890]
    "NegativeSegment", // [1891]
    "SIMILAR", // [1892]
    "SINGLEINITIATOR", // [1893]
    "SITE", // [1894]
    "SITES", // [1895]
    "SIZE", // [1896]
    "slash", // [1897]
    "slice", // [1898]
    "SONAME", // [1899]
    "SOURCE", // [1900]
    "SPACE", // [1901]
    "SPATIAL", // [1902]
    "SPECIFIC_NAME", // [1903]
    "SPECIFICTYPE", // [1904]
    "SPLIT", // [1905]
    "SQL", // [1906]
    "SQL_BIG_RESULT", // [1907]
    "SQL_BIG_SELECTS", // [1908]
    "SQL_BIG_TABLES", // [1909]
    "SQL_CALC_FOUND_ROWS", // [1910]
    "SQL_LOG_OFF", // [1911]
    "SQL_LOG_UPDATE", // [1912]
    "SQL_LOW_PRIORITY_UPDATES", // [1913]
    "SQL_SELECT_LIMIT", // [1914]
    "SQL_SMALL_RESULT", // [1915]
    "SQL_WARNINGS", // [1916]
    "SQLCA", // [1917]
    "SQLCODE", // [1918]
    "SQLERROR", // [1919]
    "SQLEXCEPTION", // [1920]
    "SQLSTATE", // [1921]
    "SQLWARNING", // [1922]
    "SQRT", // [1923]
    "SSL", // [1924]
    "SSL_CONFIG", // [1925]
    "STABLE", // [1926]
    "STANDBY", // [1927]
    "star", // [1928]
    "STARTING", // [1929]
    "STARTS", // [1930]
    "STATE", // [1931]
    "TransactionStatementSegment", // [1932]
    "transaction_statement", // [1933]
    "DropTableStatementSegment", // [1934]
    "drop_table_statement", // [1935]
    "DropViewStatementSegment", // [1936]
    "drop_view_statement", // [1937]
    "CreateUserStatementSegment", // [1938]
    "create_user_statement", // [1939]
    "DropUserStatementSegment", // [1940]
    "drop_user_statement", // [1941]
    "TruncateStatementSegment", // [1942]
    "truncate_table", // [1943]
    "AccessStatementSegment", // [1944]
    "access_statement", // [1945]
    "CreateTableStatementSegment", // [1946]
    "create_table_statement", // [1947]
    "CreateRoleStatementSegment", // [1948]
    "create_role_statement", // [1949]
    "DropRoleStatementSegment", // [1950]
    "drop_role_statement", // [1951]
    "AlterTableStatementSegment", // [1952]
    "alter_table_statement", // [1953]
    "CreateSchemaStatementSegment", // [1954]
    "create_schema_statement", // [1955]
    "SetSchemaStatementSegment", // [1956]
    "set_schema_statement", // [1957]
    "DropSchemaStatementSegment", // [1958]
    "drop_schema_statement", // [1959]
    "DropTypeStatementSegment", // [1960]
    "drop_type_statement", // [1961]
    "CreateDatabaseStatementSegment", // [1962]
    "create_database_statement", // [1963]
    "DropDatabaseStatementSegment", // [1964]
    "drop_database_statement", // [1965]
    "CreateIndexStatementSegment", // [1966]
    "create_index_statement", // [1967]
    "DropIndexStatementSegment", // [1968]
    "drop_index_statement", // [1969]
    "CreateViewStatementSegment", // [1970]
    "create_view_statement", // [1971]
    "CreateCastStatementSegment", // [1972]
    "create_cast_statement", // [1973]
    "DropCastStatementSegment", // [1974]
    "drop_cast_statement", // [1975]
    "CreateFunctionStatementSegment", // [1976]
    "create_function_statement", // [1977]
    "DropFunctionStatementSegment", // [1978]
    "drop_function_statement", // [1979]
    "CreateModelStatementSegment", // [1980]
    "create_model_statement", // [1981]
    "DropModelStatementSegment", // [1982]
    "drop_MODELstatement", // [1983]
    "DescribeStatementSegment", // [1984]
    "describe_statement", // [1985]
    "UseStatementSegment", // [1986]
    "use_statement", // [1987]
    "ExplainStatementSegment", // [1988]
    "explain_statement", // [1989]
    "CreateSequenceStatementSegment", // [1990]
    "create_sequence_statement", // [1991]
    "AlterSequenceStatementSegment", // [1992]
    "alter_sequence_statement", // [1993]
    "DropSequenceStatementSegment", // [1994]
    "drop_sequence_statement", // [1995]
    "CreateTriggerStatementSegment", // [1996]
    "create_trigger", // [1997]
    "DropTriggerStatementSegment", // [1998]
    "drop_trigger", // [1999]
    "CreateExternalTableSegment", // [2000]
    "create_external_table_statement", // [2001]
    "CreateTableLikeStatementSegment", // [2002]
    "create_table_like_statement", // [2003]
    "CreateTableAsStatementSegment", // [2004]
    "create_table_as_statement", // [2005]
    "CreateProjectionStatementSegment", // [2006]
    "create_projection_statement", // [2007]
    "AlterDefaultPrivilegesGrantSegment", // [2008]
    "alter_default_privileges_grant", // [2009]
    "DropProjectionStatementSegment", // [2010]
    "drop_projection_statement", // [2011]
    "AlterViewStatementSegment", // [2012]
    "alter_view_statement", // [2013]
    "SetStatementSegment", // [2014]
    "set_statement", // [2015]
    "CommentOnStatementSegment", // [2016]
    "comment_clause", // [2017]
    "TransactionalStatements", // [2018]
    "transactional_statement", // [2019]
    "AlterSessionStatements", // [2020]
    "alter_session_statement", // [2021]
    "CopyStatementSegment", // [2022]
    "copy_statement", // [2023]
    "AlterSchemaStatementSegment", // [2024]
    "alter_schema_statement", // [2025]
    "STATISTICS", // [2026]
    "STDDEV_POP", // [2027]
    "STDDEV_SAMP", // [2028]
    "STDOUT", // [2029]
    "STEMMER", // [2030]
    "STORAGE", // [2031]
    "STORED", // [2032]
    "STRAIGHT_JOIN", // [2033]
    "STRENGTH", // [2034]
    "STRICT", // [2035]
    "ConcatSegment", // [2036]
    "STRING", // [2037]
    "STRUCTURE", // [2038]
    "STYLE", // [2039]
    "SUBCLASS_ORIGIN", // [2040]
    "SUBCLUSTER", // [2041]
    "SUBJECT", // [2042]
    "SUBLIST", // [2043]
    "SUBMULTISET", // [2044]
    "SUBNET", // [2045]
    "SUBSTRING", // [2046]
    "SUCCESSFUL", // [2047]
    "SUITES", // [2048]
    "SUM", // [2049]
    "SUPERUSER", // [2050]
    "SYMMETRIC", // [2051]
    "SYNONYM", // [2052]
    "SYSID", // [2053]
    "SYSTEM_USER", // [2054]
    "PrimaryKeyGrammar", // [2055]
    "TABLE_NAME", // [2056]
    "Expression_A_Unary_Operator_Grammar", // [2057]
    "Expression_C_Grammar", // [2058]
    "Expression_B_Unary_Operator_Grammar", // [2059]
    "TEMPLATE", // [2060]
    "TRANSIENT", // [2061]
    "TransientKeywordSegment", // [2062]
    "TERMINATE", // [2063]
    "TERMINATED", // [2064]
    "TEXT", // [2065]
    "TEXTSIZE", // [2066]
    "THAN", // [2067]
    "tilde", // [2068]
    "WITHOUT", // [2069]
    "WithoutKeywordSegment", // [2070]
    "TimeseriesKeywordSegment", // [2071]
    "TIMESTAMPADD", // [2072]
    "TIMESTAMPDIFF", // [2073]
    "TIMESTAMPTZ", // [2074]
    "TIMETZ", // [2075]
    "TIMEZONE", // [2076]
    "TINYBLOB", // [2077]
    "TINYTEXT", // [2078]
    "TLS", // [2079]
    "TLSMODE", // [2080]
    "TOAST", // [2081]
    "TOKENIZER", // [2082]
    "TOP", // [2083]
    "TOP_LEVEL_COUNT", // [2084]
    "TRAN", // [2085]
    "BeginKeywordSegment", // [2086]
    "RollbackKeywordSegment", // [2087]
    "WORK", // [2088]
    "WorkKeywordSegment", // [2089]
    "NameKeywordSegment", // [2090]
    "ChainKeywordSegment", // [2091]
    "TRANSACTION_ACTIVE", // [2092]
    "TRANSACTIONS", // [2093]
    "TRANSACTIONS_COMMITTED", // [2094]
    "TRANSACTIONS_ROLLED_BACK", // [2095]
    "TRANSFORMS", // [2096]
    "TRANSLATE", // [2097]
    "TRANSLATION", // [2098]
    "TREAT", // [2099]
    "TRICKLE", // [2100]
    "TRIGGER_CATALOG", // [2101]
    "TRIGGER_NAME", // [2102]
    "TRIGGER_SCHEMA", // [2103]
    "BothKeywordSegment", // [2104]
    "LeadingKeywordSegment", // [2105]
    "TRUSTED", // [2106]
    "TSEQUAL", // [2107]
    "TUNING", // [2108]
    "StructLiteralSegment", // [2109]
    "TYPES", // [2110]
    "UESCAPE", // [2111]
    "UID", // [2112]
    "UNCOMMITTED", // [2113]
    "NaturalJoinKeywordsGrammar", // [2114]
    "UnconditionalCrossJoinKeywordsGrammar", // [2115]
    "HorizontalJoinKeywordsGrammar", // [2116]
    "UNDER", // [2117]
    "UNDO", // [2118]
    "UNENCRYPTED", // [2119]
    "UNI", // [2120]
    "UNINDEXED", // [2121]
    "UnionKeywordSegment", // [2122]
    "UNKNOWN", // [2123]
    "UNLIMITED", // [2124]
    "UNLISTEN", // [2125]
    "UNLOCK", // [2126]
    "UNNAMED", // [2127]
    "UNNEST", // [2128]
    "UNSIGNED", // [2129]
    "UNTIL", // [2130]
    "UPDATETEXT", // [2131]
    "UPPER", // [2132]
    "USER_DEFINED_TYPE_CATALOG", // [2133]
    "USER_DEFINED_TYPE_CODE", // [2134]
    "USER_DEFINED_TYPE_NAME", // [2135]
    "USER_DEFINED_TYPE_SCHEMA", // [2136]
    "USERS", // [2137]
    "UTC_DATE", // [2138]
    "UTC_TIME", // [2139]
    "UTC_TIMESTAMP", // [2140]
    "VACUUM", // [2141]
    "VALID", // [2142]
    "VALIDATE", // [2143]
    "VALIDATOR", // [2144]
    "VALINDEX", // [2145]
    "VAR_POP", // [2146]
    "VAR_SAMP", // [2147]
    "VARCHAR2", // [2148]
    "VARCHARACTER", // [2149]
    "VARIABLE", // [2150]
    "VARIABLES", // [2151]
    "VERBOSE", // [2152]
    "VERSION", // [2153]
    "VOLATILE", // [2154]
    "WAIT", // [2155]
    "WAITFOR", // [2156]
    "WAREHOUSES", // [2157]
    "WEBHDFS_ADDRESS", // [2158]
    "WEBSERVICE_HOSTNAME", // [2159]
    "WEBSERVICE_PORT", // [2160]
    "indented_then", // [2161]
    "indented_then_contents", // [2162]
    "WHENEVER", // [2163]
    "WHILE", // [2164]
    "whitespace", // [2165]
    "WhitespaceSegment", // [2166]
    "WIDTH_BUCKET", // [2167]
    "PartitionClauseSegment", // [2168]
    "FrameClauseSegment", // [2169]
    "frame_clause", // [2170]
    "RecursiveKeywordSegment", // [2171]
    "indented_ctes", // [2172]
    "CTEDefinitionSegment", // [2173]
    "common_table_expression", // [2174]
    "NonWithNonSelectableGrammar", // [2175]
    "BindingKeywordSegment", // [2176]
    "WithinKeywordSegment", // [2177]
    "WRITETEXT", // [2178]
    "X509", // [2179]
    "XML", // [2180]
    "XOR", // [2181]
    "YAML", // [2182]
    "YEAR_MONTH", // [2183]
    "ZEROFILL", // [2184]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 8, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0,
    1, 0, 1, 0, 1, 0, 0, 0, 4294967295, 4294967295, 0, 210, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 10, 1, 2, 1, 1, 4, 3, 1, 4, 5, 211,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 212, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 214, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 215, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 216, 1, 2, 1, 1, 4, 3, 1, 4, 5, 217, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 218, 1, 2, 1, 1, 4, 3, 1, 4, 5, 219,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 220, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 82, 233, 234, 1, 233, 3, 3, 4, 235, 236, 237, 234, 1,
    237, 3, 3, 4, 235, 1, 2, 1, 1, 2, 1, 0, 1, 0, 1, 2,
    1, 0, 244, 1, 2, 1, 1, 4, 3, 1, 4, 5, 11, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 249, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    250, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 0, 0, 1,
    0, 0, 1, 0, 0, 253, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 1,
    2, 1, 0, 1, 2, 1, 1, 2, 1, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1, 0,
    1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 0, 0, 1, 0, 0, 1,
    0, 1, 0, 0, 4294967295, 4294967295, 0, 404, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 405, 406, 234, 1, 406, 3, 3, 4, 235, 407, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 408, 1, 2, 1, 1, 4, 3, 1, 4, 5, 409, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 410, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 410, 411, 412, 1, 411, 3, 3, 411, 4, 413, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 414, 1, 2, 1, 1, 4, 3, 1, 4, 5, 415,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 98, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 416, 1, 2, 1, 1, 4, 3, 1, 4, 5, 417, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 207, 441, 234, 1, 441,
    3, 3, 4, 235, 442, 443, 234, 1, 443, 3, 3, 4, 235, 1, 2, 0,
    356, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 1,
    2, 0, 228, 1, 2, 1, 1, 4, 3, 1, 4, 5, 446, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 447, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    448, 1, 2, 1, 1, 4, 3, 1, 4, 5, 449, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 450, 1, 2, 1, 1, 4, 3, 1, 4, 5, 451, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 452, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 453, 1, 2, 1, 1, 4, 3, 1, 4, 5, 454, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 455, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    456, 1, 2, 1, 1, 4, 3, 1, 4, 5, 457, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 460, 1, 2, 1, 1, 4, 3, 1, 4, 5, 458, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 461, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 462, 1, 2, 1, 1, 4, 3, 1, 4, 5, 463, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 464, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    465, 1, 2, 1, 1, 4, 3, 1, 4, 5, 466, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 467, 1, 2, 1, 1, 4, 3, 1, 4, 5, 468, 469,
    470, 471, 472, 473, 750, 6, 474, 475, 1, 474, 2, 3, 4, 0, 479, 480,
    0, 2, 3, 479, 3, 481, 0, 1, 3, 482, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 483, 1, 2, 1, 1, 4, 3, 1, 4, 5, 484, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 485, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 486, 1, 2, 1, 1, 4, 3, 1, 4, 5, 487, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 488, 1, 2, 1, 1, 4, 3, 1, 4, 5, 489,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 490, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 411, 412, 9, 3, 3, 411, 4, 500, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 501, 1, 2, 1, 1, 4, 3, 1, 4, 5, 502,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 503, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 424, 411, 234, 1, 411, 3, 3, 4, 235, 508, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 509, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    510, 1, 2, 1, 1, 4, 3, 1, 4, 5, 511, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 512, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 515,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 518, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 519, 520,
    0, 2, 3, 519, 1, 2, 1, 525, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 526, 1, 2, 1, 1, 4, 3, 1, 4, 5, 527, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 528, 1, 2, 1, 1, 4, 3, 1, 4, 5, 529,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 530, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 293, 1, 2, 1, 1, 4, 3, 1, 4, 5, 531, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 532, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 533, 1, 2, 1, 1, 4, 3, 1, 4, 5, 534, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 538, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 282, 1, 2, 1, 1, 4, 3, 1, 4, 5, 539,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 540, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 541, 1, 2, 1, 1, 4, 3, 1, 4, 5, 279, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 542, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0, 543, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 557, 1, 2, 1, 1, 4, 3, 1, 4, 5, 558, 559,
    234, 1, 559, 3, 3, 4, 235, 12, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 560, 1, 2, 1, 1, 4, 3, 1, 4, 5, 561, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 562, 1, 2, 1, 1, 4, 3, 1, 4, 5, 563,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 564, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 565, 1, 2, 1, 1, 4, 3, 1, 4, 5, 566, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 567, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 568, 1, 2, 1, 1, 4, 3, 1, 4, 5, 569, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 570, 1, 2, 1, 1, 4, 3, 1, 4, 5, 571,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 572, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 573, 1, 2, 1, 1, 4, 3, 1, 4, 5, 574, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 575, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 576, 1, 2, 1, 1, 4, 3, 1, 4, 5, 326, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 577, 1, 2, 1, 1, 4, 3, 1, 4, 5, 578,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 579, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 580, 1, 2, 1, 1, 4, 3, 1, 4, 5, 581, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 582, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 307, 1, 2, 1, 1, 4, 3, 1, 4, 5, 583, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 585,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 586, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 587, 1, 2, 1, 1, 4, 3, 1, 4, 5, 588, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 4, 475, 9, 2, 3, 4, 589, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 590, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 0, 594, 1, 2, 1, 1, 4, 3, 1, 4, 5, 595, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 596, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 597, 1, 2, 1, 1, 4, 3, 1, 4, 5, 598, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 599, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    600, 1, 2, 1, 1, 4, 3, 1, 4, 5, 601, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 602, 1, 2, 1, 1, 4, 3, 1, 4, 5, 440, 603,
    234, 1, 603, 3, 3, 4, 235, 440, 604, 234, 1, 604, 3, 3, 4, 235,
    440, 605, 234, 1, 605, 3, 3, 4, 235, 606, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 2, 1, 0, 0, 0, 0, 0, 1, 2, 1, 0,
    4294967295, 4294967295, 0, 350, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0,
    1, 2, 1, 0, 626, 1, 2, 1, 1, 4, 3, 1, 4, 5, 628, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 629, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 87, 630, 234, 1, 630, 3, 3, 4, 235, 631, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 632, 1, 2, 1, 1, 4, 3, 1, 4, 5, 633,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 646, 647,
    10, 3, 3, 646, 4, 648, 1, 2, 1, 1, 4, 3, 1, 4, 5, 649,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 617, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 650, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 315, 659,
    9, 3, 3, 315, 4, 660, 1, 2, 1, 1, 4, 3, 1, 4, 5, 661,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 411, 662, 0, 2, 3, 411, 315,
    663, 0, 2, 3, 315, 664, 1, 2, 1, 1, 4, 3, 1, 4, 5, 533,
    665, 666, 667, 668, 1919, 5, 669, 2, 1, 669, 4, 3, 1, 4, 5, 670,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 671, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 672, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 686, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 99, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 687, 1, 2, 1, 1, 4, 3, 1, 4, 5, 688, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 327, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    689, 1, 2, 1, 1, 4, 3, 1, 4, 5, 690, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 691, 1, 2, 1, 1, 4, 3, 1, 4, 5, 692, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 693, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 694, 1, 2, 1, 1, 4, 3, 1, 4, 5, 695, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 696, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    697, 1, 2, 1, 1, 4, 3, 1, 4, 5, 698, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 699, 1, 2, 1, 1, 4, 3, 1, 4, 5, 708, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 4294967295, 1, 0, 0, 4294967295, 4294967295,
    0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2,
    1, 1, 0, 0, 0, 758, 1, 2, 1, 1, 4, 3, 1, 4, 5, 759,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 760, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 761, 1, 2, 1, 1, 4, 3, 1, 4, 5, 762, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 763, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 764, 1, 2, 1, 1, 4, 3, 1, 4, 5, 765, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 3, 4, 1, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1,
    0, 1, 2, 1, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 0,
    1, 0, 1, 2, 1, 100, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 2, 0, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0,
    4294967295, 4294967295, 0, 0, 0, 4294967295, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0,
    0, 0, 0, 4294967295, 1, 0, 1, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1,
    0, 4294967295, 1, 0, 0, 0, 4294967295, 1, 0, 0, 1, 0, 0, 1, 0, 0,
    0, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 892, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 893, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 894, 1, 2, 1, 1, 4, 3, 1, 4, 5, 895, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 674, 1, 2, 1, 1, 4, 3, 1, 4, 5, 896,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 897, 898, 475, 1, 898, 2, 3,
    4, 897, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 904,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 905, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 906, 1, 2, 1, 1, 4, 3, 1, 4, 5, 468, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 907, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 908, 1, 2, 1, 1, 4, 3, 1, 4, 5, 909, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 910, 1, 2, 1, 1, 4, 3, 1, 4, 5, 469,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 470, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 911, 1, 2, 1, 1, 4, 3, 1, 4, 5, 912, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 913, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 914, 1, 2, 1, 1, 4, 3, 1, 4, 5, 915, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 916, 1, 2, 1, 1, 4, 3, 1, 4, 5, 283,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 52, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 13, 1, 2, 1, 1, 4, 3, 1, 4, 5, 917, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 918, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 919, 475, 1, 919, 2, 3, 4, 0, 0, 0, 1, 2, 1,
    0, 1, 2, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 357, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 984, 985, 986, 987, 2711, 4, 898, 475, 1,
    898, 2, 3, 4, 0, 1, 0, 1, 2, 1, 0, 239, 992, 994, 3, 992,
    993, 239, 3, 3, 993, 4, 985, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    995, 1, 2, 1, 1, 4, 3, 1, 4, 5, 923, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 824, 825, 826, 827, 828, 829, 830, 831, 832, 833, 834, 836,
    837, 838, 839, 840, 841, 842, 843, 844, 845, 846, 847, 848, 849, 2772, 25, 996,
    475, 1, 996, 2, 3, 4, 997, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    998, 1, 2, 1, 1, 4, 3, 1, 4, 5, 999, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 825, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1000, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1001, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1002, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1003, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1004, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1005, 1, 2, 1, 1, 4, 3, 1, 4, 5, 826, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1006, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1007, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1008, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1009, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1010, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1011, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    951, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1012, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1013, 1, 2, 1, 1, 4, 3, 1, 4, 5, 266, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1015, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 866, 1, 2, 1, 1, 4, 3, 1, 4, 5, 881, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1016, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1017, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1018, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1019, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1020, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1021, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 101, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1027, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 701, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1030, 1, 2, 1, 1, 4, 3, 1, 4, 5, 618, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1031, 1, 2, 1, 1, 4, 3, 1, 4, 5, 619, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1032, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1033, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1034, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1035, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1036, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1037, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1038, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1039, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1041, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1042, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1043, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1044, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1045, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1046, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1047, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1048, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1049, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1050, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1051, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1052, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1053, 1, 2, 1, 1, 4, 3, 1, 4, 5, 332, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1054, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1055, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 267, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1057, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1058, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1059, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1060, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1061, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1062, 1, 2, 1, 1, 4, 3, 1, 4, 5, 420, 411,
    234, 1, 411, 3, 3, 4, 235, 1063, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 14, 1, 2, 1, 1, 4, 3, 1, 4, 5, 592, 1064, 234, 1, 1064,
    3, 3, 4, 235, 928, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 3,
    4, 1, 254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 0,
    1065, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1066, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1067, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1068, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1069, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1070, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1071, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 883, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1072, 1, 2, 1, 1, 4, 3, 1, 4, 5, 551, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1073, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    1, 1078, 1, 2, 1, 1, 4, 3, 1, 4, 5, 333, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 702, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1079,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 386, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 460, 511, 510, 534, 617, 618, 619, 620, 621, 622, 623, 624, 625,
    3715, 13, 1080, 2, 1, 1080, 4, 3, 1, 4, 5, 1081, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1082, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1083,
    1084, 234, 1, 1084, 3, 3, 4, 235, 553, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 703, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1085, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1086, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    830, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1087, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1089, 1, 2, 1, 1, 4, 3, 1, 4, 5, 716, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 704, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1090, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1091, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1092, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1093, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1094, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1095, 1, 2, 1, 1, 4, 3, 1, 4, 5, 717, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1096, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 272, 1, 2, 1, 1, 4, 3, 1, 4, 5, 854, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1097, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1098, 1, 2, 1, 1, 4, 3, 1, 4, 5, 102, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 134, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1099, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1100, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1101, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1102, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1103, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1104, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1105, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1113, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2, 1, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 0, 1, 2, 1, 0, 1, 0, 1,
    2, 1, 0, 0, 0, 263, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1164,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1165, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 15, 1, 2, 1, 1, 4, 3, 1, 4, 5, 987, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1166, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1167, 1, 2, 1, 1, 4, 3, 1, 4, 5, 358, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 358, 1168, 1169, 1, 1168, 5, 3, 1, 993, 4, 5,
    1170, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1171, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 0, 0, 1172, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1180, 1, 2, 1, 1, 4, 3, 1, 4, 5, 16, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 1, 4294967295, 4294967295, 0, 1, 0,
    0, 4294967295, 4294967295, 0, 1186, 1, 2, 1, 1, 4, 3, 1, 4, 5, 705, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1187, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 2, 1, 1188, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1191,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 368, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1192, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1193, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1194, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1195, 1, 2, 1, 1, 4, 3, 1, 4, 5, 937, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 932, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1196,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1197, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1198, 1, 2, 1, 1, 4, 3, 1, 4, 5, 321, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 323, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 17, 1, 2, 1, 1, 4, 3, 1, 4, 5, 177, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1199, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1200,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1201, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 0, 0, 0, 0, 0, 1210, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1211, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1212, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1213, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 0, 0, 1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2,
    1, 0, 1, 2, 1, 0, 224, 1250, 1, 0, 0, 4294967295, 4294967295, 0, 240, 1250,
    1, 1, 2, 1, 0, 91, 1, 2, 1, 1, 4, 3, 1, 4, 5, 676,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1251, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 2, 1, 0, 0, 0, 0, 4294967295, 1, 0, 1, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 18, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 5, 898, 1272, 2, 898, 5, 3, 3,
    4, 5, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0,
    0, 4294967295, 1, 0, 0, 67, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1284,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 19, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1285, 1, 2, 1, 1, 4, 3, 1, 4, 5, 620, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1286, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1287, 1, 2, 1, 1, 4, 3, 1, 4, 5, 959, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 960, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1288,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1289, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1290, 1290, 659, 3, 1290, 315, 1290, 3, 3, 315, 4, 818, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1291, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1292, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1293, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 189, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 0, 1301, 1, 2, 1, 1, 4, 3, 1, 4, 5, 133, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1306, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 0, 195, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1308,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 1, 0, 1, 0, 1309,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 665,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 621, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1315, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    0, 0, 1216, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1316, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1317, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1318, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1319, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1320, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1321, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1322, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1323, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1324, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1325, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1326, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1327, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1328, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1329, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1330, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1331, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1332, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1333, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    831, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1334, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1335, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1336, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1337, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1338, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1339, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1340, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1341, 1342, 9, 3, 3, 1341, 4, 1343, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1344, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1345, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1346, 1, 2, 1, 1, 4, 3, 1, 4, 5, 352,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1269, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1117, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1352, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 879, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 340, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1353, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1354, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    103, 1, 2, 1, 1, 4, 3, 1, 4, 5, 62, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 0, 1, 2, 1, 0, 273, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 855, 1, 2, 1, 1, 4, 3, 1, 4, 5, 284,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 787, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1357, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1358, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1359, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1360, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1361, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1362, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1363, 1, 2, 1, 1, 4, 3, 1, 4, 5, 867, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 677, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1364, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1365, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1366, 1, 2, 1, 1, 4, 3, 1, 4, 5, 104, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1372, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 769, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1373, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 860, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1374,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1375, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1376, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1377, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 946, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 943, 1, 2, 1, 1, 4, 3, 1, 4, 5, 421, 411, 234, 1, 411,
    3, 3, 4, 235, 941, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2, 4294967295,
    1378, 994, 1, 1378, 3, 3, 993, 4, 20, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1379, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1380, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1123, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1217, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1385, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 359, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1393, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1394, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1370, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1395, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 493, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1401, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1122, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1402, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1403, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295,
    4294967295, 0, 224, 1408, 1, 0, 240, 1408, 1, 0, 1406, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 545, 1417, 1, 1, 2, 1, 0, 240, 1417, 1, 0, 0,
    0, 0, 1, 0, 1, 2, 1, 1427, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1428, 1, 2, 1, 1, 4, 3, 1, 4, 5, 613, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1429, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1430,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1431, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1432, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 9,
    4, 3, 1, 4, 5, 1433, 1, 2, 1, 1, 4, 3, 1, 4, 5, 811,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1435, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1436, 1437, 234, 1, 1437, 3, 3, 4, 235, 1, 0, 1, 2,
    1, 0, 1439, 1, 2, 1, 1, 4, 3, 1, 4, 5, 21, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 92, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1440, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1441, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1442, 1, 2, 1, 1, 4, 3, 1, 4, 5, 835, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1255, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1443, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1444, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 678, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1445, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1446, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1447, 1, 2, 1, 1, 4, 3, 1, 4, 5, 635, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 852, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 495, 495, 659, 3, 495, 315, 495, 3, 3, 315, 4, 0, 0,
    1453, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 0,
    0, 801, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1455, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1456, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1457,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 993, 1169, 9, 5, 3, 1,
    993, 4, 5, 993, 994, 9, 3, 3, 993, 4, 1464, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1465, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1466, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 741, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1467, 1, 2, 1, 1, 4, 3, 1, 4, 5, 471, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 472, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1468, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1469, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1470, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1471, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1472, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1473, 1, 2, 1, 1, 4, 3, 1, 4, 5, 965, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1474, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1475, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1476, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1477, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1478, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1479, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 666, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1480, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 2, 1, 2, 3, 1, 105, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1482, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1483, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1484, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 22, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1485, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1486, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 274, 1, 2, 1, 1, 4, 3, 1, 4, 5, 23, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1487, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1488, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1489, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1490, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1491, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1492, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1493, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1494, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1495, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    285, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1496, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1497, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1498, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1499, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1500, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1501, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1502, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1181, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 0, 0,
    1, 2, 1, 0, 224, 1408, 1, 240, 1408, 1, 341, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1523, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1524, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1525, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 770, 1, 2, 1, 1, 4, 3, 1, 4, 5, 836, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1526, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    838, 1, 2, 1, 1, 4, 3, 1, 4, 5, 839, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1527, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1528, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1218, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 355, 411, 234, 1, 411, 3, 3, 4, 235, 840, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1529, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1530,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1531, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 286, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1246, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1532, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1533, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1534, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 793, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1535,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 106, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1536, 1, 2, 1, 1, 4, 3, 1, 4, 5, 418, 411, 234,
    1, 411, 3, 3, 4, 235, 952, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    40, 1, 2, 1, 1, 4, 3, 1, 4, 5, 841, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1537, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1538, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1539, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1540, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1541, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 419, 411, 234, 1, 411, 3, 3, 4, 235, 1542,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1543, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1544, 1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1545,
    1342, 1, 1545, 3, 3, 1341, 4, 1546, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 2, 1, 0, 1, 0, 1551, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1552, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1397, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1397, 1553, 1169, 1, 1553, 5, 3, 1, 993, 4, 5,
    1554, 1, 2, 1, 1, 4, 3, 1, 4, 5, 748, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 679, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1555, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1556, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 355, 1557, 234, 1, 1557, 3, 3, 4, 235, 1558, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 875,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1560, 1561, 12, 3, 3, 1560, 4,
    1174, 1, 2, 1, 1, 4, 3, 1, 4, 5, 287, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1562, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1563, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 288, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1564, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1565, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1566, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1567, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1568, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 289, 1, 2, 1, 1, 4, 3, 1, 4, 5, 636, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1569, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1570, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1571, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1572, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 1, 2, 1, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0,
    1580, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1581, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 290, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1582, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1583, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 381, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 381, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1587, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1588, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1589, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1126,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1590, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1591, 1592, 234, 1, 1592, 3, 3, 4, 235, 492, 1593, 234, 1,
    1593, 3, 3, 4, 235, 360, 1, 2, 1, 1, 4, 3, 1, 4, 5, 360,
    1553, 1169, 1, 1553, 5, 3, 1, 993, 4, 5, 1595, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 680, 1, 2, 1, 1, 4, 3, 1, 4, 5, 737, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1596, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1351, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1597, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 953, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    954, 1, 2, 1, 1, 4, 3, 1, 4, 5, 97, 97, 994, 3, 97, 993,
    97, 3, 3, 993, 4, 95, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 364, 1601, 234, 1, 1601, 3, 3, 4, 235, 1, 2, 0, 0, 0, 1605,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1606, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1607, 1, 2, 1, 1, 4, 3, 1, 4, 5, 864, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1608, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1609, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1219, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1610, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1611, 1, 2, 1, 1, 4, 3, 1, 4, 5, 871, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 164, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1612,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1176, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1613, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1614, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1615, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1616, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1617, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 107, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1618,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1619, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1620, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1621, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1622, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 261, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1623, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 794, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1624,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 494, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 494, 411, 412, 1, 411, 3, 3, 411, 4, 749, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 1, 0, 0, 222, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1631, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1632, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1633, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1634, 1, 2, 1, 1, 4, 3, 1, 4, 5, 338,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1635, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1636, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    0, 1638, 1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1, 0, 1162,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1640, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1641, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1368, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 268, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 108, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1642, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1643, 1, 2, 1, 1, 4, 3, 1, 4, 5, 311,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4294967295, 1644, 475, 1, 1644, 2,
    3, 4, 1645, 1644, 234, 1, 1644, 3, 3, 4, 235, 1646, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1647, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1648,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1649, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1650, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1651, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1265, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 750, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1652, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1653, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1, 0, 1, 2, 1, 0, 4294967295,
    4294967295, 0, 1, 0, 1, 2, 1, 0, 0, 373, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 402, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1656, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1657, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1658, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1659, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1660, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1661,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1662, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1663, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1664, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1665, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1666, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1667, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1668, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1669,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1670, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1671, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1672, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 5, 1674, 1272, 2, 1674, 5, 3, 3, 4,
    5, 1, 0, 0, 1675, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1676, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1677, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1678, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1679, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1680, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1681, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1682, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 137, 1, 2, 1, 1, 4, 3, 1, 4, 5, 425, 1683,
    234, 1, 1683, 3, 3, 4, 235, 1684, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1685, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1686, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1687, 1, 2, 1, 1, 4, 3, 1, 4, 5, 354,
    411, 234, 1, 411, 3, 3, 4, 235, 1688, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 128, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1689, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1690, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1691, 1, 2, 1, 1, 4, 3, 1, 4, 5, 354, 1557, 234, 1, 1557, 3,
    3, 4, 235, 0, 4294967295, 1, 0, 1696, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1697, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1207, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 931, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1698,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1699, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1700, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1701, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1702, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1703, 1, 2, 1, 1, 4, 3, 1, 4, 5, 821, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1382, 1, 2, 1, 1, 4, 3, 1, 4, 5, 328,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1704, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1129, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1705, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 142, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1706, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1707, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 24, 1, 2, 1, 1, 4, 3, 1, 4, 5, 68,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1708, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1709, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1710, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 637, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 388, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1711, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 206, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1712,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1220, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 2, 1, 842, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1715, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1716, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1717, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1273,
    1718, 1342, 3, 1718, 1341, 1273, 3, 3, 1341, 4, 239, 1719, 994, 3, 1719, 993,
    239, 3, 3, 993, 4, 317, 1719, 994, 3, 1719, 993, 317, 3, 3, 993, 4,
    0, 1720, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1721, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1722, 1, 2, 1, 1, 4, 3, 1, 4, 5, 807,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1723, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 314, 1724, 234, 1, 1724, 3, 3, 4, 235, 423, 1724, 234, 1,
    1724, 3, 3, 4, 235, 968, 1, 2, 1, 1, 4, 3, 1, 4, 5, 422,
    1724, 234, 1, 1724, 3, 3, 4, 235, 491, 1724, 234, 1, 1724, 3, 3, 4,
    235, 109, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1725, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1726, 1, 2, 1, 1, 4, 3, 1, 4, 5, 939,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1727, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1728, 1, 2, 1, 1, 4, 3, 1, 4, 5, 718, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1729, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1730, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1731, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 4294967295, 1, 0, 0, 111, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 110, 1, 2, 1, 1, 4, 3, 1, 4, 5, 868, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1739, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1740, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1741, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1742, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1743, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1744, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1745, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1746,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1747, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1748, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1749, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1750, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 719, 1, 2, 1, 1, 4, 3, 1, 4, 5, 720, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1751, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1752,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1753, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 374, 1, 2, 1, 1, 4, 3, 1, 4, 5, 269, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 375, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1754, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1755, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1626, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1756,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1757, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1758, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1759, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1760, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 25, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1761, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1270, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1762,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1763, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 395, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1764, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1765, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1766, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1767, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1768, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1769,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1770, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1771, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1772, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1773, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 782, 1, 2, 1, 1, 4, 3, 1, 4, 5, 190, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 681, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    622, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1118, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 26, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1775, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1182, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 899, 898, 475, 1, 898, 2, 3, 4, 899, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1776, 1, 2, 1, 1, 4, 3, 1, 4, 5, 27, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1777, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1778, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1779, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 69, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1780, 1, 2, 1, 1, 4, 3, 1, 4, 5, 873, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1781, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1782, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1783, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1784, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1785, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1786, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    345, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1787, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1788, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1789, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1790, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1791, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1792, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 1, 2, 1, 1798, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1799, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1800, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1801, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 28, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1805, 1, 2, 1, 1, 4, 3, 1, 4, 5, 60, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1806, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1807,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1808, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1809, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1810, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1811, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1812, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1813, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 843, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1814,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1815, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1816, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1817, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1818, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1819, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2,
    1, 0, 0, 797, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1,
    0, 2, 10470, 2, 10472, 3, 493, 1058, 493, 381, 1058, 0, 112, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2,
    1, 0, 1842, 1, 2, 1, 1, 4, 3, 1, 4, 5, 682, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 683, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1029, 1843, 234, 1, 1843, 3, 3, 4, 235, 1844, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1262, 1, 2, 1, 1, 4, 3, 1, 4, 5, 29, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 70, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1847, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1848, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 53, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1849, 1, 2, 1, 1, 4, 3, 1, 4, 5, 306, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1305, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 0, 1, 4294967295, 4294967295, 0, 308, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 1881, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1312, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1882, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1883, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    201, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1884, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1885, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1, 4294967295, 4294967295, 0, 1886, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1887, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1888, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1889, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1892, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1735, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 239, 1718, 1342, 3, 1718, 1341, 239, 3, 3, 1341, 4, 1893,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1894, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1895, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1896, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 721, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 420, 1897, 234, 1, 1897, 3, 3, 4, 235, 440, 1898, 234, 1, 1898, 3,
    3, 4, 235, 925, 1, 2, 1, 1, 4, 3, 1, 4, 5, 948, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1713, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1899, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1900, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1901, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1902,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 768, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1903, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1904, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1905, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1906, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1907, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1908, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1909,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1910, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1911, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1912, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1913, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1914, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1915, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1916, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1917,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1918, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1919, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1920, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1921, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1922, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1923, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1924, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1925,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1926, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 30, 1, 2, 1, 1, 4, 3, 1, 4, 5, 71, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1856, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1927, 1, 2, 1, 1, 4, 3, 1, 4, 5, 419, 1928, 234, 1, 1928,
    3, 3, 4, 235, 814, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1929, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1930, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1931, 1, 2, 1, 1, 4, 3, 1, 4, 5, 884, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 771, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2026, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2027, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2028, 1, 2, 1, 1, 4, 3, 1, 4, 5, 742,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2029, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2030, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2031, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2032, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2033, 1, 2, 1, 1, 4, 3, 1, 4, 5, 31, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 72, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2034,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2035, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 2037, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    1, 2, 1, 2038, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2039, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2040, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2041, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2042, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2043, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2044,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2045, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2046, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2047, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2048, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2049, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2050, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 235, 234, 9, 3, 3, 4, 235, 2051, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2052, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    473, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2053, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1794, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2054, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 0, 0, 1,
    2, 1, 1, 2, 1, 0, 32, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2056, 1, 2, 1, 1, 4, 3, 1, 4, 5, 73, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1239, 1, 2, 1, 1, 4, 3, 1, 4, 5, 33, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0,
    34, 1, 2, 1, 1, 4, 3, 1, 4, 5, 74, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 113, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2060, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 114, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1867, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2063, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2064, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 730, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2065, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2066, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2067, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1510, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1179, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1130, 2068,
    234, 1, 2068, 3, 3, 4, 235, 361, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 4294967295, 4294967295, 0, 1838, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 362, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2072, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2073, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2074,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2075, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2076, 1, 2, 1, 1, 4, 3, 1, 4, 5, 845, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 846, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2077, 1, 2, 1, 1, 4, 3, 1, 4, 5, 950, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2078, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2079,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2080, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 89, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2081, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2082, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 726, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2083, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2084, 1, 2, 1, 1, 4, 3, 1, 4, 5, 722,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2085, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1873, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 2092,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 2093, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2094, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2095, 1, 2, 1, 1, 4, 3, 1, 4, 5, 638, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2096, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2061,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2097, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2098, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2099, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2100, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 115, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2101, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2102, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2103,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 706, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 363, 1, 2, 1, 1, 4, 3, 1, 4, 5, 363, 1168,
    1169, 1, 1168, 5, 3, 1, 993, 4, 5, 116, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2106, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2107, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2108, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 1, 2, 1, 35, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2110, 1, 2, 1, 1, 4, 3, 1, 4, 5, 320, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2111, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2112, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1203, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2113, 1, 2, 1, 1, 4, 3, 1, 4, 5, 667, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 2117, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2118, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2119, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2121,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1221, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 329, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2123, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2124, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2125, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2126, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2127, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2128, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 798, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2129, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2130, 1, 2, 1, 1, 4, 3, 1, 4, 5, 117, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2131, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2132, 1, 2, 1, 1, 4, 3, 1, 4, 5, 118, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 119, 1, 2, 1, 1, 4, 3, 1, 4, 5, 120, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 123, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2133, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2134, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2135, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2136, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2137, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 382, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2138, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2139, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2140, 1, 2, 1, 1, 4, 3, 1, 4, 5, 964, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2141, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2142, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2143, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2144, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2145, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 522, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 1, 0, 1, 2, 1, 1, 0, 523, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2146, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2147,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 969, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2148, 1, 2, 1, 1, 4, 3, 1, 4, 5, 753, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2149, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2150, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2151, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 980, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2152,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2153, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 743, 1, 2, 1, 1, 4, 3, 1, 4, 5, 36, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 75, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2154, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2155, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2156, 1, 2, 1, 1, 4, 3, 1, 4, 5, 37,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2157, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2158, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2159, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2160, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 847, 1, 2, 1, 1, 4, 3, 1, 4, 5, 848, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 224, 2161, 1, 545, 2162, 1, 240, 2162, 1, 240, 2161,
    1, 547, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2163, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 1025, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2164, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2165, 2166,
    12, 3, 3, 4, 2165, 2167, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 4294967295, 4294967295, 0, 1222, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 224,
    2172, 1, 1, 0, 240, 2172, 1, 224, 2172, 1, 1, 0, 240, 2172, 1, 259,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1694, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2069, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    5, 1272, 9, 3, 3, 4, 5, 2088, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1868, 1, 2, 1, 1, 4, 3, 1, 4, 5, 57, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 121, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2178,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2179, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2180, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2181, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2182, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 849, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2183, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2184, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1865,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 668, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 623, 1, 2, 1, 1, 4, 3, 1, 4, 5, 624, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 625, 1, 2, 1, 1, 4, 3, 1, 4,
    5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 30, 46, 9, 0, 42, 0, 38, 39, 41, 42, 43, 44, 45,
    46, 47, 48, 49, 50, 0, 51, 40, 54, 0, 55, 56, 0, 58, 59, 61,
    42, 0, 63, 59, 61, 42, 64, 0, 58, 66, 61, 46, 0, 63, 66, 61,
    41, 42, 46, 44, 43, 77, 0, 79, 81, 84, 86, 88, 90, 0, 93, 94,
    96, 52, 0, 122, 47, 48, 124, 41, 42, 44, 0, 125, 126, 127, 0, 129,
    130, 0, 131, 132, 0, 39, 48, 135, 136, 0, 51, 0, 122, 49, 46, 0,
    126, 127, 138, 64, 0, 139, 50, 46, 47, 0, 140, 141, 125, 143, 122, 144,
    129, 145, 146, 39, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 136,
    158, 159, 0, 58, 141, 160, 53, 161, 163, 55, 165, 166, 167, 168, 169, 170,
    171, 172, 130, 46, 0, 126, 127, 0, 173, 166, 0, 174, 165, 0, 175, 176,
    56, 178, 179, 180, 181, 182, 183, 184, 185, 186, 59, 0, 126, 187, 0, 173,
    179, 0, 174, 178, 0, 175, 188, 57, 191, 193, 70, 0, 196, 58, 77, 0,
    124, 60, 197, 0, 48, 62, 199, 0, 202, 64, 77, 66, 199, 68, 203, 205,
    71, 208, 75, 85, 95, 105, 115, 125, 135, 145, 155, 165, 175, 185, 221, 225,
    0, 224, 227, 219, 0, 230, 213, 231, 195, 204, 0, 216, 231, 238, 240, 0,
    227, 222, 220, 242, 226, 0, 245, 247, 236, 246, 256, 0, 251, 252, 163, 270,
    266, 0, 165, 245, 268, 0, 58, 178, 61, 46, 256, 258, 88, 271, 199, 258,
    0, 258, 251, 260, 276, 0, 58, 141, 262, 274, 273, 152, 145, 157, 144, 151,
    156, 264, 265, 163, 277, 0, 264, 46, 287, 256, 291, 0, 270, 271, 0, 276,
    88, 289, 256, 0, 277, 88, 199, 278, 280, 294, 0, 291, 292, 96, 294, 295,
    292, 0, 296, 96, 297, 293, 298, 299, 300, 0, 264, 171, 301, 295, 303, 0,
    264, 305, 314, 0, 309, 310, 302, 299, 0, 312, 313, 316, 0, 318, 310, 306,
    303, 0, 312, 313, 316, 0, 310, 58, 0, 309, 319, 0, 322, 312, 307, 0,
    312, 313, 316, 0, 318, 319, 0, 322, 312, 313, 58, 310, 312, 319, 0, 324,
    325, 0, 264, 331, 312, 315, 334, 335, 280, 0, 336, 337, 339, 271, 0, 277,
    88, 312, 0, 309, 318, 0, 342, 316, 339, 270, 0, 343, 344, 0, 346, 317,
    347, 348, 0, 265, 349, 351, 230, 325, 0, 312, 313, 320, 353, 365, 0, 321,
    324, 146, 349, 366, 323, 0, 322, 369, 370, 242, 160, 371, 0, 276, 324, 372,
    88, 245, 0, 264, 165, 326, 245, 350, 0, 328, 376, 0, 324, 349, 378, 242,
    379, 330, 383, 385, 339, 0, 389, 336, 334, 245, 0, 58, 389, 383, 0, 264,
    349, 242, 346, 0, 385, 389, 342, 340, 245, 0, 309, 390, 0, 309, 392, 393,
    0, 309, 55, 50, 379, 0, 265, 345, 270, 0, 309, 394, 0, 270, 394, 0,
    392, 393, 0, 265, 331, 312, 347, 278, 396, 0, 265, 349, 351, 242, 397, 398,
    0, 400, 401, 0, 276, 349, 242, 88, 242, 0, 276, 88, 348, 245, 403, 0,
    309, 46, 256, 0, 264, 166, 351, 245, 355, 0, 277, 88, 312, 0, 309, 46,
    256, 271, 0, 276, 88, 353, 312, 359, 369, 378, 388, 398, 408, 418, 427, 437,
    447, 457, 467, 477, 487, 426, 427, 428, 429, 430, 431, 432, 433, 434, 435, 436,
    509, 489, 488, 96, 437, 439, 491, 500, 0, 512, 524, 522, 444, 0, 445, 527,
    379, 0, 372, 530, 540, 550, 560, 570, 580, 590, 600, 610, 620, 630, 640, 650,
    0, 459, 660, 670, 680, 690, 700, 710, 720, 730, 740, 756, 765, 353, 476, 477,
    203, 242, 437, 0, 379, 353, 86, 372, 766, 772, 777, 787, 797, 807, 817, 827,
    837, 847, 857, 867, 496, 497, 498, 499, 868, 875, 885, 895, 905, 504, 0, 505,
    505, 506, 0, 507, 507, 915, 924, 934, 944, 954, 964, 974, 513, 514, 975, 985,
    516, 517, 986, 998, 996, 353, 1003, 1001, 242, 1006, 1012, 521, 1015, 1025, 1035, 1045,
    1055, 1065, 1075, 1085, 1095, 1105, 1115, 1125, 231, 0, 230, 535, 372, 1128, 537, 1131,
    1141, 1151, 1161, 1171, 1181, 1191, 1209, 0, 544, 545, 1201, 548, 550, 552, 554, 552,
    552, 0, 544, 437, 1205, 548, 550, 552, 554, 552, 552, 499, 86, 556, 1210, 1220,
    1230, 1239, 1249, 1259, 1269, 1279, 1289, 1299, 1309, 1319, 0, 1329, 1339, 1349, 1359, 1369,
    1379, 1389, 1399, 1409, 1419, 1429, 1439, 1449, 1459, 1469, 1479, 1489, 1499, 1509, 1519, 1529,
    1539, 1549, 1559, 0, 1565, 1575, 1587, 316, 1585, 230, 591, 593, 1588, 1598, 1608, 1618,
    1628, 1638, 1648, 1658, 1668, 1678, 1687, 1696, 1705, 1715, 607, 353, 203, 476, 0, 1723,
    0, 0, 331, 77, 1722, 0, 392, 393, 0, 609, 1716, 437, 1719, 334, 335, 0,
    610, 1720, 334, 335, 0, 611, 612, 1721, 334, 335, 614, 390, 0, 230, 379, 1724,
    0, 1727, 383, 0, 615, 616, 0, 1731, 1741, 230, 591, 593, 0, 1743, 270, 0,
    309, 394, 0, 270, 394, 1747, 607, 353, 203, 476, 437, 1744, 537, 1748, 0, 627,
    0, 0, 1758, 1768, 1778, 1787, 1797, 0, 634, 316, 1807, 0, 634, 163, 0, 1820,
    0, 1817, 165, 166, 639, 245, 0, 349, 242, 0, 331, 77, 0, 163, 77, 0,
    1818, 640, 641, 642, 168, 79, 0, 81, 0, 46, 256, 0, 643, 312, 0, 1819,
    171, 644, 77, 0, 645, 1821, 316, 393, 1822, 1829, 1839, 1849, 1859, 1869, 313, 651,
    652, 653, 654, 655, 656, 657, 658, 1870, 1877, 1887, 1897, 1903, 1909, 0, 1924, 1935,
    0, 506, 506, 1945, 1955, 673, 1965, 675, 684, 685, 1966, 1976, 1986, 1996, 2006, 2016,
    2026, 2036, 2046, 2056, 2066, 2076, 2086, 2096, 2106, 2116, 0, 242, 700, 2126, 0, 2137,
    0, 709, 0, 372, 316, 0, 710, 0, 292, 316, 711, 2136, 0, 712, 0, 372,
    316, 0, 713, 712, 0, 714, 379, 0, 176, 316, 0, 393, 0, 372, 316, 0,
    715, 316, 0, 2141, 0, 723, 163, 724, 0, 724, 725, 0, 727, 316, 0, 728,
    729, 316, 0, 731, 55, 316, 0, 732, 339, 0, 733, 339, 0, 733, 734, 339,
    0, 735, 736, 700, 0, 738, 245, 2155, 2147, 2145, 739, 0, 349, 260, 2152, 2150,
    0, 242, 739, 90, 2163, 0, 744, 745, 746, 316, 0, 744, 316, 746, 0, 747,
    245, 2158, 2156, 242, 0, 2161, 316, 2164, 751, 0, 751, 752, 754, 755, 756, 2165,
    2175, 2185, 2195, 2205, 2215, 2225, 2235, 0, 122, 766, 2245, 379, 372, 379, 258, 767,
    2249, 170, 168, 169, 0, 2248, 772, 773, 774, 775, 79, 81, 0, 322, 77, 0,
    372, 776, 0, 122, 42, 378, 777, 0, 122, 174, 165, 378, 245, 0, 2257, 2255,
    2254, 0, 242, 379, 2250, 383, 385, 0, 779, 339, 325, 271, 372, 738, 2270, 2262,
    2260, 0, 242, 739, 0, 349, 260, 2267, 2265, 0, 242, 739, 90, 316, 2271, 751,
    0, 751, 752, 754, 755, 756, 0, 122, 780, 781, 168, 378, 79, 81, 0, 783,
    379, 784, 0, 122, 780, 610, 786, 378, 788, 163, 245, 0, 2274, 2272, 790, 2277,
    0, 122, 780, 792, 378, 77, 0, 795, 2295, 2293, 0, 312, 313, 2292, 353, 2289,
    2287, 316, 372, 537, 0, 122, 639, 378, 245, 2300, 2298, 0, 242, 385, 0, 779,
    339, 372, 537, 796, 221, 800, 803, 163, 2307, 2303, 221, 796, 0, 163, 805, 806,
    808, 316, 809, 316, 803, 800, 810, 0, 122, 48, 378, 199, 0, 122, 46, 378,
    256, 2308, 0, 813, 199, 0, 270, 271, 280, 2314, 0, 291, 292, 96, 0, 815,
    258, 96, 294, 295, 2312, 0, 296, 96, 297, 2313, 298, 299, 300, 0, 122, 171,
    301, 2315, 816, 0, 122, 2320, 0, 2319, 819, 744, 781, 165, 378, 245, 2322, 0,
    163, 820, 2321, 144, 822, 344, 0, 713, 639, 2331, 2328, 2326, 0, 242, 385, 0,
    779, 339, 271, 372, 0, 823, 2335, 850, 96, 851, 537, 796, 221, 221, 796, 0,
    122, 165, 378, 245, 0, 853, 245, 2336, 856, 280, 0, 122, 2341, 0, 2340, 819,
    744, 781, 165, 378, 245, 0, 2349, 2347, 2346, 0, 242, 379, 2342, 383, 385, 0,
    779, 339, 325, 2352, 221, 796, 810, 271, 280, 398, 2357, 0, 163, 820, 2356, 144,
    822, 344, 0, 713, 639, 0, 122, 155, 858, 2361, 861, 370, 0, 862, 863, 2365,
    2364, 145, 144, 0, 157, 863, 2362, 242, 865, 163, 865, 163, 163, 245, 2373, 0,
    869, 870, 872, 372, 312, 874, 872, 372, 312, 0, 90, 245, 2368, 0, 392, 876,
    0, 876, 2367, 0, 877, 878, 0, 877, 880, 0, 322, 882, 2369, 872, 885, 0,
    886, 2370, 437, 0, 129, 169, 887, 888, 0, 122, 124, 199, 0, 122, 780, 166,
    378, 245, 160, 372, 2380, 2377, 537, 890, 2381, 2391, 2401, 2411, 2421, 2431, 2441, 2449,
    0, 2459, 900, 901, 2460, 902, 2463, 2473, 2483, 2493, 2503, 2513, 2523, 2533, 2543, 2553,
    2563, 2573, 2583, 2593, 2603, 2613, 2623, 2633, 2643, 2653, 2663, 2682, 2673, 230, 365, 0,
    2700, 920, 921, 922, 924, 0, 926, 927, 0, 929, 930, 0, 933, 934, 936, 938,
    940, 942, 944, 945, 947, 949, 0, 2683, 955, 956, 957, 958, 934, 0, 2684, 961,
    962, 2685, 96, 963, 0, 966, 752, 934, 0, 967, 934, 2688, 970, 0, 966, 971,
    972, 973, 974, 2696, 2692, 2689, 437, 975, 977, 0, 2699, 0, 2697, 979, 970, 2698,
    981, 0, 93, 94, 0, 0, 230, 982, 983, 934, 2701, 2715, 988, 0, 2727, 2725,
    2724, 851, 989, 0, 2730, 921, 990, 991, 926, 2731, 927, 2742, 2752, 2762, 2797, 2806,
    2816, 2826, 2836, 2846, 2856, 2866, 2876, 2886, 2896, 2906, 2916, 2926, 2936, 2946, 2956, 2966,
    2976, 2986, 2996, 3006, 0, 270, 1014, 3016, 3026, 3036, 3046, 3056, 3066, 3076, 3086, 3096,
    3106, 0, 144, 1022, 1024, 3116, 1028, 3126, 3136, 3146, 3156, 3166, 3176, 3186, 3196, 3206,
    3216, 3226, 3236, 3246, 0, 1040, 365, 77, 3256, 3266, 3276, 3286, 3296, 3306, 3316, 3326,
    3336, 3346, 3356, 3366, 3376, 3386, 3396, 3406, 0, 1056, 316, 3416, 3426, 3436, 3446, 3456,
    3466, 3476, 3486, 3495, 3505, 3515, 3524, 3534, 396, 278, 0, 265, 766, 3535, 379, 372,
    379, 397, 0, 265, 42, 351, 777, 397, 0, 265, 168, 351, 79, 0, 265, 786,
    351, 788, 397, 3538, 0, 265, 792, 351, 77, 0, 265, 639, 351, 3548, 245, 397,
    0, 265, 48, 351, 230, 0, 265, 46, 351, 256, 397, 0, 265, 171, 301, 0,
    265, 781, 165, 351, 3550, 245, 397, 0, 265, 155, 351, 858, 0, 265, 50, 351,
    77, 397, 0, 265, 124, 351, 199, 0, 265, 166, 351, 245, 397, 3552, 3562, 3572,
    3582, 3592, 3602, 3612, 3622, 3632, 0, 550, 437, 3642, 3652, 3662, 0, 1074, 1076, 3665,
    3675, 3685, 3695, 3705, 0, 3728, 3739, 3749, 3759, 3768, 3778, 3788, 3798, 3808, 3818, 1088,
    3828, 3838, 3848, 3858, 3868, 3878, 3888, 3898, 3908, 3918, 3928, 3938, 3948, 3958, 3968, 3978,
    3988, 3998, 4008, 4018, 4028, 4038, 4048, 4058, 0, 1106, 4068, 537, 1107, 1109, 1111, 4069,
    1114, 0, 1115, 4080, 4079, 1116, 0, 556, 1115, 1119, 0, 645, 392, 1120, 1121, 0,
    0, 1124, 1114, 1125, 1127, 0, 392, 808, 1128, 809, 1115, 4084, 1131, 0, 1132, 1133,
    1134, 1135, 0, 1136, 4087, 4086, 0, 4085, 496, 497, 499, 1136, 4091, 1131, 0, 1132,
    1133, 4100, 0, 1137, 4092, 537, 0, 4095, 1138, 1139, 4096, 1141, 607, 86, 0, 4115,
    476, 203, 4105, 4104, 1143, 437, 537, 4102, 4101, 353, 242, 203, 1144, 437, 1146, 1147,
    0, 4108, 1074, 1148, 4111, 4109, 437, 0, 379, 4114, 316, 96, 1150, 1151, 1152, 1144,
    1153, 86, 1154, 4116, 1155, 353, 477, 1157, 1159, 242, 1161, 4117, 0, 4127, 4137, 4147,
    4157, 4167, 4177, 4187, 4197, 4208, 4218, 0, 1173, 4228, 369, 1175, 4229, 96, 437, 872,
    4230, 872, 344, 4231, 1177, 0, 258, 1178, 4232, 4242, 4252, 0, 4262, 1183, 4270, 1184,
    4266, 1183, 4272, 1183, 4276, 4286, 4296, 0, 1189, 4306, 0, 1190, 437, 4309, 4319, 4329,
    4339, 4349, 4359, 4369, 4379, 4389, 4399, 4409, 4419, 4429, 4439, 0, 51, 612, 4449, 4459,
    4469, 4479, 4489, 0, 1202, 4504, 4503, 0, 1204, 872, 0, 4501, 96, 4500, 0, 316,
    1205, 926, 0, 926, 4499, 927, 316, 851, 1206, 4502, 1208, 1209, 0, 808, 809, 4505,
    344, 806, 4506, 4516, 4526, 4536, 0, 90, 4546, 1214, 4548, 1190, 800, 0, 196, 292,
    0, 1223, 292, 1224, 1225, 1226, 1227, 890, 1229, 1173, 1231, 4561, 0, 1232, 4552, 4549,
    1233, 1235, 247, 4553, 1237, 1238, 1241, 1242, 0, 258, 1231, 247, 1238, 1244, 4558, 0,
    4554, 1242, 4580, 4577, 0, 4565, 1245, 1248, 4562, 1214, 0, 1223, 292, 0, 196, 292,
    4566, 4570, 0, 4569, 1242, 1241, 0, 1223, 292, 0, 196, 292, 4574, 4581, 4591, 4601,
    4611, 1252, 437, 4631, 437, 4615, 4612, 1254, 0, 437, 372, 4616, 379, 1152, 0, 1256,
    437, 90, 90, 437, 0, 4617, 851, 437, 4618, 0, 90, 437, 0, 322, 437, 0,
    1257, 4624, 1258, 4622, 1259, 1260, 0, 1263, 353, 0, 379, 4625, 316, 230, 242, 61,
    379, 4626, 316, 230, 242, 0, 1257, 4627, 316, 242, 437, 0, 394, 1264, 4629, 0,
    312, 313, 4628, 316, 1150, 96, 1266, 1268, 1271, 0, 4635, 989, 0, 372, 316, 0,
    45, 365, 4638, 4648, 0, 4658, 0, 230, 982, 520, 4662, 887, 1274, 520, 4665, 0,
    312, 4663, 0, 1275, 50, 379, 4664, 0, 1275, 50, 379, 4668, 4666, 1276, 4676, 0,
    0, 1277, 1278, 1279, 0, 0, 79, 4671, 1277, 1280, 1281, 888, 4672, 1283, 4677, 4687,
    4697, 4707, 4717, 4727, 4737, 4747, 4757, 4767, 4777, 4787, 4798, 4808, 4818, 4828, 4838, 0,
    251, 4848, 0, 1294, 163, 1296, 0, 48, 199, 0, 148, 163, 124, 197, 77, 88,
    1298, 4849, 0, 258, 251, 260, 0, 258, 1300, 260, 0, 738, 1204, 132, 0, 1302,
    292, 4850, 1303, 1304, 197, 4851, 4861, 0, 507, 1088, 507, 4871, 0, 196, 292, 0,
    4882, 4881, 242, 96, 437, 1307, 4884, 0, 1223, 292, 800, 1224, 1225, 1226, 1173, 4885,
    4895, 0, 4909, 4908, 242, 96, 437, 4905, 1307, 4911, 0, 1310, 1311, 4924, 4922, 4921,
    1313, 902, 4927, 4937, 4947, 0, 1224, 4960, 4957, 437, 4961, 0, 1223, 292, 800, 1225,
    1226, 1173, 4962, 4972, 4982, 4992, 5002, 5012, 5022, 5032, 5042, 5052, 5062, 5072, 5082, 5092,
    5102, 5112, 0, 5122, 5132, 5142, 5152, 5162, 5172, 5182, 5192, 5202, 5212, 5222, 5232, 5239,
    5249, 5259, 5269, 0, 1347, 1137, 5279, 0, 1347, 392, 1137, 5289, 0, 5299, 1348, 1349,
    1350, 5300, 5310, 5320, 5330, 5340, 5350, 5360, 5370, 0, 392, 61, 5386, 5383, 5382, 5380,
    1114, 537, 203, 5387, 5397, 5407, 0, 230, 5417, 1355, 1356, 5418, 5428, 5438, 5448, 5458,
    5468, 5478, 5488, 5498, 5508, 5518, 5528, 5538, 5548, 0, 145, 1367, 1369, 245, 5558, 537,
    0, 160, 537, 1371, 5559, 5569, 5579, 5589, 5599, 5609, 5619, 5629, 5639, 5649, 5659, 5668,
    5678, 5688, 5698, 5708, 0, 1381, 5718, 1383, 1175, 1384, 5719, 5729, 5739, 0, 926, 5751,
    0, 96, 5749, 316, 851, 316, 0, 316, 5750, 316, 851, 5752, 0, 1386, 0, 88,
    0, 1386, 934, 5762, 1387, 1388, 1389, 1390, 1391, 1392, 5763, 5773, 5783, 5793, 5803, 1151,
    1396, 1398, 1150, 1399, 0, 645, 392, 1257, 90, 5804, 1400, 5814, 5824, 5834, 5844, 5865,
    0, 1404, 1405, 1248, 5854, 1407, 0, 5858, 1409, 5861, 1411, 1413, 5862, 0, 1414, 1405,
    1248, 1409, 0, 1415, 1248, 5866, 0, 1416, 0, 0, 163, 5876, 5882, 5879, 437, 5883,
    5889, 1418, 1419, 1420, 0, 1421, 1418, 1422, 0, 5886, 1423, 1424, 1425, 337, 0, 1426,
    5888, 1422, 0, 5887, 1425, 1424, 1423, 337, 0, 394, 5892, 5890, 230, 5895, 5905, 5915,
    5925, 5935, 5945, 5955, 5965, 5973, 5983, 0, 1434, 96, 5993, 6003, 0, 6017, 312, 6014,
    6012, 312, 1438, 437, 6018, 6028, 6038, 6048, 6058, 0, 6068, 6078, 6088, 6098, 6108, 6118,
    6128, 6138, 0, 505, 1088, 505, 6148, 6158, 0, 0, 392, 1448, 1114, 0, 712, 1115,
    6168, 853, 1449, 1450, 6169, 6179, 0, 6191, 0, 6190, 1451, 1452, 389, 271, 6192, 0,
    1454, 6206, 6203, 6202, 96, 437, 58, 6208, 0, 1231, 6207, 96, 437, 0, 86, 96,
    1266, 6209, 6219, 6229, 0, 6239, 6249, 316, 96, 1150, 1132, 1151, 1152, 1458, 1460, 1462,
    6250, 6259, 6266, 6276, 6286, 0, 6296, 6306, 6316, 6326, 6336, 6346, 6356, 6366, 6376, 6386,
    6396, 6406, 6416, 6426, 6436, 6446, 6456, 6466, 6476, 0, 1481, 982, 230, 6489, 0, 792,
    77, 0, 86, 6486, 537, 6492, 6502, 6512, 0, 6522, 6532, 0, 6542, 6552, 6562, 6572,
    6582, 6592, 6602, 6612, 6622, 6632, 6642, 6652, 6662, 6672, 6682, 6692, 6702, 6712, 6722, 6732,
    6742, 144, 0, 145, 160, 1281, 0, 1503, 1369, 6752, 6762, 1504, 1506, 0, 886, 1508,
    0, 809, 437, 1509, 6766, 1511, 1513, 0, 886, 392, 1508, 0, 809, 437, 1509, 1515,
    0, 1517, 6767, 245, 1518, 394, 6771, 245, 1518, 0, 6768, 537, 247, 0, 203, 247,
    6772, 1411, 6775, 1519, 0, 157, 1521, 6778, 6788, 6798, 6808, 6818, 6828, 6838, 6848, 6858,
    6868, 6878, 6888, 6898, 6907, 6917, 6927, 6937, 6947, 6957, 6967, 6977, 6987, 6997, 7007, 7017,
    7027, 7037, 7046, 7056, 7066, 7076, 7086, 7096, 7106, 7116, 7126, 7135, 7145, 7155, 7165, 7175,
    0, 230, 372, 7188, 230, 7185, 1547, 0, 1226, 7189, 1549, 7191, 7201, 7211, 7221, 7232,
    7242, 0, 1426, 675, 7252, 7262, 7272, 7282, 0, 7291, 7301, 7311, 7321, 7328, 7338, 7348,
    7358, 7368, 7378, 7388, 7398, 7408, 7418, 7428, 7438, 7448, 7458, 7468, 7478, 7497, 1281, 1573,
    7488, 1155, 7491, 1574, 7494, 1576, 1577, 0, 7498, 1109, 1107, 1111, 1578, 7503, 1254, 7502,
    7499, 1155, 1576, 7504, 7514, 7524, 7534, 0, 7544, 7554, 0, 7564, 0, 1585, 1088, 0,
    505, 507, 7565, 1586, 7575, 7585, 7595, 7605, 7615, 7625, 7635, 7644, 1594, 7653, 7663, 7674,
    7684, 7694, 7704, 7714, 7724, 7734, 7744, 7754, 7765, 0, 316, 1598, 444, 7786, 7775, 1599,
    7777, 7789, 982, 0, 982, 982, 7790, 163, 372, 394, 86, 1205, 1602, 1603, 1604, 556,
    1598, 1183, 1241, 7791, 7801, 7811, 7821, 7831, 7841, 0, 1231, 7851, 96, 437, 872, 7852,
    872, 344, 7853, 7863, 7873, 7883, 7893, 7903, 7913, 7923, 7933, 7943, 7953, 7963, 7973, 7983,
    7993, 8003, 8013, 8023, 8033, 8043, 8053, 8063, 8073, 8083, 0, 865, 1625, 8092, 0, 1223,
    292, 8105, 0, 8102, 242, 96, 437, 8103, 1355, 1356, 0, 1350, 8104, 369, 1627, 1628,
    800, 1202, 8107, 800, 1224, 1225, 1226, 1202, 1263, 1173, 8108, 8118, 1223, 1630, 8119, 8129,
    8139, 8149, 8159, 8169, 8179, 0, 1268, 1637, 8192, 230, 8189, 1547, 8193, 0, 1639, 8206,
    0, 8203, 1152, 86, 1152, 242, 8207, 8217, 8227, 8237, 8247, 8257, 8267, 8277, 8287, 8297,
    8306, 8315, 8325, 8335, 8345, 8355, 8365, 8375, 8385, 8395, 8405, 0, 805, 292, 8430, 8419,
    0, 8415, 242, 437, 203, 607, 8427, 8425, 0, 8421, 242, 203, 607, 0, 196, 292,
    403, 803, 403, 0, 805, 8440, 0, 292, 8439, 8436, 8434, 437, 1654, 1655, 872, 0,
    1424, 1416, 8441, 8451, 8461, 8471, 8481, 8491, 8501, 8511, 8521, 8531, 8541, 8551, 8561, 8571,
    8581, 8591, 8601, 8611, 8621, 8643, 0, 1673, 8641, 8631, 1673, 316, 8644, 0, 8654, 8664,
    8674, 8684, 8694, 8704, 8714, 8724, 8734, 8743, 8753, 8763, 8773, 8783, 8792, 8802, 8812, 8822,
    8832, 8842, 8851, 1266, 1692, 1693, 0, 8855, 8865, 0, 8875, 8885, 8895, 8905, 8915, 8925,
    8935, 8945, 8955, 8965, 0, 611, 612, 8975, 8985, 8995, 9005, 9015, 9025, 9035, 9045, 9055,
    9065, 9075, 9085, 9095, 9105, 9115, 9125, 9135, 0, 1131, 96, 9145, 0, 499, 9155, 58,
    1275, 1714, 9156, 537, 9159, 9169, 9179, 9189, 9199, 9232, 0, 9210, 0, 9221, 9233, 9243,
    9253, 9263, 9273, 9283, 9292, 9301, 9311, 9320, 9329, 9339, 9349, 9359, 9369, 9379, 9389, 9399,
    9409, 9419, 0, 151, 245, 160, 1732, 9429, 0, 163, 144, 1733, 0, 163, 157, 1733,
    0, 1734, 9433, 1423, 1736, 1737, 9434, 9444, 9454, 9464, 396, 278, 0, 309, 393, 0,
    713, 1738, 0, 309, 270, 9465, 9475, 9485, 9495, 9505, 9515, 9525, 9535, 9545, 9555, 9565,
    9575, 9585, 9595, 9605, 9615, 9625, 9635, 9645, 9655, 9665, 9675, 9685, 9695, 9705, 9715, 9725,
    9735, 9745, 9755, 9765, 9775, 9785, 9795, 9805, 9815, 9825, 9835, 9845, 9855, 9865, 9875, 9885,
    9895, 9905, 9915, 0, 1774, 0, 251, 260, 322, 9925, 0, 1294, 163, 1296, 0, 48,
    199, 0, 148, 163, 124, 197, 197, 90, 1298, 397, 9926, 9936, 9946, 9956, 230, 9966,
    9976, 9986, 9994, 10004, 10014, 10024, 10034, 10044, 10054, 10064, 10074, 10084, 10094, 10104, 10114, 10124,
    10134, 10144, 10154, 10164, 10174, 10184, 10194, 10204, 0, 1793, 10214, 1795, 1796, 10215, 96, 0,
    10218, 1797, 10219, 96, 10222, 10232, 10242, 10252, 10262, 0, 10272, 1802, 1803, 1804, 46, 141,
    10273, 10283, 10293, 10303, 10313, 10323, 10333, 10343, 10353, 10363, 10373, 10383, 10393, 10403, 10413, 10423,
    10433, 0, 10450, 0, 1820, 58, 1655, 0, 1821, 292, 10449, 203, 10446, 10444, 0, 10443,
    242, 96, 437, 607, 58, 1655, 10451, 10461, 1822, 0, 444, 247, 10462, 1257, 58, 0,
    152, 1824, 10463, 1826, 1828, 10475, 90, 10465, 1190, 0, 1223, 292, 800, 1639, 1227, 1173,
    10476, 0, 1829, 1022, 1024, 803, 1831, 1161, 1833, 221, 1835, 800, 1837, 1833, 1227, 890,
    1229, 10497, 10489, 10486, 1574, 10493, 10490, 1840, 1841, 10494, 537, 10498, 10508, 10518, 10528, 10537,
    10547, 10557, 10567, 0, 1845, 96, 0, 713, 1845, 10568, 0, 1846, 96, 0, 713, 1846,
    10569, 10579, 10589, 10599, 10609, 10619, 10629, 0, 309, 10639, 1850, 0, 242, 313, 10641, 353,
    476, 203, 242, 437, 1281, 270, 0, 1576, 10642, 0, 1227, 1576, 221, 800, 1833, 10646,
    10657, 1852, 0, 10656, 1853, 1854, 58, 1855, 0, 309, 46, 378, 256, 0, 309, 10679,
    0, 10658, 1857, 1858, 1859, 1860, 1861, 88, 312, 0, 1862, 10659, 88, 313, 10662, 10660,
    312, 270, 0, 48, 10667, 1863, 270, 0, 58, 0, 1854, 10663, 312, 10665, 312, 0,
    990, 1864, 88, 10668, 312, 316, 0, 305, 1866, 313, 10669, 316, 312, 270, 0, 305,
    10678, 0, 813, 10670, 312, 270, 0, 1869, 88, 10671, 163, 1870, 0, 1871, 372, 1872,
    312, 0, 10672, 1874, 1875, 1876, 10673, 1863, 0, 313, 270, 316, 0, 10674, 1877, 1878,
    10675, 1863, 0, 313, 270, 316, 0, 1879, 88, 10676, 163, 1870, 0, 1880, 88, 10677,
    312, 270, 1863, 10680, 10690, 10700, 10710, 10720, 10730, 10740, 0, 10750, 1138, 1139, 10752, 0,
    10751, 1205, 1602, 379, 1141, 10756, 10766, 10776, 10786, 10796, 1890, 1891, 10797, 10807, 10817, 365,
    1274, 982, 10818, 230, 0, 230, 591, 1258, 10820, 10831, 10841, 10851, 10861, 0, 975, 208,
    10871, 10881, 10890, 10899, 10909, 10919, 10929, 10939, 10949, 10959, 10969, 10979, 10989, 10999, 11009, 11019,
    11029, 11039, 11049, 11059, 11069, 11079, 11089, 11099, 11109, 11119, 11129, 11139, 11149, 11159, 11169, 11179,
    11189, 11199, 11209, 11219, 11229, 11239, 11249, 11259, 11268, 11278, 11288, 11298, 11308, 11318, 537, 1578,
    1107, 1932, 1934, 1936, 1938, 1940, 1942, 1944, 1946, 1948, 1950, 1952, 1954, 1956, 1958, 1960,
    1962, 1964, 1966, 1968, 1970, 1111, 1109, 1972, 1974, 1976, 1978, 1980, 1982, 1984, 1986, 1988,
    1990, 1992, 1994, 1996, 1998, 2000, 2002, 2004, 2006, 2008, 2010, 2012, 2014, 2016, 2018, 2020,
    2022, 2024, 1183, 11319, 11329, 11339, 11349, 11359, 11369, 11379, 11389, 11399, 11409, 11419, 11429, 11439,
    11449, 11459, 2036, 11460, 11472, 11470, 0, 444, 247, 0, 11475, 11485, 11495, 11505, 11515, 11525,
    11535, 11545, 11555, 11565, 11575, 11585, 11595, 11605, 11612, 11622, 11632, 11642, 11652, 11662, 0, 0,
    331, 77, 11678, 0, 2055, 160, 11672, 334, 335, 0, 609, 11673, 437, 11676, 334, 335,
    0, 610, 160, 11677, 334, 335, 0, 51, 612, 160, 614, 0, 11685, 1281, 476, 203,
    245, 11679, 537, 11682, 1578, 11686, 11696, 11706, 11716, 11726, 0, 11736, 2057, 556, 2058, 0,
    11740, 2059, 2058, 11744, 11754, 11764, 11774, 0, 11784, 153, 154, 11785, 11795, 2062, 781, 11796,
    11806, 11816, 11826, 11836, 11846, 11856, 11866, 11876, 11886, 11895, 0, 11905, 990, 991, 934, 0,
    11906, 258, 2070, 990, 1864, 11907, 0, 823, 990, 1864, 437, 0, 2071, 247, 372, 316,
    1266, 11911, 11921, 11931, 11941, 11951, 11961, 11971, 11981, 11991, 12001, 12011, 12021, 12031, 12041, 12051,
    12061, 12071, 12081, 12091, 12101, 12111, 12121, 12131, 0, 12141, 815, 2086, 820, 2087, 552, 12142,
    1872, 2089, 0, 2090, 230, 0, 809, 713, 2091, 12143, 0, 2086, 12153, 0, 1107, 1028,
    0, 1109, 1028, 0, 1111, 1028, 0, 1155, 1028, 552, 12157, 12167, 12177, 12187, 12197, 12207,
    12217, 12227, 12237, 12247, 12257, 12267, 12277, 12287, 12297, 12307, 2104, 2105, 735, 12308, 12318, 12329,
    0, 156, 165, 245, 12339, 12349, 12359, 12371, 12369, 444, 12374, 0, 975, 1458, 0, 1074,
    2109, 12384, 12394, 12404, 12414, 12424, 12434, 12444, 0, 12454, 2114, 2115, 2116, 12455, 12465, 12475,
    12485, 12495, 0, 2122, 12505, 1257, 58, 12506, 0, 610, 12516, 12526, 0, 12536, 12546, 12556,
    12566, 12576, 0, 1829, 1022, 1024, 803, 1831, 1161, 1833, 1227, 890, 1229, 221, 800, 1837,
    0, 1576, 12586, 0, 1227, 1576, 12590, 12600, 12610, 12620, 0, 157, 245, 247, 309, 1521,
    1022, 1024, 12630, 12640, 12650, 12660, 0, 139, 777, 12670, 12680, 230, 12690, 12700, 12710, 12720,
    12730, 12740, 12750, 12760, 12770, 12780, 12790, 12800, 12810, 12820, 12830, 12840, 0, 12850, 1384, 1014,
    12857, 0, 872, 12854, 12852, 12851, 270, 353, 437, 12859, 12869, 12879, 12889, 12899, 12909, 12919,
    12929, 12939, 12949, 12959, 12969, 12979, 12989, 12999, 13009, 13019, 13029, 13039, 13049, 13059, 13069, 13079,
    13089, 13099, 0, 886, 0, 437, 13109, 1509, 13112, 437, 13115, 13118, 13121, 13131, 0, 1190,
    13144, 13141, 437, 13145, 800, 0, 196, 292, 0, 1223, 292, 1224, 1225, 1226, 1639, 1173,
    13146, 13156, 13166, 13173, 0, 84, 0, 13184, 13183, 0, 230, 591, 0, 1258, 982, 1258,
    13188, 0, 230, 13198, 805, 1223, 2168, 221, 2169, 0, 258, 2171, 13199, 13202, 2173, 152,
    13204, 2175, 0, 258, 2171, 13207, 13210, 2173, 152, 13212, 1841, 0, 258, 0, 713, 55,
    0, 13215, 0, 258, 713, 46, 2176, 0, 2177, 196, 13225, 221, 13228, 13238, 13248, 13255,
    13265, 13275, 13285, 13295, 13305, 13315, 13325, 13335, 13345, 13355, 13365, 13375, 13385, 13395, 13405, 13415,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 80, 83, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 162, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 194, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 78, 4294967295, 200, 4294967295, 204, 4294967295,
    4294967295, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 223, 4294967295,
    4294967295, 224, 229, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 240, 4294967295,
    229, 4294967295, 4294967295, 243, 4294967295, 4294967295, 246, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 257, 4294967295, 4294967295, 4294967295, 200, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 257, 4294967295, 4294967295, 4294967295, 275, 4294967295, 4294967295,
    4294967295, 4294967295, 257, 4294967295, 4294967295, 4294967295, 200, 4294967295, 281, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 302, 4294967295, 304, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    330, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 281, 4294967295, 4294967295, 4294967295, 4294967295, 275, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 367, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 377, 4294967295, 4294967295, 4294967295, 4294967295, 243,
    380, 4294967295, 384, 387, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 384, 4294967295, 4294967295,
    4294967295, 243, 4294967295, 4294967295, 387, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 391, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 399,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 243, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295,
    4294967295, 4294967295, 257, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    257, 275, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 411, 411, 4294967295, 411, 411,
    4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 478,
    204, 243, 438, 4294967295, 380, 4294967295, 4294967295, 4294967295, 479, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 411, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 519, 4294967295, 524, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 536, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 549, 4294967295, 4294967295, 555, 4294967295,
    4294967295, 4294967295, 4294967295, 438, 4294967295, 549, 4294967295, 4294967295, 555, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 608, 4294967295, 204, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 391, 4294967295, 4294967295, 380, 4294967295,
    4294967295, 4294967295, 384, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 608, 4294967295, 204, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 243, 4294967295, 4294967295, 78, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 83, 4294967295, 4294967295, 257, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 646, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 315,
    315, 315, 315, 315, 315, 4294967295, 4294967295, 315, 4294967295, 4294967295, 411, 315, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 707, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 707, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 740, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 243, 740, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    246, 4294967295, 4294967295, 243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 757, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295, 380, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 83, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 778, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 243, 380, 4294967295, 384, 387, 4294967295, 4294967295, 4294967295, 330, 275, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 243, 740, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 740, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 757, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 83, 4294967295, 4294967295,
    380, 785, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 789, 4294967295, 246, 4294967295, 4294967295, 4294967295, 791, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 243, 387, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 799, 223, 802, 804, 4294967295, 4294967295, 4294967295, 223, 799, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 804, 802, 812, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295,
    257, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 275, 281, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    302, 4294967295, 817, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 387, 4294967295,
    4294967295, 4294967295, 275, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 799, 223, 223, 799, 4294967295,
    4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 246, 4294967295, 857, 281, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 380, 4294967295, 384, 387, 4294967295,
    4294967295, 4294967295, 330, 4294967295, 223, 799, 812, 275, 281, 399, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 859, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 889, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 891, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 80, 80, 4294967295, 903, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 935, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 935, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 935, 4294967295, 4294967295, 935, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 976, 978, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 935, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1023, 1026, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295,
    380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 778, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295,
    4294967295, 789, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 257, 4294967295, 4294967295, 4294967295, 4294967295, 302, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 859, 4294967295, 4294967295, 4294967295, 4294967295,
    78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 1075, 1077, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1108, 1110, 1112, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1140, 4294967295, 1142, 608, 4294967295, 4294967295, 4294967295,
    4294967295, 204, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 243, 204, 1145, 438, 4294967295, 85,
    4294967295, 4294967295, 1075, 1149, 4294967295, 4294967295, 438, 4294967295, 380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1145,
    4294967295, 4294967295, 4294967295, 4294967295, 1156, 4294967295, 478, 1158, 1160, 243, 1163, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1185,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1215, 4294967295, 4294967295, 802, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1228, 891, 1230, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1234, 1236, 248, 4294967295, 4294967295, 1240, 4294967295, 1243, 4294967295, 4294967295, 4294967295, 248, 1240, 4294967295, 4294967295, 4294967295,
    4294967295, 1243, 4294967295, 4294967295, 4294967295, 4294967295, 1247, 1249, 4294967295, 1215, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    224, 4294967295, 4294967295, 4294967295, 1243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295,
    4294967295, 1253, 438, 4294967295, 438, 4294967295, 4294967295, 524, 4294967295, 438, 4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295,
    438, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 438, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1261, 4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295, 243, 4294967295,
    380, 4294967295, 4294967295, 4294967295, 243, 4294967295, 4294967295, 4294967295, 4294967295, 243, 438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 1267, 4294967295, 1158, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 519, 4294967295, 4294967295, 4294967295, 519, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 80, 889, 4294967295, 4294967295, 4294967295, 80, 4294967295, 80, 80, 1282, 889, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1295, 4294967295, 1297, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 198, 78, 4294967295,
    1299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 243, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 802, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1314, 903, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 802, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1341, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 935, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1249, 4294967295, 4294967295, 4294967295, 224, 1410, 4294967295, 1412, 4294967295, 240, 4294967295, 4294967295, 4294967295,
    1249, 1410, 4294967295, 4294967295, 1249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 438, 240,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 275, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295,
    1267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 1459, 1461, 1463,
    993, 993, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1282, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1505, 1507, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 438, 4294967295, 4294967295, 1512, 1514, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 1516,
    4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 204, 248,
    224, 1412, 240, 1520, 4294967295, 4294967295, 1522, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1548, 4294967295, 4294967295, 4294967295, 1550, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1560, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1282, 1156,
    4294967295, 1156, 4294967295, 1575, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1110, 1108, 1112, 1579, 4294967295, 524, 4294967295,
    4294967295, 1156, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1600,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1629,
    802, 4294967295, 4294967295, 802, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1548, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 243, 438, 204, 608, 4294967295, 4294967295, 4294967295, 4294967295, 243, 204, 608, 4294967295, 4294967295, 4294967295,
    4294967295, 804, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1267, 4294967295, 1695, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1295, 4294967295, 1297, 4294967295, 4294967295,
    200, 4294967295, 4294967295, 4294967295, 4294967295, 198, 198, 4294967295, 1299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295,
    243, 4294967295, 438, 608, 4294967295, 4294967295, 4294967295, 4294967295, 1823, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1825, 4294967295, 1827, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 802, 4294967295, 1228, 4294967295,
    4294967295, 4294967295, 1830, 1023, 1026, 804, 1832, 1163, 1834, 223, 1836, 802, 1839, 1834, 1228, 891,
    1230, 4294967295, 4294967295, 4294967295, 1575, 4294967295, 4294967295, 1575, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1851, 4294967295, 243, 315, 4294967295, 4294967295,
    4294967295, 204, 243, 438, 1282, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1228, 4294967295, 223, 802, 1834, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 257, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 315, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1140, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 380, 1142, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 976, 209,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1579,
    1108, 1933, 1935, 1937, 1939, 1941, 1943, 1945, 1947, 1949, 1951, 1953, 1955, 1957, 1959, 1961,
    1963, 1965, 1967, 1969, 1971, 1112, 1110, 1973, 1975, 1977, 1979, 1981, 1983, 1985, 1987, 1989,
    1991, 1993, 1995, 1997, 1999, 2001, 2003, 2005, 2007, 2009, 2011, 2013, 2015, 2017, 2019, 2021,
    2023, 2025, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 411, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1282, 4294967295, 204,
    246, 4294967295, 4294967295, 4294967295, 1579, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 935, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 248, 4294967295, 4294967295,
    1267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1108, 4294967295,
    4294967295, 1110, 4294967295, 4294967295, 1112, 4294967295, 4294967295, 1156, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 976, 1459, 4294967295, 1075,
    1077, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1830, 1023, 1026, 804, 1832, 1163, 1834, 1228, 891, 1230, 223, 802, 1839,
    4294967295, 4294967295, 4294967295, 4294967295, 1228, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 248, 4294967295, 1522,
    1023, 1026, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 778, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 438, 224, 4294967295, 224, 438, 240, 240, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 438, 4294967295, 802, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2165, 4294967295, 4294967295, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 399, 223, 2170, 4294967295, 4294967295, 4294967295, 224, 4294967295, 2174, 4294967295,
    240, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 2174, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 223, 4294967295, 4294967295, 5, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 66, 4294967295, 4294967295, 4294967295, 4294967295, 66, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 79, 81, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 161, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 191, 193, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 77, 4294967295, 199, 4294967295, 203, 4294967295,
    4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295,
    4294967295, 225, 227, 4294967295, 4294967295, 4294967295, 4294967295, 231, 4294967295, 4294967295, 4294967295, 4294967295, 231, 4294967295, 241, 4294967295,
    227, 4294967295, 4294967295, 242, 4294967295, 4294967295, 245, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 252, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 199, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 271, 4294967295, 4294967295,
    4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 199, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 301, 4294967295, 303, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 271, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 366, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 376, 4294967295, 4294967295, 4294967295, 4294967295, 242,
    379, 4294967295, 383, 385, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 383, 4294967295, 4294967295,
    4294967295, 242, 4294967295, 4294967295, 385, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 390, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 398,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    256, 271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 432, 433, 4294967295, 435, 436,
    4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 477,
    203, 242, 437, 4294967295, 379, 4294967295, 4294967295, 4294967295, 480, 481, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 412, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 520, 4294967295, 521, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 231, 4294967295, 4294967295, 535, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 546, 4294967295, 548, 4294967295, 4294967295, 554, 4294967295,
    4294967295, 4294967295, 4294967295, 437, 4294967295, 548, 4294967295, 4294967295, 554, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 475, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 607, 4294967295, 203, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 390, 4294967295, 4294967295, 379, 4294967295,
    4294967295, 4294967295, 383, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 607, 4294967295, 203, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 242, 4294967295, 4294967295, 77, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 81, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 647, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 313, 651,
    652, 653, 654, 655, 656, 4294967295, 4294967295, 659, 4294967295, 4294967295, 662, 663, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 700, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 700, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 739, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 242, 739, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    245, 4294967295, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 756, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 379, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 81, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 777, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 242, 379, 4294967295, 383, 385, 4294967295, 4294967295, 4294967295, 325, 271, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 242, 739, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 739, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 756, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 81, 4294967295, 4294967295,
    379, 784, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 788, 4294967295, 245, 4294967295, 4294967295, 4294967295, 790, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 313, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 242, 385, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 796, 221, 800, 803, 4294967295, 4294967295, 4294967295, 221, 796, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 803, 800, 810, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295,
    256, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 271, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    301, 4294967295, 816, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 385, 4294967295,
    4294967295, 4294967295, 271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 796, 221, 221, 796, 4294967295,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 245, 4294967295, 856, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 379, 4294967295, 383, 385, 4294967295,
    4294967295, 4294967295, 325, 4294967295, 221, 796, 810, 271, 280, 398, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 858, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 888, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 890, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 900, 901, 4294967295, 902, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 934, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 934, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 934, 4294967295, 4294967295, 934, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 975, 977, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 934, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1022, 1024, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295,
    379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 777, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295,
    4294967295, 788, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 301, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 858, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 1074, 1076, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1107, 1109, 1111, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1132, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1132,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1139, 4294967295, 1141, 607, 4294967295, 4294967295, 4294967295,
    4294967295, 203, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 242, 203, 1144, 437, 4294967295, 1147,
    4294967295, 4294967295, 1074, 1148, 4294967295, 4294967295, 437, 4294967295, 379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1144,
    4294967295, 4294967295, 4294967295, 4294967295, 1155, 4294967295, 477, 1157, 1159, 242, 1161, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1184,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1214, 4294967295, 4294967295, 800, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1227, 890, 1229, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1233, 1235, 247, 4294967295, 4294967295, 1238, 4294967295, 1242, 4294967295, 4294967295, 4294967295, 247, 1238, 4294967295, 4294967295, 4294967295,
    4294967295, 1242, 4294967295, 4294967295, 4294967295, 4294967295, 1245, 1248, 4294967295, 1214, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    225, 4294967295, 4294967295, 4294967295, 1242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295,
    4294967295, 1252, 437, 4294967295, 437, 4294967295, 4294967295, 1254, 4294967295, 437, 4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295,
    437, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 437, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1260, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295, 242, 4294967295,
    379, 4294967295, 4294967295, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 242, 437, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 313, 4294967295, 4294967295, 4294967295, 4294967295, 1266, 4294967295, 1271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 520, 4294967295, 4294967295, 4294967295, 520, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1277, 1278, 4294967295, 4294967295, 4294967295, 79, 4294967295, 1277, 1280, 1281, 888, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1294, 4294967295, 1296, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 197, 77, 4294967295,
    1298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 242, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 800, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1313, 902, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 800, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1342, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 934, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1248, 4294967295, 4294967295, 4294967295, 225, 1409, 4294967295, 1411, 4294967295, 241, 4294967295, 4294967295, 4294967295,
    1248, 1409, 4294967295, 4294967295, 1248, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 546, 4294967295, 4294967295, 437, 241,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 271, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295,
    1266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1132, 4294967295, 4294967295, 1458, 1460, 1462,
    1169, 994, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1281, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1504, 1506, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 437, 4294967295, 4294967295, 1511, 1513, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 1515,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 247, 4294967295, 203, 247,
    225, 1411, 241, 1519, 4294967295, 4294967295, 1521, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1547, 4294967295, 4294967295, 4294967295, 1549, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1561, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1281, 1573,
    4294967295, 1155, 4294967295, 1574, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1109, 1107, 1111, 1578, 4294967295, 1254, 4294967295,
    4294967295, 1155, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1599,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1628,
    800, 4294967295, 4294967295, 800, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1547, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 242, 437, 203, 607, 4294967295, 4294967295, 4294967295, 4294967295, 242, 203, 607, 4294967295, 4294967295, 4294967295,
    4294967295, 803, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1266, 4294967295, 1693, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1294, 4294967295, 1296, 4294967295, 4294967295,
    199, 4294967295, 4294967295, 4294967295, 4294967295, 197, 197, 4294967295, 1298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295,
    242, 4294967295, 437, 607, 4294967295, 4294967295, 4294967295, 4294967295, 1822, 4294967295, 4294967295, 247, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1824, 4294967295, 1826, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 800, 4294967295, 1227, 4294967295,
    4294967295, 4294967295, 1829, 1022, 1024, 803, 1831, 1161, 1833, 221, 1835, 800, 1837, 1833, 1227, 890,
    1229, 4294967295, 4294967295, 4294967295, 1574, 4294967295, 4294967295, 1840, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1850, 4294967295, 242, 313, 4294967295, 4294967295,
    4294967295, 203, 242, 437, 1281, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1227, 4294967295, 221, 800, 1833, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 313, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1139, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 379, 1141, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 975, 208,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1578,
    1107, 1932, 1934, 1936, 1938, 1940, 1942, 1944, 1946, 1948, 1950, 1952, 1954, 1956, 1958, 1960,
    1962, 1964, 1966, 1968, 1970, 1111, 1109, 1972, 1974, 1976, 1978, 1980, 1982, 1984, 1986, 1988,
    1990, 1992, 1994, 1996, 1998, 2000, 2002, 2004, 2006, 2008, 2010, 2012, 2014, 2016, 2018, 2020,
    2022, 2024, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2036, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1281, 4294967295, 203,
    245, 4294967295, 4294967295, 4294967295, 1578, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 934, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 247, 4294967295, 4294967295,
    1266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1107, 4294967295,
    4294967295, 1109, 4294967295, 4294967295, 1111, 4294967295, 4294967295, 1155, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 975, 1458, 4294967295, 1074,
    2109, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1829, 1022, 1024, 803, 1831, 1161, 1833, 1227, 890, 1229, 221, 800, 1837,
    4294967295, 4294967295, 4294967295, 4294967295, 1227, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 247, 4294967295, 1521,
    1022, 1024, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 777, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 437, 225, 4294967295, 546, 437, 241, 241, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 437, 4294967295, 800, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2166, 4294967295, 4294967295, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2168, 221, 2169, 4294967295, 4294967295, 4294967295, 225, 4294967295, 2173, 4294967295,
    241, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 2173, 4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 4294967295, 1272, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (2894, 1),
    (3288, 1),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (38, 0, 2),
    (41, 2, 2),
    (46, 4, 2),
    (53, 6, 2),
    (55, 8, 2),
    (56, 10, 2),
    (57, 12, 3),
    (99, 15, 2),
    (135, 17, 2),
    (184, 19, 2),
    (185, 21, 2),
    (190, 23, 2),
    (194, 25, 3),
    (198, 28, 3),
    (202, 31, 2),
    (204, 33, 3),
    (206, 36, 2),
    (209, 38, 2),
    (222, 40, 2),
    (225, 42, 4),
    (226, 46, 2),
    (231, 48, 2),
    (236, 50, 2),
    (238, 52, 5),
    (240, 57, 2),
    (243, 59, 3),
    (246, 62, 3),
    (247, 65, 2),
    (253, 67, 2),
    (259, 69, 3),
    (266, 72, 3),
    (270, 75, 3),
    (297, 78, 3),
    (301, 81, 2),
    (306, 83, 3),
    (310, 86, 3),
    (312, 89, 2),
    (332, 91, 3),
    (334, 94, 2),
    (346, 96, 2),
    (355, 98, 2),
    (369, 100, 2),
    (384, 102, 2),
    (392, 104, 2),
    (397, 106, 2),
    (426, 108, 2),
    (435, 110, 2),
    (441, 112, 3),
    (449, 115, 3),
    (454, 118, 3),
    (458, 121, 2),
    (463, 123, 3),
    (464, 126, 2),
    (466, 128, 2),
    (467, 130, 2),
    (473, 132, 3),
    (477, 135, 2),
    (481, 137, 3),
    (484, 140, 2),
    (488, 142, 3),
    (491, 145, 2),
    (500, 147, 2),
    (525, 149, 3),
    (527, 152, 2),
    (534, 154, 3),
    (536, 157, 3),
    (541, 160, 3),
    (546, 163, 3),
    (551, 166, 3),
    (560, 169, 3),
    (561, 172, 2),
    (587, 174, 2),
    (588, 176, 2),
    (590, 178, 2),
    (591, 180, 2),
    (596, 182, 2),
    (608, 184, 2),
    (639, 186, 2),
    (640, 188, 2),
    (641, 190, 3),
    (642, 193, 2),
    (644, 195, 2),
    (648, 197, 2),
    (649, 199, 1),
    (664, 200, 3),
    (696, 203, 3),
    (697, 206, 2),
    (699, 208, 2),
    (712, 210, 2),
    (715, 212, 2),
    (729, 214, 4),
    (731, 218, 2),
    (734, 220, 2),
    (739, 222, 2),
    (741, 224, 2),
    (744, 226, 2),
    (786, 228, 2),
    (810, 230, 2),
    (812, 232, 2),
    (819, 234, 2),
    (827, 236, 2),
    (843, 238, 2),
    (846, 240, 2),
    (850, 242, 2),
    (870, 244, 2),
    (872, 246, 2),
    (874, 248, 2),
    (901, 250, 3),
    (904, 253, 3),
    (907, 256, 2),
    (910, 258, 2),
    (917, 260, 2),
    (919, 262, 2),
    (922, 264, 3),
    (930, 267, 2),
    (936, 269, 3),
    (942, 272, 2),
    (943, 274, 2),
    (944, 276, 2),
    (945, 278, 2),
    (946, 280, 2),
    (947, 282, 2),
    (948, 284, 2),
    (951, 286, 3),
    (954, 289, 2),
    (955, 291, 2),
    (987, 293, 3),
    (988, 296, 2),
    (1014, 298, 2),
    (1059, 300, 2),
    (1062, 302, 3),
    (1066, 305, 2),
    (1073, 307, 3),
    (1074, 310, 2),
    (1088, 312, 3),
    (1091, 315, 3),
    (1102, 318, 2),
    (1115, 320, 2),
    (1117, 322, 2),
    (1130, 324, 2),
    (1131, 326, 2),
    (1134, 328, 2),
    (1142, 330, 3),
    (1148, 333, 3),
    (1154, 336, 3),
    (1155, 339, 2),
    (1157, 341, 2),
    (1158, 343, 2),
    (1162, 345, 2),
    (1163, 347, 2),
    (1170, 349, 3),
    (1171, 352, 2),
    (1178, 354, 3),
    (1179, 357, 2),
    (1189, 359, 2),
    (1196, 361, 2),
    (1197, 363, 2),
    (1200, 365, 2),
    (1201, 367, 2),
    (1208, 369, 3),
    (1210, 372, 3),
    (1214, 375, 2),
    (1221, 377, 2),
    (1228, 379, 2),
    (1240, 381, 3),
    (1244, 384, 3),
    (1245, 387, 2),
    (1251, 389, 2),
    (1252, 391, 2),
    (1253, 393, 2),
    (1254, 395, 2),
    (1258, 397, 2),
    (1259, 399, 2),
    (1268, 401, 2),
    (1269, 403, 2),
    (1270, 405, 2),
    (1275, 407, 3),
    (1280, 410, 3),
    (1284, 413, 3),
    (1287, 416, 2),
    (1288, 418, 2),
    (1312, 420, 3),
    (1314, 423, 2),
    (1325, 425, 3),
    (1341, 428, 3),
    (1342, 431, 2),
    (1346, 433, 2),
    (1355, 435, 2),
    (1356, 437, 2),
    (1357, 439, 2),
    (1358, 441, 2),
    (1363, 443, 3),
    (1366, 446, 3),
    (1368, 449, 2),
    (1369, 451, 2),
    (1380, 453, 3),
    (1386, 456, 3),
    (1387, 459, 2),
    (1389, 461, 2),
    (1390, 463, 2),
    (1394, 465, 2),
    (1396, 467, 2),
    (1397, 469, 2),
    (1398, 471, 2),
    (1399, 473, 2),
    (1400, 475, 2),
    (1401, 477, 2),
    (1416, 479, 3),
    (1431, 482, 3),
    (1437, 485, 3),
    (1451, 488, 3),
    (1474, 491, 2),
    (1479, 493, 2),
    (1483, 495, 3),
    (1489, 498, 3),
    (1495, 501, 2),
    (1506, 503, 2),
    (1507, 505, 2),
    (1509, 507, 2),
    (1549, 509, 2),
    (1564, 511, 2),
    (1575, 513, 2),
    (1578, 515, 2),
    (1590, 517, 2),
    (1591, 519, 2),
    (1592, 521, 2),
    (1609, 523, 2),
    (1667, 525, 2),
    (1668, 527, 2),
    (1687, 529, 2),
    (1726, 531, 2),
    (1728, 533, 2),
    (1734, 535, 3),
    (1740, 538, 2),
    (1745, 540, 3),
    (1752, 543, 2),
    (1758, 545, 3),
    (1769, 548, 3),
    (1774, 551, 3),
    (1781, 554, 3),
    (1787, 557, 3),
    (1792, 560, 2),
    (1798, 562, 3),
    (1803, 565, 3),
    (1816, 568, 2),
    (1821, 570, 2),
    (1822, 572, 2),
    (1868, 574, 2),
    (1869, 576, 2),
    (1870, 578, 2),
    (1902, 580, 2),
    (1919, 582, 2),
    (1929, 584, 2),
    (1931, 586, 2),
    (1932, 588, 2),
    (1937, 590, 2),
    (1941, 592, 2),
    (1946, 594, 3),
    (1947, 597, 2),
    (1948, 599, 2),
    (1949, 601, 2),
    (1951, 603, 3),
    (1954, 606, 2),
    (1955, 608, 2),
    (1958, 610, 2),
    (1960, 612, 2),
    (1967, 614, 2),
    (1972, 616, 2),
    (1974, 618, 2),
    (1975, 620, 2),
    (1976, 622, 2),
    (1977, 624, 3),
    (1978, 627, 2),
    (1998, 629, 2),
    (2015, 631, 2),
    (2028, 633, 2),
    (2089, 635, 2),
    (2092, 637, 2),
    (2102, 639, 2),
    (2103, 641, 2),
    (2104, 643, 2),
    (2112, 645, 2),
    (2113, 647, 2),
    (2114, 653, 2),
    (2117, 649, 2),
    (2119, 651, 2),
    (2123, 655, 2),
    (2124, 657, 2),
    (2129, 659, 2),
    (2134, 661, 2),
    (2135, 663, 2),
    (2137, 665, 2),
    (2148, 667, 2),
    (2161, 669, 2),
    (2162, 671, 2),
    (2164, 673, 2),
    (2167, 675, 2),
    (2169, 677, 2),
    (2172, 679, 2),
    (2176, 681, 2),
    (2179, 683, 2),
    (2183, 685, 2),
    (2187, 687, 2),
    (2190, 689, 2),
    (2197, 691, 2),
    (2202, 693, 2),
    (2206, 695, 3),
    (2208, 698, 2),
    (2212, 700, 3),
    (2217, 703, 3),
    (2218, 706, 2),
    (2225, 708, 2),
    (2230, 710, 2),
    (2232, 712, 2),
    (2249, 714, 2),
    (2253, 716, 2),
    (2261, 718, 2),
    (2266, 720, 2),
    (2273, 722, 2),
    (2274, 724, 2),
    (2278, 732, 2),
    (2280, 726, 2),
    (2281, 728, 2),
    (2282, 730, 2),
    (2283, 734, 2),
    (2307, 736, 2),
    (2309, 738, 2),
    (2312, 740, 3),
    (2317, 743, 3),
    (2318, 746, 2),
    (2320, 748, 2),
    (2340, 750, 3),
    (2354, 753, 3),
    (2356, 756, 2),
    (2362, 758, 2),
    (2372, 760, 3),
    (2374, 763, 2),
    (2384, 765, 2),
    (2385, 767, 2),
    (2393, 769, 2),
    (2398, 771, 2),
    (2430, 773, 3),
    (2466, 776, 2),
    (2493, 778, 3),
    (2546, 781, 2),
    (2579, 783, 2),
    (2584, 785, 2),
    (2586, 787, 2),
    (2592, 789, 2),
    (2593, 791, 2),
    (2596, 793, 2),
    (2606, 795, 2),
    (2644, 797, 4),
    (2659, 801, 2),
    (2701, 803, 2),
    (2709, 805, 2),
    (2716, 807, 2),
    (2720, 809, 2),
    (2730, 811, 2),
    (2733, 813, 2),
    (2734, 815, 2),
    (2735, 817, 2),
    (2736, 819, 5),
    (2737, 824, 3),
    (2768, 827, 2),
    (2805, 829, 2),
    (2811, 831, 2),
    (2812, 833, 2),
    (2818, 835, 2),
    (2821, 837, 2),
    (2822, 839, 2),
    (2829, 841, 2),
    (2831, 843, 2),
    (2835, 845, 3),
    (2839, 848, 3),
    (2844, 851, 2),
    (2846, 853, 2),
    (2847, 855, 2),
    (2849, 857, 2),
    (2851, 859, 2),
    (2854, 861, 2),
    (2902, 863, 2),
    (2906, 865, 2),
    (2924, 867, 3),
    (2942, 870, 2),
    (2943, 872, 2),
    (2945, 874, 2),
    (2947, 876, 2),
    (2953, 878, 2),
    (2954, 880, 2),
    (2955, 882, 2),
    (2956, 884, 2),
    (2958, 886, 2),
    (2961, 888, 2),
    (3007, 890, 2),
    (3037, 892, 2),
    (3076, 894, 3),
    (3078, 897, 2),
    (3087, 899, 2),
    (3088, 901, 2),
    (3091, 903, 2),
    (3115, 905, 2),
    (3125, 907, 3),
    (3154, 910, 3),
    (3155, 913, 2),
    (3156, 915, 2),
    (3157, 917, 2),
    (3162, 919, 3),
    (3163, 922, 2),
    (3164, 924, 2),
    (3169, 926, 2),
    (3179, 928, 2),
    (3235, 930, 2),
    (3237, 932, 2),
    (3316, 934, 3),
    (3403, 937, 2),
    (3405, 939, 2),
    (3408, 941, 3),
    (3413, 944, 3),
    (3414, 947, 3),
    (3416, 950, 2),
    (3499, 952, 2),
    (3504, 954, 3),
    (3506, 957, 2),
    (3507, 959, 2),
    (3512, 961, 2),
    (3515, 963, 2),
    (3521, 965, 2),
    (3523, 967, 2),
    (3532, 969, 2),
    (3534, 971, 2),
    (3538, 973, 2),
    (3539, 975, 2),
    (3540, 977, 2),
    (3541, 979, 2),
    (3542, 981, 2),
    (3543, 983, 2),
    (3544, 985, 2),
    (3545, 987, 2),
    (3546, 989, 2),
    (3547, 991, 2),
    (3548, 993, 2),
    (3549, 995, 2),
    (3550, 997, 2),
    (3551, 999, 2),
    (3552, 1001, 2),
    (3556, 1003, 2),
    (3559, 1005, 2),
    (3594, 1007, 2),
    (3596, 1009, 3),
    (3597, 1012, 2),
    (3601, 1014, 2),
    (3602, 1016, 3),
    (3603, 1019, 2),
    (3604, 1021, 2),
    (3610, 1023, 2),
    (3612, 1025, 2),
    (3613, 1027, 2),
    (3614, 1029, 2),
    (3628, 1031, 3),
    (3645, 1034, 2),
    (3673, 1036, 2),
    (3705, 1038, 2),
    (3715, 1040, 2),
    (3741, 1042, 2),
    (3747, 1044, 2),
    (3748, 1046, 2),
    (3774, 1048, 2),
    (3775, 1050, 2),
    (3823, 1052, 2),
    (3824, 1054, 2),
    (3825, 1056, 2),
    (3826, 1058, 2),
    (3827, 1060, 2),
    (3828, 1062, 2),
    (3829, 1064, 2),
    (3830, 1066, 2),
    (3831, 1068, 2),
    (3832, 1070, 2),
    (3833, 1072, 2),
    (3834, 1074, 2),
    (3835, 1076, 2),
    (3836, 1078, 2),
    (3837, 1080, 2),
    (3838, 1082, 2),
    (3839, 1084, 2),
    (3840, 1086, 2),
    (3841, 1088, 2),
    (3842, 1090, 2),
    (3843, 1092, 2),
    (3844, 1094, 2),
    (3845, 1096, 2),
    (3846, 1098, 2),
    (3847, 1100, 2),
    (3848, 1102, 2),
    (3849, 1104, 2),
    (3850, 1106, 2),
    (3851, 1108, 2),
    (3852, 1110, 2),
    (3853, 1112, 2),
    (3854, 1114, 2),
    (3855, 1116, 2),
    (3856, 1118, 2),
    (3857, 1120, 2),
    (3858, 1122, 2),
    (3859, 1124, 2),
    (3860, 1126, 2),
    (3861, 1128, 2),
    (3862, 1130, 2),
    (3863, 1132, 2),
    (3864, 1134, 2),
    (3865, 1136, 2),
    (3866, 1138, 2),
    (3867, 1140, 2),
    (3868, 1142, 2),
    (3869, 1144, 2),
    (3870, 1146, 2),
    (3871, 1148, 2),
    (3872, 1150, 2),
    (3873, 1152, 2),
    (3890, 1154, 2),
    (3896, 1156, 2),
    (3911, 1158, 3),
    (3921, 1161, 2),
    (3932, 1163, 2),
    (3949, 1165, 2),
    (3951, 1167, 2),
    (3952, 1169, 3),
    (3956, 1172, 2),
    (3998, 1174, 2),
    (4010, 1176, 2),
    (4013, 1178, 2),
    (4016, 1180, 2),
    (4062, 1182, 2),
    (4065, 1184, 2),
    (4068, 1186, 2),
    (4071, 1188, 2),
    (4099, 1190, 3),
    (4108, 1193, 2),
    (4109, 1195, 2),
    (4111, 1197, 2),
    (4112, 1199, 2),
    (4147, 1201, 2),
    (4148, 1203, 2),
    (4149, 1205, 2),
    (4150, 1207, 2),
    (4151, 1209, 2),
    (4152, 1211, 2),
    (4153, 1213, 2),
    (4154, 1215, 2),
    (4155, 1217, 2),
    (4156, 1219, 2),
    (4157, 1221, 2),
    (4158, 1223, 2),
    (4159, 1225, 2),
    (4164, 1227, 2),
    (4172, 1229, 3),
    (4173, 1232, 2),
    (4175, 1234, 2),
    (4176, 1236, 2),
    (4177, 1238, 2),
    (4184, 1240, 3),
    (4216, 1243, 2),
    (4245, 1245, 2),
    (4249, 1247, 2),
    (4258, 1249, 2),
    (4260, 1251, 2),
    (4274, 1253, 3),
    (4277, 1256, 3),
    (4294, 1259, 2),
    (4295, 1261, 2),
    (4296, 1263, 2),
    (4302, 1265, 2),
    (4311, 1267, 2),
    (4331, 1269, 2),
    (4334, 1271, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    65, 3, 76, 3, 76, 3, 3, 78, 3, 80, 3, 83, 3, 78, 85, 65,
    3, 162, 3, 3, 192, 3, 194, 3, 78, 3, 78, 198, 3, 78, 200, 3,
    78, 3, 78, 200, 3, 204, 209, 3, 3, 223, 3, 224, 226, 4, 229, 3,
    3, 232, 3, 232, 3, 240, 224, 226, 4, 229, 3, 3, 243, 78, 3, 78,
    246, 248, 3, 255, 3, 3, 78, 246, 3, 78, 257, 3, 78, 200, 3, 78,
    257, 3, 275, 3, 78, 257, 3, 78, 200, 3, 281, 3, 78, 302, 304, 3,
    3, 315, 3, 315, 3, 315, 3, 330, 3, 281, 3, 275, 3, 315, 3, 367,
    3, 243, 78, 3, 78, 246, 3, 78, 246, 377, 3, 3, 243, 78, 3, 380,
    3, 384, 3, 387, 3, 78, 246, 3, 384, 3, 243, 78, 3, 387, 3, 78,
    246, 3, 391, 3, 380, 3, 243, 78, 3, 399, 3, 243, 78, 3, 243, 78,
    3, 78, 246, 3, 78, 257, 3, 78, 246, 3, 78, 257, 3, 275, 3, 411,
    3, 411, 3, 411, 3, 411, 3, 438, 3, 380, 3, 478, 3, 204, 3, 243,
    78, 3, 438, 3, 380, 3, 479, 3, 3, 411, 4, 3, 243, 78, 3, 519,
    3, 524, 3, 232, 3, 536, 3, 224, 226, 4, 3, 549, 3, 555, 3, 438,
    3, 549, 3, 555, 3, 4, 3, 608, 3, 204, 3, 78, 3, 438, 3, 391,
    3, 380, 3, 384, 3, 608, 3, 204, 3, 438, 3, 78, 246, 3, 243, 78,
    3, 78, 3, 78, 3, 80, 3, 83, 3, 78, 257, 3, 78, 3, 646, 4,
    3, 315, 3, 315, 3, 315, 3, 315, 3, 315, 3, 315, 3, 315, 3, 315,
    4, 3, 411, 3, 315, 3, 243, 78, 3, 707, 3, 380, 3, 707, 3, 78,
    246, 3, 740, 3, 243, 78, 3, 740, 3, 78, 246, 3, 243, 78, 3, 757,
    3, 380, 3, 380, 3, 80, 3, 83, 3, 78, 3, 778, 78, 3, 78, 246,
    3, 243, 78, 3, 380, 3, 384, 3, 387, 3, 330, 3, 275, 3, 243, 78,
    3, 740, 3, 243, 78, 3, 740, 3, 757, 3, 80, 3, 83, 3, 380, 3,
    785, 3, 789, 78, 3, 78, 246, 3, 791, 3, 78, 3, 315, 3, 78, 246,
    3, 243, 78, 3, 387, 3, 799, 3, 223, 3, 802, 3, 804, 3, 223, 3,
    799, 3, 804, 3, 802, 3, 812, 3, 78, 200, 3, 78, 257, 3, 78, 200,
    3, 275, 3, 281, 3, 78, 302, 3, 817, 3, 78, 246, 3, 243, 78, 3,
    387, 3, 275, 3, 799, 3, 223, 3, 223, 3, 799, 3, 78, 246, 3, 78,
    246, 3, 857, 3, 281, 3, 78, 246, 3, 243, 78, 3, 380, 3, 384, 3,
    387, 3, 330, 3, 223, 3, 799, 3, 812, 3, 275, 3, 281, 3, 399, 3,
    78, 859, 3, 243, 78, 3, 78, 246, 3, 78, 246, 3, 438, 3, 889, 3,
    78, 200, 3, 78, 246, 3, 891, 3, 80, 3, 80, 3, 903, 3, 935, 3,
    935, 3, 935, 3, 935, 3, 438, 976, 3, 3, 978, 3, 935, 3, 1023, 3,
    1026, 3, 78, 3, 380, 3, 380, 3, 778, 78, 3, 80, 3, 789, 78, 3,
    78, 3, 78, 246, 3, 78, 257, 3, 78, 302, 3, 78, 246, 3, 78, 859,
    3, 78, 3, 78, 200, 3, 78, 246, 3, 438, 3, 1075, 3, 1077, 3, 1108,
    3, 1110, 3, 1112, 3, 97, 3, 97, 3, 1140, 3, 1142, 3, 608, 3, 204,
    3, 438, 3, 243, 78, 3, 204, 3, 1145, 3, 438, 3, 78, 85, 3, 1075,
    3, 1149, 3, 438, 3, 380, 3, 1145, 3, 1156, 3, 478, 3, 1158, 1160, 3,
    3, 243, 78, 3, 1163, 3, 438, 3, 1185, 3, 438, 3, 1215, 3, 802, 3,
    1228, 3, 891, 3, 1230, 3, 1234, 3, 1236, 3, 1240, 3, 1243, 248, 3, 248,
    3, 3, 1240, 3, 1243, 3, 1247, 3, 1249, 3, 1215, 3, 1243, 3, 1253, 3,
    438, 3, 438, 3, 524, 3, 438, 3, 380, 3, 438, 3, 438, 3, 438, 3,
    438, 3, 438, 1261, 3, 3, 380, 3, 243, 78, 3, 380, 3, 243, 78, 3,
    243, 78, 3, 438, 3, 315, 3, 1267, 3, 1158, 3, 519, 3, 519, 3, 380,
    3, 380, 3, 80, 3, 889, 3, 80, 3, 80, 3, 1282, 3, 80, 3, 889,
    1295, 3, 1297, 3, 3, 78, 200, 3, 78, 198, 3, 78, 1299, 3, 3, 78,
    198, 3, 243, 78, 3, 438, 3, 802, 3, 243, 78, 3, 438, 3, 1314, 3,
    903, 3, 438, 3, 802, 3, 1341, 4, 3, 204, 3, 78, 246, 3, 935, 3,
    1249, 3, 1410, 3, 1412, 3, 1249, 3, 1410, 3, 1249, 3, 438, 3, 1, 4,
    5, 3, 438, 3, 275, 3, 438, 3, 438, 3, 1267, 3, 97, 1459, 3, 3,
    1461, 3, 1463, 3, 1, 993, 4, 5, 3, 993, 4, 3, 78, 3, 1282, 3,
    1505, 3, 1507, 3, 438, 3, 1512, 3, 1514, 3, 438, 3, 1516, 3, 78, 246,
    3, 78, 246, 248, 3, 3, 204, 248, 3, 3, 1412, 3, 1520, 3, 1522, 3,
    1548, 3, 1550, 3, 1560, 4, 3, 1282, 3, 1156, 3, 1156, 3, 1575, 3, 1110,
    3, 1108, 3, 1112, 3, 1579, 3, 524, 3, 1156, 3, 1600, 3, 438, 3, 243,
    78, 3, 438, 3, 1629, 3, 802, 3, 802, 3, 1548, 3, 243, 78, 3, 243,
    78, 3, 438, 3, 204, 3, 608, 3, 243, 78, 3, 204, 3, 608, 3, 804,
    3, 438, 3, 1267, 3, 1695, 3, 78, 246, 1295, 3, 1297, 3, 3, 78, 200,
    3, 78, 198, 3, 78, 198, 1299, 3, 3, 204, 3, 243, 78, 3, 438, 3,
    608, 3, 1823, 248, 3, 3, 1825, 3, 1827, 3, 802, 3, 1228, 3, 1830, 3,
    1023, 3, 1026, 3, 804, 3, 1832, 3, 1163, 3, 1834, 3, 223, 3, 1836, 3,
    802, 3, 1839, 3, 1834, 3, 1228, 3, 891, 3, 1230, 3, 1575, 3, 1575, 3,
    1851, 3, 243, 78, 3, 315, 3, 204, 3, 243, 78, 3, 438, 3, 1282, 3,
    1228, 3, 223, 3, 802, 3, 1834, 3, 78, 257, 3, 315, 3, 315, 3, 315,
    3, 315, 3, 1140, 3, 380, 3, 1142, 976, 3, 209, 3, 3, 1579, 3, 1108,
    3, 1933, 3, 1935, 3, 1937, 3, 1939, 3, 1941, 3, 1943, 1945, 3, 3, 1947,
    3, 1949, 3, 1951, 1953, 3, 3, 1955, 3, 1957, 3, 1959, 3, 1961, 3, 1963,
    3, 1965, 3, 1967, 3, 1969, 3, 1971, 3, 1112, 3, 1110, 3, 1973, 3, 1975,
    3, 1977, 3, 1979, 3, 1981, 3, 1983, 3, 1985, 3, 1987, 3, 1989, 3, 1991,
    1993, 3, 3, 1995, 3, 1997, 3, 1999, 3, 2001, 3, 2003, 3, 2005, 3, 2007,
    2009, 3, 3, 2011, 2013, 3, 3, 2015, 3, 2017, 3, 2019, 2021, 3, 3, 2023,
    2025, 3, 3, 411, 248, 3, 3, 4, 235, 3, 78, 3, 438, 3, 1282, 3,
    204, 3, 78, 246, 3, 1579, 3, 935, 3, 438, 248, 3, 3, 1267, 3, 1108,
    3, 1110, 3, 1112, 3, 1156, 3, 78, 246, 976, 3, 1459, 3, 3, 1075, 3,
    1077, 3, 1830, 3, 1023, 3, 1026, 3, 804, 3, 1832, 3, 1163, 3, 1834, 3,
    1228, 3, 891, 3, 1230, 3, 223, 3, 802, 3, 1839, 3, 1228, 3, 78, 246,
    248, 3, 3, 1522, 3, 1023, 3, 1026, 3, 778, 78, 3, 438, 3, 438, 3,
    438, 3, 438, 3, 802, 3, 4, 2165, 3, 78, 85, 3, 399, 3, 223, 3,
    2170, 3, 2174, 3, 2174, 3, 223, 3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"[A-Z_][A-Z0-9_]*"#, // [0]
    r#"^(NOT)$"#, // [1]
    r#"[0-9]+"#, // [2]
    r#"[\p{L}_][\p{L}\p{N}$_]*"#, // [3]
    r#"^(ALL|AND|ANY|ARRAY|AS|ASC|AUTHORIZATION|BETWEEN|BINARY|BOTH|CASE|CAST|CHECK|COLLATE|COLUMN|CONSTRAINT|CORRELATION|CREATE|CROSS|CURRENT_DATABASE|CURRENT_DATE|CURRENT_SCHEMA|CURRENT_TIME|CURRENT_TIMESTAMP|CURRENT_USER|DEFAULT|DEFERRABLE|DESC|DISTINCT|ELSE|END|EXCEPT|FALSE|FOR|FOREIGN|FROM|FULL|GRANT|HAVING|IGNORE|IN|INITIALLY|INNER|INTERSECT|INTERVAL|INTERVALYM|INTO|IS|JOIN|LATERAL|LEADING|LEFT|LIKE|LIMIT|LOCALTIME|LOCALTIMESTAMP|MATCH|NATURAL|NEW|NOT|NOTNULL|NULL|NULLSEQUAL|OFFSET|OLD|ON|ONLY|OR|ORDER|OUTER|OVER|OVERLAPS|PARTITION|PRIMARY|REFERENCES|RESPECT|RIGHT|SELECT|SESSION_USER|SET|SIMILAR|SOME|SYSDATE|TABLE|THEN|TIMESERIES|TO|TRAILING|TRUE|UNBOUNDED|UNION|UNIQUE|USER|USING|WHEN|WHERE|WINDOW|WITH|WITHIN)$"#, // [4]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 10, 11, 12, 13, 14, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
    26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
    36, 37, 25, 40, 37, 13, 14, 20, 21, 28,
    12, 26, 33, 35, 17, 52, 53, 53, 52, 57,
    11, 60, 62, 19, 15, 16, 18, 22, 23, 24,
    27, 28, 29, 30, 31, 32, 34, 36, 15, 16,
    67, 22, 23, 68, 69, 60, 70, 71, 72, 73,
    74, 75, 13, 28, 82, 87, 89, 91, 92, 95,
    97, 11, 98, 99, 100, 101, 102, 103, 104, 105,
    106, 40, 107, 108, 109, 110, 111, 112, 113, 114,
    115, 116, 117, 118, 119, 120, 121, 100, 13, 20,
    26, 123, 37, 123, 98, 22, 128, 102, 34, 105,
    133, 134, 118, 134, 118, 11, 98, 99, 100, 101,
    102, 103, 104, 106, 40, 107, 108, 109, 110, 111,
    112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
    15, 16, 18, 22, 23, 137, 24, 27, 28, 29,
    30, 31, 32, 34, 36, 137, 119, 12, 28, 103,
    142, 99, 101, 104, 106, 107, 108, 109, 111, 110,
    112, 113, 114, 115, 116, 117, 120, 121, 164, 32,
    36, 30, 18, 24, 27, 29, 31, 23, 15, 16,
    177, 73, 75, 71, 67, 68, 69, 70, 72, 74,
    189, 190, 189, 190, 195, 201, 206, 207, 210, 211,
    212, 213, 214, 215, 216, 217, 218, 219, 220, 222,
    228, 236, 239, 244, 249, 250, 11, 253, 101, 254,
    104, 110, 112, 116, 117, 259, 261, 263, 253, 101,
    254, 104, 110, 112, 116, 117, 253, 254, 266, 267,
    268, 269, 266, 272, 273, 274, 269, 268, 279, 267,
    282, 283, 284, 285, 286, 287, 288, 289, 290, 222,
    284, 293, 286, 287, 285, 287, 282, 288, 282, 288,
    283, 289, 283, 289, 290, 222, 306, 307, 308, 308,
    311, 314, 317, 239, 307, 82, 311, 320, 321, 82,
    11, 216, 253, 267, 272, 323, 273, 274, 268, 308,
    326, 327, 17, 328, 329, 327, 332, 333, 333, 332,
    323, 338, 214, 340, 341, 340, 345, 341, 350, 352,
    354, 355, 356, 357, 358, 359, 360, 361, 362, 363,
    207, 364, 317, 97, 239, 216, 106, 82, 219, 368,
    219, 368, 368, 228, 89, 216, 253, 267, 254, 272,
    323, 273, 274, 268, 373, 374, 269, 375, 308, 326,
    327, 266, 381, 360, 328, 110, 308, 329, 382, 386,
    11, 388, 388, 254, 386, 308, 266, 308, 382, 381,
    360, 266, 381, 308, 382, 279, 395, 395, 373, 374,
    402, 375, 272, 273, 274, 268, 269, 308, 404, 405,
    407, 408, 409, 410, 413, 414, 415, 416, 417, 418,
    405, 419, 354, 355, 420, 421, 422, 423, 424, 425,
    354, 355, 420, 421, 419, 418, 425, 424, 422, 423,
    440, 442, 356, 446, 447, 448, 449, 450, 451, 452,
    453, 454, 455, 456, 457, 458, 460, 461, 462, 463,
    464, 465, 466, 467, 468, 469, 470, 471, 472, 473,
    359, 482, 483, 484, 485, 486, 487, 488, 489, 490,
    491, 418, 405, 419, 354, 355, 420, 421, 422, 492,
    314, 423, 410, 493, 494, 424, 425, 495, 410, 494,
    491, 422, 492, 314, 423, 493, 495, 500, 501, 502,
    503, 508, 509, 510, 511, 512, 494, 515, 358, 363,
    363, 358, 518, 82, 112, 522, 523, 525, 526, 527,
    528, 529, 530, 531, 532, 533, 534, 82, 112, 522,
    523, 259, 538, 539, 540, 541, 542, 543, 547, 551,
    553, 557, 558, 560, 561, 562, 563, 564, 565, 566,
    567, 568, 569, 570, 571, 572, 573, 574, 575, 576,
    326, 577, 578, 579, 580, 581, 582, 583, 584, 585,
    586, 587, 588, 589, 590, 592, 594, 595, 596, 597,
    598, 599, 600, 601, 602, 606, 326, 327, 381, 360,
    328, 110, 329, 326, 381, 360, 328, 110, 329, 381,
    360, 329, 328, 613, 460, 511, 510, 534, 617, 618,
    619, 620, 621, 622, 623, 624, 625, 308, 382, 626,
    628, 629, 631, 632, 633, 220, 408, 350, 327, 18,
    635, 636, 637, 28, 29, 32, 638, 36, 637, 32,
    36, 637, 220, 408, 18, 638, 220, 408, 638, 638,
    636, 635, 29, 635, 493, 360, 317, 239, 648, 649,
    617, 650, 492, 491, 422, 495, 660, 661, 664, 533,
    665, 666, 667, 668, 670, 671, 672, 674, 414, 676,
    677, 678, 679, 680, 681, 682, 683, 686, 687, 688,
    689, 690, 691, 692, 693, 694, 695, 696, 697, 698,
    699, 701, 702, 703, 704, 705, 177, 287, 360, 706,
    708, 701, 702, 703, 704, 287, 704, 287, 705, 706,
    0, 701, 702, 703, 716, 704, 717, 705, 177, 287,
    360, 718, 719, 720, 721, 722, 706, 716, 726, 717,
    718, 730, 719, 720, 721, 722, 737, 82, 350, 741,
    742, 743, 317, 239, 741, 742, 741, 742, 743, 748,
    749, 750, 748, 753, 749, 750, 758, 759, 760, 761,
    762, 763, 764, 765, 768, 693, 18, 769, 770, 24,
    27, 771, 693, 769, 770, 771, 693, 769, 771, 769,
    771, 770, 210, 326, 327, 266, 386, 381, 360, 328,
    110, 308, 329, 382, 113, 114, 782, 787, 793, 794,
    797, 798, 801, 195, 801, 164, 222, 797, 798, 164,
    222, 797, 798, 807, 811, 457, 266, 267, 282, 283,
    284, 285, 286, 287, 288, 289, 290, 222, 814, 814,
    818, 741, 113, 114, 818, 741, 818, 287, 164, 101,
    821, 821, 82, 272, 273, 274, 824, 825, 826, 827,
    828, 829, 830, 831, 832, 833, 834, 835, 836, 837,
    838, 839, 840, 841, 842, 843, 844, 845, 846, 847,
    848, 849, 97, 835, 824, 825, 826, 827, 828, 829,
    830, 831, 832, 833, 834, 836, 837, 838, 839, 840,
    841, 842, 843, 844, 845, 846, 847, 848, 849, 852,
    272, 854, 273, 855, 274, 267, 272, 273, 811, 274,
    222, 373, 797, 798, 219, 484, 860, 860, 864, 101,
    104, 117, 866, 321, 91, 867, 381, 868, 547, 868,
    871, 873, 875, 866, 867, 381, 866, 866, 867, 867,
    879, 881, 883, 873, 884, 884, 5, 892, 893, 894,
    895, 896, 897, 897, 899, 899, 904, 905, 906, 468,
    907, 908, 909, 910, 469, 470, 911, 912, 913, 914,
    915, 916, 917, 918, 361, 362, 357, 923, 925, 825,
    831, 840, 841, 843, 849, 928, 931, 932, 937, 939,
    941, 943, 946, 948, 950, 951, 952, 953, 954, 951,
    954, 953, 952, 959, 960, 960, 959, 964, 965, 753,
    965, 490, 531, 965, 968, 969, 965, 969, 969, 968,
    356, 207, 490, 570, 92, 980, 980, 984, 985, 986,
    987, 357, 359, 361, 362, 361, 362, 985, 995, 997,
    998, 999, 825, 1000, 1001, 1002, 1003, 1004, 1005, 826,
    1006, 1007, 1008, 1009, 1010, 1011, 1012, 1013, 523, 1015,
    1016, 1017, 1018, 1019, 1020, 1021, 1025, 1027, 1029, 1030,
    618, 1031, 619, 1032, 1033, 1034, 1035, 1036, 1037, 1038,
    1039, 1041, 1042, 1043, 1044, 1045, 1046, 1047, 1048, 1049,
    1050, 1051, 1052, 1053, 1054, 1055, 1057, 1058, 1059, 1060,
    1061, 1062, 1063, 1065, 1066, 1067, 1068, 1069, 1070, 1071,
    1072, 1073, 1078, 1079, 1081, 1082, 1083, 1085, 1086, 830,
    1087, 1089, 1090, 1091, 1092, 1093, 1094, 1095, 1096, 272,
    854, 1097, 1098, 1099, 1100, 1101, 1102, 1103, 1104, 1105,
    82, 101, 104, 112, 117, 522, 523, 259, 1113, 1117,
    852, 381, 1118, 62, 381, 1122, 1123, 1126, 488, 381,
    354, 355, 381, 1129, 1130, 354, 355, 1130, 1129, 491,
    418, 405, 419, 354, 355, 420, 421, 422, 492, 314,
    423, 493, 424, 425, 495, 354, 355, 1130, 357, 358,
    359, 360, 361, 362, 363, 317, 97, 239, 1162, 1164,
    1165, 987, 1166, 1167, 1170, 1171, 1172, 368, 1174, 1174,
    873, 345, 1176, 259, 1176, 1179, 1180, 82, 1029, 253,
    485, 633, 648, 708, 100, 101, 1039, 254, 553, 1105,
    189, 104, 1181, 190, 1182, 112, 308, 814, 116, 117,
    119, 522, 523, 259, 82, 253, 485, 633, 648, 708,
    100, 101, 1039, 254, 553, 1105, 189, 104, 1181, 190,
    1182, 112, 308, 814, 116, 117, 119, 522, 523, 259,
    1186, 1187, 1188, 1191, 1192, 1193, 1194, 1195, 1196, 1197,
    1198, 1199, 1200, 1201, 807, 345, 488, 905, 825, 831,
    359, 840, 841, 843, 1203, 849, 317, 97, 239, 905,
    825, 831, 359, 840, 841, 843, 1203, 849, 317, 97,
    239, 825, 831, 359, 840, 841, 843, 1203, 849, 317,
    97, 239, 825, 831, 359, 840, 841, 843, 849, 317,
    239, 825, 831, 840, 841, 843, 849, 317, 239, 1203,
    1198, 1207, 1207, 1210, 1211, 1212, 1213, 1095, 1172, 195,
    1216, 1217, 801, 1218, 1219, 222, 1220, 1221, 1025, 1222,
    259, 1216, 1220, 1222, 1095, 1217, 1218, 1221, 1219, 1239,
    1246, 676, 1251, 518, 1255, 722, 321, 91, 1262, 1265,
    358, 363, 317, 97, 239, 1269, 1270, 1273, 5, 1273,
    522, 523, 1284, 1285, 620, 1286, 1287, 1288, 1289, 1290,
    1291, 1292, 1293, 708, 259, 1301, 1305, 1306, 1172, 1216,
    801, 222, 1220, 1222, 1308, 1309, 1312, 665, 621, 1315,
    1172, 801, 222, 1220, 1222, 1316, 1317, 1318, 1319, 1320,
    1321, 1322, 1323, 1324, 1325, 1326, 1327, 1328, 1329, 1330,
    1331, 1332, 1333, 831, 1334, 1335, 1336, 1337, 1338, 1339,
    1340, 1343, 1344, 1345, 1346, 1269, 1270, 1351, 1117, 1352,
    1353, 1354, 273, 855, 446, 1038, 1357, 1358, 1359, 1360,
    1361, 1362, 1363, 677, 1364, 1365, 1366, 1368, 1370, 82,
    266, 112, 522, 523, 259, 1372, 1373, 1374, 1375, 1376,
    1377, 1379, 1380, 1174, 1382, 1382, 522, 1217, 1385, 317,
    97, 239, 824, 825, 826, 827, 828, 829, 830, 831,
    832, 833, 834, 836, 837, 838, 839, 840, 841, 842,
    843, 844, 845, 846, 847, 848, 849, 317, 239, 849,
    841, 840, 843, 1393, 1394, 1395, 1397, 1401, 1402, 1403,
    1406, 164, 382, 683, 682, 680, 676, 678, 681, 678,
    681, 679, 676, 677, 678, 681, 1427, 1428, 1429, 1430,
    1431, 1432, 1433, 1435, 1436, 1439, 1440, 1441, 1442, 1255,
    1443, 1444, 1445, 1446, 1447, 1117, 852, 1118, 1118, 854,
    855, 1453, 1455, 1456, 1457, 364, 1464, 1465, 1466, 1467,
    471, 472, 1468, 1469, 1470, 1471, 1472, 1473, 1474, 1475,
    1476, 1477, 1478, 1479, 666, 1480, 1482, 1483, 1484, 1485,
    1486, 274, 1487, 1488, 1489, 1490, 1491, 1492, 1493, 1494,
    1495, 285, 1496, 1497, 1498, 1499, 1500, 1501, 1502, 1181,
    1510, 101, 117, 1523, 1524, 1525, 836, 1526, 838, 839,
    1527, 1528, 1218, 1529, 1530, 1531, 286, 1532, 1533, 1534,
    1535, 1536, 1537, 1538, 1539, 1540, 1541, 1542, 1543, 1544,
    1546, 1551, 1552, 1554, 1555, 1556, 1558, 1559, 1562, 1563,
    1564, 1565, 1566, 1567, 1568, 1569, 1570, 1571, 1572, 101,
    104, 1181, 117, 82, 112, 1580, 1581, 290, 1582, 1583,
    1584, 491, 1587, 1588, 1589, 1590, 1591, 1595, 1596, 1597,
    1605, 1606, 1607, 1608, 1609, 1610, 1611, 1612, 1613, 1614,
    1615, 1616, 1617, 1618, 1619, 1620, 1621, 1622, 1623, 1624,
    1626, 368, 1440, 1172, 1216, 801, 1220, 807, 345, 1262,
    1222, 1631, 1632, 1633, 1634, 1635, 1636, 1638, 1640, 1641,
    1642, 1643, 1645, 1646, 1647, 1648, 1649, 1650, 1651, 1652,
    1653, 487, 293, 678, 1569, 873, 1656, 1657, 1658, 1659,
    1660, 1661, 1662, 1663, 1664, 1665, 1666, 1667, 1668, 1669,
    1670, 1671, 1672, 420, 317, 239, 1675, 1676, 1677, 1678,
    1679, 1680, 1681, 1682, 1684, 1685, 1686, 1687, 1688, 1689,
    1690, 1691, 1694, 1696, 1697, 1698, 1699, 1700, 1701, 1702,
    1703, 1704, 1705, 1706, 1707, 1708, 1709, 1710, 1711, 1712,
    11, 415, 1713, 1713, 842, 1715, 1716, 1717, 317, 1720,
    1721, 1722, 1723, 1725, 1726, 1727, 1728, 1729, 1730, 1731,
    279, 287, 395, 308, 676, 1653, 1735, 1735, 1739, 1740,
    1741, 1742, 1743, 1744, 1745, 1746, 1747, 1748, 1749, 1750,
    1751, 1752, 1753, 1754, 1755, 1756, 1757, 1758, 1759, 1760,
    1761, 1762, 1763, 1764, 1765, 1766, 1767, 1768, 1769, 1770,
    1771, 1772, 1773, 622, 1775, 1182, 1776, 1777, 1778, 1779,
    1780, 1781, 1782, 1783, 1784, 1785, 1786, 1787, 1788, 1789,
    1790, 1791, 1792, 486, 1794, 1794, 1798, 1799, 1800, 1801,
    1805, 1806, 1807, 1808, 1809, 1810, 1811, 1812, 1813, 1814,
    1815, 1816, 1817, 1818, 1819, 798, 797, 11, 1058, 1095,
    1172, 91, 1217, 801, 1218, 222, 1162, 1221, 1025, 1838,
    82, 259, 1842, 1844, 1847, 1848, 1849, 1221, 1095, 1217,
    995, 1090, 1393, 1467, 1761, 26, 1813, 306, 1856, 361,
    995, 1090, 1393, 1467, 1856, 1856, 314, 89, 1865, 1761,
    306, 457, 461, 575, 1293, 1346, 1501, 1541, 1788, 1867,
    1868, 1608, 164, 1873, 1293, 1346, 1788, 314, 1581, 317,
    239, 1501, 1867, 1867, 1868, 1881, 1882, 1883, 1884, 1885,
    558, 1591, 1886, 1887, 1888, 1889, 1892, 1893, 1894, 1895,
    1896, 1899, 1900, 1901, 1902, 1903, 1904, 1905, 1906, 1907,
    1908, 1909, 1910, 1911, 1912, 1913, 1914, 1915, 1916, 1917,
    1918, 1919, 1920, 1921, 1922, 1923, 1924, 1925, 1926, 1927,
    1929, 1930, 1931, 485, 648, 553, 1182, 814, 2026, 2027,
    2028, 2029, 2030, 2031, 2032, 2033, 2034, 2035, 2037, 2038,
    2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048,
    2049, 2050, 2051, 2052, 473, 2053, 2054, 326, 17, 328,
    329, 2056, 2060, 113, 114, 2061, 2061, 2063, 2064, 2065,
    2066, 2067, 259, 2069, 2069, 2072, 2073, 2074, 2075, 2076,
    845, 846, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084,
    2085, 1873, 2088, 2088, 2092, 101, 104, 112, 117, 2093,
    2094, 2095, 2096, 2097, 2098, 2099, 2100, 2101, 2102, 2103,
    2106, 2107, 2108, 2110, 2111, 2112, 2113, 667, 2117, 2118,
    2119, 2120, 2121, 2123, 2124, 2125, 2126, 2127, 2128, 2129,
    2130, 2131, 2132, 2133, 2134, 2135, 2136, 2137, 2138, 2139,
    2140, 2141, 2142, 2143, 2144, 2145, 82, 873, 2146, 2147,
    2148, 2149, 2150, 2151, 2152, 2153, 2154, 2155, 2156, 2157,
    2158, 2159, 2160, 847, 848, 2163, 1172, 195, 1216, 801,
    222, 1162, 1220, 1222, 2164, 2167, 222, 373, 2178, 2179,
    2180, 2181, 2182, 2183, 2184, 668, 623, 624, 625,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 27, token_types_start: 32, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 44, raw_values_count: 1, token_types_start: 45, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 45, raw_values_count: 2, token_types_start: 47, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 47, raw_values_count: 1, token_types_start: 48, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 49, raw_values_count: 1, token_types_start: 50, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 50, raw_values_count: 1, token_types_start: 51, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 54, raw_values_count: 14, token_types_start: 68, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 68, raw_values_count: 14, token_types_start: 82, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 82, raw_values_count: 2, token_types_start: 84, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 90, raw_values_count: 0, token_types_start: 90, token_types_count: 1 }, // [37]
    SimpleHintData { raw_values_start: 91, raw_values_count: 26, token_types_start: 117, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 117, raw_values_count: 1, token_types_start: 118, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 118, raw_values_count: 5, token_types_start: 123, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 131, raw_values_count: 2, token_types_start: 133, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 133, raw_values_count: 1, token_types_start: 134, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 134, raw_values_count: 1, token_types_start: 135, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 135, raw_values_count: 25, token_types_start: 160, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 160, raw_values_count: 15, token_types_start: 175, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 175, raw_values_count: 1, token_types_start: 176, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 176, raw_values_count: 1, token_types_start: 177, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 177, raw_values_count: 2, token_types_start: 179, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 179, raw_values_count: 1, token_types_start: 180, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 180, raw_values_count: 1, token_types_start: 181, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 181, raw_values_count: 1, token_types_start: 182, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 182, raw_values_count: 1, token_types_start: 183, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 184, raw_values_count: 1, token_types_start: 185, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 185, raw_values_count: 1, token_types_start: 186, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 186, raw_values_count: 1, token_types_start: 187, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 187, raw_values_count: 1, token_types_start: 188, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 188, raw_values_count: 1, token_types_start: 189, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 216, raw_values_count: 1, token_types_start: 217, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 217, raw_values_count: 1, token_types_start: 218, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 220, raw_values_count: 2, token_types_start: 222, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 223, raw_values_count: 1, token_types_start: 224, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 224, raw_values_count: 1, token_types_start: 225, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 225, raw_values_count: 1, token_types_start: 226, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 226, raw_values_count: 1, token_types_start: 227, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 233, raw_values_count: 1, token_types_start: 234, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 234, raw_values_count: 1, token_types_start: 235, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 237, raw_values_count: 1, token_types_start: 238, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 238, raw_values_count: 1, token_types_start: 239, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 239, raw_values_count: 1, token_types_start: 240, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 242, raw_values_count: 0, token_types_start: 242, token_types_count: 1 }, // [119]
    SimpleHintData { raw_values_start: 243, raw_values_count: 1, token_types_start: 244, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 244, raw_values_count: 1, token_types_start: 245, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 245, raw_values_count: 1, token_types_start: 246, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 246, raw_values_count: 9, token_types_start: 255, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 255, raw_values_count: 1, token_types_start: 256, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 256, raw_values_count: 1, token_types_start: 257, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 257, raw_values_count: 1, token_types_start: 258, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 258, raw_values_count: 8, token_types_start: 266, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 266, raw_values_count: 1, token_types_start: 267, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 267, raw_values_count: 1, token_types_start: 268, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 268, raw_values_count: 4, token_types_start: 272, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 272, raw_values_count: 1, token_types_start: 273, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 273, raw_values_count: 3, token_types_start: 276, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 276, raw_values_count: 1, token_types_start: 277, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 277, raw_values_count: 1, token_types_start: 278, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 278, raw_values_count: 1, token_types_start: 279, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 279, raw_values_count: 1, token_types_start: 280, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 280, raw_values_count: 10, token_types_start: 290, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 290, raw_values_count: 1, token_types_start: 291, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 291, raw_values_count: 1, token_types_start: 292, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 292, raw_values_count: 2, token_types_start: 294, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 294, raw_values_count: 2, token_types_start: 296, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 296, raw_values_count: 2, token_types_start: 298, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 298, raw_values_count: 1, token_types_start: 299, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 299, raw_values_count: 1, token_types_start: 300, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 300, raw_values_count: 2, token_types_start: 302, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 302, raw_values_count: 1, token_types_start: 303, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 303, raw_values_count: 1, token_types_start: 304, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 304, raw_values_count: 2, token_types_start: 306, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 306, raw_values_count: 1, token_types_start: 307, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 307, raw_values_count: 2, token_types_start: 309, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 309, raw_values_count: 1, token_types_start: 310, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 310, raw_values_count: 1, token_types_start: 311, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 311, raw_values_count: 1, token_types_start: 312, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 312, raw_values_count: 0, token_types_start: 312, token_types_count: 2 }, // [154]
    SimpleHintData { raw_values_start: 314, raw_values_count: 1, token_types_start: 315, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 315, raw_values_count: 2, token_types_start: 317, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 317, raw_values_count: 1, token_types_start: 318, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 318, raw_values_count: 1, token_types_start: 319, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 319, raw_values_count: 2, token_types_start: 321, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 321, raw_values_count: 9, token_types_start: 330, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 330, raw_values_count: 5, token_types_start: 335, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 335, raw_values_count: 1, token_types_start: 336, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 336, raw_values_count: 2, token_types_start: 338, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 338, raw_values_count: 1, token_types_start: 339, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 339, raw_values_count: 1, token_types_start: 340, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 340, raw_values_count: 1, token_types_start: 341, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 341, raw_values_count: 1, token_types_start: 342, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 342, raw_values_count: 3, token_types_start: 345, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 345, raw_values_count: 1, token_types_start: 346, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 346, raw_values_count: 1, token_types_start: 347, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 347, raw_values_count: 1, token_types_start: 348, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 348, raw_values_count: 1, token_types_start: 349, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 349, raw_values_count: 1, token_types_start: 350, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 350, raw_values_count: 12, token_types_start: 362, token_types_count: 3 }, // [174]
    SimpleHintData { raw_values_start: 365, raw_values_count: 2, token_types_start: 367, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 367, raw_values_count: 3, token_types_start: 370, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 370, raw_values_count: 2, token_types_start: 372, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 372, raw_values_count: 1, token_types_start: 373, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 373, raw_values_count: 2, token_types_start: 375, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 375, raw_values_count: 14, token_types_start: 389, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 389, raw_values_count: 10, token_types_start: 399, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 399, raw_values_count: 1, token_types_start: 400, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 400, raw_values_count: 2, token_types_start: 402, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 402, raw_values_count: 1, token_types_start: 403, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 403, raw_values_count: 3, token_types_start: 406, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 406, raw_values_count: 3, token_types_start: 409, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 409, raw_values_count: 1, token_types_start: 410, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 410, raw_values_count: 1, token_types_start: 411, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 411, raw_values_count: 3, token_types_start: 414, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 414, raw_values_count: 1, token_types_start: 415, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 415, raw_values_count: 2, token_types_start: 417, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 417, raw_values_count: 1, token_types_start: 418, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 418, raw_values_count: 1, token_types_start: 419, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 419, raw_values_count: 1, token_types_start: 420, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 420, raw_values_count: 1, token_types_start: 421, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 421, raw_values_count: 1, token_types_start: 422, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 422, raw_values_count: 6, token_types_start: 428, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 428, raw_values_count: 1, token_types_start: 429, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 429, raw_values_count: 1, token_types_start: 430, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 430, raw_values_count: 1, token_types_start: 431, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 431, raw_values_count: 1, token_types_start: 432, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 432, raw_values_count: 1, token_types_start: 433, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 433, raw_values_count: 1, token_types_start: 434, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 434, raw_values_count: 1, token_types_start: 435, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 435, raw_values_count: 1, token_types_start: 436, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 436, raw_values_count: 1, token_types_start: 437, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 437, raw_values_count: 1, token_types_start: 438, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 438, raw_values_count: 1, token_types_start: 439, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 439, raw_values_count: 11, token_types_start: 450, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 450, raw_values_count: 1, token_types_start: 451, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 451, raw_values_count: 1, token_types_start: 452, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 452, raw_values_count: 1, token_types_start: 453, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 453, raw_values_count: 1, token_types_start: 454, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 454, raw_values_count: 1, token_types_start: 455, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 455, raw_values_count: 1, token_types_start: 456, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 456, raw_values_count: 1, token_types_start: 457, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 457, raw_values_count: 1, token_types_start: 458, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 458, raw_values_count: 1, token_types_start: 459, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 459, raw_values_count: 1, token_types_start: 460, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 460, raw_values_count: 1, token_types_start: 461, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 461, raw_values_count: 1, token_types_start: 462, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 462, raw_values_count: 1, token_types_start: 463, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 463, raw_values_count: 1, token_types_start: 464, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 464, raw_values_count: 1, token_types_start: 465, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 465, raw_values_count: 1, token_types_start: 466, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 466, raw_values_count: 1, token_types_start: 467, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 467, raw_values_count: 1, token_types_start: 468, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 468, raw_values_count: 1, token_types_start: 469, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 469, raw_values_count: 1, token_types_start: 470, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 470, raw_values_count: 1, token_types_start: 471, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 471, raw_values_count: 1, token_types_start: 472, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 472, raw_values_count: 1, token_types_start: 473, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 473, raw_values_count: 1, token_types_start: 474, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 474, raw_values_count: 1, token_types_start: 475, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 475, raw_values_count: 1, token_types_start: 476, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 476, raw_values_count: 1, token_types_start: 477, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 477, raw_values_count: 1, token_types_start: 478, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 478, raw_values_count: 1, token_types_start: 479, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 479, raw_values_count: 1, token_types_start: 480, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 480, raw_values_count: 1, token_types_start: 481, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 481, raw_values_count: 1, token_types_start: 482, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 482, raw_values_count: 1, token_types_start: 483, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 483, raw_values_count: 1, token_types_start: 484, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 484, raw_values_count: 6, token_types_start: 490, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 490, raw_values_count: 1, token_types_start: 491, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 491, raw_values_count: 1, token_types_start: 492, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 492, raw_values_count: 1, token_types_start: 493, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 493, raw_values_count: 1, token_types_start: 494, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 494, raw_values_count: 1, token_types_start: 495, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 495, raw_values_count: 1, token_types_start: 496, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 496, raw_values_count: 1, token_types_start: 497, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 497, raw_values_count: 1, token_types_start: 498, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 498, raw_values_count: 1, token_types_start: 499, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 499, raw_values_count: 1, token_types_start: 500, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 500, raw_values_count: 17, token_types_start: 517, token_types_count: 1 }, // [255]
    SimpleHintData { raw_values_start: 518, raw_values_count: 2, token_types_start: 520, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 520, raw_values_count: 6, token_types_start: 526, token_types_count: 1 }, // [257]
    SimpleHintData { raw_values_start: 527, raw_values_count: 1, token_types_start: 528, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 528, raw_values_count: 1, token_types_start: 529, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 529, raw_values_count: 1, token_types_start: 530, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 530, raw_values_count: 1, token_types_start: 531, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 531, raw_values_count: 1, token_types_start: 532, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 532, raw_values_count: 1, token_types_start: 533, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 533, raw_values_count: 1, token_types_start: 534, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 534, raw_values_count: 1, token_types_start: 535, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 535, raw_values_count: 1, token_types_start: 536, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 536, raw_values_count: 1, token_types_start: 537, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 537, raw_values_count: 1, token_types_start: 538, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 538, raw_values_count: 2, token_types_start: 540, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 540, raw_values_count: 1, token_types_start: 541, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 541, raw_values_count: 1, token_types_start: 542, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 542, raw_values_count: 1, token_types_start: 543, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 543, raw_values_count: 4, token_types_start: 547, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 547, raw_values_count: 1, token_types_start: 548, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 548, raw_values_count: 1, token_types_start: 549, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 549, raw_values_count: 1, token_types_start: 550, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 550, raw_values_count: 1, token_types_start: 551, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 551, raw_values_count: 1, token_types_start: 552, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 552, raw_values_count: 1, token_types_start: 553, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 553, raw_values_count: 1, token_types_start: 554, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 554, raw_values_count: 1, token_types_start: 555, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 555, raw_values_count: 1, token_types_start: 556, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 556, raw_values_count: 1, token_types_start: 557, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 557, raw_values_count: 5, token_types_start: 562, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 562, raw_values_count: 1, token_types_start: 563, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 563, raw_values_count: 1, token_types_start: 564, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 564, raw_values_count: 1, token_types_start: 565, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 565, raw_values_count: 1, token_types_start: 566, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 566, raw_values_count: 1, token_types_start: 567, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 567, raw_values_count: 1, token_types_start: 568, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 568, raw_values_count: 1, token_types_start: 569, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 569, raw_values_count: 1, token_types_start: 570, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 570, raw_values_count: 1, token_types_start: 571, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 571, raw_values_count: 1, token_types_start: 572, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 572, raw_values_count: 1, token_types_start: 573, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 573, raw_values_count: 1, token_types_start: 574, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 574, raw_values_count: 1, token_types_start: 575, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 575, raw_values_count: 1, token_types_start: 576, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 576, raw_values_count: 1, token_types_start: 577, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 577, raw_values_count: 1, token_types_start: 578, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 578, raw_values_count: 1, token_types_start: 579, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 579, raw_values_count: 1, token_types_start: 580, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 580, raw_values_count: 1, token_types_start: 581, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 581, raw_values_count: 1, token_types_start: 582, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 582, raw_values_count: 1, token_types_start: 583, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 583, raw_values_count: 1, token_types_start: 584, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 584, raw_values_count: 1, token_types_start: 585, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 585, raw_values_count: 1, token_types_start: 586, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 586, raw_values_count: 1, token_types_start: 587, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 587, raw_values_count: 1, token_types_start: 588, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 588, raw_values_count: 1, token_types_start: 589, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 589, raw_values_count: 1, token_types_start: 590, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 590, raw_values_count: 1, token_types_start: 591, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 591, raw_values_count: 1, token_types_start: 592, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 592, raw_values_count: 1, token_types_start: 593, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 593, raw_values_count: 1, token_types_start: 594, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 594, raw_values_count: 1, token_types_start: 595, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 595, raw_values_count: 1, token_types_start: 596, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 596, raw_values_count: 1, token_types_start: 597, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 597, raw_values_count: 1, token_types_start: 598, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 598, raw_values_count: 1, token_types_start: 599, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 599, raw_values_count: 1, token_types_start: 600, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 600, raw_values_count: 1, token_types_start: 601, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 601, raw_values_count: 1, token_types_start: 602, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 602, raw_values_count: 1, token_types_start: 603, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 603, raw_values_count: 1, token_types_start: 604, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 604, raw_values_count: 1, token_types_start: 605, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 605, raw_values_count: 1, token_types_start: 606, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 606, raw_values_count: 1, token_types_start: 607, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 607, raw_values_count: 1, token_types_start: 608, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 608, raw_values_count: 1, token_types_start: 609, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 609, raw_values_count: 1, token_types_start: 610, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 610, raw_values_count: 1, token_types_start: 611, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 611, raw_values_count: 1, token_types_start: 612, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 612, raw_values_count: 1, token_types_start: 613, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 613, raw_values_count: 1, token_types_start: 614, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 614, raw_values_count: 1, token_types_start: 615, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 615, raw_values_count: 1, token_types_start: 616, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 616, raw_values_count: 7, token_types_start: 623, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 623, raw_values_count: 6, token_types_start: 629, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 629, raw_values_count: 2, token_types_start: 631, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 631, raw_values_count: 1, token_types_start: 632, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 632, raw_values_count: 1, token_types_start: 633, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 633, raw_values_count: 1, token_types_start: 634, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 634, raw_values_count: 13, token_types_start: 647, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 647, raw_values_count: 2, token_types_start: 649, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 649, raw_values_count: 1, token_types_start: 650, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 650, raw_values_count: 1, token_types_start: 651, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 651, raw_values_count: 1, token_types_start: 652, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 652, raw_values_count: 1, token_types_start: 653, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 653, raw_values_count: 1, token_types_start: 654, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 654, raw_values_count: 1, token_types_start: 655, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 655, raw_values_count: 13, token_types_start: 668, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 668, raw_values_count: 3, token_types_start: 671, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 671, raw_values_count: 1, token_types_start: 672, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 672, raw_values_count: 4, token_types_start: 676, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 676, raw_values_count: 3, token_types_start: 679, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 679, raw_values_count: 1, token_types_start: 680, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 680, raw_values_count: 1, token_types_start: 681, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 681, raw_values_count: 2, token_types_start: 683, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 683, raw_values_count: 1, token_types_start: 684, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 684, raw_values_count: 1, token_types_start: 685, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 685, raw_values_count: 1, token_types_start: 686, token_types_count: 2 }, // [363]
    SimpleHintData { raw_values_start: 688, raw_values_count: 1, token_types_start: 689, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 689, raw_values_count: 1, token_types_start: 690, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 690, raw_values_count: 1, token_types_start: 691, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 691, raw_values_count: 1, token_types_start: 692, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 692, raw_values_count: 1, token_types_start: 693, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 693, raw_values_count: 2, token_types_start: 695, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 695, raw_values_count: 0, token_types_start: 695, token_types_count: 1 }, // [370]
    SimpleHintData { raw_values_start: 696, raw_values_count: 1, token_types_start: 697, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 697, raw_values_count: 1, token_types_start: 698, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 698, raw_values_count: 1, token_types_start: 699, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 699, raw_values_count: 5, token_types_start: 704, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 704, raw_values_count: 1, token_types_start: 705, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 705, raw_values_count: 1, token_types_start: 706, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 706, raw_values_count: 1, token_types_start: 707, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 707, raw_values_count: 1, token_types_start: 708, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 708, raw_values_count: 9, token_types_start: 717, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 717, raw_values_count: 1, token_types_start: 718, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 718, raw_values_count: 1, token_types_start: 719, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 719, raw_values_count: 1, token_types_start: 720, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 720, raw_values_count: 1, token_types_start: 721, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 721, raw_values_count: 1, token_types_start: 722, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 722, raw_values_count: 1, token_types_start: 723, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 723, raw_values_count: 1, token_types_start: 724, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 724, raw_values_count: 1, token_types_start: 725, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 725, raw_values_count: 1, token_types_start: 726, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 726, raw_values_count: 1, token_types_start: 727, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 727, raw_values_count: 1, token_types_start: 728, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 728, raw_values_count: 1, token_types_start: 729, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 729, raw_values_count: 1, token_types_start: 730, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 730, raw_values_count: 1, token_types_start: 731, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 731, raw_values_count: 9, token_types_start: 740, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 740, raw_values_count: 1, token_types_start: 741, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 741, raw_values_count: 1, token_types_start: 742, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 742, raw_values_count: 1, token_types_start: 743, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 743, raw_values_count: 1, token_types_start: 744, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 744, raw_values_count: 2, token_types_start: 746, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 746, raw_values_count: 1, token_types_start: 747, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 747, raw_values_count: 1, token_types_start: 748, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 748, raw_values_count: 1, token_types_start: 749, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 750, raw_values_count: 17, token_types_start: 767, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 767, raw_values_count: 1, token_types_start: 768, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 768, raw_values_count: 1, token_types_start: 769, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 769, raw_values_count: 1, token_types_start: 770, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 770, raw_values_count: 1, token_types_start: 771, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 771, raw_values_count: 1, token_types_start: 772, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 772, raw_values_count: 1, token_types_start: 773, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 773, raw_values_count: 1, token_types_start: 774, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 774, raw_values_count: 1, token_types_start: 775, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 775, raw_values_count: 1, token_types_start: 776, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 776, raw_values_count: 1, token_types_start: 777, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 777, raw_values_count: 2, token_types_start: 779, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 779, raw_values_count: 3, token_types_start: 782, token_types_count: 2 }, // [416]
    SimpleHintData { raw_values_start: 784, raw_values_count: 2, token_types_start: 786, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 786, raw_values_count: 1, token_types_start: 787, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 787, raw_values_count: 1, token_types_start: 788, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 788, raw_values_count: 1, token_types_start: 789, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 789, raw_values_count: 3, token_types_start: 792, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 792, raw_values_count: 1, token_types_start: 793, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 793, raw_values_count: 1, token_types_start: 794, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 794, raw_values_count: 1, token_types_start: 795, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 795, raw_values_count: 1, token_types_start: 796, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 796, raw_values_count: 1, token_types_start: 797, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 797, raw_values_count: 1, token_types_start: 798, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 798, raw_values_count: 1, token_types_start: 799, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 799, raw_values_count: 1, token_types_start: 800, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 800, raw_values_count: 1, token_types_start: 801, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 801, raw_values_count: 1, token_types_start: 802, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 802, raw_values_count: 1, token_types_start: 803, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 803, raw_values_count: 1, token_types_start: 804, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 804, raw_values_count: 1, token_types_start: 805, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 805, raw_values_count: 7, token_types_start: 812, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 812, raw_values_count: 4, token_types_start: 816, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 816, raw_values_count: 3, token_types_start: 819, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 819, raw_values_count: 1, token_types_start: 820, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 820, raw_values_count: 1, token_types_start: 821, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 821, raw_values_count: 1, token_types_start: 822, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 822, raw_values_count: 12, token_types_start: 834, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 834, raw_values_count: 2, token_types_start: 836, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 836, raw_values_count: 1, token_types_start: 837, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 837, raw_values_count: 1, token_types_start: 838, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 838, raw_values_count: 1, token_types_start: 839, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 839, raw_values_count: 1, token_types_start: 840, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 840, raw_values_count: 2, token_types_start: 842, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 842, raw_values_count: 1, token_types_start: 843, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 843, raw_values_count: 6, token_types_start: 849, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 849, raw_values_count: 4, token_types_start: 853, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 853, raw_values_count: 1, token_types_start: 854, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 854, raw_values_count: 1, token_types_start: 855, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 855, raw_values_count: 3, token_types_start: 858, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 858, raw_values_count: 11, token_types_start: 869, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 869, raw_values_count: 1, token_types_start: 870, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 870, raw_values_count: 4, token_types_start: 874, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 874, raw_values_count: 2, token_types_start: 876, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 876, raw_values_count: 1, token_types_start: 877, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 877, raw_values_count: 2, token_types_start: 879, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 879, raw_values_count: 2, token_types_start: 881, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 881, raw_values_count: 1, token_types_start: 882, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 882, raw_values_count: 4, token_types_start: 886, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 886, raw_values_count: 26, token_types_start: 912, token_types_count: 1 }, // [463]
    SimpleHintData { raw_values_start: 913, raw_values_count: 1, token_types_start: 914, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 914, raw_values_count: 25, token_types_start: 939, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 939, raw_values_count: 1, token_types_start: 940, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 940, raw_values_count: 5, token_types_start: 945, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 945, raw_values_count: 9, token_types_start: 954, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 954, raw_values_count: 3, token_types_start: 957, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 957, raw_values_count: 1, token_types_start: 958, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 958, raw_values_count: 1, token_types_start: 959, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 959, raw_values_count: 3, token_types_start: 962, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 962, raw_values_count: 7, token_types_start: 969, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 969, raw_values_count: 1, token_types_start: 970, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 970, raw_values_count: 1, token_types_start: 971, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 971, raw_values_count: 1, token_types_start: 972, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 972, raw_values_count: 1, token_types_start: 973, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 973, raw_values_count: 3, token_types_start: 976, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 976, raw_values_count: 1, token_types_start: 977, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 977, raw_values_count: 2, token_types_start: 979, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 979, raw_values_count: 1, token_types_start: 980, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 980, raw_values_count: 1, token_types_start: 981, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 981, raw_values_count: 1, token_types_start: 982, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 982, raw_values_count: 1, token_types_start: 983, token_types_count: 0 }, // [484]
    SimpleHintData { raw_values_start: 983, raw_values_count: 2, token_types_start: 985, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 985, raw_values_count: 1, token_types_start: 986, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 986, raw_values_count: 0, token_types_start: 986, token_types_count: 1 }, // [487]
    SimpleHintData { raw_values_start: 987, raw_values_count: 1, token_types_start: 988, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 988, raw_values_count: 1, token_types_start: 989, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 989, raw_values_count: 1, token_types_start: 990, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 990, raw_values_count: 1, token_types_start: 991, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 991, raw_values_count: 1, token_types_start: 992, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 992, raw_values_count: 1, token_types_start: 993, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 993, raw_values_count: 2, token_types_start: 995, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 995, raw_values_count: 1, token_types_start: 996, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 996, raw_values_count: 1, token_types_start: 997, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 997, raw_values_count: 1, token_types_start: 998, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 998, raw_values_count: 1, token_types_start: 999, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 999, raw_values_count: 1, token_types_start: 1000, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 1000, raw_values_count: 1, token_types_start: 1001, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 1001, raw_values_count: 1, token_types_start: 1002, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 1002, raw_values_count: 1, token_types_start: 1003, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 1003, raw_values_count: 1, token_types_start: 1004, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 1004, raw_values_count: 1, token_types_start: 1005, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 1005, raw_values_count: 1, token_types_start: 1006, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 1006, raw_values_count: 1, token_types_start: 1007, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 1007, raw_values_count: 1, token_types_start: 1008, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 1008, raw_values_count: 1, token_types_start: 1009, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 1009, raw_values_count: 1, token_types_start: 1010, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 1010, raw_values_count: 1, token_types_start: 1011, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 1011, raw_values_count: 1, token_types_start: 1012, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 1012, raw_values_count: 1, token_types_start: 1013, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 1013, raw_values_count: 1, token_types_start: 1014, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 1014, raw_values_count: 2, token_types_start: 1016, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 1016, raw_values_count: 1, token_types_start: 1017, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 1017, raw_values_count: 1, token_types_start: 1018, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 1018, raw_values_count: 1, token_types_start: 1019, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 1019, raw_values_count: 6, token_types_start: 1025, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 1025, raw_values_count: 1, token_types_start: 1026, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 1026, raw_values_count: 1, token_types_start: 1027, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 1027, raw_values_count: 1, token_types_start: 1028, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 1028, raw_values_count: 1, token_types_start: 1029, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 1029, raw_values_count: 1, token_types_start: 1030, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 1030, raw_values_count: 1, token_types_start: 1031, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 1031, raw_values_count: 1, token_types_start: 1032, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 1032, raw_values_count: 1, token_types_start: 1033, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 1033, raw_values_count: 1, token_types_start: 1034, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 1034, raw_values_count: 1, token_types_start: 1035, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 1035, raw_values_count: 4, token_types_start: 1039, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 1039, raw_values_count: 1, token_types_start: 1040, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 1040, raw_values_count: 1, token_types_start: 1041, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 1041, raw_values_count: 1, token_types_start: 1042, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 1042, raw_values_count: 1, token_types_start: 1043, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 1043, raw_values_count: 2, token_types_start: 1045, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 1045, raw_values_count: 1, token_types_start: 1046, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 1046, raw_values_count: 1, token_types_start: 1047, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 1047, raw_values_count: 1, token_types_start: 1048, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 1048, raw_values_count: 2, token_types_start: 1050, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 1050, raw_values_count: 1, token_types_start: 1051, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 1051, raw_values_count: 5, token_types_start: 1056, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 1056, raw_values_count: 2, token_types_start: 1058, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 1058, raw_values_count: 1, token_types_start: 1059, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 1059, raw_values_count: 1, token_types_start: 1060, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 1060, raw_values_count: 2, token_types_start: 1062, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 1062, raw_values_count: 2, token_types_start: 1064, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 1064, raw_values_count: 2, token_types_start: 1066, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 1066, raw_values_count: 1, token_types_start: 1067, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 1067, raw_values_count: 4, token_types_start: 1071, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 1071, raw_values_count: 4, token_types_start: 1075, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 1075, raw_values_count: 1, token_types_start: 1076, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 1076, raw_values_count: 1, token_types_start: 1077, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 1077, raw_values_count: 1, token_types_start: 1078, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 1078, raw_values_count: 1, token_types_start: 1079, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 1079, raw_values_count: 1, token_types_start: 1080, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 1080, raw_values_count: 1, token_types_start: 1081, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 1081, raw_values_count: 1, token_types_start: 1082, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 1082, raw_values_count: 1, token_types_start: 1083, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 1083, raw_values_count: 1, token_types_start: 1084, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 1084, raw_values_count: 1, token_types_start: 1085, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 1085, raw_values_count: 1, token_types_start: 1086, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 1086, raw_values_count: 1, token_types_start: 1087, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 1087, raw_values_count: 1, token_types_start: 1088, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 1088, raw_values_count: 1, token_types_start: 1089, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 1089, raw_values_count: 1, token_types_start: 1090, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 1090, raw_values_count: 1, token_types_start: 1091, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 1091, raw_values_count: 1, token_types_start: 1092, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 1092, raw_values_count: 1, token_types_start: 1093, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 1093, raw_values_count: 1, token_types_start: 1094, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 1094, raw_values_count: 1, token_types_start: 1095, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 1095, raw_values_count: 1, token_types_start: 1096, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 1096, raw_values_count: 1, token_types_start: 1097, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 1097, raw_values_count: 1, token_types_start: 1098, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 1098, raw_values_count: 1, token_types_start: 1099, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 1099, raw_values_count: 1, token_types_start: 1100, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 1100, raw_values_count: 1, token_types_start: 1101, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 1101, raw_values_count: 1, token_types_start: 1102, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 1102, raw_values_count: 1, token_types_start: 1103, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 1103, raw_values_count: 1, token_types_start: 1104, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 1104, raw_values_count: 1, token_types_start: 1105, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 1105, raw_values_count: 1, token_types_start: 1106, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 1106, raw_values_count: 1, token_types_start: 1107, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 1107, raw_values_count: 1, token_types_start: 1108, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 1108, raw_values_count: 1, token_types_start: 1109, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1109, raw_values_count: 1, token_types_start: 1110, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1110, raw_values_count: 1, token_types_start: 1111, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1111, raw_values_count: 1, token_types_start: 1112, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1112, raw_values_count: 1, token_types_start: 1113, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1113, raw_values_count: 1, token_types_start: 1114, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 1114, raw_values_count: 1, token_types_start: 1115, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1115, raw_values_count: 1, token_types_start: 1116, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1116, raw_values_count: 1, token_types_start: 1117, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1117, raw_values_count: 1, token_types_start: 1118, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1118, raw_values_count: 1, token_types_start: 1119, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1119, raw_values_count: 1, token_types_start: 1120, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1120, raw_values_count: 1, token_types_start: 1121, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1121, raw_values_count: 1, token_types_start: 1122, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 1, token_types_start: 1123, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1123, raw_values_count: 1, token_types_start: 1124, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1124, raw_values_count: 1, token_types_start: 1125, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1125, raw_values_count: 1, token_types_start: 1126, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1126, raw_values_count: 1, token_types_start: 1127, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1127, raw_values_count: 1, token_types_start: 1128, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1128, raw_values_count: 1, token_types_start: 1129, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1129, raw_values_count: 1, token_types_start: 1130, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1130, raw_values_count: 1, token_types_start: 1131, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1131, raw_values_count: 1, token_types_start: 1132, token_types_count: 0 }, // [606]
    SimpleHintData { raw_values_start: 1132, raw_values_count: 1, token_types_start: 1133, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1133, raw_values_count: 1, token_types_start: 1134, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1134, raw_values_count: 1, token_types_start: 1135, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 1, token_types_start: 1136, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1136, raw_values_count: 1, token_types_start: 1137, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1137, raw_values_count: 1, token_types_start: 1138, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1138, raw_values_count: 1, token_types_start: 1139, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1139, raw_values_count: 1, token_types_start: 1140, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1140, raw_values_count: 1, token_types_start: 1141, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1141, raw_values_count: 1, token_types_start: 1142, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1142, raw_values_count: 1, token_types_start: 1143, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1143, raw_values_count: 1, token_types_start: 1144, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1144, raw_values_count: 1, token_types_start: 1145, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1145, raw_values_count: 1, token_types_start: 1146, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1146, raw_values_count: 1, token_types_start: 1147, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 1, token_types_start: 1148, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1148, raw_values_count: 1, token_types_start: 1149, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1149, raw_values_count: 1, token_types_start: 1150, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1150, raw_values_count: 1, token_types_start: 1151, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1151, raw_values_count: 1, token_types_start: 1152, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1152, raw_values_count: 1, token_types_start: 1153, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 1, token_types_start: 1154, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1154, raw_values_count: 1, token_types_start: 1155, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1155, raw_values_count: 1, token_types_start: 1156, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1156, raw_values_count: 1, token_types_start: 1157, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1157, raw_values_count: 1, token_types_start: 1158, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1158, raw_values_count: 1, token_types_start: 1159, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1159, raw_values_count: 1, token_types_start: 1160, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1160, raw_values_count: 1, token_types_start: 1161, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1161, raw_values_count: 1, token_types_start: 1162, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1162, raw_values_count: 1, token_types_start: 1163, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1163, raw_values_count: 1, token_types_start: 1164, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1164, raw_values_count: 1, token_types_start: 1165, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1165, raw_values_count: 1, token_types_start: 1166, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1166, raw_values_count: 1, token_types_start: 1167, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1167, raw_values_count: 1, token_types_start: 1168, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1168, raw_values_count: 1, token_types_start: 1169, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1169, raw_values_count: 1, token_types_start: 1170, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1170, raw_values_count: 1, token_types_start: 1171, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1171, raw_values_count: 1, token_types_start: 1172, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 1, token_types_start: 1173, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1173, raw_values_count: 1, token_types_start: 1174, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1174, raw_values_count: 1, token_types_start: 1175, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1175, raw_values_count: 1, token_types_start: 1176, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1176, raw_values_count: 1, token_types_start: 1177, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1177, raw_values_count: 1, token_types_start: 1178, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1178, raw_values_count: 1, token_types_start: 1179, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 1, token_types_start: 1180, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1180, raw_values_count: 8, token_types_start: 1188, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 1, token_types_start: 1189, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1189, raw_values_count: 4, token_types_start: 1193, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 2, token_types_start: 1195, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1195, raw_values_count: 1, token_types_start: 1196, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1196, raw_values_count: 1, token_types_start: 1197, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1197, raw_values_count: 1, token_types_start: 1198, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1198, raw_values_count: 2, token_types_start: 1200, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1200, raw_values_count: 5, token_types_start: 1205, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1205, raw_values_count: 2, token_types_start: 1207, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1207, raw_values_count: 1, token_types_start: 1208, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1208, raw_values_count: 1, token_types_start: 1209, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1209, raw_values_count: 15, token_types_start: 1224, token_types_count: 1 }, // [667]
    SimpleHintData { raw_values_start: 1225, raw_values_count: 3, token_types_start: 1228, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1228, raw_values_count: 7, token_types_start: 1235, token_types_count: 3 }, // [669]
    SimpleHintData { raw_values_start: 1238, raw_values_count: 1, token_types_start: 1239, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1239, raw_values_count: 1, token_types_start: 1240, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1240, raw_values_count: 1, token_types_start: 1241, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1241, raw_values_count: 1, token_types_start: 1242, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1242, raw_values_count: 1, token_types_start: 1243, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1243, raw_values_count: 1, token_types_start: 1244, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1244, raw_values_count: 1, token_types_start: 1245, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1245, raw_values_count: 1, token_types_start: 1246, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1246, raw_values_count: 1, token_types_start: 1247, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1247, raw_values_count: 2, token_types_start: 1249, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1249, raw_values_count: 1, token_types_start: 1250, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1250, raw_values_count: 2, token_types_start: 1252, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 2, token_types_start: 1254, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 1, token_types_start: 1255, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1255, raw_values_count: 1, token_types_start: 1256, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1256, raw_values_count: 1, token_types_start: 1257, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1257, raw_values_count: 27, token_types_start: 1284, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1284, raw_values_count: 26, token_types_start: 1310, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1310, raw_values_count: 1, token_types_start: 1311, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1311, raw_values_count: 1, token_types_start: 1312, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1312, raw_values_count: 1, token_types_start: 1313, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1313, raw_values_count: 1, token_types_start: 1314, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1314, raw_values_count: 1, token_types_start: 1315, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1315, raw_values_count: 1, token_types_start: 1316, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1316, raw_values_count: 1, token_types_start: 1317, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1317, raw_values_count: 1, token_types_start: 1318, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 1, token_types_start: 1319, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1319, raw_values_count: 1, token_types_start: 1320, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 1, token_types_start: 1321, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1321, raw_values_count: 1, token_types_start: 1322, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1322, raw_values_count: 1, token_types_start: 1323, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1323, raw_values_count: 1, token_types_start: 1324, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1324, raw_values_count: 2, token_types_start: 1326, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1326, raw_values_count: 10, token_types_start: 1336, token_types_count: 3 }, // [703]
    SimpleHintData { raw_values_start: 1339, raw_values_count: 9, token_types_start: 1348, token_types_count: 3 }, // [704]
    SimpleHintData { raw_values_start: 1351, raw_values_count: 8, token_types_start: 1359, token_types_count: 3 }, // [705]
    SimpleHintData { raw_values_start: 1362, raw_values_count: 7, token_types_start: 1369, token_types_count: 2 }, // [706]
    SimpleHintData { raw_values_start: 1371, raw_values_count: 6, token_types_start: 1377, token_types_count: 2 }, // [707]
    SimpleHintData { raw_values_start: 1379, raw_values_count: 1, token_types_start: 1380, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1380, raw_values_count: 2, token_types_start: 1382, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1382, raw_values_count: 1, token_types_start: 1383, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1383, raw_values_count: 1, token_types_start: 1384, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1384, raw_values_count: 1, token_types_start: 1385, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1385, raw_values_count: 1, token_types_start: 1386, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1386, raw_values_count: 1, token_types_start: 1387, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1387, raw_values_count: 14, token_types_start: 1401, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1401, raw_values_count: 1, token_types_start: 1402, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1402, raw_values_count: 1, token_types_start: 1403, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1403, raw_values_count: 1, token_types_start: 1404, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1404, raw_values_count: 4, token_types_start: 1408, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1408, raw_values_count: 1, token_types_start: 1409, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1409, raw_values_count: 1, token_types_start: 1410, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1410, raw_values_count: 1, token_types_start: 1411, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1411, raw_values_count: 1, token_types_start: 1412, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1412, raw_values_count: 1, token_types_start: 1413, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1413, raw_values_count: 3, token_types_start: 1416, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1416, raw_values_count: 2, token_types_start: 1418, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1418, raw_values_count: 1, token_types_start: 1419, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1419, raw_values_count: 1, token_types_start: 1420, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1420, raw_values_count: 2, token_types_start: 1422, token_types_count: 3 }, // [729]
    SimpleHintData { raw_values_start: 1425, raw_values_count: 2, token_types_start: 1427, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1427, raw_values_count: 0, token_types_start: 1427, token_types_count: 2 }, // [731]
    SimpleHintData { raw_values_start: 1429, raw_values_count: 0, token_types_start: 1429, token_types_count: 1 }, // [732]
    SimpleHintData { raw_values_start: 1430, raw_values_count: 2, token_types_start: 1432, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1432, raw_values_count: 1, token_types_start: 1433, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1433, raw_values_count: 1, token_types_start: 1434, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1434, raw_values_count: 1, token_types_start: 1435, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1435, raw_values_count: 1, token_types_start: 1436, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1436, raw_values_count: 1, token_types_start: 1437, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1437, raw_values_count: 1, token_types_start: 1438, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1438, raw_values_count: 1, token_types_start: 1439, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1439, raw_values_count: 0, token_types_start: 1439, token_types_count: 1 }, // [741]
    SimpleHintData { raw_values_start: 1440, raw_values_count: 1, token_types_start: 1441, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1441, raw_values_count: 1, token_types_start: 1442, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1442, raw_values_count: 1, token_types_start: 1443, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1443, raw_values_count: 2, token_types_start: 1445, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1445, raw_values_count: 1, token_types_start: 1446, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1446, raw_values_count: 1, token_types_start: 1447, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1447, raw_values_count: 1, token_types_start: 1448, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1448, raw_values_count: 6, token_types_start: 1454, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1454, raw_values_count: 1, token_types_start: 1455, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1455, raw_values_count: 1, token_types_start: 1456, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1456, raw_values_count: 1, token_types_start: 1457, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1457, raw_values_count: 1, token_types_start: 1458, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1458, raw_values_count: 1, token_types_start: 1459, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1459, raw_values_count: 1, token_types_start: 1460, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1460, raw_values_count: 5, token_types_start: 1465, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 1, token_types_start: 1466, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1466, raw_values_count: 1, token_types_start: 1467, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1467, raw_values_count: 1, token_types_start: 1468, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1468, raw_values_count: 1, token_types_start: 1469, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1469, raw_values_count: 1, token_types_start: 1470, token_types_count: 0 }, // [761]
    SimpleHintData { raw_values_start: 1470, raw_values_count: 1, token_types_start: 1471, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1471, raw_values_count: 1, token_types_start: 1472, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1472, raw_values_count: 1, token_types_start: 1473, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1473, raw_values_count: 1, token_types_start: 1474, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1474, raw_values_count: 1, token_types_start: 1475, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1475, raw_values_count: 1, token_types_start: 1476, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1476, raw_values_count: 1, token_types_start: 1477, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1477, raw_values_count: 1, token_types_start: 1478, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1478, raw_values_count: 1, token_types_start: 1479, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1479, raw_values_count: 1, token_types_start: 1480, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1480, raw_values_count: 1, token_types_start: 1481, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1481, raw_values_count: 1, token_types_start: 1482, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1482, raw_values_count: 1, token_types_start: 1483, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1483, raw_values_count: 1, token_types_start: 1484, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1484, raw_values_count: 1, token_types_start: 1485, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1485, raw_values_count: 1, token_types_start: 1486, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1486, raw_values_count: 1, token_types_start: 1487, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1487, raw_values_count: 1, token_types_start: 1488, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1488, raw_values_count: 1, token_types_start: 1489, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1489, raw_values_count: 1, token_types_start: 1490, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1490, raw_values_count: 1, token_types_start: 1491, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 1, token_types_start: 1492, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 1, token_types_start: 1493, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1493, raw_values_count: 1, token_types_start: 1494, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1494, raw_values_count: 1, token_types_start: 1495, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1495, raw_values_count: 1, token_types_start: 1496, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1496, raw_values_count: 1, token_types_start: 1497, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1497, raw_values_count: 1, token_types_start: 1498, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 1, token_types_start: 1499, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1499, raw_values_count: 1, token_types_start: 1500, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1500, raw_values_count: 1, token_types_start: 1501, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1501, raw_values_count: 1, token_types_start: 1502, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1502, raw_values_count: 1, token_types_start: 1503, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1503, raw_values_count: 1, token_types_start: 1504, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1504, raw_values_count: 2, token_types_start: 1506, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1506, raw_values_count: 1, token_types_start: 1507, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1507, raw_values_count: 1, token_types_start: 1508, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1508, raw_values_count: 1, token_types_start: 1509, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1509, raw_values_count: 1, token_types_start: 1510, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 1, token_types_start: 1511, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1511, raw_values_count: 1, token_types_start: 1512, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1512, raw_values_count: 1, token_types_start: 1513, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1513, raw_values_count: 1, token_types_start: 1514, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1514, raw_values_count: 1, token_types_start: 1515, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 1515, raw_values_count: 1, token_types_start: 1516, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 1516, raw_values_count: 1, token_types_start: 1517, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1517, raw_values_count: 1, token_types_start: 1518, token_types_count: 0 }, // [808]
    SimpleHintData { raw_values_start: 1518, raw_values_count: 1, token_types_start: 1519, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 1519, raw_values_count: 6, token_types_start: 1525, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1525, raw_values_count: 1, token_types_start: 1526, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 1526, raw_values_count: 1, token_types_start: 1527, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 1527, raw_values_count: 1, token_types_start: 1528, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 1528, raw_values_count: 1, token_types_start: 1529, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 1529, raw_values_count: 1, token_types_start: 1530, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 1530, raw_values_count: 1, token_types_start: 1531, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 1531, raw_values_count: 1, token_types_start: 1532, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 1532, raw_values_count: 1, token_types_start: 1533, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 1533, raw_values_count: 2, token_types_start: 1535, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 1, token_types_start: 1537, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 1537, raw_values_count: 1, token_types_start: 1538, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 1538, raw_values_count: 1, token_types_start: 1539, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 1539, raw_values_count: 0, token_types_start: 1539, token_types_count: 3 }, // [824]
    SimpleHintData { raw_values_start: 1542, raw_values_count: 25, token_types_start: 1567, token_types_count: 2 }, // [825]
    SimpleHintData { raw_values_start: 1569, raw_values_count: 1, token_types_start: 1570, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 1570, raw_values_count: 1, token_types_start: 1571, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 1571, raw_values_count: 1, token_types_start: 1572, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 1572, raw_values_count: 1, token_types_start: 1573, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 1573, raw_values_count: 1, token_types_start: 1574, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 1574, raw_values_count: 1, token_types_start: 1575, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 1575, raw_values_count: 1, token_types_start: 1576, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 1576, raw_values_count: 1, token_types_start: 1577, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 1577, raw_values_count: 1, token_types_start: 1578, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 1578, raw_values_count: 1, token_types_start: 1579, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 1579, raw_values_count: 1, token_types_start: 1580, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 1580, raw_values_count: 1, token_types_start: 1581, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 1581, raw_values_count: 2, token_types_start: 1583, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 1583, raw_values_count: 1, token_types_start: 1584, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 1584, raw_values_count: 1, token_types_start: 1585, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 1585, raw_values_count: 1, token_types_start: 1586, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 1586, raw_values_count: 3, token_types_start: 1589, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 1589, raw_values_count: 1, token_types_start: 1590, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 1590, raw_values_count: 1, token_types_start: 1591, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 1591, raw_values_count: 1, token_types_start: 1592, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 1592, raw_values_count: 4, token_types_start: 1596, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 1596, raw_values_count: 1, token_types_start: 1597, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 1597, raw_values_count: 1, token_types_start: 1598, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 1598, raw_values_count: 1, token_types_start: 1599, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 1599, raw_values_count: 1, token_types_start: 1600, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 1600, raw_values_count: 1, token_types_start: 1601, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 1601, raw_values_count: 1, token_types_start: 1602, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 1602, raw_values_count: 1, token_types_start: 1603, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 1603, raw_values_count: 1, token_types_start: 1604, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 1604, raw_values_count: 1, token_types_start: 1605, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 1605, raw_values_count: 1, token_types_start: 1606, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 1606, raw_values_count: 1, token_types_start: 1607, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 1607, raw_values_count: 1, token_types_start: 1608, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 1608, raw_values_count: 1, token_types_start: 1609, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 1609, raw_values_count: 1, token_types_start: 1610, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 1610, raw_values_count: 1, token_types_start: 1611, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 1611, raw_values_count: 1, token_types_start: 1612, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 1612, raw_values_count: 1, token_types_start: 1613, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 1613, raw_values_count: 1, token_types_start: 1614, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 1614, raw_values_count: 1, token_types_start: 1615, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 1615, raw_values_count: 3, token_types_start: 1618, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 1618, raw_values_count: 1, token_types_start: 1619, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 1619, raw_values_count: 2, token_types_start: 1621, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 1621, raw_values_count: 1, token_types_start: 1622, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 1622, raw_values_count: 1, token_types_start: 1623, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 1623, raw_values_count: 1, token_types_start: 1624, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 1624, raw_values_count: 1, token_types_start: 1625, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 1625, raw_values_count: 1, token_types_start: 1626, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 1626, raw_values_count: 1, token_types_start: 1627, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 1627, raw_values_count: 1, token_types_start: 1628, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 1628, raw_values_count: 1, token_types_start: 1629, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 1629, raw_values_count: 1, token_types_start: 1630, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 1630, raw_values_count: 1, token_types_start: 1631, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 1631, raw_values_count: 1, token_types_start: 1632, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 1632, raw_values_count: 1, token_types_start: 1633, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 1633, raw_values_count: 1, token_types_start: 1634, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 1634, raw_values_count: 1, token_types_start: 1635, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 1635, raw_values_count: 1, token_types_start: 1636, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 1636, raw_values_count: 1, token_types_start: 1637, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 1637, raw_values_count: 1, token_types_start: 1638, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 1638, raw_values_count: 1, token_types_start: 1639, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 1639, raw_values_count: 1, token_types_start: 1640, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 1640, raw_values_count: 1, token_types_start: 1641, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 1641, raw_values_count: 1, token_types_start: 1642, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 1642, raw_values_count: 1, token_types_start: 1643, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 1643, raw_values_count: 1, token_types_start: 1644, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 1644, raw_values_count: 1, token_types_start: 1645, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 1645, raw_values_count: 1, token_types_start: 1646, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 1646, raw_values_count: 1, token_types_start: 1647, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 1647, raw_values_count: 1, token_types_start: 1648, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 1648, raw_values_count: 1, token_types_start: 1649, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 1649, raw_values_count: 1, token_types_start: 1650, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 1650, raw_values_count: 1, token_types_start: 1651, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 1651, raw_values_count: 1, token_types_start: 1652, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 1652, raw_values_count: 1, token_types_start: 1653, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 1653, raw_values_count: 1, token_types_start: 1654, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 1654, raw_values_count: 1, token_types_start: 1655, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 1655, raw_values_count: 1, token_types_start: 1656, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 1656, raw_values_count: 1, token_types_start: 1657, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 1657, raw_values_count: 1, token_types_start: 1658, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 1658, raw_values_count: 1, token_types_start: 1659, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 1659, raw_values_count: 1, token_types_start: 1660, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 1660, raw_values_count: 1, token_types_start: 1661, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 1, token_types_start: 1662, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 1662, raw_values_count: 1, token_types_start: 1663, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 1663, raw_values_count: 1, token_types_start: 1664, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 1664, raw_values_count: 1, token_types_start: 1665, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 1665, raw_values_count: 1, token_types_start: 1666, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 1666, raw_values_count: 1, token_types_start: 1667, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 1667, raw_values_count: 1, token_types_start: 1668, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 1668, raw_values_count: 1, token_types_start: 1669, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 1669, raw_values_count: 1, token_types_start: 1670, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 1670, raw_values_count: 1, token_types_start: 1671, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 2, token_types_start: 1673, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 1, token_types_start: 1674, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 1674, raw_values_count: 1, token_types_start: 1675, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 1675, raw_values_count: 1, token_types_start: 1676, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 1676, raw_values_count: 1, token_types_start: 1677, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 1677, raw_values_count: 1, token_types_start: 1678, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 1678, raw_values_count: 1, token_types_start: 1679, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 1679, raw_values_count: 1, token_types_start: 1680, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 1680, raw_values_count: 1, token_types_start: 1681, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 1681, raw_values_count: 1, token_types_start: 1682, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 1682, raw_values_count: 1, token_types_start: 1683, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 1683, raw_values_count: 1, token_types_start: 1684, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 1684, raw_values_count: 1, token_types_start: 1685, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 1685, raw_values_count: 1, token_types_start: 1686, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 1686, raw_values_count: 1, token_types_start: 1687, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 1687, raw_values_count: 1, token_types_start: 1688, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 1688, raw_values_count: 1, token_types_start: 1689, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 1689, raw_values_count: 1, token_types_start: 1690, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 1690, raw_values_count: 1, token_types_start: 1691, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 1, token_types_start: 1692, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 1, token_types_start: 1693, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 1693, raw_values_count: 1, token_types_start: 1694, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 1694, raw_values_count: 1, token_types_start: 1695, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 1695, raw_values_count: 1, token_types_start: 1696, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 1696, raw_values_count: 1, token_types_start: 1697, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 1697, raw_values_count: 1, token_types_start: 1698, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 1698, raw_values_count: 1, token_types_start: 1699, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 1699, raw_values_count: 1, token_types_start: 1700, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 1700, raw_values_count: 1, token_types_start: 1701, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 1701, raw_values_count: 1, token_types_start: 1702, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 1702, raw_values_count: 1, token_types_start: 1703, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 1703, raw_values_count: 1, token_types_start: 1704, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 1704, raw_values_count: 1, token_types_start: 1705, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 1705, raw_values_count: 1, token_types_start: 1706, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 1706, raw_values_count: 1, token_types_start: 1707, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 1707, raw_values_count: 1, token_types_start: 1708, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 1708, raw_values_count: 1, token_types_start: 1709, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 1709, raw_values_count: 1, token_types_start: 1710, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 1710, raw_values_count: 1, token_types_start: 1711, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 1711, raw_values_count: 1, token_types_start: 1712, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 1712, raw_values_count: 1, token_types_start: 1713, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 1713, raw_values_count: 1, token_types_start: 1714, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 1714, raw_values_count: 1, token_types_start: 1715, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 1715, raw_values_count: 1, token_types_start: 1716, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 1716, raw_values_count: 1, token_types_start: 1717, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 1717, raw_values_count: 1, token_types_start: 1718, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 1718, raw_values_count: 1, token_types_start: 1719, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 1719, raw_values_count: 4, token_types_start: 1723, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 1723, raw_values_count: 2, token_types_start: 1725, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 1725, raw_values_count: 1, token_types_start: 1726, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 1726, raw_values_count: 1, token_types_start: 1727, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 1727, raw_values_count: 1, token_types_start: 1728, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 1728, raw_values_count: 1, token_types_start: 1729, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 1729, raw_values_count: 1, token_types_start: 1730, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 1730, raw_values_count: 1, token_types_start: 1731, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 1731, raw_values_count: 1, token_types_start: 1732, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 1732, raw_values_count: 1, token_types_start: 1733, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 1733, raw_values_count: 1, token_types_start: 1734, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 1734, raw_values_count: 1, token_types_start: 1735, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 1735, raw_values_count: 1, token_types_start: 1736, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 1736, raw_values_count: 1, token_types_start: 1737, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 1737, raw_values_count: 1, token_types_start: 1738, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 1738, raw_values_count: 1, token_types_start: 1739, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 1739, raw_values_count: 1, token_types_start: 1740, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 1740, raw_values_count: 1, token_types_start: 1741, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 1741, raw_values_count: 1, token_types_start: 1742, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 1742, raw_values_count: 1, token_types_start: 1743, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 1743, raw_values_count: 1, token_types_start: 1744, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 1744, raw_values_count: 1, token_types_start: 1745, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 1745, raw_values_count: 1, token_types_start: 1746, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 1746, raw_values_count: 1, token_types_start: 1747, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 1747, raw_values_count: 1, token_types_start: 1748, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 1748, raw_values_count: 1, token_types_start: 1749, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 1749, raw_values_count: 1, token_types_start: 1750, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 1750, raw_values_count: 1, token_types_start: 1751, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 1751, raw_values_count: 1, token_types_start: 1752, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 1752, raw_values_count: 1, token_types_start: 1753, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 1753, raw_values_count: 1, token_types_start: 1754, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 1754, raw_values_count: 1, token_types_start: 1755, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 1755, raw_values_count: 1, token_types_start: 1756, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 1756, raw_values_count: 1, token_types_start: 1757, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 1757, raw_values_count: 1, token_types_start: 1758, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 1758, raw_values_count: 1, token_types_start: 1759, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 1759, raw_values_count: 1, token_types_start: 1760, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 1760, raw_values_count: 1, token_types_start: 1761, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 1761, raw_values_count: 2, token_types_start: 1763, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 1763, raw_values_count: 8, token_types_start: 1771, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 1771, raw_values_count: 1, token_types_start: 1772, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 1772, raw_values_count: 1, token_types_start: 1773, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 1773, raw_values_count: 1, token_types_start: 1774, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 1774, raw_values_count: 1, token_types_start: 1775, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 1775, raw_values_count: 1, token_types_start: 1776, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 1776, raw_values_count: 1, token_types_start: 1777, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 1777, raw_values_count: 1, token_types_start: 1778, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 1778, raw_values_count: 1, token_types_start: 1779, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 1779, raw_values_count: 1, token_types_start: 1780, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 1780, raw_values_count: 1, token_types_start: 1781, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 1781, raw_values_count: 1, token_types_start: 1782, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 1782, raw_values_count: 1, token_types_start: 1783, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 1783, raw_values_count: 1, token_types_start: 1784, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 1784, raw_values_count: 1, token_types_start: 1785, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 1785, raw_values_count: 1, token_types_start: 1786, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 1786, raw_values_count: 1, token_types_start: 1787, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 1787, raw_values_count: 1, token_types_start: 1788, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 1788, raw_values_count: 1, token_types_start: 1789, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 1789, raw_values_count: 1, token_types_start: 1790, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 1790, raw_values_count: 1, token_types_start: 1791, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 1791, raw_values_count: 5, token_types_start: 1796, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 1796, raw_values_count: 1, token_types_start: 1797, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 1797, raw_values_count: 1, token_types_start: 1798, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 1798, raw_values_count: 1, token_types_start: 1799, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 1799, raw_values_count: 1, token_types_start: 1800, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 1800, raw_values_count: 1, token_types_start: 1801, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 1801, raw_values_count: 1, token_types_start: 1802, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 1802, raw_values_count: 1, token_types_start: 1803, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 1803, raw_values_count: 1, token_types_start: 1804, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 1804, raw_values_count: 1, token_types_start: 1805, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 1805, raw_values_count: 1, token_types_start: 1806, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 1806, raw_values_count: 1, token_types_start: 1807, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 1807, raw_values_count: 1, token_types_start: 1808, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 1808, raw_values_count: 1, token_types_start: 1809, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 1809, raw_values_count: 1, token_types_start: 1810, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 1810, raw_values_count: 1, token_types_start: 1811, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 1811, raw_values_count: 1, token_types_start: 1812, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 1812, raw_values_count: 1, token_types_start: 1813, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 1813, raw_values_count: 1, token_types_start: 1814, token_types_count: 2 }, // [1044]
    SimpleHintData { raw_values_start: 1816, raw_values_count: 1, token_types_start: 1817, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 1817, raw_values_count: 1, token_types_start: 1818, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 1818, raw_values_count: 1, token_types_start: 1819, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 1819, raw_values_count: 1, token_types_start: 1820, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 1820, raw_values_count: 1, token_types_start: 1821, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 1821, raw_values_count: 1, token_types_start: 1822, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 1822, raw_values_count: 1, token_types_start: 1823, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 1823, raw_values_count: 1, token_types_start: 1824, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 1824, raw_values_count: 1, token_types_start: 1825, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 1825, raw_values_count: 1, token_types_start: 1826, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 1826, raw_values_count: 1, token_types_start: 1827, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 1827, raw_values_count: 1, token_types_start: 1828, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 1828, raw_values_count: 1, token_types_start: 1829, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 1829, raw_values_count: 1, token_types_start: 1830, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 1830, raw_values_count: 1, token_types_start: 1831, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 1831, raw_values_count: 1, token_types_start: 1832, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 1832, raw_values_count: 1, token_types_start: 1833, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 1833, raw_values_count: 1, token_types_start: 1834, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 1834, raw_values_count: 1, token_types_start: 1835, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 1835, raw_values_count: 1, token_types_start: 1836, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 1836, raw_values_count: 1, token_types_start: 1837, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 1837, raw_values_count: 1, token_types_start: 1838, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 1838, raw_values_count: 1, token_types_start: 1839, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 1839, raw_values_count: 1, token_types_start: 1840, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 1840, raw_values_count: 1, token_types_start: 1841, token_types_count: 0 }, // [1069]
    SimpleHintData { raw_values_start: 1841, raw_values_count: 1, token_types_start: 1842, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 1842, raw_values_count: 1, token_types_start: 1843, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 1843, raw_values_count: 1, token_types_start: 1844, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 1844, raw_values_count: 1, token_types_start: 1845, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 1845, raw_values_count: 1, token_types_start: 1846, token_types_count: 0 }, // [1074]
    SimpleHintData { raw_values_start: 1846, raw_values_count: 1, token_types_start: 1847, token_types_count: 0 }, // [1075]
    SimpleHintData { raw_values_start: 1847, raw_values_count: 1, token_types_start: 1848, token_types_count: 0 }, // [1076]
    SimpleHintData { raw_values_start: 1848, raw_values_count: 1, token_types_start: 1849, token_types_count: 0 }, // [1077]
    SimpleHintData { raw_values_start: 1849, raw_values_count: 1, token_types_start: 1850, token_types_count: 0 }, // [1078]
    SimpleHintData { raw_values_start: 1850, raw_values_count: 3, token_types_start: 1853, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 1853, raw_values_count: 1, token_types_start: 1854, token_types_count: 0 }, // [1080]
    SimpleHintData { raw_values_start: 1854, raw_values_count: 1, token_types_start: 1855, token_types_count: 0 }, // [1081]
    SimpleHintData { raw_values_start: 1855, raw_values_count: 1, token_types_start: 1856, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 1856, raw_values_count: 1, token_types_start: 1857, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 1857, raw_values_count: 1, token_types_start: 1858, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 1858, raw_values_count: 0, token_types_start: 1858, token_types_count: 1 }, // [1085]
    SimpleHintData { raw_values_start: 1859, raw_values_count: 1, token_types_start: 1860, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 1860, raw_values_count: 1, token_types_start: 1861, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 1861, raw_values_count: 1, token_types_start: 1862, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 1862, raw_values_count: 1, token_types_start: 1863, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 1863, raw_values_count: 1, token_types_start: 1864, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 1864, raw_values_count: 1, token_types_start: 1865, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 1865, raw_values_count: 1, token_types_start: 1866, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 1866, raw_values_count: 1, token_types_start: 1867, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 1867, raw_values_count: 1, token_types_start: 1868, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 1868, raw_values_count: 1, token_types_start: 1869, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 1869, raw_values_count: 1, token_types_start: 1870, token_types_count: 0 }, // [1096]
    SimpleHintData { raw_values_start: 1870, raw_values_count: 4, token_types_start: 1874, token_types_count: 0 }, // [1097]
    SimpleHintData { raw_values_start: 1874, raw_values_count: 3, token_types_start: 1877, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 1877, raw_values_count: 1, token_types_start: 1878, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 1878, raw_values_count: 1, token_types_start: 1879, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 1879, raw_values_count: 1, token_types_start: 1880, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 1880, raw_values_count: 1, token_types_start: 1881, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 1881, raw_values_count: 1, token_types_start: 1882, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 1882, raw_values_count: 1, token_types_start: 1883, token_types_count: 0 }, // [1104]
    SimpleHintData { raw_values_start: 1883, raw_values_count: 1, token_types_start: 1884, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 1884, raw_values_count: 1, token_types_start: 1885, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 1885, raw_values_count: 1, token_types_start: 1886, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 1886, raw_values_count: 1, token_types_start: 1887, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 1887, raw_values_count: 1, token_types_start: 1888, token_types_count: 0 }, // [1109]
    SimpleHintData { raw_values_start: 1888, raw_values_count: 1, token_types_start: 1889, token_types_count: 0 }, // [1110]
    SimpleHintData { raw_values_start: 1889, raw_values_count: 1, token_types_start: 1890, token_types_count: 0 }, // [1111]
    SimpleHintData { raw_values_start: 1890, raw_values_count: 1, token_types_start: 1891, token_types_count: 0 }, // [1112]
    SimpleHintData { raw_values_start: 1891, raw_values_count: 1, token_types_start: 1892, token_types_count: 0 }, // [1113]
    SimpleHintData { raw_values_start: 1892, raw_values_count: 1, token_types_start: 1893, token_types_count: 0 }, // [1114]
    SimpleHintData { raw_values_start: 1893, raw_values_count: 1, token_types_start: 1894, token_types_count: 0 }, // [1115]
    SimpleHintData { raw_values_start: 1894, raw_values_count: 1, token_types_start: 1895, token_types_count: 0 }, // [1116]
    SimpleHintData { raw_values_start: 1895, raw_values_count: 1, token_types_start: 1896, token_types_count: 0 }, // [1117]
    SimpleHintData { raw_values_start: 1896, raw_values_count: 1, token_types_start: 1897, token_types_count: 0 }, // [1118]
    SimpleHintData { raw_values_start: 1897, raw_values_count: 1, token_types_start: 1898, token_types_count: 0 }, // [1119]
    SimpleHintData { raw_values_start: 1898, raw_values_count: 1, token_types_start: 1899, token_types_count: 0 }, // [1120]
    SimpleHintData { raw_values_start: 1899, raw_values_count: 1, token_types_start: 1900, token_types_count: 0 }, // [1121]
    SimpleHintData { raw_values_start: 1900, raw_values_count: 1, token_types_start: 1901, token_types_count: 0 }, // [1122]
    SimpleHintData { raw_values_start: 1901, raw_values_count: 1, token_types_start: 1902, token_types_count: 0 }, // [1123]
    SimpleHintData { raw_values_start: 1902, raw_values_count: 1, token_types_start: 1903, token_types_count: 0 }, // [1124]
    SimpleHintData { raw_values_start: 1903, raw_values_count: 1, token_types_start: 1904, token_types_count: 0 }, // [1125]
    SimpleHintData { raw_values_start: 1904, raw_values_count: 1, token_types_start: 1905, token_types_count: 0 }, // [1126]
    SimpleHintData { raw_values_start: 1905, raw_values_count: 1, token_types_start: 1906, token_types_count: 0 }, // [1127]
    SimpleHintData { raw_values_start: 1906, raw_values_count: 1, token_types_start: 1907, token_types_count: 0 }, // [1128]
    SimpleHintData { raw_values_start: 1907, raw_values_count: 1, token_types_start: 1908, token_types_count: 0 }, // [1129]
    SimpleHintData { raw_values_start: 1908, raw_values_count: 1, token_types_start: 1909, token_types_count: 0 }, // [1130]
    SimpleHintData { raw_values_start: 1909, raw_values_count: 1, token_types_start: 1910, token_types_count: 0 }, // [1131]
    SimpleHintData { raw_values_start: 1910, raw_values_count: 1, token_types_start: 1911, token_types_count: 0 }, // [1132]
    SimpleHintData { raw_values_start: 1911, raw_values_count: 1, token_types_start: 1912, token_types_count: 0 }, // [1133]
    SimpleHintData { raw_values_start: 1912, raw_values_count: 1, token_types_start: 1913, token_types_count: 0 }, // [1134]
    SimpleHintData { raw_values_start: 1913, raw_values_count: 1, token_types_start: 1914, token_types_count: 0 }, // [1135]
    SimpleHintData { raw_values_start: 1914, raw_values_count: 1, token_types_start: 1915, token_types_count: 0 }, // [1136]
    SimpleHintData { raw_values_start: 1915, raw_values_count: 1, token_types_start: 1916, token_types_count: 0 }, // [1137]
    SimpleHintData { raw_values_start: 1916, raw_values_count: 1, token_types_start: 1917, token_types_count: 0 }, // [1138]
    SimpleHintData { raw_values_start: 1917, raw_values_count: 1, token_types_start: 1918, token_types_count: 0 }, // [1139]
    SimpleHintData { raw_values_start: 1918, raw_values_count: 1, token_types_start: 1919, token_types_count: 0 }, // [1140]
    SimpleHintData { raw_values_start: 1919, raw_values_count: 1, token_types_start: 1920, token_types_count: 0 }, // [1141]
    SimpleHintData { raw_values_start: 1920, raw_values_count: 1, token_types_start: 1921, token_types_count: 0 }, // [1142]
    SimpleHintData { raw_values_start: 1921, raw_values_count: 1, token_types_start: 1922, token_types_count: 0 }, // [1143]
    SimpleHintData { raw_values_start: 1922, raw_values_count: 1, token_types_start: 1923, token_types_count: 0 }, // [1144]
    SimpleHintData { raw_values_start: 1923, raw_values_count: 1, token_types_start: 1924, token_types_count: 0 }, // [1145]
    SimpleHintData { raw_values_start: 1924, raw_values_count: 1, token_types_start: 1925, token_types_count: 0 }, // [1146]
    SimpleHintData { raw_values_start: 1925, raw_values_count: 1, token_types_start: 1926, token_types_count: 0 }, // [1147]
    SimpleHintData { raw_values_start: 1926, raw_values_count: 1, token_types_start: 1927, token_types_count: 0 }, // [1148]
    SimpleHintData { raw_values_start: 1927, raw_values_count: 1, token_types_start: 1928, token_types_count: 0 }, // [1149]
    SimpleHintData { raw_values_start: 1928, raw_values_count: 1, token_types_start: 1929, token_types_count: 0 }, // [1150]
    SimpleHintData { raw_values_start: 1929, raw_values_count: 1, token_types_start: 1930, token_types_count: 0 }, // [1151]
    SimpleHintData { raw_values_start: 1930, raw_values_count: 1, token_types_start: 1931, token_types_count: 0 }, // [1152]
    SimpleHintData { raw_values_start: 1931, raw_values_count: 1, token_types_start: 1932, token_types_count: 0 }, // [1153]
    SimpleHintData { raw_values_start: 1932, raw_values_count: 1, token_types_start: 1933, token_types_count: 0 }, // [1154]
    SimpleHintData { raw_values_start: 1933, raw_values_count: 2, token_types_start: 1935, token_types_count: 0 }, // [1155]
    SimpleHintData { raw_values_start: 1935, raw_values_count: 1, token_types_start: 1936, token_types_count: 0 }, // [1156]
    SimpleHintData { raw_values_start: 1936, raw_values_count: 1, token_types_start: 1937, token_types_count: 0 }, // [1157]
    SimpleHintData { raw_values_start: 1937, raw_values_count: 1, token_types_start: 1938, token_types_count: 0 }, // [1158]
    SimpleHintData { raw_values_start: 1938, raw_values_count: 1, token_types_start: 1939, token_types_count: 0 }, // [1159]
    SimpleHintData { raw_values_start: 1939, raw_values_count: 1, token_types_start: 1940, token_types_count: 0 }, // [1160]
    SimpleHintData { raw_values_start: 1940, raw_values_count: 1, token_types_start: 1941, token_types_count: 0 }, // [1161]
    SimpleHintData { raw_values_start: 1941, raw_values_count: 1, token_types_start: 1942, token_types_count: 0 }, // [1162]
    SimpleHintData { raw_values_start: 1942, raw_values_count: 1, token_types_start: 1943, token_types_count: 0 }, // [1163]
    SimpleHintData { raw_values_start: 1943, raw_values_count: 1, token_types_start: 1944, token_types_count: 0 }, // [1164]
    SimpleHintData { raw_values_start: 1944, raw_values_count: 1, token_types_start: 1945, token_types_count: 0 }, // [1165]
    SimpleHintData { raw_values_start: 1945, raw_values_count: 1, token_types_start: 1946, token_types_count: 0 }, // [1166]
    SimpleHintData { raw_values_start: 1946, raw_values_count: 1, token_types_start: 1947, token_types_count: 0 }, // [1167]
    SimpleHintData { raw_values_start: 1947, raw_values_count: 1, token_types_start: 1948, token_types_count: 0 }, // [1168]
    SimpleHintData { raw_values_start: 1948, raw_values_count: 1, token_types_start: 1949, token_types_count: 0 }, // [1169]
    SimpleHintData { raw_values_start: 1949, raw_values_count: 1, token_types_start: 1950, token_types_count: 0 }, // [1170]
    SimpleHintData { raw_values_start: 1950, raw_values_count: 1, token_types_start: 1951, token_types_count: 0 }, // [1171]
    SimpleHintData { raw_values_start: 1951, raw_values_count: 1, token_types_start: 1952, token_types_count: 0 }, // [1172]
    SimpleHintData { raw_values_start: 1952, raw_values_count: 1, token_types_start: 1953, token_types_count: 0 }, // [1173]
    SimpleHintData { raw_values_start: 1953, raw_values_count: 1, token_types_start: 1954, token_types_count: 0 }, // [1174]
    SimpleHintData { raw_values_start: 1954, raw_values_count: 1, token_types_start: 1955, token_types_count: 0 }, // [1175]
    SimpleHintData { raw_values_start: 1955, raw_values_count: 1, token_types_start: 1956, token_types_count: 0 }, // [1176]
    SimpleHintData { raw_values_start: 1956, raw_values_count: 1, token_types_start: 1957, token_types_count: 0 }, // [1177]
    SimpleHintData { raw_values_start: 1957, raw_values_count: 2, token_types_start: 1959, token_types_count: 0 }, // [1178]
    SimpleHintData { raw_values_start: 1959, raw_values_count: 10, token_types_start: 1969, token_types_count: 0 }, // [1179]
    SimpleHintData { raw_values_start: 1969, raw_values_count: 1, token_types_start: 1970, token_types_count: 0 }, // [1180]
    SimpleHintData { raw_values_start: 1970, raw_values_count: 2, token_types_start: 1972, token_types_count: 0 }, // [1181]
    SimpleHintData { raw_values_start: 1972, raw_values_count: 1, token_types_start: 1973, token_types_count: 0 }, // [1182]
    SimpleHintData { raw_values_start: 1973, raw_values_count: 1, token_types_start: 1974, token_types_count: 0 }, // [1183]
    SimpleHintData { raw_values_start: 1974, raw_values_count: 1, token_types_start: 1975, token_types_count: 0 }, // [1184]
    SimpleHintData { raw_values_start: 1975, raw_values_count: 1, token_types_start: 1976, token_types_count: 0 }, // [1185]
    SimpleHintData { raw_values_start: 1976, raw_values_count: 1, token_types_start: 1977, token_types_count: 0 }, // [1186]
    SimpleHintData { raw_values_start: 1977, raw_values_count: 1, token_types_start: 1978, token_types_count: 0 }, // [1187]
    SimpleHintData { raw_values_start: 1978, raw_values_count: 2, token_types_start: 1980, token_types_count: 0 }, // [1188]
    SimpleHintData { raw_values_start: 1980, raw_values_count: 10, token_types_start: 1990, token_types_count: 0 }, // [1189]
    SimpleHintData { raw_values_start: 1990, raw_values_count: 5, token_types_start: 1995, token_types_count: 0 }, // [1190]
    SimpleHintData { raw_values_start: 1995, raw_values_count: 1, token_types_start: 1996, token_types_count: 0 }, // [1191]
    SimpleHintData { raw_values_start: 1996, raw_values_count: 2, token_types_start: 1998, token_types_count: 0 }, // [1192]
    SimpleHintData { raw_values_start: 1998, raw_values_count: 1, token_types_start: 1999, token_types_count: 0 }, // [1193]
    SimpleHintData { raw_values_start: 1999, raw_values_count: 2, token_types_start: 2001, token_types_count: 0 }, // [1194]
    SimpleHintData { raw_values_start: 2001, raw_values_count: 10, token_types_start: 2011, token_types_count: 0 }, // [1195]
    SimpleHintData { raw_values_start: 2011, raw_values_count: 2, token_types_start: 2013, token_types_count: 0 }, // [1196]
    SimpleHintData { raw_values_start: 2013, raw_values_count: 1, token_types_start: 2014, token_types_count: 0 }, // [1197]
    SimpleHintData { raw_values_start: 2014, raw_values_count: 3, token_types_start: 2017, token_types_count: 0 }, // [1198]
    SimpleHintData { raw_values_start: 2017, raw_values_count: 2, token_types_start: 2019, token_types_count: 2 }, // [1199]
    SimpleHintData { raw_values_start: 2021, raw_values_count: 2, token_types_start: 2023, token_types_count: 0 }, // [1200]
    SimpleHintData { raw_values_start: 2023, raw_values_count: 1, token_types_start: 2024, token_types_count: 0 }, // [1201]
    SimpleHintData { raw_values_start: 2024, raw_values_count: 1, token_types_start: 2025, token_types_count: 0 }, // [1202]
    SimpleHintData { raw_values_start: 2025, raw_values_count: 1, token_types_start: 2026, token_types_count: 0 }, // [1203]
    SimpleHintData { raw_values_start: 2026, raw_values_count: 1, token_types_start: 2027, token_types_count: 0 }, // [1204]
    SimpleHintData { raw_values_start: 2027, raw_values_count: 1, token_types_start: 2028, token_types_count: 0 }, // [1205]
    SimpleHintData { raw_values_start: 2028, raw_values_count: 1, token_types_start: 2029, token_types_count: 0 }, // [1206]
    SimpleHintData { raw_values_start: 2029, raw_values_count: 1, token_types_start: 2030, token_types_count: 0 }, // [1207]
    SimpleHintData { raw_values_start: 2030, raw_values_count: 2, token_types_start: 2032, token_types_count: 0 }, // [1208]
    SimpleHintData { raw_values_start: 2032, raw_values_count: 1, token_types_start: 2033, token_types_count: 0 }, // [1209]
    SimpleHintData { raw_values_start: 2033, raw_values_count: 1, token_types_start: 2034, token_types_count: 0 }, // [1210]
    SimpleHintData { raw_values_start: 2034, raw_values_count: 1, token_types_start: 2035, token_types_count: 0 }, // [1211]
    SimpleHintData { raw_values_start: 2035, raw_values_count: 1, token_types_start: 2036, token_types_count: 0 }, // [1212]
    SimpleHintData { raw_values_start: 2036, raw_values_count: 1, token_types_start: 2037, token_types_count: 0 }, // [1213]
    SimpleHintData { raw_values_start: 2037, raw_values_count: 1, token_types_start: 2038, token_types_count: 0 }, // [1214]
    SimpleHintData { raw_values_start: 2038, raw_values_count: 1, token_types_start: 2039, token_types_count: 0 }, // [1215]
    SimpleHintData { raw_values_start: 2039, raw_values_count: 1, token_types_start: 2040, token_types_count: 0 }, // [1216]
    SimpleHintData { raw_values_start: 2040, raw_values_count: 1, token_types_start: 2041, token_types_count: 0 }, // [1217]
    SimpleHintData { raw_values_start: 2041, raw_values_count: 1, token_types_start: 2042, token_types_count: 0 }, // [1218]
    SimpleHintData { raw_values_start: 2042, raw_values_count: 1, token_types_start: 2043, token_types_count: 0 }, // [1219]
    SimpleHintData { raw_values_start: 2043, raw_values_count: 1, token_types_start: 2044, token_types_count: 0 }, // [1220]
    SimpleHintData { raw_values_start: 2044, raw_values_count: 1, token_types_start: 2045, token_types_count: 0 }, // [1221]
    SimpleHintData { raw_values_start: 2045, raw_values_count: 1, token_types_start: 2046, token_types_count: 0 }, // [1222]
    SimpleHintData { raw_values_start: 2046, raw_values_count: 1, token_types_start: 2047, token_types_count: 0 }, // [1223]
    SimpleHintData { raw_values_start: 2047, raw_values_count: 1, token_types_start: 2048, token_types_count: 0 }, // [1224]
    SimpleHintData { raw_values_start: 2048, raw_values_count: 1, token_types_start: 2049, token_types_count: 0 }, // [1225]
    SimpleHintData { raw_values_start: 2049, raw_values_count: 1, token_types_start: 2050, token_types_count: 0 }, // [1226]
    SimpleHintData { raw_values_start: 2050, raw_values_count: 1, token_types_start: 2051, token_types_count: 0 }, // [1227]
    SimpleHintData { raw_values_start: 2051, raw_values_count: 1, token_types_start: 2052, token_types_count: 0 }, // [1228]
    SimpleHintData { raw_values_start: 2052, raw_values_count: 1, token_types_start: 2053, token_types_count: 0 }, // [1229]
    SimpleHintData { raw_values_start: 2053, raw_values_count: 1, token_types_start: 2054, token_types_count: 0 }, // [1230]
    SimpleHintData { raw_values_start: 2054, raw_values_count: 1, token_types_start: 2055, token_types_count: 0 }, // [1231]
    SimpleHintData { raw_values_start: 2055, raw_values_count: 1, token_types_start: 2056, token_types_count: 0 }, // [1232]
    SimpleHintData { raw_values_start: 2056, raw_values_count: 1, token_types_start: 2057, token_types_count: 0 }, // [1233]
    SimpleHintData { raw_values_start: 2057, raw_values_count: 1, token_types_start: 2058, token_types_count: 0 }, // [1234]
    SimpleHintData { raw_values_start: 2058, raw_values_count: 1, token_types_start: 2059, token_types_count: 0 }, // [1235]
    SimpleHintData { raw_values_start: 2059, raw_values_count: 1, token_types_start: 2060, token_types_count: 0 }, // [1236]
    SimpleHintData { raw_values_start: 2060, raw_values_count: 1, token_types_start: 2061, token_types_count: 0 }, // [1237]
    SimpleHintData { raw_values_start: 2061, raw_values_count: 1, token_types_start: 2062, token_types_count: 0 }, // [1238]
    SimpleHintData { raw_values_start: 2062, raw_values_count: 1, token_types_start: 2063, token_types_count: 0 }, // [1239]
    SimpleHintData { raw_values_start: 2063, raw_values_count: 1, token_types_start: 2064, token_types_count: 0 }, // [1240]
    SimpleHintData { raw_values_start: 2064, raw_values_count: 1, token_types_start: 2065, token_types_count: 0 }, // [1241]
    SimpleHintData { raw_values_start: 2065, raw_values_count: 1, token_types_start: 2066, token_types_count: 0 }, // [1242]
    SimpleHintData { raw_values_start: 2066, raw_values_count: 1, token_types_start: 2067, token_types_count: 0 }, // [1243]
    SimpleHintData { raw_values_start: 2067, raw_values_count: 1, token_types_start: 2068, token_types_count: 0 }, // [1244]
    SimpleHintData { raw_values_start: 2068, raw_values_count: 1, token_types_start: 2069, token_types_count: 0 }, // [1245]
    SimpleHintData { raw_values_start: 2069, raw_values_count: 1, token_types_start: 2070, token_types_count: 0 }, // [1246]
    SimpleHintData { raw_values_start: 2070, raw_values_count: 1, token_types_start: 2071, token_types_count: 0 }, // [1247]
    SimpleHintData { raw_values_start: 2071, raw_values_count: 1, token_types_start: 2072, token_types_count: 0 }, // [1248]
    SimpleHintData { raw_values_start: 2072, raw_values_count: 1, token_types_start: 2073, token_types_count: 0 }, // [1249]
    SimpleHintData { raw_values_start: 2073, raw_values_count: 5, token_types_start: 2078, token_types_count: 0 }, // [1250]
    SimpleHintData { raw_values_start: 2078, raw_values_count: 1, token_types_start: 2079, token_types_count: 0 }, // [1251]
    SimpleHintData { raw_values_start: 2079, raw_values_count: 1, token_types_start: 2080, token_types_count: 0 }, // [1252]
    SimpleHintData { raw_values_start: 2080, raw_values_count: 1, token_types_start: 2081, token_types_count: 0 }, // [1253]
    SimpleHintData { raw_values_start: 2081, raw_values_count: 1, token_types_start: 2082, token_types_count: 0 }, // [1254]
    SimpleHintData { raw_values_start: 2082, raw_values_count: 1, token_types_start: 2083, token_types_count: 0 }, // [1255]
    SimpleHintData { raw_values_start: 2083, raw_values_count: 1, token_types_start: 2084, token_types_count: 0 }, // [1256]
    SimpleHintData { raw_values_start: 2084, raw_values_count: 1, token_types_start: 2085, token_types_count: 0 }, // [1257]
    SimpleHintData { raw_values_start: 2085, raw_values_count: 1, token_types_start: 2086, token_types_count: 0 }, // [1258]
    SimpleHintData { raw_values_start: 2086, raw_values_count: 1, token_types_start: 2087, token_types_count: 0 }, // [1259]
    SimpleHintData { raw_values_start: 2087, raw_values_count: 1, token_types_start: 2088, token_types_count: 0 }, // [1260]
    SimpleHintData { raw_values_start: 2088, raw_values_count: 1, token_types_start: 2089, token_types_count: 0 }, // [1261]
    SimpleHintData { raw_values_start: 2089, raw_values_count: 1, token_types_start: 2090, token_types_count: 0 }, // [1262]
    SimpleHintData { raw_values_start: 2090, raw_values_count: 1, token_types_start: 2091, token_types_count: 0 }, // [1263]
    SimpleHintData { raw_values_start: 2091, raw_values_count: 1, token_types_start: 2092, token_types_count: 0 }, // [1264]
    SimpleHintData { raw_values_start: 2092, raw_values_count: 1, token_types_start: 2093, token_types_count: 0 }, // [1265]
    SimpleHintData { raw_values_start: 2093, raw_values_count: 1, token_types_start: 2094, token_types_count: 0 }, // [1266]
    SimpleHintData { raw_values_start: 2094, raw_values_count: 1, token_types_start: 2095, token_types_count: 0 }, // [1267]
    SimpleHintData { raw_values_start: 2095, raw_values_count: 1, token_types_start: 2096, token_types_count: 0 }, // [1268]
    SimpleHintData { raw_values_start: 2096, raw_values_count: 1, token_types_start: 2097, token_types_count: 0 }, // [1269]
    SimpleHintData { raw_values_start: 2097, raw_values_count: 1, token_types_start: 2098, token_types_count: 0 }, // [1270]
    SimpleHintData { raw_values_start: 2098, raw_values_count: 1, token_types_start: 2099, token_types_count: 0 }, // [1271]
    SimpleHintData { raw_values_start: 2099, raw_values_count: 1, token_types_start: 2100, token_types_count: 0 }, // [1272]
    SimpleHintData { raw_values_start: 2100, raw_values_count: 1, token_types_start: 2101, token_types_count: 0 }, // [1273]
    SimpleHintData { raw_values_start: 2101, raw_values_count: 1, token_types_start: 2102, token_types_count: 0 }, // [1274]
    SimpleHintData { raw_values_start: 2102, raw_values_count: 1, token_types_start: 2103, token_types_count: 0 }, // [1275]
    SimpleHintData { raw_values_start: 2103, raw_values_count: 1, token_types_start: 2104, token_types_count: 0 }, // [1276]
    SimpleHintData { raw_values_start: 2104, raw_values_count: 1, token_types_start: 2105, token_types_count: 0 }, // [1277]
    SimpleHintData { raw_values_start: 2105, raw_values_count: 1, token_types_start: 2106, token_types_count: 0 }, // [1278]
    SimpleHintData { raw_values_start: 2106, raw_values_count: 1, token_types_start: 2107, token_types_count: 0 }, // [1279]
    SimpleHintData { raw_values_start: 2107, raw_values_count: 4, token_types_start: 2111, token_types_count: 0 }, // [1280]
    SimpleHintData { raw_values_start: 2111, raw_values_count: 1, token_types_start: 2112, token_types_count: 0 }, // [1281]
    SimpleHintData { raw_values_start: 2112, raw_values_count: 1, token_types_start: 2113, token_types_count: 0 }, // [1282]
    SimpleHintData { raw_values_start: 2113, raw_values_count: 3, token_types_start: 2116, token_types_count: 0 }, // [1283]
    SimpleHintData { raw_values_start: 2116, raw_values_count: 1, token_types_start: 2117, token_types_count: 0 }, // [1284]
    SimpleHintData { raw_values_start: 2117, raw_values_count: 1, token_types_start: 2118, token_types_count: 0 }, // [1285]
    SimpleHintData { raw_values_start: 2118, raw_values_count: 1, token_types_start: 2119, token_types_count: 0 }, // [1286]
    SimpleHintData { raw_values_start: 2119, raw_values_count: 1, token_types_start: 2120, token_types_count: 0 }, // [1287]
    SimpleHintData { raw_values_start: 2120, raw_values_count: 1, token_types_start: 2121, token_types_count: 0 }, // [1288]
    SimpleHintData { raw_values_start: 2121, raw_values_count: 1, token_types_start: 2122, token_types_count: 0 }, // [1289]
    SimpleHintData { raw_values_start: 2122, raw_values_count: 2, token_types_start: 2124, token_types_count: 0 }, // [1290]
    SimpleHintData { raw_values_start: 2124, raw_values_count: 1, token_types_start: 2125, token_types_count: 0 }, // [1291]
    SimpleHintData { raw_values_start: 2125, raw_values_count: 1, token_types_start: 2126, token_types_count: 0 }, // [1292]
    SimpleHintData { raw_values_start: 2126, raw_values_count: 1, token_types_start: 2127, token_types_count: 0 }, // [1293]
    SimpleHintData { raw_values_start: 2127, raw_values_count: 1, token_types_start: 2128, token_types_count: 0 }, // [1294]
    SimpleHintData { raw_values_start: 2128, raw_values_count: 1, token_types_start: 2129, token_types_count: 0 }, // [1295]
    SimpleHintData { raw_values_start: 2129, raw_values_count: 1, token_types_start: 2130, token_types_count: 0 }, // [1296]
    SimpleHintData { raw_values_start: 2130, raw_values_count: 1, token_types_start: 2131, token_types_count: 0 }, // [1297]
    SimpleHintData { raw_values_start: 2131, raw_values_count: 1, token_types_start: 2132, token_types_count: 0 }, // [1298]
    SimpleHintData { raw_values_start: 2132, raw_values_count: 1, token_types_start: 2133, token_types_count: 0 }, // [1299]
    SimpleHintData { raw_values_start: 2133, raw_values_count: 1, token_types_start: 2134, token_types_count: 0 }, // [1300]
    SimpleHintData { raw_values_start: 2134, raw_values_count: 1, token_types_start: 2135, token_types_count: 0 }, // [1301]
    SimpleHintData { raw_values_start: 2135, raw_values_count: 1, token_types_start: 2136, token_types_count: 0 }, // [1302]
    SimpleHintData { raw_values_start: 2136, raw_values_count: 1, token_types_start: 2137, token_types_count: 0 }, // [1303]
    SimpleHintData { raw_values_start: 2137, raw_values_count: 1, token_types_start: 2138, token_types_count: 0 }, // [1304]
    SimpleHintData { raw_values_start: 2138, raw_values_count: 1, token_types_start: 2139, token_types_count: 0 }, // [1305]
    SimpleHintData { raw_values_start: 2139, raw_values_count: 1, token_types_start: 2140, token_types_count: 0 }, // [1306]
    SimpleHintData { raw_values_start: 2140, raw_values_count: 1, token_types_start: 2141, token_types_count: 0 }, // [1307]
    SimpleHintData { raw_values_start: 2141, raw_values_count: 2, token_types_start: 2143, token_types_count: 0 }, // [1308]
    SimpleHintData { raw_values_start: 2143, raw_values_count: 1, token_types_start: 2144, token_types_count: 0 }, // [1309]
    SimpleHintData { raw_values_start: 2144, raw_values_count: 1, token_types_start: 2145, token_types_count: 0 }, // [1310]
    SimpleHintData { raw_values_start: 2145, raw_values_count: 4, token_types_start: 2149, token_types_count: 0 }, // [1311]
    SimpleHintData { raw_values_start: 2149, raw_values_count: 1, token_types_start: 2150, token_types_count: 0 }, // [1312]
    SimpleHintData { raw_values_start: 2150, raw_values_count: 1, token_types_start: 2151, token_types_count: 0 }, // [1313]
    SimpleHintData { raw_values_start: 2151, raw_values_count: 1, token_types_start: 2152, token_types_count: 0 }, // [1314]
    SimpleHintData { raw_values_start: 2152, raw_values_count: 1, token_types_start: 2153, token_types_count: 0 }, // [1315]
    SimpleHintData { raw_values_start: 2153, raw_values_count: 1, token_types_start: 2154, token_types_count: 0 }, // [1316]
    SimpleHintData { raw_values_start: 2154, raw_values_count: 1, token_types_start: 2155, token_types_count: 0 }, // [1317]
    SimpleHintData { raw_values_start: 2155, raw_values_count: 1, token_types_start: 2156, token_types_count: 0 }, // [1318]
    SimpleHintData { raw_values_start: 2156, raw_values_count: 1, token_types_start: 2157, token_types_count: 0 }, // [1319]
    SimpleHintData { raw_values_start: 2157, raw_values_count: 1, token_types_start: 2158, token_types_count: 0 }, // [1320]
    SimpleHintData { raw_values_start: 2158, raw_values_count: 1, token_types_start: 2159, token_types_count: 0 }, // [1321]
    SimpleHintData { raw_values_start: 2159, raw_values_count: 1, token_types_start: 2160, token_types_count: 0 }, // [1322]
    SimpleHintData { raw_values_start: 2160, raw_values_count: 1, token_types_start: 2161, token_types_count: 0 }, // [1323]
    SimpleHintData { raw_values_start: 2161, raw_values_count: 1, token_types_start: 2162, token_types_count: 0 }, // [1324]
    SimpleHintData { raw_values_start: 2162, raw_values_count: 1, token_types_start: 2163, token_types_count: 0 }, // [1325]
    SimpleHintData { raw_values_start: 2163, raw_values_count: 1, token_types_start: 2164, token_types_count: 0 }, // [1326]
    SimpleHintData { raw_values_start: 2164, raw_values_count: 1, token_types_start: 2165, token_types_count: 0 }, // [1327]
    SimpleHintData { raw_values_start: 2165, raw_values_count: 1, token_types_start: 2166, token_types_count: 0 }, // [1328]
    SimpleHintData { raw_values_start: 2166, raw_values_count: 1, token_types_start: 2167, token_types_count: 0 }, // [1329]
    SimpleHintData { raw_values_start: 2167, raw_values_count: 1, token_types_start: 2168, token_types_count: 0 }, // [1330]
    SimpleHintData { raw_values_start: 2168, raw_values_count: 1, token_types_start: 2169, token_types_count: 0 }, // [1331]
    SimpleHintData { raw_values_start: 2169, raw_values_count: 1, token_types_start: 2170, token_types_count: 0 }, // [1332]
    SimpleHintData { raw_values_start: 2170, raw_values_count: 1, token_types_start: 2171, token_types_count: 0 }, // [1333]
    SimpleHintData { raw_values_start: 2171, raw_values_count: 1, token_types_start: 2172, token_types_count: 0 }, // [1334]
    SimpleHintData { raw_values_start: 2172, raw_values_count: 1, token_types_start: 2173, token_types_count: 0 }, // [1335]
    SimpleHintData { raw_values_start: 2173, raw_values_count: 1, token_types_start: 2174, token_types_count: 0 }, // [1336]
    SimpleHintData { raw_values_start: 2174, raw_values_count: 1, token_types_start: 2175, token_types_count: 0 }, // [1337]
    SimpleHintData { raw_values_start: 2175, raw_values_count: 1, token_types_start: 2176, token_types_count: 0 }, // [1338]
    SimpleHintData { raw_values_start: 2176, raw_values_count: 1, token_types_start: 2177, token_types_count: 0 }, // [1339]
    SimpleHintData { raw_values_start: 2177, raw_values_count: 1, token_types_start: 2178, token_types_count: 0 }, // [1340]
    SimpleHintData { raw_values_start: 2178, raw_values_count: 1, token_types_start: 2179, token_types_count: 0 }, // [1341]
    SimpleHintData { raw_values_start: 2179, raw_values_count: 1, token_types_start: 2180, token_types_count: 0 }, // [1342]
    SimpleHintData { raw_values_start: 2180, raw_values_count: 1, token_types_start: 2181, token_types_count: 0 }, // [1343]
    SimpleHintData { raw_values_start: 2181, raw_values_count: 1, token_types_start: 2182, token_types_count: 0 }, // [1344]
    SimpleHintData { raw_values_start: 2182, raw_values_count: 1, token_types_start: 2183, token_types_count: 0 }, // [1345]
    SimpleHintData { raw_values_start: 2183, raw_values_count: 1, token_types_start: 2184, token_types_count: 0 }, // [1346]
    SimpleHintData { raw_values_start: 2184, raw_values_count: 1, token_types_start: 2185, token_types_count: 0 }, // [1347]
    SimpleHintData { raw_values_start: 2185, raw_values_count: 1, token_types_start: 2186, token_types_count: 0 }, // [1348]
    SimpleHintData { raw_values_start: 2186, raw_values_count: 1, token_types_start: 2187, token_types_count: 0 }, // [1349]
    SimpleHintData { raw_values_start: 2187, raw_values_count: 1, token_types_start: 2188, token_types_count: 0 }, // [1350]
    SimpleHintData { raw_values_start: 2188, raw_values_count: 1, token_types_start: 2189, token_types_count: 0 }, // [1351]
    SimpleHintData { raw_values_start: 2189, raw_values_count: 1, token_types_start: 2190, token_types_count: 0 }, // [1352]
    SimpleHintData { raw_values_start: 2190, raw_values_count: 1, token_types_start: 2191, token_types_count: 0 }, // [1353]
    SimpleHintData { raw_values_start: 2191, raw_values_count: 1, token_types_start: 2192, token_types_count: 0 }, // [1354]
    SimpleHintData { raw_values_start: 2192, raw_values_count: 1, token_types_start: 2193, token_types_count: 0 }, // [1355]
    SimpleHintData { raw_values_start: 2193, raw_values_count: 1, token_types_start: 2194, token_types_count: 0 }, // [1356]
    SimpleHintData { raw_values_start: 2194, raw_values_count: 1, token_types_start: 2195, token_types_count: 0 }, // [1357]
    SimpleHintData { raw_values_start: 2195, raw_values_count: 1, token_types_start: 2196, token_types_count: 0 }, // [1358]
    SimpleHintData { raw_values_start: 2196, raw_values_count: 2, token_types_start: 2198, token_types_count: 0 }, // [1359]
    SimpleHintData { raw_values_start: 2198, raw_values_count: 1, token_types_start: 2199, token_types_count: 0 }, // [1360]
    SimpleHintData { raw_values_start: 2199, raw_values_count: 1, token_types_start: 2200, token_types_count: 0 }, // [1361]
    SimpleHintData { raw_values_start: 2200, raw_values_count: 1, token_types_start: 2201, token_types_count: 0 }, // [1362]
    SimpleHintData { raw_values_start: 2201, raw_values_count: 1, token_types_start: 2202, token_types_count: 0 }, // [1363]
    SimpleHintData { raw_values_start: 2202, raw_values_count: 1, token_types_start: 2203, token_types_count: 0 }, // [1364]
    SimpleHintData { raw_values_start: 2203, raw_values_count: 1, token_types_start: 2204, token_types_count: 0 }, // [1365]
    SimpleHintData { raw_values_start: 2204, raw_values_count: 1, token_types_start: 2205, token_types_count: 0 }, // [1366]
    SimpleHintData { raw_values_start: 2205, raw_values_count: 1, token_types_start: 2206, token_types_count: 0 }, // [1367]
    SimpleHintData { raw_values_start: 2206, raw_values_count: 1, token_types_start: 2207, token_types_count: 0 }, // [1368]
    SimpleHintData { raw_values_start: 2207, raw_values_count: 1, token_types_start: 2208, token_types_count: 0 }, // [1369]
    SimpleHintData { raw_values_start: 2208, raw_values_count: 1, token_types_start: 2209, token_types_count: 0 }, // [1370]
    SimpleHintData { raw_values_start: 2209, raw_values_count: 1, token_types_start: 2210, token_types_count: 0 }, // [1371]
    SimpleHintData { raw_values_start: 2210, raw_values_count: 1, token_types_start: 2211, token_types_count: 0 }, // [1372]
    SimpleHintData { raw_values_start: 2211, raw_values_count: 1, token_types_start: 2212, token_types_count: 0 }, // [1373]
    SimpleHintData { raw_values_start: 2212, raw_values_count: 1, token_types_start: 2213, token_types_count: 0 }, // [1374]
    SimpleHintData { raw_values_start: 2213, raw_values_count: 1, token_types_start: 2214, token_types_count: 0 }, // [1375]
    SimpleHintData { raw_values_start: 2214, raw_values_count: 1, token_types_start: 2215, token_types_count: 0 }, // [1376]
    SimpleHintData { raw_values_start: 2215, raw_values_count: 1, token_types_start: 2216, token_types_count: 0 }, // [1377]
    SimpleHintData { raw_values_start: 2216, raw_values_count: 8, token_types_start: 2224, token_types_count: 0 }, // [1378]
    SimpleHintData { raw_values_start: 2224, raw_values_count: 1, token_types_start: 2225, token_types_count: 0 }, // [1379]
    SimpleHintData { raw_values_start: 2225, raw_values_count: 1, token_types_start: 2226, token_types_count: 0 }, // [1380]
    SimpleHintData { raw_values_start: 2226, raw_values_count: 2, token_types_start: 2228, token_types_count: 0 }, // [1381]
    SimpleHintData { raw_values_start: 2228, raw_values_count: 1, token_types_start: 2229, token_types_count: 0 }, // [1382]
    SimpleHintData { raw_values_start: 2229, raw_values_count: 1, token_types_start: 2230, token_types_count: 0 }, // [1383]
    SimpleHintData { raw_values_start: 2230, raw_values_count: 1, token_types_start: 2231, token_types_count: 0 }, // [1384]
    SimpleHintData { raw_values_start: 2231, raw_values_count: 1, token_types_start: 2232, token_types_count: 0 }, // [1385]
    SimpleHintData { raw_values_start: 2232, raw_values_count: 1, token_types_start: 2233, token_types_count: 0 }, // [1386]
    SimpleHintData { raw_values_start: 2233, raw_values_count: 1, token_types_start: 2234, token_types_count: 0 }, // [1387]
    SimpleHintData { raw_values_start: 2234, raw_values_count: 1, token_types_start: 2235, token_types_count: 0 }, // [1388]
    SimpleHintData { raw_values_start: 2235, raw_values_count: 1, token_types_start: 2236, token_types_count: 0 }, // [1389]
    SimpleHintData { raw_values_start: 2236, raw_values_count: 1, token_types_start: 2237, token_types_count: 0 }, // [1390]
    SimpleHintData { raw_values_start: 2237, raw_values_count: 1, token_types_start: 2238, token_types_count: 0 }, // [1391]
    SimpleHintData { raw_values_start: 2238, raw_values_count: 1, token_types_start: 2239, token_types_count: 0 }, // [1392]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 0, 5, 0, 6, 7, 7, 8, 9, 10, 11, 12, 13,
    14, 15, 16, 17, 18, 19, 19, 20, 21, 22, 22, 23, 24, 24, 25, 26,
    10, 27, 27, 25, 26, 10, 28, 24, 24, 29, 26, 14, 27, 27, 29, 26,
    30, 10, 14, 0, 0, 0, 0, 0, 31, 0, 32, 33, 34, 35, 35, 36,
    37, 38, 39, 39, 40, 16, 41, 9, 10, 12, 42, 42, 43, 44, 45, 45,
    46, 47, 47, 48, 8, 8, 49, 50, 51, 52, 52, 39, 39, 53, 14, 43,
    43, 44, 54, 28, 55, 55, 56, 14, 15, 57, 57, 58, 42, 59, 39, 60,
    45, 61, 62, 8, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 51,
    74, 75, 24, 24, 58, 31, 38, 38, 76, 28, 77, 78, 79, 80, 81, 82,
    83, 84, 46, 14, 43, 43, 44, 85, 85, 78, 86, 86, 77, 87, 87, 88,
    29, 89, 90, 91, 92, 93, 94, 95, 96, 97, 25, 43, 43, 0, 85, 85,
    90, 86, 86, 89, 87, 87, 0, 98, 99, 100, 0, 101, 101, 0, 0, 41,
    41, 0, 0, 16, 16, 0, 0, 102, 102, 0, 0, 0, 0, 0, 0, 103,
    104, 104, 105, 5, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 0,
    0, 0, 117, 0, 0, 0, 31, 0, 31, 118, 31, 31, 0, 119, 0, 0,
    117, 31, 0, 0, 120, 0, 0, 0, 24, 121, 122, 99, 99, 123, 76, 0,
    0, 0, 77, 0, 24, 24, 24, 89, 26, 14, 0, 124, 33, 0, 0, 124,
    124, 124, 99, 125, 123, 24, 24, 58, 126, 127, 127, 68, 61, 73, 60, 67,
    72, 128, 129, 76, 128, 128, 128, 14, 0, 0, 130, 131, 131, 132, 133, 133,
    33, 0, 0, 134, 134, 33, 0, 135, 136, 137, 138, 138, 139, 37, 140, 141,
    142, 143, 143, 37, 144, 145, 146, 147, 148, 128, 128, 83, 0, 137, 137, 128,
    128, 149, 150, 151, 151, 152, 0, 31, 0, 0, 153, 154, 155, 155, 152, 156,
    31, 0, 0, 153, 154, 152, 152, 24, 151, 151, 157, 158, 158, 0, 31, 0,
    0, 153, 154, 155, 155, 157, 158, 158, 0, 159, 24, 31, 0, 160, 111, 111,
    161, 128, 128, 162, 0, 163, 164, 165, 136, 166, 166, 167, 0, 132, 134, 134,
    33, 0, 151, 151, 168, 109, 109, 0, 0, 131, 169, 169, 170, 171, 171, 0,
    0, 0, 129, 129, 172, 173, 0, 0, 0, 0, 153, 0, 174, 0, 175, 175,
    111, 62, 172, 0, 176, 177, 177, 178, 114, 0, 31, 129, 133, 133, 179, 117,
    33, 0, 128, 128, 77, 0, 0, 180, 160, 160, 160, 111, 111, 172, 173, 0,
    0, 181, 181, 182, 183, 184, 184, 31, 0, 0, 24, 24, 184, 181, 128, 128,
    172, 0, 185, 182, 182, 184, 31, 0, 0, 151, 151, 186, 151, 151, 187, 188,
    151, 151, 22, 18, 0, 129, 129, 189, 131, 151, 151, 190, 131, 131, 190, 187,
    187, 188, 129, 129, 162, 0, 191, 135, 192, 129, 129, 172, 173, 0, 191, 193,
    194, 194, 195, 133, 133, 172, 0, 33, 0, 133, 133, 33, 0, 0, 196, 151,
    151, 14, 0, 128, 128, 78, 0, 0, 197, 134, 134, 33, 0, 151, 151, 14,
    0, 132, 133, 133, 33, 0, 0, 198, 199, 200, 201, 202, 203, 203, 204, 205,
    206, 42, 207, 208, 209, 210, 211, 212, 213, 214, 215, 199, 216, 217, 218, 219,
    104, 0, 0, 37, 0, 220, 104, 221, 0, 222, 104, 0, 0, 222, 222, 104,
    0, 117, 117, 117, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234,
    235, 235, 236, 235, 237, 238, 239, 240, 241, 242, 243, 244, 0, 174, 244, 245,
    0, 0, 0, 0, 0, 174, 32, 117, 0, 0, 246, 247, 248, 249, 250, 251,
    252, 253, 254, 255, 209, 216, 256, 257, 0, 258, 259, 260, 261, 199, 218, 218,
    218, 216, 219, 219, 219, 217, 262, 263, 264, 265, 266, 256, 203, 267, 268, 269,
    270, 271, 272, 31, 174, 174, 31, 0, 0, 0, 31, 273, 274, 275, 276, 277,
    278, 279, 139, 280, 281, 282, 283, 31, 0, 0, 0, 31, 117, 31, 284, 285,
    143, 286, 287, 288, 135, 289, 290, 290, 290, 0, 291, 291, 292, 293, 292, 293,
    293, 290, 290, 0, 291, 291, 292, 293, 292, 293, 293, 257, 32, 255, 290, 294,
    295, 15, 296, 297, 298, 299, 300, 301, 302, 303, 0, 304, 305, 306, 307, 308,
    309, 310, 311, 312, 313, 314, 315, 316, 317, 318, 319, 155, 320, 321, 322, 323,
    324, 325, 0, 0, 326, 327, 0, 154, 0, 0, 328, 0, 329, 330, 331, 332,
    333, 334, 335, 336, 337, 220, 220, 220, 338, 0, 0, 174, 0, 244, 181, 181,
    339, 162, 162, 0, 340, 341, 187, 188, 313, 313, 31, 0, 163, 164, 165, 342,
    342, 163, 164, 165, 343, 343, 344, 163, 164, 165, 67, 186, 0, 0, 0, 31,
    0, 181, 181, 182, 182, 345, 0, 172, 0, 0, 328, 0, 186, 186, 131, 346,
    151, 190, 131, 131, 190, 0, 0, 174, 0, 244, 0, 31, 284, 347, 0, 0,
    0, 0, 348, 349, 32, 350, 351, 352, 352, 154, 352, 352, 352, 76, 353, 353,
    354, 354, 77, 78, 355, 0, 172, 172, 0, 162, 162, 0, 76, 76, 0, 356,
    357, 115, 201, 358, 80, 0, 31, 31, 14, 14, 0, 359, 359, 0, 360, 360,
    83, 361, 0, 362, 362, 363, 154, 188, 0, 364, 365, 366, 367, 257, 153, 368,
    219, 218, 219, 218, 369, 370, 362, 0, 371, 372, 0, 0, 373, 374, 374, 375,
    216, 216, 216, 376, 377, 378, 0, 379, 378, 0, 380, 59, 381, 382, 162, 383,
    384, 385, 386, 387, 388, 389, 390, 391, 392, 393, 0, 0, 394, 395, 394, 394,
    396, 396, 117, 117, 154, 397, 397, 139, 139, 154, 398, 399, 400, 400, 117, 117,
    154, 401, 401, 400, 402, 402, 0, 88, 88, 154, 188, 188, 117, 117, 154, 403,
    403, 154, 404, 404, 1, 1, 76, 405, 405, 405, 406, 407, 407, 154, 408, 408,
    409, 154, 410, 410, 22, 154, 411, 411, 0, 412, 412, 0, 412, 412, 281, 0,
    413, 413, 414, 394, 395, 395, 0, 415, 31, 0, 0, 172, 172, 125, 31, 0,
    0, 0, 0, 34, 416, 417, 418, 419, 374, 154, 418, 418, 154, 374, 420, 420,
    0, 31, 0, 0, 154, 154, 154, 421, 422, 422, 422, 423, 424, 425, 404, 426,
    427, 428, 429, 430, 431, 432, 433, 39, 39, 294, 31, 0, 117, 0, 124, 434,
    435, 82, 80, 81, 436, 437, 438, 439, 387, 440, 0, 31, 158, 158, 0, 117,
    117, 226, 39, 39, 10, 173, 0, 39, 39, 86, 77, 173, 0, 31, 31, 0,
    0, 0, 0, 0, 441, 181, 182, 105, 105, 0, 161, 132, 117, 395, 415, 31,
    0, 0, 0, 0, 172, 172, 125, 31, 0, 0, 0, 0, 34, 154, 421, 422,
    422, 422, 423, 424, 425, 404, 39, 39, 267, 442, 80, 173, 0, 31, 443, 443,
    0, 117, 39, 39, 267, 342, 444, 173, 0, 76, 0, 31, 31, 0, 0, 39,
    39, 39, 267, 445, 173, 0, 446, 446, 31, 0, 0, 0, 153, 174, 174, 104,
    154, 154, 117, 284, 39, 39, 355, 173, 0, 31, 0, 0, 0, 182, 105, 105,
    0, 117, 284, 447, 116, 448, 101, 76, 449, 450, 116, 447, 76, 76, 193, 451,
    252, 154, 203, 154, 101, 448, 452, 39, 39, 16, 173, 0, 39, 39, 14, 173,
    0, 453, 234, 234, 0, 131, 131, 132, 136, 454, 138, 138, 139, 37, 455, 455,
    124, 37, 140, 141, 142, 143, 143, 37, 144, 145, 146, 147, 148, 39, 39, 83,
    0, 454, 454, 39, 39, 456, 456, 457, 458, 418, 442, 77, 173, 0, 459, 76,
    76, 364, 460, 60, 461, 170, 401, 401, 355, 462, 31, 0, 0, 0, 182, 105,
    105, 0, 132, 117, 228, 228, 463, 464, 37, 465, 284, 447, 116, 116, 447, 39,
    39, 77, 173, 0, 466, 466, 0, 467, 467, 136, 39, 39, 456, 456, 457, 458,
    418, 442, 77, 173, 0, 31, 31, 0, 0, 0, 0, 0, 441, 181, 182, 105,
    105, 0, 161, 468, 116, 447, 452, 132, 136, 193, 459, 76, 76, 364, 460, 60,
    461, 170, 401, 401, 355, 39, 39, 71, 0, 469, 248, 114, 470, 470, 471, 472,
    472, 61, 60, 73, 73, 471, 0, 0, 267, 76, 267, 76, 76, 0, 473, 474,
    474, 475, 476, 117, 0, 477, 476, 117, 0, 34, 34, 0, 478, 187, 187, 479,
    480, 479, 481, 481, 481, 482, 481, 481, 483, 158, 158, 484, 485, 476, 486, 291,
    291, 31, 0, 45, 45, 81, 487, 31, 39, 39, 41, 0, 39, 39, 267, 78,
    173, 0, 31, 117, 284, 31, 284, 124, 488, 489, 490, 491, 378, 492, 493, 493,
    494, 494, 493, 495, 31, 0, 496, 497, 498, 499, 500, 501, 502, 503, 504, 505,
    506, 507, 508, 509, 510, 511, 146, 22, 10, 512, 513, 0, 0, 0, 0, 0,
    0, 514, 515, 516, 517, 245, 245, 518, 519, 519, 520, 521, 521, 31, 522, 523,
    524, 525, 253, 526, 527, 528, 529, 529, 530, 531, 532, 533, 31, 534, 534, 535,
    536, 31, 37, 537, 538, 539, 423, 31, 304, 304, 31, 540, 254, 541, 539, 542,
    280, 543, 268, 544, 104, 104, 0, 222, 222, 0, 0, 545, 545, 306, 254, 546,
    547, 35, 35, 36, 0, 0, 0, 328, 0, 31, 515, 548, 548, 31, 31, 0,
    0, 465, 0, 549, 549, 515, 550, 551, 245, 119, 518, 552, 553, 516, 465, 554,
    555, 556, 557, 558, 559, 560, 561, 562, 563, 564, 565, 566, 567, 568, 569, 570,
    530, 571, 572, 131, 131, 131, 573, 574, 479, 483, 575, 576, 577, 578, 579, 580,
    60, 60, 60, 34, 581, 582, 583, 396, 584, 585, 586, 587, 588, 589, 590, 591,
    592, 593, 594, 595, 595, 595, 0, 0, 596, 597, 598, 599, 600, 601, 602, 603,
    604, 605, 606, 607, 608, 165, 609, 610, 136, 136, 154, 136, 611, 612, 613, 614,
    615, 616, 212, 617, 11, 328, 519, 191, 192, 135, 129, 129, 294, 31, 0, 117,
    0, 191, 129, 129, 10, 173, 0, 191, 129, 129, 80, 173, 0, 129, 129, 444,
    173, 0, 191, 129, 129, 129, 445, 173, 0, 129, 129, 355, 173, 0, 0, 191,
    129, 129, 16, 173, 0, 129, 129, 14, 173, 0, 191, 129, 129, 83, 0, 129,
    129, 442, 77, 173, 0, 0, 191, 129, 129, 71, 173, 0, 129, 129, 18, 173,
    0, 191, 129, 129, 41, 173, 0, 129, 129, 78, 173, 0, 191, 618, 619, 620,
    621, 622, 623, 624, 484, 625, 292, 292, 0, 292, 626, 31, 0, 0, 31, 627,
    164, 397, 628, 182, 345, 345, 629, 630, 631, 293, 398, 632, 633, 634, 635, 153,
    636, 405, 400, 637, 638, 639, 640, 641, 642, 407, 643, 644, 645, 646, 647, 45,
    50, 648, 649, 650, 651, 652, 653, 654, 654, 654, 655, 284, 61, 73, 60, 656,
    0, 0, 0, 0, 0, 657, 255, 255, 0, 658, 362, 362, 187, 0, 659, 660,
    660, 660, 0, 661, 0, 662, 187, 252, 0, 203, 0, 663, 664, 664, 664, 665,
    187, 666, 0, 0, 667, 667, 667, 667, 209, 216, 257, 0, 668, 664, 664, 664,
    665, 0, 650, 650, 31, 284, 0, 0, 0, 290, 228, 228, 0, 32, 0, 0,
    244, 0, 31, 0, 0, 0, 284, 0, 0, 174, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 31, 0, 0, 0, 0, 669, 154, 37, 269, 188, 549, 0,
    0, 32, 104, 0, 68, 174, 245, 0, 0, 0, 670, 126, 0, 671, 672, 86,
    673, 674, 675, 271, 271, 676, 677, 678, 678, 679, 178, 680, 0, 37, 0, 476,
    681, 476, 170, 682, 683, 124, 124, 684, 678, 685, 87, 686, 583, 583, 687, 687,
    583, 583, 583, 583, 688, 402, 689, 690, 690, 31, 581, 581, 0, 690, 691, 178,
    692, 693, 694, 695, 522, 521, 696, 697, 698, 158, 166, 19, 19, 344, 19, 88,
    699, 700, 701, 702, 702, 703, 704, 497, 497, 476, 705, 705, 37, 706, 154, 154,
    295, 245, 706, 245, 707, 518, 154, 465, 708, 709, 710, 698, 252, 252, 203, 702,
    170, 451, 711, 712, 713, 714, 34, 34, 0, 0, 715, 581, 448, 101, 101, 139,
    116, 116, 139, 716, 717, 718, 719, 124, 124, 678, 720, 0, 0, 0, 0, 31,
    0, 0, 0, 0, 715, 721, 0, 0, 124, 124, 720, 0, 721, 0, 31, 0,
    0, 0, 0, 31, 0, 0, 722, 0, 31, 0, 116, 116, 139, 101, 101, 139,
    0, 0, 0, 0, 0, 0, 116, 116, 139, 101, 101, 139, 0, 34, 723, 724,
    0, 0, 0, 0, 0, 273, 31, 273, 0, 0, 117, 0, 0, 549, 725, 725,
    0, 34, 34, 0, 0, 0, 465, 0, 726, 34, 34, 0, 158, 158, 0, 0,
    612, 0, 214, 0, 0, 116, 727, 727, 174, 0, 0, 0, 154, 0, 0, 26,
    0, 0, 154, 0, 0, 0, 612, 0, 154, 0, 0, 190, 190, 728, 0, 0,
    0, 153, 729, 154, 269, 37, 0, 730, 0, 31, 31, 0, 117, 117, 154, 13,
    13, 0, 80, 487, 0, 0, 0, 0, 328, 31, 731, 487, 732, 31, 0, 0,
    0, 0, 206, 206, 18, 0, 0, 206, 206, 18, 0, 31, 0, 0, 0, 548,
    548, 548, 31, 0, 0, 0, 0, 0, 548, 0, 733, 31, 0, 0, 92, 734,
    27, 735, 736, 737, 738, 536, 535, 739, 740, 741, 458, 742, 743, 744, 99, 99,
    99, 0, 38, 38, 76, 0, 16, 16, 0, 64, 64, 76, 41, 0, 0, 33,
    0, 745, 124, 124, 99, 125, 124, 124, 113, 125, 395, 395, 497, 48, 746, 746,
    139, 0, 507, 747, 0, 746, 48, 219, 219, 153, 219, 748, 101, 101, 139, 0,
    0, 0, 0, 37, 0, 749, 749, 116, 116, 139, 448, 716, 717, 718, 678, 101,
    750, 0, 0, 0, 0, 37, 0, 31, 749, 751, 751, 751, 752, 31, 0, 0,
    494, 0, 753, 754, 755, 716, 716, 0, 31, 0, 756, 116, 116, 139, 448, 717,
    718, 678, 716, 757, 758, 759, 760, 761, 762, 763, 764, 765, 766, 767, 768, 769,
    770, 771, 0, 772, 773, 774, 775, 776, 777, 778, 779, 780, 781, 782, 0, 783,
    784, 785, 786, 173, 173, 650, 173, 173, 173, 187, 650, 787, 730, 730, 787, 788,
    789, 790, 791, 482, 169, 792, 793, 57, 26, 658, 187, 26, 0, 31, 0, 0,
    0, 284, 0, 794, 795, 138, 0, 0, 796, 223, 594, 444, 797, 798, 799, 800,
    801, 802, 803, 481, 804, 805, 806, 807, 61, 61, 61, 808, 809, 0, 810, 284,
    31, 31, 284, 131, 811, 438, 812, 470, 813, 814, 815, 816, 526, 525, 213, 524,
    0, 12, 817, 818, 660, 660, 819, 820, 680, 821, 660, 822, 823, 245, 245, 824,
    37, 37, 825, 154, 465, 154, 154, 154, 825, 154, 465, 245, 518, 518, 33, 33,
    518, 518, 31, 518, 826, 827, 557, 775, 828, 829, 830, 831, 809, 832, 0, 188,
    833, 0, 269, 0, 362, 362, 187, 612, 34, 362, 659, 834, 659, 835, 836, 0,
    0, 0, 837, 0, 0, 0, 0, 0, 0, 838, 76, 190, 0, 0, 0, 837,
    0, 0, 0, 0, 0, 837, 837, 837, 0, 76, 76, 0, 0, 31, 0, 0,
    379, 205, 839, 840, 841, 841, 205, 804, 842, 842, 723, 843, 844, 167, 845, 845,
    846, 804, 842, 842, 844, 843, 723, 167, 190, 190, 31, 0, 0, 847, 848, 344,
    849, 850, 851, 852, 0, 853, 452, 452, 452, 37, 854, 855, 0, 0, 0, 31,
    0, 0, 855, 0, 856, 13, 35, 857, 858, 0, 859, 464, 860, 861, 862, 843,
    863, 864, 218, 218, 153, 218, 865, 361, 657, 657, 187, 866, 0, 400, 400, 0,
    866, 466, 867, 790, 466, 370, 467, 467, 868, 868, 795, 645, 184, 132, 869, 448,
    448, 0, 31, 0, 37, 0, 24, 0, 720, 720, 0, 37, 0, 32, 32, 37,
    0, 448, 870, 871, 0, 872, 174, 154, 37, 269, 664, 188, 549, 104, 222, 873,
    0, 0, 874, 875, 876, 0, 418, 877, 878, 879, 880, 881, 882, 883, 884, 885,
    539, 886, 887, 888, 889, 890, 891, 892, 893, 722, 722, 328, 0, 31, 445, 445,
    0, 32, 32, 31, 284, 47, 894, 895, 0, 896, 43, 0, 897, 898, 899, 85,
    900, 901, 902, 903, 904, 905, 906, 907, 908, 909, 910, 911, 912, 913, 914, 915,
    916, 60, 61, 61, 31, 733, 917, 917, 809, 917, 291, 291, 291, 291, 291, 898,
    203, 203, 0, 918, 919, 73, 60, 291, 291, 187, 898, 203, 203, 0, 918, 61,
    917, 917, 0, 0, 0, 190, 0, 0, 0, 31, 31, 284, 0, 0, 0, 0,
    0, 76, 0, 291, 73, 73, 151, 171, 920, 921, 922, 440, 923, 924, 925, 926,
    927, 928, 929, 211, 828, 930, 931, 932, 933, 722, 934, 935, 936, 445, 937, 62,
    938, 215, 533, 8, 827, 939, 940, 941, 942, 943, 214, 944, 945, 946, 0, 947,
    0, 0, 117, 0, 0, 31, 0, 718, 718, 0, 0, 948, 949, 833, 833, 950,
    422, 845, 845, 379, 845, 951, 952, 211, 0, 953, 954, 477, 0, 680, 401, 955,
    956, 144, 957, 958, 959, 960, 961, 147, 359, 962, 963, 964, 965, 273, 733, 68,
    31, 68, 31, 124, 31, 273, 31, 0, 966, 73, 61, 60, 917, 273, 273, 967,
    31, 68, 273, 968, 969, 970, 971, 0, 972, 973, 0, 369, 974, 974, 153, 218,
    218, 219, 187, 661, 187, 975, 976, 977, 661, 978, 979, 368, 368, 188, 188, 980,
    841, 414, 981, 789, 982, 532, 531, 37, 36, 154, 154, 220, 0, 873, 154, 154,
    873, 328, 328, 328, 328, 328, 0, 76, 117, 190, 32, 295, 979, 104, 31, 255,
    220, 583, 0, 983, 984, 985, 471, 986, 987, 720, 720, 0, 37, 0, 476, 681,
    476, 170, 720, 988, 989, 475, 76, 990, 683, 991, 992, 993, 994, 995, 63, 996,
    997, 998, 999, 1000, 125, 1001, 446, 1002, 267, 267, 267, 267, 1003, 424, 116, 116,
    139, 0, 0, 0, 0, 37, 0, 796, 223, 594, 789, 789, 1004, 178, 857, 0,
    448, 702, 1005, 448, 716, 717, 718, 702, 727, 678, 116, 148, 116, 970, 1006, 1007,
    1008, 1009, 167, 1010, 1011, 0, 730, 1012, 0, 0, 31, 0, 1012, 670, 670, 0,
    31, 31, 549, 32, 549, 0, 670, 1013, 1014, 808, 134, 64, 1015, 1016, 152, 0,
    1017, 1018, 1019, 1020, 1021, 1022, 1023, 728, 425, 1024, 1025, 193, 193, 139, 0, 0,
    0, 0, 0, 0, 0, 0, 31, 0, 0, 0, 0, 0, 0, 101, 101, 139,
    196, 101, 196, 193, 193, 1026, 139, 139, 0, 31, 0, 0, 251, 962, 476, 843,
    843, 837, 193, 195, 1027, 1028, 1029, 1030, 1031, 1032, 1033, 1034, 1035, 1036, 1037, 1038,
    1039, 1040, 1041, 1042, 1043, 1044, 212, 212, 487, 487, 212, 154, 1045, 0, 1046, 1047,
    1048, 1049, 1050, 1051, 1052, 54, 216, 1053, 1054, 1055, 1056, 210, 1057, 44, 1058, 1059,
    1060, 210, 0, 0, 690, 1061, 0, 1062, 1063, 0, 710, 520, 1064, 1065, 1066, 1067,
    1068, 1069, 461, 820, 343, 343, 344, 343, 1070, 666, 1071, 58, 1072, 1073, 81, 93,
    1074, 1075, 1076, 355, 184, 1077, 103, 1078, 664, 664, 37, 717, 257, 257, 1079, 24,
    206, 1080, 31, 284, 1081, 1082, 1083, 1084, 732, 154, 119, 119, 1085, 1085, 1086, 1087,
    1088, 451, 1089, 153, 219, 543, 218, 974, 65, 1090, 1091, 523, 1092, 1093, 408, 1094,
    1095, 1096, 67, 67, 0, 31, 897, 76, 76, 76, 60, 1097, 76, 76, 73, 1097,
    897, 897, 1098, 723, 1025, 1099, 66, 67, 474, 1097, 192, 135, 151, 151, 188, 401,
    401, 107, 151, 151, 131, 1100, 1101, 1102, 1103, 1104, 1105, 1106, 1107, 1108, 1109, 1110,
    1111, 410, 411, 1112, 1113, 1114, 194, 133, 196, 1115, 1116, 1003, 1117, 1118, 1119, 1120,
    1121, 7, 1122, 788, 1123, 1124, 192, 1125, 1126, 1127, 1128, 1129, 1130, 1131, 1132, 1133,
    1134, 443, 100, 100, 100, 99, 99, 125, 158, 0, 38, 38, 76, 0, 16, 16,
    0, 64, 64, 76, 41, 0, 0, 34, 0, 191, 844, 1135, 867, 16, 0, 1136,
    1137, 495, 495, 1138, 82, 1139, 1140, 1141, 94, 1142, 476, 1143, 1144, 1145, 1146, 1147,
    1148, 170, 1149, 1150, 1151, 1152, 1153, 1154, 721, 721, 1155, 250, 1156, 31, 37, 1116,
    1116, 1116, 31, 37, 1157, 1158, 1159, 1160, 14, 132, 132, 794, 644, 899, 14, 58,
    1161, 25, 1162, 1163, 1164, 1165, 1166, 1167, 1168, 1169, 829, 1170, 1171, 1172, 1173, 1174,
    1175, 447, 447, 1176, 1176, 24, 962, 1177, 1177, 139, 0, 0, 31, 0, 0, 0,
    0, 37, 0, 0, 24, 962, 1177, 0, 0, 0, 0, 0, 1178, 612, 24, 68,
    68, 1178, 0, 0, 1179, 1179, 34, 0, 581, 116, 116, 139, 448, 670, 719, 678,
    68, 68, 68, 34, 581, 101, 716, 670, 718, 116, 678, 448, 1180, 718, 719, 124,
    124, 284, 1181, 31, 124, 1181, 31, 124, 273, 31, 284, 1182, 840, 839, 583, 1183,
    727, 83, 141, 909, 909, 37, 401, 401, 909, 140, 933, 933, 37, 401, 401, 933,
    95, 1184, 1185, 21, 1186, 149, 747, 151, 151, 0, 0, 0, 0, 153, 0, 174,
    244, 0, 0, 0, 733, 131, 273, 273, 719, 719, 719, 273, 116, 448, 718, 151,
    719, 1187, 1188, 1188, 822, 642, 24, 929, 151, 151, 14, 173, 0, 151, 151, 1189,
    1190, 1190, 553, 637, 830, 877, 1191, 33, 0, 1169, 1169, 1192, 33, 153, 0, 0,
    0, 131, 16, 16, 0, 969, 131, 24, 24, 642, 642, 0, 0, 0, 0, 550,
    550, 1193, 33, 0, 0, 154, 1194, 149, 1122, 153, 0, 154, 0, 131, 149, 149,
    1195, 234, 234, 0, 0, 131, 237, 237, 33, 1196, 76, 986, 311, 311, 117, 1197,
    0, 1198, 1198, 744, 786, 1150, 1199, 969, 153, 153, 131, 154, 1200, 1200, 915, 1201,
    1199, 969, 153, 153, 131, 154, 943, 943, 33, 1196, 76, 986, 1202, 1202, 33, 0,
    0, 131, 969, 1203, 752, 1204, 1205, 102, 1206, 1207, 0, 0, 0, 290, 1208, 1208,
    1208, 295, 979, 0, 228, 1209, 1210, 1211, 1212, 664, 210, 211, 1213, 1099, 0, 0,
    732, 328, 0, 0, 0, 0, 328, 214, 119, 1214, 1215, 1216, 1217, 222, 222, 104,
    412, 212, 220, 517, 527, 1080, 1218, 1219, 1220, 1221, 434, 1222, 1223, 1224, 1225, 1226,
    1227, 1228, 1229, 1230, 1231, 1232, 1233, 1234, 1235, 1236, 1237, 1238, 1239, 1240, 1241, 1242,
    1243, 1244, 1245, 79, 91, 1191, 1246, 214, 455, 1247, 1248, 1249, 486, 687, 284, 917,
    61, 1250, 129, 129, 39, 129, 72, 98, 39, 39, 129, 128, 39, 151, 129, 129,
    39, 129, 39, 129, 39, 60, 73, 39, 129, 39, 129, 39, 129, 595, 55, 654,
    39, 128, 129, 39, 129, 39, 39, 39, 39, 99, 129, 128, 151, 352, 249, 128,
    395, 128, 583, 439, 1251, 1252, 1253, 419, 1254, 1255, 1256, 1257, 1258, 84, 96, 1259,
    1260, 216, 216, 1261, 31, 0, 0, 0, 0, 0, 1262, 1263, 1264, 1265, 1266, 1267,
    1268, 1269, 1270, 1271, 1272, 1273, 1274, 0, 1275, 1276, 1277, 1278, 1156, 1279, 161, 162,
    162, 0, 1280, 343, 343, 31, 163, 164, 165, 313, 313, 31, 0, 163, 164, 165,
    342, 342, 31, 163, 164, 165, 19, 19, 344, 31, 67, 0, 0, 733, 244, 0,
    0, 31, 284, 31, 917, 77, 1281, 89, 721, 17, 0, 663, 663, 255, 0, 0,
    668, 668, 0, 46, 97, 69, 1282, 0, 442, 69, 70, 70, 1283, 1284, 442, 1201,
    1285, 1286, 409, 1287, 1288, 1289, 918, 684, 665, 550, 514, 514, 550, 551, 31, 1290,
    1290, 124, 1291, 550, 1193, 228, 228, 228, 550, 1193, 0, 1180, 1180, 0, 117, 154,
    0, 1180, 551, 1292, 1293, 1294, 1295, 1296, 1297, 1298, 1299, 528, 1300, 1301, 1302, 33,
    1303, 1304, 406, 1305, 1306, 413, 1307, 1197, 1250, 1250, 455, 249, 364, 1137, 293, 1308,
    1197, 1309, 947, 947, 0, 203, 203, 401, 302, 1310, 249, 249, 1311, 61, 61, 583,
    73, 73, 583, 60, 60, 583, 68, 68, 583, 293, 1312, 1313, 1314, 358, 1315, 1284,
    1316, 1317, 1318, 1319, 71, 1320, 1321, 1322, 403, 725, 272, 860, 413, 270, 270, 72,
    72, 72, 77, 0, 1323, 1324, 1325, 31, 0, 0, 18, 222, 222, 104, 0, 0,
    31, 1326, 157, 1327, 1328, 708, 1329, 1330, 0, 0, 845, 0, 0, 1331, 1332, 1333,
    1334, 1335, 1187, 1187, 1178, 612, 24, 1187, 342, 342, 342, 1336, 0, 1337, 1338, 1339,
    1340, 1341, 68, 68, 34, 581, 101, 716, 670, 718, 719, 124, 124, 116, 448, 1180,
    273, 273, 719, 719, 719, 273, 1176, 1342, 1343, 73, 73, 73, 0, 0, 151, 151,
    34, 581, 1344, 1345, 51, 55, 55, 55, 0, 74, 41, 0, 1346, 1347, 1348, 1349,
    1350, 190, 1351, 1352, 1353, 537, 1354, 1355, 1356, 1357, 1358, 821, 733, 733, 821, 573,
    1359, 1359, 476, 31, 0, 0, 131, 174, 0, 573, 1360, 1361, 542, 1362, 423, 1363,
    1364, 1365, 547, 1366, 1367, 420, 78, 90, 1368, 1369, 1370, 9, 1371, 1372, 1373, 1374,
    1375, 1376, 291, 291, 0, 0, 0, 918, 0, 0, 0, 0, 291, 1377, 581, 581,
    0, 31, 0, 1378, 448, 101, 101, 139, 116, 116, 139, 716, 717, 718, 670, 678,
    581, 1379, 0, 1380, 0, 0, 0, 0, 0, 0, 0, 328, 214, 214, 328, 214,
    718, 0, 0, 1381, 193, 116, 193, 116, 702, 124, 124, 1095, 0, 0, 0, 68,
    0, 966, 124, 124, 1095, 0, 0, 0, 68, 0, 273, 124, 124, 401, 401, 22,
    0, 124, 124, 124, 401, 14, 258, 1061, 1061, 101, 31, 116, 1061, 1291, 0, 1309,
    1202, 23, 75, 1382, 1383, 1384, 1385, 1386, 826, 1387, 1388, 1193, 1389, 1390, 1391, 1392,
];

pub fn get_vertica_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &VERTICA_TABLES }),
        "AbsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &VERTICA_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &VERTICA_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &VERTICA_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(4), tables: &VERTICA_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(65), tables: &VERTICA_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(134), tables: &VERTICA_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(137), tables: &VERTICA_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(160), tables: &VERTICA_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(183), tables: &VERTICA_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(186), tables: &VERTICA_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(208), tables: &VERTICA_TABLES }),
        "AccessrankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(210), tables: &VERTICA_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &VERTICA_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &VERTICA_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &VERTICA_TABLES }),
        "ActivateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(214), tables: &VERTICA_TABLES }),
        "ActivepartitioncountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(215), tables: &VERTICA_TABLES }),
        "AdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(216), tables: &VERTICA_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(217), tables: &VERTICA_TABLES }),
        "AddressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(218), tables: &VERTICA_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(219), tables: &VERTICA_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(220), tables: &VERTICA_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(221), tables: &VERTICA_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(222), tables: &VERTICA_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(223), tables: &VERTICA_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(244), tables: &VERTICA_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(245), tables: &VERTICA_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(248), tables: &VERTICA_TABLES }),
        "AllocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(249), tables: &VERTICA_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(250), tables: &VERTICA_TABLES }),
        "AlterDefaultPrivilegesGrantSegment" => Some(RootGrammar { grammar_id: GrammarId(251), tables: &VERTICA_TABLES }),
        "AlterDefaultPrivilegesObjectPrivilegesSegment" => Some(RootGrammar { grammar_id: GrammarId(276), tables: &VERTICA_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(292), tables: &VERTICA_TABLES }),
        "AlterSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(293), tables: &VERTICA_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(313), tables: &VERTICA_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(329), tables: &VERTICA_TABLES }),
        "AlterSessionStatements" => Some(RootGrammar { grammar_id: GrammarId(335), tables: &VERTICA_TABLES }),
        "AlterTableActionSegment" => Some(RootGrammar { grammar_id: GrammarId(381), tables: &VERTICA_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(418), tables: &VERTICA_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(423), tables: &VERTICA_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(450), tables: &VERTICA_TABLES }),
        "AlterViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(547), tables: &VERTICA_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(567), tables: &VERTICA_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(568), tables: &VERTICA_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(569), tables: &VERTICA_TABLES }),
        "AnalyticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(570), tables: &VERTICA_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(571), tables: &VERTICA_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(572), tables: &VERTICA_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(573), tables: &VERTICA_TABLES }),
        "AnnotatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &VERTICA_TABLES }),
        "AntiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(575), tables: &VERTICA_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(576), tables: &VERTICA_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(577), tables: &VERTICA_TABLES }),
        "ArchiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(578), tables: &VERTICA_TABLES }),
        "AreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(579), tables: &VERTICA_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(580), tables: &VERTICA_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(592), tables: &VERTICA_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(600), tables: &VERTICA_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(601), tables: &VERTICA_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(602), tables: &VERTICA_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(605), tables: &VERTICA_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(609), tables: &VERTICA_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(611), tables: &VERTICA_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(612), tables: &VERTICA_TABLES }),
        "AsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(613), tables: &VERTICA_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(614), tables: &VERTICA_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(615), tables: &VERTICA_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(616), tables: &VERTICA_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(617), tables: &VERTICA_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(618), tables: &VERTICA_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(619), tables: &VERTICA_TABLES }),
        "AttributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(620), tables: &VERTICA_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(621), tables: &VERTICA_TABLES }),
        "AuthenticationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(622), tables: &VERTICA_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(623), tables: &VERTICA_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(624), tables: &VERTICA_TABLES }),
        "AutoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(626), tables: &VERTICA_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(627), tables: &VERTICA_TABLES }),
        "AutocommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(628), tables: &VERTICA_TABLES }),
        "AvailableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(629), tables: &VERTICA_TABLES }),
        "AvgKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(630), tables: &VERTICA_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(631), tables: &VERTICA_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(632), tables: &VERTICA_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(633), tables: &VERTICA_TABLES }),
        "BalanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(634), tables: &VERTICA_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(635), tables: &VERTICA_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(636), tables: &VERTICA_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(648), tables: &VERTICA_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(649), tables: &VERTICA_TABLES }),
        "BasenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(650), tables: &VERTICA_TABLES }),
        "BatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(651), tables: &VERTICA_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(652), tables: &VERTICA_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(653), tables: &VERTICA_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(654), tables: &VERTICA_TABLES }),
        "BestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(655), tables: &VERTICA_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(656), tables: &VERTICA_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(657), tables: &VERTICA_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(658), tables: &VERTICA_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(659), tables: &VERTICA_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(664), tables: &VERTICA_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(665), tables: &VERTICA_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(666), tables: &VERTICA_TABLES }),
        "Bit_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(667), tables: &VERTICA_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(668), tables: &VERTICA_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(669), tables: &VERTICA_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(670), tables: &VERTICA_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(673), tables: &VERTICA_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(674), tables: &VERTICA_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(677), tables: &VERTICA_TABLES }),
        "BlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(678), tables: &VERTICA_TABLES }),
        "BlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(679), tables: &VERTICA_TABLES }),
        "Block_dictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(680), tables: &VERTICA_TABLES }),
        "Blockdict_compKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(681), tables: &VERTICA_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(682), tables: &VERTICA_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(683), tables: &VERTICA_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(686), tables: &VERTICA_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(687), tables: &VERTICA_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(690), tables: &VERTICA_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(691), tables: &VERTICA_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(694), tables: &VERTICA_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(697), tables: &VERTICA_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(698), tables: &VERTICA_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(700), tables: &VERTICA_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(701), tables: &VERTICA_TABLES }),
        "BroadcastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(702), tables: &VERTICA_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(703), tables: &VERTICA_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(704), tables: &VERTICA_TABLES }),
        "BundleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(705), tables: &VERTICA_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(706), tables: &VERTICA_TABLES }),
        "ByteaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(707), tables: &VERTICA_TABLES }),
        "BytesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(708), tables: &VERTICA_TABLES }),
        "BzipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(709), tables: &VERTICA_TABLES }),
        "Bzip_compKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(710), tables: &VERTICA_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(711), tables: &VERTICA_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(713), tables: &VERTICA_TABLES }),
        "CaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(719), tables: &VERTICA_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(720), tables: &VERTICA_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(721), tables: &VERTICA_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(722), tables: &VERTICA_TABLES }),
        "CardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(723), tables: &VERTICA_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(724), tables: &VERTICA_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(725), tables: &VERTICA_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(726), tables: &VERTICA_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(750), tables: &VERTICA_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(751), tables: &VERTICA_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(752), tables: &VERTICA_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(753), tables: &VERTICA_TABLES }),
        "Catalog_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(754), tables: &VERTICA_TABLES }),
        "CatalogpathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(755), tables: &VERTICA_TABLES }),
        "CeilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(756), tables: &VERTICA_TABLES }),
        "CeilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(757), tables: &VERTICA_TABLES }),
        "CertificateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(758), tables: &VERTICA_TABLES }),
        "CertificatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(759), tables: &VERTICA_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(760), tables: &VERTICA_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(761), tables: &VERTICA_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(762), tables: &VERTICA_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(763), tables: &VERTICA_TABLES }),
        "Char_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(764), tables: &VERTICA_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(765), tables: &VERTICA_TABLES }),
        "Character_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(766), tables: &VERTICA_TABLES }),
        "Character_set_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(767), tables: &VERTICA_TABLES }),
        "Character_set_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(768), tables: &VERTICA_TABLES }),
        "Character_set_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(769), tables: &VERTICA_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(770), tables: &VERTICA_TABLES }),
        "CharactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(771), tables: &VERTICA_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(772), tables: &VERTICA_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(773), tables: &VERTICA_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(774), tables: &VERTICA_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(775), tables: &VERTICA_TABLES }),
        "CipherKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(776), tables: &VERTICA_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(777), tables: &VERTICA_TABLES }),
        "Class_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(778), tables: &VERTICA_TABLES }),
        "ClearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(779), tables: &VERTICA_TABLES }),
        "ClobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(780), tables: &VERTICA_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(781), tables: &VERTICA_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(782), tables: &VERTICA_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(783), tables: &VERTICA_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(784), tables: &VERTICA_TABLES }),
        "CobolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(785), tables: &VERTICA_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(786), tables: &VERTICA_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(787), tables: &VERTICA_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(788), tables: &VERTICA_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(789), tables: &VERTICA_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(790), tables: &VERTICA_TABLES }),
        "Collation_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(796), tables: &VERTICA_TABLES }),
        "Collation_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(797), tables: &VERTICA_TABLES }),
        "Collation_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(798), tables: &VERTICA_TABLES }),
        "CollectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &VERTICA_TABLES }),
        "CollectioncloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(800), tables: &VERTICA_TABLES }),
        "CollectiondelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(801), tables: &VERTICA_TABLES }),
        "CollectionencloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(802), tables: &VERTICA_TABLES }),
        "CollectionnullelementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(803), tables: &VERTICA_TABLES }),
        "CollectionopenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(804), tables: &VERTICA_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(805), tables: &VERTICA_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(806), tables: &VERTICA_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(807), tables: &VERTICA_TABLES }),
        "ColsizesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(808), tables: &VERTICA_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(809), tables: &VERTICA_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(814), tables: &VERTICA_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(844), tables: &VERTICA_TABLES }),
        "ColumnEncodingSegment" => Some(RootGrammar { grammar_id: GrammarId(851), tables: &VERTICA_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(854), tables: &VERTICA_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(855), tables: &VERTICA_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &VERTICA_TABLES }),
        "ColumnSetSegment" => Some(RootGrammar { grammar_id: GrammarId(860), tables: &VERTICA_TABLES }),
        "Column_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(877), tables: &VERTICA_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(878), tables: &VERTICA_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(879), tables: &VERTICA_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(880), tables: &VERTICA_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(881), tables: &VERTICA_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(882), tables: &VERTICA_TABLES }),
        "Columns_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(883), tables: &VERTICA_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(884), tables: &VERTICA_TABLES }),
        "Command_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(885), tables: &VERTICA_TABLES }),
        "Command_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(886), tables: &VERTICA_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(887), tables: &VERTICA_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(890), tables: &VERTICA_TABLES }),
        "CommentOnStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(891), tables: &VERTICA_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(936), tables: &VERTICA_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(937), tables: &VERTICA_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(938), tables: &VERTICA_TABLES }),
        "Commondelta_compKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(939), tables: &VERTICA_TABLES }),
        "CommunalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(940), tables: &VERTICA_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(941), tables: &VERTICA_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(951), tables: &VERTICA_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(952), tables: &VERTICA_TABLES }),
        "ComplexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(953), tables: &VERTICA_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(954), tables: &VERTICA_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(955), tables: &VERTICA_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(956), tables: &VERTICA_TABLES }),
        "CompressionType" => Some(RootGrammar { grammar_id: GrammarId(957), tables: &VERTICA_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(959), tables: &VERTICA_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(960), tables: &VERTICA_TABLES }),
        "ConditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(963), tables: &VERTICA_TABLES }),
        "Condition_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(964), tables: &VERTICA_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(965), tables: &VERTICA_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(966), tables: &VERTICA_TABLES }),
        "ConfigurationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(970), tables: &VERTICA_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(971), tables: &VERTICA_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(972), tables: &VERTICA_TABLES }),
        "Connection_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(973), tables: &VERTICA_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(974), tables: &VERTICA_TABLES }),
        "Constraint_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(975), tables: &VERTICA_TABLES }),
        "Constraint_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(976), tables: &VERTICA_TABLES }),
        "Constraint_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(977), tables: &VERTICA_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(978), tables: &VERTICA_TABLES }),
        "ConstructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(979), tables: &VERTICA_TABLES }),
        "ContainsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(980), tables: &VERTICA_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(981), tables: &VERTICA_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(982), tables: &VERTICA_TABLES }),
        "ControlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(983), tables: &VERTICA_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(984), tables: &VERTICA_TABLES }),
        "ConvertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(985), tables: &VERTICA_TABLES }),
        "CopyColumnOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(986), tables: &VERTICA_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(989), tables: &VERTICA_TABLES }),
        "CopyOptionsForColumnsSegment" => Some(RootGrammar { grammar_id: GrammarId(990), tables: &VERTICA_TABLES }),
        "CopyOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(1026), tables: &VERTICA_TABLES }),
        "CopyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1060), tables: &VERTICA_TABLES }),
        "CorrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1103), tables: &VERTICA_TABLES }),
        "CorrelationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1104), tables: &VERTICA_TABLES }),
        "CorrespondingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1105), tables: &VERTICA_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1106), tables: &VERTICA_TABLES }),
        "Covar_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1107), tables: &VERTICA_TABLES }),
        "Covar_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1108), tables: &VERTICA_TABLES }),
        "CpuaffinitymodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1109), tables: &VERTICA_TABLES }),
        "CpuaffinitysetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1110), tables: &VERTICA_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1111), tables: &VERTICA_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1138), tables: &VERTICA_TABLES }),
        "CreateExternalTableSegment" => Some(RootGrammar { grammar_id: GrammarId(1143), tables: &VERTICA_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1190), tables: &VERTICA_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1202), tables: &VERTICA_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1215), tables: &VERTICA_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1216), tables: &VERTICA_TABLES }),
        "CreateProjectionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1236), tables: &VERTICA_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1271), tables: &VERTICA_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1276), tables: &VERTICA_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(1289), tables: &VERTICA_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1309), tables: &VERTICA_TABLES }),
        "CreateTableAsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1315), tables: &VERTICA_TABLES }),
        "CreateTableLikeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1359), tables: &VERTICA_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1370), tables: &VERTICA_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1413), tables: &VERTICA_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1480), tables: &VERTICA_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1484), tables: &VERTICA_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1496), tables: &VERTICA_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1497), tables: &VERTICA_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1498), tables: &VERTICA_TABLES }),
        "CronKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1499), tables: &VERTICA_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1500), tables: &VERTICA_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1501), tables: &VERTICA_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1502), tables: &VERTICA_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1503), tables: &VERTICA_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1504), tables: &VERTICA_TABLES }),
        "Cume_distKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1510), tables: &VERTICA_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1511), tables: &VERTICA_TABLES }),
        "Current_databaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1512), tables: &VERTICA_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1513), tables: &VERTICA_TABLES }),
        "Current_default_transform_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1514), tables: &VERTICA_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1515), tables: &VERTICA_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1516), tables: &VERTICA_TABLES }),
        "Current_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1517), tables: &VERTICA_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1518), tables: &VERTICA_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1519), tables: &VERTICA_TABLES }),
        "Current_transform_group_for_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1520), tables: &VERTICA_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1521), tables: &VERTICA_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1522), tables: &VERTICA_TABLES }),
        "Cursor_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1523), tables: &VERTICA_TABLES }),
        "CustomKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1524), tables: &VERTICA_TABLES }),
        "Custom_partitionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1525), tables: &VERTICA_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1526), tables: &VERTICA_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1527), tables: &VERTICA_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1528), tables: &VERTICA_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &VERTICA_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1529), tables: &VERTICA_TABLES }),
        "DatapathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1530), tables: &VERTICA_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1531), tables: &VERTICA_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(1535), tables: &VERTICA_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1610), tables: &VERTICA_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(1611), tables: &VERTICA_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1612), tables: &VERTICA_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1613), tables: &VERTICA_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1619), tables: &VERTICA_TABLES }),
        "DatediffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1627), tables: &VERTICA_TABLES }),
        "DatestyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1628), tables: &VERTICA_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1629), tables: &VERTICA_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(1630), tables: &VERTICA_TABLES }),
        "Datetime_interval_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1631), tables: &VERTICA_TABLES }),
        "Datetime_interval_precisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1632), tables: &VERTICA_TABLES }),
        "DatetimesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1633), tables: &VERTICA_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1634), tables: &VERTICA_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1635), tables: &VERTICA_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1636), tables: &VERTICA_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1637), tables: &VERTICA_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1638), tables: &VERTICA_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1639), tables: &VERTICA_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1640), tables: &VERTICA_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1641), tables: &VERTICA_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1642), tables: &VERTICA_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1643), tables: &VERTICA_TABLES }),
        "DeactivateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1644), tables: &VERTICA_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1645), tables: &VERTICA_TABLES }),
        "DebugKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1646), tables: &VERTICA_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1647), tables: &VERTICA_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1648), tables: &VERTICA_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1649), tables: &VERTICA_TABLES }),
        "DecodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1650), tables: &VERTICA_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(238), tables: &VERTICA_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1651), tables: &VERTICA_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(1652), tables: &VERTICA_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1655), tables: &VERTICA_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1656), tables: &VERTICA_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1657), tables: &VERTICA_TABLES }),
        "DefineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1658), tables: &VERTICA_TABLES }),
        "DefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1659), tables: &VERTICA_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1660), tables: &VERTICA_TABLES }),
        "DegreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1661), tables: &VERTICA_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1662), tables: &VERTICA_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1663), tables: &VERTICA_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1664), tables: &VERTICA_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1665), tables: &VERTICA_TABLES }),
        "DelimitedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1669), tables: &VERTICA_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1670), tables: &VERTICA_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1671), tables: &VERTICA_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1672), tables: &VERTICA_TABLES }),
        "Deltarange_compKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1673), tables: &VERTICA_TABLES }),
        "Deltarange_comp_spKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1674), tables: &VERTICA_TABLES }),
        "DeltavalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1675), tables: &VERTICA_TABLES }),
        "Dense_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1676), tables: &VERTICA_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1677), tables: &VERTICA_TABLES }),
        "DependsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1678), tables: &VERTICA_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1679), tables: &VERTICA_TABLES }),
        "DerefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1680), tables: &VERTICA_TABLES }),
        "DerivedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1681), tables: &VERTICA_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1682), tables: &VERTICA_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1683), tables: &VERTICA_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1684), tables: &VERTICA_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1688), tables: &VERTICA_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1689), tables: &VERTICA_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1690), tables: &VERTICA_TABLES }),
        "DeterminesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1691), tables: &VERTICA_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1692), tables: &VERTICA_TABLES }),
        "DiagnosticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1693), tables: &VERTICA_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1694), tables: &VERTICA_TABLES }),
        "DirectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1695), tables: &VERTICA_TABLES }),
        "DirectcolsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1696), tables: &VERTICA_TABLES }),
        "DirectedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1697), tables: &VERTICA_TABLES }),
        "DirectgroupedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1698), tables: &VERTICA_TABLES }),
        "DirectprojKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1699), tables: &VERTICA_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1700), tables: &VERTICA_TABLES }),
        "DisabledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1701), tables: &VERTICA_TABLES }),
        "DisconnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1702), tables: &VERTICA_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1703), tables: &VERTICA_TABLES }),
        "DiskQuotaSegment" => Some(RootGrammar { grammar_id: GrammarId(1704), tables: &VERTICA_TABLES }),
        "Disk_quotaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1707), tables: &VERTICA_TABLES }),
        "DispatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1708), tables: &VERTICA_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1709), tables: &VERTICA_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1710), tables: &VERTICA_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1711), tables: &VERTICA_TABLES }),
        "DistvalindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1712), tables: &VERTICA_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1713), tables: &VERTICA_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(1714), tables: &VERTICA_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1715), tables: &VERTICA_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1716), tables: &VERTICA_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(1717), tables: &VERTICA_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1718), tables: &VERTICA_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1719), tables: &VERTICA_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1722), tables: &VERTICA_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1730), tables: &VERTICA_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1736), tables: &VERTICA_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1741), tables: &VERTICA_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1747), tables: &VERTICA_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1748), tables: &VERTICA_TABLES }),
        "DropProjectionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1753), tables: &VERTICA_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1760), tables: &VERTICA_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1765), tables: &VERTICA_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1771), tables: &VERTICA_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1775), tables: &VERTICA_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1783), tables: &VERTICA_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1788), tables: &VERTICA_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1794), tables: &VERTICA_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1799), tables: &VERTICA_TABLES }),
        "DryrunKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1805), tables: &VERTICA_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1806), tables: &VERTICA_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1807), tables: &VERTICA_TABLES }),
        "DurableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1808), tables: &VERTICA_TABLES }),
        "DynamicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1809), tables: &VERTICA_TABLES }),
        "Dynamic_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1810), tables: &VERTICA_TABLES }),
        "Dynamic_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1811), tables: &VERTICA_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1812), tables: &VERTICA_TABLES }),
        "ElementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1813), tables: &VERTICA_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1814), tables: &VERTICA_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1817), tables: &VERTICA_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1818), tables: &VERTICA_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(1819), tables: &VERTICA_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1820), tables: &VERTICA_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1823), tables: &VERTICA_TABLES }),
        "EnabledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1824), tables: &VERTICA_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1825), tables: &VERTICA_TABLES }),
        "EncodedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1826), tables: &VERTICA_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1827), tables: &VERTICA_TABLES }),
        "EncodingType" => Some(RootGrammar { grammar_id: GrammarId(1828), tables: &VERTICA_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1830), tables: &VERTICA_TABLES }),
        "End-execKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1831), tables: &VERTICA_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(233), tables: &VERTICA_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(1832), tables: &VERTICA_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1833), tables: &VERTICA_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &VERTICA_TABLES }),
        "EnforcelengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1834), tables: &VERTICA_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1835), tables: &VERTICA_TABLES }),
        "EphemeralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1836), tables: &VERTICA_TABLES }),
        "EpochKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1837), tables: &VERTICA_TABLES }),
        "EqualsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1838), tables: &VERTICA_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(1839), tables: &VERTICA_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1840), tables: &VERTICA_TABLES }),
        "ErrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1841), tables: &VERTICA_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1842), tables: &VERTICA_TABLES }),
        "Escape_string_warningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1843), tables: &VERTICA_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1844), tables: &VERTICA_TABLES }),
        "EventKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1845), tables: &VERTICA_TABLES }),
        "EventsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1846), tables: &VERTICA_TABLES }),
        "EveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1847), tables: &VERTICA_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1848), tables: &VERTICA_TABLES }),
        "ExceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1849), tables: &VERTICA_TABLES }),
        "ExceptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1850), tables: &VERTICA_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1851), tables: &VERTICA_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1852), tables: &VERTICA_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1853), tables: &VERTICA_TABLES }),
        "ExecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1854), tables: &VERTICA_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1855), tables: &VERTICA_TABLES }),
        "ExecutionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1856), tables: &VERTICA_TABLES }),
        "ExecutionparallelismKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1857), tables: &VERTICA_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1858), tables: &VERTICA_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1859), tables: &VERTICA_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1860), tables: &VERTICA_TABLES }),
        "ExpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1861), tables: &VERTICA_TABLES }),
        "ExpireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1862), tables: &VERTICA_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1863), tables: &VERTICA_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1864), tables: &VERTICA_TABLES }),
        "ExportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1871), tables: &VERTICA_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1872), tables: &VERTICA_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1873), tables: &VERTICA_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1899), tables: &VERTICA_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1906), tables: &VERTICA_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1916), tables: &VERTICA_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1921), tables: &VERTICA_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1934), tables: &VERTICA_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(1971), tables: &VERTICA_TABLES }),
        "ExtendKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1979), tables: &VERTICA_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1980), tables: &VERTICA_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1981), tables: &VERTICA_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &VERTICA_TABLES }),
        "ExtensionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1982), tables: &VERTICA_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1983), tables: &VERTICA_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1984), tables: &VERTICA_TABLES }),
        "Failed_login_attemptsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1985), tables: &VERTICA_TABLES }),
        "FallthroughKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1986), tables: &VERTICA_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1987), tables: &VERTICA_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(1988), tables: &VERTICA_TABLES }),
        "FaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1989), tables: &VERTICA_TABLES }),
        "FencedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1990), tables: &VERTICA_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1991), tables: &VERTICA_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2008), tables: &VERTICA_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2009), tables: &VERTICA_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2010), tables: &VERTICA_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(2011), tables: &VERTICA_TABLES }),
        "FilesystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2020), tables: &VERTICA_TABLES }),
        "FillerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2021), tables: &VERTICA_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2022), tables: &VERTICA_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2023), tables: &VERTICA_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2029), tables: &VERTICA_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2030), tables: &VERTICA_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2031), tables: &VERTICA_TABLES }),
        "FixedwidthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2032), tables: &VERTICA_TABLES }),
        "FlexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2033), tables: &VERTICA_TABLES }),
        "FlexibleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2034), tables: &VERTICA_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2035), tables: &VERTICA_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2036), tables: &VERTICA_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2037), tables: &VERTICA_TABLES }),
        "FloorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2038), tables: &VERTICA_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2039), tables: &VERTICA_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2040), tables: &VERTICA_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2041), tables: &VERTICA_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2042), tables: &VERTICA_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2043), tables: &VERTICA_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2046), tables: &VERTICA_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2047), tables: &VERTICA_TABLES }),
        "FortranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2048), tables: &VERTICA_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2049), tables: &VERTICA_TABLES }),
        "FoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2050), tables: &VERTICA_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2051), tables: &VERTICA_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(2079), tables: &VERTICA_TABLES }),
        "FreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2082), tables: &VERTICA_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2083), tables: &VERTICA_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2084), tables: &VERTICA_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2085), tables: &VERTICA_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2086), tables: &VERTICA_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2090), tables: &VERTICA_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2107), tables: &VERTICA_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2130), tables: &VERTICA_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2157), tables: &VERTICA_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2158), tables: &VERTICA_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2159), tables: &VERTICA_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2160), tables: &VERTICA_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2163), tables: &VERTICA_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2233), tables: &VERTICA_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2236), tables: &VERTICA_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2242), tables: &VERTICA_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2243), tables: &VERTICA_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2244), tables: &VERTICA_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2254), tables: &VERTICA_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(2267), tables: &VERTICA_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(2270), tables: &VERTICA_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2286), tables: &VERTICA_TABLES }),
        "FusionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2287), tables: &VERTICA_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2288), tables: &VERTICA_TABLES }),
        "GKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2289), tables: &VERTICA_TABLES }),
        "GcddeltaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2290), tables: &VERTICA_TABLES }),
        "GeneralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2291), tables: &VERTICA_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2292), tables: &VERTICA_TABLES }),
        "GeographyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2293), tables: &VERTICA_TABLES }),
        "GeometryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2294), tables: &VERTICA_TABLES }),
        "GetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2295), tables: &VERTICA_TABLES }),
        "GlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2296), tables: &VERTICA_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2297), tables: &VERTICA_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2298), tables: &VERTICA_TABLES }),
        "GoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2299), tables: &VERTICA_TABLES }),
        "GotoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2300), tables: &VERTICA_TABLES }),
        "GraceperiodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2301), tables: &VERTICA_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2302), tables: &VERTICA_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2303), tables: &VERTICA_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2341), tables: &VERTICA_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2342), tables: &VERTICA_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2343), tables: &VERTICA_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2346), tables: &VERTICA_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2347), tables: &VERTICA_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2348), tables: &VERTICA_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2358), tables: &VERTICA_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2367), tables: &VERTICA_TABLES }),
        "GroupedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2368), tables: &VERTICA_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(2369), tables: &VERTICA_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2377), tables: &VERTICA_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2378), tables: &VERTICA_TABLES }),
        "GzipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2386), tables: &VERTICA_TABLES }),
        "Gzip_compKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2387), tables: &VERTICA_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2388), tables: &VERTICA_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2389), tables: &VERTICA_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2394), tables: &VERTICA_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2402), tables: &VERTICA_TABLES }),
        "HcatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2403), tables: &VERTICA_TABLES }),
        "Hcatalog_connection_timeoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2404), tables: &VERTICA_TABLES }),
        "Hcatalog_dbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2405), tables: &VERTICA_TABLES }),
        "Hcatalog_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2406), tables: &VERTICA_TABLES }),
        "Hcatalog_slow_transfer_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2407), tables: &VERTICA_TABLES }),
        "Hcatalog_slow_transfer_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2408), tables: &VERTICA_TABLES }),
        "Hcatalog_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2409), tables: &VERTICA_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2410), tables: &VERTICA_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2411), tables: &VERTICA_TABLES }),
        "HierarchyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2412), tables: &VERTICA_TABLES }),
        "HighKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2413), tables: &VERTICA_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2414), tables: &VERTICA_TABLES }),
        "Hiveserver2_hostnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2415), tables: &VERTICA_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2416), tables: &VERTICA_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2417), tables: &VERTICA_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2418), tables: &VERTICA_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2419), tables: &VERTICA_TABLES }),
        "HostnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2420), tables: &VERTICA_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2421), tables: &VERTICA_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2422), tables: &VERTICA_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2423), tables: &VERTICA_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2424), tables: &VERTICA_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2425), tables: &VERTICA_TABLES }),
        "HoursKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2426), tables: &VERTICA_TABLES }),
        "IcebergKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2427), tables: &VERTICA_TABLES }),
        "IdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2428), tables: &VERTICA_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2429), tables: &VERTICA_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2430), tables: &VERTICA_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2431), tables: &VERTICA_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2432), tables: &VERTICA_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2433), tables: &VERTICA_TABLES }),
        "IdlesessiontimeoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2434), tables: &VERTICA_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2435), tables: &VERTICA_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2438), tables: &VERTICA_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2439), tables: &VERTICA_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2443), tables: &VERTICA_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2444), tables: &VERTICA_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2449), tables: &VERTICA_TABLES }),
        "IlikebKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2450), tables: &VERTICA_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2451), tables: &VERTICA_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2452), tables: &VERTICA_TABLES }),
        "ImplementationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2453), tables: &VERTICA_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(729), tables: &VERTICA_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2454), tables: &VERTICA_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2455), tables: &VERTICA_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2456), tables: &VERTICA_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2457), tables: &VERTICA_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2467), tables: &VERTICA_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2468), tables: &VERTICA_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2469), tables: &VERTICA_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(225), tables: &VERTICA_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2470), tables: &VERTICA_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2475), tables: &VERTICA_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &VERTICA_TABLES }),
        "IndicatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2476), tables: &VERTICA_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2477), tables: &VERTICA_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2478), tables: &VERTICA_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2479), tables: &VERTICA_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2480), tables: &VERTICA_TABLES }),
        "InitialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2481), tables: &VERTICA_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2482), tables: &VERTICA_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2483), tables: &VERTICA_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2484), tables: &VERTICA_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2485), tables: &VERTICA_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2486), tables: &VERTICA_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2487), tables: &VERTICA_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2488), tables: &VERTICA_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2489), tables: &VERTICA_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2500), tables: &VERTICA_TABLES }),
        "InstanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2501), tables: &VERTICA_TABLES }),
        "InstantiableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2502), tables: &VERTICA_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2503), tables: &VERTICA_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2504), tables: &VERTICA_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2505), tables: &VERTICA_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2506), tables: &VERTICA_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2507), tables: &VERTICA_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2508), tables: &VERTICA_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2509), tables: &VERTICA_TABLES }),
        "IntegerDivideSegment" => Some(RootGrammar { grammar_id: GrammarId(2510), tables: &VERTICA_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2511), tables: &VERTICA_TABLES }),
        "IntegerSegment" => Some(RootGrammar { grammar_id: GrammarId(2512), tables: &VERTICA_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2513), tables: &VERTICA_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2514), tables: &VERTICA_TABLES }),
        "InterfaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2515), tables: &VERTICA_TABLES }),
        "InterpolateGrammar" => Some(RootGrammar { grammar_id: GrammarId(2516), tables: &VERTICA_TABLES }),
        "InterpolateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2522), tables: &VERTICA_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2523), tables: &VERTICA_TABLES }),
        "IntersectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2524), tables: &VERTICA_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2525), tables: &VERTICA_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2539), tables: &VERTICA_TABLES }),
        "IntervalLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2540), tables: &VERTICA_TABLES }),
        "IntervalUnitsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2547), tables: &VERTICA_TABLES }),
        "IntervalstyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2554), tables: &VERTICA_TABLES }),
        "IntervalymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2555), tables: &VERTICA_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2556), tables: &VERTICA_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2557), tables: &VERTICA_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2558), tables: &VERTICA_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(2564), tables: &VERTICA_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2569), tables: &VERTICA_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(2570), tables: &VERTICA_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2571), tables: &VERTICA_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2572), tables: &VERTICA_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2573), tables: &VERTICA_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2574), tables: &VERTICA_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2575), tables: &VERTICA_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2597), tables: &VERTICA_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2598), tables: &VERTICA_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2600), tables: &VERTICA_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(2601), tables: &VERTICA_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2608), tables: &VERTICA_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2632), tables: &VERTICA_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2637), tables: &VERTICA_TABLES }),
        "KKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2638), tables: &VERTICA_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2639), tables: &VERTICA_TABLES }),
        "Key_memberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2640), tables: &VERTICA_TABLES }),
        "Key_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2641), tables: &VERTICA_TABLES }),
        "KeymanagerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2642), tables: &VERTICA_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2643), tables: &VERTICA_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2644), tables: &VERTICA_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2645), tables: &VERTICA_TABLES }),
        "KsafeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2646), tables: &VERTICA_TABLES }),
        "KsafeSegment" => Some(RootGrammar { grammar_id: GrammarId(2647), tables: &VERTICA_TABLES }),
        "LabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2650), tables: &VERTICA_TABLES }),
        "LambdaArrowSegment" => Some(RootGrammar { grammar_id: GrammarId(2651), tables: &VERTICA_TABLES }),
        "LambdaExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2652), tables: &VERTICA_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2660), tables: &VERTICA_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2661), tables: &VERTICA_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2662), tables: &VERTICA_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2663), tables: &VERTICA_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2664), tables: &VERTICA_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2665), tables: &VERTICA_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2666), tables: &VERTICA_TABLES }),
        "LatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2667), tables: &VERTICA_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2668), tables: &VERTICA_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2669), tables: &VERTICA_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2670), tables: &VERTICA_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2671), tables: &VERTICA_TABLES }),
        "LengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2672), tables: &VERTICA_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2673), tables: &VERTICA_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2674), tables: &VERTICA_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2677), tables: &VERTICA_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2678), tables: &VERTICA_TABLES }),
        "LibraryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2679), tables: &VERTICA_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2680), tables: &VERTICA_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(2688), tables: &VERTICA_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2692), tables: &VERTICA_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2693), tables: &VERTICA_TABLES }),
        "LikeOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(2694), tables: &VERTICA_TABLES }),
        "LikebKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2702), tables: &VERTICA_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2703), tables: &VERTICA_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2721), tables: &VERTICA_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2722), tables: &VERTICA_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2723), tables: &VERTICA_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2724), tables: &VERTICA_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2725), tables: &VERTICA_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2726), tables: &VERTICA_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2736), tables: &VERTICA_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2737), tables: &VERTICA_TABLES }),
        "LnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2738), tables: &VERTICA_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2739), tables: &VERTICA_TABLES }),
        "LoaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2740), tables: &VERTICA_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(2741), tables: &VERTICA_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2742), tables: &VERTICA_TABLES }),
        "LocaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2743), tables: &VERTICA_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2744), tables: &VERTICA_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2745), tables: &VERTICA_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2746), tables: &VERTICA_TABLES }),
        "LocatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2747), tables: &VERTICA_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2748), tables: &VERTICA_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2749), tables: &VERTICA_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2750), tables: &VERTICA_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2751), tables: &VERTICA_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2752), tables: &VERTICA_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2753), tables: &VERTICA_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2754), tables: &VERTICA_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2755), tables: &VERTICA_TABLES }),
        "LowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2756), tables: &VERTICA_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2757), tables: &VERTICA_TABLES }),
        "LowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2758), tables: &VERTICA_TABLES }),
        "LzoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2759), tables: &VERTICA_TABLES }),
        "MKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2760), tables: &VERTICA_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2761), tables: &VERTICA_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2773), tables: &VERTICA_TABLES }),
        "ManagedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2774), tables: &VERTICA_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2775), tables: &VERTICA_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2776), tables: &VERTICA_TABLES }),
        "MaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2777), tables: &VERTICA_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2778), tables: &VERTICA_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(2779), tables: &VERTICA_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2780), tables: &VERTICA_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2781), tables: &VERTICA_TABLES }),
        "MaterializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2782), tables: &VERTICA_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2783), tables: &VERTICA_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2784), tables: &VERTICA_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2785), tables: &VERTICA_TABLES }),
        "MaxconcurrencyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2786), tables: &VERTICA_TABLES }),
        "MaxconcurrencygraceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2787), tables: &VERTICA_TABLES }),
        "MaxconnectionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2788), tables: &VERTICA_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2789), tables: &VERTICA_TABLES }),
        "MaxmemorysizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2790), tables: &VERTICA_TABLES }),
        "MaxpayloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2791), tables: &VERTICA_TABLES }),
        "MaxquerymemorysizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2792), tables: &VERTICA_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2793), tables: &VERTICA_TABLES }),
        "MediumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2794), tables: &VERTICA_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2795), tables: &VERTICA_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2796), tables: &VERTICA_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2797), tables: &VERTICA_TABLES }),
        "MemberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2798), tables: &VERTICA_TABLES }),
        "MemorycapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2799), tables: &VERTICA_TABLES }),
        "MemorysizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2800), tables: &VERTICA_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2801), tables: &VERTICA_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2802), tables: &VERTICA_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2806), tables: &VERTICA_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2809), tables: &VERTICA_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(2810), tables: &VERTICA_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2813), tables: &VERTICA_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2823), tables: &VERTICA_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2832), tables: &VERTICA_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2852), tables: &VERTICA_TABLES }),
        "MergeoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2855), tables: &VERTICA_TABLES }),
        "Message_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2856), tables: &VERTICA_TABLES }),
        "Message_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2857), tables: &VERTICA_TABLES }),
        "Message_textKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2858), tables: &VERTICA_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2859), tables: &VERTICA_TABLES }),
        "MicrosecondsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2860), tables: &VERTICA_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2861), tables: &VERTICA_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2862), tables: &VERTICA_TABLES }),
        "MillisecondsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2863), tables: &VERTICA_TABLES }),
        "MinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2864), tables: &VERTICA_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2865), tables: &VERTICA_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2866), tables: &VERTICA_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(2867), tables: &VERTICA_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2868), tables: &VERTICA_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2869), tables: &VERTICA_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2870), tables: &VERTICA_TABLES }),
        "MinutesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2871), tables: &VERTICA_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2872), tables: &VERTICA_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2873), tables: &VERTICA_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2874), tables: &VERTICA_TABLES }),
        "ModKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2875), tables: &VERTICA_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2876), tables: &VERTICA_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2877), tables: &VERTICA_TABLES }),
        "ModifiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2878), tables: &VERTICA_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2879), tables: &VERTICA_TABLES }),
        "ModuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2880), tables: &VERTICA_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(2881), tables: &VERTICA_TABLES }),
        "MoneyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2882), tables: &VERTICA_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2883), tables: &VERTICA_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2884), tables: &VERTICA_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2885), tables: &VERTICA_TABLES }),
        "MoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2886), tables: &VERTICA_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2887), tables: &VERTICA_TABLES }),
        "MoveoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2888), tables: &VERTICA_TABLES }),
        "MultipleactiveresultsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2889), tables: &VERTICA_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(2890), tables: &VERTICA_TABLES }),
        "MultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2891), tables: &VERTICA_TABLES }),
        "MumpsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2892), tables: &VERTICA_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2893), tables: &VERTICA_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2894), tables: &VERTICA_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2895), tables: &VERTICA_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2896), tables: &VERTICA_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(2903), tables: &VERTICA_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2907), tables: &VERTICA_TABLES }),
        "NamespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2908), tables: &VERTICA_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2909), tables: &VERTICA_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2910), tables: &VERTICA_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2911), tables: &VERTICA_TABLES }),
        "NativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2912), tables: &VERTICA_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2913), tables: &VERTICA_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2916), tables: &VERTICA_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2917), tables: &VERTICA_TABLES }),
        "NclobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2918), tables: &VERTICA_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(2919), tables: &VERTICA_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(2920), tables: &VERTICA_TABLES }),
        "NestingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2921), tables: &VERTICA_TABLES }),
        "NetworkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2922), tables: &VERTICA_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2923), tables: &VERTICA_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(2924), tables: &VERTICA_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2925), tables: &VERTICA_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2926), tables: &VERTICA_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2927), tables: &VERTICA_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2928), tables: &VERTICA_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2929), tables: &VERTICA_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2930), tables: &VERTICA_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2931), tables: &VERTICA_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2932), tables: &VERTICA_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2933), tables: &VERTICA_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2934), tables: &VERTICA_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2935), tables: &VERTICA_TABLES }),
        "NodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2936), tables: &VERTICA_TABLES }),
        "NodesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2937), tables: &VERTICA_TABLES }),
        "NoglobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2938), tables: &VERTICA_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2939), tables: &VERTICA_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2940), tables: &VERTICA_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2941), tables: &VERTICA_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2951), tables: &VERTICA_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2952), tables: &VERTICA_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2957), tables: &VERTICA_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2963), tables: &VERTICA_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2964), tables: &VERTICA_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2965), tables: &VERTICA_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2966), tables: &VERTICA_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2967), tables: &VERTICA_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2968), tables: &VERTICA_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2969), tables: &VERTICA_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2970), tables: &VERTICA_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2971), tables: &VERTICA_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2978), tables: &VERTICA_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(2979), tables: &VERTICA_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2980), tables: &VERTICA_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2981), tables: &VERTICA_TABLES }),
        "NotifierKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2982), tables: &VERTICA_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2983), tables: &VERTICA_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2984), tables: &VERTICA_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2985), tables: &VERTICA_TABLES }),
        "NullCastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2986), tables: &VERTICA_TABLES }),
        "NullEqualsOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2987), tables: &VERTICA_TABLES }),
        "NullEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(2988), tables: &VERTICA_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2989), tables: &VERTICA_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2990), tables: &VERTICA_TABLES }),
        "NullableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2991), tables: &VERTICA_TABLES }),
        "NullawareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2992), tables: &VERTICA_TABLES }),
        "NullcolsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2993), tables: &VERTICA_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2994), tables: &VERTICA_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2995), tables: &VERTICA_TABLES }),
        "NullsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2996), tables: &VERTICA_TABLES }),
        "NumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2997), tables: &VERTICA_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2998), tables: &VERTICA_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2999), tables: &VERTICA_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3000), tables: &VERTICA_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(3001), tables: &VERTICA_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3005), tables: &VERTICA_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(3009), tables: &VERTICA_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &VERTICA_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3014), tables: &VERTICA_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3027), tables: &VERTICA_TABLES }),
        "Octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3028), tables: &VERTICA_TABLES }),
        "OctetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3029), tables: &VERTICA_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3030), tables: &VERTICA_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3031), tables: &VERTICA_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3032), tables: &VERTICA_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3033), tables: &VERTICA_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3042), tables: &VERTICA_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3043), tables: &VERTICA_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3044), tables: &VERTICA_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3045), tables: &VERTICA_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3046), tables: &VERTICA_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3047), tables: &VERTICA_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3048), tables: &VERTICA_TABLES }),
        "OpenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3049), tables: &VERTICA_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3050), tables: &VERTICA_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3051), tables: &VERTICA_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3052), tables: &VERTICA_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3053), tables: &VERTICA_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3054), tables: &VERTICA_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3055), tables: &VERTICA_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3056), tables: &VERTICA_TABLES }),
        "OptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3057), tables: &VERTICA_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3058), tables: &VERTICA_TABLES }),
        "OptimizerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3059), tables: &VERTICA_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3060), tables: &VERTICA_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3061), tables: &VERTICA_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3062), tables: &VERTICA_TABLES }),
        "OptverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3063), tables: &VERTICA_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3064), tables: &VERTICA_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3065), tables: &VERTICA_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(3066), tables: &VERTICA_TABLES }),
        "OrcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3069), tables: &VERTICA_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3070), tables: &VERTICA_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(3090), tables: &VERTICA_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3098), tables: &VERTICA_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(3099), tables: &VERTICA_TABLES }),
        "OrderingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3102), tables: &VERTICA_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3103), tables: &VERTICA_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3104), tables: &VERTICA_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3105), tables: &VERTICA_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3106), tables: &VERTICA_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3107), tables: &VERTICA_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3108), tables: &VERTICA_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3109), tables: &VERTICA_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3116), tables: &VERTICA_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3117), tables: &VERTICA_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3126), tables: &VERTICA_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3127), tables: &VERTICA_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3128), tables: &VERTICA_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3129), tables: &VERTICA_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3130), tables: &VERTICA_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3131), tables: &VERTICA_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3132), tables: &VERTICA_TABLES }),
        "PadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3133), tables: &VERTICA_TABLES }),
        "ParameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3134), tables: &VERTICA_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3135), tables: &VERTICA_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3136), tables: &VERTICA_TABLES }),
        "Parameter_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3137), tables: &VERTICA_TABLES }),
        "Parameter_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3138), tables: &VERTICA_TABLES }),
        "Parameter_ordinal_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3139), tables: &VERTICA_TABLES }),
        "Parameter_specific_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3140), tables: &VERTICA_TABLES }),
        "Parameter_specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3141), tables: &VERTICA_TABLES }),
        "Parameter_specific_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3142), tables: &VERTICA_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3143), tables: &VERTICA_TABLES }),
        "ParquetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3144), tables: &VERTICA_TABLES }),
        "ParserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3145), tables: &VERTICA_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3146), tables: &VERTICA_TABLES }),
        "PartitionByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3147), tables: &VERTICA_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3171), tables: &VERTICA_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3186), tables: &VERTICA_TABLES }),
        "PartitioningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3187), tables: &VERTICA_TABLES }),
        "PascalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3188), tables: &VERTICA_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3189), tables: &VERTICA_TABLES }),
        "Password_grace_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3190), tables: &VERTICA_TABLES }),
        "Password_life_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3191), tables: &VERTICA_TABLES }),
        "Password_lock_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3192), tables: &VERTICA_TABLES }),
        "Password_max_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3193), tables: &VERTICA_TABLES }),
        "Password_min_char_changeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3194), tables: &VERTICA_TABLES }),
        "Password_min_digitsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3195), tables: &VERTICA_TABLES }),
        "Password_min_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3196), tables: &VERTICA_TABLES }),
        "Password_min_lettersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3197), tables: &VERTICA_TABLES }),
        "Password_min_life_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3198), tables: &VERTICA_TABLES }),
        "Password_min_lowercase_lettersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3199), tables: &VERTICA_TABLES }),
        "Password_min_symbolsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3200), tables: &VERTICA_TABLES }),
        "Password_min_uppercase_lettersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3201), tables: &VERTICA_TABLES }),
        "Password_reuse_maxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3202), tables: &VERTICA_TABLES }),
        "Password_reuse_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3203), tables: &VERTICA_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3204), tables: &VERTICA_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(3205), tables: &VERTICA_TABLES }),
        "PatternKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3212), tables: &VERTICA_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(3213), tables: &VERTICA_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3214), tables: &VERTICA_TABLES }),
        "PercentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3215), tables: &VERTICA_TABLES }),
        "Percent_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3216), tables: &VERTICA_TABLES }),
        "Percentile_contKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3217), tables: &VERTICA_TABLES }),
        "Percentile_discKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3218), tables: &VERTICA_TABLES }),
        "PermanentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3219), tables: &VERTICA_TABLES }),
        "PinnedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3220), tables: &VERTICA_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3221), tables: &VERTICA_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(3222), tables: &VERTICA_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3223), tables: &VERTICA_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3224), tables: &VERTICA_TABLES }),
        "PlannedconcurrencyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3225), tables: &VERTICA_TABLES }),
        "PliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3226), tables: &VERTICA_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(3227), tables: &VERTICA_TABLES }),
        "PointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3228), tables: &VERTICA_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3229), tables: &VERTICA_TABLES }),
        "PoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3230), tables: &VERTICA_TABLES }),
        "PortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3231), tables: &VERTICA_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3232), tables: &VERTICA_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(3233), tables: &VERTICA_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3234), tables: &VERTICA_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3238), tables: &VERTICA_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3239), tables: &VERTICA_TABLES }),
        "PowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3240), tables: &VERTICA_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3241), tables: &VERTICA_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3242), tables: &VERTICA_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3243), tables: &VERTICA_TABLES }),
        "PreferKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3244), tables: &VERTICA_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3245), tables: &VERTICA_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3246), tables: &VERTICA_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3247), tables: &VERTICA_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3248), tables: &VERTICA_TABLES }),
        "PrepassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3249), tables: &VERTICA_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3250), tables: &VERTICA_TABLES }),
        "PreviousKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3251), tables: &VERTICA_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(3252), tables: &VERTICA_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3255), tables: &VERTICA_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3256), tables: &VERTICA_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3257), tables: &VERTICA_TABLES }),
        "PriorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3258), tables: &VERTICA_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3259), tables: &VERTICA_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3260), tables: &VERTICA_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3261), tables: &VERTICA_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3262), tables: &VERTICA_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3263), tables: &VERTICA_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3264), tables: &VERTICA_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3265), tables: &VERTICA_TABLES }),
        "ProfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3266), tables: &VERTICA_TABLES }),
        "ProjectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3267), tables: &VERTICA_TABLES }),
        "ProjectionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3268), tables: &VERTICA_TABLES }),
        "PsdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3269), tables: &VERTICA_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3270), tables: &VERTICA_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3271), tables: &VERTICA_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3272), tables: &VERTICA_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3275), tables: &VERTICA_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3276), tables: &VERTICA_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3284), tables: &VERTICA_TABLES }),
        "QueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3285), tables: &VERTICA_TABLES }),
        "QueuetimeoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3286), tables: &VERTICA_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3287), tables: &VERTICA_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3288), tables: &VERTICA_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3289), tables: &VERTICA_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3294), tables: &VERTICA_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3295), tables: &VERTICA_TABLES }),
        "RandomKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3296), tables: &VERTICA_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3297), tables: &VERTICA_TABLES }),
        "RankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3298), tables: &VERTICA_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(3299), tables: &VERTICA_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(3300), tables: &VERTICA_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3301), tables: &VERTICA_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(3302), tables: &VERTICA_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(3303), tables: &VERTICA_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3304), tables: &VERTICA_TABLES }),
        "ReadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3305), tables: &VERTICA_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3306), tables: &VERTICA_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3307), tables: &VERTICA_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3308), tables: &VERTICA_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3309), tables: &VERTICA_TABLES }),
        "RecordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3310), tables: &VERTICA_TABLES }),
        "RecoverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3311), tables: &VERTICA_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3312), tables: &VERTICA_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3313), tables: &VERTICA_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3314), tables: &VERTICA_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(3328), tables: &VERTICA_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3334), tables: &VERTICA_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3335), tables: &VERTICA_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3336), tables: &VERTICA_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3337), tables: &VERTICA_TABLES }),
        "RefreshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3349), tables: &VERTICA_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3350), tables: &VERTICA_TABLES }),
        "Regr_avgxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3351), tables: &VERTICA_TABLES }),
        "Regr_avgyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3352), tables: &VERTICA_TABLES }),
        "Regr_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3353), tables: &VERTICA_TABLES }),
        "Regr_interceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3354), tables: &VERTICA_TABLES }),
        "Regr_r2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3355), tables: &VERTICA_TABLES }),
        "Regr_slopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3356), tables: &VERTICA_TABLES }),
        "Regr_sxxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3357), tables: &VERTICA_TABLES }),
        "Regr_sxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3358), tables: &VERTICA_TABLES }),
        "Regr_syyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3359), tables: &VERTICA_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3360), tables: &VERTICA_TABLES }),
        "RejectedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3361), tables: &VERTICA_TABLES }),
        "RejectmaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3362), tables: &VERTICA_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3363), tables: &VERTICA_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3364), tables: &VERTICA_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3365), tables: &VERTICA_TABLES }),
        "RemoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3366), tables: &VERTICA_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3367), tables: &VERTICA_TABLES }),
        "ReorganizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3368), tables: &VERTICA_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3369), tables: &VERTICA_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3370), tables: &VERTICA_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3371), tables: &VERTICA_TABLES }),
        "ReplicateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3372), tables: &VERTICA_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3373), tables: &VERTICA_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3374), tables: &VERTICA_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3375), tables: &VERTICA_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3376), tables: &VERTICA_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3377), tables: &VERTICA_TABLES }),
        "Resource_poolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3378), tables: &VERTICA_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3379), tables: &VERTICA_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3380), tables: &VERTICA_TABLES }),
        "RestoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3381), tables: &VERTICA_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3382), tables: &VERTICA_TABLES }),
        "ResultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3383), tables: &VERTICA_TABLES }),
        "ResultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3384), tables: &VERTICA_TABLES }),
        "RetentionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3385), tables: &VERTICA_TABLES }),
        "RetryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3386), tables: &VERTICA_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3387), tables: &VERTICA_TABLES }),
        "Returned_cardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3388), tables: &VERTICA_TABLES }),
        "Returned_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3389), tables: &VERTICA_TABLES }),
        "Returned_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3390), tables: &VERTICA_TABLES }),
        "Returned_sqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3391), tables: &VERTICA_TABLES }),
        "ReturnrejectedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3392), tables: &VERTICA_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3393), tables: &VERTICA_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3394), tables: &VERTICA_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3395), tables: &VERTICA_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3418), tables: &VERTICA_TABLES }),
        "RleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3419), tables: &VERTICA_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3420), tables: &VERTICA_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3421), tables: &VERTICA_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3422), tables: &VERTICA_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3423), tables: &VERTICA_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3424), tables: &VERTICA_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3425), tables: &VERTICA_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3426), tables: &VERTICA_TABLES }),
        "RouteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3427), tables: &VERTICA_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3428), tables: &VERTICA_TABLES }),
        "Routine_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3429), tables: &VERTICA_TABLES }),
        "Routine_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3430), tables: &VERTICA_TABLES }),
        "Routine_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3431), tables: &VERTICA_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3432), tables: &VERTICA_TABLES }),
        "RoutingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3433), tables: &VERTICA_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3434), tables: &VERTICA_TABLES }),
        "Row_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3435), tables: &VERTICA_TABLES }),
        "Row_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3436), tables: &VERTICA_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3437), tables: &VERTICA_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3438), tables: &VERTICA_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3439), tables: &VERTICA_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3440), tables: &VERTICA_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3441), tables: &VERTICA_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3442), tables: &VERTICA_TABLES }),
        "RuntimecapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3443), tables: &VERTICA_TABLES }),
        "RuntimepriorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3444), tables: &VERTICA_TABLES }),
        "RuntimeprioritythresholdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3445), tables: &VERTICA_TABLES }),
        "RwithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3446), tables: &VERTICA_TABLES }),
        "SaltKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3447), tables: &VERTICA_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3448), tables: &VERTICA_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3460), tables: &VERTICA_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3461), tables: &VERTICA_TABLES }),
        "ScaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3462), tables: &VERTICA_TABLES }),
        "ScheduleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3463), tables: &VERTICA_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3464), tables: &VERTICA_TABLES }),
        "SchemaPrivilegesSegment" => Some(RootGrammar { grammar_id: GrammarId(3465), tables: &VERTICA_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &VERTICA_TABLES }),
        "Schema_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3472), tables: &VERTICA_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3473), tables: &VERTICA_TABLES }),
        "SchemataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3474), tables: &VERTICA_TABLES }),
        "ScopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3475), tables: &VERTICA_TABLES }),
        "Scope_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3476), tables: &VERTICA_TABLES }),
        "Scope_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3477), tables: &VERTICA_TABLES }),
        "Scope_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3478), tables: &VERTICA_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3479), tables: &VERTICA_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3480), tables: &VERTICA_TABLES }),
        "Search_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3481), tables: &VERTICA_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3482), tables: &VERTICA_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3483), tables: &VERTICA_TABLES }),
        "SecondaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3484), tables: &VERTICA_TABLES }),
        "SecondsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3485), tables: &VERTICA_TABLES }),
        "SectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3486), tables: &VERTICA_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3487), tables: &VERTICA_TABLES }),
        "Security_algorithmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3488), tables: &VERTICA_TABLES }),
        "SegmentedByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3489), tables: &VERTICA_TABLES }),
        "SegmentedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3510), tables: &VERTICA_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(3511), tables: &VERTICA_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3516), tables: &VERTICA_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3519), tables: &VERTICA_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3525), tables: &VERTICA_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3536), tables: &VERTICA_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3537), tables: &VERTICA_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3553), tables: &VERTICA_TABLES }),
        "SelfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3563), tables: &VERTICA_TABLES }),
        "SemiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3564), tables: &VERTICA_TABLES }),
        "SemiallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3565), tables: &VERTICA_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(3566), tables: &VERTICA_TABLES }),
        "SensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3567), tables: &VERTICA_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3568), tables: &VERTICA_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3569), tables: &VERTICA_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(3570), tables: &VERTICA_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(3577), tables: &VERTICA_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &VERTICA_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3584), tables: &VERTICA_TABLES }),
        "SerialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3585), tables: &VERTICA_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3586), tables: &VERTICA_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3587), tables: &VERTICA_TABLES }),
        "Server_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3588), tables: &VERTICA_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3589), tables: &VERTICA_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3590), tables: &VERTICA_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(3591), tables: &VERTICA_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3595), tables: &VERTICA_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3606), tables: &VERTICA_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3615), tables: &VERTICA_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(3616), tables: &VERTICA_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3624), tables: &VERTICA_TABLES }),
        "SetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3629), tables: &VERTICA_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3731), tables: &VERTICA_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3732), tables: &VERTICA_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3733), tables: &VERTICA_TABLES }),
        "ShardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3734), tables: &VERTICA_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3735), tables: &VERTICA_TABLES }),
        "SharedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3736), tables: &VERTICA_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3737), tables: &VERTICA_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(3738), tables: &VERTICA_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3749), tables: &VERTICA_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3750), tables: &VERTICA_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3751), tables: &VERTICA_TABLES }),
        "SignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3752), tables: &VERTICA_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(3753), tables: &VERTICA_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3756), tables: &VERTICA_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3757), tables: &VERTICA_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(3758), tables: &VERTICA_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(3762), tables: &VERTICA_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(3764), tables: &VERTICA_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3768), tables: &VERTICA_TABLES }),
        "SingleinitiatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3769), tables: &VERTICA_TABLES }),
        "SiteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3770), tables: &VERTICA_TABLES }),
        "SitesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3771), tables: &VERTICA_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3772), tables: &VERTICA_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3773), tables: &VERTICA_TABLES }),
        "SkipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3776), tables: &VERTICA_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(3777), tables: &VERTICA_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(3778), tables: &VERTICA_TABLES }),
        "SmalldatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3779), tables: &VERTICA_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3780), tables: &VERTICA_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3781), tables: &VERTICA_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3782), tables: &VERTICA_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3783), tables: &VERTICA_TABLES }),
        "SpaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3784), tables: &VERTICA_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3785), tables: &VERTICA_TABLES }),
        "SpecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3786), tables: &VERTICA_TABLES }),
        "Specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3787), tables: &VERTICA_TABLES }),
        "SpecifictypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3788), tables: &VERTICA_TABLES }),
        "SplitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3789), tables: &VERTICA_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3790), tables: &VERTICA_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3791), tables: &VERTICA_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3792), tables: &VERTICA_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3793), tables: &VERTICA_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3794), tables: &VERTICA_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3795), tables: &VERTICA_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3796), tables: &VERTICA_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3797), tables: &VERTICA_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3798), tables: &VERTICA_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3799), tables: &VERTICA_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3800), tables: &VERTICA_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3801), tables: &VERTICA_TABLES }),
        "SqlcodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3802), tables: &VERTICA_TABLES }),
        "SqlerrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3803), tables: &VERTICA_TABLES }),
        "SqlexceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3804), tables: &VERTICA_TABLES }),
        "SqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3805), tables: &VERTICA_TABLES }),
        "SqlwarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3806), tables: &VERTICA_TABLES }),
        "SqrtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3807), tables: &VERTICA_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3808), tables: &VERTICA_TABLES }),
        "Ssl_configKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3809), tables: &VERTICA_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3810), tables: &VERTICA_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3811), tables: &VERTICA_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3812), tables: &VERTICA_TABLES }),
        "Standard_conforming_stringsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3813), tables: &VERTICA_TABLES }),
        "StandbyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3814), tables: &VERTICA_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(3815), tables: &VERTICA_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(232), tables: &VERTICA_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(3008), tables: &VERTICA_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3816), tables: &VERTICA_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(598), tables: &VERTICA_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3817), tables: &VERTICA_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3818), tables: &VERTICA_TABLES }),
        "StateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3819), tables: &VERTICA_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3820), tables: &VERTICA_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3821), tables: &VERTICA_TABLES }),
        "StaticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3875), tables: &VERTICA_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3876), tables: &VERTICA_TABLES }),
        "Stddev_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3877), tables: &VERTICA_TABLES }),
        "Stddev_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3878), tables: &VERTICA_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3879), tables: &VERTICA_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3880), tables: &VERTICA_TABLES }),
        "StemmerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3881), tables: &VERTICA_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3882), tables: &VERTICA_TABLES }),
        "StoredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3883), tables: &VERTICA_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3884), tables: &VERTICA_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3885), tables: &VERTICA_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3886), tables: &VERTICA_TABLES }),
        "StrengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3887), tables: &VERTICA_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3888), tables: &VERTICA_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3889), tables: &VERTICA_TABLES }),
        "StringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3891), tables: &VERTICA_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3892), tables: &VERTICA_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3897), tables: &VERTICA_TABLES }),
        "StructureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3898), tables: &VERTICA_TABLES }),
        "StyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3899), tables: &VERTICA_TABLES }),
        "Subclass_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3900), tables: &VERTICA_TABLES }),
        "SubclusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3901), tables: &VERTICA_TABLES }),
        "SubjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3902), tables: &VERTICA_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3903), tables: &VERTICA_TABLES }),
        "SubmultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3904), tables: &VERTICA_TABLES }),
        "SubnetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3905), tables: &VERTICA_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3906), tables: &VERTICA_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3907), tables: &VERTICA_TABLES }),
        "SuitesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3908), tables: &VERTICA_TABLES }),
        "SumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3909), tables: &VERTICA_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3910), tables: &VERTICA_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(3911), tables: &VERTICA_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3912), tables: &VERTICA_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3913), tables: &VERTICA_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3914), tables: &VERTICA_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3915), tables: &VERTICA_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3916), tables: &VERTICA_TABLES }),
        "System_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3917), tables: &VERTICA_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(3918), tables: &VERTICA_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3947), tables: &VERTICA_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3948), tables: &VERTICA_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3957), tables: &VERTICA_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &VERTICA_TABLES }),
        "Table_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3958), tables: &VERTICA_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3959), tables: &VERTICA_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3960), tables: &VERTICA_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3961), tables: &VERTICA_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &VERTICA_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &VERTICA_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(3962), tables: &VERTICA_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(3967), tables: &VERTICA_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3971), tables: &VERTICA_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3972), tables: &VERTICA_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3973), tables: &VERTICA_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3974), tables: &VERTICA_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(3975), tables: &VERTICA_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(3976), tables: &VERTICA_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3979), tables: &VERTICA_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(3980), tables: &VERTICA_TABLES }),
        "TempspacecapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3983), tables: &VERTICA_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3984), tables: &VERTICA_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3985), tables: &VERTICA_TABLES }),
        "TerminatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3986), tables: &VERTICA_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3987), tables: &VERTICA_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3988), tables: &VERTICA_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3989), tables: &VERTICA_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3990), tables: &VERTICA_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3991), tables: &VERTICA_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(3992), tables: &VERTICA_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3993), tables: &VERTICA_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(3994), tables: &VERTICA_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(4005), tables: &VERTICA_TABLES }),
        "TimeseriesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4011), tables: &VERTICA_TABLES }),
        "TimeseriesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4017), tables: &VERTICA_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4018), tables: &VERTICA_TABLES }),
        "TimestampaddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4019), tables: &VERTICA_TABLES }),
        "TimestampdiffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4020), tables: &VERTICA_TABLES }),
        "TimestamptzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4021), tables: &VERTICA_TABLES }),
        "TimetzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4022), tables: &VERTICA_TABLES }),
        "TimezoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4023), tables: &VERTICA_TABLES }),
        "Timezone_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4024), tables: &VERTICA_TABLES }),
        "Timezone_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4025), tables: &VERTICA_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4026), tables: &VERTICA_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4027), tables: &VERTICA_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4028), tables: &VERTICA_TABLES }),
        "TlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4029), tables: &VERTICA_TABLES }),
        "TlsmodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4030), tables: &VERTICA_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4031), tables: &VERTICA_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4032), tables: &VERTICA_TABLES }),
        "TokenizerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4033), tables: &VERTICA_TABLES }),
        "ToleranceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4034), tables: &VERTICA_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4035), tables: &VERTICA_TABLES }),
        "Top_level_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4036), tables: &VERTICA_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4037), tables: &VERTICA_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4038), tables: &VERTICA_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4039), tables: &VERTICA_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &VERTICA_TABLES }),
        "Transaction_activeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4057), tables: &VERTICA_TABLES }),
        "TransactionalStatements" => Some(RootGrammar { grammar_id: GrammarId(4058), tables: &VERTICA_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4074), tables: &VERTICA_TABLES }),
        "Transactions_committedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4075), tables: &VERTICA_TABLES }),
        "Transactions_rolled_backKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4076), tables: &VERTICA_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4077), tables: &VERTICA_TABLES }),
        "TransformsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4078), tables: &VERTICA_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4079), tables: &VERTICA_TABLES }),
        "TranslateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4080), tables: &VERTICA_TABLES }),
        "TranslationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4081), tables: &VERTICA_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4082), tables: &VERTICA_TABLES }),
        "TrickleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4083), tables: &VERTICA_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4084), tables: &VERTICA_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &VERTICA_TABLES }),
        "Trigger_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4085), tables: &VERTICA_TABLES }),
        "Trigger_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4086), tables: &VERTICA_TABLES }),
        "Trigger_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4087), tables: &VERTICA_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4088), tables: &VERTICA_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(4089), tables: &VERTICA_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4093), tables: &VERTICA_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(4094), tables: &VERTICA_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4095), tables: &VERTICA_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4096), tables: &VERTICA_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4100), tables: &VERTICA_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4101), tables: &VERTICA_TABLES }),
        "TuningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4102), tables: &VERTICA_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(4103), tables: &VERTICA_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4106), tables: &VERTICA_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4107), tables: &VERTICA_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4110), tables: &VERTICA_TABLES }),
        "TypesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4113), tables: &VERTICA_TABLES }),
        "UdparameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4114), tables: &VERTICA_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4115), tables: &VERTICA_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4116), tables: &VERTICA_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4117), tables: &VERTICA_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4118), tables: &VERTICA_TABLES }),
        "UncompressedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4119), tables: &VERTICA_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4120), tables: &VERTICA_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4121), tables: &VERTICA_TABLES }),
        "UnderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4125), tables: &VERTICA_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4126), tables: &VERTICA_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4127), tables: &VERTICA_TABLES }),
        "UniKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4128), tables: &VERTICA_TABLES }),
        "UnindexedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4129), tables: &VERTICA_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4130), tables: &VERTICA_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4135), tables: &VERTICA_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(4136), tables: &VERTICA_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4138), tables: &VERTICA_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4139), tables: &VERTICA_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4140), tables: &VERTICA_TABLES }),
        "UnlimitedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4141), tables: &VERTICA_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4142), tables: &VERTICA_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4143), tables: &VERTICA_TABLES }),
        "UnnamedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4144), tables: &VERTICA_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4145), tables: &VERTICA_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4146), tables: &VERTICA_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4160), tables: &VERTICA_TABLES }),
        "UnsegmentedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4166), tables: &VERTICA_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4167), tables: &VERTICA_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4168), tables: &VERTICA_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4169), tables: &VERTICA_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4170), tables: &VERTICA_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4178), tables: &VERTICA_TABLES }),
        "UpperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4179), tables: &VERTICA_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4180), tables: &VERTICA_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4181), tables: &VERTICA_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4182), tables: &VERTICA_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4185), tables: &VERTICA_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4186), tables: &VERTICA_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4187), tables: &VERTICA_TABLES }),
        "User_defined_type_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4188), tables: &VERTICA_TABLES }),
        "User_defined_type_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4189), tables: &VERTICA_TABLES }),
        "User_defined_type_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4190), tables: &VERTICA_TABLES }),
        "User_defined_type_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4191), tables: &VERTICA_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4192), tables: &VERTICA_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4193), tables: &VERTICA_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4194), tables: &VERTICA_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4195), tables: &VERTICA_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4196), tables: &VERTICA_TABLES }),
        "UuidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4197), tables: &VERTICA_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4198), tables: &VERTICA_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4199), tables: &VERTICA_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4200), tables: &VERTICA_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4201), tables: &VERTICA_TABLES }),
        "ValindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4202), tables: &VERTICA_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4203), tables: &VERTICA_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4204), tables: &VERTICA_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4217), tables: &VERTICA_TABLES }),
        "Var_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4218), tables: &VERTICA_TABLES }),
        "Var_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4219), tables: &VERTICA_TABLES }),
        "VarbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4220), tables: &VERTICA_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4221), tables: &VERTICA_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4222), tables: &VERTICA_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4223), tables: &VERTICA_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4224), tables: &VERTICA_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4225), tables: &VERTICA_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4226), tables: &VERTICA_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4227), tables: &VERTICA_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4228), tables: &VERTICA_TABLES }),
        "VerticaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4229), tables: &VERTICA_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4230), tables: &VERTICA_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4231), tables: &VERTICA_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4232), tables: &VERTICA_TABLES }),
        "WaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4233), tables: &VERTICA_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4234), tables: &VERTICA_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4235), tables: &VERTICA_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4236), tables: &VERTICA_TABLES }),
        "Webhdfs_addressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4237), tables: &VERTICA_TABLES }),
        "Webservice_hostnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4238), tables: &VERTICA_TABLES }),
        "Webservice_portKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4239), tables: &VERTICA_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4240), tables: &VERTICA_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4241), tables: &VERTICA_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4242), tables: &VERTICA_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4252), tables: &VERTICA_TABLES }),
        "WheneverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4253), tables: &VERTICA_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4254), tables: &VERTICA_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4259), tables: &VERTICA_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4272), tables: &VERTICA_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4273), tables: &VERTICA_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(4274), tables: &VERTICA_TABLES }),
        "Width_bucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4275), tables: &VERTICA_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4276), tables: &VERTICA_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4278), tables: &VERTICA_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4288), tables: &VERTICA_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(4289), tables: &VERTICA_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4297), tables: &VERTICA_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4306), tables: &VERTICA_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4315), tables: &VERTICA_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(4320), tables: &VERTICA_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4321), tables: &VERTICA_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4322), tables: &VERTICA_TABLES }),
        "WithinGroupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4327), tables: &VERTICA_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4332), tables: &VERTICA_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4333), tables: &VERTICA_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(4334), tables: &VERTICA_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4335), tables: &VERTICA_TABLES }),
        "WorkloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4336), tables: &VERTICA_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4337), tables: &VERTICA_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4338), tables: &VERTICA_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4339), tables: &VERTICA_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4340), tables: &VERTICA_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4341), tables: &VERTICA_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4342), tables: &VERTICA_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4343), tables: &VERTICA_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4344), tables: &VERTICA_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4345), tables: &VERTICA_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4346), tables: &VERTICA_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4347), tables: &VERTICA_TABLES }),
        "ZstdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4348), tables: &VERTICA_TABLES }),
        "Zstd_compKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4349), tables: &VERTICA_TABLES }),
        "Zstd_fast_compKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4350), tables: &VERTICA_TABLES }),
        "Zstd_high_compKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4351), tables: &VERTICA_TABLES }),
        _ => None,
    }
}

pub fn get_vertica_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterDefaultPrivilegesGrantSegment" => Some("alter_default_privileges_grant"),
        "AlterDefaultPrivilegesObjectPrivilegesSegment" => Some("alter_default_privileges_object_privilege"),
        "AlterSchemaStatementSegment" => Some("alter_schema_statement"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterSessionStatements" => Some("alter_session_statement"),
        "AlterTableActionSegment" => Some("alter_table_action_segment"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AlterViewStatementSegment" => Some("alter_view_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CaseExpressionSegment" => Some("case_expression"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnEncodingSegment" => Some("column_encoding"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnSetSegment" => Some("column_set_segment"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentOnStatementSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "ConcatSegment" => Some("binary_operator"),
        "CopyColumnOptionsSegment" => Some("copy_column_options"),
        "CopyOptionsForColumnsSegment" => Some("copy_options_for_columns"),
        "CopyOptionsSegment" => Some("copy_options"),
        "CopyStatementSegment" => Some("copy_statement"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateExternalTableSegment" => Some("create_external_table_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateProjectionStatementSegment" => Some("create_projection_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateTableAsStatementSegment" => Some("create_table_as_statement"),
        "CreateTableLikeStatementSegment" => Some("create_table_like_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "Dedent" => Some("dedent"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DiskQuotaSegment" => Some("disk_quota_segment"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropProjectionStatementSegment" => Some("drop_projection_statement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileSegment" => Some("file"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "IdentifierSegment" => Some("identifier"),
        "ImplicitIndent" => Some("indent"),
        "Indent" => Some("indent"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "KsafeSegment" => Some("ksafe_segment"),
        "LambdaExpressionSegment" => Some("lambda_function"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LikeOptionSegment" => Some("like_option_segment"),
        "LimitClauseSegment" => Some("limit_clause"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "NullEqualsSegment" => Some("comparison_operator"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionByClauseSegment" => Some("partitionby_clause"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaPrivilegesSegment" => Some("schema_privileges_segment"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SegmentedByClauseSegment" => Some("segmentedby_clause"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "SetStatementSegment" => Some("set_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TimeseriesClauseSegment" => Some("timeseries_clause_statement"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TransactionalStatements" => Some("transactional_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "ValuesClauseSegment" => Some("values_clause"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WithinGroupClauseSegment" => Some("within_group_clause_statement"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static VERTICA_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_vertica_root_grammar_id() -> GrammarId {
    GrammarId(2011)
}
pub fn get_vertica_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_vertica_root_grammar_id(),
        tables: &VERTICA_TABLES,
    }
}
pub fn get_vertica_root_grammar() -> RootGrammar {
    get_vertica_root_grammar_table()
}
