/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      7681 × 20 bytes =   153620 bytes
//   Child IDs:         6837 ×  4 bytes =    27348 bytes
//   Terminators:         92 ×  4 bytes =      368 bytes
//   Strings:           2318 strings      =    36739 bytes
//   Aux Data:         11344 ×  4 bytes =    45376 bytes
//   Aux Offsets:       7681 ×  4 bytes =    30724 bytes
//   Regex Patterns:      11 patterns     =      934 bytes
//   Simple Hints:      1225 hints        =    19600 bytes
//   Hint Indices:      2921 ×  4 bytes =    11684 bytes
//   TOTAL:                                    ≈   326393 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [3] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [4] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [5] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [6] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [50] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [51] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [60] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [61] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [62] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [63] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [64] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [131] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [133] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [134] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [135] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [136] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [137] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [138] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [139] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [140] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] Ref(SemiStructuredAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 212, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 214, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] StringParser(")")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 217, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 219, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 219, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 226, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 226, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 226, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] Meta("dedent")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 226, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 226, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] StringParser("ALLOW_CONNECTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 242, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 259, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 267, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 267, child_count: 3, min_times: 1, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] AnyNumberOf(3 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 270, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 272, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] Ref(Allow_connectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 272, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 272, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] Ref(Is_templateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 295, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 297, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [323] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 309, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 309, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 309, child_count: 7, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [328] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [329] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] Ref(AlterDefaultPrivilegesObjectPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [332] Ref(AlterDefaultPrivilegesSchemaObjectsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [333] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 1, _padding: 0 }, // [334] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 318, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [335] Ref(AlterDefaultPrivilegesToFromRolesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 318, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [336] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 318, child_count: 3, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [337] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [338] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [339] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [340] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [341] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 323, child_count: 2, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [342] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [343] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 325, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [344] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 2, min_times: 0, first_terminator_idx: 4, terminator_count: 1, _padding: 0 }, // [345] Delimited(10 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 327, child_count: 10, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [346] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [347] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [348] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [349] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [350] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [351] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [352] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [353] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [354] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [355] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [356] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [357] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 8, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [358] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 345, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [359] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 345, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [360] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [361] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [362] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [363] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [364] Ref(AlterDefaultPrivilegesObjectPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [365] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [366] Ref(AlterDefaultPrivilegesSchemaObjectsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [367] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [368] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [369] Ref(AlterDefaultPrivilegesToFromRolesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [370] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [371] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [372] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 6, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [373] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [374] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [375] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [376] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [377] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [378] Ref(TypesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [379] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [380] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [381] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [382] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [383] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 362, child_count: 3, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [384] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 365, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [385] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 365, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [386] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [387] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [388] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 3, _padding: 0 }, // [389] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [390] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [391] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [392] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [393] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 369, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [394] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [395] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [396] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 2, min_times: 0, first_terminator_idx: 10, terminator_count: 2, _padding: 0 }, // [397] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [398] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [399] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [400] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [401] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 376, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [402] Ref(AlterDefaultPrivilegesGrantSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 376, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [403] Ref(AlterDefaultPrivilegesRevokeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 376, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [404] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [405] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [406] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [407] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [408] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [409] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [410] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [411] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [412] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [413] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 394, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [414] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [415] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [416] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [417] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [418] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [419] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [420] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [421] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 402, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [422] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 404, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [423] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 404, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [424] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 404, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [425] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 404, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [426] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 404, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [427] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [428] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 408, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [429] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 410, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [430] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 410, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [431] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 410, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [432] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [433] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [434] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [435] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [436] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [437] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [438] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 416, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [439] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [440] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [441] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [442] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [443] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [444] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [445] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [446] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 423, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [447] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 425, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [448] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 425, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [449] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 425, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [450] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [451] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [452] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [453] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [454] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [455] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [456] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [457] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [458] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [459] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [460] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [461] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [462] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [463] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [464] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [465] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [466] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [467] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [468] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [469] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [470] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [471] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [472] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [473] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [474] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [475] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [476] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [477] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [478] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [479] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [480] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [481] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [482] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 455, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [483] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [484] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [485] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 457, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [486] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 460, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [487] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 460, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [488] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 460, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [489] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [490] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [491] Ref(Current_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [492] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [493] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [494] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [495] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [496] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 473, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [497] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 475, child_count: 13, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [498] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 488, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [499] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [500] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [501] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [502] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [503] Ref(ConversionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [504] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [505] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [506] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [507] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [508] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 495, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [509] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 495, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [510] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 495, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [511] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 495, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [512] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [513] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [514] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [515] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [516] Ref(ProceduralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [517] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [518] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [519] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [520] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [521] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 502, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [522] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 502, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [523] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 502, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [524] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [525] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [526] Ref(DictionaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [527] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [528] Ref(TemplateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [529] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [530] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [531] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [532] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [533] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [534] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [535] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [536] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [537] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [538] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [539] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [540] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [541] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 516, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [542] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 519, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [543] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 520, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [544] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 520, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [545] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [546] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [547] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [548] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [549] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [550] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [551] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [552] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [553] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 533, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [554] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 533, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [555] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 533, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [556] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 533, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [557] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 535, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [558] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [559] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 537, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [560] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [561] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [562] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [563] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 541, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [564] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 543, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [565] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 543, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [566] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 546, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [567] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 548, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [568] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [569] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [570] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [571] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [572] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [573] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [574] Ref(ClassKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [575] Ref(FamilyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [576] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [577] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [578] Ref(IndexAccessMethodSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [579] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [580] Ref(TransformKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [581] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [582] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [583] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 24, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [584] OneOf(24 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [585] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [586] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [587] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [588] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [589] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [590] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 592, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [591] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [592] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [593] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 594, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [594] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [595] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [596] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [597] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [598] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [599] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [600] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [601] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [602] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [603] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [604] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [605] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 13, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [606] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [607] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 622, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [608] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [609] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [610] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [611] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [612] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 624, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [613] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [614] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [615] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 626, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [616] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [617] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [618] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 629, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [619] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 629, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [620] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [621] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [622] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [623] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 633, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [624] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 637, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [625] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 637, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [626] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 637, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [627] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 637, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [628] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 637, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [629] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [630] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [631] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [632] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 642, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [633] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [634] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [635] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [636] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [637] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [638] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 647, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [639] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 647, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [640] Ref(ExpressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 647, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [641] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 647, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [642] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [643] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [644] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [645] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [646] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [647] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [648] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [649] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [650] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [651] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 657, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [652] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 660, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [653] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [654] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [655] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [656] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [657] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [658] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [659] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [660] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [661] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [662] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [663] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [664] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [665] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [666] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [667] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [668] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [669] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [670] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [671] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [672] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [673] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [674] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [675] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 681, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [676] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [677] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 684, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [678] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 684, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [679] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 684, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [680] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 684, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [681] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [682] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [683] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [684] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [685] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [686] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [687] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 691, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [688] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 691, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [689] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 691, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [690] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [691] Ref(PlainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [692] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [693] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [694] Ref(MainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [695] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [696] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [697] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [698] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [699] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [700] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [701] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [702] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [703] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [704] Ref(TableConstraintUsingIndexSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [705] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 707, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [706] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 707, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [707] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 707, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [708] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 707, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [709] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [710] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [711] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [712] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [713] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 711, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [714] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 713, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [715] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [716] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [717] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [718] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [719] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [720] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [721] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 720, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [722] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 720, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [723] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 720, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [724] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 720, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [725] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [726] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [727] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [728] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [729] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [730] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [731] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 728, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [732] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [733] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [734] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [735] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [736] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [737] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [738] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [739] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 733, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [740] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [741] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [742] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [743] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [744] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [745] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [746] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [747] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [748] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 746, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [749] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 746, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [750] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 746, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [751] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [752] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [753] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [754] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [755] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [756] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [757] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [758] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [759] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [760] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [761] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [762] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [763] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [764] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [765] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [766] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [767] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [768] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [769] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [770] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [771] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [772] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [773] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [774] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [775] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [776] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [777] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [778] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [779] Ref(OidsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [780] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [781] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [782] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [783] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [784] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [785] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [786] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [787] Ref(LoggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [788] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [789] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [790] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [791] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [792] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [793] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [794] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [795] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 782, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [796] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 782, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [797] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 782, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [798] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 782, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [799] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [800] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [801] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [802] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [803] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [804] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 786, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [805] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [806] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [807] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [808] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [809] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [810] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [811] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [812] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [813] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 796, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [814] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 796, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [815] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 796, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [816] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 800, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [817] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 800, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [818] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [819] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [820] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [821] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [822] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [823] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [824] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 806, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [825] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 809, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [826] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 809, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [827] Ref(COLUMN)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 809, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [828] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 809, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [829] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 809, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [830] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 812, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [831] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [832] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 817, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [833] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [834] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [835] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [836] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [837] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 820, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [838] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 820, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [839] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 824, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [840] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 824, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [841] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 824, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [842] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 824, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [843] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 829, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [844] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 829, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [845] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 829, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [846] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 829, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [847] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 829, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [848] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [849] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [850] Ref(AlterForeignTableActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [851] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 839, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [852] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 839, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [853] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 839, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [854] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 839, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [855] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 839, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [856] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 839, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [857] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [858] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [859] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [860] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [861] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [862] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [863] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [864] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [865] Ref(CalledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [866] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [867] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [868] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [869] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [870] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [871] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [872] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [873] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [874] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [875] Ref(StrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [876] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 869, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [877] Ref(ImmutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 869, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [878] Ref(StableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 869, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [879] Ref(VolatileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 869, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [880] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [881] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [882] Ref(LeakproofKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [883] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [884] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [885] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [886] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [887] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [888] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [889] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [890] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [891] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [892] Ref(UnsafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [893] Ref(RestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [894] Ref(SafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [895] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [896] Ref(CostKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [897] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [898] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [899] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [900] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [901] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [902] Ref(SupportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [903] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [904] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [905] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [906] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [907] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [908] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [909] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [910] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [911] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [912] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [913] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [914] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 899, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [915] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 899, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [916] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [917] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [918] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [919] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 903, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [920] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 903, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [921] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [922] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [923] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [924] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [925] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [926] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [927] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [928] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [929] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 913, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [930] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 913, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [931] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 913, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [932] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [933] Ref(AlterFunctionActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [934] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [935] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [936] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [937] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [938] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [939] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [940] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [941] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [942] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 927, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [943] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 927, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [944] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 927, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [945] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [946] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [947] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [948] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [949] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [950] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [951] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [952] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [953] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [954] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [955] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [956] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [957] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 938, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [958] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [959] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [960] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [961] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [962] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 946, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [963] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 946, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [964] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 946, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [965] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [966] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [967] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [968] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [969] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [970] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [971] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [972] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [973] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [974] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [975] Ref(AttachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [976] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [977] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [978] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 967, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [979] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 967, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [980] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 967, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [981] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 967, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [982] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 967, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [983] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 967, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [984] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 969, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [985] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 969, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [986] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 972, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [987] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 974, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [988] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 976, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [989] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 976, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [990] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 978, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [991] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 978, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [992] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 978, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [993] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [994] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [995] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 983, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [996] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [997] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [998] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [999] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1000] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1001] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1002] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1003] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1004] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 991, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1005] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1000, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1006] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1000, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1007] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1000, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1008] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1000, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1009] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1000, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1010] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1011] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1012] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1013] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1005, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1014] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1005, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1015] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1005, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1016] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1005, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1017] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1005, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1018] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1005, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1019] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1005, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1020] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1021] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1022] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1023] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1024] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1025] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1020, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1026] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1027] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1028] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1029] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1030] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1025, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1031] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1025, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1032] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1028, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1033] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1030, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1034] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1035] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1036] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1037] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1038] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1035, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1039] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1035, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1040] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1038, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1041] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1042] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1043] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1043, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1044] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1043, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1045] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1043, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1046] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1047] Ref(PlainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1048] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1049] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1050] Ref(MainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1051] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1050, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1052] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1050, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1053] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1050, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1054] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1050, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1055] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1056] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1057] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1058] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1059] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1060] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1061] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1062] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1063] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1064] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1065] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1061, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1066] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1063, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1067] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1068] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1065, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1069] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1067, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1070] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1067, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1071] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1067, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1072] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1073] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1074] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1075] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1074, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1076] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1074, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1077] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1078] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1079] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1080] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1081] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1082] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1083] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1084] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1085] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1086] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1087] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1088] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1089] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1090] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1091] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1092] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1093] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1094] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1095] Ref(AlterMaterializedViewActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1096] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1097] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1102, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1098] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1099] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1100] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1101] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1102] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1103] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1104] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1105] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1106] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1107] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1108] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1109] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1110] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1111] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1112] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1113] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1114] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1115] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1116] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1117] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1118] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1119] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1120] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1121] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1122] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1124, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1123] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1124] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1125] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1126] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1127] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1128] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1129] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1130] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1129, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1131] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1130, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1132] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1130, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1133] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1134] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1135] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1136] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1137] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1138] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1139] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1138, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1140] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1141] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1142] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1143] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 3, min_times: 1, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1144] AnySetOf(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1145] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1146] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1147] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1148] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1149] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1150] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1151] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1152] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1153] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1154] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1155] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1156] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1158, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1157] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1158, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1158] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1159] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1160] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1161] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1164, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1162] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1164, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1163] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1166, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1164] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1169, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1165] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1166] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1167] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1172, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1168] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1169] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1170] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1171] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1172] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1173] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1177, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1174] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1175] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1181, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1176] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1177] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1178] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1179] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1180] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1181] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1182] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1183] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1184] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1185] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1186] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1187] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1188] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1192, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1189] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1192, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1190] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1192, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1191] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1192, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1192] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1193] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1194] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1195] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1198, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1197] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1198] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1199] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1200] Ref(AlterProcedureActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1201] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1202] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1203] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1204] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1205] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1206] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1207] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1208] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1209] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1210] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1211] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1212] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1217, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1213] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1219, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1214] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1215] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1216] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1217] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1223, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1218] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1219] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1220] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1221] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1222] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1223] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1224] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1225] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1226] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1227] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1234, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1228] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1229] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1230] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1231] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1232] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1233] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1234] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1235] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1236] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1237] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1238] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1239] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1240] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1245, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1241] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1245, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1242] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1245, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1243] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1245, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1244] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1251, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1245] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1246] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1247] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1248] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1249] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1250] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1257, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1251] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1257, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1252] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1253] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1254] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1261, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1255] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1261, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1256] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1263, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1257] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1263, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1258] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1259] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1260] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1261] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1262] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1263] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1264] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1265] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1266] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1267] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1268] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1269] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1270] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1271] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1272] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1273] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1274] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1280, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1275] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1280, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1276] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1280, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1277] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1278] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1286, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1279] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1280] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1281] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1282] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1283] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1284] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1290, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1285] AnySetOf(10 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1286] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1287] Ref(SuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1288] Ref(NosuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1289] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1290] Ref(CreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1291] Ref(NocreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1292] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1293] Ref(CreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1294] Ref(NocreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1295] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1296] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1297] Ref(NoinheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1298] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1310, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1299] Ref(LoginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1310, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1300] Ref(NologinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1310, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1301] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1302] Ref(ReplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1303] Ref(NoreplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1304] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1305] Ref(BypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1306] Ref(NobypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1307] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1308] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1309] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1310] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1311] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1320, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1312] Ref(EncryptedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1320, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1313] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1320, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1314] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1322, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1315] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1322, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1316] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1322, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1317] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1318] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1319] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1320] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1321] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1322] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1323] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1324] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1325] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1326] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1327] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1333, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1328] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1329] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1330] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1331] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1332] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1333] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1338, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1334] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1335] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1336] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1337] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1338] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1339] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1340] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1341] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1342] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1343] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1344] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1345] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1346] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1347] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1348] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1349] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1356, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1350] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1351] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1352] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1353] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1354] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1361, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1355] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1361, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1356] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1361, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1357] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1363, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1358] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1363, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1359] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1360] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1361] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1362] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1369, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1363] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1369, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1364] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1369, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1365] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1369, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1366] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1371, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1367] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1368] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1369] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1370] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1371] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1372] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1373] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1374] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1375] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1376] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1389, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1377] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1389, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1378] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1389, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1379] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1380] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1393, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1381] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1393, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1382] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1383] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1384] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1385] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1386] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1387] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1388] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1389] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1390] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1391] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1392] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1393] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1405, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1394] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1405, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1395] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1405, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1396] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1405, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1397] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1407, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1398] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1407, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1399] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1407, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1400] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1401] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1402] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1403] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1404] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1405] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1406] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1407] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1408] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1409] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1418, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1410] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1418, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1411] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1418, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1412] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1418, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1413] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1418, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1414] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1415] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1416] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1417] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1418] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1419] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1420] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1425, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1421] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1425, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1422] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1425, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1423] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1427, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1424] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1427, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1425] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1427, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1426] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1427] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1428] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1432, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1429] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1430] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1431] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1436, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1432] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1433] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1434] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1435] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1436] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1437] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1438] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1439] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1440] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1441] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1446, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1442] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1446, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1443] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1446, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1444] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1446, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1445] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1446] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1447] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1448] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1449] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1450] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1451] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1452] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1453] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1454] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1455] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1456] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1457] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1458] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1464, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1459] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1460] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1461] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1462] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1463] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1464] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1465] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1466] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1467] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1468] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1469] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1470] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1471] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1472] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1473] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1474] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1475] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1476] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1477] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1478] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1479] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1480] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1481] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1482] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1490, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1483] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1484] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1485] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1492, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1486] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1487] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1488] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1489] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1490] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1491] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1492] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1493] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1501, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1494] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1503, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1495] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1503, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1496] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1503, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1497] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1506, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1498] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1506, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1499] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1506, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1500] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1501] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1502] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1503] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1504] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1505] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1506] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1507] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1508] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1509] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1510] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1517, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1511] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1517, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1512] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1517, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1513] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1517, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1514] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1515] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1516] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1517] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1518] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1519] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1520] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1521] Ref(Current_sessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1522] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1523] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1524] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1525] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1526] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1530, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1527] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1530, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1528] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1530, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1529] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1532, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1530] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1531] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1532] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1533] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1534] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1535] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1536] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1537] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1538] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1542, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1539] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1546, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1540] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1546, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1541] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1546, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1542] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1546, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1543] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1546, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1544] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1545] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1546] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1550, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1547] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1550, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1548] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1550, child_count: 23, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1549] OneOf(23 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1550] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1551] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1552] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1553] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1554] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1555] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1580, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1556] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1557] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1558] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1582, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1559] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1560] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1561] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1562] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1563] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1564] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1565] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1566] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1567] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1592, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1568] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1592, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1569] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1592, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1570] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1592, child_count: 13, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1571] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1572] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1610, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1573] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1574] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1575] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1576] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1577] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1612, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1578] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1579] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1580] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1614, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1581] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1582] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1583] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1584] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1585] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1586] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1587] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1588] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1589] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1590] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1591] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1592] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1593] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1594] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1627, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1595] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1627, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1596] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1627, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1597] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1630, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1598] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1599] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1600] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1601] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1602] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1603] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1604] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1605] Ref(ExpressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1606] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1607] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1608] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1609] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1610] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1643, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1611] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1643, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1612] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1613] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1614] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1615] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1645, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1616] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1645, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1617] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1648, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1618] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1619] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1620] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1621] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1653, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1622] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1623] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1624] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1625] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1626] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1627] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1660, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1628] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1660, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1629] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1660, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1630] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1631] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1632] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1633] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1664, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1634] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1664, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1635] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1666, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1636] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1666, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1637] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1666, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1638] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1639] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1640] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1669, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1641] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1642] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1643] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1644] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1645] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1646] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1647] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1648] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1649] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1676, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1650] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1676, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1651] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1676, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1652] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1653] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1654] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1655] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1656] Ref(PlainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1657] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1658] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1659] Ref(MainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1660] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1661] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1662] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1663] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1664] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1665] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1666] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1667] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1690, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1668] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1690, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1669] Ref(TableConstraintUsingIndexSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1690, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1670] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1671] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1672] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1673] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1695, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1674] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1697, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1675] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1697, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1676] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1699, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1677] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1699, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1678] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1699, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1679] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1701, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1680] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1681] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1682] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1683] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1684] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1685] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1686] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1687] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1688] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1689] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1690] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1691] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1692] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1693] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1694] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1695] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1696] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1716, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1697] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1698] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1699] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1700] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1701] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1721, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1702] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1721, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1703] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1721, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1704] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1721, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1705] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1725, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1706] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1725, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1707] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1708] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1709] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1710] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1711] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1712] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1730, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1713] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1714] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1715] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1716] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1717] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1718] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1719] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1720] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1721] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1722] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1738, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1723] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1738, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1724] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1725] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1726] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1727] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1728] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1729] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1748, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1730] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1748, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1731] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1748, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1732] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1748, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1733] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1748, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1734] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1748, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1735] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1736] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1737] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1738] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1739] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1740] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1741] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1742] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1743] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1744] Ref(OidsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1745] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1746] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1747] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1748] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1749] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1750] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1751] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1752] Ref(LoggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1753] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1754] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1755] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1756] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1765, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1757] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1758] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1759] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1760] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1761] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1762] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1763] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1764] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1765] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1766] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1767] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1768] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1769] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1770] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1777, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1771] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1777, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1772] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1777, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1773] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1774] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1775] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1776] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1777] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1778] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1779] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1780] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1781] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1782] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1783] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1784] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1785] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1786] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1787] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1788] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1789] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1790] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1791] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1792] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1793] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1794] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1795] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1796] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1797] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1798] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1799] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1800] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1804, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1801] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1808, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1802] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1810, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1803] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1810, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1804] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1810, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1805] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1810, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1806] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1810, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1807] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1808] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1809] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1810] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1811] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1812] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1813] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1814] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1815] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1816] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1819, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1817] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1818] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1819] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1820] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1821] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1822] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1823] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1824] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1827, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1825] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1832, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1826] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1832, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1827] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1832, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1828] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1832, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1829] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1832, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1830] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1835, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1831] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1832] Ref(AlterTableActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1833] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1834] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1835] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1836] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1837] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1838] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1839] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1840] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1841] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1842] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1843] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1844] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1845] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1846] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1847] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1848] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1854, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1849] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1850] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1851] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1852] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1853] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1854] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1855] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1856] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1857] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1858] Ref(AttachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1859] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1860] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1861] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1866, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1862] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1863] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1864] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1865] Ref(PartitionBoundSpecSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1866] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1867] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1868] Ref(DetachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1869] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1870] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1871] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1872] Ref(FinalizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1873] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1874] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1875] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1876] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1877] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1883, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1878] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1886, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1879] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1886, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1880] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1886, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1881] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1882] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1883] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1884] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1885] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1888, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1886] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1887] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1888] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1889] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1890] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1891] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1892] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 8, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1893] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1902, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1894] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1895] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1896] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1897] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1898] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1899] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1900] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1901] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1902] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1903] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1904] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1905] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1906] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1907] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1908] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1909] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1910] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1922, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1911] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1922, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1912] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1913] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1914] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1915] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1916] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1917] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1918] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 8, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1919] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1920] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1921] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1922] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1923] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1924] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1925] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1926] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1927] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1928] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1929] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1943, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1930] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1943, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1931] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1943, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1932] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1943, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1933] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1943, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1934] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1935] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1936] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1937] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1938] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1939] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1948, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1940] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1941] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1942] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1943] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1944] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1945] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1946] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1947] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1948] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1949] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1950] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1951] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1952] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1953] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1954] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1955] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1956] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1957] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1958] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1966, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1959] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1960] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1961] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1962] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1963] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1974, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1964] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1974, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1965] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1974, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1966] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1974, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1967] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1974, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1968] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1974, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1969] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1970] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1971] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1972] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1973] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1974] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1975] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1976] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1977] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1978] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1979] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1980] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1981] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1982] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1983] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1984] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1985] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1986] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1987] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1988] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1989] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1990] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1991] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1992] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1993] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1994] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1995] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2002, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1996] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2004, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1997] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2008, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1998] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2014, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1999] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2014, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2000] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2014, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2001] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2014, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2002] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2014, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2003] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2016, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2004] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2016, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2005] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2016, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2006] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2016, child_count: 8, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2007] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2008] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2009] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2010] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2024, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2011] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2012] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2013] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2014] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2015] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2026, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2016] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2017] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2018] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2019] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2020] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2021] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2022] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2023] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2024] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2025] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2026] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2027] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2028] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2029] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2030] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2031] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2032] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2033] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2034] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2035] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2044, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2036] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2037] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2044, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2038] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2039] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2040] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2041] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2042] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2043] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2053, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2044] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2053, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2045] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2053, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2046] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2053, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2047] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2053, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2048] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2060, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2049] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2050] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2051] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2052] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2053] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2066, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2054] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2055] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2056] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2057] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2058] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2059] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2060] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2061] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2062] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2063] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2064] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2065] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2066] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2067] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2068] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2069] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2070] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2071] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2072] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2073] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2074] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2075] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2076] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2077] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2078] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2079] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2080] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2081] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2082] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2083] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2084] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2085] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2086] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2087] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2088] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2089] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2090] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2091] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2102, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2092] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2102, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2093] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2094] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2095] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2096] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2097] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2106, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2098] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2109, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2099] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2100] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2101] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2102] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2103] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2111, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2104] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2105] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2114, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2106] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2107] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2108] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2116, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2109] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2118, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2110] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2111] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2112] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2113] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2114] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2115] Ref(Skip_lockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2116] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2117] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2127, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2118] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2119] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2120] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2131, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2121] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2122] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2123] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2124] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2125] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2126] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2136, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2127] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2136, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2128] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2129] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2130] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2131] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2132] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2133] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2134] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2135] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2136] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2137] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2138] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2139] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2140] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2153, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2141] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2153, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2142] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2153, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2143] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2153, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2144] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2156, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2145] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2146] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2147] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2148] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2149] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2159, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2150] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2151] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2152] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2153] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2154] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2155] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2156] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2157] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2158] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2165, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2159] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2160] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2161] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2162] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2170, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2163] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2170, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2164] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2174, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2165] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2174, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2166] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2174, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2167] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2168] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2169] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2170] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2171] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2172] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2173] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2174] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2175] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2176] StringParser("ATTACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2177] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2178] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2179] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2180] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2176, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2181] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2176, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2182] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2176, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2183] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2176, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2184] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2176, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2185] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2176, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2186] MultiStringParser(12 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2176, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2187] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2178, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 2, _padding: 0 }, // [2188] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2189] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2190] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2191] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2192] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2193] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2194] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2196] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2197] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2198] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2199] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2200] Ref(CompositeValueExpansionSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2201] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2202] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2203] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2204] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2205] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2206] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2207] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2208] StringParser("BIGSERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2187, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2209] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 5, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2210] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2211] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2212] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2213] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2214] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2215] Ref(JsonOperatorSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2216] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2217] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2218] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2219] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2220] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2192, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2221] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2222] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2223] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2224] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2225] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2226] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2227] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2228] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2229] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2230] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2231] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2232] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2233] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2234] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2235] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2236] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2237] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2238] StringParser("BOX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2239] StringParser("BPCHAR")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2240] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2203, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2241] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2242] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2243] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2244] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2245] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2246] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2247] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2248] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2249] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2250] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2251] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2252] StringParser("BUFFERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2253] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2254] StringParser("BY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2255] StringParser("BYPASSRLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2256] StringParser("BYTEA")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2213, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2257] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2258] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 6, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2259] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2260] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2261] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2262] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2222, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2263] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2264] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2265] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2266] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2227, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2267] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2227, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2268] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2229, child_count: 7, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2269] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2236, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2270] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2236, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2271] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2272] Ref(BreadthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2273] Ref(DepthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2274] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2275] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2276] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2277] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2278] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 6, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2279] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2280] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2281] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2282] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2283] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2284] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2285] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2286] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2287] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2288] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2246, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2289] Ref(CallKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2246, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2290] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2246, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2291] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2246, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2292] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2246, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2293] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2294] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2295] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2296] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 3, _padding: 0 }, // [2297] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2250, child_count: 6, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2298] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2299] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2256, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2300] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2256, child_count: 1, min_times: 0, first_terminator_idx: 14, terminator_count: 2, _padding: 0 }, // [2301] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2302] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2303] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2304] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 1, _padding: 0 }, // [2305] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2306] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2307] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 7, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2308] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2264, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2309] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2264, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2310] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2264, child_count: 1, min_times: 0, first_terminator_idx: 17, terminator_count: 2, _padding: 0 }, // [2311] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2312] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2313] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2314] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [2315] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2316] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2317] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2318] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2319] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2320] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2321] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2322] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2323] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2324] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2325] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2326] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2327] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2328] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2329] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2330] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2331] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2332] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2333] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2334] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2335] StringParser("CIDR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2336] StringParser("CIRCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2337] StringParser("CIRCULARSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2338] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2339] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2340] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2341] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2268, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2342] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2268, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2343] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2268, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2344] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2270, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2345] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2346] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2272, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2347] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2274, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2348] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2274, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2349] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2274, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2350] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2351] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2352] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2353] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2354] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2355] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2356] Token("raw")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2357] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2358] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2359] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2360] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2361] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2279, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 1, _padding: 0 }, // [2362] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2363] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2364] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2365] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2366] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2367] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2368] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2369] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2370] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2371] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2372] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2373] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2374] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2289, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2375] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2291, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2376] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2291, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2377] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2291, child_count: 8, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2378] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2299, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2379] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2301, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2380] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2381] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2382] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2383] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2384] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2385] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2307, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2386] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2309, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2387] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2309, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2388] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2309, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2389] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2390] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 5, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2391] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2392] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2393] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2394] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2395] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2396] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2316, child_count: 5, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2397] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2398] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2399] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2400] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2401] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2402] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 5, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2403] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2404] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2405] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2406] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2407] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2330, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2408] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2330, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2409] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2330, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2410] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2330, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2411] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2330, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2412] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2333, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2413] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2414] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2415] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2338, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2416] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2338, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2417] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2418] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2419] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2420] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2341, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2421] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2422] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2423] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2343, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2424] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2425] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2426] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2427] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2428] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2429] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2430] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2431] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2351, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2432] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2353, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2433] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2353, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2434] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2353, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2435] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2436] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2437] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2438] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2439] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2440] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2357, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2441] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2442] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2443] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2444] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2445] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2361, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2446] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2363, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2447] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2448] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2449] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2450] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2451] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2452] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2367, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2453] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2454] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2455] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2370, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2456] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2372, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2457] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2372, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2458] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2459] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2460] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2461] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2462] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2374, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2463] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2464] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 2376, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2465] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2466] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2467] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2468] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2469] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2470] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2382, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 12, _padding: 0 }, // [2471] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2384, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2472] Ref(SingleIdentifierFullGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2384, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2473] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2386, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2474] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2386, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2475] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2476] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2477] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2478] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2479] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2480] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2481] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2482] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2483] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2484] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2485] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2486] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2487] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2488] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2489] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2490] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2491] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2492] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2493] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2494] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2495] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2496] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2497] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2498] StringParser(",")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2391, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2499] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2393, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2500] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2393, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2501] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2393, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2502] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2393, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2503] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2504] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2505] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2506] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2398, child_count: 11, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2507] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2409, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2508] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2411, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2509] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2510] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2511] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2512] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2513] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2514] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2515] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2516] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2517] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2518] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2519] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2520] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2521] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2522] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2523] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2524] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2525] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2526] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2527] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2528] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2529] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2530] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2531] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2532] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2533] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2534] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2535] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2433, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2536] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2537] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2538] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2436, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2539] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2540] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2541] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2542] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2543] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2544] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2545] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2546] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2547] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2444, child_count: 24, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2548] OneOf(24 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2549] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2550] Ref(ConversionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2551] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2552] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2553] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2554] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2555] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2556] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2557] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2558] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2559] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2560] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2561] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2562] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2563] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2564] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2565] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2566] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2567] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2568] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2569] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2570] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2571] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2572] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2573] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2475, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2574] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2477, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2575] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2477, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2576] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2477, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2577] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2578] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2579] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2580] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2581] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2582] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2583] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2482, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2584] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2585] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2586] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2587] Ref(DictionaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2588] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2589] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2590] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2591] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2592] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2491, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2593] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2491, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2594] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2491, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2595] Ref(TemplateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2491, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2596] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2491, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2597] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2598] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2599] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2600] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2601] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2602] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2603] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2604] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2605] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2499, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2606] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2502, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2607] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2608] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2503, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2609] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2505, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2610] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2505, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2611] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2612] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2613] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2614] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2615] StringParser("COMMENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2616] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2617] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2618] StringParser("COMMUTATOR")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2507, child_count: 17, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2619] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2620] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2621] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2622] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2623] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2624] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2625] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2626] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2627] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2527, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2628] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2527, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2629] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2527, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2630] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2527, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2631] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2632] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2633] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2634] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2635] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2636] Ref(OverlapSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2637] Ref(NotExtendRightSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2638] Ref(NotExtendLeftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2639] Ref(AdjacentSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2640] Ref(PostgisOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2641] Ref(PgvectorOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2642] Ref(PgTrgmOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2643] Ref(QualifiedOperatorSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2644] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2645] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2646] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2647] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2648] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2534, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2649] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2537, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2650] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2537, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2651] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2537, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2652] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2537, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2653] StringParser("COMPOUNDCURVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2537, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2654] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2537, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2655] StringParser("COMPRESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2537, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2656] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2537, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2657] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2658] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2659] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2660] StringParser("CONCURRENTLY")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2661] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2662] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2663] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2664] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2665] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2542, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2666] StringParser("CONFIGURATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2667] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2544, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2668] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2544, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2669] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2670] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2671] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2672] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2673] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2674] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2556, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2675] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2676] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2562, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2677] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2562, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2678] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2562, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2679] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2564, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2680] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2564, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2681] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2564, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2682] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2568, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2683] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2684] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2685] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2686] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2687] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2688] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2576, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2689] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2578, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2690] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2580, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2691] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2580, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2692] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2580, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2693] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2583, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2694] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2586, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2695] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2588, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2696] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2588, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2697] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2588, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2698] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2699] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2591, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2700] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2701] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2702] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2703] StringParser("CONFLICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2704] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2705] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2597, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2706] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2707] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2602, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2708] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2605, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2709] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2608, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2710] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2608, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2711] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2712] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2713] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2611, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2714] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2613, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2715] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2613, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2716] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2613, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2717] Ref(OperationClassReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2613, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2718] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2615, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2719] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2615, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2720] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2615, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2721] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2722] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2723] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2724] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2725] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2726] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2727] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2728] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2729] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2730] StringParser("CONTENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2731] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2732] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2733] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2734] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2735] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2736] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2737] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2629, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2738] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2631, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2739] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2631, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2740] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2634, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2741] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2636, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2742] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2636, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2743] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2636, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2744] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2638, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2745] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2641, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2746] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2641, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2747] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2643, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2748] Ref(ProgramKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2643, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2749] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2645, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2750] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2645, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2751] Ref(PsqlVariableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2645, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2752] Ref(PsqlVariableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2645, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2753] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2646, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2754] Ref(StdinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2646, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2755] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2756] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2757] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2758] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2653, child_count: 11, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2759] AnySetOf(11 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2664, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2760] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2666, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2761] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2666, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2762] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2666, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2763] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2668, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2764] Ref(FreezeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2668, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2765] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2668, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2766] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2670, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2767] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2670, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2768] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2670, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2769] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2672, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2770] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2672, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2771] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2672, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2772] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2674, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2773] Ref(HeaderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2674, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2774] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2674, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2775] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2676, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2776] Ref(QuoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2676, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2777] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2676, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2778] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2678, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2779] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2678, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2780] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2678, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2781] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2782] Ref(Force_quoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2783] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2682, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2784] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2685, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2785] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2786] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2787] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2687, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2788] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2689, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2789] Ref(Force_not_nullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2689, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2790] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2692, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2791] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2694, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2792] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2694, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2793] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2696, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2794] Ref(Force_nullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2696, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2795] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2699, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2796] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2701, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2797] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2701, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2798] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2799] Ref(EncodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2800] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2703, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2801] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2705, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2802] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2705, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2803] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2705, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2804] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2805] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2806] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2712, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2807] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2712, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2808] Ref(PsqlVariableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2712, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2809] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2713, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2810] Ref(StdinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2713, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2811] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2715, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2812] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2715, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2813] AnySetOf(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2719, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2814] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2815] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2816] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2723, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2817] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2723, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2818] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2723, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2819] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2723, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2820] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2726, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2821] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2726, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2822] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2726, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2823] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2726, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2824] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2728, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2825] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2728, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2826] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2732, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2827] Ref(HeaderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2732, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2828] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2735, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2829] Ref(QuoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2735, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2830] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2735, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2831] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2735, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2832] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2738, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2833] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2738, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2834] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2738, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2835] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2738, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2836] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2742, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2837] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2742, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2838] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2742, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2839] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2742, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2840] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2744, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2841] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2744, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2842] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2748, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2843] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2750, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2844] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2845] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2846] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2847] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2848] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2756, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2849] Ref(StdoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2756, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2850] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2760, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2851] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2762, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2852] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2765, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2853] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2765, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2854] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2765, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2855] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2856] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2857] Ref(PsqlVariableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2858] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2769, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2859] Ref(StdoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2769, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2860] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2771, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2861] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2771, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2862] AnySetOf(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2775, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2863] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2776, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2864] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2776, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2865] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2866] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2867] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2868] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2869] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2782, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2870] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2782, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2871] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2782, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2872] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2782, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2873] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2784, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2874] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2784, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2875] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2876] Ref(HeaderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2877] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2791, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2878] Ref(QuoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2791, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2879] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2791, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2880] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2791, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2881] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2794, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2882] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2794, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2883] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2794, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2884] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2794, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2885] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2797, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2886] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2797, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2887] Ref(QuoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2797, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2888] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2889] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2890] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2804, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2891] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2804, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2892] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2804, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2893] StringParser("CORRELATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2804, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2894] StringParser("COST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2804, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2895] StringParser("COSTS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2804, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2896] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2897] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2898] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2899] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2900] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2901] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2813, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2902] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2903] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2904] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2818, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2905] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2818, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2906] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2818, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2907] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2823, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2908] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2823, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2909] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2823, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2910] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2823, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2911] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2912] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2913] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2914] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2915] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2830, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2916] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2917] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2832, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2918] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2832, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2919] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2832, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2920] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2834, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2921] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2834, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2922] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2834, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2923] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2836, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2924] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2838, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2925] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2838, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2926] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2838, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2927] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2840, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2928] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2840, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2929] Ref(ImplicitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2840, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2930] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2845, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2931] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2845, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2932] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2845, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2933] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2845, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2934] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2845, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2935] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2847, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2936] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2850, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2937] Delimited(6 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2852, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2938] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2858, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2939] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2940] Ref(LocaleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2941] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2942] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2861, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2943] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2944] Ref(Lc_collateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2945] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2946] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2947] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2948] Ref(Lc_ctypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2949] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2950] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2951] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2870, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2952] Ref(ProviderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2870, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2953] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2870, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2954] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2955] Ref(IcuKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2872, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2956] Ref(LibcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2872, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2957] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2875, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2958] Ref(DeterministicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2875, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2959] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2875, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2960] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2875, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2961] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2962] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2963] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2964] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2965] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2880, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2966] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2880, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2967] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2880, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2968] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2885, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2969] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2885, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2970] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2885, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2971] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2885, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2972] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2885, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2973] AnyNumberOf(8 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2893, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2974] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2975] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2976] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2977] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2978] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2979] Ref(TemplateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2980] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2981] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2982] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2902, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2983] Ref(EncodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2902, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2984] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2902, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2985] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2986] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2987] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2988] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2906, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2989] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2909, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2990] Ref(LocaleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2909, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2991] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2909, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2992] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2909, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2993] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2911, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2994] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2914, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2995] Ref(Lc_collateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2914, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2996] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2914, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2997] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2914, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2998] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2917, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2999] Ref(Lc_ctypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2917, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3000] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2917, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3001] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2917, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3002] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3003] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3004] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2920, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3005] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2922, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3006] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2922, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3007] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2922, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3008] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3009] Ref(Allow_connectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3010] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2925, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3011] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2925, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3012] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3013] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3014] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3015] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3016] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3017] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2932, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3018] Ref(Is_templateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2932, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3019] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2932, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3020] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2932, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3021] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3022] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3023] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2940, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3024] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2940, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3025] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3026] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3027] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2941, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3028] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3029] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3030] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2943, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3031] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3032] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3033] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2945, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3034] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2946, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3035] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2948, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3036] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3037] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3038] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3039] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3040] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3041] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3042] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2954, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3043] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2956, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3044] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2956, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3045] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2956, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3046] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2965, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3047] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2965, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3048] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2965, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3049] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2965, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3050] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2965, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3051] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2965, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3052] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2967, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3053] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2967, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3054] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2967, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3055] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2969, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3056] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2969, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3057] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2969, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3058] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3059] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3060] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3061] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 11, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3062] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3063] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3064] Ref(ForeignDataWrapperGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3065] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3066] Ref(HandlerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3067] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3068] Ref(ValidatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3069] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3070] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2985, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3071] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2985, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3072] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2985, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3073] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2985, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3074] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2987, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3075] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2994, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3076] Ref(CreateForeignTableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2994, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3077] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2994, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3078] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2994, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3079] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3080] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2999, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3081] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3082] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3006, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3083] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3006, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3084] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3006, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3085] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3006, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3086] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3008, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3087] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3008, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3088] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3008, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3089] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3090] Ref(ForeignTableColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3091] Ref(ForeignTableTableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3009, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3092] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3011, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3093] Ref(InheritsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3011, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3094] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3014, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3095] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3016, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3096] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3016, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3097] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3018, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3098] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3018, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3099] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3018, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3100] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3018, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3101] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3024, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3102] Ref(CreateForeignTableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3024, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3103] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3024, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3104] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3024, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3105] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3029, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3106] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3029, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3107] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3029, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3108] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3029, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3109] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3110] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3034, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3111] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3112] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3039, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3113] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3039, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3114] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3115] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3116] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3041, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3117] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3118] Ref(ForeignTableColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3119] Ref(ForeignTableTableConstraintSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3120] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3121] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3047, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3122] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3047, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3123] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3047, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3124] Ref(PartitionBoundSpecSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3047, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3125] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3047, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3126] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3049, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3127] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3049, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3128] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3049, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3129] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3049, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3130] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3131] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3132] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3133] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3134] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3135] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3136] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3137] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3058, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3138] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3060, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3139] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3060, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3140] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3063, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3141] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3065, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3142] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3065, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3143] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3068, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3144] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3145] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3146] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3147] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3074, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3148] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3074, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3149] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3076, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3150] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3076, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3151] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3076, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3152] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3078, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3153] Ref(SetofKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3078, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3154] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3078, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3155] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3078, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3156] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3078, child_count: 15, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3157] Sequence(15 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3093, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3158] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3093, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3159] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3093, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3160] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3093, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3161] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3093, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3162] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3095, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3163] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3095, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3164] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3095, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3165] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3095, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3166] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3095, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3167] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3095, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3168] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3097, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3169] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3097, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3170] Ref(IndexAccessMethodSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3097, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3171] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3100, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3172] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3102, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3173] Ref(IndexElementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3102, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3104, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3175] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3104, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3176] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3107, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3177] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3109, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3178] Ref(IndexElementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3109, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3179] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3112, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3180] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3112, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3181] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3112, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3182] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3112, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3183] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3184] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3185] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3114, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3186] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3187] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3116, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3188] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3116, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3189] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3118, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3190] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3118, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3191] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3118, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3192] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3118, child_count: 13, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3193] Sequence(13 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3131, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3194] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3131, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3195] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3131, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3196] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3131, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3197] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3131, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3198] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3131, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3199] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3131, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3200] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3131, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3201] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3202] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3203] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3133, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3204] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3135, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3205] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3135, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3206] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3135, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3207] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3137, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3208] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3137, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3209] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3137, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3210] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3137, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3211] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3141, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3212] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3143, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3213] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3146, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3214] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3146, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3215] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3148, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3216] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3151, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3217] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3153, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3218] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3153, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3219] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3153, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3220] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3153, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3221] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3155, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3222] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3158, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3223] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3160, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3224] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3160, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3225] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3160, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3226] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3160, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3227] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3168, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3228] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3168, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3229] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3168, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3230] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3168, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3231] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3168, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3232] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3168, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3233] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3170, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3234] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3170, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3235] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3236] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3175, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3237] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3178, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3238] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3178, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3239] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3178, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3240] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3180, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3241] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3180, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3242] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3183, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3243] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3244] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3245] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3246] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3185, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3247] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3189, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3248] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3189, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3249] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3189, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3250] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3190, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3251] RegexParser
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3190, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3252] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3193, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3253] Delimited(6 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3195, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3254] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3201, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3255] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3204, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3256] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3257] Ref(LeftargKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3258] Ref(RightargKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3259] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3260] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3206, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3261] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3209, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3262] Ref(CommutatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3209, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3263] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3209, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3264] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3210, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3265] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3210, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3266] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3267] Ref(NegatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3268] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3213, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3269] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3214, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3270] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3214, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3271] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3217, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3272] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3220, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3273] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3220, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3274] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3220, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3275] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3276] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3277] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3278] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3279] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3280] Ref(HashesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3281] Ref(MergesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3222, child_count: 10, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3282] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3232, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3283] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3232, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3284] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3232, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3285] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3232, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3286] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3232, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3287] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3232, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3288] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3234, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3289] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3234, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3290] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3236, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3291] Ref(PermissiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3236, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3292] Ref(RestrictiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3236, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3293] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3238, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3294] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3238, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3295] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3243, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3296] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3243, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3297] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3243, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3298] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3243, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3299] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3243, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3300] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3243, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3301] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3245, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3302] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3245, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3303] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3247, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3304] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3305] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3306] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3307] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3308] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3309] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3252, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3310] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3311] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3312] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3313] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3257, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3314] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3315] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3316] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3260, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3317] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3263, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3318] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3263, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3319] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3320] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3269, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3321] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3322] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3323] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3324] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3325] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3326] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3274, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3327] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3274, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3328] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3274, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3329] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3274, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3330] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3276, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3331] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3332] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3333] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3334] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3335] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3336] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3337] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3338] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3283, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3339] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3285, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3340] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3285, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3341] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3285, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3342] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3289, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3343] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3289, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3344] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3345] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3346] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3347] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3291, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3348] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3293, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3349] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3293, child_count: 16, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3350] AnySetOf(16 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3309, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3351] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3311, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3352] Ref(SuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3311, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3353] Ref(NosuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3311, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3354] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3355] Ref(CreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3356] Ref(NocreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3357] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3358] Ref(CreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3359] Ref(NocreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3315, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3360] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3317, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3361] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3317, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3362] Ref(NoinheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3317, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3363] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3319, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3364] Ref(LoginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3319, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3365] Ref(NologinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3319, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3366] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3321, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3367] Ref(ReplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3321, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3368] Ref(NoreplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3321, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3369] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3323, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3370] Ref(BypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3323, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3371] Ref(NobypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3323, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3372] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3373] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3374] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3375] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3376] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3329, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3377] Ref(EncryptedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3329, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3378] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3329, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3379] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3331, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3380] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3331, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3381] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3331, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3382] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3383] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3384] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3385] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3386] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3337, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3387] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3337, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3388] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3337, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3389] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3337, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3390] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3391] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3392] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3393] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3394] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3342, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3395] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3342, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3396] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3342, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3397] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3344, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3398] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3344, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3399] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3344, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3400] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3346, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3401] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3346, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3402] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3346, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3403] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3348, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3404] Ref(SysidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3348, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3405] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3348, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3406] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3352, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3407] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3352, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3408] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3352, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3409] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3352, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3410] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3354, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3411] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3357, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3412] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3357, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3413] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3357, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3414] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3357, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3415] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3357, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3416] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3365, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3417] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3367, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3418] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3367, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3419] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3367, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3420] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3371, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3421] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3371, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3422] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3371, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3423] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3371, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3424] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3371, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3425] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3373, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3426] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3376, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3427] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3376, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3428] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3376, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3429] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3376, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3430] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3378, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3431] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3378, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3432] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3378, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3433] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3380, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3434] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3383, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3435] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3383, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3436] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3383, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3437] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3383, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3438] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3385, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3439] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3385, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3440] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3385, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3441] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3389, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3442] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3389, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3443] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3389, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3444] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3389, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3445] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3389, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3446] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3447] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3448] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3449] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3393, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3450] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3393, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3451] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3395, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3452] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3395, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3453] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3395, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3454] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3455] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3456] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3457] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3400, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3458] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3400, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3459] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3400, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3460] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3461] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3462] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3463] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3464] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3406, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3465] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3406, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3466] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3407, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3467] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3407, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3468] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3469] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3470] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3471] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3472] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3416, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3473] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3474] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3475] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3418, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3476] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3420, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3477] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3420, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3478] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3420, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3479] Ref(ForeignDataWrapperGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3420, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3480] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3420, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3481] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3420, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3482] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3428, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3483] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3428, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3484] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3428, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3485] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3430, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3486] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3430, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3487] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3430, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3488] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3489] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3435, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3490] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3439, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3491] Ref(DependenciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3439, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3492] Ref(McvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3439, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3493] Ref(NdistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3439, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3494] Ref(CorrelationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3439, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3495] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3439, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3496] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3441, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3497] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3443, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3498] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3443, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3499] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3443, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3500] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3443, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3501] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3443, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3502] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3451, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3503] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3451, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3504] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3451, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3505] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3451, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3506] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3451, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3507] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3451, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3508] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3451, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3509] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3453, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3510] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3453, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3511] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3455, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3512] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3455, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3513] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3455, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3514] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3515] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3464, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3516] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3517] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3468, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3518] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3519] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3520] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3521] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3522] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3523] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3524] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3525] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3470, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3526] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3475, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3527] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3478, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3528] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3480, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3529] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3480, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3530] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3531] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3532] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3482, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3533] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3484, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3534] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3535] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3486, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3536] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3489, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3537] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3491, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3538] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3493, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3539] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3493, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3540] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3495, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3541] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3495, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3542] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3497, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3543] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3497, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3544] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3497, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3545] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3546] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3547] Ref(OidsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3499, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3548] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3502, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3549] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3502, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3550] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3502, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3551] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3552] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3553] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3554] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3507, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3555] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3509, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3556] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3509, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3557] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3509, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3558] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3509, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3559] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3511, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3560] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3511, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3561] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3511, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3562] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3511, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3563] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3515, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3564] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3517, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3565] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3520, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3566] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3520, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3567] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3522, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3568] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3569] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3527, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3570] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3527, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3571] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3527, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3572] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3527, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3573] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3574] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3532, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3575] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3534, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3576] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3534, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3577] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3534, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3578] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3534, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3579] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3541, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3580] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3541, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3581] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3543, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3582] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3545, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3583] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3547, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3584] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3547, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3585] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3547, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3586] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3547, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3587] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3547, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3588] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3547, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3589] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3547, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3590] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3547, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3591] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3550, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3592] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3552, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3593] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3555, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3594] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3557, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3595] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3560, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3596] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3560, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3597] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3560, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3598] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3563, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3599] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3563, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3600] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3563, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3601] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3564, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3602] Ref(LikeOptionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3564, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3603] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3566, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3604] Ref(InheritsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3566, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3605] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3569, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3606] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3571, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3607] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3571, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3608] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3574, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3609] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3574, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3610] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3574, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3611] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3577, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3612] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3579, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3613] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3581, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3614] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3584, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3615] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3584, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3616] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3586, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3617] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3586, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3618] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3586, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3619] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3587, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3620] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3587, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3621] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3587, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3622] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3592, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3623] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3592, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3624] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3592, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3625] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3592, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3626] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3595, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3627] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3597, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3628] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3599, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3629] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3602, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3630] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3602, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3631] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3604, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3632] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3604, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3633] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3604, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3634] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3605, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3635] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3605, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3636] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3605, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3637] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3638] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3610, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3639] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3610, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3640] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3610, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3641] Ref(PartitionBoundSpecSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3610, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3642] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3610, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3643] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3615, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3644] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3619, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3645] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3619, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3646] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3619, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3647] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3622, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3648] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3622, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3649] Ref(ListKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3622, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3650] Ref(HashKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3622, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3651] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3625, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3652] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3626, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3653] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3628, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3654] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3630, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3655] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3632, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3656] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3632, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3657] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3632, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3658] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3634, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3659] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3636, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3660] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3636, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3661] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3636, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3662] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3636, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3663] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3664] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3665] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3666] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3640, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3667] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3668] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3642, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3669] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3642, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3670] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3644, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3671] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3644, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3672] Ref(OidsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3644, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3673] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3647, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3674] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3647, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3675] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3647, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3676] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3650, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3677] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3652, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3678] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3652, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3679] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3652, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3680] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3681] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3682] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3654, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3683] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3654, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3684] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3685] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3656, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3686] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3656, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3687] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3662, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3688] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3662, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3689] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3662, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3690] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3662, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3691] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3662, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3692] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3662, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3693] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3665, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3694] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3667, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3695] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3670, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3696] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3670, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3697] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3670, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3698] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3670, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3699] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3673, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3700] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3673, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3701] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3673, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3702] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3673, child_count: 11, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3703] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3684, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3704] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3684, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3705] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3684, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3706] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3684, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3707] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3684, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3708] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3684, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3709] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3687, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3710] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3687, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3711] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3687, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3712] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3713] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3714] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3715] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3691, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3716] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3717] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3718] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3719] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3720] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3697, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3721] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3697, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3722] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3723] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3699, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 2, _padding: 0 }, // [3724] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3701, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3725] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3701, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3726] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3701, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3727] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3701, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3728] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3701, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3729] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3701, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3730] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3701, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3731] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3706, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3732] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3708, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3733] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3708, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3734] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3708, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3735] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3710, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3736] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3712, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3737] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3712, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3738] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3712, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3739] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3714, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3740] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3714, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3741] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3716, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3742] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3718, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3743] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3718, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3744] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3718, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3745] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3720, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3746] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3720, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3747] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3720, child_count: 6, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3748] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3726, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3749] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3726, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3750] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3728, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3751] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3728, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3752] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3728, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3753] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3728, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3754] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3728, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3755] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3728, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3756] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3757] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3734, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3758] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3734, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3759] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3734, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3760] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3734, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3761] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3734, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3762] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3734, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3763] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3764] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3737, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3765] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3766] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3739, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3767] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3739, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3768] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3739, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3769] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3741, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3770] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3741, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3771] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3744, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3772] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3744, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3773] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3747, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3774] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3747, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3775] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3749, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3776] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3749, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3777] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3749, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3778] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3749, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3779] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3754, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3780] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3754, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3781] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3754, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3782] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3754, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3783] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3756, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3784] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3756, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3785] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3758, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3786] Ref(EnumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3758, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3787] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3758, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3788] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3761, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3789] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3763, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3790] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3763, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3791] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3766, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3792] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3766, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3793] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3766, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3794] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3766, child_count: 7, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3795] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3773, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3796] Ref(CreateUserMappingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3773, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3797] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3773, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3798] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3773, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3799] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3800] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3801] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3802] Ref(SessionInformationUserFunctionsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3803] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3804] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3805] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3806] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3779, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3807] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3779, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3808] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3779, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3809] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3779, child_count: 11, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3810] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3790, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3811] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3790, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3812] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3790, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3813] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3790, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3814] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3790, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3815] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3790, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3816] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3790, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3817] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3790, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3818] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3792, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3819] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3792, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3820] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3792, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3821] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3792, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3822] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3794, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3823] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3796, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3824] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3799, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3825] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3799, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3826] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3799, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3827] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3799, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3828] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3799, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3829] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3799, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3830] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3799, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3831] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3799, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3832] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3799, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3833] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3799, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3834] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3799, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3835] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3801, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3836] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3837] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3838] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3839] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3840] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3841] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3842] StringParser("CURRENT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3843] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3844] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3845] StringParser("CURRENT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3846] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3847] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3848] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3849] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3850] StringParser("CURVEPOLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3851] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3852] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3853] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3806, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 1, _padding: 0 }, // [3854] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3808, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3855] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3808, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3856] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3808, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3857] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3808, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3858] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3808, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3859] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3810, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3860] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3810, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3861] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3862] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3863] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 3814, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3864] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3865] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3866] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3816, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3867] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3822, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3868] Ref(WellKnownTextGeometrySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3822, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3869] Ref(DateTimeTypeIdentifier)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3822, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3870] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3822, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3871] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3822, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3872] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3823, child_count: 38, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3873] OneOf(38 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3874] Ref(SmallintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3875] Ref(IntegerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3876] Ref(IntKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3877] Ref(Int2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3878] Ref(Int4KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3879] Ref(Int8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3880] Ref(BigintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3881] Ref(Float4KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3882] Ref(Float8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3883] Ref(RealKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3861, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3884] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3863, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3885] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3863, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3886] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3863, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3887] Ref(SmallserialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3863, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3888] Ref(SerialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3863, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3889] Ref(Serial2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3863, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3890] Ref(Serial4KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3863, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3891] Ref(Serial8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3863, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3892] Ref(BigserialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3863, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3865, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3894] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3866, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3895] Ref(FloatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3866, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3896] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3866, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3897] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3868, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3898] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3870, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3899] Ref(DecimalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3870, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3900] Ref(NumericKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3870, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3901] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3870, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3902] Ref(MoneyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3870, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3903] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3872, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3904] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3874, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3905] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3880, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3906] Ref(BpcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3880, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3907] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3880, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3908] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3882, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3909] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3882, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3910] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3882, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3911] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3882, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3912] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3913] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3914] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3915] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3916] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3917] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3918] Ref(ByteaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3919] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3886, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3920] Ref(BooleanKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3886, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3921] Ref(BoolKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3886, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3922] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3893, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3923] Ref(PointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3893, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3924] Ref(LineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3893, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3925] Ref(LsegKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3893, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3926] Ref(BoxKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3893, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3927] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3893, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3928] Ref(PolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3893, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3929] Ref(CircleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3893, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3930] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3897, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3931] Ref(CidrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3897, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3932] Ref(InetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3897, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3933] Ref(MacaddrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3897, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3934] Ref(Macaddr8KeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3897, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3935] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3899, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3936] Ref(TsvectorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3899, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3937] Ref(TsqueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3899, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3938] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3902, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3939] Ref(BitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3902, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3940] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3941] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3942] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3943] Ref(UuidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3944] Ref(XmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3945] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3946] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3947] Ref(JsonbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3948] Ref(Int4rangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3949] Ref(Int8rangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3950] Ref(NumrangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3951] Ref(TsrangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3952] Ref(TstzrangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3953] Ref(DaterangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3954] Ref(Pg_lsnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3955] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3907, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3956] Ref(VectorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3907, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3957] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3907, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3958] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3909, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3959] Ref(DatatypeIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3909, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3960] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3909, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3961] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3912, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3962] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3963] Ref(ArrayTypeSuffixSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3964] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3965] Ref(SizedArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3966] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3967] MultiStringParser(0 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3968] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3913, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3969] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3914, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3970] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3917, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3971] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3919, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3972] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3921, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3973] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3921, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3974] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3921, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3975] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3923, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3976] Ref(DateTimeTypeIdentifier)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3923, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3977] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3923, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3978] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3926, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3979] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3926, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3980] Ref(TimeWithTZGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3926, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3981] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3928, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3982] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3931, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3983] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3931, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3984] Ref(TimetzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3931, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3985] Ref(TimestamptzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3931, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3986] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3987] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3988] StringParser("DATERANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3989] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3990] MultiStringParser(24 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3991] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3992] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3993] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3994] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3995] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3996] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3997] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3998] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3999] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4000] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4001] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3934, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4002] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3937, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4003] Ref(DeallocateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3937, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4004] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3937, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4005] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3939, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4006] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3939, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4007] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3939, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4008] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3939, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4009] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3939, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4010] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3939, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4011] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3939, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4012] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4013] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4014] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4015] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4016] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4017] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4018] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4019] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3943, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4020] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3943, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4021] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3943, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4022] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3945, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4023] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3945, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4024] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3947, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4025] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3947, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4026] Ref(DefinitionArgumentValueGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3947, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4027] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3950, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4028] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4029] Ref(DefinitionParameterSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4030] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4031] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3952, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4032] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3952, child_count: 10, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4033] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3962, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4034] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3962, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4035] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3962, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4036] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3962, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4037] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3962, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4038] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3962, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4039] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3962, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4040] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4041] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4042] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3968, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4043] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4044] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4045] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4046] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3970, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4047] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3972, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4048] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3976, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4049] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3976, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4050] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3976, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4051] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3976, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4052] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3976, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4053] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3976, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4054] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3980, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4055] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3980, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4056] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3982, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4057] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3982, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4058] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3984, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4059] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4060] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4061] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4062] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4063] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4064] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4065] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4066] StringParser("DEPENDENCIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4067] StringParser("DEPENDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4068] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4069] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4070] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3986, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4071] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4072] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4073] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4074] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4075] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4076] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4077] StringParser("DETACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4078] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4079] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4080] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4081] StringParser("DISABLE_PAGE_SKIPPING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3989, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4082] StringParser("DISCARD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4083] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3991, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4084] Ref(DiscardKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3991, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4085] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4086] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4087] Ref(PlansKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4088] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4089] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4090] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4091] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4092] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4093] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4094] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4095] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4096] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4097] StringParser("DO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4098] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3998, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4099] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3998, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4100] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4000, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4101] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4002, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4102] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4002, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4103] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4002, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4104] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4105] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4106] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4107] StringParser("DOCUMENT")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4108] TypedParser("dollar_numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4109] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4110] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4111] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4004, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4112] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4113] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4114] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4115] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4116] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4011, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4117] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4013, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4118] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4013, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4119] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4016, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4120] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4016, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4121] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4016, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4122] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4016, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4123] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4016, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4124] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4018, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4125] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4018, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4126] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4018, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4127] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4128] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4129] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4130] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4131] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4132] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4133] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4134] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4028, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4135] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4028, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4136] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4033, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4137] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4033, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4138] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4033, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4139] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4033, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4140] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4033, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4141] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4033, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4142] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4038, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4143] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4038, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4144] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4038, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4145] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4038, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4146] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4038, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4147] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4040, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4148] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4040, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4149] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4043, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4150] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4043, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4151] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4048, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4152] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4048, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4153] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4048, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4154] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4048, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4155] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4156] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4157] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4050, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4158] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4159] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4160] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4161] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4162] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4163] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4055, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4164] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4061, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4165] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4061, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4166] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4061, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4167] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4061, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4168] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4061, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4169] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4063, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4170] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4063, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4171] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4063, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4172] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4068, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4173] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4068, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4174] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4068, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4175] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4068, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4176] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4070, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4178] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4179] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4180] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4181] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4078, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4182] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4078, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4183] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4078, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4184] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4078, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4185] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4078, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4186] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4080, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4187] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4080, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4188] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4080, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4189] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4080, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4190] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4191] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4192] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4193] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4194] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4195] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4088, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4196] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4088, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4197] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4088, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4198] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4199] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4200] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4201] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4202] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4203] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4097, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4204] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4097, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4205] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4097, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4206] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4097, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4207] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4099, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4208] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4103, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4209] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4103, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4210] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4103, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4211] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4103, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4212] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4103, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4213] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4103, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4214] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4110, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4215] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4110, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4216] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4110, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4217] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4110, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4218] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4110, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4219] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4110, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4220] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4110, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4221] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4110, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4222] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4223] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4224] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4225] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4115, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4226] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4117, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4227] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4119, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4228] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4119, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4229] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4119, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4230] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4231] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4121, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4232] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4121, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4233] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4126, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4234] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4126, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4235] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4126, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4236] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4126, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4237] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4128, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4238] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4128, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4239] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4128, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4240] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4241] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4242] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4243] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4244] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4132, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4245] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4137, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4246] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4137, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4247] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4137, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4248] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4137, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4249] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4137, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4250] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4137, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4251] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4142, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4252] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4142, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4253] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4142, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4254] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4142, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4255] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4144, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4256] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4144, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4257] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4144, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4258] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4259] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4260] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4261] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4149, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4262] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4151, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4263] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4151, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4264] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4153, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4265] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4153, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4266] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4153, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4267] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4158, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4268] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4158, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4269] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4158, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4270] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4158, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4271] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4158, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4272] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4160, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4273] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4160, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4274] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4160, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4275] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4166, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4276] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4166, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4277] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4166, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4278] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4166, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4279] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4166, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4280] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4168, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4281] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4168, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4282] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4168, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4283] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4175, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4284] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4175, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4285] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4175, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4286] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4175, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4287] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4175, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4288] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4175, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4289] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4175, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4290] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4175, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4291] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4182, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4292] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4182, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4293] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4182, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4294] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4182, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4295] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4182, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4296] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4182, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4297] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4182, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4298] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4182, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4299] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4187, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4300] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4187, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4301] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4187, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4302] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4187, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4303] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4189, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4304] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4189, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4305] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4189, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4306] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4307] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4308] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4309] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4310] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4311] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4198, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4312] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4198, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4313] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4198, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4314] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4198, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4315] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4200, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4316] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4200, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4317] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4200, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4318] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4200, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4319] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4200, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4320] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4200, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4321] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4204, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4322] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4204, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4323] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4204, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4324] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4204, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4325] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4204, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4326] StringParser("EMPTY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4204, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4327] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4206, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4328] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4329] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4330] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4331] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4332] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4333] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4334] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4335] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4336] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4337] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4338] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4339] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4340] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4341] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4342] StringParser("EVENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4343] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4344] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4345] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4208, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4346] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4211, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4347] Ref(IndexElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4211, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4348] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4211, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4349] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4211, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4350] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4211, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4351] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4211, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4352] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4353] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4354] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4214, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4355] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4217, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4356] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4219, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4357] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4219, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4358] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4219, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4359] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4219, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4360] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4219, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4361] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4219, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4362] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4221, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4363] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4223, child_count: 9, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4364] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4365] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4366] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4367] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4368] Ref(CostsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4369] Ref(SettingsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4370] Ref(BuffersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4371] Ref(WalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4372] Ref(TimingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4373] Ref(SummaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4374] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4375] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4376] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4234, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4377] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4238, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4378] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4238, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4379] Ref(XmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4238, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4380] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4238, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4381] Ref(YamlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4238, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4382] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4241, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4383] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4241, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4384] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4243, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4385] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4245, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4386] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4247, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4387] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4247, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4388] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4247, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4389] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4247, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4390] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4250, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4391] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4252, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4392] Ref(ExplainOptionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4252, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4393] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4256, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4394] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4256, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4395] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4256, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4396] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4256, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4397] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4256, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4398] StringParser("EXPRESSION")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4256, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4399] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4256, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4400] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4258, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4401] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4258, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4402] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4259, child_count: 10, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4403] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4269, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4404] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4269, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4405] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4271, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4406] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4271, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4407] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4271, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4408] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4271, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4409] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4271, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4410] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4274, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4411] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4274, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4412] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4274, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4413] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4274, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4414] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4274, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4415] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4274, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4416] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4274, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4417] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4279, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4418] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4279, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4419] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4279, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4420] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4279, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4421] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4279, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4422] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4279, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4423] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4281, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4424] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4281, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4425] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4281, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4426] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4285, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4427] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4286, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4428] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4287, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4429] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4287, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4430] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4287, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4431] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4287, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4432] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4287, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4433] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4289, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4434] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4289, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4435] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4290, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4436] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4291, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4437] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4293, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4438] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4296, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4439] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4296, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4440] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4296, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4441] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4296, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4442] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4296, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4443] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4298, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4444] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4299, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4445] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4300, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4446] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4300, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4447] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4300, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4448] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4302, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4449] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4302, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4450] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4304, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 1, _padding: 0 }, // [4451] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4307, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4452] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4309, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4453] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4309, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4454] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4312, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4455] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4312, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4456] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4314, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4457] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4316, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4458] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4316, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4459] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4316, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4460] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4317, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4461] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4317, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4462] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4317, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4463] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4317, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4464] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4319, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4465] Ref(FullTextSearchOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4319, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4466] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4319, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4467] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4321, child_count: 9, min_times: 0, first_terminator_idx: 40, terminator_count: 1, _padding: 0 }, // [4468] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4330, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4469] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4330, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4470] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 4330, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4471] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4333, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4472] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4337, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4473] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4337, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4474] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4337, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4475] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4337, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4476] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4339, child_count: 5, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4477] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4344, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4478] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4344, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4479] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4344, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4480] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4344, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4481] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4344, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4482] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4344, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4483] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4344, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4484] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4344, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4485] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4346, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4486] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4348, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4487] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4348, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4488] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4348, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4489] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4351, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4490] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4353, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4491] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4353, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4492] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4355, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4493] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4355, child_count: 5, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4494] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4360, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4495] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4360, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4496] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4360, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4497] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4360, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4498] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4360, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4499] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4360, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4500] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4360, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4501] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4360, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4502] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4360, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4503] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4360, child_count: 7, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4504] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4367, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4505] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4367, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4506] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4367, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4507] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4367, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4508] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4367, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4509] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4367, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4510] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4367, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4511] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4367, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4512] StringParser("EXTENDED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4367, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4513] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4367, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4514] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4370, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4515] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4370, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4516] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4372, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4517] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4372, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4518] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4374, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4519] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4377, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4520] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4377, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4521] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4379, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4522] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4379, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4523] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4379, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4524] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4379, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4525] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4379, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4526] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4379, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4527] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4379, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4528] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4379, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4529] StringParser("FAMILY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4379, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4530] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4384, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4531] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4384, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4532] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4386, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4533] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4386, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4534] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4386, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4535] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4388, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4536] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4388, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4537] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4389, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4538] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4389, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4539] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4391, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4540] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4391, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4541] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4391, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4542] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4393, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4543] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4393, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4544] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4395, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4545] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4395, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4546] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4395, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4547] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4395, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4548] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4395, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4549] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4395, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4550] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4399, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4551] Ref(PostgresCopyStdinDataStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4399, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4552] Ref(PsqlCopyMetaCommandStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4399, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4553] Ref(PsqlSetMetaCommandStatementSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 4399, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4554] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4401, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4555] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4401, child_count: 1, min_times: 1, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4556] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4402, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4557] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4402, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4558] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4402, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4559] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4404, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4560] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4404, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4561] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4407, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4562] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4563] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4564] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4565] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4566] StringParser("FINALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4567] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4568] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4569] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4570] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4571] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4572] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4409, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4573] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4413, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4574] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4413, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4575] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4417, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4576] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4417, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4577] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4578] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4579] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4580] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4581] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4420, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4582] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4583] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4422, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4584] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4422, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4585] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4424, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4586] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4424, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4587] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4426, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4588] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4426, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4589] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4428, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4590] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4428, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4591] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4430, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4592] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4430, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4593] Ref(LockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4430, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4594] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4430, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4595] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4430, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4596] StringParser("FORCE_NOT_NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4430, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4597] StringParser("FORCE_NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4430, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4598] StringParser("FORCE_QUOTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4430, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4599] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4433, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4600] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4433, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4601] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4433, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4602] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4433, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4603] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4435, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4604] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4435, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4605] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4435, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4606] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4435, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4607] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4437, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4608] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4439, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4609] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4439, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4610] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4439, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4611] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4443, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4612] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4445, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4613] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4445, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4614] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4445, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4615] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4448, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4616] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4448, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4617] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4451, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4618] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4451, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4619] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4620] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4453, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4621] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4453, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4622] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4455, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4623] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4455, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4624] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4460, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4625] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4460, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4626] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4460, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4627] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4460, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4628] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4460, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4629] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4460, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4630] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4465, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4631] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4465, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4632] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4465, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4633] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4465, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4634] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4465, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4635] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4465, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4636] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4467, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4637] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4469, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4638] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4469, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4639] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4469, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4640] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4472, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4641] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4472, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4642] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4475, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4643] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4475, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4644] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4645] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4646] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4647] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4648] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4649] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4480, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4650] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4480, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4651] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4482, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4652] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4484, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4653] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4486, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4654] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4486, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4655] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4486, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4656] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4488, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4657] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4492, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4658] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4492, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4659] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4492, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4660] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4492, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4661] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4492, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4662] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4494, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4663] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4494, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4664] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4494, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4665] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4498, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4666] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4498, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4667] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4498, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4668] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4669] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4670] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4504, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4671] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4506, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4672] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4506, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4673] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4506, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4674] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4506, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4675] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4506, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4676] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4508, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4677] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4508, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4678] Ref(OthersKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4508, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4679] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4680] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4681] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4682] Ref(GroupsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4511, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4683] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4511, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4684] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4511, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4685] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4686] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4513, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4687] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4513, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4688] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4515, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4689] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4515, child_count: 13, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4690] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4528, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4691] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4528, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4692] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4528, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4693] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4530, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4694] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4530, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4695] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4530, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4696] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4697] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4698] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4699] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4700] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4701] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4702] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4703] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4704] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4705] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4706] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4707] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4532, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4708] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4534, child_count: 7, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4709] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4541, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4710] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4541, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4711] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4543, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4712] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4546, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4713] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4546, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4714] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4546, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4715] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4547, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4716] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4551, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4717] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4551, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4718] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4551, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4719] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4551, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4720] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4551, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4721] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4554, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4722] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4554, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4723] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4554, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4724] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4554, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4725] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4554, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4726] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4554, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4727] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4557, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4728] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4559, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4729] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4560, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4730] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4560, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4731] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4562, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4732] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4565, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4733] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4571, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 2, _padding: 0 }, // [4734] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4574, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4735] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4574, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4736] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4574, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4737] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4577, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4738] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4577, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4739] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4579, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4740] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4579, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4741] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4579, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4742] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4581, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4743] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4581, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4744] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4581, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4745] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4581, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 2, _padding: 0 }, // [4746] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4582, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4747] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4583, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4748] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4585, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4749] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4585, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4750] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4585, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4751] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4587, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4752] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4587, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4753] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4587, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4754] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4755] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4756] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4757] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4758] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4759] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4760] TypedParser("full_text_search_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4589, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4761] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4589, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4762] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4592, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4763] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4592, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4764] Ref(NamedArgumentSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4592, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4765] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4594, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4766] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4594, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4767] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4596, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4768] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4596, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4769] Ref(NamedArgumentSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4596, child_count: 14, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4770] AnyNumberOf(14 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4610, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4771] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4610, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4772] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4773] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4615, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4774] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4615, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4775] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4618, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4776] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4618, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4777] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4618, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4778] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4618, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4779] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4622, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4780] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4622, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4781] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4623, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4782] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4623, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4783] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4623, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4784] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4623, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4785] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4625, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4786] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4627, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4787] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4627, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4788] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4627, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4789] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4629, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4790] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4631, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4791] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4631, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4792] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4631, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4793] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4633, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4794] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4633, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4795] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4633, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4796] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4635, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4797] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4635, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4798] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4635, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4799] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4637, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4800] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4637, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4801] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4639, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4802] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4639, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4803] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4641, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4804] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4641, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4805] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4641, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4806] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4643, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4807] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4643, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4808] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4643, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4809] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4646, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4810] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4650, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4811] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4650, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4812] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4650, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4813] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4650, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4814] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4650, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4815] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4650, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4816] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4654, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4817] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4654, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4818] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4654, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4819] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4654, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4820] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4654, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4821] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4654, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4822] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4654, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4823] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4654, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4824] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4656, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4825] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4659, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4826] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4659, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4827] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4661, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4828] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4661, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4829] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4661, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4830] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4661, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4831] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4662, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4832] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4833] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4665, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4834] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4667, child_count: 15, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4835] AnyNumberOf(15 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4682, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4836] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4682, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4837] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4838] Ref(TransformKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4839] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4840] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4841] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4842] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4843] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4689, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4844] Ref(ImmutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4689, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4845] Ref(StableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4689, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4846] Ref(VolatileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4689, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4847] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4691, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4848] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4691, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4849] Ref(LeakproofKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4691, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4850] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4694, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4851] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4698, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4852] Ref(CalledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4698, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4853] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4698, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4854] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4698, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4855] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4698, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4856] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4703, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4857] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4703, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4858] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4703, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4859] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4703, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4860] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4703, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4861] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4703, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4862] Ref(StrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4703, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4863] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4706, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4864] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4706, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4865] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4706, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4866] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4708, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4867] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4708, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4868] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4708, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4869] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4710, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4870] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4710, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4871] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4872] Ref(UnsafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4873] Ref(RestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4874] Ref(SafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4713, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4875] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4715, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4876] Ref(CostKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4715, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4877] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4715, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4878] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4717, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4879] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4717, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4880] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4717, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4881] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4719, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4882] Ref(SupportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4719, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4883] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4719, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4884] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4722, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4885] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4722, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4886] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4722, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4887] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4724, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4888] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4726, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4889] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4728, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4890] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4728, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4891] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4728, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4892] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4730, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4893] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4732, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4894] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4732, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4895] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4732, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4896] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4734, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4897] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4734, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4898] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4734, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4899] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4736, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4900] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4736, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4901] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4738, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4902] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4738, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4903] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4741, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4904] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4741, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4905] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4741, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4906] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4741, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4907] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4743, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4908] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4743, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4909] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4743, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4910] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4749, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4911] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4749, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4912] Ref(AtomicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4749, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4913] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4753, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4914] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4755, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4915] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4755, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4916] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4755, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4917] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4757, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4918] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4757, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4919] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4757, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4920] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4759, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4921] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4762, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4922] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4762, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4923] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4762, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4924] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4762, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4925] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4762, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4926] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4765, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4927] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4765, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4928] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4765, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4929] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4765, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4930] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4765, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4931] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4767, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4932] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4767, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4933] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4770, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4934] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4772, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4935] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4772, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4936] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4772, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4937] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4772, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4938] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4774, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 1, _padding: 0 }, // [4939] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4775, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4940] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4777, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4941] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4777, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4942] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4777, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [4943] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4777, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 1, _padding: 0 }, // [4944] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4779, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [4945] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4779, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [4946] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4779, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [4947] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4779, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4948] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4782, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4949] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4786, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4950] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4786, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4951] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4786, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4952] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4786, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4953] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4786, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4954] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4788, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4955] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4788, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4956] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4791, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4957] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4791, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4958] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4795, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4959] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4795, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4960] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4795, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4961] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4795, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4962] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4795, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4963] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4797, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4964] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4797, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4965] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4797, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4966] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4799, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4967] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4802, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4968] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4802, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4969] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4802, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4970] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4802, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4971] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4802, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4972] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4805, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4973] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4807, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4974] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4807, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4975] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4810, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4976] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4812, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4977] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4812, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4978] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4812, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4979] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4812, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4980] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4814, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4981] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4816, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4982] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4817, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4983] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4984] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4985] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4986] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4987] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4988] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4989] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4990] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4991] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4992] StringParser("GEOGRAPHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4993] StringParser("GEOMETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4994] StringParser("GEOMETRYCOLLECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4995] StringParser("GEOMETRYCOLLECTIONM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4996] StringParser("GEOMETRYCOLLECTIONZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4997] StringParser("GEOMETRYCOLLECTIONZM")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4998] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [4999] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4820, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5000] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4820, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5001] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4826, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5002] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4826, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5003] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4830, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5004] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5005] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5006] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5007] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4833, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5008] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4835, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5009] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4835, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5010] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4835, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5011] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4839, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5012] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4839, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5013] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4839, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5014] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4839, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5015] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4839, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5016] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4839, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5017] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4839, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5018] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4839, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5019] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4842, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5020] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4845, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5021] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4845, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5022] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4845, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5023] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4845, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5024] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4848, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5025] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4848, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5026] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4848, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5027] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4848, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5028] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5029] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5030] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5031] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4851, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5032] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5033] Ref(GrantedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4854, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5034] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4854, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5035] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5036] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5037] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5038] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5039] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5040] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4857, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5041] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4859, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5042] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4859, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5043] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4859, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5044] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4859, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5045] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4859, child_count: 5, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5046] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4864, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5047] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4864, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5048] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4864, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 6, _padding: 0 }, // [5049] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4866, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5050] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4872, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5051] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4872, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5052] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4872, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5053] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4872, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5054] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4872, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5055] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4872, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5056] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4874, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5057] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4876, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5058] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4876, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5059] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4876, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5060] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4876, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5061] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4876, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5062] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4876, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5063] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4876, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5064] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4876, child_count: 6, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5065] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4882, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4884, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5067] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4884, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5068] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4884, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5069] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4884, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5070] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4884, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5071] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4884, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5072] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4884, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5073] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4884, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5074] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4884, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5075] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4885, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 1, _padding: 0 }, // [5076] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4887, child_count: 4, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5077] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4891, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5078] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4891, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5079] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4891, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5080] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4891, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5081] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4893, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5082] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4893, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5083] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4893, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5084] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4896, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5085] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4896, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5086] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4896, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5087] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4899, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5088] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4901, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5089] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4903, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5090] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4903, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5091] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4903, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5092] StringParser("GROUPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4903, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5093] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4903, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5094] StringParser("HASH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4903, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5095] StringParser("HASHES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4903, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5096] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4907, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5097] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4907, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5098] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4909, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5099] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4912, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5100] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4912, child_count: 5, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5101] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4917, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5102] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5103] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5104] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5105] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5106] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5107] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5108] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5109] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5110] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5111] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5112] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5113] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5114] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5115] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5116] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5117] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5118] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5119] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5120] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5121] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5122] StringParser("ICU")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5123] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5124] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5125] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5126] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4919, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5127] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4919, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5128] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4921, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5129] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4921, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5130] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4921, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5131] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4921, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5132] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5133] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5134] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5135] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4924, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5136] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4924, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5137] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4926, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5138] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4928, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5139] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4928, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5140] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4928, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5141] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4928, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5142] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4928, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5143] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4928, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5144] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4928, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5145] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4928, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5146] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4931, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5147] Ref(ImportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4931, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5148] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4931, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5149] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4931, child_count: 9, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5150] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4940, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5151] Ref(ImportForeignSchemaGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4940, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5152] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4940, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5153] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4942, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5154] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4944, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5155] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4946, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5156] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4946, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5157] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4946, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5158] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4946, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5159] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4949, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5160] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5161] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5162] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5163] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5164] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5165] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5166] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5167] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5168] StringParser("IMPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5169] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4951, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5170] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4951, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5171] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4954, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5172] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4954, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5173] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4954, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5174] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 4956, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5175] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4959, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5176] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4961, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5177] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4963, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5178] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4963, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5179] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4963, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5180] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4963, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5181] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4963, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5182] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4963, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5183] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4963, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5184] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4963, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5185] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4965, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5186] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4965, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5187] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4967, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5188] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4967, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5189] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4967, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5190] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4971, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4973, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5192] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4973, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5193] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4973, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5194] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4975, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5195] Ref(OperatorClassReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4976, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4978, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5197] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4978, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5198] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4980, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5199] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4980, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5200] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4980, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5201] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4980, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5202] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4982, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5203] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4982, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5204] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4982, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5205] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4984, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5206] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4984, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5207] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4986, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5208] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4986, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5209] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4986, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5210] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4988, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5211] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4991, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5212] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4991, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5213] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4991, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5214] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4994, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5215] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4994, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5216] Ref(IndexElementOptionsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4994, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5217] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4994, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5218] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4997, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5219] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4999, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5220] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4999, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5221] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4999, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5222] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5001, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5223] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5001, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5224] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5001, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5225] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5226] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5227] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5228] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5229] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5230] StringParser("INDEX_CLEANUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5231] StringParser("INDEXES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5232] StringParser("INET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5233] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5234] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5235] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5236] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5237] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5238] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5239] StringParser("INLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5240] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5241] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5242] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5243] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5005, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5244] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5005, child_count: 9, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5245] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5014, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5246] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5014, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5247] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5014, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5248] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5014, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5249] Ref(AsAliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5014, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5250] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5014, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5251] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5017, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5252] Ref(OverridingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5017, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5253] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5019, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5254] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5019, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5255] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5019, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5256] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5019, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5257] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5021, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5258] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5259] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5260] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5261] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5023, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5262] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5027, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5263] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5027, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5264] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5027, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5265] Ref(ConflictTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5027, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5266] Ref(ConflictActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5027, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5267] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5031, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5268] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5031, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5269] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5033, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5270] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5033, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5271] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5035, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5272] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5273] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5274] Ref(AsAliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5275] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5276] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5277] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5278] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5279] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5280] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5281] StringParser("INT4RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5282] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5283] StringParser("INT8RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5284] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5285] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5286] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5287] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5288] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5037, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5289] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5039, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5290] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5039, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5291] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5042, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5292] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5044, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5293] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5044, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5294] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5046, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5295] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5046, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5296] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5046, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5297] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5046, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5298] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5299] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5300] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5050, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5301] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5050, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5302] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5050, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5303] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5050, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5304] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5056, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5305] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5056, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5306] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5056, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5307] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5056, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5308] Ref(HourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5056, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5309] Ref(MinuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5056, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5310] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5056, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5311] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5060, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5312] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5060, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5313] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5063, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5314] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5063, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5315] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5063, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5316] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5063, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5317] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5063, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5318] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5063, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5319] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5063, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5320] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5063, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5321] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5069, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5322] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5069, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5323] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5069, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5324] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5069, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5325] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5069, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5326] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5069, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5327] Ref(JsonTestGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5069, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5328] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5329] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5330] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5331] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5332] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5333] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5334] Ref(IsnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5335] StringParser("IS_SUPERUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5336] StringParser("IS_TEMPLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5337] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5338] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5339] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5073, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5340] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5073, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5341] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5076, child_count: 7, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5342] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5083, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5343] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5083, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5344] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5083, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5345] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5083, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5346] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5084, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5347] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5084, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5348] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5088, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5349] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5088, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5350] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5088, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5351] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5090, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5352] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5090, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5353] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5090, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5354] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5090, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5355] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5096, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5356] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5096, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5357] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5096, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5358] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5096, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5359] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5096, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5360] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5100, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5361] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5100, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5362] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5100, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5363] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5100, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5364] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5101, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5365] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5101, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5366] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5101, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5367] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5105, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5368] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5105, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5369] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5105, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5370] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5371] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5110, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5372] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5110, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5373] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5110, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5374] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5112, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5375] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5112, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5376] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5114, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5377] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5117, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5378] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5117, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5379] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5117, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5380] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5117, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5381] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5117, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5382] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5121, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5383] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5121, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5384] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5124, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5385] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5126, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5386] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5126, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5387] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5126, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5388] TypedParser("json_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5126, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5389] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5129, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5390] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5129, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5391] Ref(JsonTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5129, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5392] Ref(JsonUniqueKeysGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5129, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5393] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5133, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5394] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5133, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5395] Ref(ScalarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5133, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5396] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5133, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5397] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5133, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5398] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5136, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5399] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5138, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5400] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5138, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5401] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5138, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5402] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5138, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5403] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5139, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5404] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5139, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5405] StringParser("JSONB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5139, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5406] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5139, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5407] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5139, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5408] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5139, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5409] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5139, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5410] StringParser("LABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5139, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5411] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5139, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5412] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5141, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5413] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5141, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5414] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5415] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5416] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5417] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5418] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5419] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5420] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5421] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5422] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5423] StringParser("LC_COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5424] StringParser("LC_CTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5425] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5426] StringParser("LEAKPROOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5427] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5428] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5429] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5430] StringParser("LEFTARG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5431] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5143, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5432] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5145, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5433] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5145, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5434] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5145, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5435] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5145, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5436] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5145, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5437] StringParser("LIBC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5145, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5438] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5148, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5439] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5440] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5441] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5150, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5442] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5150, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5443] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5152, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5444] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5152, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5445] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5152, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5446] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5155, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5447] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5155, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5448] Ref(IlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5155, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5449] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5157, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5450] Ref(SimilarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5157, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5451] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5157, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5452] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5157, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5453] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5157, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5454] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5159, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5455] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5161, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5456] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5161, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5457] Ref(ExcludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5161, child_count: 9, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5458] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5459] Ref(CommentsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5460] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5461] Ref(DefaultsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5462] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5463] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5464] Ref(IndexesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5465] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5466] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5467] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 5, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5468] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5175, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5469] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5175, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5470] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5471] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5180, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5472] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5473] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5474] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5475] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5183, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5476] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5185, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5477] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5187, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5478] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5187, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5479] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5189, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5480] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5189, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5481] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5189, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5482] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5483] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5484] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5485] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5486] StringParser("LINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5487] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5488] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5489] StringParser("LINESTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5490] StringParser("LINESTRINGM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5491] StringParser("LINESTRINGZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5492] StringParser("LINESTRINGZM")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5493] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5494] StringParser("LIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5191, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5495] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5191, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5496] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5193, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5497] Ref(ListenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5193, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5498] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5193, child_count: 11, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5499] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5500] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5501] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5502] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5503] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5504] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5505] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5506] Ref(DollarNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5507] Ref(PsqlVariableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5508] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5509] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5510] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5511] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5512] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5204, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5513] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5514] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5515] Ref(LoadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5516] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5517] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5518] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5519] StringParser("LOCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5520] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5521] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5522] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5206, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5523] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5206, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5524] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5212, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5525] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5212, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5526] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5212, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5527] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5212, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5528] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5214, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5529] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5216, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5530] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5216, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5531] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5216, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5532] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5219, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5533] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5219, child_count: 8, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5534] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5227, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5535] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5229, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5536] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5229, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5537] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5229, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5538] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5231, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5539] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5231, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5540] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5231, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5541] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5233, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5542] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5233, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5543] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5233, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5544] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5236, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5545] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5236, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5546] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5236, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5547] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5236, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5548] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5236, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5549] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5550] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5551] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5552] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5239, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5553] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5239, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5554] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5555] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5556] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5557] Ref(ModeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5558] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5559] StringParser("LOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5560] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5561] StringParser("LOGGED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5562] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5563] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5564] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5565] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5566] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5567] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5568] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5241, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5569] StringParser("LSEG")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5241, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5570] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5245, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5571] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5245, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5572] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5245, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5573] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5245, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5574] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5249, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5575] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5251, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5576] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5251, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5577] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5251, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5578] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5253, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5579] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5253, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5580] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5581] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5582] StringParser("MACADDR8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5583] StringParser("MACADDR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5584] StringParser("MAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5585] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5586] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5587] StringParser("MAPPING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5588] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5589] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5590] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5591] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5592] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5593] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5594] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5595] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5596] StringParser("MCV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5597] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5598] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5599] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5600] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5256, child_count: 5, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5601] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5261, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5602] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5261, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5603] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5261, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5604] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5261, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5605] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5606] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5607] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5608] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5263, child_count: 3, min_times: 1, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5609] AnyNumberOf(3 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5266, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5610] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5266, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5611] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5266, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5612] Ref(MergeNotMatchedBySourceClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5266, child_count: 7, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5613] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5614] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5273, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5615] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5273, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5616] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5275, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5617] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5275, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5618] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5275, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5619] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5275, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5620] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5278, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5621] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5278, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5622] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5278, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5623] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5280, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5624] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5280, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5625] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5280, child_count: 10, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5626] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5627] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5628] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5629] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5630] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5631] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5290, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5632] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5633] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5634] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5635] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5636] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5637] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5295, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5638] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5295, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5639] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5297, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5640] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5297, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5641] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5297, child_count: 9, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5642] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5306, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5643] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5306, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5644] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5306, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5645] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5306, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5646] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5308, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5647] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5308, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5648] Ref(TargetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5308, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5649] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5310, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5650] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5310, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5651] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5310, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5652] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5310, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5653] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5312, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5654] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5312, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5655] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5314, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5656] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5314, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5657] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5314, child_count: 12, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5658] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5659] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5660] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5328, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5661] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5328, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5662] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5328, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5663] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5328, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5664] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5332, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5665] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5332, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5666] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5332, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5667] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5334, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5668] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5337, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5669] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5337, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5670] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5337, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5671] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5339, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5672] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5339, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5673] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5339, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5674] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5339, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5675] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5339, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5676] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5339, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5677] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5339, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5678] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5343, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5679] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5343, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5680] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5343, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5681] StringParser("MERGES")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5343, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5682] TypedParser("meta_command_query_buffer")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5343, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5683] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5344, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5684] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5345, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5685] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5686] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5687] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5688] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5689] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5690] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5691] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5692] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5693] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5694] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5695] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5696] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5697] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5698] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5699] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5700] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5701] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5702] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5703] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5704] StringParser("MODULUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5705] StringParser("MONEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5706] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5707] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5708] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5709] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5710] StringParser("MULTICURVE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5347, child_count: 1, min_times: 1, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5711] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5712] Ref(MultilineConcatenateNewline)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5713] TypedParser("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5714] StringParser("MULTILINESTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5715] StringParser("MULTILINESTRINGM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5716] StringParser("MULTILINESTRINGZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5717] StringParser("MULTILINESTRINGZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5718] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5719] StringParser("MULTIPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5720] StringParser("MULTIPOINTM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5721] StringParser("MULTIPOINTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5722] StringParser("MULTIPOINTZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5723] StringParser("MULTIPOLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5724] StringParser("MULTIPOLYGONM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5725] StringParser("MULTIPOLYGONZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5726] StringParser("MULTIPOLYGONZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5727] StringParser("MULTISURFACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5728] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5729] TypedParser("word")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5730] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5731] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5348, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5732] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5351, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5733] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5351, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5734] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5735] Ref(RightArrowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5736] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5737] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5738] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5739] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5357, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5740] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5357, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5741] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5357, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5742] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5359, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5743] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5359, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5744] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5362, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5745] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5362, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5746] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5366, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5747] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5366, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5748] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5368, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5749] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5368, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5750] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5368, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5751] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5368, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5752] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5368, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5753] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5368, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5754] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5755] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5756] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5757] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5758] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5759] StringParser("NDISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5760] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5761] StringParser("NEGATOR")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5762] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5763] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5764] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5765] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5766] StringParser("NFC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5767] StringParser("NFD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5768] StringParser("NFKC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5769] StringParser("NFKD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5770] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5771] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5772] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5773] StringParser("NOBYPASSRLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5774] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5775] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5776] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5777] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5778] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5779] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5780] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5781] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5370, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5782] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5370, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5783] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5376, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5784] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5376, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5785] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5376, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5786] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5787] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5379, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5788] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5382, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5789] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5382, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5790] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5385, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5791] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5385, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5792] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5385, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5793] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5385, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5794] OneOf(0 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5385, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5795] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5391, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5796] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5391, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5797] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5393, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5798] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5396, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5799] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5396, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5800] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5396, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5801] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5396, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5802] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5396, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5803] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5396, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5804] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5396, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5805] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5396, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5806] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5396, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5807] StringParser("NOREPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5396, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5808] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5396, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5809] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5398, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5810] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5402, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5811] Ref(NfcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5402, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5812] Ref(NfdKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5402, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5813] Ref(NfkcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5402, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5814] Ref(NfkdKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5402, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5815] Ref(NormalizedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5402, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5816] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5402, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5817] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5402, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5818] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5402, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5819] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5404, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5820] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5821] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5406, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5822] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5406, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5823] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5408, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5824] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5408, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5825] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5408, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5826] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5410, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5827] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5410, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5828] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5410, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5829] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5412, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5830] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5412, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5831] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5412, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5832] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5412, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5833] Ref(NotnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5412, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5834] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5412, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5835] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5412, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5836] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5412, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5837] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5415, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5838] Ref(NotifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5415, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5839] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5415, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5840] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5841] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5842] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5843] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5844] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5845] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5846] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5847] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5848] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5849] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5850] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5851] StringParser("NUMRANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5417, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5852] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5417, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5853] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5420, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5854] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5420, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5855] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5420, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5856] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5420, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5857] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5423, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5858] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5425, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5859] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5425, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5860] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5425, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5861] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5427, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5862] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5427, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5863] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5864] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5865] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5429, child_count: 12, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5866] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5867] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5868] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5869] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5870] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5871] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5872] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5873] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5874] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5875] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5876] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5877] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5878] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5879] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5880] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5441, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5881] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5441, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5882] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5445, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5883] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5445, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5884] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5885] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5886] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5887] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5888] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5889] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5890] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5891] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5892] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5893] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5894] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5895] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5896] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5897] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5898] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5899] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5900] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5901] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5902] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5903] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5447, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5904] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5447, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5905] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5449, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5906] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5449, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5907] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5452, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5908] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5453, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5909] Ref(OptionsListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5453, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5910] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5453, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5911] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5454, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5912] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5456, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5913] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5458, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5914] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5458, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5915] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5458, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5916] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5458, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5917] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5458, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5918] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5460, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5919] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5460, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5920] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5460, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5921] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5465, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5922] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5465, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5923] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5465, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 2, _padding: 0 }, // [5924] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5467, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5925] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5471, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5926] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5474, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5927] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5474, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5928] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5474, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5929] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5474, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5930] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5931] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5477, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5932] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5477, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5933] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5479, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5934] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5479, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5935] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5479, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5936] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5481, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5937] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5481, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5938] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5483, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5939] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5483, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5940] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5483, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5941] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5483, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5942] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5483, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5943] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5483, child_count: 8, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5944] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5491, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5945] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5491, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5946] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5491, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5947] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5491, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5948] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5491, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5949] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5491, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5950] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5491, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5951] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5493, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5952] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5493, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5953] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5493, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5954] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5493, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5955] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5493, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5956] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5495, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5957] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5495, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5958] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5495, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5959] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5495, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5960] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5495, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5961] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5495, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5962] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5495, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5963] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5495, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5964] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5500, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5965] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5500, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5966] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5500, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5967] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5968] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5502, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5969] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5505, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5970] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5505, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5971] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5505, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5972] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5507, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5973] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5507, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5974] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5507, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5975] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5510, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5976] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5512, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5977] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5513, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5978] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5518, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5979] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5522, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5980] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5522, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5981] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5522, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5982] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5522, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5983] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5522, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5984] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5522, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5985] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5526, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5986] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5526, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5987] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5526, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5988] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5526, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5989] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5526, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5990] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5526, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5991] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5526, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5992] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5528, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5993] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5529, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5994] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5534, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5995] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5538, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5996] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5538, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5997] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5538, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5998] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5538, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5999] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5538, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6000] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5538, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6001] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6002] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6003] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6004] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6005] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6006] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6007] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6008] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6009] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6010] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6011] StringParser("OWNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6012] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6013] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6014] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6015] StringParser("PARALLEL")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6016] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6017] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6018] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6019] StringParser("PARSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5542, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6020] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5542, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6021] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5545, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6022] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5547, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6023] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5547, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6024] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5550, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6025] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5552, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6026] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5552, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6027] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5556, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6028] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5556, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6029] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5559, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6030] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5561, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6031] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5564, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6032] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5564, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6033] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5564, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6034] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5564, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6035] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5564, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6036] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5567, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6037] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5569, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6038] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5572, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6039] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5572, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6040] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5572, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6041] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5572, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6042] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5574, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6043] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5574, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6044] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5577, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6045] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5582, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6046] Ref(ModulusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5582, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6047] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5582, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6048] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5582, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6049] Ref(RemainderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5582, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6050] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5582, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6051] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5587, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6052] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5587, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6053] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5587, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6054] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5589, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6055] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5592, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6056] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5594, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6057] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5594, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6058] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5594, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6059] StringParser("PASSING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5594, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6060] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5594, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6061] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5594, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6062] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5596, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6063] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5598, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6064] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5598, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6065] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6066] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6067] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6068] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6069] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6070] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6071] StringParser("PERMISSIVE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6072] TypedParser("pg_trgm_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6073] StringParser("PG_LSN")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6074] TypedParser("pgvector_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6075] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6076] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6077] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6078] StringParser("PLAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6079] StringParser("PLANS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6080] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6081] StringParser("POINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6082] StringParser("POINTM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6083] StringParser("POINTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6084] StringParser("POINTZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6085] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6086] StringParser("POLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6087] StringParser("POLYGONM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6088] StringParser("POLYGONZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6089] StringParser("POLYGONZM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6090] StringParser("POLYHEDRALSURFACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6091] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5600, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6092] StringParser("+")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5600, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6093] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5603, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6094] Ref(WithinGroupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5603, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6095] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5603, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6096] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5603, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6097] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5603, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6098] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5603, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6099] TypedParser("postgis_operator")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5603, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6100] TypedParser("postgres_copy_stdin_data_block")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5603, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6101] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5604, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6102] Ref(PostgresCopyStdinDataSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5604, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6103] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5605, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6104] Ref(LateralKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5605, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6105] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5605, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6106] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5605, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6107] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5605, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6108] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5605, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6109] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5605, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6110] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5610, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6111] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5610, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6112] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5610, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6113] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5613, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6114] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5615, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6115] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5615, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6116] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5615, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6117] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5617, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6118] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5617, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6119] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5617, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6120] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5617, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6121] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5617, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6122] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6123] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6124] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6125] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6126] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6127] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6128] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6129] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6130] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6131] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6132] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6133] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6134] StringParser("PROCESS_TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6135] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6136] StringParser("PROGRAM")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6137] TypedParser("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6138] StringParser("PROVIDER")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5619, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6139] TypedParser("psql_copy_command")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5619, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6140] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5620, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6141] Ref(PsqlCopyMetaCommandSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5620, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6142] TypedParser("psql_set_command")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5620, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6143] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5621, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6144] Ref(PsqlSetMetaCommandSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5621, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6145] StringParser(":")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5621, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6146] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5622, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6147] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5624, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6148] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5627, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6149] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5629, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6150] Ref(PsqlVariableColonSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5629, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6151] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5631, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6152] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5631, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6153] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5631, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6154] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5631, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6155] StringParser("PUBLICATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5631, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6156] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5633, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6157] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5635, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6158] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5635, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 1, _padding: 0 }, // [6159] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5637, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6160] Ref(PublicationTableSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5637, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6161] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5639, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6162] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5639, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6163] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6164] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6165] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6166] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5645, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6167] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5645, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6168] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5645, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6169] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5645, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 1, _padding: 0 }, // [6170] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5647, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6171] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5649, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6172] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5649, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6173] Ref(Current_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5649, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5651, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6175] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5651, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6176] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5653, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6177] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5653, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6178] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5653, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6179] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5653, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6180] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6181] Ref(ExtendedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6182] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5656, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6183] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5658, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6184] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5658, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6185] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5661, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6186] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5661, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6187] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5661, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6188] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5663, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6189] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5663, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6190] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5663, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5665, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6192] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5665, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6193] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5668, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6194] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6195] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5671, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6196] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5671, child_count: 1, min_times: 1, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6197] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5672, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6198] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5672, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5674, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6200] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 5674, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6201] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5676, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6202] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5676, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6203] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5676, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6204] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5676, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6205] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5679, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6206] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5679, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6207] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5682, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6208] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5682, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6209] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5682, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6210] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5682, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6211] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5685, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6212] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5685, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6213] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5685, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6214] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5685, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6215] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5687, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6216] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5687, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6217] TypedParser("unicode_double_quote")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5687, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6218] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5691, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6219] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5693, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6220] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5695, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6221] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5695, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6222] TypedParser("escaped_single_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5695, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6223] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5697, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6224] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5697, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6225] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5697, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6226] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5699, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6227] TypedParser("bit_string_literal")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5699, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6228] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6229] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5701, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6230] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5701, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6231] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5704, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6232] TypedParser("unicode_single_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5704, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6233] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5706, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6234] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5706, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6235] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5706, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6236] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5708, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6237] Ref(UescapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5708, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6238] RegexParser
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5708, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6239] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5710, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6240] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5712, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6241] TypedParser("dollar_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5712, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6242] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6243] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6244] TypedParser("dollar_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6245] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6246] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6247] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6248] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6249] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6250] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6251] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6252] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6253] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6254] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6255] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5714, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6256] StringParser("REASSIGN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5714, child_count: 6, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6257] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5720, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6258] Ref(ReassignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5720, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6259] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5720, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6260] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5720, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6261] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5722, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6262] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5726, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6263] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5726, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6264] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5726, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6265] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5726, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6266] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5726, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6267] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5726, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6268] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5730, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6269] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5730, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6270] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5730, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6271] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5730, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6272] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5730, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6273] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5730, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6274] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5730, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6275] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5730, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6276] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5730, child_count: 5, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6277] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5735, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6278] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5735, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6279] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5735, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6280] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5735, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6281] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5735, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6282] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5737, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6283] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6284] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6285] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6286] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5740, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6287] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5743, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6288] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5743, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6289] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5743, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6290] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5743, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6291] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5745, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6292] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5745, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6293] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5748, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6294] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5748, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6295] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5748, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6296] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5748, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6297] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5748, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6298] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5748, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6299] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5748, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6300] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5752, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6301] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5752, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6302] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5755, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6303] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5755, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6304] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5757, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6305] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5757, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6306] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5757, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6307] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5760, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6308] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6309] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6310] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5762, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6311] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5764, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6312] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5764, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6313] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5764, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6314] StringParser("REFRESH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5764, child_count: 6, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6315] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5770, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6316] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5770, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6317] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5770, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6318] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5770, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6319] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5770, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6320] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5770, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6321] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5770, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6322] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5770, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6323] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5770, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6324] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5773, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6325] Ref(ReindexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5773, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6326] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5776, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6327] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5778, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6328] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5781, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6329] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5783, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6330] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5783, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6331] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5783, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6332] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5785, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6333] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5785, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6334] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5785, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6335] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6336] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6337] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5787, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6338] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5791, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6339] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5794, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6340] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5794, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6341] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5794, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6342] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5794, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6343] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6344] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6345] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6346] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6347] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5800, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6348] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5800, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6349] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5800, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6350] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5800, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6351] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5803, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6352] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5805, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6353] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5805, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6354] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5805, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6355] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5805, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6356] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5805, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6357] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5808, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6358] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5808, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6359] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5810, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6360] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5810, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6361] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5810, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6362] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5812, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6363] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5812, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6364] Ref(DefinitionArgumentValueGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5812, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6365] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5815, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6366] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6367] Ref(RelationOptionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6368] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6369] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6370] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6371] StringParser("REMAINDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6372] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6373] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6374] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6375] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6376] StringParser("REPLICA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6377] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6378] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6379] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5817, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6380] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5820, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6381] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5820, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6382] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5820, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6383] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5820, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6384] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5822, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6385] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5822, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6386] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6387] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6388] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6389] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6390] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6391] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6392] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6393] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6394] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6395] StringParser("RESTRICTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6396] StringParser("RESTRICTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6397] StringParser("RETRIEVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6398] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6399] StringParser("RETURNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6400] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5825, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6401] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5825, child_count: 6, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6402] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5831, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6403] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5831, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6404] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5834, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6405] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5834, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6406] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5834, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6407] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5834, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6408] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5838, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6409] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5841, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6410] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5841, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6411] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5841, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6412] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5841, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6413] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5843, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6414] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5843, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6415] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5843, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6416] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6417] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6418] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6419] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6420] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6421] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6422] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6423] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6424] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6425] StringParser("=>")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6426] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6427] StringParser("RIGHTARG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6428] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6429] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6430] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6431] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6432] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6433] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6434] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6435] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6436] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6437] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6438] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6439] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6440] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6441] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6442] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6443] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5847, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6444] StringParser("SAFE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5847, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6445] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5851, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6446] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5851, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6447] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5853, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6448] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5853, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6449] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5853, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6450] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5856, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6451] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5856, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6452] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5858, child_count: 1, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6453] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5859, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6454] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5859, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6455] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5862, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6456] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5862, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6457] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5862, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6458] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5862, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6459] StringParser("SCALAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5862, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6460] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5862, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6461] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5862, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6462] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5862, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6463] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5862, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6464] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5862, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6465] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5862, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6466] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5862, child_count: 7, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6467] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5869, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6468] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5869, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6469] Ref(LabelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5869, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6470] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6471] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6472] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5871, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6473] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5871, child_count: 21, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6474] OneOf(21 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5892, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6475] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5894, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6476] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5894, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6477] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5894, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6478] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6479] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6480] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6481] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5899, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6482] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5899, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6483] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5899, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6484] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5902, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6485] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5902, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6486] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5904, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6487] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5904, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6488] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5904, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6489] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5906, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6490] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5906, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6491] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5906, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6492] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5909, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6493] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5909, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6494] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5909, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6495] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5909, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6496] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6497] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6498] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6499] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6500] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5915, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6501] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5915, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6502] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5915, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6503] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5917, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6504] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5920, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6505] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5920, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6506] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5923, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6507] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5923, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6508] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5923, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6509] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5923, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6510] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5926, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6511] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5926, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6512] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5926, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6513] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5926, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6514] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5929, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6515] Ref(ProceduralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5929, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6516] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5929, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6517] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5929, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6518] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5932, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6519] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5932, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6520] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5932, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6521] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5934, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6522] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5937, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6523] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5937, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6524] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5939, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6525] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5939, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6526] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5939, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6527] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5941, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6528] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5941, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6529] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5941, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6530] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5944, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6531] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5944, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6532] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5944, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6533] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5946, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6534] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5949, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6535] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5949, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6536] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5951, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6537] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5951, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6538] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5951, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6539] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6540] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6541] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6542] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5955, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6543] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5955, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6544] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5955, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6545] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5957, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6546] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5957, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6547] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5957, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6548] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5959, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6549] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5959, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6550] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5959, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6551] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5961, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6552] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5961, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6553] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5961, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6554] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5961, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6555] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5963, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6556] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5963, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6557] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5963, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6558] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5965, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6559] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5965, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6560] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5967, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6561] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5967, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6562] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5967, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6563] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5969, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6564] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5971, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6565] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5971, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6566] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5973, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6567] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5973, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6568] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5976, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6569] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5978, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6570] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5978, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6571] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 5978, child_count: 5, min_times: 0, first_terminator_idx: 58, terminator_count: 12, _padding: 0 }, // [6572] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5983, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6573] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5983, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6574] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(13), first_child_idx: 5983, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6575] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6576] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6577] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6578] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5985, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6579] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5985, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6580] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5987, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6581] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5987, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6582] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5987, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6583] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5989, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6584] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5989, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6585] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5989, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6586] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5989, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6587] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5989, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6588] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5989, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6589] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5989, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6590] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5992, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6591] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5992, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6592] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5992, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6593] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5992, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6594] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5992, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6595] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5992, child_count: 9, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6596] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6001, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6597] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 6001, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6598] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6002, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6599] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6002, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6600] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6002, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6601] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6004, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6602] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6004, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6603] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6004, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6604] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6004, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6605] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6004, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6606] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6004, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6607] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6004, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6608] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6004, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6609] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 6004, child_count: 14, min_times: 0, first_terminator_idx: 70, terminator_count: 7, _padding: 0 }, // [6610] Sequence(14 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6611] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6612] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6613] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6614] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6615] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6616] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6617] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6618] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6619] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6620] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6621] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6622] Ref(OffsetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6623] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6624] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6625] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6626] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6627] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6018, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6628] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6020, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6629] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6020, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6630] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6020, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6631] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6020, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6632] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6020, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [6633] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6020, child_count: 4, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6634] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6024, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6635] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6026, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6636] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6029, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6637] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6029, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6638] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6031, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6639] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6034, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6640] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6034, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6641] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6034, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6642] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6037, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6643] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6037, child_count: 4, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6644] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6645] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6646] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6041, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6647] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6041, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6648] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6043, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6649] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6045, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6650] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6045, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6651] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6045, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6652] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6045, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6653] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6045, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6654] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6045, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6655] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6045, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6656] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6047, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6657] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6049, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6658] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6049, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6659] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6049, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6660] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6051, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6661] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6051, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6662] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6051, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6663] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6053, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6664] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6055, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6665] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6055, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6666] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6055, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6667] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6057, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6668] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6057, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6669] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6057, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6670] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6057, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6671] StringParser("SERIAL2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6057, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6672] StringParser("SERIAL4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6057, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6673] StringParser("SERIAL8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6057, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6674] StringParser("SERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6057, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6675] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6057, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6676] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6057, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6677] StringParser("SERVER_ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6057, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6678] StringParser("SERVER_VERSION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6057, child_count: 4, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6679] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6061, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6680] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6061, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6681] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6061, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6682] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6061, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6683] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6061, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6684] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6061, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6685] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6061, child_count: 4, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6686] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6065, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6687] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6065, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6688] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6067, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6689] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6067, child_count: 1, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6690] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6068, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6691] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6070, child_count: 5, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6692] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6075, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6693] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6075, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6694] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6075, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6695] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6075, child_count: 6, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6696] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6081, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6697] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6081, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6698] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6081, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6699] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6081, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6700] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6081, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6701] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6081, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6702] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6081, child_count: 1, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6703] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6082, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6704] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6082, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6705] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6085, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6706] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6088, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6707] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6090, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6708] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6090, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6709] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6090, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6710] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6093, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6711] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6095, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6712] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6095, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6713] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6097, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6714] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6099, child_count: 6, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6715] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6105, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6716] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6105, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6717] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6105, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6718] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6105, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6719] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6105, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6720] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6105, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6721] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6105, child_count: 1, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6722] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6106, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6723] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6106, child_count: 4, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6724] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6110, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6725] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6110, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6726] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6110, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6727] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6112, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6728] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6112, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6729] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6114, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6730] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6114, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6731] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6116, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6732] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6116, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6733] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6116, child_count: 5, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6734] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6121, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6735] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6121, child_count: 1, min_times: 1, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6736] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6122, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6737] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6124, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6738] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6124, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6739] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6124, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6740] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6124, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6741] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6124, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6742] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6124, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6743] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6124, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6744] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6127, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6745] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6127, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6746] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6129, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6747] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6131, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6748] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6131, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6749] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6131, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6750] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6131, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6751] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6131, child_count: 4, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6752] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6135, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6753] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6135, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6754] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6135, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6755] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6135, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6756] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6135, child_count: 4, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6757] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6139, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6758] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6139, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6759] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6141, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6760] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6143, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6761] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6143, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6762] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6143, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6763] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6145, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6764] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6145, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6765] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6145, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6766] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6145, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6767] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6768] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6769] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6147, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6770] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6150, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6771] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6150, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6772] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6152, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6773] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6152, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6774] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6152, child_count: 4, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6775] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6156, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6776] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6159, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6777] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6159, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6778] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6161, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6779] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6161, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6780] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6161, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6781] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6163, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6782] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6163, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6783] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6165, child_count: 4, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6784] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6169, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6785] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6169, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6786] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6169, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6787] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6169, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6788] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6169, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6789] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6172, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6790] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6172, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6791] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6172, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6792] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6175, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6793] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6175, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6794] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6175, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6795] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6175, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6796] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6177, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6797] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6177, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6798] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6177, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6799] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6179, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6800] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6179, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6801] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6181, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6802] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6181, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6803] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6181, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6804] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6181, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6805] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6181, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6806] StringParser("SETTINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6181, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6807] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6181, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6808] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6181, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6809] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6181, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6810] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6183, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6811] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6185, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6812] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6185, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6813] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6185, child_count: 1, min_times: 1, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6814] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6186, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6815] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6189, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6816] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6189, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6817] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6189, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6818] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6189, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6819] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6189, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6820] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6191, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6821] Ref(ShowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6191, child_count: 7, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6822] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6198, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6823] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6198, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6824] Ref(Is_superuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6198, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6825] Ref(Lc_collateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6198, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6826] Ref(Lc_ctypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6198, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6827] Ref(Server_encodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6198, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6828] Ref(Server_versionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6198, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6829] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6198, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6830] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6198, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6831] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6198, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6832] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6200, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6833] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6200, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6834] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6200, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6835] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6200, child_count: 1, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6836] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6201, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6837] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6201, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6838] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6201, child_count: 3, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6839] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6204, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6840] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6204, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6841] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6204, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6842] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6204, child_count: 2, min_times: 0, first_terminator_idx: 77, terminator_count: 1, _padding: 0 }, // [6843] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6206, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6844] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6206, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6845] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6206, child_count: 0, min_times: 0, first_terminator_idx: 77, terminator_count: 0, _padding: 0 }, // [6846] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6206, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6847] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6208, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6848] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6208, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6849] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6211, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6850] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6211, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6851] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6211, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6852] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6211, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6853] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6211, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6854] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6855] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6856] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6857] StringParser("SKIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6858] StringParser("SKIP_LOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6859] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6860] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6861] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6862] StringParser("SMALLSERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6863] StringParser("SNAPSHOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6864] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6865] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6866] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6867] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6868] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6869] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6870] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6871] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6872] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6873] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6874] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6875] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6876] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6877] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6878] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6879] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6880] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6881] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6882] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6883] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6884] StringParser("STANDALONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6885] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6886] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6887] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6888] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6213, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6889] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6213, child_count: 121, min_times: 0, first_terminator_idx: 78, terminator_count: 1, _padding: 0 }, // [6890] OneOf(121 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6891] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6892] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6893] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6894] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6895] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6896] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6897] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6898] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6899] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6900] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6901] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6902] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6903] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6904] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6905] Ref(CreateSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6906] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6907] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6908] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6909] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6910] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6911] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6912] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6913] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6914] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6915] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6916] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6917] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6918] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6919] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6920] Ref(CreateModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6921] Ref(DropModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6922] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6923] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6924] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6925] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6926] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6927] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6928] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6929] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6930] Ref(AlterDefaultPrivilegesStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6931] Ref(DropOwnedStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6932] Ref(ReassignOwnedStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6933] Ref(CommentOnStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6934] Ref(AnalyzeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6935] Ref(CreateTableAsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6936] Ref(AlterTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6937] Ref(SetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6938] Ref(AlterSystemStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6939] Ref(AlterPolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6940] Ref(CreatePolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6941] Ref(DropPolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6942] Ref(CreateDomainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6943] Ref(AlterDomainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6944] Ref(DropDomainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6945] Ref(CreateMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6946] Ref(AlterMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6947] Ref(DropMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6948] Ref(RefreshMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6949] Ref(AlterDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6950] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6951] Ref(VacuumStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6952] Ref(AlterFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6953] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6954] Ref(AlterViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6955] Ref(ListenStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6956] Ref(NotifyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6957] Ref(UnlistenStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6958] Ref(LoadStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6959] Ref(ResetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6960] Ref(DiscardStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6961] Ref(AlterProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6962] Ref(CreateProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6963] Ref(DropProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6964] Ref(CopyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6965] Ref(DoStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6966] Ref(AlterIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6967] Ref(ReindexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6968] Ref(AlterRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6969] Ref(CreateExtensionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6970] Ref(DropExtensionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6971] Ref(AlterExtensionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6972] Ref(CreateSubscriptionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6973] Ref(AlterSubscriptionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6974] Ref(DropSubscriptionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6975] Ref(CreatePublicationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6976] Ref(AlterPublicationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6977] Ref(DropPublicationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6978] Ref(CreateTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6979] Ref(AlterTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6980] Ref(CreateTextSearchConfigurationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6981] Ref(AlterTextSearchConfigurationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6982] Ref(DropTextSearchConfigurationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6983] Ref(AlterSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6984] Ref(LockTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6985] Ref(ClusterStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6986] Ref(CreateCollationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6987] Ref(DropCollationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6988] Ref(CallStoredProcedureSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6989] Ref(CreateServerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6990] Ref(CreateUserMappingStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6991] Ref(ImportForeignSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6992] Ref(CreateForeignTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6993] Ref(DropAggregateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6994] Ref(CreateAggregateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6995] Ref(AlterAggregateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6996] Ref(CreateStatisticsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6997] Ref(AlterStatisticsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6998] Ref(DropStatisticsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [6999] Ref(ShowStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7000] Ref(SetConstraintsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7001] Ref(CreateForeignDataWrapperStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7002] Ref(MetaCommandQueryBufferStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7003] Ref(DropForeignTableStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7004] Ref(CreateOperatorStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7005] Ref(AlterForeignTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7006] Ref(SecurityLabelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7007] Ref(PrepareStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7008] Ref(ExecuteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7009] Ref(DeallocateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7010] Ref(SetSessionAuthorizationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7011] Ref(ResetSessionAuthorizationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7012] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7013] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7014] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7015] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7016] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7017] StringParser("STORED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7018] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7019] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7020] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6334, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7021] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6334, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7022] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6336, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7023] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6336, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7024] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6336, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7025] StringParser("STRIP")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6336, child_count: 3, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7026] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6339, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7027] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6341, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7028] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7029] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7030] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7031] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7032] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7033] StringParser("SUBSCRIPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7034] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7035] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7036] StringParser("SUMMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7037] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7038] StringParser("SUPPORT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7039] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7040] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7041] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7042] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7043] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6343, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7044] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6343, child_count: 3, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7045] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6346, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7046] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6348, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7047] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6348, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7048] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6348, child_count: 5, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7049] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6353, child_count: 3, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7050] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6356, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7051] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6356, child_count: 3, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7052] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6359, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7053] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6359, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7054] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7055] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7056] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6361, child_count: 4, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7057] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6365, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7058] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6365, child_count: 3, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7059] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6368, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7060] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6368, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7061] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6368, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7062] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6368, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7063] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6368, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7064] Ref(IndexParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6368, child_count: 3, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7065] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6371, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7066] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6371, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7067] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6371, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7068] Ref(IndexParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6371, child_count: 5, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7069] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6376, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7070] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6376, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7071] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6378, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7072] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6378, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7073] Ref(IndexAccessMethodSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6378, child_count: 3, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7074] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6381, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7075] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6383, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7076] Ref(ExclusionConstraintElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6383, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7077] Ref(IndexParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6383, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7078] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7079] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 3, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7080] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6388, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7081] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6388, child_count: 4, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7082] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6392, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7083] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6392, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7084] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6392, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7085] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6392, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7086] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6392, child_count: 4, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7087] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6396, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7088] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6398, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7089] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6398, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7090] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6400, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7091] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6400, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7092] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6400, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7093] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6402, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7094] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6404, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7095] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6404, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7096] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6404, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7097] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6406, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7098] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6406, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7099] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6406, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6408, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7101] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6408, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7102] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6408, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7103] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6410, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7104] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6410, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7105] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6410, child_count: 4, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7106] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6414, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7107] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6416, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7108] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6416, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7109] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6416, child_count: 4, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7110] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6420, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7111] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6422, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7112] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6422, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7113] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6422, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7114] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6422, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7115] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6422, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7116] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6422, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7117] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6424, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7118] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6424, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7119] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6426, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7120] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6426, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7121] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6426, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7122] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6428, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7123] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6430, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7124] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6430, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7125] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6430, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7126] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7127] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7128] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6432, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7129] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 6, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7130] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6438, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7131] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6438, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7132] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6438, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7133] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6440, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7134] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6440, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7135] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6442, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7136] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6442, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7137] Ref(OrdinalityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6442, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7138] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6442, child_count: 3, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7139] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6445, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7140] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6445, child_count: 3, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7141] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6448, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7142] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6448, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7143] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6448, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7144] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6448, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7145] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6448, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7146] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6448, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7147] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6450, child_count: 1, min_times: 0, first_terminator_idx: 79, terminator_count: 1, _padding: 0 }, // [7148] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6451, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7149] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6451, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7150] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6451, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7151] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6451, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7152] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6453, child_count: 1, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7153] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6454, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7154] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6454, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7155] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6454, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7156] StringParser("TARGET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6454, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7157] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6454, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7158] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6454, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7159] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6454, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7160] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6454, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7161] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6454, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7162] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6456, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7163] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6456, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7164] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6456, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7165] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6456, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7166] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6458, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7167] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6458, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7168] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6458, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7169] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6458, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7170] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6458, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7171] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6458, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7172] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6458, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7173] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6458, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7174] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6458, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7175] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6458, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7176] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6458, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7177] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6458, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7178] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6461, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7179] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6463, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7180] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6463, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7181] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6463, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7182] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6463, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7183] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6466, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7184] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6468, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7185] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6468, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7186] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6468, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7187] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6468, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7188] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6468, child_count: 1, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7189] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6469, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7190] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7191] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7192] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7193] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7194] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7195] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7196] StringParser("TIMESTAMPTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7197] StringParser("TIMETZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7198] StringParser("TIMING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7199] StringParser("TIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7200] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7201] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7202] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7203] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7204] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7205] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7206] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7207] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7208] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6473, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7209] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6477, child_count: 5, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7210] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7211] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7212] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7213] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7214] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7215] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6482, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7216] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6484, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7217] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6484, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7218] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6484, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7219] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7220] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7221] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6486, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7222] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7223] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7224] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7225] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7226] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7227] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7228] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7229] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7230] StringParser("TRIANGLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7231] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6489, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7232] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6489, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7233] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6492, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7234] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6492, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7235] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6492, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7236] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6492, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7237] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6492, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7238] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6492, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7239] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6492, child_count: 5, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7240] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6497, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7241] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6497, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7242] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6497, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7243] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6499, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7244] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6501, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7245] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6503, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7246] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6503, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7247] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6503, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7248] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6505, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7249] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6505, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7250] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6505, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7251] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6507, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7252] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7253] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7254] Ref(ContinueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7255] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7256] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7257] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7258] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7259] StringParser("TSQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7260] StringParser("TSRANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7261] StringParser("TSTZRANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7262] StringParser("TSVECTOR")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7263] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6512, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7264] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7265] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6514, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7266] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6514, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7267] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6517, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7268] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6519, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7269] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6519, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7270] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7271] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7272] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7273] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6521, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7274] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6521, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7275] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6523, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7276] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6523, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7277] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6523, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7278] StringParser("TYPES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6523, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7279] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6523, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7280] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6523, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7281] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6523, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7282] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6523, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7283] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6523, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7284] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7285] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7286] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7287] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7288] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6526, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7289] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6526, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7290] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6528, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7291] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6528, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7292] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6530, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7293] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6530, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7294] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6530, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7295] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6530, child_count: 1, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7296] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6531, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7297] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6531, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7298] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6531, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7299] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6531, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7300] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6531, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7301] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6531, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7302] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6533, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7303] Ref(UnlistenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6533, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7304] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6535, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7305] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6535, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7306] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6535, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7307] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6535, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7308] StringParser("UNLOGGED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 6535, child_count: 8, min_times: 0, first_terminator_idx: 80, terminator_count: 10, _padding: 0 }, // [7309] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7310] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7311] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7312] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7313] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7314] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7315] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7316] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7317] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7318] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7319] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7320] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7321] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6543, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7322] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6543, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7323] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6546, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7324] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6546, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7325] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6546, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7326] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6546, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7327] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6548, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7328] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6548, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7329] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6548, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7330] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6548, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7331] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6548, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7332] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6548, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7333] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6550, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7334] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6550, child_count: 1, min_times: 1, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7335] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6551, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7336] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7337] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7338] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7339] StringParser("UNSAFE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7340] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7341] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6553, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7342] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6553, child_count: 10, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7343] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6563, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7344] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6563, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7345] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6563, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7346] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 6563, child_count: 1, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7347] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6564, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7348] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6564, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7349] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6564, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7350] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6564, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7351] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6566, child_count: 4, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7352] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6570, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7353] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6570, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7354] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6570, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7355] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6570, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7356] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6570, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7357] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6570, child_count: 4, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7358] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6574, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7359] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6574, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7360] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6576, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7361] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6576, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7362] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6578, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7363] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6580, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7364] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6580, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7365] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6580, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7366] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6580, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7367] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6580, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7368] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6580, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7369] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7370] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7371] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7372] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7373] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7374] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7375] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7376] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7377] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7378] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7379] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7380] StringParser("UUID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6582, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7381] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6582, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7382] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6585, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7383] Ref(VacuumKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6585, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7384] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6587, child_count: 4, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7385] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6591, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7386] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6591, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7387] Ref(FreezeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6591, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7388] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6591, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7389] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6593, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7390] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6593, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7391] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6593, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7392] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6596, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7393] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6598, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7394] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6600, child_count: 11, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7395] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6611, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7396] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6611, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7397] Ref(FreezeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6611, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7398] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6611, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7399] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6611, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7400] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6611, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7401] Ref(Disable_page_skippingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6611, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7402] Ref(Skip_lockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6611, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7403] Ref(Index_cleanupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6611, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7404] Ref(Process_toastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6611, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7405] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6611, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7406] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6611, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7407] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6614, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7408] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6614, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7409] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6614, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7410] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6614, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7411] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6616, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7412] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7413] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7414] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7415] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7416] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7417] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6618, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7418] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6618, child_count: 5, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7419] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6623, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7420] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6623, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7421] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 6625, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7422] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6628, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7423] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6630, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7424] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7425] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7426] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7427] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7428] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7429] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7430] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7431] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7432] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7433] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7434] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7435] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7436] StringParser("VARIADIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7437] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7438] StringParser("VECTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6632, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7439] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6632, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7440] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7441] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7442] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7443] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7444] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7445] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7446] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7447] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7448] StringParser("WAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7449] StringParser(":=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7450] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7451] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7452] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6634, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7453] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6634, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7454] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6636, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7455] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6638, child_count: 36, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7456] OneOf(36 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7457] Ref(PointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7458] Ref(LinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7459] Ref(PolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7460] Ref(MultipointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7461] Ref(MultilinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7462] Ref(MultipolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7463] Ref(GeometrycollectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7464] Ref(PointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7465] Ref(LinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7466] Ref(PolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7467] Ref(MultipointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7468] Ref(MultilinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7469] Ref(MultipolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7470] Ref(GeometrycollectionzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7471] Ref(PointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7472] Ref(LinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7473] Ref(PolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7474] Ref(MultipointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7475] Ref(MultilinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7476] Ref(MultipolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7477] Ref(GeometrycollectionmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7478] Ref(PointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7479] Ref(LinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7480] Ref(PolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7481] Ref(MultipointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7482] Ref(MultilinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7483] Ref(MultipolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7484] Ref(GeometrycollectionzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7485] Ref(CircularstringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7486] Ref(CompoundcurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7487] Ref(CurvepolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7488] Ref(MulticurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7489] Ref(MultisurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7490] Ref(PolyhedralsurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7491] Ref(TriangleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7492] Ref(TinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6674, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7493] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6677, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7494] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6679, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7495] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6682, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7496] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6684, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7497] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6687, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7498] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6689, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7499] Ref(SimpleGeometryGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6689, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7500] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6692, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7501] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6694, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7502] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6697, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7503] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6699, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7504] Ref(SimpleGeometryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6699, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7505] Ref(WellKnownTextGeometrySegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6699, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7506] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6701, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7507] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6703, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7508] Ref(GeometryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6703, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7509] Ref(GeographyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6703, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7510] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6706, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7511] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6708, child_count: 38, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7512] OneOf(38 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7513] Ref(PointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7514] Ref(LinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7515] Ref(PolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7516] Ref(MultipointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7517] Ref(MultilinestringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7518] Ref(MultipolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7519] Ref(GeometrycollectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7520] Ref(PointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7521] Ref(LinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7522] Ref(PolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7523] Ref(MultipointzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7524] Ref(MultilinestringzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7525] Ref(MultipolygonzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7526] Ref(GeometrycollectionzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7527] Ref(PointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7528] Ref(LinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7529] Ref(PolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7530] Ref(MultipointmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7531] Ref(MultilinestringmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7532] Ref(MultipolygonmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7533] Ref(GeometrycollectionmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7534] Ref(PointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7535] Ref(LinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7536] Ref(PolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7537] Ref(MultipointzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7538] Ref(MultilinestringzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7539] Ref(MultipolygonzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7540] Ref(GeometrycollectionzmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7541] Ref(CircularstringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7542] Ref(CompoundcurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7543] Ref(CurvepolygonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7544] Ref(MulticurveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7545] Ref(MultisurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7546] Ref(PolyhedralsurfaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7547] Ref(TriangleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7548] Ref(TinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7549] Ref(GeometryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6746, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7550] Ref(GeographyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6746, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7551] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6748, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7552] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6748, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7553] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6748, child_count: 8, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7554] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6756, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7555] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6756, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7556] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6759, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7557] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6759, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7558] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6759, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7559] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6759, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7560] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6759, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7561] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6759, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7562] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6759, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7563] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6759, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7564] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6759, child_count: 4, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7565] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6763, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7566] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6763, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7567] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6765, child_count: 3, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7568] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6768, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7569] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6768, child_count: 10, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7570] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6778, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7571] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6778, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7572] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6780, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7573] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6780, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7574] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6780, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7575] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7576] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7577] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7578] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7579] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7580] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7581] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7582] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7583] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6784, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7584] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6784, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7585] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6784, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7586] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6784, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7587] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6784, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7588] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6784, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7589] StringParser("WHITESPACE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6784, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7590] Token("whitespace")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6784, child_count: 1, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7591] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6785, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7592] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6785, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7593] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6787, child_count: 1, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7594] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6788, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7595] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6790, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7596] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6792, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7597] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6792, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7598] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6792, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7599] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6794, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7600] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6794, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7601] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6794, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7602] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6794, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7603] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6794, child_count: 4, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7604] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 6798, child_count: 1, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7605] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6799, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7606] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6801, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7607] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6801, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7608] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6801, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7609] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6801, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7610] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6801, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7611] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6801, child_count: 4, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7612] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6805, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7613] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6805, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7614] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7615] Ref(CascadedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7616] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7617] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6807, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7618] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6807, child_count: 6, min_times: 0, first_terminator_idx: 91, terminator_count: 0, _padding: 0 }, // [7619] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6813, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7620] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6813, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7621] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6813, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7622] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6813, child_count: 2, min_times: 0, first_terminator_idx: 90, terminator_count: 1, _padding: 0 }, // [7623] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6815, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7624] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6815, child_count: 0, min_times: 0, first_terminator_idx: 90, terminator_count: 0, _padding: 0 }, // [7625] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6815, child_count: 0, min_times: 0, first_terminator_idx: 91, terminator_count: 0, _padding: 0 }, // [7626] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6815, child_count: 0, min_times: 0, first_terminator_idx: 91, terminator_count: 0, _padding: 0 }, // [7627] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6815, child_count: 6, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7628] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6821, child_count: 0, min_times: 0, first_terminator_idx: 91, terminator_count: 0, _padding: 0 }, // [7629] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6821, child_count: 0, min_times: 0, first_terminator_idx: 91, terminator_count: 0, _padding: 0 }, // [7630] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6821, child_count: 0, min_times: 0, first_terminator_idx: 91, terminator_count: 0, _padding: 0 }, // [7631] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6821, child_count: 2, min_times: 0, first_terminator_idx: 91, terminator_count: 1, _padding: 0 }, // [7632] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6823, child_count: 0, min_times: 0, first_terminator_idx: 91, terminator_count: 0, _padding: 0 }, // [7633] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6823, child_count: 0, min_times: 0, first_terminator_idx: 91, terminator_count: 0, _padding: 0 }, // [7634] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 6823, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7635] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6823, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7636] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6823, child_count: 3, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7637] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6826, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7638] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6826, child_count: 1, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7639] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7640] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7641] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7642] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6827, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7643] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6827, child_count: 4, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7644] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6831, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7645] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6831, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7646] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6831, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7647] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6831, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7648] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6831, child_count: 3, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7649] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6834, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7650] Ref(WithinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6834, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7651] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6834, child_count: 3, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7652] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7653] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7654] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7655] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7656] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7657] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7658] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7659] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7660] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7661] StringParser("X509")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7662] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7663] StringParser("XMLATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7664] StringParser("XMLCONCAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7665] StringParser("XMLELEMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7666] StringParser("XMLEXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7667] StringParser("XMLFOREST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7668] StringParser("XMLNAMESPACES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7669] StringParser("XMLPARSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7670] StringParser("XMLPI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7671] StringParser("XMLROOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7672] StringParser("XMLSERIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7673] StringParser("XMLTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7674] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7675] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7676] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7677] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7678] StringParser("YES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7679] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6837, child_count: 0, min_times: 0, first_terminator_idx: 92, terminator_count: 0, _padding: 0 }, // [7680] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    4, 5, 60, 6, 50, 7, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
    20, 27, 32, 37, 38, 43, 8, 9, 21, 22, 23, 24, 25, 26, 28, 29,
    30, 31, 33, 34, 35, 36, 39, 40, 41, 42, 44, 45, 46, 47, 48, 49,
    51, 57, 52, 53, 56, 54, 55, 61, 62, 63, 65, 73, 77, 80, 83, 88,
    66, 67, 68, 69, 70, 71, 72, 74, 75, 76, 78, 79, 81, 82, 84, 85,
    86, 87, 89, 132, 90, 99, 104, 107, 108, 109, 110, 111, 112, 113, 114, 115,
    116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 91, 92,
    93, 94, 97, 98, 95, 96, 100, 101, 102, 103, 105, 106, 130, 131, 134, 57,
    137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 150, 153, 156, 148, 149,
    151, 152, 154, 155, 157, 158, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169,
    170, 173, 176, 179, 171, 172, 174, 175, 177, 178, 180, 181, 183, 184, 186, 190,
    194, 198, 202, 204, 206, 187, 188, 189, 57, 191, 192, 193, 57, 195, 196, 197,
    57, 199, 200, 201, 57, 203, 57, 205, 57, 208, 209, 215, 216, 217, 223, 224,
    225, 239, 226, 232, 227, 228, 229, 230, 231, 233, 234, 235, 230, 231, 236, 57,
    237, 238, 242, 243, 248, 249, 250, 251, 256, 252, 230, 231, 253, 254, 255, 257,
    261, 269, 258, 259, 260, 262, 263, 264, 265, 266, 267, 268, 270, 271, 272, 274,
    275, 276, 277, 278, 291, 295, 303, 307, 323, 279, 280, 281, 284, 288, 282, 283,
    285, 286, 287, 289, 290, 292, 293, 294, 296, 297, 298, 299, 300, 301, 302, 304,
    305, 306, 308, 309, 310, 311, 320, 312, 315, 313, 314, 316, 317, 318, 319, 321,
    322, 324, 325, 326, 327, 329, 330, 331, 332, 333, 334, 337, 335, 57, 338, 339,
    340, 342, 345, 343, 344, 346, 57, 347, 348, 349, 350, 351, 352, 353, 354, 355,
    356, 359, 360, 364, 365, 366, 367, 368, 372, 361, 362, 363, 369, 57, 374, 375,
    376, 377, 378, 379, 381, 382, 383, 384, 394, 401, 385, 386, 389, 387, 388, 390,
    57, 395, 396, 397, 398, 57, 402, 403, 405, 408, 406, 407, 410, 411, 412, 413,
    414, 418, 421, 427, 442, 450, 456, 460, 468, 472, 415, 416, 417, 419, 420, 422,
    425, 426, 423, 424, 428, 429, 432, 439, 430, 431, 433, 436, 434, 435, 437, 438,
    440, 441, 443, 444, 445, 446, 447, 448, 449, 451, 452, 453, 454, 455, 457, 458,
    459, 461, 462, 463, 464, 465, 466, 467, 469, 470, 471, 473, 474, 475, 477, 478,
    479, 480, 481, 486, 492, 482, 483, 484, 485, 487, 488, 489, 490, 491, 493, 496,
    494, 495, 497, 531, 539, 545, 552, 561, 579, 498, 530, 499, 502, 503, 504, 505,
    508, 512, 515, 518, 519, 520, 521, 529, 500, 501, 506, 507, 509, 510, 511, 513,
    514, 516, 517, 522, 523, 524, 525, 526, 527, 528, 532, 538, 533, 536, 537, 534,
    535, 540, 541, 542, 543, 230, 231, 544, 546, 547, 548, 230, 231, 549, 550, 551,
    553, 557, 554, 555, 556, 558, 57, 559, 560, 562, 563, 564, 572, 565, 566, 567,
    230, 231, 568, 57, 569, 570, 571, 573, 576, 577, 578, 574, 575, 580, 581, 582,
    583, 585, 596, 602, 699, 702, 705, 721, 725, 731, 740, 747, 759, 770, 774, 780,
    784, 789, 792, 795, 799, 802, 805, 813, 824, 586, 587, 588, 589, 590, 591, 594,
    592, 593, 595, 597, 598, 599, 600, 601, 603, 604, 605, 606, 607, 620, 629, 632,
    638, 642, 655, 673, 677, 681, 684, 687, 695, 608, 611, 612, 613, 616, 609, 610,
    614, 615, 617, 618, 619, 621, 622, 623, 624, 625, 626, 627, 628, 630, 631, 633,
    636, 637, 634, 635, 639, 640, 641, 643, 644, 645, 650, 651, 652, 646, 647, 648,
    649, 653, 230, 231, 654, 656, 657, 665, 668, 658, 659, 660, 661, 662, 663, 664,
    666, 667, 669, 670, 671, 672, 674, 675, 676, 678, 679, 680, 682, 683, 685, 686,
    688, 689, 690, 691, 692, 693, 694, 696, 697, 698, 700, 701, 703, 704, 706, 707,
    708, 709, 714, 710, 711, 712, 713, 715, 718, 716, 717, 719, 720, 722, 723, 724,
    726, 727, 728, 729, 730, 732, 735, 736, 733, 734, 737, 738, 739, 741, 742, 745,
    746, 743, 744, 748, 757, 758, 749, 750, 751, 754, 752, 753, 755, 756, 760, 767,
    768, 769, 761, 762, 763, 764, 765, 766, 771, 772, 773, 775, 776, 777, 778, 779,
    781, 782, 783, 785, 786, 787, 788, 790, 791, 793, 794, 796, 797, 798, 800, 801,
    803, 804, 806, 807, 808, 809, 810, 811, 812, 814, 815, 816, 817, 818, 822, 823,
    819, 820, 821, 825, 829, 830, 826, 827, 828, 831, 230, 231, 832, 57, 833, 837,
    838, 834, 835, 836, 840, 841, 842, 843, 844, 845, 846, 847, 848, 849, 851, 857,
    850, 57, 852, 853, 854, 855, 856, 858, 859, 860, 862, 924, 863, 876, 880, 883,
    889, 895, 898, 901, 904, 919, 864, 869, 875, 865, 866, 867, 868, 870, 871, 872,
    873, 874, 877, 878, 879, 881, 882, 884, 885, 886, 887, 888, 890, 891, 892, 893,
    894, 896, 897, 899, 900, 902, 903, 905, 906, 907, 908, 916, 909, 912, 910, 911,
    913, 914, 915, 917, 918, 920, 921, 922, 923, 926, 927, 928, 932, 929, 57, 930,
    931, 933, 934, 938, 942, 952, 935, 936, 937, 939, 940, 941, 943, 944, 945, 946,
    949, 950, 951, 947, 948, 953, 954, 955, 956, 957, 959, 960, 961, 962, 1005, 963,
    964, 965, 966, 970, 974, 978, 984, 993, 998, 967, 968, 969, 971, 972, 973, 975,
    976, 977, 979, 980, 981, 982, 983, 985, 986, 987, 230, 231, 988, 57, 989, 990,
    991, 992, 994, 995, 996, 230, 231, 997, 57, 999, 1000, 1001, 1002, 1003, 1004, 1006,
    1007, 1008, 1009, 1010, 1015, 1016, 1017, 1018, 1011, 1012, 1013, 1014, 57, 1021, 1055, 1059,
    1063, 1072, 1077, 1022, 1023, 1024, 1025, 1026, 1030, 1038, 1043, 1051, 1027, 1028, 1029, 1031,
    1032, 1033, 230, 231, 1034, 57, 1035, 1036, 1037, 1039, 1040, 1041, 230, 231, 1042, 57,
    1044, 1045, 1046, 1047, 1048, 1049, 1050, 1052, 1053, 1054, 1056, 1057, 1058, 1060, 1061, 1062,
    1064, 1065, 1066, 230, 231, 1067, 57, 1068, 1069, 1070, 1071, 1073, 1074, 1075, 230, 231,
    1076, 57, 1078, 1079, 1080, 1081, 1082, 1083, 1084, 1086, 1087, 1088, 1089, 1090, 1111, 1118,
    1091, 1092, 1093, 1094, 1096, 1103, 1107, 1095, 57, 1097, 1098, 1100, 1101, 1102, 1099, 1104,
    1105, 1106, 1108, 1109, 1110, 1112, 1113, 1114, 1115, 1116, 1117, 1119, 1120, 1121, 1122, 1123,
    1128, 1129, 1130, 1131, 1124, 1125, 1126, 1127, 57, 1132, 1134, 1135, 1136, 1137, 1138, 1139,
    1140, 1144, 1141, 1142, 1143, 1145, 1154, 1158, 1146, 1147, 1148, 57, 1149, 1150, 1151, 1152,
    1153, 1155, 1156, 1157, 230, 231, 1159, 1160, 1161, 1162, 230, 231, 1164, 1191, 1165, 1171,
    1186, 1166, 1167, 1168, 1169, 1170, 1172, 1173, 1174, 1175, 1183, 1176, 1179, 1177, 1178, 1180,
    1181, 1182, 1184, 1185, 1187, 1188, 1189, 1190, 1193, 1194, 1195, 1199, 1196, 57, 1197, 1198,
    1200, 1201, 1205, 1209, 1224, 1234, 1202, 1203, 1204, 1206, 1207, 1208, 1210, 1211, 1212, 1213,
    1221, 1214, 1217, 1215, 1216, 1218, 57, 1219, 1220, 1222, 1223, 1225, 1226, 1227, 1228, 1231,
    1232, 1233, 1229, 1230, 1235, 1236, 1237, 1238, 1239, 1241, 1242, 1243, 1244, 1245, 1248, 1252,
    1256, 1260, 1264, 1246, 1247, 1249, 1250, 1251, 57, 1253, 1254, 1255, 57, 1257, 1258, 1259,
    57, 1261, 1262, 1263, 1265, 1266, 1267, 1268, 1269, 1270, 1271, 1273, 1274, 1277, 1275, 1276,
    1278, 1321, 1327, 1279, 1284, 1285, 1280, 1281, 1282, 1283, 1286, 1289, 1292, 1295, 1298, 1301,
    1304, 1307, 1311, 1317, 1287, 1288, 1290, 1291, 1293, 1294, 1296, 1297, 1299, 1300, 1302, 1303,
    1305, 1306, 1308, 1309, 1310, 1312, 1313, 1314, 1315, 1316, 1318, 1319, 1320, 1322, 1323, 1324,
    1325, 1326, 1328, 1334, 1338, 1329, 1330, 1331, 1332, 1333, 1335, 1336, 1337, 1339, 1357, 1340,
    1341, 1342, 1343, 1354, 1344, 1347, 1345, 1346, 1348, 1349, 1350, 57, 1351, 1352, 1353, 1355,
    1356, 1358, 1359, 1360, 1361, 1363, 1364, 1365, 1366, 1367, 1371, 1368, 1369, 1370, 1372, 1373,
    1374, 1376, 1379, 1384, 1392, 1400, 1404, 1409, 1414, 1417, 1420, 1377, 1378, 1380, 1381, 1382,
    1383, 1385, 1389, 1386, 1387, 1388, 1390, 1391, 1393, 1397, 1394, 1395, 1396, 1398, 1399, 1401,
    1402, 1403, 1405, 1406, 1407, 1408, 1410, 1411, 1412, 1413, 1415, 1416, 1418, 1419, 1421, 1422,
    1423, 1424, 1425, 1427, 1428, 1429, 1430, 1431, 1432, 1434, 1441, 1445, 1433, 1435, 1436, 1437,
    1438, 1439, 1440, 1442, 1443, 1444, 1446, 1447, 1448, 1450, 1451, 1452, 1453, 1454, 1455, 1465,
    1469, 1456, 1457, 1458, 1459, 1462, 1463, 1464, 1460, 1461, 1466, 1467, 1468, 1470, 1471, 1472,
    1475, 1473, 1474, 1476, 1477, 1479, 1480, 1481, 1482, 1483, 1486, 1497, 1503, 1504, 1505, 1508,
    1514, 1522, 1484, 1485, 1487, 1491, 1492, 1494, 1488, 1489, 1490, 1493, 57, 1495, 1496, 1498,
    1499, 1500, 1501, 1502, 1506, 1507, 1509, 1510, 1511, 1512, 1513, 230, 231, 1515, 1516, 1517,
    1518, 1519, 1520, 1521, 1523, 1524, 1525, 1527, 1528, 1529, 1530, 1544, 1531, 1532, 1533, 1536,
    1534, 1535, 1537, 1538, 1539, 57, 1540, 1541, 1542, 1543, 1545, 1546, 1547, 1548, 1550, 1561,
    1567, 1664, 1667, 1670, 1686, 1690, 1696, 1705, 1712, 1724, 1735, 1739, 1745, 1749, 1754, 1757,
    1760, 1764, 1767, 1770, 1778, 1551, 1552, 1553, 1554, 1555, 1556, 1559, 1557, 1558, 1560, 1562,
    1563, 1564, 1565, 1566, 1568, 1569, 1570, 1571, 1572, 1585, 1594, 1597, 1603, 1607, 1620, 1638,
    1642, 1646, 1649, 1652, 1660, 1573, 1576, 1577, 1578, 1581, 1574, 1575, 1579, 1580, 1582, 1583,
    1584, 1586, 1587, 1588, 1589, 1590, 1591, 1592, 1593, 1595, 1596, 1598, 1601, 1602, 1599, 1600,
    1604, 1605, 1606, 1608, 1609, 1610, 1615, 1616, 1617, 1611, 1612, 1613, 1614, 1618, 230, 231,
    1619, 1621, 1622, 1630, 1633, 1623, 1624, 1625, 1626, 1627, 1628, 1629, 1631, 1632, 1634, 1635,
    1636, 1637, 1639, 1640, 1641, 1643, 1644, 1645, 1647, 1648, 1650, 1651, 1653, 1654, 1655, 1656,
    1657, 1658, 1659, 1661, 1662, 1663, 1665, 1666, 1668, 1669, 1671, 1672, 1673, 1674, 1679, 1675,
    1676, 1677, 1678, 1680, 1683, 1681, 1682, 1684, 1685, 1687, 1688, 1689, 1691, 1692, 1693, 1694,
    1695, 1697, 1700, 1701, 1698, 1699, 1702, 1703, 1704, 1706, 1707, 1710, 1711, 1708, 1709, 1713,
    1722, 1723, 1714, 1715, 1716, 1719, 1717, 1718, 1720, 1721, 1725, 1732, 1733, 1734, 1726, 1727,
    1728, 1729, 1730, 1731, 1736, 1737, 1738, 1740, 1741, 1742, 1743, 1744, 1746, 1747, 1748, 1750,
    1751, 1752, 1753, 1755, 1756, 1758, 1759, 1761, 1762, 1763, 1765, 1766, 1768, 1769, 1771, 1772,
    1773, 1774, 1775, 1776, 1777, 1779, 1780, 1781, 1782, 1783, 1787, 1788, 1784, 1785, 1786, 1790,
    1791, 1792, 1793, 1795, 1801, 1814, 1815, 1796, 1797, 1798, 1799, 1800, 1802, 1805, 1806, 1807,
    1803, 1804, 1808, 1813, 1809, 1812, 1810, 1811, 1816, 1817, 1820, 1818, 1819, 1822, 1823, 1824,
    1825, 1845, 1873, 1826, 1827, 1828, 1829, 1830, 1831, 1833, 1839, 1832, 57, 1834, 1835, 1836,
    1837, 1838, 1840, 1841, 1842, 1843, 1844, 1846, 1847, 1848, 1849, 1853, 1857, 1867, 1850, 1851,
    1852, 1854, 1855, 1856, 1858, 1859, 1860, 1861, 1862, 1866, 1863, 1864, 1865, 1868, 1869, 1870,
    1871, 1872, 1874, 1875, 1876, 1877, 1878, 1883, 1884, 1885, 1886, 1879, 1880, 1881, 1882, 57,
    1888, 1889, 1890, 1891, 1892, 1893, 1894, 1903, 1912, 1919, 1929, 1936, 1940, 1948, 1895, 1896,
    1897, 1898, 1900, 1901, 1899, 57, 1902, 57, 1904, 1905, 1906, 1907, 1909, 1910, 1908, 57,
    1911, 57, 1913, 1914, 1915, 1916, 1917, 1918, 1920, 1921, 1922, 1923, 1925, 1926, 1927, 1928,
    1924, 57, 1930, 1931, 1932, 1933, 1934, 1935, 57, 1937, 1938, 1939, 1941, 1942, 1943, 1944,
    1945, 1946, 1947, 1949, 1950, 1951, 1953, 1954, 1955, 1956, 1957, 1958, 1959, 1963, 1960, 1961,
    1962, 1964, 1965, 1966, 1967, 1968, 1970, 1971, 1972, 1973, 1974, 1982, 1988, 1992, 1996, 2033,
    1975, 1976, 1977, 1978, 1979, 1980, 1981, 1983, 1984, 1985, 1986, 1987, 1989, 1990, 1991, 1993,
    1994, 1995, 1997, 57, 1998, 2007, 2020, 2026, 1999, 2000, 2001, 2002, 2003, 2006, 2004, 2005,
    2008, 2009, 2010, 2011, 2014, 2015, 2016, 2019, 2012, 2013, 2017, 2018, 2021, 2022, 2023, 2024,
    2025, 2027, 2028, 2029, 2030, 2031, 2032, 2034, 2035, 2036, 2037, 2038, 2039, 2042, 2040, 2041,
    2044, 2045, 2046, 2047, 2048, 2049, 2065, 2073, 2079, 2083, 2087, 2096, 2050, 2051, 2052, 2053,
    2054, 2062, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2063, 2064, 2066, 2067, 2068, 2069, 2070,
    2071, 2072, 2074, 2075, 2076, 2077, 2078, 2080, 2081, 2082, 2084, 2085, 2086, 2088, 2089, 2090,
    230, 231, 2091, 57, 2092, 2093, 2094, 2095, 2097, 2098, 2099, 230, 231, 2100, 57, 2106,
    2109, 2118, 2107, 2108, 2110, 2117, 2111, 230, 231, 2112, 57, 2113, 2116, 2114, 2115, 2119,
    57, 2120, 2121, 2122, 230, 231, 2123, 57, 2129, 2130, 2131, 2132, 2133, 2134, 2135, 2136,
    2137, 2138, 2140, 2154, 2155, 2141, 2142, 2143, 2144, 2145, 2149, 2150, 2146, 2147, 2148, 2151,
    2152, 2153, 2159, 2154, 2155, 2160, 57, 2163, 2154, 2155, 223, 2165, 2166, 239, 2168, 2181,
    2188, 2200, 2189, 2190, 2191, 2192, 2193, 2194, 2195, 2196, 2197, 2211, 2212, 2213, 2214, 2215,
    2222, 2223, 2226, 2227, 2231, 2232, 2235, 2236, 2241, 230, 231, 2242, 57, 2244, 230, 231,
    2245, 57, 2248, 230, 231, 2258, 230, 231, 2260, 2261, 2262, 2263, 2266, 2268, 2264, 2265,
    2267, 230, 231, 2269, 2279, 2270, 2271, 2274, 2275, 2276, 2277, 2278, 2272, 2273, 2280, 2281,
    2282, 2283, 2284, 2285, 2289, 2290, 2294, 2295, 2298, 2308, 2299, 2300, 2301, 2305, 239, 2307,
    2302, 2309, 2310, 2300, 2311, 2315, 239, 2317, 2312, 2342, 2343, 2344, 2345, 2350, 2346, 2347,
    2348, 2349, 2351, 2352, 2353, 2361, 2362, 2363, 2364, 2370, 2371, 2372, 2373, 2375, 2378, 2441,
    2446, 2376, 2377, 2379, 2382, 2389, 2397, 2403, 2415, 2429, 2440, 2380, 2381, 2383, 2384, 2386,
    2385, 230, 231, 2387, 2388, 2390, 2391, 2392, 2393, 2394, 2395, 2396, 2398, 2399, 2400, 2401,
    2402, 2404, 2405, 2410, 2411, 2412, 2406, 2407, 2408, 2409, 2413, 230, 231, 2414, 2416, 2417,
    2421, 2424, 2418, 2419, 2420, 2422, 2423, 2425, 2426, 2427, 2428, 2430, 2431, 2432, 2435, 2433,
    2434, 2436, 2437, 2438, 2439, 2442, 2443, 2444, 2445, 2447, 2450, 2448, 2449, 2451, 2452, 2454,
    2455, 2456, 2457, 2458, 2459, 2460, 2464, 2465, 2466, 2471, 2467, 2468, 2469, 2470, 2472, 2473,
    2474, 2475, 2476, 2477, 2490, 2491, 2492, 2500, 2501, 2504, 2505, 2506, 2507, 2609, 2508, 2513,
    2520, 2523, 2530, 2533, 2536, 2541, 2544, 2547, 2600, 2509, 2512, 2510, 2511, 2514, 2515, 2516,
    230, 231, 2517, 2518, 2519, 2521, 2522, 2524, 2525, 2526, 2527, 2528, 2529, 2531, 2532, 2534,
    2535, 2537, 2538, 2539, 2540, 2542, 2543, 2545, 2546, 2548, 2596, 2597, 2549, 2550, 2551, 2552,
    2553, 2554, 2555, 2556, 2557, 2558, 2559, 2560, 2561, 2562, 2563, 2564, 2567, 2570, 2574, 2577,
    2580, 2584, 2588, 2592, 2565, 2566, 2568, 2569, 2571, 2572, 2573, 2575, 2576, 2578, 2579, 2581,
    2582, 2583, 2585, 2586, 2587, 2589, 2590, 2591, 2593, 2594, 2595, 2598, 2599, 2601, 2605, 2606,
    2602, 2603, 2604, 2607, 230, 231, 2608, 2610, 2611, 2612, 2613, 2620, 2621, 2622, 2623, 2624,
    2625, 2626, 2627, 2631, 2636, 2637, 2638, 2639, 2640, 2641, 2642, 2643, 2628, 2629, 2630, 2632,
    2633, 2634, 2635, 2649, 2651, 2652, 2650, 230, 231, 2658, 2659, 2663, 2664, 2665, 2668, 2669,
    2670, 2671, 2672, 223, 2673, 2300, 2674, 239, 2700, 239, 2675, 57, 2676, 2682, 2693, 2677,
    2678, 2679, 2680, 2681, 2683, 2686, 2687, 2688, 2684, 230, 231, 2685, 57, 2689, 230, 231,
    2690, 57, 2691, 2692, 2694, 2697, 2698, 2695, 230, 231, 2696, 57, 2699, 230, 231, 2701,
    2702, 2705, 2721, 2706, 2718, 2707, 230, 231, 2708, 57, 2709, 2714, 2717, 2710, 2711, 2713,
    2712, 230, 231, 2715, 2716, 2719, 2720, 2722, 2723, 2724, 2735, 2736, 2737, 2804, 2842, 2850,
    2738, 2743, 2744, 2755, 2801, 2739, 2740, 2741, 230, 231, 2742, 57, 2745, 2753, 2746, 2747,
    2752, 2748, 2749, 2750, 2751, 2754, 2756, 2757, 2758, 230, 231, 2759, 57, 2760, 2763, 2766,
    2769, 2772, 2775, 2778, 2781, 2788, 2793, 2798, 2761, 2762, 2764, 2765, 2767, 2768, 2770, 2771,
    2773, 2774, 2776, 2777, 2779, 2780, 2782, 2783, 2784, 2787, 2785, 230, 231, 2786, 57, 2789,
    2790, 2791, 230, 231, 2792, 57, 2794, 2795, 2796, 230, 231, 2797, 57, 2799, 2800, 2802,
    2803, 2738, 2805, 2806, 2811, 2807, 2808, 2809, 2810, 2812, 2813, 2814, 2816, 2820, 2824, 2815,
    2817, 2818, 2819, 2821, 2822, 2823, 2825, 2826, 2827, 2828, 2832, 2836, 2829, 2830, 2831, 2833,
    2834, 2835, 2837, 2838, 2839, 2840, 2841, 57, 2843, 2846, 2847, 2755, 2738, 2844, 2845, 230,
    231, 2745, 2848, 2849, 2851, 2854, 2855, 2860, 2738, 2852, 2853, 230, 231, 2856, 2857, 2858,
    2859, 2861, 2862, 2863, 2865, 2869, 2873, 2864, 2866, 2867, 2868, 2870, 2871, 2872, 2874, 2875,
    2876, 2877, 2881, 2885, 2878, 2879, 2880, 2882, 2883, 2884, 2886, 2887, 2888, 2889, 2892, 2890,
    230, 231, 2891, 57, 2897, 2898, 2899, 2900, 2901, 2903, 2902, 230, 231, 2905, 2906, 2907,
    2911, 2923, 2908, 2909, 2910, 230, 231, 2912, 2917, 2920, 2913, 2914, 2915, 2916, 2918, 2919,
    2921, 2922, 2924, 2927, 2925, 2926, 2928, 2929, 2931, 2932, 2933, 2934, 2935, 2936, 2965, 2937,
    230, 231, 2938, 57, 2939, 2943, 2947, 2951, 2957, 2961, 2940, 2941, 2942, 2944, 2945, 2946,
    2948, 2949, 2950, 2952, 2953, 2954, 2955, 2956, 2958, 2959, 2960, 2962, 2963, 2964, 2966, 2967,
    2969, 2970, 2971, 2972, 2973, 2974, 2978, 2982, 2988, 3002, 3008, 3012, 3017, 2975, 2976, 2977,
    2979, 2980, 2981, 2983, 2984, 2985, 2986, 2987, 2989, 2993, 2990, 2991, 2992, 2994, 2998, 2995,
    2996, 2997, 2999, 3000, 3001, 3003, 3004, 3005, 3006, 3007, 3009, 3010, 3011, 3013, 3014, 3015,
    3016, 3018, 3019, 3020, 3022, 3023, 3024, 3025, 3027, 3028, 3031, 3034, 3026, 3029, 3030, 3032,
    3033, 3035, 3036, 3039, 3037, 3038, 3040, 3043, 3041, 3042, 3044, 3045, 3047, 3048, 3049, 3050,
    3051, 3052, 3055, 3058, 3061, 3053, 3054, 3056, 3057, 3059, 3060, 3063, 3064, 3065, 223, 3066,
    3067, 239, 223, 3068, 3069, 239, 3071, 3072, 3073, 3075, 3101, 3076, 3077, 3078, 3079, 3092,
    3097, 3100, 3080, 230, 231, 3081, 57, 3082, 3091, 3083, 3084, 3085, 3086, 3089, 3087, 3088,
    3090, 3093, 3094, 3095, 230, 231, 3096, 57, 3098, 3099, 3102, 3103, 3104, 3105, 3126, 3129,
    3106, 3107, 3108, 3109, 3120, 3110, 230, 231, 3111, 57, 3112, 3119, 3113, 3114, 3117, 3115,
    3116, 3118, 3121, 3125, 3122, 3123, 3124, 3127, 3128, 3131, 3132, 3133, 3134, 3135, 3136, 3137,
    3138, 3156, 3139, 3140, 3141, 3152, 3155, 3142, 3143, 3144, 230, 231, 3145, 57, 3146, 3147,
    3148, 3149, 3150, 3151, 3153, 3154, 3158, 3159, 3160, 3161, 3162, 3165, 3166, 3167, 3168, 3171,
    3174, 3179, 3183, 3186, 3189, 3163, 3164, 3169, 3170, 3172, 230, 231, 3173, 57, 3175, 3176,
    3177, 230, 231, 3178, 57, 3180, 3181, 3182, 3184, 3185, 3187, 3188, 3190, 3191, 3194, 3195,
    3196, 3197, 3198, 3199, 3200, 3201, 3204, 3207, 3210, 3211, 3226, 3202, 3203, 3205, 3206, 3208,
    3209, 3212, 3215, 3220, 3221, 3213, 3214, 3214, 230, 231, 3216, 3217, 3217, 230, 231, 3218,
    3219, 3222, 3223, 3223, 230, 231, 3224, 3225, 3228, 3229, 3230, 3231, 3232, 3233, 3245, 3246,
    3234, 3235, 3236, 230, 231, 3237, 57, 3238, 3239, 3240, 3241, 3242, 3243, 2154, 2155, 3244,
    57, 3248, 3249, 3250, 3252, 3251, 3253, 230, 231, 3254, 57, 3255, 3261, 3266, 3271, 3280,
    3281, 3256, 3259, 3260, 3257, 3258, 3262, 3263, 3264, 3265, 3267, 3268, 3269, 3270, 3272, 3278,
    3279, 3273, 3274, 3275, 3276, 3277, 3283, 3284, 3285, 3286, 3287, 3288, 3293, 3301, 3310, 3314,
    3289, 3290, 3291, 3292, 3294, 3295, 3296, 3297, 3298, 3299, 3300, 3302, 3303, 3304, 57, 3305,
    3306, 3307, 3308, 3309, 3311, 3312, 3313, 230, 231, 3315, 3316, 3317, 3318, 230, 231, 3320,
    3321, 3322, 3323, 3324, 3325, 3327, 3328, 3329, 3330, 3339, 3331, 3335, 3332, 3333, 3334, 3336,
    3337, 3338, 57, 3340, 3341, 3343, 3344, 3347, 3348, 3345, 3346, 3349, 3350, 3351, 3354, 3357,
    3360, 3363, 3366, 3369, 3372, 3376, 3382, 3386, 3390, 3394, 3397, 3400, 3403, 3352, 3353, 3355,
    3356, 3358, 3359, 3361, 3362, 3364, 3365, 3367, 3368, 3370, 3371, 3373, 3374, 3375, 3377, 3378,
    3379, 3380, 3381, 3383, 3384, 3385, 3387, 3388, 3389, 3391, 3392, 3393, 3395, 3396, 3398, 3399,
    3401, 3402, 3404, 3405, 3407, 3408, 3409, 3410, 3411, 3415, 3412, 3413, 3414, 3417, 3420, 3425,
    3433, 3441, 3446, 3449, 3454, 3418, 3419, 3421, 3422, 3423, 3424, 3426, 3430, 3427, 3428, 3429,
    3431, 3432, 3434, 3438, 3435, 3436, 3437, 3439, 3440, 3442, 3443, 3444, 3445, 3447, 3448, 3450,
    3451, 3452, 3453, 3455, 3456, 3457, 3458, 3459, 3461, 3462, 3463, 3464, 3465, 3466, 3467, 3469,
    3470, 3471, 3472, 3473, 3476, 3479, 3480, 3481, 3474, 3475, 3477, 3478, 3483, 3484, 3485, 3488,
    3495, 3496, 3500, 3501, 3486, 3487, 3489, 230, 231, 3490, 57, 3491, 3492, 3493, 3494, 3497,
    57, 3498, 3499, 3503, 3504, 3505, 3506, 3507, 3508, 3509, 3511, 3510, 57, 3512, 3513, 3515,
    3516, 3523, 3524, 3525, 3526, 3562, 3563, 3578, 3517, 3522, 3518, 3521, 3519, 3520, 3527, 3530,
    3533, 3548, 3559, 3528, 230, 231, 3529, 57, 3531, 3532, 3534, 3545, 3535, 3536, 3537, 230,
    231, 3538, 57, 3539, 3540, 3541, 3542, 3543, 3544, 3546, 3547, 3549, 3550, 3551, 3552, 3555,
    3558, 3553, 3554, 3556, 3557, 3560, 3561, 3564, 3567, 3572, 3573, 3565, 3566, 3566, 230, 231,
    3568, 3569, 3569, 230, 231, 3570, 3571, 3574, 3575, 3575, 230, 231, 3576, 3577, 3580, 3581,
    3588, 3589, 3590, 3591, 3643, 3582, 3587, 3583, 3586, 3584, 3585, 3592, 3608, 3622, 3593, 3603,
    3594, 230, 231, 3595, 57, 3596, 3597, 3598, 3599, 3600, 3601, 3602, 3604, 3605, 3606, 230,
    231, 3607, 57, 3609, 3610, 3611, 3612, 230, 231, 3613, 57, 3614, 3621, 3615, 3616, 3619,
    3617, 3618, 3620, 3623, 3624, 3625, 3626, 3637, 3627, 230, 231, 3628, 57, 3629, 3636, 3630,
    3631, 3634, 3632, 3633, 3635, 3638, 3642, 3639, 3640, 3641, 3644, 3663, 3666, 3673, 3684, 3645,
    3646, 3647, 3651, 3648, 3649, 3650, 3652, 230, 231, 3653, 3654, 57, 3655, 3658, 3656, 3657,
    3659, 3662, 3660, 3661, 3664, 3665, 3667, 3670, 3668, 3669, 3671, 3672, 3674, 3675, 3676, 3677,
    3680, 3683, 3678, 3679, 3681, 3682, 3685, 3686, 3688, 3689, 3690, 3691, 3692, 3693, 3694, 230,
    231, 3695, 3699, 3696, 3697, 3698, 3700, 3701, 3702, 3704, 3705, 3706, 3707, 3708, 3709, 3715,
    3729, 3730, 3731, 3773, 3710, 3711, 3712, 3713, 3714, 3716, 3728, 3717, 3718, 3719, 3720, 3721,
    3722, 3723, 3724, 3725, 57, 3732, 3735, 3748, 3763, 3769, 3733, 3734, 3736, 3739, 3737, 3738,
    3740, 3741, 3742, 3745, 3743, 3744, 3746, 3747, 3749, 3750, 3753, 3754, 3755, 3756, 3751, 3752,
    3757, 3760, 3761, 3762, 3758, 3759, 3764, 3765, 3766, 3767, 3768, 3770, 3771, 3772, 230, 231,
    3774, 3775, 3778, 3776, 3777, 3780, 3781, 3782, 3783, 3788, 3784, 3785, 3786, 3787, 3789, 230,
    231, 3790, 57, 3792, 3793, 3794, 3796, 3797, 3798, 3799, 3803, 3804, 3805, 3800, 3801, 3802,
    3807, 3808, 3809, 3811, 3812, 3813, 3814, 3815, 3816, 3817, 3818, 3821, 3822, 3827, 3819, 3820,
    3823, 3826, 3824, 3825, 3825, 230, 231, 3836, 3839, 3837, 3838, 3840, 230, 231, 3855, 3856,
    3860, 3861, 3862, 3864, 3867, 3961, 3865, 3866, 3868, 3869, 3870, 3871, 3872, 3958, 3873, 3874,
    3875, 3876, 3877, 3878, 3879, 3880, 3881, 3882, 3883, 3884, 3887, 3888, 3889, 3890, 3891, 3892,
    3893, 3897, 3902, 3903, 3918, 3919, 3922, 3930, 3935, 3938, 3943, 3944, 3945, 3948, 3949, 3950,
    3951, 3952, 3953, 3954, 3955, 3885, 3886, 3894, 3896, 3895, 3898, 3901, 3899, 3900, 3904, 3917,
    3905, 3916, 3906, 3907, 3908, 3911, 3912, 3915, 3909, 3910, 3913, 3914, 3920, 3921, 3923, 3924,
    3925, 3926, 3927, 3928, 3929, 3931, 3932, 3933, 3934, 3936, 3937, 3939, 3940, 3942, 3941, 3946,
    3947, 3956, 3957, 3959, 3960, 3962, 3964, 3965, 3963, 3970, 3971, 230, 231, 3972, 57, 3973,
    3974, 3976, 3977, 3979, 3980, 3981, 3982, 3986, 3983, 3984, 3985, 3987, 230, 231, 4003, 4004,
    4005, 4006, 4007, 4013, 4014, 4020, 4021, 4023, 4024, 4025, 4026, 4028, 230, 231, 4029, 57,
    4034, 4035, 4036, 4037, 4038, 4039, 4040, 4046, 4047, 4054, 4041, 223, 4042, 239, 4043, 57,
    4044, 4045, 4048, 4053, 4049, 4050, 4051, 4052, 4055, 223, 4056, 239, 4057, 4058, 4059, 57,
    4060, 4061, 4072, 4073, 4074, 4084, 4085, 4086, 4087, 4088, 4089, 4090, 4099, 4100, 4101, 4104,
    4102, 4103, 4105, 4106, 4113, 4114, 4115, 4116, 4123, 4117, 57, 4118, 4119, 4120, 4121, 4122,
    4125, 4126, 4128, 4129, 4130, 4131, 4135, 4132, 4133, 4134, 230, 231, 4137, 4138, 4139, 4140,
    4141, 4143, 4144, 4145, 4146, 4147, 4148, 4149, 4150, 230, 231, 4152, 4153, 4154, 4155, 4157,
    4156, 57, 4159, 4160, 4161, 4162, 4163, 4165, 4166, 4167, 4168, 4169, 4171, 4170, 57, 4173,
    4174, 4175, 4176, 4180, 4177, 57, 4178, 4179, 4182, 4183, 4184, 4185, 4186, 4188, 4187, 57,
    4191, 4192, 4193, 4194, 4195, 4197, 4196, 57, 4199, 4200, 4201, 4202, 4204, 4205, 4206, 4207,
    4213, 4208, 57, 4209, 4210, 4211, 4212, 4215, 4216, 4217, 4218, 4219, 4220, 4221, 4223, 4224,
    4225, 4226, 4230, 4227, 57, 4228, 4229, 4231, 4232, 4234, 4235, 4236, 4237, 4239, 4238, 57,
    4241, 4242, 4243, 4244, 4246, 4247, 4248, 4249, 4250, 4252, 4253, 4254, 4255, 4257, 4256, 57,
    4259, 4260, 4261, 4262, 4264, 4263, 57, 4265, 4266, 4268, 4269, 4270, 4271, 4272, 4273, 4274,
    4276, 4277, 4278, 4279, 4280, 4282, 4281, 57, 4284, 4285, 4286, 4287, 4288, 4289, 4290, 4292,
    4293, 4294, 4295, 4296, 4297, 4298, 4300, 4301, 4302, 4303, 4305, 4304, 57, 4307, 4308, 4309,
    4310, 4312, 4313, 4314, 4315, 4317, 4316, 57, 4322, 2300, 4323, 239, 230, 231, 4329, 4330,
    4347, 4348, 4349, 4353, 4354, 4355, 4356, 230, 231, 4357, 57, 4363, 4375, 4364, 4374, 4365,
    4366, 4367, 4368, 4369, 4370, 4371, 4372, 4373, 4376, 4377, 4378, 4379, 4380, 4381, 4383, 4384,
    4393, 4385, 4390, 4386, 4389, 4387, 4388, 4391, 230, 231, 4392, 57, 4394, 4395, 4396, 4397,
    4401, 4402, 4403, 4404, 4405, 4408, 4409, 4410, 4414, 4415, 4416, 4417, 4423, 4406, 4407, 4411,
    4412, 4413, 4418, 4419, 4420, 4421, 4422, 4424, 4425, 4427, 4430, 4431, 4432, 4428, 4429, 4434,
    4435, 4436, 4437, 4438, 4442, 4439, 4440, 4441, 4444, 4447, 4445, 4446, 4449, 4450, 4451, 4464,
    4452, 4456, 4462, 4453, 4454, 4455, 230, 231, 4457, 4460, 4458, 4459, 4461, 4465, 4466, 4468,
    4503, 4469, 4470, 4471, 4483, 4484, 4485, 4492, 4500, 4501, 4472, 230, 231, 4473, 4474, 4475,
    4476, 4477, 57, 4478, 4479, 4480, 4481, 4482, 4486, 4489, 4487, 4488, 4490, 230, 231, 4491,
    57, 4493, 4494, 4495, 4496, 4497, 4498, 4499, 4505, 4506, 4507, 4508, 4509, 4510, 4511, 4515,
    4516, 4521, 4517, 4518, 4519, 4520, 4520, 230, 231, 4522, 4523, 4531, 4532, 4535, 4539, 4542,
    4533, 4534, 4536, 4537, 4538, 4540, 4541, 4543, 4544, 4545, 4546, 4551, 4552, 4553, 4554, 4555,
    4556, 4557, 4560, 4561, 4562, 230, 231, 4563, 4564, 4574, 4575, 4585, 4589, 4576, 4577, 4581,
    4582, 4578, 4579, 4580, 4583, 4584, 4586, 4587, 4588, 57, 4590, 4591, 4592, 4593, 4600, 4601,
    4602, 4604, 4605, 4608, 4611, 4609, 4610, 4612, 4615, 4622, 4630, 4613, 4614, 4616, 4617, 4619,
    4618, 230, 231, 4620, 4621, 4623, 4624, 4625, 4626, 4627, 4628, 4629, 4631, 4632, 4633, 4634,
    4635, 4637, 4640, 4638, 4639, 4641, 4642, 4644, 4643, 230, 231, 4645, 4646, 4650, 4651, 4668,
    4652, 4665, 4653, 4656, 4654, 4655, 4657, 4662, 4658, 4659, 4660, 4661, 4663, 4664, 4666, 4652,
    4667, 4652, 4669, 4670, 4671, 4674, 4675, 4676, 4672, 4673, 4677, 4678, 4680, 4681, 4682, 4687,
    4688, 4689, 57, 4691, 4692, 4693, 4696, 4699, 4700, 4701, 4702, 4703, 4704, 4705, 4706, 4707,
    4694, 4695, 4697, 4698, 4709, 4727, 4710, 4711, 4714, 4715, 4721, 4725, 4726, 4712, 4713, 4713,
    230, 231, 4716, 4717, 4718, 4719, 4720, 4722, 4723, 4724, 4728, 230, 231, 4709, 4729, 4730,
    4732, 4733, 4733, 230, 231, 2300, 4734, 239, 4745, 4746, 4757, 4735, 4736, 4737, 4738, 230,
    231, 4740, 4741, 4743, 4744, 4747, 4748, 4749, 4750, 4752, 4753, 4755, 4756, 4763, 4764, 4765,
    4766, 4767, 4768, 4769, 4771, 4772, 4775, 4779, 4785, 4796, 4799, 4805, 4806, 4809, 4821, 4822,
    4823, 4824, 4773, 4774, 4774, 230, 231, 4776, 4777, 4778, 4780, 4781, 4783, 4784, 4782, 4786,
    4789, 4787, 4788, 4790, 4793, 4791, 4792, 4794, 4795, 4797, 4798, 4800, 4801, 4802, 4803, 4804,
    57, 4807, 4808, 4810, 4815, 4816, 4811, 4812, 4813, 4814, 4817, 4818, 4819, 4820, 4825, 57,
    4826, 4827, 4830, 4828, 4829, 4832, 4833, 230, 231, 4835, 4931, 4836, 4837, 4842, 4843, 4847,
    4850, 4863, 4869, 4875, 4878, 4881, 4884, 4899, 4907, 4910, 4838, 4839, 4840, 4841, 4844, 4845,
    4846, 4848, 4849, 4851, 4856, 4862, 4852, 4853, 4854, 4855, 4857, 4858, 4859, 4860, 4861, 4864,
    4865, 4866, 4867, 4868, 4870, 4871, 4872, 4873, 4874, 4876, 4877, 4879, 4880, 4882, 4883, 4885,
    4886, 4887, 4888, 4896, 4889, 4892, 4890, 4891, 4893, 57, 4894, 4895, 4897, 4898, 4900, 4901,
    4902, 4903, 4904, 4905, 4906, 4908, 4909, 4911, 4912, 223, 4913, 239, 4930, 4914, 4917, 4920,
    4926, 4915, 4916, 4918, 4919, 4921, 4925, 4922, 4923, 4924, 4927, 4928, 4929, 4932, 4933, 4934,
    230, 231, 4935, 57, 4939, 4944, 4940, 4941, 4942, 4945, 4946, 4949, 4954, 4966, 4950, 4951,
    4952, 4953, 4955, 4956, 4957, 4958, 4963, 4959, 4960, 4961, 4962, 4964, 4965, 4967, 4971, 4968,
    4969, 4970, 4973, 230, 231, 4974, 57, 4976, 4979, 4980, 4977, 4978, 4981, 4988, 4982, 4987,
    4983, 4984, 4985, 4986, 5002, 5003, 5017, 5018, 5019, 5032, 5004, 5008, 5011, 5016, 5005, 5006,
    5007, 5009, 5010, 5012, 5013, 5014, 5015, 5020, 5024, 5028, 5021, 5022, 5023, 5025, 5026, 5027,
    5029, 5030, 5031, 5033, 5034, 5035, 5036, 5037, 5038, 5042, 5043, 5047, 5048, 223, 5049, 239,
    5050, 57, 5051, 5052, 5053, 5054, 5055, 5056, 230, 231, 5058, 5059, 5066, 5069, 5070, 5071,
    5072, 5073, 5067, 5068, 5076, 5077, 57, 5078, 5079, 5080, 5081, 230, 231, 5085, 5086, 5087,
    5088, 230, 231, 5089, 57, 5090, 5091, 5097, 2300, 5098, 239, 5099, 5100, 5100, 230, 231,
    5102, 5105, 5106, 5107, 5108, 5103, 5104, 5129, 5130, 5133, 5134, 5135, 5138, 5141, 5139, 5140,
    5147, 5148, 5149, 5151, 5152, 5153, 5162, 5163, 5164, 5165, 5166, 5167, 5154, 5159, 5155, 5158,
    5156, 5157, 5160, 230, 231, 5161, 57, 5172, 5173, 5174, 5175, 5180, 5176, 230, 231, 5177,
    5179, 5178, 57, 5186, 5187, 5188, 5189, 5191, 5194, 5202, 5205, 5192, 5193, 5195, 5201, 5196,
    5197, 5198, 5199, 5200, 5203, 5204, 5206, 5207, 5208, 5209, 5211, 5216, 5212, 5213, 5214, 5215,
    230, 231, 5219, 5222, 5225, 5220, 5221, 5223, 5224, 5226, 5227, 5228, 5229, 5246, 5247, 5248,
    5249, 5250, 5251, 5257, 5262, 5267, 5252, 5253, 5256, 5254, 5255, 5258, 5261, 5259, 5260, 5263,
    5264, 5265, 5266, 5268, 223, 5269, 239, 5270, 5271, 5272, 57, 5273, 5274, 5290, 5291, 5292,
    5297, 5298, 5293, 5294, 5295, 5296, 5299, 5300, 5301, 5302, 5305, 5306, 5307, 5308, 5309, 5310,
    5312, 5313, 5317, 5318, 5314, 5315, 5316, 5322, 5323, 5324, 5325, 5326, 5327, 5329, 5330, 5331,
    5332, 5342, 5355, 5360, 5343, 5344, 223, 5345, 5346, 239, 5348, 5347, 5349, 5350, 5351, 5354,
    5352, 5353, 5356, 5357, 223, 5358, 5359, 239, 5361, 223, 5362, 239, 5365, 5368, 5369, 5370,
    5373, 5371, 5372, 5372, 230, 231, 5375, 5376, 5377, 5381, 5378, 5379, 5380, 5383, 223, 5384,
    239, 5385, 230, 231, 5386, 57, 5390, 5391, 5392, 5394, 5395, 5396, 5397, 5399, 5402, 5403,
    5400, 5401, 5404, 5413, 5414, 5415, 5416, 5433, 5434, 5439, 5442, 5443, 5440, 5441, 5444, 5445,
    5447, 5448, 5449, 5450, 5451, 5455, 5458, 5456, 5457, 5459, 5460, 5461, 5462, 5463, 5464, 5465,
    5466, 5467, 5469, 223, 5470, 5476, 239, 5471, 5472, 5472, 230, 231, 5473, 5474, 5475, 5477,
    5482, 5478, 5479, 5480, 5481, 5483, 5484, 5497, 5498, 5500, 5501, 5502, 5503, 5504, 5505, 5506,
    5507, 5508, 5509, 5510, 5515, 5516, 5525, 5526, 5527, 5528, 5532, 5558, 5529, 5531, 5530, 57,
    5533, 5534, 5557, 5535, 5538, 5541, 5544, 5548, 5549, 5553, 5554, 5536, 5537, 5539, 5540, 5542,
    5543, 5545, 5546, 5547, 5550, 5551, 5552, 5555, 5556, 5571, 5572, 5573, 5574, 5575, 5578, 230,
    231, 5576, 5577, 5579, 5580, 5581, 230, 231, 5602, 223, 5603, 239, 5604, 5606, 5607, 5610,
    5611, 5612, 5614, 5615, 5616, 5619, 223, 5620, 239, 5617, 5618, 5621, 5622, 5623, 5624, 5625,
    5627, 5628, 5629, 5630, 5631, 5632, 5635, 223, 5636, 239, 5633, 5634, 5637, 5638, 5639, 5640,
    5641, 5643, 5644, 5645, 5646, 5649, 5652, 223, 5653, 239, 5647, 5648, 5650, 5651, 5654, 5655,
    5656, 5657, 5659, 223, 5660, 239, 5663, 223, 5664, 239, 5674, 5675, 5676, 5677, 5661, 5662,
    5665, 5666, 5667, 5671, 5668, 5670, 5669, 230, 231, 5672, 5673, 5679, 223, 5680, 239, 5684,
    5685, 5686, 5687, 5712, 5733, 5734, 5738, 5735, 5736, 5737, 5740, 5741, 5742, 5743, 5744, 5745,
    230, 231, 5747, 223, 5748, 239, 5749, 57, 5755, 5756, 5784, 5785, 5786, 5788, 5790, 5792,
    5787, 230, 231, 5789, 230, 231, 5791, 230, 231, 5796, 5797, 5800, 5801, 5802, 5803, 5798,
    5799, 5799, 230, 231, 5810, 5815, 5811, 5812, 5813, 5814, 5820, 5823, 5821, 5822, 5824, 5825,
    5827, 5828, 5830, 5831, 5838, 5839, 5840, 5841, 5842, 5854, 5855, 5856, 5858, 5860, 4335, 5859,
    57, 5862, 5863, 5864, 5865, 5867, 5868, 5869, 5870, 5871, 5872, 5873, 5874, 5875, 5876, 5877,
    2246, 5883, 223, 5884, 239, 5885, 5886, 5906, 5907, 5908, 230, 231, 5909, 5912, 5913, 57,
    5914, 5915, 5919, 5920, 5922, 5923, 223, 5924, 239, 5925, 57, 5926, 5930, 5936, 5941, 5927,
    5928, 5929, 5931, 5932, 5933, 5934, 5935, 5937, 5938, 5939, 5940, 5945, 5946, 5947, 5948, 5949,
    5950, 5951, 5954, 5952, 5953, 5957, 5958, 223, 5965, 5966, 5967, 239, 5968, 5969, 5970, 230,
    231, 5973, 5974, 5976, 5991, 5992, 5977, 5990, 5978, 5979, 5984, 5985, 230, 231, 5980, 5981,
    5982, 5983, 5986, 5987, 5988, 5989, 5993, 6006, 5994, 5995, 6000, 6001, 230, 231, 5996, 5997,
    5998, 5999, 6002, 6003, 6004, 6005, 6022, 6027, 6042, 6023, 6024, 6025, 230, 231, 6026, 57,
    6028, 6029, 6035, 6036, 6030, 230, 231, 6031, 57, 6032, 6033, 6034, 6037, 230, 231, 6038,
    57, 6039, 6040, 6041, 6043, 6044, 6045, 230, 231, 6046, 6047, 6048, 6049, 6050, 6052, 6053,
    223, 6054, 239, 6055, 6056, 6056, 230, 231, 6057, 57, 6063, 6068, 6064, 6065, 6066, 6067,
    6094, 6095, 6096, 6102, 6104, 6111, 6112, 6113, 6116, 6117, 6114, 230, 231, 6115, 57, 6118,
    6119, 6123, 6124, 6141, 6144, 6147, 6148, 6149, 6149, 230, 231, 6150, 6151, 6152, 6153, 6157,
    6166, 6158, 6159, 6160, 57, 6162, 6163, 6164, 6165, 6167, 6168, 6169, 6170, 6171, 57, 6172,
    6173, 6175, 6176, 6177, 6178, 6181, 6182, 6183, 6184, 6185, 6186, 230, 231, 6189, 6190, 6192,
    6193, 6194, 230, 231, 6195, 6196, 6197, 6198, 6200, 6201, 6202, 6203, 6206, 6207, 6211, 6208,
    6209, 6210, 6212, 230, 231, 6216, 6217, 6219, 6226, 6231, 6239, 6220, 6223, 6221, 6222, 6224,
    6225, 6227, 6228, 6229, 6230, 6232, 6233, 6236, 6234, 6235, 6237, 6238, 6240, 57, 6241, 6242,
    6243, 6244, 6258, 6259, 6260, 6261, 6267, 6268, 6262, 57, 6263, 6264, 6265, 6266, 6269, 6270,
    6271, 6272, 6278, 6279, 6280, 6281, 6282, 6283, 6287, 6284, 6285, 6286, 6288, 6289, 6290, 6292,
    6293, 6294, 6295, 6296, 6301, 6302, 6310, 6311, 6303, 6304, 6307, 6305, 6306, 6308, 230, 231,
    6309, 57, 6312, 6313, 6316, 6317, 6318, 6319, 6320, 6321, 6325, 6326, 6338, 6327, 230, 231,
    6328, 57, 6329, 6332, 6335, 6330, 6331, 6333, 6334, 6336, 6337, 6339, 6343, 6347, 6351, 6340,
    6341, 6342, 6344, 6345, 6346, 6348, 6349, 6350, 6352, 6355, 6356, 6353, 6354, 6358, 6359, 6362,
    6360, 6361, 6363, 6364, 6366, 230, 231, 6367, 57, 6381, 6382, 6383, 6385, 6386, 6387, 6388,
    6389, 6403, 6404, 6408, 6422, 6423, 6424, 6405, 6406, 6407, 6409, 6413, 6416, 6421, 6410, 6411,
    6412, 6414, 6415, 6417, 6418, 6419, 6420, 6446, 6447, 6450, 6452, 6448, 6449, 6451, 230, 231,
    6453, 6455, 6454, 6456, 230, 231, 6468, 6469, 6470, 6473, 6474, 6554, 6555, 6471, 6472, 6475,
    6478, 6481, 6486, 6489, 6492, 6496, 6500, 6506, 6510, 6514, 6518, 6524, 6527, 6530, 6536, 6539,
    6542, 6545, 6548, 6551, 6476, 6477, 6479, 6480, 6482, 6483, 6484, 6485, 230, 231, 6487, 6488,
    6490, 6491, 6493, 6494, 6495, 6497, 6498, 6499, 6501, 6502, 6503, 6504, 6505, 6505, 230, 231,
    6507, 6508, 6509, 6511, 6512, 6513, 6515, 6516, 6517, 6519, 6520, 6521, 6522, 6523, 6523, 230,
    231, 6525, 6526, 6528, 6529, 6531, 6532, 6533, 6534, 6535, 6535, 230, 231, 6537, 6538, 6540,
    6541, 6543, 6544, 6546, 6547, 6549, 6550, 6552, 6553, 6556, 6557, 6559, 6560, 6561, 6562, 6564,
    6571, 6565, 6566, 6567, 6568, 6569, 230, 231, 6570, 57, 6573, 6574, 223, 6575, 239, 6576,
    57, 6581, 6582, 6584, 6585, 6591, 6592, 6593, 6597, 6598, 6600, 6601, 6604, 6605, 6606, 6607,
    6608, 6599, 6602, 6603, 6611, 6612, 6613, 6614, 6615, 6616, 6617, 6618, 6619, 6620, 6621, 6622,
    6623, 6624, 6629, 6630, 6635, 6638, 6641, 6642, 6636, 6637, 6637, 230, 231, 6639, 6640, 6640,
    230, 231, 6643, 230, 231, 6645, 6646, 6647, 6648, 6649, 6652, 6650, 6651, 6657, 6660, 6658,
    6659, 6661, 6662, 6664, 6667, 6665, 6666, 6668, 6669, 6680, 6681, 6682, 6683, 6687, 223, 6688,
    239, 6689, 57, 6691, 6692, 6705, 6693, 6694, 6695, 6696, 6703, 6697, 6698, 6699, 6700, 6701,
    6702, 6704, 6706, 6709, 6710, 6707, 230, 231, 6708, 57, 6711, 230, 231, 6712, 6713, 6714,
    57, 6715, 6722, 6716, 6717, 6718, 6719, 6720, 6721, 6723, 6725, 6726, 6727, 6731, 6728, 6729,
    6730, 57, 6732, 6733, 6735, 6736, 6740, 6741, 6742, 6737, 6738, 6739, 6745, 6746, 6751, 6747,
    6750, 6748, 6749, 6753, 6754, 6755, 6756, 6758, 6759, 6766, 6767, 6760, 6763, 6761, 6762, 6764,
    6765, 6768, 6769, 6771, 6772, 6775, 6773, 6774, 6776, 6789, 6796, 6799, 6777, 6778, 6781, 6779,
    6780, 6782, 6783, 6784, 57, 6785, 6786, 6787, 6788, 6790, 6791, 6792, 6793, 6794, 6795, 6797,
    6798, 6800, 6801, 6802, 6803, 6811, 6814, 6812, 6813, 6815, 6816, 6817, 6818, 6821, 6822, 6823,
    6824, 6825, 6826, 6827, 6828, 6829, 6833, 6834, 6837, 6840, 6841, 6842, 6844, 6845, 6848, 57,
    6850, 6851, 6852, 6855, 6856, 6891, 6892, 6893, 6894, 6895, 6896, 6897, 6898, 6899, 6900, 6901,
    6902, 6903, 6904, 6905, 6906, 6907, 6908, 6909, 6910, 6911, 6912, 6913, 6914, 6915, 6916, 6917,
    6918, 6919, 6920, 6921, 6922, 6923, 6924, 6925, 6926, 6927, 6928, 6929, 6930, 6931, 6932, 6933,
    6934, 6935, 6936, 6937, 6938, 6939, 6940, 6941, 6942, 6943, 6944, 6945, 6946, 6947, 6948, 6949,
    6950, 6951, 6952, 6953, 6954, 6955, 6956, 6957, 6958, 6959, 6960, 6961, 6962, 6963, 6964, 6965,
    6966, 6967, 6968, 6969, 6970, 6971, 6972, 6973, 6974, 6975, 6976, 6977, 6978, 6979, 6980, 6981,
    6982, 6983, 6984, 6985, 6986, 6987, 6988, 6989, 6990, 6991, 6992, 6993, 6994, 6995, 6996, 6997,
    6998, 6999, 7000, 7001, 7002, 7003, 7004, 7005, 7006, 7007, 7008, 7009, 7010, 7011, 7023, 7024,
    7027, 230, 231, 7028, 57, 7029, 7030, 7046, 7049, 7087, 7047, 7048, 7050, 7057, 7065, 7069,
    7082, 7051, 7052, 7054, 7053, 230, 231, 7055, 7056, 7058, 7059, 7063, 7064, 7060, 7061, 7062,
    7066, 7067, 7068, 7070, 7071, 7074, 7077, 7078, 7072, 7073, 7075, 230, 231, 7076, 57, 7079,
    7080, 7081, 230, 231, 7083, 7084, 7085, 7086, 7088, 7093, 7100, 7103, 7089, 7090, 7091, 7092,
    7094, 7097, 7095, 7096, 7098, 7099, 7101, 7102, 7104, 7105, 7107, 7110, 7117, 7122, 7108, 7109,
    7111, 7114, 7115, 7116, 7112, 7113, 7118, 7119, 7120, 7121, 7123, 7126, 7124, 7125, 7127, 7128,
    7131, 7132, 7133, 7138, 7139, 7141, 7134, 7135, 7136, 7137, 7140, 230, 231, 7142, 230, 231,
    7148, 7151, 7149, 7153, 7155, 7154, 7163, 7164, 7167, 7168, 7179, 7182, 7183, 7180, 7181, 7184,
    7187, 7188, 7185, 7186, 7190, 7191, 7192, 7193, 7194, 7210, 7216, 7219, 7222, 7211, 7212, 7213,
    7214, 7215, 7217, 7218, 7220, 7221, 7223, 7224, 7225, 7234, 7235, 7236, 7241, 7242, 7243, 7251,
    7256, 7244, 57, 7245, 7248, 7246, 7247, 7249, 7250, 7252, 7255, 7253, 7254, 7264, 230, 231,
    7265, 57, 7268, 7273, 7274, 7269, 7270, 7271, 7272, 7276, 7277, 7285, 7286, 7287, 7291, 7292,
    7293, 7294, 7297, 7303, 7304, 7305, 7306, 7310, 7311, 7312, 7313, 7314, 7315, 7316, 7317, 7324,
    7325, 7326, 7328, 7329, 7334, 7335, 7336, 7337, 7338, 7344, 7345, 223, 7346, 7347, 239, 7349,
    7350, 7351, 7358, 7348, 7352, 7357, 7353, 7354, 7355, 7356, 7359, 223, 7360, 239, 7361, 7362,
    7363, 57, 7364, 7365, 7370, 7371, 7383, 7384, 7411, 7385, 7392, 7386, 7387, 7388, 7389, 7390,
    7391, 7393, 230, 231, 7394, 57, 7395, 7407, 7396, 7397, 7398, 7399, 7400, 7401, 7402, 7403,
    7404, 7405, 7406, 7408, 7409, 7410, 7412, 57, 7413, 7414, 7420, 7421, 7427, 7428, 7429, 7422,
    57, 7423, 230, 231, 7424, 57, 7425, 7426, 7441, 7442, 7455, 7506, 7456, 7493, 7457, 7458,
    7459, 7460, 7461, 7462, 7463, 7464, 7465, 7466, 7467, 7468, 7469, 7470, 7471, 7472, 7473, 7474,
    7475, 7476, 7477, 7478, 7479, 7480, 7481, 7482, 7483, 7484, 7485, 7486, 7487, 7488, 7489, 7490,
    7491, 7492, 7494, 230, 231, 7495, 57, 7496, 7500, 7505, 7497, 7498, 7498, 230, 231, 7499,
    57, 7501, 230, 231, 7502, 57, 7503, 230, 231, 7504, 57, 7507, 7510, 7508, 7509, 7511,
    230, 231, 7512, 7551, 7513, 7514, 7515, 7516, 7517, 7518, 7519, 7520, 7521, 7522, 7523, 7524,
    7525, 7526, 7527, 7528, 7529, 7530, 7531, 7532, 7533, 7534, 7535, 7536, 7537, 7538, 7539, 7540,
    7541, 7542, 7543, 7544, 7545, 7546, 7547, 7548, 7549, 7550, 7552, 7553, 7555, 7556, 7558, 7559,
    7560, 7561, 7562, 7563, 2300, 7557, 239, 7566, 2300, 7567, 239, 7568, 7569, 7569, 230, 231,
    7571, 7572, 7575, 7578, 7579, 7580, 7581, 7582, 7583, 7586, 7573, 7574, 7576, 7577, 7584, 7585,
    7592, 7594, 7602, 7595, 7596, 7599, 7597, 7598, 7600, 7601, 7605, 7609, 7610, 7611, 7606, 7607,
    7608, 7613, 7614, 7617, 7618, 7615, 7616, 7620, 7621, 7622, 7623, 7626, 7627, 7624, 57, 7629,
    7630, 7631, 7632, 7635, 7636, 7633, 57, 7638, 7639, 7641, 7640, 7645, 7646, 7647, 7648, 7650,
    7651, 7652, 7653, 230, 231,
];

pub static TERMINATORS: &[u32] = &[
    58, 59, 135, 336, 357, 370, 371, 391, 392, 393, 399, 400, 2198, 2199, 2303, 2304,
    2306, 2313, 2314, 2316, 2318, 2319, 2320, 2365, 2478, 2479, 2480, 2481, 2482, 2483, 2484, 2485,
    2486, 2487, 2488, 2246, 3726, 3727, 3857, 4463, 4502, 4739, 4742, 4751, 4754, 4943, 4947, 5057,
    5060, 5061, 5062, 5063, 5064, 5082, 5942, 5943, 6161, 6174, 6577, 6578, 6579, 6580, 6583, 6586,
    6587, 6588, 6589, 6590, 6594, 6595, 6625, 6626, 6627, 6628, 6631, 6632, 6633, 6846, 7012, 7150,
    7318, 7319, 7320, 7321, 7322, 7323, 7327, 7330, 7331, 7332, 7625, 7634,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABSOLUTE", // [6]
    "ACCESS", // [7]
    "AccountKeywordSegment", // [8]
    "ACCOUNT", // [9]
    "ALL", // [10]
    "CATALOG", // [11]
    "DATABASE", // [12]
    "DOMAIN", // [13]
    "EXTERNAL", // [14]
    "FILE", // [15]
    "FOREIGN", // [16]
    "FUNCTION", // [17]
    "FUTURE", // [18]
    "INTEGRATION", // [19]
    "LANGUAGE", // [20]
    "MASKING", // [21]
    "MATERIALIZED", // [22]
    "PROCEDURE", // [23]
    "RESOURCE", // [24]
    "ROLE", // [25]
    "ROUTINE", // [26]
    "SCHEMA", // [27]
    "SEQUENCE", // [28]
    "STAGE", // [29]
    "STREAM", // [30]
    "TABLE", // [31]
    "TABLESPACE", // [32]
    "TASK", // [33]
    "TYPE", // [34]
    "VIEW", // [35]
    "WAREHOUSE", // [36]
    "ResourceKeywordSegment", // [37]
    "MonitorKeywordSegment", // [38]
    "MONITOR", // [39]
    "WarehouseKeywordSegment", // [40]
    "DatabaseKeywordSegment", // [41]
    "DomainKeywordSegment", // [42]
    "IntegrationKeywordSegment", // [43]
    "LanguageKeywordSegment", // [44]
    "SchemaKeywordSegment", // [45]
    "CatalogKeywordSegment", // [46]
    "RoleKeywordSegment", // [47]
    "TablespaceKeywordSegment", // [48]
    "TypeKeywordSegment", // [49]
    "ForeignKeywordSegment", // [50]
    "DATA", // [51]
    "SERVER", // [52]
    "ServerKeywordSegment", // [53]
    "DataKeywordSegment", // [54]
    "WrapperKeywordSegment", // [55]
    "WRAPPER", // [56]
    "AllKeywordSegment", // [57]
    "SchemasKeywordSegment", // [58]
    "SCHEMAS", // [59]
    "InKeywordSegment", // [60]
    "IN", // [61]
    "FutureKeywordSegment", // [62]
    "AccessSchemaObjectSegment", // [63]
    "access_schema_object", // [64]
    "AccessSchemaPluralObjectSegment", // [65]
    "FUNCTIONS", // [66]
    "PROCEDURES", // [67]
    "ROUTINES", // [68]
    "SEQUENCES", // [69]
    "STAGES", // [70]
    "STREAMS", // [71]
    "TABLES", // [72]
    "TASKS", // [73]
    "VIEWS", // [74]
    "access_schema_plural_object", // [75]
    "ObjectReferenceSegment", // [76]
    "object_reference", // [77]
    "FunctionNameSegment", // [78]
    "function_name", // [79]
    "FunctionParameterListGrammar", // [80]
    "(", // [81]
    "function_parameter_list", // [82]
    "WildcardIdentifierSegment", // [83]
    "wildcard_identifier", // [84]
    "CommaSegment", // [85]
    ",", // [86]
    "ToKeywordSegment", // [87]
    "TO", // [88]
    "FromKeywordSegment", // [89]
    "FROM", // [90]
    "LARGE", // [91]
    "LargeKeywordSegment", // [92]
    "ObjectKeywordSegment", // [93]
    "OBJECT", // [94]
    "NumericLiteralSegment", // [95]
    "numeric_literal", // [96]
    "APPLY", // [97]
    "CONNECT", // [98]
    "CREATE", // [99]
    "DELETE", // [100]
    "EXECUTE", // [101]
    "IMPORTED", // [102]
    "INSERT", // [103]
    "MANAGE", // [104]
    "MODIFY", // [105]
    "OPERATE", // [106]
    "OWNERSHIP", // [107]
    "READ", // [108]
    "REFERENCES", // [109]
    "REFERENCE_USAGE", // [110]
    "SELECT", // [111]
    "TEMP", // [112]
    "TEMPORARY", // [113]
    "TRIGGER", // [114]
    "TRUNCATE", // [115]
    "UPDATE", // [116]
    "USAGE", // [117]
    "USE", // [118]
    "USE_ANY_ROLE", // [119]
    "WRITE", // [120]
    "CreateKeywordSegment", // [121]
    "USER", // [122]
    "UserKeywordSegment", // [123]
    "ApplyKeywordSegment", // [124]
    "MaskingKeywordSegment", // [125]
    "PolicyKeywordSegment", // [126]
    "POLICY", // [127]
    "ExecuteKeywordSegment", // [128]
    "TaskKeywordSegment", // [129]
    "ManageKeywordSegment", // [130]
    "GrantsKeywordSegment", // [131]
    "GRANTS", // [132]
    "ExecutionKeywordSegment", // [133]
    "UsageKeywordSegment", // [134]
    "PIPE", // [135]
    "PipeKeywordSegment", // [136]
    "UseKeywordSegment", // [137]
    "ImportedKeywordSegment", // [138]
    "PrivilegesKeywordSegment", // [139]
    "PRIVILEGES", // [140]
    "ConnectKeywordSegment", // [141]
    "DeleteKeywordSegment", // [142]
    "InsertKeywordSegment", // [143]
    "ModifyKeywordSegment", // [144]
    "OperateKeywordSegment", // [145]
    "OwnershipKeywordSegment", // [146]
    "ReadKeywordSegment", // [147]
    "Reference_usageKeywordSegment", // [148]
    "ReferencesKeywordSegment", // [149]
    "SelectKeywordSegment", // [150]
    "TempKeywordSegment", // [151]
    "TemporaryKeywordSegment", // [152]
    "TriggerKeywordSegment", // [153]
    "TruncateKeywordSegment", // [154]
    "UpdateKeywordSegment", // [155]
    "Use_any_roleKeywordSegment", // [156]
    "WriteKeywordSegment", // [157]
    "BracketedColumnReferenceListGrammar", // [158]
    "AccessPermissionSegment", // [159]
    "access_permission", // [160]
    "OnKeywordSegment", // [161]
    "ON", // [162]
    "TableKeywordSegment", // [163]
    "ViewKeywordSegment", // [164]
    "StageKeywordSegment", // [165]
    "FunctionKeywordSegment", // [166]
    "ProcedureKeywordSegment", // [167]
    "RoutineKeywordSegment", // [168]
    "SequenceKeywordSegment", // [169]
    "StreamKeywordSegment", // [170]
    "MaterializedKeywordSegment", // [171]
    "ExternalKeywordSegment", // [172]
    "FileKeywordSegment", // [173]
    "FormatKeywordSegment", // [174]
    "FORMAT", // [175]
    "TablesKeywordSegment", // [176]
    "ViewsKeywordSegment", // [177]
    "StagesKeywordSegment", // [178]
    "FunctionsKeywordSegment", // [179]
    "ProceduresKeywordSegment", // [180]
    "RoutinesKeywordSegment", // [181]
    "SequencesKeywordSegment", // [182]
    "StreamsKeywordSegment", // [183]
    "TasksKeywordSegment", // [184]
    "PoliciesKeywordSegment", // [185]
    "FormatsKeywordSegment", // [186]
    "GRANT", // [187]
    "REVOKE", // [188]
    "GrantStatementSegment", // [189]
    "grant_statement", // [190]
    "RevokeStatementSegment", // [191]
    "revoke_statement", // [192]
    "GROUP", // [193]
    "GroupKeywordSegment", // [194]
    "UserReferenceSegment", // [195]
    "user_reference", // [196]
    "RoleReferenceSegment", // [197]
    "role_reference", // [198]
    "SHARE", // [199]
    "ShareKeywordSegment", // [200]
    "FunctionSegment", // [201]
    "function", // [202]
    "PublicKeywordSegment", // [203]
    "PUBLIC", // [204]
    ".", // [205]
    "[", // [206]
    "ArrayAccessorSegment", // [207]
    "array_accessor", // [208]
    "SemiStructuredAccessorSegment", // [209]
    "semi_structured_expression", // [210]
    "ACCOUNTS", // [211]
    "ACTION", // [212]
    "ADD", // [213]
    "-", // [214]
    "MinusSegment", // [215]
    "PipeSegment", // [216]
    "|", // [217]
    "ADMIN", // [218]
    "AFTER", // [219]
    "AGGREGATE", // [220]
    "OrderByClauseSegment", // [221]
    "ORDER", // [222]
    "orderby_clause", // [223]
    "indent", // [224]
    "Indent", // [225]
    "meta", // [226]
    "AsAliasOperatorSegment", // [227]
    "AS", // [228]
    "alias_operator", // [229]
    "SingleIdentifierGrammar", // [230]
    "SingleIdentifierListSegment", // [231]
    "identifier_list", // [232]
    "start_bracket", // [233]
    "SymbolSegment", // [234]
    "symbol", // [235]
    ")", // [236]
    "end_bracket", // [237]
    "ParameterNameSegment", // [238]
    "DatatypeSegment", // [239]
    "data_type", // [240]
    "dedent", // [241]
    "Dedent", // [242]
    "ALIAS", // [243]
    "TableReferenceSegment", // [244]
    "table_reference", // [245]
    "AliasExpressionSegment", // [246]
    "alias_expression", // [247]
    "ALLOW_CONNECTIONS", // [248]
    "ALSO", // [249]
    "ALTER", // [250]
    "AlterKeywordSegment", // [251]
    "AggregateKeywordSegment", // [252]
    "StarSegment", // [253]
    "*", // [254]
    "OWNER", // [255]
    "RENAME", // [256]
    "SET", // [257]
    "RenameKeywordSegment", // [258]
    "OwnerKeywordSegment", // [259]
    "Current_roleKeywordSegment", // [260]
    "CURRENT_ROLE", // [261]
    "Current_userKeywordSegment", // [262]
    "CURRENT_USER", // [263]
    "Session_userKeywordSegment", // [264]
    "SESSION_USER", // [265]
    "SetKeywordSegment", // [266]
    "SchemaReferenceSegment", // [267]
    "schema_reference", // [268]
    "DatabaseReferenceSegment", // [269]
    "database_reference", // [270]
    "CONNECTION", // [271]
    "IS_TEMPLATE", // [272]
    "RESET", // [273]
    "WITH", // [274]
    "WithKeywordSegment", // [275]
    "Allow_connectionsKeywordSegment", // [276]
    "BooleanLiteralGrammar", // [277]
    "FALSE", // [278]
    "TRUE", // [279]
    "ConnectionKeywordSegment", // [280]
    "LimitKeywordSegment", // [281]
    "LIMIT", // [282]
    "Is_templateKeywordSegment", // [283]
    "TablespaceReferenceSegment", // [284]
    "tablespace_reference", // [285]
    "=", // [286]
    "EqualsSegment", // [287]
    "comparison_operator", // [288]
    "DefaultKeywordSegment", // [289]
    "DEFAULT", // [290]
    "LiteralGrammar", // [291]
    "+", // [292]
    ":", // [293]
    "ARRAY", // [294]
    "DATE", // [295]
    "INTERVAL", // [296]
    "NULL", // [297]
    "TIME", // [298]
    "TIMESTAMP", // [299]
    "TIMESTAMPTZ", // [300]
    "TIMETZ", // [301]
    "VARIADIC", // [302]
    "{", // [303]
    "bit_string_literal", // [304]
    "dollar_numeric_literal", // [305]
    "dollar_quote", // [306]
    "escaped_single_quote", // [307]
    "newline", // [308]
    "single_quote", // [309]
    "unicode_single_quote", // [310]
    "NakedIdentifierSegment", // [311]
    "QuotedIdentifierSegment", // [312]
    "double_quote", // [313]
    "unicode_double_quote", // [314]
    "CurrentKeywordSegment", // [315]
    "CURRENT", // [316]
    "ResetKeywordSegment", // [317]
    "GrantKeywordSegment", // [318]
    "AlterDefaultPrivilegesObjectPrivilegesSegment", // [319]
    "alter_default_privileges_object_privilege", // [320]
    "AlterDefaultPrivilegesSchemaObjectsSegment", // [321]
    "TYPES", // [322]
    "alter_default_privileges_schema_object", // [323]
    "AlterDefaultPrivilegesToFromRolesSegment", // [324]
    "alter_default_privileges_to_from_roles", // [325]
    "OptionKeywordSegment", // [326]
    "OPTION", // [327]
    "RevokeKeywordSegment", // [328]
    "ForKeywordSegment", // [329]
    "FOR", // [330]
    "RestrictKeywordSegment", // [331]
    "RESTRICT", // [332]
    "CascadeKeywordSegment", // [333]
    "CASCADE", // [334]
    "DropBehaviorGrammar", // [335]
    "TypesKeywordSegment", // [336]
    "AlterDefaultPrivilegesGrantSegment", // [337]
    "alter_default_privileges_grant", // [338]
    "AlterDefaultPrivilegesRevokeSegment", // [339]
    "alter_default_privileges_revoke", // [340]
    "DROP", // [341]
    "VALIDATE", // [342]
    "ExpressionSegment", // [343]
    "expression", // [344]
    "DropKeywordSegment", // [345]
    "NotKeywordSegment", // [346]
    "NOT", // [347]
    "NullKeywordSegment", // [348]
    "AddKeywordSegment", // [349]
    "CONSTRAINT", // [350]
    "ConstraintKeywordSegment", // [351]
    "CHECK", // [352]
    "CheckKeywordSegment", // [353]
    "ValidKeywordSegment", // [354]
    "VALID", // [355]
    "IfExistsGrammar", // [356]
    "IF", // [357]
    "ValidateKeywordSegment", // [358]
    "ExtensionKeywordSegment", // [359]
    "EXTENSION", // [360]
    "ExtensionReferenceSegment", // [361]
    "extension_reference", // [362]
    "Current_schemaKeywordSegment", // [363]
    "CURRENT_SCHEMA", // [364]
    "CAST", // [365]
    "COLLATION", // [366]
    "CONVERSION", // [367]
    "EVENT", // [368]
    "OPERATOR", // [369]
    "PROCEDURAL", // [370]
    "TEXT", // [371]
    "TRANSFORM", // [372]
    "AccessKeywordSegment", // [373]
    "MethodKeywordSegment", // [374]
    "METHOD", // [375]
    "CollationKeywordSegment", // [376]
    "ConversionKeywordSegment", // [377]
    "EventKeywordSegment", // [378]
    "ProceduralKeywordSegment", // [379]
    "TextKeywordSegment", // [380]
    "SearchKeywordSegment", // [381]
    "SEARCH", // [382]
    "CONFIGURATION", // [383]
    "DICTIONARY", // [384]
    "PARSER", // [385]
    "TEMPLATE", // [386]
    "ConfigurationKeywordSegment", // [387]
    "DictionaryKeywordSegment", // [388]
    "ParserKeywordSegment", // [389]
    "TemplateKeywordSegment", // [390]
    "CastKeywordSegment", // [391]
    "AsKeywordSegment", // [392]
    "OperatorKeywordSegment", // [393]
    "CLASS", // [394]
    "FAMILY", // [395]
    "ClassKeywordSegment", // [396]
    "FamilyKeywordSegment", // [397]
    "UsingKeywordSegment", // [398]
    "USING", // [399]
    "IndexAccessMethodSegment", // [400]
    "index_access_method", // [401]
    "TransformKeywordSegment", // [402]
    "CLUSTER", // [403]
    "DISABLE", // [404]
    "ENABLE", // [405]
    "FORCE", // [406]
    "INHERIT", // [407]
    "NO", // [408]
    "OF", // [409]
    "OPTIONS", // [410]
    "REPLICA", // [411]
    "ColumnKeywordSegment", // [412]
    "COLUMN", // [413]
    "IfNotExistsGrammar", // [414]
    "ColumnReferenceSegment", // [415]
    "column_reference", // [416]
    "COLLATE", // [417]
    "CollateKeywordSegment", // [418]
    "CollationReferenceSegment", // [419]
    "collation_reference", // [420]
    "GENERATED", // [421]
    "PRIMARY", // [422]
    "UNIQUE", // [423]
    "ColumnConstraintSegment", // [424]
    "column_constraint_segment", // [425]
    "RESTART", // [426]
    "BareFunctionSegment", // [427]
    "CURRENT_CATALOG", // [428]
    "CURRENT_DATE", // [429]
    "CURRENT_TIME", // [430]
    "CURRENT_TIMESTAMP", // [431]
    "LOCALTIME", // [432]
    "LOCALTIMESTAMP", // [433]
    "SYSTEM_USER", // [434]
    "ExpressionKeywordSegment", // [435]
    "EXPRESSION", // [436]
    "GeneratedKeywordSegment", // [437]
    "ALWAYS", // [438]
    "BY", // [439]
    "AlwaysKeywordSegment", // [440]
    "ByKeywordSegment", // [441]
    "IdentityKeywordSegment", // [442]
    "IDENTITY", // [443]
    "CACHE", // [444]
    "CYCLE", // [445]
    "INCREMENT", // [446]
    "MAXVALUE", // [447]
    "MINVALUE", // [448]
    "OWNED", // [449]
    "START", // [450]
    "AlterSequenceOptionsSegment", // [451]
    "alter_sequence_options_segment", // [452]
    "RestartKeywordSegment", // [453]
    "StatisticsKeywordSegment", // [454]
    "STATISTICS", // [455]
    "RelationOptionsSegment", // [456]
    "relation_options", // [457]
    "StorageKeywordSegment", // [458]
    "STORAGE", // [459]
    "EXTENDED", // [460]
    "MAIN", // [461]
    "PLAIN", // [462]
    "PlainKeywordSegment", // [463]
    "ExtendedKeywordSegment", // [464]
    "MainKeywordSegment", // [465]
    "CompressionKeywordSegment", // [466]
    "COMPRESSION", // [467]
    "TableConstraintSegment", // [468]
    "EXCLUDE", // [469]
    "table_constraint", // [470]
    "TableConstraintUsingIndexSegment", // [471]
    "DEFERRABLE", // [472]
    "DeferrableKeywordSegment", // [473]
    "INITIALLY", // [474]
    "InitiallyKeywordSegment", // [475]
    "DeferredKeywordSegment", // [476]
    "DEFERRED", // [477]
    "ImmediateKeywordSegment", // [478]
    "IMMEDIATE", // [479]
    "EnableKeywordSegment", // [480]
    "DisableKeywordSegment", // [481]
    "ReplicaKeywordSegment", // [482]
    "RuleKeywordSegment", // [483]
    "RULE", // [484]
    "ForceKeywordSegment", // [485]
    "NoKeywordSegment", // [486]
    "RowKeywordSegment", // [487]
    "ROW", // [488]
    "LevelKeywordSegment", // [489]
    "LEVEL", // [490]
    "SecurityKeywordSegment", // [491]
    "SECURITY", // [492]
    "ClusterKeywordSegment", // [493]
    "WithoutKeywordSegment", // [494]
    "WITHOUT", // [495]
    "OIDS", // [496]
    "OidsKeywordSegment", // [497]
    "LOGGED", // [498]
    "UNLOGGED", // [499]
    "LoggedKeywordSegment", // [500]
    "UnloggedKeywordSegment", // [501]
    "InheritKeywordSegment", // [502]
    "OfKeywordSegment", // [503]
    "FULL", // [504]
    "NOTHING", // [505]
    "IndexKeywordSegment", // [506]
    "INDEX", // [507]
    "IndexReferenceSegment", // [508]
    "index_reference", // [509]
    "FullKeywordSegment", // [510]
    "NothingKeywordSegment", // [511]
    "OptionsKeywordSegment", // [512]
    "QuotedLiteralSegment", // [513]
    "OnlyKeywordSegment", // [514]
    "ONLY", // [515]
    "AlterForeignTableActionSegment", // [516]
    "alter_foreign_table_action_segment", // [517]
    "alter_table_action_segment", // [518]
    "CALLED", // [519]
    "COST", // [520]
    "IMMUTABLE", // [521]
    "LEAKPROOF", // [522]
    "PARALLEL", // [523]
    "RETURNS", // [524]
    "ROWS", // [525]
    "STABLE", // [526]
    "STRICT", // [527]
    "SUPPORT", // [528]
    "VOLATILE", // [529]
    "CalledKeywordSegment", // [530]
    "InputKeywordSegment", // [531]
    "INPUT", // [532]
    "ReturnsKeywordSegment", // [533]
    "StrictKeywordSegment", // [534]
    "ImmutableKeywordSegment", // [535]
    "StableKeywordSegment", // [536]
    "VolatileKeywordSegment", // [537]
    "LeakproofKeywordSegment", // [538]
    "DEFINER", // [539]
    "INVOKER", // [540]
    "DefinerKeywordSegment", // [541]
    "InvokerKeywordSegment", // [542]
    "ParallelKeywordSegment", // [543]
    "RESTRICTED", // [544]
    "SAFE", // [545]
    "UNSAFE", // [546]
    "UnsafeKeywordSegment", // [547]
    "RestrictedKeywordSegment", // [548]
    "SafeKeywordSegment", // [549]
    "CostKeywordSegment", // [550]
    "RowsKeywordSegment", // [551]
    "SupportKeywordSegment", // [552]
    "DEPENDS", // [553]
    "AlterFunctionActionSegment", // [554]
    "alter_function_action_segment", // [555]
    "DependsKeywordSegment", // [556]
    "ATTACH", // [557]
    "AttachKeywordSegment", // [558]
    "PartitionKeywordSegment", // [559]
    "PARTITION", // [560]
    "OwnedKeywordSegment", // [561]
    "NowaitKeywordSegment", // [562]
    "NOWAIT", // [563]
    "QualifiedParameterNameGrammar", // [564]
    "AlterMaterializedViewActionSegment", // [565]
    "alter_materialized_view_action_segment", // [566]
    "AlterProcedureActionSegment", // [567]
    "alter_procedure_action_segment", // [568]
    "PublicationKeywordSegment", // [569]
    "PUBLICATION", // [570]
    "PublicationReferenceSegment", // [571]
    "publication_reference", // [572]
    "DefinitionParametersSegment", // [573]
    "definition_parameters", // [574]
    "PublicationObjectsSegment", // [575]
    "publication_objects", // [576]
    "BYPASSRLS", // [577]
    "CREATEDB", // [578]
    "CREATEROLE", // [579]
    "ENCRYPTED", // [580]
    "LOGIN", // [581]
    "NOBYPASSRLS", // [582]
    "NOCREATEDB", // [583]
    "NOCREATEROLE", // [584]
    "NOINHERIT", // [585]
    "NOLOGIN", // [586]
    "NOREPLICATION", // [587]
    "NOSUPERUSER", // [588]
    "PASSWORD", // [589]
    "REPLICATION", // [590]
    "SUPERUSER", // [591]
    "SuperuserKeywordSegment", // [592]
    "NosuperuserKeywordSegment", // [593]
    "CreatedbKeywordSegment", // [594]
    "NocreatedbKeywordSegment", // [595]
    "CreateroleKeywordSegment", // [596]
    "NocreateroleKeywordSegment", // [597]
    "NoinheritKeywordSegment", // [598]
    "LoginKeywordSegment", // [599]
    "NologinKeywordSegment", // [600]
    "ReplicationKeywordSegment", // [601]
    "NoreplicationKeywordSegment", // [602]
    "BypassrlsKeywordSegment", // [603]
    "NobypassrlsKeywordSegment", // [604]
    "EncryptedKeywordSegment", // [605]
    "PasswordKeywordSegment", // [606]
    "UntilKeywordSegment", // [607]
    "UNTIL", // [608]
    "OnKeywordAsIdentifierSegment", // [609]
    "IncrementKeywordSegment", // [610]
    "SignedSegmentGrammar", // [611]
    "MinvalueKeywordSegment", // [612]
    "MaxvalueKeywordSegment", // [613]
    "NameKeywordSegment", // [614]
    "NAME", // [615]
    "SequenceReferenceSegment", // [616]
    "sequence_reference", // [617]
    "StartKeywordSegment", // [618]
    "CacheKeywordSegment", // [619]
    "CycleKeywordSegment", // [620]
    "NoneKeywordSegment", // [621]
    "NONE", // [622]
    "StatisticsReferenceSegment", // [623]
    "statistics_reference", // [624]
    "SubscriptionKeywordSegment", // [625]
    "SUBSCRIPTION", // [626]
    "SubscriptionReferenceSegment", // [627]
    "subscription_reference", // [628]
    "REFRESH", // [629]
    "SKIP", // [630]
    "RefreshKeywordSegment", // [631]
    "SkipKeywordSegment", // [632]
    "RawEqualsSegment", // [633]
    "Current_sessionKeywordSegment", // [634]
    "SystemKeywordSegment", // [635]
    "SYSTEM", // [636]
    "ColumnDefinitionSegment", // [637]
    "column_definition", // [638]
    "FIRST", // [639]
    "FirstKeywordSegment", // [640]
    "AfterKeywordSegment", // [641]
    "AlterTableDropColumnGrammar", // [642]
    "AlterTableActionSegment", // [643]
    "DETACH", // [644]
    "ValuesKeywordSegment", // [645]
    "VALUES", // [646]
    "PartitionBoundSpecSegment", // [647]
    "partition_bound_spec", // [648]
    "DetachKeywordSegment", // [649]
    "ConcurrentlyKeywordSegment", // [650]
    "CONCURRENTLY", // [651]
    "FinalizeKeywordSegment", // [652]
    "FINALIZE", // [653]
    "MappingKeywordSegment", // [654]
    "MAPPING", // [655]
    "ReplaceKeywordSegment", // [656]
    "REPLACE", // [657]
    "TriggerReferenceSegment", // [658]
    "trigger_reference", // [659]
    "ValueKeywordSegment", // [660]
    "VALUE", // [661]
    "AttributeKeywordSegment", // [662]
    "ATTRIBUTE", // [663]
    "CascadeRestrictGrammar", // [664]
    "BEFORE", // [665]
    "BeforeKeywordSegment", // [666]
    "&", // [667]
    "ampersand", // [668]
    "ANALYSE", // [669]
    "ANALYZE", // [670]
    "AnalyzeKeywordSegment", // [671]
    "AnalyseKeywordSegment", // [672]
    "VERBOSE", // [673]
    "SKIP_LOCKED", // [674]
    "VerboseKeywordSegment", // [675]
    "Skip_lockedKeywordSegment", // [676]
    "AND", // [677]
    "binary_operator", // [678]
    "BinaryOperatorSegment", // [679]
    "ANY", // [680]
    "%", // [681]
    "/", // [682]
    "<", // [683]
    ">", // [684]
    "^", // [685]
    "PlusSegment", // [686]
    "DivideSegment", // [687]
    "MultiplySegment", // [688]
    "ModuloSegment", // [689]
    "BitwiseAndSegment", // [690]
    "BitwiseOrSegment", // [691]
    "BitwiseXorSegment", // [692]
    "BitwiseLShiftSegment", // [693]
    "BitwiseRShiftSegment", // [694]
    "QualifiedNumericLiteralSegment", // [695]
    "SliceSegment", // [696]
    "start_square_bracket", // [697]
    "]", // [698]
    "end_square_bracket", // [699]
    "BaseExpressionElementGrammar", // [700]
    "ArrayKeywordSegment", // [701]
    "ASC", // [702]
    "ASSERTION", // [703]
    "ASSIGNMENT", // [704]
    "ASYMMETRIC", // [705]
    "AT", // [706]
    "ATOMIC", // [707]
    "AUDIT", // [708]
    "AUTHORIZATION", // [709]
    "AUTO_INCREMENT", // [710]
    "Auto_incrementKeywordSegment", // [711]
    "AVG_ROW_LENGTH", // [712]
    "BACKUP", // [713]
    "BACKWARD", // [714]
    "bare_function", // [715]
    "CodeSegment", // [716]
    "IntervalExpressionSegment", // [717]
    "interval_expression", // [718]
    "CompositeValueExpansionSegment", // [719]
    "composite_value_expansion", // [720]
    "file", // [721]
    "BaseFileSegment", // [722]
    "BaseSegment", // [723]
    "BEGIN", // [724]
    "BERNOULLI", // [725]
    "BETWEEN", // [726]
    "BIGINT", // [727]
    "BIGSERIAL", // [728]
    "BINARY", // [729]
    "!", // [730]
    "IS", // [731]
    "OR", // [732]
    "json_operator", // [733]
    "like_operator", // [734]
    "pg_trgm_operator", // [735]
    "pgvector_operator", // [736]
    "postgis_operator", // [737]
    "ArithmeticBinaryOperatorGrammar", // [738]
    "StringBinaryOperatorGrammar", // [739]
    "BooleanBinaryOperatorGrammar", // [740]
    "ComparisonOperatorGrammar", // [741]
    "JsonOperatorSegment", // [742]
    "BINDING", // [743]
    "BIT", // [744]
    "BITVAR", // [745]
    "AmpersandSegment", // [746]
    "RawLessThanSegment", // [747]
    "RawGreaterThanSegment", // [748]
    "BOOL", // [749]
    "AndOperatorGrammar", // [750]
    "OrOperatorGrammar", // [751]
    "BOOLEAN", // [752]
    "TrueSegment", // [753]
    "FalseSegment", // [754]
    "BOTH", // [755]
    "BOX", // [756]
    "BPCHAR", // [757]
    "bracketed", // [758]
    "BracketedSegment", // [759]
    "SetExpressionSegment", // [760]
    "set_expression", // [761]
    "BREADTH", // [762]
    "BREAK", // [763]
    "BROWSE", // [764]
    "BUFFERS", // [765]
    "BULK", // [766]
    "BYTEA", // [767]
    "CTEColumnList", // [768]
    "cte_column_list", // [769]
    "SelectableGrammar", // [770]
    "DEPTH", // [771]
    "BreadthKeywordSegment", // [772]
    "DepthKeywordSegment", // [773]
    "CALL", // [774]
    "CallKeywordSegment", // [775]
    "CASCADED", // [776]
    "CASE", // [777]
    "CaseKeywordSegment", // [778]
    "implicit_indent", // [779]
    "ImplicitIndent", // [780]
    "WHEN", // [781]
    "WhenClauseSegment", // [782]
    "when_clause", // [783]
    "ElseKeywordSegment", // [784]
    "ELSE", // [785]
    "EndKeywordSegment", // [786]
    "END", // [787]
    "ElseClauseSegment", // [788]
    "else_clause", // [789]
    "BinaryOperatorGrammar", // [790]
    "::", // [791]
    "casting_operator", // [792]
    "CHAIN", // [793]
    "CHANGE", // [794]
    "CHAR", // [795]
    "CHARACTER", // [796]
    "CHARACTERISTICS", // [797]
    "CHECKED", // [798]
    "CHECKPOINT", // [799]
    "CHECKSUM", // [800]
    "CIDR", // [801]
    "CIRCLE", // [802]
    "CIRCULARSTRING", // [803]
    "CLOSE", // [804]
    "CLUSTERED", // [805]
    "COALESCE", // [806]
    "ObjectReferenceDelimiterGrammar", // [807]
    "ObjectReferenceTerminatorGrammar", // [808]
    "colon_delimiter", // [809]
    "colon_prefix", // [810]
    "colon", // [811]
    "ShorthandCastSegment", // [812]
    "cast_expression", // [813]
    "StoredKeywordSegment", // [814]
    "STORED", // [815]
    "UniqueKeywordSegment", // [816]
    "NULLS", // [817]
    "NullsKeywordSegment", // [818]
    "DistinctKeywordSegment", // [819]
    "DISTINCT", // [820]
    "PrimaryKeywordSegment", // [821]
    "KeyKeywordSegment", // [822]
    "KEY", // [823]
    "ReferenceDefinitionGrammar", // [824]
    "DotSegment", // [825]
    "SingleIdentifierFullGrammar", // [826]
    "CastOperatorSegment", // [827]
    "StartSquareBracketSegment", // [828]
    "StartBracketSegment", // [829]
    "ColonSegment", // [830]
    "DelimiterGrammar", // [831]
    ";", // [832]
    "JoinLikeClauseGrammar", // [833]
    "ColumnsExpressionNameGrammar", // [834]
    "COLUMNS", // [835]
    "comma", // [836]
    "COMMENT", // [837]
    "CommentKeywordSegment", // [838]
    "IsKeywordSegment", // [839]
    "comment", // [840]
    "CommentSegment", // [841]
    "COMMENTS", // [842]
    "COMMIT", // [843]
    "COMMITTED", // [844]
    "COMMUTATOR", // [845]
    "GreaterThanSegment", // [846]
    "LessThanSegment", // [847]
    "GreaterThanOrEqualToSegment", // [848]
    "LessThanOrEqualToSegment", // [849]
    "NotEqualToSegment", // [850]
    "LikeOperatorSegment", // [851]
    "OverlapSegment", // [852]
    "NotExtendRightSegment", // [853]
    "NotExtendLeftSegment", // [854]
    "AdjacentSegment", // [855]
    "PostgisOperatorSegment", // [856]
    "PgvectorOperatorSegment", // [857]
    "PgTrgmOperatorSegment", // [858]
    "QualifiedOperatorSegment", // [859]
    "qualified_operator", // [860]
    "ComparisonOperatorSegment", // [861]
    "COMPLETION", // [862]
    "CompositeBinaryOperatorSegment", // [863]
    "CompositeComparisonOperatorSegment", // [864]
    "COMPOUNDCURVE", // [865]
    "COMPRESS", // [866]
    "COMPUTE", // [867]
    "JoinTypeKeywordsGrammar", // [868]
    "INNER", // [869]
    "LEFT", // [870]
    "RIGHT", // [871]
    "ConditionalCrossJoinKeywordsGrammar", // [872]
    "NonStandardJoinTypeKeywordsGrammar", // [873]
    "DO", // [874]
    "DoKeywordSegment", // [875]
    "WHERE", // [876]
    "WhereKeywordSegment", // [877]
    "CONFLICT", // [878]
    "OperationClassReferenceSegment", // [879]
    "operation_class_reference", // [880]
    "CONSTRAINTS", // [881]
    "CONTAINSTABLE", // [882]
    "CONTENT", // [883]
    "CONTINUE", // [884]
    "COPY", // [885]
    "CopyKeywordSegment", // [886]
    "PROGRAM", // [887]
    "STDIN", // [888]
    "ProgramKeywordSegment", // [889]
    "PsqlVariableGrammar", // [890]
    "psql_variable", // [891]
    "StdinKeywordSegment", // [892]
    "DELIMITER", // [893]
    "ENCODING", // [894]
    "ESCAPE", // [895]
    "FORCE_NOT_NULL", // [896]
    "FORCE_NULL", // [897]
    "FORCE_QUOTE", // [898]
    "FREEZE", // [899]
    "HEADER", // [900]
    "QUOTE", // [901]
    "FreezeKeywordSegment", // [902]
    "DelimiterKeywordSegment", // [903]
    "HeaderKeywordSegment", // [904]
    "QuoteKeywordSegment", // [905]
    "EscapeKeywordSegment", // [906]
    "Force_quoteKeywordSegment", // [907]
    "Force_not_nullKeywordSegment", // [908]
    "Force_nullKeywordSegment", // [909]
    "EncodingKeywordSegment", // [910]
    "CSV", // [911]
    "BinaryKeywordSegment", // [912]
    "CsvKeywordSegment", // [913]
    "STDOUT", // [914]
    "StdoutKeywordSegment", // [915]
    "CORRELATION", // [916]
    "COSTS", // [917]
    "OrReplaceGrammar", // [918]
    "InoutKeywordSegment", // [919]
    "INOUT", // [920]
    "AssignmentKeywordSegment", // [921]
    "ImplicitKeywordSegment", // [922]
    "IMPLICIT", // [923]
    "DETERMINISTIC", // [924]
    "LC_COLLATE", // [925]
    "LC_CTYPE", // [926]
    "LOCALE", // [927]
    "PROVIDER", // [928]
    "VERSION", // [929]
    "LocaleKeywordSegment", // [930]
    "Lc_collateKeywordSegment", // [931]
    "Lc_ctypeKeywordSegment", // [932]
    "ProviderKeywordSegment", // [933]
    "ICU", // [934]
    "LIBC", // [935]
    "IcuKeywordSegment", // [936]
    "LibcKeywordSegment", // [937]
    "DeterministicKeywordSegment", // [938]
    "VersionKeywordSegment", // [939]
    "VersionIdentifierSegment", // [940]
    "version_identifier", // [941]
    "ForeignDataWrapperGrammar", // [942]
    "HandlerKeywordSegment", // [943]
    "HANDLER", // [944]
    "ValidatorKeywordSegment", // [945]
    "VALIDATOR", // [946]
    "CreateForeignTableGrammar", // [947]
    "OptionsGrammar", // [948]
    "ForeignTableColumnConstraintSegment", // [949]
    "ForeignTableTableConstraintSegment", // [950]
    "INHERITS", // [951]
    "InheritsKeywordSegment", // [952]
    "ServerReferenceSegment", // [953]
    "server_reference", // [954]
    "TemporaryGrammar", // [955]
    "ColumnTypeReferenceSegment", // [956]
    "column_type_reference", // [957]
    "SETOF", // [958]
    "SetofKeywordSegment", // [959]
    "FunctionDefinitionGrammar", // [960]
    "RETURN", // [961]
    "WINDOW", // [962]
    "function_definition", // [963]
    "IndexElementSegment", // [964]
    "index_element", // [965]
    "INCLUDE", // [966]
    "IncludeKeywordSegment", // [967]
    "ValuesClauseSegment", // [968]
    "values_clause", // [969]
    "WithDataClauseSegment", // [970]
    "with_data_clause", // [971]
    "ModelKeywordSegment", // [972]
    "MODEL", // [973]
    "commutator", // [974]
    "HASHES", // [975]
    "JOIN", // [976]
    "LEFTARG", // [977]
    "MERGES", // [978]
    "NEGATOR", // [979]
    "RIGHTARG", // [980]
    "LeftargKeywordSegment", // [981]
    "RightargKeywordSegment", // [982]
    "CommutatorKeywordSegment", // [983]
    "NegatorKeywordSegment", // [984]
    "negator", // [985]
    "JoinKeywordSegment", // [986]
    "HashesKeywordSegment", // [987]
    "MergesKeywordSegment", // [988]
    "PERMISSIVE", // [989]
    "RESTRICTIVE", // [990]
    "PermissiveKeywordSegment", // [991]
    "RestrictiveKeywordSegment", // [992]
    "SYSID", // [993]
    "AdminKeywordSegment", // [994]
    "SysidKeywordSegment", // [995]
    "AuthorizationKeywordSegment", // [996]
    "CreateSequenceOptionsSegment", // [997]
    "create_sequence_options_segment", // [998]
    "DEPENDENCIES", // [999]
    "MCV", // [1000]
    "NDISTINCT", // [1001]
    "DependenciesKeywordSegment", // [1002]
    "McvKeywordSegment", // [1003]
    "NdistinctKeywordSegment", // [1004]
    "CorrelationKeywordSegment", // [1005]
    "GLOBAL", // [1006]
    "LOCAL", // [1007]
    "GlobalKeywordSegment", // [1008]
    "LocalKeywordSegment", // [1009]
    "CommitKeywordSegment", // [1010]
    "PRESERVE", // [1011]
    "PreserveKeywordSegment", // [1012]
    "LIKE", // [1013]
    "LikeKeywordSegment", // [1014]
    "EXCLUDING", // [1015]
    "INCLUDING", // [1016]
    "LikeOptionSegment", // [1017]
    "like_option_segment", // [1018]
    "HASH", // [1019]
    "LIST", // [1020]
    "RANGE", // [1021]
    "RangeKeywordSegment", // [1022]
    "ListKeywordSegment", // [1023]
    "HashKeywordSegment", // [1024]
    "INSTEAD", // [1025]
    "InsteadKeywordSegment", // [1026]
    "OrKeywordSegment", // [1027]
    "REFERENCING", // [1028]
    "ReferencingKeywordSegment", // [1029]
    "NEW", // [1030]
    "OLD", // [1031]
    "OldKeywordSegment", // [1032]
    "NewKeywordSegment", // [1033]
    "EachKeywordSegment", // [1034]
    "EACH", // [1035]
    "STATEMENT", // [1036]
    "StatementKeywordSegment", // [1037]
    "WhenKeywordSegment", // [1038]
    "ENUM", // [1039]
    "EnumKeywordSegment", // [1040]
    "CreateUserMappingGrammar", // [1041]
    "SessionInformationUserFunctionsGrammar", // [1042]
    "RecursiveKeywordSegment", // [1043]
    "RECURSIVE", // [1044]
    "WithCheckOptionSegment", // [1045]
    "with_check_option", // [1046]
    "CREATEUSER", // [1047]
    "CROSS", // [1048]
    "CUBE", // [1049]
    "function_name_identifier", // [1050]
    "ROLLUP", // [1051]
    "CubeFunctionNameSegment", // [1052]
    "RollupFunctionNameSegment", // [1053]
    "GroupingExpressionList", // [1054]
    "grouping_expression_list", // [1055]
    "CURSOR", // [1056]
    "CURVEPOLYGON", // [1057]
    "DATABASES", // [1058]
    "data_type_identifier", // [1059]
    "WellKnownTextGeometrySegment", // [1060]
    "GEOGRAPHY", // [1061]
    "GEOMETRY", // [1062]
    "GEOMETRYCOLLECTION", // [1063]
    "GEOMETRYCOLLECTIONM", // [1064]
    "GEOMETRYCOLLECTIONZ", // [1065]
    "GEOMETRYCOLLECTIONZM", // [1066]
    "LINESTRING", // [1067]
    "LINESTRINGM", // [1068]
    "LINESTRINGZ", // [1069]
    "LINESTRINGZM", // [1070]
    "MULTICURVE", // [1071]
    "MULTILINESTRING", // [1072]
    "MULTILINESTRINGM", // [1073]
    "MULTILINESTRINGZ", // [1074]
    "MULTILINESTRINGZM", // [1075]
    "MULTIPOINT", // [1076]
    "MULTIPOINTM", // [1077]
    "MULTIPOINTZ", // [1078]
    "MULTIPOINTZM", // [1079]
    "MULTIPOLYGON", // [1080]
    "MULTIPOLYGONM", // [1081]
    "MULTIPOLYGONZ", // [1082]
    "MULTIPOLYGONZM", // [1083]
    "MULTISURFACE", // [1084]
    "POINT", // [1085]
    "POINTM", // [1086]
    "POINTZ", // [1087]
    "POINTZM", // [1088]
    "POLYGON", // [1089]
    "POLYGONM", // [1090]
    "POLYGONZ", // [1091]
    "POLYGONZM", // [1092]
    "POLYHEDRALSURFACE", // [1093]
    "TIN", // [1094]
    "TRIANGLE", // [1095]
    "wkt_geometry_type", // [1096]
    "DateTimeTypeIdentifier", // [1097]
    "datetime_type_identifier", // [1098]
    "StructTypeSegment", // [1099]
    "struct_type", // [1100]
    "MapTypeSegment", // [1101]
    "map_type", // [1102]
    "DATERANGE", // [1103]
    "DECIMAL", // [1104]
    "DOUBLE", // [1105]
    "FLOAT", // [1106]
    "FLOAT4", // [1107]
    "FLOAT8", // [1108]
    "INET", // [1109]
    "INT", // [1110]
    "INT2", // [1111]
    "INT4", // [1112]
    "INT4RANGE", // [1113]
    "INT8", // [1114]
    "INT8RANGE", // [1115]
    "INTEGER", // [1116]
    "JSON", // [1117]
    "JSONB", // [1118]
    "LINE", // [1119]
    "LSEG", // [1120]
    "MACADDR", // [1121]
    "MACADDR8", // [1122]
    "MONEY", // [1123]
    "NUMERIC", // [1124]
    "NUMRANGE", // [1125]
    "PATH", // [1126]
    "PG_LSN", // [1127]
    "REAL", // [1128]
    "SERIAL", // [1129]
    "SERIAL2", // [1130]
    "SERIAL4", // [1131]
    "SERIAL8", // [1132]
    "SMALLINT", // [1133]
    "SMALLSERIAL", // [1134]
    "TSQUERY", // [1135]
    "TSRANGE", // [1136]
    "TSTZRANGE", // [1137]
    "TSVECTOR", // [1138]
    "UUID", // [1139]
    "VARCHAR", // [1140]
    "VECTOR", // [1141]
    "XML", // [1142]
    "SmallintKeywordSegment", // [1143]
    "IntegerKeywordSegment", // [1144]
    "IntKeywordSegment", // [1145]
    "Int2KeywordSegment", // [1146]
    "Int4KeywordSegment", // [1147]
    "Int8KeywordSegment", // [1148]
    "BigintKeywordSegment", // [1149]
    "Float4KeywordSegment", // [1150]
    "Float8KeywordSegment", // [1151]
    "RealKeywordSegment", // [1152]
    "DoubleKeywordSegment", // [1153]
    "PrecisionKeywordSegment", // [1154]
    "PRECISION", // [1155]
    "SmallserialKeywordSegment", // [1156]
    "SerialKeywordSegment", // [1157]
    "Serial2KeywordSegment", // [1158]
    "Serial4KeywordSegment", // [1159]
    "Serial8KeywordSegment", // [1160]
    "BigserialKeywordSegment", // [1161]
    "FloatKeywordSegment", // [1162]
    "BracketedArguments", // [1163]
    "bracketed_arguments", // [1164]
    "DecimalKeywordSegment", // [1165]
    "NumericKeywordSegment", // [1166]
    "MoneyKeywordSegment", // [1167]
    "BpcharKeywordSegment", // [1168]
    "CharKeywordSegment", // [1169]
    "VaryingKeywordSegment", // [1170]
    "VARYING", // [1171]
    "CharacterKeywordSegment", // [1172]
    "VarcharKeywordSegment", // [1173]
    "ByteaKeywordSegment", // [1174]
    "BooleanKeywordSegment", // [1175]
    "BoolKeywordSegment", // [1176]
    "PointKeywordSegment", // [1177]
    "LineKeywordSegment", // [1178]
    "LsegKeywordSegment", // [1179]
    "BoxKeywordSegment", // [1180]
    "PathKeywordSegment", // [1181]
    "PolygonKeywordSegment", // [1182]
    "CircleKeywordSegment", // [1183]
    "CidrKeywordSegment", // [1184]
    "InetKeywordSegment", // [1185]
    "MacaddrKeywordSegment", // [1186]
    "Macaddr8KeywordSegment", // [1187]
    "TsvectorKeywordSegment", // [1188]
    "TsqueryKeywordSegment", // [1189]
    "BitKeywordSegment", // [1190]
    "UuidKeywordSegment", // [1191]
    "XmlKeywordSegment", // [1192]
    "JsonKeywordSegment", // [1193]
    "JsonbKeywordSegment", // [1194]
    "Int4rangeKeywordSegment", // [1195]
    "Int8rangeKeywordSegment", // [1196]
    "NumrangeKeywordSegment", // [1197]
    "TsrangeKeywordSegment", // [1198]
    "TstzrangeKeywordSegment", // [1199]
    "DaterangeKeywordSegment", // [1200]
    "Pg_lsnKeywordSegment", // [1201]
    "VectorKeywordSegment", // [1202]
    "DatatypeIdentifierSegment", // [1203]
    "ArrayTypeSuffixSegment", // [1204]
    "array_type_suffix", // [1205]
    "ArrayTypeSegment", // [1206]
    "array_type", // [1207]
    "SizedArrayTypeSegment", // [1208]
    "sized_array_type", // [1209]
    "DatePartFunctionName", // [1210]
    "DatetimeUnitSegment", // [1211]
    "CENTURY", // [1212]
    "DAY", // [1213]
    "DAYOFYEAR", // [1214]
    "DECADE", // [1215]
    "DOW", // [1216]
    "DOY", // [1217]
    "EPOCH", // [1218]
    "HOUR", // [1219]
    "ISODOW", // [1220]
    "ISOYEAR", // [1221]
    "MICROSECONDS", // [1222]
    "MILLENNIUM", // [1223]
    "MILLISECOND", // [1224]
    "MILLISECONDS", // [1225]
    "MINUTE", // [1226]
    "MONTH", // [1227]
    "QUARTER", // [1228]
    "SECOND", // [1229]
    "TIMEZONE", // [1230]
    "TIMEZONE_HOUR", // [1231]
    "TIMEZONE_MINUTE", // [1232]
    "WEEK", // [1233]
    "WEEKDAY", // [1234]
    "YEAR", // [1235]
    "FunctionContentsGrammar", // [1236]
    "DateKeywordSegment", // [1237]
    "TimeWithTZGrammar", // [1238]
    "IntervalKeywordSegment", // [1239]
    "TimetzKeywordSegment", // [1240]
    "TimestamptzKeywordSegment", // [1241]
    "DATETIME", // [1242]
    "date_part", // [1243]
    "DAY_HOUR", // [1244]
    "DAY_MICROSECOND", // [1245]
    "DAY_MINUTE", // [1246]
    "DAY_SECOND", // [1247]
    "DAYOFMONTH", // [1248]
    "DAYOFWEEK", // [1249]
    "DAYS", // [1250]
    "DBCC", // [1251]
    "DEALLOCATE", // [1252]
    "DeallocateKeywordSegment", // [1253]
    "PrepareKeywordSegment", // [1254]
    "PREPARE", // [1255]
    "DEC", // [1256]
    "DECLARE", // [1257]
    "DEFAULTS", // [1258]
    "PropertiesNakedIdentifierSegment", // [1259]
    "DefinitionArgumentValueGrammar", // [1260]
    "DefinitionParameterSegment", // [1261]
    "definition_parameter", // [1262]
    "DELAY_KEY_WRITE", // [1263]
    "DELAYED", // [1264]
    "TableExpressionSegment", // [1265]
    "table_expression", // [1266]
    "JoinClauseSegment", // [1267]
    "join_clause", // [1268]
    "WhereClauseSegment", // [1269]
    "where_clause", // [1270]
    "RETURNING", // [1271]
    "ReturningKeywordSegment", // [1272]
    "SemicolonSegment", // [1273]
    "DELIMITERS", // [1274]
    "DENY", // [1275]
    "DESC", // [1276]
    "DESCRIBE", // [1277]
    "DescribeKeywordSegment", // [1278]
    "DESTROY", // [1279]
    "DESTRUCTOR", // [1280]
    "DISABLE_PAGE_SKIPPING", // [1281]
    "DISCARD", // [1282]
    "DiscardKeywordSegment", // [1283]
    "PLANS", // [1284]
    "PlansKeywordSegment", // [1285]
    "DISK", // [1286]
    "DISTINCTROW", // [1287]
    "DISTRIBUTED", // [1288]
    "DIV", // [1289]
    "LanguageClauseSegment", // [1290]
    "language_clause", // [1291]
    "DOCUMENT", // [1292]
    "literal", // [1293]
    "LiteralSegment", // [1294]
    "dot", // [1295]
    "DUMMY", // [1296]
    "DUMP", // [1297]
    "ELSEIF", // [1298]
    "EMPTY", // [1299]
    "EmptyStructLiteralBracketsSegment", // [1300]
    "struct_literal", // [1301]
    "ENCLOSED", // [1302]
    "}", // [1303]
    "end_curly_bracket", // [1304]
    "ERRLVL", // [1305]
    "ESCAPED", // [1306]
    "EXCEPT", // [1307]
    "EXCLUSIVE", // [1308]
    "EXISTING", // [1309]
    "EXISTS", // [1310]
    "EXIT", // [1311]
    "EXPLAIN", // [1312]
    "SETTINGS", // [1313]
    "SUMMARY", // [1314]
    "TIMING", // [1315]
    "WAL", // [1316]
    "CostsKeywordSegment", // [1317]
    "SettingsKeywordSegment", // [1318]
    "BuffersKeywordSegment", // [1319]
    "WalKeywordSegment", // [1320]
    "TimingKeywordSegment", // [1321]
    "SummaryKeywordSegment", // [1322]
    "YAML", // [1323]
    "YamlKeywordSegment", // [1324]
    "ExplainKeywordSegment", // [1325]
    "ExplainOptionSegment", // [1326]
    "explain_option", // [1327]
    "InsertStatementSegment", // [1328]
    "insert_statement", // [1329]
    "UpdateStatementSegment", // [1330]
    "update_statement", // [1331]
    "DeleteStatementSegment", // [1332]
    "delete_statement", // [1333]
    "Expression_A_Grammar", // [1334]
    "Tail_Recurse_Expression_A_Grammar", // [1335]
    "LikeExpressionGrammar", // [1336]
    "ILIKE", // [1337]
    "SIMILAR", // [1338]
    "InOperatorGrammar", // [1339]
    "QuantifiedComparisonOperatorGrammar", // [1340]
    "IsClauseGrammar", // [1341]
    "NAN", // [1342]
    "NFC", // [1343]
    "NFD", // [1344]
    "NFKC", // [1345]
    "NFKD", // [1346]
    "NORMALIZED", // [1347]
    "UNKNOWN", // [1348]
    "IsNullGrammar", // [1349]
    "ISNULL", // [1350]
    "NotNullGrammar", // [1351]
    "NOTNULL", // [1352]
    "CollateGrammar", // [1353]
    "BetweenKeywordSegment", // [1354]
    "Expression_B_Grammar", // [1355]
    "AndKeywordSegment", // [1356]
    "PatternMatchingGrammar", // [1357]
    "PRIOR", // [1358]
    "~", // [1359]
    "TildeSegment", // [1360]
    "NotOperatorGrammar", // [1361]
    "PriorKeywordSegment", // [1362]
    "Tail_Recurse_Expression_B_Grammar", // [1363]
    "WalrusOperatorSegment", // [1364]
    ":=", // [1365]
    "ExistsKeywordSegment", // [1366]
    "Expression_D_Grammar", // [1367]
    "CaseExpressionSegment", // [1368]
    "case_expression", // [1369]
    "TimeZoneGrammar", // [1370]
    "time_zone_grammar", // [1371]
    "FullTextSearchOperatorSegment", // [1372]
    "full_text_search_operator", // [1373]
    "LateralColumnAliasExpressionGrammar", // [1374]
    "LocalAliasSegment", // [1375]
    "local_alias_segment", // [1376]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [1377]
    "SingleIdentifierWildcardSegment", // [1378]
    "NullLiteralSegment", // [1379]
    "DateTimeLiteralGrammar", // [1380]
    "datetime_literal", // [1381]
    "ListComprehensionGrammar", // [1382]
    "AccessorGrammar", // [1383]
    "SelectStatementSegment", // [1384]
    "select_statement", // [1385]
    "TypedStructLiteralSegment", // [1386]
    "typed_struct_literal", // [1387]
    "ArrayExpressionSegment", // [1388]
    "array_expression", // [1389]
    "OverlapsClauseSegment", // [1390]
    "overlaps_clause", // [1391]
    "EXTRACT", // [1392]
    "boolean_literal", // [1393]
    "LiteralKeywordSegment", // [1394]
    "FETCH", // [1395]
    "FetchKeywordSegment", // [1396]
    "NEXT", // [1397]
    "NextKeywordSegment", // [1398]
    "TiesKeywordSegment", // [1399]
    "TIES", // [1400]
    "FIELDS", // [1401]
    "IMPORT", // [1402]
    "LISTEN", // [1403]
    "LOAD", // [1404]
    "LOCK", // [1405]
    "MERGE", // [1406]
    "NOTIFY", // [1407]
    "REASSIGN", // [1408]
    "REINDEX", // [1409]
    "ROLLBACK", // [1410]
    "SHOW", // [1411]
    "UNLISTEN", // [1412]
    "VACUUM", // [1413]
    "postgres_copy_stdin_data_block", // [1414]
    "psql_copy_command", // [1415]
    "psql_set_command", // [1416]
    "PostgresCopyStdinDataStatementSegment", // [1417]
    "postgres_copy_stdin_data_statement", // [1418]
    "PsqlCopyMetaCommandStatementSegment", // [1419]
    "psql_copy_meta_command_statement", // [1420]
    "PsqlSetMetaCommandStatementSegment", // [1421]
    "psql_set_meta_command_statement", // [1422]
    "StatementSegment", // [1423]
    "statement", // [1424]
    "FILLFACTOR", // [1425]
    "FILTER", // [1426]
    "FilterKeywordSegment", // [1427]
    "FLUSH", // [1428]
    "FOLLOWING", // [1429]
    "LockedKeywordSegment", // [1430]
    "LOCKED", // [1431]
    "FORWARD", // [1432]
    "GROUPS", // [1433]
    "FrameClauseUnitGrammar", // [1434]
    "UnboundedKeywordSegment", // [1435]
    "UNBOUNDED", // [1436]
    "PRECEDING", // [1437]
    "PrecedingKeywordSegment", // [1438]
    "FollowingKeywordSegment", // [1439]
    "ExcludeKeywordSegment", // [1440]
    "OthersKeywordSegment", // [1441]
    "OTHERS", // [1442]
    "GroupsKeywordSegment", // [1443]
    "FREETEXT", // [1444]
    "FREETEXTTABLE", // [1445]
    "FromExpressionSegment", // [1446]
    "from_expression", // [1447]
    "HAVING", // [1448]
    "INTERSECT", // [1449]
    "MINUS", // [1450]
    "OFFSET", // [1451]
    "QUALIFY", // [1452]
    "UNION", // [1453]
    "LimitClauseSegment", // [1454]
    "limit_clause", // [1455]
    "OrderKeywordSegment", // [1456]
    "HavingKeywordSegment", // [1457]
    "QualifyKeywordSegment", // [1458]
    "WindowKeywordSegment", // [1459]
    "SetOperatorSegment", // [1460]
    "set_operator", // [1461]
    "WithNoSchemaBindingClauseSegment", // [1462]
    "with_no_schema_binding_clause", // [1463]
    "OffsetKeywordSegment", // [1464]
    "ForClauseSegment", // [1465]
    "for_clause", // [1466]
    "PreTableFunctionKeywordsGrammar", // [1467]
    "LATERAL", // [1468]
    "TemporalQuerySegment", // [1469]
    "temporal_query", // [1470]
    "FromClauseTerminatorGrammar", // [1471]
    "SamplingExpressionSegment", // [1472]
    "TABLESAMPLE", // [1473]
    "sample_expression", // [1474]
    "PostTableExpressionGrammar", // [1475]
    "MLTableExpressionSegment", // [1476]
    "ML", // [1477]
    "ml_table_expression", // [1478]
    "FromExpressionElementSegment", // [1479]
    "from_expression_element", // [1480]
    "indented_joins", // [1481]
    "FULLTEXT", // [1482]
    "NamedArgumentSegment", // [1483]
    "named_argument", // [1484]
    "VariadicKeywordSegment", // [1485]
    "LEADING", // [1486]
    "TRAILING", // [1487]
    "TrimParametersGrammar", // [1488]
    "FunctionContentsExpressionGrammar", // [1489]
    "AggregateOrderByClause", // [1490]
    "aggregate_order_by", // [1491]
    "SEPARATOR", // [1492]
    "SeparatorKeywordSegment", // [1493]
    "IgnoreRespectNullsGrammar", // [1494]
    "IGNORE", // [1495]
    "RESPECT", // [1496]
    "IndexColumnDefinitionSegment", // [1497]
    "index_column_definition", // [1498]
    "EmptyStructLiteralSegment", // [1499]
    "ReturnKeywordSegment", // [1500]
    "BeginKeywordSegment", // [1501]
    "AtomicKeywordSegment", // [1502]
    "WithCompoundStatementSegment", // [1503]
    "with_compound_statement", // [1504]
    "FunctionNameIdentifierSegment", // [1505]
    "OUT", // [1506]
    "OutKeywordSegment", // [1507]
    "FunctionParameterGrammar", // [1508]
    "DatePartFunctionNameSegment", // [1509]
    "DateTimeFunctionContentsSegment", // [1510]
    "function_contents", // [1511]
    "ColumnsExpressionGrammar", // [1512]
    "ColumnsExpressionFunctionNameSegment", // [1513]
    "FunctionContentsSegment", // [1514]
    "PostFunctionGrammar", // [1515]
    "glob_operator", // [1516]
    "AccessPermissionsSegment", // [1517]
    "access_permissions", // [1518]
    "AccessObjectSegment", // [1519]
    "access_object", // [1520]
    "AccessTargetSegment", // [1521]
    "access_target", // [1522]
    "GRANTED", // [1523]
    "GrantedKeywordSegment", // [1524]
    "GREATEST", // [1525]
    "CubeRollupClauseSegment", // [1526]
    "cube_rollup_clause", // [1527]
    "GroupingSetsClauseSegment", // [1528]
    "GROUPING", // [1529]
    "grouping_sets_clause", // [1530]
    "GroupByClauseTerminatorGrammar", // [1531]
    "GroupingKeywordSegment", // [1532]
    "SetsKeywordSegment", // [1533]
    "SETS", // [1534]
    "HEAP", // [1535]
    "HIGH_PRIORITY", // [1536]
    "HOLD", // [1537]
    "HOLDLOCK", // [1538]
    "HOST", // [1539]
    "HOSTS", // [1540]
    "HOUR_MICROSECOND", // [1541]
    "HOUR_MINUTE", // [1542]
    "HOUR_SECOND", // [1543]
    "IDENTIFIED", // [1544]
    "identifier", // [1545]
    "IdentifierSegment", // [1546]
    "IDENTITY_INSERT", // [1547]
    "IDENTITYCOL", // [1548]
    "IfKeywordSegment", // [1549]
    "IgnoreKeywordSegment", // [1550]
    "RespectKeywordSegment", // [1551]
    "ImportKeywordSegment", // [1552]
    "ImportForeignSchemaGrammar", // [1553]
    "ExceptKeywordSegment", // [1554]
    "NakedIdentifierFullSegment", // [1555]
    "IntoKeywordSegment", // [1556]
    "INTO", // [1557]
    "AscKeywordSegment", // [1558]
    "DescKeywordSegment", // [1559]
    "OperatorClassReferenceSegment", // [1560]
    "LAST", // [1561]
    "LastKeywordSegment", // [1562]
    "operator_class_reference", // [1563]
    "IndexElementOptionsSegment", // [1564]
    "index_element_options", // [1565]
    "INDEX_CLEANUP", // [1566]
    "INDEXES", // [1567]
    "INFILE", // [1568]
    "INFIX", // [1569]
    "INITIALIZE", // [1570]
    "INLINE", // [1571]
    "INSENSITIVE", // [1572]
    "AsAliasExpressionSegment", // [1573]
    "OVERRIDING", // [1574]
    "OverridingKeywordSegment", // [1575]
    "ConflictKeywordSegment", // [1576]
    "ConflictTargetSegment", // [1577]
    "conflict_target", // [1578]
    "ConflictActionSegment", // [1579]
    "conflict_action", // [1580]
    "INSERT_ID", // [1581]
    "INT1", // [1582]
    "INT3", // [1583]
    "INTEGRATIONS", // [1584]
    "YearKeywordSegment", // [1585]
    "MonthKeywordSegment", // [1586]
    "DayKeywordSegment", // [1587]
    "HourKeywordSegment", // [1588]
    "MinuteKeywordSegment", // [1589]
    "SecondKeywordSegment", // [1590]
    "NanLiteralSegment", // [1591]
    "UnknownLiteralSegment", // [1592]
    "NormalizedGrammar", // [1593]
    "JsonTestGrammar", // [1594]
    "IsnullKeywordSegment", // [1595]
    "IS_SUPERUSER", // [1596]
    "ISAM", // [1597]
    "ISOLATION", // [1598]
    "ITERATE", // [1599]
    "ConditionalJoinKeywordsGrammar", // [1600]
    "JoinKeywordsGrammar", // [1601]
    "NestedJoinGrammar", // [1602]
    "indented_using_on", // [1603]
    "MatchConditionSegment", // [1604]
    "match_condition", // [1605]
    "JoinOnConditionSegment", // [1606]
    "join_on_condition", // [1607]
    "JoinUsingConditionGrammar", // [1608]
    "UnconditionalJoinKeywordsGrammar", // [1609]
    "ExtendedNaturalJoinKeywordsGrammar", // [1610]
    "indented_on_contents", // [1611]
    "InnerKeywordSegment", // [1612]
    "LeftKeywordSegment", // [1613]
    "RightKeywordSegment", // [1614]
    "OuterKeywordSegment", // [1615]
    "OUTER", // [1616]
    "JsonTypeGrammar", // [1617]
    "SCALAR", // [1618]
    "JsonUniqueKeysGrammar", // [1619]
    "ScalarKeywordSegment", // [1620]
    "KEYS", // [1621]
    "KeysKeywordSegment", // [1622]
    "KILL", // [1623]
    "LABEL", // [1624]
    "LANCOMPILER", // [1625]
    "SingleQuotedIdentifierSegment", // [1626]
    "LAST_INSERT_ID", // [1627]
    "LEAST", // [1628]
    "LEAVE", // [1629]
    "LESS", // [1630]
    "LikeGrammar", // [1631]
    "IlikeKeywordSegment", // [1632]
    "SimilarKeywordSegment", // [1633]
    "IncludingKeywordSegment", // [1634]
    "ExcludingKeywordSegment", // [1635]
    "CommentsKeywordSegment", // [1636]
    "ConstraintsKeywordSegment", // [1637]
    "DefaultsKeywordSegment", // [1638]
    "IndexesKeywordSegment", // [1639]
    "LINENO", // [1640]
    "LINES", // [1641]
    "ListenKeywordSegment", // [1642]
    "DollarNumericLiteralSegment", // [1643]
    "ArrayLiteralSegment", // [1644]
    "array_literal", // [1645]
    "TypedArrayLiteralSegment", // [1646]
    "typed_array_literal", // [1647]
    "ObjectLiteralSegment", // [1648]
    "object_literal", // [1649]
    "LoadKeywordSegment", // [1650]
    "LOCATION", // [1651]
    "LockKeywordSegment", // [1652]
    "ExclusiveKeywordSegment", // [1653]
    "ModeKeywordSegment", // [1654]
    "MODE", // [1655]
    "LOCKS", // [1656]
    "LOGS", // [1657]
    "LONG", // [1658]
    "LONGBLOB", // [1659]
    "LONGTEXT", // [1660]
    "LOOP", // [1661]
    "LOW_PRIORITY", // [1662]
    "MlKeywordSegment", // [1663]
    "MATCH", // [1664]
    "MATCHED", // [1665]
    "MAX_ROWS", // [1666]
    "MAXEXTENTS", // [1667]
    "MEDIUMBLOB", // [1668]
    "MEDIUMINT", // [1669]
    "MEDIUMTEXT", // [1670]
    "MergeKeywordSegment", // [1671]
    "MergeMatchedClauseSegment", // [1672]
    "merge_when_matched_clause", // [1673]
    "MergeNotMatchedClauseSegment", // [1674]
    "merge_when_not_matched_clause", // [1675]
    "MergeNotMatchedBySourceClauseSegment", // [1676]
    "merge_when_not_matched_by_source_clause", // [1677]
    "MatchedKeywordSegment", // [1678]
    "ThenKeywordSegment", // [1679]
    "THEN", // [1680]
    "MergeUpdateClauseSegment", // [1681]
    "merge_update_clause", // [1682]
    "MergeDeleteClauseSegment", // [1683]
    "merge_delete_clause", // [1684]
    "SourceKeywordSegment", // [1685]
    "SOURCE", // [1686]
    "TargetKeywordSegment", // [1687]
    "TARGET", // [1688]
    "MergeInsertClauseSegment", // [1689]
    "merge_insert_clause", // [1690]
    "MergeIntoLiteralGrammar", // [1691]
    "AliasedTableReferenceGrammar", // [1692]
    "MergeMatchSegment", // [1693]
    "merge_match", // [1694]
    "SetClauseListSegment", // [1695]
    "set_clause_list", // [1696]
    "meta_command_query_buffer", // [1697]
    "meta_command", // [1698]
    "MetaCommandQueryBufferSegment", // [1699]
    "MIDDLEINT", // [1700]
    "MIN_ROWS", // [1701]
    "MINUTE_MICROSECOND", // [1702]
    "MINUTE_SECOND", // [1703]
    "MLSLABEL", // [1704]
    "MODULUS", // [1705]
    "MONTHNAME", // [1706]
    "MOVE", // [1707]
    "MultilineConcatenateNewline", // [1708]
    "NewlineSegment", // [1709]
    "MYISAM", // [1710]
    "naked_identifier_all", // [1711]
    "naked_identifier", // [1712]
    "=>", // [1713]
    "RightArrowSegment", // [1714]
    "WindowSpecificationSegment", // [1715]
    "window_specification", // [1716]
    "NamedWindowExpressionSegment", // [1717]
    "named_window_expression", // [1718]
    "NAMES", // [1719]
    "null_literal", // [1720]
    "NATIONAL", // [1721]
    "NATURAL", // [1722]
    "NaturalKeywordSegment", // [1723]
    "NCHAR", // [1724]
    "sign_indicator", // [1725]
    "NO_WRITE_TO_BINLOG", // [1726]
    "NOAUDIT", // [1727]
    "NOCACHE", // [1728]
    "NOCHECK", // [1729]
    "NOCOMPRESS", // [1730]
    "NOCREATEUSER", // [1731]
    "NOCYCLE", // [1732]
    "UnorderedSelectStatementSegment", // [1733]
    "NonSetSelectableGrammar", // [1734]
    "BracketedSetExpressionGrammar", // [1735]
    "NONCLUSTERED", // [1736]
    "NOORDER", // [1737]
    "NORMALIZE", // [1738]
    "NfcKeywordSegment", // [1739]
    "NfdKeywordSegment", // [1740]
    "NfkcKeywordSegment", // [1741]
    "NfkdKeywordSegment", // [1742]
    "NormalizedKeywordSegment", // [1743]
    "RawNotSegment", // [1744]
    "NotnullKeywordSegment", // [1745]
    "NotifyKeywordSegment", // [1746]
    "NULLIF", // [1747]
    "ObjectLiteralElementSegment", // [1748]
    "object_literal_element", // [1749]
    "start_curly_bracket", // [1750]
    "OBJECTS", // [1751]
    "OFF", // [1752]
    "OFFLINE", // [1753]
    "OFFSETS", // [1754]
    "ONLINE", // [1755]
    "OPENDATASOURCE", // [1756]
    "OPENQUERY", // [1757]
    "OPENROWSET", // [1758]
    "OPENXML", // [1759]
    "OPERATION", // [1760]
    "OPTIMIZE", // [1761]
    "OPTIONALLY", // [1762]
    "OptionsListGrammar", // [1763]
    "WithFillSegment", // [1764]
    "with_fill", // [1765]
    "NoorderKeywordSegment", // [1766]
    "ORDINALITY", // [1767]
    "OUTFILE", // [1768]
    "OverKeywordSegment", // [1769]
    "OVER", // [1770]
    "OverlapsKeywordSegment", // [1771]
    "OVERLAPS", // [1772]
    "OVERLAY", // [1773]
    "OVERWRITE", // [1774]
    "PACK_KEYS", // [1775]
    "parameter", // [1776]
    "?", // [1777]
    "PARAMETERS", // [1778]
    "PARTIAL", // [1779]
    "ModulusKeywordSegment", // [1780]
    "RemainderKeywordSegment", // [1781]
    "REMAINDER", // [1782]
    "PASSING", // [1783]
    "SlashSegment", // [1784]
    "path_segment", // [1785]
    "WordSegment", // [1786]
    "PCTFREE", // [1787]
    "pipe", // [1788]
    "PLACING", // [1789]
    "POSITION", // [1790]
    "WithinGroupClauseSegment", // [1791]
    "WITHIN", // [1792]
    "withingroup_clause", // [1793]
    "OverClauseSegment", // [1794]
    "over_clause", // [1795]
    "FilterClauseGrammar", // [1796]
    "POSTFIX", // [1797]
    "PostgresCopyStdinDataSegment", // [1798]
    "LateralKeywordSegment", // [1799]
    "PREFIX", // [1800]
    "PREORDER", // [1801]
    "MergeStatementSegment", // [1802]
    "merge_statement", // [1803]
    "PREPARED", // [1804]
    "PRINT", // [1805]
    "PROC", // [1806]
    "PROCESS", // [1807]
    "PROCESS_TOAST", // [1808]
    "PROCESSLIST", // [1809]
    "properties_naked_identifier", // [1810]
    "PsqlCopyMetaCommandSegment", // [1811]
    "PsqlSetMetaCommandSegment", // [1812]
    "psql_variable_colon", // [1813]
    "PsqlVariableColonSegment", // [1814]
    "PublicationTableSegment", // [1815]
    "publication_table", // [1816]
    "ExtendedTableReferenceGrammar", // [1817]
    "PURGE", // [1818]
    "operator", // [1819]
    "SOME", // [1820]
    "AnyKeywordSegment", // [1821]
    "SomeKeywordSegment", // [1822]
    "quoted_identifier", // [1823]
    "quoted_literal", // [1824]
    "MultilineConcatenateDelimiterGrammar", // [1825]
    "UESCAPE", // [1826]
    "UescapeKeywordSegment", // [1827]
    "unicode_escape_value", // [1828]
    "RAID0", // [1829]
    "RAISERROR", // [1830]
    "raw_comparison_operator", // [1831]
    "RAW", // [1832]
    "READTEXT", // [1833]
    "ReassignKeywordSegment", // [1834]
    "RECHECK", // [1835]
    "RECONFIGURE", // [1836]
    "REF", // [1837]
    "ReferenceMatchGrammar", // [1838]
    "ReferentialActionGrammar", // [1839]
    "MatchKeywordSegment", // [1840]
    "SIMPLE", // [1841]
    "PartialKeywordSegment", // [1842]
    "SimpleKeywordSegment", // [1843]
    "ActionKeywordSegment", // [1844]
    "REGEXP", // [1845]
    "ReindexKeywordSegment", // [1846]
    "RelationOptionSegment", // [1847]
    "relation_option", // [1848]
    "RELATIVE", // [1849]
    "RELEASE", // [1850]
    "RELOAD", // [1851]
    "REPEAT", // [1852]
    "REPEATABLE", // [1853]
    "REQUIRE", // [1854]
    "SessionKeywordSegment", // [1855]
    "SESSION", // [1856]
    "RESIGNAL", // [1857]
    "RETRIEVE", // [1858]
    "right_arrow", // [1859]
    "RLIKE", // [1860]
    "ROLES", // [1861]
    "ROWCOUNT", // [1862]
    "ROWGUIDCOL", // [1863]
    "ROWID", // [1864]
    "ROWNUM", // [1865]
    "TablesampleKeywordSegment", // [1866]
    "BernoulliKeywordSegment", // [1867]
    "RepeatableKeywordSegment", // [1868]
    "SAVE", // [1869]
    "SAVEPOINT", // [1870]
    "SCROLL", // [1871]
    "SECOND_MICROSECOND", // [1872]
    "LabelKeywordSegment", // [1873]
    "WildcardExpressionSegment", // [1874]
    "wildcard_expression", // [1875]
    "SelectClauseModifierSegment", // [1876]
    "select_clause_modifier", // [1877]
    "SelectClauseElementSegment", // [1878]
    "select_clause_element", // [1879]
    "SelectClauseSegment", // [1880]
    "select_clause", // [1881]
    "IntoClauseSegment", // [1882]
    "into_clause", // [1883]
    "FromClauseSegment", // [1884]
    "from_clause", // [1885]
    "GroupByClauseSegment", // [1886]
    "groupby_clause", // [1887]
    "HavingClauseSegment", // [1888]
    "having_clause", // [1889]
    "NamedWindowSegment", // [1890]
    "named_window", // [1891]
    "OffsetClauseSegment", // [1892]
    "offset_clause", // [1893]
    "FetchClauseSegment", // [1894]
    "fetch_clause", // [1895]
    "WithCompoundNonSelectStatementSegment", // [1896]
    "NonWithSelectableGrammar", // [1897]
    "statement_terminator", // [1898]
    "SERIALIZABLE", // [1899]
    "SERVER_ENCODING", // [1900]
    "SERVER_VERSION", // [1901]
    "SetClauseSegment", // [1902]
    "set_clause", // [1903]
    "UnionGrammar", // [1904]
    "IntersectKeywordSegment", // [1905]
    "MinusKeywordSegment", // [1906]
    "TimeKeywordSegment", // [1907]
    "ZoneKeywordSegment", // [1908]
    "ZONE", // [1909]
    "SETUSER", // [1910]
    "SHARES", // [1911]
    "ShowKeywordSegment", // [1912]
    "Is_superuserKeywordSegment", // [1913]
    "Server_encodingKeywordSegment", // [1914]
    "Server_versionKeywordSegment", // [1915]
    "SHUTDOWN", // [1916]
    "SIGNAL", // [1917]
    "PositiveSegment", // [1918]
    "NegativeSegment", // [1919]
    "slash", // [1920]
    "slice", // [1921]
    "SNAPSHOT", // [1922]
    "SONAME", // [1923]
    "SPATIAL", // [1924]
    "SQL", // [1925]
    "SQL_BIG_RESULT", // [1926]
    "SQL_BIG_SELECTS", // [1927]
    "SQL_BIG_TABLES", // [1928]
    "SQL_CALC_FOUND_ROWS", // [1929]
    "SQL_LOG_OFF", // [1930]
    "SQL_LOG_UPDATE", // [1931]
    "SQL_LOW_PRIORITY_UPDATES", // [1932]
    "SQL_SELECT_LIMIT", // [1933]
    "SQL_SMALL_RESULT", // [1934]
    "SQL_WARNINGS", // [1935]
    "SQLCA", // [1936]
    "SSL", // [1937]
    "STANDALONE", // [1938]
    "star", // [1939]
    "STARTING", // [1940]
    "STARTS", // [1941]
    "TransactionStatementSegment", // [1942]
    "transaction_statement", // [1943]
    "DropTableStatementSegment", // [1944]
    "drop_table_statement", // [1945]
    "DropViewStatementSegment", // [1946]
    "drop_view_statement", // [1947]
    "CreateUserStatementSegment", // [1948]
    "create_user_statement", // [1949]
    "DropUserStatementSegment", // [1950]
    "drop_user_statement", // [1951]
    "TruncateStatementSegment", // [1952]
    "truncate_table", // [1953]
    "AccessStatementSegment", // [1954]
    "access_statement", // [1955]
    "CreateTableStatementSegment", // [1956]
    "create_table_statement", // [1957]
    "CreateRoleStatementSegment", // [1958]
    "create_role_statement", // [1959]
    "DropRoleStatementSegment", // [1960]
    "drop_role_statement", // [1961]
    "AlterTableStatementSegment", // [1962]
    "alter_table_statement", // [1963]
    "CreateSchemaStatementSegment", // [1964]
    "create_schema_statement", // [1965]
    "SetSchemaStatementSegment", // [1966]
    "set_schema_statement", // [1967]
    "DropSchemaStatementSegment", // [1968]
    "drop_schema_statement", // [1969]
    "DropTypeStatementSegment", // [1970]
    "drop_type_statement", // [1971]
    "CreateDatabaseStatementSegment", // [1972]
    "create_database_statement", // [1973]
    "DropDatabaseStatementSegment", // [1974]
    "drop_database_statement", // [1975]
    "CreateIndexStatementSegment", // [1976]
    "create_index_statement", // [1977]
    "DropIndexStatementSegment", // [1978]
    "drop_index_statement", // [1979]
    "CreateViewStatementSegment", // [1980]
    "create_view_statement", // [1981]
    "CreateCastStatementSegment", // [1982]
    "create_cast_statement", // [1983]
    "DropCastStatementSegment", // [1984]
    "drop_cast_statement", // [1985]
    "CreateFunctionStatementSegment", // [1986]
    "create_function_statement", // [1987]
    "DropFunctionStatementSegment", // [1988]
    "drop_function_statement", // [1989]
    "CreateModelStatementSegment", // [1990]
    "create_model_statement", // [1991]
    "DropModelStatementSegment", // [1992]
    "drop_MODELstatement", // [1993]
    "DescribeStatementSegment", // [1994]
    "describe_statement", // [1995]
    "UseStatementSegment", // [1996]
    "use_statement", // [1997]
    "ExplainStatementSegment", // [1998]
    "explain_statement", // [1999]
    "CreateSequenceStatementSegment", // [2000]
    "create_sequence_statement", // [2001]
    "AlterSequenceStatementSegment", // [2002]
    "alter_sequence_statement", // [2003]
    "DropSequenceStatementSegment", // [2004]
    "drop_sequence_statement", // [2005]
    "CreateTriggerStatementSegment", // [2006]
    "create_trigger", // [2007]
    "DropTriggerStatementSegment", // [2008]
    "drop_trigger", // [2009]
    "AlterDefaultPrivilegesStatementSegment", // [2010]
    "alter_default_privileges_statement", // [2011]
    "DropOwnedStatementSegment", // [2012]
    "drop_owned_statement", // [2013]
    "ReassignOwnedStatementSegment", // [2014]
    "reassign_owned_statement", // [2015]
    "CommentOnStatementSegment", // [2016]
    "comment_clause", // [2017]
    "AnalyzeStatementSegment", // [2018]
    "analyze_statement", // [2019]
    "CreateTableAsStatementSegment", // [2020]
    "create_table_as_statement", // [2021]
    "AlterTriggerStatementSegment", // [2022]
    "alter_trigger", // [2023]
    "SetStatementSegment", // [2024]
    "set_statement", // [2025]
    "AlterSystemStatementSegment", // [2026]
    "alter_system_statement", // [2027]
    "AlterPolicyStatementSegment", // [2028]
    "alter_policy_statement", // [2029]
    "CreatePolicyStatementSegment", // [2030]
    "create_policy_statement", // [2031]
    "DropPolicyStatementSegment", // [2032]
    "drop_policy_statement", // [2033]
    "CreateDomainStatementSegment", // [2034]
    "create_domain_statement", // [2035]
    "AlterDomainStatementSegment", // [2036]
    "alter_domain_statement", // [2037]
    "DropDomainStatementSegment", // [2038]
    "drop_domain_statement", // [2039]
    "CreateMaterializedViewStatementSegment", // [2040]
    "create_materialized_view_statement", // [2041]
    "AlterMaterializedViewStatementSegment", // [2042]
    "alter_materialized_view_statement", // [2043]
    "DropMaterializedViewStatementSegment", // [2044]
    "drop_materialized_view_statement", // [2045]
    "RefreshMaterializedViewStatementSegment", // [2046]
    "refresh_materialized_view_statement", // [2047]
    "AlterDatabaseStatementSegment", // [2048]
    "alter_database_statement", // [2049]
    "VacuumStatementSegment", // [2050]
    "vacuum_statement", // [2051]
    "AlterFunctionStatementSegment", // [2052]
    "alter_function_statement", // [2053]
    "AlterViewStatementSegment", // [2054]
    "alter_view_statement", // [2055]
    "ListenStatementSegment", // [2056]
    "listen_statement", // [2057]
    "NotifyStatementSegment", // [2058]
    "notify_statement", // [2059]
    "UnlistenStatementSegment", // [2060]
    "unlisten_statement", // [2061]
    "LoadStatementSegment", // [2062]
    "load_statement", // [2063]
    "ResetStatementSegment", // [2064]
    "reset_statement", // [2065]
    "DiscardStatementSegment", // [2066]
    "discard_statement", // [2067]
    "AlterProcedureStatementSegment", // [2068]
    "alter_procedure_statement", // [2069]
    "CreateProcedureStatementSegment", // [2070]
    "create_procedure_statement", // [2071]
    "DropProcedureStatementSegment", // [2072]
    "drop_procedure_statement", // [2073]
    "CopyStatementSegment", // [2074]
    "copy_statement", // [2075]
    "DoStatementSegment", // [2076]
    "do_statement", // [2077]
    "AlterIndexStatementSegment", // [2078]
    "alter_index_statement", // [2079]
    "ReindexStatementSegment", // [2080]
    "reindex_statement_segment", // [2081]
    "AlterRoleStatementSegment", // [2082]
    "alter_role_statement", // [2083]
    "CreateExtensionStatementSegment", // [2084]
    "create_extension_statement", // [2085]
    "DropExtensionStatementSegment", // [2086]
    "drop_extension_statement", // [2087]
    "AlterExtensionStatementSegment", // [2088]
    "alter_extension_statement", // [2089]
    "CreateSubscriptionStatementSegment", // [2090]
    "create_subscription", // [2091]
    "AlterSubscriptionStatementSegment", // [2092]
    "alter_subscription", // [2093]
    "DropSubscriptionStatementSegment", // [2094]
    "drop_subscription", // [2095]
    "CreatePublicationStatementSegment", // [2096]
    "create_publication_statement", // [2097]
    "AlterPublicationStatementSegment", // [2098]
    "alter_publication_statement", // [2099]
    "DropPublicationStatementSegment", // [2100]
    "drop_publication_statement", // [2101]
    "CreateTypeStatementSegment", // [2102]
    "create_type_statement", // [2103]
    "AlterTypeStatementSegment", // [2104]
    "alter_type_statement", // [2105]
    "CreateTextSearchConfigurationStatementSegment", // [2106]
    "create_text_search_configuration_statement", // [2107]
    "AlterTextSearchConfigurationStatementSegment", // [2108]
    "alter_text_search_configuration_statement", // [2109]
    "DropTextSearchConfigurationStatementSegment", // [2110]
    "drop_text_search_configuration_statement", // [2111]
    "AlterSchemaStatementSegment", // [2112]
    "alter_schema_statement", // [2113]
    "LockTableStatementSegment", // [2114]
    "lock_table_statement", // [2115]
    "ClusterStatementSegment", // [2116]
    "cluster_statement", // [2117]
    "CreateCollationStatementSegment", // [2118]
    "create_collation_statement", // [2119]
    "DropCollationStatementSegment", // [2120]
    "drop_collation_statement", // [2121]
    "CallStoredProcedureSegment", // [2122]
    "call_statement", // [2123]
    "CreateServerStatementSegment", // [2124]
    "create_server_statement", // [2125]
    "CreateUserMappingStatementSegment", // [2126]
    "create_user_mapping_statement", // [2127]
    "ImportForeignSchemaStatementSegment", // [2128]
    "import_foreign_schema_statement", // [2129]
    "CreateForeignTableStatementSegment", // [2130]
    "create_foreign_table_statement", // [2131]
    "DropAggregateStatementSegment", // [2132]
    "drop_aggregate_statement", // [2133]
    "CreateAggregateStatementSegment", // [2134]
    "create_aggregate_statement", // [2135]
    "AlterAggregateStatementSegment", // [2136]
    "alter_aggregate_statement", // [2137]
    "CreateStatisticsStatementSegment", // [2138]
    "create_statistics_statement", // [2139]
    "AlterStatisticsStatementSegment", // [2140]
    "alter_statistics_statement", // [2141]
    "DropStatisticsStatementSegment", // [2142]
    "drop_statistics_statement", // [2143]
    "ShowStatementSegment", // [2144]
    "show_statement", // [2145]
    "SetConstraintsStatementSegment", // [2146]
    "set_constraint_statement", // [2147]
    "CreateForeignDataWrapperStatementSegment", // [2148]
    "create_foreign_data_wrapper", // [2149]
    "MetaCommandQueryBufferStatement", // [2150]
    "meta_command_statement", // [2151]
    "DropForeignTableStatement", // [2152]
    "drop_foreign_table_statement", // [2153]
    "CreateOperatorStatementSegment", // [2154]
    "create_operator_statement", // [2155]
    "AlterForeignTableStatementSegment", // [2156]
    "alter_foreign_table_statement", // [2157]
    "SecurityLabelStatementSegment", // [2158]
    "security_label_statement", // [2159]
    "PrepareStatementSegment", // [2160]
    "prepare_statement", // [2161]
    "ExecuteStatementSegment", // [2162]
    "execute_statement", // [2163]
    "DeallocateStatementSegment", // [2164]
    "deallocate_statement", // [2165]
    "SetSessionAuthorizationStatementSegment", // [2166]
    "set_session_authorization_statement", // [2167]
    "ResetSessionAuthorizationStatementSegment", // [2168]
    "reset_session_authorization_statement", // [2169]
    "STRAIGHT_JOIN", // [2170]
    "ConcatSegment", // [2171]
    "STRIP", // [2172]
    "SUBLIST", // [2173]
    "SUBSTRING", // [2174]
    "SUCCESSFUL", // [2175]
    "SYMMETRIC", // [2176]
    "SYNONYM", // [2177]
    "SYSDATE", // [2178]
    "IndexParametersSegment", // [2179]
    "index_parameters", // [2180]
    "PrimaryKeyGrammar", // [2181]
    "ExclusionConstraintElementSegment", // [2182]
    "exclusion_constraint_element", // [2183]
    "OrdinalityKeywordSegment", // [2184]
    "Expression_A_Unary_Operator_Grammar", // [2185]
    "Expression_C_Grammar", // [2186]
    "Expression_B_Unary_Operator_Grammar", // [2187]
    "TRANSIENT", // [2188]
    "TransientKeywordSegment", // [2189]
    "TERMINATE", // [2190]
    "TERMINATED", // [2191]
    "TEXTSIZE", // [2192]
    "THAN", // [2193]
    "tilde", // [2194]
    "TimestampKeywordSegment", // [2195]
    "AtKeywordSegment", // [2196]
    "TINYBLOB", // [2197]
    "TINYINT", // [2198]
    "TINYTEXT", // [2199]
    "TOAST", // [2200]
    "TOP", // [2201]
    "TRAN", // [2202]
    "TRANSACTION", // [2203]
    "RollbackKeywordSegment", // [2204]
    "WORK", // [2205]
    "TransactionKeywordSegment", // [2206]
    "WorkKeywordSegment", // [2207]
    "ChainKeywordSegment", // [2208]
    "TRANSACTIONS", // [2209]
    "TREAT", // [2210]
    "TRIM", // [2211]
    "BothKeywordSegment", // [2212]
    "LeadingKeywordSegment", // [2213]
    "TrailingKeywordSegment", // [2214]
    "ContinueKeywordSegment", // [2215]
    "TRUSTED", // [2216]
    "TSEQUAL", // [2217]
    "StructLiteralSegment", // [2218]
    "UID", // [2219]
    "UNCOMMITTED", // [2220]
    "CrossKeywordSegment", // [2221]
    "NaturalJoinKeywordsGrammar", // [2222]
    "UnconditionalCrossJoinKeywordsGrammar", // [2223]
    "HorizontalJoinKeywordsGrammar", // [2224]
    "UNDO", // [2225]
    "UNENCRYPTED", // [2226]
    "UnionKeywordSegment", // [2227]
    "UnlistenKeywordSegment", // [2228]
    "UNLOCK", // [2229]
    "UNSIGNED", // [2230]
    "UPDATETEXT", // [2231]
    "USERS", // [2232]
    "UTC_DATE", // [2233]
    "UTC_TIME", // [2234]
    "UTC_TIMESTAMP", // [2235]
    "VacuumKeywordSegment", // [2236]
    "Disable_page_skippingKeywordSegment", // [2237]
    "Index_cleanupKeywordSegment", // [2238]
    "Process_toastKeywordSegment", // [2239]
    "VARCHAR2", // [2240]
    "VARCHARACTER", // [2241]
    "VARIABLE", // [2242]
    "VARIABLES", // [2243]
    "WAITFOR", // [2244]
    "assignment_operator", // [2245]
    "WAREHOUSES", // [2246]
    "LinestringKeywordSegment", // [2247]
    "MultipointKeywordSegment", // [2248]
    "MultilinestringKeywordSegment", // [2249]
    "MultipolygonKeywordSegment", // [2250]
    "GeometrycollectionKeywordSegment", // [2251]
    "PointzKeywordSegment", // [2252]
    "LinestringzKeywordSegment", // [2253]
    "PolygonzKeywordSegment", // [2254]
    "MultipointzKeywordSegment", // [2255]
    "MultilinestringzKeywordSegment", // [2256]
    "MultipolygonzKeywordSegment", // [2257]
    "GeometrycollectionzKeywordSegment", // [2258]
    "PointmKeywordSegment", // [2259]
    "LinestringmKeywordSegment", // [2260]
    "PolygonmKeywordSegment", // [2261]
    "MultipointmKeywordSegment", // [2262]
    "MultilinestringmKeywordSegment", // [2263]
    "MultipolygonmKeywordSegment", // [2264]
    "GeometrycollectionmKeywordSegment", // [2265]
    "PointzmKeywordSegment", // [2266]
    "LinestringzmKeywordSegment", // [2267]
    "PolygonzmKeywordSegment", // [2268]
    "MultipointzmKeywordSegment", // [2269]
    "MultilinestringzmKeywordSegment", // [2270]
    "MultipolygonzmKeywordSegment", // [2271]
    "GeometrycollectionzmKeywordSegment", // [2272]
    "CircularstringKeywordSegment", // [2273]
    "CompoundcurveKeywordSegment", // [2274]
    "CurvepolygonKeywordSegment", // [2275]
    "MulticurveKeywordSegment", // [2276]
    "MultisurfaceKeywordSegment", // [2277]
    "PolyhedralsurfaceKeywordSegment", // [2278]
    "TriangleKeywordSegment", // [2279]
    "TinKeywordSegment", // [2280]
    "SimpleGeometryGrammar", // [2281]
    "GeometryKeywordSegment", // [2282]
    "GeographyKeywordSegment", // [2283]
    "indented_then", // [2284]
    "indented_then_contents", // [2285]
    "WHILE", // [2286]
    "WHITESPACE", // [2287]
    "whitespace", // [2288]
    "WhitespaceSegment", // [2289]
    "PartitionClauseSegment", // [2290]
    "partitionby_clause", // [2291]
    "FrameClauseSegment", // [2292]
    "frame_clause", // [2293]
    "CascadedKeywordSegment", // [2294]
    "indented_ctes", // [2295]
    "CTEDefinitionSegment", // [2296]
    "common_table_expression", // [2297]
    "NonWithNonSelectableGrammar", // [2298]
    "BindingKeywordSegment", // [2299]
    "WithinKeywordSegment", // [2300]
    "WRITETEXT", // [2301]
    "X509", // [2302]
    "XMLATTRIBUTES", // [2303]
    "XMLCONCAT", // [2304]
    "XMLELEMENT", // [2305]
    "XMLEXISTS", // [2306]
    "XMLFOREST", // [2307]
    "XMLNAMESPACES", // [2308]
    "XMLPARSE", // [2309]
    "XMLPI", // [2310]
    "XMLROOT", // [2311]
    "XMLSERIALIZE", // [2312]
    "XMLTABLE", // [2313]
    "XOR", // [2314]
    "YEAR_MONTH", // [2315]
    "YES", // [2316]
    "ZEROFILL", // [2317]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,
    1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 4294967295, 4294967295,
    0, 9, 1, 2, 1, 1, 4, 3, 1, 4, 5, 211, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 212, 1, 2, 1, 1, 4, 3, 1, 4, 5, 213,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 218, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 219, 1, 2, 1, 1, 4, 3, 1, 4, 5, 220, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 81, 233, 234, 1, 233, 3, 3, 4, 235,
    236, 237, 234, 1, 237, 3, 3, 4, 235, 1, 2, 1, 1, 0, 1, 2,
    1, 0, 243, 1, 2, 1, 1, 4, 3, 1, 4, 5, 10, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 248, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    249, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 0,
    1, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0,
    1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 1, 2, 1, 1, 2, 1, 0, 1, 0, 0, 1, 0,
    1, 2, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0,
    0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 1, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0,
    1, 2, 1, 1, 0, 1, 2, 1, 0, 1, 0, 0, 250, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1,
    0, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 0, 1, 0,
    0, 1, 0, 0, 0, 1, 0, 1, 2, 1, 1, 2, 1, 1, 4294967295, 1,
    0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0,
    0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 4294967295, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0,
    0, 0, 1, 0, 3, 4, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1, 0,
    1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1,
    0, 0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1,
    0, 438, 1, 2, 1, 1, 4, 3, 1, 4, 5, 667, 668, 234, 1, 668,
    3, 3, 4, 235, 669, 1, 2, 1, 1, 4, 3, 1, 4, 5, 670, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 1, 2, 1, 0,
    1, 0, 1, 2, 1, 1, 0, 677, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 677, 678, 679, 1, 678, 3, 3, 678, 4, 680, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 97, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 0, 206, 697, 234, 1, 697, 3, 3, 4, 235, 698, 699, 234, 1, 699,
    3, 3, 4, 235, 1, 2, 0, 294, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 1, 2, 0, 1, 2, 0, 228, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 702, 1, 2, 1, 1, 4, 3, 1, 4, 5, 703, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 704, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 705, 1, 2, 1, 1, 4, 3, 1, 4, 5, 706, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 707, 1, 2, 1, 1, 4, 3, 1, 4, 5, 557,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 663, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 708, 1, 2, 1, 1, 4, 3, 1, 4, 5, 709, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 710, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 712, 1, 2, 1, 1, 4, 3, 1, 4, 5, 713, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 714, 1, 2, 1, 1, 4, 3, 1, 4, 5, 428,
    429, 261, 364, 430, 431, 263, 432, 433, 265, 434, 122, 831, 12, 715, 716, 1,
    715, 2, 3, 4, 0, 0, 721, 722, 0, 2, 3, 721, 3, 723, 0, 1,
    3, 665, 1, 2, 1, 1, 4, 3, 1, 4, 5, 724, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 725, 1, 2, 1, 1, 4, 3, 1, 4, 5, 726,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 727, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 728, 1, 2, 1, 1, 4, 3, 1, 4, 5, 729, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 678, 679, 9, 3, 3, 678, 4, 743,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 744, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 745, 1, 2, 1, 1, 4, 3, 1, 4, 5, 685, 678, 234,
    1, 678, 3, 3, 4, 235, 749, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 752, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 755, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 756, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    757, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1,
    0, 1, 2, 1, 758, 759, 0, 2, 3, 758, 1, 2, 1, 762, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 763, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 764, 1, 2, 1, 1, 4, 3, 1, 4, 5, 765, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 766, 1, 2, 1, 1, 4, 3, 1, 4, 5, 439,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 577, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 767, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    1, 2, 1, 0, 0, 444, 1, 2, 1, 1, 4, 3, 1, 4, 5, 774,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 519, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 334, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 776, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0,
    0, 777, 1, 2, 1, 1, 4, 3, 1, 4, 5, 365, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 791, 792, 234, 1, 792, 3, 3, 4, 235, 11, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 793, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 794, 1, 2, 1, 1, 4, 3, 1, 4, 5, 795, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 796, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    797, 1, 2, 1, 1, 4, 3, 1, 4, 5, 352, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 798, 1, 2, 1, 1, 4, 3, 1, 4, 5, 799, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 800, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 801, 1, 2, 1, 1, 4, 3, 1, 4, 5, 802, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 803, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    394, 1, 2, 1, 1, 4, 3, 1, 4, 5, 804, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 403, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 805,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 806, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 4, 716, 9, 2, 3, 4, 417, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 366, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0,
    293, 809, 234, 1, 809, 3, 3, 4, 235, 293, 810, 234, 1, 810, 3, 3,
    4, 235, 293, 811, 234, 1, 811, 3, 3, 4, 235, 0, 1, 2, 1, 0,
    0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 0, 0, 0, 4294967295, 4294967295, 0, 413,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 835, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 86, 836, 234, 1, 836, 3, 3, 4, 235,
    837, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 0,
    1, 2, 1, 0, 0, 840, 841, 10, 3, 3, 840, 4, 842, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 843, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    844, 1, 2, 1, 1, 4, 3, 1, 4, 5, 845, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 288, 861, 9, 3, 3, 288, 4, 862, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 678, 863, 0, 2, 3, 678, 288, 864, 0, 2,
    3, 288, 1, 2, 1, 865, 1, 2, 1, 1, 4, 3, 1, 4, 5, 866,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 467, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 867, 1, 2, 1, 1, 4, 3, 1, 4, 5, 651, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 383, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 0,
    1, 2, 1, 1, 2, 1, 0, 1, 0, 0, 878, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 98, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 271, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 350, 1, 2, 1, 1, 4, 3, 1, 4, 5, 881, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 882, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    883, 1, 2, 1, 1, 4, 3, 1, 4, 5, 884, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 367, 1, 2, 1, 1, 4, 3, 1, 4, 5, 885, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 0, 0,
    1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1,
    0, 4294967295, 1, 0, 1, 0, 1, 2, 1, 0, 1, 0, 0, 0, 4294967295, 1,
    0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0, 1, 0, 1, 2, 1,
    0, 0, 0, 4294967295, 1, 0, 0, 916, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 520, 1, 2, 1, 1, 4, 3, 1, 4, 5, 917, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 1, 3, 4, 1, 0, 0, 0, 0, 1,
    0, 1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    0, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1, 1,
    0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1, 0, 0,
    0, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 0, 1,
    2, 1, 99, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0,
    1, 2, 1, 0, 1, 2, 1, 0, 0, 1, 0, 1, 2, 0, 0, 1,
    0, 1, 2, 1, 0, 4294967295, 974, 234, 1, 974, 3, 3, 4, 235, 0, 4294967295,
    4294967295, 0, 0, 0, 4294967295, 974, 234, 1, 974, 3, 3, 4, 235, 0, 4294967295, 4294967295,
    0, 0, 4294967295, 985, 234, 1, 985, 3, 3, 4, 235, 0, 4294967295, 4294967295, 0, 0,
    0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 1, 0, 1, 2, 1, 1,
    2, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4294967295,
    1, 0, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1,
    2, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 1, 2,
    1, 0, 1, 2, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0,
    1, 2, 1, 1, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1,
    2, 1, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 0,
    0, 4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 0, 0,
    4294967295, 4294967295, 0, 0, 1, 2, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0,
    0, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 1, 2, 1,
    0, 1, 2, 1, 0, 0, 578, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    579, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1047, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1048, 1, 2, 1, 1, 4, 3, 1, 4, 5, 911, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1049, 1050, 716, 1, 1050, 2, 3, 4,
    1049, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 316, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 428, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 429, 1, 2, 1, 1, 4, 3, 1, 4, 5, 261, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 364, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    430, 1, 2, 1, 1, 4, 3, 1, 4, 5, 431, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 263, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1056, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1057, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 445, 1, 2, 1, 1, 4, 3, 1, 4, 5, 51, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 12, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 1058, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1059, 716,
    1, 1059, 2, 3, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 4294967295, 4294967295, 0, 0, 295, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2497, 0, 1050, 716, 1, 1050, 2, 3, 4, 0, 1, 0, 1, 2, 1,
    0, 1, 2, 1, 0, 1103, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1242,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1212, 1213, 1214, 1215, 1216, 1217, 1218,
    1219, 1220, 1221, 1222, 1223, 1224, 1225, 1226, 1227, 1228, 1229, 1230, 1231, 1232, 1233, 1234,
    1235, 2537, 24, 1243, 716, 1, 1243, 2, 3, 4, 1213, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1244, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1245, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1246, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1247, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1248, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1249, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1214, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1250, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1251, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1252, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1256, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1104, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1257, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 290, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1258, 1, 2, 1, 1, 4, 3, 1, 4, 5, 472, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 477, 1, 2, 1, 1, 4, 3, 1, 4, 5, 539,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1263,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1264, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 100, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0,
    1, 0, 0, 893, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1274, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1275, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 999, 1, 2, 1, 1, 4, 3, 1, 4, 5, 553, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 771, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1276,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1277, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1279, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1280, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 644, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 924, 1, 2, 1, 1, 4, 3, 1, 4, 5, 384, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 404, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1281,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1282, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1286, 1, 2, 1, 1, 4, 3, 1, 4, 5, 820, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1287, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1288, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1289, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 682, 678, 234, 1, 678, 3, 3, 4, 235, 874,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1292, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 305, 305, 1294, 3, 305, 1293, 305, 3, 3, 1293, 4, 13,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 205, 1295, 234, 1, 1295, 3, 3,
    4, 235, 1105, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0,
    3, 4, 1, 1, 2, 1, 1, 0, 1, 0, 1, 0, 1, 0, 341, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 0, 1, 0, 0,
    1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1296, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1297, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1035, 1, 2, 1, 1, 4, 3, 1, 4, 5, 785, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1298, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1299, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 1, 405, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1302, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 894, 1, 2, 1, 1, 4, 3, 1, 4, 5, 580, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1303, 1304, 234, 1, 1304, 3, 3, 4, 235, 787, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1039, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1305, 1, 2, 1, 1, 4, 3, 1, 4, 5, 895, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1306, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    368, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1307, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 469, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1015, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1308, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 101, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2,
    1, 1309, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1310, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1311, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1312,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 1,
    2, 1, 0, 0, 436, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2, 1, 0, 0,
    4294967295, 4294967295, 0, 0, 0, 0, 1, 0, 0, 1, 2, 1, 0, 1, 0, 1,
    2, 1, 0, 0, 0, 460, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    2, 1, 0, 0, 360, 1, 2, 1, 1, 4, 3, 1, 4, 5, 14, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1392, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 278, 1, 2, 1, 1, 4, 3, 1, 4, 5, 278, 1393, 1394, 1,
    1393, 5, 3, 1, 1293, 4, 5, 395, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 0, 1395, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1401,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 15, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1425, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1426, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 653, 1, 2, 1, 1, 4, 3, 1, 4, 5, 639, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1107, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1108, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1106, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1428, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1429, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 330, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 406, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 896, 1, 2, 1, 1, 4, 3, 1, 4, 5, 897, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 898, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    16, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 1,
    2, 1, 175, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1432, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 0, 1444, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1445, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    899, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2, 1,
    0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 1, 2, 1, 0, 224, 1481,
    1, 0, 0, 4294967295, 4294967295, 0, 241, 1481, 1, 1, 2, 1, 0, 90, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 504, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1373, 1373, 1294, 3, 1373, 1293, 1373, 3, 3, 1293, 4, 1482, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 1, 2, 1, 0, 0, 0, 4294967295, 1,
    0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1,
    0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0,
    4294967295, 4294967295, 0, 1, 0, 1, 2, 1, 17, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 3, 4294967295, 1050, 716, 1, 1050, 2, 3, 4, 0, 4294967295, 4294967295, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 66, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 18, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    421, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1061, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1062, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1063, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1064, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1065, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1066, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1516, 1516, 861, 3, 1516, 288, 1516, 3, 3, 288,
    4, 1006, 1, 2, 1, 1, 4, 3, 1, 4, 5, 187, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 1523, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 132, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1525, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 1, 0, 1, 0, 0, 193, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1, 1, 0, 1, 0, 1529, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1433, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 944, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1019, 1, 2, 1, 1, 4, 3, 1, 4, 5, 975, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 1448, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 900, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1535, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1536, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1537, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1538, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1539, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1540, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1219, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1541, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1542, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1543, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 934, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1544, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1545, 1546, 9, 3, 3, 1545, 4, 443, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1547, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1548, 1, 2, 1, 1, 4, 3, 1, 4, 5, 357, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1495, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1337, 1, 2, 1, 1, 4, 3, 1, 4, 5, 479, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 521, 1, 2, 1, 1, 4, 3, 1, 4, 5, 923, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1402, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 102, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 61, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1,
    2, 1, 0, 966, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1016, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 446, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 0, 1, 2, 1, 0, 507, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1566, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1567, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1109, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1568, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1569, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 407, 1, 2, 1, 1, 4, 3, 1, 4, 5, 951,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1570, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 474, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1571, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 869, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 920, 1, 2, 1, 1, 4, 3, 1, 4, 5, 532, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1572, 1, 2, 1, 1, 4, 3, 1, 4, 5, 103,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 0, 1581, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1025, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1111, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1583, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1112, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1113, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1114, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1115, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1110, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1116, 1, 2, 1, 1, 4, 3, 1, 4, 5, 19, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1584, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1449, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 296, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 1557, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 540, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 731, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1596, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 272, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1597, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1350, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1598, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1599, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 224, 1603, 1, 0, 241, 1603, 1, 0,
    976, 1, 2, 1, 1, 4, 3, 1, 4, 5, 779, 1611, 1, 1, 2, 1,
    0, 241, 1611, 1, 0, 0, 1, 0, 1, 2, 1, 1117, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 733, 678, 234, 3, 678, 235, 733, 3, 3, 4, 235,
    0, 0, 1118, 1, 2, 1, 1, 4, 3, 1, 4, 5, 823, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1621, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 9, 4, 3, 1, 4, 5, 1623, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1624, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1625, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 20, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 91, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1561, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1627, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1468,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 925, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 926, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1486, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 522, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1628, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1629, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 870, 1, 2, 1, 1, 4, 3, 1, 4, 5, 977,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1630, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 490, 1, 2, 1, 1, 4, 3, 1, 4, 5, 935, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1013, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 734, 734, 861, 3, 734, 288, 734, 3, 3, 288, 4, 0, 0, 0,
    1, 2, 1, 0, 0, 0, 282, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1119, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1640, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1641, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1067, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1068, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1069, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1070, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1020, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1403, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1293, 1394, 9, 5, 3,
    1, 1293, 4, 5, 1293, 1294, 9, 3, 3, 1293, 4, 1404, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1007, 1, 2, 1, 1, 4, 3, 1, 4, 5, 927,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 432, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 433, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1651, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1405, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 0, 0, 1431, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1656,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 498, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 581, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1657, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1658, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1659, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1660, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1661, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1662,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1120, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 2, 3, 1, 1122, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1121, 1, 2, 1, 1, 4, 3, 1, 4, 5, 461, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 104, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 655, 1, 2, 1, 1, 4, 3, 1, 4, 5, 21, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1664, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1665,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 22, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1666, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1667, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 447, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1000, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1668, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1669, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1670,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1406, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 4294967295, 4294967295, 0, 0, 0, 0, 0, 1, 2, 1, 0, 224,
    1603, 1, 241, 1603, 1, 978, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1697,
    1698, 234, 3, 1698, 235, 1697, 3, 3, 4, 235, 0, 4294967295, 4294967295, 0, 375, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1700, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1224, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1701, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1450, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    214, 678, 234, 1, 678, 3, 3, 4, 235, 1226, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1702, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1703, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 448, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1477, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1704, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1655, 1, 2, 1, 1, 4, 3, 1, 4, 5, 973,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 105, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 681, 678, 234, 1, 678, 3, 3, 4, 235, 1705, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1123, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    39, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1227, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1706, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1707, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1071, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 4294967295, 4294967295, 0, 308, 308, 1709, 1, 308, 3, 3, 308, 4, 1072,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1073, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1074, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1075, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 254, 678, 234, 1, 678, 3, 3, 4, 235,
    1076, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1077, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1078, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1079, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1080, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1081, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1082, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1083, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1084, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1710, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 5, 1711, 1546, 3, 1711, 1545, 5, 3, 3, 1545, 4, 4,
    5, 1712, 1546, 1, 1712, 3, 3, 1545, 4, 615, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 2, 1, 0, 1, 0, 1719, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1342, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1342, 1720,
    1394, 1, 1720, 5, 3, 1, 1293, 4, 5, 1721, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1722, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1724, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1001, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 214, 1725, 234, 1, 1725, 3, 3, 4, 235, 979, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1030, 1, 2, 1, 1, 4, 3, 1, 4, 5, 308, 1709,
    12, 3, 3, 308, 4, 1397, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1343,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1344, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1345, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1346, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 408, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1726, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1727, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1728,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1729, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1730, 1, 2, 1, 1, 4, 3, 1, 4, 5, 583, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 584, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1731, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1732, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 585, 1, 2, 1, 1, 4, 3, 1, 4, 5, 586,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 1,
    2, 1, 0, 0, 1, 2, 1, 0, 0, 1736, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 622, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1737, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 587, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1738, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1347, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 588, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 347, 1, 2, 1, 1, 4, 3, 1, 4, 5, 347, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 505, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1407,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1352, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 563, 1, 2, 1, 1, 4, 3, 1, 4, 5, 297, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 297, 1720, 1394, 1, 1720, 5, 3, 1, 1293,
    4, 5, 1747, 1, 2, 1, 1, 4, 3, 1, 4, 5, 817, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1124, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    96, 96, 1294, 3, 96, 1293, 96, 3, 3, 1293, 4, 1125, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 94, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 303, 1750, 234, 1, 1750, 3, 3, 4, 235, 1, 2, 0, 0, 0, 1751,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 409, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1752, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1753, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1451, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1754, 1, 2, 1, 1, 4, 3, 1, 4, 5, 496, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1031, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    162, 1712, 1546, 1, 1712, 3, 3, 1545, 4, 162, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1755, 1, 2, 1, 1, 4, 3, 1, 4, 5, 515, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1756, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1757, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1758, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1759, 1, 2, 1, 1, 4, 3, 1, 4, 5, 106,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1760, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 369, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1761, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 327, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1762, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 1,
    2, 1, 410, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 732,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 732, 678, 679, 1, 678, 3, 3,
    678, 4, 0, 0, 0, 1, 0, 0, 222, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1767, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1442, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1506, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1616, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1768, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 1, 0, 1770, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 3, 4, 1, 0, 0, 0, 3, 4, 1, 0, 1772,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1773, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1574, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1774, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 449, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 255, 1, 2, 1, 1, 4, 3, 1, 4, 5, 107, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1775, 1, 2, 1, 1, 4, 3, 1, 4, 5, 523,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 4294967295, 1776, 716, 1, 1776, 2,
    3, 4, 1777, 1776, 234, 1, 1776, 3, 3, 4, 235, 1778, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 385, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1779,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 2, 1, 0, 1, 0,
    1, 2, 1, 0, 560, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1783, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 589, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1126, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 1785, 1786, 2,
    1785, 5, 3, 3, 4, 5, 1, 0, 0, 1787, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 989, 1, 2, 1, 1, 4, 3, 1, 4, 5, 735, 678, 234,
    3, 678, 235, 735, 3, 3, 4, 235, 1127, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 736, 678, 234, 3, 678, 235, 736, 3, 3, 4, 235, 135, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 217, 1788, 234, 1, 1788, 3, 3, 4, 235,
    1789, 1, 2, 1, 1, 4, 3, 1, 4, 5, 462, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1284, 1, 2, 1, 1, 4, 3, 1, 4, 5, 292, 678,
    234, 1, 678, 3, 3, 4, 235, 1085, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1086, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1087, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1088, 1, 2, 1, 1, 4, 3, 1, 4, 5, 127,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1089, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1090, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1091, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1092, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1093, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1790, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 292, 1725, 234, 1, 1725, 3, 3, 4, 235, 0, 4294967295,
    4294967295, 0, 1797, 1, 2, 1, 1, 4, 3, 1, 4, 5, 737, 678, 234, 3,
    678, 235, 737, 3, 3, 4, 235, 1414, 1418, 716, 3, 1418, 4, 1414, 2, 3,
    4, 0, 1437, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1155, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1800, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1801, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1255, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 1804, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1011, 1, 2, 1, 1, 4, 3, 1, 4, 5, 422, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1805, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1358, 1, 2, 1, 1, 4, 3, 1, 4, 5, 140, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1806, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    370, 1, 2, 1, 1, 4, 3, 1, 4, 5, 23, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 67, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1807, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1808, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1809, 1, 2, 1, 1, 4, 3, 1, 4, 5, 887, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 5, 1810, 716, 3, 1810, 4, 5, 2, 3, 4,
    928, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1415, 1415, 716, 3, 1415, 4,
    1415, 2, 3, 4, 1416, 1416, 716, 3, 1416, 4, 1416, 2, 3, 4, 293, 1813,
    234, 1, 1813, 3, 3, 4, 235, 0, 1, 2, 1, 0, 204, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 570, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 0, 0, 1, 0, 0, 0, 1, 2, 1, 1818, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 7, 4294967295, 1819, 234, 1, 1819, 3, 3, 4, 235, 1,
    4294967295, 4294967295, 0, 1, 2, 1, 1452, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 2, 1, 1228, 1, 2, 1, 1, 4, 3, 1, 4, 5, 901, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 313, 1823, 1546, 3, 1823, 1545, 313, 3,
    3, 1545, 4, 314, 1823, 1546, 3, 1823, 1545, 314, 3, 3, 1545, 4, 0, 309,
    1824, 1294, 3, 1824, 1293, 309, 3, 3, 1293, 4, 307, 1824, 1294, 3, 1824, 1293,
    307, 3, 3, 1293, 4, 0, 309, 1824, 1294, 3, 1824, 1293, 309, 3, 3, 1293,
    4, 0, 4294967295, 4294967295, 0, 304, 1824, 1294, 3, 1824, 1293, 304, 3, 3, 1293, 4,
    8, 4294967295, 1824, 1294, 1, 1824, 3, 3, 1293, 4, 0, 4294967295, 4294967295, 0, 310, 1824,
    1294, 3, 1824, 1293, 310, 3, 3, 1293, 4, 9, 4294967295, 1824, 1294, 1, 1824, 3,
    3, 1293, 4, 0, 4294967295, 4294967295, 0, 10, 4294967295, 1828, 716, 1, 1828, 2, 3, 4,
    306, 1824, 1294, 3, 1824, 1293, 306, 3, 3, 1293, 4, 306, 1824, 1294, 3, 1824,
    1293, 306, 3, 3, 1293, 4, 0, 4294967295, 4294967295, 0, 0, 1, 0, 0, 1829, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1830, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1021, 1, 2, 1, 1, 4, 3, 1, 4, 5, 286, 1831, 234, 1,
    1831, 3, 3, 4, 235, 684, 1831, 234, 1, 1831, 3, 3, 4, 235, 1832, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 683, 1831, 234, 1, 1831, 3, 3, 4,
    235, 730, 1831, 234, 1, 1831, 3, 3, 4, 235, 108, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1833, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1128, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1408, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 0, 0, 1835, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1836, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1044, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1837, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295,
    1, 0, 0, 110, 1, 2, 1, 1, 4, 3, 1, 4, 5, 109, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1028, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 1, 2, 1, 0, 629, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1845, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1409, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 0, 0, 1, 0,
    1, 2, 1, 1849, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1850, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1851, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1782, 1, 2, 1, 1, 4, 3, 1, 4, 5, 256, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1852, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1853,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 657, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 411, 1, 2, 1, 1, 4, 3, 1, 4, 5, 590, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1854, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 273, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1857, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 24, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1496, 1, 2, 1, 1, 4, 3, 1, 4, 5, 426, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 332, 1, 2, 1, 1, 4, 3, 1, 4, 5, 544, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 990, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1858, 1, 2, 1, 1, 4, 3, 1, 4, 5, 961, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1271, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    524, 1, 2, 1, 1, 4, 3, 1, 4, 5, 188, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1713, 1859, 234, 1, 1859, 3, 3, 4, 235, 871, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 980, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1860, 1, 2, 1, 1, 4, 3, 1, 4, 5, 25, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1861, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1410, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1051, 1050, 716, 1, 1050, 2,
    3, 4, 1051, 1, 2, 1, 1, 4, 3, 1, 4, 5, 26, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 68, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    488, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1862, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1863, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1864, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1865, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 525, 1, 2, 1, 1, 4, 3, 1, 4, 5, 484, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 545, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 2, 1, 0, 1, 2, 1, 1869, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1870, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1618, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 27, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    59, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1871, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 382, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1229, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1872, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 492, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1,
    2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 0, 0, 0, 1, 0,
    1, 2, 1, 0, 1, 0, 2, 8875, 2, 8877, 3, 731, 820, 731, 347, 820,
    0, 111, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1,
    2, 1, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 832, 1898, 234, 1, 1898,
    3, 3, 4, 235, 1492, 1, 2, 1, 1, 4, 3, 1, 4, 5, 28, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 69, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1130, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1131, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1132, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1129, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1899, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 52, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1900, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1901, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1856, 1, 2, 1, 1, 4, 3, 1, 4, 5, 265,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 0, 4294967295, 4294967295, 0,
    1, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 1, 0, 0, 1, 2, 1,
    0, 1, 0, 0, 0, 1, 4294967295, 4294967295, 0, 257, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
    958, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1534, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1313, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1910, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 199, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1911, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295,
    0, 1411, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1916, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1917, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1338, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 1841,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 309, 1823, 1546,
    3, 1823, 1545, 309, 3, 3, 1545, 4, 630, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 674, 1, 2, 1, 1, 4, 3, 1, 4, 5, 682, 1920, 234, 1,
    1920, 3, 3, 4, 235, 293, 1921, 234, 1, 1921, 3, 3, 4, 235, 1133, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1134, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1922, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1820, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1923, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1686, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1924, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1925, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1926, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1927, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1928, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1929, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1930, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1931, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1932, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1933, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1934, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1935, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1936, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1937, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 526, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    29, 1, 2, 1, 1, 4, 3, 1, 4, 5, 70, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1938, 1, 2, 1, 1, 4, 3, 1, 4, 5, 254, 1939,
    234, 1, 1939, 3, 3, 4, 235, 450, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1940, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1941, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1036, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    455, 1, 2, 1, 1, 4, 3, 1, 4, 5, 888, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 914, 1, 2, 1, 1, 4, 3, 1, 4, 5, 459, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 815, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2170, 1, 2, 1, 1, 4, 3, 1, 4, 5, 30, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 71, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    527, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2172, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 2173, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 626, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2174, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2175, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1314, 1, 2, 1, 1, 4, 3, 1, 4, 5, 591, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 528, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    235, 234, 9, 3, 3, 4, 235, 2176, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2177, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2178, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 993, 1, 2, 1, 1, 4, 3, 1, 4, 5, 636,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 0, 1, 2,
    1, 1, 2, 1, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 1, 2,
    1, 1, 2, 1, 0, 31, 1, 2, 1, 1, 4, 3, 1, 4, 5, 72,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1473, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 32, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295,
    0, 0, 4294967295, 4294967295, 0, 1688, 1, 2, 1, 1, 4, 3, 1, 4, 5, 33,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 73, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 112, 1, 2, 1, 1, 4, 3, 1, 4, 5, 386, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 113, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 2190, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2191, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 371, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2192, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2193, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1680, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1400,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1359, 2194, 234, 1, 2194, 3, 3,
    4, 235, 298, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 4294967295,
    4294967295, 0, 299, 1, 2, 1, 1, 4, 3, 1, 4, 5, 300, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 301, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1315, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1094, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2197, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2198, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2199, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 88, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2200, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2201, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1487, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2202, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2203, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    2209, 1, 2, 1, 1, 4, 3, 1, 4, 5, 372, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2188, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2210, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1095, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 114, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2211, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 279, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 279, 1393, 1394, 1, 1393, 5, 3, 1, 1293, 4, 5, 115, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 2216, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2217, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1135, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1136, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1137, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1138, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 34, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 322, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1826,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2219, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1436, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2220, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 2225, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2226, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1453, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 423, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1348, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1348, 1720, 1394, 1, 1720,
    5, 3, 1, 1293, 4, 5, 1412, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 2229, 1, 2, 1, 1, 4, 3, 1, 4, 5, 499, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 546, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2230, 1, 2, 1, 1, 4, 3, 1, 4, 5, 608, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 116, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 0, 2231, 1, 2, 1, 1, 4, 3, 1, 4, 5, 117,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 118, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 119, 1, 2, 1, 1, 4, 3, 1, 4, 5, 122, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2232, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 399, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2233, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2234, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2235,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1139, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1413, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0,
    1, 0, 1, 2, 1, 0, 1, 0, 355, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 342, 1, 2, 1, 1, 4, 3, 1, 4, 5, 946, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 661, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 0, 1, 2, 1, 1, 0, 646, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2240, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1140, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2241, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2242, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2243, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 302, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1171, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1141, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 673, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 929, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 35, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 74, 1, 2, 1, 1, 4, 3, 1, 4, 5, 529, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2244, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1316,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1365, 2245, 234, 1, 2245, 3, 3,
    4, 235, 36, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2246, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1233, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1234, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1,
    0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2,
    1, 0, 0, 1, 2, 1, 0, 224, 2284, 1, 779, 2285, 1, 241, 2285, 1,
    241, 2284, 1, 781, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    0, 0, 876, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2286, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2287, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2288, 2289, 12, 3, 3, 4, 2288, 0, 0, 4294967295, 4294967295, 0, 962, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 224, 2295, 1, 1, 0, 241, 2295, 1,
    224, 2295, 1, 1, 0, 241, 2295, 1, 274, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 2, 1, 1792, 1, 2, 1, 1, 4, 3, 1, 4, 5, 495,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 1786, 9, 3, 3, 4, 5,
    2205, 1, 2, 1, 1, 4, 3, 1, 4, 5, 56, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2301, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2302, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1142, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2303, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2304, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2305, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2306, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2307, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2308, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2309, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2310, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2311, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2312, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2313, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2314, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1323, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1235, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2315, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2316, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2317, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1909, 1, 2, 1, 1, 4, 3, 1, 4, 5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 36, 8, 0, 32, 0, 37, 38, 40, 41, 42, 43, 44, 45,
    46, 47, 48, 49, 0, 50, 30, 53, 0, 54, 55, 0, 57, 58, 60, 41,
    0, 62, 58, 60, 41, 63, 0, 57, 65, 60, 45, 0, 62, 65, 60, 31,
    41, 45, 34, 33, 76, 0, 78, 80, 83, 85, 87, 89, 0, 92, 93, 95,
    42, 0, 121, 37, 47, 123, 40, 41, 43, 0, 124, 125, 126, 0, 128, 129,
    0, 130, 131, 0, 38, 38, 133, 134, 0, 41, 0, 121, 39, 45, 0, 125,
    126, 136, 63, 0, 137, 40, 45, 46, 0, 138, 139, 124, 141, 121, 142, 128,
    143, 144, 38, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 134, 156,
    157, 0, 57, 139, 158, 43, 159, 161, 45, 163, 164, 165, 166, 167, 168, 169,
    170, 129, 45, 0, 125, 126, 0, 171, 164, 0, 172, 163, 0, 173, 174, 46,
    176, 177, 178, 179, 180, 181, 182, 183, 184, 58, 0, 125, 185, 0, 171, 177,
    0, 172, 176, 0, 173, 186, 47, 189, 191, 60, 0, 194, 48, 76, 0, 123,
    50, 195, 0, 47, 52, 197, 0, 200, 54, 76, 56, 197, 58, 201, 203, 61,
    207, 209, 65, 75, 85, 95, 0, 215, 216, 215, 105, 115, 125, 221, 0, 224,
    227, 161, 0, 230, 153, 231, 135, 144, 0, 230, 158, 156, 0, 238, 239, 241,
    162, 0, 244, 246, 172, 182, 192, 0, 251, 252, 76, 203, 202, 80, 0, 253,
    207, 0, 258, 87, 78, 0, 259, 87, 206, 260, 262, 264, 197, 0, 266, 45,
    267, 0, 251, 41, 269, 217, 0, 275, 208, 0, 276, 277, 0, 280, 281, 95,
    0, 283, 277, 0, 258, 87, 269, 0, 259, 87, 212, 76, 260, 262, 264, 0,
    266, 48, 284, 0, 266, 238, 215, 0, 213, 87, 287, 214, 289, 291, 311, 312,
    0, 89, 315, 0, 317, 216, 57, 238, 0, 318, 319, 161, 321, 87, 218, 324,
    275, 0, 275, 318, 326, 223, 0, 57, 139, 221, 220, 121, 142, 128, 143, 149,
    150, 153, 154, 155, 134, 161, 0, 328, 0, 318, 326, 329, 319, 161, 321, 89,
    224, 324, 331, 333, 335, 226, 176, 179, 181, 182, 336, 58, 0, 251, 289, 139,
    0, 329, 227, 47, 123, 228, 76, 60, 318, 328, 0, 60, 45, 230, 267, 318,
    328, 232, 337, 339, 233, 0, 194, 197, 203, 0, 251, 42, 76, 238, 0, 266,
    289, 343, 0, 345, 289, 0, 234, 266, 345, 346, 348, 0, 349, 0, 351, 76,
    235, 0, 346, 348, 0, 353, 343, 0, 346, 354, 0, 345, 351, 356, 76, 236,
    331, 333, 0, 258, 351, 76, 87, 76, 0, 358, 351, 76, 0, 259, 87, 237,
    76, 260, 262, 264, 0, 258, 87, 76, 0, 266, 45, 76, 0, 251, 359, 361,
    262, 0, 155, 0, 87, 291, 0, 266, 45, 239, 267, 363, 0, 240, 349, 345,
    261, 0, 242, 0, 373, 374, 376, 377, 42, 0, 378, 153, 0, 50, 54, 55,
    0, 50, 163, 0, 379, 44, 45, 169, 53, 0, 380, 381, 241, 387, 388, 389,
    390, 49, 76, 0, 243, 0, 171, 164, 163, 164, 244, 0, 252, 76, 244, 0,
    0, 0, 391, 247, 0, 76, 392, 76, 0, 250, 166, 167, 168, 251, 0, 78,
    80, 0, 393, 260, 0, 76, 256, 254, 253, 239, 85, 239, 0, 259, 396, 397,
    76, 398, 400, 0, 402, 329, 49, 238, 300, 0, 349, 412, 414, 415, 239, 0,
    418, 419, 263, 424, 0, 345, 412, 356, 415, 335, 0, 251, 412, 415, 282, 0,
    0, 266, 54, 49, 239, 0, 418, 419, 0, 398, 267, 343, 0, 266, 289, 269,
    268, 291, 201, 427, 343, 0, 345, 289, 0, 270, 266, 345, 346, 348, 0, 345,
    435, 356, 0, 349, 437, 271, 440, 0, 441, 289, 392, 442, 276, 272, 451, 0,
    280, 0, 266, 437, 279, 440, 0, 441, 289, 0, 266, 451, 0, 453, 0, 275,
    95, 0, 345, 442, 356, 0, 266, 454, 95, 0, 266, 456, 0, 317, 456, 0,
    266, 458, 281, 463, 172, 464, 465, 0, 266, 466, 238, 0, 349, 468, 0, 349,
    471, 0, 251, 351, 238, 283, 473, 0, 346, 473, 284, 0, 475, 476, 0, 475,
    478, 0, 358, 351, 238, 0, 345, 351, 356, 238, 335, 0, 285, 480, 481, 153,
    286, 238, 57, 123, 0, 480, 287, 482, 440, 153, 238, 0, 288, 480, 481, 0,
    480, 482, 0, 480, 483, 483, 238, 0, 289, 481, 480, 485, 0, 486, 485, 487,
    489, 491, 0, 493, 161, 238, 0, 266, 494, 290, 493, 497, 0, 266, 48, 284,
    0, 266, 291, 500, 501, 0, 266, 456, 0, 317, 456, 0, 486, 502, 244, 0,
    503, 238, 0, 346, 503, 0, 259, 87, 292, 238, 260, 262, 264, 0, 482, 442,
    293, 289, 0, 398, 506, 508, 510, 511, 0, 0, 251, 413, 415, 512, 297, 295,
    0, 294, 349, 266, 345, 230, 513, 0, 251, 50, 163, 0, 356, 514, 244, 253,
    303, 301, 516, 0, 258, 412, 415, 87, 415, 0, 258, 87, 244, 0, 312, 304,
    0, 530, 161, 348, 531, 0, 533, 348, 161, 348, 531, 534, 305, 535, 536, 537,
    0, 346, 538, 0, 172, 491, 306, 541, 542, 0, 543, 307, 547, 548, 549, 0,
    550, 95, 0, 551, 95, 0, 552, 238, 0, 266, 238, 310, 0, 308, 87, 287,
    309, 291, 311, 289, 0, 89, 315, 0, 317, 311, 57, 238, 331, 0, 251, 166,
    313, 0, 78, 80, 317, 554, 0, 258, 87, 78, 0, 266, 45, 267, 0, 259,
    87, 316, 315, 238, 312, 260, 262, 264, 0, 486, 556, 161, 359, 361, 0, 251,
    506, 331, 0, 356, 508, 328, 0, 258, 87, 508, 0, 266, 48, 284, 0, 558,
    559, 508, 0, 486, 556, 161, 359, 361, 0, 266, 320, 318, 0, 238, 0, 287,
    291, 0, 317, 325, 323, 238, 0, 251, 412, 95, 266, 454, 95, 0, 57, 60,
    48, 284, 0, 561, 441, 329, 197, 266, 48, 284, 562, 332, 365, 0, 251, 412,
    415, 353, 0, 266, 454, 95, 0, 266, 344, 342, 0, 564, 287, 291, 0, 317,
    349, 347, 564, 0, 266, 458, 352, 463, 172, 464, 465, 0, 266, 466, 238, 0,
    493, 161, 238, 0, 266, 494, 493, 0, 266, 356, 354, 0, 564, 0, 287, 291,
    0, 317, 361, 359, 564, 0, 259, 87, 364, 76, 260, 262, 264, 0, 251, 171,
    164, 371, 0, 356, 244, 368, 366, 565, 0, 258, 0, 412, 415, 87, 415, 0,
    258, 87, 244, 0, 266, 45, 267, 0, 244, 486, 556, 161, 359, 361, 0, 57,
    60, 48, 284, 0, 561, 441, 369, 76, 266, 48, 284, 0, 562, 0, 251, 126,
    76, 161, 244, 385, 0, 258, 87, 76, 381, 0, 87, 373, 372, 197, 203, 260,
    262, 264, 0, 398, 375, 343, 0, 275, 353, 378, 343, 0, 391, 0, 172, 491,
    386, 541, 542, 0, 266, 238, 389, 0, 387, 87, 287, 388, 291, 311, 289, 0,
    89, 315, 0, 317, 390, 57, 238, 331, 0, 251, 167, 392, 0, 78, 80, 401,
    567, 0, 258, 87, 78, 0, 266, 45, 267, 0, 266, 238, 398, 0, 394, 87,
    287, 396, 395, 238, 291, 0, 89, 315, 0, 259, 87, 400, 399, 238, 312, 260,
    262, 264, 0, 486, 556, 161, 359, 361, 0, 251, 569, 571, 409, 0, 266, 573,
    0, 349, 402, 575, 0, 266, 404, 575, 0, 345, 406, 575, 0, 258, 87, 571,
    0, 259, 87, 408, 260, 262, 264, 197, 0, 251, 410, 47, 123, 433, 0, 411,
    260, 262, 264, 197, 275, 420, 412, 592, 593, 413, 594, 595, 414, 596, 597, 415,
    502, 598, 416, 599, 600, 417, 601, 602, 418, 603, 604, 0, 280, 281, 95, 0,
    605, 606, 419, 513, 348, 0, 354, 607, 513, 0, 197, 0, 258, 87, 197, 0,
    424, 260, 262, 264, 57, 197, 0, 60, 41, 269, 432, 0, 266, 238, 430, 0,
    425, 87, 287, 429, 289, 427, 426, 291, 311, 609, 0, 89, 315, 0, 317, 431,
    238, 57, 0, 251, 45, 267, 434, 0, 258, 87, 267, 0, 259, 87, 197, 438,
    0, 392, 239, 0, 610, 441, 611, 95, 435, 0, 612, 611, 95, 0, 486, 612,
    436, 0, 613, 611, 95, 0, 486, 613, 0, 169, 614, 616, 0, 618, 275, 611,
    95, 0, 453, 275, 611, 95, 0, 619, 95, 0, 486, 620, 0, 561, 441, 437,
    621, 415, 0, 251, 169, 356, 616, 444, 439, 451, 0, 259, 87, 443, 238, 262,
    264, 0, 258, 87, 616, 0, 266, 45, 267, 0, 251, 454, 356, 623, 448, 0,
    259, 87, 446, 445, 238, 312, 260, 262, 264, 0, 258, 87, 623, 0, 266, 447,
    0, 45, 267, 0, 454, 95, 0, 251, 625, 627, 456, 0, 280, 513, 0, 449,
    266, 349, 345, 569, 450, 571, 0, 275, 573, 0, 631, 569, 0, 275, 573, 480,
    481, 0, 266, 573, 0, 632, 452, 238, 633, 343, 0, 259, 87, 455, 76, 260,
    262, 634, 0, 258, 87, 627, 0, 251, 635, 463, 0, 266, 238, 457, 87, 287,
    461, 289, 459, 458, 291, 311, 312, 609, 0, 317, 462, 57, 238, 495, 0, 349,
    412, 414, 415, 239, 0, 418, 419, 464, 424, 0, 345, 412, 356, 415, 335, 0,
    251, 412, 415, 483, 0, 0, 266, 54, 49, 239, 0, 418, 419, 0, 398, 468,
    343, 0, 266, 289, 470, 469, 291, 201, 427, 343, 0, 345, 289, 0, 471, 266,
    345, 346, 348, 0, 345, 435, 356, 0, 349, 437, 472, 440, 0, 441, 289, 392,
    442, 477, 473, 451, 0, 481, 0, 266, 437, 480, 440, 0, 441, 289, 0, 266,
    451, 0, 453, 0, 275, 95, 0, 345, 442, 356, 0, 266, 454, 95, 0, 266,
    456, 0, 317, 456, 0, 266, 458, 482, 463, 172, 464, 465, 0, 266, 466, 238,
    0, 349, 468, 0, 349, 471, 0, 251, 351, 238, 484, 473, 0, 346, 473, 485,
    0, 475, 476, 0, 475, 478, 0, 358, 351, 238, 0, 345, 351, 356, 238, 335,
    0, 486, 480, 481, 153, 487, 238, 57, 123, 0, 480, 488, 482, 440, 153, 238,
    0, 489, 480, 481, 0, 480, 482, 0, 480, 483, 483, 238, 0, 490, 481, 480,
    485, 0, 486, 485, 487, 489, 491, 0, 493, 161, 238, 0, 266, 494, 491, 493,
    497, 0, 266, 48, 284, 0, 266, 492, 500, 501, 0, 266, 456, 0, 317, 456,
    0, 486, 502, 244, 0, 503, 238, 0, 346, 503, 0, 259, 87, 493, 238, 260,
    262, 264, 0, 482, 442, 494, 289, 0, 398, 506, 508, 510, 511, 0, 345, 412,
    356, 230, 501, 0, 238, 287, 496, 291, 311, 0, 497, 349, 144, 412, 637, 499,
    0, 498, 640, 641, 415, 158, 642, 0, 258, 500, 392, 87, 244, 0, 251, 163,
    509, 0, 356, 514, 244, 253, 504, 502, 643, 0, 258, 412, 415, 87, 415, 0,
    258, 351, 238, 87, 238, 0, 356, 244, 506, 0, 258, 87, 244, 0, 266, 45,
    267, 0, 558, 559, 238, 505, 0, 329, 645, 647, 289, 0, 649, 559, 238, 650,
    652, 0, 57, 60, 48, 284, 0, 561, 441, 507, 76, 266, 48, 284, 562, 0,
    251, 380, 381, 387, 76, 523, 0, 349, 654, 329, 510, 76, 275, 512, 76, 0,
    251, 654, 329, 514, 76, 275, 516, 76, 0, 251, 654, 656, 76, 275, 76, 0,
    251, 654, 329, 518, 76, 656, 76, 275, 76, 0, 345, 654, 356, 329, 520, 76,
    0, 258, 87, 76, 0, 259, 87, 522, 76, 260, 262, 264, 0, 266, 45, 76,
    0, 251, 153, 658, 161, 244, 524, 0, 258, 87, 658, 0, 486, 556, 161, 359,
    361, 0, 251, 49, 239, 530, 0, 259, 87, 525, 262, 264, 260, 76, 0, 258,
    660, 513, 87, 513, 0, 258, 87, 76, 0, 266, 45, 267, 527, 526, 0, 349,
    662, 415, 239, 0, 418, 419, 664, 0, 251, 662, 415, 0, 266, 54, 49, 239,
    0, 418, 419, 664, 0, 345, 662, 356, 415, 664, 0, 258, 662, 415, 87, 415,
    664, 0, 349, 660, 414, 513, 0, 529, 666, 641, 513, 0, 251, 164, 356, 244,
    544, 0, 251, 412, 415, 532, 0, 266, 289, 531, 291, 201, 427, 343, 0, 345,
    289, 0, 259, 87, 533, 76, 260, 262, 264, 0, 258, 412, 415, 87, 415, 0,
    258, 87, 244, 0, 266, 45, 267, 0, 266, 536, 534, 0, 564, 0, 287, 291,
    0, 317, 541, 539, 564, 545, 555, 564, 574, 0, 584, 671, 672, 591, 588, 586,
    0, 585, 675, 676, 277, 675, 597, 0, 244, 594, 592, 415, 599, 609, 618, 628,
    638, 686, 215, 687, 688, 689, 690, 691, 692, 693, 694, 660, 641, 695, 95, 343,
    0, 639, 695, 95, 343, 696, 640, 695, 95, 343, 642, 651, 0, 663, 675, 673,
    700, 701, 678, 343, 0, 227, 230, 0, 392, 681, 691, 701, 711, 721, 731, 741,
    751, 761, 771, 781, 0, 711, 791, 801, 811, 821, 843, 853, 852, 291, 427, 717,
    201, 415, 343, 0, 239, 291, 85, 392, 719, 854, 860, 865, 875, 885, 895, 905,
    915, 925, 935, 738, 739, 740, 741, 742, 936, 943, 953, 963, 746, 0, 747, 747,
    216, 0, 748, 748, 973, 982, 992, 750, 751, 993, 1003, 753, 754, 1004, 1014, 1024,
    1036, 1034, 291, 1041, 1039, 415, 1044, 1050, 760, 1053, 1063, 1073, 1083, 1093, 1103, 1113,
    1123, 1133, 231, 0, 230, 768, 392, 0, 346, 171, 1136, 770, 1140, 0, 381, 1139,
    772, 773, 640, 441, 415, 266, 415, 0, 620, 415, 266, 415, 398, 415, 1141, 1151,
    0, 775, 201, 1161, 1171, 1181, 333, 331, 1182, 1200, 0, 778, 779, 1192, 782, 784,
    786, 788, 786, 786, 0, 778, 343, 1196, 782, 784, 786, 788, 786, 786, 741, 85,
    790, 1201, 1211, 1221, 1230, 1240, 1250, 0, 1260, 1270, 1280, 1290, 1300, 1310, 1320, 1330,
    1340, 1350, 1360, 1370, 1380, 0, 493, 675, 1390, 0, 244, 0, 398, 508, 0, 508,
    161, 244, 1391, 1401, 1411, 0, 1417, 1427, 1439, 513, 1437, 230, 807, 808, 1440, 1449,
    1458, 1467, 812, 291, 201, 427, 0, 0, 351, 76, 1480, 0, 346, 348, 0, 353,
    1468, 343, 0, 486, 502, 0, 289, 1471, 812, 291, 201, 427, 343, 0, 437, 440,
    392, 343, 814, 0, 437, 1472, 440, 0, 441, 289, 392, 442, 1477, 1473, 451, 0,
    816, 0, 818, 346, 819, 0, 275, 573, 0, 398, 506, 48, 284, 0, 821, 822,
    0, 275, 573, 0, 398, 506, 48, 284, 824, 1481, 473, 0, 346, 473, 1482, 0,
    475, 476, 0, 475, 478, 0, 230, 239, 1483, 424, 0, 418, 419, 0, 1487, 0,
    230, 0, 1497, 825, 0, 825, 825, 1499, 826, 1498, 825, 0, 825, 825, 161, 392,
    398, 85, 827, 828, 829, 790, 830, 831, 833, 0, 415, 689, 49, 0, 834, 0,
    0, 1501, 1511, 0, 838, 513, 1520, 0, 838, 161, 0, 1539, 0, 1530, 163, 164,
    244, 0, 391, 1531, 0, 76, 392, 76, 0, 412, 415, 0, 351, 76, 0, 161,
    42, 76, 0, 41, 269, 0, 359, 361, 0, 166, 78, 0, 80, 0, 506, 508,
    0, 45, 267, 0, 1534, 376, 377, 42, 44, 126, 569, 47, 483, 169, 53, 454,
    625, 48, 153, 49, 0, 373, 374, 0, 378, 153, 0, 50, 54, 55, 0, 50,
    163, 0, 171, 164, 0, 380, 381, 387, 0, 380, 381, 388, 0, 380, 381, 389,
    0, 380, 381, 390, 76, 0, 161, 76, 0, 1535, 252, 167, 168, 76, 1536, 0,
    0, 0, 839, 1540, 513, 348, 1541, 1548, 1558, 1568, 1578, 1588, 287, 846, 847, 848,
    849, 850, 851, 0, 839, 819, 89, 0, 839, 346, 819, 89, 852, 853, 854, 855,
    856, 857, 858, 859, 1589, 1596, 1606, 1612, 0, 1618, 343, 825, 253, 1621, 1631, 1641,
    1651, 0, 216, 216, 1661, 0, 1671, 868, 872, 873, 1672, 0, 875, 1705, 511, 0,
    155, 266, 1703, 1702, 0, 415, 287, 1682, 343, 289, 0, 1685, 1683, 415, 287, 487,
    1691, 1689, 1688, 343, 289, 0, 1696, 1694, 415, 287, 1699, 770, 0, 877, 343, 1706,
    1725, 0, 1722, 1720, 0, 1719, 415, 1716, 343, 201, 0, 418, 419, 879, 0, 877,
    343, 0, 161, 351, 238, 1726, 1736, 1746, 1756, 1766, 1776, 1786, 1796, 1806, 0, 886,
    1878, 0, 0, 244, 1818, 1816, 415, 89, 1823, 1822, 513, 0, 889, 1821, 513, 890,
    890, 0, 892, 0, 275, 1846, 1844, 1840, 0, 174, 230, 0, 902, 277, 0, 903,
    513, 0, 348, 513, 0, 904, 277, 0, 905, 513, 0, 906, 513, 0, 907, 1829,
    1826, 1824, 415, 253, 0, 908, 1832, 1830, 415, 0, 909, 1837, 1835, 415, 0, 910,
    513, 0, 877, 343, 0, 89, 1849, 513, 890, 0, 892, 0, 275, 1853, 0, 912,
    0, 903, 392, 513, 0, 348, 392, 513, 0, 913, 1852, 904, 0, 905, 392, 513,
    0, 906, 392, 513, 0, 485, 346, 348, 1850, 415, 0, 1860, 1857, 770, 87, 1861,
    0, 915, 0, 1865, 1862, 770, 87, 1866, 513, 890, 0, 915, 0, 275, 1874, 0,
    912, 0, 903, 392, 513, 0, 348, 392, 513, 0, 913, 1873, 904, 0, 905, 392,
    513, 0, 906, 392, 513, 0, 485, 905, 1872, 1869, 1867, 415, 253, 1879, 1889, 1899,
    0, 121, 918, 252, 76, 1909, 0, 80, 0, 121, 391, 1912, 239, 392, 239, 1915,
    0, 275, 166, 78, 80, 0, 494, 166, 0, 275, 919, 1916, 0, 392, 921, 0,
    392, 922, 0, 121, 376, 414, 76, 1924, 1921, 1919, 1918, 0, 930, 287, 513, 0,
    931, 287, 513, 0, 932, 287, 513, 0, 933, 287, 1917, 936, 937, 0, 938, 287,
    277, 0, 939, 287, 513, 0, 89, 76, 0, 121, 41, 269, 275, 1932, 0, 259,
    287, 76, 0, 390, 287, 76, 0, 910, 287, 1925, 513, 289, 1930, 0, 930, 287,
    513, 1926, 0, 931, 287, 513, 0, 932, 287, 513, 0, 48, 287, 1931, 284, 289,
    0, 276, 287, 277, 0, 280, 281, 287, 95, 0, 283, 287, 277, 0, 121, 42,
    76, 0, 392, 239, 0, 418, 419, 0, 289, 343, 1937, 0, 0, 351, 76, 1936,
    0, 346, 348, 0, 353, 343, 0, 121, 359, 414, 361, 275, 0, 45, 267, 0,
    939, 940, 0, 89, 940, 333, 0, 121, 942, 230, 943, 230, 945, 230, 0, 121,
    50, 163, 1967, 0, 947, 414, 244, 1948, 1946, 1945, 0, 415, 239, 948, 0, 418,
    419, 1941, 949, 950, 0, 952, 1953, 1951, 244, 0, 53, 953, 948, 0, 947, 414,
    244, 0, 559, 503, 244, 1963, 1961, 1960, 0, 415, 0, 275, 512, 1956, 949, 950,
    1966, 0, 329, 645, 647, 289, 0, 53, 953, 948, 0, 121, 918, 955, 166, 414,
    78, 80, 0, 533, 1975, 0, 163, 1972, 1970, 1969, 239, 0, 415, 1968, 239, 956,
    0, 959, 239, 239, 960, 0, 121, 816, 506, 650, 0, 414, 508, 161, 514, 244,
    0, 398, 400, 1978, 1976, 964, 0, 967, 1983, 1981, 964, 0, 818, 346, 819, 0,
    275, 456, 0, 48, 284, 0, 877, 343, 1986, 0, 121, 918, 171, 164, 414, 244,
    158, 0, 398, 238, 0, 275, 456, 0, 48, 284, 392, 2008, 1999, 1996, 770, 2003,
    2000, 0, 163, 244, 968, 2007, 2004, 0, 128, 201, 970, 0, 121, 918, 972, 414,
    76, 0, 512, 2017, 2015, 0, 238, 287, 2014, 291, 2011, 2009, 513, 392, 770, 0,
    121, 393, 2030, 2020, 2068, 2066, 2065, 0, 2034, 981, 982, 287, 76, 0, 983, 287,
    2045, 2035, 0, 984, 287, 2059, 2049, 0, 2064, 331, 986, 2063, 167, 166, 287, 78,
    987, 988, 0, 121, 126, 76, 161, 244, 0, 392, 2071, 991, 992, 0, 329, 2072,
    57, 150, 143, 155, 142, 0, 87, 2074, 2073, 76, 203, 260, 262, 264, 0, 398,
    2076, 343, 0, 275, 353, 2079, 343, 0, 121, 918, 167, 78, 80, 960, 0, 121,
    569, 571, 2084, 0, 329, 57, 176, 0, 329, 2082, 575, 0, 275, 573, 0, 121,
    2085, 47, 123, 197, 0, 275, 2094, 2086, 592, 593, 2087, 594, 595, 2088, 596, 597,
    2089, 502, 598, 2090, 599, 600, 2091, 601, 602, 2092, 603, 604, 0, 280, 281, 95,
    0, 605, 606, 2093, 513, 348, 0, 354, 607, 513, 0, 60, 47, 197, 0, 60,
    194, 197, 0, 47, 197, 0, 994, 197, 0, 123, 197, 0, 995, 95, 0, 121,
    45, 414, 2098, 0, 267, 996, 197, 267, 2103, 0, 392, 239, 0, 610, 441, 611,
    95, 2099, 0, 612, 611, 95, 0, 486, 612, 2100, 0, 613, 611, 95, 0, 486,
    613, 0, 618, 275, 611, 95, 0, 619, 95, 2101, 620, 0, 486, 620, 0, 561,
    441, 2102, 621, 415, 0, 121, 955, 169, 414, 616, 2104, 997, 0, 121, 53, 414,
    953, 0, 49, 513, 0, 939, 940, 942, 76, 948, 0, 121, 454, 0, 414, 623,
    2111, 2109, 2108, 1002, 1003, 1004, 1005, 161, 2115, 2114, 415, 343, 89, 244, 0, 121,
    625, 627, 280, 513, 569, 2117, 571, 0, 275, 573, 0, 121, 2120, 0, 2119, 1008,
    1009, 955, 501, 163, 414, 244, 2134, 2123, 2121, 415, 0, 398, 238, 2132, 0, 275,
    2129, 2127, 0, 238, 0, 287, 2126, 291, 311, 0, 494, 497, 0, 161, 1010, 2133,
    0, 1012, 551, 0, 142, 551, 345, 0, 48, 284, 392, 2150, 2141, 2138, 770, 2145,
    2142, 0, 163, 244, 968, 2149, 2146, 0, 128, 201, 970, 0, 121, 2152, 0, 2151,
    1008, 1009, 955, 501, 163, 414, 244, 2189, 0, 2160, 2158, 2157, 637, 468, 0, 1014,
    244, 2153, 1017, 0, 952, 2165, 2163, 244, 0, 503, 238, 2175, 2173, 2172, 0, 415,
    0, 275, 512, 2168, 424, 468, 0, 559, 503, 244, 2185, 2183, 2182, 0, 415, 0,
    275, 512, 2178, 424, 468, 2188, 0, 329, 645, 647, 289, 2207, 0, 559, 441, 2190,
    1022, 1023, 1024, 2202, 2198, 2196, 0, 2191, 415, 201, 2192, 0, 418, 419, 238, 0,
    398, 238, 2205, 0, 275, 456, 0, 494, 497, 0, 161, 1010, 2206, 0, 1012, 551,
    0, 142, 551, 345, 0, 48, 284, 0, 121, 380, 381, 387, 76, 2212, 2211, 0,
    389, 287, 76, 0, 886, 287, 76, 0, 121, 918, 351, 153, 658, 2215, 666, 641,
    0, 1026, 503, 2219, 2218, 143, 142, 154, 0, 155, 0, 503, 2216, 415, 1027, 161,
    1027, 161, 244, 2229, 0, 89, 244, 2222, 0, 346, 473, 0, 473, 2221, 0, 475,
    478, 0, 475, 476, 0, 1029, 2223, 1032, 1033, 163, 392, 244, 0, 2224, 1032, 1033,
    163, 392, 244, 0, 329, 1034, 2225, 487, 1037, 0, 1038, 2226, 343, 0, 128, 2233,
    166, 167, 201, 0, 121, 49, 239, 0, 392, 2234, 1040, 1022, 2237, 2235, 0, 0,
    121, 123, 654, 0, 1041, 414, 329, 2240, 203, 230, 1042, 53, 953, 948, 0, 121,
    123, 197, 0, 121, 918, 955, 1043, 164, 244, 158, 0, 275, 456, 392, 2245, 2244,
    2241, 770, 968, 1045, 2246, 2256, 2266, 2276, 2286, 2296, 2304, 0, 2314, 1052, 1053, 2315,
    1054, 2318, 2328, 2338, 2348, 2358, 2368, 2378, 2388, 2398, 2408, 2418, 2428, 2438, 2448, 230,
    807, 808, 2450, 2469, 2460, 230, 311, 0, 0, 230, 825, 2481, 1060, 1097, 1099, 1101,
    0, 2480, 1143, 1144, 1145, 1146, 1147, 1148, 1149, 1150, 1151, 1152, 0, 1153, 1154, 1156,
    1157, 1158, 1159, 1160, 1161, 0, 2470, 1162, 1163, 0, 2471, 1165, 1166, 1163, 1167, 2473,
    0, 2472, 1168, 1169, 0, 1169, 1170, 1172, 0, 1172, 1170, 1173, 1163, 380, 1174, 2474,
    1175, 1176, 2475, 1177, 1178, 1179, 1180, 1181, 1182, 1183, 2476, 1184, 1185, 1186, 1187, 2477,
    1188, 1189, 0, 1190, 2478, 1170, 1163, 1191, 1192, 2479, 1193, 1194, 1195, 1196, 1197, 1198,
    1199, 1200, 1201, 0, 1202, 1163, 0, 1203, 1163, 2486, 2482, 1204, 1206, 1208, 2487, 2497,
    1210, 0, 2509, 2507, 2506, 1211, 1236, 0, 1097, 513, 2516, 1237, 1238, 0, 2512, 1239,
    1240, 1241, 2513, 95, 2517, 2527, 2561, 2570, 2580, 2590, 2600, 2610, 2620, 2630, 2640, 2650,
    2660, 2670, 0, 1253, 1254, 2680, 76, 57, 2681, 2691, 2701, 2711, 0, 289, 645, 2721,
    2731, 2741, 2751, 2761, 291, 1259, 0, 1259, 0, 287, 1260, 2764, 2762, 1261, 2767, 2777,
    2787, 0, 142, 89, 514, 244, 253, 246, 0, 398, 2797, 0, 1265, 246, 1267, 2799,
    0, 877, 315, 503, 76, 1269, 0, 1272, 2802, 253, 2800, 0, 343, 246, 1273, 2803,
    2813, 2823, 2833, 2843, 2853, 2863, 2873, 0, 1278, 311, 76, 2883, 2893, 2903, 2913, 2923,
    2933, 2943, 2953, 0, 1283, 2963, 57, 1285, 182, 152, 151, 2964, 2974, 2984, 2994, 3004,
    3014, 3023, 0, 875, 3033, 0, 1290, 513, 0, 513, 1290, 3034, 3044, 3055, 3065, 3074,
    0, 345, 252, 356, 3085, 0, 76, 3084, 80, 0, 253, 335, 3087, 331, 333, 0,
    345, 391, 356, 3088, 239, 392, 239, 335, 0, 345, 376, 356, 76, 335, 0, 345,
    41, 356, 269, 0, 275, 3091, 485, 0, 345, 42, 356, 3094, 76, 335, 0, 345,
    359, 356, 361, 335, 0, 345, 50, 163, 356, 3096, 244, 664, 0, 345, 166, 356,
    3098, 0, 76, 80, 335, 0, 345, 506, 650, 356, 3100, 508, 335, 3102, 0, 345,
    171, 164, 356, 3112, 244, 335, 0, 345, 972, 356, 76, 0, 345, 561, 441, 3115,
    3114, 260, 262, 264, 197, 335, 0, 345, 126, 356, 76, 161, 244, 335, 0, 345,
    167, 356, 3117, 0, 78, 80, 3119, 333, 331, 0, 345, 569, 356, 3120, 571, 335,
    0, 345, 47, 356, 230, 0, 345, 45, 356, 267, 335, 0, 345, 169, 356, 3122,
    616, 335, 0, 345, 454, 356, 3124, 623, 3126, 333, 331, 0, 345, 625, 356, 627,
    3127, 333, 331, 0, 345, 955, 163, 356, 3128, 244, 335, 0, 345, 380, 381, 387,
    356, 76, 335, 0, 345, 153, 356, 658, 161, 244, 335, 0, 345, 49, 356, 3130,
    239, 335, 0, 345, 123, 356, 197, 0, 345, 164, 356, 3132, 244, 335, 3134, 3144,
    3154, 0, 784, 343, 3164, 3174, 3184, 3194, 0, 1099, 1300, 3197, 3207, 3217, 3227, 3237,
    3246, 3256, 633, 3266, 3276, 3286, 3296, 3306, 3316, 3326, 0, 964, 275, 741, 3336, 3346,
    0, 128, 76, 3358, 3356, 343, 3361, 3371, 3381, 3391, 3403, 0, 3401, 671, 672, 675,
    1317, 1318, 1319, 1320, 1321, 1322, 277, 0, 174, 3402, 380, 1192, 1193, 1324, 0, 1325,
    3410, 0, 3404, 671, 672, 675, 3407, 3405, 1326, 3411, 770, 1328, 1330, 1332, 3412, 1334,
    0, 1335, 3423, 3422, 1336, 0, 790, 1335, 1339, 1340, 0, 839, 346, 1341, 1349, 1351,
    1353, 0, 346, 1354, 1355, 1356, 1335, 0, 1357, 1334, 3427, 611, 0, 695, 1360, 1361,
    1362, 0, 1363, 3430, 3429, 0, 3428, 738, 739, 741, 1363, 3434, 611, 0, 695, 1360,
    0, 1364, 3444, 3443, 0, 1366, 3435, 770, 0, 3438, 1367, 1368, 3439, 1370, 812, 85,
    0, 1372, 812, 0, 3459, 427, 201, 3449, 3448, 1374, 343, 770, 3446, 3445, 291, 415,
    201, 1375, 343, 1377, 1378, 0, 3452, 1099, 1101, 3455, 3453, 343, 0, 239, 3458, 513,
    95, 277, 1379, 1380, 1375, 1382, 85, 1383, 3460, 1384, 291, 717, 1386, 1388, 415, 1390,
    3461, 0, 3475, 244, 0, 514, 3474, 3471, 244, 0, 244, 253, 3476, 3486, 3496, 3506,
    3516, 3527, 0, 1396, 3537, 640, 1398, 3538, 95, 343, 487, 3539, 487, 551, 3540, 514,
    0, 275, 1399, 3541, 3551, 3561, 3577, 1417, 1419, 1421, 3575, 1423, 3571, 831, 3581, 0,
    1427, 3591, 0, 877, 343, 3594, 3604, 3614, 3624, 3634, 3644, 3654, 3664, 0, 329, 3674,
    155, 0, 486, 822, 155, 200, 0, 822, 200, 0, 503, 3675, 244, 3677, 562, 0,
    632, 1430, 3678, 3688, 3698, 3708, 3718, 0, 50, 54, 55, 0, 50, 822, 3728, 0,
    0, 351, 76, 3742, 0, 346, 348, 0, 353, 3738, 343, 0, 486, 502, 0, 289,
    3741, 812, 291, 201, 427, 343, 0, 437, 440, 392, 343, 814, 0, 0, 351, 76,
    0, 353, 3743, 343, 0, 486, 502, 3746, 3756, 0, 1434, 3769, 3768, 0, 315, 487,
    0, 3766, 95, 717, 1435, 415, 3767, 1438, 1439, 0, 1354, 1356, 0, 1440, 3770, 0,
    315, 487, 194, 1399, 0, 486, 1441, 3771, 1022, 551, 1443, 3772, 3782, 3792, 0, 89,
    3802, 1446, 3804, 877, 1454, 0, 194, 441, 0, 1456, 441, 1457, 1458, 1459, 1460, 1462,
    970, 1396, 1464, 1465, 3817, 0, 1467, 3808, 3805, 1265, 1469, 246, 3809, 1471, 1472, 833,
    1267, 0, 275, 1464, 246, 1472, 1475, 3814, 0, 3810, 1267, 3836, 3833, 0, 3821, 1476,
    1479, 3818, 1446, 0, 1456, 441, 0, 194, 441, 3822, 3826, 0, 3825, 1267, 833, 0,
    1456, 441, 0, 194, 441, 3830, 3837, 3847, 3857, 3868, 3879, 343, 1483, 0, 1485, 3878,
    343, 1483, 3897, 343, 3883, 3880, 760, 0, 343, 392, 239, 0, 1488, 343, 89, 89,
    343, 0, 3884, 1211, 343, 3885, 0, 89, 343, 0, 329, 343, 0, 1272, 239, 0,
    819, 3891, 253, 3889, 1489, 1490, 0, 1493, 291, 0, 3892, 513, 230, 415, 343, 60,
    3893, 513, 230, 415, 343, 1494, 1497, 1499, 3895, 0, 343, 3894, 660, 830, 343, 0,
    3901, 1236, 0, 3919, 1290, 0, 402, 329, 49, 238, 1459, 3904, 535, 536, 537, 0,
    346, 538, 3905, 0, 530, 161, 348, 531, 0, 533, 348, 161, 348, 531, 534, 0,
    172, 491, 3906, 542, 541, 0, 543, 3907, 547, 548, 549, 0, 550, 95, 0, 551,
    95, 0, 552, 238, 0, 266, 238, 3912, 0, 3908, 87, 287, 3910, 3909, 238, 291,
    0, 89, 315, 0, 392, 3913, 513, 0, 513, 85, 513, 0, 1500, 343, 0, 1501,
    1502, 3915, 0, 1328, 1273, 0, 1330, 1273, 0, 3914, 1503, 760, 1384, 1273, 0, 1500,
    343, 1273, 786, 0, 275, 3925, 3923, 238, 3928, 3938, 0, 3947, 0, 230, 825, 759,
    3951, 1505, 312, 759, 0, 3952, 60, 1507, 919, 1485, 3955, 239, 0, 238, 3953, 60,
    1507, 919, 1485, 3954, 239, 956, 0, 3956, 289, 287, 1364, 343, 3959, 3957, 1508, 3963,
    0, 1509, 1510, 1512, 0, 0, 78, 3962, 1509, 1513, 968, 1514, 1515, 3964, 3974, 3984,
    3994, 4004, 4014, 4024, 4034, 4044, 4054, 4065, 4075, 0, 318, 4085, 0, 1517, 161, 1519,
    0, 47, 197, 0, 146, 161, 123, 195, 76, 87, 1521, 4086, 0, 275, 318, 326,
    0, 275, 994, 326, 0, 886, 315, 131, 0, 1524, 441, 4087, 262, 264, 195, 4088,
    4098, 0, 748, 633, 748, 4108, 0, 194, 441, 4122, 4121, 415, 95, 1526, 1528, 343,
    4118, 0, 1456, 441, 281, 1457, 1458, 1459, 1460, 4124, 0, 1456, 441, 1454, 1457, 1458,
    1459, 1396, 4125, 0, 4139, 4138, 415, 95, 343, 4135, 1531, 4141, 0, 1532, 1533, 4154,
    4152, 4151, 1526, 1054, 4157, 4167, 4177, 4187, 0, 1457, 4200, 4197, 343, 4201, 0, 1456,
    441, 1454, 1458, 1459, 1396, 4202, 4212, 4222, 4232, 4242, 4252, 0, 4262, 4272, 4282, 4292,
    4302, 4312, 4322, 4332, 4342, 4349, 4359, 4369, 0, 1549, 1366, 4379, 0, 1549, 346, 1366,
    4389, 0, 4399, 1550, 1551, 818, 4400, 4410, 4420, 4430, 0, 1552, 50, 45, 0, 1553,
    267, 0, 4440, 0, 281, 87, 1554, 4443, 4441, 1555, 89, 53, 953, 1556, 267, 948,
    4446, 4456, 4466, 0, 346, 60, 4482, 4479, 4478, 4476, 1334, 770, 201, 4483, 4493, 4503,
    230, 0, 230, 4513, 1558, 1559, 0, 0, 418, 419, 0, 1560, 0, 818, 4514, 640,
    1562, 456, 4515, 1558, 1559, 0, 818, 4516, 640, 1562, 0, 4520, 415, 201, 4517, 343,
    1564, 4521, 0, 0, 967, 158, 0, 275, 573, 0, 398, 506, 48, 284, 4531, 4541,
    4551, 4561, 4571, 4581, 4591, 4601, 4611, 4621, 4631, 4641, 4651, 4661, 4671, 0, 143, 1556,
    244, 1573, 158, 0, 1575, 4681, 635, 123, 660, 4682, 0, 289, 645, 770, 0, 161,
    1576, 1577, 1579, 0, 1272, 4685, 253, 4683, 0, 343, 1573, 4686, 4696, 4706, 4716, 4726,
    4736, 4746, 4756, 4766, 4776, 4786, 4796, 4806, 4816, 0, 1239, 4828, 0, 95, 4826, 513,
    1211, 513, 0, 513, 4827, 513, 1211, 4829, 4839, 1585, 1586, 1587, 1588, 1589, 1590, 0,
    1556, 4840, 152, 151, 501, 163, 244, 4841, 4851, 4861, 1379, 1591, 1592, 277, 1593, 1594,
    0, 839, 346, 819, 89, 4862, 1595, 4872, 4882, 4892, 4902, 4912, 4922, 4943, 0, 1600,
    1601, 1479, 4932, 1602, 0, 4936, 1604, 4939, 1606, 1608, 4940, 0, 1609, 1601, 1479, 1604,
    0, 1610, 1479, 4944, 0, 986, 0, 0, 161, 4954, 4960, 4957, 343, 4961, 4965, 1612,
    0, 4964, 510, 1613, 1614, 1615, 0, 398, 4968, 4966, 230, 4971, 4981, 0, 1193, 1617,
    1619, 4992, 660, 1620, 701, 93, 0, 4993, 275, 494, 816, 0, 1622, 4994, 5004, 5014,
    5024, 5032, 5042, 5052, 0, 44, 5062, 311, 1626, 5063, 5073, 5083, 5093, 0, 5103, 5113,
    5123, 5133, 5143, 5153, 5163, 5173, 5183, 5193, 0, 747, 633, 747, 5203, 5213, 0, 0,
    346, 1631, 1334, 0, 906, 1335, 5223, 1014, 1632, 0, 1633, 87, 5224, 5234, 0, 5245,
    1634, 1635, 5246, 1636, 1637, 1638, 437, 442, 1639, 454, 458, 57, 0, 281, 5251, 5248,
    5247, 95, 343, 57, 5253, 0, 1464, 5252, 95, 343, 0, 85, 95, 5254, 5264, 5274,
    5284, 5294, 5304, 5314, 5324, 0, 5334, 5344, 0, 1642, 230, 5354, 513, 95, 277, 695,
    1379, 1380, 1643, 890, 1644, 1646, 1648, 5355, 5364, 5371, 0, 1650, 513, 0, 5381, 5391,
    5401, 5411, 5421, 5431, 0, 1652, 163, 514, 5443, 5441, 244, 253, 0, 60, 5444, 0,
    373, 200, 0, 487, 200, 0, 487, 1653, 0, 200, 155, 1653, 200, 0, 200, 487,
    1653, 1653, 0, 373, 1653, 1654, 562, 5445, 5455, 5465, 5475, 5485, 5495, 5505, 5515, 5525,
    5535, 5545, 0, 1663, 825, 230, 5558, 0, 972, 76, 0, 85, 5555, 770, 5561, 5571,
    5581, 5591, 0, 5601, 5611, 0, 5621, 5631, 5641, 5651, 5661, 5671, 5681, 5691, 5701, 5711,
    142, 0, 143, 158, 968, 0, 1671, 1556, 5721, 5731, 1672, 1674, 1676, 0, 1038, 1678,
    0, 1356, 343, 1679, 5735, 1681, 1683, 0, 875, 511, 0, 1038, 346, 1678, 441, 1685,
    0, 1356, 343, 1679, 5736, 1681, 1683, 0, 875, 511, 0, 1038, 346, 1678, 0, 441,
    1687, 0, 1356, 343, 1679, 5737, 1689, 0, 875, 511, 0, 1691, 5738, 244, 1692, 398,
    5742, 244, 1692, 0, 5739, 770, 246, 0, 201, 246, 5743, 1606, 5746, 1693, 0, 155,
    1695, 5749, 5759, 0, 5770, 0, 770, 1699, 5774, 5784, 5794, 5804, 5814, 5824, 5833, 5843,
    5853, 5863, 5873, 5883, 5893, 5903, 5913, 5923, 5932, 5942, 5952, 5962, 5972, 5982, 5992, 6002,
    1708, 6006, 6015, 6025, 6035, 6045, 6055, 6064, 6074, 6084, 6094, 6104, 6114, 6124, 6134, 6144,
    6154, 6164, 6175, 6185, 0, 311, 6195, 1714, 1364, 287, 343, 0, 230, 392, 6199, 230,
    6196, 1715, 0, 1459, 6200, 1717, 6202, 6212, 6222, 6233, 0, 1723, 868, 6243, 6253, 6263,
    6273, 6282, 0, 6292, 6302, 6309, 6319, 6329, 6339, 6349, 6359, 6369, 6379, 6389, 6399, 6409,
    6419, 6429, 6439, 6449, 6459, 6469, 6479, 6498, 968, 1733, 6489, 1384, 6492, 1503, 6495, 1734,
    1735, 0, 6499, 6504, 760, 6503, 6500, 1384, 1734, 1330, 1328, 1332, 6505, 6515, 6525, 6535,
    6545, 0, 6555, 1739, 1740, 1741, 1742, 1743, 6556, 6566, 0, 6576, 0, 1744, 633, 0,
    747, 748, 0, 746, 747, 0, 746, 748, 6577, 1745, 6587, 6597, 6607, 0, 1746, 230,
    0, 85, 513, 6617, 6627, 6637, 6647, 6658, 6668, 6678, 6688, 6699, 6709, 0, 513, 830,
    700, 6730, 6719, 1748, 6721, 6733, 825, 0, 825, 825, 6734, 161, 392, 398, 85, 827,
    828, 829, 790, 830, 831, 833, 6735, 6745, 6755, 6765, 0, 1464, 6775, 95, 343, 6776,
    6786, 6796, 6806, 6816, 6825, 6835, 6845, 6855, 6865, 6875, 6885, 6895, 6905, 6915, 6925, 6935,
    6945, 0, 512, 6959, 6955, 1763, 6962, 0, 6973, 6972, 1555, 513, 6975, 6985, 0, 1027,
    656, 0, 1456, 441, 6997, 0, 6994, 415, 95, 343, 6995, 1558, 1559, 0, 398, 741,
    0, 818, 6996, 640, 1562, 1764, 1454, 1434, 6999, 281, 1457, 1458, 1459, 1434, 1493, 0,
    275, 54, 1465, 7000, 7010, 1456, 1766, 7011, 7021, 7031, 7041, 7051, 0, 1494, 1769, 7064,
    230, 7061, 1715, 7065, 0, 746, 746, 0, 7080, 0, 7077, 7075, 415, 1380, 812, 1489,
    85, 7076, 415, 1380, 812, 1489, 415, 1771, 7086, 0, 7083, 7081, 415, 1380, 812, 1489,
    85, 7082, 415, 1380, 812, 1489, 415, 7087, 7097, 7107, 7117, 7127, 7137, 7147, 7157, 7167,
    7177, 7186, 7195, 7205, 7215, 7245, 0, 60, 7227, 7225, 343, 0, 89, 7233, 7231, 7230,
    343, 612, 613, 87, 7239, 7237, 7236, 343, 612, 613, 0, 275, 7242, 0, 1780, 95,
    85, 1781, 95, 0, 559, 441, 7251, 7248, 7246, 343, 7252, 7262, 7272, 7282, 7304, 0,
    1784, 7302, 7292, 1784, 513, 0, 7305, 7315, 7325, 7336, 7346, 7357, 7367, 7376, 7386, 7396,
    7406, 7415, 7425, 7435, 7445, 7455, 7465, 7475, 7485, 7495, 7505, 7515, 7525, 7534, 1791, 1794,
    1796, 0, 7538, 7548, 7559, 0, 1798, 7569, 1799, 7570, 7580, 7590, 7600, 7610, 0, 1254,
    76, 7622, 7620, 239, 392, 7625, 770, 1802, 7626, 7636, 0, 821, 822, 7646, 7656, 7666,
    7676, 7686, 7696, 7706, 7716, 7726, 7736, 7746, 7756, 7766, 7776, 7786, 0, 1811, 7796, 0,
    1812, 7806, 0, 7819, 7816, 0, 1814, 7815, 238, 513, 7820, 7830, 7847, 0, 163, 7840,
    1815, 0, 85, 7842, 163, 176, 0, 176, 60, 45, 7844, 7843, 267, 363, 0, 85,
    7846, 163, 176, 230, 0, 1817, 158, 0, 877, 7848, 343, 7851, 0, 611, 95, 0,
    393, 7875, 0, 311, 825, 7871, 7861, 0, 238, 0, 825, 238, 7878, 0, 741, 7888,
    57, 1821, 1822, 7889, 770, 7892, 7902, 7934, 7912, 7923, 8061, 0, 7957, 7935, 7946, 7969,
    1825, 7958, 0, 7973, 7994, 1825, 7984, 0, 7998, 8019, 1825, 8009, 0, 1827, 8023, 8059,
    8058, 8032, 8054, 1825, 8043, 8062, 8072, 8082, 8092, 8101, 8110, 8120, 8129, 8138, 8148, 8158,
    8168, 0, 1834, 561, 441, 8179, 8178, 260, 262, 264, 197, 87, 8181, 260, 262, 264,
    197, 8182, 8192, 8202, 8212, 0, 149, 244, 158, 1838, 8222, 0, 161, 142, 1839, 0,
    161, 155, 1839, 0, 1840, 8226, 510, 1842, 1843, 8227, 8237, 8247, 8263, 333, 0, 266,
    8257, 289, 348, 8260, 8258, 415, 331, 0, 486, 1844, 8264, 0, 631, 171, 164, 650,
    244, 970, 8274, 8284, 0, 1846, 8297, 8295, 8294, 0, 650, 277, 0, 48, 284, 0,
    675, 277, 8301, 0, 506, 650, 508, 0, 163, 650, 244, 0, 45, 650, 267, 0,
    8300, 41, 635, 650, 269, 0, 1259, 0, 825, 1259, 0, 287, 1260, 8304, 8302, 1847,
    8307, 8317, 8327, 8337, 8347, 8357, 8367, 8377, 8387, 8397, 8407, 8417, 0, 317, 1855, 996,
    0, 317, 8427, 57, 47, 238, 8428, 8438, 8448, 8458, 8468, 8478, 8488, 8498, 8508, 8518,
    8528, 8538, 0, 328, 0, 318, 326, 329, 8548, 0, 1517, 161, 1519, 0, 47, 197,
    0, 146, 161, 123, 195, 195, 89, 1521, 335, 8549, 8558, 8568, 8578, 8588, 230, 8598,
    8608, 8618, 8626, 8636, 8646, 8656, 8666, 8676, 8686, 8696, 8706, 8716, 8726, 0, 1866, 8736,
    1867, 635, 8737, 95, 0, 8740, 1868, 8741, 95, 8744, 8754, 8764, 8774, 8784, 8794, 8804,
    8814, 8824, 8834, 0, 491, 1873, 0, 329, 76, 161, 8859, 0, 163, 244, 0, 412,
    415, 0, 252, 78, 8844, 80, 0, 41, 269, 0, 42, 76, 0, 378, 153, 76,
    0, 50, 163, 244, 0, 166, 78, 8850, 8847, 1508, 0, 92, 93, 95, 0, 171,
    164, 244, 0, 379, 44, 76, 0, 167, 78, 8854, 8851, 1508, 0, 569, 76, 0,
    47, 197, 0, 168, 78, 8858, 8855, 1508, 0, 45, 267, 0, 169, 76, 0, 625,
    76, 0, 48, 76, 0, 49, 76, 0, 164, 244, 839, 8860, 513, 348, 8861, 1874,
    0, 700, 246, 8867, 0, 819, 0, 161, 8864, 8862, 343, 57, 0, 150, 1876, 8868,
    1878, 1556, 89, 877, 0, 1456, 441, 0, 161, 1576, 281, 1272, 1771, 1460, 0, 275,
    486, 54, 1045, 1699, 8880, 1556, 89, 8870, 877, 0, 1456, 441, 281, 1272, 85, 1460,
    1699, 8881, 0, 1880, 1882, 1884, 1269, 1886, 1888, 1390, 1890, 1890, 221, 1454, 1892, 1894,
    1465, 1460, 1462, 970, 0, 161, 1576, 1272, 1045, 1699, 8902, 8894, 8891, 1503, 8898, 8895,
    1896, 1897, 8899, 770, 0, 825, 230, 207, 8903, 0, 825, 230, 207, 8907, 8916, 8926,
    8936, 0, 613, 95, 0, 486, 613, 8937, 0, 612, 95, 0, 486, 612, 8938, 8948,
    8958, 8968, 8978, 8988, 8998, 9008, 9018, 9028, 123, 260, 262, 264, 9029, 9039, 0, 266,
    9049, 1902, 0, 9072, 0, 415, 207, 287, 9051, 291, 427, 201, 415, 343, 289, 9052,
    812, 0, 9058, 9056, 415, 287, 9069, 9068, 770, 9066, 0, 9061, 291, 427, 201, 415,
    343, 289, 9062, 812, 0, 266, 1637, 9075, 57, 9073, 76, 9076, 476, 478, 0, 1734,
    9077, 0, 1460, 1734, 221, 1454, 1890, 9081, 9092, 1904, 0, 9091, 1905, 1554, 57, 1906,
    0, 266, 45, 414, 267, 0, 266, 9093, 0, 1009, 1855, 0, 1855, 1855, 996, 9094,
    197, 289, 0, 266, 9095, 1855, 1009, 9103, 0, 238, 9096, 87, 287, 9100, 289, 9098,
    9097, 291, 311, 312, 609, 0, 1907, 1908, 9101, 513, 1009, 289, 0, 45, 513, 0,
    47, 9102, 621, 197, 9104, 9114, 9124, 9134, 9144, 9154, 0, 9164, 1367, 1368, 9165, 0,
    827, 239, 1370, 9169, 0, 1912, 9179, 57, 1913, 931, 932, 1914, 1915, 238, 9180, 9190,
    9200, 1918, 1919, 9201, 9211, 95, 9215, 9225, 311, 312, 1555, 9226, 311, 312, 825, 9227,
    230, 0, 230, 807, 253, 9229, 0, 1206, 207, 9240, 9250, 9260, 9269, 9278, 9288, 9298,
    9308, 9318, 9328, 9338, 9348, 9358, 9368, 9378, 9388, 9398, 9408, 9418, 9428, 9438, 9448, 9458,
    9468, 9478, 9488, 9498, 9508, 9518, 9527, 9537, 9547, 9557, 9567, 770, 1802, 1328, 1942, 1944,
    1946, 1948, 1950, 1952, 1954, 1956, 1958, 1960, 1962, 1964, 1966, 1968, 1970, 1972, 1974, 1976,
    1978, 1980, 1332, 1330, 1982, 1984, 1986, 1988, 1990, 1992, 1994, 1996, 1998, 2000, 2002, 2004,
    2006, 2008, 2010, 2012, 2014, 2016, 2018, 2020, 2022, 2024, 2026, 2028, 2030, 2032, 2034, 2036,
    2038, 2040, 2042, 2044, 2046, 2048, 1974, 2050, 2052, 1980, 2054, 2056, 2058, 2060, 2062, 2064,
    2066, 2068, 2070, 2072, 2074, 2076, 2078, 2080, 2082, 2084, 2086, 2088, 2090, 2092, 2094, 2096,
    2098, 2100, 2102, 2104, 2106, 2108, 2110, 2112, 2114, 2116, 2118, 2120, 2122, 2124, 2126, 2128,
    2130, 2132, 2134, 2136, 2138, 2140, 2142, 2144, 2146, 2148, 2150, 2152, 2154, 2156, 2158, 2160,
    2162, 2164, 2166, 2168, 831, 9568, 9578, 9588, 9598, 9608, 9618, 9628, 9638, 9648, 9658, 2171,
    418, 9659, 9671, 9669, 0, 700, 246, 0, 9674, 9684, 9694, 9704, 9714, 9724, 9734, 9744,
    9751, 9761, 9771, 9781, 9791, 0, 0, 351, 76, 9812, 0, 353, 9801, 343, 0, 486,
    502, 0, 816, 0, 818, 346, 819, 158, 2179, 0, 2181, 158, 2179, 0, 1440, 0,
    398, 400, 9806, 9804, 2182, 2179, 0, 877, 9809, 343, 0, 50, 822, 158, 824, 9815,
    9813, 473, 0, 346, 473, 9814, 0, 475, 476, 0, 475, 478, 0, 346, 354, 0,
    486, 502, 0, 0, 351, 76, 0, 9819, 816, 2181, 398, 506, 508, 9820, 473, 0,
    346, 473, 9821, 0, 475, 476, 0, 475, 478, 0, 9828, 968, 427, 0, 201, 0,
    275, 2184, 244, 9822, 770, 9825, 1802, 9829, 9839, 9849, 9859, 0, 9869, 2185, 790, 2186,
    0, 9873, 2187, 2186, 9877, 9887, 9897, 9907, 9917, 0, 9927, 151, 152, 9928, 9938, 2189,
    955, 9939, 9949, 9959, 9969, 9979, 9989, 9999, 10009, 10018, 0, 10028, 1907, 2195, 1163, 0,
    10029, 275, 494, 1907, 1908, 10030, 0, 2196, 1907, 1908, 343, 10034, 10044, 10054, 10064, 10074,
    10084, 10094, 10104, 10114, 10124, 10134, 10144, 10154, 10164, 0, 10174, 618, 1501, 1010, 2204, 786,
    10175, 2206, 2207, 0, 614, 230, 0, 1356, 486, 2208, 10176, 10186, 10196, 10206, 10216, 10226,
    10236, 10246, 2212, 2213, 2214, 10247, 10257, 10268, 0, 154, 163, 10279, 10278, 0, 514, 244,
    0, 244, 253, 0, 10281, 453, 2215, 442, 335, 10282, 10292, 10302, 10312, 10322, 10332, 10344,
    10342, 700, 10347, 0, 0, 1485, 0, 311, 1364, 1206, 1644, 0, 1099, 2218, 10357, 10367,
    10377, 10387, 10397, 2221, 10407, 2222, 2223, 2224, 10408, 10418, 0, 2227, 10428, 819, 57, 10429,
    0, 816, 10439, 10449, 10459, 10470, 0, 2228, 10480, 230, 253, 10481, 10491, 0, 1880, 1882,
    1884, 1269, 1886, 1888, 1390, 1890, 1460, 1462, 970, 221, 1454, 0, 275, 486, 54, 0,
    161, 1576, 1272, 1045, 1699, 0, 1734, 10501, 0, 1460, 1734, 10505, 10515, 10525, 10535, 0,
    155, 514, 244, 246, 266, 1695, 1884, 10545, 0, 877, 315, 503, 76, 1269, 0, 1272,
    10548, 253, 10546, 0, 343, 246, 10549, 10559, 10569, 0, 137, 269, 10579, 10589, 230, 10599,
    10609, 10619, 10629, 10639, 10649, 10659, 0, 2236, 10677, 0, 510, 902, 675, 10669, 671, 672,
    10674, 10672, 0, 10670, 510, 902, 675, 671, 672, 2237, 676, 2238, 2239, 154, 543, 10671,
    291, 311, 609, 10678, 0, 244, 158, 10680, 10690, 10700, 10710, 0, 645, 10726, 10723, 10721,
    10720, 343, 289, 246, 221, 1454, 10728, 10738, 10748, 10758, 10768, 10778, 10788, 10798, 10808, 10818,
    10828, 513, 311, 10829, 10839, 10849, 10859, 10869, 10879, 10889, 10898, 10908, 10918, 10928, 10966, 0,
    10938, 1177, 2247, 1182, 2248, 2249, 2250, 2251, 2252, 2253, 2254, 2255, 2256, 2257, 2258, 2259,
    2260, 2261, 2262, 2263, 2264, 2265, 2266, 2267, 2268, 2269, 2270, 2271, 2272, 2273, 2274, 2275,
    2276, 2277, 2278, 2279, 2280, 10958, 10956, 10955, 10944, 10941, 10939, 2281, 10952, 10950, 10947, 10945,
    2281, 1060, 0, 10961, 2282, 2283, 10963, 0, 10962, 1177, 2247, 1182, 2248, 2249, 2250, 2251,
    2252, 2253, 2254, 2255, 2256, 2257, 2258, 2259, 2260, 2261, 2262, 2263, 2264, 2265, 2266, 2267,
    2268, 2269, 2270, 2271, 2272, 2273, 2274, 2275, 2276, 2277, 2278, 2279, 2280, 2282, 2283, 0,
    85, 95, 0, 1038, 0, 343, 10967, 1679, 10970, 343, 10973, 10976, 10979, 0, 877, 10992,
    10989, 343, 10993, 281, 0, 194, 441, 0, 1456, 441, 1457, 1458, 1459, 1771, 1272, 0,
    161, 1576, 1465, 10994, 11004, 11014, 11024, 0, 83, 0, 11032, 11031, 0, 230, 807, 0,
    253, 825, 253, 11036, 0, 230, 11046, 559, 1456, 2290, 221, 2292, 0, 275, 11047, 2294,
    1009, 353, 326, 0, 275, 1043, 11048, 11051, 2296, 150, 11053, 2298, 0, 275, 1043, 11056,
    11059, 2296, 150, 11061, 1897, 0, 275, 0, 486, 54, 0, 11064, 0, 275, 486, 45,
    2299, 0, 2300, 194, 11074, 221, 11077, 11087, 11097, 11104, 11114, 11124, 11134, 11144, 11154, 11164,
    11174, 11184, 11194, 11204, 11214, 11224, 11234, 11244, 11254, 11264, 11274, 11284, 11294, 11304, 11314, 11324,
    11334,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 75, 4294967295, 4294967295, 4294967295, 4294967295, 75, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 79, 82, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 64, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 160, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 190, 192, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295,
    4294967295, 196, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 77, 4294967295, 198, 4294967295, 202, 4294967295, 4294967295,
    208, 210, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 223, 4294967295, 224,
    229, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 241,
    4294967295, 4294967295, 245, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 82, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295,
    268, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 320, 4294967295, 323, 4294967295, 4294967295, 325,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 320, 4294967295, 323, 4294967295,
    4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295,
    4294967295, 4294967295, 338, 340, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 77, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 362,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 77, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79,
    82, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 240, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 401, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 240, 4294967295,
    4294967295, 420, 4294967295, 425, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 420, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 202, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 452, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 452, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 457, 4294967295, 4294967295, 457, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 470, 4294967295, 4294967295,
    470, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 457, 4294967295, 4294967295, 457, 4294967295, 4294967295, 4294967295, 245, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295,
    4294967295, 4294967295, 517, 4294967295, 4294967295, 4294967295, 416, 4294967295, 416, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 79, 82, 4294967295, 555, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295,
    4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 566, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 416, 4294967295,
    4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 268, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295,
    4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 82, 4294967295,
    568, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 572, 4294967295, 4294967295, 4294967295, 574,
    4294967295, 4294967295, 4294967295, 576, 4294967295, 4294967295, 4294967295, 576, 4294967295, 4294967295, 4294967295, 576, 4294967295, 4294967295, 4294967295, 572,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 198, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 198, 4294967295,
    4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 617, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 617, 4294967295, 4294967295, 452, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 617, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 624, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 624, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 628, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 572, 4294967295, 4294967295, 574, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 574, 4294967295,
    4294967295, 4294967295, 4294967295, 574, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 628, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 416, 240, 4294967295, 4294967295, 420, 4294967295, 425, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295,
    4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 420, 4294967295, 4294967295, 4294967295,
    344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 452, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    452, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    457, 4294967295, 4294967295, 457, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 470, 4294967295, 4294967295, 470, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 457, 4294967295, 4294967295, 457,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 638, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 518, 4294967295, 4294967295, 4294967295, 416, 4294967295, 416, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295,
    268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 648, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 285, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 77, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 659, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 659, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    362, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 416, 240, 4294967295, 4294967295, 420, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 240,
    4294967295, 4294967295, 420, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 416,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245,
    4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 344, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 416, 4294967295,
    4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 678, 678, 4294967295, 678, 678, 4294967295, 4294967295, 96, 4294967295, 344,
    4294967295, 4294967295, 96, 4294967295, 344, 4294967295, 4294967295, 96, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 344, 4294967295, 229, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 718,
    202, 416, 344, 4294967295, 240, 4294967295, 4294967295, 4294967295, 720, 721, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 678, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 758, 4294967295, 761, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 232, 4294967295, 4294967295, 769, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 416, 4294967295, 4294967295, 416, 4294967295, 416, 4294967295, 416, 4294967295, 4294967295,
    4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 783, 4294967295,
    4294967295, 789, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 783, 4294967295, 4294967295, 789, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 509, 4294967295, 509,
    4294967295, 245, 4294967295, 4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 813, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 813, 4294967295, 202, 4294967295, 344, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 452, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 574, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 574, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 425, 4294967295, 4294967295, 420, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    245, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 77, 4294967295, 4294967295, 416, 4294967295, 4294967295, 77, 4294967295, 4294967295,
    4294967295, 77, 4294967295, 4294967295, 270, 4294967295, 4294967295, 362, 4294967295, 4294967295, 79, 4294967295, 82, 4294967295, 4294967295, 509,
    4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 840, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 288, 288, 288,
    288, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 288, 288, 288,
    4294967295, 4294967295, 4294967295, 860, 288, 4294967295, 678, 288, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 288, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 416, 288, 4294967295,
    4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 416, 288, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 344, 202, 4294967295, 4294967295, 420, 880, 4294967295, 4294967295,
    344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 891,
    891, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 891, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 891, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 240, 4294967295,
    4294967295, 4294967295, 4294967295, 79, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295,
    4294967295, 288, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288,
    4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295,
    288, 77, 4294967295, 4294967295, 288, 77, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288,
    4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 288, 4294967295, 285, 4294967295,
    4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 240, 4294967295, 4294967295, 420, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 268, 4294967295,
    4294967295, 941, 4294967295, 4294967295, 941, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 416, 240, 4294967295, 4294967295, 4294967295,
    420, 4294967295, 425, 470, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 954, 4294967295, 4294967295, 4294967295, 4294967295,
    245, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 425, 470,
    4294967295, 4294967295, 4294967295, 4294967295, 648, 4294967295, 4294967295, 4294967295, 954, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    79, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 416, 4294967295, 240, 957,
    4294967295, 4294967295, 240, 240, 963, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 245,
    4294967295, 4294967295, 401, 4294967295, 4294967295, 965, 4294967295, 4294967295, 4294967295, 4294967295, 965, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 457, 4294967295, 4294967295, 285, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 457, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 245, 969, 4294967295, 4294967295, 4294967295, 4294967295, 202, 971, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 77, 4294967295, 4294967295, 288,
    4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 79,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 79, 82, 963, 4294967295, 4294967295,
    4294967295, 572, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 576, 4294967295, 4294967295, 574, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295,
    4294967295, 198, 4294967295, 4294967295, 198, 4294967295, 4294967295, 198, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 198, 268, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 617, 4294967295, 998, 4294967295, 4294967295, 4294967295, 4294967295,
    954, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 941, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 624,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 344, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 628, 4294967295, 4294967295, 4294967295, 4294967295, 572, 4294967295, 4294967295, 574, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 245, 969, 4294967295, 4294967295, 4294967295, 4294967295, 202, 971, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 638, 470, 4294967295, 4294967295,
    245, 4294967295, 1018, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416,
    4294967295, 4294967295, 4294967295, 4294967295, 425, 470, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295,
    4294967295, 4294967295, 4294967295, 425, 470, 4294967295, 4294967295, 4294967295, 4294967295, 648, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 202, 4294967295, 4294967295, 4294967295, 420, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 457, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295,
    4294967295, 288, 77, 4294967295, 4294967295, 288, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 659, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295,
    4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 954, 4294967295, 4294967295, 4294967295,
    4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 457, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 969, 1046, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 79, 4294967295,
    1055, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1096, 1098, 1100, 1102,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1164, 4294967295, 4294967295, 4294967295, 4294967295, 1164, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1164, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1164, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1164, 4294967295, 4294967295, 1164, 4294967295, 4294967295, 1205, 1207, 1209, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1098, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 1262, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 247, 4294967295, 4294967295, 4294967295, 4294967295, 1266, 247, 1268, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 77, 1270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 247, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1291, 4294967295, 4294967295, 4294967295, 1291, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 77, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 245, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 79, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 572, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    617, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 628,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 659, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1100, 1301, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 965, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1327, 4294967295, 4294967295, 1329, 1331, 1333, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 96, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 96, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1369, 4294967295, 1371, 813, 4294967295,
    4294967295, 4294967295, 813, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 416,
    202, 1376, 344, 4294967295, 84, 4294967295, 4294967295, 1100, 1102, 4294967295, 4294967295, 344, 4294967295, 240, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1381, 1376, 4294967295, 4294967295, 4294967295, 4294967295, 1385, 4294967295, 718, 1387, 1389, 416, 1391,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1418, 1420, 1422, 4294967295, 1424, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 813, 4294967295, 202, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 718, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1447, 4294967295, 4294967295, 1455, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1461, 1463,
    971, 4294967295, 4294967295, 1466, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1266, 1470, 247, 4294967295, 4294967295, 1474, 4294967295,
    1268, 4294967295, 4294967295, 4294967295, 247, 1474, 4294967295, 4294967295, 4294967295, 4294967295, 1268, 4294967295, 4294967295, 4294967295, 4294967295, 1478,
    1480, 4294967295, 1447, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 4294967295, 1268, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 1484, 4294967295, 4294967295, 4294967295,
    344, 1484, 4294967295, 344, 4294967295, 4294967295, 761, 4294967295, 344, 4294967295, 240, 4294967295, 4294967295, 344, 4294967295, 4294967295,
    344, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 344, 4294967295, 4294967295, 240, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1491, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 344, 4294967295,
    4294967295, 4294967295, 4294967295, 416, 344, 4294967295, 1498, 1387, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 344, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1291, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1329, 4294967295, 4294967295, 1331, 4294967295, 4294967295, 4294967295, 1504, 761, 1385, 4294967295, 4294967295, 4294967295,
    344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 758,
    4294967295, 4294967295, 4294967295, 758, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 240, 957, 4294967295, 4294967295, 4294967295, 288, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 79, 1511, 4294967295, 4294967295, 4294967295, 79, 4294967295, 79, 79, 969, 1511, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1518, 4294967295, 1520,
    4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 196, 77, 4294967295, 1522, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 196, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 1527, 1530, 344,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1461, 4294967295, 4294967295, 4294967295, 4294967295, 1455, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1527, 1055, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295,
    4294967295, 1455, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1545, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 954, 4294967295, 268, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 420, 4294967295, 1563, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 457, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 202, 4294967295, 344,
    1565, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 574, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    245, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1578, 1580, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1480, 4294967295, 4294967295, 4294967295, 224, 1605, 4294967295, 1607, 4294967295, 241, 4294967295, 4294967295, 4294967295, 1480, 1605,
    4294967295, 4294967295, 1480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 4294967295, 344, 241, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 96,
    4294967295, 1381, 4294967295, 891, 1645, 1647, 1649, 1293, 1293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 969, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1673, 1675, 1677, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 344, 4294967295, 4294967295, 1682, 1684, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 344, 4294967295, 4294967295, 1682, 1684, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 1690, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295,
    4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 247, 4294967295, 202, 247, 224, 1607, 241, 1694, 4294967295, 4294967295,
    1696, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1716, 4294967295, 4294967295, 4294967295, 1718, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 308, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 969, 1385, 4294967295, 1385, 4294967295, 1504, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 761, 4294967295, 4294967295, 1385, 4294967295, 1331, 1329, 1333, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1749, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1765, 1455, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1466, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1716, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 1381, 813, 4294967295,
    4294967295, 4294967295, 416, 1381, 813, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 1381, 813, 4294967295,
    4294967295, 4294967295, 416, 1381, 813, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1793, 1795,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 1803, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1816, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    245, 971, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 268, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295, 1848,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1518, 4294967295, 1520, 4294967295, 4294967295, 198,
    4294967295, 4294967295, 4294967295, 4294967295, 196, 196, 4294967295, 1522, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295,
    416, 4294967295, 4294967295, 79, 4294967295, 82, 4294967295, 4294967295, 270, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 245, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 198, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 77, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 77, 4294967295, 4294967295, 77, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1875,
    4294967295, 4294967295, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 1877, 4294967295,
    1879, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1461, 4294967295, 4294967295,
    4294967295, 4294967295, 1046, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1461,
    4294967295, 4294967295, 4294967295, 1881, 1883, 1885, 1270, 1887, 1889, 1391, 1891, 1891, 223, 1455, 1893, 1895,
    1466, 1461, 1463, 971, 4294967295, 4294967295, 4294967295, 4294967295, 1046, 4294967295, 4294967295, 4294967295, 4294967295, 1504, 4294967295, 4294967295,
    1504, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 208, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1903, 4294967295, 4294967295, 4294967295, 416, 208, 288, 4294967295, 4294967295, 4294967295, 202, 416, 344, 4294967295, 4294967295,
    813, 4294967295, 4294967295, 4294967295, 416, 288, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 416,
    344, 4294967295, 4294967295, 813, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1461, 4294967295, 223, 1455, 1891, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 288, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1369, 4294967295, 4294967295,
    4294967295, 240, 1371, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1207, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1803, 1329, 1943, 1945,
    1947, 1949, 1951, 1953, 1955, 1957, 1959, 1961, 1963, 1965, 1967, 1969, 1971, 1973, 1975, 1977,
    1979, 1981, 1333, 1331, 1983, 1985, 1987, 1989, 1991, 1993, 1995, 1997, 1999, 2001, 2003, 2005,
    2007, 2009, 2011, 2013, 2015, 2017, 2019, 2021, 2023, 2025, 2027, 2029, 2031, 2033, 2035, 2037,
    2039, 2041, 2043, 2045, 2047, 2049, 1975, 2051, 2053, 1981, 2055, 2057, 2059, 2061, 2063, 2065,
    2067, 2069, 2071, 2073, 2075, 2077, 2079, 2081, 2083, 2085, 2087, 2089, 2091, 2093, 2095, 2097,
    2099, 2101, 2103, 2105, 2107, 2109, 2111, 2113, 2115, 2117, 2119, 2121, 2123, 2125, 2127, 2129,
    2131, 2133, 2135, 2137, 2139, 2141, 2143, 2145, 2147, 2149, 2151, 2153, 2155, 2157, 2159, 2161,
    2163, 2165, 2167, 2169, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 678,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2180, 4294967295, 4294967295, 4294967295, 2180, 4294967295, 4294967295, 4294967295,
    4294967295, 401, 4294967295, 4294967295, 2183, 2180, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 509, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 969, 4294967295, 4294967295, 202, 4294967295,
    4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 1803, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1164, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245,
    4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1207, 1645, 4294967295, 1100, 1301, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1881, 1883,
    1885, 1270, 1887, 1889, 1391, 1891, 1461, 1463, 971, 223, 1455, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1046, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1461, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 245, 247, 4294967295, 1696, 1885, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 1270, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 344, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 247, 223, 1455, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1096, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 344, 224, 4294967295, 224, 344, 241, 241, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1466, 4294967295, 4294967295, 4294967295, 2288, 4294967295, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2291, 223, 2293, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 224, 4294967295, 2297, 4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 224,
    4294967295, 2297, 4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 223, 4294967295, 4294967295, 5, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 63, 4294967295, 4294967295, 65, 4294967295, 4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 78, 80, 83, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 63, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 159, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 189, 191, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295,
    4294967295, 195, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 76, 4294967295, 197, 4294967295, 201, 4294967295, 4294967295,
    207, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 225,
    227, 4294967295, 4294967295, 4294967295, 4294967295, 231, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 242,
    4294967295, 4294967295, 244, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 80, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295,
    267, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 319, 4294967295, 321, 4294967295, 4294967295, 324,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 319, 4294967295, 321, 4294967295,
    4294967295, 324, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295,
    4294967295, 4294967295, 337, 339, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 76, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295,
    76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 361,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 76, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78,
    80, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 239, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295,
    76, 4294967295, 400, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 239, 4294967295,
    4294967295, 419, 4294967295, 424, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 419, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 201, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 451, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 451, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 456, 4294967295, 4294967295, 456, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 468, 4294967295, 4294967295,
    471, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 284,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 456, 4294967295, 4294967295, 456, 4294967295, 4294967295, 4294967295, 244, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 508, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295,
    4294967295, 4294967295, 516, 4294967295, 4294967295, 4294967295, 415, 4294967295, 415, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 78, 80, 4294967295, 554, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 508, 4294967295, 4294967295, 4294967295, 4294967295, 508, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295,
    4294967295, 508, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 565, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 415, 4294967295,
    4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 267, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295,
    4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    76, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 80, 4294967295,
    567, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 571, 4294967295, 4294967295, 4294967295, 573,
    4294967295, 4294967295, 4294967295, 575, 4294967295, 4294967295, 4294967295, 575, 4294967295, 4294967295, 4294967295, 575, 4294967295, 4294967295, 4294967295, 571,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 197, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 197, 4294967295,
    4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 616, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 616, 4294967295, 4294967295, 451, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 616, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 623, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 623, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 627, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 571, 4294967295, 4294967295, 573, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 573, 4294967295,
    4294967295, 4294967295, 4294967295, 573, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 627, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 415, 239, 4294967295, 4294967295, 419, 4294967295, 424, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295,
    4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 419, 4294967295, 4294967295, 4294967295,
    343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 451, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    451, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    456, 4294967295, 4294967295, 456, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 468, 4294967295, 4294967295, 471, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 456, 4294967295, 4294967295, 456,
    4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 508, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 637, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 643, 4294967295, 4294967295, 4294967295, 415, 4294967295, 415, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295,
    267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 647, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 284, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 76, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76,
    4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76,
    4294967295, 4294967295, 4294967295, 658, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 658, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    361, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 415, 239, 4294967295, 4294967295, 419, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 239,
    4294967295, 4294967295, 419, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 415,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244,
    4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 343, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 415, 4294967295,
    4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 690, 691, 4294967295, 693, 694, 4294967295, 4294967295, 695, 4294967295, 343,
    4294967295, 4294967295, 695, 4294967295, 343, 4294967295, 4294967295, 695, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 343, 4294967295, 227, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 717,
    201, 415, 343, 4294967295, 239, 4294967295, 4294967295, 4294967295, 719, 722, 723, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 679, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 759, 4294967295, 760, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 231, 4294967295, 4294967295, 768, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 415, 4294967295, 4294967295, 415, 4294967295, 415, 4294967295, 415, 4294967295, 4294967295,
    4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 780, 4294967295, 782, 4294967295,
    4294967295, 788, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 782, 4294967295, 4294967295, 788, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 508, 4294967295, 508,
    4294967295, 244, 4294967295, 4294967295, 716, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 812, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 812, 4294967295, 201, 4294967295, 343, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 451, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 573, 4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 573, 4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 424, 4294967295, 4294967295, 419, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    244, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 76, 4294967295, 4294967295, 415, 4294967295, 4294967295, 76, 4294967295, 4294967295,
    4294967295, 76, 4294967295, 4294967295, 269, 4294967295, 4294967295, 361, 4294967295, 4294967295, 78, 4294967295, 80, 4294967295, 4294967295, 508,
    4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 841, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 846, 847, 848,
    849, 850, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 852, 853, 854, 855,
    4294967295, 4294967295, 4294967295, 859, 861, 4294967295, 863, 864, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 287, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 415, 287, 4294967295,
    4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 415, 287, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 343, 201, 4294967295, 4294967295, 419, 879, 4294967295, 4294967295,
    343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 890,
    890, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 890, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 890, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 239, 4294967295,
    4294967295, 4294967295, 4294967295, 78, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295,
    4294967295, 287, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287,
    4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295,
    287, 76, 4294967295, 4294967295, 287, 76, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287,
    4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 287, 4294967295, 284, 4294967295,
    4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295,
    76, 4294967295, 4294967295, 239, 4294967295, 4294967295, 419, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 267, 4294967295,
    4294967295, 940, 4294967295, 4294967295, 940, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 415, 239, 4294967295, 4294967295, 4294967295,
    419, 4294967295, 949, 950, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 953, 4294967295, 4294967295, 4294967295, 4294967295,
    244, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 949, 950,
    4294967295, 4294967295, 4294967295, 4294967295, 647, 4294967295, 4294967295, 4294967295, 953, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    78, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 415, 4294967295, 239, 956,
    4294967295, 4294967295, 239, 239, 960, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 508, 4294967295, 4294967295, 244,
    4294967295, 4294967295, 400, 4294967295, 4294967295, 964, 4294967295, 4294967295, 4294967295, 4294967295, 964, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 456, 4294967295, 4294967295, 284, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 456, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 244, 968, 4294967295, 4294967295, 4294967295, 4294967295, 201, 970, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 76, 4294967295, 4294967295, 287,
    4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 78,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 78, 80, 960, 4294967295, 4294967295,
    4294967295, 571, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 575, 4294967295, 4294967295, 573, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295,
    4294967295, 197, 4294967295, 4294967295, 197, 4294967295, 4294967295, 197, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 197, 267, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 616, 4294967295, 997, 4294967295, 4294967295, 4294967295, 4294967295,
    953, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 940, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 623,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 343, 4294967295, 244, 4294967295, 4294967295,
    4294967295, 627, 4294967295, 4294967295, 4294967295, 4294967295, 571, 4294967295, 4294967295, 573, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 244, 968, 4294967295, 4294967295, 4294967295, 4294967295, 201, 970, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 637, 468, 4294967295, 4294967295,
    244, 4294967295, 1017, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415,
    4294967295, 4294967295, 4294967295, 4294967295, 424, 468, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295,
    4294967295, 4294967295, 4294967295, 424, 468, 4294967295, 4294967295, 4294967295, 4294967295, 647, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 201, 4294967295, 4294967295, 4294967295, 419, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 456, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295,
    4294967295, 287, 76, 4294967295, 4294967295, 287, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 658, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295,
    4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 953, 4294967295, 4294967295, 4294967295,
    4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 456, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 968, 1045, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1052, 1053, 4294967295,
    1054, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1060, 1097, 1099, 1101,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1163, 4294967295, 4294967295, 4294967295, 4294967295, 1163, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1163, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1163, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1163, 4294967295, 4294967295, 1163, 4294967295, 4294967295, 1204, 1206, 1208, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1097, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 1261, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 1265, 246, 1267, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 76, 1269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 246, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1290, 4294967295, 4294967295, 4294967295, 1290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 361, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 76, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 508, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 244, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 78, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 571, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    616, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 623, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 627,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 658, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1099, 1300, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 964, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 76, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1326, 4294967295, 4294967295, 1328, 1330, 1332, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 695, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 695, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1368, 4294967295, 1370, 812, 4294967295,
    4294967295, 4294967295, 812, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 415,
    201, 1375, 343, 4294967295, 1378, 4294967295, 4294967295, 1099, 1101, 4294967295, 4294967295, 343, 4294967295, 239, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1380, 1375, 4294967295, 4294967295, 4294967295, 4294967295, 1384, 4294967295, 717, 1386, 1388, 415, 1390,
    4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1417, 1419, 1421, 4294967295, 1423, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 812, 4294967295, 201, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 76,
    4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 717, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1446, 4294967295, 4294967295, 1454, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1460, 1462,
    970, 4294967295, 4294967295, 1465, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1265, 1469, 246, 4294967295, 4294967295, 1472, 4294967295,
    1267, 4294967295, 4294967295, 4294967295, 246, 1472, 4294967295, 4294967295, 4294967295, 4294967295, 1267, 4294967295, 4294967295, 4294967295, 4294967295, 1476,
    1479, 4294967295, 1446, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 1267, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 1483, 4294967295, 4294967295, 4294967295,
    343, 1483, 4294967295, 343, 4294967295, 4294967295, 760, 4294967295, 343, 4294967295, 239, 4294967295, 4294967295, 343, 4294967295, 4294967295,
    343, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 343, 4294967295, 4294967295, 239, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1490, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 343, 4294967295,
    4294967295, 4294967295, 4294967295, 415, 343, 4294967295, 1497, 1499, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 343, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1328, 4294967295, 4294967295, 1330, 4294967295, 4294967295, 4294967295, 1503, 760, 1384, 4294967295, 4294967295, 4294967295,
    343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 759,
    4294967295, 4294967295, 4294967295, 759, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 239, 956, 4294967295, 4294967295, 4294967295, 287, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1509, 1510, 4294967295, 4294967295, 4294967295, 78, 4294967295, 1509, 1513, 968, 1514, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1517, 4294967295, 1519,
    4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 195, 76, 4294967295, 1521, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 195, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 1526, 1528, 343,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1460, 4294967295, 4294967295, 4294967295, 4294967295, 1454, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1526, 1054, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295,
    4294967295, 1454, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1546, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 953, 4294967295, 267, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 419, 4294967295, 1560, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 456, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 201, 4294967295, 343,
    1564, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 573, 4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    244, 1573, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1577, 1579, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 1573, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1479, 4294967295, 4294967295, 4294967295, 225, 1604, 4294967295, 1606, 4294967295, 242, 4294967295, 4294967295, 4294967295, 1479, 1604,
    4294967295, 4294967295, 1479, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 780, 4294967295, 4294967295, 343, 242, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    2, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 695,
    4294967295, 1380, 4294967295, 890, 1644, 1646, 1648, 1394, 1294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 968, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1672, 1674, 1676, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 343, 4294967295, 4294967295, 1681, 1683, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 343, 4294967295, 4294967295, 1681, 1683, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 1689, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295,
    4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 201, 246, 225, 1606, 242, 1693, 4294967295, 4294967295,
    1695, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1715, 4294967295, 4294967295, 4294967295, 1717, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1709, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 968, 1733, 4294967295, 1384, 4294967295, 1503, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 760, 4294967295, 4294967295, 1384, 4294967295, 1330, 1328, 1332, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1748, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1764, 1454, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1465, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1715, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 1380, 812, 4294967295,
    4294967295, 4294967295, 415, 1380, 812, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 1380, 812, 4294967295,
    4294967295, 4294967295, 415, 1380, 812, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1791, 1794,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    76, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 1802, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1815, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    244, 970, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 508, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 267, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295, 1847,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1517, 4294967295, 1519, 4294967295, 4294967295, 197,
    4294967295, 4294967295, 4294967295, 4294967295, 195, 195, 4294967295, 1521, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295,
    415, 4294967295, 4294967295, 78, 4294967295, 80, 4294967295, 4294967295, 269, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 76,
    4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 244, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 197, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 76, 4294967295, 4294967295,
    76, 4294967295, 4294967295, 76, 4294967295, 4294967295, 76, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1874,
    4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 1876, 4294967295,
    1878, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1460, 4294967295, 4294967295,
    4294967295, 4294967295, 1045, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1460,
    4294967295, 4294967295, 4294967295, 1880, 1882, 1884, 1269, 1886, 1888, 1390, 1890, 1890, 221, 1454, 1892, 1894,
    1465, 1460, 1462, 970, 4294967295, 4294967295, 4294967295, 4294967295, 1045, 4294967295, 4294967295, 4294967295, 4294967295, 1503, 4294967295, 4294967295,
    1896, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 207, 4294967295, 4294967295, 4294967295, 4294967295, 207, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1902, 4294967295, 4294967295, 4294967295, 415, 207, 287, 4294967295, 4294967295, 4294967295, 201, 415, 343, 4294967295, 4294967295,
    812, 4294967295, 4294967295, 4294967295, 415, 287, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 415,
    343, 4294967295, 4294967295, 812, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1460, 4294967295, 221, 1454, 1890, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 287, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1368, 4294967295, 4294967295,
    4294967295, 239, 1370, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1206, 207, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1802, 1328, 1942, 1944,
    1946, 1948, 1950, 1952, 1954, 1956, 1958, 1960, 1962, 1964, 1966, 1968, 1970, 1972, 1974, 1976,
    1978, 1980, 1332, 1330, 1982, 1984, 1986, 1988, 1990, 1992, 1994, 1996, 1998, 2000, 2002, 2004,
    2006, 2008, 2010, 2012, 2014, 2016, 2018, 2020, 2022, 2024, 2026, 2028, 2030, 2032, 2034, 2036,
    2038, 2040, 2042, 2044, 2046, 2048, 1974, 2050, 2052, 1980, 2054, 2056, 2058, 2060, 2062, 2064,
    2066, 2068, 2070, 2072, 2074, 2076, 2078, 2080, 2082, 2084, 2086, 2088, 2090, 2092, 2094, 2096,
    2098, 2100, 2102, 2104, 2106, 2108, 2110, 2112, 2114, 2116, 2118, 2120, 2122, 2124, 2126, 2128,
    2130, 2132, 2134, 2136, 2138, 2140, 2142, 2144, 2146, 2148, 2150, 2152, 2154, 2156, 2158, 2160,
    2162, 2164, 2166, 2168, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2171,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2179, 4294967295, 4294967295, 4294967295, 2179, 4294967295, 4294967295, 4294967295,
    4294967295, 400, 4294967295, 4294967295, 2182, 2179, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 508, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 968, 4294967295, 4294967295, 201, 4294967295,
    4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 1802, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1163, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244,
    4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1206, 1644, 4294967295, 1099, 2218, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1880, 1882,
    1884, 1269, 1886, 1888, 1390, 1890, 1460, 1462, 970, 221, 1454, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1045, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1460, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 244, 246, 4294967295, 1695, 1884, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 76, 1269, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 343, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 246, 221, 1454, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1060, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 343, 225, 4294967295, 780, 343, 242, 242, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 343, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1465, 4294967295, 4294967295, 4294967295, 2289, 4294967295, 83, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2290, 221, 2292, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 2296, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 225,
    4294967295, 2296, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 4294967295, 1786, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (5730, 2),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (37, 0, 2),
    (40, 2, 2),
    (45, 4, 2),
    (52, 6, 2),
    (54, 8, 2),
    (55, 10, 2),
    (56, 12, 3),
    (98, 15, 2),
    (134, 17, 2),
    (183, 19, 2),
    (184, 21, 2),
    (189, 23, 2),
    (193, 25, 3),
    (197, 28, 3),
    (201, 31, 2),
    (203, 33, 3),
    (205, 36, 2),
    (208, 38, 2),
    (209, 40, 2),
    (221, 42, 2),
    (223, 44, 4),
    (224, 48, 2),
    (229, 50, 2),
    (238, 52, 2),
    (239, 54, 5),
    (242, 59, 3),
    (243, 62, 2),
    (250, 64, 2),
    (253, 66, 2),
    (260, 68, 2),
    (268, 70, 3),
    (272, 73, 3),
    (276, 76, 3),
    (294, 79, 3),
    (299, 82, 2),
    (306, 84, 3),
    (314, 87, 2),
    (330, 89, 2),
    (332, 91, 2),
    (335, 93, 2),
    (364, 95, 2),
    (366, 97, 2),
    (369, 99, 2),
    (390, 101, 2),
    (398, 103, 3),
    (402, 106, 2),
    (403, 108, 2),
    (407, 110, 3),
    (412, 113, 2),
    (417, 115, 2),
    (431, 117, 2),
    (438, 119, 2),
    (446, 121, 2),
    (453, 123, 2),
    (455, 125, 2),
    (459, 127, 2),
    (464, 129, 2),
    (471, 131, 2),
    (475, 133, 2),
    (479, 135, 3),
    (490, 138, 3),
    (530, 141, 2),
    (538, 143, 3),
    (541, 146, 2),
    (549, 148, 2),
    (551, 150, 2),
    (559, 152, 2),
    (560, 154, 2),
    (565, 156, 2),
    (569, 158, 2),
    (571, 160, 2),
    (576, 162, 2),
    (578, 164, 2),
    (589, 166, 3),
    (590, 169, 2),
    (593, 171, 3),
    (595, 174, 2),
    (600, 176, 3),
    (605, 179, 3),
    (612, 182, 2),
    (615, 184, 3),
    (619, 187, 2),
    (626, 189, 2),
    (628, 191, 2),
    (654, 193, 2),
    (667, 195, 2),
    (683, 197, 2),
    (686, 199, 2),
    (701, 201, 2),
    (704, 203, 2),
    (783, 205, 3),
    (791, 208, 2),
    (794, 210, 2),
    (798, 212, 3),
    (821, 215, 3),
    (828, 218, 3),
    (846, 221, 3),
    (850, 224, 3),
    (854, 227, 3),
    (856, 230, 3),
    (860, 233, 3),
    (911, 236, 2),
    (930, 238, 2),
    (931, 240, 2),
    (933, 242, 2),
    (937, 244, 2),
    (941, 246, 3),
    (957, 249, 3),
    (964, 252, 3),
    (969, 255, 3),
    (973, 258, 3),
    (977, 261, 3),
    (983, 264, 3),
    (991, 267, 2),
    (1009, 269, 3),
    (1014, 272, 3),
    (1017, 275, 3),
    (1024, 278, 3),
    (1036, 281, 2),
    (1070, 283, 2),
    (1081, 285, 2),
    (1092, 287, 3),
    (1095, 290, 2),
    (1100, 292, 3),
    (1102, 295, 3),
    (1106, 298, 3),
    (1110, 301, 3),
    (1112, 304, 3),
    (1117, 307, 3),
    (1122, 310, 3),
    (1127, 313, 2),
    (1130, 315, 3),
    (1136, 318, 2),
    (1138, 320, 3),
    (1143, 323, 2),
    (1149, 325, 3),
    (1157, 328, 2),
    (1162, 330, 2),
    (1178, 332, 2),
    (1197, 334, 2),
    (1198, 336, 2),
    (1200, 338, 2),
    (1204, 340, 2),
    (1208, 342, 3),
    (1216, 345, 2),
    (1239, 347, 3),
    (1243, 350, 3),
    (1247, 353, 2),
    (1251, 355, 2),
    (1255, 357, 2),
    (1259, 359, 2),
    (1263, 361, 3),
    (1271, 364, 3),
    (1283, 367, 3),
    (1322, 370, 3),
    (1326, 373, 3),
    (1333, 376, 3),
    (1337, 379, 3),
    (1346, 382, 2),
    (1365, 384, 3),
    (1370, 387, 3),
    (1374, 390, 3),
    (1378, 393, 2),
    (1403, 395, 3),
    (1425, 398, 3),
    (1430, 401, 3),
    (1433, 404, 2),
    (1444, 406, 3),
    (1448, 409, 3),
    (1453, 412, 3),
    (1468, 415, 3),
    (1474, 418, 3),
    (1481, 421, 3),
    (1493, 424, 3),
    (1496, 427, 2),
    (1502, 429, 2),
    (1507, 431, 2),
    (1513, 433, 2),
    (1518, 435, 2),
    (1525, 437, 3),
    (1535, 440, 2),
    (1554, 442, 3),
    (1555, 445, 2),
    (1558, 447, 3),
    (1560, 450, 2),
    (1565, 452, 3),
    (1570, 455, 3),
    (1577, 458, 2),
    (1580, 460, 3),
    (1584, 463, 2),
    (1591, 465, 2),
    (1593, 467, 2),
    (1619, 469, 2),
    (1632, 471, 2),
    (1648, 473, 2),
    (1651, 475, 2),
    (1666, 477, 2),
    (1669, 479, 2),
    (1748, 481, 3),
    (1756, 484, 2),
    (1759, 486, 2),
    (1763, 488, 3),
    (1786, 491, 3),
    (1797, 494, 2),
    (1806, 496, 2),
    (1812, 498, 3),
    (1820, 501, 3),
    (1828, 504, 3),
    (1832, 507, 2),
    (1836, 509, 3),
    (1838, 512, 3),
    (1847, 515, 3),
    (1852, 518, 3),
    (1856, 521, 3),
    (1865, 524, 2),
    (1877, 526, 3),
    (1882, 529, 2),
    (1885, 531, 3),
    (1892, 534, 2),
    (1899, 536, 2),
    (1902, 538, 2),
    (1908, 540, 2),
    (1911, 542, 2),
    (1916, 544, 2),
    (1918, 546, 2),
    (1924, 548, 2),
    (1926, 550, 2),
    (1928, 552, 2),
    (1935, 554, 2),
    (1939, 556, 2),
    (1944, 558, 2),
    (1951, 560, 2),
    (1955, 562, 3),
    (1957, 565, 3),
    (1962, 568, 3),
    (1968, 571, 3),
    (1972, 574, 2),
    (1981, 576, 2),
    (1991, 578, 2),
    (1995, 580, 3),
    (2001, 583, 3),
    (2002, 586, 2),
    (2005, 588, 3),
    (2010, 591, 3),
    (2015, 594, 2),
    (2018, 596, 3),
    (2024, 599, 3),
    (2029, 602, 3),
    (2031, 605, 3),
    (2047, 608, 3),
    (2052, 611, 3),
    (2059, 614, 2),
    (2061, 616, 2),
    (2069, 618, 2),
    (2076, 620, 3),
    (2078, 623, 3),
    (2082, 626, 3),
    (2086, 629, 3),
    (2094, 632, 2),
    (2120, 634, 3),
    (2123, 637, 3),
    (2134, 640, 2),
    (2135, 642, 2),
    (2137, 644, 2),
    (2138, 646, 2),
    (2141, 648, 2),
    (2143, 650, 2),
    (2146, 652, 2),
    (2148, 654, 2),
    (2151, 656, 2),
    (2153, 658, 2),
    (2163, 660, 2),
    (2165, 662, 2),
    (2191, 664, 2),
    (2192, 666, 2),
    (2193, 668, 3),
    (2194, 671, 2),
    (2196, 673, 2),
    (2200, 675, 2),
    (2201, 677, 2),
    (2202, 679, 1),
    (2216, 680, 3),
    (2245, 683, 3),
    (2246, 686, 2),
    (2248, 688, 2),
    (2258, 690, 2),
    (2261, 692, 2),
    (2276, 694, 3),
    (2278, 697, 3),
    (2281, 700, 3),
    (2283, 703, 3),
    (2285, 706, 3),
    (2290, 709, 2),
    (2300, 711, 4),
    (2302, 715, 2),
    (2305, 717, 2),
    (2310, 719, 2),
    (2312, 721, 2),
    (2315, 723, 2),
    (2346, 725, 3),
    (2349, 728, 3),
    (2351, 731, 3),
    (2353, 734, 3),
    (2356, 737, 2),
    (2370, 739, 2),
    (2372, 741, 2),
    (2377, 743, 2),
    (2385, 745, 2),
    (2392, 747, 2),
    (2394, 749, 2),
    (2396, 751, 2),
    (2401, 753, 2),
    (2414, 755, 2),
    (2423, 757, 2),
    (2428, 759, 3),
    (2434, 762, 2),
    (2439, 764, 3),
    (2455, 767, 2),
    (2457, 769, 2),
    (2460, 771, 3),
    (2490, 774, 3),
    (2512, 777, 3),
    (2517, 780, 2),
    (2519, 782, 2),
    (2522, 784, 3),
    (2525, 787, 2),
    (2529, 789, 2),
    (2532, 791, 3),
    (2535, 794, 3),
    (2538, 797, 2),
    (2540, 799, 2),
    (2543, 801, 3),
    (2546, 804, 3),
    (2596, 807, 2),
    (2599, 809, 2),
    (2605, 811, 2),
    (2614, 813, 3),
    (2620, 816, 2),
    (2621, 818, 2),
    (2622, 820, 2),
    (2623, 822, 2),
    (2624, 824, 2),
    (2625, 826, 2),
    (2636, 828, 2),
    (2637, 830, 2),
    (2638, 832, 2),
    (2639, 834, 2),
    (2643, 836, 2),
    (2644, 838, 3),
    (2646, 841, 2),
    (2647, 843, 2),
    (2650, 845, 2),
    (2677, 847, 3),
    (2678, 850, 2),
    (2680, 852, 2),
    (2685, 854, 3),
    (2686, 857, 2),
    (2691, 859, 2),
    (2696, 861, 3),
    (2697, 864, 2),
    (2702, 866, 2),
    (2710, 868, 3),
    (2712, 871, 2),
    (2713, 873, 2),
    (2716, 875, 3),
    (2717, 878, 3),
    (2720, 881, 2),
    (2739, 883, 3),
    (2742, 886, 3),
    (2751, 889, 2),
    (2752, 891, 2),
    (2786, 893, 3),
    (2792, 896, 3),
    (2797, 899, 3),
    (2803, 902, 2),
    (2808, 904, 2),
    (2841, 906, 3),
    (2857, 909, 2),
    (2891, 911, 3),
    (2900, 914, 2),
    (2903, 916, 2),
    (2908, 918, 2),
    (2910, 920, 2),
    (2915, 922, 2),
    (2916, 924, 2),
    (2934, 926, 2),
    (2941, 928, 2),
    (2945, 930, 2),
    (2949, 932, 2),
    (2953, 934, 2),
    (2959, 936, 2),
    (2963, 938, 2),
    (2967, 940, 2),
    (2971, 942, 3),
    (2976, 945, 2),
    (2977, 947, 2),
    (2980, 949, 2),
    (2981, 951, 2),
    (2984, 953, 2),
    (2991, 955, 2),
    (2996, 957, 2),
    (3000, 959, 2),
    (3004, 961, 2),
    (3006, 963, 3),
    (3010, 966, 2),
    (3015, 968, 2),
    (3019, 970, 2),
    (3024, 972, 2),
    (3027, 974, 2),
    (3030, 976, 3),
    (3033, 979, 2),
    (3038, 981, 2),
    (3045, 983, 2),
    (3050, 985, 3),
    (3054, 988, 3),
    (3057, 991, 2),
    (3060, 993, 2),
    (3078, 995, 3),
    (3083, 998, 3),
    (3084, 1001, 2),
    (3088, 1003, 3),
    (3090, 1006, 2),
    (3091, 1008, 2),
    (3096, 1010, 3),
    (3099, 1013, 3),
    (3104, 1016, 3),
    (3108, 1019, 3),
    (3113, 1022, 3),
    (3118, 1025, 2),
    (3119, 1027, 2),
    (3124, 1029, 2),
    (3128, 1031, 3),
    (3136, 1034, 2),
    (3137, 1036, 2),
    (3146, 1038, 2),
    (3148, 1040, 3),
    (3150, 1043, 2),
    (3151, 1045, 2),
    (3154, 1047, 2),
    (3155, 1049, 2),
    (3156, 1051, 2),
    (3164, 1053, 3),
    (3167, 1056, 3),
    (3170, 1059, 2),
    (3173, 1061, 2),
    (3178, 1063, 2),
    (3185, 1065, 2),
    (3188, 1067, 3),
    (3191, 1070, 2),
    (3199, 1072, 3),
    (3206, 1075, 2),
    (3209, 1077, 3),
    (3219, 1080, 3),
    (3220, 1083, 2),
    (3225, 1085, 2),
    (3226, 1087, 2),
    (3232, 1089, 2),
    (3239, 1091, 2),
    (3259, 1093, 2),
    (3260, 1095, 2),
    (3263, 1097, 2),
    (3268, 1099, 2),
    (3278, 1101, 2),
    (3279, 1103, 2),
    (3285, 1105, 2),
    (3287, 1107, 3),
    (3305, 1110, 2),
    (3313, 1112, 2),
    (3318, 1114, 2),
    (3323, 1116, 2),
    (3324, 1118, 2),
    (3325, 1120, 2),
    (3329, 1122, 3),
    (3338, 1125, 2),
    (3341, 1127, 2),
    (3347, 1129, 3),
    (3389, 1132, 3),
    (3393, 1135, 3),
    (3396, 1138, 3),
    (3399, 1141, 3),
    (3402, 1144, 3),
    (3412, 1147, 3),
    (3414, 1150, 3),
    (3415, 1153, 3),
    (3419, 1156, 2),
    (3459, 1158, 3),
    (3465, 1161, 3),
    (3467, 1164, 2),
    (3472, 1166, 3),
    (3478, 1169, 2),
    (3480, 1171, 2),
    (3487, 1173, 3),
    (3498, 1176, 3),
    (3499, 1179, 2),
    (3501, 1181, 3),
    (3505, 1184, 3),
    (3510, 1187, 3),
    (3513, 1190, 2),
    (3525, 1192, 3),
    (3529, 1195, 3),
    (3541, 1198, 2),
    (3561, 1200, 3),
    (3571, 1203, 3),
    (3572, 1206, 2),
    (3577, 1208, 2),
    (3578, 1210, 2),
    (3590, 1212, 3),
    (3596, 1215, 2),
    (3597, 1217, 2),
    (3600, 1219, 3),
    (3602, 1222, 2),
    (3607, 1224, 3),
    (3615, 1227, 3),
    (3620, 1230, 2),
    (3621, 1232, 2),
    (3625, 1234, 3),
    (3630, 1237, 3),
    (3635, 1240, 2),
    (3636, 1242, 2),
    (3641, 1244, 2),
    (3656, 1246, 3),
    (3657, 1249, 2),
    (3661, 1251, 3),
    (3669, 1254, 2),
    (3686, 1256, 3),
    (3692, 1259, 2),
    (3697, 1261, 2),
    (3698, 1263, 2),
    (3701, 1265, 2),
    (3702, 1267, 2),
    (3708, 1269, 3),
    (3725, 1272, 3),
    (3730, 1275, 3),
    (3734, 1278, 3),
    (3755, 1281, 3),
    (3762, 1284, 3),
    (3772, 1287, 2),
    (3778, 1289, 2),
    (3782, 1291, 2),
    (3804, 1293, 3),
    (3809, 1296, 3),
    (3816, 1299, 3),
    (3820, 1302, 2),
    (3826, 1304, 2),
    (3827, 1306, 2),
    (3837, 1308, 2),
    (3838, 1310, 2),
    (3840, 1312, 2),
    (3868, 1314, 2),
    (3869, 1316, 2),
    (3870, 1318, 2),
    (3871, 1320, 2),
    (3896, 1322, 2),
    (3901, 1324, 2),
    (3916, 1326, 2),
    (3942, 1328, 2),
    (3957, 1330, 2),
    (3960, 1332, 2),
    (3963, 1334, 2),
    (3964, 1336, 2),
    (3965, 1338, 2),
    (3976, 1340, 2),
    (4006, 1342, 2),
    (4025, 1344, 2),
    (4029, 1346, 2),
    (4037, 1348, 3),
    (4039, 1351, 2),
    (4044, 1353, 2),
    (4045, 1355, 2),
    (4046, 1357, 2),
    (4052, 1359, 2),
    (4053, 1361, 2),
    (4060, 1363, 2),
    (4061, 1365, 2),
    (4074, 1367, 2),
    (4102, 1369, 2),
    (4106, 1371, 2),
    (4118, 1373, 2),
    (4120, 1375, 2),
    (4132, 1377, 2),
    (4134, 1379, 2),
    (4140, 1381, 2),
    (4146, 1383, 3),
    (4156, 1386, 2),
    (4162, 1388, 3),
    (4170, 1391, 3),
    (4178, 1394, 2),
    (4179, 1396, 2),
    (4187, 1398, 3),
    (4196, 1401, 3),
    (4202, 1404, 2),
    (4212, 1406, 3),
    (4218, 1409, 2),
    (4220, 1411, 3),
    (4228, 1414, 2),
    (4229, 1416, 2),
    (4238, 1418, 3),
    (4249, 1421, 3),
    (4256, 1424, 3),
    (4263, 1427, 3),
    (4271, 1430, 3),
    (4281, 1433, 3),
    (4289, 1436, 2),
    (4295, 1438, 3),
    (4297, 1441, 3),
    (4304, 1444, 2),
    (4310, 1446, 3),
    (4316, 1449, 3),
    (4323, 1452, 2),
    (4329, 1454, 2),
    (4330, 1456, 2),
    (4347, 1458, 2),
    (4354, 1460, 2),
    (4357, 1462, 2),
    (4392, 1464, 2),
    (4395, 1466, 2),
    (4396, 1468, 2),
    (4397, 1470, 2),
    (4429, 1472, 2),
    (4446, 1474, 2),
    (4459, 1476, 2),
    (4461, 1478, 2),
    (4462, 1480, 2),
    (4466, 1482, 2),
    (4470, 1484, 2),
    (4474, 1486, 2),
    (4479, 1488, 3),
    (4480, 1491, 2),
    (4481, 1493, 2),
    (4482, 1495, 2),
    (4484, 1497, 3),
    (4487, 1500, 2),
    (4488, 1502, 2),
    (4491, 1504, 2),
    (4493, 1506, 2),
    (4499, 1508, 2),
    (4500, 1510, 2),
    (4505, 1512, 2),
    (4507, 1514, 2),
    (4508, 1516, 2),
    (4509, 1518, 2),
    (4510, 1520, 3),
    (4511, 1523, 2),
    (4515, 1525, 3),
    (4520, 1528, 3),
    (4522, 1531, 3),
    (4537, 1534, 2),
    (4551, 1536, 2),
    (4552, 1538, 2),
    (4553, 1540, 2),
    (4555, 1542, 2),
    (4564, 1544, 2),
    (4588, 1546, 3),
    (4610, 1549, 2),
    (4618, 1551, 2),
    (4625, 1553, 2),
    (4627, 1555, 2),
    (4629, 1557, 2),
    (4634, 1559, 2),
    (4639, 1561, 2),
    (4643, 1563, 2),
    (4659, 1565, 2),
    (4661, 1567, 3),
    (4689, 1570, 2),
    (4692, 1572, 2),
    (4702, 1574, 2),
    (4703, 1576, 2),
    (4704, 1578, 2),
    (4707, 1580, 2),
    (4713, 1582, 2),
    (4714, 1584, 2),
    (4715, 1590, 2),
    (4718, 1586, 2),
    (4720, 1588, 2),
    (4724, 1592, 2),
    (4725, 1594, 2),
    (4730, 1596, 2),
    (4735, 1598, 2),
    (4736, 1600, 2),
    (4738, 1602, 2),
    (4749, 1604, 2),
    (4763, 1606, 2),
    (4764, 1608, 2),
    (4768, 1610, 2),
    (4769, 1612, 2),
    (4771, 1614, 2),
    (4774, 1616, 2),
    (4776, 1618, 2),
    (4778, 1620, 2),
    (4781, 1622, 2),
    (4784, 1624, 2),
    (4788, 1626, 2),
    (4792, 1628, 2),
    (4795, 1630, 2),
    (4798, 1632, 2),
    (4805, 1634, 2),
    (4813, 1636, 3),
    (4814, 1639, 2),
    (4819, 1641, 3),
    (4820, 1644, 2),
    (4822, 1646, 2),
    (4823, 1648, 2),
    (4826, 1650, 2),
    (4830, 1652, 2),
    (4836, 1654, 2),
    (4891, 1656, 2),
    (4909, 1658, 2),
    (4915, 1660, 2),
    (4918, 1662, 2),
    (4922, 1664, 2),
    (4923, 1666, 2),
    (4924, 1668, 2),
    (4928, 1670, 2),
    (4943, 1672, 2),
    (4947, 1674, 2),
    (4955, 1676, 2),
    (4964, 1678, 2),
    (4965, 1680, 2),
    (4969, 1682, 2),
    (4971, 1684, 2),
    (4977, 1686, 2),
    (4978, 1688, 2),
    (4982, 1696, 2),
    (4984, 1690, 2),
    (4985, 1692, 2),
    (4986, 1694, 2),
    (4987, 1698, 2),
    (5005, 1700, 2),
    (5007, 1702, 2),
    (5010, 1704, 3),
    (5015, 1707, 3),
    (5016, 1710, 2),
    (5018, 1712, 2),
    (5038, 1714, 3),
    (5051, 1717, 3),
    (5053, 1720, 2),
    (5054, 1722, 2),
    (5055, 1724, 2),
    (5064, 1726, 2),
    (5069, 1728, 2),
    (5078, 1730, 3),
    (5080, 1733, 2),
    (5090, 1735, 2),
    (5091, 1737, 2),
    (5100, 1739, 2),
    (5105, 1741, 2),
    (5124, 1743, 3),
    (5152, 1746, 3),
    (5164, 1749, 3),
    (5166, 1752, 3),
    (5180, 1755, 2),
    (5193, 1757, 3),
    (5195, 1760, 3),
    (5201, 1763, 2),
    (5212, 1765, 3),
    (5213, 1768, 2),
    (5215, 1770, 2),
    (5216, 1772, 2),
    (5224, 1774, 2),
    (5229, 1776, 3),
    (5248, 1779, 3),
    (5249, 1782, 2),
    (5265, 1784, 2),
    (5266, 1786, 2),
    (5273, 1788, 2),
    (5274, 1790, 2),
    (5318, 1792, 3),
    (5345, 1795, 2),
    (5350, 1797, 2),
    (5352, 1799, 2),
    (5358, 1801, 2),
    (5359, 1803, 2),
    (5362, 1805, 2),
    (5372, 1807, 2),
    (5408, 1809, 4),
    (5474, 1813, 2),
    (5481, 1815, 2),
    (5503, 1817, 2),
    (5505, 1819, 2),
    (5507, 1821, 2),
    (5508, 1823, 2),
    (5509, 1825, 2),
    (5510, 1827, 2),
    (5511, 1829, 5),
    (5512, 1834, 3),
    (5530, 1837, 3),
    (5577, 1840, 2),
    (5604, 1842, 2),
    (5610, 1844, 2),
    (5611, 1846, 2),
    (5612, 1848, 2),
    (5618, 1850, 2),
    (5621, 1852, 2),
    (5622, 1854, 2),
    (5634, 1856, 2),
    (5637, 1858, 2),
    (5638, 1860, 2),
    (5651, 1862, 2),
    (5654, 1864, 2),
    (5661, 1866, 3),
    (5665, 1869, 3),
    (5670, 1872, 2),
    (5672, 1874, 2),
    (5673, 1876, 2),
    (5675, 1878, 2),
    (5677, 1880, 2),
    (5680, 1882, 2),
    (5737, 1884, 2),
    (5738, 1886, 2),
    (5745, 1888, 2),
    (5749, 1890, 2),
    (5764, 1892, 3),
    (5784, 1895, 2),
    (5785, 1897, 2),
    (5787, 1899, 2),
    (5789, 1901, 2),
    (5796, 1903, 2),
    (5799, 1905, 2),
    (5801, 1907, 2),
    (5802, 1909, 2),
    (5803, 1911, 2),
    (5859, 1913, 2),
    (5886, 1915, 2),
    (5927, 1917, 3),
    (5929, 1920, 2),
    (5941, 1922, 2),
    (5942, 1924, 2),
    (5954, 1926, 2),
    (5970, 1928, 2),
    (5980, 1930, 3),
    (5981, 1933, 2),
    (5982, 1935, 2),
    (5986, 1937, 3),
    (5987, 1940, 2),
    (5988, 1942, 2),
    (5990, 1944, 3),
    (5996, 1947, 3),
    (5997, 1950, 2),
    (5998, 1952, 2),
    (6002, 1954, 3),
    (6003, 1957, 2),
    (6004, 1959, 2),
    (6006, 1961, 3),
    (6026, 1964, 2),
    (6032, 1966, 2),
    (6039, 1968, 2),
    (6057, 1970, 2),
    (6094, 1972, 2),
    (6095, 1974, 2),
    (6112, 1976, 2),
    (6115, 1978, 2),
    (6119, 1980, 2),
    (6160, 1982, 2),
    (6172, 1984, 3),
    (6186, 1987, 2),
    (6266, 1989, 3),
    (6272, 1992, 3),
    (6279, 1995, 3),
    (6309, 1998, 3),
    (6320, 2001, 3),
    (6321, 2004, 2),
    (6334, 2006, 3),
    (6342, 2009, 3),
    (6346, 2012, 3),
    (6350, 2015, 3),
    (6356, 2018, 3),
    (6363, 2021, 2),
    (6367, 2023, 2),
    (6410, 2025, 2),
    (6412, 2027, 2),
    (6415, 2029, 3),
    (6420, 2032, 3),
    (6421, 2035, 3),
    (6423, 2038, 2),
    (6472, 2040, 2),
    (6477, 2042, 3),
    (6480, 2045, 3),
    (6483, 2048, 2),
    (6485, 2050, 2),
    (6488, 2052, 3),
    (6491, 2055, 2),
    (6495, 2057, 2),
    (6499, 2059, 3),
    (6502, 2062, 2),
    (6513, 2064, 3),
    (6517, 2067, 2),
    (6520, 2069, 2),
    (6526, 2071, 2),
    (6529, 2073, 3),
    (6532, 2076, 2),
    (6538, 2078, 3),
    (6541, 2081, 2),
    (6544, 2083, 2),
    (6547, 2085, 2),
    (6550, 2087, 2),
    (6553, 2089, 3),
    (6559, 2092, 2),
    (6562, 2094, 2),
    (6570, 2096, 2),
    (6574, 2098, 2),
    (6576, 2100, 2),
    (6589, 2102, 2),
    (6594, 2104, 2),
    (6607, 2106, 2),
    (6611, 2108, 2),
    (6612, 2110, 2),
    (6613, 2112, 2),
    (6614, 2114, 2),
    (6615, 2116, 2),
    (6616, 2118, 2),
    (6617, 2120, 2),
    (6618, 2122, 2),
    (6619, 2124, 2),
    (6620, 2126, 2),
    (6621, 2128, 2),
    (6622, 2130, 2),
    (6623, 2132, 2),
    (6624, 2134, 2),
    (6625, 2136, 2),
    (6626, 2138, 2),
    (6627, 2140, 2),
    (6632, 2142, 2),
    (6637, 2144, 2),
    (6640, 2146, 2),
    (6647, 2148, 2),
    (6652, 2150, 2),
    (6689, 2152, 2),
    (6693, 2154, 3),
    (6694, 2157, 2),
    (6695, 2159, 2),
    (6699, 2161, 2),
    (6700, 2163, 3),
    (6701, 2166, 2),
    (6704, 2168, 2),
    (6708, 2170, 3),
    (6709, 2173, 2),
    (6718, 2175, 2),
    (6719, 2177, 3),
    (6720, 2180, 2),
    (6723, 2182, 2),
    (6730, 2184, 2),
    (6738, 2186, 2),
    (6740, 2188, 2),
    (6741, 2190, 2),
    (6742, 2192, 2),
    (6756, 2194, 3),
    (6768, 2197, 3),
    (6780, 2200, 2),
    (6803, 2202, 3),
    (6813, 2205, 2),
    (6817, 2207, 2),
    (6818, 2209, 2),
    (6855, 2211, 2),
    (6856, 2213, 2),
    (6892, 2215, 2),
    (6893, 2217, 2),
    (6894, 2219, 2),
    (6895, 2221, 2),
    (6896, 2223, 2),
    (6897, 2225, 2),
    (6898, 2227, 2),
    (6899, 2229, 2),
    (6900, 2231, 2),
    (6901, 2233, 2),
    (6902, 2235, 2),
    (6903, 2237, 2),
    (6904, 2239, 2),
    (6905, 2241, 2),
    (6906, 2243, 2),
    (6907, 2245, 2),
    (6908, 2247, 2),
    (6909, 2249, 2),
    (6910, 2251, 2),
    (6911, 2253, 2),
    (6912, 2255, 2),
    (6913, 2257, 2),
    (6914, 2259, 2),
    (6915, 2261, 2),
    (6916, 2263, 2),
    (6917, 2265, 2),
    (6918, 2267, 2),
    (6919, 2269, 2),
    (6920, 2271, 2),
    (6921, 2273, 2),
    (6922, 2275, 2),
    (6923, 2277, 2),
    (6924, 2279, 2),
    (6925, 2281, 2),
    (6926, 2283, 2),
    (6927, 2285, 2),
    (6928, 2287, 2),
    (6929, 2289, 2),
    (6930, 2291, 2),
    (6931, 2293, 2),
    (6932, 2295, 2),
    (6933, 2297, 2),
    (6934, 2299, 2),
    (6935, 2301, 2),
    (6936, 2303, 2),
    (6937, 2305, 2),
    (6938, 2307, 2),
    (6939, 2309, 2),
    (6940, 2311, 2),
    (6941, 2313, 2),
    (6942, 2315, 2),
    (6943, 2317, 2),
    (6944, 2319, 2),
    (6945, 2321, 2),
    (6946, 2323, 2),
    (6947, 2325, 2),
    (6948, 2327, 2),
    (6949, 2329, 2),
    (6950, 2331, 2),
    (6951, 2333, 2),
    (6952, 2335, 2),
    (6953, 2337, 2),
    (6954, 2339, 2),
    (6955, 2341, 2),
    (6956, 2343, 2),
    (6957, 2345, 2),
    (6958, 2347, 2),
    (6959, 2349, 2),
    (6960, 2351, 2),
    (6961, 2353, 2),
    (6962, 2355, 2),
    (6963, 2357, 2),
    (6964, 2359, 2),
    (6965, 2361, 2),
    (6966, 2363, 2),
    (6967, 2365, 2),
    (6968, 2367, 2),
    (6969, 2369, 2),
    (6970, 2371, 2),
    (6971, 2373, 2),
    (6972, 2375, 2),
    (6973, 2377, 2),
    (6974, 2379, 2),
    (6975, 2381, 2),
    (6976, 2383, 2),
    (6977, 2385, 2),
    (6978, 2387, 2),
    (6979, 2389, 2),
    (6980, 2391, 2),
    (6981, 2393, 2),
    (6982, 2395, 2),
    (6983, 2397, 2),
    (6984, 2399, 2),
    (6985, 2401, 2),
    (6986, 2403, 2),
    (6987, 2405, 2),
    (6988, 2407, 2),
    (6989, 2409, 2),
    (6990, 2411, 2),
    (6991, 2413, 2),
    (6992, 2415, 2),
    (6993, 2417, 2),
    (6994, 2419, 2),
    (6995, 2421, 2),
    (6996, 2423, 2),
    (6997, 2425, 2),
    (6998, 2427, 2),
    (6999, 2429, 2),
    (7000, 2431, 2),
    (7001, 2433, 2),
    (7002, 2435, 2),
    (7003, 2437, 2),
    (7004, 2439, 2),
    (7005, 2441, 2),
    (7006, 2443, 2),
    (7007, 2445, 2),
    (7008, 2447, 2),
    (7009, 2449, 2),
    (7010, 2451, 2),
    (7011, 2453, 2),
    (7023, 2455, 2),
    (7030, 2457, 2),
    (7039, 2459, 3),
    (7048, 2462, 2),
    (7053, 2464, 2),
    (7064, 2466, 2),
    (7068, 2468, 2),
    (7073, 2470, 2),
    (7076, 2472, 2),
    (7077, 2474, 2),
    (7081, 2476, 2),
    (7109, 2478, 2),
    (7116, 2480, 3),
    (7131, 2483, 2),
    (7134, 2485, 2),
    (7138, 2487, 3),
    (7142, 2490, 2),
    (7182, 2492, 2),
    (7194, 2494, 2),
    (7247, 2496, 3),
    (7249, 2499, 3),
    (7273, 2502, 2),
    (7274, 2504, 2),
    (7276, 2506, 2),
    (7277, 2508, 2),
    (7310, 2510, 2),
    (7311, 2512, 2),
    (7312, 2514, 2),
    (7313, 2516, 2),
    (7314, 2518, 2),
    (7315, 2520, 2),
    (7316, 2522, 2),
    (7317, 2524, 2),
    (7318, 2526, 2),
    (7319, 2528, 2),
    (7320, 2530, 2),
    (7321, 2532, 2),
    (7322, 2534, 2),
    (7331, 2536, 2),
    (7337, 2538, 2),
    (7346, 2540, 3),
    (7347, 2543, 2),
    (7349, 2545, 2),
    (7350, 2547, 2),
    (7356, 2549, 2),
    (7357, 2551, 2),
    (7364, 2553, 2),
    (7365, 2555, 2),
    (7371, 2557, 3),
    (7413, 2560, 3),
    (7425, 2563, 2),
    (7427, 2565, 2),
    (7428, 2567, 2),
    (7429, 2569, 2),
    (7505, 2571, 2),
    (7557, 2573, 2),
    (7561, 2575, 2),
    (7569, 2577, 2),
    (7586, 2579, 2),
    (7590, 2581, 3),
    (7592, 2584, 3),
    (7609, 2587, 2),
    (7610, 2589, 2),
    (7611, 2591, 2),
    (7624, 2593, 2),
    (7633, 2595, 2),
    (7653, 2597, 2),
    (7656, 2599, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    64, 3, 75, 3, 75, 3, 3, 77, 3, 79, 3, 82, 3, 77, 84, 64,
    3, 160, 3, 3, 190, 3, 192, 3, 77, 3, 77, 196, 3, 77, 198, 3,
    77, 3, 77, 198, 3, 202, 208, 3, 3, 210, 3, 223, 3, 224, 226, 4,
    229, 3, 3, 232, 3, 240, 3, 241, 224, 226, 4, 3, 77, 245, 247, 3,
    3, 77, 3, 82, 3, 79, 3, 77, 198, 3, 77, 268, 3, 270, 77, 3,
    270, 77, 3, 77, 3, 77, 285, 3, 288, 320, 3, 323, 3, 325, 3, 320,
    3, 323, 3, 325, 3, 3, 77, 3, 77, 268, 338, 3, 340, 3, 3, 77,
    198, 3, 77, 3, 344, 3, 77, 3, 344, 3, 77, 3, 77, 3, 77, 3,
    77, 3, 77, 3, 77, 3, 77, 3, 362, 77, 3, 77, 268, 3, 77, 3,
    77, 245, 3, 77, 3, 77, 3, 77, 3, 79, 3, 82, 3, 77, 3, 240,
    3, 240, 3, 77, 3, 401, 3, 416, 77, 3, 240, 3, 420, 77, 3, 425,
    3, 416, 77, 3, 416, 77, 3, 240, 3, 420, 77, 3, 344, 3, 202, 3,
    344, 452, 3, 452, 3, 3, 457, 3, 457, 3, 470, 3, 470, 3, 77, 285,
    3, 457, 3, 457, 3, 77, 245, 3, 509, 77, 3, 416, 77, 3, 77, 245,
    517, 518, 3, 3, 416, 77, 3, 416, 77, 3, 77, 245, 3, 288, 3, 79,
    3, 82, 555, 3, 3, 79, 3, 77, 268, 3, 362, 77, 3, 509, 77, 3,
    509, 77, 3, 77, 285, 3, 509, 77, 3, 362, 77, 3, 288, 3, 77, 285,
    3, 77, 198, 3, 77, 285, 3, 416, 77, 3, 288, 3, 288, 3, 77, 3,
    77, 245, 566, 3, 3, 416, 77, 3, 416, 77, 3, 77, 245, 3, 77, 268,
    3, 77, 245, 3, 362, 77, 3, 77, 285, 3, 77, 3, 77, 285, 3, 77,
    3, 77, 245, 3, 77, 3, 77, 198, 3, 344, 3, 344, 3, 288, 3, 79,
    3, 82, 568, 3, 3, 79, 3, 77, 268, 3, 288, 3, 362, 77, 3, 77,
    572, 3, 574, 3, 576, 3, 576, 3, 576, 3, 77, 572, 3, 77, 198, 3,
    77, 198, 3, 77, 198, 3, 77, 198, 3, 77, 198, 3, 270, 77, 3, 288,
    3, 77, 268, 3, 77, 268, 3, 77, 198, 3, 240, 3, 77, 617, 3, 416,
    77, 3, 77, 617, 452, 3, 3, 77, 617, 3, 77, 268, 3, 77, 624, 3,
    77, 624, 3, 77, 268, 3, 77, 628, 3, 77, 572, 3, 574, 3, 574, 3,
    574, 3, 344, 3, 77, 3, 77, 628, 3, 288, 3, 416, 77, 3, 240, 3,
    420, 77, 3, 425, 3, 416, 77, 3, 416, 77, 3, 240, 3, 420, 77, 3,
    344, 3, 202, 3, 344, 452, 3, 452, 3, 3, 457, 3, 457, 3, 470, 3,
    470, 3, 77, 285, 3, 457, 3, 457, 3, 77, 245, 3, 509, 77, 3, 288,
    3, 638, 3, 416, 77, 3, 77, 245, 3, 77, 245, 518, 3, 3, 416, 77,
    3, 416, 77, 3, 77, 245, 3, 77, 245, 3, 77, 268, 3, 648, 3, 77,
    285, 3, 77, 3, 77, 285, 3, 77, 3, 77, 3, 77, 3, 77, 3, 77,
    3, 77, 3, 77, 3, 77, 3, 77, 3, 77, 3, 77, 3, 77, 3, 77,
    3, 77, 3, 77, 659, 3, 77, 245, 3, 77, 659, 3, 362, 77, 3, 240,
    3, 77, 3, 77, 3, 77, 268, 3, 416, 77, 3, 240, 3, 420, 77, 3,
    416, 77, 3, 240, 3, 420, 77, 3, 416, 77, 3, 416, 77, 3, 416, 77,
    3, 77, 245, 3, 416, 77, 3, 202, 3, 344, 3, 77, 3, 416, 77, 3,
    416, 77, 3, 77, 245, 3, 77, 268, 3, 288, 3, 77, 245, 3, 416, 77,
    3, 678, 3, 678, 3, 678, 3, 678, 3, 96, 3, 344, 3, 96, 3, 344,
    3, 96, 3, 344, 3, 344, 229, 3, 3, 718, 3, 202, 3, 416, 77, 3,
    344, 3, 240, 3, 720, 3, 721, 3, 3, 678, 4, 3, 416, 77, 3, 758,
    3, 761, 3, 232, 3, 769, 3, 416, 77, 3, 416, 77, 3, 416, 77, 3,
    416, 77, 3, 416, 77, 3, 202, 3, 224, 226, 4, 3, 783, 3, 789, 3,
    344, 3, 783, 3, 789, 3, 77, 245, 3, 509, 77, 3, 509, 77, 3, 77,
    245, 3, 4, 3, 813, 3, 202, 3, 77, 3, 344, 3, 813, 3, 202, 3,
    344, 3, 344, 452, 3, 3, 574, 3, 77, 285, 3, 574, 3, 77, 285, 3,
    240, 3, 425, 3, 420, 77, 3, 416, 77, 3, 77, 245, 3, 77, 3, 77,
    3, 416, 77, 3, 77, 3, 77, 3, 270, 77, 3, 362, 77, 3, 79, 3,
    82, 3, 509, 77, 3, 77, 268, 3, 77, 3, 77, 3, 77, 3, 840, 4,
    3, 288, 3, 288, 3, 288, 3, 288, 3, 288, 3, 288, 3, 288, 3, 288,
    3, 288, 3, 288, 3, 860, 3, 288, 4, 3, 678, 3, 288, 3, 344, 3,
    416, 77, 3, 288, 3, 344, 3, 416, 77, 3, 288, 3, 344, 3, 416, 77,
    3, 288, 3, 344, 3, 416, 77, 3, 344, 3, 202, 3, 420, 77, 3, 77,
    880, 3, 344, 3, 77, 245, 3, 416, 77, 3, 891, 3, 891, 3, 416, 77,
    3, 416, 77, 3, 416, 77, 3, 344, 3, 891, 3, 416, 77, 3, 891, 3,
    416, 77, 3, 77, 3, 82, 3, 240, 3, 240, 3, 79, 3, 82, 3, 77,
    3, 288, 3, 288, 3, 288, 3, 288, 3, 288, 3, 288, 3, 77, 3, 270,
    77, 3, 288, 3, 77, 3, 288, 3, 77, 3, 288, 3, 288, 3, 288, 3,
    288, 3, 288, 3, 77, 285, 3, 288, 3, 288, 3, 288, 3, 77, 3, 240,
    3, 420, 77, 3, 344, 3, 77, 3, 344, 3, 362, 77, 3, 77, 268, 3,
    941, 3, 941, 3, 77, 245, 3, 416, 77, 3, 240, 3, 420, 77, 3, 425,
    3, 470, 3, 77, 245, 3, 77, 954, 3, 77, 245, 3, 77, 245, 3, 416,
    77, 3, 425, 3, 470, 3, 648, 3, 77, 954, 3, 79, 3, 82, 3, 240,
    3, 416, 77, 3, 240, 3, 957, 3, 240, 3, 240, 3, 963, 3, 509, 77,
    3, 77, 245, 3, 401, 3, 965, 3, 965, 3, 457, 3, 77, 285, 3, 344,
    3, 77, 245, 3, 457, 3, 77, 285, 3, 77, 245, 3, 969, 3, 202, 3,
    971, 3, 77, 3, 288, 3, 288, 3, 77, 3, 288, 3, 288, 3, 288, 3,
    79, 3, 77, 3, 77, 245, 3, 77, 3, 344, 3, 344, 3, 79, 3, 82,
    3, 963, 3, 77, 572, 3, 576, 3, 574, 3, 77, 198, 3, 77, 198, 3,
    77, 198, 3, 77, 198, 3, 77, 198, 3, 77, 198, 3, 77, 268, 3, 77,
    198, 3, 77, 268, 3, 240, 3, 416, 77, 3, 77, 617, 3, 998, 3, 77,
    954, 3, 941, 3, 77, 3, 77, 624, 3, 416, 77, 3, 344, 3, 77, 245,
    3, 77, 628, 3, 77, 572, 3, 574, 3, 77, 245, 3, 416, 77, 3, 288,
    3, 77, 285, 3, 77, 245, 3, 969, 3, 202, 3, 971, 3, 77, 245, 3,
    638, 3, 470, 3, 77, 245, 3, 1018, 3, 77, 245, 3, 416, 77, 3, 425,
    3, 470, 3, 77, 245, 3, 416, 77, 3, 425, 3, 470, 3, 648, 3, 416,
    77, 3, 202, 3, 420, 77, 3, 457, 3, 77, 285, 3, 77, 3, 288, 3,
    77, 3, 288, 3, 77, 3, 77, 659, 3, 416, 77, 3, 77, 245, 3, 77,
    245, 3, 77, 245, 3, 77, 245, 3, 344, 3, 202, 3, 240, 3, 77, 954,
    3, 77, 198, 3, 77, 245, 3, 457, 3, 969, 3, 1046, 3, 79, 3, 79,
    3, 1055, 3, 1096, 3, 1098, 3, 1100, 3, 1102, 3, 1164, 3, 1164, 3, 1164,
    3, 1164, 3, 1164, 3, 1164, 1205, 3, 1207, 3, 3, 1209, 3, 1098, 3, 77,
    3, 288, 3, 1262, 3, 77, 245, 247, 3, 3, 1266, 247, 3, 3, 1268, 3,
    77, 3, 1270, 3, 344, 247, 3, 3, 77, 3, 1291, 3, 1291, 3, 77, 3,
    82, 3, 240, 3, 240, 3, 77, 3, 270, 77, 3, 77, 3, 362, 77, 3,
    77, 245, 3, 77, 3, 82, 3, 509, 77, 3, 77, 245, 3, 77, 3, 77,
    198, 3, 77, 3, 77, 245, 3, 79, 3, 82, 3, 77, 572, 3, 77, 268,
    3, 77, 617, 3, 77, 624, 3, 77, 628, 3, 77, 245, 3, 77, 3, 77,
    659, 3, 77, 245, 3, 240, 3, 77, 198, 3, 77, 245, 3, 344, 3, 1100,
    3, 1301, 3, 965, 3, 77, 3, 344, 3, 1327, 3, 1329, 3, 1331, 3, 1333,
    3, 96, 3, 96, 3, 1369, 3, 1371, 3, 813, 3, 813, 3, 202, 3, 344,
    3, 416, 77, 3, 202, 3, 1376, 3, 344, 3, 77, 84, 3, 1100, 3, 1102,
    3, 344, 3, 240, 3, 1381, 3, 1376, 3, 1385, 3, 718, 3, 1387, 1389, 3,
    3, 416, 77, 3, 1391, 3, 77, 245, 3, 77, 245, 3, 77, 245, 3, 344,
    3, 1418, 3, 1420, 3, 1422, 3, 1424, 3, 344, 3, 77, 245, 3, 77, 3,
    344, 3, 813, 3, 202, 3, 344, 3, 344, 3, 77, 3, 344, 3, 718, 3,
    416, 77, 3, 1447, 3, 1455, 3, 1461, 3, 1463, 3, 971, 3, 1466, 3, 1266,
    3, 1470, 3, 1474, 3, 1268, 247, 3, 247, 3, 3, 1474, 3, 1268, 3, 1478,
    3, 1480, 3, 1447, 3, 1268, 3, 344, 3, 1484, 3, 344, 3, 1484, 3, 344,
    3, 761, 3, 344, 3, 240, 3, 344, 3, 344, 3, 344, 3, 344, 3, 344,
    3, 240, 1491, 3, 3, 416, 77, 3, 344, 3, 416, 77, 3, 344, 3, 1498,
    3, 1387, 3, 344, 3, 344, 3, 1291, 3, 288, 3, 344, 3, 1329, 3, 1331,
    3, 1504, 3, 761, 3, 1385, 3, 344, 3, 758, 3, 758, 3, 240, 3, 240,
    3, 957, 3, 288, 3, 344, 3, 79, 3, 1511, 3, 79, 3, 79, 3, 969,
    3, 79, 3, 1511, 1518, 3, 1520, 3, 3, 77, 198, 3, 77, 196, 3, 77,
    1522, 3, 3, 77, 196, 3, 416, 77, 3, 1527, 3, 1530, 3, 344, 3, 1461,
    3, 1455, 3, 416, 77, 3, 344, 3, 1527, 3, 1055, 3, 344, 3, 1455, 3,
    1545, 4, 3, 77, 268, 3, 77, 954, 3, 77, 268, 3, 202, 3, 420, 77,
    3, 77, 1563, 3, 457, 3, 416, 77, 3, 202, 3, 344, 3, 1565, 3, 574,
    3, 77, 285, 3, 77, 245, 247, 3, 3, 1578, 3, 1580, 3, 344, 247, 3,
    3, 77, 245, 3, 1480, 3, 1605, 3, 1607, 3, 1480, 3, 1605, 3, 1480, 3,
    344, 3, 1, 4, 5, 3, 344, 3, 344, 3, 96, 3, 1381, 3, 891, 1645,
    3, 3, 1647, 3, 1649, 3, 1, 1293, 4, 5, 3, 1293, 4, 3, 77, 245,
    3, 77, 3, 969, 3, 1673, 3, 1675, 3, 1677, 3, 344, 3, 1682, 3, 1684,
    3, 344, 3, 1682, 3, 1684, 3, 344, 3, 1690, 3, 77, 245, 3, 77, 245,
    247, 3, 3, 202, 247, 3, 3, 1607, 3, 1694, 3, 1696, 3, 288, 3, 344,
    3, 1716, 3, 1718, 3, 308, 4, 3, 969, 3, 1385, 3, 1385, 3, 1504, 3,
    761, 3, 1385, 3, 1331, 3, 1329, 3, 1333, 3, 1749, 3, 344, 3, 416, 77,
    3, 344, 3, 1765, 3, 1455, 3, 1466, 3, 1716, 3, 416, 77, 3, 1381, 3,
    813, 3, 416, 77, 3, 1381, 3, 813, 3, 416, 77, 3, 416, 77, 3, 1381,
    3, 813, 3, 416, 77, 3, 1381, 3, 813, 3, 416, 77, 3, 344, 3, 344,
    3, 344, 3, 344, 3, 1793, 3, 1795, 3, 77, 3, 240, 3, 1803, 3, 1816,
    3, 77, 268, 3, 344, 3, 77, 198, 3, 77, 198, 3, 77, 245, 3, 416,
    77, 3, 77, 245, 3, 971, 3, 77, 285, 3, 509, 77, 3, 77, 245, 3,
    77, 268, 3, 270, 77, 3, 288, 3, 1848, 1518, 3, 1520, 3, 3, 77, 198,
    3, 77, 196, 3, 77, 196, 1522, 3, 3, 77, 3, 77, 245, 3, 416, 77,
    3, 79, 3, 82, 3, 270, 77, 3, 77, 3, 77, 3, 77, 245, 3, 79,
    3, 77, 245, 3, 77, 3, 79, 3, 77, 3, 77, 198, 3, 79, 3, 77,
    268, 3, 77, 3, 77, 3, 77, 3, 77, 3, 77, 245, 3, 1875, 247, 3,
    3, 344, 3, 1877, 3, 1879, 3, 1461, 3, 1046, 3, 1461, 3, 1881, 3, 1883,
    3, 1885, 3, 1270, 3, 1887, 3, 1889, 3, 1391, 3, 1891, 3, 1891, 3, 223,
    3, 1455, 3, 1893, 3, 1895, 3, 1466, 3, 1461, 3, 1463, 3, 971, 3, 1046,
    3, 1504, 3, 1504, 208, 3, 208, 3, 3, 1903, 3, 416, 77, 208, 3, 3,
    288, 3, 202, 3, 416, 77, 3, 344, 3, 813, 3, 416, 77, 3, 288, 3,
    202, 3, 416, 77, 3, 344, 3, 813, 3, 77, 3, 1461, 3, 223, 3, 1455,
    3, 1891, 3, 77, 268, 3, 77, 198, 3, 288, 3, 77, 198, 3, 1369, 3,
    240, 3, 1371, 1207, 3, 208, 3, 3, 1803, 3, 1329, 3, 1943, 3, 1945, 3,
    1947, 3, 1949, 3, 1951, 3, 1953, 1955, 3, 3, 1957, 3, 1959, 3, 1961, 1963,
    3, 3, 1965, 3, 1967, 3, 1969, 3, 1971, 3, 1973, 3, 1975, 3, 1977, 3,
    1979, 3, 1981, 3, 1333, 3, 1331, 3, 1983, 3, 1985, 3, 1987, 3, 1989, 3,
    1991, 3, 1993, 3, 1995, 3, 1997, 3, 1999, 3, 2001, 2003, 3, 3, 2005, 3,
    2007, 3, 2009, 2011, 3, 3, 2013, 3, 2015, 3, 2017, 2019, 3, 3, 2021, 2023,
    3, 3, 2025, 2027, 3, 2029, 3, 3, 2031, 3, 2033, 3, 2035, 2037, 3, 3,
    2039, 3, 2041, 2043, 3, 3, 2045, 3, 2047, 2049, 3, 3, 1975, 3, 2051, 2053,
    3, 3, 1981, 2055, 3, 3, 2057, 3, 2059, 3, 2061, 3, 2063, 3, 2065, 3,
    2067, 2069, 3, 3, 2071, 3, 2073, 3, 2075, 3, 2077, 2079, 3, 3, 2081, 2083,
    3, 3, 2085, 3, 2087, 2089, 3, 3, 2091, 2093, 3, 3, 2095, 3, 2097, 2099,
    3, 3, 2101, 3, 2103, 2105, 3, 3, 2107, 2109, 3, 3, 2111, 2113, 3, 3,
    2115, 3, 2117, 3, 2119, 3, 2121, 3, 2123, 3, 2125, 3, 2127, 3, 2129, 3,
    2131, 3, 2133, 3, 2135, 2137, 3, 3, 2139, 2141, 3, 3, 2143, 3, 2145, 3,
    2147, 3, 2149, 3, 2151, 3, 2153, 3, 2155, 2157, 3, 3, 2159, 3, 2161, 3,
    2163, 3, 2165, 3, 2167, 3, 2169, 3, 678, 247, 3, 3, 4, 235, 3, 77,
    3, 344, 3, 2180, 3, 2180, 3, 401, 3, 2183, 3, 2180, 3, 344, 3, 77,
    3, 509, 77, 3, 969, 3, 202, 3, 77, 245, 3, 1803, 3, 1164, 3, 344,
    3, 77, 245, 3, 77, 245, 1207, 3, 1645, 3, 3, 1100, 3, 1301, 3, 1881,
    3, 1883, 3, 1885, 3, 1270, 3, 1887, 3, 1889, 3, 1391, 3, 1891, 3, 1461,
    3, 1463, 3, 971, 3, 223, 3, 1455, 3, 1046, 3, 1461, 3, 77, 245, 247,
    3, 3, 1696, 3, 1885, 3, 77, 3, 1270, 3, 344, 247, 3, 3, 270, 77,
    3, 77, 245, 3, 344, 247, 3, 3, 223, 3, 1455, 3, 1096, 3, 344, 3,
    344, 3, 344, 3, 1466, 3, 4, 2288, 3, 77, 84, 3, 2291, 3, 223, 3,
    2293, 3, 2297, 3, 2297, 3, 223, 3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"^[+\-*/<>=~!@#%^&|`?]+$"#, // [0]
    r#"[A-Z_][A-Z0-9_]*"#, // [1]
    r#"^(NOT)$"#, // [2]
    r#"[A-Z_][A-Z0-9_$]*"#, // [3]
    r#"[\p{L}_][\p{L}\p{N}_$]*"#, // [4]
    r#"^(ALL|ANALYSE|ANALYZE|AND|ANY|ARRAY|AS|ASC|ASYMMETRIC|AUTHORIZATION|BINARY|BOTH|CASE|CAST|CHECK|COLLATE|COLUMN|COMMUTATOR|CONCURRENTLY|CONNECT|CONSTRAINT|CREATE|CROSS|CURRENT_CATALOG|CURRENT_DATE|CURRENT_ROLE|CURRENT_SCHEMA|CURRENT_TIME|CURRENT_TIMESTAMP|DEFAULT|DEFERRABLE|DESC|DISTINCT|DO|ELSE|END|EXCEPT|FALSE|FETCH|FOR|FOREIGN|FREEZE|FROM|FULL|GRANT|GROUP|HASHES|HAVING|ILIKE|IN|INITIALLY|INNER|INTERSECT|INTO|IS|ISNULL|JOIN|LATERAL|LEADING|LEFT|LIKE|LIMIT|LOCALTIME|LOCALTIMESTAMP|MERGES|NATURAL|NEGATOR|NOT|NOTNULL|NULL|OFFSET|ON|ONLY|OR|ORDER|OUTER|OVERLAPS|PLACING|PRIMARY|REFERENCES|RETURNING|RIGHT|RIGHTARG|SELECT|SESSION_USER|SIMILAR|SOME|SYMMETRIC|TABLESAMPLE|THEN|TO|TRAILING|TRUE|UNION|UNIQUE|USING|VARIADIC|VERBOSE|WHEN|WHERE|WINDOW|WITH)$"#, // [5]
    r#"[A-Z_][A-Z0-9_$]*|"[^"]*""#, // [6]
    r#"[!<>=~@#%^&|`?+\-*/]+"#, // [7]
    r#"(?i)'[0-9a-f]*'"#, // [8]
    r#"'([^']|'')*'"#, // [9]
    r#"'[^0-9A-Fa-f'+\-\s)]'"#, // [10]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 9, 10, 11, 12, 13, 14, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
    26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
    36, 24, 39, 36, 12, 13, 19, 20, 27, 11,
    25, 32, 34, 16, 51, 52, 52, 51, 56, 10,
    59, 61, 18, 14, 15, 17, 21, 22, 23, 26,
    27, 28, 29, 30, 31, 33, 35, 14, 15, 66,
    21, 22, 67, 68, 59, 69, 70, 71, 72, 73,
    74, 12, 27, 81, 86, 88, 90, 91, 94, 96,
    10, 97, 98, 99, 100, 101, 102, 103, 104, 105,
    39, 106, 107, 108, 109, 110, 111, 112, 113, 114,
    115, 116, 117, 118, 119, 120, 99, 12, 19, 25,
    122, 36, 122, 97, 21, 127, 101, 33, 104, 132,
    117, 10, 97, 98, 99, 100, 101, 102, 103, 105,
    39, 106, 107, 108, 109, 110, 111, 112, 113, 114,
    115, 116, 117, 118, 119, 120, 14, 15, 17, 21,
    22, 135, 23, 26, 27, 28, 29, 30, 31, 33,
    35, 135, 118, 11, 27, 102, 140, 98, 100, 103,
    105, 106, 107, 108, 110, 109, 111, 112, 113, 114,
    115, 116, 119, 120, 162, 31, 35, 29, 17, 23,
    26, 28, 30, 22, 14, 15, 175, 72, 74, 70,
    66, 67, 68, 69, 71, 73, 187, 188, 187, 188,
    193, 199, 204, 205, 206, 206, 205, 211, 212, 213,
    214, 217, 218, 219, 220, 222, 228, 236, 243, 248,
    249, 250, 254, 255, 256, 257, 256, 255, 261, 263,
    265, 257, 248, 271, 272, 255, 256, 273, 257, 274,
    248, 271, 272, 274, 274, 248, 271, 272, 278, 279,
    271, 282, 272, 286, 90, 88, 286, 88, 286, 290,
    81, 292, 214, 293, 294, 295, 278, 296, 297, 298,
    299, 300, 301, 279, 302, 206, 303, 304, 305, 306,
    307, 308, 96, 309, 310, 313, 314, 316, 273, 10,
    99, 100, 101, 103, 109, 111, 114, 115, 116, 117,
    66, 68, 59, 69, 72, 322, 327, 99, 100, 101,
    103, 109, 111, 114, 115, 116, 117, 330, 332, 334,
    334, 332, 322, 25, 122, 213, 341, 255, 256, 257,
    342, 341, 341, 257, 347, 297, 350, 352, 347, 297,
    347, 297, 352, 355, 357, 342, 360, 213, 341, 257,
    116, 364, 213, 341, 7, 220, 365, 366, 367, 13,
    368, 16, 17, 20, 22, 369, 370, 23, 26, 27,
    28, 52, 31, 371, 372, 34, 35, 7, 366, 367,
    13, 368, 16, 20, 370, 27, 28, 52, 371, 34,
    375, 366, 367, 368, 20, 370, 370, 371, 382, 383,
    384, 385, 386, 383, 384, 385, 386, 22, 31, 35,
    365, 17, 23, 26, 369, 394, 395, 394, 395, 399,
    372, 213, 250, 403, 404, 341, 405, 406, 407, 408,
    347, 409, 410, 255, 411, 273, 257, 342, 413, 417,
    352, 350, 290, 421, 347, 297, 422, 109, 423, 213,
    341, 347, 273, 426, 257, 34, 257, 34, 428, 429,
    261, 364, 430, 431, 263, 432, 433, 265, 434, 122,
    341, 347, 257, 436, 421, 438, 439, 438, 439, 443,
    228, 444, 445, 446, 447, 448, 408, 449, 426, 28,
    450, 426, 257, 426, 455, 459, 460, 14, 461, 462,
    462, 460, 461, 467, 352, 350, 469, 16, 422, 423,
    350, 422, 423, 472, 347, 472, 474, 477, 479, 404,
    405, 405, 404, 438, 411, 411, 484, 404, 405, 406,
    408, 406, 408, 488, 490, 492, 403, 495, 403, 496,
    496, 498, 499, 498, 499, 407, 408, 407, 409, 290,
    504, 505, 399, 507, 504, 505, 250, 410, 410, 213,
    341, 257, 304, 306, 307, 308, 309, 310, 515, 213,
    250, 403, 404, 341, 405, 406, 407, 408, 347, 409,
    410, 255, 256, 411, 273, 257, 342, 519, 520, 14,
    521, 522, 347, 523, 273, 524, 525, 492, 257, 526,
    527, 528, 529, 519, 524, 527, 519, 532, 524, 527,
    521, 526, 529, 521, 526, 529, 522, 347, 522, 14,
    492, 539, 540, 539, 540, 523, 544, 545, 546, 546,
    544, 545, 520, 525, 528, 519, 520, 553, 14, 521,
    522, 408, 347, 255, 523, 256, 273, 524, 525, 492,
    257, 526, 527, 528, 529, 553, 408, 553, 250, 557,
    553, 408, 256, 273, 257, 557, 560, 449, 563, 250,
    403, 255, 273, 257, 273, 257, 250, 403, 255, 256,
    273, 257, 256, 88, 399, 274, 88, 399, 274, 14,
    273, 492, 257, 553, 14, 408, 255, 256, 273, 492,
    257, 570, 213, 341, 255, 256, 257, 31, 72, 577,
    271, 578, 579, 580, 407, 581, 582, 583, 584, 585,
    586, 587, 588, 589, 590, 591, 355, 588, 591, 591,
    588, 578, 583, 578, 583, 579, 584, 579, 584, 407,
    585, 585, 581, 586, 581, 586, 587, 590, 590, 587,
    577, 582, 577, 582, 580, 589, 580, 589, 297, 304,
    306, 307, 308, 309, 310, 608, 255, 256, 446, 292,
    214, 448, 408, 448, 447, 408, 447, 615, 450, 444,
    445, 408, 445, 622, 228, 444, 445, 446, 447, 448,
    408, 449, 255, 256, 426, 28, 257, 450, 27, 455,
    626, 213, 271, 404, 341, 405, 255, 629, 256, 257,
    630, 629, 630, 636, 213, 250, 403, 404, 341, 405,
    406, 407, 408, 347, 409, 255, 411, 273, 257, 342,
    213, 105, 81, 219, 639, 219, 639, 639, 228, 88,
    213, 250, 403, 404, 341, 405, 406, 407, 408, 347,
    409, 255, 256, 411, 273, 257, 342, 557, 644, 256,
    257, 290, 330, 646, 90, 61, 274, 644, 651, 653,
    213, 250, 341, 255, 256, 257, 655, 657, 553, 408,
    256, 661, 213, 250, 341, 256, 663, 219, 665, 665,
    250, 255, 256, 273, 257, 667, 669, 670, 669, 670,
    81, 673, 674, 673, 673, 674, 677, 680, 681, 667,
    254, 292, 214, 682, 683, 684, 685, 217, 292, 682,
    681, 685, 683, 684, 293, 698, 294, 702, 703, 704,
    705, 706, 707, 708, 709, 710, 712, 713, 714, 296,
    724, 725, 726, 727, 728, 729, 730, 681, 667, 254,
    292, 214, 682, 683, 286, 684, 677, 417, 731, 369,
    732, 685, 217, 733, 734, 735, 736, 737, 417, 217,
    677, 732, 730, 667, 214, 683, 286, 684, 731, 369,
    734, 735, 736, 737, 733, 743, 744, 745, 749, 732,
    752, 279, 278, 755, 756, 757, 81, 111, 646, 762,
    763, 764, 765, 766, 767, 22, 347, 81, 100, 103,
    111, 116, 646, 274, 445, 382, 762, 771, 771, 774,
    776, 777, 781, 785, 787, 791, 793, 794, 795, 796,
    797, 798, 799, 800, 801, 802, 803, 804, 805, 806,
    352, 290, 421, 347, 297, 422, 109, 423, 815, 423,
    817, 820, 422, 823, 352, 417, 350, 290, 421, 347,
    297, 422, 109, 423, 832, 835, 837, 7, 220, 365,
    366, 413, 350, 367, 12, 13, 368, 360, 16, 17,
    507, 20, 22, 127, 23, 570, 25, 26, 484, 27,
    28, 52, 455, 626, 31, 32, 371, 114, 34, 35,
    31, 35, 7, 366, 367, 13, 368, 16, 20, 22,
    127, 570, 25, 484, 28, 52, 455, 626, 32, 371,
    114, 34, 220, 23, 26, 731, 842, 843, 844, 845,
    730, 683, 734, 737, 736, 735, 862, 865, 866, 867,
    504, 869, 870, 871, 874, 505, 116, 876, 878, 81,
    162, 881, 882, 883, 884, 885, 81, 293, 887, 888,
    304, 306, 307, 308, 309, 310, 81, 293, 887, 304,
    306, 307, 308, 309, 310, 887, 81, 293, 304, 306,
    307, 308, 309, 310, 81, 293, 888, 81, 274, 893,
    894, 895, 896, 897, 898, 175, 899, 900, 297, 901,
    899, 893, 900, 901, 895, 898, 81, 254, 896, 897,
    894, 81, 293, 888, 304, 306, 307, 308, 309, 310,
    729, 911, 893, 297, 274, 729, 911, 893, 297, 911,
    895, 406, 900, 901, 81, 293, 887, 914, 304, 306,
    307, 308, 309, 310, 914, 81, 293, 914, 304, 306,
    307, 308, 309, 310, 916, 917, 274, 495, 920, 923,
    81, 90, 924, 925, 926, 927, 928, 929, 927, 925,
    926, 928, 934, 935, 934, 935, 924, 929, 248, 271,
    894, 272, 925, 926, 927, 255, 32, 386, 290, 304,
    306, 307, 308, 309, 310, 925, 926, 927, 925, 926,
    352, 350, 347, 297, 944, 946, 352, 350, 290, 421,
    347, 297, 352, 350, 951, 112, 113, 958, 228, 724,
    519, 520, 14, 521, 20, 522, 347, 523, 961, 524,
    525, 492, 257, 526, 527, 528, 372, 529, 962, 274,
    966, 81, 100, 101, 103, 111, 31, 116, 646, 274,
    81, 31, 81, 101, 973, 845, 17, 975, 976, 977,
    978, 979, 23, 332, 980, 977, 980, 977, 980, 979,
    17, 976, 23, 332, 976, 17, 23, 975, 978, 989,
    990, 989, 990, 10, 100, 103, 111, 116, 218, 577,
    271, 578, 579, 580, 61, 407, 581, 582, 583, 584,
    585, 586, 587, 588, 589, 590, 25, 591, 993, 122,
    355, 274, 218, 577, 271, 578, 579, 580, 61, 407,
    581, 582, 583, 584, 585, 586, 587, 588, 589, 590,
    25, 591, 993, 122, 355, 993, 228, 444, 445, 446,
    447, 448, 408, 449, 450, 916, 999, 1000, 1001, 999,
    1000, 1001, 1006, 1007, 112, 113, 499, 1006, 1007, 112,
    113, 1006, 1007, 1006, 1007, 81, 162, 32, 399, 274,
    495, 100, 341, 1011, 1011, 81, 409, 560, 1013, 1015,
    1016, 162, 560, 32, 399, 274, 495, 1019, 1020, 1021,
    1021, 1020, 1019, 885, 385, 219, 665, 1025, 1025, 100,
    103, 115, 116, 472, 330, 90, 474, 347, 1028, 781,
    472, 474, 347, 472, 474, 1028, 1030, 1031, 1031, 1030,
    1035, 488, 1036, 1036, 1039, 1021, 1039, 261, 263, 265,
    122, 1044, 1047, 1048, 1049, 1049, 1051, 1051, 428, 429,
    430, 431, 1056, 1057, 1058, 803, 865, 1057, 1061, 1062,
    1063, 1064, 1065, 1066, 1067, 1068, 1069, 1070, 1071, 1072,
    1073, 1074, 1075, 1076, 1077, 1078, 1079, 1080, 1081, 1082,
    1083, 1084, 1085, 1086, 1087, 1088, 1089, 1090, 1091, 1092,
    1093, 1094, 1095, 295, 296, 298, 299, 300, 301, 727,
    728, 744, 749, 752, 756, 757, 767, 795, 796, 801,
    802, 1103, 1104, 1105, 1106, 1107, 1108, 1109, 1110, 1111,
    1112, 1113, 1114, 1115, 1116, 1117, 1118, 1119, 1120, 1121,
    1122, 1123, 1124, 1125, 1126, 1127, 1085, 1089, 1128, 1129,
    1130, 1131, 1132, 1133, 1134, 371, 1135, 1136, 1137, 1138,
    1139, 1140, 1141, 1142, 1133, 1116, 1110, 1111, 1112, 1114,
    1107, 1108, 1128, 1105, 1155, 1134, 1129, 1130, 1131, 1132,
    1106, 1104, 1124, 1104, 1124, 1123, 757, 795, 796, 371,
    1140, 757, 795, 796, 1140, 1171, 1140, 749, 752, 756,
    802, 1119, 1120, 1126, 1085, 1089, 1085, 1119, 1120, 1126,
    1089, 801, 1109, 1121, 1122, 1109, 1121, 1122, 1135, 1138,
    1138, 1135, 1139, 1142, 1117, 1118, 1117, 1118, 1113, 1115,
    1125, 1136, 1137, 1103, 1127, 1141, 294, 206, 295, 1212,
    1213, 1214, 1215, 1216, 1217, 1218, 1219, 1220, 1221, 1222,
    1223, 1224, 1225, 1226, 1227, 1228, 1229, 1230, 1231, 1232,
    1233, 1234, 1235, 295, 296, 298, 299, 300, 301, 304,
    306, 307, 308, 309, 310, 298, 299, 296, 300, 301,
    301, 300, 1242, 1213, 1244, 1245, 1246, 1247, 1248, 1249,
    1214, 1250, 1251, 1252, 1255, 1256, 1257, 1258, 81, 292,
    214, 293, 294, 295, 278, 296, 297, 298, 299, 300,
    301, 279, 302, 206, 303, 304, 305, 306, 307, 308,
    96, 309, 310, 5, 5, 1263, 1264, 1271, 1274, 1275,
    1276, 1277, 1279, 1280, 1281, 1282, 10, 1284, 69, 112,
    113, 1284, 1286, 1287, 1288, 1289, 20, 304, 306, 307,
    308, 309, 310, 1292, 305, 1296, 1297, 1298, 1299, 1302,
    1303, 1305, 1306, 1307, 469, 1015, 1308, 1309, 1310, 1311,
    1312, 669, 670, 765, 917, 175, 1313, 1314, 1315, 673,
    1316, 669, 670, 765, 917, 1313, 1314, 1315, 673, 1316,
    1313, 1316, 1315, 1314, 1117, 371, 1142, 1323, 1323, 81,
    669, 670, 673, 669, 670, 673, 1337, 1013, 347, 1338,
    61, 347, 278, 1117, 1342, 1343, 1344, 1345, 1346, 1347,
    297, 279, 1348, 1350, 1352, 726, 347, 292, 214, 347,
    1358, 1359, 1359, 1358, 730, 681, 667, 254, 292, 214,
    682, 683, 286, 684, 417, 731, 369, 685, 217, 734,
    735, 736, 737, 292, 214, 1359, 1365, 1373, 295, 278,
    296, 297, 298, 299, 300, 301, 279, 304, 306, 307,
    308, 96, 309, 310, 1392, 1395, 639, 1397, 1397, 488,
    525, 515, 274, 1400, 1401, 81, 250, 669, 670, 724,
    774, 403, 837, 843, 885, 99, 1252, 100, 1277, 1282,
    874, 341, 787, 101, 1312, 187, 1402, 103, 1403, 1404,
    1405, 1406, 1407, 1255, 1408, 629, 1409, 273, 188, 1410,
    492, 111, 257, 1411, 450, 115, 1412, 116, 118, 1413,
    646, 274, 1414, 1415, 1416, 1414, 1415, 1416, 81, 250,
    669, 670, 724, 774, 403, 837, 843, 885, 99, 1252,
    100, 1277, 1282, 874, 341, 787, 101, 1312, 187, 1402,
    103, 1403, 1404, 1405, 1406, 1407, 1255, 1408, 629, 1409,
    273, 188, 1410, 492, 111, 257, 1411, 450, 115, 1412,
    116, 118, 1413, 646, 274, 1425, 1426, 1428, 1429, 823,
    408, 199, 116, 563, 630, 1431, 352, 290, 421, 347,
    297, 1432, 1433, 1021, 525, 1436, 1429, 1437, 1437, 316,
    193, 408, 1400, 1442, 1433, 1444, 1445, 1307, 1395, 330,
    193, 1448, 1449, 282, 1450, 1451, 222, 1452, 1453, 876,
    962, 274, 1448, 1452, 962, 1307, 1449, 1450, 1453, 1451,
    1468, 1473, 1477, 1482, 302, 755, 1486, 1487, 330, 90,
    1492, 1495, 1496, 293, 661, 228, 724, 519, 520, 14,
    521, 20, 522, 347, 523, 961, 524, 525, 492, 257,
    526, 527, 528, 372, 529, 962, 961, 81, 103, 961,
    111, 116, 646, 274, 81, 111, 646, 274, 61, 920,
    1506, 302, 1506, 1365, 286, 290, 1061, 1062, 1063, 1064,
    1065, 1066, 1516, 885, 274, 1523, 1525, 1529, 1395, 1448,
    282, 222, 1452, 962, 1534, 1395, 282, 222, 1452, 962,
    1535, 1536, 1537, 1538, 1539, 1540, 1219, 1541, 1542, 1543,
    1544, 1547, 1548, 1495, 1496, 1337, 1402, 1307, 282, 1557,
    1016, 702, 1276, 639, 1561, 1561, 966, 399, 274, 1566,
    1567, 1568, 1569, 1570, 1571, 869, 1572, 1574, 636, 122,
    81, 290, 100, 103, 111, 116, 646, 274, 1581, 1582,
    1583, 1584, 1449, 304, 306, 307, 308, 96, 309, 310,
    1212, 1213, 1214, 1215, 1216, 1217, 1218, 1219, 1220, 1221,
    1222, 1223, 1224, 1225, 1226, 1227, 1228, 1229, 1230, 1231,
    1232, 1233, 1234, 1235, 304, 306, 307, 308, 309, 310,
    1213, 1219, 1226, 1227, 1229, 1235, 1235, 1227, 1226, 1229,
    112, 113, 499, 1342, 1348, 1343, 1344, 1345, 1346, 1347,
    1596, 1597, 1598, 1599, 162, 399, 504, 870, 871, 870,
    871, 1616, 294, 94, 1618, 661, 1618, 1621, 1623, 1624,
    1625, 309, 1627, 1486, 1628, 1629, 1630, 1337, 1013, 1338,
    1338, 10, 842, 881, 1258, 421, 443, 1567, 455, 459,
    86, 1451, 1640, 1641, 1067, 1068, 1069, 1070, 1403, 294,
    302, 303, 1404, 432, 433, 1651, 1405, 7, 1308, 488,
    199, 1655, 1656, 1657, 1658, 1659, 1660, 1661, 1662, 1664,
    1665, 1666, 1667, 1668, 1669, 1670, 1406, 1680, 100, 874,
    116, 1686, 1688, 874, 103, 1697, 1700, 1224, 1701, 1450,
    1702, 1703, 1704, 1705, 1706, 1707, 1071, 308, 1072, 1073,
    1074, 1075, 1076, 1077, 1078, 1079, 1080, 1081, 1082, 1083,
    1084, 1710, 1365, 286, 1713, 1713, 1719, 1721, 1722, 1724,
    1343, 1344, 1345, 1346, 1726, 1727, 1728, 1729, 1730, 1731,
    1732, 81, 100, 103, 111, 116, 646, 81, 111, 1736,
    1737, 1738, 1343, 1344, 1345, 1346, 1347, 730, 1407, 1747,
    1751, 1752, 1753, 1754, 1755, 1756, 1757, 1758, 1759, 1760,
    1761, 1762, 304, 306, 307, 308, 309, 310, 5, 702,
    1276, 399, 330, 1433, 1448, 282, 1452, 1021, 525, 1492,
    962, 274, 1737, 222, 1767, 1768, 1770, 1772, 1773, 1774,
    1775, 1777, 1778, 1779, 1782, 1783, 682, 304, 306, 307,
    308, 309, 310, 1787, 1789, 1086, 1087, 1088, 1090, 1091,
    1092, 1093, 1790, 1792, 1797, 1800, 1801, 81, 100, 103,
    1406, 111, 116, 646, 274, 1804, 1805, 1806, 1807, 1808,
    1809, 1818, 10, 680, 1820, 1820, 1228, 313, 314, 307,
    309, 307, 308, 309, 304, 310, 1826, 306, 308, 306,
    1829, 1830, 1832, 1833, 1408, 1835, 1836, 1837, 334, 408,
    332, 257, 504, 1779, 1841, 1841, 290, 297, 1845, 1409,
    651, 32, 673, 12, 507, 27, 636, 31, 12, 636,
    1849, 1850, 1851, 1852, 1853, 1854, 1856, 1857, 1858, 1860,
    1861, 1410, 1862, 1863, 1864, 1865, 725, 636, 1869, 1870,
    1871, 1872, 220, 413, 12, 13, 368, 16, 17, 20,
    91, 22, 370, 23, 570, 25, 26, 27, 28, 626,
    31, 32, 34, 35, 10, 820, 86, 1307, 90, 1449,
    1557, 282, 1450, 222, 1271, 1453, 876, 1697, 1899, 1900,
    1901, 477, 479, 1453, 1307, 1449, 1007, 1856, 298, 1909,
    290, 1007, 304, 306, 307, 308, 309, 310, 1910, 1911,
    1411, 1916, 1917, 1922, 1923, 1924, 1925, 1926, 1927, 1928,
    1929, 1930, 1931, 1932, 1933, 1934, 1935, 1936, 1937, 1938,
    1940, 1941, 724, 843, 787, 1410, 450, 1413, 1412, 2170,
    2172, 2173, 2174, 2175, 2176, 2177, 2178, 352, 469, 16,
    422, 423, 472, 474, 408, 347, 422, 423, 112, 113,
    2188, 2188, 2190, 2191, 2192, 2193, 299, 1094, 2197, 2198,
    2199, 2200, 2201, 1487, 2202, 2203, 2203, 2205, 2205, 2209,
    2210, 1095, 2211, 884, 426, 2216, 2217, 2219, 2220, 2225,
    2226, 2229, 2230, 2231, 2232, 2233, 2234, 2235, 81, 669,
    670, 899, 504, 673, 669, 670, 899, 504, 673, 669,
    670, 1281, 899, 504, 1566, 523, 1808, 674, 115, 673,
    2240, 2241, 2242, 2243, 2244, 2246, 1233, 1234, 803, 865,
    1057, 1063, 1064, 1065, 1066, 1067, 1068, 1069, 1070, 1071,
    1072, 1073, 1074, 1075, 1076, 1077, 1078, 1079, 1080, 1081,
    1082, 1083, 1084, 1085, 1086, 1087, 1088, 1089, 1090, 1091,
    1092, 1093, 1094, 1095, 81, 803, 865, 1057, 1061, 1062,
    1063, 1064, 1065, 1066, 1067, 1068, 1069, 1070, 1071, 1072,
    1073, 1074, 1075, 1076, 1077, 1078, 1079, 1080, 1081, 1082,
    1083, 1084, 1085, 1086, 1087, 1088, 1089, 1090, 1091, 1092,
    1093, 1094, 1095, 96, 81, 96, 1061, 1062, 330, 193,
    1448, 282, 162, 222, 1772, 1452, 1271, 962, 2286, 2287,
    222, 560, 776, 1007, 2301, 2302, 2303, 2304, 2305, 2306,
    2307, 2308, 2309, 2310, 2311, 2312, 2313, 2314, 2315, 2316,
    2317,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 27, token_types_start: 31, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 31, raw_values_count: 1, token_types_start: 32, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 44, raw_values_count: 2, token_types_start: 46, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 46, raw_values_count: 1, token_types_start: 47, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 47, raw_values_count: 1, token_types_start: 48, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 49, raw_values_count: 1, token_types_start: 50, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 50, raw_values_count: 1, token_types_start: 51, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 53, raw_values_count: 14, token_types_start: 67, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 67, raw_values_count: 14, token_types_start: 81, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 81, raw_values_count: 2, token_types_start: 83, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 83, raw_values_count: 1, token_types_start: 84, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 89, raw_values_count: 0, token_types_start: 89, token_types_count: 1 }, // [36]
    SimpleHintData { raw_values_start: 90, raw_values_count: 26, token_types_start: 116, token_types_count: 0 }, // [37]
    SimpleHintData { raw_values_start: 116, raw_values_count: 1, token_types_start: 117, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 117, raw_values_count: 5, token_types_start: 122, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 122, raw_values_count: 1, token_types_start: 123, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 131, raw_values_count: 25, token_types_start: 156, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 156, raw_values_count: 15, token_types_start: 171, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 171, raw_values_count: 1, token_types_start: 172, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 172, raw_values_count: 1, token_types_start: 173, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 173, raw_values_count: 2, token_types_start: 175, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 175, raw_values_count: 1, token_types_start: 176, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 176, raw_values_count: 1, token_types_start: 177, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 177, raw_values_count: 1, token_types_start: 178, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 178, raw_values_count: 1, token_types_start: 179, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 179, raw_values_count: 1, token_types_start: 180, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 180, raw_values_count: 1, token_types_start: 181, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 181, raw_values_count: 1, token_types_start: 182, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 182, raw_values_count: 1, token_types_start: 183, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 184, raw_values_count: 1, token_types_start: 185, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 185, raw_values_count: 1, token_types_start: 186, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 186, raw_values_count: 1, token_types_start: 187, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 187, raw_values_count: 1, token_types_start: 188, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 188, raw_values_count: 1, token_types_start: 189, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 216, raw_values_count: 2, token_types_start: 218, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 220, raw_values_count: 1, token_types_start: 221, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 221, raw_values_count: 1, token_types_start: 222, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 223, raw_values_count: 2, token_types_start: 225, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 225, raw_values_count: 1, token_types_start: 226, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 226, raw_values_count: 1, token_types_start: 227, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 233, raw_values_count: 1, token_types_start: 234, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 234, raw_values_count: 1, token_types_start: 235, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 237, raw_values_count: 1, token_types_start: 238, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 238, raw_values_count: 1, token_types_start: 239, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 239, raw_values_count: 1, token_types_start: 240, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 243, raw_values_count: 3, token_types_start: 246, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 246, raw_values_count: 1, token_types_start: 247, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 247, raw_values_count: 1, token_types_start: 248, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 248, raw_values_count: 1, token_types_start: 249, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 249, raw_values_count: 1, token_types_start: 250, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 250, raw_values_count: 1, token_types_start: 251, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 251, raw_values_count: 1, token_types_start: 252, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 252, raw_values_count: 8, token_types_start: 260, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 260, raw_values_count: 4, token_types_start: 264, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 264, raw_values_count: 1, token_types_start: 265, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 265, raw_values_count: 3, token_types_start: 268, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 268, raw_values_count: 2, token_types_start: 270, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 270, raw_values_count: 1, token_types_start: 271, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 271, raw_values_count: 1, token_types_start: 272, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 272, raw_values_count: 1, token_types_start: 273, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 273, raw_values_count: 3, token_types_start: 276, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 276, raw_values_count: 2, token_types_start: 278, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 278, raw_values_count: 1, token_types_start: 279, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 279, raw_values_count: 1, token_types_start: 280, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 280, raw_values_count: 17, token_types_start: 297, token_types_count: 8 }, // [139]
    SimpleHintData { raw_values_start: 305, raw_values_count: 0, token_types_start: 305, token_types_count: 2 }, // [140]
    SimpleHintData { raw_values_start: 307, raw_values_count: 1, token_types_start: 308, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 308, raw_values_count: 1, token_types_start: 309, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 309, raw_values_count: 11, token_types_start: 320, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 320, raw_values_count: 6, token_types_start: 326, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 326, raw_values_count: 1, token_types_start: 327, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 327, raw_values_count: 10, token_types_start: 337, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 337, raw_values_count: 1, token_types_start: 338, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 338, raw_values_count: 1, token_types_start: 339, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 339, raw_values_count: 1, token_types_start: 340, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 340, raw_values_count: 2, token_types_start: 342, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 342, raw_values_count: 1, token_types_start: 343, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 343, raw_values_count: 2, token_types_start: 345, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 345, raw_values_count: 6, token_types_start: 351, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 351, raw_values_count: 1, token_types_start: 352, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 352, raw_values_count: 2, token_types_start: 354, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 354, raw_values_count: 1, token_types_start: 355, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 355, raw_values_count: 1, token_types_start: 356, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 356, raw_values_count: 1, token_types_start: 357, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 357, raw_values_count: 3, token_types_start: 360, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 360, raw_values_count: 2, token_types_start: 362, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 362, raw_values_count: 1, token_types_start: 363, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 363, raw_values_count: 1, token_types_start: 364, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 364, raw_values_count: 1, token_types_start: 365, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 365, raw_values_count: 1, token_types_start: 366, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 366, raw_values_count: 1, token_types_start: 367, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 367, raw_values_count: 4, token_types_start: 371, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 371, raw_values_count: 1, token_types_start: 372, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 372, raw_values_count: 2, token_types_start: 374, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 374, raw_values_count: 23, token_types_start: 397, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 397, raw_values_count: 13, token_types_start: 410, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 410, raw_values_count: 1, token_types_start: 411, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 411, raw_values_count: 1, token_types_start: 412, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 412, raw_values_count: 1, token_types_start: 413, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 413, raw_values_count: 1, token_types_start: 414, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 414, raw_values_count: 2, token_types_start: 416, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 416, raw_values_count: 1, token_types_start: 417, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 417, raw_values_count: 1, token_types_start: 418, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 418, raw_values_count: 1, token_types_start: 419, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 419, raw_values_count: 4, token_types_start: 423, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 423, raw_values_count: 1, token_types_start: 424, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 424, raw_values_count: 1, token_types_start: 425, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 425, raw_values_count: 1, token_types_start: 426, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 426, raw_values_count: 1, token_types_start: 427, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 427, raw_values_count: 3, token_types_start: 430, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 430, raw_values_count: 1, token_types_start: 431, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 431, raw_values_count: 3, token_types_start: 434, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 434, raw_values_count: 1, token_types_start: 435, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 435, raw_values_count: 2, token_types_start: 437, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 437, raw_values_count: 1, token_types_start: 438, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 438, raw_values_count: 1, token_types_start: 439, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 439, raw_values_count: 1, token_types_start: 440, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 440, raw_values_count: 1, token_types_start: 441, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 441, raw_values_count: 17, token_types_start: 458, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 458, raw_values_count: 1, token_types_start: 459, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 459, raw_values_count: 1, token_types_start: 460, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 460, raw_values_count: 9, token_types_start: 469, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 469, raw_values_count: 7, token_types_start: 476, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 476, raw_values_count: 2, token_types_start: 478, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 478, raw_values_count: 12, token_types_start: 490, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 490, raw_values_count: 3, token_types_start: 493, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 493, raw_values_count: 1, token_types_start: 494, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 494, raw_values_count: 1, token_types_start: 495, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 495, raw_values_count: 2, token_types_start: 497, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 497, raw_values_count: 1, token_types_start: 498, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 498, raw_values_count: 1, token_types_start: 499, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 499, raw_values_count: 1, token_types_start: 500, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 500, raw_values_count: 11, token_types_start: 511, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 511, raw_values_count: 2, token_types_start: 513, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 513, raw_values_count: 1, token_types_start: 514, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 514, raw_values_count: 1, token_types_start: 515, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 515, raw_values_count: 1, token_types_start: 516, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 516, raw_values_count: 4, token_types_start: 520, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 520, raw_values_count: 1, token_types_start: 521, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 521, raw_values_count: 1, token_types_start: 522, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 522, raw_values_count: 1, token_types_start: 523, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 523, raw_values_count: 1, token_types_start: 524, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 524, raw_values_count: 6, token_types_start: 530, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 530, raw_values_count: 3, token_types_start: 533, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 533, raw_values_count: 2, token_types_start: 535, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 535, raw_values_count: 1, token_types_start: 536, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 536, raw_values_count: 1, token_types_start: 537, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 537, raw_values_count: 1, token_types_start: 538, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 538, raw_values_count: 1, token_types_start: 539, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 539, raw_values_count: 2, token_types_start: 541, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 541, raw_values_count: 1, token_types_start: 542, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 542, raw_values_count: 1, token_types_start: 543, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 543, raw_values_count: 2, token_types_start: 545, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 545, raw_values_count: 1, token_types_start: 546, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 546, raw_values_count: 1, token_types_start: 547, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 547, raw_values_count: 4, token_types_start: 551, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 551, raw_values_count: 1, token_types_start: 552, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 552, raw_values_count: 1, token_types_start: 553, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 553, raw_values_count: 1, token_types_start: 554, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 554, raw_values_count: 1, token_types_start: 555, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 555, raw_values_count: 1, token_types_start: 556, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 556, raw_values_count: 1, token_types_start: 557, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 557, raw_values_count: 1, token_types_start: 558, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 558, raw_values_count: 2, token_types_start: 560, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 560, raw_values_count: 1, token_types_start: 561, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 561, raw_values_count: 2, token_types_start: 563, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 563, raw_values_count: 1, token_types_start: 564, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 564, raw_values_count: 1, token_types_start: 565, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 565, raw_values_count: 2, token_types_start: 567, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 567, raw_values_count: 1, token_types_start: 568, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 568, raw_values_count: 1, token_types_start: 569, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 569, raw_values_count: 4, token_types_start: 573, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 573, raw_values_count: 1, token_types_start: 574, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 574, raw_values_count: 1, token_types_start: 575, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 575, raw_values_count: 1, token_types_start: 576, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 576, raw_values_count: 2, token_types_start: 578, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 578, raw_values_count: 1, token_types_start: 579, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 579, raw_values_count: 3, token_types_start: 582, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 582, raw_values_count: 0, token_types_start: 582, token_types_count: 6 }, // [253]
    SimpleHintData { raw_values_start: 588, raw_values_count: 1, token_types_start: 589, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 589, raw_values_count: 18, token_types_start: 607, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 607, raw_values_count: 16, token_types_start: 623, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 623, raw_values_count: 3, token_types_start: 626, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 626, raw_values_count: 1, token_types_start: 627, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 627, raw_values_count: 1, token_types_start: 628, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 628, raw_values_count: 1, token_types_start: 629, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 629, raw_values_count: 1, token_types_start: 630, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 630, raw_values_count: 3, token_types_start: 633, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 633, raw_values_count: 1, token_types_start: 634, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 634, raw_values_count: 1, token_types_start: 635, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 635, raw_values_count: 1, token_types_start: 636, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 636, raw_values_count: 2, token_types_start: 638, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 638, raw_values_count: 1, token_types_start: 639, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 639, raw_values_count: 2, token_types_start: 641, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 641, raw_values_count: 2, token_types_start: 643, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 643, raw_values_count: 1, token_types_start: 644, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 644, raw_values_count: 1, token_types_start: 645, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 645, raw_values_count: 1, token_types_start: 646, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 646, raw_values_count: 3, token_types_start: 649, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 649, raw_values_count: 1, token_types_start: 650, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 650, raw_values_count: 1, token_types_start: 651, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 651, raw_values_count: 1, token_types_start: 652, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 652, raw_values_count: 1, token_types_start: 653, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 653, raw_values_count: 1, token_types_start: 654, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 654, raw_values_count: 1, token_types_start: 655, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 655, raw_values_count: 20, token_types_start: 675, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 675, raw_values_count: 2, token_types_start: 677, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 677, raw_values_count: 1, token_types_start: 678, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 678, raw_values_count: 7, token_types_start: 685, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 685, raw_values_count: 1, token_types_start: 686, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 686, raw_values_count: 1, token_types_start: 687, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 687, raw_values_count: 1, token_types_start: 688, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 688, raw_values_count: 1, token_types_start: 689, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 689, raw_values_count: 5, token_types_start: 694, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 694, raw_values_count: 2, token_types_start: 696, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 696, raw_values_count: 6, token_types_start: 702, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 702, raw_values_count: 4, token_types_start: 706, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 706, raw_values_count: 3, token_types_start: 709, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 709, raw_values_count: 4, token_types_start: 713, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 713, raw_values_count: 8, token_types_start: 721, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 721, raw_values_count: 1, token_types_start: 722, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 722, raw_values_count: 5, token_types_start: 727, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 727, raw_values_count: 2, token_types_start: 729, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 729, raw_values_count: 18, token_types_start: 747, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 747, raw_values_count: 2, token_types_start: 749, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 751, raw_values_count: 2, token_types_start: 753, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 753, raw_values_count: 1, token_types_start: 754, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 754, raw_values_count: 1, token_types_start: 755, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 755, raw_values_count: 2, token_types_start: 757, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 757, raw_values_count: 1, token_types_start: 758, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 758, raw_values_count: 1, token_types_start: 759, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 759, raw_values_count: 2, token_types_start: 761, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 761, raw_values_count: 1, token_types_start: 762, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 762, raw_values_count: 2, token_types_start: 764, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 764, raw_values_count: 1, token_types_start: 765, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 765, raw_values_count: 1, token_types_start: 766, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 766, raw_values_count: 2, token_types_start: 768, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 768, raw_values_count: 1, token_types_start: 769, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 769, raw_values_count: 1, token_types_start: 770, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 770, raw_values_count: 2, token_types_start: 772, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 772, raw_values_count: 1, token_types_start: 773, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 773, raw_values_count: 1, token_types_start: 774, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 774, raw_values_count: 2, token_types_start: 776, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 776, raw_values_count: 1, token_types_start: 777, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 777, raw_values_count: 1, token_types_start: 778, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 778, raw_values_count: 1, token_types_start: 779, token_types_count: 6 }, // [322]
    SimpleHintData { raw_values_start: 785, raw_values_count: 1, token_types_start: 786, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 786, raw_values_count: 2, token_types_start: 788, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 788, raw_values_count: 1, token_types_start: 789, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 789, raw_values_count: 2, token_types_start: 791, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 791, raw_values_count: 2, token_types_start: 793, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 793, raw_values_count: 1, token_types_start: 794, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 794, raw_values_count: 2, token_types_start: 796, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 796, raw_values_count: 1, token_types_start: 797, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 797, raw_values_count: 1, token_types_start: 798, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 798, raw_values_count: 1, token_types_start: 799, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 799, raw_values_count: 1, token_types_start: 800, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 800, raw_values_count: 2, token_types_start: 802, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 802, raw_values_count: 1, token_types_start: 803, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 803, raw_values_count: 1, token_types_start: 804, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 804, raw_values_count: 14, token_types_start: 818, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 818, raw_values_count: 2, token_types_start: 820, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 820, raw_values_count: 1, token_types_start: 821, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 821, raw_values_count: 10, token_types_start: 831, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 831, raw_values_count: 1, token_types_start: 832, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 832, raw_values_count: 1, token_types_start: 833, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 833, raw_values_count: 1, token_types_start: 834, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 834, raw_values_count: 16, token_types_start: 850, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 850, raw_values_count: 2, token_types_start: 852, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 852, raw_values_count: 3, token_types_start: 855, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 855, raw_values_count: 2, token_types_start: 857, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 857, raw_values_count: 1, token_types_start: 858, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 858, raw_values_count: 2, token_types_start: 860, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 860, raw_values_count: 17, token_types_start: 877, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 877, raw_values_count: 4, token_types_start: 881, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 881, raw_values_count: 2, token_types_start: 883, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 883, raw_values_count: 1, token_types_start: 884, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 884, raw_values_count: 3, token_types_start: 887, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 887, raw_values_count: 1, token_types_start: 888, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 888, raw_values_count: 1, token_types_start: 889, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 889, raw_values_count: 1, token_types_start: 890, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 890, raw_values_count: 6, token_types_start: 896, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 896, raw_values_count: 1, token_types_start: 897, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 897, raw_values_count: 1, token_types_start: 898, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 898, raw_values_count: 3, token_types_start: 901, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 901, raw_values_count: 1, token_types_start: 902, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 902, raw_values_count: 4, token_types_start: 906, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 906, raw_values_count: 1, token_types_start: 907, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 907, raw_values_count: 2, token_types_start: 909, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 909, raw_values_count: 1, token_types_start: 910, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 910, raw_values_count: 5, token_types_start: 915, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 915, raw_values_count: 1, token_types_start: 916, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 916, raw_values_count: 1, token_types_start: 917, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 917, raw_values_count: 1, token_types_start: 918, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 918, raw_values_count: 2, token_types_start: 920, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 920, raw_values_count: 2, token_types_start: 922, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 922, raw_values_count: 2, token_types_start: 924, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 924, raw_values_count: 1, token_types_start: 925, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 925, raw_values_count: 1, token_types_start: 926, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 926, raw_values_count: 1, token_types_start: 927, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 927, raw_values_count: 1, token_types_start: 928, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 928, raw_values_count: 10, token_types_start: 938, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 938, raw_values_count: 1, token_types_start: 939, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 939, raw_values_count: 1, token_types_start: 940, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 940, raw_values_count: 1, token_types_start: 941, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 941, raw_values_count: 1, token_types_start: 942, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 942, raw_values_count: 1, token_types_start: 943, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 943, raw_values_count: 1, token_types_start: 944, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 944, raw_values_count: 1, token_types_start: 945, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 945, raw_values_count: 1, token_types_start: 946, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 946, raw_values_count: 1, token_types_start: 947, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 947, raw_values_count: 1, token_types_start: 948, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 948, raw_values_count: 1, token_types_start: 949, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 949, raw_values_count: 1, token_types_start: 950, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 950, raw_values_count: 1, token_types_start: 951, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 951, raw_values_count: 1, token_types_start: 952, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 952, raw_values_count: 1, token_types_start: 953, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 953, raw_values_count: 1, token_types_start: 954, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 954, raw_values_count: 1, token_types_start: 955, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 955, raw_values_count: 1, token_types_start: 956, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 956, raw_values_count: 1, token_types_start: 957, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 957, raw_values_count: 1, token_types_start: 958, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 958, raw_values_count: 1, token_types_start: 959, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 959, raw_values_count: 1, token_types_start: 960, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 960, raw_values_count: 1, token_types_start: 961, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 961, raw_values_count: 1, token_types_start: 962, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 962, raw_values_count: 1, token_types_start: 963, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 963, raw_values_count: 1, token_types_start: 964, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 964, raw_values_count: 1, token_types_start: 965, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 965, raw_values_count: 1, token_types_start: 966, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 966, raw_values_count: 17, token_types_start: 983, token_types_count: 5 }, // [407]
    SimpleHintData { raw_values_start: 988, raw_values_count: 2, token_types_start: 990, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 990, raw_values_count: 2, token_types_start: 992, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 992, raw_values_count: 8, token_types_start: 1000, token_types_count: 4 }, // [410]
    SimpleHintData { raw_values_start: 1004, raw_values_count: 0, token_types_start: 1004, token_types_count: 1 }, // [411]
    SimpleHintData { raw_values_start: 1005, raw_values_count: 1, token_types_start: 1006, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 1006, raw_values_count: 1, token_types_start: 1007, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 1007, raw_values_count: 1, token_types_start: 1008, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 1008, raw_values_count: 1, token_types_start: 1009, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 1009, raw_values_count: 1, token_types_start: 1010, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 1010, raw_values_count: 1, token_types_start: 1011, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 1011, raw_values_count: 1, token_types_start: 1012, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 1012, raw_values_count: 1, token_types_start: 1013, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 1013, raw_values_count: 1, token_types_start: 1014, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 1014, raw_values_count: 1, token_types_start: 1015, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 1015, raw_values_count: 1, token_types_start: 1016, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 1016, raw_values_count: 3, token_types_start: 1019, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 1019, raw_values_count: 1, token_types_start: 1020, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 1020, raw_values_count: 1, token_types_start: 1021, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 1021, raw_values_count: 1, token_types_start: 1022, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 1022, raw_values_count: 1, token_types_start: 1023, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 1023, raw_values_count: 1, token_types_start: 1024, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 1024, raw_values_count: 1, token_types_start: 1025, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 1025, raw_values_count: 2, token_types_start: 1027, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 1027, raw_values_count: 7, token_types_start: 1034, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 1034, raw_values_count: 2, token_types_start: 1036, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 1036, raw_values_count: 2, token_types_start: 1038, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 1038, raw_values_count: 1, token_types_start: 1039, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 1039, raw_values_count: 1, token_types_start: 1040, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 1040, raw_values_count: 1, token_types_start: 1041, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 1041, raw_values_count: 1, token_types_start: 1042, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 1042, raw_values_count: 1, token_types_start: 1043, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 1043, raw_values_count: 1, token_types_start: 1044, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 1044, raw_values_count: 1, token_types_start: 1045, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 1045, raw_values_count: 1, token_types_start: 1046, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 1046, raw_values_count: 1, token_types_start: 1047, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 1047, raw_values_count: 1, token_types_start: 1048, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 1048, raw_values_count: 1, token_types_start: 1049, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 1049, raw_values_count: 1, token_types_start: 1050, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 1050, raw_values_count: 1, token_types_start: 1051, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 1051, raw_values_count: 1, token_types_start: 1052, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 1052, raw_values_count: 1, token_types_start: 1053, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 1053, raw_values_count: 1, token_types_start: 1054, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 1054, raw_values_count: 1, token_types_start: 1055, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 1055, raw_values_count: 1, token_types_start: 1056, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 1056, raw_values_count: 1, token_types_start: 1057, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 1057, raw_values_count: 1, token_types_start: 1058, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 1058, raw_values_count: 1, token_types_start: 1059, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 1059, raw_values_count: 1, token_types_start: 1060, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 1060, raw_values_count: 8, token_types_start: 1068, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 1068, raw_values_count: 1, token_types_start: 1069, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 1069, raw_values_count: 1, token_types_start: 1070, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 1070, raw_values_count: 1, token_types_start: 1071, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 1071, raw_values_count: 1, token_types_start: 1072, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 1072, raw_values_count: 1, token_types_start: 1073, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 1073, raw_values_count: 1, token_types_start: 1074, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 1074, raw_values_count: 10, token_types_start: 1084, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 1084, raw_values_count: 1, token_types_start: 1085, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 1085, raw_values_count: 1, token_types_start: 1086, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 1086, raw_values_count: 1, token_types_start: 1087, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 1087, raw_values_count: 33, token_types_start: 1120, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 1120, raw_values_count: 2, token_types_start: 1122, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 20, token_types_start: 1142, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 1142, raw_values_count: 3, token_types_start: 1145, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 1145, raw_values_count: 1, token_types_start: 1146, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 1146, raw_values_count: 1, token_types_start: 1147, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 1, token_types_start: 1148, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 1148, raw_values_count: 1, token_types_start: 1149, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 1149, raw_values_count: 1, token_types_start: 1150, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 1150, raw_values_count: 2, token_types_start: 1152, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 1152, raw_values_count: 0, token_types_start: 1152, token_types_count: 1 }, // [477]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 0, token_types_start: 1153, token_types_count: 1 }, // [478]
    SimpleHintData { raw_values_start: 1154, raw_values_count: 0, token_types_start: 1154, token_types_count: 1 }, // [479]
    SimpleHintData { raw_values_start: 1155, raw_values_count: 0, token_types_start: 1155, token_types_count: 1 }, // [480]
    SimpleHintData { raw_values_start: 1156, raw_values_count: 1, token_types_start: 1157, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 1157, raw_values_count: 1, token_types_start: 1158, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 1158, raw_values_count: 1, token_types_start: 1159, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 1159, raw_values_count: 1, token_types_start: 1160, token_types_count: 0 }, // [484]
    SimpleHintData { raw_values_start: 1160, raw_values_count: 4, token_types_start: 1164, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 1164, raw_values_count: 1, token_types_start: 1165, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 1165, raw_values_count: 2, token_types_start: 1167, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 1167, raw_values_count: 1, token_types_start: 1168, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 1168, raw_values_count: 1, token_types_start: 1169, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 1169, raw_values_count: 2, token_types_start: 1171, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 1171, raw_values_count: 1, token_types_start: 1172, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 1, token_types_start: 1173, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 1173, raw_values_count: 1, token_types_start: 1174, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 1174, raw_values_count: 1, token_types_start: 1175, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 1175, raw_values_count: 1, token_types_start: 1176, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 1176, raw_values_count: 4, token_types_start: 1180, token_types_count: 6 }, // [496]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 3, token_types_start: 1189, token_types_count: 6 }, // [497]
    SimpleHintData { raw_values_start: 1195, raw_values_count: 1, token_types_start: 1196, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 1196, raw_values_count: 2, token_types_start: 1198, token_types_count: 6 }, // [499]
    SimpleHintData { raw_values_start: 1204, raw_values_count: 2, token_types_start: 1206, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 1206, raw_values_count: 1, token_types_start: 1207, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 1207, raw_values_count: 2, token_types_start: 1209, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 1209, raw_values_count: 11, token_types_start: 1220, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 1220, raw_values_count: 1, token_types_start: 1221, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 1221, raw_values_count: 1, token_types_start: 1222, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 1222, raw_values_count: 1, token_types_start: 1223, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 1223, raw_values_count: 1, token_types_start: 1224, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 1224, raw_values_count: 1, token_types_start: 1225, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 1225, raw_values_count: 1, token_types_start: 1226, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 1226, raw_values_count: 2, token_types_start: 1228, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 1228, raw_values_count: 1, token_types_start: 1229, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 1229, raw_values_count: 1, token_types_start: 1230, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 1230, raw_values_count: 1, token_types_start: 1231, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 1231, raw_values_count: 3, token_types_start: 1234, token_types_count: 6 }, // [514]
    SimpleHintData { raw_values_start: 1240, raw_values_count: 5, token_types_start: 1245, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 1245, raw_values_count: 4, token_types_start: 1249, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 1249, raw_values_count: 1, token_types_start: 1250, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 1250, raw_values_count: 4, token_types_start: 1254, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 4, token_types_start: 1258, token_types_count: 6 }, // [519]
    SimpleHintData { raw_values_start: 1264, raw_values_count: 1, token_types_start: 1265, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 1265, raw_values_count: 3, token_types_start: 1268, token_types_count: 6 }, // [521]
    SimpleHintData { raw_values_start: 1274, raw_values_count: 1, token_types_start: 1275, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 1275, raw_values_count: 1, token_types_start: 1276, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 1276, raw_values_count: 2, token_types_start: 1278, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 1278, raw_values_count: 1, token_types_start: 1279, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 1279, raw_values_count: 1, token_types_start: 1280, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 1280, raw_values_count: 2, token_types_start: 1282, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 1282, raw_values_count: 6, token_types_start: 1288, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 1288, raw_values_count: 1, token_types_start: 1289, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 1289, raw_values_count: 1, token_types_start: 1290, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 1290, raw_values_count: 1, token_types_start: 1291, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 1291, raw_values_count: 1, token_types_start: 1292, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 1292, raw_values_count: 2, token_types_start: 1294, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 1294, raw_values_count: 1, token_types_start: 1295, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 1295, raw_values_count: 1, token_types_start: 1296, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 1296, raw_values_count: 1, token_types_start: 1297, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 1297, raw_values_count: 1, token_types_start: 1298, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 1298, raw_values_count: 10, token_types_start: 1308, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 1308, raw_values_count: 1, token_types_start: 1309, token_types_count: 6 }, // [539]
    SimpleHintData { raw_values_start: 1315, raw_values_count: 3, token_types_start: 1318, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 2, token_types_start: 1320, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 4, token_types_start: 1324, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 1324, raw_values_count: 1, token_types_start: 1325, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 1325, raw_values_count: 1, token_types_start: 1326, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 1326, raw_values_count: 6, token_types_start: 1332, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 1332, raw_values_count: 2, token_types_start: 1334, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 1334, raw_values_count: 1, token_types_start: 1335, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 1335, raw_values_count: 2, token_types_start: 1337, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 1337, raw_values_count: 1, token_types_start: 1338, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 1338, raw_values_count: 22, token_types_start: 1360, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 1360, raw_values_count: 1, token_types_start: 1361, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 1361, raw_values_count: 9, token_types_start: 1370, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 1370, raw_values_count: 2, token_types_start: 1372, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 1372, raw_values_count: 2, token_types_start: 1374, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 1374, raw_values_count: 1, token_types_start: 1375, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 1375, raw_values_count: 10, token_types_start: 1385, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 1385, raw_values_count: 2, token_types_start: 1387, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 1387, raw_values_count: 1, token_types_start: 1388, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 1388, raw_values_count: 1, token_types_start: 1389, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 1389, raw_values_count: 1, token_types_start: 1390, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 1390, raw_values_count: 4, token_types_start: 1394, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 1394, raw_values_count: 1, token_types_start: 1395, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 1395, raw_values_count: 2, token_types_start: 1397, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 1397, raw_values_count: 1, token_types_start: 1398, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 1398, raw_values_count: 1, token_types_start: 1399, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 1399, raw_values_count: 2, token_types_start: 1401, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 1401, raw_values_count: 1, token_types_start: 1402, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 1402, raw_values_count: 1, token_types_start: 1403, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 1403, raw_values_count: 5, token_types_start: 1408, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 1408, raw_values_count: 24, token_types_start: 1432, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 1432, raw_values_count: 23, token_types_start: 1455, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 1455, raw_values_count: 1, token_types_start: 1456, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 1456, raw_values_count: 9, token_types_start: 1465, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 4, token_types_start: 1469, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 1469, raw_values_count: 1, token_types_start: 1470, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 1470, raw_values_count: 1, token_types_start: 1471, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 1471, raw_values_count: 1, token_types_start: 1472, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 1472, raw_values_count: 5, token_types_start: 1477, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 1477, raw_values_count: 4, token_types_start: 1481, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 1481, raw_values_count: 2, token_types_start: 1483, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 1483, raw_values_count: 1, token_types_start: 1484, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 1484, raw_values_count: 1, token_types_start: 1485, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 1485, raw_values_count: 6, token_types_start: 1491, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 3, token_types_start: 1494, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1494, raw_values_count: 1, token_types_start: 1495, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1495, raw_values_count: 3, token_types_start: 1498, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 1, token_types_start: 1499, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1499, raw_values_count: 2, token_types_start: 1501, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 1501, raw_values_count: 6, token_types_start: 1507, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1507, raw_values_count: 3, token_types_start: 1510, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 1, token_types_start: 1511, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1511, raw_values_count: 1, token_types_start: 1512, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1512, raw_values_count: 1, token_types_start: 1513, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1513, raw_values_count: 2, token_types_start: 1515, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1515, raw_values_count: 3, token_types_start: 1518, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1518, raw_values_count: 1, token_types_start: 1519, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1519, raw_values_count: 4, token_types_start: 1523, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1523, raw_values_count: 7, token_types_start: 1530, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1530, raw_values_count: 3, token_types_start: 1533, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1533, raw_values_count: 2, token_types_start: 1535, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 2, token_types_start: 1538, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1538, raw_values_count: 1, token_types_start: 1539, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1539, raw_values_count: 1, token_types_start: 1540, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1540, raw_values_count: 1, token_types_start: 1541, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1541, raw_values_count: 2, token_types_start: 1543, token_types_count: 0 }, // [606]
    SimpleHintData { raw_values_start: 1543, raw_values_count: 1, token_types_start: 1544, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1544, raw_values_count: 2, token_types_start: 1546, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1546, raw_values_count: 1, token_types_start: 1547, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1547, raw_values_count: 4, token_types_start: 1551, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1551, raw_values_count: 1, token_types_start: 1552, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1552, raw_values_count: 1, token_types_start: 1553, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1553, raw_values_count: 1, token_types_start: 1554, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1554, raw_values_count: 1, token_types_start: 1555, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1555, raw_values_count: 2, token_types_start: 1557, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1557, raw_values_count: 1, token_types_start: 1558, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1558, raw_values_count: 1, token_types_start: 1559, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1559, raw_values_count: 1, token_types_start: 1560, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1560, raw_values_count: 1, token_types_start: 1561, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1561, raw_values_count: 1, token_types_start: 1562, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1562, raw_values_count: 1, token_types_start: 1563, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1563, raw_values_count: 1, token_types_start: 1564, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1564, raw_values_count: 1, token_types_start: 1565, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1565, raw_values_count: 38, token_types_start: 1603, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1603, raw_values_count: 6, token_types_start: 1609, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1609, raw_values_count: 55, token_types_start: 1664, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1664, raw_values_count: 1, token_types_start: 1665, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1665, raw_values_count: 1, token_types_start: 1666, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1666, raw_values_count: 1, token_types_start: 1667, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1667, raw_values_count: 1, token_types_start: 1668, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1668, raw_values_count: 1, token_types_start: 1669, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1669, raw_values_count: 1, token_types_start: 1670, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1670, raw_values_count: 1, token_types_start: 1671, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 1, token_types_start: 1672, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1672, raw_values_count: 1, token_types_start: 1673, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 1, token_types_start: 1674, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1674, raw_values_count: 1, token_types_start: 1675, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1675, raw_values_count: 1, token_types_start: 1676, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1676, raw_values_count: 1, token_types_start: 1677, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1677, raw_values_count: 1, token_types_start: 1678, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1678, raw_values_count: 1, token_types_start: 1679, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1679, raw_values_count: 1, token_types_start: 1680, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1680, raw_values_count: 1, token_types_start: 1681, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1681, raw_values_count: 2, token_types_start: 1683, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1683, raw_values_count: 1, token_types_start: 1684, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1684, raw_values_count: 1, token_types_start: 1685, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1685, raw_values_count: 1, token_types_start: 1686, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1686, raw_values_count: 5, token_types_start: 1691, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 4, token_types_start: 1695, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1695, raw_values_count: 1, token_types_start: 1696, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1696, raw_values_count: 1, token_types_start: 1697, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1697, raw_values_count: 2, token_types_start: 1699, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1699, raw_values_count: 7, token_types_start: 1706, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1706, raw_values_count: 1, token_types_start: 1707, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1707, raw_values_count: 1, token_types_start: 1708, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1708, raw_values_count: 1, token_types_start: 1709, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1709, raw_values_count: 1, token_types_start: 1710, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1710, raw_values_count: 1, token_types_start: 1711, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1711, raw_values_count: 4, token_types_start: 1715, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1715, raw_values_count: 1, token_types_start: 1716, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1716, raw_values_count: 1, token_types_start: 1717, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1717, raw_values_count: 1, token_types_start: 1718, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1718, raw_values_count: 2, token_types_start: 1720, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1720, raw_values_count: 1, token_types_start: 1721, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1721, raw_values_count: 1, token_types_start: 1722, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1722, raw_values_count: 1, token_types_start: 1723, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1723, raw_values_count: 1, token_types_start: 1724, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1724, raw_values_count: 2, token_types_start: 1726, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1726, raw_values_count: 1, token_types_start: 1727, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1727, raw_values_count: 1, token_types_start: 1728, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1728, raw_values_count: 1, token_types_start: 1729, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1729, raw_values_count: 1, token_types_start: 1730, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1730, raw_values_count: 1, token_types_start: 1731, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1731, raw_values_count: 1, token_types_start: 1732, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1732, raw_values_count: 1, token_types_start: 1733, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1733, raw_values_count: 1, token_types_start: 1734, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1734, raw_values_count: 1, token_types_start: 1735, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1735, raw_values_count: 1, token_types_start: 1736, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1736, raw_values_count: 2, token_types_start: 1738, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1738, raw_values_count: 1, token_types_start: 1739, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1739, raw_values_count: 0, token_types_start: 1739, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1739, raw_values_count: 24, token_types_start: 1763, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1763, raw_values_count: 6, token_types_start: 1769, token_types_count: 6 }, // [683]
    SimpleHintData { raw_values_start: 1775, raw_values_count: 2, token_types_start: 1777, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1777, raw_values_count: 3, token_types_start: 1780, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1780, raw_values_count: 1, token_types_start: 1781, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1781, raw_values_count: 1, token_types_start: 1782, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1782, raw_values_count: 1, token_types_start: 1783, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1783, raw_values_count: 1, token_types_start: 1784, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1784, raw_values_count: 1, token_types_start: 1785, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1785, raw_values_count: 1, token_types_start: 1786, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1786, raw_values_count: 1, token_types_start: 1787, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1787, raw_values_count: 1, token_types_start: 1788, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1788, raw_values_count: 1, token_types_start: 1789, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1789, raw_values_count: 1, token_types_start: 1790, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1790, raw_values_count: 1, token_types_start: 1791, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1791, raw_values_count: 1, token_types_start: 1792, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1792, raw_values_count: 1, token_types_start: 1793, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1793, raw_values_count: 1, token_types_start: 1794, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1794, raw_values_count: 1, token_types_start: 1795, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1795, raw_values_count: 1, token_types_start: 1796, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1796, raw_values_count: 1, token_types_start: 1797, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1797, raw_values_count: 1, token_types_start: 1798, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1798, raw_values_count: 17, token_types_start: 1815, token_types_count: 9 }, // [704]
    SimpleHintData { raw_values_start: 1824, raw_values_count: 0, token_types_start: 1824, token_types_count: 1 }, // [705]
    SimpleHintData { raw_values_start: 1825, raw_values_count: 1, token_types_start: 1826, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1826, raw_values_count: 1, token_types_start: 1827, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1827, raw_values_count: 1, token_types_start: 1828, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1828, raw_values_count: 1, token_types_start: 1829, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1829, raw_values_count: 1, token_types_start: 1830, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1830, raw_values_count: 1, token_types_start: 1831, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1831, raw_values_count: 1, token_types_start: 1832, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1832, raw_values_count: 1, token_types_start: 1833, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1833, raw_values_count: 1, token_types_start: 1834, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1834, raw_values_count: 1, token_types_start: 1835, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1835, raw_values_count: 1, token_types_start: 1836, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1836, raw_values_count: 5, token_types_start: 1841, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1841, raw_values_count: 1, token_types_start: 1842, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1842, raw_values_count: 1, token_types_start: 1843, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1843, raw_values_count: 1, token_types_start: 1844, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1844, raw_values_count: 1, token_types_start: 1845, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1845, raw_values_count: 1, token_types_start: 1846, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1846, raw_values_count: 1, token_types_start: 1847, token_types_count: 6 }, // [723]
    SimpleHintData { raw_values_start: 1853, raw_values_count: 1, token_types_start: 1854, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1854, raw_values_count: 0, token_types_start: 1854, token_types_count: 1 }, // [725]
    SimpleHintData { raw_values_start: 1855, raw_values_count: 1, token_types_start: 1856, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1856, raw_values_count: 1, token_types_start: 1857, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1857, raw_values_count: 1, token_types_start: 1858, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1858, raw_values_count: 1, token_types_start: 1859, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1859, raw_values_count: 1, token_types_start: 1860, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1860, raw_values_count: 1, token_types_start: 1861, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1861, raw_values_count: 1, token_types_start: 1862, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1862, raw_values_count: 1, token_types_start: 1863, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1863, raw_values_count: 1, token_types_start: 1864, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1864, raw_values_count: 1, token_types_start: 1865, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1865, raw_values_count: 1, token_types_start: 1866, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1866, raw_values_count: 1, token_types_start: 1867, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1867, raw_values_count: 1, token_types_start: 1868, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1868, raw_values_count: 1, token_types_start: 1869, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1869, raw_values_count: 1, token_types_start: 1870, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1870, raw_values_count: 1, token_types_start: 1871, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1871, raw_values_count: 10, token_types_start: 1881, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1881, raw_values_count: 9, token_types_start: 1890, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1890, raw_values_count: 1, token_types_start: 1891, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1891, raw_values_count: 1, token_types_start: 1892, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1892, raw_values_count: 1, token_types_start: 1893, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1893, raw_values_count: 1, token_types_start: 1894, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1894, raw_values_count: 4, token_types_start: 1898, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1898, raw_values_count: 1, token_types_start: 1899, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1899, raw_values_count: 4, token_types_start: 1903, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1903, raw_values_count: 3, token_types_start: 1906, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1906, raw_values_count: 4, token_types_start: 1910, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1910, raw_values_count: 2, token_types_start: 1912, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1912, raw_values_count: 11, token_types_start: 1923, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1923, raw_values_count: 1, token_types_start: 1924, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1924, raw_values_count: 1, token_types_start: 1925, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1925, raw_values_count: 2, token_types_start: 1927, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1927, raw_values_count: 5, token_types_start: 1932, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1932, raw_values_count: 1, token_types_start: 1933, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1933, raw_values_count: 1, token_types_start: 1934, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1934, raw_values_count: 15, token_types_start: 1949, token_types_count: 4 }, // [761]
    SimpleHintData { raw_values_start: 1953, raw_values_count: 3, token_types_start: 1956, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1956, raw_values_count: 1, token_types_start: 1957, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1957, raw_values_count: 0, token_types_start: 1957, token_types_count: 1 }, // [764]
    SimpleHintData { raw_values_start: 1958, raw_values_count: 9, token_types_start: 1967, token_types_count: 7 }, // [765]
    SimpleHintData { raw_values_start: 1974, raw_values_count: 1, token_types_start: 1975, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1975, raw_values_count: 1, token_types_start: 1976, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1976, raw_values_count: 2, token_types_start: 1978, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1978, raw_values_count: 1, token_types_start: 1979, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1979, raw_values_count: 2, token_types_start: 1981, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1981, raw_values_count: 2, token_types_start: 1983, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1983, raw_values_count: 1, token_types_start: 1984, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1984, raw_values_count: 1, token_types_start: 1985, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1985, raw_values_count: 47, token_types_start: 2032, token_types_count: 3 }, // [774]
    SimpleHintData { raw_values_start: 2035, raw_values_count: 0, token_types_start: 2035, token_types_count: 1 }, // [775]
    SimpleHintData { raw_values_start: 2036, raw_values_count: 0, token_types_start: 2036, token_types_count: 1 }, // [776]
    SimpleHintData { raw_values_start: 2037, raw_values_count: 0, token_types_start: 2037, token_types_count: 1 }, // [777]
    SimpleHintData { raw_values_start: 2038, raw_values_count: 47, token_types_start: 2085, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 2085, raw_values_count: 1, token_types_start: 2086, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 2086, raw_values_count: 1, token_types_start: 2087, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 2087, raw_values_count: 1, token_types_start: 2088, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 2088, raw_values_count: 1, token_types_start: 2089, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 2089, raw_values_count: 4, token_types_start: 2093, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 2093, raw_values_count: 2, token_types_start: 2095, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 2095, raw_values_count: 1, token_types_start: 2096, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 2096, raw_values_count: 5, token_types_start: 2101, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 2101, raw_values_count: 1, token_types_start: 2102, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 2102, raw_values_count: 3, token_types_start: 2105, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 2105, raw_values_count: 1, token_types_start: 2106, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 2106, raw_values_count: 2, token_types_start: 2108, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 2108, raw_values_count: 1, token_types_start: 2109, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 2109, raw_values_count: 4, token_types_start: 2113, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 2113, raw_values_count: 1, token_types_start: 2114, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 2114, raw_values_count: 1, token_types_start: 2115, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 2115, raw_values_count: 1, token_types_start: 2116, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 2116, raw_values_count: 1, token_types_start: 2117, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 2117, raw_values_count: 15, token_types_start: 2132, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 2132, raw_values_count: 1, token_types_start: 2133, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 2133, raw_values_count: 1, token_types_start: 2134, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 2134, raw_values_count: 1, token_types_start: 2135, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 2135, raw_values_count: 4, token_types_start: 2139, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 2139, raw_values_count: 1, token_types_start: 2140, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 2140, raw_values_count: 1, token_types_start: 2141, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 2141, raw_values_count: 1, token_types_start: 2142, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 2142, raw_values_count: 1, token_types_start: 2143, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 2143, raw_values_count: 1, token_types_start: 2144, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 2144, raw_values_count: 1, token_types_start: 2145, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 2145, raw_values_count: 3, token_types_start: 2148, token_types_count: 0 }, // [808]
    SimpleHintData { raw_values_start: 2148, raw_values_count: 2, token_types_start: 2150, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 2150, raw_values_count: 1, token_types_start: 2151, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 2151, raw_values_count: 2, token_types_start: 2153, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 2153, raw_values_count: 2, token_types_start: 2155, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 2155, raw_values_count: 21, token_types_start: 2176, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 2176, raw_values_count: 1, token_types_start: 2177, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 2177, raw_values_count: 7, token_types_start: 2184, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 2184, raw_values_count: 4, token_types_start: 2188, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 2188, raw_values_count: 4, token_types_start: 2192, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 2192, raw_values_count: 1, token_types_start: 2193, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 2193, raw_values_count: 3, token_types_start: 2196, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 2196, raw_values_count: 1, token_types_start: 2197, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 2197, raw_values_count: 1, token_types_start: 2198, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 2198, raw_values_count: 1, token_types_start: 2199, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 2199, raw_values_count: 1, token_types_start: 2200, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 2200, raw_values_count: 1, token_types_start: 2201, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 2201, raw_values_count: 1, token_types_start: 2202, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 2202, raw_values_count: 0, token_types_start: 2202, token_types_count: 1 }, // [826]
    SimpleHintData { raw_values_start: 2203, raw_values_count: 2, token_types_start: 2205, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 2205, raw_values_count: 1, token_types_start: 2206, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 2206, raw_values_count: 1, token_types_start: 2207, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 2207, raw_values_count: 1, token_types_start: 2208, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 2208, raw_values_count: 6, token_types_start: 2214, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 2214, raw_values_count: 1, token_types_start: 2215, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 2215, raw_values_count: 5, token_types_start: 2220, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 2220, raw_values_count: 1, token_types_start: 2221, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 2221, raw_values_count: 1, token_types_start: 2222, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 2222, raw_values_count: 1, token_types_start: 2223, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 2223, raw_values_count: 1, token_types_start: 2224, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 2224, raw_values_count: 1, token_types_start: 2225, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 2225, raw_values_count: 1, token_types_start: 2226, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 2226, raw_values_count: 1, token_types_start: 2227, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 2227, raw_values_count: 1, token_types_start: 2228, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 2228, raw_values_count: 1, token_types_start: 2229, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 2229, raw_values_count: 1, token_types_start: 2230, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 2230, raw_values_count: 1, token_types_start: 2231, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 2231, raw_values_count: 1, token_types_start: 2232, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 2232, raw_values_count: 1, token_types_start: 2233, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 2233, raw_values_count: 1, token_types_start: 2234, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 2234, raw_values_count: 1, token_types_start: 2235, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 2235, raw_values_count: 1, token_types_start: 2236, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 2236, raw_values_count: 1, token_types_start: 2237, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 2237, raw_values_count: 2, token_types_start: 2239, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 2239, raw_values_count: 1, token_types_start: 2240, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 2240, raw_values_count: 1, token_types_start: 2241, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 2241, raw_values_count: 2, token_types_start: 2243, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 2243, raw_values_count: 2, token_types_start: 2245, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 2245, raw_values_count: 1, token_types_start: 2246, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 2246, raw_values_count: 3, token_types_start: 2249, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 2249, raw_values_count: 1, token_types_start: 2250, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 2250, raw_values_count: 1, token_types_start: 2251, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 2251, raw_values_count: 1, token_types_start: 2252, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 2252, raw_values_count: 1, token_types_start: 2253, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 2253, raw_values_count: 1, token_types_start: 2254, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 2254, raw_values_count: 1, token_types_start: 2255, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 2255, raw_values_count: 1, token_types_start: 2256, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 2256, raw_values_count: 1, token_types_start: 2257, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 2257, raw_values_count: 1, token_types_start: 2258, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 2258, raw_values_count: 2, token_types_start: 2260, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 2260, raw_values_count: 8, token_types_start: 2268, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 2268, raw_values_count: 1, token_types_start: 2269, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 2269, raw_values_count: 1, token_types_start: 2270, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 2270, raw_values_count: 1, token_types_start: 2271, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 2271, raw_values_count: 1, token_types_start: 2272, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 2272, raw_values_count: 1, token_types_start: 2273, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 2273, raw_values_count: 0, token_types_start: 2273, token_types_count: 7 }, // [874]
    SimpleHintData { raw_values_start: 2280, raw_values_count: 24, token_types_start: 2304, token_types_count: 6 }, // [875]
    SimpleHintData { raw_values_start: 2310, raw_values_count: 6, token_types_start: 2316, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 2316, raw_values_count: 1, token_types_start: 2317, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 2317, raw_values_count: 1, token_types_start: 2318, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 2318, raw_values_count: 1, token_types_start: 2319, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 2319, raw_values_count: 1, token_types_start: 2320, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 2320, raw_values_count: 3, token_types_start: 2323, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 2323, raw_values_count: 1, token_types_start: 2324, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 2324, raw_values_count: 1, token_types_start: 2325, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 2325, raw_values_count: 5, token_types_start: 2330, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 2330, raw_values_count: 1, token_types_start: 2331, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 2331, raw_values_count: 1, token_types_start: 2332, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 2332, raw_values_count: 1, token_types_start: 2333, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 2333, raw_values_count: 1, token_types_start: 2334, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 2334, raw_values_count: 2, token_types_start: 2336, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 2336, raw_values_count: 3, token_types_start: 2339, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 2339, raw_values_count: 1, token_types_start: 2340, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 2340, raw_values_count: 1, token_types_start: 2341, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 2341, raw_values_count: 1, token_types_start: 2342, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 2342, raw_values_count: 4, token_types_start: 2346, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 2346, raw_values_count: 1, token_types_start: 2347, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 2347, raw_values_count: 1, token_types_start: 2348, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 2348, raw_values_count: 1, token_types_start: 2349, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 2349, raw_values_count: 1, token_types_start: 2350, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 2350, raw_values_count: 1, token_types_start: 2351, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 2351, raw_values_count: 0, token_types_start: 2351, token_types_count: 1 }, // [900]
    SimpleHintData { raw_values_start: 2352, raw_values_count: 1, token_types_start: 2353, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 2353, raw_values_count: 1, token_types_start: 2354, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 2354, raw_values_count: 1, token_types_start: 2355, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 2355, raw_values_count: 1, token_types_start: 2356, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 2356, raw_values_count: 1, token_types_start: 2357, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 2357, raw_values_count: 3, token_types_start: 2360, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 2360, raw_values_count: 1, token_types_start: 2361, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 2361, raw_values_count: 9, token_types_start: 2370, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 2370, raw_values_count: 2, token_types_start: 2372, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 2372, raw_values_count: 1, token_types_start: 2373, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 2373, raw_values_count: 1, token_types_start: 2374, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 2374, raw_values_count: 1, token_types_start: 2375, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 2375, raw_values_count: 1, token_types_start: 2376, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 2376, raw_values_count: 1, token_types_start: 2377, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 2377, raw_values_count: 1, token_types_start: 2378, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 2378, raw_values_count: 1, token_types_start: 2379, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 2379, raw_values_count: 2, token_types_start: 2381, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 2381, raw_values_count: 1, token_types_start: 2382, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 2382, raw_values_count: 1, token_types_start: 2383, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 2383, raw_values_count: 1, token_types_start: 2384, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 2384, raw_values_count: 1, token_types_start: 2385, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 2385, raw_values_count: 1, token_types_start: 2386, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 2386, raw_values_count: 1, token_types_start: 2387, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 2387, raw_values_count: 4, token_types_start: 2391, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 2391, raw_values_count: 1, token_types_start: 2392, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 2392, raw_values_count: 1, token_types_start: 2393, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 2393, raw_values_count: 1, token_types_start: 2394, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 2394, raw_values_count: 1, token_types_start: 2395, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 2395, raw_values_count: 1, token_types_start: 2396, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 2396, raw_values_count: 1, token_types_start: 2397, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 2397, raw_values_count: 1, token_types_start: 2398, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 2398, raw_values_count: 1, token_types_start: 2399, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 2399, raw_values_count: 1, token_types_start: 2400, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 2400, raw_values_count: 1, token_types_start: 2401, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 2401, raw_values_count: 1, token_types_start: 2402, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 2402, raw_values_count: 1, token_types_start: 2403, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 2403, raw_values_count: 1, token_types_start: 2404, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 2404, raw_values_count: 1, token_types_start: 2405, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 2405, raw_values_count: 1, token_types_start: 2406, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 2406, raw_values_count: 1, token_types_start: 2407, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 2407, raw_values_count: 1, token_types_start: 2408, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 2408, raw_values_count: 3, token_types_start: 2411, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 2411, raw_values_count: 1, token_types_start: 2412, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 2412, raw_values_count: 1, token_types_start: 2413, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 2413, raw_values_count: 2, token_types_start: 2415, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 2415, raw_values_count: 0, token_types_start: 2415, token_types_count: 1 }, // [946]
    SimpleHintData { raw_values_start: 2416, raw_values_count: 1, token_types_start: 2417, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 2417, raw_values_count: 1, token_types_start: 2418, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 2418, raw_values_count: 1, token_types_start: 2419, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 2419, raw_values_count: 1, token_types_start: 2420, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 2420, raw_values_count: 1, token_types_start: 2421, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 2421, raw_values_count: 1, token_types_start: 2422, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 2422, raw_values_count: 1, token_types_start: 2423, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 2423, raw_values_count: 1, token_types_start: 2424, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 2424, raw_values_count: 1, token_types_start: 2425, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 2425, raw_values_count: 1, token_types_start: 2426, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 2426, raw_values_count: 1, token_types_start: 2427, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 2427, raw_values_count: 0, token_types_start: 2427, token_types_count: 1 }, // [958]
    SimpleHintData { raw_values_start: 2428, raw_values_count: 1, token_types_start: 2429, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 2429, raw_values_count: 1, token_types_start: 2430, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 2430, raw_values_count: 1, token_types_start: 2431, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 2431, raw_values_count: 1, token_types_start: 2432, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 2432, raw_values_count: 1, token_types_start: 2433, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 2433, raw_values_count: 1, token_types_start: 2434, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 2434, raw_values_count: 1, token_types_start: 2435, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 2435, raw_values_count: 1, token_types_start: 2436, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 2436, raw_values_count: 1, token_types_start: 2437, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 2437, raw_values_count: 1, token_types_start: 2438, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 2438, raw_values_count: 1, token_types_start: 2439, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 2439, raw_values_count: 1, token_types_start: 2440, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 2440, raw_values_count: 1, token_types_start: 2441, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 2441, raw_values_count: 1, token_types_start: 2442, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 2442, raw_values_count: 3, token_types_start: 2445, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 2445, raw_values_count: 1, token_types_start: 2446, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 2446, raw_values_count: 1, token_types_start: 2447, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 2447, raw_values_count: 1, token_types_start: 2448, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 2448, raw_values_count: 1, token_types_start: 2449, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 2449, raw_values_count: 1, token_types_start: 2450, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 2450, raw_values_count: 1, token_types_start: 2451, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 2451, raw_values_count: 1, token_types_start: 2452, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 2452, raw_values_count: 1, token_types_start: 2453, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 2453, raw_values_count: 1, token_types_start: 2454, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 2454, raw_values_count: 1, token_types_start: 2455, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 2455, raw_values_count: 1, token_types_start: 2456, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 2456, raw_values_count: 1, token_types_start: 2457, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 2457, raw_values_count: 1, token_types_start: 2458, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 2458, raw_values_count: 1, token_types_start: 2459, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 2459, raw_values_count: 1, token_types_start: 2460, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 2460, raw_values_count: 1, token_types_start: 2461, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 2461, raw_values_count: 6, token_types_start: 2467, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 2467, raw_values_count: 2, token_types_start: 2469, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 2469, raw_values_count: 1, token_types_start: 2470, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 2470, raw_values_count: 1, token_types_start: 2471, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 2471, raw_values_count: 1, token_types_start: 2472, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 2472, raw_values_count: 4, token_types_start: 2476, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 2476, raw_values_count: 1, token_types_start: 2477, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 2477, raw_values_count: 1, token_types_start: 2478, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 2478, raw_values_count: 1, token_types_start: 2479, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 2479, raw_values_count: 1, token_types_start: 2480, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 2480, raw_values_count: 1, token_types_start: 2481, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 2481, raw_values_count: 1, token_types_start: 2482, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 2482, raw_values_count: 1, token_types_start: 2483, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 2483, raw_values_count: 1, token_types_start: 2484, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 2484, raw_values_count: 1, token_types_start: 2485, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 2485, raw_values_count: 1, token_types_start: 2486, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 2486, raw_values_count: 1, token_types_start: 2487, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 2487, raw_values_count: 1, token_types_start: 2488, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 2488, raw_values_count: 1, token_types_start: 2489, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 2489, raw_values_count: 1, token_types_start: 2490, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 2490, raw_values_count: 1, token_types_start: 2491, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 2491, raw_values_count: 1, token_types_start: 2492, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 2492, raw_values_count: 0, token_types_start: 2492, token_types_count: 7 }, // [1012]
    SimpleHintData { raw_values_start: 2499, raw_values_count: 3, token_types_start: 2502, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 2502, raw_values_count: 10, token_types_start: 2512, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 2512, raw_values_count: 2, token_types_start: 2514, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 2514, raw_values_count: 1, token_types_start: 2515, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 2515, raw_values_count: 1, token_types_start: 2516, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 2516, raw_values_count: 1, token_types_start: 2517, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 2517, raw_values_count: 1, token_types_start: 2518, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 2518, raw_values_count: 1, token_types_start: 2519, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 2519, raw_values_count: 1, token_types_start: 2520, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 2520, raw_values_count: 1, token_types_start: 2521, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 2521, raw_values_count: 1, token_types_start: 2522, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 2522, raw_values_count: 1, token_types_start: 2523, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 2523, raw_values_count: 1, token_types_start: 2524, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 2524, raw_values_count: 1, token_types_start: 2525, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 2525, raw_values_count: 1, token_types_start: 2526, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 2526, raw_values_count: 1, token_types_start: 2527, token_types_count: 6 }, // [1028]
    SimpleHintData { raw_values_start: 2533, raw_values_count: 1, token_types_start: 2534, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 2534, raw_values_count: 1, token_types_start: 2535, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 2535, raw_values_count: 1, token_types_start: 2536, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 2536, raw_values_count: 1, token_types_start: 2537, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 2537, raw_values_count: 1, token_types_start: 2538, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 2538, raw_values_count: 1, token_types_start: 2539, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 2539, raw_values_count: 1, token_types_start: 2540, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 2540, raw_values_count: 1, token_types_start: 2541, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 2541, raw_values_count: 1, token_types_start: 2542, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 2542, raw_values_count: 1, token_types_start: 2543, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 2543, raw_values_count: 1, token_types_start: 2544, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 2544, raw_values_count: 1, token_types_start: 2545, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 2545, raw_values_count: 1, token_types_start: 2546, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 2546, raw_values_count: 1, token_types_start: 2547, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 2547, raw_values_count: 8, token_types_start: 2555, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 2555, raw_values_count: 1, token_types_start: 2556, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 2556, raw_values_count: 1, token_types_start: 2557, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 2557, raw_values_count: 1, token_types_start: 2558, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 2558, raw_values_count: 1, token_types_start: 2559, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 2559, raw_values_count: 1, token_types_start: 2560, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 2560, raw_values_count: 1, token_types_start: 2561, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 2561, raw_values_count: 1, token_types_start: 2562, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 2562, raw_values_count: 3, token_types_start: 2565, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 2565, raw_values_count: 1, token_types_start: 2566, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 2566, raw_values_count: 1, token_types_start: 2567, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 2567, raw_values_count: 0, token_types_start: 2567, token_types_count: 1 }, // [1054]
    SimpleHintData { raw_values_start: 2568, raw_values_count: 0, token_types_start: 2568, token_types_count: 1 }, // [1055]
    SimpleHintData { raw_values_start: 2569, raw_values_count: 0, token_types_start: 2569, token_types_count: 2 }, // [1056]
    SimpleHintData { raw_values_start: 2571, raw_values_count: 0, token_types_start: 2571, token_types_count: 1 }, // [1057]
    SimpleHintData { raw_values_start: 2572, raw_values_count: 0, token_types_start: 2572, token_types_count: 2 }, // [1058]
    SimpleHintData { raw_values_start: 2574, raw_values_count: 0, token_types_start: 2574, token_types_count: 1 }, // [1059]
    SimpleHintData { raw_values_start: 2575, raw_values_count: 0, token_types_start: 2575, token_types_count: 1 }, // [1060]
    SimpleHintData { raw_values_start: 2576, raw_values_count: 1, token_types_start: 2577, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 2577, raw_values_count: 0, token_types_start: 2577, token_types_count: 2 }, // [1062]
    SimpleHintData { raw_values_start: 2579, raw_values_count: 0, token_types_start: 2579, token_types_count: 1 }, // [1063]
    SimpleHintData { raw_values_start: 2580, raw_values_count: 1, token_types_start: 2581, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 2581, raw_values_count: 1, token_types_start: 2582, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 2582, raw_values_count: 1, token_types_start: 2583, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 2583, raw_values_count: 1, token_types_start: 2584, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 2584, raw_values_count: 1, token_types_start: 2585, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 2585, raw_values_count: 1, token_types_start: 2586, token_types_count: 0 }, // [1069]
    SimpleHintData { raw_values_start: 2586, raw_values_count: 1, token_types_start: 2587, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 2587, raw_values_count: 1, token_types_start: 2588, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 2588, raw_values_count: 4, token_types_start: 2592, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 2592, raw_values_count: 3, token_types_start: 2595, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 2595, raw_values_count: 1, token_types_start: 2596, token_types_count: 0 }, // [1074]
    SimpleHintData { raw_values_start: 2596, raw_values_count: 2, token_types_start: 2598, token_types_count: 0 }, // [1075]
    SimpleHintData { raw_values_start: 2598, raw_values_count: 1, token_types_start: 2599, token_types_count: 0 }, // [1076]
    SimpleHintData { raw_values_start: 2599, raw_values_count: 1, token_types_start: 2600, token_types_count: 0 }, // [1077]
    SimpleHintData { raw_values_start: 2600, raw_values_count: 3, token_types_start: 2603, token_types_count: 0 }, // [1078]
    SimpleHintData { raw_values_start: 2603, raw_values_count: 5, token_types_start: 2608, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 2608, raw_values_count: 2, token_types_start: 2610, token_types_count: 0 }, // [1080]
    SimpleHintData { raw_values_start: 2610, raw_values_count: 1, token_types_start: 2611, token_types_count: 0 }, // [1081]
    SimpleHintData { raw_values_start: 2611, raw_values_count: 1, token_types_start: 2612, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 2612, raw_values_count: 1, token_types_start: 2613, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 2613, raw_values_count: 1, token_types_start: 2614, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 2614, raw_values_count: 1, token_types_start: 2615, token_types_count: 0 }, // [1085]
    SimpleHintData { raw_values_start: 2615, raw_values_count: 1, token_types_start: 2616, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 2616, raw_values_count: 1, token_types_start: 2617, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 2617, raw_values_count: 1, token_types_start: 2618, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 2618, raw_values_count: 1, token_types_start: 2619, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 2619, raw_values_count: 1, token_types_start: 2620, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 2620, raw_values_count: 1, token_types_start: 2621, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 2621, raw_values_count: 1, token_types_start: 2622, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 2622, raw_values_count: 1, token_types_start: 2623, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 2623, raw_values_count: 1, token_types_start: 2624, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 2624, raw_values_count: 1, token_types_start: 2625, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 2625, raw_values_count: 1, token_types_start: 2626, token_types_count: 0 }, // [1096]
    SimpleHintData { raw_values_start: 2626, raw_values_count: 2, token_types_start: 2628, token_types_count: 0 }, // [1097]
    SimpleHintData { raw_values_start: 2628, raw_values_count: 1, token_types_start: 2629, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 2629, raw_values_count: 1, token_types_start: 2630, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 2630, raw_values_count: 1, token_types_start: 2631, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 2631, raw_values_count: 1, token_types_start: 2632, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 2632, raw_values_count: 22, token_types_start: 2654, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 2654, raw_values_count: 2, token_types_start: 2656, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 2656, raw_values_count: 11, token_types_start: 2667, token_types_count: 1 }, // [1104]
    SimpleHintData { raw_values_start: 2668, raw_values_count: 1, token_types_start: 2669, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 2669, raw_values_count: 1, token_types_start: 2670, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 2670, raw_values_count: 1, token_types_start: 2671, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 2671, raw_values_count: 2, token_types_start: 2673, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 2673, raw_values_count: 1, token_types_start: 2674, token_types_count: 0 }, // [1109]
    SimpleHintData { raw_values_start: 2674, raw_values_count: 2, token_types_start: 2676, token_types_count: 0 }, // [1110]
    SimpleHintData { raw_values_start: 2676, raw_values_count: 2, token_types_start: 2678, token_types_count: 0 }, // [1111]
    SimpleHintData { raw_values_start: 2678, raw_values_count: 1, token_types_start: 2679, token_types_count: 0 }, // [1112]
    SimpleHintData { raw_values_start: 2679, raw_values_count: 1, token_types_start: 2680, token_types_count: 0 }, // [1113]
    SimpleHintData { raw_values_start: 2680, raw_values_count: 2, token_types_start: 2682, token_types_count: 6 }, // [1114]
    SimpleHintData { raw_values_start: 2688, raw_values_count: 1, token_types_start: 2689, token_types_count: 0 }, // [1115]
    SimpleHintData { raw_values_start: 2689, raw_values_count: 1, token_types_start: 2690, token_types_count: 0 }, // [1116]
    SimpleHintData { raw_values_start: 2690, raw_values_count: 1, token_types_start: 2691, token_types_count: 0 }, // [1117]
    SimpleHintData { raw_values_start: 2691, raw_values_count: 1, token_types_start: 2692, token_types_count: 0 }, // [1118]
    SimpleHintData { raw_values_start: 2692, raw_values_count: 1, token_types_start: 2693, token_types_count: 0 }, // [1119]
    SimpleHintData { raw_values_start: 2693, raw_values_count: 1, token_types_start: 2694, token_types_count: 0 }, // [1120]
    SimpleHintData { raw_values_start: 2694, raw_values_count: 1, token_types_start: 2695, token_types_count: 0 }, // [1121]
    SimpleHintData { raw_values_start: 2695, raw_values_count: 1, token_types_start: 2696, token_types_count: 0 }, // [1122]
    SimpleHintData { raw_values_start: 2696, raw_values_count: 1, token_types_start: 2697, token_types_count: 0 }, // [1123]
    SimpleHintData { raw_values_start: 2697, raw_values_count: 1, token_types_start: 2698, token_types_count: 0 }, // [1124]
    SimpleHintData { raw_values_start: 2698, raw_values_count: 1, token_types_start: 2699, token_types_count: 0 }, // [1125]
    SimpleHintData { raw_values_start: 2699, raw_values_count: 1, token_types_start: 2700, token_types_count: 0 }, // [1126]
    SimpleHintData { raw_values_start: 2700, raw_values_count: 1, token_types_start: 2701, token_types_count: 0 }, // [1127]
    SimpleHintData { raw_values_start: 2701, raw_values_count: 1, token_types_start: 2702, token_types_count: 0 }, // [1128]
    SimpleHintData { raw_values_start: 2702, raw_values_count: 1, token_types_start: 2703, token_types_count: 0 }, // [1129]
    SimpleHintData { raw_values_start: 2703, raw_values_count: 1, token_types_start: 2704, token_types_count: 0 }, // [1130]
    SimpleHintData { raw_values_start: 2704, raw_values_count: 1, token_types_start: 2705, token_types_count: 0 }, // [1131]
    SimpleHintData { raw_values_start: 2705, raw_values_count: 1, token_types_start: 2706, token_types_count: 0 }, // [1132]
    SimpleHintData { raw_values_start: 2706, raw_values_count: 1, token_types_start: 2707, token_types_count: 0 }, // [1133]
    SimpleHintData { raw_values_start: 2707, raw_values_count: 1, token_types_start: 2708, token_types_count: 0 }, // [1134]
    SimpleHintData { raw_values_start: 2708, raw_values_count: 1, token_types_start: 2709, token_types_count: 0 }, // [1135]
    SimpleHintData { raw_values_start: 2709, raw_values_count: 1, token_types_start: 2710, token_types_count: 0 }, // [1136]
    SimpleHintData { raw_values_start: 2710, raw_values_count: 1, token_types_start: 2711, token_types_count: 0 }, // [1137]
    SimpleHintData { raw_values_start: 2711, raw_values_count: 1, token_types_start: 2712, token_types_count: 0 }, // [1138]
    SimpleHintData { raw_values_start: 2712, raw_values_count: 5, token_types_start: 2717, token_types_count: 0 }, // [1139]
    SimpleHintData { raw_values_start: 2717, raw_values_count: 1, token_types_start: 2718, token_types_count: 0 }, // [1140]
    SimpleHintData { raw_values_start: 2718, raw_values_count: 1, token_types_start: 2719, token_types_count: 0 }, // [1141]
    SimpleHintData { raw_values_start: 2719, raw_values_count: 1, token_types_start: 2720, token_types_count: 0 }, // [1142]
    SimpleHintData { raw_values_start: 2720, raw_values_count: 1, token_types_start: 2721, token_types_count: 0 }, // [1143]
    SimpleHintData { raw_values_start: 2721, raw_values_count: 1, token_types_start: 2722, token_types_count: 0 }, // [1144]
    SimpleHintData { raw_values_start: 2722, raw_values_count: 1, token_types_start: 2723, token_types_count: 0 }, // [1145]
    SimpleHintData { raw_values_start: 2723, raw_values_count: 1, token_types_start: 2724, token_types_count: 0 }, // [1146]
    SimpleHintData { raw_values_start: 2724, raw_values_count: 1, token_types_start: 2725, token_types_count: 0 }, // [1147]
    SimpleHintData { raw_values_start: 2725, raw_values_count: 1, token_types_start: 2726, token_types_count: 0 }, // [1148]
    SimpleHintData { raw_values_start: 2726, raw_values_count: 1, token_types_start: 2727, token_types_count: 0 }, // [1149]
    SimpleHintData { raw_values_start: 2727, raw_values_count: 5, token_types_start: 2732, token_types_count: 0 }, // [1150]
    SimpleHintData { raw_values_start: 2732, raw_values_count: 4, token_types_start: 2736, token_types_count: 0 }, // [1151]
    SimpleHintData { raw_values_start: 2736, raw_values_count: 2, token_types_start: 2738, token_types_count: 0 }, // [1152]
    SimpleHintData { raw_values_start: 2738, raw_values_count: 3, token_types_start: 2741, token_types_count: 0 }, // [1153]
    SimpleHintData { raw_values_start: 2741, raw_values_count: 1, token_types_start: 2742, token_types_count: 0 }, // [1154]
    SimpleHintData { raw_values_start: 2742, raw_values_count: 1, token_types_start: 2743, token_types_count: 0 }, // [1155]
    SimpleHintData { raw_values_start: 2743, raw_values_count: 1, token_types_start: 2744, token_types_count: 0 }, // [1156]
    SimpleHintData { raw_values_start: 2744, raw_values_count: 1, token_types_start: 2745, token_types_count: 0 }, // [1157]
    SimpleHintData { raw_values_start: 2745, raw_values_count: 1, token_types_start: 2746, token_types_count: 0 }, // [1158]
    SimpleHintData { raw_values_start: 2746, raw_values_count: 1, token_types_start: 2747, token_types_count: 0 }, // [1159]
    SimpleHintData { raw_values_start: 2747, raw_values_count: 1, token_types_start: 2748, token_types_count: 0 }, // [1160]
    SimpleHintData { raw_values_start: 2748, raw_values_count: 1, token_types_start: 2749, token_types_count: 0 }, // [1161]
    SimpleHintData { raw_values_start: 2749, raw_values_count: 1, token_types_start: 2750, token_types_count: 0 }, // [1162]
    SimpleHintData { raw_values_start: 2750, raw_values_count: 1, token_types_start: 2751, token_types_count: 0 }, // [1163]
    SimpleHintData { raw_values_start: 2751, raw_values_count: 1, token_types_start: 2752, token_types_count: 0 }, // [1164]
    SimpleHintData { raw_values_start: 2752, raw_values_count: 1, token_types_start: 2753, token_types_count: 0 }, // [1165]
    SimpleHintData { raw_values_start: 2753, raw_values_count: 1, token_types_start: 2754, token_types_count: 0 }, // [1166]
    SimpleHintData { raw_values_start: 2754, raw_values_count: 1, token_types_start: 2755, token_types_count: 0 }, // [1167]
    SimpleHintData { raw_values_start: 2755, raw_values_count: 1, token_types_start: 2756, token_types_count: 0 }, // [1168]
    SimpleHintData { raw_values_start: 2756, raw_values_count: 2, token_types_start: 2758, token_types_count: 0 }, // [1169]
    SimpleHintData { raw_values_start: 2758, raw_values_count: 1, token_types_start: 2759, token_types_count: 0 }, // [1170]
    SimpleHintData { raw_values_start: 2759, raw_values_count: 1, token_types_start: 2760, token_types_count: 0 }, // [1171]
    SimpleHintData { raw_values_start: 2760, raw_values_count: 1, token_types_start: 2761, token_types_count: 0 }, // [1172]
    SimpleHintData { raw_values_start: 2761, raw_values_count: 1, token_types_start: 2762, token_types_count: 0 }, // [1173]
    SimpleHintData { raw_values_start: 2762, raw_values_count: 1, token_types_start: 2763, token_types_count: 0 }, // [1174]
    SimpleHintData { raw_values_start: 2763, raw_values_count: 2, token_types_start: 2765, token_types_count: 0 }, // [1175]
    SimpleHintData { raw_values_start: 2765, raw_values_count: 1, token_types_start: 2766, token_types_count: 0 }, // [1176]
    SimpleHintData { raw_values_start: 2766, raw_values_count: 1, token_types_start: 2767, token_types_count: 0 }, // [1177]
    SimpleHintData { raw_values_start: 2767, raw_values_count: 1, token_types_start: 2768, token_types_count: 0 }, // [1178]
    SimpleHintData { raw_values_start: 2768, raw_values_count: 1, token_types_start: 2769, token_types_count: 0 }, // [1179]
    SimpleHintData { raw_values_start: 2769, raw_values_count: 1, token_types_start: 2770, token_types_count: 0 }, // [1180]
    SimpleHintData { raw_values_start: 2770, raw_values_count: 1, token_types_start: 2771, token_types_count: 0 }, // [1181]
    SimpleHintData { raw_values_start: 2771, raw_values_count: 1, token_types_start: 2772, token_types_count: 0 }, // [1182]
    SimpleHintData { raw_values_start: 2772, raw_values_count: 1, token_types_start: 2773, token_types_count: 0 }, // [1183]
    SimpleHintData { raw_values_start: 2773, raw_values_count: 1, token_types_start: 2774, token_types_count: 0 }, // [1184]
    SimpleHintData { raw_values_start: 2774, raw_values_count: 1, token_types_start: 2775, token_types_count: 0 }, // [1185]
    SimpleHintData { raw_values_start: 2775, raw_values_count: 1, token_types_start: 2776, token_types_count: 0 }, // [1186]
    SimpleHintData { raw_values_start: 2776, raw_values_count: 1, token_types_start: 2777, token_types_count: 0 }, // [1187]
    SimpleHintData { raw_values_start: 2777, raw_values_count: 1, token_types_start: 2778, token_types_count: 0 }, // [1188]
    SimpleHintData { raw_values_start: 2778, raw_values_count: 6, token_types_start: 2784, token_types_count: 0 }, // [1189]
    SimpleHintData { raw_values_start: 2784, raw_values_count: 5, token_types_start: 2789, token_types_count: 0 }, // [1190]
    SimpleHintData { raw_values_start: 2789, raw_values_count: 11, token_types_start: 2800, token_types_count: 0 }, // [1191]
    SimpleHintData { raw_values_start: 2800, raw_values_count: 1, token_types_start: 2801, token_types_count: 0 }, // [1192]
    SimpleHintData { raw_values_start: 2801, raw_values_count: 1, token_types_start: 2802, token_types_count: 0 }, // [1193]
    SimpleHintData { raw_values_start: 2802, raw_values_count: 1, token_types_start: 2803, token_types_count: 0 }, // [1194]
    SimpleHintData { raw_values_start: 2803, raw_values_count: 1, token_types_start: 2804, token_types_count: 0 }, // [1195]
    SimpleHintData { raw_values_start: 2804, raw_values_count: 1, token_types_start: 2805, token_types_count: 0 }, // [1196]
    SimpleHintData { raw_values_start: 2805, raw_values_count: 1, token_types_start: 2806, token_types_count: 0 }, // [1197]
    SimpleHintData { raw_values_start: 2806, raw_values_count: 1, token_types_start: 2807, token_types_count: 0 }, // [1198]
    SimpleHintData { raw_values_start: 2807, raw_values_count: 1, token_types_start: 2808, token_types_count: 0 }, // [1199]
    SimpleHintData { raw_values_start: 2808, raw_values_count: 36, token_types_start: 2844, token_types_count: 0 }, // [1200]
    SimpleHintData { raw_values_start: 2844, raw_values_count: 39, token_types_start: 2883, token_types_count: 1 }, // [1201]
    SimpleHintData { raw_values_start: 2884, raw_values_count: 1, token_types_start: 2885, token_types_count: 1 }, // [1202]
    SimpleHintData { raw_values_start: 2886, raw_values_count: 2, token_types_start: 2888, token_types_count: 0 }, // [1203]
    SimpleHintData { raw_values_start: 2888, raw_values_count: 10, token_types_start: 2898, token_types_count: 0 }, // [1204]
    SimpleHintData { raw_values_start: 2898, raw_values_count: 1, token_types_start: 2899, token_types_count: 0 }, // [1205]
    SimpleHintData { raw_values_start: 2899, raw_values_count: 1, token_types_start: 2900, token_types_count: 0 }, // [1206]
    SimpleHintData { raw_values_start: 2900, raw_values_count: 2, token_types_start: 2902, token_types_count: 0 }, // [1207]
    SimpleHintData { raw_values_start: 2902, raw_values_count: 2, token_types_start: 2904, token_types_count: 0 }, // [1208]
    SimpleHintData { raw_values_start: 2904, raw_values_count: 1, token_types_start: 2905, token_types_count: 0 }, // [1209]
    SimpleHintData { raw_values_start: 2905, raw_values_count: 1, token_types_start: 2906, token_types_count: 0 }, // [1210]
    SimpleHintData { raw_values_start: 2906, raw_values_count: 1, token_types_start: 2907, token_types_count: 0 }, // [1211]
    SimpleHintData { raw_values_start: 2907, raw_values_count: 1, token_types_start: 2908, token_types_count: 0 }, // [1212]
    SimpleHintData { raw_values_start: 2908, raw_values_count: 1, token_types_start: 2909, token_types_count: 0 }, // [1213]
    SimpleHintData { raw_values_start: 2909, raw_values_count: 1, token_types_start: 2910, token_types_count: 0 }, // [1214]
    SimpleHintData { raw_values_start: 2910, raw_values_count: 1, token_types_start: 2911, token_types_count: 0 }, // [1215]
    SimpleHintData { raw_values_start: 2911, raw_values_count: 1, token_types_start: 2912, token_types_count: 0 }, // [1216]
    SimpleHintData { raw_values_start: 2912, raw_values_count: 1, token_types_start: 2913, token_types_count: 0 }, // [1217]
    SimpleHintData { raw_values_start: 2913, raw_values_count: 1, token_types_start: 2914, token_types_count: 0 }, // [1218]
    SimpleHintData { raw_values_start: 2914, raw_values_count: 1, token_types_start: 2915, token_types_count: 0 }, // [1219]
    SimpleHintData { raw_values_start: 2915, raw_values_count: 1, token_types_start: 2916, token_types_count: 0 }, // [1220]
    SimpleHintData { raw_values_start: 2916, raw_values_count: 1, token_types_start: 2917, token_types_count: 0 }, // [1221]
    SimpleHintData { raw_values_start: 2917, raw_values_count: 1, token_types_start: 2918, token_types_count: 0 }, // [1222]
    SimpleHintData { raw_values_start: 2918, raw_values_count: 1, token_types_start: 2919, token_types_count: 0 }, // [1223]
    SimpleHintData { raw_values_start: 2919, raw_values_count: 1, token_types_start: 2920, token_types_count: 0 }, // [1224]
    SimpleHintData { raw_values_start: 2920, raw_values_count: 1, token_types_start: 2921, token_types_count: 0 }, // [1225]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 0, 4, 0, 5, 6, 6, 7, 8, 9, 10, 11, 12, 13,
    14, 15, 16, 17, 18, 18, 19, 20, 21, 21, 22, 23, 23, 24, 25, 9,
    26, 26, 24, 25, 9, 27, 23, 23, 28, 25, 13, 26, 26, 28, 25, 29,
    9, 13, 0, 0, 0, 0, 0, 30, 0, 31, 32, 33, 34, 34, 35, 36,
    37, 38, 38, 39, 15, 40, 8, 9, 11, 41, 41, 42, 43, 44, 44, 45,
    46, 46, 47, 7, 7, 0, 0, 48, 49, 49, 38, 38, 50, 13, 42, 42,
    43, 51, 27, 52, 52, 53, 13, 14, 54, 54, 55, 41, 56, 38, 57, 44,
    58, 59, 7, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 48, 71,
    72, 23, 23, 55, 30, 37, 37, 73, 27, 74, 75, 76, 77, 78, 79, 80,
    81, 45, 13, 42, 42, 43, 82, 82, 75, 83, 83, 74, 84, 84, 85, 28,
    86, 87, 88, 89, 90, 91, 92, 93, 94, 24, 42, 42, 0, 82, 82, 87,
    83, 83, 86, 84, 84, 0, 95, 96, 97, 0, 98, 98, 0, 0, 40, 40,
    0, 0, 15, 15, 0, 0, 99, 99, 0, 0, 0, 0, 0, 0, 100, 101,
    102, 103, 4, 104, 105, 106, 107, 107, 108, 107, 109, 110, 111, 112, 0, 0,
    113, 0, 0, 0, 30, 0, 30, 114, 0, 0, 30, 0, 0, 0, 0, 0,
    115, 0, 0, 0, 23, 116, 117, 118, 118, 111, 0, 30, 0, 30, 0, 119,
    120, 121, 121, 32, 0, 122, 122, 32, 0, 123, 124, 125, 0, 126, 126, 13,
    0, 118, 118, 9, 0, 127, 128, 129, 130, 116, 116, 131, 132, 132, 133, 36,
    134, 134, 131, 121, 121, 32, 0, 122, 122, 32, 0, 0, 123, 124, 125, 126,
    126, 16, 0, 126, 126, 0, 135, 136, 136, 32, 137, 0, 138, 139, 0, 140,
    33, 33, 141, 142, 142, 0, 23, 0, 96, 96, 143, 73, 144, 32, 0, 0,
    129, 129, 129, 96, 145, 143, 23, 23, 55, 146, 146, 38, 57, 44, 58, 64,
    65, 68, 69, 70, 48, 73, 97, 97, 96, 96, 145, 147, 143, 73, 144, 33,
    0, 0, 148, 149, 150, 144, 86, 89, 91, 92, 151, 24, 118, 118, 138, 55,
    147, 147, 152, 15, 40, 0, 0, 25, 96, 97, 25, 25, 13, 0, 0, 96,
    97, 95, 96, 97, 0, 0, 98, 0, 100, 118, 118, 10, 0, 153, 126, 126,
    138, 0, 154, 154, 138, 155, 155, 126, 154, 156, 157, 106, 106, 158, 158, 0,
    159, 160, 156, 157, 161, 161, 0, 156, 156, 162, 154, 154, 158, 163, 0, 150,
    148, 149, 121, 121, 158, 0, 32, 0, 164, 164, 158, 0, 122, 122, 32, 0,
    0, 123, 124, 125, 121, 121, 32, 0, 126, 126, 13, 0, 118, 118, 165, 0,
    166, 70, 70, 32, 32, 139, 126, 126, 13, 0, 0, 167, 168, 168, 106, 154,
    169, 170, 170, 3, 3, 171, 172, 173, 10, 174, 174, 68, 18, 18, 21, 22,
    18, 18, 74, 175, 176, 12, 13, 80, 20, 177, 177, 178, 179, 180, 181, 182,
    183, 17, 0, 184, 184, 82, 82, 75, 74, 75, 0, 111, 111, 0, 30, 0,
    0, 185, 185, 30, 0, 0, 113, 0, 186, 186, 77, 78, 79, 0, 0, 0,
    30, 187, 187, 0, 0, 0, 30, 0, 0, 0, 31, 0, 188, 188, 189, 190,
    0, 191, 0, 192, 192, 147, 17, 0, 193, 106, 106, 194, 163, 0, 0, 195,
    195, 0, 196, 196, 154, 154, 194, 163, 0, 150, 118, 118, 194, 0, 197, 198,
    126, 126, 21, 17, 0, 195, 195, 0, 191, 191, 0, 0, 126, 126, 138, 0,
    0, 139, 0, 199, 0, 154, 154, 138, 200, 155, 126, 154, 156, 157, 154, 154,
    201, 163, 106, 106, 202, 203, 204, 205, 205, 138, 113, 206, 30, 207, 207, 208,
    208, 126, 126, 202, 203, 204, 205, 205, 138, 126, 126, 207, 209, 209, 129, 129,
    36, 154, 154, 206, 163, 126, 126, 210, 36, 126, 126, 30, 142, 142, 30, 126,
    126, 211, 212, 213, 83, 214, 215, 126, 126, 216, 0, 106, 106, 217, 106, 106,
    218, 118, 118, 158, 0, 219, 220, 156, 156, 220, 221, 221, 221, 222, 221, 221,
    223, 164, 164, 158, 0, 154, 154, 158, 163, 0, 150, 224, 224, 225, 226, 68,
    0, 0, 23, 40, 225, 225, 227, 228, 204, 68, 0, 224, 224, 225, 226, 225,
    225, 228, 225, 225, 229, 229, 0, 230, 230, 226, 225, 231, 232, 232, 231, 233,
    234, 235, 236, 236, 73, 0, 126, 126, 237, 238, 236, 239, 126, 126, 16, 0,
    126, 126, 240, 241, 242, 126, 126, 30, 142, 142, 30, 243, 232, 244, 0, 245,
    245, 0, 156, 156, 245, 122, 122, 32, 0, 0, 123, 124, 125, 228, 228, 206,
    246, 138, 191, 191, 247, 0, 248, 249, 250, 118, 118, 0, 0, 251, 30, 0,
    0, 252, 106, 126, 154, 0, 253, 118, 118, 18, 74, 0, 163, 254, 0, 119,
    255, 193, 193, 121, 121, 194, 0, 32, 0, 121, 121, 32, 0, 256, 256, 257,
    258, 258, 73, 157, 259, 260, 260, 157, 73, 157, 259, 261, 262, 263, 264, 265,
    266, 156, 267, 268, 83, 235, 269, 270, 271, 272, 272, 273, 274, 275, 276, 277,
    277, 36, 278, 278, 36, 279, 279, 0, 126, 126, 0, 135, 136, 136, 32, 137,
    0, 139, 0, 138, 33, 33, 141, 142, 142, 0, 23, 0, 148, 118, 118, 77,
    0, 0, 0, 30, 280, 256, 121, 121, 32, 0, 126, 126, 13, 0, 122, 122,
    32, 0, 0, 0, 140, 123, 124, 125, 281, 232, 282, 73, 165, 0, 118, 118,
    247, 0, 0, 163, 0, 283, 121, 121, 32, 0, 126, 126, 16, 0, 284, 284,
    285, 0, 281, 232, 282, 73, 165, 0, 126, 126, 30, 0, 0, 0, 137, 137,
    139, 142, 142, 30, 0, 0, 118, 118, 194, 36, 126, 210, 36, 23, 23, 25,
    16, 0, 286, 286, 205, 0, 0, 126, 16, 0, 287, 118, 288, 118, 118, 194,
    0, 289, 126, 126, 210, 36, 126, 126, 30, 0, 0, 0, 137, 139, 142, 142,
    30, 0, 0, 126, 126, 211, 212, 213, 83, 214, 215, 126, 126, 216, 0, 236,
    236, 73, 0, 126, 126, 237, 236, 126, 126, 30, 0, 0, 0, 137, 137, 139,
    142, 142, 30, 0, 0, 122, 122, 32, 0, 0, 123, 124, 125, 118, 118, 82,
    75, 0, 0, 163, 0, 290, 288, 288, 121, 121, 194, 194, 0, 32, 0, 121,
    121, 32, 0, 126, 126, 13, 0, 0, 0, 232, 282, 73, 165, 0, 23, 23,
    25, 16, 0, 286, 286, 205, 0, 0, 126, 16, 0, 287, 287, 118, 118, 43,
    0, 73, 0, 291, 121, 121, 32, 0, 292, 32, 32, 0, 0, 0, 100, 123,
    124, 125, 191, 191, 30, 0, 129, 129, 161, 30, 0, 293, 293, 268, 83, 235,
    269, 270, 271, 126, 126, 0, 135, 136, 136, 32, 137, 0, 139, 0, 138, 33,
    33, 141, 142, 142, 0, 23, 0, 148, 118, 118, 78, 0, 0, 0, 30, 294,
    293, 121, 121, 32, 0, 126, 126, 13, 0, 126, 126, 0, 135, 136, 136, 32,
    137, 0, 0, 0, 139, 33, 33, 141, 122, 122, 32, 0, 0, 0, 140, 123,
    124, 125, 281, 232, 282, 73, 165, 0, 118, 118, 295, 0, 296, 126, 126, 30,
    106, 106, 297, 297, 126, 126, 297, 297, 154, 154, 297, 297, 121, 121, 32, 0,
    122, 122, 32, 0, 123, 124, 125, 0, 118, 118, 152, 15, 40, 0, 0, 0,
    123, 124, 125, 0, 129, 298, 299, 300, 301, 302, 303, 304, 305, 306, 307, 308,
    244, 309, 310, 311, 312, 313, 314, 315, 316, 317, 318, 132, 132, 133, 36, 319,
    320, 321, 322, 253, 157, 162, 162, 323, 253, 0, 0, 121, 121, 32, 0, 0,
    0, 123, 124, 125, 23, 0, 25, 25, 9, 0, 289, 126, 126, 0, 135, 136,
    136, 32, 137, 0, 138, 0, 0, 139, 0, 73, 33, 33, 141, 142, 142, 0,
    0, 23, 118, 118, 13, 0, 324, 121, 121, 32, 0, 122, 122, 32, 0, 207,
    113, 113, 0, 325, 325, 205, 326, 36, 327, 328, 328, 326, 36, 232, 232, 328,
    329, 330, 330, 326, 36, 232, 232, 330, 80, 80, 331, 0, 332, 332, 129, 326,
    36, 209, 209, 129, 326, 36, 333, 333, 36, 334, 232, 335, 286, 286, 205, 0,
    336, 0, 118, 118, 80, 163, 0, 337, 207, 207, 122, 122, 32, 0, 0, 124,
    125, 121, 121, 32, 0, 126, 126, 13, 0, 118, 118, 210, 163, 0, 120, 122,
    122, 32, 0, 0, 0, 140, 123, 124, 125, 121, 121, 32, 0, 126, 126, 338,
    13, 13, 0, 210, 210, 36, 118, 118, 339, 0, 340, 132, 132, 253, 252, 252,
    126, 106, 154, 295, 0, 0, 129, 129, 30, 341, 341, 295, 129, 129, 30, 225,
    226, 126, 126, 30, 342, 342, 30, 0, 137, 0, 122, 122, 32, 0, 0, 123,
    124, 0, 121, 121, 32, 0, 118, 118, 343, 289, 126, 126, 0, 136, 32, 137,
    0, 138, 0, 0, 139, 0, 140, 73, 142, 142, 0, 23, 0, 344, 106, 106,
    194, 163, 0, 0, 195, 195, 0, 196, 196, 154, 154, 194, 163, 0, 150, 118,
    118, 194, 0, 197, 198, 126, 126, 21, 17, 0, 195, 195, 0, 191, 191, 0,
    0, 126, 126, 138, 0, 0, 139, 0, 199, 0, 154, 154, 138, 200, 155, 126,
    154, 156, 157, 154, 154, 201, 163, 106, 106, 202, 203, 204, 205, 205, 138, 113,
    206, 30, 207, 207, 208, 208, 126, 126, 202, 203, 204, 205, 205, 138, 126, 126,
    207, 209, 209, 129, 129, 36, 154, 154, 206, 163, 126, 126, 210, 36, 126, 126,
    30, 142, 142, 30, 126, 126, 211, 212, 213, 83, 214, 215, 126, 126, 216, 0,
    106, 106, 217, 106, 106, 218, 118, 118, 158, 0, 219, 220, 156, 156, 220, 221,
    221, 221, 222, 221, 221, 223, 164, 164, 158, 0, 154, 154, 158, 163, 0, 150,
    224, 224, 225, 226, 68, 0, 0, 23, 40, 225, 225, 227, 228, 204, 68, 0,
    224, 224, 225, 226, 225, 225, 228, 225, 225, 229, 229, 0, 230, 230, 226, 225,
    231, 232, 232, 231, 233, 234, 235, 236, 236, 73, 0, 126, 126, 237, 238, 236,
    239, 126, 126, 16, 0, 126, 126, 240, 241, 242, 126, 126, 30, 142, 142, 30,
    243, 232, 244, 0, 245, 245, 0, 156, 156, 245, 122, 122, 32, 0, 0, 123,
    124, 125, 228, 228, 206, 246, 138, 191, 191, 247, 0, 248, 249, 154, 154, 194,
    163, 0, 0, 0, 0, 137, 0, 139, 0, 345, 345, 106, 59, 194, 0, 346,
    347, 347, 348, 110, 0, 30, 154, 121, 121, 349, 113, 32, 0, 118, 118, 74,
    0, 0, 163, 254, 0, 119, 350, 344, 344, 121, 121, 194, 0, 32, 0, 121,
    121, 158, 0, 32, 0, 0, 163, 0, 351, 121, 121, 32, 0, 126, 126, 13,
    0, 284, 284, 285, 0, 352, 147, 147, 353, 354, 138, 355, 355, 285, 0, 356,
    357, 23, 23, 25, 16, 0, 286, 286, 205, 0, 0, 126, 16, 0, 287, 118,
    118, 177, 178, 180, 0, 358, 106, 106, 359, 147, 0, 0, 129, 0, 0, 118,
    118, 359, 147, 0, 0, 129, 0, 0, 118, 118, 359, 360, 0, 129, 0, 118,
    118, 359, 147, 0, 0, 360, 0, 129, 0, 154, 154, 359, 163, 147, 0, 0,
    121, 121, 32, 0, 122, 122, 32, 0, 0, 123, 124, 125, 126, 126, 13, 0,
    118, 118, 68, 0, 73, 0, 361, 121, 121, 32, 0, 281, 232, 282, 73, 165,
    0, 118, 118, 17, 0, 358, 122, 122, 32, 0, 124, 125, 123, 0, 121, 121,
    362, 253, 32, 253, 121, 121, 32, 0, 126, 126, 13, 0, 363, 363, 106, 106,
    364, 0, 0, 195, 195, 0, 150, 118, 118, 364, 0, 126, 126, 21, 17, 0,
    195, 195, 0, 150, 154, 154, 364, 163, 0, 150, 121, 121, 364, 0, 32, 0,
    150, 106, 106, 362, 163, 253, 365, 365, 366, 110, 253, 118, 118, 75, 163, 0,
    367, 118, 118, 194, 0, 155, 126, 126, 138, 0, 139, 0, 199, 0, 154, 154,
    138, 122, 122, 32, 0, 0, 123, 124, 125, 121, 121, 194, 0, 32, 0, 121,
    121, 32, 0, 126, 126, 13, 0, 126, 126, 30, 0, 0, 0, 137, 137, 139,
    142, 142, 30, 0, 0, 204, 368, 369, 370, 371, 371, 370, 369, 372, 30, 373,
    373, 373, 374, 375, 131, 374, 0, 0, 0, 30, 0, 0, 376, 376, 377, 41,
    378, 379, 107, 380, 119, 381, 368, 108, 382, 383, 384, 102, 0, 326, 36, 0,
    0, 0, 326, 36, 0, 385, 0, 326, 36, 0, 102, 386, 0, 387, 102, 0,
    0, 387, 102, 0, 0, 113, 0, 113, 113, 113, 388, 389, 390, 391, 392, 393,
    284, 364, 394, 395, 396, 396, 396, 397, 398, 399, 199, 0, 0, 139, 199, 400,
    0, 0, 0, 0, 0, 139, 31, 113, 30, 0, 0, 366, 401, 402, 403, 404,
    405, 406, 407, 378, 408, 409, 410, 411, 0, 412, 413, 414, 368, 383, 383, 383,
    108, 384, 384, 384, 382, 415, 409, 376, 416, 417, 131, 418, 419, 420, 421, 422,
    30, 139, 139, 30, 0, 0, 0, 30, 423, 424, 425, 426, 427, 428, 205, 317,
    429, 30, 0, 0, 0, 30, 113, 430, 156, 82, 30, 431, 432, 178, 178, 433,
    424, 434, 348, 205, 0, 126, 0, 335, 335, 0, 126, 0, 191, 0, 333, 435,
    435, 435, 0, 258, 149, 150, 149, 148, 436, 437, 437, 437, 0, 438, 438, 439,
    440, 439, 440, 440, 437, 437, 0, 438, 438, 439, 440, 439, 440, 440, 410, 31,
    407, 437, 185, 441, 14, 442, 443, 0, 444, 445, 446, 161, 447, 448, 449, 450,
    451, 452, 189, 453, 236, 236, 236, 374, 0, 0, 0, 191, 191, 0, 0, 0,
    73, 0, 454, 455, 0, 0, 195, 172, 0, 253, 0, 0, 103, 0, 385, 385,
    385, 0, 0, 139, 0, 199, 196, 158, 158, 0, 456, 160, 156, 157, 161, 161,
    30, 0, 232, 232, 244, 138, 138, 0, 0, 139, 0, 199, 0, 202, 202, 204,
    113, 0, 457, 202, 202, 203, 204, 205, 205, 138, 113, 206, 30, 207, 207, 458,
    458, 459, 459, 156, 460, 129, 129, 30, 191, 191, 247, 16, 0, 461, 461, 462,
    129, 129, 30, 191, 191, 247, 16, 0, 64, 219, 220, 156, 156, 220, 221, 221,
    221, 222, 221, 221, 223, 0, 0, 0, 463, 196, 195, 195, 0, 0, 194, 0,
    0, 103, 103, 103, 103, 103, 103, 0, 0, 103, 103, 103, 103, 103, 73, 113,
    191, 31, 441, 102, 30, 407, 385, 464, 0, 0, 0, 381, 17, 0, 0, 0,
    0, 465, 31, 466, 466, 253, 466, 466, 466, 73, 467, 467, 468, 468, 74, 75,
    0, 185, 185, 30, 0, 0, 113, 0, 194, 194, 0, 158, 158, 0, 73, 73,
    10, 0, 9, 9, 0, 165, 165, 0, 77, 77, 0, 30, 30, 247, 247, 0,
    13, 13, 0, 469, 469, 172, 173, 10, 12, 43, 295, 15, 229, 80, 20, 210,
    339, 16, 68, 17, 3, 3, 171, 174, 174, 68, 18, 18, 21, 22, 18, 18,
    74, 82, 82, 75, 177, 177, 178, 180, 177, 177, 178, 181, 177, 177, 178, 182,
    177, 177, 178, 183, 0, 73, 73, 0, 470, 470, 111, 78, 79, 0, 30, 0,
    0, 471, 471, 322, 253, 157, 0, 472, 473, 474, 475, 410, 137, 384, 383, 384,
    383, 476, 477, 471, 471, 460, 33, 471, 471, 156, 460, 33, 368, 368, 368, 107,
    478, 479, 480, 187, 0, 481, 0, 0, 30, 30, 0, 103, 119, 482, 483, 216,
    484, 108, 108, 108, 356, 0, 0, 485, 0, 0, 180, 486, 486, 487, 249, 70,
    70, 126, 0, 0, 0, 0, 137, 0, 0, 138, 30, 30, 0, 0, 137, 233,
    30, 0, 0, 0, 138, 30, 30, 0, 0, 137, 30, 431, 488, 488, 0, 489,
    490, 30, 30, 0, 0, 0, 0, 30, 0, 0, 195, 195, 0, 0, 488, 488,
    0, 73, 73, 158, 0, 56, 132, 158, 491, 492, 493, 494, 173, 495, 495, 495,
    0, 0, 0, 0, 30, 0, 0, 33, 496, 497, 253, 498, 498, 499, 253, 500,
    500, 501, 501, 502, 129, 30, 503, 503, 85, 85, 0, 504, 504, 131, 505, 505,
    253, 157, 157, 253, 506, 506, 131, 507, 507, 253, 508, 508, 253, 509, 509, 510,
    30, 0, 0, 119, 511, 511, 30, 0, 0, 512, 512, 30, 0, 0, 513, 513,
    253, 488, 488, 0, 0, 33, 514, 253, 500, 501, 501, 515, 129, 516, 406, 406,
    505, 505, 113, 253, 157, 157, 113, 253, 517, 517, 518, 506, 507, 507, 113, 253,
    508, 508, 113, 253, 231, 231, 156, 157, 0, 0, 0, 0, 30, 431, 32, 519,
    520, 520, 0, 0, 30, 431, 32, 521, 253, 500, 520, 520, 515, 129, 516, 406,
    406, 505, 505, 113, 253, 157, 157, 113, 253, 517, 517, 518, 506, 507, 507, 113,
    253, 508, 508, 113, 253, 231, 231, 507, 510, 30, 0, 0, 119, 522, 277, 523,
    38, 38, 416, 111, 0, 30, 0, 30, 38, 38, 185, 30, 0, 113, 0, 524,
    129, 129, 77, 0, 30, 237, 237, 77, 129, 129, 525, 113, 113, 113, 390, 113,
    113, 526, 38, 38, 172, 163, 0, 527, 30, 528, 528, 529, 529, 137, 253, 530,
    530, 137, 253, 531, 531, 137, 253, 532, 532, 137, 533, 534, 535, 536, 536, 137,
    131, 537, 537, 137, 253, 33, 33, 0, 38, 38, 9, 0, 129, 538, 122, 122,
    137, 0, 183, 183, 137, 0, 513, 513, 137, 539, 253, 138, 540, 529, 529, 137,
    253, 541, 530, 530, 137, 253, 531, 531, 137, 253, 16, 16, 137, 0, 0, 138,
    116, 116, 137, 131, 132, 132, 133, 137, 36, 134, 134, 137, 131, 38, 38, 10,
    0, 113, 113, 0, 195, 195, 0, 138, 138, 0, 542, 542, 158, 158, 0, 159,
    160, 156, 157, 161, 161, 0, 38, 38, 165, 163, 0, 129, 13, 13, 0, 537,
    537, 0, 33, 33, 0, 149, 38, 38, 18, 0, 543, 0, 544, 0, 38, 38,
    18, 74, 38, 38, 38, 163, 0, 30, 0, 0, 0, 0, 0, 251, 195, 195,
    0, 545, 545, 546, 547, 547, 30, 0, 0, 20, 20, 0, 251, 38, 38, 163,
    0, 285, 285, 245, 0, 30, 0, 0, 0, 0, 129, 129, 251, 545, 545, 546,
    352, 147, 147, 353, 354, 138, 20, 20, 0, 251, 38, 38, 416, 548, 77, 163,
    0, 30, 260, 260, 0, 74, 74, 30, 0, 0, 0, 0, 0, 0, 0, 0,
    549, 549, 0, 0, 550, 38, 38, 458, 247, 356, 0, 163, 0, 73, 254, 0,
    191, 191, 0, 30, 0, 0, 551, 551, 30, 0, 0, 459, 459, 156, 460, 129,
    129, 30, 16, 16, 0, 488, 488, 0, 38, 38, 38, 416, 82, 75, 163, 0,
    30, 191, 191, 0, 129, 129, 30, 16, 16, 0, 113, 552, 431, 30, 431, 553,
    30, 74, 74, 0, 353, 554, 30, 44, 44, 0, 129, 38, 38, 416, 555, 163,
    0, 251, 251, 30, 0, 0, 0, 137, 139, 139, 102, 253, 253, 113, 431, 38,
    38, 187, 0, 0, 30, 556, 556, 557, 557, 558, 559, 137, 0, 475, 475, 137,
    0, 0, 560, 560, 137, 0, 0, 561, 561, 148, 562, 563, 78, 77, 137, 0,
    564, 565, 38, 38, 43, 0, 73, 0, 113, 113, 566, 567, 568, 147, 147, 569,
    23, 65, 58, 70, 57, 32, 32, 0, 0, 0, 100, 123, 124, 125, 191, 191,
    30, 0, 129, 129, 161, 30, 0, 38, 38, 416, 78, 0, 30, 550, 38, 38,
    295, 0, 147, 147, 147, 23, 86, 147, 147, 297, 297, 129, 129, 30, 38, 38,
    152, 15, 40, 0, 570, 129, 571, 299, 300, 301, 302, 303, 304, 305, 306, 307,
    308, 244, 309, 310, 311, 312, 313, 314, 315, 316, 317, 318, 132, 132, 133, 36,
    319, 320, 321, 322, 253, 157, 162, 162, 323, 253, 25, 25, 15, 0, 25, 25,
    98, 0, 15, 15, 0, 109, 109, 0, 40, 40, 0, 572, 572, 36, 38, 38,
    13, 163, 0, 0, 0, 395, 0, 0, 573, 113, 113, 0, 325, 325, 205, 326,
    36, 327, 328, 328, 326, 36, 232, 232, 328, 329, 330, 330, 326, 36, 232, 232,
    330, 332, 332, 129, 326, 36, 333, 333, 36, 334, 335, 232, 232, 335, 286, 286,
    205, 0, 336, 0, 38, 38, 548, 80, 163, 0, 573, 573, 38, 38, 20, 163,
    0, 17, 17, 253, 537, 537, 0, 18, 0, 251, 38, 38, 210, 0, 163, 0,
    30, 574, 574, 575, 576, 577, 522, 73, 0, 0, 0, 0, 33, 0, 38, 38,
    339, 0, 132, 253, 295, 0, 0, 129, 129, 30, 38, 38, 578, 579, 580, 581,
    582, 548, 242, 74, 163, 0, 583, 30, 0, 0, 191, 191, 0, 524, 129, 129,
    30, 0, 0, 0, 137, 137, 0, 139, 0, 237, 237, 239, 73, 73, 473, 584,
    585, 585, 278, 57, 57, 278, 154, 16, 16, 0, 113, 552, 431, 30, 431, 553,
    30, 74, 74, 0, 353, 554, 30, 44, 44, 0, 129, 38, 38, 578, 579, 580,
    581, 582, 548, 242, 74, 163, 0, 586, 30, 30, 0, 0, 0, 217, 587, 587,
    0, 588, 588, 547, 547, 30, 0, 0, 245, 245, 0, 30, 0, 0, 0, 0,
    129, 129, 251, 196, 196, 217, 285, 285, 245, 0, 30, 0, 0, 0, 0, 129,
    129, 251, 196, 196, 217, 352, 147, 147, 353, 354, 138, 589, 285, 285, 205, 590,
    591, 592, 593, 30, 0, 0, 0, 0, 0, 0, 0, 195, 195, 0, 0, 191,
    191, 0, 524, 129, 129, 30, 237, 237, 239, 73, 73, 473, 584, 585, 585, 278,
    57, 57, 278, 154, 16, 16, 0, 38, 38, 177, 178, 180, 0, 30, 594, 182,
    182, 137, 0, 495, 495, 137, 0, 38, 38, 416, 158, 68, 0, 595, 366, 110,
    596, 596, 245, 597, 597, 58, 57, 69, 70, 70, 245, 245, 0, 0, 416, 73,
    416, 73, 0, 598, 33, 33, 0, 599, 156, 156, 220, 600, 220, 221, 221, 221,
    223, 221, 221, 222, 601, 601, 602, 603, 604, 74, 113, 0, 602, 602, 603, 604,
    74, 113, 0, 147, 147, 605, 606, 233, 607, 438, 438, 30, 0, 44, 44, 563,
    77, 78, 0, 38, 38, 17, 0, 113, 113, 608, 609, 591, 30, 0, 0, 38,
    38, 40, 359, 38, 38, 163, 147, 0, 100, 0, 610, 20, 0, 251, 38, 38,
    40, 0, 38, 38, 416, 548, 611, 75, 0, 30, 129, 129, 30, 113, 431, 431,
    30, 431, 353, 129, 303, 306, 612, 613, 517, 614, 614, 615, 615, 614, 616, 30,
    0, 141, 617, 618, 123, 167, 619, 620, 124, 621, 622, 335, 21, 9, 0, 0,
    103, 0, 623, 0, 0, 0, 0, 0, 0, 0, 103, 0, 624, 625, 0, 0,
    626, 626, 627, 628, 629, 630, 631, 632, 404, 633, 634, 635, 636, 636, 637, 638,
    639, 640, 641, 642, 405, 643, 643, 643, 30, 644, 644, 645, 646, 30, 647, 648,
    649, 649, 422, 444, 444, 444, 650, 445, 445, 445, 650, 651, 30, 177, 429, 652,
    417, 415, 653, 654, 655, 656, 421, 657, 658, 451, 659, 450, 660, 661, 662, 663,
    664, 665, 413, 413, 650, 650, 30, 666, 667, 668, 669, 670, 671, 672, 673, 674,
    675, 676, 677, 678, 678, 30, 0, 0, 30, 679, 102, 102, 387, 387, 680, 681,
    681, 30, 30, 0, 0, 682, 0, 683, 625, 253, 625, 680, 684, 685, 685, 400,
    686, 687, 30, 36, 676, 688, 682, 689, 690, 691, 692, 693, 694, 695, 696, 697,
    698, 699, 699, 699, 700, 0, 0, 23, 701, 645, 702, 138, 138, 138, 353, 703,
    220, 222, 270, 704, 139, 705, 705, 705, 137, 137, 704, 30, 705, 705, 706, 707,
    57, 57, 57, 33, 254, 0, 119, 0, 191, 191, 0, 0, 0, 0, 0, 488,
    488, 488, 141, 245, 0, 488, 708, 708, 0, 119, 0, 0, 0, 0, 464, 505,
    709, 710, 575, 282, 434, 711, 712, 712, 712, 0, 0, 713, 714, 355, 536, 181,
    226, 715, 716, 716, 716, 717, 23, 718, 92, 67, 66, 719, 460, 720, 721, 722,
    380, 486, 486, 486, 723, 723, 12, 253, 253, 253, 12, 724, 725, 10, 103, 636,
    154, 154, 111, 163, 0, 0, 0, 0, 30, 0, 119, 150, 150, 148, 149, 154,
    154, 185, 163, 30, 0, 113, 0, 150, 154, 154, 172, 163, 0, 150, 154, 154,
    9, 163, 0, 502, 129, 30, 231, 154, 154, 10, 163, 0, 0, 150, 154, 154,
    165, 163, 0, 150, 154, 154, 18, 74, 163, 0, 0, 150, 154, 154, 77, 163,
    0, 0, 0, 30, 150, 154, 154, 247, 356, 163, 0, 0, 150, 154, 154, 154,
    82, 75, 163, 0, 0, 150, 154, 154, 555, 163, 0, 154, 154, 286, 205, 0,
    0, 123, 124, 125, 0, 150, 154, 154, 43, 163, 0, 73, 0, 150, 154, 154,
    78, 163, 0, 0, 0, 30, 150, 149, 148, 154, 154, 295, 163, 0, 0, 150,
    154, 154, 15, 163, 0, 154, 154, 13, 163, 0, 150, 154, 154, 80, 163, 0,
    0, 150, 154, 154, 210, 163, 0, 0, 150, 149, 148, 154, 154, 339, 163, 0,
    150, 149, 148, 154, 154, 548, 74, 163, 0, 0, 150, 154, 154, 177, 178, 180,
    163, 0, 150, 154, 154, 68, 163, 0, 73, 0, 150, 154, 154, 17, 163, 0,
    0, 150, 154, 154, 40, 163, 0, 154, 154, 75, 163, 0, 0, 150, 726, 727,
    605, 439, 439, 0, 439, 728, 729, 30, 0, 0, 30, 225, 730, 513, 320, 731,
    440, 609, 137, 732, 508, 733, 174, 734, 735, 736, 0, 0, 129, 410, 737, 44,
    44, 44, 0, 30, 0, 0, 738, 739, 740, 741, 742, 743, 743, 370, 369, 374,
    523, 744, 427, 745, 746, 747, 131, 85, 85, 748, 177, 667, 669, 749, 741, 741,
    750, 751, 371, 370, 369, 374, 30, 742, 742, 431, 431, 58, 70, 57, 201, 0,
    0, 0, 0, 0, 752, 407, 407, 0, 753, 410, 471, 471, 156, 754, 755, 756,
    0, 757, 156, 403, 0, 376, 0, 0, 0, 0, 758, 326, 326, 326, 759, 156,
    760, 0, 0, 761, 761, 761, 761, 378, 408, 410, 0, 762, 326, 326, 326, 759,
    0, 763, 0, 0, 739, 739, 30, 431, 0, 0, 0, 437, 392, 392, 0, 31,
    0, 764, 0, 0, 0, 199, 0, 30, 0, 0, 0, 431, 0, 0, 139, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 30, 0, 0, 0, 0, 765, 253,
    36, 131, 157, 683, 0, 0, 31, 101, 0, 65, 139, 400, 0, 0, 0, 0,
    214, 0, 0, 0, 254, 254, 0, 30, 0, 0, 0, 119, 165, 83, 766, 419,
    419, 190, 767, 767, 768, 348, 769, 0, 36, 0, 233, 770, 233, 278, 771, 254,
    129, 129, 772, 767, 773, 84, 774, 775, 776, 777, 778, 778, 464, 464, 779, 780,
    780, 30, 488, 488, 0, 780, 357, 348, 633, 634, 643, 781, 782, 147, 147, 783,
    70, 232, 232, 462, 70, 99, 462, 462, 99, 245, 245, 0, 0, 784, 287, 342,
    342, 785, 147, 231, 511, 512, 509, 18, 18, 21, 22, 18, 18, 462, 18, 545,
    158, 158, 0, 786, 160, 156, 157, 161, 161, 30, 0, 232, 232, 244, 138, 138,
    0, 0, 139, 0, 199, 0, 202, 202, 204, 113, 0, 457, 546, 158, 158, 0,
    161, 161, 30, 0, 232, 232, 244, 85, 787, 788, 788, 0, 0, 141, 141, 233,
    0, 0, 36, 400, 789, 0, 790, 791, 782, 403, 403, 376, 735, 735, 792, 141,
    141, 233, 98, 772, 232, 232, 793, 788, 591, 278, 794, 795, 796, 504, 33, 33,
    0, 0, 797, 488, 133, 98, 98, 205, 112, 112, 205, 798, 799, 800, 801, 129,
    129, 767, 802, 147, 0, 0, 803, 0, 30, 0, 0, 0, 0, 797, 804, 0,
    0, 129, 129, 802, 0, 804, 0, 30, 0, 0, 0, 0, 30, 0, 0, 805,
    0, 30, 0, 112, 112, 205, 98, 98, 205, 0, 0, 0, 0, 0, 0, 112,
    112, 205, 98, 98, 205, 0, 33, 248, 764, 806, 0, 0, 0, 807, 807, 0,
    0, 0, 0, 0, 423, 30, 423, 0, 0, 113, 0, 808, 808, 0, 33, 33,
    0, 0, 0, 682, 0, 809, 33, 33, 0, 147, 147, 0, 708, 708, 0, 0,
    460, 0, 119, 0, 0, 112, 810, 810, 139, 0, 0, 253, 0, 0, 0, 25,
    0, 253, 0, 0, 0, 811, 0, 0, 0, 0, 0, 812, 362, 385, 0, 30,
    30, 0, 550, 813, 12, 192, 192, 147, 17, 0, 800, 262, 263, 264, 265, 266,
    156, 267, 257, 258, 258, 73, 157, 259, 260, 260, 157, 73, 157, 259, 261, 268,
    83, 235, 269, 271, 270, 272, 272, 273, 274, 275, 276, 277, 277, 36, 278, 278,
    36, 279, 279, 0, 126, 126, 0, 135, 136, 136, 32, 137, 0, 0, 0, 139,
    33, 33, 141, 113, 113, 253, 253, 253, 253, 31, 253, 814, 814, 0, 401, 401,
    393, 815, 58, 58, 464, 70, 70, 464, 816, 816, 129, 423, 65, 464, 814, 814,
    0, 464, 440, 129, 129, 30, 0, 0, 77, 0, 0, 0, 0, 0, 103, 30,
    0, 0, 140, 30, 0, 817, 25, 818, 525, 807, 0, 0, 0, 0, 817, 25,
    818, 525, 807, 0, 0, 0, 819, 819, 138, 137, 763, 0, 30, 0, 0, 0,
    681, 681, 30, 0, 0, 0, 0, 0, 681, 0, 353, 30, 0, 89, 26, 202,
    820, 821, 822, 823, 824, 825, 826, 581, 96, 96, 96, 0, 37, 37, 73, 0,
    15, 15, 0, 61, 61, 73, 40, 0, 0, 32, 0, 827, 129, 129, 96, 145,
    129, 129, 109, 145, 495, 495, 141, 47, 828, 828, 205, 0, 124, 125, 0, 828,
    47, 384, 384, 137, 384, 829, 98, 98, 205, 0, 0, 0, 36, 615, 830, 0,
    30, 112, 112, 205, 133, 798, 799, 800, 801, 831, 112, 112, 205, 133, 798, 799,
    800, 767, 98, 0, 0, 0, 0, 36, 0, 30, 831, 830, 830, 830, 832, 30,
    0, 0, 615, 0, 794, 543, 593, 564, 798, 798, 0, 30, 0, 833, 112, 112,
    205, 133, 799, 800, 767, 798, 506, 834, 835, 836, 837, 0, 838, 839, 840, 841,
    842, 843, 534, 844, 0, 206, 845, 846, 163, 163, 739, 163, 163, 163, 156, 739,
    847, 811, 811, 847, 848, 459, 849, 223, 263, 526, 850, 850, 18, 13, 850, 850,
    0, 851, 851, 133, 133, 32, 734, 30, 705, 705, 33, 20, 0, 852, 0, 251,
    850, 54, 25, 753, 156, 25, 0, 30, 0, 0, 0, 431, 0, 551, 853, 325,
    0, 0, 0, 854, 388, 711, 0, 195, 195, 0, 0, 0, 459, 459, 855, 348,
    856, 30, 854, 388, 711, 459, 459, 855, 348, 856, 0, 0, 0, 0, 30, 0,
    0, 247, 857, 551, 551, 30, 129, 129, 30, 191, 191, 247, 16, 0, 858, 859,
    660, 860, 861, 244, 547, 862, 221, 863, 864, 525, 259, 865, 58, 58, 58, 852,
    0, 0, 30, 866, 866, 867, 343, 40, 362, 868, 138, 138, 353, 431, 73, 73,
    489, 490, 486, 708, 708, 0, 119, 0, 0, 0, 0, 869, 596, 870, 630, 871,
    631, 671, 632, 672, 629, 628, 11, 872, 873, 400, 400, 874, 36, 36, 875, 253,
    682, 253, 253, 253, 875, 253, 682, 400, 876, 877, 878, 689, 840, 879, 880, 852,
    852, 881, 67, 66, 242, 74, 0, 852, 271, 754, 157, 882, 883, 131, 884, 669,
    471, 471, 156, 460, 33, 471, 755, 885, 134, 886, 755, 887, 888, 0, 0, 0,
    562, 0, 0, 0, 0, 0, 0, 889, 73, 191, 0, 0, 0, 562, 0, 0,
    0, 0, 0, 562, 562, 562, 0, 73, 73, 0, 0, 30, 0, 0, 485, 864,
    890, 890, 248, 891, 892, 893, 191, 191, 30, 0, 0, 669, 411, 669, 669, 894,
    524, 894, 362, 895, 387, 35, 524, 524, 129, 237, 458, 896, 896, 670, 462, 896,
    0, 897, 898, 899, 12, 12, 0, 0, 900, 12, 34, 856, 901, 0, 803, 530,
    531, 902, 267, 903, 904, 891, 558, 905, 383, 383, 137, 383, 234, 535, 752, 752,
    156, 906, 0, 508, 508, 0, 906, 587, 849, 907, 907, 32, 587, 477, 588, 588,
    853, 736, 908, 472, 491, 703, 202, 206, 859, 210, 211, 23, 133, 133, 0, 30,
    0, 36, 0, 23, 909, 802, 802, 0, 36, 0, 31, 31, 36, 133, 655, 910,
    911, 912, 913, 914, 915, 0, 592, 916, 916, 916, 0, 139, 253, 36, 131, 326,
    157, 683, 725, 500, 102, 917, 918, 0, 0, 919, 919, 919, 253, 0, 582, 529,
    920, 921, 922, 923, 923, 923, 74, 254, 0, 0, 0, 119, 25, 25, 924, 3,
    3, 99, 233, 233, 99, 233, 233, 737, 99, 99, 70, 737, 99, 99, 99, 233,
    737, 737, 3, 3, 737, 925, 287, 785, 926, 241, 311, 927, 928, 929, 930, 931,
    932, 656, 805, 805, 103, 0, 30, 555, 555, 0, 31, 31, 30, 431, 662, 661,
    215, 46, 0, 359, 42, 0, 933, 934, 82, 935, 936, 330, 576, 937, 938, 939,
    57, 58, 58, 30, 353, 940, 940, 852, 940, 438, 438, 438, 438, 438, 438, 934,
    376, 376, 0, 941, 942, 70, 57, 486, 486, 249, 438, 438, 156, 934, 205, 943,
    376, 376, 0, 941, 942, 70, 57, 486, 486, 249, 438, 438, 156, 934, 205, 205,
    944, 376, 376, 0, 941, 945, 58, 486, 486, 249, 940, 940, 0, 0, 0, 191,
    0, 0, 0, 30, 30, 431, 0, 0, 0, 0, 0, 73, 0, 438, 70, 70,
    126, 565, 946, 431, 431, 431, 431, 946, 171, 947, 948, 949, 950, 107, 879, 951,
    952, 328, 805, 953, 925, 555, 59, 381, 954, 647, 7, 878, 955, 956, 957, 958,
    958, 958, 959, 960, 961, 962, 119, 963, 964, 965, 966, 967, 968, 969, 970, 971,
    972, 705, 0, 331, 0, 0, 973, 974, 763, 137, 0, 0, 0, 113, 0, 0,
    30, 0, 800, 800, 0, 0, 975, 882, 882, 976, 977, 977, 485, 977, 978, 577,
    107, 560, 0, 604, 0, 769, 979, 980, 981, 982, 232, 983, 984, 318, 985, 986,
    987, 304, 307, 988, 989, 309, 312, 423, 353, 65, 30, 65, 30, 129, 30, 423,
    30, 0, 681, 990, 423, 991, 30, 65, 423, 70, 58, 57, 992, 336, 993, 315,
    994, 884, 995, 979, 980, 981, 982, 996, 996, 301, 0, 476, 997, 997, 137, 383,
    383, 384, 368, 368, 383, 368, 368, 384, 156, 756, 156, 249, 998, 998, 998, 0,
    31, 31, 253, 756, 287, 157, 157, 999, 459, 646, 36, 673, 35, 253, 253, 385,
    0, 918, 253, 253, 918, 103, 103, 103, 103, 103, 0, 73, 113, 191, 31, 441,
    102, 30, 407, 385, 464, 0, 1000, 245, 1001, 1002, 802, 802, 0, 36, 0, 802,
    1003, 239, 603, 73, 73, 1004, 254, 1005, 1006, 1007, 1008, 60, 1009, 187, 1010, 145,
    1011, 251, 251, 30, 1012, 1012, 251, 1012, 1012, 1012, 705, 253, 416, 416, 416, 416,
    360, 112, 112, 205, 0, 0, 0, 0, 36, 0, 1013, 388, 711, 191, 191, 410,
    459, 459, 855, 348, 856, 0, 133, 788, 1014, 133, 798, 799, 800, 788, 810, 129,
    129, 21, 147, 112, 1015, 112, 993, 1016, 793, 818, 893, 1017, 0, 811, 1018, 0,
    0, 30, 0, 1018, 368, 368, 368, 0, 0, 30, 30, 0, 0, 683, 0, 0,
    31, 0, 0, 683, 0, 0, 0, 1019, 0, 30, 30, 0, 0, 683, 0, 0,
    31, 0, 0, 683, 0, 0, 0, 1019, 1020, 866, 1021, 286, 122, 61, 1022, 272,
    0, 1023, 1024, 182, 1025, 354, 25, 25, 30, 0, 0, 33, 33, 30, 0, 0,
    0, 328, 330, 32, 30, 0, 0, 0, 328, 330, 129, 129, 30, 954, 954, 36,
    31, 1026, 36, 285, 285, 205, 0, 30, 0, 0, 285, 1027, 321, 657, 1028, 380,
    380, 705, 705, 380, 253, 0, 1029, 567, 480, 677, 479, 51, 108, 1030, 213, 718,
    379, 654, 1031, 1032, 1033, 43, 658, 1034, 1035, 1036, 1037, 1038, 379, 0, 1039, 0,
    780, 0, 1040, 478, 775, 775, 775, 803, 803, 791, 637, 1041, 1042, 700, 700, 700,
    0, 30, 0, 0, 113, 1043, 431, 940, 1044, 585, 461, 461, 462, 461, 1045, 760,
    55, 1046, 176, 78, 90, 1047, 1048, 1049, 498, 705, 532, 776, 776, 776, 777, 777,
    777, 385, 500, 500, 30, 385, 385, 0, 0, 253, 100, 295, 297, 74, 74, 0,
    0, 31, 31, 297, 74, 86, 86, 86, 25, 13, 0, 0, 0, 167, 31, 31,
    297, 74, 86, 0, 0, 0, 30, 488, 488, 30, 0, 1050, 326, 326, 36, 187,
    187, 30, 0, 0, 103, 0, 0, 0, 0, 103, 103, 0, 799, 410, 410, 1051,
    23, 377, 1052, 30, 431, 1053, 507, 140, 1054, 1055, 253, 1056, 1056, 900, 1057, 1058,
    958, 900, 1059, 1059, 0, 958, 0, 1060, 1060, 0, 958, 0, 1061, 1061, 0, 1062,
    1062, 1063, 1062, 958, 1063, 1064, 1065, 591, 137, 384, 1066, 383, 997, 62, 1067, 635,
    1068, 1068, 1068, 286, 205, 0, 0, 123, 124, 125, 0, 32, 0, 123, 124, 125,
    0, 1069, 1070, 611, 1071, 64, 64, 0, 30, 933, 73, 73, 73, 57, 1072, 73,
    73, 70, 1072, 933, 933, 1073, 248, 1025, 1074, 63, 64, 601, 1072, 149, 126, 126,
    1075, 138, 157, 30, 0, 0, 148, 232, 232, 105, 341, 341, 341, 82, 75, 356,
    0, 129, 1076, 1077, 1077, 1077, 30, 1078, 1078, 356, 356, 131, 16, 16, 0, 374,
    374, 131, 1079, 247, 247, 356, 0, 74, 74, 356, 0, 13, 13, 356, 0, 1080,
    1080, 9, 343, 356, 0, 705, 705, 103, 103, 705, 137, 137, 704, 30, 705, 705,
    1081, 1082, 1083, 1026, 121, 1084, 1085, 360, 228, 314, 1086, 142, 142, 142, 1087, 395,
    142, 142, 0, 23, 15, 0, 1088, 6, 848, 209, 148, 275, 568, 1089, 814, 708,
    260, 97, 97, 97, 96, 96, 145, 147, 0, 37, 37, 73, 0, 15, 15, 0,
    61, 61, 73, 40, 0, 0, 33, 0, 150, 974, 892, 559, 1090, 15, 0, 1091,
    1092, 616, 616, 79, 91, 233, 1093, 1094, 1095, 1096, 278, 229, 276, 804, 804, 1097,
    402, 343, 30, 36, 1085, 1085, 1085, 30, 36, 1098, 1099, 895, 13, 24, 1100, 178,
    880, 1101, 235, 235, 235, 898, 147, 147, 0, 73, 1102, 74, 74, 0, 194, 194,
    0, 111, 111, 0, 30, 30, 9, 9, 0, 10, 10, 0, 174, 174, 68, 0,
    18, 18, 74, 0, 77, 77, 0, 0, 30, 0, 34, 34, 35, 36, 82, 82,
    75, 0, 175, 176, 12, 0, 78, 78, 0, 0, 30, 0, 295, 295, 0, 15,
    15, 0, 79, 79, 0, 0, 30, 0, 13, 13, 0, 80, 80, 0, 339, 339,
    0, 16, 16, 0, 17, 17, 0, 75, 75, 0, 471, 322, 253, 157, 0, 0,
    0, 0, 0, 1103, 460, 460, 73, 73, 30, 0, 0, 23, 65, 65, 1103, 0,
    0, 852, 33, 488, 112, 112, 205, 73, 73, 489, 133, 708, 1019, 801, 129, 129,
    232, 21, 129, 946, 1104, 852, 33, 0, 488, 112, 112, 205, 133, 708, 31, 801,
    946, 65, 65, 65, 852, 33, 488, 98, 798, 0, 800, 800, 112, 133, 802, 767,
    147, 801, 129, 129, 73, 73, 489, 708, 129, 946, 431, 502, 30, 129, 502, 30,
    129, 990, 30, 431, 103, 103, 0, 102, 101, 103, 103, 0, 102, 464, 810, 80,
    329, 330, 330, 36, 232, 232, 330, 327, 328, 328, 36, 232, 232, 328, 92, 640,
    641, 642, 639, 1105, 20, 1106, 1107, 610, 40, 123, 124, 125, 1087, 125, 126, 126,
    0, 0, 0, 0, 0, 0, 102, 137, 0, 139, 199, 0, 0, 0, 138, 0,
    0, 30, 30, 0, 0, 137, 30, 0, 431, 0, 0, 0, 139, 199, 0, 0,
    0, 138, 0, 0, 126, 126, 491, 0, 23, 0, 0, 1108, 222, 223, 423, 423,
    801, 801, 801, 423, 112, 133, 800, 126, 801, 1109, 1110, 1110, 873, 734, 23, 950,
    126, 126, 13, 163, 0, 126, 126, 1111, 1111, 582, 1087, 1087, 1087, 1087, 395, 0,
    0, 138, 126, 126, 1111, 1087, 582, 0, 0, 0, 136, 32, 137, 0, 138, 0,
    0, 139, 0, 140, 73, 1112, 1112, 1113, 1114, 253, 582, 138, 13, 13, 253, 15,
    15, 0, 336, 0, 549, 832, 744, 1115, 99, 1116, 0, 0, 0, 437, 441, 441,
    441, 0, 392, 1117, 1117, 1117, 0, 23, 885, 530, 531, 1106, 1107, 0, 1118, 1119,
    326, 379, 107, 907, 36, 36, 1074, 0, 0, 140, 705, 0, 0, 140, 103, 0,
    0, 0, 0, 103, 119, 900, 387, 387, 102, 342, 375, 380, 385, 627, 638, 1120,
    1052, 1121, 943, 1122, 1123, 1124, 1125, 1126, 1127, 1128, 1129, 1130, 1131, 1132, 1133, 1134,
    1135, 264, 76, 88, 1136, 119, 332, 1137, 1138, 607, 778, 431, 940, 58, 1139, 154,
    154, 38, 154, 69, 95, 38, 38, 154, 118, 38, 126, 154, 154, 38, 154, 38,
    154, 38, 57, 70, 38, 154, 38, 154, 38, 154, 712, 52, 741, 38, 118, 154,
    38, 154, 118, 154, 1068, 466, 371, 38, 118, 126, 118, 118, 38, 154, 38, 118,
    154, 38, 118, 154, 341, 118, 154, 1140, 118, 38, 118, 916, 998, 1141, 919, 142,
    716, 118, 38, 154, 495, 486, 118, 1077, 118, 38, 154, 118, 38, 118, 154, 38,
    118, 154, 38, 118, 38, 118, 154, 118, 923, 236, 38, 154, 435, 38, 38, 850,
    38, 154, 38, 118, 38, 118, 154, 1117, 126, 38, 431, 154, 38, 118, 235, 700,
    44, 699, 126, 142, 464, 210, 501, 520, 211, 457, 1142, 81, 93, 261, 408, 108,
    195, 1143, 30, 0, 0, 0, 0, 0, 1144, 339, 1145, 1146, 747, 300, 279, 0,
    1147, 1148, 1149, 572, 343, 217, 158, 158, 0, 1150, 161, 161, 30, 0, 232, 232,
    244, 458, 458, 459, 459, 156, 460, 30, 857, 461, 461, 30, 857, 735, 735, 191,
    191, 0, 30, 0, 0, 857, 488, 488, 30, 0, 18, 18, 462, 30, 64, 1151,
    219, 220, 156, 156, 220, 221, 221, 221, 222, 221, 221, 223, 156, 156, 162, 232,
    232, 244, 218, 158, 158, 0, 1152, 1152, 458, 461, 191, 247, 0, 219, 220, 156,
    156, 220, 221, 221, 221, 222, 221, 221, 223, 0, 0, 353, 199, 0, 0, 129,
    129, 1016, 0, 30, 431, 30, 940, 74, 86, 804, 16, 0, 758, 758, 407, 0,
    0, 762, 762, 0, 944, 45, 94, 66, 183, 0, 548, 66, 67, 67, 1153, 1154,
    548, 1155, 1156, 177, 1157, 1158, 941, 772, 759, 1112, 684, 684, 1112, 1159, 30, 524,
    524, 129, 237, 1112, 1113, 392, 392, 392, 1112, 1113, 0, 1159, 687, 686, 746, 1160,
    1161, 1162, 1163, 32, 1164, 1165, 1166, 1167, 1168, 1139, 1139, 332, 401, 473, 1092, 440,
    1169, 1168, 1170, 331, 331, 0, 376, 376, 232, 442, 1171, 192, 1154, 1172, 1173, 68,
    1174, 808, 420, 902, 1166, 418, 418, 69, 69, 69, 74, 0, 0, 0, 254, 0,
    0, 0, 119, 1175, 1175, 209, 494, 206, 150, 1176, 1177, 665, 674, 675, 664, 30,
    0, 0, 17, 917, 807, 807, 0, 0, 763, 387, 102, 0, 0, 30, 151, 1061,
    1178, 789, 1179, 613, 0, 977, 613, 0, 1180, 1181, 1109, 1109, 1103, 460, 23, 1109,
    458, 458, 458, 883, 883, 1141, 1141, 1141, 0, 0, 119, 1182, 242, 65, 65, 852,
    33, 488, 98, 798, 0, 800, 801, 129, 129, 112, 133, 129, 129, 232, 21, 73,
    73, 489, 708, 129, 946, 423, 423, 801, 801, 801, 423, 274, 1183, 323, 70, 70,
    70, 254, 0, 0, 126, 126, 33, 488, 488, 488, 141, 245, 0, 488, 708, 708,
    0, 119, 0, 0, 0, 0, 1184, 48, 52, 52, 52, 0, 71, 40, 0, 1185,
    191, 1186, 1187, 1188, 666, 1140, 1140, 1140, 1189, 1190, 248, 504, 374, 371, 370, 369,
    30, 1191, 1191, 1191, 248, 504, 374, 370, 369, 715, 375, 858, 1048, 69, 272, 0,
    139, 0, 73, 0, 0, 0, 30, 162, 164, 544, 362, 353, 353, 30, 30, 0,
    0, 0, 138, 0, 112, 133, 353, 1192, 651, 1193, 1194, 1195, 807, 650, 678, 374,
    0, 253, 0, 537, 75, 87, 265, 1196, 745, 763, 8, 1197, 1198, 1199, 624, 1200,
    1200, 654, 912, 658, 963, 959, 967, 822, 1032, 914, 1035, 965, 961, 969, 824, 1031,
    913, 1034, 964, 960, 968, 823, 1033, 915, 1036, 966, 962, 970, 825, 452, 482, 622,
    957, 971, 1037, 1173, 1160, 30, 1201, 1201, 1202, 30, 36, 36, 30, 30, 30, 36,
    36, 624, 1203, 1203, 821, 820, 30, 624, 624, 654, 912, 658, 963, 959, 967, 822,
    1032, 914, 1035, 965, 961, 969, 824, 1031, 913, 1034, 964, 960, 968, 823, 1033, 915,
    1036, 966, 962, 970, 825, 452, 482, 622, 957, 971, 1037, 1173, 1160, 821, 820, 31,
    31, 36, 438, 438, 0, 0, 0, 941, 0, 0, 0, 0, 438, 488, 488, 0,
    30, 0, 1204, 133, 98, 98, 205, 112, 112, 205, 798, 799, 800, 1019, 708, 73,
    73, 489, 147, 488, 1205, 1206, 0, 0, 0, 0, 0, 0, 0, 0, 103, 119,
    119, 103, 119, 800, 0, 0, 1207, 285, 112, 285, 112, 788, 129, 129, 1208, 436,
    582, 161, 145, 129, 129, 611, 0, 0, 0, 65, 0, 681, 129, 129, 611, 0,
    0, 0, 65, 0, 990, 129, 129, 232, 232, 21, 0, 129, 129, 129, 232, 13,
    412, 1039, 1039, 98, 30, 112, 1039, 237, 0, 1170, 22, 72, 1209, 1210, 667, 1211,
    1212, 1213, 1214, 1215, 1216, 1217, 1218, 1219, 1220, 1221, 1222, 749, 877, 1223, 1224, 1225,
    1113,
];

pub fn get_postgres_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &POSTGRES_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &POSTGRES_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &POSTGRES_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &POSTGRES_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(64), tables: &POSTGRES_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(133), tables: &POSTGRES_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(136), tables: &POSTGRES_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(159), tables: &POSTGRES_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(182), tables: &POSTGRES_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(185), tables: &POSTGRES_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(207), tables: &POSTGRES_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(210), tables: &POSTGRES_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &POSTGRES_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &POSTGRES_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &POSTGRES_TABLES }),
        "AdjacentSegment" => Some(RootGrammar { grammar_id: GrammarId(214), tables: &POSTGRES_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(218), tables: &POSTGRES_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(219), tables: &POSTGRES_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(220), tables: &POSTGRES_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(221), tables: &POSTGRES_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(222), tables: &POSTGRES_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(240), tables: &POSTGRES_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(241), tables: &POSTGRES_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(244), tables: &POSTGRES_TABLES }),
        "Allow_connectionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(245), tables: &POSTGRES_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(246), tables: &POSTGRES_TABLES }),
        "AlterAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(247), tables: &POSTGRES_TABLES }),
        "AlterDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(273), tables: &POSTGRES_TABLES }),
        "AlterDefaultPrivilegesGrantSegment" => Some(RootGrammar { grammar_id: GrammarId(328), tables: &POSTGRES_TABLES }),
        "AlterDefaultPrivilegesObjectPrivilegesSegment" => Some(RootGrammar { grammar_id: GrammarId(341), tables: &POSTGRES_TABLES }),
        "AlterDefaultPrivilegesRevokeSegment" => Some(RootGrammar { grammar_id: GrammarId(358), tables: &POSTGRES_TABLES }),
        "AlterDefaultPrivilegesSchemaObjectsSegment" => Some(RootGrammar { grammar_id: GrammarId(373), tables: &POSTGRES_TABLES }),
        "AlterDefaultPrivilegesStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(380), tables: &POSTGRES_TABLES }),
        "AlterDefaultPrivilegesToFromRolesSegment" => Some(RootGrammar { grammar_id: GrammarId(404), tables: &POSTGRES_TABLES }),
        "AlterDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(409), tables: &POSTGRES_TABLES }),
        "AlterExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(476), tables: &POSTGRES_TABLES }),
        "AlterForeignTableActionSegment" => Some(RootGrammar { grammar_id: GrammarId(584), tables: &POSTGRES_TABLES }),
        "AlterForeignTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(839), tables: &POSTGRES_TABLES }),
        "AlterFunctionActionSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &POSTGRES_TABLES }),
        "AlterFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(925), tables: &POSTGRES_TABLES }),
        "AlterIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(958), tables: &POSTGRES_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1019), tables: &POSTGRES_TABLES }),
        "AlterMaterializedViewActionSegment" => Some(RootGrammar { grammar_id: GrammarId(1020), tables: &POSTGRES_TABLES }),
        "AlterMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1085), tables: &POSTGRES_TABLES }),
        "AlterPolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1133), tables: &POSTGRES_TABLES }),
        "AlterProcedureActionSegment" => Some(RootGrammar { grammar_id: GrammarId(1163), tables: &POSTGRES_TABLES }),
        "AlterProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1192), tables: &POSTGRES_TABLES }),
        "AlterPublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1240), tables: &POSTGRES_TABLES }),
        "AlterRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1272), tables: &POSTGRES_TABLES }),
        "AlterSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1362), tables: &POSTGRES_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(1375), tables: &POSTGRES_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1426), tables: &POSTGRES_TABLES }),
        "AlterStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1449), tables: &POSTGRES_TABLES }),
        "AlterSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1478), tables: &POSTGRES_TABLES }),
        "AlterSystemStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1526), tables: &POSTGRES_TABLES }),
        "AlterTableActionSegment" => Some(RootGrammar { grammar_id: GrammarId(1549), tables: &POSTGRES_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(1789), tables: &POSTGRES_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1794), tables: &POSTGRES_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1821), tables: &POSTGRES_TABLES }),
        "AlterTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1887), tables: &POSTGRES_TABLES }),
        "AlterTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1952), tables: &POSTGRES_TABLES }),
        "AlterTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1969), tables: &POSTGRES_TABLES }),
        "AlterViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2043), tables: &POSTGRES_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2101), tables: &POSTGRES_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(2102), tables: &POSTGRES_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2103), tables: &POSTGRES_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2104), tables: &POSTGRES_TABLES }),
        "AnalyzeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2105), tables: &POSTGRES_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2124), tables: &POSTGRES_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2125), tables: &POSTGRES_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2126), tables: &POSTGRES_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2127), tables: &POSTGRES_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2128), tables: &POSTGRES_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(2139), tables: &POSTGRES_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2156), tables: &POSTGRES_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2157), tables: &POSTGRES_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2158), tables: &POSTGRES_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2161), tables: &POSTGRES_TABLES }),
        "ArrayTypeSuffixSegment" => Some(RootGrammar { grammar_id: GrammarId(2162), tables: &POSTGRES_TABLES }),
        "AsAliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2164), tables: &POSTGRES_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2167), tables: &POSTGRES_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2169), tables: &POSTGRES_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2170), tables: &POSTGRES_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2171), tables: &POSTGRES_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2172), tables: &POSTGRES_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2173), tables: &POSTGRES_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2174), tables: &POSTGRES_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2175), tables: &POSTGRES_TABLES }),
        "AttachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2176), tables: &POSTGRES_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2177), tables: &POSTGRES_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2178), tables: &POSTGRES_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2179), tables: &POSTGRES_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(2180), tables: &POSTGRES_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2182), tables: &POSTGRES_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2183), tables: &POSTGRES_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2184), tables: &POSTGRES_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2185), tables: &POSTGRES_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(2186), tables: &POSTGRES_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(2187), tables: &POSTGRES_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(2201), tables: &POSTGRES_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(2202), tables: &POSTGRES_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2203), tables: &POSTGRES_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2204), tables: &POSTGRES_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2205), tables: &POSTGRES_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2206), tables: &POSTGRES_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2207), tables: &POSTGRES_TABLES }),
        "BigserialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2208), tables: &POSTGRES_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2209), tables: &POSTGRES_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2210), tables: &POSTGRES_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2216), tables: &POSTGRES_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2217), tables: &POSTGRES_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2218), tables: &POSTGRES_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2219), tables: &POSTGRES_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(2220), tables: &POSTGRES_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(2221), tables: &POSTGRES_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(2224), tables: &POSTGRES_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(2225), tables: &POSTGRES_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(2228), tables: &POSTGRES_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2229), tables: &POSTGRES_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2230), tables: &POSTGRES_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2233), tables: &POSTGRES_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2234), tables: &POSTGRES_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2237), tables: &POSTGRES_TABLES }),
        "BoxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2238), tables: &POSTGRES_TABLES }),
        "BpcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2239), tables: &POSTGRES_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(2240), tables: &POSTGRES_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(2243), tables: &POSTGRES_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(2246), tables: &POSTGRES_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2247), tables: &POSTGRES_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2249), tables: &POSTGRES_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2250), tables: &POSTGRES_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2251), tables: &POSTGRES_TABLES }),
        "BuffersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2252), tables: &POSTGRES_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2253), tables: &POSTGRES_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2254), tables: &POSTGRES_TABLES }),
        "BypassrlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2255), tables: &POSTGRES_TABLES }),
        "ByteaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2256), tables: &POSTGRES_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(2257), tables: &POSTGRES_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2259), tables: &POSTGRES_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2286), tables: &POSTGRES_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2287), tables: &POSTGRES_TABLES }),
        "CallStoredProcedureSegment" => Some(RootGrammar { grammar_id: GrammarId(2288), tables: &POSTGRES_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2291), tables: &POSTGRES_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2292), tables: &POSTGRES_TABLES }),
        "CascadeRestrictGrammar" => Some(RootGrammar { grammar_id: GrammarId(2293), tables: &POSTGRES_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2296), tables: &POSTGRES_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2297), tables: &POSTGRES_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2321), tables: &POSTGRES_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2322), tables: &POSTGRES_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2323), tables: &POSTGRES_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2324), tables: &POSTGRES_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2325), tables: &POSTGRES_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2326), tables: &POSTGRES_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(2327), tables: &POSTGRES_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2328), tables: &POSTGRES_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2329), tables: &POSTGRES_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2330), tables: &POSTGRES_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2331), tables: &POSTGRES_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2332), tables: &POSTGRES_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2333), tables: &POSTGRES_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2334), tables: &POSTGRES_TABLES }),
        "CidrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2335), tables: &POSTGRES_TABLES }),
        "CircleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2336), tables: &POSTGRES_TABLES }),
        "CircularstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2337), tables: &POSTGRES_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2338), tables: &POSTGRES_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2339), tables: &POSTGRES_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2340), tables: &POSTGRES_TABLES }),
        "ClusterStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2341), tables: &POSTGRES_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2354), tables: &POSTGRES_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2355), tables: &POSTGRES_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(2356), tables: &POSTGRES_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(2357), tables: &POSTGRES_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2358), tables: &POSTGRES_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2359), tables: &POSTGRES_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2360), tables: &POSTGRES_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(2366), tables: &POSTGRES_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(2367), tables: &POSTGRES_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(2368), tables: &POSTGRES_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(2369), tables: &POSTGRES_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(2374), tables: &POSTGRES_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2453), tables: &POSTGRES_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2461), tables: &POSTGRES_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2462), tables: &POSTGRES_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2463), tables: &POSTGRES_TABLES }),
        "ColumnTypeReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2489), tables: &POSTGRES_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2493), tables: &POSTGRES_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2494), tables: &POSTGRES_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2495), tables: &POSTGRES_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(2496), tables: &POSTGRES_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2497), tables: &POSTGRES_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(2498), tables: &POSTGRES_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2499), tables: &POSTGRES_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2502), tables: &POSTGRES_TABLES }),
        "CommentOnStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2503), tables: &POSTGRES_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(2614), tables: &POSTGRES_TABLES }),
        "CommentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2615), tables: &POSTGRES_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2616), tables: &POSTGRES_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2617), tables: &POSTGRES_TABLES }),
        "CommutatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2618), tables: &POSTGRES_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2619), tables: &POSTGRES_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2644), tables: &POSTGRES_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2645), tables: &POSTGRES_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2646), tables: &POSTGRES_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2647), tables: &POSTGRES_TABLES }),
        "CompositeValueExpansionSegment" => Some(RootGrammar { grammar_id: GrammarId(2648), tables: &POSTGRES_TABLES }),
        "CompoundcurveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2653), tables: &POSTGRES_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2654), tables: &POSTGRES_TABLES }),
        "CompressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2655), tables: &POSTGRES_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2656), tables: &POSTGRES_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(2657), tables: &POSTGRES_TABLES }),
        "ConcurrentlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2660), tables: &POSTGRES_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2661), tables: &POSTGRES_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2662), tables: &POSTGRES_TABLES }),
        "ConfigurationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2666), tables: &POSTGRES_TABLES }),
        "ConflictActionSegment" => Some(RootGrammar { grammar_id: GrammarId(2667), tables: &POSTGRES_TABLES }),
        "ConflictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2703), tables: &POSTGRES_TABLES }),
        "ConflictTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(2704), tables: &POSTGRES_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2725), tables: &POSTGRES_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2726), tables: &POSTGRES_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2727), tables: &POSTGRES_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2728), tables: &POSTGRES_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2729), tables: &POSTGRES_TABLES }),
        "ContentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2730), tables: &POSTGRES_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2731), tables: &POSTGRES_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2732), tables: &POSTGRES_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2733), tables: &POSTGRES_TABLES }),
        "CopyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2734), tables: &POSTGRES_TABLES }),
        "CorrelationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2893), tables: &POSTGRES_TABLES }),
        "CostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2894), tables: &POSTGRES_TABLES }),
        "CostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2895), tables: &POSTGRES_TABLES }),
        "CreateAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2896), tables: &POSTGRES_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2904), tables: &POSTGRES_TABLES }),
        "CreateCollationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2930), tables: &POSTGRES_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2968), tables: &POSTGRES_TABLES }),
        "CreateDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3021), tables: &POSTGRES_TABLES }),
        "CreateExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3046), tables: &POSTGRES_TABLES }),
        "CreateForeignDataWrapperStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3062), tables: &POSTGRES_TABLES }),
        "CreateForeignTableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3070), tables: &POSTGRES_TABLES }),
        "CreateForeignTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3074), tables: &POSTGRES_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3130), tables: &POSTGRES_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3157), tables: &POSTGRES_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3192), tables: &POSTGRES_TABLES }),
        "CreateMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3193), tables: &POSTGRES_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3227), tables: &POSTGRES_TABLES }),
        "CreateOperatorStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3247), tables: &POSTGRES_TABLES }),
        "CreatePolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3282), tables: &POSTGRES_TABLES }),
        "CreateProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3319), tables: &POSTGRES_TABLES }),
        "CreatePublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3326), tables: &POSTGRES_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3342), tables: &POSTGRES_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3406), tables: &POSTGRES_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(3416), tables: &POSTGRES_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3460), tables: &POSTGRES_TABLES }),
        "CreateServerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3468), tables: &POSTGRES_TABLES }),
        "CreateStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3482), tables: &POSTGRES_TABLES }),
        "CreateSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3502), tables: &POSTGRES_TABLES }),
        "CreateTableAsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3514), tables: &POSTGRES_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3579), tables: &POSTGRES_TABLES }),
        "CreateTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3687), tables: &POSTGRES_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3703), tables: &POSTGRES_TABLES }),
        "CreateTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3779), tables: &POSTGRES_TABLES }),
        "CreateUserMappingGrammar" => Some(RootGrammar { grammar_id: GrammarId(3791), tables: &POSTGRES_TABLES }),
        "CreateUserMappingStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3795), tables: &POSTGRES_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3806), tables: &POSTGRES_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3810), tables: &POSTGRES_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3828), tables: &POSTGRES_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3829), tables: &POSTGRES_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3830), tables: &POSTGRES_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3831), tables: &POSTGRES_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3832), tables: &POSTGRES_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3833), tables: &POSTGRES_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3834), tables: &POSTGRES_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3835), tables: &POSTGRES_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3841), tables: &POSTGRES_TABLES }),
        "Current_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3842), tables: &POSTGRES_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3843), tables: &POSTGRES_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3844), tables: &POSTGRES_TABLES }),
        "Current_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3845), tables: &POSTGRES_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3846), tables: &POSTGRES_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3847), tables: &POSTGRES_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3848), tables: &POSTGRES_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3849), tables: &POSTGRES_TABLES }),
        "CurvepolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3850), tables: &POSTGRES_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3851), tables: &POSTGRES_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3852), tables: &POSTGRES_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3853), tables: &POSTGRES_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3858), tables: &POSTGRES_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3859), tables: &POSTGRES_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3863), tables: &POSTGRES_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3966), tables: &POSTGRES_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(3967), tables: &POSTGRES_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3968), tables: &POSTGRES_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(3969), tables: &POSTGRES_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(3975), tables: &POSTGRES_TABLES }),
        "DateTimeTypeIdentifier" => Some(RootGrammar { grammar_id: GrammarId(3978), tables: &POSTGRES_TABLES }),
        "DaterangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3988), tables: &POSTGRES_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3989), tables: &POSTGRES_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(3990), tables: &POSTGRES_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3991), tables: &POSTGRES_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3992), tables: &POSTGRES_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3993), tables: &POSTGRES_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3994), tables: &POSTGRES_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3995), tables: &POSTGRES_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3996), tables: &POSTGRES_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3997), tables: &POSTGRES_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3998), tables: &POSTGRES_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3999), tables: &POSTGRES_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4000), tables: &POSTGRES_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4001), tables: &POSTGRES_TABLES }),
        "DeallocateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4002), tables: &POSTGRES_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4008), tables: &POSTGRES_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4009), tables: &POSTGRES_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4010), tables: &POSTGRES_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(239), tables: &POSTGRES_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4011), tables: &POSTGRES_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(4012), tables: &POSTGRES_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4015), tables: &POSTGRES_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4016), tables: &POSTGRES_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4017), tables: &POSTGRES_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4018), tables: &POSTGRES_TABLES }),
        "DefinitionArgumentValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(4019), tables: &POSTGRES_TABLES }),
        "DefinitionParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(4022), tables: &POSTGRES_TABLES }),
        "DefinitionParametersSegment" => Some(RootGrammar { grammar_id: GrammarId(4027), tables: &POSTGRES_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4030), tables: &POSTGRES_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4031), tables: &POSTGRES_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4032), tables: &POSTGRES_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4033), tables: &POSTGRES_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(4062), tables: &POSTGRES_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4063), tables: &POSTGRES_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4064), tables: &POSTGRES_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4065), tables: &POSTGRES_TABLES }),
        "DependenciesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4066), tables: &POSTGRES_TABLES }),
        "DependsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4067), tables: &POSTGRES_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4068), tables: &POSTGRES_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4069), tables: &POSTGRES_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4070), tables: &POSTGRES_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4071), tables: &POSTGRES_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4075), tables: &POSTGRES_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4076), tables: &POSTGRES_TABLES }),
        "DetachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4077), tables: &POSTGRES_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4078), tables: &POSTGRES_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4079), tables: &POSTGRES_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4080), tables: &POSTGRES_TABLES }),
        "Disable_page_skippingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4081), tables: &POSTGRES_TABLES }),
        "DiscardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4082), tables: &POSTGRES_TABLES }),
        "DiscardStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4083), tables: &POSTGRES_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4091), tables: &POSTGRES_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &POSTGRES_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4093), tables: &POSTGRES_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4094), tables: &POSTGRES_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4095), tables: &POSTGRES_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(4096), tables: &POSTGRES_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4097), tables: &POSTGRES_TABLES }),
        "DoStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4098), tables: &POSTGRES_TABLES }),
        "DocumentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4107), tables: &POSTGRES_TABLES }),
        "DollarNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4108), tables: &POSTGRES_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4109), tables: &POSTGRES_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(4110), tables: &POSTGRES_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4111), tables: &POSTGRES_TABLES }),
        "DropAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4112), tables: &POSTGRES_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4124), tables: &POSTGRES_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4127), tables: &POSTGRES_TABLES }),
        "DropCollationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4136), tables: &POSTGRES_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4142), tables: &POSTGRES_TABLES }),
        "DropDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4151), tables: &POSTGRES_TABLES }),
        "DropExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4158), tables: &POSTGRES_TABLES }),
        "DropForeignTableStatement" => Some(RootGrammar { grammar_id: GrammarId(4164), tables: &POSTGRES_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4172), tables: &POSTGRES_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4181), tables: &POSTGRES_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4189), tables: &POSTGRES_TABLES }),
        "DropMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4190), tables: &POSTGRES_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4198), tables: &POSTGRES_TABLES }),
        "DropOwnedStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4203), tables: &POSTGRES_TABLES }),
        "DropPolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4214), tables: &POSTGRES_TABLES }),
        "DropProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4222), tables: &POSTGRES_TABLES }),
        "DropPublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4233), tables: &POSTGRES_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4240), tables: &POSTGRES_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4245), tables: &POSTGRES_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4251), tables: &POSTGRES_TABLES }),
        "DropStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4258), tables: &POSTGRES_TABLES }),
        "DropSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4267), tables: &POSTGRES_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4275), tables: &POSTGRES_TABLES }),
        "DropTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4283), tables: &POSTGRES_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4291), tables: &POSTGRES_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4299), tables: &POSTGRES_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4306), tables: &POSTGRES_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4311), tables: &POSTGRES_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4318), tables: &POSTGRES_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4319), tables: &POSTGRES_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4320), tables: &POSTGRES_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4321), tables: &POSTGRES_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4324), tables: &POSTGRES_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4325), tables: &POSTGRES_TABLES }),
        "EmptyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4326), tables: &POSTGRES_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(4327), tables: &POSTGRES_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4328), tables: &POSTGRES_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4331), tables: &POSTGRES_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4332), tables: &POSTGRES_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4333), tables: &POSTGRES_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4334), tables: &POSTGRES_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(231), tables: &POSTGRES_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(4335), tables: &POSTGRES_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4336), tables: &POSTGRES_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2155), tables: &POSTGRES_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4337), tables: &POSTGRES_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(4338), tables: &POSTGRES_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4339), tables: &POSTGRES_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4340), tables: &POSTGRES_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4341), tables: &POSTGRES_TABLES }),
        "EventKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4342), tables: &POSTGRES_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4343), tables: &POSTGRES_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4344), tables: &POSTGRES_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4345), tables: &POSTGRES_TABLES }),
        "ExclusionConstraintElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4346), tables: &POSTGRES_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4350), tables: &POSTGRES_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4351), tables: &POSTGRES_TABLES }),
        "ExecuteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4352), tables: &POSTGRES_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4358), tables: &POSTGRES_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4359), tables: &POSTGRES_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4360), tables: &POSTGRES_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4361), tables: &POSTGRES_TABLES }),
        "ExplainOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(4362), tables: &POSTGRES_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4382), tables: &POSTGRES_TABLES }),
        "ExpressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4398), tables: &POSTGRES_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4399), tables: &POSTGRES_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4400), tables: &POSTGRES_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4426), tables: &POSTGRES_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4433), tables: &POSTGRES_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4443), tables: &POSTGRES_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4448), tables: &POSTGRES_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4467), tables: &POSTGRES_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(4504), tables: &POSTGRES_TABLES }),
        "ExtendedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4512), tables: &POSTGRES_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4513), tables: &POSTGRES_TABLES }),
        "ExtendedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(4514), tables: &POSTGRES_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4524), tables: &POSTGRES_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4525), tables: &POSTGRES_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4526), tables: &POSTGRES_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4527), tables: &POSTGRES_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(4528), tables: &POSTGRES_TABLES }),
        "FamilyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4529), tables: &POSTGRES_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4530), tables: &POSTGRES_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4547), tables: &POSTGRES_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4548), tables: &POSTGRES_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4549), tables: &POSTGRES_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(4550), tables: &POSTGRES_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4558), tables: &POSTGRES_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(4559), tables: &POSTGRES_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4565), tables: &POSTGRES_TABLES }),
        "FinalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4566), tables: &POSTGRES_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4567), tables: &POSTGRES_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4568), tables: &POSTGRES_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4569), tables: &POSTGRES_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4570), tables: &POSTGRES_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4571), tables: &POSTGRES_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4572), tables: &POSTGRES_TABLES }),
        "ForClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4573), tables: &POSTGRES_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4594), tables: &POSTGRES_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4595), tables: &POSTGRES_TABLES }),
        "Force_not_nullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4596), tables: &POSTGRES_TABLES }),
        "Force_nullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4597), tables: &POSTGRES_TABLES }),
        "Force_quoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4598), tables: &POSTGRES_TABLES }),
        "ForeignDataWrapperGrammar" => Some(RootGrammar { grammar_id: GrammarId(4599), tables: &POSTGRES_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(4603), tables: &POSTGRES_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4606), tables: &POSTGRES_TABLES }),
        "ForeignTableColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(4607), tables: &POSTGRES_TABLES }),
        "ForeignTableTableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(4636), tables: &POSTGRES_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4647), tables: &POSTGRES_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4648), tables: &POSTGRES_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4649), tables: &POSTGRES_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(4679), tables: &POSTGRES_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4683), tables: &POSTGRES_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4684), tables: &POSTGRES_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4685), tables: &POSTGRES_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4686), tables: &POSTGRES_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4690), tables: &POSTGRES_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4708), tables: &POSTGRES_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4731), tables: &POSTGRES_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4758), tables: &POSTGRES_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4759), tables: &POSTGRES_TABLES }),
        "FullTextSearchOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(4760), tables: &POSTGRES_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4761), tables: &POSTGRES_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4762), tables: &POSTGRES_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4770), tables: &POSTGRES_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(4831), tables: &POSTGRES_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4834), tables: &POSTGRES_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4936), tables: &POSTGRES_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4937), tables: &POSTGRES_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4938), tables: &POSTGRES_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(4948), tables: &POSTGRES_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(4972), tables: &POSTGRES_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(4975), tables: &POSTGRES_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4989), tables: &POSTGRES_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4990), tables: &POSTGRES_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4991), tables: &POSTGRES_TABLES }),
        "GeographyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4992), tables: &POSTGRES_TABLES }),
        "GeometryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4993), tables: &POSTGRES_TABLES }),
        "GeometrycollectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4994), tables: &POSTGRES_TABLES }),
        "GeometrycollectionmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4995), tables: &POSTGRES_TABLES }),
        "GeometrycollectionzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4996), tables: &POSTGRES_TABLES }),
        "GeometrycollectionzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4997), tables: &POSTGRES_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(4998), tables: &POSTGRES_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4999), tables: &POSTGRES_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5000), tables: &POSTGRES_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5001), tables: &POSTGRES_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5039), tables: &POSTGRES_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5040), tables: &POSTGRES_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(5041), tables: &POSTGRES_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(5044), tables: &POSTGRES_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5045), tables: &POSTGRES_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5046), tables: &POSTGRES_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5065), tables: &POSTGRES_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5074), tables: &POSTGRES_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(5075), tables: &POSTGRES_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5083), tables: &POSTGRES_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5084), tables: &POSTGRES_TABLES }),
        "GroupsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5092), tables: &POSTGRES_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5093), tables: &POSTGRES_TABLES }),
        "HashKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5094), tables: &POSTGRES_TABLES }),
        "HashesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5095), tables: &POSTGRES_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5096), tables: &POSTGRES_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5101), tables: &POSTGRES_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5109), tables: &POSTGRES_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5110), tables: &POSTGRES_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5111), tables: &POSTGRES_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5112), tables: &POSTGRES_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5113), tables: &POSTGRES_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5114), tables: &POSTGRES_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5115), tables: &POSTGRES_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5116), tables: &POSTGRES_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5117), tables: &POSTGRES_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5118), tables: &POSTGRES_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5119), tables: &POSTGRES_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5120), tables: &POSTGRES_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5121), tables: &POSTGRES_TABLES }),
        "IcuKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5122), tables: &POSTGRES_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5123), tables: &POSTGRES_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5124), tables: &POSTGRES_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5125), tables: &POSTGRES_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5126), tables: &POSTGRES_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5127), tables: &POSTGRES_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5128), tables: &POSTGRES_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5131), tables: &POSTGRES_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5132), tables: &POSTGRES_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5136), tables: &POSTGRES_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5137), tables: &POSTGRES_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5142), tables: &POSTGRES_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5143), tables: &POSTGRES_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5144), tables: &POSTGRES_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(2300), tables: &POSTGRES_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5145), tables: &POSTGRES_TABLES }),
        "ImportForeignSchemaGrammar" => Some(RootGrammar { grammar_id: GrammarId(5146), tables: &POSTGRES_TABLES }),
        "ImportForeignSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5150), tables: &POSTGRES_TABLES }),
        "ImportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5168), tables: &POSTGRES_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5169), tables: &POSTGRES_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5170), tables: &POSTGRES_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5171), tables: &POSTGRES_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5181), tables: &POSTGRES_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5182), tables: &POSTGRES_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5183), tables: &POSTGRES_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(223), tables: &POSTGRES_TABLES }),
        "IndexAccessMethodSegment" => Some(RootGrammar { grammar_id: GrammarId(5184), tables: &POSTGRES_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(5185), tables: &POSTGRES_TABLES }),
        "IndexElementOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(5190), tables: &POSTGRES_TABLES }),
        "IndexElementSegment" => Some(RootGrammar { grammar_id: GrammarId(5210), tables: &POSTGRES_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5217), tables: &POSTGRES_TABLES }),
        "IndexParametersSegment" => Some(RootGrammar { grammar_id: GrammarId(5218), tables: &POSTGRES_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "Index_cleanupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5230), tables: &POSTGRES_TABLES }),
        "IndexesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5231), tables: &POSTGRES_TABLES }),
        "InetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5232), tables: &POSTGRES_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5233), tables: &POSTGRES_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5234), tables: &POSTGRES_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5235), tables: &POSTGRES_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5236), tables: &POSTGRES_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5237), tables: &POSTGRES_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5238), tables: &POSTGRES_TABLES }),
        "InlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5239), tables: &POSTGRES_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5240), tables: &POSTGRES_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5241), tables: &POSTGRES_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5242), tables: &POSTGRES_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5243), tables: &POSTGRES_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5244), tables: &POSTGRES_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5245), tables: &POSTGRES_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5275), tables: &POSTGRES_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5276), tables: &POSTGRES_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5277), tables: &POSTGRES_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5278), tables: &POSTGRES_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5279), tables: &POSTGRES_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5280), tables: &POSTGRES_TABLES }),
        "Int4rangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5281), tables: &POSTGRES_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5282), tables: &POSTGRES_TABLES }),
        "Int8rangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5283), tables: &POSTGRES_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5284), tables: &POSTGRES_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5285), tables: &POSTGRES_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5286), tables: &POSTGRES_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5287), tables: &POSTGRES_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5288), tables: &POSTGRES_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5289), tables: &POSTGRES_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5303), tables: &POSTGRES_TABLES }),
        "IntervalUnitsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5304), tables: &POSTGRES_TABLES }),
        "IntoClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5311), tables: &POSTGRES_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5319), tables: &POSTGRES_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5320), tables: &POSTGRES_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(5321), tables: &POSTGRES_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(5328), tables: &POSTGRES_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5333), tables: &POSTGRES_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(5334), tables: &POSTGRES_TABLES }),
        "Is_superuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5335), tables: &POSTGRES_TABLES }),
        "Is_templateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5336), tables: &POSTGRES_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5337), tables: &POSTGRES_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5338), tables: &POSTGRES_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5339), tables: &POSTGRES_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5340), tables: &POSTGRES_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5341), tables: &POSTGRES_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5363), tables: &POSTGRES_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5364), tables: &POSTGRES_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(5366), tables: &POSTGRES_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(5367), tables: &POSTGRES_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5374), tables: &POSTGRES_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5382), tables: &POSTGRES_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5387), tables: &POSTGRES_TABLES }),
        "JsonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5388), tables: &POSTGRES_TABLES }),
        "JsonTestGrammar" => Some(RootGrammar { grammar_id: GrammarId(5389), tables: &POSTGRES_TABLES }),
        "JsonTypeGrammar" => Some(RootGrammar { grammar_id: GrammarId(5393), tables: &POSTGRES_TABLES }),
        "JsonUniqueKeysGrammar" => Some(RootGrammar { grammar_id: GrammarId(5398), tables: &POSTGRES_TABLES }),
        "JsonbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5405), tables: &POSTGRES_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5406), tables: &POSTGRES_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5407), tables: &POSTGRES_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5408), tables: &POSTGRES_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5409), tables: &POSTGRES_TABLES }),
        "LabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5410), tables: &POSTGRES_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5411), tables: &POSTGRES_TABLES }),
        "LanguageClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5412), tables: &POSTGRES_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5417), tables: &POSTGRES_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5418), tables: &POSTGRES_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5419), tables: &POSTGRES_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5420), tables: &POSTGRES_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5421), tables: &POSTGRES_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5422), tables: &POSTGRES_TABLES }),
        "Lc_collateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5423), tables: &POSTGRES_TABLES }),
        "Lc_ctypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5424), tables: &POSTGRES_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5425), tables: &POSTGRES_TABLES }),
        "LeakproofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5426), tables: &POSTGRES_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5427), tables: &POSTGRES_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5428), tables: &POSTGRES_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5429), tables: &POSTGRES_TABLES }),
        "LeftargKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5430), tables: &POSTGRES_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5431), tables: &POSTGRES_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(5432), tables: &POSTGRES_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(5435), tables: &POSTGRES_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5436), tables: &POSTGRES_TABLES }),
        "LibcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5437), tables: &POSTGRES_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5438), tables: &POSTGRES_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(5446), tables: &POSTGRES_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5452), tables: &POSTGRES_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5453), tables: &POSTGRES_TABLES }),
        "LikeOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(5454), tables: &POSTGRES_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5468), tables: &POSTGRES_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5485), tables: &POSTGRES_TABLES }),
        "LineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5486), tables: &POSTGRES_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5487), tables: &POSTGRES_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5488), tables: &POSTGRES_TABLES }),
        "LinestringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5489), tables: &POSTGRES_TABLES }),
        "LinestringmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5490), tables: &POSTGRES_TABLES }),
        "LinestringzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5491), tables: &POSTGRES_TABLES }),
        "LinestringzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5492), tables: &POSTGRES_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5493), tables: &POSTGRES_TABLES }),
        "ListKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5494), tables: &POSTGRES_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5495), tables: &POSTGRES_TABLES }),
        "ListenStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5496), tables: &POSTGRES_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(5499), tables: &POSTGRES_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5511), tables: &POSTGRES_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5512), tables: &POSTGRES_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5513), tables: &POSTGRES_TABLES }),
        "LoadStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5514), tables: &POSTGRES_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(5517), tables: &POSTGRES_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5518), tables: &POSTGRES_TABLES }),
        "LocaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5519), tables: &POSTGRES_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5520), tables: &POSTGRES_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5521), tables: &POSTGRES_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5522), tables: &POSTGRES_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5523), tables: &POSTGRES_TABLES }),
        "LockTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5524), tables: &POSTGRES_TABLES }),
        "LockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5559), tables: &POSTGRES_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5560), tables: &POSTGRES_TABLES }),
        "LoggedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5561), tables: &POSTGRES_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5562), tables: &POSTGRES_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5563), tables: &POSTGRES_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5564), tables: &POSTGRES_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5565), tables: &POSTGRES_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5566), tables: &POSTGRES_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5567), tables: &POSTGRES_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5568), tables: &POSTGRES_TABLES }),
        "LsegKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5569), tables: &POSTGRES_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5570), tables: &POSTGRES_TABLES }),
        "Macaddr8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5582), tables: &POSTGRES_TABLES }),
        "MacaddrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5583), tables: &POSTGRES_TABLES }),
        "MainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5584), tables: &POSTGRES_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5585), tables: &POSTGRES_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(5586), tables: &POSTGRES_TABLES }),
        "MappingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5587), tables: &POSTGRES_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5588), tables: &POSTGRES_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(5589), tables: &POSTGRES_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5590), tables: &POSTGRES_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5591), tables: &POSTGRES_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5592), tables: &POSTGRES_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5593), tables: &POSTGRES_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5594), tables: &POSTGRES_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5595), tables: &POSTGRES_TABLES }),
        "McvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5596), tables: &POSTGRES_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5597), tables: &POSTGRES_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5598), tables: &POSTGRES_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5599), tables: &POSTGRES_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5600), tables: &POSTGRES_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5601), tables: &POSTGRES_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(5605), tables: &POSTGRES_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5608), tables: &POSTGRES_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(5609), tables: &POSTGRES_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5613), tables: &POSTGRES_TABLES }),
        "MergeNotMatchedBySourceClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5626), tables: &POSTGRES_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5642), tables: &POSTGRES_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5658), tables: &POSTGRES_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5678), tables: &POSTGRES_TABLES }),
        "MergesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5681), tables: &POSTGRES_TABLES }),
        "MetaCommandQueryBufferSegment" => Some(RootGrammar { grammar_id: GrammarId(5682), tables: &POSTGRES_TABLES }),
        "MetaCommandQueryBufferStatement" => Some(RootGrammar { grammar_id: GrammarId(5683), tables: &POSTGRES_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5688), tables: &POSTGRES_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5689), tables: &POSTGRES_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5690), tables: &POSTGRES_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5691), tables: &POSTGRES_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5692), tables: &POSTGRES_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(5693), tables: &POSTGRES_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5694), tables: &POSTGRES_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5695), tables: &POSTGRES_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5696), tables: &POSTGRES_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5697), tables: &POSTGRES_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5698), tables: &POSTGRES_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5699), tables: &POSTGRES_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5700), tables: &POSTGRES_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5701), tables: &POSTGRES_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5702), tables: &POSTGRES_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(5703), tables: &POSTGRES_TABLES }),
        "ModulusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5704), tables: &POSTGRES_TABLES }),
        "MoneyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5705), tables: &POSTGRES_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5706), tables: &POSTGRES_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5707), tables: &POSTGRES_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5708), tables: &POSTGRES_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5709), tables: &POSTGRES_TABLES }),
        "MulticurveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5710), tables: &POSTGRES_TABLES }),
        "MultilineConcatenateDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(5711), tables: &POSTGRES_TABLES }),
        "MultilineConcatenateNewline" => Some(RootGrammar { grammar_id: GrammarId(5713), tables: &POSTGRES_TABLES }),
        "MultilinestringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5714), tables: &POSTGRES_TABLES }),
        "MultilinestringmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5715), tables: &POSTGRES_TABLES }),
        "MultilinestringzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5716), tables: &POSTGRES_TABLES }),
        "MultilinestringzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5717), tables: &POSTGRES_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(5718), tables: &POSTGRES_TABLES }),
        "MultipointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5719), tables: &POSTGRES_TABLES }),
        "MultipointmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5720), tables: &POSTGRES_TABLES }),
        "MultipointzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5721), tables: &POSTGRES_TABLES }),
        "MultipointzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5722), tables: &POSTGRES_TABLES }),
        "MultipolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5723), tables: &POSTGRES_TABLES }),
        "MultipolygonmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5724), tables: &POSTGRES_TABLES }),
        "MultipolygonzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5725), tables: &POSTGRES_TABLES }),
        "MultipolygonzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5726), tables: &POSTGRES_TABLES }),
        "MultisurfaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5727), tables: &POSTGRES_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5728), tables: &POSTGRES_TABLES }),
        "NakedIdentifierFullSegment" => Some(RootGrammar { grammar_id: GrammarId(5729), tables: &POSTGRES_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5730), tables: &POSTGRES_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5731), tables: &POSTGRES_TABLES }),
        "NamedArgumentSegment" => Some(RootGrammar { grammar_id: GrammarId(5732), tables: &POSTGRES_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5739), tables: &POSTGRES_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(5746), tables: &POSTGRES_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5750), tables: &POSTGRES_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5751), tables: &POSTGRES_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5752), tables: &POSTGRES_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5753), tables: &POSTGRES_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5754), tables: &POSTGRES_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5757), tables: &POSTGRES_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5758), tables: &POSTGRES_TABLES }),
        "NdistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5759), tables: &POSTGRES_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(5760), tables: &POSTGRES_TABLES }),
        "NegatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5761), tables: &POSTGRES_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(5762), tables: &POSTGRES_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5763), tables: &POSTGRES_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(5764), tables: &POSTGRES_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5765), tables: &POSTGRES_TABLES }),
        "NfcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5766), tables: &POSTGRES_TABLES }),
        "NfdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5767), tables: &POSTGRES_TABLES }),
        "NfkcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5768), tables: &POSTGRES_TABLES }),
        "NfkdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5769), tables: &POSTGRES_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5770), tables: &POSTGRES_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5771), tables: &POSTGRES_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5772), tables: &POSTGRES_TABLES }),
        "NobypassrlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5773), tables: &POSTGRES_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5774), tables: &POSTGRES_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5775), tables: &POSTGRES_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5776), tables: &POSTGRES_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5777), tables: &POSTGRES_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5778), tables: &POSTGRES_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5779), tables: &POSTGRES_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5780), tables: &POSTGRES_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5781), tables: &POSTGRES_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5782), tables: &POSTGRES_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(5783), tables: &POSTGRES_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5793), tables: &POSTGRES_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(5794), tables: &POSTGRES_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(5795), tables: &POSTGRES_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5804), tables: &POSTGRES_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5805), tables: &POSTGRES_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5806), tables: &POSTGRES_TABLES }),
        "NoreplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5807), tables: &POSTGRES_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5808), tables: &POSTGRES_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(5809), tables: &POSTGRES_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5816), tables: &POSTGRES_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5817), tables: &POSTGRES_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(5818), tables: &POSTGRES_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(5819), tables: &POSTGRES_TABLES }),
        "NotExtendLeftSegment" => Some(RootGrammar { grammar_id: GrammarId(5826), tables: &POSTGRES_TABLES }),
        "NotExtendRightSegment" => Some(RootGrammar { grammar_id: GrammarId(5829), tables: &POSTGRES_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5832), tables: &POSTGRES_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(5833), tables: &POSTGRES_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5834), tables: &POSTGRES_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5835), tables: &POSTGRES_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5836), tables: &POSTGRES_TABLES }),
        "NotifyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5837), tables: &POSTGRES_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5843), tables: &POSTGRES_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5844), tables: &POSTGRES_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5845), tables: &POSTGRES_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5846), tables: &POSTGRES_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5847), tables: &POSTGRES_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5848), tables: &POSTGRES_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5849), tables: &POSTGRES_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5850), tables: &POSTGRES_TABLES }),
        "NumrangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5851), tables: &POSTGRES_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5852), tables: &POSTGRES_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(5853), tables: &POSTGRES_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5857), tables: &POSTGRES_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(5861), tables: &POSTGRES_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5866), tables: &POSTGRES_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5878), tables: &POSTGRES_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5879), tables: &POSTGRES_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5880), tables: &POSTGRES_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5881), tables: &POSTGRES_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5882), tables: &POSTGRES_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5887), tables: &POSTGRES_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5888), tables: &POSTGRES_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5889), tables: &POSTGRES_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5890), tables: &POSTGRES_TABLES }),
        "OnKeywordAsIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5891), tables: &POSTGRES_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5892), tables: &POSTGRES_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5893), tables: &POSTGRES_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5894), tables: &POSTGRES_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5895), tables: &POSTGRES_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5896), tables: &POSTGRES_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5897), tables: &POSTGRES_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5898), tables: &POSTGRES_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5899), tables: &POSTGRES_TABLES }),
        "OperationClassReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5900), tables: &POSTGRES_TABLES }),
        "OperatorClassReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5901), tables: &POSTGRES_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5902), tables: &POSTGRES_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5903), tables: &POSTGRES_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5904), tables: &POSTGRES_TABLES }),
        "OptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5905), tables: &POSTGRES_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5910), tables: &POSTGRES_TABLES }),
        "OptionsListGrammar" => Some(RootGrammar { grammar_id: GrammarId(5911), tables: &POSTGRES_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5916), tables: &POSTGRES_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5917), tables: &POSTGRES_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(5918), tables: &POSTGRES_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5921), tables: &POSTGRES_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(5944), tables: &POSTGRES_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5955), tables: &POSTGRES_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(5956), tables: &POSTGRES_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5959), tables: &POSTGRES_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5960), tables: &POSTGRES_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5961), tables: &POSTGRES_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5962), tables: &POSTGRES_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5963), tables: &POSTGRES_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5964), tables: &POSTGRES_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5971), tables: &POSTGRES_TABLES }),
        "OverlapSegment" => Some(RootGrammar { grammar_id: GrammarId(5972), tables: &POSTGRES_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5975), tables: &POSTGRES_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6007), tables: &POSTGRES_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6008), tables: &POSTGRES_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6009), tables: &POSTGRES_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6010), tables: &POSTGRES_TABLES }),
        "OwnedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6011), tables: &POSTGRES_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6012), tables: &POSTGRES_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6013), tables: &POSTGRES_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6014), tables: &POSTGRES_TABLES }),
        "ParallelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6015), tables: &POSTGRES_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(6016), tables: &POSTGRES_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(6017), tables: &POSTGRES_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6018), tables: &POSTGRES_TABLES }),
        "ParserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6019), tables: &POSTGRES_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6020), tables: &POSTGRES_TABLES }),
        "PartitionBoundSpecSegment" => Some(RootGrammar { grammar_id: GrammarId(6021), tables: &POSTGRES_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6051), tables: &POSTGRES_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6058), tables: &POSTGRES_TABLES }),
        "PassingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6059), tables: &POSTGRES_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6060), tables: &POSTGRES_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6061), tables: &POSTGRES_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(6062), tables: &POSTGRES_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(6069), tables: &POSTGRES_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6070), tables: &POSTGRES_TABLES }),
        "PermissiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6071), tables: &POSTGRES_TABLES }),
        "PgTrgmOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6072), tables: &POSTGRES_TABLES }),
        "Pg_lsnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6073), tables: &POSTGRES_TABLES }),
        "PgvectorOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6074), tables: &POSTGRES_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6075), tables: &POSTGRES_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(6076), tables: &POSTGRES_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6077), tables: &POSTGRES_TABLES }),
        "PlainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6078), tables: &POSTGRES_TABLES }),
        "PlansKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6079), tables: &POSTGRES_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(6080), tables: &POSTGRES_TABLES }),
        "PointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6081), tables: &POSTGRES_TABLES }),
        "PointmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6082), tables: &POSTGRES_TABLES }),
        "PointzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6083), tables: &POSTGRES_TABLES }),
        "PointzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6084), tables: &POSTGRES_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6085), tables: &POSTGRES_TABLES }),
        "PolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6086), tables: &POSTGRES_TABLES }),
        "PolygonmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6087), tables: &POSTGRES_TABLES }),
        "PolygonzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6088), tables: &POSTGRES_TABLES }),
        "PolygonzmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6089), tables: &POSTGRES_TABLES }),
        "PolyhedralsurfaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6090), tables: &POSTGRES_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6091), tables: &POSTGRES_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(6092), tables: &POSTGRES_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6093), tables: &POSTGRES_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6097), tables: &POSTGRES_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6098), tables: &POSTGRES_TABLES }),
        "PostgisOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6099), tables: &POSTGRES_TABLES }),
        "PostgresCopyStdinDataSegment" => Some(RootGrammar { grammar_id: GrammarId(6100), tables: &POSTGRES_TABLES }),
        "PostgresCopyStdinDataStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6101), tables: &POSTGRES_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(6103), tables: &POSTGRES_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6105), tables: &POSTGRES_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6106), tables: &POSTGRES_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6107), tables: &POSTGRES_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6108), tables: &POSTGRES_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6109), tables: &POSTGRES_TABLES }),
        "PrepareStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6110), tables: &POSTGRES_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6120), tables: &POSTGRES_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6121), tables: &POSTGRES_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(6122), tables: &POSTGRES_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6125), tables: &POSTGRES_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6126), tables: &POSTGRES_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6127), tables: &POSTGRES_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6128), tables: &POSTGRES_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6129), tables: &POSTGRES_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6130), tables: &POSTGRES_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6131), tables: &POSTGRES_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6132), tables: &POSTGRES_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6133), tables: &POSTGRES_TABLES }),
        "Process_toastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6134), tables: &POSTGRES_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6135), tables: &POSTGRES_TABLES }),
        "ProgramKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6136), tables: &POSTGRES_TABLES }),
        "PropertiesNakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6137), tables: &POSTGRES_TABLES }),
        "ProviderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6138), tables: &POSTGRES_TABLES }),
        "PsqlCopyMetaCommandSegment" => Some(RootGrammar { grammar_id: GrammarId(6139), tables: &POSTGRES_TABLES }),
        "PsqlCopyMetaCommandStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6140), tables: &POSTGRES_TABLES }),
        "PsqlSetMetaCommandSegment" => Some(RootGrammar { grammar_id: GrammarId(6142), tables: &POSTGRES_TABLES }),
        "PsqlSetMetaCommandStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6143), tables: &POSTGRES_TABLES }),
        "PsqlVariableColonSegment" => Some(RootGrammar { grammar_id: GrammarId(6145), tables: &POSTGRES_TABLES }),
        "PsqlVariableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6146), tables: &POSTGRES_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6154), tables: &POSTGRES_TABLES }),
        "PublicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6155), tables: &POSTGRES_TABLES }),
        "PublicationObjectsSegment" => Some(RootGrammar { grammar_id: GrammarId(6156), tables: &POSTGRES_TABLES }),
        "PublicationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(6179), tables: &POSTGRES_TABLES }),
        "PublicationTableSegment" => Some(RootGrammar { grammar_id: GrammarId(6180), tables: &POSTGRES_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6187), tables: &POSTGRES_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6188), tables: &POSTGRES_TABLES }),
        "QualifiedOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6191), tables: &POSTGRES_TABLES }),
        "QualifiedParameterNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(6199), tables: &POSTGRES_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6204), tables: &POSTGRES_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6205), tables: &POSTGRES_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6213), tables: &POSTGRES_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6214), tables: &POSTGRES_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6215), tables: &POSTGRES_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6218), tables: &POSTGRES_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6245), tables: &POSTGRES_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6246), tables: &POSTGRES_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6247), tables: &POSTGRES_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(6248), tables: &POSTGRES_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(6249), tables: &POSTGRES_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6250), tables: &POSTGRES_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(6251), tables: &POSTGRES_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(6252), tables: &POSTGRES_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6253), tables: &POSTGRES_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6254), tables: &POSTGRES_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6255), tables: &POSTGRES_TABLES }),
        "ReassignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6256), tables: &POSTGRES_TABLES }),
        "ReassignOwnedStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6257), tables: &POSTGRES_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6273), tables: &POSTGRES_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6274), tables: &POSTGRES_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6275), tables: &POSTGRES_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6276), tables: &POSTGRES_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6277), tables: &POSTGRES_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(6291), tables: &POSTGRES_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6297), tables: &POSTGRES_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6298), tables: &POSTGRES_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6299), tables: &POSTGRES_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6300), tables: &POSTGRES_TABLES }),
        "RefreshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6314), tables: &POSTGRES_TABLES }),
        "RefreshMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6315), tables: &POSTGRES_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6322), tables: &POSTGRES_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6323), tables: &POSTGRES_TABLES }),
        "ReindexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6324), tables: &POSTGRES_TABLES }),
        "RelationOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(6357), tables: &POSTGRES_TABLES }),
        "RelationOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(6365), tables: &POSTGRES_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6368), tables: &POSTGRES_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6369), tables: &POSTGRES_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6370), tables: &POSTGRES_TABLES }),
        "RemainderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6371), tables: &POSTGRES_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6372), tables: &POSTGRES_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6373), tables: &POSTGRES_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6374), tables: &POSTGRES_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6375), tables: &POSTGRES_TABLES }),
        "ReplicaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6376), tables: &POSTGRES_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6377), tables: &POSTGRES_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6378), tables: &POSTGRES_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6379), tables: &POSTGRES_TABLES }),
        "ResetSessionAuthorizationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6380), tables: &POSTGRES_TABLES }),
        "ResetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6384), tables: &POSTGRES_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6390), tables: &POSTGRES_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6391), tables: &POSTGRES_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6392), tables: &POSTGRES_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6393), tables: &POSTGRES_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6394), tables: &POSTGRES_TABLES }),
        "RestrictedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6395), tables: &POSTGRES_TABLES }),
        "RestrictiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6396), tables: &POSTGRES_TABLES }),
        "RetrieveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6397), tables: &POSTGRES_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6398), tables: &POSTGRES_TABLES }),
        "ReturningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6399), tables: &POSTGRES_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6400), tables: &POSTGRES_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6401), tables: &POSTGRES_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6402), tables: &POSTGRES_TABLES }),
        "RightArrowSegment" => Some(RootGrammar { grammar_id: GrammarId(6425), tables: &POSTGRES_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6426), tables: &POSTGRES_TABLES }),
        "RightargKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6427), tables: &POSTGRES_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6428), tables: &POSTGRES_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6429), tables: &POSTGRES_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(6430), tables: &POSTGRES_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6431), tables: &POSTGRES_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6432), tables: &POSTGRES_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(6433), tables: &POSTGRES_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6434), tables: &POSTGRES_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6435), tables: &POSTGRES_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6436), tables: &POSTGRES_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6437), tables: &POSTGRES_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6438), tables: &POSTGRES_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6439), tables: &POSTGRES_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6440), tables: &POSTGRES_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6441), tables: &POSTGRES_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6442), tables: &POSTGRES_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6443), tables: &POSTGRES_TABLES }),
        "SafeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6444), tables: &POSTGRES_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(6445), tables: &POSTGRES_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6457), tables: &POSTGRES_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6458), tables: &POSTGRES_TABLES }),
        "ScalarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6459), tables: &POSTGRES_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6460), tables: &POSTGRES_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6461), tables: &POSTGRES_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6462), tables: &POSTGRES_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6463), tables: &POSTGRES_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6464), tables: &POSTGRES_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6465), tables: &POSTGRES_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6466), tables: &POSTGRES_TABLES }),
        "SecurityLabelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6467), tables: &POSTGRES_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(6558), tables: &POSTGRES_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6563), tables: &POSTGRES_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6572), tables: &POSTGRES_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6596), tables: &POSTGRES_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6609), tables: &POSTGRES_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6610), tables: &POSTGRES_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6634), tables: &POSTGRES_TABLES }),
        "SemiStructuredAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(6644), tables: &POSTGRES_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(6653), tables: &POSTGRES_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6654), tables: &POSTGRES_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6655), tables: &POSTGRES_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(6656), tables: &POSTGRES_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(6663), tables: &POSTGRES_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6670), tables: &POSTGRES_TABLES }),
        "Serial2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6671), tables: &POSTGRES_TABLES }),
        "Serial4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6672), tables: &POSTGRES_TABLES }),
        "Serial8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6673), tables: &POSTGRES_TABLES }),
        "SerialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6674), tables: &POSTGRES_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6675), tables: &POSTGRES_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6676), tables: &POSTGRES_TABLES }),
        "ServerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "Server_encodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6677), tables: &POSTGRES_TABLES }),
        "Server_versionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6678), tables: &POSTGRES_TABLES }),
        "SessionInformationUserFunctionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(6679), tables: &POSTGRES_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6684), tables: &POSTGRES_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6685), tables: &POSTGRES_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(6686), tables: &POSTGRES_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6690), tables: &POSTGRES_TABLES }),
        "SetConstraintsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6724), tables: &POSTGRES_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(6734), tables: &POSTGRES_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6743), tables: &POSTGRES_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6744), tables: &POSTGRES_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6752), tables: &POSTGRES_TABLES }),
        "SetSessionAuthorizationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6757), tables: &POSTGRES_TABLES }),
        "SetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6770), tables: &POSTGRES_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6804), tables: &POSTGRES_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6805), tables: &POSTGRES_TABLES }),
        "SettingsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6806), tables: &POSTGRES_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6807), tables: &POSTGRES_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6808), tables: &POSTGRES_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6809), tables: &POSTGRES_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(6810), tables: &POSTGRES_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6819), tables: &POSTGRES_TABLES }),
        "ShowStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6820), tables: &POSTGRES_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6830), tables: &POSTGRES_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6831), tables: &POSTGRES_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(6832), tables: &POSTGRES_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6835), tables: &POSTGRES_TABLES }),
        "SimpleGeometryGrammar" => Some(RootGrammar { grammar_id: GrammarId(6836), tables: &POSTGRES_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6838), tables: &POSTGRES_TABLES }),
        "SingleIdentifierFullGrammar" => Some(RootGrammar { grammar_id: GrammarId(6839), tables: &POSTGRES_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(6843), tables: &POSTGRES_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(6847), tables: &POSTGRES_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(6849), tables: &POSTGRES_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6853), tables: &POSTGRES_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(6854), tables: &POSTGRES_TABLES }),
        "SkipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6857), tables: &POSTGRES_TABLES }),
        "Skip_lockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6858), tables: &POSTGRES_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(6859), tables: &POSTGRES_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(6860), tables: &POSTGRES_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6861), tables: &POSTGRES_TABLES }),
        "SmallserialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6862), tables: &POSTGRES_TABLES }),
        "SnapshotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6863), tables: &POSTGRES_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6864), tables: &POSTGRES_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6865), tables: &POSTGRES_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6866), tables: &POSTGRES_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6867), tables: &POSTGRES_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6868), tables: &POSTGRES_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6869), tables: &POSTGRES_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6870), tables: &POSTGRES_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6871), tables: &POSTGRES_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6872), tables: &POSTGRES_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6873), tables: &POSTGRES_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6874), tables: &POSTGRES_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6875), tables: &POSTGRES_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6876), tables: &POSTGRES_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6877), tables: &POSTGRES_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6878), tables: &POSTGRES_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6879), tables: &POSTGRES_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6880), tables: &POSTGRES_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6881), tables: &POSTGRES_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6882), tables: &POSTGRES_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6883), tables: &POSTGRES_TABLES }),
        "StandaloneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6884), tables: &POSTGRES_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(6885), tables: &POSTGRES_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(230), tables: &POSTGRES_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(5860), tables: &POSTGRES_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6886), tables: &POSTGRES_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2154), tables: &POSTGRES_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6887), tables: &POSTGRES_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6888), tables: &POSTGRES_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6889), tables: &POSTGRES_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6890), tables: &POSTGRES_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7013), tables: &POSTGRES_TABLES }),
        "StatisticsReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7014), tables: &POSTGRES_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7015), tables: &POSTGRES_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7016), tables: &POSTGRES_TABLES }),
        "StoredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7017), tables: &POSTGRES_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7018), tables: &POSTGRES_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7019), tables: &POSTGRES_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7020), tables: &POSTGRES_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7021), tables: &POSTGRES_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(7022), tables: &POSTGRES_TABLES }),
        "StripKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7025), tables: &POSTGRES_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7026), tables: &POSTGRES_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(7031), tables: &POSTGRES_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7032), tables: &POSTGRES_TABLES }),
        "SubscriptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7033), tables: &POSTGRES_TABLES }),
        "SubscriptionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7034), tables: &POSTGRES_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7035), tables: &POSTGRES_TABLES }),
        "SummaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7036), tables: &POSTGRES_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7037), tables: &POSTGRES_TABLES }),
        "SupportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7038), tables: &POSTGRES_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(7039), tables: &POSTGRES_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7040), tables: &POSTGRES_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7041), tables: &POSTGRES_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7042), tables: &POSTGRES_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7043), tables: &POSTGRES_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7044), tables: &POSTGRES_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(7045), tables: &POSTGRES_TABLES }),
        "TableConstraintUsingIndexSegment" => Some(RootGrammar { grammar_id: GrammarId(7106), tables: &POSTGRES_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7129), tables: &POSTGRES_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7130), tables: &POSTGRES_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7143), tables: &POSTGRES_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7144), tables: &POSTGRES_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7145), tables: &POSTGRES_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7146), tables: &POSTGRES_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(7147), tables: &POSTGRES_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(7152), tables: &POSTGRES_TABLES }),
        "TargetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7156), tables: &POSTGRES_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7157), tables: &POSTGRES_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7158), tables: &POSTGRES_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7159), tables: &POSTGRES_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7160), tables: &POSTGRES_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(7161), tables: &POSTGRES_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(7162), tables: &POSTGRES_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7165), tables: &POSTGRES_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(7166), tables: &POSTGRES_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7169), tables: &POSTGRES_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7170), tables: &POSTGRES_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7171), tables: &POSTGRES_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7172), tables: &POSTGRES_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7173), tables: &POSTGRES_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7174), tables: &POSTGRES_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7175), tables: &POSTGRES_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(7176), tables: &POSTGRES_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7177), tables: &POSTGRES_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(7178), tables: &POSTGRES_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(7189), tables: &POSTGRES_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7195), tables: &POSTGRES_TABLES }),
        "TimestamptzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7196), tables: &POSTGRES_TABLES }),
        "TimetzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7197), tables: &POSTGRES_TABLES }),
        "TimingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7198), tables: &POSTGRES_TABLES }),
        "TinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7199), tables: &POSTGRES_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7200), tables: &POSTGRES_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7201), tables: &POSTGRES_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7202), tables: &POSTGRES_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7203), tables: &POSTGRES_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7204), tables: &POSTGRES_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7205), tables: &POSTGRES_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7206), tables: &POSTGRES_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7207), tables: &POSTGRES_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7208), tables: &POSTGRES_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7209), tables: &POSTGRES_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7226), tables: &POSTGRES_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7227), tables: &POSTGRES_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7228), tables: &POSTGRES_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7229), tables: &POSTGRES_TABLES }),
        "TriangleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7230), tables: &POSTGRES_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7231), tables: &POSTGRES_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &POSTGRES_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7232), tables: &POSTGRES_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(7233), tables: &POSTGRES_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7237), tables: &POSTGRES_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(7238), tables: &POSTGRES_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7239), tables: &POSTGRES_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7240), tables: &POSTGRES_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7257), tables: &POSTGRES_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7258), tables: &POSTGRES_TABLES }),
        "TsqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7259), tables: &POSTGRES_TABLES }),
        "TsrangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7260), tables: &POSTGRES_TABLES }),
        "TstzrangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7261), tables: &POSTGRES_TABLES }),
        "TsvectorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7262), tables: &POSTGRES_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(7263), tables: &POSTGRES_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7266), tables: &POSTGRES_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7267), tables: &POSTGRES_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7275), tables: &POSTGRES_TABLES }),
        "TypesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7278), tables: &POSTGRES_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7279), tables: &POSTGRES_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7280), tables: &POSTGRES_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7281), tables: &POSTGRES_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7282), tables: &POSTGRES_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(7283), tables: &POSTGRES_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(7284), tables: &POSTGRES_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7288), tables: &POSTGRES_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7289), tables: &POSTGRES_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(7290), tables: &POSTGRES_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7295), tables: &POSTGRES_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(7296), tables: &POSTGRES_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7298), tables: &POSTGRES_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7299), tables: &POSTGRES_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7300), tables: &POSTGRES_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7301), tables: &POSTGRES_TABLES }),
        "UnlistenStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7302), tables: &POSTGRES_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7307), tables: &POSTGRES_TABLES }),
        "UnloggedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7308), tables: &POSTGRES_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7309), tables: &POSTGRES_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7333), tables: &POSTGRES_TABLES }),
        "UnsafeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7339), tables: &POSTGRES_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7340), tables: &POSTGRES_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7341), tables: &POSTGRES_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7342), tables: &POSTGRES_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7343), tables: &POSTGRES_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7366), tables: &POSTGRES_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7367), tables: &POSTGRES_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7368), tables: &POSTGRES_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7369), tables: &POSTGRES_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7372), tables: &POSTGRES_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7373), tables: &POSTGRES_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(7374), tables: &POSTGRES_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7375), tables: &POSTGRES_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7376), tables: &POSTGRES_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7377), tables: &POSTGRES_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7378), tables: &POSTGRES_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7379), tables: &POSTGRES_TABLES }),
        "UuidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7380), tables: &POSTGRES_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7381), tables: &POSTGRES_TABLES }),
        "VacuumStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7382), tables: &POSTGRES_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7415), tables: &POSTGRES_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7416), tables: &POSTGRES_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7417), tables: &POSTGRES_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7418), tables: &POSTGRES_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7419), tables: &POSTGRES_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7430), tables: &POSTGRES_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7431), tables: &POSTGRES_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7432), tables: &POSTGRES_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7433), tables: &POSTGRES_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7434), tables: &POSTGRES_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7435), tables: &POSTGRES_TABLES }),
        "VariadicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7436), tables: &POSTGRES_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7437), tables: &POSTGRES_TABLES }),
        "VectorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7438), tables: &POSTGRES_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7439), tables: &POSTGRES_TABLES }),
        "VersionIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(7440), tables: &POSTGRES_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7443), tables: &POSTGRES_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7444), tables: &POSTGRES_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7445), tables: &POSTGRES_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7446), tables: &POSTGRES_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7447), tables: &POSTGRES_TABLES }),
        "WalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7448), tables: &POSTGRES_TABLES }),
        "WalrusOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(7449), tables: &POSTGRES_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7450), tables: &POSTGRES_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7451), tables: &POSTGRES_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7452), tables: &POSTGRES_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7453), tables: &POSTGRES_TABLES }),
        "WellKnownTextGeometrySegment" => Some(RootGrammar { grammar_id: GrammarId(7454), tables: &POSTGRES_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7554), tables: &POSTGRES_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7564), tables: &POSTGRES_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7565), tables: &POSTGRES_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(7570), tables: &POSTGRES_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7587), tables: &POSTGRES_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7588), tables: &POSTGRES_TABLES }),
        "WhitespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7589), tables: &POSTGRES_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(7590), tables: &POSTGRES_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7591), tables: &POSTGRES_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(7593), tables: &POSTGRES_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7603), tables: &POSTGRES_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(7604), tables: &POSTGRES_TABLES }),
        "WithCheckOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(7612), tables: &POSTGRES_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7619), tables: &POSTGRES_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7628), tables: &POSTGRES_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7637), tables: &POSTGRES_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(7642), tables: &POSTGRES_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7643), tables: &POSTGRES_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7644), tables: &POSTGRES_TABLES }),
        "WithinGroupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7649), tables: &POSTGRES_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7654), tables: &POSTGRES_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7655), tables: &POSTGRES_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(7656), tables: &POSTGRES_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7657), tables: &POSTGRES_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7658), tables: &POSTGRES_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7659), tables: &POSTGRES_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7660), tables: &POSTGRES_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7661), tables: &POSTGRES_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7662), tables: &POSTGRES_TABLES }),
        "XmlattributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7663), tables: &POSTGRES_TABLES }),
        "XmlconcatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7664), tables: &POSTGRES_TABLES }),
        "XmlelementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7665), tables: &POSTGRES_TABLES }),
        "XmlexistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7666), tables: &POSTGRES_TABLES }),
        "XmlforestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7667), tables: &POSTGRES_TABLES }),
        "XmlnamespacesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7668), tables: &POSTGRES_TABLES }),
        "XmlparseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7669), tables: &POSTGRES_TABLES }),
        "XmlpiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7670), tables: &POSTGRES_TABLES }),
        "XmlrootKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7671), tables: &POSTGRES_TABLES }),
        "XmlserializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7672), tables: &POSTGRES_TABLES }),
        "XmltableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7673), tables: &POSTGRES_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7674), tables: &POSTGRES_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7675), tables: &POSTGRES_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7676), tables: &POSTGRES_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7677), tables: &POSTGRES_TABLES }),
        "YesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7678), tables: &POSTGRES_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7679), tables: &POSTGRES_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7680), tables: &POSTGRES_TABLES }),
        _ => None,
    }
}

pub fn get_postgres_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AdjacentSegment" => Some("comparison_operator"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterAggregateStatementSegment" => Some("alter_aggregate_statement"),
        "AlterDatabaseStatementSegment" => Some("alter_database_statement"),
        "AlterDefaultPrivilegesGrantSegment" => Some("alter_default_privileges_grant"),
        "AlterDefaultPrivilegesObjectPrivilegesSegment" => Some("alter_default_privileges_object_privilege"),
        "AlterDefaultPrivilegesRevokeSegment" => Some("alter_default_privileges_revoke"),
        "AlterDefaultPrivilegesSchemaObjectsSegment" => Some("alter_default_privileges_schema_object"),
        "AlterDefaultPrivilegesStatementSegment" => Some("alter_default_privileges_statement"),
        "AlterDefaultPrivilegesToFromRolesSegment" => Some("alter_default_privileges_to_from_roles"),
        "AlterDomainStatementSegment" => Some("alter_domain_statement"),
        "AlterExtensionStatementSegment" => Some("alter_extension_statement"),
        "AlterForeignTableActionSegment" => Some("alter_foreign_table_action_segment"),
        "AlterForeignTableStatementSegment" => Some("alter_foreign_table_statement"),
        "AlterFunctionActionSegment" => Some("alter_function_action_segment"),
        "AlterFunctionStatementSegment" => Some("alter_function_statement"),
        "AlterIndexStatementSegment" => Some("alter_index_statement"),
        "AlterMaterializedViewActionSegment" => Some("alter_materialized_view_action_segment"),
        "AlterMaterializedViewStatementSegment" => Some("alter_materialized_view_statement"),
        "AlterPolicyStatementSegment" => Some("alter_policy_statement"),
        "AlterProcedureActionSegment" => Some("alter_procedure_action_segment"),
        "AlterProcedureStatementSegment" => Some("alter_procedure_statement"),
        "AlterPublicationStatementSegment" => Some("alter_publication_statement"),
        "AlterRoleStatementSegment" => Some("alter_role_statement"),
        "AlterSchemaStatementSegment" => Some("alter_schema_statement"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterStatisticsStatementSegment" => Some("alter_statistics_statement"),
        "AlterSubscriptionStatementSegment" => Some("alter_subscription"),
        "AlterSystemStatementSegment" => Some("alter_system_statement"),
        "AlterTableActionSegment" => Some("alter_table_action_segment"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AlterTextSearchConfigurationStatementSegment" => Some("alter_text_search_configuration_statement"),
        "AlterTriggerStatementSegment" => Some("alter_trigger"),
        "AlterTypeStatementSegment" => Some("alter_type_statement"),
        "AlterViewStatementSegment" => Some("alter_view_statement"),
        "AnalyzeStatementSegment" => Some("analyze_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "ArrayTypeSuffixSegment" => Some("array_type_suffix"),
        "AsAliasExpressionSegment" => Some("alias_expression"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CallStoredProcedureSegment" => Some("call_statement"),
        "CaseExpressionSegment" => Some("case_expression"),
        "ClusterStatementSegment" => Some("cluster_statement"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnTypeReferenceSegment" => Some("column_type_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentOnStatementSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeValueExpansionSegment" => Some("composite_value_expansion"),
        "ConcatSegment" => Some("binary_operator"),
        "ConflictActionSegment" => Some("conflict_action"),
        "ConflictTargetSegment" => Some("conflict_target"),
        "CopyStatementSegment" => Some("copy_statement"),
        "CreateAggregateStatementSegment" => Some("create_aggregate_statement"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateCollationStatementSegment" => Some("create_collation_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateDomainStatementSegment" => Some("create_domain_statement"),
        "CreateExtensionStatementSegment" => Some("create_extension_statement"),
        "CreateForeignDataWrapperStatementSegment" => Some("create_foreign_data_wrapper"),
        "CreateForeignTableStatementSegment" => Some("create_foreign_table_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateMaterializedViewStatementSegment" => Some("create_materialized_view_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateOperatorStatementSegment" => Some("create_operator_statement"),
        "CreatePolicyStatementSegment" => Some("create_policy_statement"),
        "CreateProcedureStatementSegment" => Some("create_procedure_statement"),
        "CreatePublicationStatementSegment" => Some("create_publication_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateServerStatementSegment" => Some("create_server_statement"),
        "CreateStatisticsStatementSegment" => Some("create_statistics_statement"),
        "CreateSubscriptionStatementSegment" => Some("create_subscription"),
        "CreateTableAsStatementSegment" => Some("create_table_as_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTextSearchConfigurationStatementSegment" => Some("create_text_search_configuration_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateTypeStatementSegment" => Some("create_type_statement"),
        "CreateUserMappingStatementSegment" => Some("create_user_mapping_statement"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "DateTimeLiteralGrammar" => Some("datetime_literal"),
        "DateTimeTypeIdentifier" => Some("datetime_type_identifier"),
        "DeallocateStatementSegment" => Some("deallocate_statement"),
        "Dedent" => Some("dedent"),
        "DefinitionParameterSegment" => Some("definition_parameter"),
        "DefinitionParametersSegment" => Some("definition_parameters"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DiscardStatementSegment" => Some("discard_statement"),
        "DoStatementSegment" => Some("do_statement"),
        "DropAggregateStatementSegment" => Some("drop_aggregate_statement"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropCollationStatementSegment" => Some("drop_collation_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropDomainStatementSegment" => Some("drop_domain_statement"),
        "DropExtensionStatementSegment" => Some("drop_extension_statement"),
        "DropForeignTableStatement" => Some("drop_foreign_table_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropMaterializedViewStatementSegment" => Some("drop_materialized_view_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropOwnedStatementSegment" => Some("drop_owned_statement"),
        "DropPolicyStatementSegment" => Some("drop_policy_statement"),
        "DropProcedureStatementSegment" => Some("drop_procedure_statement"),
        "DropPublicationStatementSegment" => Some("drop_publication_statement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropStatisticsStatementSegment" => Some("drop_statistics_statement"),
        "DropSubscriptionStatementSegment" => Some("drop_subscription"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTextSearchConfigurationStatementSegment" => Some("drop_text_search_configuration_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExclusionConstraintElementSegment" => Some("exclusion_constraint_element"),
        "ExecuteStatementSegment" => Some("execute_statement"),
        "ExplainOptionSegment" => Some("explain_option"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileSegment" => Some("file"),
        "ForClauseSegment" => Some("for_clause"),
        "ForeignTableColumnConstraintSegment" => Some("column_constraint_segment"),
        "ForeignTableTableConstraintSegment" => Some("table_constraint"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "IdentifierSegment" => Some("identifier"),
        "ImplicitIndent" => Some("indent"),
        "ImportForeignSchemaStatementSegment" => Some("import_foreign_schema_statement"),
        "Indent" => Some("indent"),
        "IndexAccessMethodSegment" => Some("index_access_method"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexElementOptionsSegment" => Some("index_element_options"),
        "IndexElementSegment" => Some("index_element"),
        "IndexParametersSegment" => Some("index_parameters"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "IntoClauseSegment" => Some("into_clause"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "LanguageClauseSegment" => Some("language_clause"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LikeOptionSegment" => Some("like_option_segment"),
        "LimitClauseSegment" => Some("limit_clause"),
        "ListenStatementSegment" => Some("listen_statement"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LoadStatementSegment" => Some("load_statement"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "LockTableStatementSegment" => Some("lock_table_statement"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedBySourceClauseSegment" => Some("merge_when_not_matched_by_source_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "MetaCommandQueryBufferStatement" => Some("meta_command_statement"),
        "NamedArgumentSegment" => Some("named_argument"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "NotExtendLeftSegment" => Some("comparison_operator"),
        "NotExtendRightSegment" => Some("comparison_operator"),
        "NotifyStatementSegment" => Some("notify_statement"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OperationClassReferenceSegment" => Some("operation_class_reference"),
        "OperatorClassReferenceSegment" => Some("operator_class_reference"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapSegment" => Some("comparison_operator"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionBoundSpecSegment" => Some("partition_bound_spec"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "PostgresCopyStdinDataStatementSegment" => Some("postgres_copy_stdin_data_statement"),
        "PrepareStatementSegment" => Some("prepare_statement"),
        "PsqlCopyMetaCommandStatementSegment" => Some("psql_copy_meta_command_statement"),
        "PsqlSetMetaCommandStatementSegment" => Some("psql_set_meta_command_statement"),
        "PsqlVariableGrammar" => Some("psql_variable"),
        "PublicationObjectsSegment" => Some("publication_objects"),
        "PublicationReferenceSegment" => Some("publication_reference"),
        "PublicationTableSegment" => Some("publication_table"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "QualifiedOperatorSegment" => Some("qualified_operator"),
        "ReassignOwnedStatementSegment" => Some("reassign_owned_statement"),
        "RefreshMaterializedViewStatementSegment" => Some("refresh_materialized_view_statement"),
        "ReindexStatementSegment" => Some("reindex_statement_segment"),
        "RelationOptionSegment" => Some("relation_option"),
        "RelationOptionsSegment" => Some("relation_options"),
        "ResetSessionAuthorizationStatementSegment" => Some("reset_session_authorization_statement"),
        "ResetStatementSegment" => Some("reset_statement"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SecurityLabelStatementSegment" => Some("security_label_statement"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SemiStructuredAccessorSegment" => Some("semi_structured_expression"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "ServerReferenceSegment" => Some("server_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetConstraintsStatementSegment" => Some("set_constraint_statement"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "SetSessionAuthorizationStatementSegment" => Some("set_session_authorization_statement"),
        "SetStatementSegment" => Some("set_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "ShowStatementSegment" => Some("show_statement"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StatisticsReferenceSegment" => Some("statistics_reference"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SubscriptionReferenceSegment" => Some("subscription_reference"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableConstraintUsingIndexSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnlistenStatementSegment" => Some("unlisten_statement"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "VacuumStatementSegment" => Some("vacuum_statement"),
        "ValuesClauseSegment" => Some("values_clause"),
        "VersionIdentifierSegment" => Some("version_identifier"),
        "WellKnownTextGeometrySegment" => Some("wkt_geometry_type"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCheckOptionSegment" => Some("with_check_option"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WithinGroupClauseSegment" => Some("withingroup_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static POSTGRES_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_postgres_root_grammar_id() -> GrammarId {
    GrammarId(4550)
}
pub fn get_postgres_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_postgres_root_grammar_id(),
        tables: &POSTGRES_TABLES,
    }
}
pub fn get_postgres_root_grammar() -> RootGrammar {
    get_postgres_root_grammar_table()
}
